/**
 * LyTreemapController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyTreemapController.java 36510 2012-08-17 15:43:06Z jlsaouli $
 */

package eu.w4.ae.control;

import java.text.*;
import java.util.*;

import leon.app.*;
import leon.app.behavior.*;
import leon.control.*;
import leon.control.builder.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;
import leon.view.event.*;
import leon.view.setinterface.*;
import eu.w4.ae.app.behaviorinterface.*;
import eu.w4.ae.control.builder.*;


/**
 * This class implements generic processing of the treemap. This controller is associated with a
 * behavior class which implements a LyTreemapBehaviorInterface. Usually, the behavior is a
 * LyTreemapBehavior instance.
 *
 * @see leon.app.behaviorinterface.LyTreemapBehaviorInterface
 * @see leon.app.behavior.LyTreemapBehavior
 */
public class LyTreemapController extends LyPanelController
	implements LyWorkSpaceListener,
		LyFilterEndInterface
{
	/** File Version */
	public static final String						 __VERSION =
		"$Id: LyTreemapController.java 36510 2012-08-17 15:43:06Z jlsaouli $";

	/** Separator id for tree map node. */
	public final static String						 SEP_ID = "@".intern();

	/** Separator id for hierarchy tree map node. */
	public final static String						 SEP_HIERARCHY_ID = "*".intern();

	/** Identifier of an input field : hierarchy fields. */
	public static final String						 HIERARCHY_FIELDS = "_hierarchyFields".intern();

	/** Identifier of an input field : hierarchy levels. */
	public static final String						 HIERARCHY_LEVELS = "_hierarchyLevels".intern();

	/** Identifier of an input field : size field. */
	public static final String						 SIZE_FIELD = "_sizeField".intern();

	/** Identifier of an input field : color field. */
	public static final String						 COLOR_FIELD = "_colorField".intern();

	/** Identifier of an input field : aggregation type. */
	public static final String						 AGGREGATION_TYPE = "_aggregationType".intern();

	/** Identifier of an input field : aggregation field. */
	public static final String						 AGGREGATION_FIELD = "_aggregationField"
			.intern();

	/** Identifier of an input field : filter expression. */
	public static final String						 FILTER_EXPRESSION = "_filterExpression"
			.intern();

	/** Identifier of an input field : filter activation. */
	public static final String						 FILTER_ACTIVE = "_filterActive".intern();

	/** Identifier of an input field : filter type. */
	public static final String						 FILTER_TYPE = "_filterType".intern();

	/** Identifier of an input field : search text. */
	public static final String						 SEARCH_TEXT = "_searchText".intern();

	/** Identifier of an input field : search list. */
	public static final String						 SEARCH_LIST = "_searchList".intern();

	/** Identifier of an input field : minimum color. */
	public static final String						 MIN_COLOR = "_minColor".intern();

	/** Identifier of an input field : maximum color. */
	public static final String						 MAX_COLOR = "_maxColor".intern();

	/** Identifier of an input field : threshold color value. */
	public static final String						 THRESHOLD_COLOR_VALUE = "_thresholdColorValue"
			.intern();

	/** Identifier of an input field : threshold color before. */
	public static final String						 THRESHOLD_COLOR_BEFORE = "_thresholdColorBefore"
			.intern();

	/** Identifier of an input field : threshold color after. */
	public static final String						 THRESHOLD_COLOR_AFTER = "_thresholdColorAfter"
			.intern();

	/** Identifier of an input field : threshold color update. */
	public static final String						 THRESHOLD_COLOR_UPDATE = "_thresholdColorUpdate"
			.intern();

	/** Identifier of an input field : threshold colors. */
	public static final String						 THRESHOLD_COLORS = "_thresholdColors".intern();

	/** Identifier for modifying filter. */
	public static final String						 DEMAND_SET_FILTER = "_setFilter".intern();

	/** Identifier for modifying extended filter. */
	public static final String						 DEMAND_SET_EXTENDED_FILTER = "_setExtendedFilter"
			.intern();

	/** Identifier for refreshing search list. */
	public static final String						 DEMAND_REFRESH_SEARCH_LIST = "_refreshSearchList"
			.intern();

	/** Identifier for updating threshold color. */
	public static final String						 DEMAND_UPDATE_THRESHOLD_COLOR = "_updateThresholdColor"
			.intern();

	/** Identifier for showing configuration form. */
	public static final String						 DEMAND_SHOW_CONFIGURATION = "_showConfigurationForm"
			.intern();

	/** Identifier for hiding configuration form. */
	public static final String						 DEMAND_HIDE_CONFIGURATION = "_hideConfigurationForm"
			.intern();

	/** Identifier for showing hierarchy form. */
	public static final String						 DEMAND_SHOW_HIERARCHY_FORM = "_showHierarchyForm"
			.intern();

	/** Identifier for showing representation form. */
	public static final String						 DEMAND_SHOW_REPRESENTATION_FORM = "_showRepresentationForm"
			.intern();

	/** Identifier for showing filter form. */
	public static final String						 DEMAND_SHOW_FILTER_FORM = "_showFilterForm"
			.intern();

	/** Identifier for showing colors form. */
	public static final String						 DEMAND_SHOW_COLORS_FORM = "_showColorsForm"
			.intern();

	/** Identifier for playing the treemap. */
	public static final String						 DEMAND_PLAY = "_play".intern();

	/** Identifier for pausing the treemap. */
	public static final String						 DEMAND_PAUSE = "_pause".intern();

	/** Identifier for zoom in. */
	public static final String						 DEMAND_ZOOM_IN = "_zoomIn".intern();

	/** Identifier for zoom out. */
	public static final String						 DEMAND_ZOOM_OUT = "_zoomOut".intern();

	/** Identifier for zoom out. */
	public static final String						 DEMAND_ZOOM_TOP = "_zoomTop".intern();

	/** Identifier for saving configuration. */
	public static final String						 DEMAND_SAVE_CONFIGURATION = "_saveConfiguration"
			.intern();

	/** Identifier for deleting configuration. */
	public static final String						 DEMAND_DELETE_CONFIGURATION = "_deleteConfiguration"
			.intern();

	/** Identifier for loading configuration. */
	public static final String						 DEMAND_LOAD_CONFIGURATION = "_loadConfiguration"
			.intern();

	/** Minimum algorithm for aggregation. */
	public static final String						 MINIMUM_AGGREGATION = "_minimumAggregation"
			.intern();

	/** Maximum algorithm for aggregation. */
	public static final String						 MAXIMUM_AGGREGATION = "_maximumAggregation"
			.intern();

	/** Average algorithm for aggregation. */
	public static final String						 AVERAGE_AGGREGATION = "_averageAggregation"
			.intern();

	/** Balanced algorithm for aggregation. */
	public static final String						 BALANCED_AGGREGATION = "_balancedAggregation"
			.intern();

	/** Prefix used to save last configuration. */
	public static final String						 LAST_CONFIGURATION_PREFIX = "_last_configuration_"
			.intern();

	/** Class info. */
	protected LyClassInfo							 _classInfo;

	/** Indicates if global configuration form must be used;. */
	protected boolean								 _useGlobalConfiguration;

	/** Fields used to display tooltips. */
	protected LyInfoList<LyFieldInfo>				 _tooltipFields;

	/** Fields used to display small images. */
	protected LyInfoList<LyFieldInfo>				 _pictoFields;

	/** Hierarchy fields. */
	protected LyInfoList<LyFieldInfo>				 _hierarchyFields;

	/** Hierarchy levels. */
	protected int									 _hierarchyLevels;

	/** Size field. */
	protected LyFieldInfo							 _sizeField;

	/** Indicates if size field has been graphically set. */
	protected boolean								 _sizeFieldSet;

	/** Color field. */
	protected LyFieldInfo							 _colorField;

	/** Indicates if color field has been graphically set. */
	protected boolean								 _colorFieldSet;

	/** Minimum color. */
	protected String								 _minColor;

	/** Maximum color. */
	protected String								 _maxColor;

	/** Aggregation type. */
	protected String								 _aggregationType;

	/** Aggregation field. */
	protected LyFieldInfo							 _aggregationField;

	/** Object workSpace. */
	protected LyWorkSpace							 _workSpace;

	/** Initial object workSpace. */
	protected LyWorkSpace							 _initialWorkSpace;

	/** Root node of the maptree hierarchy. */
	protected _LyTreeMapNode						 _rootNode;

	/** Visible root node of the maptree hierarchy. */
	protected _LyTreeMapNode						 _visibleRootNode;

	/** List of displayed map regions and associated node of the maptree hierarchy. */
	protected Hashtable<LyMapRegion, _LyTreeMapNode> _displayedMapRegions;

	/** Minimum value used for color computation. */
	protected float									 _minColorValue;

	/** Maximum value used for color computation. */
	protected float									 _maxColorValue;

	/** List of exception colors (keys are numeric values and values are colors). */
	protected Hashtable<Float, String>				 _exceptionColors;

	/**
	 * List of threshold colors (keys are numeric values and values are min and max colors in an
	 * array).
	 */
	protected Hashtable<Float, String[]>			 _thresholdColors;

	/** Value of selected color in the table of threshold colors. */
	protected Float									 _selectedThresholdValue;

	/** Class filter. */
	protected LyFilter								 _classFilter;

	/** Indicates if all the objects must be shown (even if not match filter). */
	protected boolean								 _filterShowAll;

	/** Indicates if configuration form is shown. */
	protected boolean								 _configurationFormShown;

	/** Indicates if pause mode is set. */
	protected boolean								 _pause;

	/** Identifier of the hierarchy form controller. */
	protected String								 _hierarchyCtrlId;

	/** Identifier of the representation form controller. */
	protected String								 _representationCtrlId;

	/** Identifier of the filter and search form controller. */
	protected String								 _filterSearchCtrlId;

	/** Identifier of the filter form controller. */
	protected String								 _filterCtrlId;

	/** Identifier of the extended filter form controller. */
	protected String								 _filterEditorCtrlId;

	/** Identifier of the colors form controller. */
	protected String								 _colorsCtrlId;

	/** Identifier of the update colors form controller. */
	protected String								 _updateColorsCtrlId;

	/** Current treemap configuration. */
	protected LyTreemapConfiguration				 _configuration;

	/** Border sizes used by current configuration. */
	protected String								 _borderSizes;

	/** Label styles used by current configuration. */
	protected String								 _labelStyles;

	/** Indicates when configuration form needs to be updated. */
	protected boolean								 _updateConfigurationForm;

	/** Current user. */
	protected LyObject								 _currentUser;

	/** Indicates whether a cancel demand has been registered or not. */
	protected boolean								 _cancelDemand;

	/**
	 * Constructor.
	 *
	 * @param session   current session associated to the controller. The controller automatically
	 *                  registers on the specified session.
	 * @param parent    parent controller (or null for 'root' controller). Controller hierarchy
	 *                  indicates a child controller (for example a dialog created from a main view)
	 * @param action    related action. If an action is specified, the controller uses action
	 *                  parameters (action name, action specific XML view description ...)
	 * @param classInfo class info for this controller
	 * @param context   current context (may be null if the view is not contextual to a list of
	 *                  objects).
	 */
	public LyTreemapController(LySession session, LyController parent, LyAction action,
		LyClassInfo classInfo, LyObjectList context)
	{
		super(session, parent, action, context);

		_classInfo = classInfo;
		_useGlobalConfiguration = true;
		_tooltipFields = null;
		_pictoFields = null;
		_hierarchyFields = null;
		_hierarchyLevels = -1;
		_sizeField = null;
		_sizeFieldSet = false;
		_colorField = null;
		_colorFieldSet = false;
		_minColor = null;
		_maxColor = null;
		_aggregationType = null;
		_aggregationField = null;
		_rootNode = null;
		_visibleRootNode = null;
		_displayedMapRegions = null;
		_exceptionColors = null;
		_thresholdColors = null;
		_selectedThresholdValue = null;
		_classFilter = null;
		_filter = null;
		_filterActive = true;
		_filterShowAll = true;
		_configurationFormShown = true;
		_pause = false;
		_hierarchyCtrlId = null;
		_representationCtrlId = null;
		_filterSearchCtrlId = null;
		_filterCtrlId = null;
		_filterEditorCtrlId = null;
		_colorsCtrlId = null;
		_updateColorsCtrlId = null;
		_configuration = null;
		_borderSizes = null;
		_labelStyles = null;
		_updateConfigurationForm = false;
		_currentUser = null;
		_cancelDemand = false;

		init();
	}

	/**
	 * Method that gives the interface that must be implemented by the behavior associated to this
	 * controller.
	 *
	 * @return Class representing the behavior interface of this controller.
	 */
	public Class<?> getBehaviorInterface()
	{
		return LyTreemapBehaviorInterface.class;
	}

	/**
	 * Initialize map controller.
	 */
	protected void init()
	{
		// Retrieve current user
		LySession session = getSession();

		_currentUser = session.getUser();

		// Retrieve class info and filters
		_classInfo = (LyClassInfo)getParameter("_classInfo");
		_classFilter = (LyFilter)getParameter("_classFilter");
		_filter = (LyFilter)getParameter("_filter");

		// Retrieve _useGlobalConfiguration parameter
		String useGlobalConfiguration = (String)getParameter("_useGlobalConfiguration");

		_useGlobalConfiguration = ((useGlobalConfiguration != null)
					&& (useGlobalConfiguration.equals("true")));

		// Retrieve exception colors
		int		 i = 1;
		String   colorString = (String)getParameter("_exceptionColor" + i, true);
		String[] values;

		while (colorString != null)
		{
			colorString = (String)getParameter("_exceptionColor" + i, true);

			if (colorString == null)
				break;

			values = LyString.stringToList(colorString, '#');

			if (_exceptionColors == null)
				_exceptionColors = new Hashtable<Float, String>();

			_exceptionColors.put(new Float(values[0]), values[1]);
			i++;
		}

		// Retrieve fields used by tooltips
		BitSet tooltipFieldMarks = getParameterMarks("_tooltipFieldMarks", LyBitSet.Type.F_MARKS);

		if (tooltipFieldMarks != null)
			_tooltipFields = _classInfo.getFieldInfos(tooltipFieldMarks);
		else
		{
			// If no mark specified for tooltips, use fields that are not hidden and not secret
			_tooltipFields = _classInfo.getFieldInfos();
			_tooltipFields = _tooltipFields.getInfos(LyFieldInfo.MARK_HIDDEN, true);

			if (_tooltipFields != null)
				_tooltipFields = _tooltipFields.getInfos(LyFieldInfo.MARK_SECRET, true);
		}

		// Retrieve fields used by small images
		BitSet pictoFieldMarks = getParameterMarks("_pictoFieldMarks", LyBitSet.Type.F_MARKS);

		if (pictoFieldMarks != null)
			_pictoFields = _classInfo.getFieldInfos(pictoFieldMarks);

		// Retrieve threshold colors
		i = 1;
		colorString = (String)getParameter("_thresholdColor" + i, true);

		while (colorString != null)
		{
			colorString = (String)getParameter("_thresholdColor" + i, true);

			if (colorString == null)
				break;

			values = LyString.stringToList(colorString, '#');

			if (_thresholdColors == null)
				_thresholdColors = new Hashtable<Float, String[]>();

			_thresholdColors.put(new Float(values[0]), new String[] { values[1], values[2] });
			i++;
		}

		// Restore last configuration for current user
		LyTreemapBehaviorInterface behavior = (LyTreemapBehaviorInterface)getBehavior();

		behavior.completeTreemap(this);

		LyMap    map = getMapComponent();
		LyLayout layout = map.getLayout();

		_borderSizes = layout.getBorderSizes();
		_labelStyles = layout.getLabelStyles();

		if (behavior.saveUserConfiguration(this))
		{
			LyApplication					   application = getApplication();
			LyInfoList<LyTreemapConfiguration> configurations = application.getInfoList(
					LyTreemapConfiguration.class);
			LyTreemapConfiguration			   configuration;
			String							   userId,
											   currentUserId = ((_currentUser == null)
						? "" : _currentUser.getId());
			int								   n = ((configurations == null)
					? 0 : configurations.getSize());
			String							   lastConfName = getLastConfigurationName();

			for (i = 0; i < n; i++)
			{
				configuration = configurations.getInfo(i);
				userId = configuration.getUser();

				if (userId == null)
					userId = "";

				// If last configuration for current user, set this configuration
				if ((configuration.getObjectInfo().getId().equals(_action.getId())
							|| configuration.getObjectInfo().getId().equals(_classInfo.getId()))
						&& (configuration.getName().equals(lastConfName))
						&& (currentUserId.equals(userId)))
				{
					setConfiguration(configuration, false);
					break;
				}
			}
		}

		updateTools();
	} // end method init

	/**
	 * Cancel the creation of the treemap. If this method is called while treemap is being built,
	 * creation process is stopped as soon as possible.
	 */
	public void cancel()
	{
		_cancelDemand = true;
	}

	/**
	 * Return the name of the last configuration (automatically saved).
	 *
	 * @return the name of the last configuration
	 */
	protected String getLastConfigurationName()
	{
		String		 currentUserId = ((_currentUser == null) ? "" : _currentUser.getId());
		LyObjectInfo objectInfo;

		if (_action.isActionRef())
			objectInfo = _classInfo;
		else
			objectInfo = _action;

		return (LAST_CONFIGURATION_PREFIX + objectInfo.getId() + "_" + currentUserId);
	}

	/**
	 * Reset component.
	 *
	 * @param full Boolean indicating if a full reset (including reset of datas) is requiered or not
	 *             (only reset graphical area).
	 */
	public void reset(boolean full)
	{
		if (full && !_cancelDemand)
		{
			if (_workSpace != null)
			{
				try
				{
					_workSpace.free(this);
				}
				catch (Exception e)
				{
					getEnvironment().writeException(e);
				}

				_workSpace = null;
			}

			LySession   session = getSession();
			LyWorkSpace context = getContext();

			if (context == null)
				_workSpace = new LyClassWorkSpace(session, _classInfo, _classFilter, true);
			else
			{
				_workSpace = (LyWorkSpace)context.getList(session, _classInfo, true);

				if (_classFilter != null)
					_workSpace = new LySubWorkSpace(session, _workSpace, _classFilter, true);
			}

			_initialWorkSpace = _workSpace;

			if (_filterActive)
				_workSpace = new LySubWorkSpace(session, _initialWorkSpace, _filter, true);

			_workSpace.addListener(this);

			activateFilter(_filterActive, _filterShowAll, false);
		}

		updateConfigurationMenu();

		if (_cancelDemand)
			free();
		else
			super.reset(full);
	} // end method reset

	/**
	 * Update the menu for the current controller (including configuration menu).
	 */
	protected void updateMenuBar()
	{
		updateConfigurationMenu();

		super.updateMenuBar();
	}

	/**
	 * Indicates if global configuration form is used by this controller.
	 *
	 * @return true if global configuration form is used by this controller, false otherwise
	 */
	public boolean useGlobalConfiguration()
	{
		return _useGlobalConfiguration;
	}

	/**
	 * Return the user of the treemap.
	 *
	 * @return the user of the treemap or null is no user is connected
	 */
	public LyObject getUser()
	{
		return _currentUser;
	}

	/**
	 * This method is invoked to enable the the area(s) to complete the message bar of the view and
	 * add its own message items.
	 *
	 * @param messages messages bar
	 * @param level    index of message
	 * @see   LyAreaBuilder#completeMessages
	 */
	public void completeMessages(LyMessages messages, int level)
	{
		if (_action.hasMark(LyAction.MARK_NO_MESSAGES))
			return;

		LyMessage objectsMessage = buildObjectsMessage();
		LyMessage filterMessage = buildFilterMessage();

		if (objectsMessage != null)
			messages.addMessage(objectsMessage);
		if (filterMessage != null)
			messages.addMessage(filterMessage);

		if (_builder instanceof LyAreaBuilder)
			((LyAreaBuilder)_builder).completeMessages(messages, level);
	}

	/**
	 * Builds the objects message description.
	 *
	 * @return the component for the objects message description
	 */
	protected LyMessage buildObjectsMessage()
	{
		LyMessage objectsMessage = new LyMessage(this.getId() + "_objectsMessage", " ");

		objectsMessage.setSize(10);

		return objectsMessage;
	}

	/**
	 * Builds the filter message description.
	 *
	 * @return the component for the objects message description
	 */
	protected LyMessage buildFilterMessage()
	{
		// Sensitive if filtering is allowed on some fields of the classInfo
		// displayed in the table
		LyClassInfo			    classInfo = getClassInfo();
		LyInfoList<LyFieldInfo> filterFields = ((classInfo == null)
				? null : classInfo.getFieldInfos(LyFieldInfo.MARK_FILTER));
		LyMessage			    filterMessage = null;

		if ((filterFields != null)
				&& (filterFields.getSize() > 0))
		{
			filterMessage = new LyMessage(getId() + "_filterMessage", " ");
			filterMessage.setController(getId());
			filterMessage.setSize(10);
		}

		return filterMessage;
	}

	/**
	 * This method updates the content of the message bar. Current implementation indicates
	 * selection, filters ...
	 */
	protected void updateMessages()
	{
		if (_component == null)
			return;

		// If table is Idle, no need to refresh
		if (isIdle())
			return;

		LyView view = getView();

		if (view == null)
			return;

		LyMessage     objectsMessage = (LyMessage)view.getComponent(getId() + "_objectsMessage");
		LyMessage     filterMessage = (LyMessage)view.getComponent(getId() + "_filterMessage");
		int			  nSelections = ((_selections == null) ? 0 : _selections.getSize());
		LyEnvironment environment = getEnvironment();

		// Set message for filter
		if (filterMessage != null)
		{
			StringBuffer filterLabel;

			if (_filter == null)
			{
				filterMessage.setImage("LY_UNACTIVE_FILTER");
				filterLabel = new StringBuffer(environment.getMessage("LY_NO_FILTER"));
			}
			else if (_filterActive)
			{
				int filteredCount;

				filterMessage.setImage("LY_ACTIVE_FILTER");
				filterLabel = new StringBuffer(environment.getMessage("LY_ACTIVE_FILTER"));
				filterLabel.append(" ");

				if (_filterShowAll)
				{
					LySubWorkSpace wsTmp = new LySubWorkSpace(getSession(), _workSpace, _filter,
							false);

					filteredCount = wsTmp.getSize();
				}
				else
					filteredCount = ((_workSpace == null) ? 0 : _workSpace.getSize());

				filterLabel.append(getMessageFilterLabel(filteredCount));
			}
			else
			{
				filterMessage.setImage("LY_UNACTIVE_FILTER");
				filterLabel = new StringBuffer(environment.getMessage("LY_UNACTIVE_FILTER"));
			}

			filterMessage.setLabel(filterLabel.toString());
		}

		// Set message for objects
		if (objectsMessage != null)
		{
			int count;

			if (_visibleRootNode == _rootNode)
				count = ((_workSpace == null) ? 0 : _workSpace.getSize());
			else
			{
				LySimpleObjectList objects = new LySimpleObjectList();

				_visibleRootNode.computeSubObjects(objects);

				count = objects.getSize();
			}

			StringBuffer selectionsLabel = new StringBuffer(environment.getMessage(
						"LY_OBJECTS_COUNT", new Integer(count)));

			if (count > 0)
			{
				selectionsLabel.append(" (");
				selectionsLabel.append(environment.getMessage("LY_SELECTED_OBJECTS_COUNT",
						new Integer(nSelections)));
				selectionsLabel.append(")");
			}

			objectsMessage.setLabel(selectionsLabel.toString());
		}
	} // end method updateMessages

	/**
	 * Gets the selection text for the specified number of items in the view.
	 *
	 * @param  n number of objects matching filter
	 * @return the label shown in message bar.
	 */
	protected String getMessageFilterLabel(int n)
	{
		int count = ((_initialWorkSpace == null) ? 0 : _initialWorkSpace.getSize());

		return getEnvironment().getMessage("LY_FILTER_OBJECTS_COUNT", new Integer(n),
			new Integer(count));
	}

	/**
	 * Reset the content of the view (build the hierarchy and the map regions).
	 */
	public void updateContent()
	{
		updateContent(false);
	}

	/**
	 * Reset the content of the view (build the hierarchy and the map regions).
	 *
	 * @param resetVisibleRootNode indicates if current position in the hierarchy must be reset or
	 *                             not
	 */
	public void updateContent(boolean resetVisibleRootNode)
	{
		if (_cancelDemand)
			return;

		if (_component == null)
			return;

		LyMap map = getMapComponent();

		synchronized (map)
		{
			boolean  idle = isIdle();
			String[] hierarchyIds = null;

			map.setIdle(true);

			// Compute the identifiers used to reach visible root node
			if (!resetVisibleRootNode && (_visibleRootNode != _rootNode))
				hierarchyIds = getHierarchyPosition();

			// Compute new hierarchy and colors
			computeTreemapNodes();

			// Reset old navigation position in the hierarchy
			if (hierarchyIds != null)
				setHierarchyPosition(hierarchyIds);

			// Build the map regions and reset the configuration form
			buildMapRegions();
			updateHierarchyFields();
			updateAggregationFieldState();
			updateFilterText();
			updateThresholdColorList();
			updateTools();
			updateMessages();

			// Idle mode
			map.setIdle(idle);
		}
	} // end method updateContent

	/**
	 * Return current position in the hierarchy.
	 *
	 * @return an array containing the identifiers of rectangles used to go to current position in
	 *         the hierarchy.
	 */
	public String[] getHierarchyPosition()
	{
		_LyTreeMapNode		   node = _visibleRootNode;
		Vector<_LyTreeMapNode> nodes = new Vector<_LyTreeMapNode>();

		while (node != _rootNode)
		{
			nodes.addElement(node);
			node = node._parent;
		}

		int		 i, n = nodes.size();
		String[] hierarchyIds = new String[n];

		for (i = n - 1; i >= 0; i--)
		{
			node = nodes.elementAt(i);
			hierarchyIds[n - i - 1] = node._id;
		}

		return hierarchyIds;
	}

	/**
	 * Sets current position in the hierarchy.
	 *
	 * @param  hierarchyIds an array containing the identifiers of rectangles to be used to go to
	 *                      given position in the hierarchy.
	 * @return true is the position exists in the hierarchy, false otherwise
	 */
	public boolean setHierarchyPosition(String[] hierarchyIds)
	{
		int i, n = hierarchyIds.length;

		_visibleRootNode = _rootNode;

		for (i = 0; (i < n) && (_visibleRootNode != null); i++)
			_visibleRootNode = _visibleRootNode.getSon(hierarchyIds[i]);

		if (_visibleRootNode == null)
		{
			_visibleRootNode = _rootNode;
			return false;
		}

		return true;
	}

	/**
	 * Indicates if pause mode is set.
	 *
	 * @return true if pause mode is set, false otherwise
	 */
	public boolean isPaused()
	{
		return _pause;
	}

	/**
	 * Recursively updates the min and max values used to compute colors.
	 */
	protected void updateColorValues()
	{
		_minColorValue = Float.MAX_VALUE;
		_maxColorValue = -Float.MAX_VALUE;
		updateColorValues(_rootNode);
	}

	/**
	 * Recursively updates the min and max values used to compute colors for given node.
	 *
	 * @param node the node
	 */
	protected void updateColorValues(_LyTreeMapNode node)
	{
		if (_cancelDemand)
			return;

		LyTreemapBehaviorInterface behavior = (LyTreemapBehaviorInterface)getBehavior();
		_LyTreeMapNode			   son;
		LyObject				   object;
		Number					   number;
		float					   fValue;
		LySimpleObjectList		   objects;
		LyFieldInfo				   colorField = getColorField();
		int						   i, n = ((node._sons != null) ? node._sons.size() : 0);

		// Retrieve max and min values used for colors
		if (colorField != null)
		{
			object = node._object;
			objects = new LySimpleObjectList();
			objects.setSorted(false);

			// If the node has a field info, it is not a leaf node of the hierarchy :
			// retrieve all the objects that it contains to compute its color
			if (node._fieldInfo != null)
				node.computeSubObjects(objects);
			// If the node is a leaf, only its object is used to compute its color
			else if (object != null)
				objects.addObject(node._object);

			if (objects.getSize() > 0)
				number = behavior.computeColorValue(this, colorField, objects,
						(node._fieldInfo != null));
			else
				number = null;

			if (number != null)
			{
				fValue = number.floatValue();

				if (fValue > _maxColorValue)
					_maxColorValue = fValue;

				if (fValue < _minColorValue)
					_minColorValue = fValue;
			}

			for (i = 0; i < n; i++)
			{
				son = node._sons.elementAt(i);
				updateColorValues(son);
			}
		}
	} // end method updateColorValues

	/**
	 * Return the list of fields used to display tooltips on the map.
	 *
	 * @return the list of fields used to display tooltips on the map
	 */
	public LyInfoList<LyFieldInfo> getTooltipFields()
	{
		return _tooltipFields;
	}

	/**
	 * Return the list of fields used to display small images on the map.
	 *
	 * @return the list of fields used to display small images on the map
	 */
	public LyInfoList<LyFieldInfo> getPictoFields()
	{
		return _pictoFields;
	}

	/**
	 * Shows or hides the configuration form.
	 *
	 * @param show indicates whether the configuration form must be shown or hidden
	 */
	protected void showConfigurationForm(boolean show)
	{
		LyArea			component = (LyArea)getComponent();
		LyConstraint  	constraint = new LyConstraint();
		
		constraint.setFill(LyConstraint.Fill.BOTH);

		// If configuration form must be shown, rebuild the splitter with the two areas
		if (show)
		{
			LySplitter		 splitter = (LySplitter)component.getPanel(0);
			LyArea			 rightArea = new LyArea();
			LyArea			 mapArea = (LyArea)splitter.getPanel(0);
			LyTreemapBuilder builder = (LyTreemapBuilder)getBuilder();

			component.removePanel(splitter);
			splitter = new LySplitter();
			splitter.setConstraint(constraint);
			splitter.setResizable(true);
			splitter.setSizes(LyTreemapBuilder.SPLITTER_SIZES);
			rightArea.addPanel(builder.getFormArea());
			rightArea.setScrolling(LyArea.Scrolling.AUTO);
			splitter.addPanel(mapArea);
			splitter.addPanel(rightArea);
			component.addPanel(splitter);

			// If a configuration has been set while form was hidden, reset it
			if (_updateConfigurationForm)
				setConfiguration(_configuration, true);

			refreshSearchList();
		}
		// If configuration form must be hidden, remove splitter and add configuration form
		else
		{
			LySplitter splitter = (LySplitter)component.getPanel(0);
			LyArea     mapArea = (LyArea)splitter.getPanel(0);

			component.removePanel(splitter);
			splitter = new LySplitter();
			splitter.setConstraint(constraint);
			splitter.setResizable(false);
			splitter.setSizes("100");
			splitter.addPanel(mapArea);
			component.addPanel(splitter);
		}

		_configurationFormShown = show;
		updateTools();
		setLastModified(new Date().getTime());
	} // end method showConfigurationForm

	/**
	 * Sets or unsets the pause mode.
	 *
	 * @param pause indicates whether the pause mode must be set or unset
	 */
	protected void pause(boolean pause)
	{
		_pause = pause;
		updateTools();

		if (!pause)
			updateContent(false);
	}

	/**
	 * This method frees this controller.
	 *
	 * @param controller the parent controller
	 */
	public void free(LyController controller)
	{
		if (_parent != controller)
			return;

		// Save current configuration in order to reload it next time the user launch this action
		LyTreemapBehaviorInterface behavior = (LyTreemapBehaviorInterface)getBehavior();

		setIdle(true);

		if ((behavior != null)
			&& behavior.saveUserConfiguration(this))
		{
			_LySaveConfigurationBehavior saveBehavior = new _LySaveConfigurationBehavior();

			saveBehavior.init(this);
			saveBehavior.writeConfiguration(getLastConfigurationName(), false);
		}

		if (behavior != null)
			behavior.free(this);

		// Free workspaces
		if (_workSpace != null)
			_workSpace.free(this);

		// Free memory (gc)
		_workSpace = null;
		_selections = null;

		if (_displayedMapRegions != null)
		{
			_displayedMapRegions.clear();
			_displayedMapRegions = null;
		}

		if (_exceptionColors != null)
		{
			_exceptionColors.clear();
			_exceptionColors = null;
		}

		if (_thresholdColors != null)
		{
			_thresholdColors.clear();
			_thresholdColors = null;
		}

		// Retrieve internal map component
		if (_component != null)
		{
			LyMap map = getMapComponent();

			if (map != null)
				map.free();
		}

		// Free sub-forms controllers
		LyController formController;

		formController = getHierarchyFormController();
		if (formController != null)
			((LyAreaController)formController).free(this);

		formController = getRepresentationFormController();
		if (formController != null)
			((LyAreaController)formController).free(this);

		formController = getFilterFormController();
		if (formController != null)
			((LyAreaController)formController).free(this);

		formController = getColorsFormController();
		if (formController != null)
			((LyAreaController)formController).free(this);

		formController = getUpdateColorsFormController(false);
		if (formController != null)
			((LyAreaController)formController).free(this);

		super.free(controller);
	} // end method free

	/**
	 * Return the form field with given identifier.
	 *
	 * @param  fieldId the identifier of the field
	 * @return the form field with given identifier
	 */
	protected LyFormField getFormField(String fieldId)
	{
		LyComponent component = null;

		if (useGlobalConfiguration())
			component = getComponent();
		else
		{
			LyAreaController controller = null;

			if (HIERARCHY_FIELDS.equals(fieldId) || HIERARCHY_LEVELS.equals(fieldId))
				controller = getHierarchyFormController();
			else if (SIZE_FIELD.equals(fieldId) || COLOR_FIELD.equals(fieldId)
					|| AGGREGATION_FIELD.equals(fieldId) || AGGREGATION_TYPE.equals(fieldId))
				controller = getRepresentationFormController();
			else if (FILTER_EXPRESSION.equals(fieldId) || FILTER_ACTIVE.equals(fieldId)
					|| FILTER_TYPE.equals(fieldId)
					|| SEARCH_TEXT.equals(fieldId) || SEARCH_LIST.equals(fieldId))
				controller = getFilterFormController();
			else if (MIN_COLOR.equals(fieldId) || MAX_COLOR.equals(fieldId)
					|| THRESHOLD_COLOR_VALUE.equals(fieldId)
					|| THRESHOLD_COLOR_BEFORE.equals(fieldId)
					|| THRESHOLD_COLOR_AFTER.equals(fieldId) || THRESHOLD_COLORS.equals(fieldId))
				controller = getColorsFormController();

			if (controller != null)
				component = controller.getComponent();
		}

		if (component == null)
			return null;
		
		LyFormField field = (LyFormField)component.getComponent(fieldId);

		return field;
	} // end method getFormField

	/**
	 * Update the state of the hierarchy fields and levels component. The maximum value of levels is
	 * computed with the number of hierarchy fields.
	 */
	public void updateHierarchyFields()
	{
		LyInfoList<LyFieldInfo> hierarchyFieldList = getHierarchyFieldList();
		LyInfoList<LyFieldInfo> hierarchyFields = getHierarchyFields();
		LyFormField			    hierarchyFieldsField = getFormField(HIERARCHY_FIELDS);
		LyFormField			    hierarchyLevelsField = getFormField(HIERARCHY_LEVELS);
		LyLink				    link;

		// Hierarchy fields may not be displayed
		if (hierarchyFieldsField != null)
		{
			LyRelation  relation = (LyRelation)hierarchyFieldsField.getFieldItem();
			int		    i, n = relation.getLinkCount(), count = 0;
			LyFieldInfo field;

			relation.setIdle(true);

			// Remove all the links
			for (i = n - 1; i >= 0; i--)
			{
				link = relation.getLink(i);
				relation.removeLink(link);
			}

			// Add non selected links
			n = hierarchyFieldList.getSize();

			for (i = 0; i < n; i++)
			{
				field = hierarchyFieldList.getInfo(i);

				if (hierarchyFields.contains(field))
					continue;

				link = new LyLink(field.getId(), field.getName());
				relation.addLink(link);
				count++;
			}

			// Add selected links
			n = hierarchyFields.getSize();

			for (i = 0; i < n; i++)
			{
				field = hierarchyFields.getInfo(i);
				link = new LyLink(field.getId(), field.getName());
				relation.addLink(link);
				relation.selectLink(count, true);
				link.setSelected(true);
				count++;
			}

			relation.setIdle(false);
		}

		// Hierarchy fields may not be displayed
		if (hierarchyLevelsField != null)
		{
			LyNumber number = (LyNumber)hierarchyLevelsField.getFieldItem();

			number.setVMax(hierarchyFields.getSize() + 1);

			if (_hierarchyLevels > (hierarchyFields.getSize() + 1))
				_hierarchyLevels = hierarchyFields.getSize() + 1;

			if (_hierarchyLevels > 0)
				number.setLabel(_hierarchyLevels + "");
		}
	} // end method updateHierarchyFields

	/**
	 * Update the state of the aggregation field component. This component is enabled only if
	 * balanced aggregation is chosen.
	 */
	public void updateAggregationFieldState()
	{
		String				    aggregationType = getAggregationType(false);
		int					    hierarchyLevels = getHierarchyLevels();
		LyInfoList<LyFieldInfo> hierarchyFields = getHierarchyFields();
		LyFormField			    aggregationTypeField = getFormField(AGGREGATION_TYPE);
		LyFormField			    aggregationField = getFormField(AGGREGATION_FIELD);
		boolean				    groups = (hierarchyLevels <= hierarchyFields.getSize());

		// Representation form may not be displayed
		if ((aggregationTypeField == null) || (aggregationField == null))
			return;

		// Type of aggregation is enabled only if number of levels is lower than
		// number of hierarchy fields
		aggregationTypeField.setEnabled(groups);

		// Field used for aggregation is enabled only if balanced aggregation is used
		aggregationField.setEnabled(groups && BALANCED_AGGREGATION.equals(aggregationType));
	}

	/**
	 * Refresh the search list with the prefix given by the search text. The list will display map
	 * regions whose label starts with given prefix (not case sensitive).
	 */
	protected void refreshSearchList()
	{
		// First retrieve the map regions that match the given prefix
		LyFormField textField = getFormField(SEARCH_TEXT);

		// Search text may not be displayed
		if (textField == null)
			return;

		LyText			    text = (LyText)textField.getFieldItem();
		String			    prefix = text.getLabel();
		LyMapRegion		    region;
		String			    label, lowerLabel;
		Vector<LyMapRegion> regions = getMapRegions();
		Vector<String>	    labels = new Vector<String>();
		int				    i, n;

		n = regions.size();

		for (i = n - 1; i >= 0; i--)
		{
			region = regions.elementAt(i);
			label = region.getLabel();
			lowerLabel = ((label == null) ? null : label.toLowerCase());

			if ((prefix != null) && (prefix.length() != 0)
					&& (lowerLabel != null) && lowerLabel.startsWith(prefix.toLowerCase())
					&& !labels.contains(label))
				labels.addElement(label);
		}

		// Clear the search list and update its content
		LyFormField  searchField = getFormField(SEARCH_LIST);
		
		if (searchField == null)
			return;
		
		LyList		 list = (LyList)searchField.getFieldItem();
		LyListOption option;

		n = list.getListOptionCount();

		for (i = n - 1; i >= 0; i--)
		{
			option = list.getListOption(i);
			list.removeListOption(option);
		}

		n = labels.size();

		for (i = 0; i < n; i++)
		{
			label = labels.elementAt(i);
			option = new LyListOption(label, label);
			list.addListOption(option);
		}
	} // end method refreshSearchList

	/**
	 * Update the selection state of the map regions with the selected element of the search list.
	 */
	protected void updateMapRegionsSelection()
	{
		// Retrieve selected option
		LyFormField  searchField = getFormField(SEARCH_LIST);
		LyList		 list = (searchField == null ? null : (LyList)searchField.getFieldItem());
		LyListOption option;
		String		 label = null;
		int			 i, n = (list == null ? 0 : list.getListOptionCount());

		for (i = 0; i < n; i++)
		{
			option = list.getListOption(i);

			if (option.isSelected())
			{
				label = option.getLabel();
				break;
			}
		}

		// Update selection state for all the map regions of the map
		LyMap			    map = getMapComponent();
		LyMapRegion		    region;
		Vector<LyMapRegion> regions = new Vector<LyMapRegion>();
		String			    regionLabel;
		boolean			    selected;

		synchronized (map)
		{
			boolean idle = isIdle();

			map.setIdle(true);
			regions = getMapRegions();
			n = regions.size();

			for (i = 0; i < n; i++)
			{
				region = regions.elementAt(i);
				regionLabel = region.getLabel();
				selected = (label != null) && (label.equals(regionLabel));

				if (region.isSelected() != selected)
					selectMapRegion(region.getId(), selected);
			}

			map.setIdle(idle);
		}

		updateTools();
		updateMessages();
	} // end method updateMapRegionsSelection

	/**
	 * Retrieves a a map region by its id.
	 *
	 * @param  id the map region id
	 * @return the map region corresponding to the id or null if none was found
	 */
	public LyMapRegion getMapRegion(String id)
	{
		if (_component == null)
			return null;

		if (id == null)
			return null;

		LyMap			    map = getMapComponent();
		LyMapRegion		    region;
		Vector<LyMapRegion> regions;
		int				    i, n;

		synchronized (map)
		{
			regions = getMapRegions();
			n = regions.size();

			for (i = 0; i < n; i++)
			{
				region = regions.elementAt(i);
				if (id.equals(region.getId()))
					return region;
			}
		}

		return null;
	} // end method getMapRegion

	/**
	 * Update the text representing the filter.
	 */
	protected void updateFilterText()
	{
		LyFormField filterField = getFormField(FILTER_EXPRESSION);

		// Filter form may not be displayed
		if (filterField == null)
			return;

		LyText text = (LyText)filterField.getFieldItem();
		String label = ((_filter == null)
				? null : LyFilterEditorController.getNameFilter(_filter, getEnvironment(), false));

		text.setLabel(label);
	}

	/**
	 * Updates the tool representing a color.
	 *
	 * @param fieldId the identifier of the field containing the color
	 * @param color   the new color of the tool
	 */
	protected void updateColorTool(String fieldId, String color)
	{
		LyFormField colorField = getFormField(fieldId);

		// Colors form may not be displayed
		if (colorField == null)
			return;

		LyToolBar toolbar = colorField.getToolBar();

		toolbar.setColor(color);
	}

	/**
	 * Update the graphical list of threshold colors.
	 */
	protected void updateThresholdColorList()
	{
		LyFormField colorsField = getFormField(THRESHOLD_COLORS);

		// Color form may not be displayed
		if (colorsField == null)
			return;

		LyArea   area = (LyArea)colorsField.getFieldItem();
		LyTable  table = (LyTable)area.getPanel(0);
		Float    value;
		String[] colors;

		// First empty the table
		LyRows   rows = table.getRows();
		LyRow    row;
		LyCell   cell;
		int		 i, n = rows.getRowCount();

		for (i = n - 1; i >= 0; i--)
		{
			row = rows.getRow(i);
			rows.removeRow(i);
		}

		// Then rebuild all the rows
		if ((_thresholdColors != null) && (_thresholdColors.size() > 0))
		{
			String				 image = getImageFromPrefix("LY_ACTION_DELETE",
					DEFAULT_IMAGE_FORMAT, 12);
			Vector<Float>		 values = new Vector<Float>();
			LyEnvironment		 env = getEnvironment();
			DecimalFormatSymbols format = new DecimalFormatSymbols(env.getLocale());
			char				 decimalSeparator = format.getDecimalSeparator();
			Enumeration<Float>   keys = _thresholdColors.keys();

			// Always keep the values sorted
			while (keys.hasMoreElements())
			{
				value = keys.nextElement();
				values.addElement(value);
			}

			Collections.sort(values);

			n = values.size();

			for (i = 0; i < n; i++)
			{
				value = values.elementAt(i);
				colors = _thresholdColors.get(value);
				row = new LyRow();
				row.setId(THRESHOLD_COLORS + ID_SEP + value.toString());

				// Add a cell for value
				cell = new LyCell();
				cell.setLabel(value.toString().replace('.', decimalSeparator));
				row.addCell(cell);

				// Add a cell for color before
				cell = new LyCell();
				cell.setLabel(colors[0]);
				row.addCell(cell);

				// Add a cell for color after
				cell = new LyCell();
				cell.setLabel(colors[1]);
				row.addCell(cell);

				row.setImage(image);

				if (value.equals(_selectedThresholdValue))
					row.setSelected(true);

				rows.addRow(row);
			}
		}

		// Add an empty row
		row = new LyRow();
		row.setId(THRESHOLD_COLORS + ID_SEP);
		cell = new LyCell();
		cell.setLabel("");
		row.addCell(cell);
		cell = new LyCell();
		cell.setLabel("");
		row.addCell(cell);
		cell = new LyCell();
		cell.setLabel("");
		row.addCell(cell);
		rows.addRow(row);
	} // end method updateThresholdColorList

	/**
	 * Select a row in the table of threshold colors.
	 *
	 * @param value the threshold value of the row that must be selected
	 */
	protected void selectThresholdColor(Float value)
	{
		LyFormField colorsField = getFormField(THRESHOLD_COLORS);

		// Color form may not be displayed
		if (colorsField == null)
			return;

		LyArea  area = (LyArea)colorsField.getFieldItem();
		LyTable table = (LyTable)area.getPanel(0);
		LyRows  rows = table.getRows();
		LyRow   row;
		int     i, n = rows.getRowCount();
		String  rowId, valueString = ((value == null) ? "" : value.toString());

		for (i = 0; i < n; i++)
		{
			row = rows.getRow(i);
			rowId = row.getId();

			if (rowId.equals(THRESHOLD_COLORS + ID_SEP + valueString))
				row.setSelected(true);
			else
				row.setSelected(false);
		}
	}

	/**
	 * Update the graphical components used to set or create a threshold color.
	 */
	protected void updateThresholdColor()
	{
		// Retrieve selected threshold color
		String[]    colors = ((_selectedThresholdValue == null)
				? null : (String[])_thresholdColors.get(_selectedThresholdValue));
		String	    colorBefore = ((colors == null) ? null : colors[0]);
		String	    colorAfter = ((colors == null) ? null : colors[1]);
		LyFormField valueField = getFormField(THRESHOLD_COLOR_VALUE);
		LyFormField colorBeforeField = getFormField(THRESHOLD_COLOR_BEFORE);
		LyFormField colorAfterField = getFormField(THRESHOLD_COLOR_AFTER);
		LyNumber    number = (LyNumber)valueField.getFieldItem();
		LyRelation  color = (LyRelation)colorBeforeField.getFieldItem();
		LyLink	    link;
		int		    i, n = color.getLinkCount();
		boolean     selected;

		// Update the value component
		number.setLabel(((_selectedThresholdValue == null) ? ""
														   : _selectedThresholdValue.toString()));

		// Update the color before component
		for (i = 0; i < n; i++)
		{
			link = color.getLink(i);

			if (colorBefore == null)
				selected = (i == 0);
			else
				selected = colorBefore.equals(link.getId());

			link.setSelected(selected);

			if (selected)
				updateColorTool(THRESHOLD_COLOR_BEFORE, link.getId());
		}

		// Update the color after component
		color = (LyRelation)colorAfterField.getFieldItem();
		n = color.getLinkCount();

		for (i = 0; i < n; i++)
		{
			link = color.getLink(i);

			if (colorAfter == null)
				selected = (i == 0);
			else
				selected = colorAfter.equals(link.getId());

			link.setSelected(selected);

			if (selected)
				updateColorTool(THRESHOLD_COLOR_AFTER, link.getId());
		}
	} // end method updateThresholdColor

	/**
	 * Set the color in the list of threshold colors with the values specified by the graphical
	 * components. If there is no color at given index, a new color is created.
	 */
	protected void setThresholdColor()
	{
		// Retrieve value, color before and color after
		LyFormField valueField = getFormField(THRESHOLD_COLOR_VALUE);
		LyNumber    valueItem = (LyNumber)valueField.getFieldItem();

		if ((valueItem.getLabel() == null) || (valueItem.getLabel().length() == 0))
		{
			// No value has been selected
			showError("LY_TREEMAP_THRESHOLD_COLOR_NOTNULL");
			return;
		}

		// Retrieve the decimal separator
		LyEnvironment		 environment = getEnvironment();
		DecimalFormatSymbols format = new DecimalFormatSymbols(environment.getLocale());
		char				 decimalSeparator = format.getDecimalSeparator();
		String				 valueItemLabel = valueItem.getLabel();

		valueItemLabel = valueItemLabel.replace(decimalSeparator, '.');

		Float	    value = new Float(valueItemLabel);
		LyFormField colorBeforeField = getFormField(THRESHOLD_COLOR_BEFORE);
		LyRelation  colorBeforeItem = (LyRelation)colorBeforeField.getFieldItem();
		LyFormField colorAfterField = getFormField(THRESHOLD_COLOR_AFTER);
		LyRelation  colorAfterItem = (LyRelation)colorAfterField.getFieldItem();
		LyLink	    link;
		int		    i, n = colorBeforeItem.getLinkCount();
		String	    colorBefore = null, colorAfter = null;

		for (i = 0; i < n; i++)
		{
			link = colorBeforeItem.getLink(i);

			if (link.isSelected())
			{
				colorBefore = link.getId();
				break;
			}
		}

		n = colorAfterItem.getLinkCount();

		for (i = 0; i < n; i++)
		{
			link = colorAfterItem.getLink(i);

			if (link.isSelected())
			{
				colorAfter = link.getId();
				break;
			}
		}

		// Remove existing color
		if (_selectedThresholdValue != null)
			_thresholdColors.remove(_selectedThresholdValue);

		// Create a new color
		if (_thresholdColors == null)
			_thresholdColors = new Hashtable<Float, String[]>();

		_thresholdColors.put(value, new String[] { colorBefore, colorAfter });
		_selectedThresholdValue = value;
		updateThresholdColorList();
		updateMapRegionsColor();
	} // end method setThresholdColor

	/**
	 * Update the map regions of the map with the treemap nodes hierarchy. This method does not
	 * compute the hierarchy but only reset the regions (colors, sizes...).
	 */
	protected void buildMapRegions()
	{
		if (_component == null)
			return;

		LyMap map = getMapComponent();

		synchronized (map)
		{
			// Retrieve selected map regions
			LyMapRegion		    region;
			Vector<LyMapRegion> regions;
			Vector<String>	    selectedRegions = new Vector<String>();
			String			    regionId;
			int				    i, n;

			regions = getMapRegions();
			n = regions.size();

			for (i = 0; i < n; i++)
			{
				region = regions.elementAt(i);
				if (region.isSelected())
					selectedRegions.addElement(region.getId());
			}

			boolean idle = isIdle();

			map.setIdle(true);
			clearMapObjects();

			if ((_workSpace != null) && (_workSpace.getSize() > 0))
			{
				// Build the root region with the visible root node and compute the groups
				LyMapRegion rootRegion = buildMapRegion(_visibleRootNode);
				int		    levels = getHierarchyLevels();

				if (rootRegion != null)
				{
					n = rootRegion.getMapRegionCount();

					computeHierarchyGroups(rootRegion, levels);
					updateColorValues();
					updateMapRegionsColor();

					if (n > 0)
					{
						for (i = n - 1; i >= 0; i--)
						{
							region = rootRegion.getMapRegion(i);
							map.addMapRegion(region);
						}
					}
					else
						map.addMapRegion(rootRegion);
				}

				// Set the title of the map
				LyTreemapBehaviorInterface behavior = (LyTreemapBehaviorInterface)getBehavior();
				String[][]				   labels;
				_LyTreeMapNode			   node = _visibleRootNode;
				Vector<_LyTreeMapNode>     nodes = new Vector<_LyTreeMapNode>();

				while (node != _rootNode)
				{
					nodes.addElement(node);
					node = node._parent;
				}

				n = nodes.size();
				labels = new String[n][2];

				for (i = 0; i < n; i++)
				{
					node = nodes.elementAt(i);
					labels[n - 1 - i][0] = ((node._fieldInfo != null) ? node._fieldInfo.getName()
																	  : getClassInfo().getName());
					labels[n - 1 - i][1] = behavior.getMapRegionLabel(this, node._fieldInfo,
							node._object);
				}

				map.setLabel(behavior.getTreemapTitle(this, labels));

				// Restore selection
				n = selectedRegions.size();

				for (i = 0; i < n; i++)
				{
					regionId = selectedRegions.elementAt(i);
					selectMapRegion(regionId, true);
				}

				if (n > 0)
					updateTools();
			}

			// Idle mode
			map.setIdle(idle);
			map.resetContent();
		}

		refreshSearchList();
		updateMessages();
	} // end method buildMapRegions

	/**
	 * Recursively computes the groups in the hierarchy of the treemap. This method removes from the
	 * map regions that should not appear since they are placed in a hierarchy group.
	 *
	 * @param region      the root region that must be used for computation
	 * @param levelOffset the level offset between current region level in the hierarchy and the
	 *                    number of levels that should be displayed
	 */
	protected void computeHierarchyGroups(LyMapRegion region, int levelOffset)
	{
		if (_cancelDemand)
			return;

		LyTreemapBehaviorInterface behavior = (LyTreemapBehaviorInterface)getBehavior();
		_LyTreeMapNode			   node = _displayedMapRegions.get(region);

		// Update map region (label, tooltip, small image...)
		if (node != null)
		{
			String label = behavior.getMapRegionLabel(this, node._fieldInfo, node._object);
			String smallImage = behavior.getMapRegionSmallImage(this, node._fieldInfo,
					node._object);
			String smallImageTooltip = behavior.getMapRegionSmallImageTooltip(this, node._fieldInfo,
					node._object);

			region.setLabel(label);
			region.setSmallImage(smallImage);
			region.setSmallImageTooltip(smallImageTooltip);
		}

		if ((region == null) || (region.getMapRegionCount() == 0))
			return;

		LyMapRegion subRegion;
		int		    i, n = region.getMapRegionCount();

		if (levelOffset > 0)
		{
			// If there are sub-regions, remove this region from the internal list
			// of displayed regions (no need to compute its color) and recurse
			_displayedMapRegions.remove(region);

			for (i = 0; i < n; i++)
			{
				subRegion = region.getMapRegion(i);
				computeHierarchyGroups(subRegion, levelOffset - 1);
			}
		}
		else
		{
			// If this region is a leaf in the hierarchy, remove its sub-regions
			for (i = n - 1; i >= 0; i--)
			{
				subRegion = region.getMapRegion(i);
				region.removeMapRegion(subRegion);
				removeDisplayedMapRegion(subRegion);
			}
		}
	} // end method computeHierarchyGroups

	/**
	 * Remove a map region from the internal list of displayed map regions. This method also removes
	 * recursively of the sub-regions of the given region.
	 *
	 * @param region the region that should be removed from the list
	 */
	protected void removeDisplayedMapRegion(LyMapRegion region)
	{
		_displayedMapRegions.remove(region);

		int		    i, n = region.getMapRegionCount();
		LyMapRegion subRegion;

		for (i = 0; i < n; i++)
		{
			subRegion = region.getMapRegion(i);
			removeDisplayedMapRegion(subRegion);
		}
	}

	/**
	 * Empty the map from all map objects.
	 */
	public void clearMapObjects()
	{
		if (_component == null)
			return;

		synchronized (getMapComponent())
		{
			LyMap	    map = getMapComponent();
			int		    i, n = map.getComponentCount();
			LyComponent mapObject;

			map.resetContent();

			// Remove every regions
			for (i = n - 1; i >= 0; i--)
			{
				mapObject = map.getComponent(i);

				if (mapObject instanceof LyMapRegion)
					map.removeMapRegion((LyMapRegion)mapObject);
			}
		}

		// Clear list of displayed map regions
		if (_displayedMapRegions != null)
			_displayedMapRegions.clear();

		// Clear list of selected objects
		if (_selections != null)
			_selections.reset();

		updateTools();
		updateMessages();
	} // end method clearMapObjects

	/**
	 * Compute the treemap hierarchy according to the fields and the number of levels that must be
	 * used for the hierarchy.
	 */
	protected void computeTreemapNodes()
	{
		LySort				    sort = new LySort();
		LyInfoList<LyFieldInfo> fields = getHierarchyFields();
		LyFieldInfo			    field;
		LyObject			    object, prevObject = null;
		LyValue				    value, prevValue;
		_LyTreeMapNode		    node, parentNode;
		int					    i, j, k, kStart, kEnd;
		int					    nFields = fields.getSize(), mObjects = _workSpace.getSize(),
								currentLevel = 0;

		// Build a sort with the hierarchy fields
		for (i = 0; i < nFields; i++)
		{
			field = fields.getInfo(i);
			sort.addCriterion(field, true);
		}

		_workSpace.setSort(sort);
		_rootNode = new _LyTreeMapNode(null, null);
		_visibleRootNode = _rootNode;
		parentNode = _rootNode;

		synchronized (_workSpace)
		{
			for (i = 0; i < mObjects; i++)
			{
				if (_cancelDemand)
					return;

				object = _workSpace.getObject(i);

				// Retrieve the first different value between object and previousObject
				if (prevObject != null)
				{
					for (j = 0; j < nFields; j++)
					{
						field = fields.getInfo(j);
						value = object.getFieldValue(field);
						prevValue = prevObject.getFieldValue(field);

						if (((value != null) && (prevValue == null))
								|| ((value == null) && (prevValue != null))
								|| ((value != null) && !(value.equals(prevValue))))
							break;
					}
				}
				else
					j = nFields;

				if (j > currentLevel)
				{
					kStart = currentLevel;
					kEnd = j;
				}
				else if (j < currentLevel)
				{
					for (k = j; k < currentLevel; k++)
						parentNode = parentNode._parent;

					kStart = j;
					kEnd = currentLevel;
				}
				else
				{
					kStart = currentLevel;
					kEnd = currentLevel;
				}

				for (k = kStart; k < kEnd; k++)
				{
					field = fields.getInfo(k);
					node = new _LyTreeMapNode(field, object);
					parentNode.addSon(node);
					node._id = computeId(node);
					parentNode = node;
				}

				node = new _LyTreeMapNode(null, object);
				parentNode.addSon(node);
				node._id = computeId(node);
				prevObject = object;
				currentLevel = nFields;
			}
		}
	} // end method computeTreemapNodes

	/**
	 * Compute the identifier for a given tree map node of the hierarchy.
	 *
	 * @param  node the node whose identifier must be computed
	 * @return the identifier of the tree map node
	 */
	protected String computeId(_LyTreeMapNode node)
	{
		Vector<_LyTreeMapNode> sons = node._sons;
		int                   n = ((sons == null) ? 0 : sons.size());
		String                id = null;

		if ((n == 0) && (node._fieldInfo == null) && (node._object != null))
			id = node._object.getId();
		else if (node._object != null)
		{
			Object value = node._object.getValue(node._fieldInfo);

			id = node._fieldInfo.getId() + SEP_HIERARCHY_ID + node._fieldInfo.toString(value);

			if ((node._parent != null) && (node._parent._id != null))
				id = node._parent._id + SEP_ID + id;
		}

		return id;
	}

	/**
	 * Build a map region for a given node of the hierarchy.
	 *
	 * @param  node the node of the hierarchy
	 * @return the map region built for the given node of the hierarchy
	 */
	protected LyMapRegion buildMapRegion(_LyTreeMapNode node)
	{
		if (_cancelDemand)
			return null;

		LyTreemapBehaviorInterface behavior = (LyTreemapBehaviorInterface)getBehavior();
		LyFieldInfo				   sizeField = getSizeField();
		Vector<_LyTreeMapNode>	   sons = node._sons;
		_LyTreeMapNode			   son;
		LyMapRegion				   region = new LyMapRegion(), subRegion;
		LyPolygon				   polygon = new LyPolygon();
		int						   i, n = ((sons == null) ? 0 : sons.size());
		float					   weight = 0;

		// Add a polygon with default values (position will be computed automatically
		// by the treemap layout)
		polygon.addPoint(new LyPoint(0, 0, 0));
		polygon.addPoint(new LyPoint(0, 0, 0));
		polygon.addPoint(new LyPoint(0, 0, 0));
		polygon.addPoint(new LyPoint(0, 0, 0));
		region.addPolygon(polygon);

		// If region has no child, compute its weight
		if ((n == 0) && (node._fieldInfo == null) && (node._object != null))
		{
			Number number = behavior.computeSizeValue(this, sizeField, node._object);

			weight = ((number == null) ? 1 : number.floatValue());
		}

		region.setId(computeId(node));

		// Recursively build map regions for the hierarchy
		for (i = 0; i < n; i++)
		{
			son = sons.elementAt(i);
			subRegion = buildMapRegion(son);

			if (subRegion != null)
			{
				region.addMapRegion(subRegion);
				weight += subRegion.getWeight();
			}
		}

		region.setWeight(Math.max(weight, 0));

		if (_displayedMapRegions == null)
			_displayedMapRegions = new Hashtable<LyMapRegion, _LyTreeMapNode>();

		// Map region is not displayed for root node
		if ((node != _visibleRootNode) || (_visibleRootNode._sons == null))
			_displayedMapRegions.put(region, node);

		return region;
	} // end method buildMapRegion

	/**
	 * Update color of displayed map regions. Only "final" map regions (ie map regions that have no
	 * sub-map regions) will have their color computed.
	 */
	protected void updateMapRegionsColor()
	{
		if (_displayedMapRegions == null)
			return;

		LyTreemapBehaviorInterface     behavior = (LyTreemapBehaviorInterface)getBehavior();
		LyEnvironment				   env = getEnvironment();
		Enumeration<LyMapRegion>	   regions = _displayedMapRegions.keys();
		LyMapRegion					   region;
		LyPolygon					   polygon;
		_LyTreeMapNode				   node;
		LyObject					   object;
		Number						   number;
		float						   fValue = 0;
		Hashtable<LyMapRegion, Number> regionValues = new Hashtable<LyMapRegion, Number>();
		LySimpleObjectList			   objects;
		LyFieldInfo					   colorField = getColorField();
		String						   minColorValue = getMinColor();
		String						   maxColorValue = getMaxColor();
		LyColor						   minColor = env.getLyColor(minColorValue);
		LyColor						   maxColor = env.getLyColor(maxColorValue);
		String						   color;
		String						   tooltip;

		// Retrieve max and min values used for colors
		if (colorField != null)
		{
			while (regions.hasMoreElements())
			{
				region = regions.nextElement();
				node = _displayedMapRegions.get(region);
				object = node._object;
				objects = new LySimpleObjectList();
				objects.setSorted(false);

				// If the node has a field info, it is not a leaf node of the hierarchy :
				// retrieve all the objects that it contains to compute its color
				if (node._fieldInfo != null)
					node.computeSubObjects(objects);
				// If the node is a leaf, only its object is used to compute its color
				else if (object != null)
					objects.addObject(node._object);

				// Remove from the list, objects that do not match the filter if filter is
				// applied on colors
				if (_filterActive && _filterShowAll)
				{
					int i, n = objects.getSize();

					for (i = n - 1; i >= 0; i--)
					{
						object = objects.getObject(i);

						if ((_filter != null) && !_filter.match(object))
							objects.removeObject(object);
					}
				}

				if (objects.getSize() > 0)
					number = behavior.computeColorValue(this, colorField, objects,
							(node._fieldInfo != null));
				else
					number = null;

				if (number != null)
					regionValues.put(region, number);
			}
		}

		regions = _displayedMapRegions.keys();

		while (regions.hasMoreElements())
		{
			region = regions.nextElement();
			node = _displayedMapRegions.get(region);
			polygon = region.getPolygon(0);
			number = regionValues.get(region);

			// If there is no field for color, use default color
			if ((colorField == null) || (number == null))
				polygon.setColor(null);
			else if (number != null)
			{
				fValue = number.floatValue();
				color = behavior.computeColor(this, fValue, _minColorValue, _maxColorValue,
						minColor, maxColor);
				polygon.setColor(color);
			}

			// Update tooltip for current region
			tooltip = behavior.getMapRegionTooltip(this, node._fieldInfo, node._object,
					region.getWeight(), fValue);
			region.setTooltip(tooltip);
		}
	} // end method updateMapRegionsColor

	/**
	 * Return the list of exception colors registered for this action.
	 *
	 * @return the list of exception colors registered for this action or null if no exception color
	 *         has been defined.
	 */
	public Hashtable<Float, String> getExceptionColors()
	{
		return _exceptionColors;
	}

	/**
	 * Return the list of threshold colors registered for this action.
	 *
	 * @return the list of threshold colors registered for this action or null if no threshold color
	 *         has been defined.
	 */
	public Hashtable<Float, String[]> getThresholdColors()
	{
		return _thresholdColors;
	}

	/**
	 * Select or unselect a map region.
	 *
	 * @param id     the id of the map region that must be selected or unselected. If this id is
	 *               null, all the map regions area selected or unselected.
	 * @param select indicates whether the map region must be selected or unselected
	 */
	protected void selectMapRegion(String id, boolean select)
	{
		LyMapRegion		    region = null;
		Vector<LyMapRegion> regions;

		regions = getMapRegions();
		;

		int i, n = regions.size();

		if (id == null)
		{
			// Select or unselect all the map regions
			for (i = 0; i < n; i++)
			{
				region = regions.elementAt(i);
				if (region.isSelected() != select)
					region.setSelected(select);
			}

			if (select)
			{
				_selections = new LySimpleObjectList(_workSpace);
				_selections.setSorted(false);
			}
			else
				_selections = null;
		}
		else
		{
			// Select or unselect the map region of given id
			boolean objectRegion = id.startsWith(getClassInfo().getId() + LyObjectId.CLASS_SEP);

			for (i = 0; i < n; i++)
			{
				region = regions.elementAt(i);
				if (id.equals(region.getId()))
					break;
			}

			if ((region == null) || (i == n))
				return;

			if (region.isSelected() != select)
				region.setSelected(select);

			// If selected region corresponds to a real object of current class info,
			// add it to selection.
			if (objectRegion)
			{
				LyObjectId oid = getObjectId(id);
				LyObject   object = _workSpace.getObject(oid);

				if (object != null)
				{
					if (select)
					{
						if (_selections == null)
						{
							_selections = new LySimpleObjectList();
							_selections.setSorted(false);
						}

						// If object was not previously selected, update selection
						if (!_selections.contains(object))
							_selections.addObject(object);
					}
					else
					{
						// If object was previously selected, update selection
						if ((_selections != null) && _selections.contains(object))
							_selections.removeObject(object);
					}
				}
			}
		}
	} // end method selectMapRegion

	/**
	 * updates the status of Tool and menu items.
	 */
	protected void updateTools()
	{
		if (_component == null)
			return;

		// If table is Idle, no need to refresh
		if (isIdle())
			return;

		LyViewManager viewManager = getViewManager();
		boolean		  notifySelect = viewManager.hasProperty("NOTIFY_SELECT");
		boolean		  hasMenu = viewManager.hasProperty("MENU");

		// Update ToolBar
		LyToolBar     toolBar = (LyToolBar)_component.getComponent("_toolBar");

		if (toolBar == null)
			return;

		// get all the actions of the toolbar
		LyAction			 rootAction = getAction();
		LyInfoList<LyAction> actions = new LyInfoList<LyAction>();

		if (rootAction != null)
			actions.addList(rootAction.getActions()); // add sub-actions of the root action

		LyClassInfo classInfo = getClassInfo();

		if (classInfo.getActionCount() > 0)
			actions.addList(classInfo.getActions()); // add actions of the class info

		// Update toolbar actions
		LyAction    action;
		LyTool	    tool = null;
		boolean     enabled;
		int		    i, n = actions.getSize();
		LyComponent comp;

		for (i = 0; i < n; i++)
		{
			action = actions.getInfo(i);
			enabled = enableAction(action);

			// action tool ?
			comp = toolBar.getComponent(action.getId());

			if ((comp != null) && (comp instanceof LyTool))
			{
				tool = (LyTool)comp;
				tool.setEnabled(enabled);
			}
		}

		// Update zoom in / out tools
		if (!notifySelect)
		{
			// Retrieve selected regions
			boolean			    zoomIn = false;
			Vector<LyMapRegion> selectedRegions = getMapRegions();
			LyMapRegion		    mapRegion;

			n = selectedRegions.size();
			for (i = n - 1; i >= 0; i--)
			{
				mapRegion = selectedRegions.elementAt(i);
				if (!mapRegion.isSelected())
					selectedRegions.removeElementAt(i);
			}

			if (selectedRegions.size() == 1)
			{
				LyMapRegion    region = selectedRegions.elementAt(0);
				_LyTreeMapNode node = _displayedMapRegions.get(region);
				_LyTreeMapNode nodeTmp = node;

				while ((nodeTmp != null) && (nodeTmp._parent != _visibleRootNode))
					nodeTmp = nodeTmp._parent;

				if ((nodeTmp != null) && (nodeTmp._sons != null) && (nodeTmp._sons.size() > 0))
					zoomIn = true;
			}

			tool = (LyTool)toolBar.getComponent(DEMAND_ZOOM_IN);
			
			if (tool != null)
				tool.setEnabled(zoomIn);
			
			tool = (LyTool)toolBar.getComponent(DEMAND_ZOOM_OUT);
			
			if (tool != null)
				tool.setEnabled((_visibleRootNode != _rootNode));
			
			tool = (LyTool)toolBar.getComponent(DEMAND_ZOOM_TOP);
			
			if (tool != null)
				tool.setEnabled((_visibleRootNode != _rootNode));
		}

		// Update show / hide tools
		if (useGlobalConfiguration())
		{
			tool = (LyTool)toolBar.getComponent(DEMAND_SHOW_CONFIGURATION);
			
			if (tool != null)
				tool.setEnabled(!_configurationFormShown);

			tool = (LyTool)toolBar.getComponent(DEMAND_HIDE_CONFIGURATION);
			
			if (tool != null)
				tool.setEnabled(_configurationFormShown);
		}

		// Update tools for managing configurations
		if (!hasMenu)
		{
			LyInfoList<LyTreemapConfiguration> configurations = getCurrentConfigurations();

			n = ((configurations == null) ? 0 : configurations.getSize());

			tool = (LyTool)toolBar.getComponent(DEMAND_DELETE_CONFIGURATION);
			
			if (tool != null)
				tool.setEnabled(n > 0);
			
			tool = (LyTool)toolBar.getComponent(DEMAND_LOAD_CONFIGURATION);
			
			if (tool != null)
				tool.setEnabled(n > 0);
		}

		// Update play / pause tools
		tool = (LyTool)toolBar.getComponent(DEMAND_PLAY);
		
		if (tool != null)
			tool.setEnabled(_pause);
		
		tool = (LyTool)toolBar.getComponent(DEMAND_PAUSE);
		
		if (tool != null)
			tool.setEnabled(!_pause);
	} // end method updateTools

	/**
	 * Update the configuration menu.
	 */
	public void updateConfigurationMenu()
	{
		// If treemap is Idle, no need to refresh
		if (isIdle())
			return;
		
		LyArea	    area = (LyArea)getComponent();
		LyTitleBar  titleBar = area.getTitleBar();
		LyToolBar   rightToolBar;
		LyFormField prefsField = null;

		if (titleBar != null)
		{
			rightToolBar = titleBar.getToolBar();
			if (rightToolBar != null)
				prefsField = (LyFormField)rightToolBar.getComponent(getId() + "_configurationsField");
		}

		if (prefsField == null)
		{
			rightToolBar = area.getTopRightToolBar();
			if (rightToolBar != null)
				prefsField = (LyFormField)rightToolBar.getComponent(getId() + "_configurationsField");
		}

		if (prefsField == null)
			return;

		LyInfoList<LyTreemapConfiguration> configurations = getCurrentConfigurations();
		int		n = ((configurations == null) ? 0 : configurations.getSize());

		int						 i;
		String					 id, label;
		LyTreemapConfiguration	 configuration;
		LyRelation				 relation = (LyRelation)prefsField.getFieldItem();
		LyLink					 link;

		prefsField.setHidden(n == 0);

		if (_configuration == null)
		{
			link = (LyLink)relation.getComponent("LY_NO_CHOICE");
			link.setSelected(true);
		}
		
		for (i = 0; i < n; i++)
		{
			configuration = configurations.getInfo(i);
			id = configuration.getId();
			label = configuration.getName();
			link = (LyLink)relation.getComponent(id);
			
			if (link == null)
			{
				link = new LyLink(id, label);
				relation.addLink(link);
			}

			link.setSelected(configuration == _configuration);
		}
		
		LyViewManager viewManager = getViewManager();
		boolean		  hasMenu = viewManager.hasProperty("MENU");
		LyView		  view = ((_component == null) ? null : _component.getParentView());

		if (_initialized || (view == null))
			view = getView();

		LyMenuBar menuBar = ((view == null) ? null : (LyMenuBar)view.getComponent("_menuBar"));

		if (!hasMenu || (menuBar == null))
			return;

		menuBar.resetChildrenCache();
		
		LyItem deleteItem = (LyItem)menuBar.getComponent(DEMAND_DELETE_CONFIGURATION + getId());

		if (deleteItem != null)
			deleteItem.setEnabled(n > 0);
	} // end method updateConfigurationMenu

	/**
	 * Return the list of visible configurations.
	 *
	 * @return the list of visible configurations
	 */
	protected LyInfoList<LyTreemapConfiguration> getCurrentConfigurations()
	{
		LyApplication					   application = getApplication();
		LyInfoList<LyTreemapConfiguration> configurations = application.getInfoList(LyTreemapConfiguration.class);
		LyInfoList<LyTreemapConfiguration> result = new LyInfoList<LyTreemapConfiguration>();
		LyTreemapConfiguration			   configuration;
		String							   userId;
		String							   currentUserId = ((_currentUser == null)
				? "" : _currentUser.getId());
		int								   i, n;

		n = ((configurations == null) ? 0 : configurations.getSize());

		// Sort by name
		configurations = ((configurations == null) ? null : configurations.getInfosByName());

		for (i = 0; i < n; i++)
		{
			configuration = configurations.getInfo(i);
			userId = configuration.getUser();

			if (userId == null)
				userId = "";

			// If configuration is not applied on current action, do not add it.
			if (!configuration.getObjectInfo().getId().equals(_action.getId())
					&& !configuration.getObjectInfo().getId().equals(_classInfo.getId()))
				continue;

			// If configuration was saved automatically, do not display it
			if (configuration.getName().startsWith(LAST_CONFIGURATION_PREFIX))
				continue;

			// If current user does not own the configuration do not show it
			if (!userId.equals("") && (!currentUserId.equals(userId)))
				continue;

			result.addInfo(configuration);
		}

		return result;
	} // end method getCurrentConfigurations

	/**
	 * Indicates is end-user may choose hierarchy fields.
	 * @param fieldId  the id of the field to display
	 *
	 * @return true if end-user may choose hierarchy fields, false otherwise
	 */
	public boolean displayField(String fieldId)
	{
		if ((fieldId == null) || !(fieldId.startsWith("_")))
			return false;

		fieldId = "_choose" + fieldId.substring(1, 2).toUpperCase() + fieldId.substring(2);

		String parameter = (String)getParameter(fieldId);

		return ("true".equalsIgnoreCase(parameter));
	}

	/**
	 * Returns the list of fields that may be used to compute hierarchy of the treemap.
	 *
	 * @return the list of fields
	 */
	public LyInfoList<LyFieldInfo> getHierarchyFieldList()
	{
		LyTreemapBehaviorInterface behavior = (LyTreemapBehaviorInterface)getBehavior();

		return behavior.getHierarchyFieldList(this);
	}

	/**
	 * Returns the list of fields used to compute hierarchy of the treemap.
	 *
	 * @return the list of fields
	 */
	public LyInfoList<LyFieldInfo> getHierarchyFields()
	{
		if (_hierarchyFields != null)
			return _hierarchyFields;

		LyTreemapBehaviorInterface behavior = (LyTreemapBehaviorInterface)getBehavior();

		_hierarchyFields = behavior.getHierarchyFields(this);

		return _hierarchyFields;
	}

	/**
	 * Returns the number of levels used in the hierarchy.
	 *
	 * @return the number of levels used in the hierarchy
	 */
	public int getHierarchyLevels()
	{
		if (_hierarchyLevels > 0)
			return _hierarchyLevels;

		LyTreemapBehaviorInterface behavior = (LyTreemapBehaviorInterface)getBehavior();

		_hierarchyLevels = behavior.getHierarchyLevels(this);

		return _hierarchyLevels;
	}

	/**
	 * Returns the list of fields that may be used to compute size of the treemap elements.
	 *
	 * @return the list of fields
	 */
	public LyInfoList<LyFieldInfo> getSizeFieldList()
	{
		LyTreemapBehaviorInterface behavior = (LyTreemapBehaviorInterface)getBehavior();

		return behavior.getSizeFieldList(this);
	}

	/**
	 * Returns the field used to compute size of the treemap elements.
	 *
	 * @return the field used to compute size of the treemap elements
	 */
	public LyFieldInfo getSizeField()
	{
		if (_sizeFieldSet)
			return _sizeField;

		LyTreemapBehaviorInterface behavior = (LyTreemapBehaviorInterface)getBehavior();

		_sizeField = behavior.getSizeField(this);

		return _sizeField;
	}

	/**
	 * Returns the list of fields that may be used to compute color of the treemap elements.
	 *
	 * @return the list of fields
	 */
	public LyInfoList<LyFieldInfo> getColorFieldList()
	{
		LyTreemapBehaviorInterface behavior = (LyTreemapBehaviorInterface)getBehavior();

		return behavior.getColorFieldList(this);
	}

	/**
	 * Returns the field used to compute color of the treemap elements.
	 *
	 * @return the field used to compute color of the treemap elements
	 */
	public LyFieldInfo getColorField()
	{
		if (_colorFieldSet)
			return _colorField;

		LyTreemapBehaviorInterface behavior = (LyTreemapBehaviorInterface)getBehavior();

		_colorField = behavior.getColorField(this);

		return _colorField;
	}

	/**
	 * Returns the minimum color code used to compute color of the treemap elements.
	 *
	 * @return the minimum color code used to compute color of the treemap elements
	 */
	public String getMinColor()
	{
		if (_minColor != null)
			return _minColor;

		LyTreemapBehaviorInterface behavior = (LyTreemapBehaviorInterface)getBehavior();

		_minColor = behavior.getMinColor(this);

		return _minColor;
	}

	/**
	 * Returns the maximum color code used to compute color of the treemap elements.
	 *
	 * @return the maximum color code used to compute color of the treemap elements
	 */
	public String getMaxColor()
	{
		if (_maxColor != null)
			return _maxColor;

		LyTreemapBehaviorInterface behavior = (LyTreemapBehaviorInterface)getBehavior();

		_maxColor = behavior.getMaxColor(this);

		return _maxColor;
	}

	/**
	 * Returns the list of aggregation types that may be used to compute color of the treemap
	 * elements.
	 *
	 * @return the list of aggregation type ids and labels
	 */
	public String[][] getAggregationTypes()
	{
		LyTreemapBehaviorInterface behavior = (LyTreemapBehaviorInterface)getBehavior();

		return behavior.getAggregationTypes(this);
	}

	/**
	 * Returns the aggregation type id used to compute color of the treemap elements.
	 *
	 * @param  ignoreState indicates if state of the component must be taken into account
	 * @return the aggregation type id used to compute color of the treemap elements
	 */
	public String getAggregationType(boolean ignoreState)
	{
		// If there is no group, aggregation type is disabled and aggregation type
		// should not be taken into account.
		int					    hierarchyLevels = getHierarchyLevels();
		LyInfoList<LyFieldInfo> hierarchyFields = getHierarchyFields();
		boolean				    groups = (hierarchyLevels <= hierarchyFields.getSize());

		if (!groups)
			return null;

		if (_aggregationType != null)
			return _aggregationType;

		LyTreemapBehaviorInterface behavior = (LyTreemapBehaviorInterface)getBehavior();

		_aggregationType = behavior.getAggregationType(this);

		return _aggregationType;
	}

	/**
	 * Returns the list of fields that may be used by balanced aggregation.
	 *
	 * @return the list of fields
	 */
	public LyInfoList<LyFieldInfo> getAggregationFieldList()
	{
		LyTreemapBehaviorInterface behavior = (LyTreemapBehaviorInterface)getBehavior();

		return behavior.getAggregationFieldList(this);
	}

	/**
	 * Returns the field used by balanced aggregation.
	 *
	 * @return the field used by balanced aggregation
	 */
	public LyFieldInfo getAggregationField()
	{
		if (_aggregationField != null)
			return _aggregationField;

		LyTreemapBehaviorInterface behavior = (LyTreemapBehaviorInterface)getBehavior();

		_aggregationField = behavior.getAggregationField(this);

		return _aggregationField;
	}

	/**
	 * Saving current configuration.
	 */
	protected void saveConfiguration()
	{
		// Create a dialog to type in the search parameters
		LySession			  session = getSession();
		LyUserInputController prefCtrl;
		LyApplication		  application = session.getApplication();
		LyAction			  action = application.getActionWithId("__saveTreemapConfiguration", getClassInfo());
		String				  title = null;

		if ((_configuration != null)
				&& (_configuration.getName() != null)
				&& (!_configuration.getName().startsWith(LAST_CONFIGURATION_PREFIX)))
			title = _configuration.getName();

		prefCtrl = new LyUserInputController(session, this, action);
		
		_LySaveConfigurationBehavior	behavior = new _LySaveConfigurationBehavior(prefCtrl);
		
		prefCtrl.setBehavior(behavior);
		behavior.init(prefCtrl);
		prefCtrl.setBuilder(new _LyConfigurationInputBuilder(prefCtrl));
		prefCtrl.setTitleBar("LY_ITEM_SAVE_CONFIGURATION");
		prefCtrl.setImage(getImage(_action));

		prefCtrl.addText(DEMAND_SAVE_CONFIGURATION, "LY_TREEMAP_CONFIGURATION_NAME", title, 40,
			null, false);

		prefCtrl.showView();

		// Waiting for userInput event.
	}

	/**
	 * Delete configurations.
	 */
	protected void deleteConfigurations()
	{
		// Create a dialog to type in the search parameters
		LySession			  session = getSession();
		LyUserInputController confCtrl;
		LyApplication		  application = session.getApplication();
		LyAction			  action = application.getActionWithId("__deleteTreemapConfiguration", getClassInfo());

		confCtrl = new LyUserInputController(session, this, action);
		
		_LyDeleteConfigurationBehavior	behavior = new _LyDeleteConfigurationBehavior();
		
		confCtrl.setBehavior(behavior);
		behavior.init(confCtrl);
		confCtrl.setBuilder(new _LyConfigurationInputBuilder(confCtrl));
		confCtrl.setTitleBar("LY_ACTION_DELETE");
		confCtrl.setImage(getImage(_action));

		LyInfoList<LyTreemapConfiguration> confs = getCurrentConfigurations();
		LyTreemapConfiguration			   conf;
		int								   i, n = ((confs == null) ? 0 : confs.getSize());
		LyOption						   option;
		LyChoice						   choice = confCtrl.addChoice(DEMAND_DELETE_CONFIGURATION,
				null, true, true);

		choice.setFill(LyChoice.Fill.VERTICAL);
		choice.setMaxSpan(6);

		if (n == 0)
			return;

		for (i = 0; i < n; i++)
		{
			conf = confs.getInfo(i);
			option = new LyOption(conf.getId(), conf.getName());
			choice.addOption(option);
		}

		confCtrl.showView();
		// Waiting for userInput event.
	} // end method deleteConfigurations

	/**
	 * Deleting configuration files.
	 *
	 * @param configurations the identifiers of the configurations
	 */
	protected void deleteConfigurationFile(String configurations)
	{
		LyArea		  area = (LyArea)getComponent();
		LyTitleBar    titleBar = area.getTitleBar();
		LyToolBar     rightToolBar;
		LyFormField   prefsField = null;

		LySession			   session = getSession();
		LyApplication		   application = session.getApplication();
		String[]			   ids = LyString.stringToList(configurations, LyTools.OBJECT_SEP);
		String				   prefId;
		LyTreemapConfiguration configuration;
		int					   i, n = ids.length;

		if (titleBar != null)
		{
			rightToolBar = titleBar.getToolBar();
			if (rightToolBar != null)
				prefsField = (LyFormField)rightToolBar.getComponent(getId() + "_configurationsField");
		}

		if (prefsField == null)
		{
			rightToolBar = area.getTopRightToolBar();
			if (rightToolBar != null)
				prefsField = (LyFormField)rightToolBar.getComponent(getId() + "_configurationsField");
		}

		if (prefsField != null)
		{
			LyRelation relation = (LyRelation)prefsField.getFieldItem();
			LyLink     link;

			for (i = 0; i < n; i++)
			{
				prefId = ids[i];
				configuration = (LyTreemapConfiguration)application.getInfo(prefId);

				if (configuration != null)
				{
					// Reset current configuration if deleted
					if (configuration == _configuration)
						_configuration = null;

                    if (application.deleteExtraInfo(configuration))
					{
						// Remove link in configuration field
						link = (LyLink)relation.getComponent(prefId);
						relation.removeLink(link);
					}
				}
			}
		}

		// Update configuration menu
		updateConfigurationMenu();
		
		// Update configuration menu
		updateConfigurationMenu();
		updateTools();
	} // end method deleteConfigurationFile

	/**
	 * Activate or deactivate the filter.
	 *
	 * @param active        indicates whether filter is active or not
	 * @param showAll       indicates whether all the objects must be shown or not
	 * @param updateContent indicates whether a content update must be performed or not
	 */
	protected void activateFilter(boolean active, boolean showAll, boolean updateContent)
	{
		boolean onlyUpdateColors = ((_filterShowAll == showAll) && (showAll == true));

		_filterActive = active;
		_filterShowAll = showAll;

		if (_initialWorkSpace == null) // not initialized
			return;

		// Update filter type state
		LyFormField field = getFormField(FILTER_ACTIVE);

		// Filter form may not be displayed
		if (field != null)
		{
			LyChoice choice = (LyChoice)field.getFieldItem();
			LyOption option = choice.getOption(0);

			option.setChecked(_filterActive);
			field.setEnabled(_filter != null);

			field = getFormField(FILTER_TYPE);
			choice = (LyChoice)field.getFieldItem();
			option = choice.getOption(0);
			option.setChecked(_filterShowAll);
			option = choice.getOption(1);
			option.setChecked(!_filterShowAll);
			field.setEnabled((_filter != null) && _filterActive);
		}

		if (active && !_filterShowAll)
		{
			_workSpace.removeListener(this);

			if (_workSpace == _initialWorkSpace)
				_workSpace = new LySubWorkSpace(getSession(), _initialWorkSpace, (LyFilter)null, true);

			((LySubWorkSpace)_workSpace).setFilter(_filter);
			_workSpace.addListener(this);
		}
		else
		{
			LyWorkSpace temp = _workSpace;
			boolean     free = (_workSpace != _initialWorkSpace);

			_workSpace = _initialWorkSpace;
			_workSpace.addListener(this);

			if (free)
				temp.free(this);

			temp = null;
		}

		if (updateContent)
		{
			if (onlyUpdateColors)
			{
				LyMap   map = getMapComponent();
				boolean idle = isIdle();

				map.setIdle(true);
				updateMapRegionsColor();
				map.setIdle(idle);
			}
			else
				updateContent(false);
		}

		updateMessages();
	} // end method activateFilter

	/**
	 * Implementation of LyFilterEndInterface.
	 * @param parent the parent controller
	 * @param actionDemand the demand
	 * @param filter  the filter to be used
	 */
	public void endFilter(LyController parent, String actionDemand, LyFilter filter)
	{
		if (filter == null)
			return;

		_filter = filter;
		activateFilter(_filterActive, _filterShowAll, _filterActive);
		updateFilterText();
	}

	/**
	 * Implementation of LyFilterEndInterface.
	 * @return the current filter
	 */
	public LyFilter getFilter()
	{
		return _filter;
	}

	/**
	 * Sets the configuration for this controller.
	 * @param configuration the configuration to set
	 * @param reset true to reset the controller
	 */
	public void setConfiguration(LyTreemapConfiguration configuration, boolean reset)
	{
		if (configuration == null)
			return;

		// Update filter
		LyFilter filter = configuration.getFilter();

		_filter = ((filter == null) ? null : filter.cloneFilter());

		activateFilter(configuration.isFilterActive(), configuration.isFilterShowAll(), false);

		// Update hierarchy fields and levels
		LyInfoList<LyFieldInfo> hierarchyFields = configuration.getHierarchyFieldInfos();

		if (hierarchyFields == null)
			_hierarchyFields = new LyInfoList<LyFieldInfo>();
		else
			_hierarchyFields = new LyInfoList<LyFieldInfo>(hierarchyFields);

		_hierarchyLevels = configuration.getHierarchyLevels();

		// Update size and color fields and components
		_sizeField = configuration.getSizeFieldInfo();
		_colorField = configuration.getColorFieldInfo();
		_sizeFieldSet = true;
		_colorFieldSet = true;

		selectLink(SIZE_FIELD, ((_sizeField == null) ? null : _sizeField.getId()));
		selectLink(COLOR_FIELD, ((_colorField == null) ? null : _colorField.getId()));

		// Update aggregation type and field
        LyTreemapConfiguration.Aggregation aggregationType = configuration.getAggregationType();

		if (aggregationType == LyTreemapConfiguration.Aggregation.MINIMUM)
			_aggregationType = MINIMUM_AGGREGATION;
		else if (aggregationType == LyTreemapConfiguration.Aggregation.MAXIMUM)
			_aggregationType = MAXIMUM_AGGREGATION;
		else if (aggregationType == LyTreemapConfiguration.Aggregation.AVERAGE)
			_aggregationType = AVERAGE_AGGREGATION;
		else
			_aggregationType = BALANCED_AGGREGATION;

		_aggregationField = configuration.getAggregationFieldInfo();

		selectLink(AGGREGATION_TYPE, _aggregationType);
		selectLink(AGGREGATION_FIELD,
			((_aggregationField == null) ? null : _aggregationField.getId()));

		// Update min and max colors
		_minColor = configuration.getMinColor();
		_maxColor = configuration.getMaxColor();

		selectLink(MIN_COLOR, _minColor);
		selectLink(MAX_COLOR, _maxColor);
		updateColorTool(MIN_COLOR, _minColor);
		updateColorTool(MAX_COLOR, _maxColor);

		// Update threshold color list
		String[] thresholdColors = configuration.getThresholdColors();
		int		 i, n = ((thresholdColors == null) ? 0 : thresholdColors.length);
		String[] thresholdColor;

		if (_thresholdColors != null)
			_thresholdColors.clear();

		_thresholdColors = null;

		for (i = 0; i < n; i++)
		{
			thresholdColor = LyString.stringToList(thresholdColors[i], '#');

			if (_thresholdColors == null)
				_thresholdColors = new Hashtable<Float, String[]>(n);

			_thresholdColors.put(new Float(thresholdColor[0]),
				new String[] { thresholdColor[1], thresholdColor[2] });
		}

		// Update border sizes and label styles
		setBorderSizes(configuration.getBorderSizes());
		setLabelStyles(configuration.getLabelStyles());

		// Reset
		_configuration = configuration;

		// If global configuration form is used, check if it is displayed.
		// If not displayed, form must be updated next time it will be shown.
		if (useGlobalConfiguration())
		{
			LyComponent component = getComponent();
			
                        if (component != null)
                        { 
                            LyComponent form = component.getComponent("_treemapForm");

                            _updateConfigurationForm = (form == null);
                        }
		}

		if (reset)
			updateContent(false);
	} // end method setConfiguration

	/**
	 * Set the border sizes of the treemap.
	 *
	 * @param borderSizes the new border sizes
	 */
	public void setBorderSizes(String borderSizes)
	{
		LyMap    map = getMapComponent();
		
                if (map != null)
                {
                    LyLayout layout = map.getLayout();

                    if (layout != null)
                            layout.setBorderSizes(borderSizes);
                }
		_borderSizes = borderSizes;
	}

	/**
	 * Return the border sizes of the treemap.
	 *
	 * @return the border sizes of the treemap
	 */
	public String getBorderSizes()
	{
		return _borderSizes;
	}

	/**
	 * Set the label styles of the treemap.
	 *
	 * @param labelStyles the new label styles
	 */
	public void setLabelStyles(String labelStyles)
	{
		LyMap    map = getMapComponent();
	     
                if (map != null)
                {
                    LyLayout layout = map.getLayout();

                    if (layout != null)
                            layout.setLabelStyles(labelStyles);
                }
		_labelStyles = labelStyles;
	}

	/**
	 * Return the label styles of the treemap.
	 *
	 * @return the label styles of the treemap
	 */
	public String getLabelStyles()
	{
		return _labelStyles;
	}

	/**
	 * Select a link in a relation.
	 *
	 * @param relationId     the identifier of the form field containing the relation
	 * @param selectedLinkId the identifier of the link that must be selected within the relation
	 */
	protected void selectLink(String relationId, String selectedLinkId)
	{
		LyLink	    link;
		String	    linkId;
		boolean     selected;
		LyFormField field = getFormField(relationId);

		if (field != null)
		{
			LyRelation relation = (LyRelation)field.getFieldItem();
			int		   i, n = relation.getLinkCount();

			for (i = 0; i < n; i++)
			{
				link = relation.getLink(i);
				linkId = link.getId();
				selected = ((selectedLinkId != null) && (selectedLinkId.equals(linkId)));

				if (link.isSelected() != selected)
				{
					relation.selectLink(i, selected);
					link.setSelected(selected);
				}
			}
		}
	}

	/**
	 * Invoked when an action occurs. (LyDemandListener interface)
	 *
	 * @param demand action event created from the view. User interaction.
	 */
	public void actionDemand(LyActionDemand demand)
	{
		String source = demand.getSource();
		String value = demand.getValue();

		if (demand.getType() == LyActionDemand.TYPE_ACTION)
		{
			if (DEMAND_HIDE_CONFIGURATION.equals(source))
			{
				showConfigurationForm(false);
				setLastModified(new Date().getTime());
				return;
			}
			else if (DEMAND_SHOW_CONFIGURATION.equals(source))
			{
				showConfigurationForm(true);
				setLastModified(new Date().getTime());
				return;
			}
			else if (DEMAND_SHOW_HIERARCHY_FORM.equals(source))
			{
				LyController controller = getHierarchyFormController();

				controller.showView();
				updateHierarchyFields();
				setLastModified(new Date().getTime());
				return;
			}
			else if (DEMAND_SHOW_REPRESENTATION_FORM.equals(source))
			{
				LyController controller = getRepresentationFormController();

				controller.showView();
				selectLink(SIZE_FIELD, ((_sizeField == null) ? null : _sizeField.getId()));
				selectLink(COLOR_FIELD, ((_colorField == null) ? null : _colorField.getId()));
				selectLink(AGGREGATION_TYPE, _aggregationType);
				selectLink(AGGREGATION_FIELD,
					((_aggregationField == null) ? null : _aggregationField.getId()));
				updateAggregationFieldState();
				setLastModified(new Date().getTime());
				return;
			}
			else if (DEMAND_SHOW_FILTER_FORM.equals(source))
			{
				LyController controller = getFilterFormController();

				controller.showView();
				activateFilter(_filterActive, _filterShowAll, false);
				updateFilterText();
				setLastModified(new Date().getTime());
				return;
			}
			else if (DEMAND_SHOW_COLORS_FORM.equals(source))
			{
				LyController controller = getColorsFormController();

				controller.showView();
				selectLink(MIN_COLOR, _minColor);
				selectLink(MAX_COLOR, _maxColor);
				updateColorTool(MIN_COLOR, _minColor);
				updateColorTool(MAX_COLOR, _maxColor);
				updateThresholdColorList();
				setLastModified(new Date().getTime());
				return;
			}
			else if (DEMAND_PAUSE.equals(source))
			{
				pause(true);
				setLastModified(new Date().getTime());
				return;
			}
			else if (DEMAND_PLAY.equals(source))
			{
				pause(false);
				setLastModified(new Date().getTime());
				return;
			}
			else if (DEMAND_SAVE_CONFIGURATION.equals(source))
			{
				saveConfiguration();
				setLastModified(new Date().getTime());
				return;
			}
			else if ((DEMAND_DELETE_CONFIGURATION + getId()).equals(source))
			{
				deleteConfigurations();
				setLastModified(new Date().getTime());
				return;
			}
			else if ((source.equals("_map") && (value != null))
					|| source.startsWith(DEMAND_ZOOM_IN))
			{
				if (source.startsWith(DEMAND_ZOOM_IN))
				{
					int pos = source.indexOf(ID_SEP);

					value = source.substring(pos + ID_SEP.length());
				}

				// Default action on a region : go one step deeper in the hierarchy
				LyMapRegion region = getMapRegion(value);

				if (region == null)
				{
					Vector<LyMapRegion> selectedRegions = getMapRegions();
					LyMapRegion		    mapRegion;

					int				    i, n = selectedRegions.size();

					for (i = n - 1; i >= 0; i--)
					{
						mapRegion = selectedRegions.elementAt(i);
						if (!mapRegion.isSelected())
							selectedRegions.removeElementAt(i);
					}

					if (selectedRegions.size() > 0)
						region = selectedRegions.elementAt(0);
				}

				if (region != null)
				{
					_LyTreeMapNode node = _displayedMapRegions.get(region);
					_LyTreeMapNode nodeTmp = node;

					while ((nodeTmp != null) && (nodeTmp._parent != _visibleRootNode))
						nodeTmp = nodeTmp._parent;

					// If node has no son, do nothing
					if ((nodeTmp != null) && (nodeTmp._sons != null) && (nodeTmp._sons.size() > 0))
					{
						_visibleRootNode = nodeTmp;
						selectMapRegion(null, false);
						buildMapRegions();
					}

					setLastModified(new Date().getTime());
				}

				return;
			}
			else if ((source.equals("_map") && (value == null))
					|| DEMAND_ZOOM_OUT.equals(source))
			{
				// Default action on the map : go one step upper in the hierarchy
				if (_visibleRootNode != _rootNode)
					_visibleRootNode = _visibleRootNode._parent;

				selectMapRegion(null, false);
				buildMapRegions();
				setLastModified(new Date().getTime());
				return;
			}
			else if (DEMAND_ZOOM_TOP.equals(source))
			{
				// Go to the top of the hierarchy
				if (_visibleRootNode != _rootNode)
					_visibleRootNode = _rootNode;

				selectMapRegion(null, false);
				buildMapRegions();
				setLastModified(new Date().getTime());
				return;
			}
			else if (source.equals(getId() + "_filterMessage"))
			{
				if (_filter == null)
					buildFilter();
				else
				{
					activateFilter(!_filterActive, _filterShowAll, true);
					setLastModified(new Date().getTime());
				}
				return;
			}
		}
		else if (demand.getType() == LyActionDemand.TYPE_VALUE_CHANGED)
		{
			if ((source != null)
					&& source.equals(getId() + "_configurationsField"))
			{
				LySession     session = getSession();
				LyApplication application = session.getApplication();
				
				LyTreemapConfiguration conf = (LyTreemapConfiguration)application.getInfo(value);

				setConfiguration(conf, true);
			}
		}

		super.actionDemand(demand);
	} // end method actionDemand

	/**
	 * Build a filter controller to create a filter.
	 */
	protected void buildFilter()
	{
		// Create a new filter controller
		String		 actionId = "__treemapFilter";
		LySession    session = getSession();
		LyController filterController;

		if (_filter != null)
			actionId = actionId + "_set";
		else
			actionId = actionId + "_new";

		if (_filterCtrlId != null)
		{
			// try to find the corresponding controller
			filterController = session.getController(_filterCtrlId);

			if (filterController != null)
			{
				filterController.showView();
				return;
			}
		}

		LyApplication application = session.getApplication();
		LyAction	  action = application.getActionWithId(actionId, getClassInfo());

		filterController = invokeAction(action, _classInfo);

		if (filterController != null)
			_filterCtrlId = filterController.getId();
	} // end method buildFilter

	/**
	 * Invoked when a selection occurs. (LyDemandListener interface)
	 *
	 * @param demand selection event created from the view. User interaction.
	 */
	public void selectionDemand(LySelectionDemand demand)
	{
		if (_component == null)
			return;

		LyMap					   map = getMapComponent();
		boolean					   idle = isIdle();
		boolean					   extend = demand.isExtend();
		boolean					   select = false;
		int						   selectionType = demand.getType();

		LyTreemapBehaviorInterface behavior = (LyTreemapBehaviorInterface)getBehavior();

		if (behavior.refreshViewOnSelection(this))
			map.setIdle(true);

		if ((!extend) || (selectionType == LySelectionDemand.TYPE_UNSELECT_ALL))
			selectMapRegion(null, false);

		// If a user selection occurs, deselect all map regions that are not final
		LyMapRegion		    region = null;
		Vector<LyMapRegion> regions = getMapRegions();
		int				    i, n = regions.size();

		for (i = 0; i < n; i++)
		{
			region = regions.elementAt(i);
			if (region.isSelected())
				if (region.getMapRegionCount() > 0)
					region.setSelected(false);
		}

		if ((selectionType == LySelectionDemand.TYPE_SELECT)
				|| (selectionType == LySelectionDemand.TYPE_UNSELECT))
		{
			n = demand.getSelectionSize();

			if (selectionType == LySelectionDemand.TYPE_SELECT)
				select = true;
			else if (selectionType == LySelectionDemand.TYPE_UNSELECT)
				select = false;

			if (n != 0)
			{
				String     key;
				LyObjectId oid;
				LyObject   object;

				for (i = 0; i < n; i++)
				{
					key = demand.getSelectionId(i);
					object = null;

					if ((key != null)
						&& key.startsWith(getClassInfo().getId() + LyObjectId.CLASS_SEP))
					{
						oid = getObjectId(key);
						object = _workSpace.getObject(oid);

						if (object == null)
						{
							// Display a message (the object was deleted) except if
							// selection on a hierarchy region (SEP_ID not in the key).
							if ((n == 1) && (key != null) && (key.indexOf(SEP_ID) < 0))
							{
								LyEnvironment env = getEnvironment();
								String		  message = env.getMessage("LY_ER_OBJECT_NOT_IN_VIEW");

								getViewManager().showError(getMapComponent(), message);

								// Refresh map
								updateContent(false);

								break;
							}
							else
								continue;
						}
					}

					selectMapRegion(key, select);
				}
			}
		}

		refreshSearchList();
		updateContextClient();
		updateTools();
		updateMessages();

		setLastModified(new Date().getTime());

		if (behavior.refreshViewOnSelection(this))
			map.setIdle(idle);
	} // end method selectionDemand

	/**
	 * Invoked when a menu action occurs.
	 *
	 * @param demand menu demand
	 */
	public void menuDemand(LyMenuDemand demand)
	{
		if (demand == null)
			return;

		if (demand.getSource().equals("_map"))
		{
			if (_component == null)
				return;

			// Find target object, or null if popup on 'background'
			String	    key = demand.getItem();
			boolean     onObject = true, zoomOut = false;
			LyMapRegion zoomInRegion = null;
			LyAction    action = getAction();

			if (action.hasMark(LyAction.MARK_NO_MENU))
				return;

			// Check if menu demand is on region that represents a real object
			if ((key == null) || (key.indexOf(LyObjectId.CLASS_SEP) < 0))
				onObject = false;

			// Check if zoom in and zoom out may be performed
			zoomOut = (_visibleRootNode != _rootNode);

			if (key != null)
			{
				LyMapRegion    region = getMapRegion(key);
				_LyTreeMapNode node = _displayedMapRegions.get(region);
				_LyTreeMapNode nodeTmp = node;

				while ((nodeTmp != null) && (nodeTmp._parent != _visibleRootNode))
					nodeTmp = nodeTmp._parent;

				if ((nodeTmp != null) && (nodeTmp._sons != null) && (nodeTmp._sons.size() > 0))
					zoomInRegion = getMapRegion(nodeTmp._id);
			}

			LyPopupMenu popupMenu = new LyPopupMenu();
			LyClassInfo classInfo = getClassInfo();
			LySession   session = getSession();
			LyItem	    item;
			int		    i, n = classInfo.getActionCount();
			boolean     enabled;
			String	    image;

			if (zoomInRegion != null)
			{
				item = new LyItem(DEMAND_ZOOM_IN + ID_SEP + getMapRegion(key).getId(),
						"LY_TREEMAP_ZOOM_IN" + " : " + zoomInRegion.getLabel());
				item.setController(getId());
				popupMenu.addItem(item);
			}

			if (zoomOut)
			{
				item = new LyItem(DEMAND_ZOOM_OUT, "LY_TREEMAP_ZOOM_OUT");
				item.setController(getId());
				popupMenu.addItem(item);

				item = new LyItem(DEMAND_ZOOM_TOP, "LY_TREEMAP_ZOOM_TOP");
				item.setController(getId());
				popupMenu.addItem(item);
			}

			if (((zoomInRegion != null) || (zoomOut)) && onObject)
				popupMenu.addSep();

			if (onObject)
			{
				for (i = 0; i < n; i++)
				{
					action = classInfo.getAction(i);

					// Skip actions which should not be here
					if (action.hasMark(LyAction.MARK_HIDDEN))
						continue;
					if (!session.validateAction(action, this))
						continue;

					// If clic on selection, the action must enable selected items
					if (action.getMax() == 0)
						continue;

					enabled = session.enableAction(action, this, _selections, false);
					image = getMenuItemImage(action);

					item = new LyItem(action.getId(), action.getName());
					item.setController(getId());
					item.setEnabled(enabled);
					item.setImage(image);

					// Add in Popup menu
					popupMenu.addItem(item);
				}
			}

			// Coordinates
			int[] p = demand.getPosition();

			popupMenu.setX(p[0]);
			popupMenu.setY(p[1]);

			LyMap map = getMapComponent();

			if (map == null)
				// An error occured, just return
				return;

			popupMenu.setController(getId());
			popupMenu.setEnvironment(map.getEnvironment());
			map.setPopupMenu(popupMenu);
		}
		else
			super.menuDemand(demand);
	} // end method menuDemand

	/**
	 * Return the controller of the hierarchy form.
	 *
	 * @return the controller of the hierarchy form (create it if necessary)
	 */
	public LyAreaController getHierarchyFormController()
	{
		LySession			  session = getSession();
		LyUserInputController hierarchyCtrl;

		if (_hierarchyCtrlId != null)
		{
			// try to find the controller with this ID
			hierarchyCtrl = (LyUserInputController)session.getController(_hierarchyCtrlId);

			if (hierarchyCtrl != null)
				return hierarchyCtrl;
			else
				_hierarchyCtrlId = null;
		}

		LyApplication application = session.getApplication();
		LyAction	  action = application.getActionWithId("__treemapHierarchy", getClassInfo());

		hierarchyCtrl = new LyUserInputController(getSession(), this, action);
		
		_LyHierarchyFormBehavior	behavior = new _LyHierarchyFormBehavior();
		
		hierarchyCtrl.setBehavior(behavior);
		behavior.init(hierarchyCtrl);
		_hierarchyCtrlId = hierarchyCtrl.getId();

		LyTreemapBuilder builder = (LyTreemapBuilder)getBuilder();
		LyBuilder		 hierarchyBuilder = builder.getHierarchyFormBuilder(hierarchyCtrl);

		hierarchyCtrl.setBuilder(hierarchyBuilder);

		return hierarchyCtrl;
	} // end method getHierarchyFormController

	/**
	 * Return the controller of the representation form.
	 *
	 * @return the controller of the representation form (create it if necessary)
	 */
	public LyAreaController getRepresentationFormController()
	{
		LySession			  session = getSession();
		LyUserInputController representationCtrl;

		if (_representationCtrlId != null)
		{
			// try to find the controller with this ID
			representationCtrl = (LyUserInputController)session.getController(
					_representationCtrlId);

			if (representationCtrl != null)
				return representationCtrl;
			else
				_representationCtrlId = null;
		}

		LyApplication application = session.getApplication();
		LyAction	  action = application.getActionWithId("__treemapRepresentation", getClassInfo());

		representationCtrl = new LyUserInputController(getSession(), this, action);
		
		_LyRepresentationFormBehavior	behavior = new _LyRepresentationFormBehavior();
		
		representationCtrl.setBehavior(behavior);
		behavior.init(representationCtrl);
		_representationCtrlId = representationCtrl.getId();

		LyTreemapBuilder builder = (LyTreemapBuilder)getBuilder();
		LyBuilder		 representationBuilder = builder.getRepresentationFormBuilder(
				representationCtrl);

		representationCtrl.setBuilder(representationBuilder);

		return representationCtrl;
	} // end method getRepresentationFormController

	/**
	 * Return the controller of the filter form.
	 *
	 * @return the controller of the filter form (create it if necessary)
	 */
	public LyAreaController getFilterFormController()
	{
		LySession			  session = getSession();
		LyUserInputController filterCtrl;

		if (_filterSearchCtrlId != null)
		{
			// try to find the controller with this ID
			filterCtrl = (LyUserInputController)session.getController(_filterSearchCtrlId);

			if (filterCtrl != null)
				return filterCtrl;
			else
				_filterSearchCtrlId = null;
		}

		LyApplication application = session.getApplication();
		LyAction	  action = application.getActionWithId("__treemapFilter", getClassInfo());

		filterCtrl = new LyUserInputController(getSession(), this, action);
		
		_LyFilterFormBehavior	behavior = new _LyFilterFormBehavior();
		
		filterCtrl.setBehavior(behavior);
		behavior.init(filterCtrl);
		_filterSearchCtrlId = filterCtrl.getId();

		LyTreemapBuilder builder = (LyTreemapBuilder)getBuilder();
		LyBuilder		 filterBuilder = builder.getFilterFormBuilder(filterCtrl);

		filterCtrl.setBuilder(filterBuilder);

		return filterCtrl;
	} // end method getFilterFormController

	/**
	 * Return the controller of the colors form.
	 *
	 * @return the controller of the colors form (create it if necessary)
	 */
	public LyAreaController getColorsFormController()
	{
		LySession			  session = getSession();
		LyUserInputController colorsCtrl;

		if (_colorsCtrlId != null)
		{
			// try to find the controller with this ID
			colorsCtrl = (LyUserInputController)session.getController(_colorsCtrlId);

			if (colorsCtrl != null)
				return colorsCtrl;
			else
				_colorsCtrlId = null;
		}

		LyApplication application = session.getApplication();
		LyAction	  action = application.getActionWithId("__treemapColors", getClassInfo());

		colorsCtrl = new LyUserInputController(getSession(), this, action);
		
		_LyColorsFormBehavior	behavior = new _LyColorsFormBehavior();
		
		colorsCtrl.setBehavior(behavior);
		behavior.init(colorsCtrl);
		_colorsCtrlId = colorsCtrl.getId();

		LyTreemapBuilder builder = (LyTreemapBuilder)getBuilder();
		LyBuilder		 colorsBuilder = builder.getColorsFormBuilder(colorsCtrl);

		colorsCtrl.setBuilder(colorsBuilder);

		return colorsCtrl;
	} // end method getColorsFormController

	/**
	 * Return the controller of the update colors form.
	 *
	 * @return the controller of the update colors form (create it if necessary)
	 */
	public LyAreaController getUpdateColorsFormController()
	{
		return getUpdateColorsFormController(true);
	}

	/**
	 * Return the controller of the update colors form.
	 *
	 * @param  build Indicates if we must built this controller if it doesn't exists.
	 * @return the controller of the update colors form (create it if necessary)
	 */
	public LyAreaController getUpdateColorsFormController(boolean build)
	{
		LySession			  session = getSession();
		LyUserInputController updateColorsCtrl;

		if (_updateColorsCtrlId != null)
		{
			// try to find the controller with this ID
			updateColorsCtrl = (LyUserInputController)session.getController(_updateColorsCtrlId);

			if (updateColorsCtrl != null)
				return updateColorsCtrl;
			else
				_updateColorsCtrlId = null;
		}

		if (!build)
			return null;

		LyApplication application = session.getApplication();
		LyAction	  action = application.getActionWithId("__treemapColors", getClassInfo());

		updateColorsCtrl = new LyUserInputController(getSession(), this, action);
		
		_LyColorsFormBehavior	behavior = new _LyColorsFormBehavior();
		
		updateColorsCtrl.setBehavior(behavior);
		behavior.init(updateColorsCtrl);
		_updateColorsCtrlId = updateColorsCtrl.getId();

		return updateColorsCtrl;
	} // end method getUpdateColorsFormController

	/**
	 * Get content of the treemap.
	 *
	 * @return the list of objects being displayed
	 */
	public LyObjectList getContent()
	{
		return _workSpace;
	}

	/**
	 * Selected objects.
	 *
	 * @return the list of objects currently selected or null if no object is selected
	 */
	public LyObjectList getSelection()
	{
		return ((_selections == null) ? null : new LySimpleObjectList(_selections, true));
	}

	/**
	 * Accessor to panel component.
	 *
	 * @see leon.control.LyPanelController#getPanelComponent()
	 */
	public LyPanel getPanelComponent()
	{
		return getMapComponent();
	}

	/**
	 * Retrieves the view Builder.
	 *
	 * @return the builder instance;
	 */
	protected LyBuilder getDefaultBuilder()
	{
		return new LyTreemapBuilder(this);
	}

	/**
	 * Accessor to the Map component.
	 *
	 * @return the map component
	 */
	public LyMap getMapComponent()
	{
		// Retrieve internal table component
		LyComponent component = getComponent();

		if (component == null)
			return null;

		LyMap map = (LyMap)component.getComponent("_map");

		if (map == null)
			throw new IllegalStateException("BF AppEngine : can't retrieve map for controller " + this);

		return map;
	}

	/**
	 * Gets list of map regions.
	 * @return a list of map regions
	 */
	protected Vector<LyMapRegion> getMapRegions()
	{
		LyMap			    map = getMapComponent();
		Vector<LyMapRegion> regions = new Vector<LyMapRegion>();

		map.getComponents(LyMapRegion.class, regions);
		return regions;
	}

	/* Implementation of LyWorkSpaceListener */

	/**
	 * Called when an object is added in the dynamic object list.
	 *
	 * @param  event event
	 * @return true is demand was successfull.
	 * @see    LyWorkSpaceListener#objectAdded
	 */
	public boolean objectAdded(LyWorkSpaceEvent event)
	{
		if (_pause)
			return true;

		updateContent(false);
		setLastModified(new Date().getTime());

		return true;
	}

	/**
	 * Called when an object has changed in the dynamic object list.
	 *
	 * @param  event event
	 * @return true is demand was successfull.
	 * @see    LyWorkSpaceListener#objectChanged
	 */
	public boolean objectChanged(LyWorkSpaceEvent event)
	{
		if (_pause)
			return true;

		LyValueSet values = event.getValues();

		if (values != null)
		{
			// Check if one of the hierarchy fields has been changed, or size field
			// or color field
			LyInfoList<LyFieldInfo> hierarchyFields = getHierarchyFields();
			LyInfoList<LyFieldInfo> pictoFields = getPictoFields();
			LyInfoList<LyFieldInfo> tooltipFields = getTooltipFields();
			LyFieldInfo			    sizeField = getSizeField();
			LyFieldInfo			    colorField = getColorField();
			boolean				    updateContent = false, updateRegions = false,
									updateColors = false, updateTooltip = false;
			LyMap				    map = getMapComponent();
			boolean				    idle = map.isIdle();

			// Idle mode
			map.setIdle(true);

			int		    i, n = values.getSize();
			LyValue     value;
			LyFieldInfo fieldInfo;

			for (i = 0; i < n; i++)
			{
				value = values.getFieldValue(i);
				fieldInfo = (LyFieldInfo)value.getField();

				if ((hierarchyFields != null) && hierarchyFields.contains(fieldInfo))
					updateContent = true;
				else if ((sizeField == fieldInfo)
						|| ((pictoFields != null) && pictoFields.contains(fieldInfo)))
					updateRegions = true;
				else if (colorField == fieldInfo)
					updateColors = true;
				else if ((tooltipFields != null) && tooltipFields.contains(fieldInfo))
					updateTooltip = true;
			}

			if (updateContent)
				reset();
			else if (updateRegions)
				buildMapRegions();
			else if (updateColors)
			{
				updateColorValues();
				updateMapRegionsColor();
			}
			else if (updateTooltip)
			{
				LyObject    object = event.getObject();
				LyMapRegion region = getMapRegion(object.getId());

				if (region != null)
				{
					LyTreemapBehaviorInterface behavior = (LyTreemapBehaviorInterface)getBehavior();
					String					   tooltip = behavior.getMapRegionTooltip(this, null,
							object, 0, 0);

					region.setTooltip(tooltip);
				}
			}

			map.setIdle(idle);

			// Update the last modification time.
			setLastModified(new Date().getTime());
			setRefreshTime(new Date().getTime());
		}

		return true;
	} // end method objectChanged

	/**
	 * Called when an object is removed from the dynamic object list.
	 *
	 * @param  event event
	 * @return true is demand was successfull.
	 * @see    LyWorkSpaceListener#objectRemoved
	 */
	public boolean objectRemoved(LyWorkSpaceEvent event)
	{
		if (_pause)
			return true;

		updateContent(false);
		setLastModified(new Date().getTime());

		return true;
	}

	/* Internal classes */

	/**
	 * Internal class used to represent the treemap hierarchy.
	 */
	protected class _LyTreeMapNode
	{
		String				   _id;
		_LyTreeMapNode		   _parent;
		Vector<_LyTreeMapNode> _sons;
		LyFieldInfo			   _fieldInfo;
		LyObject			   _object;

		/**
		 * Constructor.
		 *
		 * @param fieldInfo field info associated to the node
		 * @param object    object associated to the node
		 */
		_LyTreeMapNode(LyFieldInfo fieldInfo, LyObject object)
		{
			_id = null;
			_parent = null;
			_fieldInfo = fieldInfo;
			_object = object;
		}

		/**
		 * Add a son to current node.
		 *
		 * @param son the son to be added
		 */
		protected void addSon(_LyTreeMapNode son)
		{
			if (_sons == null)
				_sons = new Vector<_LyTreeMapNode>();

			_sons.addElement(son);
			son._parent = _LyTreeMapNode.this;
		}

		/**
		 * Return the son of given identifier.
		 *
		 * @param  id the identifier
		 * @return the son of given identifier or null if none was found
		 */
		protected _LyTreeMapNode getSon(String id)
		{
			if ((_sons == null) || (_sons.size() == 0) || (id == null))
				return null;

			int			   i, n = _sons.size();
			_LyTreeMapNode son;

			for (i = 0; i < n; i++)
			{
				son = _sons.elementAt(i);

				if (id.equals(son._id))
					return son;
			}

			return null;
		}

		/**
		 * Compute the list of sub-objects for current node. This method calls itself recursively.
		 *
		 * @param objects the list of objects that must be filled with son objects
		 */
		protected void computeSubObjects(LySimpleObjectList objects)
		{
			if (_object != null)
				objects.addObject(_object);

			if ((_sons == null) || (_sons.size() == 0))
				return;

			int			   i, n = _sons.size();
			_LyTreeMapNode son;

			for (i = 0; i < n; i++)
			{
				son = _sons.elementAt(i);
				son.computeSubObjects(objects);
			}
		}
	} // end class _LyTreeMapNode


	/**
	 * Internal class used to implement the behavior of sub-forms.
	 */
	class _LyFormBehavior extends LyUserInputBehavior
	{

		/**
		 * Invoked when the user validates the content of the input Default closes and free the view
		 * if the button is VALIDATE action.
		 *
		 * @param controller the current LyUserInputController instance
		 * @param demand     demand parameters sent by the view component.
		 */
		public void userInputDemand(LyController controller, LyDemand demand)
		{
			if (useGlobalConfiguration() && (demand.getType() == LyActionDemand.TYPE_ACTION))
			{
				// Gets current action (source of the demand)
				String id = demand.getSource();

				// Do not free controller when validating form (may happen when user press
				// enter key or escape)
				if ((id.equals(LyFormController.DEMAND_VALIDATE))
						|| (id.equals(LyFormController.DEMAND_CANCEL)))
					return;
			}

			super.userInputDemand(controller, demand);
		}
	}


	/**
	 * Internal class used to implement the behavior of hierarchy form.
	 */
	class _LyHierarchyFormBehavior extends _LyFormBehavior
	{
		public boolean handleDemand(LyController controller, LyDemand demand)
		{
			if (demand instanceof LyActionDemand)
			{
				LyActionDemand    actionDemand = (LyActionDemand)demand;
				String			  source = actionDemand.getSource();
				LyParameterValues paramValues = actionDemand.getParameterValues();

				if (paramValues != null)
				{
					LyComponent component = controller.getComponent();

					if (useGlobalConfiguration())
						component = getComponent();

					paramValues.updateViewFromValues(component);
					setLastModified(new Date().getTime());
				}

				if (demand.getType() == LyActionDemand.TYPE_VALUE_CHANGED)
				{
					if (HIERARCHY_FIELDS.equals(source))
					{
						LyInfoList<LyFieldInfo> fields = getHierarchyFieldList();

						if (paramValues != null)
						{
							// Update fields list
							String[]    fieldIds = paramValues.getParameterValues(HIERARCHY_FIELDS);
							LyFieldInfo fieldInfo;
							int		    i, n = ((fieldIds == null) ? 0 : fieldIds.length);

							_hierarchyFields = new LyInfoList<LyFieldInfo>();

							for (i = 0; i < n; i++)
							{
								fieldInfo = fields.getInfo(fieldIds[i]);
								_hierarchyFields.addInfo(fieldInfo);
							}

							// Update the hierachy levels with the new hierarchy
							_hierarchyLevels = n + 1;
						}

						// If no real object selected, unselect all selected regions
						// (may be hierarchy regions)
						if ((_selections == null) || (_selections.getSize() == 0))
							selectMapRegion(null, false);

						updateContent(true);

						return true;
					}
					else if (HIERARCHY_LEVELS.equals(source))
					{
						String value = actionDemand.getValue();

						_hierarchyLevels = LyTools.intFromString(value);
						buildMapRegions();

						// If no real object selected, unselect all selected regions
						// (may be hierarchy regions)
						if ((_selections == null) || (_selections.getSize() == 0))
							selectMapRegion(null, false);

						updateAggregationFieldState();
						return true;
					}
				}
			}

			return super.handleDemand(controller, demand);
		} // end method handleDemand
	} // end class _LyHierarchyFormBehavior


	/**
	 * Internal class used to implement the behavior of representation form.
	 */
	class _LyRepresentationFormBehavior extends _LyFormBehavior
	{
		public boolean handleDemand(LyController controller, LyDemand demand)
		{
			if (demand instanceof LyActionDemand)
			{
				LyMap			  map = getMapComponent();
				LyActionDemand    actionDemand = (LyActionDemand)demand;
				String			  source = actionDemand.getSource();
				String			  value = actionDemand.getValue();
				LyParameterValues paramValues = actionDemand.getParameterValues();

				if (paramValues != null)
				{
					LyComponent component = controller.getComponent();

					if (useGlobalConfiguration())
						component = getComponent();

					paramValues.updateViewFromValues(component);
					setLastModified(new Date().getTime());
				}

				if (demand.getType() == LyActionDemand.TYPE_VALUE_CHANGED)
				{
					if (SIZE_FIELD.equals(source))
					{
						LyInfoList<LyFieldInfo> fields = getSizeFieldList();

						_sizeField = fields.getInfo(value);
						_sizeFieldSet = true;
						buildMapRegions();
						return true;
					}
					else if (COLOR_FIELD.equals(source))
					{
						LyInfoList<LyFieldInfo> fields = getColorFieldList();
						boolean				    idle = isIdle();

						_colorField = fields.getInfo(value);
						_colorFieldSet = true;
						map.setIdle(true);
						updateColorValues();
						updateMapRegionsColor();
						map.setIdle(idle);
						map.resetContent();

						return true;
					}
					else if (AGGREGATION_TYPE.equals(source))
					{
						boolean idle = isIdle();

						_aggregationType = value;
						map.setIdle(true);
						updateMapRegionsColor();
						updateAggregationFieldState();
						map.setIdle(idle);
						map.resetContent();

						return true;
					}
					else if (AGGREGATION_FIELD.equals(source))
					{
						LyInfoList<LyFieldInfo> fields = getAggregationFieldList();
						boolean				    idle = isIdle();

						_aggregationField = fields.getInfo(value);
						map.setIdle(true);
						updateMapRegionsColor();
						map.setIdle(idle);
						map.resetContent();

						return true;
					}
				}
			}

			return super.handleDemand(controller, demand);
		} // end method handleDemand
	} // end class _LyRepresentationFormBehavior


	/**
	 * Internal class used to implement the behavior of filter form.
	 */
	class _LyFilterFormBehavior extends _LyFormBehavior
	{
		public boolean handleDemand(LyController controller, LyDemand demand)
		{
			if (demand instanceof LyActionDemand)
			{
				LyActionDemand    actionDemand = (LyActionDemand)demand;
				String			  source = actionDemand.getSource();
				LyParameterValues paramValues = actionDemand.getParameterValues();

				if (paramValues != null)
				{
					LyComponent component = controller.getComponent();

					if (useGlobalConfiguration())
						component = getComponent();

					paramValues.updateViewFromValues(component);
					setLastModified(new Date().getTime());
				}

				if (demand.getType() == LyActionDemand.TYPE_ACTION)
				{
					if (DEMAND_SET_FILTER.equals(source))
					{
						buildFilter();
						return true;
					}
					else if (DEMAND_SET_EXTENDED_FILTER.equals(source))
					{
						// Create a new extended filter controller
						String		 actionId = "__filter_extended";
						LySession    session = getSession();
						LyController filterController;

						if (_filter != null)
							actionId = actionId + "_set";
						else
							actionId = actionId + "_new";

						if (_filterEditorCtrlId != null)
						{
							// try to find the corresponding controller
							filterController = session.getController(_filterEditorCtrlId);

							if (filterController != null)
							{
								filterController.showView();

								return true;
							}
						}

						LyApplication application = session.getApplication();
						LyAction	  action = application.getActionWithId(actionId, _classInfo);

						filterController = LyTreemapController.this.invokeAction(action, _classInfo);

						if (filterController != null)
							_filterEditorCtrlId = filterController.getId();

						return true;
					}
					else if (DEMAND_REFRESH_SEARCH_LIST.equals(source))
					{
						refreshSearchList();

						return true;
					}
				}
				else if (demand.getType() == LyActionDemand.TYPE_VALUE_CHANGED)
				{
					if (SEARCH_TEXT.equals(source))
					{
						refreshSearchList();

						return true;
					}
					else if (SEARCH_LIST.equals(source))
					{
						LyMap map = getMapComponent();

						updateMapRegionsSelection();
						map.resetContent();

						return true;
					}
					else if (FILTER_ACTIVE.equals(source)
							|| FILTER_TYPE.equals(source))
					{
						LyFormField field = getFormField(FILTER_ACTIVE);
						LyChoice    choice = (LyChoice)field.getFieldItem();
						LyOption    option = choice.getOption(0);
						boolean     activate = option.isChecked();
						boolean     showAll;

						field = getFormField(FILTER_TYPE);
						choice = (LyChoice)field.getFieldItem();
						option = choice.getOption(0);
						showAll = option.isChecked();

						activateFilter(activate, showAll, true);
					}
				}
			}

			return super.handleDemand(controller, demand);
		} // end method handleDemand
	} // end class _LyFilterFormBehavior


	/**
	 * Internal class used to implement the behavior of colors form.
	 */
	class _LyColorsFormBehavior extends _LyFormBehavior
		implements LyQuestionAnswerInterface
	{
		public boolean handleDemand(LyController controller, LyDemand demand)
		{
			LyParameterValues paramValues = demand.getParameterValues();

			if (paramValues != null)
			{
				LyComponent component = controller.getComponent();

				if (useGlobalConfiguration())
					component = getComponent();

				paramValues.updateViewFromValues(component);
			}

			setLastModified(new Date().getTime());

			// Selection in the list of threshold colors
			if (demand instanceof LySelectionDemand)
			{
				LySelectionDemand selectionDemand = (LySelectionDemand)demand;

				if (demand.getType() == LySelectionDemand.TYPE_SELECT)
				{
					String id = selectionDemand.getSelectionId();

					if (id == null)
					{
						_selectedThresholdValue = null;
						selectThresholdColor(null);
						return true;
					}

					// Selection in the table of threshold colors
					if (id.startsWith(THRESHOLD_COLORS))
					{
						String[] values = LyString.stringToList(id, ID_SEP);
						String   floatValue = ((values.length > 1) ? values[1] : null);
						Float    value = (((floatValue == null) || (floatValue.length() == 0))
								? null : new Float(floatValue));

						_selectedThresholdValue = value;
						selectThresholdColor(value);
						updateThresholdColor();
						return true;
					}
				}
			}
			else if (demand instanceof LyActionDemand)
			{
				LyMap		   map = getMapComponent();
				LyActionDemand actionDemand = (LyActionDemand)demand;
				String		   source = actionDemand.getSource();
				String		   value = actionDemand.getValue();

				if (demand.getType() == LyActionDemand.TYPE_VALUE_CHANGED)
				{
					if (MIN_COLOR.equals(source))
					{
						boolean idle = isIdle();

						_minColor = value;
						map.setIdle(true);
						updateColorTool(MIN_COLOR, _minColor);
						updateMapRegionsColor();
						map.setIdle(idle);
						map.resetContent();
					}
					else if (MAX_COLOR.equals(source))
					{
						boolean idle = isIdle();

						_maxColor = value;
						map.setIdle(true);
						updateColorTool(MAX_COLOR, _maxColor);
						updateMapRegionsColor();
						map.setIdle(idle);
						map.resetContent();
					}
					else if (THRESHOLD_COLOR_BEFORE.equals(source))
						updateColorTool(THRESHOLD_COLOR_BEFORE, value);
					else if (THRESHOLD_COLOR_AFTER.equals(source))
						updateColorTool(THRESHOLD_COLOR_AFTER, value);

					// Update the last modification time.
					controller.setLastModified(new Date().getTime());
					setLastModified(new Date().getTime());

					return true;
				}
				else if (demand.getType() == LyActionDemand.TYPE_ACTION)
				{
					if (DEMAND_UPDATE_THRESHOLD_COLOR.equals(source))
					{
						boolean idle = isIdle();

						map.setIdle(true);
						setThresholdColor();
						map.setIdle(idle);

						LyAreaController areaController = getColorsFormController();

						areaController.setLastModified(new Date().getTime());
						controller.setLastModified(new Date().getTime());

						return true;
					}
				}
				else if (demand.getType() == LyActionDemand.TYPE_STATUS)
				{
					// Default action in the threshold color table : delete
					String[]	  values = LyString.stringToList(value, ID_SEP);
					Float		  floatValue = ((values.length > 1) ? new Float(values[1]) : null);
					LyEnvironment env = getEnvironment();
					String		  message = env.getMessage("LY_TREEMAP_THRESHOLD_COLOR_CONFIRM_DEL",
							floatValue);

					getViewManager().askQuestion(getComponent(), message, this, floatValue);

					return true;
				}
			}

			return super.handleDemand(controller, demand);
		} // end method handleDemand

		public void questionAnswer(boolean result, Object clientData)
		{
			if (result)
			{
				LyMap   map = getMapComponent();
				Float   value = (Float)clientData;
				boolean idle = isIdle();

				map.setIdle(true);
				_thresholdColors.remove(value);
				updateThresholdColorList();
				updateMapRegionsColor();
				map.setIdle(idle);

				// Update the last modification time.
				setLastModified(new Date().getTime());
			}
		}
	} // end class _LyColorsFormBehavior


	/**
	 * Internal class used to implement the behavior of configuration save.
	 */
	class _LySaveConfigurationBehavior extends LyUserInputBehavior
		implements LyQuestionAnswerInterface
	{
		LyUserInputController _controller;

		public _LySaveConfigurationBehavior()
		{
			this(null);
		}

		public _LySaveConfigurationBehavior(LyUserInputController controller)
		{
			super();

			_controller = controller;
		}

		/**
		 * Invoked when the user validates the content of the input.
		 *
		 * @param controller the current LyUserInputController instance
		 * @param demand     demand parameters sent by the view component.
		 */
		public void userInputDemand(LyController controller, LyDemand demand)
		{
			// Close find dialog
			String id = demand.getSource();

			if (id.equals(LyFormController.DEMAND_VALIDATE))
			{
				LyParameterValues paramValues = demand.getParameterValues();

				if (paramValues != null)
				{
					String configName = paramValues.getParameter(DEMAND_SAVE_CONFIGURATION);

					if ((configName == null) || (configName.length() == 0))
					{
						LyViewManager viewManager = getViewManager();
						LyComponent   root = getBuilder().getComponent();
						String		  message = getEnvironment().getMessage("LY_ER_NOTNULL",
								"LY_TREEMAP_CONFIGURATION_NAME");

						viewManager.showError(root, message);
						return;
					}

					writeConfiguration(configName, true);
				}
			}
			else
				super.userInputDemand(controller, demand);
		} // end method userInputDemand

		/**
		 * Writing a configuration.
		 *
		 * @param name the name of the configuration
		 * @param askQuestion true to ask the user to confirm the save
		 */
		protected void writeConfiguration(String name, boolean askQuestion)
		{
			LySession						   session = getSession();
			LyApplication					   application = session.getApplication();
			LyTreemapConfiguration			   configuration = null;

			// Search if a configuration with the given name already exists
			LyInfoList<LyTreemapConfiguration> configurations = application.getInfoList(LyTreemapConfiguration.class);
			LyTreemapConfiguration			   oldConfiguration = null;
			int								   j,
											   n = ((configurations == null)
						? 0 : configurations.getSize());

			for (j = 0; j < n; j++)
			{
				configuration = configurations.getInfo(j);

				if (configuration.getName().equals(name))
				{
					oldConfiguration = configuration;
					break;
				}
			}

			if (oldConfiguration != null)
			{
				if (askQuestion)
					getViewManager().askQuestion(getView(),
						getEnvironment().getMessage("LY_CONFIRM_TREEMAP_CONFIGURATION_OVERWRITE",
							oldConfiguration.getName()), this, oldConfiguration);
				// Process goes on in questionAnswer
				else
					questionAnswer(true, oldConfiguration);
			}
			else
				questionAnswer(true, name);
		} // end method writeConfiguration

		/**
		 * Answer to a askQuestion request.
		 *
		 * @param result true if the user has clicked on Yes or Ok
		 * @param clientData data specified in the askQuestion method
		 * @see   LyQuestionAnswerInterface
		 */
		public void questionAnswer(boolean result, Object clientData)
		{
			if (!result)
				return;

			LySession			   session = getSession();
			LyApplication		   application = session.getApplication();
			LyFilter			   filter = getFilter();
			LyTreemapConfiguration configuration = null;

			if (clientData instanceof LyTreemapConfiguration)
				configuration = (LyTreemapConfiguration)clientData;

			// retrieve the name of the data directory
			if (configuration == null)
			{
                String       userFullId = (_currentUser == null ? null : _currentUser.getId().toString());
				String		 name = (String)clientData;
				LyObjectInfo objectInfo;

				if (_action.isActionRef())
					objectInfo = _classInfo;
				else
					objectInfo = _action;

				configuration = new LyTreemapConfiguration("_treemap_configuration"
						+ LyString.getRandomId(), objectInfo, name, _hierarchyLevels,
						LyTreemapConfiguration.Aggregation.BALANCED, _filterActive, _filterShowAll,
						_minColor, _maxColor, null, null, null, userFullId);
				configuration.setApplication(application);
			}

			String[] thresholdColors = null;

			if (_thresholdColors != null)
			{
				int				   i, n = ((_thresholdColors == null) ? 0 : _thresholdColors.size());
				Enumeration<Float> keys = _thresholdColors.keys();
				Float			   value;
				String[]		   colors;

				thresholdColors = new String[n];
				i = 0;

				while (keys.hasMoreElements())
				{
					value = keys.nextElement();
					colors = _thresholdColors.get(value);
					thresholdColors[i] = value + "#" + colors[0] + "#" + colors[1];
					i++;
				}
			}

            LyTreemapConfiguration.Aggregation  aggregationType = LyTreemapConfiguration.Aggregation.BALANCED;
			String aggregationTypeValue = getAggregationType(true);

			if (aggregationTypeValue == MINIMUM_AGGREGATION)
				aggregationType = LyTreemapConfiguration.Aggregation.MINIMUM;
			else if (aggregationTypeValue == MAXIMUM_AGGREGATION)
				aggregationType = LyTreemapConfiguration.Aggregation.MAXIMUM;
			else if (aggregationTypeValue == AVERAGE_AGGREGATION)
				aggregationType = LyTreemapConfiguration.Aggregation.AVERAGE;

			configuration.setHierarchyFieldInfos(getHierarchyFields());
			configuration.setHierarchyLevels(getHierarchyLevels());
			configuration.setSizeFieldInfo(getSizeField());
			configuration.setColorFieldInfo(getColorField());
			configuration.setAggregationType(aggregationType);
			configuration.setAggregationFieldInfo(getAggregationField());
			configuration.setFilterActive(_filterActive);
			configuration.setFilterShowAll(_filterShowAll);
			configuration.setMinColor(getMinColor());
			configuration.setMaxColor(getMaxColor());
			configuration.setThresholdColors(thresholdColors);
			configuration.setFilter(filter);
			configuration.setBorderSizes(_borderSizes);
			configuration.setLabelStyles(_labelStyles);

			// Create xml file for the configuration
			 getApplication().saveExtraInfo(configuration);

			setConfiguration(configuration, false);

			// Update configuration menu
			updateConfigurationMenu();
			updateTools();

			if (_controller != null)
				_controller.free();
		} // end method questionAnswer
	} // end class _LySaveConfigurationBehavior


	/**
	 * Internal class used to implement the builder of configuration actions.
	 */
	class _LyConfigurationInputBuilder extends LyUserInputBuilder
	{
		public _LyConfigurationInputBuilder()
		{
			this(null);
		}

		public _LyConfigurationInputBuilder(LyAreaController parent)
		{
			super(parent);
		}

		public LyComponent build()
		{
			LyComponent comp = super.build();

			if (comp != null)
				((LyArea)comp).setScrolling(LyArea.Scrolling.AUTO);
			return comp;
		}
	}


	/**
	 * Internal class used to implement the behavior of configuration deletion.
	 */
	class _LyDeleteConfigurationBehavior extends LyUserInputBehavior
	{

		/**
		 * Invoked when the user validates the content of the input.
		 *
		 * @param controller the current LyUserInputController instance
		 * @param demand     demand parameters sent by the view component.
		 */
		public void userInputDemand(LyController controller, LyDemand demand)
		{
			// Close find dialog
			String id = demand.getSource();

			if (id.equals(LyFormController.DEMAND_VALIDATE))
			{
				LyParameterValues paramValues = demand.getParameterValues();

				if (paramValues != null)
				{
					String configurations = paramValues.getParameter(DEMAND_DELETE_CONFIGURATION);

					if ((configurations != null) && (configurations.length() != 0))
						deleteConfigurationFile(configurations);
				}
			}

			super.userInputDemand(controller, demand);
		}
	} // end class _LyDeleteConfigurationBehavior


	/**
	 * Internal class used to implement the behavior of configuration load.
	 */
	class _LyLoadConfigurationBehavior extends LyUserInputBehavior
	{

		/**
		 * Invoked when the user validates the content of the input.
		 *
		 * @param controller the current LyUserInputController instance
		 * @param demand     demand parameters sent by the view component.
		 */
		public void userInputDemand(LyController controller, LyDemand demand)
		{
			// Close find dialog
			String id = demand.getSource();

			if (id.equals(LyFormController.DEMAND_VALIDATE))
			{
				LyParameterValues paramValues = demand.getParameterValues();

				if (paramValues != null)
				{
					String configurations = paramValues.getParameter(DEMAND_LOAD_CONFIGURATION);

					if ((configurations != null) && (configurations.length() != 0))
					{
						LySession			   session = getSession();
						LyApplication		   application = session.getApplication();
						LyTreemapConfiguration conf = (LyTreemapConfiguration)application.getInfo(
								configurations);

						setConfiguration(conf, true);
					}
				}
			}

			super.userInputDemand(controller, demand);
		}
	} // end class _LyLoadConfigurationBehavior
} // end class LyTreemapController
