/*
 * LyApplicationVersionValidator.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app;

import java.io.*;
import java.net.*;
import leon.misc.*;
import leon.view.*;
import leon.view.setinterface.*;


/**
 * This class attempts to fetch 'current' application version on a website. If the connection
 * failed, or version file is not found : nothing occur. If the connection file is found and the
 * version number is the same as LY_ROOT_VERSION properties : nothing occurs. If the version number
 * in the target file differs from current file a message is shown to the user.<br>
 * Target url is specified in files_xxx.properties LY_APPLICATION_VERSION_VALIDATOR<br>
 * Update message is specified in the message String LY_NEW_APPLICATION_VERSION_AVAILABLE
 *
 * <p>Target file format is<br/>
 * &lt;VALIDATOR&gt;<br/>
 * &lt;VERSION&gt;x.y.z build b&lt;/VERSION&gt;<br/>
 * &lt;MESSAGE&gt;optional message &lt;/MESSAGE&gt;<br/>
 * &lt;UPDATE_URL&gt;optional download url &lt;/UPDATE_URL&gt;<br/>
 * &lt;/VALIDATOR&gt;<br/>
 * File size is 1Ko Max</p>
 *
 * @version $Id: LyApplicationVersionValidator.java 34843 2012-03-19 11:13:34Z jlsaouli $
 * @author  Lyria
 */
public class LyApplicationVersionValidator
	implements LyQuestionAnswerInterface,
		Runnable
{
	LyEnvironment _environment;

	/**
	 * Attempt to connect to an URL to check Application version. See class description above
	 *
	 * @param environment current application environment
	 */
	public void checkVersion(LyEnvironment environment)
	{
		if ((environment == null)
				|| (!(environment.getViewManager() instanceof LyViewManager)))
			return;
		_environment = environment;
		((LyViewManager)_environment.getViewManager()).execGui(this, false);
	}

	/**
	 * Code to RUN In SWT Thread -from execGUI in checkVersion().
	 */
	public void run()
	{
		try
		{
			String urlStr = _environment.getFile("LY_APPLICATION_VERSION_VALIDATOR");

			if ((urlStr == null) || (urlStr.equals("LY_APPLICATION_VERSION_VALIDATOR")))
				return;

			((LyViewManager)_environment.getViewManager()).askQuestion(null,
				"LY_CHECK_APPLICATION_VERSION", this, null);
		}
		catch (Exception e)
		{
			if (_environment != null)
				_environment.writeException(e);
		}

	}

	/**
	 * Response to the question popup view. This method may be called directrly specifying the
	 * environment as clientData
	 *
	 * @param result     : answer to the question "do you want to check versions on servers". If
	 *                   false, does nothing.
	 * @param clientData shound be the environment if called directly.
	 */
	public void questionAnswer(boolean result, Object clientData)
	{
		if (result)
		{
			if (clientData instanceof LyEnvironment)
				_environment = (LyEnvironment)clientData;
			AnswerHandler h = new AnswerHandler();

			new Thread(h).start(); // Run asynch to close the first dialog
		}
	}

	class AnswerHandler
		implements Runnable
	{
		public void run()
		{
			try
			{
				try
				{
					String version = _environment.getString("LY_ROOT_VERSION");

					/*
					 * Only for Free version
					 */
					if (version.indexOf("Free") >= 0)
					{
						//int p = version.indexOf("3");

						//if (p >= 0)
						//	version = version.substring(p);
						version = version.replace(' ', '_');
						String			  site =
							"http://c20.statcounter.com/t.php?sc_project=2069430&java=1&security=84ac19d7&camefrom=&u=http%3A//studioFree.lyria.com/"
							+ URLEncoder.encode(System.getProperty("os.name") + "/" + version,
								"UTF-8") + "&sc_random=111&t=Studio"
							+ URLEncoder.encode(version, "UTF-8") + "&sc_random="
							+ System.currentTimeMillis();
						URL				  remoteUrl = new URL(site);
						HttpURLConnection remoteConnection = (HttpURLConnection)remoteUrl
								.openConnection();

						remoteConnection.setRequestProperty("Host", "www.lyria.com");
						remoteConnection.setRequestProperty("User-Agent",
							"User-Agent: Mozilla/1.2.3 (X11; U; " + System.getProperty("os.name")
							+ " i686; ja; rv:1.8.0.1) Gecko/20060124");
						remoteConnection.setRequestProperty("Accept",
							"text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5");
//                                              remoteConnection.setRequestProperty("Keep-Alive", "300");
//                                              remoteConnection.setRequestProperty("Proxy-Connection",         "keep-alive");

						remoteConnection.connect();
						InputStream  iStream = remoteConnection.getInputStream();

						int			 n = iStream.read(), count = 0;
						StringBuffer sb = new StringBuffer();

						while (n >= 0)
						{
							sb.append((char)n);
							n = iStream.read();
							count++;
							//if (count > 1024)
							if (count > 10)
								break;
						}
						iStream.close();
					}
				}
				catch (Exception e)
				{
					//e.printStackTrace();
				}

				String urlStr = _environment.getFile("LY_APPLICATION_VERSION_VALIDATOR");
				String version = _environment.getString("LY_ROOT_VERSION");
				int[]  versions = getVersions(version);

				if (versions == null)
				{
					String displayMessage = _environment.getMessage(
							"LY_ERR_CHECK_APPLICATION_VERSION", version);

					((LyViewManager)_environment.getViewManager()).showWarning(null,
						displayMessage);
					return;
				}

				urlStr = urlStr + versions[0] + '.' + versions[1] + '.' + versions[2] + ".xml";

				URL			 url = new URL(urlStr);
				InputStream  iStream = new BufferedInputStream(url.openStream());
				StringBuffer sb = new StringBuffer();
				int			 n = iStream.read(), count = 0;

				while (n >= 0)
				{
					sb.append((char)n);
					n = iStream.read();
					count++;
					if (count > 1024)
						break;
				}

				if (sb.length() > 0)
				{
					// Fast Search (useless to use an XML Parser)
					String message = "", displayMessage, updateUrl = "";
					int    n1, n2;

					n1 = sb.indexOf("<VERSION>");
					n2 = sb.indexOf("</VERSION>");

					if ((n1 > 0) && (n2 > n1))
					{
						String lastVersion = sb.substring(n1 + "<VERSION>".length(), n2);
						int[]  lastVersions = getVersions(lastVersion);

						if ((lastVersions != null)
								&& ((versions[0] < lastVersions[0]) // Major version
									|| (versions[1] < lastVersions[1]) // Minor version
									|| (versions[2] < lastVersions[2]) // Revision
									|| (versions[3] < lastVersions[3]))) // Build
						{
							n1 = sb.indexOf("<MESSAGE>");
							n2 = sb.indexOf("</MESSAGE>");
							if ((n1 > 0) && (n2 > n1))
							{
								message = sb.substring(n1 + "<MESSAGE>".length(), n2);
								message = _environment.getString(message);
								n1 = sb.indexOf("<UPDATE_URL>", n2);
								n2 = sb.indexOf("</UPDATE_URL>", n1);
								if ((n1 > 0) && (n2 > n1))
									updateUrl = sb.substring(n1 + "<UPDATE_URL>".length(), n2);
							}

							// New version or addsOn
							if ((versions[0] == lastVersions[0]) // Major version
									&& (versions[1] == lastVersions[1]) // Minor version
									&& (versions[2] == lastVersions[2]))
								displayMessage = _environment.getMessage(
										"LY_NEW_APPLICATION_PATCH_AVAILABLE", version, message,
										updateUrl);
							else
								displayMessage = _environment.getMessage(
										"LY_NEW_APPLICATION_VERSION_AVAILABLE", version, message,
										updateUrl);
							((LyViewManager)_environment.getViewManager()).showWarning(null,
								displayMessage);
							if (updateUrl.length() > 0)
								((LyViewManager)_environment.getViewManager()).showLocation(null,
									updateUrl, true);
						}
					}
				}
			}
			catch (Exception e)
			{
				_environment.writeException(e);
				// Do nothing
			}
		} // end method run
	} // end class AnswerHandler

	/**
	 * Gets an array of versions numbers from a srting formatted like "3.3.1 build 29".
	 */
	int[] getVersions(String version)
	{
		try
		{
			int n1 = version.lastIndexOf(" build");

			if (n1 < 0)
				return null;

			String rootVersion, build;
			int    n2 = version.lastIndexOf(" ", n1 - 1);

			rootVersion = ((n2 >= 0) ? version.substring(n2, n1).trim()
									 : version.substring(0, n1).trim());
			if ((rootVersion.charAt(0) == 'v') || (rootVersion.charAt(0) == 'V'))
				rootVersion = rootVersion.substring(1);

			build = version.substring(n1 + 6).trim();

			String[] versions = LyString.stringToList(rootVersion, '.');
			int		 i, nv = versions.length;
			int[]    versionNumbers = new int[nv + 1];

			for (i = 0; i < nv; i++)
				versionNumbers[i] = Integer.parseInt(versions[i]);
			versionNumbers[nv] = Integer.parseInt(build);

			return versionNumbers;
		}
		catch (Exception e)
		{
			return null;
		}
	} // end method getVersions
} // end class LyApplicationVersionValidator
