/*
 * LyGlobalEvent.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app;

/**
 * This class implements an event of creation/modification/deletion for an object within a data
 * source. The event is sent to 'registered listeners': classes which implements the {@link
 * LyDataSourceListener} interface and which were added as listener of the dataSource.
 *
 * @author  Lyria S.A.
 * @version $Id: LyGlobalEvent.java 29006 2010-12-23 17:44:46Z bmichel $
 * @see     leon.data.LyDataSource
 * @see     leon.data.LyDataSourceListener
 */
public class LyGlobalEvent
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyGlobalEvent.java 29006 2010-12-23 17:44:46Z bmichel $";

	/** Event type : ALL, all the types of events. */
	public static final short  TYPE_ALL = -1;

	/** Event type : ACTIVE_VIEW, active view change event. */
	public static final short  TYPE_ACTIVE_VIEW = 0;

	/** Event type : SELECTION, selection change event. */
	public static final short  TYPE_SELECTION = 1;

	/**
	 * Constant : LAST_TYPE, the last number type used by Leonardi. User types must be greater than
	 * this value.
	 */
	public static final short  LAST_TYPE = 100;

	/**
	 * The type of event which occured. Possible values are TYPE_XXX constants:
	 *
	 * <ul>
	 *   <li>{@link #TYPE_ACTIVE_VIEW}</li>
	 *   <li>{@link #TYPE_ALL}</li>
	 *   <li>{@link #TYPE_SELECTION}</li>
	 * </ul>
	 */
	short					   _type;

	/** Data associated to this event. The data type depends on the event type. */
	Object					   _data;

	/**
	 * Constructs a data source event of the given type with the given data.
	 *
	 * @param type Type of event which occured. Possible values are TYPE_XXX constants:
	 *
	 *             <ul>
	 *               <li>{@link #TYPE_ACTIVE_VIEW}</li>
	 *               <li>{@link #TYPE_ALL}</li>
	 *               <li>{@link #TYPE_SELECTION}</li>
	 *             </ul>
	 * @param data data associated to this event
	 */
	public LyGlobalEvent(short type, Object data)
	{
		_type = type;
		_data = data;
	}

	/**
	 * Gets the type of this event.
	 *
	 * @return Type of this event. Possible values are TYPE_XXX constants:
	 *
	 *         <ul>
	 *           <li>{@link #TYPE_ACTIVE_VIEW}</li>
	 *           <li>{@link #TYPE_ALL}</li>
	 *           <li>{@link #TYPE_SELECTION}</li>
	 *         </ul>
	 */
	public short getType()
	{
		return _type;
	}

	/**
	 * Gets data associated to this event.
	 *
	 * @return The data associated to this event.
	 */
	public Object getData()
	{
		return _data;
	}
} // end class LyGlobalEvent
