/*
 * LyGlobalEventListener.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app;

/**
 * This class is the listener interface for receiving global events.
 *
 * <p>Three types of events can be thrown: 'active view changes', 'selection changes' and 'user
 * events'. A method corresponds to each of these types.</p>
 *
 * @author  Lyria S.A.
 * @version $Id: LyGlobalEventListener.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @see     leon.data.LyEventManager
 * @see     leon.app.LyGlobalEvent
 */
public interface LyGlobalEventListener
{
	/**
	 * File version
	 */
	String __VERSION = "$Id: LyGlobalEventListener.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Method for receiving an active view change event.
	 *
	 * @param  event Event notifying the creation of an object.
	 * @return True indicating if the event was used, false otherwise.
	 */
	boolean activeViewChanged(LyGlobalEvent event);

	/**
	 * Method for receiving selection change event.
	 *
	 * @param  event Event notifying the modification of an object.
	 * @return True indicating if the event was used, false otherwise.
	 */
	boolean selectionChanged(LyGlobalEvent event);

	/**
	 * Method for receiving user event.
	 *
	 * @param  event Event notifying the deletion of an object.
	 * @return True indicating if the event was used, false otherwise.
	 */
	boolean userEvent(LyGlobalEvent event);
}