/**
 * LyModelListener.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app;

import java.lang.reflect.*;
import leon.data.LyObjectSource;
import leon.info.LyInfo;


/**
 * Model listener.
 */
public class LyModelListener
	implements LyApplicationListener
{
	protected Class<?>			_modelClass;
	protected String	    	_applicationId;
	protected LyApplication		_application;

	public LyModelListener(Class<?> modelClass, LyApplication application)
	{
		_modelClass = modelClass;
		_application = application;
		_applicationId = application.getId();

		LyApplication.addListener(this);
	}

	/**
	 * Call when a new LyApplication is added in the list of all applications.
	 * @param application the added application
	 */
	public void applicationAdded(LyApplication application)
	{
	}

	/**
	 * Call when a new LyApplication is added in the list of all applications.
	 * @param application the added application
	 */
	public void applicationStarted(LyApplication application)
	{
		// Restore application attribute when needed
		if ((_application == null)
				&& application.getId().equals(_applicationId))
			_application = application;

		// Restore identifiers
		Field[] 	fields = _modelClass.getDeclaredFields();
		int			i, n = fields.length;
		Object		value;
		String		infoId = null;
		LyInfo		info;

		for (i = 0; i < n; i++)
		{
			try
			{
				if (Modifier.isFinal(fields[i].getModifiers()))
				{
					value = fields[i].get(this);
					if (value instanceof String)
						infoId = (String)value;
				}
				else if (infoId != null)
				{
					fields[i].setAccessible(true);
					info = application.getInfo(infoId);
					if ((info != null)
							&& fields[i].getType().isAssignableFrom(info.getClass()))
						fields[i].set(this, info);
					infoId = null;
				}
			}
			catch (IllegalAccessException e)
			{
			}
		}
	}

	/**
	 * Call when a LyApplication was removed from the list of all applications.
	 * @param application the removed application
	 */
	public void applicationRemoved(LyApplication application)
	{
		if (application != _application)
			return;

		// Clear identifiers
		Field[] 	fields = _modelClass.getDeclaredFields();
		int			i, n = fields.length;

		for (i = 0; i < n; i++)
		{
			try
			{
				if (!Modifier.isFinal(fields[i].getModifiers()))
				{
					fields[i].setAccessible(true);
					fields[i].set(this, null);
				}
			}
			catch (IllegalAccessException e)
			{
			}
		}

		_application = null;
	}

	/**
	 * Call when a LyObjectSource was added on the specified application
	 * @param application the application
	 * @param objectSource the added LyObjectSource
	 */
	public void objectSourceAdded(LyApplication application, LyObjectSource objectSource)
	{
		// Nothing to do
	}
} // end class LyModelListener
