/*
 * LyFormatterAdapter.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app.adapter;

import leon.info.*;
import leon.info.infointerface.*;


/**
 * Interface implemented by classes that format field values.
 *
 * @author  Lyria S.A.
 * @version $Id: LyFormatterAdapter.java 28422 2010-11-03 12:03:38Z ehelmste $
 */
public abstract class LyFormatterAdapter
	implements LyFormatterInterface
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyFormatterAdapter.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Method to format a value to an edition string with the given unit.
	 *
	 * @param  fieldInfo the fieldInfo for which a value is formatted
	 * @param  value     the value to format
	 * @param  unit      the destination unit of the string value
	 * @return The edition string corresponding to the given value and unit.
	 */
	public abstract String toString(LyFieldInfo fieldInfo, Object value, LyUnit unit);

	/**
	 * Method to format a value to a displayable string with the given unit.
	 *
	 * @param  fieldInfo the fieldInfo for which a value is formatted
	 * @param  value     the value to format
	 * @param  unit      the destination unit of the string value
	 * @return The displayable string corresponding to the given value and unit.
	 */
	public String toName(LyFieldInfo fieldInfo, Object value, LyUnit unit)
	{
		return toString(fieldInfo, value, unit);
	}

	/**
	 * Method to format a value to a displayable short string with the given unit.
	 *
	 * @param  fieldInfo the fieldInfo for which a value is formatted
	 * @param  value     the value to format
	 * @param  unit      the destination unit of the string value
	 * @return The short displayable string corresponding to the given value and unit.
	 */
	public String toShortName(LyFieldInfo fieldInfo, Object value, LyUnit unit)
	{
		return toString(fieldInfo, value, unit);
	}

	/**
	 * Method to parse a string value with a given unit.
	 *
	 * @param  fieldInfo the fieldInfo for which the string value is parsed.
	 * @param  value     the string value to parse
	 * @param  unit      the destination unit of the string value
	 * @return The value corresponding to the given string and unit.
	 */
	public abstract Object parse(LyFieldInfo fieldInfo, String value, LyUnit unit);
} // end class LyFormatterAdapter
