/*
 * LyBehavior.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app.behavior;

import leon.app.*;
import leon.app.behaviorinterface.*;
import leon.control.*;
import leon.view.event.*;


/**
 * LyBehavior is an abstract class. It implements the default behavior interface. All behaviors
 * inherit from LyBehavior.
 *
 * @author  Lyria S.A.
 * @version $Id: LyBehavior.java 31039 2011-05-17 13:27:20Z jlsaouli $
 * @see     LyBehaviorInterface
 */
public abstract class LyBehavior
	implements LyBehaviorInterface,
			   LyGetSessionInterface
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyBehavior.java 31039 2011-05-17 13:27:20Z jlsaouli $";

	protected LySession		_session;
	
	/**
	 * Default constructor.
	 */
	public LyBehavior()
	{
	}

	/**
	 * Initializes information used within this behavior and is called immediately after its
	 * construction.<br />
	 * <i>The default implementation</i> does nothing.
	 *
	 * @param controller The controller that requested the creation of this behavior.
	 */
	public void init(LyController controller)
	{
		_session = (controller == null ? null : controller.getSession());
	}

	/**
	 * Free this behavior. This method is provided to allow behavior to unregister itself from
	 * object sources or workspaces if needed. It should be called only by the controller that built
	 * this behavior.<br />
	 * <i>The default implementation</i> does nothing.
	 */
	public void free()
	{
	}

	/**
	 * Method invoked when a demand is sent to this listener.<br />
	 * <i>The default implementation</i> does nothing and return false.
	 *
	 * @param  controller controller that routed the demand.
	 * @param  demand     demand parameters sent by the view component.
	 * @return Return true if the demand was handled, false otherwise. If false, the demand is
	 *         transmitted to the controller to try to handle it.
	 */
	public boolean handleDemand(LyController controller, LyDemand demand)
	{
		return false;
	}
	
	/**
	 * Gets the session of current object.
	 *
	 * @return the session associated to the object.
	 */
	public LySession getSession()
	{
		return _session;
	}
} // end class LyBehavior
