/*
 * LyCanvasBehavior.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app.behavior;

import leon.app.behaviorinterface.*;
import leon.control.*;
import leon.misc.*;
import leon.view.event.*;


/**
 * This class implements the default behavior for actions that are controlled by a canvas
 * controller.
 *
 * @author  Lyria S.A.
 * @version $Id: LyCanvasBehavior.java 32903 2011-10-21 11:39:48Z nhamon $
 */
public class LyCanvasBehavior extends LyActionBehavior
	implements LyCanvasBehaviorInterface
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyCanvasBehavior.java 32903 2011-10-21 11:39:48Z nhamon $";

	/**
	 * Constructor.
	 */
	public LyCanvasBehavior()
	{
	}

	/**
	 * Invoked when a demand is sent to this listener. This method only traces demands on
	 * environment output when DEBUG_TRACE mode is on.
	 *
	 * @param  controller controller that routed the demand.
	 * @param  demand     demand parameters sent by the view component.
	 * @return Return true if the demand was handled, false otherwise. If false, the demand is
	 *         transmitted to the controller to try to handle it.
	 */
	public boolean handleDemand(LyController controller, LyDemand demand)
	{
		LyParameterValues parameterValues = demand.getParameterValues();

		if (parameterValues == null)
			return false;

		LyEnvironment environment = controller.getEnvironment();

		if (environment.isDebugLevel(LyEnvironment.DEBUG_TRACE))
		{
			String[] names = parameterValues.getParameterNames();
			int		 i, n = ((names == null) ? 0 : names.length);

			environment.logTrace("\nLyCanvasBehavior.handleDemand " + demand);

			for (i = 0; i < n; i++)
				environment.logTrace("param " + names[i] + ", "
					+ parameterValues.getParameter(names[i]));
		}
		
		if ((demand instanceof LyActionDemand) && LyViewController.DEMAND_CLOSE.equals(demand.getSource()))
			return super.handleDemand(controller, demand);

		return true;
	}
} // end class LyCanvasBehavior
