/*
 * LySocialBehavior.java
 * Copyright (c) 2000-2011, Lyria-W4.
 * All rights reserved.
 */

package leon.app.behavior;

import java.io.File;

import leon.app.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;
import leon.view.event.*;

public class LyChangePhotoBehavior extends LyUserInputBehavior
{
	public static int	SIZE = 200;
	
    LyUserInputController _controller;
    
	public void init(LyController controller)
	{
		super.init(controller);
		
		LyUserInputController	userInputController = (LyUserInputController)controller;
		LyTableController		socialController = (LyTableController)userInputController.getParentAreaController();
		LySocialBehavior		socialBehavior = (LySocialBehavior)socialController.getBehavior();
		LySession				session = socialController.getSession();
		LyObject				user = session.getUser();
        LyViewManager			vm = session.getViewManager();
		String					imagePath = socialBehavior.getRowImage(socialController, user, vm.hasProperty("MANAGE_ATTACHMENT"));
		LyFile					file = userInputController.addFile("_imageFile", "LY_ACTION_CHANGE_PHOTO", imagePath,
			LyFile.Type.IMAGE, false);
		LyViewManager			viewManager = session.getViewManager();
		
		file.setAttachmentHandler("default");
		file.setAttachmentDirectory(viewManager.getTemporaryDir());
		file.setNCols(SIZE);
		file.setNRows(SIZE);
		file.setBorder(LyFile.Border.SINGLE);
	}
	
	public boolean validate(LyController controller, LyParameterValues values)
	{ 
		if (!super.validate(controller, values))
			return false;
		
		String			imageFile = values.getParameter("_imageFile");
		
		if (imageFile == null)
			return false;
		
		LyApplication	application = controller.getApplication();
		LySession		session = controller.getSession();
		LyObject		user = session.getUser();
		String			userId = user.getObjectId().toString();
		
		if (!imageFile.contains(File.separator))
		{
			LyEnvironment	environment = controller.getEnvironment();
			String 			tmpDir = environment.getViewManager().getTemporaryDir();
			
			imageFile = tmpDir + File.separator + imageFile;
		}
		
		LySocialBehavior.saveImage(application, userId, imageFile);
		
		// Post an event to force refresh on user
		LyClassInfo	classInfo = user.getClassInfo();
		
		user.refresh(classInfo.getFieldInfos(LyFieldInfo.MARK_NAME));
		return true;
	}
}
