package leon.app.behavior;

import java.io.File;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import leon.control.LyCommonServiceController;
import leon.control.LyController;
import leon.control.LyFormController;
import leon.control.builder.LyCommonServiceBuilder;
import leon.misc.LyEnvironment;
import leon.view.LyBoolean;
import leon.view.LyCell;
import leon.view.LyComponent;
import leon.view.LyDate;
import leon.view.LyFieldItem;
import leon.view.LyFile;
import leon.view.LyFormField;
import leon.view.LyLabel;
import leon.view.LyNumber;
import leon.view.LyRow;
import leon.view.LyRows;
import leon.view.LyTable;
import leon.view.LyText;
import leon.view.LyTool;
import leon.view.LyToolBar;
import leon.view.event.LyDemand;
import eu.w4.service.Service;
import eu.w4.service.ServiceInfo;

/**
 * The default implementation for common services
 * 
 *
 */
public class LyCommonServiceBehavior extends LyUserInputBehavior
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyCommonServiceBehavior.java 32310 2011-09-13 15:00:34Z bjacquet $";
	
  private Map<String, Object>       _parameters              = new HashMap<String, Object>();
  private LyCommonServiceController _commonServiceController = null;
  private LyCommonServiceBuilder    _commonServicebuilder    = null;
  private boolean                   _isDebug                 = LyCommonServiceController._isDebug;
  private Service                   _service;
  private ServiceInfo               _serviceInfo;

  /**
   * Handle a demand form the user form If the form contains an 'AutoExec' field, the form is
   * automatically closed and the service is invoked if the form is validated by user, service
   * parameters are setted and the service is invoked.
   * 
   * @param controller the controller
   * @param demand the demand
   * @return true if the demand was handled 
   */
  public boolean handleDemand(LyController controller, LyDemand demand)
  {
    LyEnvironment   environment = controller.getEnvironment();

    environment.logTrace("HANDLE DEMAND");

    boolean resp = super.handleDemand(controller, demand);

    if ((LyFormField)controller.getComponent().getComponent("AutoExec") != null)
    {
      super.close(controller);
    }

    String demandSource = demand.getSource();
    if (demandSource.equals(LyFormController.DEMAND_VALIDATE))
    {
      _commonServiceController = (LyCommonServiceController)controller;

      if (_service == null)
      {
        _service = _commonServiceController.getServiceType();
      }

      if (_serviceInfo == null)
      {
        _serviceInfo = _commonServiceController.getServiceInfoType();
      }

      if ((_parameters != null) && !_parameters.isEmpty())
      {
        _commonServiceController.setParameters(_parameters);
      }

      try
      {
        boolean close = _commonServiceController.invoke(_service, _serviceInfo);
        if(close)
        {
          super.close(controller);
        }
      }
      catch (Exception e)
      {
        e.printStackTrace();
      }
    }

    if (demandSource.equals(LyFormController.DEMAND_CANCEL))
    {
      super.close(controller);
    }

    return resp;
  }

  /**
   * handle users demand in case of dynamic service this method allows to update form fields,
   * deleting or creating them.
   * @param controller the controller
   * @param demand the demand
   */
  public void userInputDemand(LyController controller, LyDemand demand)
  {
    LyEnvironment   environment = controller.getEnvironment();
    
    if (_isDebug)
    {
      environment.logTrace("USER INPUT DEMAND");
    }

    if (_commonServiceController == null)
    {
      _commonServiceController = (LyCommonServiceController)controller;
    }
    
    List<String> classNames = _commonServiceController.getClasseNamesInPackage();
    if (_commonServicebuilder == null && classNames.size()>1)
    {
      _commonServicebuilder = (LyCommonServiceBuilder)controller.getBuilder();
    }

    if (_isDebug)
    {
      environment.logTrace("allservices = " + _commonServiceController._allServices);
    }

    if (_service == null)
    {
      _service = _commonServiceController.getServiceType();
    }

    if (_serviceInfo == null)
    {
      _serviceInfo = _commonServiceController.getServiceInfoType();
    }

    
    String       fieldId     = demand.getSource();
    String       paramName   = fieldId;
    List<String> allServices = _commonServiceController.getClasseNamesInPackage();
    for(String key : allServices)
    {
      if(fieldId.contains(key))
      {
        paramName = fieldId.replace(key+".", "");
      }
    }

    LyComponent component = controller.getComponent().getComponent(fieldId);

    if (component instanceof LyTool)
    {
      String   param = fieldId;
      Class<?> type;

      if (fieldId.endsWith("-ADD"))
      {
        String field = fieldId;
        field = field.substring(0, field.length() - 4);
        param = paramName.substring(0, paramName.length() - 4);
        type  = _commonServiceController.getType(param);
        LyFormField attributeField = (LyFormField)controller.getComponent().getComponent(field);
        LyFieldItem fieldItem      = attributeField.getFieldItem();
        String      value          = fieldItem.getValue();
        
        if ((value != null) && (value.length() > 0))
        {
          if (fieldItem.getClass() == LyText.class)
          {
            LyText text = (LyText)fieldItem;
            text.setLabel("");
          }
          else if (fieldItem.getClass() == LyFile.class)
          {
            LyFile file = (LyFile)fieldItem;
            file.setLabel("");
          }

          String  tableId = field + "-TABLE";
          LyTable table   = (LyTable)controller.getComponent().getComponent(tableId);
          LyRows  rows    = new LyRows();

          if (table.getRows() != null)
          {
            rows = table.getRows();
          }

          LyRow  row0   = rows.getRow(0);
          String row0Id = row0.getId();
          if ((row0Id != null) && (row0Id.compareTo("") == 0))
          {
            rows.removeRow(0);
          }

          LyLabel string = new LyLabel();
          string.setLabel(value);

          Calendar test = Calendar.getInstance();
          long     id   = test.getTimeInMillis();
          LyTool   sup  = new LyTool(field + "-SUP" + id, "LY_COMPONENT_DELETE");

          LyToolBar supToolBar = new LyToolBar();
          supToolBar.setController(_commonServiceController.getId());
          supToolBar.addTool(sup);

          LyCell cell = new LyCell();
          cell.setLabel(value);

          LyRow row = new LyRow();
          row.addCell(cell);
          row.addToolBar(supToolBar);
          row.setId(field + id);

          rows.addRow(row);
          table.setRows(rows);
          /************************Ajout dans la map******************/
          Map<String, Object> adding = new HashMap<String, Object>();

          if (type == List.class)
          {
            Object       list = _commonServiceController.getParam(param);
            List<String> currentList = new ArrayList<String>();

            if ((list != null) && (list instanceof List<?>))
            {
            	@SuppressWarnings("unchecked")
            	List<String> temp = (List<String>)list;
              currentList = temp;
            }

            currentList.add(value);
            adding.put(param, currentList);
          }
          else if (type == Vector.class)
          {
            Object       vector        = _commonServiceController.getParam(param);
            Vector<File> currentVector = new Vector<File>();

            if ((vector != null) && (vector instanceof Vector<?>))
            {
            	@SuppressWarnings("unchecked")
            	Vector<File> temp = (Vector<File>)vector;
              currentVector = temp;
            }

            currentVector.addElement(new File(value.toString()));
            adding.put(param, currentVector);
          }

          _commonServiceController.setParameters(adding);
          /************************Fin Ajout dans la map******************/
        }
      }
      else if (fieldId.contains("-SUP"))
      {
        String field = fieldId;

        String[] split = field.split("-SUP");
        param = paramName.split("-SUP")[0];
        type  = _commonServiceController.getType(param);
        String   tableId = split[0] + "-TABLE";

        LyTable table = (LyTable)controller.getComponent().getComponent(tableId);
        LyRows  rows = table.getRows();

        LyRow row = new LyRow();
        row = rows.getRow(split[0] + split[1]);

        for (int i = 0; i < row.getComponentCount(); i++)
        {
          if (row.getComponent(i) instanceof LyCell)
          {
            String value = ((LyCell)row.getComponent(i)).getLabel();

            if (value.compareTo("EMPTY_CELL_VALUE") != 0)
            {
              /************************Suppression dans la map******************/
              Map<String, Object> adding = new HashMap<String, Object>();

              if (type == List.class)
              {
                Object       list        = _commonServiceController.getParam(param);
                List<String> currentList = new ArrayList<String>();
                if ((list != null) && (list instanceof List<?>))
                {
                	@SuppressWarnings("unchecked")
                	List<String> temp = (List<String>)list;
                  currentList = temp;
                  currentList.remove(value);
                }

                adding.put(param, currentList);
              }
              else if (type == Vector.class)
              {
                Object       vector        = _commonServiceController.getParam(param);
                Vector<File> currentVector = new Vector<File>();

                if ((vector != null) && (vector instanceof Vector<?>))
                {
                	@SuppressWarnings("unchecked")
                	Vector<File> temp = (Vector<File>)vector;
                  currentVector = temp;
                  currentVector.removeElement(new File(value));
                }

                adding.put(param, currentVector);
              }

              _commonServiceController.setParameters(adding);
              /************************Fin Suppression dans la map******************/
            }

          }
        }

        rows.removeRow(row);
        if (rows.getRowCount() == 0)
        {
          LyRow  emptyRow = new LyRow();
          LyCell cell = new LyCell();

          emptyRow.addCell(cell);
          emptyRow.addCell(cell);
          emptyRow.setId("");
          rows.addRow(emptyRow);
        }

        table.updateViewContent();
        table.setVisible(true);
      }
    }

    if (component instanceof LyFormField)
    {
      LyFormField attributeField = (LyFormField)component;
      LyFieldItem fieldItem      = attributeField.getFieldItem();
      String      fieldValue     = fieldItem.getValue();
      if (_isDebug)
      {
        System.out.println("getSource      = " + fieldId);
        System.out.println("paramName      = " + paramName);
        System.out.println("Internal Value = " + fieldValue);
      }

      String changedComponent     = fieldValue;
      String currentComponentName = _commonServiceController._currentServiceName;

      if (_commonServiceController._allServices.contains(changedComponent)
          && (changedComponent.compareTo(currentComponentName) != 0))
      {
        if (_isDebug)
        {
          System.out.println("----------------------- changing interface");
        }

        _commonServiceController.clearParameters();
        _commonServicebuilder.removeOldParam(changedComponent);

        return;
      }

      if (changedComponent != null)
      {
        Map<String, Object> adding = new HashMap<String, Object>();
        Class<?>            type   = _commonServiceController.getType(paramName);

        if ((type == String.class) || (type == String[].class))
        {
          String string = fieldValue;
          adding.put(paramName, string);
        }
        else if (type == Date.class)
        {
          LyDate           date       = (LyDate)fieldItem;
          SimpleDateFormat simple     = new SimpleDateFormat(date.getFormat());
          Date             outputDate = null;

          try
          {
            outputDate = simple.parse(date.getValue());
          }
          catch (ParseException e)
          {
            e.printStackTrace();
          }

          adding.put(paramName, outputDate);
        }
        else if ((type == File.class) || (type == File[].class))
        {
          File file = new File(fieldValue);

          adding.put(paramName, file);
        }
        else if (type == Boolean.class)
        {
          LyBoolean bool = (LyBoolean)fieldItem;
          boolean   value;

          if (bool.getStatus().compareTo(LyBoolean.Status.CHECKED) == 0)
          {
            value = true;
          }
          else
          {
            value = false;
          }

          adding.put(paramName, value);
        }
        else if (type == Long.class)
        {
          LyNumber number = (LyNumber)fieldItem;
          String   value = number.getValue();

          if (value.contains("."))
          {
            value = value.split("\\.")[0];
          }

          long pLong = Long.parseLong(value);

          adding.put(paramName, pLong);
        }

        if ((type != List.class) && (type != Vector.class))
        {
          _commonServiceController.setParameters(adding);
        }
      }
    }
  }
}
