/**
  * LyFindBehavior.java
  * Copyright (c) 2000-2009, Lyria-W4.
  * All rights reserved.
  */
package leon.app.behavior;

import leon.control.LyAreaController;
import leon.control.LyController;
import leon.control.LyFindController;
import leon.control.LyFindInterface;
import leon.control.LyPanelController;
import leon.misc.LyEnvironment;
import leon.view.LyBoolean;
import leon.view.event.LyActionDemand;
import leon.view.event.LyDemand;
import leon.view.event.LyParameterValues;

/**
 * The default find behavior
 * 
 *
 */
public class LyFindBehavior extends LyUserInputBehavior
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyFindBehavior.java 33768 2012-01-09 17:19:43Z nhamon $";
	
	/** Indicate when an error last occured in search. */
	protected long		  _errorInSearch;

	/**
	 * Check if the controller is vertical
	 * @param findController the find controller
	 * @return true if the panel is vertical, false otherwise
	 */
	public boolean isVertical(LyFindController findController){
		 
		LyPanelController parent = ((LyPanelController)findController.getParentAreaController());
		
		return !parent.isHorizontal();
	}
	
	/**
	 * Invoked when the user validates the content of the input.
	 *
	 * @param controller the current LyUserInputController instance
	 * @param demand     demand parameters sent by the view component.
	 */
	public void userInputDemand(LyController controller, LyDemand demand)
	{
		String            id = demand.getSource();
		LyFindController  findController = (LyFindController)controller;
        
		if (id.equals(LyFindController.DEMAND_FIND)
		        || id.equals(LyFindController.DEMAND_NEXT)
		        || id.equals(LyFindController.DEMAND_PREVIOUS)
				|| id.equals(LyFindController.DEMAND_VALIDATE)
				|| id.equals(LyFindController.DEMAND_FIND_FIELD + findController.getId()))
		{
			LyParameterValues paramValues = demand.getParameterValues();

			if (!super.validate(controller, paramValues))
				return;

			String fieldId = paramValues.getParameter(LyFindController.DEMAND_FIND_FIELD + findController.getId());

			if (fieldId != null)
			{
				String  pattern = paramValues.getParameter(LyFindController.DEMAND_FIND_PATTERN + findController.getId());
				String  wholeWord = paramValues.getParameter(LyFindController.DEMAND_FIND_WHOLE_WORD + findController.getId());
				String  caseSensitive = paramValues.getParameter(LyFindController.DEMAND_FIND_CASE_SENSITIVE + findController.getId());
				boolean update = (pattern == null);

				findController.setPattern(pattern);
				findController.setFieldInfo(findController.getClassInfo().getFieldInfo(fieldId));
				findController.setWholeWord(wholeWord.equals(LyBoolean.Status.CHECKED.toString()));
				findController.setCaseSensitive(caseSensitive.equals(LyBoolean.Status.CHECKED.toString()));

				// Find start
				LyFindInterface parent = ((LyFindInterface)findController.getParentAreaController());
				int				result = parent.findObject(findController, -1, !id.equals(LyFindController.DEMAND_PREVIOUS));
				
				if ((findController.getPattern() != null)
						&& (result == -1)
						&& ((System.currentTimeMillis() - _errorInSearch) > 500))
				{
					LyEnvironment	environment = findController.getEnvironment();
					
					_errorInSearch = System.currentTimeMillis();
					environment.beep();
					findController.showWarning("LY_ER_NO_SEARCH_RESULT");
				}

				// Update Find menu
				if (update)
					findController.updateFindMenu((LyAreaController)parent);
			}
		}
		else if (id.equals(LyFindController.DEMAND_CLOSE_FIND))
			findController.hideView();
		else if (demand instanceof LyActionDemand)
		{
			// A close demand may be sent to find controller in web mode : forward to parent controller.
			LyPanelController parent = ((LyPanelController)findController.getParentAreaController());

			parent.actionDemand((LyActionDemand)demand);
		}
		else
			super.userInputDemand(controller, demand);
	} // end method userInputDemand
} // end class _LyFindBehavior
