/*
 * LyLabelBehavior.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app.behavior;

import leon.app.behaviorinterface.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;


/**
 * This class implements the default behavior for customizing labels.
 *
 * @author  Lyria S.A.
 * @version $Id: LyLabelBehavior.java 28422 2010-11-03 12:03:38Z ehelmste $
 */
public class LyLabelBehavior
	implements LyLabelBehaviorInterface
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyLabelBehavior.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Construct a label behavior.
	 */
	public LyLabelBehavior()
	{
	}

	/**
	 * Gets info label of the given type for the given class info in the context of the given area
	 * controller.<br />
	 * <i>The default implementation</i> does nothing and return null.
	 *
	 * @param  areaController the current controller instance
	 * @param  classInfo      the class info
	 * @param  type           Type of searched label. Possible values are {@link
	 *                        LyInfoLabel.Type#GRAMMATICAL}, {@link LyInfoLabel.Type#ID}, {@link
	 *                        LyInfoLabel.Type#NAME} {@link LyInfoLabel.Type#SEMANTIC} {@link
	 *                        LyInfoLabel.Type#SHORT_VALUE} {@link LyInfoLabel.Type#USER}, {@link
	 *                        LyInfoLabel.Type#VALUE}
	 * @return The info label if found, null otherwise.
	 */
	public LyInfoLabel getInfoLabel(LyAreaController areaController, LyClassInfo classInfo,
        LyInfoLabel.Type type)
	{
		return null;
	}

	/**
	 * Gets info label of the given type for the given field info in the context of the given area
	 * controller.<br />
	 * <i>The default implementation</i> does nothing and return null.
	 *
	 * @param  areaController the current controller instance
	 * @param  fieldInfo      the field info
	 * @param  type           Type of searched label. Possible values are {@link
	 *                        LyInfoLabel.Type#GRAMMATICAL}, {@link LyInfoLabel.Type#ID}, {@link
	 *                        LyInfoLabel.Type#NAME} {@link LyInfoLabel.Type#SEMANTIC} {@link
	 *                        LyInfoLabel.Type#SHORT_VALUE} {@link LyInfoLabel.Type#USER}, {@link
	 *                        LyInfoLabel.Type#VALUE}
	 * @return The info label if found, null otherwise.
	 */
	public LyInfoLabel getInfoLabel(LyAreaController areaController, LyFieldInfo fieldInfo,
        LyInfoLabel.Type type)
	{
		return null;
	}

	/**
	 * Gets the name of the object in the context of the given area controller.<br />
	 * <i>The default implementation</i> does nothing and return null.
	 *
	 * @param  areaController the current controller instance
	 * @param  object         Object whose the name is required.
	 * @return the object name
	 */
	public String getName(LyAreaController areaController, LyObject object)
	{
		return null;
	}

	/**
	 * Gets the given object label, for the given field info and the given value in the context of
	 * the given area controller.<br />
	 * <i>The default implementation</i> does nothing and return null.
	 *
	 * @param  areaController the current controller instance
	 * @param  object         the object for which a label is required.
	 * @param  fieldInfo      field info.
	 * @param  fieldValue     value of the field.
	 * @return the object label as a string.
	 */
	public String getLabel(LyAreaController areaController, LyObject object, LyFieldInfo fieldInfo,
		Object fieldValue)
	{
		return null;
	}

	/**
	 * Gets the given object short label, for the given field info and the given value in the
	 * context of the given area controller.<br />
	 * <i>The default implementation</i> does nothing and return null.
	 *
	 * @param  areaController the current controller instance
	 * @param  object         Object for which a short label is required.
	 * @param  fieldInfo      field info.
	 * @param  fieldValue     value of the field.
	 * @return the object short label as a string.
	 */
	public String getShortLabel(LyAreaController areaController, LyObject object,
		LyFieldInfo fieldInfo, Object fieldValue)
	{
		return null;
	}
} // end class LyLabelBehavior
