/*
 * LyMetricsPanelBehavior.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app.behavior;

import java.util.*;

import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.info.LyFilterExpression.*;
import leon.view.event.*;


/**
 * This class implements the default behavior for metrics chart panel.
 *
 * @author  Lyria S.A.
 * @version $Id: LyChartBehavior.java 30007 2011-03-10 17:58:16Z jlsaouli $
 */
public class LyMetricsPanelBehavior extends LyActionBehavior
{
	@Override
	public boolean handleDemand(LyController controller, LyDemand demand)
	{
		if ((demand instanceof LyActionDemand)
				&& (demand.getType() == LyActionDemand.TYPE_VALUE_CHANGED))
        {
            String source = demand.getSource();

            if ((source != null)
                    && source.equals(controller.getId() + "_periodFilterField"))
            {
            	LyCompoundAreaController	compoundController = (LyCompoundAreaController)controller;
                String						value = ((LyActionDemand)demand).getValue();

				filterPeriod(compoundController, value);

                return true;
            }
        }
		else if ((demand instanceof LyActionDemand) && (LyViewController.DEMAND_UPDATE.equals(demand.getSource())))
		{
        	LyCompoundAreaController	compoundController = (LyCompoundAreaController)controller;

			filterPeriod((LyMultiAreaController)compoundController.getParentAreaController(), "");

            return true;
		}

		return super.handleDemand(controller, demand);
	}

	/**
	 * Filter metrics objects according to the filter identifier.
	 *
	 * @param multiAreaController controller containing the charts.
	 * @param id                  filter identifier.
	 * @return the filter used to select metric objects.
	 */
	protected void filterPeriod(LyMultiAreaController multiAreaController, String id)
	{
        LyClassInfo			metricCls = multiAreaController.getApplication().getClassInfo(LyMetricsTools.METRIC_CLASS);
		LyFilter			filter = new LyFilter(LyFilter.Type.AND);
		Calendar			cal = Calendar.getInstance();
		LyFilterExpression	filterExpr = null;
		LyFilter			globalFilter;

		if (id.equals("LY_METRICS_FILTER_MONTH"))
			cal.set(Calendar.MONTH, cal.get(Calendar.MONTH) - 1);
		else if (id.equals("LY_METRICS_FILTER_WEEK"))
			cal.set(Calendar.WEEK_OF_MONTH, cal.get(Calendar.WEEK_OF_MONTH) - 1);
		else if (id.equals("LY_METRICS_FILTER_QUARTER"))
			cal.set(Calendar.MONTH, cal.get(Calendar.MONTH) - 3);
		else if (id.equals("LY_METRICS_FILTER_YEAR"))
			cal.set(Calendar.YEAR, cal.get(Calendar.YEAR) - 1);
		else
			filter = null;

		if (filter != null)
		{
			LyFieldInfo		startDate = metricCls.getFieldInfo("__leonMetricStartDate");

			filterExpr = new LyFilterExpression(startDate, Condition.GREATER_OR_EQUALS, cal.getTime());
			filter.addElement(filterExpr);
		}

		Vector<LyGraphicalChartController>	chartControllers = getChartControllers(multiAreaController);

        for (LyGraphicalChartController gcController : chartControllers)
        {
        	LyAction					action = gcController.getAction();
        	LyChartInfo					chartInfo = (LyChartInfo)action.getParameter("_chartInfo");
        	LyPreference				pref = chartInfo.getPreference();
        	LyFilter					actionFilter = (pref == null ? null : pref.getFilter());
        	LySort						sort = (pref == null ? null : pref.getSort());

			globalFilter = new LyFilter(LyFilter.Type.AND);

			if (actionFilter != null)
				globalFilter.addElement(actionFilter);

			globalFilter.addElement(filter);

			LyResponse			resp = metricCls.getObjectSource().getObjects(sort, globalFilter, -1);

			gcController.setContext(resp.getObjects());

            LyWorkSpace	context = gcController.getContext();

			context.setSort(sort);
			gcController.reloadChartComponent();
        }
	}

	/**
	 * Retrieves all the graphical chart controllers of a given multi area controller.
	 */
	private Vector<LyGraphicalChartController> getChartControllers(LyMultiAreaController multiAreaController)
	{
		Vector<LyGraphicalChartController>	result = new Vector<LyGraphicalChartController>();

        for (int i = 0; i < multiAreaController.getAreaControllersCount(); i++)
        {
        	LyAreaController	areaController = multiAreaController.getAreaController(i);

        	if (areaController instanceof LyMultiAreaController)
        	{
        		Vector<LyGraphicalChartController>	subChartControllers = getChartControllers((LyMultiAreaController)areaController);

        		result.addAll(subChartControllers);
        	}
        	else if (areaController instanceof LyGraphicalChartController)
        		result.add((LyGraphicalChartController)areaController);
        }

		return result;
	}
}
