/*
 * LyPanelBehavior.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app.behavior;

import leon.app.behaviorinterface.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.view.*;


/**
 * This class implements the default behavior for actions that are controlled by a panel controller.
 *
 * @author  Lyria S.A.
 * @version $Id: LyPanelBehavior.java 28422 2010-11-03 12:03:38Z ehelmste $
 */
public abstract class LyPanelBehavior extends LyActionBehavior
	implements LyActionBehaviorInterface
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyPanelBehavior.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Construct a panel behavior.
	 */
	public LyPanelBehavior()
	{
	}

	/**
	 * Gets the ordered list of pairs of (action, classInfo) to display in a popup menu.<br />
	 * <i>The default implementation</i> does nothing and returns the given actions data list.
	 *
	 * @param  panelController Panel controller that displays the menu.
	 * @param  actions         List of pairs (action, classInfo) that the controller is about to
	 *                         present in popup menu on current selection.
	 * @param  selection       Boolean indicating if popup menu is asked or not on selection.
	 * @return List of pairs (action, classInfo) that the controller will display in menu.
	 */
	public LyInfoDataList<LyAction, LyClassInfo> getPopupMenuActions(LyPanelController panelController,
		LyInfoDataList<LyAction, LyClassInfo> actions, boolean selection)
	{
		return actions;
	}

	/**
	 * Completes a popup menu before displaying it.
	 * <i>The default implementation</i> does nothing and returns the popup menu built by the controller.
	 * 
	 * @param panelController Panel controller that displays the menu.
	 * @param menu            Popup menu built by the panel controller.
	 */
	public void completePopupMenu(LyPanelController panelController, LyPopupMenu menu)
	{
	}

	/**
	 * Checks if the panel needs a refresh when an object changed is received.
	 *
	 * @param  panelController Panel controller that received the object change.
	 * @param  object          The object that changed.
	 * @param  oldValues       The previous values.
	 * @return True if the panel has to be refreshed, false otherwise.
	 */
	public boolean needRefresh(LyPanelController panelController,
		LyObject object, LyValueSet oldValues)
	{
		return true;
	}
    
    /**
     * Gets title corresponding to the given action.<br />
     * <i>The default implementation</i> returns the action name.
     *
     * @param  areaController the current LyAreaController instance
     * @param  action         the action that was executed
     * @return the given action name
     */
    public String getTitle(LyAreaController areaController, LyAction action)
    {
        String          title = super.getTitle(areaController, action);
        LyPreference    preference = ((LyPanelController)areaController).getPreference();
        
        if (preference != null)
            title += " { " + preference.getName() + " }";
        return title;
    }
    
    /**
     * Method used to enable a filter.
     *
     * @param panelController Panel controller that asks for the filtering.
     * @param filter            The filter that is asked.
     * @return The enabled filter.
     */
    public LyFilter validateFilter(LyPanelController panelController, LyFilter filter)
    {
        return filter;
    }
} // end class LyPanelBehavior
