/*
 * LySendObjectBehavior.java
 * Copyright (c) 2000-2011, W4.
 * All rights reserved.
 */
package leon.app.behavior;

import java.util.*;

import leon.app.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;
import leon.view.event.*;

/**
 * Implements the send object action.
 * This class handles the build of the userInput and sends an object to the selected users.
 *
 */
public class LySendObjectBehavior extends LyUserInputBehavior
{
	/**
	 * Default constructor.
	 */
	public LySendObjectBehavior()
	{
		super();
	}
	
	@Override
	public String getTitle(LyAreaController areaController, LyAction action)
	{
		LyObject		sender = (LyObject)action.getParameter("_sender");
		Date			sendDate = (Date)action.getParameter("_date");
		LyEnvironment	env = areaController.getEnvironment();
		String			format = env.getStringDateFormat(LyDateFieldInfo.Type.DATE_TIME.ordinal());
		String			date = LyDateFormatter.format(sendDate, format);
		String			title = date + "<br />" + env.getMessage("LY_OBJECT_FROM", sender.getName());

		return title;
	}
	
	/**
	 * This method build dynamically the content of the message view.
	 * It always add a content text field
	 * If we cannot find users in the context of the controller, we add a relation field displaying currently
	 * connected users.
	 */
	public boolean startAction(LyAreaController areaController, LyAction action)
	{
		LyUserInputController	fileController = (LyUserInputController)areaController;
		LyFormField				field = new LyFormField("_object");
		LyCaption   			caption = new LyCaption();
		String					objectIdValue = (String)action.getParameter("_objectId");
		LyObjectId				objectId = new LyObjectId(areaController.getApplication(), objectIdValue);
		LyObject				object = objectId.getLogicalObject();
		
		caption.setLabel("LY_SEND_OBJECT_LINK");
		caption.setColon(true);
		
		LyHRef	href = new LyHRef();

		href.setId(object.getId());
		href.setLabel(object.getName());
		field.setCaption(caption);
		field.setFieldItem(href);
		
		if (fileController.getFormComponent() != null)
			fileController.getFormComponent().addFormField(field);
		
		return super.startAction(areaController, action);
	}
	
	@Override
	public boolean handleDemand(LyController controller, LyDemand demand)
	{
		if ((demand instanceof LyActionDemand) && (demand.getType() == LyActionDemand.TYPE_HREF))
		{
			String		value = ((LyActionDemand)demand).getValue();
			LyObjectId	objectId = new LyObjectId(controller.getApplication(), value);
			LyObject	object = objectId.getLogicalObject();
			
			if (object != null)
			{
				LyClassInfo	classInfo = object.getClassInfo();
				LyAction    consult = classInfo.getDefaultAction();

				if (consult == null)
					consult = classInfo.getActionWithId(LyAction.ID_CONSULT);

				if (consult != null)
				{
					LySession	session = controller.getSession();
					
					controller.getSession().invokeAction(session.getController(0), consult, classInfo, new LySimpleObjectList(object), true);
				}
			}
			
			controller.free();
			
			return true;
		}
		
		return super.handleDemand(controller, demand);
	}
}
