/**
  * LyWebServiceBehavior.java
  * Copyright (c) 2000-2009, Lyria-W4.
  * All rights reserved.
  */
  
package leon.app.behavior;

import java.util.Enumeration;
import java.util.Hashtable;

import leon.app.behaviorinterface.LyWebServicesBehaviorInterface;
import leon.control.LyController;
import leon.control.LyFormController;
import leon.control.LyUserInputController;
import leon.control.LyWebServiceController;
import leon.control.builder.LyWebServiceBuilder;
import leon.info.LyFieldInfo;
import leon.view.LyFieldItem;
import leon.view.LyFormField;
import leon.view.LyRelation;
import leon.view.event.LyDemand;

/**
 * The default behavior for web services
 * 
 *
 */
public class LyWebServiceBehavior extends LyUserInputBehavior implements LyWebServicesBehaviorInterface
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyWebServiceBehavior.java 28422 2010-11-03 12:03:38Z ehelmste $";
	
	LyWebServiceBuilder 	wsBuilder = null;
	LyWebServiceController	wsController = null;

	/**
	 * @see leon.app.behavior.LyBehavior#handleDemand(leon.control.LyController, leon.view.event.LyDemand)
	 */
	public boolean handleDemand(LyController controller, LyDemand demand)
	{
			boolean resp = super.handleDemand(controller, demand);
			if(demand.getSource().equals(LyFormController.DEMAND_VALIDATE))
			{
				LyUserInputController 		uic = (LyUserInputController)controller;
				Hashtable<String, String>	attrLst = wsController.getAttributesList();			
				LyFormField	 				attributField = null;
				
				if(attrLst != null)
				{
					Enumeration<String>		attrLst_ = attrLst.keys();
					
					while(attrLst_.hasMoreElements())
					{
						String 			clsName =  attrLst_.nextElement();
						
						attributField = (LyFormField)uic.getComponent().getComponent("parameter" + clsName);
						
						LyFieldItem 	item = attributField.getFieldItem();
						
						wsController.putAttributeValue(clsName, item.getValue());
					}
					
				}
				String operationName = ((LyFormField)uic.getComponent().getComponent("_selectionOperation")).getFieldItem().getValue();
				
				// invoke the selected Operation with the attribute list.
				wsController.invokeWebService(wsController.getAttributesList(), operationName);
			}
			return resp;
			
	}

	
	/**
	 * @see leon.app.behavior.LyUserInputBehavior#userInputDemand(leon.control.LyController, leon.view.event.LyDemand)
	 */	
	public void userInputDemand(LyController controller, LyDemand demand)
	{	
		if(wsBuilder==null)
		{
			wsBuilder = (LyWebServiceBuilder)controller.getBuilder();
		}
		if(wsController==null)
		{
			wsController = (LyWebServiceController)controller;
		}
		
		if(demand.getSource().equals("_selectionPorts"))
		{	
			String 					selectionPort = demand.getParameterValues().getParameter("_selectionPorts");
			
			wsController.selectPort(selectionPort);
			
			LyUserInputController 	uic = (LyUserInputController)controller;
			LyRelation				relation = (LyRelation)((LyFormField)uic.getComponent().getComponent("_selectionOperation")).getFieldItem();
			wsBuilder.removeLinksToRelation(relation);
			wsBuilder.addLinksToRelation(wsController.getListOfOperations(), relation, true);	
			wsBuilder.buildOperationAttributes();
		}
		
		if(demand.getSource().equals("_selectionOperation"))
		{
			String 	operation = demand.getParameterValues().getParameter("_selectionOperation");
			wsController.selectOperation(operation);
			wsBuilder.buildOperationAttributes();
		}
	}
	
	
	/**
	 * @see leon.app.behavior.LyUserInputBehavior#close(leon.control.LyController)
	 */
	public void close(LyController controller)
	{
		super.close(controller);
		wsController.freeClassInfo();
	}
	
	/**
	 * @see leon.app.behaviorinterface.LyWebServicesBehaviorInterface#updateFieldInfo(leon.info.LyFieldInfo)
	 */
	public LyFieldInfo updateFieldInfo(LyFieldInfo fieldInfo)
	{	
		return fieldInfo;
	}
	
	/**
	 * @see leon.app.behaviorinterface.LyWebServicesBehaviorInterface#getFieldInfoType(java.lang.String)
	 */
	public int getFieldInfoType(String name)
	{
		return LyWebServiceController.FIELD_TYPE_TEXT;
	}
}


