/*
 * LyBehaviorInterface.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app.behaviorinterface;

import leon.control.*;
import leon.view.event.*;


/**
 * Generic interface for all behaviors and behavior interfaces. Behaviors are used to implement a
 * specific behavior when the default one does not fullfill all the needs. Most of time, user
 * behavior inherits the default one, and just implement the specific parts.<br />
 * There are default behaviors for most of predefined actions.
 *
 * @author  Lyria S.A.
 * @version $Id: LyBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $
 */
public interface LyBehaviorInterface
{
	/**
	 * File version
	 */
	String __VERSION = "$Id: LyBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Information used within this behavior are initialized here. This method is called immediately
	 * after the behavior construction.
	 *
	 * @param controller The controller that requested the creation of this behavior.
	 */
	void init(LyController controller);

	/**
	 * Free this behavior. This method is provided to allow behavior to unregister itself from
	 * object sources or workspaces if needed. It should be called only by the controller that built
	 * this behavior.
	 */
	void free();

	/**
	 * Invoked when a demand is sent to this listener.
	 *
	 * @param  controller controller that routed the demand.
	 * @param  demand     demand parameters sent by the view component.
	 * @return Return true if the demand was handled, false otherwise. If false, the demand is
	 *         transmitted to the controller to try to handle it.
	 */
	boolean handleDemand(LyController controller, LyDemand demand);
}