/*
 * LyConsultBehaviorInterface.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app.behaviorinterface;

import leon.control.*;
import leon.data.*;
import leon.info.*;


/**
 * Generic interface for all behaviors that applies on consult action.
 *
 * @author  Lyria S.A.
 * @version $Id: LyConsultBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @see     LyConsultController
 */
public interface LyConsultBehaviorInterface extends LyFormBehaviorInterface
{
	/**
	 * File version
	 */
	String __VERSION = "$Id: LyConsultBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Gets action to use for consulting or editing the given relation with the specified class info
	 * selected on it.
	 *
	 * @param  consult   the current LyConsultController instance.
	 * @param  relation  Edited relation
	 * @param  classInfo Selected class info.
	 * @return The action to use, if null let controller decides.
	 */
	LyAction getAction(LyConsultController consult, LyRelationFieldInfo relation,
		LyClassInfo classInfo);

	/**
	 * Method called when object shown in the form is changed.
	 *
	 * @param consult the current LyConsultController instance.
	 * @param object  the object shown in the form.
	 */
	void setObject(LyConsultController consult, LyObject object);
    
    /**
     * Enables the given href object on the given relation field info.<br />
     * <i>The default implementation</i> always returns true.
     *
     * @param  consult  the current LyConsultController instance
     * @param  relation the relation that contains the href
     * @param  href     the href to enabled
     * @return true if the href is enabled on this relation, false otherwise
     */
    boolean enableRelationHRef(LyConsultController consult, LyRelationFieldInfo relation,
        LyObject href);
}