/*
 * LyDashboardBehaviorInterface.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app.behaviorinterface;

import leon.control.*;
import leon.info.*;
import leon.view.*;


/**
 * This interface defines the methods to implement in order to modify the default behavior of a
 * dashboard controller.
 *
 * @author  Lyria S.A.
 * @version $Id: LyDashboardBehaviorInterface.java 31729 2011-07-19 15:35:10Z jlsaouli $
 * @see     LyDashboardController
 */
public interface LyDashboardBehaviorInterface extends LyActionBehaviorInterface
{
	/**
	 * File version
	 */
	String __VERSION = "$Id: LyDashboardBehaviorInterface.java 31729 2011-07-19 15:35:10Z jlsaouli $";

	/**
	 * Indicates if the given sub view controller has to be freed when closed.
	 *
	 * @param      dashboard      Dashboard controller.
	 * @param      viewController View controller.
	 * @return     True if the given view controller is freed on closing, false otherwise.
	 * @deprecated Use {@link LyDashboardBehaviorInterface#isFreedOnClose(LyDashboardController,
	 *             LyAction, LyViewController)} instead.
	 * @see        LyDashboardBehaviorInterface#isFreedOnClose(LyDashboardController,LyAction,LyViewController)
	 */
	boolean isFreedOnClose(LyDashboardController dashboard, LyViewController viewController);

	/**
	 * Indicates if the given sub view controller has to be freed when closed.
	 *
	 * @param  dashboard      Dashboard controller.
	 * @param  action         The action of the sub view controller.
	 * @param  viewController View controller.
	 * @return True if the given view controller is freed on closing, false otherwise.
	 */
	boolean isFreedOnClose(LyDashboardController dashboard, LyAction action,
		LyViewController viewController);

	/**
	 * Indicates if the given action has to be opened in a new view.
	 *
	 * @param  dashboard Dashboard controller.
	 * @param  action    the action to be opened.
	 * @return True if the given action has to be opened in a new view, false otherwise.
	 */
	boolean openNewView(LyDashboardController dashboard, LyAction action);

	/**
	 * Method called when a new controller is (re)opened in the dasboard panel.
	 *
	 * @param dashboard      Dashboard controller.
	 * @param controller Area controller opened in the dashboard.
	 */
	void openController(LyDashboardController dashboard, LyAreaController controller);

	/**
	 * Method called when a controller is closed in the dasboard panel.
	 *
	 * @param dashboard      Dashboard controller.
	 * @param controller Area controller closed in the dashboard.
	 */
	void closeController(LyDashboardController dashboard, LyAreaController controller);
	
	/**
	 * Builds a tab for given area controller when the useTabs mode is active.
	 * 
	 * @param dashboard  Dashboard controller.
	 * @param controller Area controller opened in the dashboard.
	 * @return the newly created tab.
	 */
	LyTab buildTab(LyDashboardController dashboard, LyAreaController controller);
	
	/**
	 * Get the list of actions displayed in the dashboard.
	 *
	 * @param dashboard  Dashboard controller.
	 * @param actions List of actions declared in the model. Can be null.
	 * @return the actions list of the dashboard.
	 */
	LyInfoList<LyAction> getActions(LyDashboardController dashboard, LyInfoList<LyAction> actions);

	/**
	 * Get the classes list to be displayed in search tool.
	 * 	 
	 * @param dashboard  The dashboard controller.
	 * @return the classes list used for search, null to let the controller do the job.
	 */
	LyInfoList<LyClassInfo>	getSearchClasses(LyDashboardController dashboard);
}