/*
 * LyDeleteBehaviorInterface.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app.behaviorinterface;

import leon.control.*;
import leon.data.*;


/**
 * This interface defines the methods to implement in order to modify the behavior of a deletion
 * action.
 *
 * @author  Lyria S.A.
 * @version $Id: LyDeleteBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @see     leon.control.LyDeleteController
 */
public interface LyDeleteBehaviorInterface extends LyActionBehaviorInterface
{
	/**
	 * File version
	 */
	String __VERSION = "$Id: LyDeleteBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Checks if the given object deletion is enabled or not (used before asking to the end-user if
	 * he want to delete).
	 *
	 * @param  delete current LyDeleteController instance
	 * @param  object Object checked before deletion.
	 * @return Error message if deletion is not allowed, null otherwise.
	 */
	String checkDeletion(LyDeleteController delete, LyObject object);

	/**
	 * hecks if the given objects list deletion is enabled or not (used before asking to the
	 * end-user if he want to delete them).
	 *
	 * @param  delete current LyDeleteController instance
	 * @param  list   list of objects to checked before deletion.
	 * @return The list of allowed objects to delete.
	 */
	LyObjectList checkDeletions(LyDeleteController delete, LyObjectList list);

	/**
	 * Asks for the deletion message for the given object.
	 *
	 * @param  delete current LyDeleteController instance
	 * @param  object Object checked before deletion.
	 * @return Confirmation message to display to the user if necessary, null otherwise.
	 */
	String getConfirmDeletionMessage(LyDeleteController delete, LyObject object);

	/**
	 * Final step of object deletion process.
	 *
	 * @param  delete current LyDeleteController instance
	 * @return true if the deletion is complete, false otherwise
	 */
	boolean endDeletion(LyDeleteController delete);
}