/*
 * LyExportBehaviorInterface.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app.behaviorinterface;

import java.io.*;
import java.util.*;

import leon.control.*;
import leon.data.*;
import leon.info.*;


/**
 * This interface defines methods for the behavior of an export action.
 *
 * @author  Lyria S.A.
 * @version $Id: LyExportBehaviorInterface.java 40676 2012-09-05 16:27:18Z jlsaouli $
 * @see     leon.control.LyExportController
 */
public interface LyExportBehaviorInterface extends LyActionBehaviorInterface
{
	/**
	 * File version
	 */
	String __VERSION = "$Id: LyExportBehaviorInterface.java 40676 2012-09-05 16:27:18Z jlsaouli $";

    /**
     * Return the lines that will be written in the export file.
     *
     * @param  controller the current export controller
     * @param  objects    objects to export.
     * @param  fields     list of fieldInfos that should be exported to the file.
     * @param  fieldSep   separator between fields (columns)
     * @param  valueSep   separator between multiple values
     * @return the lines that will be written in the export file, null for standard export.
     * @param  exportType type of data to export : Possible values are SELECT_EXPORT_TYPE_TEXT
     *                    (.txt), SELECT_EXPORT_TYPE_RAW (.res) or SELECT_EXPORT_TYPE_SPREADSHEET
     *                    (.csv).
     */
    Vector<String> getExportLines(LyExportController controller, Iterator<LyObject> objects,
        Vector<LyFieldInfo> fields, String fieldSep, String valueSep, String exportType);
    
	/**
	 * Saves lines from the given list into the given file.
	 *
	 * @param  controller the current export controller
	 * @param  file       the file where data must be saved
	 * @param  lines      the list of lines to save
	 * @return true if file is save, false otherwise
	 */
	boolean saveFile(LyExportController controller, File file, Vector<String> lines);
	
    /**
     * Returns the header mark, as a String, to be used at the beginning of header line.
     * 
     * @param controller the export controller
     * @return the header mark
     */
	String getHeaderMark(LyExportController controller);

   /**
    * Indicates if the export form is displayed.<br />
    * Export form should not be displayed if all the fields values are given as
    * parameters of the export action.
    * 
    * @param controller the export controller
    * @return true if form is displayed, false otherwise.
    */
   public boolean isFormDisplayed(LyExportController controller);
}
