/*
 * LyFilterBehaviorInterface.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app.behaviorinterface;

import java.util.*;

import leon.control.*;
import leon.data.*;
import leon.info.*;


/**
 * This interface defines the methods to implement in order to modify the behavior of a filter
 * controller.
 *
 * @see LyFilterController
 */
public interface LyFilterBehaviorInterface extends LyBehaviorInterface
{
	/**
	 * File version
	 */
	String __VERSION = "$Id: LyFilterBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Gets relation domain (list of available links in the filter context) for the given relation
	 * fieldInfo for filter actions.
	 *
	 * @param  filter   the current LyFilterController instance
	 * @param  relation the target relation field
	 * @return a list of LyObjects available for this relation in filtering context.
	 */
	LyObjectList getFilterRelationDomain(LyFilterController filter,
		LyRelationFieldInfo relation);

	/**
	 * Gets the list of available class info for the given relation filtering.
	 *
	 * @param  filter   the current instance of "filter controller" This parameter could be both a
	 *                  LyFilterController or a LyFilterRelationController. That's why its type is
	 *                  LyFormController which is the common inheritance.
	 * @param  relation the target relation field
	 * @return a list of classInfo available for filtering the relation
	 */
	LyInfoList<LyClassInfo> getFilterRelationClasses(LyFormController filter, LyRelationFieldInfo relation);

	/**
	 * Gets all actions enabled by the given field for a filter action. A list of pairs (action,
	 * classInfo) has to be returned.
	 *
	 * @param  filter   the current LyFilterController instance
	 * @param  fieldInfo the field that may enabled some actions
	 * @return a list of actions enabled by the given field for a filter action
	 */
	LyInfoDataList<LyAction, LyClassInfo> getFilterTransverseActions(LyFilterController filter,
		LyFieldInfo fieldInfo);
	
	/**
	 * Gets all actions enabled by the given relation for a filter action. A list of pairs (action,
	 * classInfo) has to be returned.
	 *
	 * @param  filter   the current LyFilterController instance
	 * @param  relation the relation that may enabled some actions
	 * @return a list of actions enabled by the given relation for a filter action
	 */
	LyInfoDataList<LyAction, LyClassInfo> getFilterTransverseActions(LyFilterController filter,
		LyRelationFieldInfo relation);

	/**
	 * Gets domain (list of allowed options) of the given choice fieldInfo.
	 *
	 * @param  filter the current LyFilterController instance
	 * @param  choice target choice fieldInfo
	 * @return a list of LyChoiceOption allowed for this choice field info
	 */
	Vector<LyChoiceOption> getFilterChoiceDomain(LyFilterController filter,
		LyChoiceFieldInfo choice);

	/**
	 * Builds a filter element with the given parameters.
	 *
	 * @param  filter    the current LyFilterController instance
	 * @param  fieldInfo FieldInfo on which this filter expression applies.
	 * @param  not       Inverts filter condition or not.
	 * @param  condition Filter condition.
	 * @param  value     Value of this expression (may be null). This value is an instance of the
	 *                   class returned by the getValueClass method of LyFieldInfo.
	 * @param  modifiers List of modifiers (may be null).
	 * @return The corresponding filter element or null if nothing specific must be done
	 */
	LyFilterElement buildFilterElement(LyFilterController filter, LyFieldInfo fieldInfo,
		boolean not, LyFilterExpression.Condition condition, Object value, BitSet modifiers);
}