/*
 * LyFormBehaviorInterface.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app.behaviorinterface;

import leon.control.*;
import leon.data.*;
import leon.info.*;


/**
 * Generic interface for all behaviors that applies on form action.
 *
 * @see LyFormController
 */
public interface LyFormBehaviorInterface extends LyActionBehaviorInterface
{
	/**
	 * File version
	 */
	String __VERSION = "$Id: LyFormBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Gets the image associated to the given object for the given form.
	 *
	 * @param  formController the form controller
	 * @param  object         object whose image is looked for
	 * @return Logical name of the image.
	 */
	String getImage(LyFormController formController, LyObject object);

	/**
	 * Gets value of the given fieldInfo. This method must be efficient because it can be called
	 * frequently.
	 *
	 * @param  form      the current LyFormController instance.
	 * @param  fieldInfo the fieldInfo that contains the value
	 * @return the value of the given fieldInfo as an Object
	 */
	Object getValue(LyFormController form, LyFieldInfo fieldInfo);

	/**
	 * Indicates if the given fieldInfo is displayed in the given form or not.
	 *
	 * @param  form      the current LyFormController instance.
	 * @param  fieldInfo the fieldInfo which is tested for addition in the view.
	 * @return true if the field should be added, false if it should be skipped.
	 */
	boolean isDisplayed(LyFormController form, LyFieldInfo fieldInfo);
}