/*
 * LyImportTableBehaviorInterface.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.app.behaviorinterface;

import java.util.*;
import leon.control.*;
import leon.data.*;


/**
 * This interface defines the methods to implement in order to modify the behavior of an import
 * table action.
 *
 * @author  Lyria S.A.
 * @version $Id: LyImportTableBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @see     LyImportTableController
 */
public interface LyImportTableBehaviorInterface extends LyTableBehaviorInterface
{
	/**
	 * File version
	 */
	String __VERSION =
		"$Id: LyImportTableBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Checks if the import can be done with the given list of selected fields.
	 *
	 * @param  controller the current import table controller
	 * @param  objects    the list of objects we want to check
	 * @return an error message if fields are not OK, null if everything is fine
	 */
	String checkFields(LyImportTableController controller, LyObjectList objects);

	/**
	 * Returns the message to be displayed when some lines of the import file contain errors.
	 *
	 * @param  controller the current import table controller
	 * @param  errorLines a bitset with the indexes of lines containing errors.
	 * @return the message to be displayed
	 */
	String getErrorLinesMessage(LyImportTableController controller, BitSet errorLines);
}