/*
 * LyLoginBehaviorInterface.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app.behaviorinterface;

import leon.app.LySession;
import leon.control.*;
import leon.data.*;
import leon.view.event.*;


/**
 * This interface defines the methods to implement in order to manage the login of a user.
 *
 * @author  Lyria S.A.
 * @version $Id: LyLoginBehaviorInterface.java 36169 2012-07-04 07:12:20Z jlsaouli $
 * @see     LyLoginController
 */
public interface LyLoginBehaviorInterface extends LyActionBehaviorInterface
{
	/**
	 * File version
	 */
	String __VERSION = "$Id: LyLoginBehaviorInterface.java 36169 2012-07-04 07:12:20Z jlsaouli $";

	/**
	 * Login initialization of manipulated classInfo, loginFieldInfo and passwordFieldInfo.
	 *
	 * @param loginController current controller
	 */
	void initializeLogin(LyLoginController loginController);

	/**
	 * Complete login initialization with default values or extra fields adding for input.
	 *
	 * @param loginController current controller
	 */
	void completeInitializeLogin(LyLoginController loginController);

	/**
	 * Validate the login.
	 *
	 * @param      loginController current controller
	 * @param      login           value typed in by the user
	 * @param      password        value typed in by the user
	 * @param      values          extra values from the login controller validation.
	 * @return     true if login is ok, false for default login tries
	 * @deprecated since V1.2. Replaced by <code>validate(LyLoginController, String, String,
	 *             LyParameterValue, StringBuffer)</code>.
	 */
	boolean validate(LyLoginController loginController, String login, String password,
		LyParameterValues values);

	/**
	 * Validate the login: checks that one unique user exists for the given values.
	 *
	 * @param  loginController current controller
	 * @param  login           value typed in by the user
	 * @param  password        value typed in by the user
	 * @param  values          extra values from the login controller validation.
	 * @param  error           error message to show in case of invalid login. It is a result
	 *                         parameter.
	 * @return true if login is valid, false otherwise
	 */
	boolean validate(LyLoginController loginController, String login, String password,
		LyParameterValues values, StringBuffer error);

	/**
	 * Validate the login: checks that user is valid.
	 *
	 * @param  loginController current controller
	 * @param  user			   user that logged in.
	 * @param  values          extra values from the login controller validation.
	 * @param  error           error message to show in case of invalid login. It is a result
	 *                         parameter.
	 * @param  autoLogin	   boolean indicating if the call results from an auto-login.
	 * @return true if login is valid, false otherwise
	 */
	boolean validateUser(LyLoginController loginController, LyObject user, LyParameterValues values,
		StringBuffer error, boolean autoLogin);
	
	/**
	 * Ends the login.
	 *
	 * @param loginController current controller
	 * @param values          extra values from the login controller validation.
	 */
	void endLogin(LyLoginController loginController, LyParameterValues values);

	/**
	 * Gets the user associated to the given login and password values.
	 *
	 * @param  loginController current controller
	 * @param  values          values from the login controller.
	 * @return true if everything is done, false othewise and complete generic processing
	 */
	LyObject getUser(LyLoginController loginController, LyParameterValues values);
	
	/**
	 * Gets any existing session for the given user
	 * 
	 * @param  loginController current controller
	 * @param  currentUser          the current user
	 * @param  values the values for the current login view
	 * @return a session with the current user as the logged-in user.
	 */
	LySession getExistingSessionForUser(LyLoginController loginController, LyObject currentUser, LyParameterValues values);
	
	/**
	 * Returns a boolean value that indicates if the reconnect must be confirmed or not
	 * @param loginController the login controller
	 * @return true if the reconnection has to be confirmed by the user.
	 */
	boolean confirmReconnect(LyLoginController loginController);
}