/**
  * LyMailBehaviorInterface.java
  * Copyright (c) 2000-2009, Lyria-W4.
  * All rights reserved.
  */
  
package leon.app.behaviorinterface;

import leon.control.*;
import leon.data.*;
import leon.info.*;


/**
 * This interface defines methods for the behavior of a mail action.
 *
 * @author  Lyria S.A.
 * @version $Id: LyMailBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @see     leon.control.LyMailController
 */
public interface LyMailBehaviorInterface extends LyUserInputBehaviorInterface
{
	/**
	 * File version
	 */
	String __VERSION = "$Id: LyMailBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Sends a mail message. This method is called when the selected objects are sent.
	 *
	 * @param  controller      the mail controller
	 * @param  toClass         the class that contains the email address field of the email message
	 *                         receivers.
	 * @param  toField         the class field that contains the email address of the receivers.
	 * @param  selectedObjects the objects that are sent to the given receivers as the message text
	 * @return true if the message has been correctly sent through the given server, false otherwise
	 */
	boolean sendMail(LyMailController controller, LyClassInfo toClass, LyFieldInfo toField,
		LyObjectList selectedObjects);

	/**
	 * Sends a mail message. This method is called when the selected objects describe the email
	 * receivers.
	 *
	 * @param  controller    the mail controller
	 * @param  selectedUsers the list of recipient users for a custom mail
	 * @param  toField       a field for all the selected users that holds their email addresses
	 * @return true if the message has been correctly sent through the given server, false otherwise
	 */
	boolean sendMail(LyMailController controller, LyObjectList selectedUsers, LyFieldInfo toField);
}