/*
 * LyPanelBehaviorInterface.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app.behaviorinterface;

import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.view.*;


/**
 * This interface defines the methods to implement in order to modify the default behavior of a
 * panel controller.
 *
 * @author  Lyria S.A.
 * @version $Id: LyPanelBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @see     leon.control.LyPanelController
 */
public interface LyPanelBehaviorInterface extends LyActionBehaviorInterface
{
	/**
	 * File version
	 */
	String __VERSION = "$Id: LyPanelBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Gets the ordered list of pairs of (action, classInfo) to display in a popup menu.<br />
	 * <i>The default implementation</i> does nothing and returns the given actions data list.
	 *
	 * @param  panelController Panel controller that displays the menu.
	 * @param  actions         List of pairs (action, classInfo) that the controller is about to
	 *                         present in popup menu on current selection.
	 * @param  selection       Boolean indicating if popup menu is asked or not on selection.
	 * @return List of pairs (action, classInfo) that the controller will display in menu.
	 */
	LyInfoDataList<LyAction, LyClassInfo> getPopupMenuActions(LyPanelController panelController,
		LyInfoDataList<LyAction, LyClassInfo> actions, boolean selection);

	/**
	 * Completes a popup menu before displaying it.
	 * <i>The default implementation</i> does nothing and returns the popup menu built by the controller.
	 * 
	 * @param panelController Panel controller that displays the menu.
	 * @param menu            Popup menu built by the panel controller.
	 */
	void completePopupMenu(LyPanelController panelController, LyPopupMenu menu);
	
	/**
	 * Checks if the panel needs a refresh when an object changed is received. This method is given
	 * to enable optimization when object changes are received.
	 *
	 * @param  panelController Panel controller that received the object change.
	 * @param  object          The object that changed.
	 * @param  oldValues       The previous values.
	 * @return True if the panel has to be refreshed, false otherwise.
	 */
	boolean needRefresh(LyPanelController panelController,
		LyObject object, LyValueSet oldValues);
    
    /**
     * Method used to enable a filter.
     *
     * @param panelController Panel controller that asks for the filtering.
     * @param filter            The filter that is asked.
     * @return The enabled filter.
     */
    LyFilter validateFilter(LyPanelController panelController, LyFilter filter);
}