/*
 * LyPasswordBehaviorInterface.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app.behaviorinterface;

import leon.control.*;
import leon.view.event.*;


/**
 * This interface defines the method to implement in order to manage a view that allows a password
 * modification.
 *
 * @author  Lyria S.A.
 * @version $Id: LyPasswordBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @see     leon.control.LyPasswordController
 */
public interface LyPasswordBehaviorInterface extends LyActionBehaviorInterface
{
	/**
	 * File version
	 */
	String __VERSION = "$Id: LyPasswordBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Password initialization of the manipulated classInfo and passwordFieldInfo.
	 *
	 * @param passwordController current controller
	 */
	void initializePassword(LyPasswordController passwordController);

	/**
	 * Complete password initialization with default values or extra fields adding for input.
	 *
	 * @param passwordController current controller
	 */
	void completeInitializePassword(LyPasswordController passwordController);

	/**
	 * Method invoked to change the given current password with the given new password value.
	 * Specific implementation may add extra rules here.<br />
	 *
	 * @param  passwordController current controller
	 * @param  currentPassword    current password for validation.
	 * @param  newPassword        new password value typed in by the user
	 * @param  confirmPassword    confirm password request: should equals newPassword
	 * @param  values             user input values of the current, new and confirmed passwords
	 * @return true if action completed successfully, false if password is unchanged
	 */
	boolean changePassword(LyPasswordController passwordController, String currentPassword,
		String newPassword, String confirmPassword, LyParameterValues values);
}