/*
 * LySessionListenerInterface.java
 * Copyright (c) 2000-2011, Lyria-W4.
 * All rights reserved.
 */

package leon.app.behaviorinterface;

import leon.app.*;
import leon.data.*;


/**
 * This interface defines methods to listen session creation/deletion.
 */
public interface LySessionListenerInterface
{
	/**
	 * Method called when a session is added to the application.
	 *
	 * @param application Application in which a session is added.
	 * @param session     Session added to the application.
	 */
	void sessionAdded(LyApplication application, LyDataSession session);
	
	/**
	 * Method called when a user logged in a session.
	 *
	 * @param application Application in which a session is added.
	 * @param session     Session added to the application.
	 * @param user		  User that logged in.
	 */
	void sessionLoggedIn(LyApplication application, LyDataSession session, LyObject user);

	/**
	 * Method called when a session is removed from the application.
	 *
	 * @param application Application in which a session is removed.
	 * @param session     Session removed from the application.
	 */
	void sessionRemoved(LyApplication application, LyDataSession session);
}