/*
 * LyShowUrlBehaviorInterface.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app.behaviorinterface;

import java.io.*;
import java.net.*;
import leon.control.*;
import leon.misc.*;


/**
 * This interface defines the methods to implement in order to modify the behavior of a text action.
 *
 * @author  Lyria S.A.
 * @version $Id: LyShowUrlBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @see     leon.control.LyTextController
 */
public interface LyShowUrlBehaviorInterface extends LyActionBehaviorInterface
{
	/**
	 * File version
	 */
	String __VERSION = "$Id: LyShowUrlBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Gets the Input Stream associated to the given file name.
	 *
	 * @param  controller  the current editor controller
	 * @param  fileName    the file to read
	 * @param  environment the current environment
	 * @param  showError   if true, show errors thrown to the user
	 * @return the input stream of the file if found, null otherwise
	 */
	InputStream getFileInputStream(LyShowUrlController controller, String fileName,
		LyEnvironment environment, boolean showError);

	/**
	 * Gets the URL associated to the given file name.
	 *
	 * @param  controller  the current editor controller
	 * @param  fileName    the file to read
	 * @param  environment the current environment
	 * @param  showError   if true, show errors thrown to the user
	 * @return the URL of the file if found, null otherwise
	 */
	URL getFileUrl(LyShowUrlController controller, String fileName, LyEnvironment environment,
		boolean showError);

	/**
	 * Indicates whether the action should open a new window or not.
	 * 
	 * @param controller  the current editor controller
	 * @return true if a new window should be opened, false otherwise
	 */
	boolean openNewWindow(LyShowUrlController controller);
}