/*
 * LyTreeBaseBehaviorInterface.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app.behaviorinterface;

import leon.control.*;
import leon.data.*;
import leon.view.*;


/**
 * This interface defines the methods to implement in order to change the default behavior for tree
 * controllers.
 *
 * @author  Lyria S.A.
 * @version $Id: LyTreeBaseBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @see     leon.control.LyTreeBaseController
 * @see     leon.control.LyTreeController
 */
public interface LyTreeBaseBehaviorInterface extends LyPanelBehaviorInterface
{
	/**
	 * File version
	 */
	String __VERSION = "$Id: LyTreeBaseBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Gets the tree root node.
	 *
	 * @param  controller current LyTreeController instance
	 * @return the object shown as the root of the tree, or null if the tree is empty
	 */
	Object getTreeRootObject(LyTreeBaseController controller);

	/**
	 * Build a view object instance from the given application object. Node status (colors, borders,
	 * ...) should be set in {@link #updateTreeNode(Object, LyTreeNode, LyTreeBaseController)}.
	 *
	 * @param  nodeObject the node to build, usually {@link LyObject} instance but not mandatory
	 * @param  controller current LyTreeController instance
	 * @return the built node view object.
	 */
	LyTreeNode buildTreeNode(Object nodeObject, LyTreeBaseController controller);

	/**
	 * Updates the given tree node view object status (colors, label, border, style ...) from the
	 * gicen node object properties. The controller invokes this method when the given nodeObject is
	 * modified.
	 *
	 * @param nodeObject application object (reference)
	 * @param treeNode   view objects (need update)
	 * @param controller current tree controller instance
	 */
	void updateTreeNode(Object nodeObject, LyTreeNode treeNode,
		LyTreeBaseController controller);

	/**
	 * Method invoked when a validity changed in a three state tree (valid, invalid, partially).
	 * <br />
	 * <u>Note:</u> a tree is activated as a three state validity tree by the resource :
	 * LY_TREE_VALIDITY=true.
	 *
	 * @param treeNode   view objects (need update)
	 * @param controller current LyTreeBaseController instance
	 */
	void validityModified(LyTreeNode treeNode, LyTreeBaseController controller);

	/**
	 * Method invoked at the beginning of the given object drag.
	 *
	 * @param treeController Tree controller that displays the tree
	 * @param sourceObject   dragged object
	 */
	void dragStart(LyTreeController treeController, LyObject sourceObject);

	/**
	 * Method called when the given dragged object is dropped on the given dropped object.
	 *
	 * @param treeController current LyTreeController instance
	 * @param targetObject   object on where the dragged object is dropped
	 * @param sourceObject   the dragged object
	 */
	void drop(LyTreeController treeController, LyObject targetObject, LyObject sourceObject);

	/**
	 * Method called at the end of a drag-and-drop.<br />
	 * <i>The default implementation</i> does nothing.
	 *
	 * @param treeController current LyMapController instance
	 * @param type           drag-and-drop type ({@link org.eclipse.swt.dnd.DND#DROP_COPY} or {@link
	 *                       org.eclipse.swt.dnd.DND#DROP_MOVE})
	 */
	void dragFinished(LyTreeController treeController, int type);

}