/*
 * LyWaitBehaviorInterface.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.app.behaviorinterface;

import leon.control.*;


/**
 * This interface defines the methods to implement in order to have a behavior that controls the
 * evolution of the progress bar, the label, and the icon associated to a wait view shown while a
 * time consumming action is performed.
 *
 * @author  Lyria S.A.
 * @version $Id: LyWaitBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @see     leon.control.LyWaitController
 */
public interface LyWaitBehaviorInterface extends LyActionBehaviorInterface
{
	/**
	 * File version
	 */
	String __VERSION = "$Id: LyWaitBehaviorInterface.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Gets the current progress value. This method is invoked when the view is updated.
	 *
	 * @param  controller the current LyWaitController instance
	 * @return the current progress value
	 */
	int getWaitValue(LyWaitController controller);

	/**
	 * Gets the current label for the waiting view. This method is invoked when the view is updated.
	 * May update the label of the waiting view.
	 *
	 * @param  controller the current LyWaitController instance
	 * @return the label or null if the label should not change
	 */
	String getWaitLabel(LyWaitController controller);

	/**
	 * Gets the current icon for the waiting view. This method is invoked when the view is updated.
	 * May update the icon of the waiting view.
	 *
	 * @param  controller the current LyWaitController instance
	 * @return the current icon or null if the current icon should not change
	 */
	String getWaitIcon(LyWaitController controller);

	/**
	 * Increases the current offset. This method is used to simulate smoth progression of the bar.
	 *
	 * @param controller the current LyWaitController instance
	 */
	void increaseOffset(LyWaitController controller);

	/**
	 * Method called when the waiting starts. May be overriden to do extra work.
	 *
	 * @param controller current LyWaitController instance
	 */
	void startWaiting(LyWaitController controller);

	/**
	 * Method invoked at the end of the wait, when it is over. Should complete progress bar because
	 * the controller has ended the progression. May do extra work here, or close the view.<br />
	 * <i>The default implementation</i> wait until the minimum display time is over (defined by
	 * MIN_DURATION).
	 *
	 * @param controller the current LyWaitController instance
	 */
	void endWaiting(LyWaitController controller);
}