/**
 * LyAreaController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyCanvasController.java 28997 2010-12-23 10:17:15Z bmichel $
 */

package leon.control;

import leon.app.*;
import leon.app.behavior.*;
import leon.control.builder.*;
import leon.data.*;
import leon.info.*;
import leon.view.*;


/**
 * A canvas controller is the class used to integrate an external drawing area.
 */
public class LyCanvasController extends LyPanelController
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyCanvasController.java 28997 2010-12-23 10:17:15Z bmichel $";

	/**
	 * Constructor.
	 *
	 * @param session current session associated to the controller.
	 * @param parent  parent controller.
	 * @param action  related action.
	 * @param classInfo related classinfo
	 * @param context current context (may be null if the view is not contextual to a list of
	 *                objects).
	 */
	public LyCanvasController(LySession session, LyController parent,
		LyAction action, LyClassInfo classInfo, LyObjectList context)
	{
		super(session, parent, action, context);
	}

	/**
	 * Gets default builder associated to this controller.
	 *
	 * @return default builder.
	 * @see    leon.control.builder.LyCanvasBuilder
	 */
	public LyBuilder getDefaultBuilder()
	{
		return new LyCanvasBuilder(this);
	}

	/**
	 * Method that gives the interface that must be implemented by the behavior associated to this
	 * controller.
	 *
	 * @return Class representing the behavior interface of this controller.
	 */
	public Class<?> getBehaviorInterface()
	{
		return LyCanvasBehavior.class;
	}

	/**
	 * Gets selections.
	 *
	 * @return List of objects selected in canvas.
	 */
	public LyObjectList getSelection()
	{
		return null;
	}

	/**
	 * Gets content.
	 *
	 * @return List of objects contained in canvas.
	 */
	public LyObjectList getContent()
	{
		return null;
	}

	/**
	 * Accessor to panel component.
	 * @return the panel component
	 */
	public LyPanel getPanelComponent()
	{
		return (LyPanel)getComponent();
	}
} // end class LyCanvasController
