/**
 * LyChartController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyChartController.java 40675 2012-09-05 16:26:11Z jlsaouli $
 */

package leon.control;

import java.text.*;
import java.util.*;
import leon.app.*;
import leon.app.behaviorinterface.*;
import leon.control.builder.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;
import leon.view.event.*;
import leon.view.getinterface.*;


/**
 * This class implements the parameter chart edition forms. It implements a 'wizard' style
 * controller for navigation between form page. At end, it invokes a graphical chart controller.
 *
 * @see LyFormController
 * @see LyGraphicalChartController
 */
public class LyChartController extends LyFormController
{
	/**
	 * File version
	 */
	public static final String			 __VERSION =
		"$Id: LyChartController.java 40675 2012-09-05 16:26:11Z jlsaouli $";

	/** Width of the window. */
	public static int					 DEFAULT_WIDTH = 620;

	/** Height of the window. */
	public static int					 DEFAULT_HEIGHT = 460;

	/* Demand codes */

	/** Show previous window. */
	public static final String			 DEMAND_MODIFY_STEP = "_modifyStep";

	/** Input in the first window. */
	public static final String			 SELECT_Y_FIELDS = "_chartyfields";

	/** Input in the first window. */
	public static final String			 SELECT_X_FIELD = "_chartxfield";

	/** Input in the first window. */
	public static final String			 SELECT_GRAPH_TYPE = "_chartgraphtype";

	/** Input in the second window. */
	public static final String			 X_TYPE_VALUES = "_xtypeval";

	/** Input in the second window. */
	public static final String			 Y_TYPE_VALUES = "_ytypeval";

	/** Input in the second window. */
	public static final String			 SELECT_GROUP_FIELD = "_chartGroupField";

	/** Input in the second window. */
	public static final String			 LG_INT_X_AREA = "_lgintx";

	/** Input in the second window. */
	public static final String			 LG_INT_Y_AREA = "_lginty";

	/** Input in the second window. */
	public static final String			 Y_TYPE_INTS = "_ytypeints";

	/** Input in the second window. */
	public static final String			 X_TYPE_INTS = "_xtypeints";

	/** Input in the second window. */
	public static final String			 BORDER_INT_X = "_bordintx";

	/** Input in the second window. */
	public static final String			 BORDER_INT_Y = "_bordinty";

	/** Input in the second window. */
	public static final String			 PIES_BY = "_piesby";

	/** Input in the second window. */
	public static final String			 PIES_BY_X = "_piesByX".intern();

	/** Input in the second window. */
	public static final String			 PIES_BY_Y = "_piesByY".intern();

	/** Input in the third window. */
	public static final String			 TITLE_AREA = "_charttitle";

	/** Input in the third window. */
	public static final int				 TITLE_WIDTH = 50;

	/** Input in the third window. */
	public static final int				 TITLE_HEIGHT = 1;

	/** Input in the third window. */
	public static final int				 AXIS_TITLES_WIDTH = 50;

	/** Input in the third window. */
	public static final String			 X_TITLE = "_xtitle";

	/** Input in the third window. */
	public static final String			 Y_TITLE = "_ytitle";

	/** Input in the third window. */
	public static final String			 X_MIN_AXIS = "_xMinAxis";

	/** Input in the third window. */
	public static final String			 X_MAX_AXIS = "_xMaxAxis";

	/** Input in the third window. */
	public static final String			 X_GRAD_AXIS = "_xGradAxis";

	/** Input in the third window. */
	public static final String			 Y_MIN_AXIS = "_yMinAxis";

	/** Input in the third window. */
	public static final String			 Y_MAX_AXIS = "_yMaxAxis";

	/** Input in the third window. */
	public static final String			 Y_GRAD_AXIS = "_yGradAxis";

	/** Input in the third window. */
	public static final String			 CHART_DISPLAY_OPTIONS = "_dispOptions";

	/** Current chart info. */
	protected LyChartInfo				 _chartInfo;

	/** Controller of the 4th window. */
	protected LyGraphicalChartController _graphicalChartController = null;

	/** Builder of the 2nd window. */
	protected _LyChartWin2Builder		 _win2Builder = null;

	/** Builder of the 3rd window. */
	protected _LyChartWin3Builder		 _win3Builder = null;

	/** Action that must be used when showing graphical chart. */
	protected LyAction					 _graphicalChartAction = null;

	/** The number of the current step (window). */
	private short						 _step = 1;

	/** Type of Selected Y Var(s). */
	private LyChartInfo.Type       		 _typexml;

	/** Maximum number of decimals. */
	private short						 _ydec, _xdec;

	/** Determine if intervals are input. */
	private boolean						 _x_Interval;

	/** Determine if intervals are input. */
	private boolean						 _y_Interval;

	/** The Second panel in the area. */
	private LyPanel						 _panel2 = null;

	/** The Third form in the area. */
	private LyPanel						 _panel3 = null;

	/**
	 * Determine if it's the first time the third parameter window is opened, and the chart is not
	 * predefined.
	 */
	private boolean						 _first3 = true;

	/**
	 * Determine if the X axis title has to be generated by the third window. It is set to true when
	 * X variable changes (different from _chartInfo). It is set to false when X variable doesn't
	 * change in the first window, and when leaving third window.
	 */
	private boolean						 _defaultXAxisTitle = true;

	/**
	 * Determine if the Y axis title has to be generated by the third window. It is set to true when
	 * Y variables change (different from _chartInfo). It is set to false when Y variables don't
	 * change in the first window, and when leaving third window.
	 */
	private boolean						 _defaultYAxisTitle = true;

	/**
	 * Default constructor.
	 *
	 * @param session   current session associated to the controller. The controller automatically
	 *                  registers on the specified session.
	 * @param parent    parent controller (or null for 'root' controller). Controller hierarchy
	 *                  indicates a child controller (for example a dialog created from a main view)
	 * @param action    related action. If an action is specified, the controller uses action
	 *                  parameters (action name, action specific XML view description ...)
	 * @param classInfo always the classInfo of the specified objects.
	 * @param objects   current context (not null because the view is contextual to a list of
	 *                  objects).
	 */
	public LyChartController(LySession session, LyController parent, LyAction action,
		LyClassInfo classInfo, LyObjectList objects)
	{
		this(session, parent, action, classInfo, objects, null);

		// Use default graphical action
		LyApplication application = session.getApplication();

		_graphicalChartAction = application.getAction("_graphicalChart");
	}

	/**
	 * Constructor.
	 *
	 * @param session   current session associated to the controller. The controller automatically
	 *                  registers on the specified session.
	 * @param parent    parent controller (or null for 'root' controller). Controller hierarchy
	 *                  indicates a child controller (for example a dialog created from a main view)
	 * @param action    related action. If an action is specified, the controller uses action
	 *                  parameters (action name, action specific XML view description ...)
	 * @param classInfo always the classInfo of the specified objects.
	 * @param objects   current context (not null because the view is contextual to a list of
	 *                  objects).
	 * @param chartInfo the chartInfo of the predefined chart.
	 */
	public LyChartController(LySession session, LyController parent, LyAction action,
		LyClassInfo classInfo, LyObjectList objects, LyChartInfo chartInfo)
	{
		super(session, parent, action, classInfo, objects);

		if (_classInfo != classInfo)
		{
			// If the classInfo given as parameter is not the current one,
			// select all the objects corresponding to it.
			LyWorkSpace parentContext = null;

			if (parent instanceof LyContextInterface)
				parentContext = ((LyContextInterface)parent).getContext();

			if (getBehavior() instanceof LyChartBehaviorInterface)
				objects = ((LyChartBehaviorInterface)getBehavior()).getChartContext(this, session,
						getClassInfo(), objects, parentContext);
		}

		if (objects != null)
			setContext(objects);

		// if chartInfo is null, try to recover the chartInfo of the action (if there is one)
		if (null == chartInfo)
		{
			if ((LyChartInfo)action.getParameter("_chartInfo", true) != null)
			{
				LyChartInfo initChartInfo = (LyChartInfo)action.getParameter("_chartInfo", true);

				_chartInfo = initChartInfo.cloneChartInfo(initChartInfo.getId());
			}
			else
			{
				String id = ((_parent == null) ? ("_chartInfo" + LyTools.nextInt()) : _parent.getId());

				_chartInfo = new LyChartInfo(id, getClassInfo().getName(), null);
				_chartInfo.setApplication(getApplication());
			}
		}
		else
			_chartInfo = chartInfo;

		// Fix the dimensions of the window
		LyView view = getView();

		view.setSize(DEFAULT_WIDTH, DEFAULT_HEIGHT);
	}

	/**
	 * Constructor.
	 *
	 * @param session                  current session associated to the controller. The controller
	 *                                 automatically registers on the specified session.
	 * @param parent                   parent controller (or null for 'root' controller). Controller
	 *                                 hierarchy indicates a child controller (for example a dialog
	 *                                 created from a main view)
	 * @param action                   related action. If an action is specified, the controller
	 *                                 uses action parameters (action name, action specific XML view
	 *                                 description ...)
	 * @param classInfo                always the classInfo of the specified objects.
	 * @param objects                  current context (not null because the view is contextual to a
	 *                                 list of objects).
	 * @param chartInfo                the chartInfo of the predefined chart.
	 * @param graphicalChartController the controller of the predefined chart.
	 */
	public LyChartController(LySession session, LyController parent, LyAction action,
		LyClassInfo classInfo, LyObjectList objects, LyChartInfo chartInfo,
		LyGraphicalChartController graphicalChartController)
	{
		this(session, parent, action, classInfo, objects, chartInfo);

		_graphicalChartController = graphicalChartController;

		// if the chart is predefined, it's like if the user had ever filled the parameter windows
		if (_graphicalChartController != null)
			_first3 = false;
	}

	/**
	 * Constructor.
	 *
	 * @param session   current session associated to the controller. The controller automatically
	 *                  registers on the specified session.
	 * @param parent    parent controller (or null for 'root' controller). Controller hierarchy
	 *                  indicates a child controller (for example a dialog created from a main view)
	 * @param action    related action. If an action is specified, the controller uses action
	 *                  parameters (action name, action specific XML view description ...)
	 * @param classInfo always the classInfo of the specified objects.
	 */
	public LyChartController(LySession session, LyController parent,
		LyAction action, LyClassInfo classInfo)
	{
		// TODO : optimize the loaded fieldinfos.
		this(session, parent, action, classInfo,
			((parent instanceof LyAreaController)
				? ((LyAreaController)parent).getContent()
				: new LyClassWorkSpace(session, classInfo, false)));
	}

	/**
	 * Constructor.
	 *
	 * @param session current session associated to the controller. The controller automatically
	 *                registers on the specified session.
	 * @param parent  parent controller (or null for 'root' controller). Controller hierarchy
	 *                indicates a child controller (for example a dialog created from a main view)
	 * @param action  related action. If an action is specified, the controller uses action
	 *                parameters (action name, action specific XML view description ...)
	 */
	public LyChartController(LySession session, LyController parent, LyAction action)
	{
		this(session, parent, action, action.getClassInfo());
	}

	/**
	 * Gets the current LyChartInfo.
	 *
	 * @return the current LyChartInfo.
	 */
	public LyChartInfo getChartInfo()
	{
		return _chartInfo;
	}

	/**
	 * Set the action that must be used when showing graphical chart.
	 *
	 * @param action the action
	 */
	public void setGraphicalChartAction(LyAction action)
	{
		_graphicalChartAction = action;
	}

	/**
	 * Method that gives the interface that must be implemented by the behavior associated to this
	 * controller.
	 *
	 * @return Class representing the behavior interface of this controller.
	 */
	public Class<?> getBehaviorInterface()
	{
		return LyChartBehaviorInterface.class;
	}

	/**
	 * Indicates if the view for this chart controller is resizable. A call to isViewResizable() is
	 * done on the chart behavior.
	 *
	 * @return true if the view is resizable, false otherwise.
	 */
	public boolean isViewResizable()
	{
		LyChartBehaviorInterface behavior = (LyChartBehaviorInterface)getBehavior();

		if (null != behavior)
			return behavior.isViewResizable(this);
		else
			return false;
	}

	/**
	 * Invoked when an action occurs. (LyDemandListener interface)
	 *
	 * @param demand the type of demand
	 */
	public void actionDemand(LyActionDemand demand)
	{
		// Update the last modification time of the controller.
		setLastModified(new Date().getTime());

		if (demand.getType() == LyActionDemand.TYPE_VALUE_CHANGED)
		{
			if (_step == 1)
				demandValueChanged1();
			else if (_step == 2)
				demandValueChanged2();
			return;
		}
		else if (demand.getType() == LyActionDemand.TYPE_ACTION)
		{
			String id = demand.getSource();

			if (id.equals(DEMAND_NEXT_STEP))
			{
				switch (_step)
				{
				case 1:
					demandNext1();
					return;
				case 2:
					demandNext2();
					return;
				case 3:
					demandNext3();
					return;
				case 4:
					demandNext3();
					return;
				}
			}
			else if (id.equals(DEMAND_PREVIOUS_STEP))
			{
				switch (_step)
				{
				case 1:
					break;
				case 2:
					demandPrevious2();
					return;
				case 3:
					demandPrevious3();
					return;
				case 4:
					demandPrevious3();
					return;
				}
			}
		}

		super.actionDemand(demand);
	} // end method actionDemand

	/**
	 * Free this controller on request of the specified controller.
	 *
	 * @param controller controller instance which requested the freeing of this controller. If the
	 *                   controller is the parent, this controller is freed otherwise the method
	 *                   does nothing. If the LyGraphicalController has a MODIFY nutton, enable it.
	 * @see   #free()
	 */
	public void free(LyController controller)
	{
		if (_graphicalChartController != null)
		{
			if ((null != _graphicalChartController.getBuilder())
				&& (null != _graphicalChartController.getBuilder().getComponent())
				&& (null != ((LyArea)_graphicalChartController.getBuilder().getComponent()) .getCommands())
				&& (0 < ((LyArea)_graphicalChartController.getBuilder().getComponent()).getCommands().getCommandCount()))
			{
				// enable the graphicalChart modify button if there is one
				LyCommands commands = ((LyArea)_graphicalChartController.getBuilder().getComponent()).getCommands();
				LyCommand  command = commands.getCommand(0);

				if (command.getId() == DEMAND_MODIFY_STEP)
					command.setEnabled(true);
			}

			_graphicalChartController._chartController = null;
		}

		super.free(controller);

		_chartInfo = null;
	} // end method free

	/**
	 * Demand to display the window after the first one, if there is no error.
	 */
	protected void demandNext1()
	{
		saveWin1();

		boolean verify = validateWin1();

		if (verify)
		{
			resetArea2();
			_step = 2;
		}
	}

	/**
	 * Demand to display the window after the second one, if there is no error.
	 */
	protected void demandNext2()
	{
		saveWin2();
		boolean verify = validateWin2();

		if (verify)
		{
			resetArea3();
			_step = 3;
		}
	}

	/**
	 * Demand to display the window after the third one.
	 */
	protected void demandNext3()
	{
		saveWin3();

		boolean verify = validateWin3();

		if (verify)
		{
			resetArea4();

			// Hide the current view.
			hideView();
			_graphicalChartController.showView();

			_step = 4;
		}
	}

	/**
	 * Demand to display the window before the second one.
	 */
	protected void demandPrevious2()
	{
		saveWin2();
		LyArea  area = (LyArea)((LyChartBuilder)_builder).getComponent();
//              LyForm          form1 = (LyForm) ((LyChartBuilder)_builder)._form1;
		LyArea  area1 = ((LyChartBuilder)_builder)._area1;
		LyPanel panel2 = area.getPanel(0);

		area.removePanel(panel2);
		area.addPanel(area1);
		LyCommands commands = area.getCommands();

		commands.getCommand(0).setEnabled(false);
		_step = 1;
	}

	/**
	 * Demand to display the window before the third one.
	 */
	protected void demandPrevious3()
	{
		saveWin3();
		LyArea  area = (LyArea)((LyChartBuilder)_builder).getComponent();
		LyPanel panel3 = area.getPanel(0);

		area.removePanel(panel3);
		area.addPanel(_panel2);
		_step = 2;
	}

	/**
	 * Demand to display the window before the fourth one.
	 */
	protected void demandPrevious4()
	{
		_step = 3;

		_graphicalChartController.free();
		_graphicalChartController = null;
		
		showView();

		// Update the last modification time of the controller.
		setLastModified(new Date().getTime());
	}

	/**
	 * Invoked when a value is changed in the first window.
	 */

	protected void demandValueChanged1()
	{
		LyFormField formField = getFormField(LyChartController.SELECT_X_FIELD);

		formField.setEditable(true);
		formField.setEnabled(true);
		if ((null != getSelectedYFields()) && (null != getSelectedYFields().elementAt(1)))
		{
			Vector<?> selectedYFields = (Vector<?>)getSelectedYFields().elementAt(1);
			// Y contains dates : disable "function of" variable and count.
			if ((null != selectedYFields)
					&& (_chartInfo.getTypeVarY() == LyChartInfo.Type.DATE))
			{
				formField.setEditable(false);
				formField.setEnabled(false);
			}
		}
	}

	/**
	 * Invoked when a value is changed in the second window.
	 */
	protected void demandValueChanged2()
	{
		LyFormField myFormField;

		// if real values are choosen, forbid entering intervals
		// only if intervals are printed (ie the variable is not text)
		// for x values
        String	    selectedVal = getChoosenOption((LyFormField)_panel2.getComponent(
					LyChartController.X_TYPE_VALUES));
        LyChartInfo.ValType     valType = LyInfoTokens.getChartInfoValType(selectedVal);

		if (valType == LyChartInfo.ValType.REAL)
		{
			_x_Interval = false;
			myFormField = (LyFormField)_panel2.getComponent(LyChartController.LG_INT_X_AREA);
			if (myFormField != null)
				myFormField.setEditable(false);

			myFormField = (LyFormField)_panel2.getComponent(LyChartController.X_TYPE_INTS);
			if (myFormField != null)
				myFormField.setEditable(false);

			myFormField = (LyFormField)_panel2.getComponent(LyChartController.BORDER_INT_X);
			if (myFormField != null)
				myFormField.setEditable(false);
		}
		else if (valType != null)
		{
			_x_Interval = true;
			((LyFormField)_panel2.getComponent(LyChartController.LG_INT_X_AREA)).setEditable(true);
			((LyFormField)_panel2.getComponent(LyChartController.BORDER_INT_X)).setEditable(true);

			myFormField = (LyFormField)_panel2.getComponent(LyChartController.X_TYPE_INTS);
			if (myFormField != null)
			{
				int		 i;
				LyChoice mychoice = (LyChoice)myFormField.getFieldItem();

				for (i = 0; i < mychoice.getOptionCount(); i++)
					mychoice.enableOption(i, true);
				myFormField.setEditable(true);
			}
		}

		// for y values
		selectedVal = getChoosenOption((LyFormField)_panel2.getComponent(
					LyChartController.Y_TYPE_VALUES));
        valType = LyInfoTokens.getChartInfoValType(selectedVal);

		if ((LyChartInfo.ValType.REAL == valType)
                || (LyChartInfo.ValType.SUM_REAL == valType)
				|| (LyChartInfo.ValType.COUNT_REAL == valType))
		{
			_y_Interval = false;
			myFormField = (LyFormField)_panel2.getComponent(LyChartController.LG_INT_Y_AREA);
			if (myFormField != null)
				myFormField.setEditable(false);

			myFormField = (LyFormField)_panel2.getComponent(LyChartController.Y_TYPE_INTS);
			if (myFormField != null)
				myFormField.setEditable(false);

			myFormField = (LyFormField)_panel2.getComponent(LyChartController.BORDER_INT_Y);
			if (myFormField != null)
				myFormField.setEditable(false);
		}
		else
		{
			_y_Interval = true;
			((LyFormField)_panel2.getComponent(LyChartController.LG_INT_Y_AREA)).setEditable(true);
			((LyFormField)_panel2.getComponent(LyChartController.BORDER_INT_Y)).setEditable(true);

			myFormField = (LyFormField)_panel2.getComponent(LyChartController.Y_TYPE_INTS);
			if (myFormField != null)
			{
				int		 i;
				LyChoice mychoice = (LyChoice)myFormField.getFieldItem();

				for (i = 0; i < mychoice.getOptionCount(); i++)
					mychoice.enableOption(i, true);
				myFormField.setEditable(true);
			}
		}

		// Update group field
		myFormField = (LyFormField)_panel2.getComponent(LyChartController.SELECT_GROUP_FIELD);

		if (myFormField != null)
		{
			if (valType == LyChartInfo.ValType.REAL)
				myFormField.setEnabled(false);
			else
				myFormField.setEnabled(true);
		}

		// Update the last modification time of the controller.
		setLastModified(new Date().getTime());
	} // end method demandValueChanged2

	/**
	 * Save parameters given by the first window in LyChartInfo.
	 */
	protected void saveWin1()
	{
		@SuppressWarnings("unchecked")
		Vector<String> selectedYFields = (Vector<String>)getSelectedYFields().elementAt(1);
		String		   selectedXField = getSelectedXField(getFormField(
					LyChartController.SELECT_X_FIELD));
		LyChartInfo.GraphType		   graphType = null;
		String		   type = getChoosenOption(getFormField(LyChartController.SELECT_GRAPH_TYPE));

		if (type != null)
        {
            type = type.substring(SELECT_GRAPH_TYPE.length());
            graphType = LyInfoTokens.getChartInfoGraphType(type);
        }

		// Determine if we have to re-make default values for axis titles
		if ((null != _chartInfo.getSelectedXFieldInfo())
				&& (_chartInfo.getSelectedXFieldInfo().equals(selectedXField)))
			_defaultXAxisTitle = false;
		else
			_defaultXAxisTitle = true;

		if ((null != _chartInfo.getSelectedYFieldsInfo())
				&& (_chartInfo.getSelectedYFieldsInfo().equals(selectedYFields)))
			_defaultYAxisTitle = false;
		else
			_defaultYAxisTitle = true;

		_chartInfo.setSelectedYFieldsInfo(selectedYFields);
		_chartInfo.setSelectedXFieldInfo(selectedXField);
		_chartInfo.setSelectedGraphType(graphType);
	} // end method saveWin1

	/**
	 * Verify parameters given by the first window : Variables must have the same type At least one
	 * Y value must be selected. If Y contains dates and X contains a variable, no sense. If Hi-Low
	 * is demanded, only number or date Y variables are allowed.
	 *
	 * @return false if error, true if it is coherent.
	 */
	protected boolean validateWin1()
	{
		boolean		  result = true;
		LyComponent   root = _builder.getComponent();
		Vector<String> selectedYFields = _chartInfo.getSelectedYFieldsInfo();
		LyViewManager viewManager = getViewManager();

		// Variables must have the same type
		if (((Boolean)getSelectedYFields().elementAt(0)).booleanValue() == true)
		{
			result = false;
			viewManager.showError(root, "LY_ER_CHART_FIELDS");
		}
		// At least one Y value must be selected.
		else if (selectedYFields == null)
		{
			result = false;
			viewManager.showError(root, "LY_ER_CHART_YNULL");
		}
		// A graph type must be choosen
		else if (_chartInfo.getSelectedGraphType() == null)
		{
			result = false;
			viewManager.showError(root, "LY_ER_CHART_TYPENULL");
		}

		return result;
	} // end method validateWin1

	/**
	 * Save parameters given by the second window in LyChartInfo.
	 */
	protected void saveWin2()
	{
		String		          selectedVal;
		String[]              intervals;
		String		          myLabel;
        LyChartInfo.ValType   valType;

		// fill the _chartInfo with type of values (real, intervals ...) and intervals for Y
		selectedVal = getChoosenOption((LyFormField)_panel2.getComponent(
					LyChartController.Y_TYPE_VALUES));
        valType = LyInfoTokens.getChartInfoValType(selectedVal);

		if (valType == _chartInfo.getYVal())
			_defaultXAxisTitle |= false;
		else
			_defaultXAxisTitle = true;

		_chartInfo.setYVal(valType);

		if ((valType == LyChartInfo.ValType.REAL)
                || (valType == LyChartInfo.ValType.SUM_REAL)
				|| (valType == LyChartInfo.ValType.COUNT_REAL))
			_y_Interval = false;
		else
			_y_Interval = true;

		if (_y_Interval)
		{
			intervals = new String[3];

			myLabel = getNumDateLabel(
					((LyFormField)_panel2.getComponent(LyChartController.BORDER_INT_Y))
						.getFieldItem());
			intervals[0] = myLabel;

			myLabel = getNumDateLabel(
					((LyFormField)_panel2.getComponent(LyChartController.LG_INT_Y_AREA))
						.getFieldItem());
			intervals[1] = myLabel;

			if ((myLabel != null) && !myLabel.equals(_chartInfo.getYIntervalLength()))
				_defaultXAxisTitle = true;

			if ((LyFormField)_panel2.getComponent(LyChartController.Y_TYPE_INTS) != null)
			{
				String choosenOption = getChoosenOption((LyFormField)_panel2.getComponent(
							LyChartController.Y_TYPE_INTS));

				if (choosenOption != null)
				{
					choosenOption = choosenOption.substring(Y_TYPE_INTS.length());
					intervals[2] = choosenOption;

					if ((null != _chartInfo.getYIntervalUnit())
							&& choosenOption.equals(_chartInfo.getYIntervalUnit().toString()))
						_defaultXAxisTitle |= false;
					else
						_defaultXAxisTitle = true;
				}
			}
			_chartInfo.setYIntervals(intervals);
		}

		selectedVal = getChoosenOption((LyFormField)_panel2.getComponent(
					LyChartController.X_TYPE_VALUES));
        valType = LyInfoTokens.getChartInfoValType(selectedVal);

		// fill the _chartInfo with group field info
		String selectedGroupField = getSelectedGroupField(getFormField(
					LyChartController.SELECT_GROUP_FIELD));

		_chartInfo.setSelectedGroupFieldInfo(selectedGroupField);

		// fill the _chartInfo with type of values (real, intervals) and intervals for X.
		// If no X is selected, intervals for X are equal to intervals for Y
		if (_chartInfo.getSelectedXFieldInfo() != null)
		{
			_chartInfo.setXVal(valType);
			if (valType == LyChartInfo.ValType.REAL)
				_x_Interval = false;
			else
				_x_Interval = true;

			if (_x_Interval)
			{
				intervals = new String[3];

				myLabel = getNumDateLabel(
						((LyFormField)_panel2.getComponent(LyChartController.BORDER_INT_X))
							.getFieldItem());
				intervals[0] = myLabel;

				myLabel = getNumDateLabel(
						((LyFormField)_panel2.getComponent(LyChartController.LG_INT_X_AREA))
							.getFieldItem());
				intervals[1] = myLabel;

				if ((LyFormField)_panel2.getComponent(LyChartController.X_TYPE_INTS) != null)
				{
					String choosenOption = getChoosenOption((LyFormField)_panel2.getComponent(
								LyChartController.X_TYPE_INTS));

					intervals[2] = choosenOption.substring(Y_TYPE_INTS.length());
				}
				//intervals.addElement(getChoosenOption(
						// (LyFormField)_panel2.getComponent(LyChartController.X_TYPE_INTS)));
				_chartInfo.setXIntervals(intervals);
			}
		}
		else
			_chartInfo.setXVal(LyChartInfo.ValType.REAL);

		if (_chartInfo.getSelectedGraphType() == LyChartInfo.GraphType.PIE)
		{
			selectedVal = getChoosenOption((LyFormField)_panel2.getComponent(
						LyChartController.PIES_BY));
			_chartInfo.setPieBy(PIES_BY_Y.equals(selectedVal));
		}
	} // end method saveWin2

	/**
	 * Retrieve the label of the field : date or number If the field is a number, replace the local
	 * decimal separator by '.'
	 *
	 * @param  fieldItem a field which is a number or a date
	 * @return the label of the given field if it's a date or a number field
	 */
	protected String getNumDateLabel(LyFieldItem fieldItem)
	{
		if (fieldItem instanceof LyDate)
			return ((LyGetLabelInterface)fieldItem).getLabel();
		else if (fieldItem instanceof LyNumber)
		{
			// Retrieve the decimal separator
			DecimalFormatSymbols format = new DecimalFormatSymbols(_parent.getEnvironment()
						.getLocale());
			char				 decimalSeparator = format.getDecimalSeparator();

			String				 label = ((LyGetLabelInterface)fieldItem).getLabel();
			// Be sure to replace decimal separator by '.'
			if (null != label)
				label = label.replace(decimalSeparator, '.');
			return label;
		}
		else
			return null;
	}

	/**
	 * Verify parameters given by the second window : If count is demanded, only bars, stacking bars
	 * and pies are allowed. If no X is selected and real values are selected, only plot, scatter
	 * plot and Hi-Low are allowed because in the other cases, some parts of the chart hide the
	 * other ones. If pie is selected and more than one Y alphanumeric, real values are not allowed
	 *
	 * @return false if error, true if it is coherent.
	 */
	protected boolean validateWin2()
	{
		LyComponent   root = _builder.getComponent();
		LyViewManager viewManager = getViewManager();
		boolean		  result = true;

		// type of values (real, intervals ...) for Y
		LyChartInfo.ValType		  yVal = _chartInfo.getYVal();

		//verify if width of intervals is filled (if intervals are required) and if it is not null
		// for Y and for X :
		if ((yVal == LyChartInfo.ValType.COUNT_INTERVAL)
					&& ((_chartInfo.getYIntervals()[1] == null)
						|| (_chartInfo.getYIntervals()[1].toString().equals("0")))
				|| ((yVal == LyChartInfo.ValType.INTERVAL)
					&& ((_chartInfo.getXIntervals()[1] == null)
						|| (_chartInfo.getXIntervals()[1].toString().equals("0")))))
		{
			result = false;
			viewManager.showError(root, "LY_ER_CHART_INT");
		}

		//type of chart (plot, bar, pie ...)
        LyChartInfo.GraphType selectedGraphType = _chartInfo.getSelectedGraphType();

		if (((yVal == LyChartInfo.ValType.COUNT_INTERVAL)
					|| (yVal == LyChartInfo.ValType.COUNT_REAL))
				&& ((selectedGraphType == LyChartInfo.GraphType.SCATTER_PLOT)
					|| (selectedGraphType == LyChartInfo.GraphType.PLOT)
					|| (selectedGraphType == LyChartInfo.GraphType.STACK_AREA)))
		{
			result = false;
			viewManager.showError(root, "LY_ER_CHART_COUNT");
		}

		// selected X field
		String	    xFieldInfo = _chartInfo.getSelectedXFieldInfo();
		LyFieldInfo selectedXField = null;

		if (xFieldInfo != null)
			selectedXField = _classInfo.getFieldInfo(xFieldInfo, false);

		if (((yVal == LyChartInfo.ValType.REAL)
                && (selectedXField == null))
				&& ((selectedGraphType == LyChartInfo.GraphType.STACK_BAR)
					|| (selectedGraphType == LyChartInfo.GraphType.STACK_AREA)
					|| (selectedGraphType == LyChartInfo.GraphType.BAR))) //            ||(selectedGraphType == LyChartInfo.GraphType.PIE)
		{
			result = false;
			viewManager.showError(root, "LY_ER_CHART_REAL");
		}

		// if pie is selected and more than one Y alphanumeric, real values are not allowed
		if ((yVal == LyChartInfo.ValType.REAL)
				&& (_chartInfo.getTypeVarY() == LyChartInfo.Type.TEXT)
				&& (_chartInfo.getSelectedYFieldsInfo().size() > 1)
				&& (selectedGraphType == LyChartInfo.GraphType.PIE))
		{
			result = false;
			viewManager.showError(root, "LY_ER_CHART_PIE_ALPHA");
		}

		// if pie is selected and only one Y variable is selected, real and sum values are not allowed
		if (((LyChartInfo.ValType.REAL == yVal)
                    || (LyChartInfo.ValType.SUM_REAL == yVal))
				&& (_chartInfo.getSelectedYFieldsInfo().size() == 1)
				&& (selectedGraphType == LyChartInfo.GraphType.PIE))
		{
// a remettre                   result = false;
//getEnvironment().logError("LY_ER_CHART_PIE_Y");
// a remettre                   viewManager.showError(root, "LY_ER_CHART_PIE_Y");
		}

		return result;
	} // end method validateWin2

	/**
	 * Verify parameters given by the third window :
	 *
	 * @return false if error, true if it is coherent.
	 */
	protected boolean validateWin3()
	{
		boolean     result = true;
		LyFormField fieldMin, fieldMax;
		String	    minVal, maxVal;

		// Check min and max for X
		fieldMin = (LyFormField)_panel3.getComponent(LyChartController.X_MIN_AXIS);
		fieldMax = (LyFormField)_panel3.getComponent(LyChartController.X_MAX_AXIS);

		if ((fieldMin != null) && (fieldMax != null))
		{
			minVal = getNumDateLabel(fieldMin.getFieldItem());
			maxVal = getNumDateLabel(fieldMax.getFieldItem());

			if ((minVal != null) && (minVal.length() > 0)
					&& (maxVal != null) && (maxVal.length() > 0))
			{
				if (fieldMin.getFieldItem() instanceof LyDate)
				{
					LyDateFieldInfo fieldInfo = (LyDateFieldInfo)_classInfo.getFieldInfo(_chartInfo
								.getSelectedXFieldInfo(), false);

					try
					{
						if (fieldInfo.compare(fieldInfo.parse(minVal), fieldInfo.parse(maxVal))
								>= 0)
						{
							result = false;
							showError("LY_ER_CHART_X_INTERVAL");
						}
					}
					catch (LyFieldInfoFormatException e)
					{
					}
				}
				else if (fieldMin.getFieldItem() instanceof LyNumber)
				{
					float fMin = LyTools.floatFromString(minVal);
					float fMax = LyTools.floatFromString(maxVal);

					if (fMax <= fMin)
					{
						result = false;
						showError("LY_ER_CHART_X_INTERVAL");
					}
				}
			}
		}

		// Check min and max for Y
		fieldMin = (LyFormField)_panel3.getComponent(LyChartController.Y_MIN_AXIS);
		fieldMax = (LyFormField)_panel3.getComponent(LyChartController.Y_MAX_AXIS);

		if ((fieldMin != null) && (fieldMax != null))
		{
			minVal = getNumDateLabel(fieldMin.getFieldItem());
			maxVal = getNumDateLabel(fieldMax.getFieldItem());

			if ((minVal != null) && (minVal.length() > 0)
					&& (maxVal != null) && (maxVal.length() > 0))
			{
				if (fieldMin.getFieldItem() instanceof LyNumber)
				{
					float fMin = LyTools.floatFromString(minVal);
					float fMax = LyTools.floatFromString(maxVal);

					if (fMax <= fMin)
					{
						result = false;
						showError("LY_ER_CHART_Y_INTERVAL");
					}
				}
			}
		}

		return result;
	} // end method validateWin3

	/**
	 * Sets the params given by the third window in LyChartInfo : Titles (graphic and axis), Axis
	 * parameters.
	 */
	public void saveWin3()
	{
		LyFormField field = (LyFormField)_panel3.getComponent(LyChartController.TITLE_AREA);
		LyFieldItem fieldItem = field.getFieldItem();

		// General Title
		_chartInfo.setTitle(((LyGetLabelInterface)fieldItem).getLabel());

		// X axis Title
		field = (LyFormField)_panel3.getComponent(LyChartController.X_TITLE);
		if (field != null)
		{
			fieldItem = field.getFieldItem();
			_chartInfo.setXTitle(((LyGetLabelInterface)fieldItem).getLabel());
		}

		// Y axis Title
		field = (LyFormField)_panel3.getComponent(LyChartController.Y_TITLE);
		if (field != null)
		{
			fieldItem = field.getFieldItem();
			_chartInfo.setYTitle(((LyGetLabelInterface)fieldItem).getLabel());
		}

		// X axis parameters : begin, end and graduations
		// initialize the vector of axis parameters
		String[] axisParam = new String[3];

		field = (LyFormField)_panel3.getComponent(LyChartController.X_MIN_AXIS);
		if (field != null)
			axisParam[0] = getNumDateLabel(field.getFieldItem());

		field = (LyFormField)_panel3.getComponent(LyChartController.X_MAX_AXIS);
		if (field != null)
			axisParam[1] = getNumDateLabel(field.getFieldItem());

		field = (LyFormField)_panel3.getComponent(LyChartController.X_GRAD_AXIS);
		if (field != null)
			axisParam[2] = getNumDateLabel(field.getFieldItem());

		_chartInfo.setXAxis(axisParam);

		// Y axis parameters : begin, end and graduations
		// initialize the vector of axis parameters
		axisParam = new String[3];

		field = (LyFormField)_panel3.getComponent(LyChartController.Y_MIN_AXIS);
		if (field != null)
			axisParam[0] = getNumDateLabel(field.getFieldItem());

		field = (LyFormField)_panel3.getComponent(LyChartController.Y_MAX_AXIS);
		if (field != null)
			axisParam[1] = getNumDateLabel(field.getFieldItem());

		field = (LyFormField)_panel3.getComponent(LyChartController.Y_GRAD_AXIS);
		if (field != null)
			axisParam[2] = getNumDateLabel(field.getFieldItem());

		_chartInfo.setYAxis(axisParam);

		// Display options
		field = (LyFormField)_panel3.getComponent(LyChartController.CHART_DISPLAY_OPTIONS);
		fieldItem = field.getFieldItem();
		// X axis Labels form : horizontal or vertical
		LyOption option = ((LyChoice)fieldItem).getOption(2);

		if (option != null)
			_chartInfo.setXVerticalLabels(option.isChecked());
		// X grid
		option = ((LyChoice)fieldItem).getOption(0);
		if (option != null)
			_chartInfo.setXGrid(option.isChecked());
		// Y grid
		option = ((LyChoice)fieldItem).getOption(1);
		if (option != null)
			_chartInfo.setYGrid(option.isChecked());
		// 3D Effect
		option = ((LyChoice)fieldItem).getOption(3);
		if (option != null)
			_chartInfo.set3dEffect(option.isChecked());
		// Show/hide the legend
		option = ((LyChoice)fieldItem).getOption(4);
		if (option != null)
			_chartInfo.setShowLegend(option.isChecked());
		// Show all the labels (and not dwell labels)
		option = ((LyChoice)fieldItem).getOption(5);
		if (option != null)
			_chartInfo.setShowTips(option.isChecked());
		// Set ratios (percentiles).
		option = ((LyChoice)fieldItem).getOption(6);
		if (option != null)
			_chartInfo.setRatios(option.isChecked());

		// sets _first3 to false because the third window has been filled by the user
		_first3 = false;
		// don't reset axis titles next time
		_defaultXAxisTitle = false;
		_defaultYAxisTitle = false;

		return;
	} // end method saveWin3

	/**
	 * Retrieves the selected Y fields and fill the type of Y var in LyChartInfo. If types are
	 * incompatible, returns an error.
	 *
	 * @return first element : an error Boolean, second element : the selected Y fields
	 * @see    LyChartInfo
	 */
	protected Vector<Object> getSelectedYFields()
	{
		Vector<String> fields = null;
		LyFormField    formField = getFormField(LyChartController.SELECT_Y_FIELDS);
		LyRelation     relation = (LyRelation)formField.getFieldItem();
		int			   i, n = relation.getLinkCount();
		LyLink		   link;
		String		   value;
		boolean		   warning = false;

		_typexml = null; //initialisation of the type of selected var(s)

		for (i = 0; i < n; i++)
		{
			link = relation.getLink(i);

			if (link.isSelected())
			{
				if (fields == null)
					fields = new Vector<String>();

				// the link Id is "link"+Id of the field
				value = link.getId().substring(4);

				// Warning if the types of selectionned fields are different
				LyFieldInfo fieldInfo = _classInfo.getFieldInfo(value);

				fields.addElement(value);

				if (fieldInfo instanceof LyNumberFieldInfo)
				{
					if (_typexml == null)
						_typexml = LyChartInfo.Type.NUMBER;
					else if (_typexml != LyChartInfo.Type.NUMBER)
						warning = true;
				}
				else if ((fieldInfo instanceof LyTextFieldInfo)
						|| (fieldInfo instanceof LyFileFieldInfo)
						|| (fieldInfo instanceof LyRelationFieldInfo)
						|| (fieldInfo instanceof LyChoiceFieldInfo))
				{
					if (_typexml == null)
						_typexml = LyChartInfo.Type.TEXT;
					else if (_typexml != LyChartInfo.Type.TEXT)
						warning = true;
				}
				else if (fieldInfo instanceof LyDateFieldInfo)
				{
					if (_typexml == null)
						_typexml = LyChartInfo.Type.DATE;
					else if (_typexml != LyChartInfo.Type.DATE)
						warning = true;
				}
				else
				{
					if (_typexml == null)
						_typexml = LyChartInfo.Type.TEXT;
					else if (_typexml != LyChartInfo.Type.TEXT)
						warning = true;
				}
				_chartInfo.setTypeVarY(_typexml);
			}
		}

		Vector<Object> result = new Vector<Object>();

		result.addElement(Boolean.valueOf(warning));
		result.addElement(fields);
		return result;
	} // end method getSelectedYFields

	/**
	 * Retrieves the selected X field and fill the type of X var in LyChartInfo.
	 *
	 * @param  formField the field containing X selected value
	 * @return the selected X field id
	 * @see    LyChartInfo
	 */
	protected String getSelectedXField(LyFormField formField)
	{
		LyRelation  relation = (LyRelation)formField.getFieldItem();
		LyLink	    link;
		String	    value;
		LyFieldInfo fieldInfo = null;
		int		    i, n = relation.getLinkCount();

		// if the LyFormField is not editable, ignore its selection
		if (!formField.isEditable())
		{
			_chartInfo.setTypeVarX(LyChartInfo.Type.TEXT);
			return null;
		}

		for (i = 0; i < n; i++)
		{
			link = relation.getLink(i);

			if (link.isSelected())
			{
				// the link Id is "link"+Id of the field
				value = link.getId().substring(4);
				fieldInfo = _classInfo.getFieldInfo(value, false);
			}
		}

		// if an X field is selected, retrieve its type
		// (if no X field is selected, it will be equal to Y so its type is Y type) deprecated
		// if no X field is selected, it will be equal text (type of the name of Y variables)
		if (fieldInfo != null)
		{
			// If the field is a relation, retrieve the reference LyFieldInfo.
			if (fieldInfo instanceof LyRelationFieldInfo)
                _chartInfo.setTypeVarX(LyChartInfo.Type.TEXT);
			if (fieldInfo instanceof LyNumberFieldInfo)
				_chartInfo.setTypeVarX(LyChartInfo.Type.NUMBER);
			else if (fieldInfo instanceof LyDateFieldInfo)
				_chartInfo.setTypeVarX(LyChartInfo.Type.DATE);
			else
				_chartInfo.setTypeVarX(LyChartInfo.Type.TEXT);
			return fieldInfo.getId();
		}
		else
			_chartInfo.setTypeVarX(LyChartInfo.Type.TEXT);
		return null;
	} // end method getSelectedXField

	/**
	 * Retrieves the selected group field.
	 *
	 * @param  formField the field containing group selected value
	 * @return the selected group field id
	 * @see    LyChartInfo
	 */
	protected String getSelectedGroupField(LyFormField formField)
	{
		LyRelation  relation = (LyRelation)formField.getFieldItem();
		LyLink	    link;
		String	    value;
		LyFieldInfo fieldInfo = null;
		int		    i, n = relation.getLinkCount();

		// if the LyFormField is not enable, ignore its selection
		if (!formField.isEnabled())
			return null;

		for (i = 0; i < n; i++)
		{
			link = relation.getLink(i);

			if (link.isSelected())
			{
				value = link.getId();
				fieldInfo = _classInfo.getFieldInfo(value, false);
			}
		}

		return ((fieldInfo == null) ? null : fieldInfo.getId());
	}

	/**
	 * Retrieves the selected option from a choice.
	 *
	 * @param  formField the form field where the choice is.
	 * @return the selected option from a choice.
	 */
	protected String getChoosenOption(LyFormField formField)
	{
		LyFieldItem fieldItem = null;
		int		    i, n;

		if (formField != null)
		{
			fieldItem = formField.getFieldItem();

			if (fieldItem instanceof LyChoice)
			{
				LyChoice choice = (LyChoice)fieldItem;
				LyOption option;

				for (i = 0, n = choice.getOptionCount(); i < n; i++)
				{
					option = choice.getOption(i);
					if (option.isChecked())
                        return option.getId();
				}
			}
			else if (fieldItem instanceof LyList)
			{
				LyList		 list = (LyList)fieldItem;
				LyListOption option;

				for (i = 0, n = list.getListOptionCount(); i < n; i++)
				{
					option = list.getListOption(i);
					if (option.isSelected())
						return option.getId();
				}
			}
		}

        return null;
	} // end method getChoosenOption

	/**
	 * Retrieves the LyFormField corresponding to an ID.
	 *
	 * @param  id the ID of the searched LyFormField
	 * @return the LyFormField corresponding to the ID
	 */
	protected LyFormField getFormField(String id)
	{
		// Retrieve formField corresponding to id
		LyComponent root = _builder.getComponent();

		return (LyFormField)root.getComponent(id);
	}

	/**
	 * Retrieves the view Builder : The builder which creates the first parameter Window if the
	 * chart is not parametered yet. else draw the chart directly.
	 *
	 * @return the builder instance;
	 */
	protected LyBuilder getDefaultBuilder()
	{
		return new LyChartBuilder(this);
	}

	/**
	 * Retrieves the second step form Builder : the builder which creates the second parameter
	 * Window.
	 *
	 * @return the builder instance;
	 */
	protected LyBuilder getSecondBuilder()
	{
		if (_win2Builder == null)
			_win2Builder = new _LyChartWin2Builder(this);
		return _win2Builder;
	}

	/**
	 * Retrieves the third step form Builder : The builder which creates the third parameter Window.
	 *
	 * @return the builder instance;
	 */
	protected LyBuilder getThirdBuilder()
	{
		if (_win3Builder == null)
			_win3Builder = new _LyChartWin3Builder(this);
		return _win3Builder;
	}

	/**
	 * Retrieves the fourth step form Builder : The builder which creates the Chart Window.
	 *
	 * @return the builder instance;
	 */
	protected LyBuilder getFourthBuilder()
	{
		return _graphicalChartController.getBuilder();
	}

	/**
	 * Reset the first window.
	 *
	 * @return the first form.
	 */
	public LyArea resetArea1()
	{
		_LyChartWin1Builder builder1 = new _LyChartWin1Builder(this);

		builder1.build();

		LyArea area1 = (LyArea)builder1.getComponent();

		// Update the last modification time of the controller.
		setLastModified(new Date().getTime());

		return area1;
	}

	/**
	 * show the second window.
	 */
	public void resetArea2()
	{
		_y_Interval = false;
		_x_Interval = false;

		_LyChartWin2Builder builder2 = (_LyChartWin2Builder)getSecondBuilder();

		_panel2 = (LyPanel)builder2.build();

		LyArea  area = (LyArea)((LyChartBuilder)_builder).getComponent();

		// Remove the old panel
		LyPanel oldpanel = area.getPanel(0);

		area.removePanel(oldpanel);
		area.addPanel(_panel2);

		LyCommands commands = area.getCommands();

		commands.getCommand(0).setEnabled(true);

		// Update the last modification time of the controller.
		setLastModified(new Date().getTime());
	}

	/**
	 * show the third window.
	 */
	public void resetArea3()
	{
		_LyChartWin3Builder builder3 = (_LyChartWin3Builder)getThirdBuilder();

		_panel3 = (LyPanel)builder3.build();
		LyArea  area = (LyArea)((LyChartBuilder)_builder).getComponent();

		// Remove the old panel
		LyPanel oldpanel = area.getPanel(0);

		area.removePanel(oldpanel);
		area.addPanel(_panel3);

		// Update the last modification time of the controller.
		setLastModified(new Date().getTime());
	}

	/**
	 * show the graphic window.
	 */
	protected void resetArea4()
	{
		boolean first = false;
		LyArea  area = null;

		if ((_graphicalChartController == null)
				|| (_graphicalChartController.getSession() == null)
				|| (_graphicalChartController.getView() == null)
				|| !_graphicalChartController.getView().isVisible())
		{
			LyApplication application = getApplication();
			LyAction	  defaultAction = application.getAction("_graphicalChart");

			first = true;
			_graphicalChartController = new LyGraphicalChartController(_session,
					_parent.getParent(), _graphicalChartAction, _classInfo, getObjectList(), this);

			// If default action is used set the behavior of current controller to the graphical
			// chart controller
			if (defaultAction == _graphicalChartAction)
				_graphicalChartController.setBehavior(getBehavior());
		}

		if (!first)
		{
			LyView  view = _graphicalChartController.getView();
			int[]   size = view.getSize();
			boolean resize = (size == null) || (size.length != 2) || !(view instanceof LyDialog);
			boolean wasResizable = true;

			if (!resize)
			{
				((LyDialog)view).setWidth(size[0]);
				((LyDialog)view).setHeight(size[1]);
				((LyDialog)view).setResizable(false);
				wasResizable = ((LyDialog)view).isResizable();
			}

			_graphicalChartController.getComponent();
			_graphicalChartController.setChartInfo(_chartInfo);

			LyGraphicalChartBuilder builder4 = (LyGraphicalChartBuilder)getFourthBuilder();
			LyPanel				    newpanel = builder4.buildChart();

			area = (LyArea)builder4.getComponent();

			// Remove the old panel
			LyPanel oldpanel = area.getPanel(0);

			area.removePanel(oldpanel);

			area.addPanel(newpanel);

			LyChartBehaviorInterface behavior = (LyChartBehaviorInterface)_graphicalChartController
					.getBehavior();

			builder4.updateSpecificBehaviors();
			behavior.completeChart(_graphicalChartController,
				_graphicalChartController.getChartComponent());

			if (!resize)
				((LyDialog)view).setResizable(wasResizable);
		}

		// Update the last modification time of the controller.
		_graphicalChartController.setLastModified(new Date().getTime());
	} // end method resetArea4

	/**
	 * Called when current object has changed . Keep the overall content consistent.
	 *
	 * @param oldValues set of former values, or null if not specified.
	 */
	protected void updateObject(LyValueSet oldValues)
	{
	}

	/* Internal Classes */
	/* First window */
	/**
	 * This internal class constructs the first parameter window.
	 */
	class _LyChartWin1Builder extends LyFormBuilder
	{

		/**
		 * Default constructor.
		 *
		 * @param parent parent of the controller
		 */
		public _LyChartWin1Builder(LyChartController parent)
		{
			super(parent);
		}

		/**
		 * Builds the view component for the specified parent controller. The component may create
		 * the XML Tree or the internal view.
		 *
		 * @return the view Component
		 */
		public LyComponent build()
		{

			/* The chart 2nd window is an area which contains :
			        - A titleBar (description)
			        - An area component
			*/
			LyArea area = buildWin1Area();

			area.setController(this._parent.getId());

			this._component = area;

			return (area);
		}

		/**
		 * Creates the first from and sets its content.
		 *
		 * @return the first form.
		 */
		protected LyArea buildWin1Area()
		{
			LyArea			  area = new LyArea();
			LyArea			  topArea = new LyArea();
			LyArea			  bottomArea = new LyArea();
			LyForm			  form = new LyForm(this._parent.getId());
			LyForm			  bottomForm = new LyForm(this._parent.getId());
			LyFormField		  field;
			LyCaption		  caption;
			LyTip			  tip;
			LyChartController chartController = ((LyChartController)this._parent);
			LyClassInfo		  classInfo = chartController.getClassInfo();

			// Resize the areas to the maximum
			LyConstraint	  constraint = new LyConstraint();

			constraint.setRowSpan(-1);
			constraint.setRemainder(true);
			constraint.setFill(LyConstraint.Fill.BOTH);
			area.setConstraint(constraint);

			// **** Add attribute List for Y
			LyRelation				 fieldRelation = new LyRelation();
			LyFieldInfo				 fieldInfo;
			LyLink					 link;

			LyChartBehaviorInterface behavior = (LyChartBehaviorInterface)chartController
					.getBehavior();

			LyInfoList<LyFieldInfo>  fields = behavior.getYFields(chartController);
			int						 i, n = fields.getSize();

			// Get fieldinfo list
			for (i = 0; i < n; i++)
			{
				fieldInfo = fields.getInfo(i);

				// The link Id can't be the same than the fieldInfo
				link = new LyLink("link" + fieldInfo.getId(), fieldInfo.getName());

				// if _chartInfo is filled, retrieve if the field is selected.
				if ((_chartInfo != null) && (_chartInfo.getSelectedYFieldsInfo() != null)
						&& (_chartInfo.getSelectedYFieldsInfo().contains(fieldInfo.getId())))
					link.setSelected(true);

				fieldRelation.addLink(link);
			}

			fieldRelation.setMultiple(true);

			// add the relation to the field
			field = new LyFormField(LyChartController.SELECT_Y_FIELDS);

			caption = new LyCaption();
			caption.setLabel("LY_CHART_FIELD_Y");
			caption.setColon(true);

			constraint = new LyConstraint();
			constraint.setRemainder(true);
			constraint.setHAlign(LyConstraint.HAlign.LEFT);
			caption.setConstraint(constraint);

			tip = new LyTip();
			tip.setLabel("LY_CHART_FIELD_Y_TIP");
			field.setTip(tip);

			field.setCaption(caption);
			field.setFieldItem(fieldRelation);
			field.setOptional(false);
			// force the browser to check for changes
			field.setNeedCheck(true);

			constraint = new LyConstraint();
			constraint.setRemainder(true);
			constraint.setHAlign(LyConstraint.HAlign.LEFT);
//                      constraint.setColSpan(1);
			field.setConstraint(constraint);

			form.addFormField(field);

			// **** Add attribute List for X
			fieldRelation = new LyRelation();
			n = classInfo.getFieldInfoCount();

			fields = behavior.getXFields(chartController);
			n = fields.getSize();

			// the first link is blank
			link = new LyLink("LY_NO_CHOICE", "            ");
			fieldRelation.insertLink(link, 0);

			// Get fieldinfo list
			for (i = 0; i < n; i++)
			{
				fieldInfo = fields.getInfo(i);

				// The link Id can't be the same than the fieldInfo
				link = new LyLink("link" + fieldInfo.getId(), fieldInfo.getName());

				// if _chartInfo is filled, retrieve if the field is selected.
				if ((_chartInfo != null) && (null != _chartInfo.getSelectedXFieldInfo())
						&& (_chartInfo.getSelectedXFieldInfo().equals(fieldInfo.getId())))
					link.setSelected(true);

				fieldRelation.addLink(link);
			}

			fieldRelation.setMultiple(false);

			// add the relation to the field
			field = new LyFormField(LyChartController.SELECT_X_FIELD);

			caption = new LyCaption();
			caption.setLabel("LY_CHART_FIELD_X");
			caption.setColon(true);

			constraint = new LyConstraint();
			constraint.setRemainder(true);
			constraint.setHAlign(LyConstraint.HAlign.LEFT);
//                      constraint.setColSpan(LyConstraint.REMAINDER);
			caption.setConstraint(constraint);

			tip = new LyTip();
			tip.setLabel("LY_CHART_FIELD_X_TIP");
			field.setTip(tip);

			field.setCaption(caption);
			field.setFieldItem(fieldRelation);
			field.setOptional(true);

			// if Y contains dates : disable this field.
			if ((null != _chartInfo)
                 && (LyChartInfo.Type.DATE == _chartInfo.getTypeVarY()))
			{
				field.setEditable(false);
				field.setEnabled(false);
			}

			form.addFormField(field);

			// **** Add Option List for type of graphical representation
			// by default : SCATTER_PLOT is selected, but if _chartInfo is filled, retrieve which option is selected.

			LyList list = new LyList();

			list.setNRows(6);
			list.setSelectionMode(LyList.SelectionMode.SINGLE);

			LyListOption option = new LyListOption(SELECT_GRAPH_TYPE + LyChartInfo.GraphType.STACK_BAR,
					"LY_STACK_BAR");

			option.setImage("LY_CHART_STACK_BAR");

			if ((_chartInfo == null)
					|| (_chartInfo.getSelectedGraphType() == LyChartInfo.GraphType.STACK_BAR))
				option.setSelected(true);
			list.addListOption(option);

			option = new LyListOption(SELECT_GRAPH_TYPE + LyChartInfo.GraphType.BAR, "LY_BAR");
			option.setImage("LY_CHART_BAR");
			if ((_chartInfo != null) && (_chartInfo.getSelectedGraphType() == LyChartInfo.GraphType.BAR))
				option.setSelected(true);
			list.addListOption(option);

			option = new LyListOption(SELECT_GRAPH_TYPE + LyChartInfo.GraphType.PIE, "LY_PIE");
			option.setImage("LY_CHART_PIE");
			if ((_chartInfo != null) && (_chartInfo.getSelectedGraphType() == LyChartInfo.GraphType.PIE))
				option.setSelected(true);
			list.addListOption(option);

			option = new LyListOption(SELECT_GRAPH_TYPE + LyChartInfo.GraphType.SCATTER_PLOT,
					"LY_SCATTER_PLOT");
			option.setImage("LY_CHART_SCATTER_PLOT");
			if ((_chartInfo != null)
					&& (_chartInfo.getSelectedGraphType() == LyChartInfo.GraphType.SCATTER_PLOT))
				option.setSelected(true);
			list.addListOption(option);

			option = new LyListOption(SELECT_GRAPH_TYPE + LyChartInfo.GraphType.PLOT, "LY_PLOT");
			option.setImage("LY_CHART_PLOT");
			if ((_chartInfo != null) && (_chartInfo.getSelectedGraphType() == LyChartInfo.GraphType.PLOT))
				option.setSelected(true);
			list.addListOption(option);

			option = new LyListOption(SELECT_GRAPH_TYPE + LyChartInfo.GraphType.STACK_AREA, "LY_STACK_AREA");
			option.setImage("LY_CHART_STACK_AREA");
			if ((_chartInfo != null)
					&& (_chartInfo.getSelectedGraphType() == LyChartInfo.GraphType.STACK_AREA))
				option.setSelected(true);
			list.addListOption(option);

			// add the relation to the field
			field = new LyFormField(LyChartController.SELECT_GRAPH_TYPE);

			caption = new LyCaption();
			caption.setLabel("LY_CHART_GRAPH_CHOICE");
			caption.setColon(true);

			tip = new LyTip();
			tip.setLabel("LY_CHART_GRAPH_CHOICE_TIP");
			field.setTip(tip);

			field.setCaption(caption);
			field.setFieldItem(list);
			field.setOptional(false);

			bottomForm.addFormField(field);

			constraint = new LyConstraint();

			constraint.setFill(LyConstraint.Fill.NONE);
			form.setConstraint(constraint);
			bottomForm.setConstraint(constraint);

			//** Construct first parameter area
			topArea.setBorder(LyArea.Border.IN);
			topArea.addPanel(form);
			form.setBorder(false);
			bottomArea.setBorder(LyArea.Border.IN);
			bottomForm.setBorder(false);
			bottomArea.addPanel(bottomForm);
			area.addPanel(topArea);
			area.addPanel(bottomArea);

			// first parameter  area is ready
			return (area);
		} // end method buildWin1Area

		/**
		 * Creates the command bar and sets its content.
		 *
		 * @return the command bar with two buttons : next and cancel.
		 */
		protected LyCommands buildCommands()
		{
			return null;
		}

		/**
		 * Build a text field item
		 * Overriden from LyFormBuilder.
		 * @param textField the date field
		 * @param value a value
		 * @return the asociated field item (can be null)
		 */
		public LyFieldItem buildText(LyTextFieldInfo textField, Object value)
		{
			return null;
		}

		/**
		 * Build a date field item
		 * Overriden from LyFormBuilder.
		 * @param dateField the date field
		 * @param value a value
		 * @return the asociated field item (can be null)
		 */
		public LyFieldItem buildDate(LyDateFieldInfo dateField, Object value)
		{
			return null;
		}

		/**
		 * Build a number field item
		 * Overriden from LyFormBuilder.
		 * @param numberField the number field
		 * @param value a value
		 * @return the asociated field item (can be null)
		 */
		public LyFieldItem buildNumber(LyNumberFieldInfo numberField, Object value)
		{
			return null;
		}

		/**
		 * Build a relation field item
		 * Overriden from LyFormBuilder.
		 * @param relationField the relation field
		 * @param value a value
		 * @return the asociated field item (can be null)
		 */
		public LyFieldItem buildRelation(LyRelationFieldInfo relationField, Object value)
		{
			return null;
		}

		/**
		 * Build a choice field item
		 * Overriden from LyFormBuilder.
		 * @param choiceField  the choice fieldinfo
		 * @param value a vlue
		 * @return the associated field item (can be null)
		 */
		public LyFieldItem buildChoice(LyChoiceFieldInfo choiceField, Object value)
		{
			return null;
		}

		/**
		 * Build a file field item
		 * Overriden from LyFormBuilder.
		 * @param fileField the file fieldinfo
		 * @param value a value
		 * @return the associated field item (can be null)
		 */
		public LyFieldItem buildFile(LyFileFieldInfo fileField, Object value)
		{
			return null;
		}
	} // end class _LyChartWin1Builder


	/* Second window */

	/**
	 * This internal class constructs the second parameter window.
	 */
	class _LyChartWin2Builder extends LyFormBuilder
	{

		/**
		 * Default constructor.
		 *
		 * @param parent parent of the controller
		 */
		public _LyChartWin2Builder(LyChartController parent)
		{
			super(parent);
		}

		/**
		 * Builds the view component for the specified parent controller.
		 *
		 * @return the view Component
		 */
		public LyComponent build()
		{

			/* The chart 2nd window is an area which contains :
			        - A titleBar (description)
			        - A form component
			*/
			LyArea area = buildWin2Area();

			area.setController(this._parent.getId());

			this._component = area;

			return (area);
		}

		/**
		 * Creates the second window form.
		 *
		 * @return a new second window form.
		 */
		protected LyArea buildWin2Area()
		{
			LyArea			  area = new LyArea();
			LyForm			  form = new LyForm(this._parent.getId());
			LyForm			  midForm = new LyForm(this._parent.getId());
			LyForm			  bottomForm = new LyForm(this._parent.getId());
			LyFormField		  field;
			LyTip			  tip;
			LyChartInfo.Type  typeVarX = _chartInfo.getTypeVarX();
            LyChartInfo.Type  typeVarY = _chartInfo.getTypeVarY();
			LyFieldInfo		  fieldInfo = null;

			// Calendar constructed only if there are intervals of dates
			Calendar		  calendar = null;

			// last value selected for an interval parameter
			Object			  choosenIntValue = null;
			LyOption		  option = null;

			LyChartController chartController = (LyChartController)this._parent;

			// Resize the area to the maximum
			LyConstraint	  constraint = new LyConstraint();

			constraint.setRowSpan(-1);
			constraint.setRemainder(true);
			constraint.setFill(LyConstraint.Fill.BOTH);
			area.setConstraint(constraint);

			// **** Add Option List for y representation
			// by default : SUM_REAL is checked if numeric values, but if _chartInfo is filled, retrieve which option is checked.

			LyChoice choice = new LyChoice();

			choice.setFill(LyChoice.Fill.VERTICAL);
			choice.setMaxSpan(2);
			boolean  editableYInts = false; // to enable interval fields

			LyOption option1 = new LyOption(LyChartInfo.ValType.REAL.toString(),
                    "LY_REAL");

			if (LyChartInfo.ValType.REAL == _chartInfo.getYVal())
				option1.setChecked(true);
			choice.addOption(option1);

			LyOption option2 = new LyOption(LyChartInfo.ValType.SUM_REAL.toString(),
                    "LY_SUM_REAL");

			if (LyChartInfo.Type.NUMBER != typeVarY)
				option2.setEnabled(false);
			else if ((LyChartInfo.ValType.SUM_REAL == _chartInfo.getYVal())
					|| (_chartInfo.getYVal() == null))
				option2.setChecked(true);
			choice.addOption(option2);

			LyOption option3 = new LyOption(LyChartInfo.ValType.COUNT_INTERVAL.toString(),
                    "LY_COUNT_INTERVAL");

			if ((typeVarY != LyChartInfo.Type.DATE)
					&& (typeVarY != LyChartInfo.Type.NUMBER))
				option3.setEnabled(false);
			else if ((_chartInfo.getYVal() == LyChartInfo.ValType.COUNT_INTERVAL)
					|| ((_chartInfo.getYVal() == null)
						&& (typeVarY == LyChartInfo.Type.DATE)))
			{
				option3.setChecked(true);
				editableYInts = true;
			}
			choice.addOption(option3);

			LyOption option4 = new LyOption(LyChartInfo.ValType.COUNT_REAL.toString(),
                    "LY_COUNT_REAL");

			if ((_chartInfo.getYVal() == LyChartInfo.ValType.COUNT_REAL)
					|| ((_chartInfo.getYVal() == null)
						&& (typeVarY == LyChartInfo.Type.TEXT)))
				option4.setChecked(true);
			choice.addOption(option4);

			// if no option is checked, check one
			if ((!option1.isChecked()) && (!option2.isChecked()) && (!option3.isChecked())
					&& (!option4.isChecked()))
			{
				if (typeVarY == LyChartInfo.Type.DATE)
				{
					option3.setChecked(true);
					editableYInts = true;
				}
				else if (typeVarY == LyChartInfo.Type.NUMBER)
					option2.setChecked(true);
				else
					option1.setChecked(true);
			}

			// add the relation to the field
			field = new LyFormField(LyChartController.Y_TYPE_VALUES);

			LyCaption caption = new LyCaption();

			caption.setLabel("LY_CHART_Y_VAL");
			caption.setColon(true);

			// Caption Constraint
			LyConstraint capConstraint = new LyConstraint();

			capConstraint.setHAlign(LyConstraint.HAlign.LEFT);
			caption.setConstraint(capConstraint);

			tip = new LyTip();
			tip.setLabel("LY_CHART_Y_VAL_TIP");
			field.setTip(tip);

			field.setCaption(caption);
			field.setFieldItem(choice);
			field.setOptional(false);
			// force the browser to check for changes
			field.setNeedCheck(true);

			constraint = new LyConstraint();
			constraint.setHAlign(LyConstraint.HAlign.LEFT);
			field.setConstraint(constraint);

			form.addFormField(field);

			// **** Add attribute List for X group
			LyChartBehaviorInterface behavior = (LyChartBehaviorInterface)chartController
					.getBehavior();
			LyRelation				 fieldRelation = new LyRelation();
			LyInfoList<LyFieldInfo>  fields = behavior.getGroupFields(chartController);
			int						 i, n = fields.getSize();

			// the first link is blank
			LyLink					 link = new LyLink("LY_NO_CHOICE", "            ");

			fieldRelation.insertLink(link, 0);

			// Get fieldinfo list
			for (i = 0; i < n; i++)
			{
				fieldInfo = fields.getInfo(i);

				// The link Id can't be the same than the fieldInfo
				link = new LyLink(fieldInfo.getId(), fieldInfo.getName());

				// if _chartInfo is filled, retrieve if the field is selected.
				if ((null != _chartInfo.getSelectedGroupFieldInfo())
						&& (_chartInfo.getSelectedGroupFieldInfo().equals(fieldInfo.getId())))
					link.setSelected(true);

				fieldRelation.addLink(link);
			}

			fieldRelation.setMultiple(false);

			// add the relation to the field
			field = new LyFormField(LyChartController.SELECT_GROUP_FIELD);

			caption = new LyCaption();
			caption.setLabel("LY_CHART_FIELD_GROUP");
			caption.setColon(true);

			constraint = new LyConstraint();
			constraint.setHAlign(LyConstraint.HAlign.LEFT);
			caption.setConstraint(constraint);

			tip = new LyTip();
			tip.setLabel("LY_CHART_FIELD_GROUP_TIP");
			field.setTip(tip);

			field.setCaption(caption);
			field.setFieldItem(fieldRelation);
			field.setOptional(true);

			form.addFormField(field);

			// if real selected, disable.
			if ((null != _chartInfo)
                    && (LyChartInfo.ValType.REAL == _chartInfo.getYVal()))
				field.setEnabled(false);

			// Y type of date (if Y is a date)
            LyChartInfo.Type typeVar = _chartInfo.getTypeVarY();
			String dateType = null;

			if (typeVar == LyChartInfo.Type.DATE)
			{
				Vector<String> yVars = _chartInfo.getSelectedYFieldsInfo();

				dateType = LyGraphicalChartBuilder.makeYDateType(yVars, _classInfo, _chartInfo);
			}

			// **** the length of the interval for y
			// type date -> a choice (year, month ...) and one input field
			// type number -> only one input field
			// by default : if _chartInfo is filled, retrieve if LyChartInfo.ValType.COUNT_INTERVAL option is checked and then set editable.

			String label;

			if (typeVarY == LyChartInfo.Type.DATE)
			{
				// Add Option List for y intervals : years, months, ...
				// by default : TimeUnit.MONTHS is checked (or HOURS if there is no months),
				//      but if _chartInfo is filled, retrieve which option is checked.
				choice = new LyChoice();
				choice.setFill(LyChoice.Fill.VERTICAL);
				choice.setMaxSpan(3);

				if (dateType.contains("DATE"))
				{
					option = new LyOption(Y_TYPE_INTS + LyIntervals.TimeUnit.YEARS, "LY_YEARS");
					if ((_chartInfo.getYIntervals() != null)
							&& (_chartInfo.getYIntervals().length >= 3)
							&& (_chartInfo.getYIntervals()[2]
								== LyIntervals.TimeUnit.YEARS.toString()))
						option.setChecked(true);
					choice.addOption(option);

					option = new LyOption(Y_TYPE_INTS + LyIntervals.TimeUnit.MONTHS, "LY_MONTHS");
					if ((_chartInfo.getYIntervals() != null)
							&& (_chartInfo.getYIntervals().length >= 3)
							&& (_chartInfo.getYIntervals()[2]
								== LyIntervals.TimeUnit.MONTHS.toString()))
						option.setChecked(true);
					// default value
					if (_chartInfo.getYIntervals() == null)
						option.setChecked(true);
					choice.addOption(option);

					option = new LyOption(Y_TYPE_INTS + LyIntervals.TimeUnit.DAYS, "LY_DAYS");
					if ((_chartInfo.getYIntervals() != null)
							&& (_chartInfo.getYIntervals().length >= 3)
							&& (_chartInfo.getYIntervals()[2]
								== LyIntervals.TimeUnit.DAYS.toString()))
						option.setChecked(true);
					choice.addOption(option);
				}
				if (!dateType.equals("DATE"))
				{
					option = new LyOption(Y_TYPE_INTS + LyIntervals.TimeUnit.HOURS, "LY_HOURS");
					if ((_chartInfo.getYIntervals() != null)
							&& (_chartInfo.getYIntervals().length >= 3)
							&& (_chartInfo.getYIntervals()[2]
								== LyIntervals.TimeUnit.HOURS.toString()))
						option.setChecked(true);
					// default value
					if (_chartInfo.getYIntervals() == null)
						option.setChecked(true);
					choice.addOption(option);
				}
				if (!dateType.equals("DATE"))
				{
					option = new LyOption(Y_TYPE_INTS + LyIntervals.TimeUnit.MINS, "LY_MINS");
					if ((_chartInfo.getYIntervals() != null)
							&& (_chartInfo.getYIntervals().length >= 3)
							&& (_chartInfo.getYIntervals()[2]
								== LyIntervals.TimeUnit.MINS.toString()))
						option.setChecked(true);
					choice.addOption(option);
				}
				if (dateType.equals("DATE_TIME") || dateType.equals("TIME"))
				{
					option = new LyOption(Y_TYPE_INTS + LyIntervals.TimeUnit.SECS, "LY_SECS");
					if ((_chartInfo.getYIntervals() != null)
							&& (_chartInfo.getYIntervals().length >= 3)
							&& (_chartInfo.getYIntervals()[2]
								== LyIntervals.TimeUnit.SECS.toString()))
						option.setChecked(true);
					choice.addOption(option);
				}

				// add the relation to the field

				field = new LyFormField(LyChartController.Y_TYPE_INTS);

				caption = new LyCaption();
				caption.setLabel("LY_TYPE_INTS");
				caption.setColon(true);
				caption.setConstraint(capConstraint);

				field.setCaption(caption);
				field.setFieldItem(choice);
				field.setOptional(true);
				field.setEditable((_chartInfo.getYVal() == LyChartInfo.ValType.COUNT_INTERVAL)
						|| editableYInts);

				form.addFormField(field);

				//  the length of the interval
				field = new LyFormField(LyChartController.LG_INT_Y_AREA);
				label = "LY_LENGTH_INT";

				// by default : retrieve the last entered value
				if ((_chartInfo.getYIntervals() != null)
						&& (_chartInfo.getYIntervals().length >= 2))
					choosenIntValue = _chartInfo.getYIntervals()[1];
				else
					choosenIntValue = "1";

				addFieldND(form, field, label, LyChartInfo.Type.NUMBER, (short)0, dateType, null,
					choosenIntValue);
				field.setEditable((_chartInfo.getYVal() == LyChartInfo.ValType.COUNT_INTERVAL)
						|| editableYInts);
			}
			else // numeric variable
			{
				field = new LyFormField(LyChartController.LG_INT_Y_AREA);
				label = "LY_LENGTH_INT";

				// by default : retrieve the last entered value
				if ((_chartInfo.getYIntervals() != null)
						&& (_chartInfo.getYIntervals().length >= 2))
					choosenIntValue = _chartInfo.getYIntervals()[1];
				else
					choosenIntValue = "1";

				addFieldND(form, field, label, typeVarY, _ydec, dateType, null, choosenIntValue);
				field.setEditable((_chartInfo.getYVal() == LyChartInfo.ValType.COUNT_INTERVAL)
						|| editableYInts);
			}

			// **** a border of the interval for y
			// by default : if _chartInfo is filled, retrieve if LyChartInfo.ValType.COUNT_INTERVAL option is checked and then set editable.
			field = new LyFormField(LyChartController.BORDER_INT_Y);

			tip = new LyTip();
			tip.setLabel("LY_BORDER_INT_TIP");
			field.setTip(tip);
			label = "LY_BORDER_INT";
			fieldInfo = _classInfo.getFieldInfo(_chartInfo.getSelectedYFieldsInfo()
						.elementAt(0), false);

			// reset choosenIntValue
			choosenIntValue = null;

			// for dates with days and months : set the first day of the month by default
			if ((typeVarY == LyChartInfo.Type.DATE)
                    && dateType.contains("DATE"))
				// set the default value to call addFieldND(...)
				choosenIntValue = firstDayOfMonth(fieldInfo, calendar);

			// by default : retrieve the last entered value
			if ((_chartInfo.getYIntervals() != null)
			        && (_chartInfo.getYIntervals().length >= 1))
				choosenIntValue = _chartInfo.getYIntervals()[0];

			addFieldND(form, field, label, typeVarY, _ydec, dateType, fieldInfo, choosenIntValue);
			field.setEditable((_chartInfo.getYVal() == LyChartInfo.ValType.COUNT_INTERVAL)
					|| editableYInts);

			LyArea topArea = new LyArea();

			topArea.setBorder(LyArea.Border.IN);
			form.setBorder(false);
			topArea.addPanel(form);
			area.addPanel(topArea);

			// **** Add Option List for x representation
			// Only if a variable x has been choosen
			// by default : REAL is checked, but if _chartInfo is filled, retrieve which option is checked.
			if (_chartInfo.getSelectedXFieldInfo() != null)
			{
				choice = new LyChoice();
				boolean  editableXInts = false; // to enable interval fields

				LyOption optionX1 = new LyOption(LyChartInfo.ValType.REAL.toString(), "LY_REAL");

				if ((_chartInfo.getXVal() == LyChartInfo.ValType.REAL)
						|| (_chartInfo.getXVal() == null))
					optionX1.setChecked(true);
				choice.addOption(optionX1);

				LyOption optionX2 = new LyOption(LyChartInfo.ValType.INTERVAL.toString(), "LY_INTERVAL");

				choice.addOption(optionX2);
				if ((typeVarX != LyChartInfo.Type.DATE)
						&& (typeVarX != LyChartInfo.Type.NUMBER))
					optionX2.setEnabled(false);
				else if (_chartInfo.getXVal() == LyChartInfo.ValType.INTERVAL)
				{
					optionX2.setChecked(true);
					editableXInts = true;
				}

				// if no option is checked, check REAL one
				if ((!optionX1.isChecked()) && (!optionX2.isChecked()))
					optionX1.setChecked(true);

				// add the relation to the field

				field = new LyFormField(LyChartController.X_TYPE_VALUES);

				caption = new LyCaption();
				caption.setLabel("LY_CHART_X_VAL");
				caption.setColon(true);
				caption.setConstraint(capConstraint);

				tip = new LyTip();
				tip.setLabel("LY_CHART_X_VAL_TIP");
				field.setTip(tip);

				field.setCaption(caption);
				field.setFieldItem(choice);
				field.setOptional(false);
				// force the browser to check for changes
				field.setNeedCheck(true);

				midForm.addFormField(field);

				// x type of date (if x is a date)
				typeVar = _chartInfo.getTypeVarX();

				if (typeVar == LyChartInfo.Type.DATE)
				{
					LyDateFieldInfo date = (LyDateFieldInfo)_classInfo.getFieldInfo(_chartInfo
								.getSelectedXFieldInfo(), false);

					dateType = date.getType().toString();
				}

				// **** the length of the interval for x
				// type number -> only one input field
				// type date -> a choice (year, month ...) and one input field
				//              by default if _chartInfo is empty : TimeUnit.MONTHS is checked (or HOURS if there is no months),
				// by default : if _chartInfo is filled, retrieve if LyChartInfo.ValType.COUNT_INTERVAL option is checked and then set editable.

				if (typeVarX == LyChartInfo.Type.DATE)
				{
					// Add Option List for x intervals : years, months, ...
					choice = new LyChoice();
					choice.setFill(LyChoice.Fill.VERTICAL);
					choice.setMaxSpan(3);

					if (dateType.contains("DATE"))
					{
						option = new LyOption(X_TYPE_INTS + LyIntervals.TimeUnit.YEARS, "LY_YEARS");
						if ((_chartInfo.getXIntervals() != null)
								&& (_chartInfo.getXIntervals().length >= 3)
								&& "YEARS".equals(_chartInfo.getXIntervals()[2]))
							option.setChecked(true);
						choice.addOption(option);
					}
					if (!dateType.equals("DATE"))
					{
						option = new LyOption(X_TYPE_INTS + LyIntervals.TimeUnit.HOURS, "LY_HOURS");
						if ((_chartInfo.getXIntervals() != null)
								&& (_chartInfo.getXIntervals().length >= 3)
								&& "HOURS".equals(_chartInfo.getXIntervals()[2]))
							option.setChecked(true);
						// default value
						if (_chartInfo.getXIntervals() == null)
							option.setChecked(true);
						choice.addOption(option);
					}
                    if (dateType.contains("DATE"))
					{
						option = new LyOption(X_TYPE_INTS + LyIntervals.TimeUnit.MONTHS,
								"LY_MONTHS");
						if ((_chartInfo.getXIntervals() != null)
								&& (_chartInfo.getXIntervals().length >= 3)
								&& "MONTHS".equals(_chartInfo.getXIntervals()[2]))
							option.setChecked(true);
						// default value
						if (_chartInfo.getXIntervals() == null)
							option.setChecked(true);
						choice.addOption(option);
					}
					if (!dateType.equals("DATE"))
					{
						option = new LyOption(X_TYPE_INTS + LyIntervals.TimeUnit.MINS, "LY_MINS");
						if ((_chartInfo.getXIntervals() != null)
								&& (_chartInfo.getXIntervals().length >= 3)
								&& "MINS".equals(_chartInfo.getXIntervals()[2]))
							option.setChecked(true);
						choice.addOption(option);
					}
                    if (dateType.contains("DATE"))
					{
						option = new LyOption(X_TYPE_INTS + LyIntervals.TimeUnit.DAYS, "LY_DAYS");
						if ((_chartInfo.getXIntervals() != null)
								&& (_chartInfo.getXIntervals().length >= 3)
								&& "DAYS".equals(_chartInfo.getXIntervals()[2]))
							option.setChecked(true);
						choice.addOption(option);
					}
					if (dateType.equals("DATE_TIME") || dateType.equals("TIME"))
					{
						option = new LyOption(X_TYPE_INTS + LyIntervals.TimeUnit.SECS, "LY_SECS");
						if ((_chartInfo.getXIntervals() != null)
								&& (_chartInfo.getXIntervals().length >= 3)
								&& "SECS".equals(_chartInfo.getXIntervals()[2]))
							option.setChecked(true);
						choice.addOption(option);
					}

					// add the relation to the field

					field = new LyFormField(LyChartController.X_TYPE_INTS);

					caption = new LyCaption();
					caption.setLabel("LY_TYPE_INTS");
					caption.setColon(true);
					caption.setConstraint(capConstraint);

					field.setCaption(caption);
					field.setFieldItem(choice);
					field.setOptional(true);
					field.setEditable((_chartInfo.getXVal() == LyChartInfo.ValType.INTERVAL)
							|| editableXInts);

					midForm.addFormField(field);

					//  the length of the interval
					field = new LyFormField(LyChartController.LG_INT_X_AREA);
					label = "LY_LENGTH_INT";

					// by default : retrieve the last entered value
					if ((_chartInfo.getXIntervals() != null)
							&& (_chartInfo.getXIntervals().length >= 2))
						choosenIntValue = _chartInfo.getXIntervals()[1];
					else
						choosenIntValue = "1";

					addFieldND(midForm, field, label, LyChartInfo.Type.NUMBER, (short)0, dateType,
						null, choosenIntValue);
					field.setEditable((_chartInfo.getXVal() == LyChartInfo.ValType.INTERVAL)
							|| editableXInts);
				}
				else // numeric variable
				{
					field = new LyFormField(LyChartController.LG_INT_X_AREA);
					label = "LY_LENGTH_INT";

					// by default : retrieve the last entered value
					if ((_chartInfo.getXIntervals() != null)
							&& (_chartInfo.getXIntervals().length >= 2))
						choosenIntValue = _chartInfo.getXIntervals()[1];
					else
						choosenIntValue = "1";

					addFieldND(midForm, field, label, typeVarX, _xdec, dateType, null,
						choosenIntValue);
					field.setEditable((_chartInfo.getXVal() == LyChartInfo.ValType.INTERVAL)
							|| editableXInts);
				}

				// **** a border of the interval for x
				// by default : if _chartInfo is filled, retrieve if LyChartInfo.ValType.COUNT_INTERVAL option is checked and then set editable.
				field = new LyFormField(LyChartController.BORDER_INT_X);

				tip = new LyTip();
				tip.setLabel("LY_BORDER_INT_TIP");
				field.setTip(tip);
				label = "LY_BORDER_INT";
				fieldInfo = _classInfo.getFieldInfo(_chartInfo.getSelectedXFieldInfo(), false);

				// reset choosenIntValue
				choosenIntValue = null;

				// for dates with days and months : set the first day of the month by default
				if ((typeVarX == LyChartInfo.Type.DATE)
                        && dateType.contains("DATE"))
					// set the default value to call addFieldND(...)
					choosenIntValue = firstDayOfMonth(fieldInfo, calendar);

				// by default : retrieve the last entered value
				if ((_chartInfo.getXIntervals() != null)
						&& (_chartInfo.getXIntervals().length >= 1))
					choosenIntValue = _chartInfo.getXIntervals()[0];

				addFieldND(midForm, field, label, typeVarX, _xdec, dateType, fieldInfo,
					choosenIntValue);
				field.setEditable((_chartInfo.getXVal() == LyChartInfo.ValType.INTERVAL)
						|| editableXInts);

				LyArea midArea = new LyArea();

				midArea.setBorder(LyArea.Border.IN);
				midForm.setBorder(false);
				midArea.addPanel(midForm);
				area.addPanel(midArea);
			}

			// **** Add Option List for pie representation
			//Only if there are at least two Y vars and Graphic type is pie
			// by default : if _chartInfo is filled, retrieve the checked option.

//                                              if ((_chartInfo.getXIntervals() != null) && (_chartInfo.getXIntervals().size() >= 3) &&
//                                                      (LyTools.shortFromString(_chartInfo.getXIntervals().elementAt(2).toString()) == LyIntervals.TimeUnit.YEARS))
//                                                      option.setChecked(true);
			if (_chartInfo.getSelectedGraphType() == LyChartInfo.GraphType.PIE)
			{
				boolean enable = true;

				if (_chartInfo.getSelectedXFieldInfo() == null)
					enable = false;

				choice = new LyChoice();
				choice.setFill(LyChoice.Fill.VERTICAL);
				choice.setMaxSpan(2);

				option = new LyOption(PIES_BY_X, "LY_PIES_BY_X");
				option.setChecked(!_chartInfo.getPieBy() && enable);
				choice.addOption(option);

				option = new LyOption(PIES_BY_Y, "LY_PIES_BY_Y");
				option.setChecked(_chartInfo.getPieBy() || !enable);
				choice.addOption(option);

				// add the relation to the field
				field = new LyFormField(LyChartController.PIES_BY);

				caption = new LyCaption();
				caption.setColon(true);
				caption.setConstraint(capConstraint);
				caption.setLabel("LY_TYPE_PIE");

				field.setCaption(caption);
				field.setFieldItem(choice);
				field.setOptional(false);
				field.setEnabled(enable);

				bottomForm.addFormField(field);
				bottomForm.setBorder(false);

				LyArea bottomArea = new LyArea();

				bottomArea.setBorder(LyArea.Border.IN);
				bottomArea.addPanel(bottomForm);
				area.addPanel(bottomArea);
			}

			constraint = new LyConstraint();
			constraint.setFill(LyConstraint.Fill.NONE);
			form.setConstraint(constraint);
			constraint = new LyConstraint();
			constraint.setFill(LyConstraint.Fill.NONE);
			midForm.setConstraint(constraint);
			constraint = new LyConstraint();
			constraint.setFill(LyConstraint.Fill.NONE);
			bottomForm.setConstraint(constraint);

			// second parameter  area is ready
			return (area);
		} // end method buildWin2Area

		/**
		 * Add a numeric or date field to the form.
		 *
		 * @param form      the form in which the field has to be added
		 * @param field     the field to be added in the form
		 * @param label     the label of the field caption
		 * @param typeVar   the type of variable (from LyChartInfo)
		 * @param dec       the number of decimals, ignored if the field is not a number
		 * @param dateType  the type of date, ignored i the field is not a date
		 * @param fieldInfo the first selected fieldInfo. The first value of this one will be set as
		 *                  a default value if lastValue is null.
		 * @param lastValue the last entered value in this field. It will be set as default value.
		 */
		protected void addFieldND(LyForm form, LyFormField field, String label,
            LyChartInfo.Type typeVar, short dec, String dateType,
            LyFieldInfo fieldInfo, Object lastValue)
		{
			LyCaption caption = new LyCaption();
			Object    defaultValue = null;

			caption.setLabel(label);
			caption.setColon(true);
			LyConstraint capConstraint = new LyConstraint();

			capConstraint.setHAlign(LyConstraint.HAlign.LEFT);
			caption.setConstraint(capConstraint);
			field.setCaption(caption);

			// Retrieve the first selected value to make a default value for the beginning of an interval
			if (fieldInfo != null)
			{
				LyObjectList list = getObjectList();

				if ((list != null) && (list.getSize() > 0))
				{
					Object myObject = list.getObject(0); // first line

					if (myObject != null)
						defaultValue = _classInfo.getValue(myObject, fieldInfo);
				}
			}

			if (typeVar == LyChartInfo.Type.NUMBER)
			{
				LyNumber fieldNumber = new LyNumber();

				if (!LyNumber.SHOW_ARROWS)
					fieldNumber.showArrows(false);

				fieldNumber.setDecimals(dec);
				if (defaultValue != null)
				{
					//create a LyNumber label from a Number
					try
					{
						fieldNumber.setLabel(defaultValue.toString());
					}
					catch (Exception e)
					{
					}
				}
				if (lastValue != null)
				{
					//create a LyNumber label from a String
					try
					{
						fieldNumber.setLabel(lastValue.toString());
					}
					catch (Exception e)
					{
					}
				}
				field.setFieldItem(fieldNumber);
				field.setEditable(false);
				form.addFormField(field);

				// set 0 as the min value if interval legth (ie fieldInfo = null)
				if (fieldInfo == null)
					fieldNumber.setVMin(0);
			}

			if (typeVar == LyChartInfo.Type.DATE)
			{
				LyDate fieldDate = new LyDate();

				if (!LyDate.SHOW_ARROWS)
					fieldDate.showArrows(false);

				String myLabel = "0";

				if (defaultValue != null)
				{
					//create a LyDate label from a Date
					LyDateFieldInfo dateField = (LyDateFieldInfo)fieldInfo;

					myLabel = dateField.toString(defaultValue);
					fieldDate.setType(LyDate.getType(dateType));

					try
					{
						fieldDate.setLabel(myLabel);
					}
					catch (Exception e)
					{
					}
				}
				if (lastValue != null)
				{
					try
					{
						// before setting the lastValue, verify that its format is the same that the new field.
						fieldDate.setLabel(lastValue.toString());
					}
					catch (Exception e) // re-fill the date label with the old one
					{
						fieldDate.setLabel(myLabel);
					}
				}
				field.setFieldItem(fieldDate);
				field.setEditable(false);
				form.addFormField(field);
			}
		} // end method addFieldND

		/**
		 * Retrieve the first day of the month, for the first date of the LyFieldInfo.
		 *
		 * @param  fieldInfo the first selected fieldInfo.
		 * @param  calendar  used to set the first day of the month.
		 * @return the date of the first day of the month
		 */
		protected String firstDayOfMonth(LyFieldInfo fieldInfo, Calendar calendar)
		{
			LyDateFieldInfo dateField = (LyDateFieldInfo)fieldInfo;
			LyObjectList    list = getObjectList();
			Object		    value = null;

			if ((list != null) && (list.getSize() > 0))
			{
				Object myObject = list.getObject(0); // first line

				value = _classInfo.getValue(myObject, fieldInfo);
			}

			if (null != value)
			{
				try
				{
					Date myDate = LyDateFormatter.parse(dateField.toString(value),
							dateField.getStringDateFormat());

					// construct the calendar if it has not been constructed yet
					if (calendar == null)
						calendar = getEnvironment().calendarFactory();
					calendar.setTime(myDate);
					// set the first day of the month
					calendar.set(Calendar.DAY_OF_MONTH, 1);

					return LyDateFormatter.format(calendar.getTime(),
						getEnvironment().getStringDateFormat(LyEnvironment.TYPE_DATE_TIME));
				}
				catch (Exception e)
				{
					return null;
				}
			}
			// if no data in the first line, set the first time (1970/01/01)
			else
				return LyDateFormatter.format(new Date(0),
					getEnvironment().getStringDateFormat(LyEnvironment.TYPE_DATE_TIME));
		} // end method firstDayOfMonth

		/**
		 * Creates the command bar and sets its content.
		 *
		 * @return a command bar with three buttons : previous, next and cancel
		 */
		protected LyCommands buildCommands()
		{
			return null;
		}

		/**
		 * Build a text field item
		 * Overriden from LyFormBuilder.
		 * @param textField the date field
		 * @param value a value
		 * @return the asociated field item (can be null)
		 */
		public LyFieldItem buildText(LyTextFieldInfo textField, Object value)
		{
			return null;
		}

		/**
		 * Build a date field item
		 * Overriden from LyFormBuilder.
		 * @param dateField the date field
		 * @param value a value
		 * @return the asociated field item (can be null)
		 */
		public LyFieldItem buildDate(LyDateFieldInfo dateField, Object value)
		{
			return null;
		}

		/**
		 * Build a number field item
		 * Overriden from LyFormBuilder.
		 * @param numberField the number field
		 * @param value a value
		 * @return the asociated field item (can be null)
		 */
		public LyFieldItem buildNumber(LyNumberFieldInfo numberField, Object value)
		{
			return null;
		}

		/**
		 * Build a relation field item
		 * Overriden from LyFormBuilder.
		 * @param relationField the relation field
		 * @param value a value
		 * @return the asociated field item (can be null)
		 */
		public LyFieldItem buildRelation(LyRelationFieldInfo relationField, Object value)
		{
			return null;
		}

		/**
		 * Build a choice field item
		 * Overriden from LyFormBuilder.
		 * @param choiceField the choice field info
		 * @param value a value
		 * @return the associated field item
		 */
		public LyFieldItem buildChoice(LyChoiceFieldInfo choiceField, Object value)
		{
			return null;
		}

		/**
		 * Build a file field item
		 * Overriden from LyFormBuilder.
		 * @param fileField the file field info
		 * @param value a value
		 * @return the associated field item (can be null)
		 */
		public LyFieldItem buildFile(LyFileFieldInfo fileField, Object value)
		{
			return null;
		}

	} // end class _LyChartWin2Builder


	/* Third window */
	/**
	 * This internal class constructs the third parameter window.
	 */
	class _LyChartWin3Builder extends LyFormBuilder
	{

		/**
		 * Default constructor.
		 *
		 * @param parent parent of the controller
		 */
		public _LyChartWin3Builder(LyChartController parent)
		{
			super(parent);
		}

		/**
		 * Builds the view component for the specified parent controller.
		 *
		 * @return the view Component
		 */
		public LyComponent build()
		{

			/* The chart 3rd window is an area which contains :
			        - A titleBar (description)
			        - A form component
			*/
			LyArea area = buildWin3Area();

			area.setController(this._parent.getId());

			this._component = area;

			return (area);
		}

		/**
		 * Creates the third window form.
		 *
		 * @return the new third form
		 */
		protected LyArea buildWin3Area()
		{
			LyArea		  area = new LyArea();
			LyForm		  form = new LyForm(this._parent.getId());
			LyForm		  midForm = new LyForm(this._parent.getId());
			LyForm		  bottomForm = new LyForm(this._parent.getId());
			LyFormField   field;
			LyEnvironment environment = this._parent.getEnvironment();
			LyTip		  tip;

			// Resize the area to the maximum
			LyConstraint  constraint = new LyConstraint();

			constraint.setRowSpan(-1);
			constraint.setRemainder(true);
			constraint.setFill(LyConstraint.Fill.BOTH);
			area.setConstraint(constraint);

			// add the title to the field. The default title is blank of the one which is in _chartInfo
			LyText    fieldText = new LyText();
			LyCaption caption = new LyCaption();

			field = new LyFormField(LyChartController.TITLE_AREA);
			caption.setLabel("LY_CHART_TITLE");
			caption.setColon(true);
			// Caption Constraint
			LyConstraint capConstraint = new LyConstraint();

			capConstraint.setHAlign(LyConstraint.HAlign.LEFT);
			caption.setConstraint(capConstraint);
			field.setCaption(caption);

			tip = new LyTip();
			tip.setLabel("LY_CHART_TITLE_TIP");
			field.setTip(tip);

			fieldText.setNCols(TITLE_WIDTH);
			fieldText.setNRows(TITLE_HEIGHT);
			fieldText.setWrap(false);
			fieldText.setLabel(environment.translate(_chartInfo.getTitle()));
			field.setEditable(true);
			field.setFieldItem(fieldText);
			form.addFormField(field);

			//creating the intermediate area
			LyArea topArea = new LyArea();

			topArea.setBorder(LyArea.Border.IN);
			form.setBorder(false);
			topArea.addPanel(form);
			area.addPanel(topArea);

			// Add the titles of the axis only if Graphic type is not pie
			if (_chartInfo.getSelectedGraphType() != LyChartInfo.GraphType.PIE)
			{
				// X axis
				field = new LyFormField(LyChartController.X_TITLE);
				String label = "LY_X_TITLE";
				String selectedVal = getChoosenOption((LyFormField)_panel2.getComponent(
						LyChartController.Y_TYPE_VALUES));
		        LyChartInfo.ValType     valType = LyInfoTokens.getChartInfoValType(selectedVal);

				addFieldText(form, field, label);

				//default value : the last entered value if X var has change, or the name of X variable if there is one (and if it's the first time)
				LyFieldItem fieldItem = field.getFieldItem();

				if ((_first3 && (_chartInfo.getXTitle() == null))
                      || _defaultXAxisTitle)
				{
					if ((valType == LyChartInfo.ValType.COUNT_INTERVAL)
							&& (_chartInfo.getSelectedYFieldsInfo().size() == 1))
					{
						String choosenOption = getChoosenOption((LyFormField)_panel2.getComponent(
								LyChartController.Y_TYPE_INTS));
						String suffix = "";
						String intLength = getNumDateLabel(
								((LyFormField)_panel2.getComponent(LyChartController.LG_INT_Y_AREA))
									.getFieldItem());

						if (choosenOption != null)
							choosenOption = choosenOption.substring(Y_TYPE_INTS.length());

						if ("1".equals(intLength))
						{
							if ((choosenOption == null)
									|| choosenOption.equals(LyIntervals.TimeUnit.YEARS + ""))
								suffix = " / " + environment.translate("LY_YEAR_INTERVAL");
							else if (choosenOption.equals(LyIntervals.TimeUnit.MONTHS + ""))
								suffix = " / " + environment.translate("LY_MONTH_INTERVAL");
							else if (choosenOption.equals(LyIntervals.TimeUnit.DAYS + ""))
								suffix = " / " + environment.translate("LY_DAY_INTERVAL");
						}

						((LyText)fieldItem).setLabel(environment.translate(
								(_classInfo.getFieldInfo(
										_chartInfo.getSelectedYFieldsInfo().elementAt(0),
										false).getName())) + suffix);
					}
					else if (_chartInfo.getSelectedXFieldInfo() != null)
						((LyText)fieldItem).setLabel(environment.translate(
								_classInfo.getFieldInfo(_chartInfo.getSelectedXFieldInfo(),
									false).getName()));
					else
						((LyText)fieldItem).setLabel(null);
				}
				else if (_chartInfo.getXTitle() != null)
					((LyText)fieldItem).setLabel(environment.translate(_chartInfo.getXTitle()));

				tip = new LyTip();
				tip.setLabel("LY_X_TITLE_TIP");
				field.setTip(tip);

				// Y axis
				field = new LyFormField(LyChartController.Y_TITLE);
				label = "LY_Y_TITLE";
				addFieldText(form, field, label);
				//default value : the last entered value if Y vars have change, or the name of Y variable if there is one and only one (and if it's the first time)
				fieldItem = field.getFieldItem();

				if (((_first3) && (_chartInfo.getYTitle() == null)) || (_defaultYAxisTitle))
				{
					if ((valType == LyChartInfo.ValType.COUNT_REAL)
							|| (valType == LyChartInfo.ValType.COUNT_INTERVAL))
						((LyText)fieldItem).setLabel(environment.translate("LY_NB_OBJECTS"));
					else if (_chartInfo.getSelectedYFieldsInfo().size() == 1)
						((LyText)fieldItem).setLabel(environment.translate(
								(_classInfo.getFieldInfo(
										_chartInfo.getSelectedYFieldsInfo().elementAt(0),
										false).getName())));
					else
						((LyText)fieldItem).setLabel(null);
				}
				else if (_chartInfo.getYTitle() != null)
					((LyText)fieldItem).setLabel(environment.translate(_chartInfo.getYTitle()));

				tip = new LyTip();
				tip.setLabel("LY_Y_TITLE_TIP");
				field.setTip(tip);

				// X type of date (if X is a date)
				LyChartInfo.Type  typeVar = _chartInfo.getTypeVarX();
				String  dateType = null;

				if (typeVar == LyChartInfo.Type.DATE)
				{
					LyDateFieldInfo date = (LyDateFieldInfo)_classInfo.getFieldInfo(_chartInfo
								.getSelectedXFieldInfo(), false);

					dateType = date.getType().toString();
				}

				// X axis minimum value
				field = new LyFormField(LyChartController.X_MIN_AXIS);
				label = "LY_X_MIN_AXIS";
				addAxisFieldND(midForm, field, label, typeVar, _xdec, dateType);

				//default value : the value stocked in _chartInfo if there is one
				fieldItem = field.getFieldItem();
				if ((_chartInfo.getXAxis() != null)
				    && (_chartInfo.getXAxis()[0] != null))
				{
					if (fieldItem instanceof LyNumber)
						((LyNumber)fieldItem).setLabel(_chartInfo.getXAxis()[0]);
					if (fieldItem instanceof LyDate)
						((LyDate)fieldItem).setLabel(_chartInfo.getXAxis()[0]);
				}

				tip = new LyTip();
				tip.setLabel("LY_BOUNDS_AXIS_TIP");
				field.setTip(tip);

				// X axis maximum value
				field = new LyFormField(LyChartController.X_MAX_AXIS);
				label = "LY_X_MAX_AXIS";
				addAxisFieldND(midForm, field, label, typeVar, _xdec, dateType);

				//default value : the value stocked in _chartInfo if there is one
				fieldItem = field.getFieldItem();
				if ((_chartInfo.getXAxis() != null)
				    && (_chartInfo.getXAxis()[1] != null))
				{
					if (fieldItem instanceof LyNumber)
						((LyNumber)fieldItem).setLabel(_chartInfo.getXAxis()[1]);
					if (fieldItem instanceof LyDate)
						((LyDate)fieldItem).setLabel(_chartInfo.getXAxis()[1]);
				}

				tip = new LyTip();
				tip.setLabel("LY_BOUNDS_AXIS_TIP");
				field.setTip(tip);

				// X axis graduation value only for numeric variables
				if (typeVar == LyChartInfo.Type.NUMBER)
				{
					field = new LyFormField(LyChartController.X_GRAD_AXIS);
					label = "LY_X_GRAD_AXIS";
					addAxisFieldND(midForm, field, label, typeVar, _xdec, dateType);
					// Set min value to 0
					((LyNumber)field.getFieldItem()).setVMin(0);

					//default value : the value stocked in _chartInfo if there is one
					fieldItem = field.getFieldItem();
					if ((_chartInfo.getXAxis() != null)
						&& (_chartInfo.getXAxis()[2] != null))
					{
						if (fieldItem instanceof LyNumber)
							((LyNumber)fieldItem).setLabel(_chartInfo.getXAxis()[2]);
						if (fieldItem instanceof LyDate)
							((LyDate)fieldItem).setLabel(_chartInfo.getXAxis()[2]);
					}
				}

				// Y type of date (if Y is a date)
				typeVar = _chartInfo.getTypeVarY();
				if (typeVar == LyChartInfo.Type.DATE)
				{
					Vector<String> yVars = _chartInfo.getSelectedYFieldsInfo();

					dateType = LyGraphicalChartBuilder.makeYDateType(yVars, _classInfo, _chartInfo);
				}

				// Y axis minimum value
				typeVar = _chartInfo.getTypeVarY();
				field = new LyFormField(LyChartController.Y_MIN_AXIS);
				label = "LY_Y_MIN_AXIS";
				// if count, force numeric values for the Y axis and set default value to 0
				if ((_chartInfo.getYVal() == LyChartInfo.ValType.COUNT_REAL)
						|| (_chartInfo.getYVal() == LyChartInfo.ValType.COUNT_INTERVAL))
				{
					addAxisFieldND(midForm, field, label, LyChartInfo.Type.NUMBER, (short)-1,
						dateType);
					((LyNumber)field.getFieldItem()).setLabel("0");
				}
				else
					addAxisFieldND(midForm, field, label, typeVar, _ydec, dateType);

				// if numeric, set default value to 0
				if (typeVar == LyChartInfo.Type.NUMBER)
					((LyNumber)field.getFieldItem()).setLabel("0");

				//default value : the value stocked in _chartInfo if there is one
				fieldItem = field.getFieldItem();
				if ((_chartInfo.getYAxis() != null) && (_chartInfo.getYAxis()[0] != null))
				{
					if (fieldItem instanceof LyNumber)
						((LyNumber)fieldItem).setLabel(_chartInfo.getYAxis()[0]);
					if (fieldItem instanceof LyDate)
						((LyDate)fieldItem).setLabel(_chartInfo.getYAxis()[0]);
				}

				tip = new LyTip();
				tip.setLabel("LY_BOUNDS_AXIS_TIP");
				field.setTip(tip);

				// Y axis maximum value
				field = new LyFormField(LyChartController.Y_MAX_AXIS);
				label = "LY_Y_MAX_AXIS";
				// if count, force numeric values for the Y axis
				if ((_chartInfo.getYVal() == LyChartInfo.ValType.COUNT_REAL)
						|| (_chartInfo.getYVal() == LyChartInfo.ValType.COUNT_INTERVAL))
					addAxisFieldND(midForm, field, label, LyChartInfo.Type.NUMBER, (short)-1,
						dateType);
				else
					addAxisFieldND(midForm, field, label, typeVar, _ydec, dateType);

				//default value : the value stocked in _chartInfo if there is one
				fieldItem = field.getFieldItem();
				if ((_chartInfo.getYAxis() != null) && (_chartInfo.getYAxis()[1] != null))
				{
					if (fieldItem instanceof LyNumber)
						((LyNumber)fieldItem).setLabel(_chartInfo.getYAxis()[1]);
					if (fieldItem instanceof LyDate)
						((LyDate)fieldItem).setLabel(_chartInfo.getYAxis()[1]);
				}

				tip = new LyTip();
				tip.setLabel("LY_BOUNDS_AXIS_TIP");
				field.setTip(tip);

				// Y axis graduation value only for numeric variables
				if (typeVar == LyChartInfo.Type.NUMBER)
				{
					field = new LyFormField(LyChartController.Y_GRAD_AXIS);
					label = "LY_Y_GRAD_AXIS";
					// if count, force numeric values for the Y axis
					if ((_chartInfo.getYVal() == LyChartInfo.ValType.COUNT_REAL)
							|| (_chartInfo.getYVal() == LyChartInfo.ValType.COUNT_INTERVAL))
						addAxisFieldND(midForm, field, label, LyChartInfo.Type.NUMBER, (short)-1,
							dateType);
					else
						addAxisFieldND(midForm, field, label, typeVar, _ydec, dateType);

					fieldItem = field.getFieldItem();

					// Set min value to 0
					((LyNumber)fieldItem).setVMin(0);

					//default value : the value stocked in _chartInfo if there is one
					if ((_chartInfo.getYAxis() != null)
							&& (_chartInfo.getYAxis()[2] != null))
					{
						if (fieldItem instanceof LyNumber)
							((LyNumber)fieldItem).setLabel(_chartInfo.getYAxis()[2]);
						if (fieldItem instanceof LyDate)
							((LyDate)fieldItem).setLabel(_chartInfo.getYAxis()[2]);
					}
				}
				//creating the intermediate area if midForm is not empty
				if (midForm.getFormFieldCount() > 0)
				{
					LyArea midArea = new LyArea();

					midArea.setBorder(LyArea.Border.IN);
					midForm.setBorder(false);
					midArea.addPanel(midForm);
					area.addPanel(midArea);
				}
			}

			// Choice for display options
			LyChoice displayChoice = new LyChoice();

			displayChoice.setMultiple(true);
			displayChoice.setFill(LyChoice.Fill.VERTICAL);
			displayChoice.setMaxSpan(4);

			// Asking for X grid
			LyOption option = new LyOption("xGrid", "LY_X_GRID");

			displayChoice.addOption(option);
			//default value : the value stocked in _chartInfo
			option.setChecked(_chartInfo.getXGrid());

			// Asking for Y grid
			option = new LyOption("yGrid", "LY_Y_GRID");
			displayChoice.addOption(option);
			//default value : the value stocked in _chartInfo
			option.setChecked(_chartInfo.getYGrid());

			// Asking for X axis vertical labels
			option = new LyOption("_xVertic", "LY_X_VERTICAL_LABELS");
			//default value :
			// if it's the first time : true for text or date, false for numbers
			// else : the value stocked in _chartInfo
			if (_first3
                    && (_chartInfo.getTypeVarX() != LyChartInfo.Type.NUMBER))
				option.setChecked(true);
			else
				option.setChecked(_chartInfo.getXVerticalLabels());
			displayChoice.addOption(option);

			// Enabled axis appearence options only if Graphic type is not pie
			if (_chartInfo.getSelectedGraphType() != LyChartInfo.GraphType.PIE)
			{
				displayChoice.enableOption(0, true);
				displayChoice.enableOption(2, true);
			}
			else
			{
				displayChoice.enableOption(0, false);
				displayChoice.enableOption(2, false);
			}

			// Add 3D effect option (enabled if the graphic type supports it)
			option = new LyOption("_3dEffect", "LY_CHART_3D_EFFECT");
			//default value : the value stocked in _chartInfo
			option.setChecked(_chartInfo.get3dEffect());
			if ((_chartInfo.getSelectedGraphType() == LyChartInfo.GraphType.PIE)
					|| (_chartInfo.getSelectedGraphType() == LyChartInfo.GraphType.BAR)
					|| (_chartInfo.getSelectedGraphType() == LyChartInfo.GraphType.STACK_BAR))
				option.setEnabled(true);
			else
				option.setEnabled(false);
			displayChoice.addOption(option);

			// Show or hide the legend
			option = new LyOption("_showLegend", "LY_CHART_SHOW_LEGEND");
			//default value : the value stocked in _chartInfo
			option.setChecked(_chartInfo.getShowLegend());
			displayChoice.addOption(option);

			// Show the label tips always or only when the mouse is over the point
			option = new LyOption("_showTips", "LY_CHART_SHOW_TIPS");
			//default value : the value stocked in _chartInfo
			option.setChecked(_chartInfo.getShowTips());
			displayChoice.addOption(option);

			// Transform data to ratios (enabled for count or sum, not real values)
			option = new LyOption("_ratios", "LY_CHART_RATIOS");
			//default value : the value stocked in _chartInfo
			option.setChecked(_chartInfo.getRatios());
			if (LyChartInfo.ValType.REAL != _chartInfo.getYVal())
				option.setEnabled(true);
			else
			{
				// if the option is disabled, don't check it
				option.setEnabled(false);
				option.setChecked(false);
			}
			displayChoice.addOption(option);

			// add the Choice Options to a new field
			LyFormField displayField = new LyFormField(LyChartController.CHART_DISPLAY_OPTIONS);

			caption = new LyCaption();
			caption.setLabel("LY_CHART_DISPLAY_OPTIONS");
			caption.setColon(true);
			// Caption Constraint
			caption.setConstraint(capConstraint);

			displayField.setCaption(caption);
			displayField.setFieldItem(displayChoice);

			tip = new LyTip();
			tip.setLabel("LY_DISPLAY_OPTIONS_TIP");
			displayField.setTip(tip);

			bottomForm.addFormField(displayField);
			bottomForm.setBorder(false);

			//creating the intermediate area
			LyArea bottomArea = new LyArea();

			bottomArea.setBorder(LyArea.Border.IN);
			bottomForm.setBorder(false);
			bottomArea.addPanel(bottomForm);
			area.addPanel(bottomArea);

			constraint = new LyConstraint();
			constraint.setFill(LyConstraint.Fill.NONE);
			form.setConstraint(constraint);
			constraint = new LyConstraint();
			constraint.setFill(LyConstraint.Fill.NONE);
			midForm.setConstraint(constraint);
			constraint = new LyConstraint();
			constraint.setFill(LyConstraint.Fill.NONE);
			bottomForm.setConstraint(constraint);

			return (area);
		} // end method buildWin3Area

		/**
		 * Add a text field to the form.
		 *
		 * @param form  the form concerned
		 * @param field the field which label have to be updated
		 * @param label the new label of the field
		 */
		protected void addFieldText(LyForm form, LyFormField field, String label)
		{
			LyText    fieldText = new LyText();
			LyCaption caption = new LyCaption();

			;
			caption.setLabel(label);
			caption.setColon(true);
			// Caption Constraint
			LyConstraint capConstraint = new LyConstraint();

			capConstraint.setHAlign(LyConstraint.HAlign.LEFT);
			caption.setConstraint(capConstraint);

			field.setCaption(caption);
			fieldText.setWrap(false);
			fieldText.setNCols(AXIS_TITLES_WIDTH);
			field.setEditable(true);
			field.setFieldItem(fieldText);
			form.addFormField(field);
		}

		/**
		 * Add a numeric or date field to the form.
		 *
		 * @param form     the form in which the field has to be added
		 * @param field    the field to be added in the form
		 * @param label    the label of the field caption
		 * @param typeVar  the type of variable (from LyChartInfo)
		 * @param dec      the number of decimals, ignored if the field is not a number
		 * @param dateType the type of date, ignored i the field is not a date
		 */
		protected void addAxisFieldND(LyForm form, LyFormField field, String label,
            LyChartInfo.Type typeVar, short dec, String dateType)
		{
			LyCaption caption = new LyCaption();

			caption.setLabel(label);
			caption.setColon(true);
			// Caption Constraint
			LyConstraint capConstraint = new LyConstraint();

			capConstraint.setHAlign(LyConstraint.HAlign.LEFT);
			caption.setConstraint(capConstraint);

			field.setCaption(caption);

			if (typeVar == LyChartInfo.Type.NUMBER)
			{
				LyNumber fieldNumber = new LyNumber();

				if (!LyNumber.SHOW_ARROWS)
					fieldNumber.showArrows(false);

				fieldNumber.setDecimals(dec + 1);
				field.setFieldItem(fieldNumber);
				form.addFormField(field);
			}

			if (typeVar == LyChartInfo.Type.DATE)
			{
				LyDate fieldDate = new LyDate();

				fieldDate.setType(LyDate.getType(dateType));

				if (!LyDate.SHOW_ARROWS)
					fieldDate.showArrows(false);

				field.setFieldItem(fieldDate);
				form.addFormField(field);
			}
		} // end method addAxisFieldND

		/**
		 * Build the commands item
		 * Overriden from LyFormBuilder.
		 *
		 * @return the commands (can be null)
		 */
		protected LyCommands buildCommands()
		{
			return null;
		}

		/**
		 * Build a text field item
		 * Overriden from LyFormBuilder.
		 * @param textField the date field
		 * @param value a value
		 * @return the asociated field item (can be null)
		 */
		public LyFieldItem buildText(LyTextFieldInfo textField, Object value)
		{
			return null;
		}

		/**
		 * Build a date field item
		 * Overriden from LyFormBuilder.
		 * @param dateField the date field
		 * @param value a value
		 * @return the asociated field item (can be null)
		 */
		public LyFieldItem buildDate(LyDateFieldInfo dateField, Object value)
		{
			return null;
		}

		/**
		 * Build a number field item
		 * Overriden from LyFormBuilder.
		 * @param numberField the number field
		 * @param value a value
		 * @return the asociated field item (can be null)
		 */
		public LyFieldItem buildNumber(LyNumberFieldInfo numberField, Object value)
		{
			return null;
		}

		/**
		 * Build a relation field item
		 * Overriden from LyFormBuilder.
		 * @param relationField the relation field
		 * @param value a value
		 * @return the asociated field item (can be null)
		 */
		public LyFieldItem buildRelation(LyRelationFieldInfo relationField, Object value)
		{
			return null;
		}

		/**
		 * Build a choice field item
		 * Overriden from LyFormBuilder.
		 * @param choiceField the choice field info
		 * @param value a value
		 * @return the associated field item
		 */
		public LyFieldItem buildChoice(LyChoiceFieldInfo choiceField, Object value)
		{
			return null;
		}

		/**
		 * Build a file field item
		 * Overriden from LyFormBuilder.
		 * @param fileField the file field info
		 * @param value a value
		 * @return the associated field item (can be null)
		 */
		public LyFieldItem buildFile(LyFileFieldInfo fileField, Object value)
		{
			return null;
		}

	} // end class _LyChartWin3Builder

} // end class LyChartController
