package leon.control;

import leon.app.*;
import leon.app.behavior.*;
import leon.control.builder.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;

public class LyChatAskController implements LyProcessActionInterface
{

	public LyController processAction(LySession session, LyController parent,
			LyAction action, LyClassInfo classInfo, LyObjectList context)
	{
		
		LyController socialController = (LyController)session.getValue("social_controller");
		
		if (socialController != null)
		{
			LySocialBuilder socialBuilder = (LySocialBuilder)socialController.getBuilder();
			LyEnvironment env = session.getEnvironment();
			LyApplication application = session.getApplication();
			String maxChat = env.getResource("LY_CHAT_MAX_CONVERSATIONS");
			int maxChatTab = LyTools.intFromString(maxChat);
			if (socialBuilder != null && socialBuilder.getTabCount() < maxChatTab)
			{
				LyViewManager 	chatViewManager;
				LySession		targetSession;
				
				// set unavailable on session
				session.setChatting(true);
				// construct chat view
				LyAction chatAction = application.getActionWithId("_chat", null);
				LyAreaController userInputController = new LyUserInputController(session, parent, chatAction);
				userInputController.setContext(context);
				userInputController.initialize();
				
				// add chat view to social panel chat tabs
				LyTab tab = socialBuilder.addTab(userInputController);
				LyChatBehavior behavior = (LyChatBehavior)userInputController.getBehavior();
				behavior.setConversationTab(tab);
				
				LyObject currentUser = session.getUser(), user, targetSessionUser;
				// look for target user session to ask chat demand
				int nbUsers = 	context.getSize();
				for (int i = 0; i < nbUsers; i++)
				{
					user = context.getObject(i);
					for (int j = 0; j < application.getSessionCount(); j++)
					{
						targetSession = (LySession) application.getSession(j);
						targetSessionUser = targetSession.getUser();
						
						if (targetSessionUser != null && user.getId().equals(targetSessionUser.getId()))
						{
							LyEnvironment targetEnv = targetSession.getEnvironment();
							if (!checkTargetSession(targetSession, maxChatTab))
							{
								behavior.writeMessage(env.translate("LY_TARGET_MAXCONV_MSG"), user, true);
								break;
							}
							LyConversation conversation = behavior.getConversation();
							// if not already in conversation, send question
							if (conversation != null && conversation.getTargetUserSession(targetSession.getId()) == null)
							{
								chatViewManager = targetSession.getViewManager();
								StringBuffer message = new StringBuffer();
								message.append(currentUser.getName());
								message.append(" ");
								message.append(targetEnv.translate("LY_CHAT_DEMAND_MSG"));
								chatViewManager.askQuestion(null, message.toString(), behavior, targetSession);
							}
							break;
						}
					}
				}
			}
			else
			{
				session.getViewManager().showMessage(null, "", "LY_WARNING_ICON", env.translate("LY_MAXCONV_MSG"), null, null, null);
			}
		}
		return socialController;
	}
	
	
	private boolean checkTargetSession(LySession targetSession, int maxTabCount)
	{
		LyController socialController = (LyController)targetSession.getValue("social_controller");
		if (socialController != null)
		{
			LySocialBuilder targetBuilder = (LySocialBuilder)socialController.getBuilder();
			if (targetBuilder.getTabCount() >= maxTabCount)
			{
				return false;
			}
		}
		return true;
	}
}
