/*
 * LyChooseFieldsController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.control;

import leon.app.*;
import leon.app.behavior.*;
import leon.app.behaviorinterface.*;
import leon.control.builder.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;
import leon.view.event.*;


/**
 * This is the controller for the find action.
 *
 * @version $Id: LyChooseFieldsController.java 40687 2012-09-06 15:07:47Z jlsaouli $
 * @author  Lyria
 */
public class LyChooseFieldsController extends LyUserInputController
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyChooseFieldsController.java 40687 2012-09-06 15:07:47Z jlsaouli $";

	/** Menu id : ChooseFields. */
	public static final String DEMAND_CHOOSE_FIELDS = "_chooseFields".intern();

	/** Menu id : ChooseFields. */
	public static final String DEMAND_CLOSE_CHOOSE_FIELDS = "_closeChooseFields".intern();

	/**
	 * Build the choose field controller
	 * @param session the current session
	 * @param parent the parent controller
	 * @param action the associated action
	 * @param classInfo a classInfo
	 * @param context the context of this action
	 */
	public LyChooseFieldsController(LySession session, LyController parent, LyAction action,
		LyClassInfo classInfo, LyObjectList context)
	{
		super(session, parent, action, classInfo, context);

		_hasViewController = false;
		setBuilder(new _LyChooseFieldsBuilder(this));
		setBehavior(new _LyChooseFieldsBehavior());
	}

	/**
	 * Complete the component
	 */
	protected void completeComponent()
	{
		LyArea area = (LyArea)getComponent();

		// If scrolling is not set, form may appear in a strange way in html mode
		area.setScrolling(LyArea.Scrolling.AUTO);
	}

	class _LyChooseFieldsBuilder extends LyUserInputBuilder
	{
		public _LyChooseFieldsBuilder(LyController parent)
		{
			super(parent);
		}

		public void buildFormContent()
		{
			super.buildFormContent();

			if (_form != null)
			{
				LyConstraint	constraint = new LyConstraint();
				
				constraint.setFill(LyConstraint.Fill.NONE);
				_form.setConstraint(constraint);
				_form.setAlign(LyForm.Align.LEFT);
				_form.setBorder(false);
			}
			
			LyRelation  relation = addRelation(DEMAND_CHOOSE_FIELDS, "LY_CHOOSE_COLUMNS", true,
					false);
			LyFormField formField = (LyFormField)relation.getParent();

			relation.setOrdered(true);
			relation.setNCols(40);
			formField.setNeedCheck(true);

			LyCaption    caption = formField.getCaption();
			LyConstraint constraint = new LyConstraint();

			constraint.setHAlign(LyConstraint.HAlign.LEFT);
			constraint.setColSpan(LyConstraint.REMAINDER);
			constraint.setRemainder(true);
			caption.setConstraint(constraint);

			LySimpleTableController  tableController = (LySimpleTableController)
				getParentAreaController();
			LyInfoList<LyFieldInfo>  fields = tableController.getRealFields();
			int						 i, n = _classInfo.getFieldInfoCount();
			LyFieldInfo				 fieldInfo;
			LyLink					 link;

			// Add not selected field infos
			LyTableBehaviorInterface behavior = (LyTableBehaviorInterface)tableController
					.getBehavior();
			LyInfoList<LyFieldInfo>  availableFields = behavior.getChooseColumnsFieldInfos(
					tableController);

			if (availableFields != null)
			{
				n = availableFields.getSize();
				for (i = 0; i < n; i++)
				{
					fieldInfo = availableFields.getInfo(i);
					if (fields.contains(fieldInfo))
						continue;

					link = new LyLink(fieldInfo.getId(), fieldInfo.getAlias());
					relation.addLink(link);
				}
			}
			else
			{
				for (i = 0; i < n; i++)
				{
					fieldInfo = _classInfo.getFieldInfo(i);
					if (fields.contains(fieldInfo))
						continue;

					if ((fieldInfo.hasMark(LyFieldInfo.MARK_SECRET)
								|| fieldInfo.hasMark(LyFieldInfo.MARK_HIDDEN))
						/*&& ((_fieldMarks == null) || !fieldInfo.hasMarks(_fieldMarks, false))*/)
						continue;

					link = new LyLink(fieldInfo.getId(), fieldInfo.getAlias());
					relation.addLink(link);
				}
			}

			// Add selections
			for (i = 0, n = fields.getSize(); i < n; i++)
			{
				fieldInfo = fields.getInfo(i);

				if ((availableFields != null) && (!availableFields.contains(fieldInfo)))
					continue;

				link = new LyLink(fieldInfo.getId(), fieldInfo.getAlias());
				link.setSelected(true);
				relation.addLink(link);
			}
		} // end method buildFormContent

		/**
		 * Creates the command bar and sets its content.
		 *
		 * @return the commands component
		 */
		protected LyCommands buildCommands()
		{
			LyAction action = ((LyAreaController)_parent).getAction();

			if (action.hasMark(LyAction.MARK_NO_COMMANDS))
				return null;

			LyCommands commands = new LyCommands();
			LyCommand  command;
			LyEnvironment environment = _parent.getEnvironment();

			command = new LyCommand(DEMAND_CLOSE_CHOOSE_FIELDS, "LY_CLOSE");
			command.setAction(LyCommand.Action.CLOSE);
			
			if (environment.checkImage("LY_COMMAND_CLOSE"))
				command.setImage("LY_COMMAND_CLOSE");
			
			commands.addCommand(command);

			command = buildHelpCommand();
			if (command != null)
				commands.addCommand(command);

			return commands;
		} // end method buildCommands
	} // end class _LyChooseFieldsBuilder


	class _LyChooseFieldsBehavior extends LyUserInputBehavior
	{

		/**
		 * Invoked when the user validates the content of the input.
		 *
		 * @param controller the current LyUserInputController instance
		 * @param demand     demand parameters sent by the view component.
		 */
		public void userInputDemand(LyController controller, LyDemand demand)
		{
			// Close find dialog
			String id = demand.getSource();

			if (id.equals(DEMAND_CHOOSE_FIELDS))
			{
				LyParameterValues paramValues = demand.getParameterValues();

				if (paramValues != null)
				{
					// Update fields list
					String[]			    fieldIds = paramValues.getParameterValues(
							DEMAND_CHOOSE_FIELDS);
					int					    i, n = ((fieldIds == null) ? 0 : fieldIds.length);
					LyInfoList<LyFieldInfo> fields;

					if (n == 0)
						fields = null;
					else
					{
						LyFieldInfo fieldInfo;

						fields = new LyInfoList<LyFieldInfo>();
						for (i = 0; i < n; i++)
						{
							fieldInfo = _classInfo.getFieldInfo(fieldIds[i]);
							fields.addInfo(fieldInfo);
						}
					}

					// Set fields
					LySimpleTableController  tableController = (LySimpleTableController)
						getParentAreaController();
					LyTableBehaviorInterface behavior = (LyTableBehaviorInterface)tableController.getBehavior();
                    
					fields = behavior.getFields(tableController, fields);
					tableController.setFields(fields);
				}
			}
			else if (id.equals(DEMAND_CLOSE_CHOOSE_FIELDS))
			{
				hideView();
                ((LyUserInputController)controller).free(_parent);
                return;
			}

			super.userInputDemand(controller, demand);
		} // end method userInputDemand
	} // end class _LyChooseFieldsBehavior
} // end class LyChooseFieldsController
