/*
 * LyFastFilterEndInterface.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyFastFilterEndInterface.java 35839 2012-06-12 13:43:15Z nhamon $
 */

package leon.control;

import leon.info.*;


/**
 * Implemented by the client of a fast filter form (LyFastFilterEditorController).
 *
 * <P>This interface is implemented by a listener (usually a table controller. The listener is
 * notified when the filter has been set by the user in the filtering form.</P>
 *
 * @see LyFastFilterController#actionDemand
 */
public interface LyFastFilterEndInterface
{
	/** File Version */
	String __VERSION = "$Id: LyFastFilterEndInterface.java 35839 2012-06-12 13:43:15Z nhamon $";

	/**
	 * Gets current client filter.
	 * @return Current filter.
	 */
	LyFilter getFilter();
	
	/**
	 * Gets initial fast filter value : indicates to the fast filter controller the default filter
	 * value.
	 *
	 * @return the initial fast filter value
	 */
	String getFastFilterValue();

	/**
	 * Gets initial fast filter field info : indicates to the fast filter controller the default
	 * filter field info.
	 *
	 * @return the initial fast filter field info or null if no filter is set
	 */
	LyFieldInfo getFastFilterFieldInfo();

	/**
	 * Indicates if a fast filter is set.
	 *
	 * @return true if a fast filter is set, false otherwise
	 */
	boolean isFastFilterSet();
	
	/**
	 * Indicates if fast filter creation is an extension of current filter.
	 *
	 * @return true if fast filter creation is an extension of current filter, false otherwise
	 */
	boolean isFastFilterExtended();

	/**
	 * The fast filter edition form is closed : notify registered listener.
	 *
	 * @param parent       reference to the fast filter controller (filter form which is closing)
	 * @param actionDemand actionDemand on fast filter controller : id of the command such as
	 *                     LyFormController.DEMAND_VALIDATE, LyFormController.DEMAND_CANCEL
	 * @param filterValue  the filter value that was typed in the form
	 * @param filter       the filter that was created in the form
	 * @param fieldInfo    the fast filter field info
	 * @param extend       indicates if fast filter creation is an extension of current filter.
	 */
	void endFastFilter(LyController parent, String actionDemand,
		String filterValue, LyFilter filter, LyFieldInfo fieldInfo, boolean extend);
}