/*
 * LyFormInterface.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyFormInterface.java 28422 2010-11-03 12:03:38Z ehelmste $
 */

package leon.control;

import leon.data.*;
import leon.info.*;


/**
 * This interface is provided to allow sub form controllers to be called when the form is validated
 * or cancelled.
 */
public interface LyFormInterface extends LyControllerContextInterface
{
	/** File Version */
	String __VERSION = "$Id: LyFormInterface.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Return codes for validate: Cancel */
	short  STATUS_CANCEL = -2;
	/** Return codes for validate: Ignore */
	short  STATUS_IGNORE = -1;
	/** Return codes for validate: OK */
	short  STATUS_OK = 0;
	/** Return codes for validate: KO */
	short  STATUS_KO = 1;

	/**
	 * Method called at the begining of the parent form validation to check if everything is ok
	 * before completing the edition.
	 *
	 * @return One of the status codes indicating if the validation was done (STATUS_OK), if an
	 *         error occured (STATUS_KO), or if nothing was done by the user (STATUS_IGNORE).
	 */
	short startValidate();

	/**
	 * Method called at the end of the parent form validation to complete the edition.
	 *
	 * @return One of the status codes indicating if the validation was complete (STATUS_OK), if an
	 *         error occured (STATUS_KO), or if nothing was done by the user (STATUS_IGNORE).
	 */
	short endValidate();

	/**
	 * Method called when the whole modifications are committed.
	 */
	void commit();

	/**
	 * Method called when the parent form is cancelled.
	 */
	void cancel();

	/**
	 * Method for controlling current edition.
	 *
	 * @return Error status.
	 */
	short control();

	/**
	 * Checks if changes were done on current edition.
	 *
	 * @return true if the edition was changed, false otherwise.
	 */
	boolean checkIfModified();

	/**
	 * Extract current value for the specified fieldInfo in the form.
	 *
	 * @param  fieldInfo fieldInfo associated to the requested value
	 * @return current value, or null if not found or field is disabled.
	 */
	LyValue getFieldValue(LyFieldInfo fieldInfo);

	/**
	 * Set boolean indicating if the controlled form is editable or not.
	 *
	 * @param editable editable status.
	 */
	void setEditable(boolean editable);

	/**
	 * Set boolean indicating if the controlled form is enabled or not.
	 *
	 * @param enabled enabled status.
	 */
	void setEnabled(boolean enabled);
}