/**
 * LyPrintPdfController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyPrintPdfController.java 28663 2010-11-23 08:30:38Z bangiole $
 */

package leon.control;

import java.io.*;
import leon.app.*;
import leon.data.*;
import leon.info.*;
import leon.view.*;


/**
 * Print a list of objects This controller print a list of object in a PDF file.
 */
public class LyPrintPdfController extends LySimplePrintController
{
	/** File Version */
	public static final String __VERSION =
		"$Id: LyPrintPdfController.java 28663 2010-11-23 08:30:38Z bangiole $";

	/** File where data must be saved. */
	protected String		   _file;

	/**
	 * Constructor.
	 *
	 * @param session   current session associated to the controller. The controller automatically
	 *                  registers on the specified session.
	 * @param parent    parent controller (or null for 'root' controller).
	 * @param action    related action (_login).
	 * @param classInfo related class info
	 * @param objects   current action context
	 */
	public LyPrintPdfController(LySession session, LyController parent, LyAction action,
		LyClassInfo classInfo, LyObjectList objects)
	{
		super(session, parent, action, objects);

		_file = null;
	}

	/**
	 * Printing objects starts.
	 */
	public void run()
	{
		startRun();

		FileOutputStream fos = null;

		try
		{
			LyViewManager viewManager = getViewManager();

			fos = new FileOutputStream(_file);
			_printManager = viewManager.getPdfPrintManager(this, fos);
			if (_printManager == null) // Cancelled or refused
			{
				freePrinting();
				return;
			}

			// Check if an objectPrinterInterface is registred
			boolean over = false;

			if (_behavior != null)
				over = _behavior.printBegin(this);

			if (!over)
				printEnd();

			if ((!over) && (_behavior != null))
				over = _behavior.printEnd(this);
		}
		catch (IOException ioe)
		{
			String message = getEnvironment().getMessage("LY_ER_OPEN_FILE", _file);

			getParent().showError(message);
		}

		endRun();

		freePrintJob();

		try
		{
			// FFT 797 -acces sharing violation-
			if (fos != null)
			{
				fos.flush();
				fos.close();
			}
		}
		catch (IOException ioe)
		{
			getEnvironment().writeException(ioe);
		}
	} // end method run

	/**
	 * Free the current print job
	 */
	protected void freePrintJob()
	{
		LySession session = getSession();

		session.removeController(this);

		if (_printManager != null)
			_printManager.end();
	}

	/**
	 * Set the target file.
	 *
	 * @param file The path of the target file
	 */
	public void setFile(String file)
	{
		_file = file;
	}

	/**
	 * Indicates if page number must be reset when printing a new object.
	 * @return false
	 */
	protected boolean resetPageNumberForObject()
	{
		return false;
	}
} // end class LyPrintPdfController
