/**
 * LyFilterController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LySimpleFilterController.java 28997 2010-12-23 10:17:15Z bmichel $
 */

package leon.control;

import leon.app.*;
import leon.app.behaviorinterface.*;
import leon.data.*;
import leon.info.*;
import leon.view.event.*;


/**
 * This class implements the simple filter edition area. 
 *
 * @see LyFilterController
 * @see LyFilterCriterionController
 * @see LyFilterRelationController
 */
public abstract class LySimpleFilterController extends LyFormController
{
	/** File Version */
	public static final String     __VERSION =
		"$Id: LySimpleFilterController.java 28997 2010-12-23 10:17:15Z bmichel $";
    
	/**
	 * Client which may be notified when the filter forms is closed to update its content (usually a
	 * LyTableController or LyFilterEditorController).
	 */
	protected LyFilterEndInterface _filterEndInterface;

    /** Current filter. */
    protected LyFilter             _defaultFilter;

    /** Indicates if there was a modification on this controller. */
    protected boolean              _modified;
    
    
	/**
	 * Constructor.
	 *
	 * @param session   current session associated to the controller. The controller automatically
	 *                  registers on the specified session.
	 * @param parent    parent controller. If the parent implements LyFilterEndInterface, it will be
	 *                  notified when the filter form is closed.
	 * @param action    related action. (action based on '_filter' action)
	 * @param classInfo the application class associated to the filter
	 * @param objects   the context
	 */
	public LySimpleFilterController(LySession session, LyController parent, LyAction action,
		LyClassInfo classInfo, LyObjectList objects)
	{
		super(session, parent, action, classInfo, objects);

		if (parent instanceof LyFilterEndInterface)
			setFilterEndInterface((LyFilterEndInterface)parent);
        
        _modified = false;
	}

    /**
     * Sets client (listener) for closing notification.
     *
     * @param endInterface client notified when the dialog closes.
     */
    public void setFilterEndInterface(LyFilterEndInterface endInterface)
    {
        _filterEndInterface = endInterface;
        _defaultFilter = ((_filterEndInterface != null) ? _filterEndInterface.getFilter() : null);
    }
    
    /**
     * Default filter (initial values). This is given by the LyFilterEndInterface
     *
     * @return initial filter (default values)
     */
    public LyFilter getDefaultFilter()
    {
        return _defaultFilter;
    }
    
	/**
	 * Gets client (listener) for closing notification.
	 *
	 * @return client notified when the dialog closes. May be null.
	 */
	public LyFilterEndInterface getFilterEndInterface()
	{
		return _filterEndInterface;
	}

    /**
     * Method that gives the interface that must be implemented by the behavior associated to this
     * controller.
     *
     * @return Class representing the behavior interface of this controller.
     */
	public Class<?> getBehaviorInterface()
    {
        return LyFilterBehaviorInterface.class;
    }

    /**
     * Set accessor to the context client area controller which will receive contextual notification
     * when something occur on this area controller. This method is overriden to set the given
     * controller as filter client when the filter controller is used in a compound action.
     *
     * @param controller the contextual controller.
     */
    public void setContextClient(LyAreaController controller)
    {
        super.setContextClient(controller);

        if ((_filterEndInterface == null)
                && (controller instanceof LyFilterEndInterface))
            setFilterEndInterface((LyFilterEndInterface)controller);
    }
    
    /**
     * Checks if a value has been modified in the form.
     *
     * @return true if a value has been modified, false otherwise.
     */
    public boolean checkIfModified()
    {
        return _modified;
    }
    
    /**
     * Internal method to validate filter creation.
     * @param demand the demand
     */
    protected abstract void validate(LyActionDemand demand);
} // end class LyFilterController
