/**
 * LyTreeBaseController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyTreeBaseController.java 28422 2010-11-03 12:03:38Z ehelmste $
 */

package leon.control;

import leon.app.*;
import leon.app.behaviorinterface.*;
import leon.control.builder.*;
import leon.data.*;
import leon.info.*;
import leon.view.*;


/**
 * This class implements the basics of a tree controller. It is used to factorize code between all
 * tree controllers. It is associated with a builder (LyTreeBuilder) and a behavior that implements
 * LyTreeBaseBehaviorInterface. There is no default implementation of this behavior.
 *
 * @see LyTreeBuilder
 * @see leon.app.behaviorinterface.LyTreeBaseBehaviorInterface
 * @see LyTreeController
 * @see LyFilterEditorController
 */
public abstract class LyTreeBaseController extends LyPanelController
{
	/** File Version */
	public static final String			  __VERSION =
		"$Id: LyTreeBaseController.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Specific controller class to create and keep up to date the content of the view. */
	protected LyTreeBaseBehaviorInterface _treeBehaviorInterface;

	/**
	 * Constructor.
	 *
	 * @param session current session associated to the controller. The controller automatically
	 *                registers on the specified session.
	 * @param parent  parent controller (or null for 'root' controller). Controller hierarchy
	 *                indicates a child controller (for example a dialog created from a main view)
	 * @param action  related action. If an action is specified, the controller uses action
	 *                parameters (action name, action specific XML view description ...)
	 * @param context current context (may be null if the view is not contextual to a list of
	 *                objects).
	 */
	public LyTreeBaseController(LySession session, LyController parent, LyAction action,
		LyObjectList context)
	{
		super(session, parent, action, context);
		initTreeController();
	}

	/**
	 * Init tree controller.
	 */
	protected void initTreeController()
	{
	}

	/**
	 * Reset the content of the view (Nodes) The current implementation does nothing.
	 */
	public void updateContent()
	{
		//XXX
	}

	/**
	 * Reset the message bar status if any The current implementation does nothing.
	 */
	public void updateMessages()
	{
		// XXX
	}

	/**
	 * @see leon.control.LyPanelController#updateTools()
	 */
	public void updateTools()
	{
		super.updateTools();
	}

	/**
	 * Empty the tree (remove all nodes).
	 */
	public void clearTreeContent()
	{
		// Retrieve internal tree component
		LyTree tree = getTreeComponent();

		tree.setTreeNode(null);
	}

	/**
	 * Accessor to the Tree component (view).
	 *
	 * @return The tree view component
	 */
	public LyTree getTreeComponent()
	{
		if (isFreed())
			return null;
		// Retrieve internal tree component
		LyTree tree = (LyTree)getComponent().getComponent(getId() + "_tree");

		return tree;
	}

	/**
	 * Accessor to panel component.
	 * @return the panel component
	 * @see #getTreeComponent
	 */
	public LyPanel getPanelComponent()
	{
		return getTreeComponent();
	}

	/**
	 * Retrieves the default view Builder.
	 *
	 * @return the default builder instance
	 * @see    LyController#getDefaultBuilder
	 */
	protected LyBuilder getDefaultBuilder()
	{
		return (new LyTreeBuilder(this));
	}
} // end class LyTreeBaseController
