/**
 * LyBuilder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyBuilder.java 28422 2010-11-03 12:03:38Z ehelmste $
 */

package leon.control.builder;

import leon.control.*;
import leon.misc.*;
import leon.view.*;


/**
 * This class is inherited by all builder class. It implements all basic method for a builder. The
 * role of a builder is to create the view associted to a controller
 */
public abstract class LyBuilder
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyBuilder.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Parent controller. */
	protected LyController     _parent;

	/** Component constructed by this builder. */
	protected LyComponent	   _component;

	/** Boolean indicating if the component is under construction. */
	protected boolean		   _building;

	/**
	 * Default constructor.
	 *
	 * @param parent parent controller
	 */
	public LyBuilder(LyController parent)
	{
		_parent = parent;
	}

	/**
	 * Retrieves the parent controller.
	 *
	 * @return the parent controller
	 */
	public LyController getParent()
	{
		return _parent;
	}

	/**
	 * Builds the view component for the specified parent controller.
	 *
	 * @return the view Component
	 */
	public LyComponent buildComponent()
	{
		if (_component == null)
		{
			if ((_building)
					&& !(this instanceof LyViewBuilder))
			{
				LyEnvironment environment = _parent.getEnvironment();

				environment.logWarning("BF AppEngine : call to buildComponent on "
					+ _parent + " during view construction");
			}

			_building = true;
			_component = build();
			_building = false;
		}

		return _component;
	}

	/**
	 * Builds the view component for the specified parent controller. The component may create the
	 * XML description or the internal view.
	 *
	 * @return the view Component
	 */
	public abstract LyComponent build();

	/**
	 * Returns root component built by this builder.
	 *
	 * @return the root component
	 */
	public LyComponent getComponent()
	{
		return _component;
	}

	/**
	 * Accessor to the view manager.
	 *
	 * @return the view manager
	 */
	protected LyViewManager getViewManager()
	{
		return ((_parent == null) ? null : _parent.getViewManager());
	}
} // end class LyBuilder
