/**
 * LyAreaBuilder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyCanvasBuilder.java 31479 2011-06-17 14:10:55Z nhamon $
 */

package leon.control.builder;

import leon.control.*;
import leon.info.*;
import leon.view.*;


/**
 * Builder used to integrate a specific drawing area.
 *
 * @see LyCanvasController
 */
public class LyCanvasBuilder extends LyAreaBuilder
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyCanvasBuilder.java 31479 2011-06-17 14:10:55Z nhamon $";

	/**
	 * Default constructor.
	 *
	 * @param parent parent controller
	 */
	public LyCanvasBuilder(LyController parent)
	{
		super(parent);
	}

	/**
	 * Builds the hierarchy of components.
	 *
	 * @return root component.
	 */
	public LyComponent build()
	{
		LyArea area = new LyArea();

		area.setId(_parent.getId());
		area.setController(_parent.getId());
		area.setScrolling(LyArea.Scrolling.AUTO);

		// set content
		area.setToolBar(buildToolBar());
		area.setTitleBar(buildTitleBar());

		area.addPanel(buildPanel());

		_component = area;
		
		LyConstraint  constraint = new LyConstraint();
		
		constraint.setFill(LyConstraint.Fill.BOTH);
		area.setConstraint(constraint);

		updateTitleBar();
		return area;
	}

	/**
	 * Builds the draing panel.
	 *
	 * @return panel used for drawings.
	 */
	public LyPanel buildPanel()
	{
		LyAreaController parent = (LyAreaController)_parent;
		LyAction		 action = parent.getAction();
		String			 canvasClassName = (String)action.getParameter("_javaName");
		LyCanvas		 canvas = new LyCanvas(canvasClassName);

		canvas.setController(_parent.getId());
		return canvas;
	}
} // end class LyCanvasBuilder
