/**
 * LyChartBuilder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyChartBuilder.java 40644 2012-09-04 07:03:16Z jlsaouli $
 */

package leon.control.builder;

import leon.control.*;
import leon.info.*;
import leon.view.*;


/**
 * This class builds the parameter chart edition area.
 *
 * @see LyChartController
 */
public class LyChartBuilder extends LyAreaBuilder
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyChartBuilder.java 40644 2012-09-04 07:03:16Z jlsaouli $";

	/** The First area in the area. */
	public LyArea			   _area1 = null;

	/**
	 * Default constructor.
	 *
	 * @param parent the parent controller
	 */
	public LyChartBuilder(LyController parent)
	{
		super(parent);
	}

	/**
	 * Builds the view component for the specified parent controller. The component may create the
	 * XML Tree or the internal view.
	 *
	 * @return the view Component
	 */
	public LyComponent build()
	{

		/* The chart view is an area which contains :
		        - A titleBar (description)
		        - An area component
		*/
		LyArea area = new LyArea();

		area.setScrolling(LyArea.Scrolling.AUTO);
		area.setTitleBar(buildTitleBar());
		area.setCommands(buildCommands());
		area.setController(_parent.getId());

		_area1 = ((LyChartController)this._parent).resetArea1();
		area.addPanel(_area1);

		_component = area;
		return (area);
	}

	/**
	 * Add information bar into the chart form.
	 *
	 * @return the title bar
	 */
	protected LyTitleBar buildTitleBar()
	{
		LyAction	action = ((LyChartController)_parent).getAction();

		if (action.hasMark(LyAction.MARK_NO_TITLEBAR))
			return null;
		
		LyTitleBar titleBar = new LyTitleBar();

		if (_parent instanceof LyChartController)
		{
			LyClassInfo classInfo = ((LyChartController)_parent).getClassInfo();
			String	    label = "LY_ACTION_CHART" + " : " + classInfo.getName();

			if (_parent.getEnvironment().testResource("LY_TITLEBAR_LEFT_IMAGE"))
			{
				titleBar.setImage(_parent.getImage(classInfo));
				titleBar.setImageSensitive(true);
			}
			
			titleBar.setLeftLabel(label);
		}

		return titleBar;
	}

	/**
	 * Creates the command bar and sets its content.
	 *
	 * @return the command bar
	 */
	protected LyCommands buildCommands()
	{
		LyAction	action = ((LyChartController)_parent).getAction();

		if (action.hasMark(LyAction.MARK_NO_COMMANDS))
			return null;
		
		LyCommands commands = new LyCommands();
		LyCommand  command;

		command = new LyCommand(LyChartController.DEMAND_PREVIOUS_STEP, "LY_PREVIOUS");
		command.setAction(LyCommand.Action.OTHER);
		command.setEnabled(false);
		commands.addCommand(command);

		command = new LyCommand(LyChartController.DEMAND_NEXT_STEP, "LY_NEXT");
		command.setAction(LyCommand.Action.SUBMIT);
		commands.addCommand(command);

		command = new LyCommand(LyFormController.DEMAND_CANCEL, "LY_CANCEL");
		command.setAction(LyCommand.Action.CANCEL);
		commands.addCommand(command);

		command = buildHelpCommand();

		if (command != null)
			commands.addCommand(command);

		return commands;
	}
} // end class LyChartBuilder
