package leon.control.builder;

import java.io.*;
import java.util.*;

import leon.control.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;
import eu.w4.service.*;
import eu.w4.service.helper.*;


/**
 * The default builder for common service controller.
 */
public class LyCommonServiceBuilder extends LyUserInputBuilder
{

    /** File version. */
    public static final String __VERSION =
        "$Id: LyCommonServiceBuilder.java 32388 2011-09-19 12:29:25Z jlsaouli $";

    private Map<String, Integer>      _componentPosition = new HashMap<String, Integer>();
    private boolean                   _isAutomatic = true;
    private int                       _pos = 1;
    private String[]                  _currentProperty = null;
    private String[]                  _currentParameters = null;
    private ServiceInfo               _serviceInfo = null;
    private String                    _serviceName;
    private LyUserInputController     _userInputController;
    private LyCommonServiceController _commonServiceController;
    private Service                   _service;

    /**
     * Default constructor.
     *
     * @param  parent  the parent controller
     */
    public LyCommonServiceBuilder(LyController parent)
    {
        super(parent);
    }

    /**
     * Build the form, depending on what fields have already been settled. If a property or a
     * parameter is null or not settled, a corresponding field is created
     */
    protected void buildFormContent()
    {
        LyEnvironment environment = _parent.getEnvironment();

        _userInputController = (LyUserInputController) _parent;
        _commonServiceController = (LyCommonServiceController) _parent;

        Map<String, Object> parameters = new HashMap<String, Object>();
        List<String>        serviceList = _commonServiceController.getClasseNamesInPackage();
        Locale              locale = environment.getLocale(); // Locale.getDefault(); TODO
        LyRelation          relation = new LyRelation();

        for (int j = 0; j < serviceList.size(); j++)
        {
            _commonServiceController._allServices.add(serviceList.get(j));

            if (j == 0)
            {
                _serviceName = serviceList.get(j);
                _serviceInfo = _commonServiceController.getServiceInfoFromName(serviceList.get(j));
                _service = _commonServiceController.getServiceFromName(serviceList.get(j));
                _commonServiceController._currentService = _service;
                _commonServiceController._currentServiceInfo = _serviceInfo;
                _commonServiceController._currentServiceName = serviceList.get(j);
                _currentProperty = _serviceInfo.getPropertyNames();
                _currentParameters = _serviceInfo.getInputParameterNames();
            }
        }

        if (serviceList.size() > 1)
        {
            relation = _userInputController.addRelation("serviceName",
                    ServiceInfoImpl.getTranslation("serviceName", locale), false, true);

            for (int j = 0; j < serviceList.size(); j++)
                relation.addLink(new LyLink(serviceList.get(j),
                        ServiceInfoImpl.getTranslation(serviceList.get(j), locale)));
        }

        if (_currentProperty != null)
        {
            environment.logTrace("current PROPERTY " + _currentProperty[0]);
            parameters.putAll(buildFields(_serviceName, _currentProperty));
        }

        if (_currentParameters != null)
        {
            environment.logTrace("current PARAMETERS " + _currentParameters[0]);
            parameters.putAll(buildFields(_serviceName, _currentParameters));
        }

        _commonServiceController.setParameters(parameters);

        if (_isAutomatic && _commonServiceController.hasUI())
        {
            _userInputController.addLabel("AutoExec", "AutoExec", "");
            _commonServiceController.invoke(_service, _serviceInfo);
        }
    }

    @Override public LyComponent build()
    {
        return super.build();
    }

    /**
       * Remove old fields and update the form. If news fields have been added they are added to the
       * form in this method
       *
       * @param  changedComponent  : name of the field to remove
       */
    public void removeOldParam(String changedComponent)
    {
        int count = _currentProperty.length + _currentParameters.length;

        for (int i = 0; i < _currentProperty.length; i++)
        {
            Class<?> type = getType(_currentProperty[i]);

            if ((type == List.class) || (type == Vector.class))
                count++;
        }

        for (int i = 0; i < _currentParameters.length; i++)
        {
            Class<?> type = getType(_currentParameters[i]);

            if ((type == List.class) || (type == Vector.class))
                count++;
        }

        LyEnvironment environment = _parent.getEnvironment();

        environment.logTrace("REMOVING " + count + " parameters");
        environment.logTrace("formcount= " + _form.getFormFieldCount());

        for (int j = 1; j <= count; j++)
        {
            environment.logTrace("       remove n" + j);
            _form.removeFormField(1);
        }

        _form.resetContent(true);

        ServiceInfo newServiceInfo = _commonServiceController.getServiceInfoFromName(
                changedComponent);
        String[]    newProperty = newServiceInfo.getPropertyNames();
        String[]    newParameter = newServiceInfo.getInputParameterNames();

        Map<String, Object> parameters = new HashMap<String, Object>();

        _commonServiceController.setParameters(parameters);

        _serviceInfo = _commonServiceController.getServiceInfoFromName(changedComponent);
        _service = _commonServiceController.getServiceFromName(changedComponent);
        _serviceName = changedComponent;
        _commonServiceController._currentService = _service;
        _commonServiceController._currentServiceInfo = _serviceInfo;
        _commonServiceController._currentServiceName = changedComponent;

        _currentProperty = _serviceInfo.getPropertyNames();
        _currentParameters = _serviceInfo.getInputParameterNames();

        parameters.putAll(buildFields(_serviceName, newProperty));
        parameters.putAll(buildFields(_serviceName, newParameter));

        _commonServiceController.cleanParameters();
        _commonServiceController.setParameters(parameters);

        _userInputController.update();

        _form.resetContent(true);
    }

    /**
     * Build all form fields, according to their type.
     *
     * @param   inParam  : names of the properties and field to add to the form
     *
     * @return  a map containing already settled parameters and properties
     */
    protected Map<String, Object> buildFields(String serviceName, String[] inParam)
    {
        LyEnvironment       environment = _parent.getEnvironment();
        Locale              locale = environment.getLocale(); // Locale.getDefault(); TODO
        Map<String, Object> outParam = new HashMap<String, Object>();
        Map<String, Object> defaultMap = new HashMap<String, Object>();
        LyRelation          relation = new LyRelation();
        String              parameter;
        Class<?>            type;

        if (inParam != null)
        {

            for (int i = 0; i < inParam.length; i += 1)
            {

                try
                {
                    type = getType(inParam[i]);
                    environment.logTrace("Adding param ====> " + inParam[i] + "   type = " + type);

                    parameter = (String) _userInputController.getParameter("_" + inParam[i], true);

                    boolean isMandatory = _serviceInfo.isPropertyMandatory(inParam[i]);

                    if (!isMandatory)
                        isMandatory = _serviceInfo.isInputparameterMandatory(inParam[i]);

                    boolean      optional = !isMandatory;
                    List<String> serviceList = _commonServiceController.getClasseNamesInPackage();
                    String       help = _serviceInfo.getPropertyHelp(inParam[i], locale);

                    for (int j = 0; j < serviceList.size(); j++)
                    {

                        if (help == null)
                            help = _serviceInfo.getInputParameterHelp(inParam[i], locale);
                    }

                    LyTip tip = new LyTip();

                    tip.setLabel(help);

                    String id = "";

                    /*Remove variable values from the UI*/
                    if (!_commonServiceController.isParameterHardCoded(parameter))
                        parameter = null;

                    if ((null == parameter) || parameter.equals("") || parameter.equals("EMPTY"))
                    {

                        if (type == Boolean.class) // Add a boolean to the form
                        {
                            Object defaultState = getDefaultValue(inParam[i]);

                            if (defaultState.equals(true))
                            {
                                LyBoolean comp = _userInputController.addBoolean(serviceName + "."
                                        + inParam[i],
                                        ServiceInfoImpl.getTranslation(inParam[i], locale),
                                        LyBoolean.Status.CHECKED, optional);

                                applyTip(comp, tip);
                                defaultMap.put(inParam[i], true);
                            }
                            else if (defaultState.equals(false))
                            {
                                LyBoolean comp = _userInputController.addBoolean(serviceName + "."
                                        + inParam[i],
                                        ServiceInfoImpl.getTranslation(inParam[i], locale),
                                        LyBoolean.Status.UNCHECKED, optional);

                                applyTip(comp, tip);
                                defaultMap.put(inParam[i], false);
                            }
                            else
                            {
                                LyBoolean comp = _userInputController.addBoolean(serviceName + "."
                                        + inParam[i],
                                        ServiceInfoImpl.getTranslation(inParam[i], locale),
                                        LyBoolean.Status.EMPTY, optional);

                                applyTip(comp, tip);
                                defaultMap.put(inParam[i], false);
                            }
                        }
                        else if (type == Date.class) // Add a date choice to the form
                        {
                            LyDate comp = _userInputController.addDate(serviceName + "."
                                    + inParam[i],
                                    ServiceInfoImpl.getTranslation(inParam[i], locale), inParam[i],
                                    "dd/MM/yyyy hh:mm:ss", optional);

                            applyTip(comp, tip);
                            defaultMap.put(inParam[i], "");
                        }
                        else if (type == File.class) // Add a file choice to the form
                        {
                            LyFile comp = _userInputController.addFile(serviceName + "."
                                    + inParam[i],
                                    ServiceInfoImpl.getTranslation(inParam[i], locale), "",
                                    LyFile.Type.TEXT, optional);

                            applyTip(comp, tip);
                            defaultMap.put(inParam[i], "");
                        }
                        else if (type == File[].class) // Add a repertory choice to the form
                        {
                            LyFile comp = _userInputController.addFile(serviceName + "."
                                    + inParam[i],
                                    ServiceInfoImpl.getTranslation(inParam[i], locale), "",
                                    LyFile.Type.DIRECTORY, optional);

                            applyTip(comp, tip);
                            defaultMap.put(inParam[i], "");
                        }
                        else if ((type == Map.class)
                                || (_serviceInfo.isDynamic()
                                    && (inParam == _serviceInfo.getInputParameterNames())))
                        {
                            // TODO notImplemented Yet
                        }
                        else if (type == String[].class) // Add a list choice to the form
                        {
                            String[] table1 = (String[]) _serviceInfo.getDefaultInputParameterValue(
                                    inParam[i]);
                            String[] table2 = (String[]) _serviceInfo.getDefaultPropertyValue(
                                    inParam[i]);
                            String[] table =
                                new String[((table1 == null) ? 0 : table1.length)
                                    + ((table2 == null) ? 0 : table2.length)];

                            if (table1 != null)
                            {

                                for (int j = 0; j < table1.length; j++)
                                    table[j] = table1[j];
                            }

                            if (table2 != null)
                            {

                                for (int j = 0; j < table2.length; j++)
                                    table[j] = table2[j];
                            }

                            if ((table2 != null) && (table1 != null))
                            {

                                for (int k = table1.length - 1;
                                        k < (table1.length + table2.length - 1); k++)
                                    table[k] = table2[k - table1.length];
                            }

                            relation = _userInputController.addRelation(serviceName + "."
                                    + inParam[i],
                                    ServiceInfoImpl.getTranslation(inParam[i], locale), false,
                                    optional);
                            relation.addLink(new LyLink("blank", "LY_COMPONENT_CHOOSE"));

                            java.util.List<String> links = new Vector<String>();

                            for (int j = 0; j < table.length; j++)
                                links.add(table[j]);

                            for (int j = 0; j < links.size(); j++)
                                relation.addLink(new LyLink(links.get(j), links.get(j)));

                            applyTip(relation, tip);
                            defaultMap.put(inParam[i], "");
                        }
                        else if (type == Vector.class)
                        {
                            LyFile text = _userInputController.addFile(serviceName + "."
                                    + inParam[i],
                                    ServiceInfoImpl.getTranslation(inParam[i], locale), "",
                                    LyFile.Type.TEXT, optional);

                            applyTip(text, tip);
                            defaultMap.put(inParam[i], "");

                            LyFormField textFormField = (LyFormField) text.getParent();
                            LyForm      form = text.getParentForm();

                            createMultipleField(textFormField, form, serviceName, inParam[i],
                                id.toString());
                        }
                        else if (type == List.class)
                        {
                            LyText text = _userInputController.addText(serviceName + "."
                                    + inParam[i],
                                    ServiceInfoImpl.getTranslation(inParam[i], locale), "", 50, "",
                                    optional);

                            applyTip(text, tip);
                            defaultMap.put(inParam[i], "");

                            LyFormField textFormField = (LyFormField) text.getParent();
                            LyForm      form = text.getParentForm();

                            createMultipleField(textFormField, form, serviceName, inParam[i],
                                id.toString());
                        }
                        else if (type == Long.class)
                        {
                            LyNumber comp = _userInputController.addNumber(serviceName + "."
                                    + inParam[i],
                                    ServiceInfoImpl.getTranslation(inParam[i], locale), "0",
                                    LyNumberFieldInfo.Type.INT, optional, -1);

                            applyTip(comp, tip);
                            defaultMap.put(inParam[i], "");
                        }
                        else
                        {
                            int nbRow = 1;

                            if (
                                ServiceInfoImpl.STRING_MULTI_LINE_DEFAULT_VALUE.equals(
                                        _serviceInfo.getDefaultInputParameterValue(inParam[i]))
                                    || ServiceInfoImpl.STRING_MULTI_LINE_DEFAULT_VALUE.equals(
                                        _serviceInfo.getDefaultPropertyValue(inParam[i])))
                                nbRow = 5;

                            LyText comp = _userInputController.addText(serviceName + "."
                                    + inParam[i],
                                    ServiceInfoImpl.getTranslation(inParam[i], locale), "", 50,
                                    nbRow, "", optional, -1);

                            applyTip(comp, tip);
                            defaultMap.put(inParam[i], "");
                        }

                        _componentPosition.put(inParam[i], _pos++);
                        _isAutomatic = false;
                    }
                    else
                    {
                        if ((type == Boolean.class)
                                && (((String) _userInputController.getParameter("_" + inParam[i],
                                            true)).compareTo("EMPTY") != 0))
                            outParam.put(inParam[i],
                                (((String) _userInputController.getParameter("_" + inParam[i],
                                            true)).compareTo("false") == 0) ? false : true);
                        else if (type == File.class)
                            outParam.put(inParam[i], new File(parameter));
                        else
                            outParam.put(inParam[i], parameter);
                    }
                }
                catch (Exception e)
                {
                    environment.writeException(e);
                }
            }

            if ((defaultMap != null) && !defaultMap.isEmpty())
                _commonServiceController.setParameters(defaultMap);
        }

        return outParam;
    }

    private void applyTip(LyComponent component, LyTip tip)
    {
        LyFormField  formField = (LyFormField) component.getParent();
        LyConstraint constraint = new LyConstraint();

        constraint.setHAlign(LyConstraintBase.HAlign.LEFT);
        tip.setConstraint(constraint);

        LyConstraint constraint2 = new LyConstraint();

        constraint2.setFill(LyConstraintBase.Fill.HORIZONTAL);
        formField.setTip(tip);

        Class<?> type = component.getClass();

        if (type != LyDate.class)
            formField.setConstraint(constraint2);
    }

    /**
     * get the type of a named parameter.
     *
     * @param   param  : the name of the parameter
     *
     * @return  the type of the parameter
     */
    private Class<?> getType(String param)
    {
        Class<?> paramClass = null;

        paramClass = _serviceInfo.getInputParameterType(param);

        if (paramClass != null)
            return paramClass;

        paramClass = _serviceInfo.getPropertyType(param);

        if (paramClass != null)
            return paramClass;

        return null;
    }

    private void createMultipleField(LyFormField textFormField, LyForm form, String serviceName,
        String inParam, String id)
    {
        LyToolBar textToolBar = new LyToolBar();
        LyTool    add = new LyTool(serviceName + "." + inParam + "-ADD", "LY_COMPONENT_ADD");

        textToolBar.setController(_userInputController.getId());
        textToolBar.addTool(add);
        textFormField.setToolBar(textToolBar);

        LyTable table = new LyTable(serviceName + "." + inParam + "-TABLE");

        LyRows rows = new LyRows();
        LyRow  row = new LyRow();
        LyCell cell = new LyCell();

        row.addCell(cell);
        row.addCell(cell);
        row.setId("");

        rows.addRow(row);
        table.setRows(rows);
        table.setNRows(5);

        LyFormField formField = new LyFormField("");

        formField.setFieldItem(table);
        formField.setCaption(new LyCaption());

        LyConstraint constraint = new LyConstraint();

        constraint.setFill(LyConstraintBase.Fill.HORIZONTAL);

        formField.setConstraint(constraint);
        formField.setId(serviceName + "." + inParam + "-FORMFIELD");
        form.addFormField(formField);
    }

    /**
     * Get the default value of a given parameter.
     *
     * @param   param  Name of the parameter
     *
     * @return  the default value
     */
    private Object getDefaultValue(String param)
    {
        Object defaultValue = null;

        defaultValue = _serviceInfo.getDefaultInputParameterValue(param);

        if (defaultValue != null)
            return defaultValue;

        defaultValue = _serviceInfo.getDefaultPropertyValue(param);

        if (defaultValue != null)
            return defaultValue;

        return null;
    }
}
