/**
 * LySelectBuilder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyCompoundFilterBuilder.java 40687 2012-09-06 15:07:47Z jlsaouli $
 */

package leon.control.builder;

import leon.control.*;
import leon.misc.*;
import leon.view.*;


/**
 * This builder creates the compound view used  for filtering panels.
 */
public class LyCompoundFilterBuilder extends LyCompoundAreaBuilder
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyCompoundFilterBuilder.java 40687 2012-09-06 15:07:47Z jlsaouli $";   
    
	/**
	 * Default constructor.
	 *
	 * @param parent parent controller
	 */
	public LyCompoundFilterBuilder(LyController parent)
	{
		super(parent);
	}

	/**
	 * Builds the view component for the specified parent controller. The component may create the
	 * XML Tree or the internal view.
	 *
	 * @return the view Component
	 */
	public LyComponent build()
	{
		LyArea			 area = (LyArea)super.build();
        
		area.setBorder(LyArea.Border.IN);
		area.setCommands(buildCommands());
        
		// For Web mode, associate area to filter controller (not necessary otherwise)
        area.setController(((LyCompoundFilterController)_parent).getFilterController().getId());
		return area;
	}

    /**
     * Creates the command bar and sets its content.
     *
     * @return the commands bar component
     */
    protected LyCommands buildCommands()
    {
		LyEnvironment environment = _parent.getEnvironment();
        LyCommands    commands = new LyCommands();
        LyCommand     command;

        command = new LyCommand(LyCompoundFilterController.DEMAND_ADD_FILTER, "LY_ADD");
        command.setAction(LyCommand.Action.SUBMIT);
        command.setEnabled(false);
		
		if (environment.checkImage("LY_COMMAND_ADD"))
			command.setImage("LY_COMMAND_ADD");
		
        commands.addCommand(command);
 
        command = new LyCommand(LyFormController.DEMAND_VALIDATE, "LY_VALIDATE");
        command.setAction(LyCommand.Action.SUBMIT);
		
		if (environment.checkImage("LY_COMMAND_VALIDATE"))
			command.setImage("LY_COMMAND_VALIDATE");
		
        commands.addCommand(command);
 
        if (environment.testResource("LY_APPLY_BUTTON"))
        {
            LyViewManager viewManager = getViewManager();

            if (!viewManager.hasProperty("NOT_APPLY"))
            {
                command = new LyCommand(LyFormController.DEMAND_APPLY, "LY_APPLY");
                command.setAction(LyCommand.Action.SUBMIT);
    			
    			if (environment.checkImage("LY_COMMAND_APPLY"))
    				command.setImage("LY_COMMAND_APPLY");
    			
                commands.addCommand(command);
            }
        }
     
        command = new LyCommand(LyFormController.DEMAND_CANCEL, "LY_CLOSE");
        command.setAction(LyCommand.Action.CANCEL);
		
		if (environment.checkImage("LY_COMMAND_CLOSE"))
			command.setImage("LY_COMMAND_CLOSE");
		
        commands.addCommand(command);

        command = buildHelpCommand();

        if (command != null)
            commands.addCommand(command);

        return commands;
    } // end method buildCommands

} // end class LySelectBuilder
