/**
 * LyConsultBuilder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyConsultBuilder.java 40687 2012-09-06 15:07:47Z jlsaouli $
 */

package leon.control.builder;

import leon.control.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;


/**
 * This builder construct a consult form view. The associated controller is the consult controller
 *
 * @see LyConsultController
 */
public class LyConsultBuilder extends LyFormBuilder
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyConsultBuilder.java 40687 2012-09-06 15:07:47Z jlsaouli $";

	/**
	 * Default constructor.
	 *
	 * @param parent parent controller
	 */
	public LyConsultBuilder(LyController parent)
	{
		super(parent);
	}

	/**
	 * Build the toolbar of the form.
	 *
	 * @return the built toolbar
	 */
	protected LyToolBar buildToolBar()
	{
		LyToolBar		 toolBar = super.buildToolBar();
		LyFormController formController = (LyFormController)_parent;
		LyAction		 formAction = formController.getAction();
	      
	    if (formAction.hasMark(LyAction.MARK_NO_COMMANDS)
		    && !formAction.hasMark(LyAction.MARK_NO_TOOLBAR))
		{
			if (toolBar == null)
			{
				toolBar = new LyToolBar();
				toolBar.setId("_toolBar");
				toolBar.setFill(LyToolBar.Fill.HORIZONTAL);
				toolBar.setColor("LY_DASHBOARD_HORIZONTAL_BACKGROUND");
			}

			// Add close button as tool
			LyTool tool = new LyTool(LyFormController.DEMAND_CANCEL, "LY_TOOL_CLOSE");

			tool.setImage("LY_TOOL_CLOSE");
			toolBar.addTool(tool);
		}

		return toolBar;
	}

	/**
	 * Build the command bar.
	 *
	 * @return the command bar component
	 */
	protected LyCommands buildCommands()
	{
		LyAction action = ((LyConsultController)_parent).getAction();

		if (action.hasMark(LyAction.MARK_NO_COMMANDS))
			return null;

		LyCommands    commands = null;
		LyCommand     command;
		LyController  gParent = _parent.getParent();

		if ((gParent instanceof LyViewController)
		    || (gParent instanceof LyDashboardController))
		{
			LyEnvironment environment = _parent.getEnvironment();

			commands = new LyCommands();
			command = new LyCommand(LyFormController.DEMAND_CANCEL, "LY_CLOSE");
			command.setAction(LyCommand.Action.CANCEL);
			
			if (environment.checkImage("LY_COMMAND_CLOSE"))
				command.setImage("LY_COMMAND_CLOSE");
			
			commands.addCommand(command);
		}

		command = buildHelpCommand();
		if (command != null)
		{
			if (commands == null)
				commands = new LyCommands();
			commands.addCommand(command);
		}

		return commands;
	} // end method buildCommands

} // end class LyConsultBuilder
