/**
 * LyContextListBuilder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyContextListBuilder.java 28422 2010-11-03 12:03:38Z ehelmste $
 */

package leon.control.builder;

import leon.control.*;
import leon.info.*;
import leon.view.*;


/**
 * This builder is associated with the context lits controller.
 *
 * @see LyContextListController
 * @see LySimpleTableBuilder
 */
public class LyContextListBuilder extends LySimpleTableBuilder
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyContextListBuilder.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Default constructor.
	 *
	 * @param parent parent controller
	 */
	public LyContextListBuilder(LyController parent)
	{
		super(parent);
	}

	/**
	 * Builds the view component for the specified parent controller. The component may create the
	 * XML Tree or the internal view.
	 *
	 * @return the view Component
	 */
	public LyComponent build()
	{
		LyArea area = (LyArea)super.build();

		area.setCommands(buildCommands());
		return area;
	}

	/**
	 * Builds the table description.
	 *
	 * @return the component for the table description
	 */
	protected LyTable buildTable()
	{
		LyTable table = super.buildTable();

		table.setHLines(false);
		table.setVLines(false);

		if (_parent.getParent() instanceof LyDialogController)
			// When use to consult context, there's no need to offer selection on objects
			table.setSelectionMode(LyTable.SelectionMode.NONE);

		return table;
	}

	/**
	 * Builds the table header line description.
	 *
	 * @return the header line
	 */
	protected LyHeaders buildHeaders()
	{
		LyHeader  header;

		LyHeaders headers = new LyHeaders();

		header = new LyHeader();

		// Add object name
		header.setId("_objectName");
		header.setLabel("LY_CONTEXT_OBJECT_NAME");
		header.setSortable(false);

		headers.addHeader(header);

		return (headers);
	}

	/**
	 * Build the command bar.
	 *
	 * @return the command bar component
	 */
	protected LyCommands buildCommands()
	{
		if (!(_parent.getParent() instanceof LyDialogController))
			return null;

		LyAction action = ((LyAreaController)_parent).getAction();

		if (action.hasMark(LyAction.MARK_NO_COMMANDS))
			return null;

		LyCommands commands = new LyCommands();
		LyCommand  command = new LyCommand(LyFormController.DEMAND_CANCEL, "LY_CLOSE");

		command.setAction(LyCommand.Action.CANCEL);
		commands.addCommand(command);
		return commands;
	}
} // end class LyContextListBuilder
