/**
 * LyCreateBuilder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyCreateBuilder.java 40687 2012-09-06 15:07:47Z jlsaouli $
 */

package leon.control.builder;

import leon.control.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;


/**
 * This builder construct a create form view. The associated controller is the create controller
 *
 * @see LyCreateController
 * @see LySetBuilder
 */
public class LyCreateBuilder extends LySetBuilder
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyCreateBuilder.java 40687 2012-09-06 15:07:47Z jlsaouli $";

	/**
	 * Default constructor.
	 *
	 * @param parent parent controller
	 */
	public LyCreateBuilder(LyController parent)
	{
		super(parent);
	}

	/**
	 * Check if the action should be added in the form toolbar Return true if not hidden, has the
	 * form mark set and if sessions validates the action (validateAction).
	 *
	 * @param  action the action we want to test
	 * @return true if the action should be added, false otherwise
	 */
	protected boolean validateAction(LyAction action)
	{
		if (action.getMin() != 0)
			return false;

		return super.validateAction(action);
	}

	/**
	 * Build the command bar.
	 *
	 * @return the command bar component
	 */
	protected LyCommands buildCommands()
	{
		LyAction action = ((LyAreaController)_parent).getAction();

		if (action.hasMark(LyAction.MARK_NO_COMMANDS))
			return null;

		LyCommands commands = new LyCommands();
		LyCommand  command;
		boolean    hasView = false;
		LyController parent = _parent;
		
		do
		{
			parent = parent.getParent();
			
			hasView = (parent instanceof LyViewController);
		}
		while ((!hasView) && (parent instanceof LyDashboardController));
			
		
		boolean   	  hasTarget = (((LyAreaController)_parent).getTarget() != null);
		LyEnvironment environment = _parent.getEnvironment();

		if (hasView)
		{
			command = new LyCommand(LyFormController.DEMAND_VALIDATE, "LY_VALIDATE");
			command.setAction(LyCommand.Action.SUBMIT);
			
			if (environment.checkImage("LY_COMMAND_VALIDATE"))
				command.setImage("LY_COMMAND_VALIDATE");
			
			commands.addCommand(command);
		}

		if ((environment.testResource("LY_APPLY_BUTTON")
					&& environment.testResource("LY_APPLY_BUTTON_IN_CREATION"))
				|| !hasView)
		{
			command = new LyCommand(LyFormController.DEMAND_APPLY, "LY_APPLY");
			command.setAction(LyCommand.Action.SUBMIT);
			
			if (environment.checkImage("LY_COMMAND_APPLY"))
				command.setImage("LY_COMMAND_APPLY");
			
			commands.addCommand(command);
		}
		
        if (hasView
            && environment.testResource("LY_SAVE_BUTTON")
            && environment.testResource("LY_SAVE_BUTTON_IN_CREATION"))
        {
            command = new LyCommand(LyFormController.DEMAND_SAVE, "LY_SAVE");
            command.setAction(LyCommand.Action.SUBMIT);
            
            if (environment.checkImage("LY_COMMAND_SAVE"))
                command.setImage("LY_COMMAND_SAVE");
            
            commands.addCommand(command);
        }

		if (hasTarget)
		{
			command = new LyCommand(LyFilterController.DEMAND_CLOSE, "LY_CLOSE");
			command.setAction(LyCommand.Action.CLOSE);
			
			if (environment.checkImage("LY_COMMAND_CLOSE"))
				command.setImage("LY_COMMAND_CLOSE");
			
			commands.addCommand(command);
		}
		else if (hasView)
		{
			command = new LyCommand(LyFormController.DEMAND_CANCEL, "LY_CANCEL");
			command.setAction(LyCommand.Action.CANCEL);
			
			if (environment.checkImage("LY_COMMAND_CANCEL"))
				command.setImage("LY_COMMAND_CANCEL");
			
			commands.addCommand(command);
		}

		command = buildHelpCommand();
		if (command != null)
			commands.addCommand(command);

		return commands;
	} // end method buildCommands

	/**
	 * Build area for the form context.
	 *
	 * @return the area for the form context
	 */
	protected LyArea buildContext()
	{
		return null;
	}

	/**
	 * Complete the toolbar with the form context.
	 *
	 * @param toolBar the toolbar to complete
	 */
	public void completeToolBarForContext(LyToolBar toolBar)
	{
		return;
	}

} // end class LyCreateBuilder
