/**
 * LyDialogBuilder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyDialogBuilder.java 35569 2012-05-23 13:55:33Z kchenal $
 */

package leon.control.builder;

import leon.control.*;
import leon.info.*;
import leon.info.misc.*;
import leon.misc.*;
import leon.view.*;
import leon.view.web.mobile.*;


/**
 * This builder creates the view for a dialog controller.
 *
 * @see LyViewBuilder
 * @see LyDialogController
 */
public class LyDialogBuilder extends LyViewBuilder
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyDialogBuilder.java 35569 2012-05-23 13:55:33Z kchenal $";

	/**
	 * Default constructor.
	 *
	 * @param parent parent controller
	 */
	public LyDialogBuilder(LyController parent)
	{
		super(parent);
	}

	/**
	 * Build the view.
	 *
	 * @return return the root view component
	 */
	public LyComponent build()
	{
		LyArea area = buildArea();

		if (_component != null)
			// The building of the view was done by a call to getView while constructing area
			return _component;
		LyEnvironment env;
		LyComponent   parent = area.getParent();
		if(((LyViewController)_parent).getAreaController() != null &&
				((LyViewController)_parent).getAreaController().getParentAreaController() != null &&
				((LyViewController)_parent).getAreaController().getParentAreaController().getComponent() != null)
		{
			env = ((LyViewController)_parent).getAreaController().getParentAreaController().getComponent().getEnvironment();
		}
		else
		{
			env = area.getEnvironment();
		}
		
		/*LyAreaController parentController2 = ((LyViewController)_parent).getAreaController();
		
		while(parentController2 != null)
		{
			env = parentController2.getComponent().getEnvironment();
			if(env.getViewManager().hasProperty(property))
			{
				
			}
			parentController2 = parentController2.getParentAreaController();
		}*/
		//LyEnvironment env = area.getEnvironment();

		if (env == null)
			env = _parent.getEnvironment();

		LyDialog dialog;

		if (parent instanceof LyDialog)
			dialog = (LyDialog)parent;
		else
		{
			dialog = new LyDialog(_parent.getId(), env);
			dialog.setArea(area);
		}

		if (dialog.getLabel() == null)
			dialog.setLabel(_parent.getName());
		dialog.setController(_parent.getId());

		// Sets parent id
		if (_parent.getParent() != null)
		{
			LyViewController parentViewController = _parent.getParent().getViewController();

			if (parentViewController != null)
				dialog.setParentId(parentViewController.getId());
		}

		LyViewController viewController = (LyViewController)_parent;
		LyAreaController areaController = viewController.getAreaController();
		LyAction		 action = areaController.getAction();
		
		if(action != null && action.hasMark(LyAction.MARK_MOBILEWEBVIEW))
		{
			dialog.addData(LyMobileConstants.WEBVIEW_MOBILE_KEY, LyMobileConstants.TRUE);
		}
		else
		{
			LyAreaController parentController = areaController.getParentAreaController();
			boolean mobileAction = false;
			
			while(parentController != null && ! mobileAction)
			{
				LyAction parentAction = parentController.getAction();
				if(parentAction != null && parentAction.hasMark(LyAction.MARK_MOBILEWEBVIEW))
				{
					dialog.addData(LyMobileConstants.WEBVIEW_MOBILE_KEY, LyMobileConstants.TRUE);
					mobileAction = true;
				}
				parentController = parentController.getParentAreaController();
			}
		}

		dialog.setDecorated(!action.hasMark(LyAction.MARK_NO_DECORATION));

		// Sets geometry
		LyGeometry geometry = action.getGeometry();

		if (geometry != null)
		{
			dialog.setWidth(geometry.getWidth());
			dialog.setHeight(geometry.getHeight());
			dialog.setX(geometry.getX());
			dialog.setY(geometry.getY());
			dialog.setMinWidth(geometry.getMinWidth());
			dialog.setMinHeight(geometry.getMinHeight());
		}

		_component = dialog;
		return dialog;
	} // end method build
} // end class LyDialogBuilder
