/**
 * LyFileBuilder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyFileBuilder.java 28997 2010-12-23 10:17:15Z bmichel $
 */

package leon.control.builder;

import java.util.*;
import leon.control.*;
import leon.misc.*;
import leon.view.*;
import leon.view.setinterface.*;


/**
 * This builder creates a view from an XML description of this view.
 */
public class LyFileBuilder extends LyBuilder
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyFileBuilder.java 28997 2010-12-23 10:17:15Z bmichel $";

	/** XML file containing the view description. */
	String					   _xmlFile;

	/** Expected component class. */
	Class<?>				   _componentClass;

	/** Indicates if the object is build or not yet. */
	private boolean			   _isBuild;

	/**
	 * Default constructor.
	 *
	 * @param parent         parent controller
	 * @param xmlFile        XML File
	 * @param componentClass Expected component class (may be null if component is unknown).
	 */
	public LyFileBuilder(LyController parent, String xmlFile, Class<?> componentClass)
	{
		super(parent);

		if (xmlFile == null)
			throw new IllegalStateException("BF AppEngine : xmlFile parameter can't be null");

		_xmlFile = xmlFile;
		_componentClass = componentClass;
		_isBuild = false;
	}

	/**
	 * Builds the view component for the specified parent controller. The component may create the
	 * XML description or the internal view.
	 *
	 * @return the view Component
	 */
	public LyComponent build()
	{
		LyEnvironment environment = _parent.getEnvironment();
		LyViewParser  parser = new LyViewParser(environment, _xmlFile);
		LyComponent   component = (LyComponent)parser.getRootElement();

		if ((_componentClass == LyArea.class)
				&& (component instanceof LyView))
			component = ((LyView)component).getArea();
		else if ((_componentClass != null)
				&& !_componentClass.isAssignableFrom(component.getClass()))
			throw new IllegalStateException("BF AppEngine : specific XML view file '" + _xmlFile
				+ "' must define an element of class " + _componentClass);

		Vector<LySetControllerInterface> setControllers = new Vector<LySetControllerInterface>();
		LySetControllerInterface		 setController;

		component.getComponents(LySetControllerInterface.class, setControllers);
		if (component instanceof LySetControllerInterface)
			setControllers.addElement((LySetControllerInterface)component);

		int i, n = setControllers.size();

		for (i = 0; i < n; i++)
		{
			setController = setControllers.elementAt(i);
			setController.setController(_parent.getId());
		}

		//FFT01001
		if (component instanceof LyArea)
			((LyArea)component).setEnvironment(environment);

		_isBuild = true;
		return component;
	} // end method build

	/**
	 * Returns root component built by this builder.
	 *
	 * @return the root component
	 */
	public LyComponent getComponent()
	{
		if (_isBuild)
			return _component;
		else
			return buildComponent();
	}
} // end class LyFileBuilder
