/**
  * LyFindBuilder.java
  * Copyright (c) 2000-2009, Lyria-W4.
  * All rights reserved.
  */

package leon.control.builder;

import leon.app.behavior.*;
import leon.app.behaviorinterface.*;
import leon.control.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;

/**
 * Default build for the find action
 *
 */
public class LyFindBuilder extends LyUserInputBuilder
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyFindBuilder.java 40687 2012-09-06 15:07:47Z jlsaouli $";
	
	/**
	 * Default constructor
	 * @param parent the parent controller
	 */
	public LyFindBuilder(LyController parent)
	{
		super(parent);
	}

	/**
	 * @see leon.control.builder.LyUserInputBuilder#buildFormContent()
	 */
	public void buildFormContent()
	{
		super.buildFormContent();

		LyConstraint	    constraint = new LyConstraint();
		LyConstraint	    captionConstraint = new LyConstraint();
		LyFindController    findController = (LyFindController)_parent;
		boolean			    hasView = (findController.getTarget() == null);
		LyBehaviorInterface behaviorInterface = findController.getBehavior();
		LyFindBehavior	    behavior = (LyFindBehavior)behaviorInterface;
		boolean			    vertical = hasView || behavior.isVertical(findController);

		constraint.setHAlign(LyConstraint.HAlign.LEFT);
		constraint.setRemainder(vertical);
		constraint.setColSpan(1);

		captionConstraint.setHAlign(LyConstraint.HAlign.RIGHT);
		captionConstraint.setRemainder(false);
		captionConstraint.setColSpan(1);

		LyText text = addText(LyFindController.DEMAND_FIND_PATTERN + _parent.getId(), "LY_FIND_PATTERN",
				findController.getPattern(), 15, null, true);

		((LyFormField)text.getParent()).setOptional(false);
		((LyFormField)text.getParent()).setConstraint(constraint);
		((LyFormField)text.getParent()).getCaption().setConstraint(captionConstraint);

		LyRelation relation = addRelation(LyFindController.DEMAND_FIND_FIELD + _parent.getId(), "LY_FIND_COLUMN",
				false, true);

		((LyFormField)relation.getParent()).setOptional(false);
		((LyFormField)relation.getParent()).setConstraint(constraint);
		((LyFormField)relation.getParent()).getCaption().setConstraint(captionConstraint);

		LyFieldInfo			    fieldInfo;
		LyLink				    link;
		boolean				    defaultField = true;
		LyInfoList<LyFieldInfo> findFields = findController.getFields();
		int					    i, n = findFields.getSize();

		// Get fieldinfo list
		for (i = 0; i < n; i++)
		{
			fieldInfo = findFields.getInfo(i);

			// Create relation to the fieldInfo
			link = new LyLink(fieldInfo.getId(), fieldInfo.getAlias());

			// restore default
			if (findController.getFieldInfo() != null)
				link.setSelected(true);
			else if (defaultField && (fieldInfo instanceof LyTextFieldInfo))
			{
				link.setSelected(true);
				defaultField = false;
			}

			relation.addLink(link);
		}

		LyBoolean booleanComp1 = addBoolean(LyFindController.DEMAND_FIND_WHOLE_WORD + _parent.getId(),
				"LY_FIND_WHOLE_WORD", (findController.isWholeWord() ? LyBoolean.Status.CHECKED : LyBoolean.Status.UNCHECKED), true);

		((LyFormField)booleanComp1.getParent()).setOptional(false);
		((LyFormField)booleanComp1.getParent()).setConstraint(constraint);
		((LyFormField)booleanComp1.getParent()).getCaption().setConstraint(captionConstraint);

		LyBoolean booleanComp2 = addBoolean(LyFindController.DEMAND_FIND_CASE_SENSITIVE + _parent.getId(),
				"LY_FIND_CASE_SENSITIVE",
				(findController.isCaseSensitive() ? LyBoolean.Status.CHECKED : LyBoolean.Status.UNCHECKED), true);

		((LyFormField)booleanComp2.getParent()).setOptional(false);

		constraint = new LyConstraint();
		constraint.setFill(LyConstraint.Fill.NONE);
		_form.setConstraint(constraint);
		_form.setAlign(LyForm.Align.LEFT);
		_form.setBorder(false);
		
		LyArea area = _form.getParentArea();

		area.setScrolling(LyArea.Scrolling.NO);
	} // end method buildFormContent

	/**
	 * Creates the toolbar and sets its content.
	 * @param vertical tells if the toolbar is vertical
	 *
	 * @return the toolbar component
	 */
	protected LyToolBar buildToolBar(boolean vertical)
	{
		LyToolBar		 toolBar = new LyToolBar();
		LyTool			 tool;

		tool = new LyTool(LyFindController.DEMAND_FIND, "LY_FIND");
		tool.setImage("LY_TOOL_FIND");
		toolBar.addTool(tool);

		tool = new LyTool(LyFindController.DEMAND_CLOSE_FIND, "LY_CLOSE");
		tool.setImage("LY_TOOL_CLOSE");
		toolBar.addTool(tool);

		tool = buildHelpTool();
		if (tool != null)
			toolBar.addTool(tool);

		if (vertical)
		{
			toolBar.setFill(LyToolBar.Fill.HORIZONTAL);
			toolBar.setHAlign(LyToolBar.HAlign.LEFT);
			toolBar.setVAlign(LyToolBar.VAlign.TOP);
		}
		else
		{
			toolBar.setFill(LyToolBar.Fill.HORIZONTAL);
			toolBar.setHAlign(LyToolBar.HAlign.RIGHT);
			toolBar.setVAlign(LyToolBar.VAlign.MIDDLE);
		}

		return toolBar;
	} // end method buildToolBar

	/**
	 * Creates the command bar and sets its content.
	 *
	 * @return the commands component
	 */
	protected LyCommands buildCommands()
	{
		LyEnvironment environment = _parent.getEnvironment();
		LyCommands commands = new LyCommands();
		LyCommand  command;

		command = new LyCommand(LyFindController.DEMAND_NEXT, "LY_NEXT");
		command.setAction(LyCommand.Action.SUBMIT);
		
		if (environment.checkImage("LY_COMMAND_NEXT"))
			command.setImage("LY_COMMAND_NEXT");
		
		commands.addCommand(command);
		
	    command = new LyCommand(LyFindController.DEMAND_PREVIOUS, "LY_PREVIOUS");
	    command.setAction(LyCommand.Action.SUBMIT);
		
		if (environment.checkImage("LY_COMMAND_PREVIOUS"))
			command.setImage("LY_COMMAND_PREVIOUS");
		
	    commands.addCommand(command);

		command = new LyCommand(LyFindController.DEMAND_CLOSE_FIND, "LY_CLOSE");
		command.setAction(LyCommand.Action.CLOSE);
		
		if (environment.checkImage("LY_COMMAND_CLOSE"))
			command.setImage("LY_COMMAND_CLOSE");
		
		commands.addCommand(command);

		command = buildHelpCommand();
		if (command != null)
			commands.addCommand(command);

		return commands;
	} // end method buildCommands
} // end class _LyFindBuilder
