/**
 * LyImportTableBuilder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyImportTableBuilder.java 35012 2012-03-23 15:08:50Z jlsaouli $
 */

package leon.control.builder;

import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;


/**
 * This builder create the import table for the Import table controller It is based on the simple
 * table builder.
 *
 * @see LySimpleTableBuilder
 * @see LyImportTableController
 */
public class LyImportTableBuilder extends LyMultiPageTableBuilder
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyImportTableBuilder.java 35012 2012-03-23 15:08:50Z jlsaouli $";

	/**
	 * Default constructor.
	 *
	 * @param parent parent controller
	 */
	public LyImportTableBuilder(LyController parent)
	{
		super(parent);
	}

	/**
	 * Builds the view component for the specified parent controller. The component may create the
	 * XML Tree or the internal view.
	 *
	 * @return the view Component
	 */
	@SuppressWarnings("deprecation")
	public LyComponent build()
	{
		LyArea area = (LyArea)super.build();

		area.setCommands(buildCommands());

		// Build select bar only if more than one page
		LyImportTableController importTableController = (LyImportTableController)_parent;
		LyObjectList		    content = importTableController.getImportedObjects();
		LyObjectList		    deleteContent = importTableController.getObjectsToDelete();
		LyAction			    action = importTableController.getAction();
		String				    viewLimitParameter = (String)action.getParameter("_viewLimit");
		int					    viewLimit = LyTools.intFromString(viewLimitParameter);
		int					    n = ((content == null) ? 0 : content.getSize());

		n += ((deleteContent == null) ? 0 : deleteContent.getSize());

		if (n <= viewLimit)
			area.setSelectBar(null);

		return area;
	}

	/**
	 * Builds the toolbar.
	 *
	 * @return the toolbar Component
	 */
	protected LyToolBar buildToolBar()
	{
		LyAction action = ((LyAreaController)_parent).getAction();

		if (action.hasMark(LyAction.MARK_NO_TOOLBAR))
			return null;

		LyToolBar			    toolBar = completeToolBar(null);
		LyImportTableController importTableController = (LyImportTableController)_parent;

		if (importTableController.allowOperations())
		{
			if (toolBar == null)
			{
				toolBar = new LyToolBar();
				toolBar.setId("_toolBar");
				toolBar.setFill(LyToolBar.Fill.HORIZONTAL);
				toolBar.setHAlign(LyToolBar.HAlign.LEFT);
				toolBar.setVAlign(LyToolBar.VAlign.TOP);
			}
			
			if (toolBar.getComponentCount() > 0)
				toolBar.addSep();

			LyFormField field = new LyFormField(LyImportController.SELECT_OPERATIONS);
			LyChoice    fieldChoice = new LyChoice();
			LyCaption   caption = new LyCaption();
			LyOption    fieldOption;

			fieldChoice.setMultiple(true);
			fieldChoice.setFill(LyChoice.Fill.HORIZONTAL);

			caption.setLabel("LY_IMPORT_OPERATIONS");
			caption.setColon(true);

			fieldOption = new LyOption(LyImportController.SELECT_OPERATIONS_CREATION + "",
					"LY_IMPORT_CREATION");
			fieldChoice.addOption(fieldOption);
			fieldOption.setChecked(true);

			fieldOption = new LyOption(LyImportController.SELECT_OPERATIONS_MODIFICATION + "",
					"LY_IMPORT_MODIFICATION");
			fieldChoice.addOption(fieldOption);
			fieldOption.setChecked(true);

			fieldOption = new LyOption(LyImportController.SELECT_OPERATIONS_DELETION + "",
					"LY_IMPORT_DELETION");
			fieldChoice.addOption(fieldOption);

			fieldOption = new LyOption(LyImportController.SELECT_OPERATIONS_CONSULTATION + "",
					"LY_IMPORT_CONSULTATION");
			fieldChoice.addOption(fieldOption);
			fieldOption.setChecked(false);

			field.setCaption(caption);
			field.setFieldItem(fieldChoice);
			field.setOptional(false);
			field.setNeedCheck(true);

			LyTip tip = new LyTip();

			tip.setLabel("LY_IMPORT_OPERATIONS_TIP");
			field.setTip(tip);

			toolBar.addFormField(field);
		}

		return toolBar;
	} // end method buildToolBar

    /**
     * Add tools in the right toolbar.
     * @param toolbar a toolbar
     */
	protected void addRightToolbarTools(LyToolBar toolbar)
	{
        addFindTools(toolbar);
        addSortTools(toolbar);
	}

	/**
	 * Builds the commands bar.
	 *
	 * @return the commands component
	 */
	protected LyCommands buildCommands()
	{
		LyCommands commands = new LyCommands();
		LyCommand  command;

		commands.setId("_commands");

		command = new LyCommand(LyFormController.DEMAND_VALIDATE, "LY_VALIDATE");
		command.setAction(LyCommand.Action.SUBMIT);
		commands.addCommand(command);

		command = new LyCommand(LyImportTableController.DEMAND_IMPORT_SELECTION,
				"LY_IMPORT_SELECTION");
		command.setAction(LyCommand.Action.SUBMIT);
		commands.addCommand(command);

		command = new LyCommand(LyImportTableController.DEMAND_IMPORT_ALL, "LY_IMPORT_ALL");
		command.setAction(LyCommand.Action.SUBMIT);
		commands.addCommand(command);

		/*command = new LyCommand(LyImportTableController.DEMAND_VIEW_IMPORTED, "LY_VIEW_IMPORTED");
		command.setAction(LyCommand.Action.SUBMIT);
		commands.addCommand(command);*/

		command = new LyCommand(LyFormController.DEMAND_CANCEL, "LY_CANCEL");
		command.setAction(LyCommand.Action.CANCEL);
		commands.addCommand(command);

		command = buildHelpCommand();

		if (command != null)
			commands.addCommand(command);

		return commands;
	} // end method buildCommands

	/**
	 * Build a status header.
	 *
	 * @return the status headers
	 */
	protected LyHeader buildStatusHeader()
	{
		LyHeader header = new LyHeader();

		header.setId(LySimpleTableController.DEMAND_STATUS_SORT);
		header.setLabel("LY_STATUS_SORT");
		header.setSortable(false);
		header.setResizable(false);
		header.setFixed(true);
		return header;
	}
} // end class LyImportTableBuilder
