/**
 * LyLoginBuilder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.control.builder;

import java.util.*;
import leon.app.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;


/**
 * Creates a view for login action.
 *
 * @see LyLoginController
 */
public class LyLoginBuilder extends LyUserInputBuilder
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyLoginBuilder.java 41853 2012-12-26 16:17:42Z bangiole $";

	/**
	 * Default constructor
	 * @param parent the parent controller
	 */
	public LyLoginBuilder(LyController parent)
	{
		super(parent);
	}

	/**
	 * @see leon.control.builder.LyUserInputBuilder#build()
	 */
	public LyComponent build()
	{
		LyArea		 area = (LyArea)super.build();
		LyPanel		 panel = area.getPanel(0);
		LyForm		 form = new LyForm(_parent.getId() + "_mainForm");
		LyFormField  field;
		LyIcon		 icon = new LyIcon("LY_APPLICATION", "LY_LOGIN_IMAGE");
		LyConstraint constraint;

		area.removePanel(panel);

		form.setBorder(true);
		form.setAlign(LyForm.Align.CENTER);

		field = new LyFormField("_iconField");
		field.setFieldItem(icon);
		field.setEditable(false);
		constraint = new LyConstraint();
		constraint.setRemainder(false);
		constraint.setColSpan(1);
		constraint.setVAlign(LyConstraint.VAlign.MIDDLE);
		field.setConstraint(constraint);
		form.addFormField(field);

		field = new LyFormField("_areaField");
		field.setFieldItem(panel);
		field.setEditable(false);
		constraint = new LyConstraint();
		constraint.setVAlign(LyConstraint.VAlign.MIDDLE);
		field.setConstraint(constraint);
		form.addFormField(field);

		area.addPanel(form);

		return area;
	} // end method build

	/**
	 * @see leon.control.builder.LyUserInputBuilder#buildFormContent()
	 */
	public void buildFormContent()
	{
		super.buildFormContent();

		LyLoginController loginController = (LyLoginController)_parent;
		LyAction		  action = loginController.getAction();
		LySession		  session = _parent.getSession();
        LyObject          user = session.getUser();
		LyApplication     application = session.getApplication();
		int				  nCols = LyLoginController.DEFAULT_COLS;
		String selectLang = (String)loginController.getParameter("_selectLanguage");
		boolean showLang = (selectLang == null ? true : Boolean.parseBoolean(selectLang));

		if (action != null)
		{
			loginController.setImage(loginController.getImage(action));
			loginController.setTitleBar(action.getName() + " : " + "LY_APPLICATION");
		}

        if (user == null)
        {
            // Login
            LyText    loginField = addText(LyLoginController.PARAM_KEY_LOGIN, "LY_LOGIN_USER", null, nCols, null, false);
            LyAction  create = loginController.getCreateAction();

            if ((loginField != null)
                && (create != null))
            {
				LyFormField			 formField = (LyFormField)loginField.getParent();
				LyInfoList<LyAction> actions = new LyInfoList<LyAction>();

				actions.addInfo(create);

				LyToolBar toolBar = new LyToolBar();
				LyTool    tool = new LyTool(create.getId(), create.getAlias());

				tool.setTooltip(create.getName());
				tool.setImage(_parent.getImage(create,
						LyFormBuilder.DEFAULT_TRANSVERSE_ICON_WIDTH));
				tool.setDialog(true);

				toolBar.setController(_parent.getId());
				toolBar.addTool(tool);
				formField.setToolBar(toolBar);
    		}

    		// Password
    		LyText passwordField = addText(LyLoginController.PARAM_KEY_PASSWORD, "LY_LOGIN_PASSWORD",
    				null, nCols, null, false);

    		passwordField.setSecret(true);
        }
		else
		    // Display user
            addLabel(LyLoginController.PARAM_KEY_LOGIN, "LY_LOGIN_USER", user.getName());

		// Language
		LyEnvironment appEnv = application.getEnvironment();
		Locale		  appLocale = appEnv.getLocale();
		String[][]    languages = application.getSupportedLanguages();

		if (showLang && (languages.length > 1))
		{
			LyRelation field;
			LyLink     link;

			field = addRelation(LyLoginController.PARAM_KEY_LANGUAGE, "LY_LOGIN_LANGUAGE", false, false);

			for (int i = 0; i < languages.length; i++)
			{
				link = new LyLink(languages[i][0], languages[i][1]);
				field.addLink(link);
				if (languages[i][0].equals(appLocale.getLanguage()))
					field.selectLink(i, true);
			}
		}

		setFormAlignment(LyForm.Align.CENTER);

		// Remove form border from
		setFormBorder(false);
	} // end method buildFormContent

	/**
	 * Creates the command bar and sets its content.
	 *
	 * @return the commands component
	 */
	protected LyCommands buildCommands()
	{
		LyCommands commands = super.buildCommands();

		commands.setBorder(LyArea.Border.IN);
		return commands;
	}
} // end class LyLoginBuilder
