/**
 * LyMetricsPanelBuilder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyCompoundAreaBuilder.java 33190 2011-11-17 14:55:53Z kchenal $
 */

package leon.control.builder;

import leon.control.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;


/**
 * This builder construct the view for the metrics panel controller.
 */
public class LyMetricsPanelBuilder extends LyCompoundAreaBuilder
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyCompoundAreaBuilder.java 33190 2011-11-17 14:55:53Z kchenal $";

	/**
	 * Default constructor.
	 *
	 * @param parent parent controller
	 */
	public LyMetricsPanelBuilder(LyController parent)
	{
		super(parent);
	}

	/**
	 * Builds the view component for the specified parent controller. The component may create the
	 * XML Tree or the internal view.
	 *
	 * @return the view Component
	 */
	public LyComponent build()
	{
		LyArea	area = (LyArea)super.build();
		
		if (area != null)
		{
			area.setTopRightToolBar(buildRightToolBar());

			LyAction		action = ((LyAreaController)_parent).getAction();

		    if ((action == null)
			        || !action.hasMark(LyAction.MARK_NO_TOOLBAR))
		    	area.setToolBar(completeToolBar(null));
			
			LyFormField preferenceField = buildPeriodFilterField();
			
			if (preferenceField != null)
			{
			    LyToolBar   toolBar = area.getTopRightToolBar();
	
				if (toolBar == null)
				{
					toolBar = new LyToolBar();
					toolBar.setFill(LyToolBar.Fill.HORIZONTAL);
					toolBar.setStyle(LyToolBar.Style.SEPARATOR);
					toolBar.setHAlign(LyToolBar.HAlign.RIGHT);
					toolBar.setVAlign(LyToolBar.VAlign.TOP);
					toolBar.setController(_parent.getId());
				}
				else
					toolBar.addSep();
	
				toolBar.addFormField(preferenceField);
				area.setTopRightToolBar(toolBar);
			}
		}
		
		area.setScrolling(LyArea.Scrolling.AUTO);
		
		return area;
	}

	protected LyFormField buildPeriodFilterField()
	{
		LyFormField   formField = new LyFormField(_parent.getId() + "_periodFilterField");
		LyRelation    relation = new LyRelation();
		LyEnvironment environment = _parent.getEnvironment();
		String        displayString = environment.translate("LY_ITEM_FILTER_ACTIVE");

		if ((displayString != null) && (displayString.trim().length() > 0))
		{
			LyLink link = new LyLink("LY_NO_CHOICE", "LY_ALL");
			LyLink link1 = new LyLink("LY_METRICS_FILTER_WEEK", environment.getString("LY_METRICS_FILTER_WEEK"));
			LyLink link2 = new LyLink("LY_METRICS_FILTER_MONTH", environment.getString("LY_METRICS_FILTER_MONTH"));
			LyLink link3 = new LyLink("LY_METRICS_FILTER_QUARTER", environment.getString("LY_METRICS_FILTER_QUARTER"));
			LyLink link4 = new LyLink("LY_METRICS_FILTER_YEAR", environment.getString("LY_METRICS_FILTER_YEAR"));
			
			relation.addLink(link);
			relation.addLink(link1);
			relation.addLink(link2);
			relation.addLink(link3);
			relation.addLink(link4);
		}

		formField.setFieldItem(relation);
		formField.setHidden(false);
		formField.setNeedCheck(true);
		formField.setLabel(environment.getString("LY_METRICS_FILTER_PERIOD"));
		formField.setController(_parent.getId());
		
		return formField;
	}
} // end class LyMetricsPanelBuilder
