/**
 * LyMultiPageTableBuilder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyMultiPageTableBuilder.java 29563 2011-02-10 15:16:32Z nhamon $
 */

package leon.control.builder;

import leon.control.*;
import leon.view.*;


/**
 * Builder associated to a multipage view.
 *
 * @see LyMultiPageTableController
 */
public class LyMultiPageTableBuilder extends LyTableBuilder
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyMultiPageTableBuilder.java 29563 2011-02-10 15:16:32Z nhamon $";

	/**
	 * Default constructor.
	 *
	 * @param parent the parent controller
	 */
	public LyMultiPageTableBuilder(LyController parent)
	{
		super(parent);
	}

	/**
	 * Builds the view component for the specified parent controller. The component may create the
	 * XML Tree or the internal view.
	 *
	 * @return the view Component
	 */
	public LyComponent build()
	{
		LyArea					   area = (LyArea)super.build();

		// Build select bar only if view limit is set
		LyMultiPageTableController multiPageController = (LyMultiPageTableController)_parent;

		if (!multiPageController.isPaginated())
			return area;

		// add a select bar
		area.setSelectBar(buildSelectBar());
		return area;
	}

	/**
	 * Builds the page selector message in the view.
	 *
	 * @return the message component
	 */
	protected LySelectBar buildSelectBar()
	{
		LySelectBar  selectBar = new LySelectBar(_parent.getId() + "_selectBar");
		LySelectItem item = new LySelectItem(LyMultiPageTableController.SELECT_PAGE + 0, "1");

		selectBar.setController(_parent.getId());
		selectBar.addSelectItem(item);

		// If page are displayed by the names of the objects, used a step of 10 pages instead of 20
		LyMultiPageTableController controller = (LyMultiPageTableController)_parent;

		if (!controller.isDisplayPagesNumber())
			selectBar.setItemCount(10);

		// Add a number field to choose the number of objects displayed per page
		LyFormField viewLimit = buildSelectViewLimitField();

		if (viewLimit != null)
			selectBar.setFormField(viewLimit);

		return selectBar;
	}

	/**
	 * Indicates whether the view limit field must be displayed in the top toolbar.
	 * 
	 * @return true if it must be displayed, false otherwise.
	 */
	protected boolean addTopToolbarViewLimitField()
	{
		return false;
	}
} // end class LyMultiPageTableBuilder
