/*
 * LyOutlineBuilder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyOutlineBuilder.java 31479 2011-06-17 14:10:55Z nhamon $
 */
package leon.control.builder;

import leon.control.*;
import leon.view.*;


/**
 * Builder for the outline action.
 *
 * @author nla
 */
public class LyOutlineBuilder extends LyAreaBuilder
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyOutlineBuilder.java 31479 2011-06-17 14:10:55Z nhamon $";

	/** LyTable which represents the object outline. */
	protected LyTable		   _table;

	/**
	 * Default constructor.
	 *
	 * @param parent parent controller
	 */
	public LyOutlineBuilder(LyController parent)
	{
		super(parent);
		_table = null;
	}

	/**
	 * Builds the outline table.
	 *
	 * @see leon.control.builder.LyBuilder#build()
	 */
	public LyComponent build()
	{
		LyArea area = new LyArea();

		area.setId(_parent.getId());
		area.setController(_parent.getId());

		// set content
		area.setTitleBar(buildTitleBar());
		_component = area;

		// Update content
		buildTable();
		((LyArea)_component).addPanel(_table);

		LyConstraint	constraint = new LyConstraint();
		
		constraint.setFill(LyConstraint.Fill.BOTH);
		area.setConstraint(constraint);
		_table.setConstraint(constraint);
		
		updateTitleBar();
		return area;
	}

	/**
	 * Builds the table of the outline.
	 */
	protected void buildTable()
	{
		_table = new LyTable(_parent.getId() + "_table");
		_table.setController(_parent.getId());
		_table.setSelectionMode(LyTable.SelectionMode.NONE);
		_table.setHLines(true);
		_table.setVLines(true);
	}
} // end class LyOutlineBuilder
