/**
 * LyPrintBuilder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyPrintBuilder.java 30316 2011-04-01 07:24:21Z bangiole $
 */

package leon.control.builder;

import java.util.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.view.*;


/**
 * Thius builder creates the view for the PrintController.
 *
 * @see LyPrintController
 */
public class LyPrintBuilder extends LyBuilder
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyPrintBuilder.java 30316 2011-04-01 07:24:21Z bangiole $";

	/**
	 * Default constructor.
	 *
	 * @param parent parent controller
	 */
	public LyPrintBuilder(LyController parent)
	{
		super(parent);
	}

	/**
	 * Builds the component.
	 *
	 * @return the built component
	 */
	public LyComponent build()
	{
		LyArea			    area = new LyArea();
		Vector<LyConsultController>	controllers = ((LyPrintController)_parent).getSonControllers();
		LyClassInfo		    classInfo;
		LyConsultController consultCont;
		LyArea			    subArea;
		LyTitleBar		    titleBar;
		LyObject		    object;
		String			    label;
		String			    image;
		int				    i, n = controllers.size();

		for (i = 0; i < n; i++)
		{
			consultCont = (LyConsultController)controllers.elementAt(i);
			subArea = (LyArea)consultCont.getComponent();
			titleBar = new LyTitleBar();
			object = consultCont.getObject();
			classInfo = object.getClassInfo();
			label = classInfo.getName() + " : " + consultCont.getName(object);
			image = _parent.getImage(classInfo);

			titleBar.setLeftLabel(label);
			if (_parent.getEnvironment().testResource("LY_TITLEBAR_LEFT_IMAGE"))
				titleBar.setImage(image);

			subArea.setCommands(null);
			//subArea.setToolBar(null);
			subArea.setTitleBar(titleBar);
			subArea.setController(null);
			subArea.setId(null);
			
			LyConstraint constraint = new LyConstraint();
			constraint.setRemainder(true);
			subArea.setConstraint(constraint);
			

			area.addPanel(subArea);
		}

		area.setController(_parent.getId());
		area.setId(_parent.getId());
		area.setScrolling(LyArea.Scrolling.AUTO);

		return area;
	} // end method build
} // end class LyPrintBuilder
