/**
 * LyReportingBuilder.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.control.builder;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;

import leon.control.LyController;
import leon.control.LyReportingController;
import leon.view.LyArea;
import leon.view.LyComponent;
import leon.view.LyReporting;

/** 
 * Creates a view to show a report.
 */
public class LyReportingBuilder extends LyAreaBuilder
{
	/** Represents the parent controller*/
	private LyReportingController 			_parent;

	/** Area used to display the report*/
	private LyArea 							_area;
	
	private LyReporting						_report;
	
	/**
	 * Default constructor
	 * 
	 * @param parent parent controller
	 */
	public LyReportingBuilder(LyController parent)
	{
		super(parent);
		_parent = (LyReportingController) parent;
	}

	/**
	 * Builds the view component for the specified parent controller. The component may create the
	 * XML Tree or the internal view.
	 *
	 * @return the view Component
	 */
	public LyComponent build()
	{
		_area = new LyArea();		
		_area.setId(_parent.getId());
		_area.setController(_parent.getId());
		_area.setTitleBar(buildTitleBar());
		_area.addPanel(getReport());
				
		_component = _area;
		updateTitleBar();
		return _area;
	}
	
	/**
	 * Gets the current LyReporting
	 * 
	 * @return the current LyReporting
	 */
	public LyReporting getReport ()
	{
		_report = new LyReporting();
		_report.setBfReportHost(_parent.getBfreportHost());
		_report.setReportName(_parent.getReportName());
		_report.setReportDir(_parent.getReportDir());
		
		Hashtable<String, String> reportParams = new Hashtable<String, String>();
		HashMap<String, Object []> reportParameter = _parent.getReportParameters();
		Iterator <String> iterParamName = reportParameter.keySet().iterator();
		while (iterParamName.hasNext())
		{
			String paramName = iterParamName.next();
			Object [] values = reportParameter.get(paramName);
			
			for (int index = 0 ; index < values.length; index++)
			{
				Object value = values [index];
				reportParams.put(paramName+"_"+index, value.toString());
			}
		}
		_report.setDatas(reportParams);
		return _report;
	}
	
	/**
	 * Gets the current area
	 * 
	 * @return the area used to display the report
	 */
	public LyComponent getComponent ()
	{
		return _area;
	}
}// end class LyReportingBuilder
