/**
 * LySelectBuilder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LySelectBuilder.java 28422 2010-11-03 12:03:38Z ehelmste $
 */

package leon.control.builder;

import leon.control.*;
import leon.info.*;
import leon.view.*;
import leon.view.LyAreaBase.*;


/**
 * This builder creates the modification form for the set controller It is based on the consult
 * builder.
 *
 * @see LyConsultBuilder
 * @see LySetController
 */
public class LySelectBuilder extends LyCompoundAreaBuilder
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LySelectBuilder.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Default constructor.
	 *
	 * @param parent parent controller
	 */
	public LySelectBuilder(LyController parent)
	{
		super(parent);
	}

	/**
	 * Builds the view component for the specified parent controller. The component may create the
	 * XML Tree or the internal view.
	 *
	 * @return the view Component
	 */
	public LyComponent build()
	{
		LyArea			 area = (LyArea)super.build();
		LySplitter		 splitter = (LySplitter)area.getPanel(0);
		LyArea			 tableArea = ((LyArea)splitter.getPanel(1));
		LyAreaController tableController = ((LyCompoundAreaController)_parent).getAreaController(1);
		int              limitParameter = ((LyTableController)tableController).getLimit();
		
		if (limitParameter  > 0)
		{
		    LyAreaBuilder    tableBuilder = (LyAreaBuilder)tableController.getBuilder();
    		LyToolBar		 toolBar = tableBuilder.completeToolBar(null);
    
    		if (toolBar != null)
    		{
    			//When the toolbar contains the limit field, we fill it.
    			LyFormField viewLimitFormField = (LyFormField)toolBar.getComponent(tableController.getId() + "_viewLimit");
    			if(viewLimitFormField != null)
    			{
    				LyNumber viewLimitNumber = (LyNumber)viewLimitFormField.getFieldItem();
    				if(viewLimitNumber != null)
    				{
    					viewLimitNumber.setLabel(String.valueOf(limitParameter));
    				}
    			}
  
    			toolBar.setController(tableController.getId());
    			tableArea.setToolBar(toolBar);
    			}
		}
		
		tableArea.setScrolling(Scrolling.AUTO);

		area.setCommands(buildCommands());

		return area;
	}

	/**
	 * Builds commands bar.
	 *
	 * @return the commands component
	 */
	protected LyCommands buildCommands()
	{
		LyAction action = ((LyAreaController)_parent).getAction();

		if (action.hasMark(LyAction.MARK_NO_COMMANDS))
			return null;

		LyCommands commands = new LyCommands();
		LyCommand  command;

		command = new LyCommand(LyFormController.DEMAND_VALIDATE, "LY_VALIDATE");
		command.setAction(LyCommand.Action.SUBMIT);
		commands.addCommand(command);

		command = new LyCommand(LyFormController.DEMAND_CANCEL, "LY_CANCEL");
		command.setAction(LyCommand.Action.CANCEL);
		commands.addCommand(command);

		command = buildHelpCommand();
		if (command != null)
			commands.addCommand(command);

		return commands;
	}
} // end class LySelectBuilder
