/**
 * LyTableBuilder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyTableBuilder.java 41024 2012-10-04 11:55:18Z bangiole $
 */

package leon.control.builder;

import leon.app.*;
import leon.app.behavior.*;
import leon.control.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;
import leon.view.mobile.*;

import java.util.*;


/**
 * This builder creates a table view for the Table controller It is based on the simple table
 * builder.
 *
 * @see LySimpleTableBuilder
 * @see LyTableController
 */
public class LyTableBuilder extends LySimpleTableBuilder
{
    /**
     * File version
     */
    public static final String __VERSION =
        "$Id: LyTableBuilder.java 41024 2012-10-04 11:55:18Z bangiole $";

    /** Associations between actions prefix and instances of LyMenu. */
    Hashtable<String, _LyActionMenu> _actionMenus;

    /**
     * Default constructor.
     *
     * @param parent parent controller
     */
    public LyTableBuilder(LyController parent)
    {
        super(parent);

        _actionMenus = null;
    }

    /**
     * Builds the view component for the specified parent controller. The component may create the
     * XML Tree or the internal view.
     *
     * @return the view Component
     */
    public LyComponent build()
    {
        LyArea area = (LyArea) super.build();

        // set content
        area.setToolBar(buildToolBar());
        area.setTopRightToolBar(buildRightToolBar());
        area.setTitleBar(buildTitleBar());

        // Update default content
        completePanel(area);

        return area;
    }

    /**
     * Builds the table description.
     *
     * @return the table component
     */
    protected LyTable buildTable()
    {
        LyTable table = super.buildTable();

        table.setFooters(buildFooters());

        return table;
    }

    /**
     * Complete the toolbar.
     *
     * @param  toolBar the toolbar to complete
     * @return the completed toolbar
     */
    protected LyToolBar completeToolBar(LyToolBar toolBar)
    {
        LyToolBar         tools = super.completeToolBar(toolBar);
        LyTableController tableController = (LyTableController) _parent;
        LyAction          parentAction = tableController.getAction();

        if (((LyTableController) _parent).isDisplayNextPreviousTools())
        {

            if (tools == null)
                tools = buildEmptyToolBar();

            // Add tools for page loading.
            if (tools.getComponentCount() > 0)
                tools.addSep();

            boolean isAllowedNavigation = true;
            String  allowNavigationParameter = (String) parentAction.getParameter(
                    "_allowNavigation");

            if ((allowNavigationParameter != null) && "false".equals(allowNavigationParameter))
                isAllowedNavigation = false;

            if (isAllowedNavigation)
            {
                LyTool tool;
                tool = new LyTool(LySimpleTableController.DEMAND_PREVIOUS_PAGE,
                        "LY_TOOL_PREVIOUS_PAGE");
                tool.setImage("LY_TOOL_PREVIOUS_PAGE");
                tool.setEnabled(true);
                tool.setDialog(false);
                tools.addTool(tool);

                tool = new LyTool(LySimpleTableController.DEMAND_NEXT_PAGE, "LY_TOOL_NEXT_PAGE");
                tool.setImage("LY_TOOL_NEXT_PAGE");
                tool.setEnabled(true);
                tool.setDialog(false);
                tools.addTool(tool);
            }

            if (addTopToolbarViewLimitField() && !getViewManager().hasProperty(LyMobileViewManager.MOBILE_ONLY_PROPERTY))
            {
                // Add a number field to choose the number of objects displayed per page
                LyFormField viewLimit = buildSelectViewLimitField();

                if (viewLimit != null)
                    tools.addFormField(viewLimit);
            }
        }

        if (!parentAction.hasMark(LyAction.MARK_NO_TOOLBAR))
            completeToolBarForMenus(tools);

        return tools;
    } // end method completeToolBar


    /**
     * Indicates whether the view limit field must be displayed in the top toolbar.
     *
     * @return true if it must be displayed, false otherwise.
     */
    protected boolean addTopToolbarViewLimitField()
    {
        return true;
    }

    /**
     * Build the number field used to change the number of objects per page.
     *
     * @return the field used to change the number of objects per page.
     */
    protected LyFormField buildSelectViewLimitField()
    {
        LyFormField   viewLimit = new LyFormField(_parent.getId() + "_viewLimit");
        LyCaption     caption = new LyCaption();
        LyNumber      number = new LyNumber();
        LyToolBar     toolbar = new LyToolBar();
        LyEnvironment environment = _parent.getEnvironment();
        int           max = environment.getIntResource("LY_MAX_OBJECTS_PER_PAGE");

        String label = environment.translate("LY_OBJECTS_PER_PAGE");

        if ((label == null) || (label.trim().length() == 0))
            return null;

        caption.setLabel("LY_OBJECTS_PER_PAGE");
        number.setVMin(1);
        number.setVMax(max);
        number.setDecimals(0);
        number.showArrows(false);
        toolbar.setController(_parent.getId());
        viewLimit.setOptional(true);
        viewLimit.setCaption(caption);
        viewLimit.setFieldItem(number);
        viewLimit.setToolBar(toolbar);

        LyTool refreshTool = new LyTool(_parent.getId() + "_refreshViewLimit",
                "LY_TOOL_PAGINATION_UPDATE");
        refreshTool.setImage("LY_VIEW_LIMIT_REFRESH");
        toolbar.addTool(refreshTool);

        return viewLimit;
    }

    /**
     * Add tools in the right toolbar.
     * @param toolbar a toolbar
     */
    protected void addRightToolbarTools(LyToolBar toolbar)
    {
        addFindTools(toolbar);
        addFilterTools(toolbar);
        addSortTools(toolbar);
        addChooseColumnsTool(toolbar);
        addPreferencesTools(toolbar);
        addDisplayListTool(toolbar);
    } // end method completeRightToolBar

    /**
     * Add extra tools if there is no menu.
     *
     * @param tools the toolbar to complete
     */
    protected void completeToolBarForMenus(LyToolBar tools)
    {
        LyViewManager viewManager = getViewManager();

        if ((viewManager == null) || viewManager.hasProperty("MENU")
                || viewManager.hasProperty(LyMobileViewManager.MOBILE_ONLY_PROPERTY))
            return;

        LyTool        tool;
        LyClassInfo   classInfo = ((LyGetClassInfoInterface) _parent).getClassInfo();
        LyEnvironment environment = _parent.getEnvironment();

        // Add find tools if at least one fieldinfos has the find Mark
        LyInfoList<LyFieldInfo> findfields = classInfo.getFieldInfos(LyFieldInfo.MARK_FIND);

        if ((findfields != null) && (findfields.getSize() > 0))
        {
            if (tools == null)
                tools = buildEmptyToolBar();

            if (tools.getComponentCount() > 0)
                tools.addSep();

            tool = new LyTool(LyFindController.DEMAND_FIND + LyController.ID_SEP + _parent.getId(),
                    "LY_TOOL_FIND");
            tool.setImage("LY_TOOL_FIND");
            tool.setEnabled(true);
            tool.setDialog(true);
            tools.addTool(tool);

            tool = new LyTool(LyFindController.DEMAND_PREVIOUS + LyController.ID_SEP
                    + _parent.getId(), "LY_TOOL_FIND_PREVIOUS");
            tool.setImage("LY_TOOL_FIND_PREVIOUS");
            tool.setEnabled(false);
            tool.setDialog(true);
            tools.addTool(tool);

            tool = new LyTool(LyFindController.DEMAND_NEXT + LyController.ID_SEP + _parent.getId(),
                    "LY_TOOL_FIND_NEXT");
            tool.setImage("LY_TOOL_FIND_NEXT");
            tool.setEnabled(false);
            tool.setDialog(true);
            tools.addTool(tool);
        }

        // Add filter tools if filtering is allowed on some fields of the classInfo
        // displayed in the table
        LyInfoList<LyFieldInfo> filterFields = classInfo.getFieldInfos(LyFieldInfo.MARK_FILTER);

        if ((filterFields != null) && (filterFields.getSize() > 0))
        {
            if (tools == null)
                tools = buildEmptyToolBar();

            if (tools.getComponentCount() > 0)
                tools.addSep();

            tool = new LyTool(LySimpleTableController.SIMPLE_FILTER_NEW + LyController.ID_SEP
                    + _parent.getId(), "LY_TOOL_FILTER_NEW");
            tool.setImage("LY_TOOL_FILTER_NEW");
            tool.setEnabled(true);
            tool.setDialog(true);
            tools.addTool(tool);

            if (environment.testResource("LY_EXTENDED_FILTER"))
            {
                tool = new LyTool(LySimpleTableController.EXTENDED_FILTER_NEW + LyController.ID_SEP
                        + _parent.getId(), "LY_TOOL_EXTENDED_FILTER_NEW");
                tool.setImage("LY_TOOL_EXTENDED_FILTER_NEW");
                tool.setEnabled(true);
                tool.setDialog(true);
                tools.addTool(tool);
            }

            tool = new LyTool(LySimpleTableController.FILTER_SET + LyController.ID_SEP
                    + _parent.getId(), "LY_TOOL_FILTER_SET");
            tool.setImage("LY_TOOL_FILTER_SET");
            tool.setEnabled(false);
            tool.setDialog(true);
            tools.addTool(tool);

            if (environment.testResource("LY_EXTENDED_FILTER"))
            {
                tool = new LyTool(LySimpleTableController.EXTEND_SIMPLE_FILTER + LyController.ID_SEP
                        + _parent.getId(), "LY_TOOL_EXTEND_SIMPLE_FILTER");
                tool.setImage("LY_TOOL_EXTEND_SIMPLE_FILTER");
                tool.setEnabled(false);
                tool.setDialog(true);
                tools.addTool(tool);
            }
        }

        String	displayString;

    	if (classInfo.getFieldInfoCount() > 1)
    	{
    		displayString = environment.translate("LY_TOOL_CHOOSE_COLS");

    		if ((displayString != null) && (displayString.trim().length() > 0))
	        {
	            // Columns chooser
	            if (tools == null)
	                tools = buildEmptyToolBar();

	            if (tools.getComponentCount() > 0)
	                tools.addSep();

	            tool = new LyTool(LyTableController.DEMAND_CHOOSE_COLUMNS + LyController.ID_SEP
	                    + _parent.getId(), "LY_CHOOSE_COLS");
	            tool.setImage("LY_TOOL_CHOOSE_COLS");
	            tool.setEnabled(true);
	            tool.setDialog(true);
	            tools.addTool(tool);
	        }
    	}

        LyInfoList<LyFieldInfo> sortFields = classInfo.getFieldInfos(LyFieldInfo.MARK_SORT);

        if ((sortFields != null)
            && (sortFields.getSize() > 0))
        {
            displayString = environment.translate("LY_TOOL_SORT_WIZARD");

            if ((displayString != null) && (displayString.trim().length() > 0))
            {
	            // Sort wizard
	            if (tools == null)
	                tools = buildEmptyToolBar();

	            if (tools.getComponentCount() > 0)
	                tools.addSep();

	            tool = new LyTool(LyTableController.DEMAND_SORT_WIZARD + LyController.ID_SEP
	                    + _parent.getId(), "LY_SORT_BY");
	            tool.setImage("LY_TOOL_SORT_WIZARD");
	            tool.setEnabled(true);
	            tool.setDialog(true);
	            tools.addTool(tool);
            }
        }

        // ** Help **
        String   classId, actionId, rootActionId, helpFile;
        LyAction action = null, rootAction;

        if (_parent instanceof LyGetActionInterface)
            action = ((LyGetActionInterface) _parent).getAction();

        // Check Help Button if help file exists
        rootAction = ((action == null) ? null : action.getRoot());

        classId = classInfo.getId();
        actionId = ((action == null) ? null : action.getId());
        rootActionId = ((rootAction == null) ? null : rootAction.getId());

        helpFile = environment.getHelpFile(actionId, rootActionId, classId);

        if (helpFile != null)
        {
            if (tools == null)
                tools = buildEmptyToolBar();

            if (tools.getComponentCount() > 0)
                tools.addSep();

            tool = new LyTool(LySimpleTableController.DEMAND_HELP, "LY_HELP");
            tool.setImage("LY_HELP");
            tool.setEnabled(true);
            tool.setDialog(true);
            tools.addTool(tool);
        }

        // Ascending compatibility
        if (environment.getViewManager() instanceof LyMobileViewManager)
            return;

        // ** List printing **
        if (tools == null)
            tools = buildEmptyToolBar();
        else if (tools.getComponentCount() > 0)
            tools.addSep();

        tool = new LyTool(LyTableController.DEMAND_PRINT_TABLE, "LY_ACTION_PRINT_TABLE");
        tool.setImage("LY_ACTION_PRINT");
        tool.setEnabled(true);
        tool.setDialog(true);
        tools.addTool(tool);
    } // end method completeToolBarForMenus

    /**
     * Builds an empty toolbar for this area.
     *
     * @return the toolbar component
     */
    protected LyToolBar buildEmptyToolBar()
    {
        LyToolBar toolBar = new LyToolBar();

        toolBar.setId("_toolBar");
        toolBar.setFill(LyToolBar.Fill.HORIZONTAL);
        toolBar.setColor("LY_DASHBOARD_HORIZONTAL_BACKGROUND");

        return toolBar;
    }

    /**
     * Complete the file menu
     * @param menuBar the menu bar
     * @param nFile
     *
     */
    protected void completeFileMenu(LyMenuBar menuBar, int nFile)
    {
        // Ascending compatibility
        LyMenu        fileMenu = (LyMenu) menuBar.getComponentByLabel("LY_MENU_FILE");
        LyEnvironment environment = _parent.getEnvironment();
        boolean       printTable = environment.testResource("LY_TABLE_PRINT_TABLE");

        if (printTable)
        {
            String label = "LY_ACTION_PRINT_TABLE";
            String displayString = environment.translate(label);

            if ((displayString != null)
            	&& (displayString.trim().length() > 0))
            {
                // Add class actions in Actions menu
                LyItem   item;
                String   id = null, image = null;
                LyAction actionItem = null;

                // *** Add printing info ***
                if (nFile > 0)
                {
                    fileMenu.insertSep(nFile); // Sep
                    nFile++;
                }

                id = LyTableController.DEMAND_PRINT_TABLE;
                actionItem = (LyAction) _parent.getApplication().getInfo(id);
                image = ((LyAreaController) _parent).getMenuItemImage(actionItem);
                item = new LyItem(id, label);
                item.setImage(image);
                item.setController(_parent.getId());
                item.setDialog(true);
                item.setShortcut("LY_SHORTCUT_PRINT_TABLE");
                fileMenu.insertItem(item, nFile);
                nFile++;
            }
        }

        fileMenu.insertSep(nFile); // Sep
    } // end method completeFileMenu


    /**
     * @see leon.control.builder.LyPanelBuilder#completeActionsMenu(leon.view.LyMenuBar)
     */
    protected int completeActionsMenu(LyMenuBar menuBar)
    {
        // Add class actions in Actions menu
        LyClassInfo   classInfo = ((LyTableController) _parent).getClassInfo();
        LyItem        item;
        int           nFile = 0;
        LyEnvironment environment = _parent.getEnvironment();
        LyMenu        fileMenu = (LyMenu) menuBar.getComponentByLabel("LY_MENU_FILE");
        String        displayString;
        LyMenu        actionsMenu = null;
        LyAction      action;

        // Search for Actions Menu
        displayString = environment.translate("LY_MENU_ACTIONS");

        if ((displayString != null) && (displayString.length() != 0))
        {
            actionsMenu = (LyMenu) menuBar.getComponentByLabel("LY_MENU_ACTIONS");

            LyInfoDataList<LyAction, LyClassInfo> actionsList = getMenuActions((LyTableController)
                    _parent, classInfo, false);
            LyAction                              action2;
            LyMenu                                menu, subMenu;
            LyClassInfo                           classInfo2, classInfo3;
            int                                   i, j, min;
            boolean                               multiple;
            boolean                               changeLabel;
            String                                menuName;

            // FFT00293
            if (actionsList.getSize() > 0)
            {

                // We add/Complete the action Menu the actions
                // If it doesn't exist, create it
                if (actionsMenu == null)
                {
                    actionsMenu = new LyMenu("LY_MENU_ACTIONS");
                    menuBar.addMenu(actionsMenu);
                }
                else
                    actionsMenu.addSep();
            }

            // For each action in the super class, search for equivalent action
            // in subclasses.
            for (i = 0; i < actionsList.getSize(); i++)
            {
                action = actionsList.getInfo(i);
                menuName = action.getMenu();

                if (menuName == null)
                    menu = actionsMenu;
                else
                {
                    menu = (LyMenu) menuBar.getComponentByLabel(menuName);

                    if (menu == null)
                    {
                        menu = new LyMenu(menuName);
                        menuBar.addMenu(menu);
                    }
                }

                classInfo3 = actionsList.getData(i);
                min = action.getMin();
                multiple = false;
                changeLabel = false;

                // Check if we have multiple occurrences of the same action image
                // if so, build a tool with a sub menu
                subMenu = null;

                for (j = i + 1; j < actionsList.getSize(); j++)
                {
                    action2 = actionsList.getInfo(j);

                    if ((action.getRoot() == action2.getRoot())
                            && (_parent.getImage(action).equals(_parent.getImage(action2))))
                    {
                        classInfo2 = actionsList.getData(j);

                        if ((classInfo2 == classInfo) || (min == 0))
                        {

                            if ((action2.getName() != null)
                                    && (action2.getName().equals(action.getName())))
                                changeLabel = true;

                            if (subMenu == null)
                            {
                                subMenu = new LyMenu(action.getName());
                                subMenu.setController(_parent.getId());
                                subMenu.setId(action.getId());

                                item = buildItem(action, classInfo3);
                                item.setImage(((LyAreaController) _parent).getMenuItemImage(
                                        action));

                                if (changeLabel)
                                    item.setLabel(action.getName() + " : " + classInfo3.getName());

                                subMenu.addItem(item);
                            }

                            item = buildItem(action2, classInfo2);
                            item.setImage(((LyAreaController) _parent).getMenuItemImage(action2));

                            if (changeLabel)
                                item.setLabel(action2.getName() + " : " + classInfo2.getName());

                            subMenu.addItem(item);
                        }

                        multiple = true;
                        actionsList.removeInfo(action2, classInfo2);
                        j--;
                    }
                }

                if (subMenu != null)
                {

                    if ((menuName == null) && (action.getMax() == 0)) // Don't care of selections
                    {
                        // Insert in menu File
                        fileMenu.insertMenu(subMenu, nFile);
                        nFile++;
                    }
                    else
                        // Add in Actions menu
                        menu.addMenu(subMenu);
                }
                else
                {

                    if (multiple)
                        item = buildItem(action);
                    else
                        item = buildItem(action, classInfo3);

                    item.setImage(((LyAreaController) _parent).getMenuItemImage(action));
                    item.setController(_parent.getId());

                    if (action.hasMark(LyAction.MARK_DIALOG))
                        item.setDialog(true);
                        //item.setShortcut(action.getShortcut());

                    if ((menuName == null) && (action.getMax() == 0)) // Don't care of selections
                    {
                        // Insert in menu File
                        fileMenu.insertItem(item, nFile);
                        nFile++;
                    }
                    else
                        // Add in Actions menu
                        menu.addItem(item);
                }
            }
        }

        return nFile;
    } // end method completeActionsMenu

    /**
     * @see leon.control.builder.LyPanelBuilder#completeDisplayMenu(leon.view.LyMenuBar)
     */
    protected void completeDisplayMenu(LyMenuBar menuBar)
    {
        LyClassInfo   classInfo = ((LyTableController) _parent).getClassInfo();
        LyEnvironment environment = _parent.getEnvironment();
        String        displayString = environment.translate("LY_MENU_DISPLAY");

        if ((displayString != null) && (displayString.length() != 0))
        {
            // Search for display Menu
            LyMenu displayMenu = (LyMenu) menuBar.getComponentByLabel("LY_MENU_DISPLAY");

            // if menu display does not exist : create it
            if (displayMenu == null)
            {
                displayMenu = new LyMenu("LY_MENU_DISPLAY");
                displayMenu.setId("_displayMenu");
                menuBar.addMenu(displayMenu);
            }
            else
                displayMenu.addSep();

            addSortItems(displayMenu);

            addGroupItems(displayMenu);

            addFilterItems(displayMenu);

            addChooseColumnsItems(displayMenu);

            // Build fix column sub-menu
            if (classInfo.getFieldInfoCount() > 1)
                updateFixColumnSubMenu(menuBar);

            addPreferencesMenu(displayMenu);
        }
    } // end method completeDisplayMenu

    /**
     * Complete menu to add sort items
     * @param displayMenu the display menu
     */
    protected void addSortItems(LyMenu displayMenu)
    {
        LyClassInfo             classInfo = ((LyTableController) _parent).getClassInfo();
        LyItem                  item;
        LyFieldInfo             fieldInfo;
        int                     i, n;
        LyEnvironment           environment = _parent.getEnvironment();
        LyInfoList<LyFieldInfo> fields;
        String                  displayString;
        String                  id = null, image = null;
        LyAction                actionItem = null;

        displayString = environment.translate("LY_MENU_SORT");

        if ((displayString != null) && (displayString.trim().length() > 0))
        {
            // Create sort menu
            fields = classInfo.getFieldInfos(LyFieldInfo.MARK_SORT);

            if ((fields == null) || (fields.getSize() == 0))
            {
                fields = classInfo.getFieldInfos();
                fields = fields.getInfos(LyFieldInfo.MARK_HIDDEN, true);
                fields = fields.getInfos(LyFieldInfo.MARK_SECRET, true);
            }

            if ((fields != null) && (fields.getSize() > 0))
            {
                int    m;
                LyMenu sortMenu, subSortMenu = null;

                fields = classInfo.getFieldInfos(LyFieldInfo.MARK_SORT);

                sortMenu = new LyMenu("LY_MENU_SORT");
                displayMenu.addMenu(sortMenu);

                displayString = environment.translate("LY_ITEM_SORT_WIZARD");

                if ((displayString != null) && (displayString.trim().length() > 0))
                {
                    id = LyTableController.DEMAND_SORT_WIZARD;
                    actionItem = (LyAction) _parent.getApplication().getInfo(id);
                    image = ((LyAreaController) _parent).getMenuItemImage(actionItem);
                    item = new LyItem(id, "LY_ITEM_SORT_WIZARD");
                    item.setImage(image);
                    item.setEnabled(true);
                    item.setDialog(true);
                    item.setController(_parent.getId());
                    sortMenu.addItem(item);
                }
                else
                {

                    for (m = 0; m < 3; m++)
                    {
                        subSortMenu = null;

                        if (m == 0)
                        {
                            displayString = environment.translate("LY_MENU_SORT_BY");

                            if ((displayString != null) && (displayString.trim().length() > 0))
                                subSortMenu = new LyMenu("LY_MENU_SORT_BY");
                        }
                        else if (m == 1)
                        {
                            displayString = environment.translate("LY_MENU_SORT_EXTEND");

                            if ((displayString != null) && (displayString.trim().length() > 0))
                                subSortMenu = new LyMenu("LY_MENU_SORT_EXTEND");
                        }
                        else if (m == 2)
                        {
                            displayString = environment.translate("LY_MENU_SORT_REVERSE");

                            if ((displayString != null) && (displayString.trim().length() > 0))
                                subSortMenu = new LyMenu("LY_MENU_SORT_REVERSE");
                        }

                        if (sortMenu != null)
                        {
                            sortMenu.addMenu(subSortMenu);

                            for (i = 0, n = fields.getSize(); i < n; i++)
                            {
                                fieldInfo = fields.getInfo(i);

								if (!((LyTableController)_parent).allowSortOnField(fieldInfo))
									continue;

                                switch (m)
                                {

                                case 0:
                                    id = LyTableController.DEMAND_SORT_BY;
                                    break;

                                case 1:
                                    id = LyTableController.DEMAND_SORT_EXTEND;
                                    break;

                                case 2:
                                    id = LyTableController.DEMAND_SORT_REVERSE;
                                    break;

                                default:
                                    id = null;
                                    break;
                                }

                                actionItem = (LyAction) _parent.getApplication().getInfo(id);
                                image = ((LyAreaController) _parent).getMenuItemImage(actionItem);
                                item = new LyItem(id + LyController.ID_SEP + fieldInfo.getId(),
                                        fieldInfo.getName());
                                item.setImage(image);
                                item.setController(_parent.getId());
                                subSortMenu.addItem(item);
                            }
                        }
                    }

                    sortMenu.addSep();
                }

                if (classInfo.getSort() != null)
                {
                    displayString = environment.translate("LY_ITEM_SORT_DEFAULT");

                    if ((displayString != null) && (displayString.trim().length() > 0))
                    {
                        id = LyTableController.DEMAND_SORT_DEFAULT;
                        actionItem = (LyAction) _parent.getApplication().getInfo(id);
                        image = ((LyAreaController) _parent).getMenuItemImage(actionItem);
                        item = new LyItem(id, "LY_ITEM_SORT_DEFAULT");
                        item.setImage(image);
                        item.setController(_parent.getId());
                        sortMenu.addItem(item);
                    }
                }

                displayString = environment.translate("LY_ITEM_SORT_REVERSE");

                if ((displayString != null) && (displayString.trim().length() > 0))
                {
                    id = LyTableController.DEMAND_REVERSE;
                    actionItem = (LyAction) _parent.getApplication().getInfo(id);
                    image = ((LyAreaController) _parent).getMenuItemImage(actionItem);
                    item = new LyItem(id, "LY_ITEM_SORT_REVERSE");
                    item.setImage(image);
                    item.setController(_parent.getId());
                    sortMenu.addItem(item);
                }

                // if there are field with status in this classinfo, we add a header...
                fields = ((LyTableController) _parent).getStatusFieldInfos();

                LyHeader statusHeader = null;

                // SMA in list display, no headers, check if not null so.
                if ((_table.getHeaders() != null) && (_table.getHeaders().getHeaderCount() > 0))
                    statusHeader = _table.getHeaders().getHeader(0);

                boolean sepAdded = false;

                if ((fields != null) && (fields.getSize() > 0) && (statusHeader != null)
                        && statusHeader.isSortable())
                {
                    displayString = environment.translate("LY_ITEM_STATUS_SORT");

                    if ((displayString != null) && (displayString.trim().length() > 0))
                    {
                        sortMenu.addSep();
                        id = LyTableController.DEMAND_STATUS_SORT;
                        actionItem = (LyAction) _parent.getApplication().getInfo(id);
                        image = ((LyAreaController) _parent).getMenuItemImage(actionItem);
                        item = new LyItem(id, "LY_ITEM_STATUS_SORT");
                        item.setImage(image);
                        item.setController(_parent.getId());
                        sortMenu.addItem(item);
                        sepAdded = true;
                    }

                    displayString = environment.translate("LY_ITEM_INVERT_STATUS_SORT");

                    if ((displayString != null) && (displayString.trim().length() > 0))
                    {

                        if (!sepAdded)
                            sortMenu.addSep();

                        id = LyTableController.DEMAND_INVERT_STATUS_SORT;
                        actionItem = (LyAction) _parent.getApplication().getInfo(id);
                        image = ((LyAreaController) _parent).getMenuItemImage(actionItem);
                        item = new LyItem(id, "LY_ITEM_INVERT_STATUS_SORT");
                        item.setImage(image);
                        item.setController(_parent.getId());
                        sortMenu.addItem(item);
                    }
                }
            }
        }
    } // end method addSortItems

    /**
     * Add group items
     * @param displayMenu the display menu
     */
    protected void addGroupItems(LyMenu displayMenu)
    {
        LyEnvironment environment = _parent.getEnvironment();
        String        displayString;

        displayString = environment.translate("LY_MENU_GROUP_BY");

        if ((displayString != null) && (displayString.trim().length() > 0))
        {
            LyTableController       tableController = (LyTableController) _parent;
            LyTableBehavior         behavior = (LyTableBehavior) tableController.getBehavior();
            LyClassInfo             classInfo = ((LyTableController) _parent).getClassInfo();
            LyInfoList<LyFieldInfo> fields = new LyInfoList<LyFieldInfo>();
            int                     i, n = classInfo.getFieldInfoCount();
            LyFieldInfo             fieldInfo;

            for (i = 0; i < n; i++)
            {
                fieldInfo = classInfo.getFieldInfo(i);

                if (behavior.enableGroupBy(tableController, fieldInfo))
                    fields.addInfo(fieldInfo);
            }

            if (fields.getSize() > 0)
            {
                LyMenu groupMenu = new LyMenu("LY_MENU_GROUP_BY");
                LyItem item;

                displayMenu.addMenu(groupMenu);

                for (i = 0, n = fields.getSize(); i < n; i++)
                {
                    fieldInfo = fields.getInfo(i);
                    item = new LyItem(LyTableController.DEMAND_GROUP_BY + LyController.ID_SEP
                            + fieldInfo.getId(), fieldInfo.getAlias());
                    item.setController(_parent.getId());
                    item.setCheckBox(true);
                    item.setChecked(false);
                    groupMenu.addItem(item);
                }
            }
        }
    } // end method addGroupItems

    /**
     * Add sort tools
     * @param toolbar a toolbar
     */
    protected void addSortTools(LyToolBar toolbar)
    {
        LyPanelController panel = (LyPanelController) _parent;

        if (!(panel instanceof LyFilterEndInterface))
            return;

        LyClassInfo             classInfo = ((LyPanelController) _parent).getClassInfo();
        LyInfoList<LyFieldInfo> fields;

        fields = classInfo.getFieldInfos(LyFieldInfo.MARK_SORT);

        if ((fields == null) || (fields.getSize() == 0))
        {
            fields = classInfo.getFieldInfos();
            fields = fields.getInfos(LyFieldInfo.MARK_HIDDEN, true);
            fields = fields.getInfos(LyFieldInfo.MARK_SECRET, true);
        }

        if ((fields == null) || (fields.getSize() == 0))
            return;

        LyEnvironment environment = _parent.getEnvironment();
        String        displayString;

        displayString = environment.translate("LY_TOOL_SORT_WIZARD");

        if ((displayString != null)
                && (displayString.trim().length() > 0)
                && !"_null".equals(displayString))
        {
            LyTool tool = new LyTool(LyTableController.DEMAND_SORT_WIZARD + LyController.ID_SEP
                    + _parent.getId(), "");

            tool.setImage("LY_TOOL_SORT_WIZARD");
            tool.setTooltip("LY_ITEM_SORT_WIZARD");
            tool.setDialog(true);
            toolbar.addTool(tool);
        }
    }

    /**
     * Add fast filter item to filter menu
     * @param menuFilter the filter menu
     *
     */
    protected void addFastFilterItems(LyMenu menuFilter)
    {
        LyEnvironment environment = _parent.getEnvironment();
        String        displayString = environment.translate("LY_ITEM_FAST_FILTER");

        if ((displayString == null) || (displayString.length() == 0))
            return;

        // add items SIMPLE_FILTER_NEW, EXTENDED_FILTER_NEW,
        // FILTER_SET, EXTEND_SIMPLE_FILTER, FILTER_ACTIVE
        LyTableController       tableController = (LyTableController) _parent;
        LyInfoList<LyFieldInfo> fastFilterFields = tableController.getFields();

        fastFilterFields = fastFilterFields.getInfos(LyFieldInfo.MARK_FILTER);

        if ((fastFilterFields != null) && (fastFilterFields.getSize() > 0))
        {
            LyMenu      menuFastFilter = new LyMenu("LY_ITEM_FAST_FILTER");
            LyFieldInfo fastFilterField;
            int         j, m = fastFilterFields.getSize();
            LyClassInfo classInfo = ((LyPanelController) _parent).getClassInfo();
            LyItem      item;

            for (j = 0; j < m; j++)
            {
                fastFilterField = fastFilterFields.getInfo(j);

                // Struct fields may not be filtered using fast filters
                if (fastFilterField instanceof LyStructFieldInfo)
                    continue;

                // Check if field really belongs to class info (may happen in complex table actions)
                if (!classInfo.contains(fastFilterField))
                    continue;

                item = new LyItem(LyTableController.FAST_FILTER + LyController.ID_SEP
                        + fastFilterField.getId(), fastFilterField.getName());
                item.setDialog(true);
                item.setController(_parent.getId());
                menuFastFilter.addItem(item);
            }

            menuFilter.addMenu(menuFastFilter);
        }
    } // end method addFastFilterItems

    /**
     * Add the choose columns tool to a toolbar
     * @param toolbar a toolbar
     */
    protected void addChooseColumnsTool(LyToolBar toolbar)
    {
        LyClassInfo   classInfo = ((LyTableController) _parent).getClassInfo();
        LyTool        tool;
        LyEnvironment environment = _parent.getEnvironment();
        String        displayString;
        String        id = null;

        // Choose columns
        if (classInfo.getFieldInfoCount() > 1)
        {
            displayString = environment.translate("LY_TOOL_CHOOSE_COLS");
            if ((displayString != null) && (displayString.trim().length() > 0))
            {
                // Choose columns
                id = LyTableController.DEMAND_CHOOSE_COLUMNS + LyController.ID_SEP
                    + _parent.getId();
                tool = new LyTool(id, "");
                tool.setTooltip("LY_ITEM_CHOOSE_COLS");
                tool.setImage("LY_TOOL_CHOOSE_COLS");
                tool.setEnabled(true);
                tool.setDialog(true);

                toolbar.addTool(tool);
            }
        }
    } // end method addChooseColumnsTool

    /**
     * Add choose column items to display menu
     * @param displayMenu the display menu
     */
    protected void addChooseColumnsItems(LyMenu displayMenu)
    {
        LyClassInfo   classInfo = ((LyTableController) _parent).getClassInfo();
        LyItem        item;
        LyEnvironment environment = _parent.getEnvironment();
        String        displayString;
        String        id = null, image = null;
        LyAction      actionItem = null;

        // Choose columns
        if (classInfo.getFieldInfoCount() > 1)
        {
            displayString = environment.translate("LY_ITEM_CHOOSE_COLS");
            if ((displayString != null) && (displayString.trim().length() > 0))
            {
                // Choose columns
                id = LyTableController.DEMAND_CHOOSE_COLUMNS;
                actionItem = (LyAction) _parent.getApplication().getInfo(id);
                image = ((LyAreaController) _parent).getMenuItemImage(actionItem);
                item = new LyItem(id, "LY_ITEM_CHOOSE_COLS");
                item.setImage(image);
                item.setEnabled(true);
                item.setDialog(true);
                item.setController(_parent.getId());
                displayMenu.addItem(item);
            }
        }
    } // end method addChooseColumnsItems

    /**
     * Add toggle button for display switch.
     */
    protected void addDisplayListTool(LyToolBar toolbar)
    {
        LySimpleTableController tableController = (LySimpleTableController) _parent;

        if (!tableController.hasDisplayListToggleButton())
            return;

        LyEnvironment environment = getParent().getEnvironment();
        String		  displayString = environment.translate("LY_TOOL_TABLE_DISPLAYLIST");

        if ((displayString != null)
                && (displayString.length() != 0)
                && !"_null".equals(displayString))
        {
            LyTool displayTool = new LyTool(LySimpleTableController.DEMAND_LIST_DISPLAY
                    + LyController.ID_SEP + _parent.getId(), "");

            displayTool.setStyle(LyToolBase.Style.TOGGLE);
            displayTool.setTooltip("LY_TOOL_TABLE_DISPLAYLIST");
            displayTool.setImage("LY_TOOL_TABLE_DISPLAYLIST");
            displayTool.setEnabled(true);

            toolbar.addTool(displayTool);
        }
    }

    /**
     * Builds "Fix column" sub-menu.
     * @param menuBar the menu bar
     * @return a created menu
     */
    public LyMenu updateFixColumnSubMenu(LyMenuBar menuBar)
    {

        if (menuBar == null)
            return null;

        LyViewManager viewManager = getViewManager();
        LyEnvironment environment = _parent.getEnvironment();

        if ((viewManager != null) && !viewManager.hasProperty("ALLOW_FIX_COLUMN"))
            return null;

        // FFT01073 : in compound areas the id of the menu was changed.
        LyMenu displayMenu = (LyMenu) menuBar.getComponentByLabel("LY_MENU_DISPLAY");
        String displayString;

        if (displayMenu == null)
            return null;

        LyClassInfo classInfo = ((LyTableController) _parent).getClassInfo();

        if (classInfo.getFieldInfoCount() <= 1)
            return null;

        LyMenu subMenu = (LyMenu) displayMenu.getComponent("_fixColumnMenu_" + _parent.getId());
        LyItem item;
        int    i, n;

        if (subMenu != null)
        {

            for (i = subMenu.getComponentCount() - 3; i >= 0; i--)
            {
                item = (LyItem) subMenu.getComponent(i);
                subMenu.removeItem(item);
            }

            item = (LyItem) subMenu.getComponent(LyTableController.DEMAND_UNFIX_COLUMN);
            item.setEnabled(false);
        }
        else
        {
            displayString = environment.translate("LY_ITEM_FIX_COLUMN");

            if ((displayString != null) && (displayString.trim().length() > 0))
            {
                subMenu = new LyMenu("LY_ITEM_FIX_COLUMN");
                subMenu.setId("_fixColumnMenu_" + _parent.getId());
                displayMenu.addMenu(subMenu);

                subMenu.addItem(null); // separator
                displayString = environment.translate("LY_ITEM_UNFIX_COLUMN");

                if ((displayString != null) && (displayString.trim().length() > 0))
                {
                    item = new LyItem(LyTableController.DEMAND_UNFIX_COLUMN,
                            "LY_ITEM_UNFIX_COLUMN");
                    item.setEnabled(false);
                    item.setController(_parent.getId());
                    subMenu.addItem(item);
                }
            }
        }

        if (subMenu == null)
            return null;

        LyTableController       tableController = (LyTableController) _parent;
        LyInfoList<LyFieldInfo> fields = tableController.getFields();
        LyFieldInfo             fieldInfo;

        for (i = 0, n = fields.getSize() - 1; i < n; i++)
        {
            fieldInfo = fields.getInfo(i);
            item = new LyItem(LyTableController.DEMAND_FIX_COLUMN + LyController.ID_SEP
                    + Integer.toString(i), fieldInfo.getAlias());
            item.setController(_parent.getId());
            subMenu.insertItem(item, i);
        }

        return subMenu;
    } // end method updateFixColumnSubMenu

    /**
     * Builds a list of all data files.
     *
     * @param  action The action corresponding to the button.
     * @param  prefix The prefix of the files.
     * @return a menu containing a creation item and the list of files.
     */
    protected LyMenu buildSelectFileMenu(LyAction action, String prefix)
    {
        // Create menu items
        LyMenu menu = updateSelectMenu(action, prefix);

        return menu;
    }

    /**
     * Updates the list of all data files.
     *
     * @param  prefix The prefix of the files.
     * @return an area containing the list of files.
     */
    public LyArea updateSelectFile(String prefix)
    {
        LyArea area = super.updateSelectFile(prefix);

        updateSelectMenu(null, prefix);

        return area;
    }

    /**
     * Updates a list of all data files.
     *
     * @param  action The action corresponding to the button.
     * @param  prefix The prefix of the files.
     * @return a menu containing a creation item and the list of files.
     */
    protected LyMenu updateSelectMenu(LyAction action, String prefix)
    {
        _LyActionMenu actionMenu = null;
        LyMenu        menu = null;

        if ((_actionMenus != null) && (_actionMenus.get(prefix) != null))
        {
            actionMenu = _actionMenus.get(prefix);
            menu = actionMenu._menu;

            if (action == null)
                action = actionMenu._action;
        }

        if (action == null)
            return null;

        LyApplication           application = _parent.getApplication();
        LyInfoList<LyChartInfo> chartInfos = application.getInfoList(LyChartInfo.class);
        int                     i, n = chartInfos.getSize();
        LyChartInfo             chart;
        LyItem                  item;
        LyClassInfo             classInfo = null;
        boolean                 first = true;

        // Add create and delete items
        LyItem createItem = new LyItem(action.getId(), "LY_ACTION_CREATE");

        createItem.setController(_parent.getId());

        if (action.hasMark(LyAction.MARK_DIALOG))
            createItem.setDialog(true);

        createItem.setShortcut(action.getShortcut());

        LyMenu actions = null;
        int    pos = -1, j, m;

        if (menu != null)
        {
            actions = (LyMenu) menu.getParent();

            if (actions != null)
            {

                for (j = 0, m = actions.getComponentCount(); j < m; j++)
                {

                    if (menu == actions.getComponent(j))
                    {
                        pos = j;
                        break;
                    }
                }

                actions.removeMenu(menu);
            }
        }

        menu = new LyMenu(action.getName());

        menu.addItem(createItem);

        if (_parent instanceof LyGetClassInfoInterface)
            classInfo = ((LyGetClassInfoInterface) _parent).getClassInfo();

        // Sort charts by name
        chartInfos = chartInfos.getInfosByName();

        LyItem deleteItem;

        for (i = 0; i < n; i++)
        {
            chart = chartInfos.getInfo(i);

            if (!chart.getId().startsWith(prefix + '_'))
                continue;

            if (classInfo != chart.getClassInfo())
                continue;

            item = new LyItem(_parent.getId() + "_selectFile" + LyController.ID_SEP + chart.getId(),
                    chart.getName());
            item.setController(_parent.getId());

            if (first)
            {
                deleteItem = new LyItem(_parent.getId() + "_selectFile"
                        + LySimpleTableController.DEMAND_DELETE_OBJECTINFOS + LyController.ID_SEP
                        + prefix, "LY_ACTION_DELETE");

                deleteItem.setController(_parent.getId());
                deleteItem.setDialog(true);

                menu.addItem(deleteItem);
                menu.addSep();
                first = false;
            }

            menu.addItem(item);
        }

        if (actions != null)
            actions.insertMenu(menu, pos);

        // Add menu to the list of action menus
        if (_actionMenus == null)
            _actionMenus = new Hashtable<String, _LyActionMenu>(1);

        actionMenu = new _LyActionMenu(action, menu);

        _actionMenus.put(prefix, actionMenu);

        return menu;
    } // end method updateSelectMenu

    class _LyActionMenu
    {
        protected LyAction _action = null;
        protected LyMenu   _menu = null;

        _LyActionMenu(LyAction action, LyMenu menu)
        {
            _action = action;
            _menu = menu;
        }
    }
} // end class LyTableBuilder
