/**
 * LyTreeBuilder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyTreeBuilder.java 28422 2010-11-03 12:03:38Z ehelmste $
 */

package leon.control.builder;

import leon.control.*;
import leon.misc.*;
import leon.view.*;


/**
 * This builder creates a view for the tree controller.
 *
 * @see LyTreeBaseController
 * @see LyTreeController
 */
public class LyTreeBuilder extends LyPanelBuilder
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyTreeBuilder.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Current tree component. */
	protected LyTree		   _tree;

	/**
	 * Default constructor.
	 *
	 * @param parent parent controller
	 */
	public LyTreeBuilder(LyController parent)
	{
		super(parent);
	}

	/**
	 * Builds the view component for the specified parent controller. The component may create the
	 * XML Tree or the internal view.
	 *
	 * @return the view Component
	 */
	public LyComponent build()
	{

		/* The tree view is an area which contains :
		        - A toolbar (actions)
		        - A titleBar (description)
		        - A tree component
		*/
		LyArea area = new LyArea();

		area.setId(_parent.getId());
		area.setController(_parent.getId());
		// Translation is made within the presentation layer
		area.setScrolling(LyArea.Scrolling.AUTO);

		// set content
		LyTree	tree = buildTree();
		
		area.setToolBar(buildToolBar());
	    area.setTopRightToolBar(buildRightToolBar());
		area.setTitleBar(buildTitleBar());
		area.addPanel(tree);

		LyConstraint	constraint = new LyConstraint();
		
		constraint.setFill(LyConstraint.Fill.BOTH);
		area.setConstraint(constraint);
		tree.setConstraint(constraint);

		_component = area;

		// Update default content
		completePanel(area);

		return area;
	} // end method build

	/**
	 * Builds the tree description.
	 *
	 * @return the tree component
	 */
	protected LyTree buildTree()
	{
		_tree = new LyTree(_parent.getId() + "_tree");

		LyEnvironment environment = _parent.getEnvironment();

		String		  selectionMode = environment.getResource("LY_SELECTION_MODE");

		if (selectionMode != null)
		{
			if (selectionMode.equals("MULTIPLE"))
				_tree.setSelectionMode(LyTree.SelectionMode.MULTIPLE);
			else if (selectionMode.equals("SINGLE"))
				_tree.setSelectionMode(LyTree.SelectionMode.SINGLE);
			else if (selectionMode.equals("NONE"))
				_tree.setSelectionMode(LyTree.SelectionMode.NONE);
			else
				environment.logWarning("Unexpected value for property LY_SELECTION_MODE : "
					+ selectionMode + " expected MULTIPLE, SINGLE or NONE.");
		}

		_tree.setController(_parent.getId());
		_tree.setIsContext(((LyTreeBaseController)_parent).getContextClient() != null);
		return _tree;
	}
	
	/**
	 * Overridden to disable display menu for a tree action.
	 * @param menuBar the menu bar to complete
	 */
	protected void completeDisplayMenu(LyMenuBar menuBar)
	{
	}

	/**
	 * Overridden to disable edition menu for a tree action.
	 * @param menuBar the menu bar to complete 
	 */
	protected void completeEditionMenu(LyMenuBar menuBar)
	{
	}

	/**
	 * Shortcut to the tree description.
	 *
	 * @return the component for the tree description
	 */
	public LyTree getTree()
	{
		return _tree;
	}
} // end class LyTreeBuilder
