/**
 * LyWaitBuilder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyWaitBuilder.java 35056 2012-03-26 17:12:43Z jlsaouli $
 */

package leon.control.builder;

import leon.control.*;
import leon.info.*;
import leon.view.*;


/**
 * This builder create the view for the wait controller.
 *
 * @see LyWaitController
 */
public class LyWaitBuilder extends LyAreaBuilder
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyWaitBuilder.java 35056 2012-03-26 17:12:43Z jlsaouli $";

	/** Default width for the icon of the waiting view. The unit is the number of pixels. */
	public static int		   ICON_DEFAULT_WIDTH = 20;
	/** Default height for the icon of the waiting view. The unit is the number of pixels. */
	public static int		   ICON_DEFAULT_HEIGHT = 20;

	/** the notice component for this view. */
	LyNotice				   _notice;

	/**
	 * Default constructor.
	 *
	 * @param parent parent controller
	 */
	public LyWaitBuilder(LyController parent)
	{
		super(parent);
	}

	/**
	 * Builds the view component for the specified parent controller. The component may create the
	 * XML Tree or the internal view.
	 *
	 * @return the view Component
	 */
	public LyComponent build()
	{
		/* The table view is an area which contains :
		        - A toolbar (actions)
		        - A titleBar (description)
		        - A table component
		*/
		LyArea area = new LyArea();

		area.setId(_parent.getId());
		area.setController(_parent.getId());
		area.setScrolling(LyArea.Scrolling.NO);
		area.setCommands(buildCommands());

		// Set the refresh rate of the window
		String 	refreshRate = (String)((LyWaitController)_parent).getParameter("_refreshRate");
		int    	refresh;

		try
		{
			refresh = Integer.parseInt(refreshRate);
		}
		catch (NumberFormatException e)
		{
			refresh = 2;
		}
		
		area.setRefresh(refresh);

		// set content
		area.addPanel(buildNotice());

		_component = area;
		return area;
	} // end method build

	/**
	 * Builds the notice description.
	 *
	 * @return the notice component
	 */
	protected LyNotice buildNotice()
	{
		_notice = new LyNotice(_parent.getId() + "_notice");

		_notice.setIcon(buildIcon());
		_notice.setLabel(buildLabel());
		_notice.setGauge(buildGauge());

		_notice.setNCols(((LyWaitController)_parent).getNCols());
		_notice.setNRows(((LyWaitController)_parent).getNRows());

		return _notice;
	}

	/**
	 * Builds the icon description.
	 *
	 * @return the icon component
	 */
	protected LyIcon buildIcon()
	{
		LyAction action = ((LyAreaController)_parent).getAction();
		String   image;

		if (action == null)
			image = "LY_WAITING_ICON";
		else
			image = _parent.getImage(action);
		LyIcon icon = new LyIcon(_parent.getName(), image);

		// set the default size for the icon
		icon.setHeight(ICON_DEFAULT_HEIGHT);
		icon.setWidth(ICON_DEFAULT_WIDTH);

		return icon;
	}

	/**
	 * Builds the label description.
	 *
	 * @return the label component
	 */
	protected LyLabel buildLabel()
	{
		LyLabel label = new LyLabel();

		label.setLabel(_parent.getName());
		label.setHAlign(LyLabel.HAlign.CENTER);

		return label;
	}

	/**
	 * Builds the gauge description.
	 *
	 * @return the gauge component
	 */
	protected LyGauge buildGauge()
	{
		LyGauge gauge = new LyGauge("_gauge");

		gauge.setNMin(((LyWaitController)_parent).getNMin());
		gauge.setNMax(((LyWaitController)_parent).getNMax());

		return gauge;
	}

	/**
	 * Builds the commands bar.
	 *
	 * @return the commands component
	 */
	protected LyCommands buildCommands()
	{
		LyAction action = ((LyWaitController)_parent).getAction();

		if (action.hasMark(LyAction.MARK_NO_COMMANDS))
			return null;

		LyCommands commands = new LyCommands();
		LyCommand  command;

		command = new LyCommand(LyWaitController.DEMAND_CANCEL, "LY_CANCEL");
		command.setAction(LyCommand.Action.CANCEL);
		commands.addCommand(command);

		return commands;
	}
} // end class LyWaitBuilder
