/**
  * LyWebServiceBuilder.java
  * Copyright (c) 2000-2009, Lyria-W4.
  * All rights reserved.
  */
  
package leon.control.builder;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import leon.control.LyController;
import leon.control.LyUserInputController;
import leon.control.LyWebServiceController;
import leon.info.LyNumberFieldInfo;
import leon.view.LyBoolean;
import leon.view.LyFieldItem;
import leon.view.LyFormField;
import leon.view.LyLink;
import leon.view.LyRelation;

/**
 * The default webservice builder
 * 
 *
 */
public class LyWebServiceBuilder extends LyUserInputBuilder
{
	/**
	 * File version
	 */
	public static final String __VERSION =
		"$Id: LyWebServiceBuilder.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * 
	 * @param parent
	 */
	public LyWebServiceBuilder(LyController parent)
	{
		super(parent);
	}

	/**
	 * Get the webservice controller
	 * @return the webservice controller
	 */
	public LyWebServiceController getWSController()
	{
		return ((LyWebServiceController)_parent);
	}
	
	/**
	 * Creates the form content.
	 */
	protected void buildFormContent()
	{
		LyUserInputController 	userInputController = (LyUserInputController)_parent;
		String 					targetNameSpace 	= (String)userInputController.getParameter("_targetNameSpace", false);
		String 					serviceName 		= (String)userInputController.getParameter("_serviceName", false);
		String 					portName 			= (String)userInputController.getParameter("_portName", false);
		String 					operationName 		= (String)userInputController.getParameter("_operationName", false);
		String 					wsdlUrl	= (String)userInputController.getParameter("_wsdlUrl", true);
		
		
		LyWebServiceController wsController = getWSController();
		
		wsController.setWsdlUrl(wsdlUrl);
		wsController.createWsdlDefinition(wsdlUrl);
		

		if(targetNameSpace==null)
		{
			LyRelation				comboNameSpaces  	= addRelation("_selectionNameSpaces",
					wsController.getEnvironment().translate("LY_ACTION_WEB_SERVICE_NAMESPACE"), false, true);
			List<String> 			lstNameSpaces 		= wsController.getListOfNameSpaces();
	
			if(lstNameSpaces.size() == 1)
				addLinksToRelation(wsController.getListOfNameSpaces(), comboNameSpaces, true);
			else
				addLinksToRelation(wsController.getListOfNameSpaces(), comboNameSpaces, false);
		}
		else
			wsController.setTargetNameSpace(targetNameSpace);
		
		if(serviceName == null)
		{	
			List<String> lstServices = wsController.getListOfServices();
			
			if(lstServices != null)
			{
				LyRelation				comboServices 		= addRelation("_selectionServices",
						wsController.getEnvironment().translate("LY_ACTION_WEB_SERVICE_SERVICE"), false, true);
				
				if(lstServices.size() == 1)
				{
					addLinksToRelation(lstServices, comboServices, false);
					wsController.selectService(lstServices.get(0));
				}
				else
				{
					addLinksToRelation(lstServices, comboServices, true);
					wsController.selectService(lstServices.get(0));
				}
			}
		}
		else
			wsController.selectService(serviceName);
		
		if( portName == null)
		{
			List<String>	lstPorts   = wsController.getListOfPorts();
		
			if(lstPorts != null)
			{
				LyRelation	comboPorts = addRelation("_selectionPorts",
						wsController.getEnvironment().translate("LY_ACTION_WEB_SERVICE_PORT"), false, true);
				if(wsController.getService() != null)
				{
					if(lstPorts.size() == 1)
					{
						addLinksToRelation(lstPorts, comboPorts, false);
						wsController.selectPort(lstPorts.get(0));
					}
					else
					{
						addLinksToRelation(lstPorts, comboPorts, false);
						wsController.selectPort(lstPorts.get(0));
					}
				}
				else
					addLinksToRelation(lstPorts, comboPorts, true);
			}	
		}
		else
		{
			wsController.selectPort(portName);
		}
		
		if(operationName == null)
		{
			List<String> 	lstOperation = wsController.getListOfOperations();
			if(lstOperation != null)
			{
				LyRelation		comboOperations 	= addRelation("_selectionOperation",
						wsController.getEnvironment().translate("LY_ACTION_WEB_SERVICE_OPERATION"), false, true);
	
			
				if(lstOperation.size() == 1)
				{
					addLinksToRelation(lstOperation, comboOperations, false);
					wsController.selectOperation(lstOperation.get(0));
					buildOperationAttributes();
				}
				else
				{
					addLinksToRelation(wsController.getListOfOperations(), comboOperations, true);
					wsController.selectOperation(lstOperation.get(0));
				}
			}
		}
	}
	
	

	

	
	/**
	 * Add links to a relation
	 * @param list the list of element to add
	 * @param relation the relation to complete
	 * @param blank true to insert a blank line at the beginning
	 */
	public void addLinksToRelation(List<String> list, LyRelation relation, boolean blank)
	{
		if(blank)
		{
			LyLink link		 = 	new LyLink("_blank", "");
			relation.addLink(link);
		}
		
		for (int i = 0; i < list.size(); i++)
		{
			LyLink link		 = 	new LyLink(list.get(i), list.get(i));
			relation.addLink(link);
		}
	}

	/**
	 * Remove all links from a relation
	 * @param relation the relation to empty
	 */
	public void removeLinksToRelation(LyRelation relation)
	{
		Vector<LyLink> listLyLink = new Vector<LyLink>();
		
		for (int i = 0; i < relation.getLinkCount(); i++) 
		{
			listLyLink.addElement(relation.getLink(i));
		}
		
		for (int i = 0; i < listLyLink.size(); i++) 
		{
			relation.removeLink(listLyLink.elementAt(i));
		}
		
	}
	
	/**
	 * Build the operation attributes
	 */
	public void buildOperationAttributes()
	{
		Map<String, String> listOfAttNameType 	= getWSController().getListOfOperationAttribute();
		Iterator<String> 	attNameType			= null;
		
		if(listOfAttNameType != null)
		{
			attNameType 		= listOfAttNameType.keySet().iterator();
		}
		
		String 				prefix 				= "parameter";
		
		for(int i = _form.getFormFieldCount(); i > 0 ; i--)
		{
			LyFormField formField = _form.getFormField(i);
			if(formField!= null 
					&& !formField.getId().equals("")
					&& formField.getId().startsWith(prefix))
				_form.removeFormField(i);
		}
		
		if(attNameType != null)
		{
			while(attNameType.hasNext())
			{
				String attName = attNameType.next();
				getWSController().addNewAttribute(attName);
				buildAttributeByType(attName, listOfAttNameType.get(attName), attName);
				
			}
		}
		_form.resetContent();

	}
	
	/**
	 * Build an attribute from a type
	 * @param attributeName the attribute name
	 * @param attributeType the attribute type
	 * @param label a label
	 * @return the built field item
	 */
	public LyFieldItem buildAttributeByType(String attributeName, String attributeType, String label)
	{
		String 	prefix	= "parameter";

		if(attributeType.equals("string"))
			return addText(prefix + attributeName, label, "", -1, null, false);
		else if(attributeType.equals("int"))
			return addNumber(prefix + attributeName, label, label, LyNumberFieldInfo.Type.INT, false);
		else if(attributeType.equals("float"))
			return addNumber(prefix + attributeName, label, label, LyNumberFieldInfo.Type.FLOAT, false);
		else if(attributeType.equals("double"))
			return addNumber(prefix + attributeName, label, label, LyNumberFieldInfo.Type.DOUBLE, false);
		else if(attributeType.equals("long"))
			return addNumber(prefix + attributeName, label, label, LyNumberFieldInfo.Type.LONG, false);
		else if(attributeType.equals("short"))
			return addNumber(prefix + attributeName, label, label, LyNumberFieldInfo.Type.SHORT, false);
		else if(attributeType.equals("boolean"))
			return addBoolean(prefix + attributeName, label, LyBoolean.Status.UNCHECKED, false);		
		else if(attributeType.equals("list"))
			return addRelation(prefix + attributeName, label, false, false);
		else
		{
			return addText(prefix + attributeName, label, label, 150, null, false);
		}
				
	}
	

	
}
