/**
 * LyArrayFieldConsultController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyArrayFieldConsultController.java 35905 2012-06-15 09:03:33Z nhamon $
 */

package leon.control.form;

import java.util.*;
import leon.control.*;
import leon.info.*;
import leon.view.*;


/**
 * Field controller for consultation of LyArrayFieldInfo.
 * @see LyConsultController
 */
public class LyArrayFieldConsultController extends LyFieldConsultController
{
	/** File version */
	public static final String __VERSION =
		"$Id: LyArrayFieldConsultController.java 35905 2012-06-15 09:03:33Z nhamon $";

	/** Sub fields controllers. */
	protected ArrayList<LyFieldController>		   _fieldControllers;

	/**
	 * Constructor. The controller automatically registers on the specified session.
	 * @param parent parent controller
	 * @param fieldInfo the array field handled by this controller
	 */
	public LyArrayFieldConsultController(LyFormController parent, LyArrayFieldInfo fieldInfo)
	{
		super(parent, fieldInfo);

		_fieldControllers = new ArrayList<LyFieldController>();
	}

	/**
	 * Get the field controller at the given index.
	 * @param index the index
	 * @return a field controller
	 */
	public LyFieldController getFieldController(int index)
	{
		LyFieldController fieldController;

		if ((index >= 0) && (index < _fieldControllers.size()))
		{
			fieldController = _fieldControllers.get(index);
			if (fieldController != null)
				return fieldController;
		}

		LyFieldInfo fieldInfo = ((LyArrayFieldInfo)_fieldInfo).getFieldInfo();
        boolean     forceConsult = ((_formController instanceof LySetController)
                && ((LySetController)_formController).isConsult(_fieldInfo));
        
		fieldController = _formController.fieldControllerFactory(fieldInfo, forceConsult);
		if (fieldController != null)
			_fieldControllers.add(index, fieldController);
		return fieldController;
	}

	/**
	 * Free this field controller.
	 */
	public void free()
	{
		// Free field controllers
		if (_fieldControllers != null)
		{
			int				  i, n = _fieldControllers.size();
			LyFieldController fieldController;

			for (i = 0; i < n; i++)
			{
				fieldController = _fieldControllers.get(i);
				if (fieldController != null)
					fieldController.free();
			}

			_fieldControllers = null;
		}

		super.free();
	}

	/**
	 * Update the value of a fieldInfo shown in the view.
	 * @param newValue the new value of the field.
	 * @param oldValue former value of the field,or null if unknown.
	 */
	public void updateValue(Object newValue, Object oldValue)
	{
		LyArrayFieldInfo  arrayField = (LyArrayFieldInfo)_fieldInfo;
		int				  i, maxCard = arrayField.getMaxCardinality();
		int				  nn = arrayField.getValueCount(newValue);
		int				  no = _fieldControllers.size();
		LyFieldInfo		  fieldInfo = arrayField.getFieldInfo();
		Object			  newItemValue, oldItemValue;
		LyFieldController fieldController;
		LyFieldItem		  fieldItem = getFieldItem();

		if (fieldItem instanceof LyForm)
		{
			for (i = 0; i < maxCard; i++)
			{
				newItemValue = arrayField.getValue(newValue, i);
				oldItemValue = arrayField.getValue(oldValue, i);
				fieldController = getFieldController(i);
				fieldController.updateValue(newItemValue, oldItemValue);
			}
		}
		else if (fieldItem instanceof LyTable)
		{
			LyTable table = (LyTable)fieldItem;
			LyRows  rows = table.getRows();
			LyRow   row;

			table.setIdle(true);

			for (i = 0; i < nn; i++)
			{
				newItemValue = arrayField.getValue(newValue, i);
				oldItemValue = arrayField.getValue(oldValue, i);
				if (!(fieldInfo instanceof LyStructFieldInfo)
						&& (fieldInfo.compare(newItemValue, oldItemValue) == 0))
					continue;

				row = rows.getRow(i);
				if (row == null)
				{
					row = buildRow(fieldInfo, i);
					rows.addRow(row);
				}

				fieldController = getFieldController(i);
				fieldController.updateValue(newItemValue, oldItemValue);
			}
			for (i = no - 1; i >= nn; i--)
			{
				rows.removeRow(i);
				fieldController = getFieldController(i);
				_fieldControllers.remove(i);
				fieldController.free();
			}

			table.setIdle(false);
		}
		else
			super.updateValue(newValue, oldValue);
	} // end method updateValue

	/**
	 * Build a new row to edit the given field within a table.
	 * @param  fieldInfo Field to edit.
	 * @param  index     Index of the new row.
	 * @return the row
	 */
	protected LyRow buildRow(LyFieldInfo fieldInfo, int index)
	{
		LyRow row = new LyRow();

		row.setId(fieldInfo.getId() + index);

		LyFieldController fieldController = getFieldController(index);
		LyFormField		  formField;

		formField = fieldController.getFormField();
		formField.setId(formField.getId() + index);
		formField.setEditable(false);
		formField.setCaption(null);
		row.addFormField(formField);

		return row;
	}
} // end class LyArrayFieldConsultController
