/**
 * LyArrayFieldSetController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyArrayFieldSetController.java 41501 2012-11-12 14:58:14Z smaupin $
 */

package leon.control.form;

import java.util.*;

import leon.view.setinterface.LySetTableInterface;

import leon.app.*;
import leon.control.*;
import leon.control.form.builder.*;
import leon.data.*;
import leon.info.*;
import leon.info.misc.*;
import leon.misc.*;
import leon.view.*;
import leon.view.event.*;
import leon.view.setinterface.*;


/**
 * Field controller for modification of LyArrayFieldInfo.
 * @see LySetController
 */
public class LyArrayFieldSetController extends LyFieldSetController
	implements LyFormResultInterface,
		LyAdvancedWorkSpaceListener
{
	/** File version */
	public static final String     __VERSION =
		"$Id: LyArrayFieldSetController.java 41501 2012-11-12 14:58:14Z smaupin $";

	/** Name of the tranverse tool for moving up items in table. */
	public static final String     MOVE_UP_TOOL = "Up_Tool";

	/** Name of the tranverse tool for moving down items in table. */
	public static final String     MOVE_DOWN_TOOL = "Down_Tool";

	/** Sub fields controllers. */
	protected ArrayList<LyFieldSetController>			   _fieldSetControllers;

	/** Controller for creation. */
	protected LyFieldSetController _createFieldController;

	/**
	 * Current list of objects in the inner table (when any) we are listening to. This make sense
	 * only for Array of structs
	 */
	protected LyWorkSpace		   _registeredWorkSpace;

	/** Index of the last selected field. */
	protected int				   _rowIndex;

	/**
	 * Constructor.
	 * @param parent the parent set controller
	 * @param fieldInfo array field handled by this controller
	 */
	public LyArrayFieldSetController(LySetController parent, LyArrayFieldInfo fieldInfo)
	{
		super(parent, fieldInfo);

		_fieldSetControllers = new ArrayList<LyFieldSetController>();
		_createFieldController = null;
		_registeredWorkSpace = null;
	}

	/**
	 * Free this field controller.
	 */
	public void free()
	{
		if (_registeredWorkSpace != null)
		{
			_registeredWorkSpace.removeListener(this);
			_registeredWorkSpace = null;
		}

		// Free field controllers
		if (_fieldSetControllers != null)
		{
			int				  i, n = _fieldSetControllers.size();
			LyFieldController fieldController;

			for (i = 0; i < n; i++)
			{
				fieldController = _fieldSetControllers.get(i);
				if (fieldController != null)
					fieldController.free();
			}

			_fieldSetControllers = null;
		}
		if (_createFieldController != null)
		{
			_createFieldController.free();
			_createFieldController = null;
		}

		super.free();
	} // end method free

	/**
	 * Clear view cache.
	 */
	public void clearViewCache()
	{
		// Clear field controllers
		if (_fieldSetControllers != null)
		{
			int				  i, n = _fieldSetControllers.size();
			LyFieldController fieldController;

			for (i = 0; i < n; i++)
			{
				fieldController = _fieldSetControllers.get(i);
				if (fieldController != null)
					fieldController.clearViewCache();
			}
		}
		if (_createFieldController != null)
			_createFieldController.clearViewCache();

		super.clearViewCache();
	}

	/**
	 * @see leon.control.form.LyFieldController#getFieldController(int)
	 */
	public LyFieldController getFieldController(int index)
	{
		LyFieldSetController fieldController = null;

		if ((index < 0) && (_createFieldController != null))
			return _createFieldController;
		else if ((index >= 0) && (index < _fieldSetControllers.size()))
		{
			fieldController = _fieldSetControllers.get(index);
			if (fieldController != null)
				return fieldController;
		}

		LyFieldInfo subFieldInfo = ((LyArrayFieldInfo)_fieldInfo).getFieldInfo();
        boolean     forceConsult = ((LySetController)_formController).isConsult(_fieldInfo);

        if (subFieldInfo != null)
            fieldController = (LyFieldSetController)_formController.fieldControllerFactory(subFieldInfo, forceConsult);

		if (fieldController != null)
		{
			fieldController._parentFieldController = this;

			if (index >= 0)
			{
				_fieldSetControllers.add(index, fieldController);
				fieldController.setInternal(true);
			}
			else
				_createFieldController = fieldController;
		}

		return fieldController;
	} // end method getFieldController

	/**
	 * Invoked when an action occurs. At this level, the implementation manages<BR>
	 * DEMAND_VALIDATE demands. (demand.getType = LyActionDemand.TYPE_ACTION AND demand.getSource()
	 * = LyController.DEMAND_VALIDATE)<BR>
	 * DEMAND_APPLY demands. (demand.getType = LyActionDemand.TYPE_ACTION AND demand.getSource() =
	 * LyController.DEMAND_APPLY)<BR>
	 * Default action (double click)<BR>
	 * Default action (double click)<BR>
	 * VALUE_CHANGED demands. (demand.getType = LyActionDemand.TYPE_VALUE_CHANGED
	 * @param demand the action demand
	 * @see   LyController#demandPerformed
	 */
	public void actionDemand(LyActionDemand demand)
	{
		if (isEditable()
			&& isEnabled())
		{
			if (demand.getType() == LyActionDemand.TYPE_ACTION)
			{
				// Transverse action ON ARRAY?
				String   sourceId = demand.getSource().intern();
				String[] parameters = LyString.stringToList(sourceId, ID_SEP);

				if (parameters.length > 1)
				{
					if (parameters[1].equals(LyFieldSetController.STANDARD_ARRAY_TOOL))
						arrayToolSelected(true);
					else if (parameters[1].equals(MOVE_UP_TOOL))
						moveSelection(true);
					else if (parameters[1].equals(MOVE_DOWN_TOOL))
						moveSelection(false);
				}
			}
			else if (demand.getType() == LyActionDemand.TYPE_STATUS)
			{
				// May be obsolete for all but deletion on status item : see TYPE_ACTION demand
				//compute the index of row with the value of row
				String valueDemand = demand.getValue();
				int    sepDeb = valueDemand.indexOf("__LyArrayNumberRow__"); //the index is before the text __LyArrayNumberRow__
				int    index = -1;

				if (sepDeb > 0)
				{
					String indexString = valueDemand.substring(0, sepDeb);

					try
					{
						Integer indexObject = new Integer(indexString);

						index = indexObject.intValue();
					}
					catch (NumberFormatException e)
					{
						index = -1;
					}
				}
				//int index = demand.getIndex(0);
				LyValue			  value;
				int				  i, n = _fieldSetControllers.size();
				Vector<Object>			  values = new Vector<Object>(n + 1);
				LyFieldController fieldController;

				if (index >= n)
					return; // Click on empty line
				for (i = 0; i < n; i++)
				{
					if (i == index)
						continue;
					fieldController = getFieldController(i);
					value = ((LyFieldSetController)fieldController).getFieldValue(null);
					if ((value == null) || value.isNull())
						continue;
					values.add(value.getValue());
				}

				if (index < n)
				{
					fieldController = getFieldController(index);
					if (fieldController instanceof LyStructFieldSetController)
						((LyStructFieldSetController)fieldController).setStructObject(null);
				}

				if (values.size() == 0)
					value = _fieldInfo.buildFieldValue(null);
				else
					value = _fieldInfo.buildFieldValue(values.toArray());

				updateValue(value.getValue(), null);
				valueChanged();

				// Update the last modification time.
				_setController.setLastModified(new Date().getTime());

				return;
			}
		}

		super.actionDemand(demand);
	} // end method actionDemand

	/**
	 * Internal method to get the degfault value for a fieldInfo without usage of the behavior. This
	 * method is used to get the default value of the fieldInfo of the array fieldInfo. We can't
	 * used the generic method of the LyFieldSetController because the current object of the
	 * LySetController has not the fieldInfo in its classInfo.
	 * @param  fieldInfo The fieldInfo
	 * @return The default value for the specified classInfo.
	 */
	protected Object getDefaultValue(LyFieldInfo fieldInfo)
	{
		LySession session = getSession();
		Object    defaultValue = fieldInfo.getDefaultValue();
		LyValue   fieldValue;

		if ((defaultValue instanceof String)
				&& (((String)defaultValue).charAt(0) == '$'))
		{
			defaultValue = session.getValue(defaultValue);

			if ((defaultValue != null)
					&& fieldInfo.isMultiple())
			{
				Object[] tableValue = (Object[])java.lang.reflect.Array.newInstance(defaultValue
							.getClass(), 1);

				tableValue[0] = defaultValue;
				defaultValue = tableValue;
			}
		}

		// Build value from default value
		if (defaultValue != null)
		{
			fieldValue = fieldInfo.buildFieldValue(defaultValue);
			return fieldValue.getValue();
		}
		return null;
	} // end method getDefaultValue

	/**
	 * Internal method used when the tool for adding values in the array is selected.
	 * @param onClick Indicates the origin of the selection : true for a mouse click, false for an
	 *                internal origin.
	 */
	protected void arrayToolSelected(boolean onClick)
	{
		// Got it. Lets update the internal table.
		// When nothing is selected, create a new entry,
		// Wen a line is selected, update selected entry
		LyTable table = findInnerTable();

		if (table == null)
			return;

		LyRows rows = table.getRows();

		// Is a row selected ?
		LyRow  row;
		int    i = 0;
		int    indexSelection = -1, n = rows.getChildCount();

		while ((indexSelection < 0) && (i < n))
		{
			row = rows.getRow(i);
			if (row.isSelected())
				indexSelection = i;
			else
				i++;
		}

		// Build the values for the array.
		LyFieldSetController fieldController = (LyFieldSetController)getFieldController(-1);
		LyUnit				 unit = fieldController.getUnit();
		short				 status = LyFormInterface.STATUS_OK;
		LyValue				 newValue = null;

		// We must check if a value has changed in the controller.
		boolean				 modified = true;
		Object				 oldValue, currentValue;
		LyValue				 value;
		LyFieldInfo			 fieldInfo = fieldController.getFieldInfo();

		if (!onClick)
		{
			if (indexSelection == -1)
			{
				// Get the value from the user form
				value = fieldController.getFieldValue();
				currentValue = ((value == null) ? null : value.getValue());

				// Get the default (opening) value
				oldValue = getDefaultValue(fieldInfo);

				modified = !fieldInfo.equals(oldValue, currentValue);
			}
			else
			{
				// Get the value from the user form
				value = fieldController.getFieldValue();
				currentValue = ((value == null) ? null : value.getValue());

				// Get the default (opening) value
				LyFieldSetController ctrl = (LyFieldSetController)getFieldController(
						indexSelection);

				value = ctrl.getFieldValue();
				oldValue = ((value == null) ? null : value.getValue());

				modified = !fieldInfo.equals(oldValue, currentValue);
			}
		}
		if (modified)
			newValue = fieldController.getFieldValue(unit);

		// Do we need to validate an internal object in the edition wizard ?
		LySetController internalController = null;

		if ((fieldController instanceof LyStructInArraySetController)
				&& (fieldController.getAreaController() instanceof LySetController))
			internalController = (LySetController)fieldController.getAreaController();

		if (internalController != null)
		{
			// Check that we have no pending values in the GUI for this controller
			internalController.updateViewContent();

			// Validate the wizard controller : the subcontext is commited and
			status = internalController.validate(false);

			// Is everything ok so far ?
			if (status >= LyFormInterface.STATUS_OK)
				setState(status);
			if (status != LyFormInterface.STATUS_OK)
				return;

			// Try to get it again after creation : useful for creation of Structs
			if ((newValue == null) || newValue.isNull())
				newValue = fieldController.getFieldValue(unit);
		}

		if ((newValue == null) || newValue.isNull())
			return;

		// Existing values
		n = _fieldSetControllers.size();

		Vector<Object> values = new Vector<Object>(n + 1);

		for (i = 0; i < n; i++)
		{
			fieldController = (LyFieldSetController)getFieldController(i);
			if (indexSelection == i)
				value = newValue; // Set a value in the list
			else
				value = fieldController.getFieldValue(null);
			if ((value == null) || value.isNull())
				continue;
			values.add(value.getValue());
		}

		if (indexSelection < 0)
			values.add(newValue.getValue()); // Adding a value at the end

		value = _fieldInfo.buildFieldValue(values.toArray());

		// In the case of a struct in this array, set a new object on the top creation
		// form
		fieldController = (LyFieldSetController)getFieldController(-1);

		if (indexSelection < 0)
		{
			if ((internalController != null) && internalController.isCreation())
			{
				LyValue curValue = fieldController.getFieldValue(unit);
				// Do not delete the value when it is used elsewhere !
				boolean freeCurrent = (!curValue.equals(newValue));
				// Clone the object we have added below in the form on the top.
				internalController.resetFromObject(null, freeCurrent, true);
			}
			else if (fieldController instanceof LyStructFieldSetController)
				((LyStructFieldSetController)fieldController).resetFromValue(null, false, true);
		}
		updateValue(value.getValue(), null);

		fieldInfo = fieldController.getFieldInfo();

		// For a struct fieldInfo, we must link operations for
		// undo/redo management.
		boolean   linkOperation = (fieldInfo instanceof LyStructFieldInfo);
		LyContext context = _setController.getControllerContext();

		if ((context != null) && linkOperation)
			context.setLinkUndo(true);
		valueChanged();
		if ((context != null) && linkOperation)
			context.setLinkUndo(false);

		// Update the last modification time.
		_setController.setLastModified(new Date().getTime());
	} // end method arrayToolSelected

	/**
	 * Invoked when an selection notification is sent to this listener.
	 * <P>This method is called by demandPerformed and may be overriden in subclasses.</P>
	 * <P>Default implementation makes nothing.</P>
	 * @param demand demand parameters sent by the view component.
	 * @see   #demandPerformed
	 */
	public void selectionDemand(LySelectionDemand demand)
	{
		boolean selected = false;
		int     selectionType = demand.getType();

		// a row is selected in the table : refresh the top component with the value
		//compute the index of row with the value of row
		String  valueDemand = demand.getSelectionId();

		if (valueDemand == null)
		{
			updateTools(-1);
			return;
		}

		int sepDeb = valueDemand.indexOf("__LyArrayNumberRow__"); //the index is before the text __LyArrayNumberRow__
		int rowIndex = -1;

		if (sepDeb > 0)
		{
			String indexString = valueDemand.substring(0, sepDeb);

			try
			{
				Integer indexObject = new Integer(indexString);

				rowIndex = indexObject.intValue();
			}
			catch (NumberFormatException e)
			{
				rowIndex = -1;
			}
		}
		//int rowIndex = demand.getIndex(0);
		LyTable table = findInnerTable();
		LyRows  rows = table.getRows();

		// Be sure its a selection
		if (selectionType == LySelectionDemand.TYPE_SELECT)
		{
			if (rowIndex < _fieldSetControllers.size())
				selected = true; //
			else
				rowIndex = -1; // Not selected or selected a dummy blank line
		}
		else
			rowIndex = -1; // Not selected

		// Just check if the row exists or deselection
		if ((!selected)
				|| ((_fieldSetControllers != null) && (_fieldSetControllers.size() >= rowIndex)))
		{
			LyFieldController topFieldController = getFieldController(-1);

			// If selected we may modify but notr create new objects
			if (topFieldController.getAreaController() instanceof LyCreateOrSetController)
				((LyCreateOrSetController)topFieldController.getAreaController()).setCreation(
					!selected);
			else if (topFieldController instanceof LyStructFieldSetController)
				((LyStructFieldSetController)topFieldController).setCreation(!selected);

			if (selected)
			{
				// Update the top value
				LyFieldController fieldController = getFieldController(rowIndex);
				LyValue			  value = ((LyFieldSetController)fieldController).getFieldValue();

				topFieldController.resetFromValue(value.getValue(), true, true);
			}
			else
				topFieldController.resetFromValue(null, true, true); // Clear any previous selection

			// Force selection in view
            LyRow row;
            LySetTableInterface tableInterface = null;
            if (table.getInternalView() instanceof LySetTableInterface)
                tableInterface = (LySetTableInterface) table.getInternalView();
            for (int i = 0; i < rows.getRowCount(); i++)
            {
                boolean isRowSelected = (i == rowIndex);
                row = rows.getRow(i);
                row.setSelected(isRowSelected);
                if (tableInterface != null)
                {
                    tableInterface.selectRow(i, isRowSelected);
                }
            }

            if (selected)
                setEnableInnerTool(isEditable());
            else
                checkInnerTools();

			// Update the last modification time.
			_setController.setLastModified(new Date().getTime());
		}

		updateTools(rowIndex);
	} // end method selectionDemand

	/**
	 * Extract current value for the specified fieldInfo in the form.
	 * @param  unit      Unit of the value.
	 * @return current value, or null if not found or field is disabled.
	 */
	public LyValue getFieldValue(LyUnit unit)
	{
		Vector<Object>	  values = null;
		LyValue			  value;
		int				  i, n = _fieldSetControllers.size();
		LyFieldController fieldController;

		for (i = 0; i < n; i++)
		{
			fieldController = getFieldController(i);
			value = ((LyFieldSetController)fieldController).getFieldValue();
			if ((value == null) || value.isNull())
				continue;
			if (values == null)
				values = new Vector<Object>(n);
			values.add(value.getValue());
		}

		if (values == null)
			return _fieldInfo.buildFieldValue(null);
		else
			return _fieldInfo.buildFieldValue(values.toArray());
	}

	/**
	 * Extract current value for the specified fieldInfo in the form.
	 * @return current value, or null if not found or field is disabled.
	 */
	public LyValue getFieldValue()
	{
		// Existing values
		LyValue			  value;
		int				  i, n;
		LyFieldController fieldController;

		n = _fieldSetControllers.size();

		Vector<Object> values = new Vector<Object>(n);

		for (i = 0; i < n; i++)
		{
			fieldController = getFieldController(i);
			value = ((LyFieldSetController)fieldController).getFieldValue(null);

			// A null value may occur only when the number of item max in array is < MAX_CARDINALITY
			// Then, instead of a line, the max number items are shown immediately in the form,
			// without a table
			if ((value != null) && (!value.isNull()))
				values.add(value.getValue());
		}

		value = _fieldInfo.buildFieldValue(values.toArray());
		return value;
	}

	/**
	 * Internal use : update the value of an array fieldInfo in the form. <B>Application should use
	 * setFieldValue instead.</B> This method is called from updateFieldItem. (Specialization)
	 * @param newValue    the new value of the field. This value may be null, a LyValue instance or the
	 *                 internal value associated to thge fieldInfo. See LyValue.getValue()
	 * @param oldValue the latest value of the field. This value may be null, a LyValue instance or
	 *                 the internal value associated to thge fieldInfo. See LyValue.getValue()
	 * @see   #setFieldValue
	 * @see   #updateFieldItem
	 */
	protected void updateValue(Object newValue, Object oldValue)
	{
        if (_setController.isConsult(_fieldInfo))
        {
            super.updateValue(newValue, oldValue);
            return;
        }

		if ((newValue != null)
				&& (oldValue != null)
				&& _fieldInfo.equals(newValue, oldValue))
			return;

		LyArrayFieldInfo  arrayField = (LyArrayFieldInfo)_fieldInfo;
		int				  i, maxCard = arrayField.getMaxCardinality();
		LyFieldItem		  fieldItem = getFieldItem();
		int				  nn = arrayField.getValueCount(newValue);
		int				  no = _fieldSetControllers.size();
		LyFieldInfo		  fieldInfo = arrayField.getFieldInfo();
		Object			  newItemValue, oldItemValue;
		boolean			  editable = _setController.isEditable(arrayField);
		LyFieldController fieldController;
		LyValue			  value;

		if (_internal)
		{
			// Do locally, no GUI
			for (i = 0; i < nn; i++)
			{
				newItemValue = arrayField.getValue(newValue, i);
				fieldController = getFieldController(i);

				// Check inside the controller (may be alreay up to date)
				oldItemValue = fieldController.getFieldValue().getValue();
				if (fieldInfo.compare(newItemValue, oldItemValue) == 0)
					continue;
				fieldController.updateValue(newItemValue, oldItemValue);
			}
			// remove additional controllers (not used any more)
			for (i = no - 1; i >= nn; i--)
			{
				fieldController = getFieldController(i);
				_fieldSetControllers.remove(i);
				fieldController.free();
			}
			return;
		}

		if (findInnerTable() != null)
			fieldItem = findInnerTable();

		// Ready for update
		if (fieldItem instanceof LyForm)
		{
			for (i = 0; i < maxCard; i++)
			{
				newItemValue = arrayField.getValue(newValue, i);
				oldItemValue = arrayField.getValue(oldValue, i);
				fieldController = getFieldController(i);
				fieldController.updateValue(newItemValue, oldItemValue);
				((LyFieldSetController)fieldController).setEditable(editable);
			}
		}
		else if (fieldItem instanceof LyTable)
		{
			LyTable table = (LyTable)fieldItem;
			LyRows  rows = table.getRows();
			LyRow   row;
			boolean resetFromValue;
			Object  defaultValue = null;

			table.setIdle(true);

			for (i = 0; i < nn; i++)
			{
				newItemValue = arrayField.getValue(newValue, i);
				oldItemValue = arrayField.getValue(oldValue, i);
//                              if (!(fieldInfo instanceof LyStructFieldInfo))
				if (fieldInfo.compare(newItemValue, oldItemValue) == 0)
					continue;

				fieldController = getFieldController(i);
				if (fieldController instanceof LyStructFieldSetController)
					fieldController.updateValue(newItemValue, oldItemValue);

				// Check inside the controller (may be already up to date)
				value = fieldController.getFieldValue();
				row = rows.getRow(i);
				resetFromValue = true;

				if (value != null)
				{
					oldItemValue = value.getValue();
					if (fieldInfo.compare(newItemValue, oldItemValue) == 0)
						if (newItemValue instanceof LyObjectId)
							resetFromValue = false;
						else
							continue;
				}

				// Be sure the row has a remove item tool
				if ((row != null) && (row.getImage() == null))
				{
					rows.removeRow(i);
					row = null;
				}

				if ((row == null) || row.getId().equals(fieldInfo.getId())) // Row for creation
				{
					row = buildRow(fieldInfo, i, true);
					rows.insertRow(row, i);
				}

				if (resetFromValue)
					fieldController.resetFromValue(newItemValue, true, true);
				updateRow(table, i, fieldInfo, newItemValue);
			}

			// remove additional controllers (not used any more)
			boolean reset = false;

			for (i = no - 1; i >= nn; i--)
			{
				row = rows.getRow(i);
				if (row.isSelected())
					reset = true;
				rows.removeRow(i);
				fieldController = getFieldController(i);
				_fieldSetControllers.remove(i);
				fieldController.free();
			}

			// Remove blank lines if any left
			for (i = rows.getRowCount() - 1; i >= nn; i--)
			{
				row = rows.getRow(i);
				if (row.isSelected())
					reset = true;
				rows.removeRow(i);
			}

			// We add a blank row at the end for deselection
			if ((nn > 0) && ((maxCard < 0) || (nn < maxCard)))
			{
				row = buildRow(fieldInfo, rows.getRowCount(), false);
				rows.insertRow(row, nn);
			}

			if (reset)
			{
				LyFieldController topFieldController = getFieldController(-1);

				topFieldController.resetFromValue(null, true, true); // Clear any previous selection
			}

			// Set the default value of the field item above
            if (fieldInfo != null)
            {
                defaultValue = fieldInfo.getDefaultValue();
    			fieldController = getFieldController(-1);
    			fieldController.updateFieldItem(defaultValue, null, false);
            }

			checkInnerTools();

			// Register on inner objects when it makes sense.
//                      if (_registeredWorkSpace != null)
//                      {
//                              _registeredWorkSpace.removeListener(this);
//                              _registeredWorkSpace = null;
//                      }
			if ((newValue instanceof LyObjectId[])
					&& (((LyObjectId[])newValue).length > 0))
			{
				LyObjectId[] ids = (LyObjectId[])newValue;
				int			 i1, n1 = ids.length;
				LyObject     obj;

				if (_registeredWorkSpace == null)
				{
					_registeredWorkSpace = new LySimpleWorkSpace(getSession(), true);
					_registeredWorkSpace.addListener(this);
				}
//                              else
//                                      _registeredWorkSpace.removeListener(this);
				Vector<LyObject>  objects = _registeredWorkSpace.getObjects();

				for (i1 = 0; i1 < n1; i1++)
				{
					obj = ids[i1].getLogicalObject();
					if (obj != null)
					{
						if ((objects != null) && (objects.contains(obj)))
							objects.remove(obj);
						else
							((LySimpleWorkSpace)_registeredWorkSpace).addObject(obj);
					}
				}
				if (objects != null)
				{
					for (i = 0; i < objects.size(); i++)
					{
						obj = objects.elementAt(i);
						((LySimpleWorkSpace)_registeredWorkSpace).removeObject(obj);
					}
				}
			}

			table.setIdle(false);
			updateTools(-1);
		}
	} // end method updateValue

	/**
	 * Check if the inner tool is enabled or not.
	 */
	protected void checkInnerTools()
	{
		// Check if max is reached
		LyArrayFieldInfo arrayField = (LyArrayFieldInfo)_fieldInfo;
		int				 maxCard = arrayField.getMaxCardinality();
		int				 nn = _fieldSetControllers.size();

		if (_formField.isEnabled()
				&& _formField.isEditable()
				&& _setController.isEditable()
				&& ((maxCard < 0) || (nn < maxCard)))
			// We can add new lines in the widget.
			// enable the update tool.
			setEnableInnerTool(true);
		else
			// array is full or not editable :
			// disable the update tool.
			setEnableInnerTool(false);

		if (_createFieldController != null)
		{
			_createFieldController.setEditable(_formField.isEditable());
			_createFieldController.setEnabled(_formField.isEnabled());
		}
	}

	/**
	 * Change the enable/disable status of the update tool.
	 * @param status new status of the tool
	 */
	public void setEnableInnerTool(boolean status)
	{
		String	    toolId = "__update_array_value" + LyController.ID_SEP
			+ LyFieldSetController.STANDARD_ARRAY_TOOL + LyController.ID_SEP
			+ _fieldInfo.getId();

		LyFieldItem fieldItem = getFieldItem();

		if (fieldItem instanceof LyForm)
		{
			// Check if a table is nested in this form.
			LyComponent tableField = ((LyForm)fieldItem).getComponent(toolId);

			if (tableField instanceof LyTool)
				((LyTool)tableField).setEnabled(status);
		}
	}

	/**
	 * Build the update tool.
	 * @param  builder The builder asking to build the tool.
	 * @return The tool
	 */
	protected LyTool buildInnerTool(LyFieldSetBuilder builder)
	{
		LyFormController  formController = getFormController();
		LyClassInfo       classInfo = formController.getClassInfo();
		LyAction	      action = getApplication().getActionWithId(LyAction.ID_UPDATE_ARRAY_VALUE, classInfo);
		LyEnvironment     environment = getEnvironment();
	    LyTool            tool;

		tool = builder.buildTool(action, true,
				LyFieldSetController.STANDARD_ARRAY_TOOL
				+ LyController.ID_SEP + _fieldInfo.getId(),
				environment.getImageSizeInfo(LyFieldSetBuilder.DEFAULT_TRANSVERSE_ICON_WIDTH,
					LyFieldSetBuilder.DEFAULT_TRANSVERSE_ICON_WIDTH));
		return tool;
	}

	/**
	 * Find (eventually) an inner table in the Array fieldItem.
	 * @return the table if any, or null if it could not be found.
	 */
	protected LyTable findInnerTable()
	{
		LyFieldItem fieldItem = getFieldItem();

		if (fieldItem instanceof LyForm)
		{
			// Check if a table is nested in this form.
			LyComponent tableField = ((LyForm)fieldItem).getComponent("_table"
					+ LyFormController.FORM_FIELD_SEP + _fieldInfo.getId());

			if ((tableField instanceof LyFormField)
					&& (((LyFormField)tableField).getFieldItem() instanceof LyTable))
			{
				// Nested table : this is the inner table we may update
				fieldItem = ((LyFormField)tableField).getFieldItem();

				return ((LyTable)fieldItem);
			}
		}
		return null;
	}

	/**
	 * Update a row from the specified value LyTable table current table.
	 * @param table the table component
	 * @param i     index of the row
	 * @param fieldInfo the fieldInfo to update
	 * @param value new value
	 */
	protected void updateRow(LyTable table, int i, LyFieldInfo fieldInfo, Object value)
	{
		LyRows		  rows = table.getRows();
		LyRow		  row = rows.getRow(i);
		LyCell		  cell;
		LyEnvironment environment = getEnvironment();
        int           nc = row.getChildCount();

		if ((nc > 0) && (row.getComponent(nc - 1) instanceof LyCell))
		{
			if ((fieldInfo instanceof LyStructFieldInfo) && (value instanceof LyObjectId))
			{
				LyHeaders headers = table.getHeaders();

				if ((headers == null) || (headers.getChildCount() == 1))
					headers = new LyHeaders();
				else
					headers = null;

				LyObject structObj = ((LyObjectId)value).getLogicalObject();
				LyRow    newRow = new LyRow();
				String   image = row.getImage();

				if (environment.testResource("LY_ERROR_STATE_ICON"))
				{
					image = getImageFromPrefix("LY_REMOVE_VALUE", DEFAULT_IMAGE_FORMAT, 12);

					if ((structObj != null)
							&& (structObj.getErrorLevel() != LyObject.LEVEL_OK)) // May be removed : do we want to show also OK icons ?
						image = getViewManager().buildErrorImage(image, structObj.getErrorLevel());
				}

				newRow.setColor(row.getColor());
				newRow.setImage(image);
				newRow.setTooltip(row.getTooltip());
				newRow.setSelected(row.isSelected());
				newRow.setId(row.getId());

				LyCell			  structCell = null;
				LyFieldInfo		  structFieldInfo;
				LyHeader		  header;
				LyInfoLabel		  infoLabel;
				String			  name;
				Object			  structVal;
				LyFieldController fieldController;
				String			  label;

				if (structObj != null)
				{
					LyClassInfo structClassInfo = structObj.getClassInfo();

					// If there is a dynamic label we just add one cell
					// with the value.
					structFieldInfo = ((LyArrayFieldInfo)_fieldInfo).getFieldInfo();
					infoLabel = getInfoLabel(structClassInfo, LyInfoLabel.Type.NAME);
					if (infoLabel != null)
					{
						label = infoLabel.getString(structObj);
						if (label == null)
							label = "";
						LyObject object = getObject();

						if (headers != null)
						{
							header = new LyHeader();
							infoLabel = getInfoLabel(structFieldInfo, LyInfoLabel.Type.NAME);

							if ((infoLabel != null) && (object != null))
								name = infoLabel.getString(object);
							else
								name = structFieldInfo.getName();

							header.setLabel(name);
							headers.addHeader(header);
						}
						structCell = new LyCell();

						structCell.setLabel(label);
						newRow.addCell(structCell);
					}
					else
					{
						LyUnits  units = structFieldInfo.getUnits();
						LyFormat format = null;

						int		 n = ((units == null) ? 0 : units.getUnitCount());

						for (int k = 0; k < n; k++)
						{
							LyUnit   unit = units.getUnit(k);
							LyFormat tempFormat = unit.getFormat();

							if (tempFormat != null)
								format = tempFormat;
						}

						if ((structFieldInfo.getFormat() != null)
								|| (format != null))
						{
							label = structFieldInfo.toName(value);
							structCell = new LyCell();

							structCell.setLabel(label);
							newRow.addCell(structCell);
						}
						else
						{
							// sets the new content
							for (int j = 0; j < structClassInfo.getFieldInfoCount(); j++)
							{
								structFieldInfo = structClassInfo.getFieldInfo(j);

								if (structFieldInfo.hasMark(LyFieldInfo.MARK_HIDDEN))
									continue;
								if (structFieldInfo.hasMark(LyFieldInfo.MARK_ID))
									continue;

								if (headers != null)
								{
									header = new LyHeader();
									infoLabel = getInfoLabel(structFieldInfo,
											LyInfoLabel.Type.NAME);

									if (infoLabel != null)
										name = infoLabel.getString(structObj);
									else
										name = structFieldInfo.getName();

									header.setLabel(name);
									headers.addHeader(header);
								}

								structVal = structObj.getValue(structFieldInfo);

								fieldController = getFieldController(i);
								label = fieldController.getLabel(structObj, structFieldInfo,
										structVal);

								if (label == null)
									label = "";

								label = environment.translate(label);
								label = label.replace('\n', ' ');
								label = label.replace('\r', ' ');

								structCell = new LyCell();

								structCell.setLabel(label);
								newRow.addCell(structCell);
							}

							if (structCell == null)
								// add at leat one cell
								newRow.addCell(new LyCell());
						}
					}

					int idx = rows.getRowIndex(row);

					rows.removeRow(row);

					if (idx >= rows.getRowCount())
						idx = -1;
					rows.insertRow(newRow, idx);

					if ((headers != null) && (structCell != null)
							&& (headers.getChildCount() > 1))
					{
						table.setHeaders(headers);
						if (table.getInternalView(false) instanceof LyResetContentInterface)
							((LyResetContentInterface)table.getInternalView(false)).resetContent();
					}
				}
			}
			else
			{
				cell = (LyCell)row.getComponent(nc - 1);

				String   label = null;
				LyObject object = getObject();

				if (object != null)
					label = getLabel(getObject(), fieldInfo, value);

				if (label == null)
					label = fieldInfo.toName(value);

				if (label == null)
					label = "";

				label = label.replace('\n', ' ');
				label = label.replace('\r', ' ');

				label = environment.translate(label);
				cell.setLabel(label);
			}
		}
	} // end method updateRow

	/**
	 * Build a row from the given parameter
	 * @param fieldInfo
	 * @param i
	 * @param editable
	 * @return a LyRow
	 */
	protected LyRow buildRow(LyFieldInfo fieldInfo, int i, boolean editable)
	{
		LyTable table = findInnerTable();

		if (table.getController() == null)
			table.setController(getId());

		LyRow row = new LyRow();

		row.setId(i + "__LyArrayNumberRow__" + fieldInfo.getId());
		if (editable)
		{
			String image = getImageFromPrefix("LY_REMOVE_VALUE", DEFAULT_IMAGE_FORMAT, 12);

			row.setImage(image);
			row.setTooltip("LY_REMOVE_VALUE");
		}

		row.addCell(new LyCell()); // Will be updated in updateRow

		return row;
	}

	/**
	 * Changes current field status to (non-)editable.
	 * @param editable new fieldInfo status.
	 */
	public void setEditable(boolean editable)
	{
		if ((_formField != null)
			&& (_formField.isEditable() != editable))
		{
			super.setEditable(editable);

			int					 i, n = _fieldSetControllers.size();
			LyFieldSetController fieldController;

			for (i = 0; i < n; i++)
			{
				fieldController = _fieldSetControllers.get(i);
				if (fieldController != null)
					fieldController.setEditable(editable);
			}

			if (_createFieldController != null)
				_createFieldController.setEditable(editable);

			setEnableInnerTool(editable);

			LyFieldItem fieldItem = findInnerTable();

			if (fieldItem instanceof LyTable)
				((LyTable)fieldItem).setEnabled(editable);

			updateTools(-1);
		}
	} // end method setEditable

	/**
	 * Changes current field status to (non-)enabled.
	 * @param enabled  new fieldInfo status.
	 */
	public void setEnabled(boolean enabled)
	{
		if ((_formField != null)
			&& (_formField.isEnabled() != enabled))
		{
			super.setEnabled(enabled);

			int					 i, n = _fieldSetControllers.size();
			LyFieldSetController fieldController;

			for (i = 0; i < n; i++)
			{
				fieldController = _fieldSetControllers.get(i);
				if (fieldController != null)
					fieldController.setEnabled(enabled);
			}

			if (_createFieldController != null)
				_createFieldController.setEnabled(enabled);

			setEnableInnerTool(enabled);

			LyFieldItem fieldItem = findInnerTable();

			if (fieldItem instanceof LyTable)
				((LyTable)fieldItem).setEnabled(enabled);

			updateTools(-1);
		}
	}

	/**
	 * Method called at the begining of the parent form validation to check if everything is ok
	 * before completing the edition.
	 * @return One of the status codes indicating if the validation was done (STATUS_OK), if an
	 *         error occured (STATUS_KO), or if nothing was done by the user (STATUS_IGNORE).
	 */
	public short startValidate()
	{
		short status = getState();

		if (status == LyFormInterface.STATUS_KO)
			return status;

		// Start validating all controllers
		int					 i, n = _fieldSetControllers.size();
		LyFieldSetController fieldController;

		for (i = 0; (i < n) && (status != LyFormInterface.STATUS_KO); i++)
		{
			fieldController = _fieldSetControllers.get(i);
			if (fieldController != null)
				status = fieldController.startValidate();
		}

		return status;
	}

	/**
	 * Method called at the end of the parent form validation to complete the edition.
	 * @return One of the status codes indicating if the validation was complete (STATUS_OK), if an
	 *         error occured (STATUS_KO), or if nothing was done by the user (STATUS_IGNORE).
	 */
	public short endValidate()
	{
		// No need to end validate all controllers because validation has been done
		// earlier when the array tool was selected.
		int   n = _fieldSetControllers.size();
		short status = LyFormInterface.STATUS_IGNORE;

		if (n != 0)
			status = LyFormInterface.STATUS_OK;

		// For edition only, this one is not taken into account : this is handled by the control
		// method when _ucontrol_values is false
		if ((_createFieldController != null)
				&& _createFieldController.checkIfModified()
				&& (status != LyFormInterface.STATUS_KO))
			// Do not recreate any value here
			_createFieldController.resetFromValue(null, true, false);

		return status;
	}

	/**
	 * Method called when all modifications are committed.
	 */
	public void commit()
	{
		// Commit all controllers.
		int					 i, n = _fieldSetControllers.size();
		LyFieldSetController fieldController;

		for (i = 0; i < n; i++)
		{
			fieldController = _fieldSetControllers.get(i);
			if (fieldController != null)
				fieldController.commit();
		}

		// For edition only, this one is not taken into account
		if (_createFieldController != null)
			_createFieldController.cancel();
	}

	/**
	 * Method called when editions is cancelled.
	 */
	public void cancel()
	{
		// Cancel all controllers
		int					 i, n = _fieldSetControllers.size();
		LyFieldSetController fieldController;

		for (i = 0; i < n; i++)
		{
			fieldController = _fieldSetControllers.get(i);
			if (fieldController != null)
				fieldController.cancel();
		}

		if (_createFieldController != null)
			_createFieldController.cancel();

		if (_registeredWorkSpace != null)
		{
			_registeredWorkSpace.free(this);
			_registeredWorkSpace = null;
		}
	}

	/**
	 * Method called for controlling current edition.
	 * @return the result of the control
	 * @see LyFormInterface
	 */
	public short control()
	{
		// Control all field controllers.
		int					 i, n = _fieldSetControllers.size();
		LyFieldSetController fieldController;
		short				 status = LyFormInterface.STATUS_OK, fieldStatus;

		for (i = 0; i < n; i++)
		{
			fieldController = _fieldSetControllers.get(i);
			if (fieldController != null)
			{
				fieldStatus = fieldController.control();
				if ((fieldStatus != LyFormInterface.STATUS_OK)
						&& ((status == LyFormInterface.STATUS_OK) || (fieldStatus < status)))
					status = fieldStatus;
			}
		}

		return status;
	}

	/**
	 * Sets the field content from the specified value Any value modified by the user is the form is
	 * lost. Default implementation just call updateFieldItem but this may be overriden in
	 * subclasses.
	 * @param value          new value to set in the field.
	 * @param freeOldValue   indicates if the old value should be freed (deleted) when it is an
	 *                       object
	 * @param createNewValue indicates if a new empty value should created when object is null and
	 *                       when the value is an object
	 */
	public void resetFromValue(Object value, boolean freeOldValue, boolean createNewValue)
	{
		updateValue(value, null);
	}

	/**
	 * Notify object creation to parent.
	 * @param object   Object created by the creation form.
	 * @param relation Relation for which the object was created.
	 */
	public void notifyFormResult(LyObject object, LyRelationFieldInfo relation)
	{
		// This is an array of Relation : receive a transverse action notification,
		// forward this into our inner tool
		LyFieldController fieldController = getFieldController(-1);

		if (fieldController instanceof LyFormResultInterface)
			((LyFormResultInterface)fieldController).notifyFormResult(object, relation);
	}

	/**
	 * Called when an object is added in the workspace.
	 * @param  event Event notifying object add.
	 * @return True if the event is used by the listener, false otherwise.
	 */
	public boolean objectAdded(LyWorkSpaceEvent event)
	{
		// Should never happen
		return false;
	}

	/**
	 * Called when an object is removed from the workspace.
	 * @param  event Event notifying object remove.
	 * @return True if the event is used by the listener, false otherwise.
	 */
	public boolean objectRemoved(LyWorkSpaceEvent event)
	{
		return false; // Already handled since the top value is modified
	}

	/**
	 * Called when an object has changed in the workspace. This call notifies a change on the object
	 * concerned by the event either because its values have changed or because the sort on the
	 * workspace has changed and consequently the object position within the list has changed. These
	 * two cases can easily be distinguished by accessing the values set contained in the event (if
	 * it's not null, this is an change on the object, not a modification of the sort).
	 * <p>Note that object position may also change if one of the modified fields is one of the
	 * fields specified in the list sort.</p>
	 * @param  event Event notifying object change.
	 * @return True if the event is used by the listener, false otherwise.
	 */
	public boolean objectChanged(LyWorkSpaceEvent event)
	{
		// An object has been modified in the list !
		LyObject target = event.getObject();

		if (target != null)
		{
			int				  i, n = _fieldSetControllers.size();
			LyFieldController fieldController;
			LyValue			  fieldValue;
			Object			  value;

			for (i = -1; i < n; i++)
			{
				fieldController = getFieldController(i);
				fieldValue = fieldController.getFieldValue();
				value = ((fieldValue == null) ? null : fieldValue.getValue());

				if (target.getObjectId().equals(value))
				{
					if (i < 0)
						fieldController.resetFromValue(value, false, false);
					else
					{
						LyArrayFieldInfo arrayField = (LyArrayFieldInfo)_fieldInfo;
						LyFieldInfo		 fieldInfo = arrayField.getFieldInfo();
						LyTable			 table = findInnerTable();

						if (table != null)
							updateRow(table, i, fieldInfo, value);
					}
				}
			}
		}
		return false;
	} // end method objectChanged

	/**
	 * Called when an object state is changed.
	 * @param  event Event notifying object state change.
	 * @return True if the event is used by the listener, false otherwise.
	 */
	public boolean objectStateChanged(LyAdvancedWorkSpaceEvent event)
	{
		return objectChanged(event);
	}

	/**
	 * Called when an object user event occured.
	 * @param  event Event notifying object user event.
	 * @return True if the event is used by the listener, false otherwise.
	 */
	public boolean objectUserEvent(LyAdvancedWorkSpaceEvent event)
	{
		return false;
	}

	/**
	 * Called when grouped events are sent.
	 * @param  events Sorted list of events.
	 * @return True if the events are used by the listener, false otherwise.
	 */

	public boolean workSpaceEventsGrouped(ArrayList<LyWorkSpaceEvent> events)
	{
		int				 i, n = events.size();
		LyWorkSpaceEvent event;
		boolean			 used = false;

		for (i = 0; i < n; i++)
		{
			event = events.get(i);
			if (event.getType() == LyWorkSpaceEvent.Type.ADD)
				used |= objectAdded(event);
			else if (event.getType() == LyWorkSpaceEvent.Type.CHANGE)
				used |= objectChanged(event);
			else if (event.getType() == LyWorkSpaceEvent.Type.REMOVE)
				used |= objectRemoved(event);
			else if ((event.getType() == LyWorkSpaceEvent.Type.ERROR_CHANGE)
					|| (event.getType() == LyWorkSpaceEvent.Type.SELECTION_CHANGE)
					|| (event.getType() == LyWorkSpaceEvent.Type.VISIBILITY_CHANGE))
				used |= objectStateChanged((LyAdvancedWorkSpaceEvent)event);
			else if (event.getType() == LyWorkSpaceEvent.Type.USER_EVENT)
				used |= objectUserEvent((LyAdvancedWorkSpaceEvent)event);
		}
		return used;
	}

	/**
	 * Method for moving up or down the selected row.
	 * @param up Indicates if the row is moved up or down.
	 */
	protected void moveSelection(boolean up)
	{
		if (_rowIndex < 0)
			return;
		if (up && (_rowIndex == 0))
			return;
		if (!up && (_rowIndex > (_fieldSetControllers.size() - 1)))
			return;

		LyTable  table = findInnerTable();
		LyRows   rows = table.getRows();
		LyRow    row = rows.getRow(_rowIndex);
		LyValue  value = getFieldValue();
		Object[] values = (Object[])value.getValue();
		Object   temp = values[_rowIndex];
		int		 newRowIndex;

		if (up)
		{
			values[_rowIndex] = values[_rowIndex - 1];
			values[_rowIndex - 1] = temp;
			newRowIndex = _rowIndex - 1;
		}
		else
		{
			values[_rowIndex] = values[_rowIndex + 1];
			values[_rowIndex + 1] = temp;
			newRowIndex = _rowIndex + 1;
		}

		row.setSelected(false);

		updateValue(values, null);
		valueChanged();

		row = rows.getRow(newRowIndex);
		row.setSelected(true);

		LySetTableInterface tableInterface = null;

		if (table.getInternalView() instanceof LySetTableInterface)
			tableInterface = (LySetTableInterface)table.getInternalView();

		if (tableInterface != null)
		{
			// Deselect all the lines
			int i, n = rows.getRowCount();

			for (i = 0; i < n; i++)
				tableInterface.selectRow(i, false);

			// Then select newly selected line
			tableInterface.selectRow(newRowIndex, true);
			tableInterface.showRow(newRowIndex);
		}

		updateTools(newRowIndex);

		// Update the last modification time.
		_setController.setLastModified(new Date().getTime());
	} // end method moveSelection

	/**
	 * Update tools used for ordering table.
	 * @param rowIndex Index of last selected row (-1 means no row selected).
	 */
	protected void updateTools(int rowIndex)
	{
		LyFieldItem fieldItem = findInnerTable();

		if (!(fieldItem instanceof LyTable))
			return;

		LyFormField formField = (LyFormField)fieldItem.getParent();
		LyToolBar   toolBar = formField.getToolBar();
		LyTool	    tool;
		boolean		editable = isEditable() && isEnabled();

		tool = (LyTool)toolBar.getChild(0);
		tool.setEnabled(editable && rowIndex > 0);

		tool = (LyTool)toolBar.getChild(1);
		tool.setEnabled(editable && (rowIndex >= 0)
				&& (rowIndex < (_fieldSetControllers.size() - 1)));

		_rowIndex = rowIndex;
	}
} // end class LyArrayFieldSetController
