/**
 * LyArrayTableFieldSetController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyArrayTableFieldSetController.java 28422 2010-11-03 12:03:38Z ehelmste $
 */

package leon.control.form;

import java.util.*;
import leon.app.behaviorinterface.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.view.*;
import leon.view.event.*;
import leon.view.setinterface.*;


/**
 * Field controller for modification of LyArrayFieldInfo in edit table mode.
 * @see LySetController
 */
public class LyArrayTableFieldSetController extends LyFieldSetController
{
	/** File version */
	public static final String     __VERSION =
		"$Id: LyArrayTableFieldSetController.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Sub fields controllers. */
	protected ArrayList<LyFieldSetController>			   _fieldSetControllers;

	/** Controller for creation. */
	protected LyFieldSetController _createFieldController;

	/**
	 * Constructor.
	 * @param parent parent set controller
	 * @param fieldInfo array field handled by this controller
	 */
	public LyArrayTableFieldSetController(LySetController parent, LyArrayFieldInfo fieldInfo)
	{
		super(parent, fieldInfo);

		_fieldSetControllers = new ArrayList<LyFieldSetController>();
		_createFieldController = null;
	}

	/**
	 * Get the field controller at the given index.
	 * @param index the index
	 * @return the field controller
	 */
	public LyFieldController getFieldController(int index)
	{
		LyFieldSetController fieldController;

		if ((index < 0) && (_createFieldController != null))
			return _createFieldController;
		else if ((index >= 0) && (index < _fieldSetControllers.size()))
		{
			fieldController = _fieldSetControllers.get(index);
			if (fieldController != null)
				return fieldController;
		}

		LyFieldInfo fieldInfo = ((LyArrayFieldInfo)_fieldInfo).getFieldInfo();
        boolean     forceConsult = ((LySetController)_formController).isConsult(_fieldInfo);

		fieldController = (LyFieldSetController)_formController.fieldControllerFactory(fieldInfo, forceConsult);
		if (fieldController != null)
		{
			fieldController._parentFieldController = this;
			if (index >= 0)
				_fieldSetControllers.add(index, fieldController);
			else
				_createFieldController = fieldController;
		}

		return fieldController;
	}

	/**
	 * Free this field controller.
	 */
	public void free()
	{
		// Free field controllers
		if (_fieldSetControllers != null)
		{
			int				  i, n = _fieldSetControllers.size();
			LyFieldController fieldController;

			for (i = 0; i < n; i++)
			{
				fieldController = _fieldSetControllers.get(i);
				if (fieldController != null)
					fieldController.free();
			}

			_fieldSetControllers = null;
		}
		if (_createFieldController != null)
		{
			_createFieldController.free();
			_createFieldController = null;
		}
	}

	/**
	 * Invoked when an action occurs. At this level, the implementation manages<BR>
	 * DEMAND_VALIDATE demands. (demand.getType = LyActionDemand.TYPE_ACTION AND demand.getSource()
	 * = LyController.DEMAND_VALIDATE)<BR>
	 * DEMAND_APPLY demands. (demand.getType = LyActionDemand.TYPE_ACTION AND demand.getSource() =
	 * LyController.DEMAND_APPLY)<BR>
	 * Default action (double click)<BR>
	 * Default action (double click)<BR>
	 * VALUE_CHANGED demands. (demand.getType = LyActionDemand.TYPE_VALUE_CHANGED
	 * @param demand the action demand
	 * @see   LyController#demandPerformed
	 */
	public void actionDemand(LyActionDemand demand)
	{
		String demandCode = demand.getSource().intern();

		if (demand.getType() == LyActionDemand.TYPE_STATUS)
		{
			demandCode = demandCode.substring(demandCode.lastIndexOf(FORM_FIELD_SEP)
					+ FORM_FIELD_SEP.length());

			int				  index = demand.getIndex(0);
			LyValue			  value;
			int				  i, n = _fieldSetControllers.size();
			Vector<Object>	  values = new Vector<Object>(n + 1);
			LyFieldController fieldController;
			LyFieldItem		  fieldItem = _formField.getFieldItem();
			LyTable			  table = (LyTable)fieldItem;
			LyRows			  rows = table.getRows();

			if (index < 0)
			{
				String		demandValue = demand.getValue();
				
				if (demandValue != null)
				{
					int		j, m = rows.getRowCount();
					LyRow	row;
					
					for (j = 0; j < m; j++)
					{
						row = rows.getRow(j);
						
						if (demandValue.equals(row.getId()))
						{
							index = rows.getRowIndex(row);
							break;
						}
					}
				}
			}
			
			for (i = 0; i < n; i++)
			{
				if (i == index)
					continue;
				fieldController = getFieldController(i);
				value = ((LyFieldSetController)fieldController).getFieldValue(null);
				if ((value == null) || value.isNull())
					continue;
				values.add(value.getValue());
			}

			if ((index == n)
					&& (_createFieldController != null))
			{
				// Reset errors on fieldInfo for the object
				_setController.resetFieldInfoErrors(_fieldInfo);

				// Control the value.
				short				   status = LyFormInterface.STATUS_OK;
				LySetBehaviorInterface behavior = _setController.getSetBehavior();

				value = _createFieldController.getFieldValue();
				if ((value == null) || value.isNull())
					return;
				status = behavior.checkNewValue(_setController, (LyFieldInfoValue)value);
				setState(status);
				if (status != LyFormInterface.STATUS_OK)
					return;
				if (value != null)
					values.add(value.getValue());

				// Remove creation before update
				rows.removeRow(index);
				_createFieldController.free();
				_createFieldController = null;
			}

			if (values.size() == 0)
				value = _fieldInfo.buildFieldValue(null);
			else
				value = _fieldInfo.buildFieldValue(values.toArray());

			updateValue(value.getValue(), null);
			valueChanged();
		}
		else
			super.actionDemand(demand);
	} // end method actionDemand

	/**
	 * Extract current value for the specified fieldInfo in the form.
	 * @param  unit      Unit of the value.
	 * @return current value, or null if not found or field is disabled.
	 */
	public LyValue getFieldValue(LyUnit unit)
	{
		Vector<Object>			  values = null;
		LyValue			  value;
		int				  i, n = _fieldSetControllers.size();
		LyFieldController fieldController;

		for (i = 0; i < n; i++)
		{
			fieldController = getFieldController(i);
			value = ((LyFieldSetController)fieldController).getFieldValue();
			if ((value == null) || value.isNull())
				continue;
			if (values == null)
				values = new Vector<Object>(n);
			values.add(value.getValue());
		}

		if (values == null)
			return _fieldInfo.buildFieldValue(null);
		else
			return _fieldInfo.buildFieldValue(values.toArray());
	}

	/**
	 * Internal use : update the value of an array fieldInfo in the form. <B>Application should use
	 * setFieldValue instead.</B> This method is called from updateFieldItem. (Specialization)
	 * @param newValue     the new value of the field. This value may be null, a LyValue instance or
	 *                  the internal value associated to thge fieldInfo. See LyValue.getValue()
	 * @param oldValue  the latest value of the field. This value may be null, a LyValue instance or
	 *                  the internal value associated to thge fieldInfo. See LyValue.getValue()
	 * @see   #setFieldValue
	 * @see   #updateFieldItem
	 */
	protected void updateValue(Object newValue, Object oldValue)
	{
		LyArrayFieldInfo  arrayField = (LyArrayFieldInfo)_fieldInfo;
		int				  i, maxCard = arrayField.getMaxCardinality();
		LyFieldItem		  fieldItem = getFieldItem();
		int				  nn = arrayField.getValueCount(newValue);
		int				  no = _fieldSetControllers.size();
		LyFieldInfo		  fieldInfo = arrayField.getFieldInfo();
		Object			  newItemValue, oldItemValue;
		boolean			  editable = _setController.isEditable(arrayField);
		LyFieldController fieldController;

		if (fieldItem instanceof LyForm)
		{
			for (i = 0; i < maxCard; i++)
			{
				newItemValue = arrayField.getValue(newValue, i);
				oldItemValue = arrayField.getValue(oldValue, i);
				fieldController = getFieldController(i);
				fieldController.updateFieldItem(newItemValue, oldItemValue, false);
				((LyFieldSetController)fieldController).setEditable(editable);
			}
		}
		else if (fieldItem instanceof LyTable)
		{
			LyTable table = (LyTable)fieldItem;
			LyRows  rows = table.getRows();
			LyRow   row;

			table.setIdle(true);

			for (i = 0; i < nn; i++)
			{
				newItemValue = arrayField.getValue(newValue, i);
				oldItemValue = arrayField.getValue(oldValue, i);
				if (fieldInfo.compare(newItemValue, oldItemValue) == 0)
					continue;

				row = rows.getRow(i);
				if ((row == null)
						|| row.getId().equals(fieldInfo.getId())) // Row for creation
				{
					row = buildRow(fieldInfo, i, editable);
					rows.insertRow(row, i);
				}

				fieldController = getFieldController(i);
				fieldController.updateValue(newItemValue, oldItemValue);
			}
			for (i = no - 1; i >= nn; i--)
			{
				rows.removeRow(i);
				fieldController = getFieldController(i);
				_fieldSetControllers.remove(i);
				fieldController.free();
			}

			int index = -1;

			if (_setController.isEditable()
					&& ((maxCard < 0) || (nn < maxCard)))
			{
				if (_createFieldController == null)
				{
					// Add extra row to create new value
					Object defaultValue = fieldInfo.getDefaultValue();

					row = buildRow(fieldInfo, -1, editable);
					if (row != null)
					{
						rows.addRow(row);
						fieldController = getFieldController(-1);
						fieldController.updateFieldItem(defaultValue, null, false);
						index = rows.getRowCount() - 1;
					}
				}
			}
			else if (_createFieldController != null)
			{
				row = rows.getRow(nn);
				if (row != null)
					rows.removeRow(nn);
				_createFieldController.free();
				_createFieldController = null;
			}

			table.setIdle(false);
			if (_setController.isInitialized()
					&& (index > 0)
					&& (table.getInternalView() instanceof LySetTableInterface))
				((LySetTableInterface)table.getInternalView()).showRow(index);
		}
	} // end method updateValue

	/**
	 * Build a row
	 * @param fieldInfo the field
	 * @param i an index
	 * @param editable indicates if the row is editable
	 * @return a LyRow
	 */
	protected LyRow buildRow(LyFieldInfo fieldInfo, int i, boolean editable)
	{
		LyRow	row = new LyRow();
		String	prefix = System.currentTimeMillis() + "_" + (i >= 0 ? i : "");

		row.setId(fieldInfo.getId() + prefix);

		if (i >= 0)
		{
			if (editable)
			{
				row.setImage(getImageFromPrefix("LY_REMOVE_VALUE", DEFAULT_IMAGE_FORMAT, 12));
				row.setTooltip("LY_REMOVE_VALUE");
			}
		}
		else
		{
			row.setImage(getImageFromPrefix("LY_ADD_VALUE", DEFAULT_IMAGE_FORMAT, 12));
			row.setTooltip("LY_ADD_VALUE");
		}

		LyFieldController fieldController = getFieldController(i);

		if (fieldController == null)
			return null;

		LyComponent component = fieldController.getComponent();
		LyFormField formField;

		if (component instanceof LyFormField)
		{
			formField = (LyFormField)component;
			formField.setId(formField.getId() + prefix);
			formField.setCaption(null);
			formField.setEnabled(formField.isEnabled() && isEnabled());
			formField.setEditable(formField.isEditable() && isEditable());
			row.addFormField(formField);
		}
		else if (component instanceof LyForm) // May occur if choice multiple
		{
			LyForm form = (LyForm)component;
			int    j, nf;

			for (j = 0, nf = form.getFormFieldCount(); j < nf; j++)
			{
				formField = form.getFormField(j);
				row.addFormField(formField);
			}
		}

		return row;
	} // end method buildRow

	/**
	 * Changes current field status to (non-)editable.
	 * @param editable new fieldInfo status.
	 */
	public void setEditable(boolean editable)
	{
		if ((_formField != null)
				&& (_formField.isEditable() != editable))
		{
			super.setEditable(editable);

			int					 i, n = _fieldSetControllers.size();
			LyFieldSetController fieldController;

			for (i = 0; i < n; i++)
			{
				fieldController = _fieldSetControllers.get(i);
				if (fieldController != null)
					fieldController.setEditable(editable);
			}

			LyTable table = (LyTable)_formField.getFieldItem();

			table.setEnabled(editable);
			
			if (_createFieldController != null)
			{
				if (editable)
					_createFieldController.setEditable(editable);
				else
				{
					LyRows  rows = table.getRows();

					rows.removeRow(n);
					_createFieldController.free();
					_createFieldController = null;
				}
			}
		}
	} // end method setEditable

	/**
	 * Changes current field status to (non-)enabled.
	 * @param enabled  new fieldInfo status.
	 */
	public void setEnabled(boolean enabled)
	{
		if ((_formField != null)
				&& (_formField.isEnabled() != enabled))
		{
			super.setEnabled(enabled);

			int					 i, n = _fieldSetControllers.size();
			LyFieldSetController fieldController;

			for (i = 0; i < n; i++)
			{
				fieldController = _fieldSetControllers.get(i);
				if (fieldController != null)
					fieldController.setEnabled(enabled);
			}

			LyTable table = (LyTable)_formField.getFieldItem();

			table.setEnabled(enabled);
			
			if (_createFieldController != null)
			{
				if (enabled)
					_createFieldController.setEnabled(enabled);
				else
				{
					LyRows  rows = table.getRows();

					rows.removeRow(n);
					_createFieldController.free();
					_createFieldController = null;
				}
			}
		}
	} // end method setEnabled

	/**
	 * Method called at the begining of the parent form validation to check if everything is ok
	 * before completing the edition.
	 * @return One of the status codes indicating if the validation was done (STATUS_OK), if an
	 *         error occured (STATUS_KO), or if nothing was done by the user (STATUS_IGNORE).
	 */
	public short startValidate()
	{
		int					 i, n = _fieldSetControllers.size();
		LyFieldSetController fieldController;

		if (_createFieldController != null)
		{
			LyFieldInfo fieldInfo = ((LyArrayFieldInfo)_fieldInfo).getFieldInfo();
			Object	    defaultValue = fieldInfo.getDefaultValue();
			LyValue     value = _createFieldController.getFieldValue();

			if ((value != null)
					&& !fieldInfo.equals(value.getValue(), defaultValue))
			{
				Vector<Object> values = new Vector<Object>(n + 1);

				for (i = 0; i < n; i++)
				{
					fieldController = (LyFieldSetController)getFieldController(i);
					value = fieldController.getFieldValue(null);
					if ((value == null) || value.isNull())
						continue;
					values.add(value.getValue());
				}

				// Reset errors on fieldInfo for the object
				_setController.resetFieldInfoErrors(_fieldInfo);

				// Control the value.
				short				   status = LyFormInterface.STATUS_OK;
				LySetBehaviorInterface behavior = _setController.getSetBehavior();

				value = _createFieldController.getFieldValue();
				if ((value != null) && !value.isNull())
				{
					status = behavior.checkNewValue(_setController, (LyFieldInfoValue)value);
					setState(status);
					if (status != LyFormInterface.STATUS_OK)
						return status;
					if (value != null)
						values.add(value.getValue());

					// Remove creation before update
					LyFieldItem fieldItem = _formField.getFieldItem();
					LyTable     table = (LyTable)fieldItem;
					LyRows	    rows = table.getRows();

					rows.removeRow(n);
					_createFieldController.free();
					_createFieldController = null;

					if (values.size() == 0)
						value = _fieldInfo.buildFieldValue(null);
					else
						value = _fieldInfo.buildFieldValue(values.toArray());

					updateValue(value.getValue(), null);
					valueChanged();
				}
			}
		}

		// Start validating all controllers
		short status = LyFormInterface.STATUS_IGNORE;

		for (i = 0; (i < n) && (status != LyFormInterface.STATUS_KO); i++)
		{
			fieldController = _fieldSetControllers.get(i);
			if (fieldController != null)
				status = fieldController.startValidate();
		}

		return status;
	} // end method startValidate

	/**
	 * Method called at the end of the parent form validation to complete the edition.
	 * @return One of the status codes indicating if the validation was complete (STATUS_OK), if an
	 *         error occured (STATUS_KO), or if nothing was done by the user (STATUS_IGNORE).
	 */
	public short endValidate()
	{
		// End validate all controllers
		int					 i, n = _fieldSetControllers.size();
		LyFieldSetController fieldController;
		short				 status = LyFormInterface.STATUS_IGNORE;

		for (i = 0; (i < n) && (status != LyFormInterface.STATUS_KO); i++)
		{
			fieldController = _fieldSetControllers.get(i);
			if (fieldController != null)
				status = fieldController.endValidate();
		}

		if ((status != LyFormInterface.STATUS_KO)
				&& (_createFieldController != null))
			status = _createFieldController.endValidate();

		return status;
	}

	/**
	 * Method called when all modifications are committed.
	 */
	public void commit()
	{
		// Commit all controllers.
		int					 i, n = _fieldSetControllers.size();
		LyFieldSetController fieldController;

		for (i = 0; i < n; i++)
		{
			fieldController = _fieldSetControllers.get(i);
			if (fieldController != null)
				fieldController.commit();
		}

		if (_createFieldController != null)
			_createFieldController.commit();
	}

	/**
	 * Method called when editions is cancelled.
	 */
	public void cancel()
	{
		// Cancel all controllers
		int					 i, n = _fieldSetControllers.size();
		LyFieldSetController fieldController;

		for (i = 0; i < n; i++)
		{
			fieldController = _fieldSetControllers.get(i);
			if (fieldController != null)
				fieldController.cancel();
		}

		if (_createFieldController != null)
			_createFieldController.cancel();
	}

	/**
	 * Method called for controlling current edition.
	 * @return the result of the control
	 * @see LyFormInterface
	 */
	public short control()
	{
		// Control all field controllers.
		int					 i, n = _fieldSetControllers.size();
		LyFieldSetController fieldController;
		short				 status = LyFormInterface.STATUS_OK, fieldStatus;

		for (i = 0; i < n; i++)
		{
			fieldController = _fieldSetControllers.get(i);
			if (fieldController != null)
			{
				fieldStatus = fieldController.control();
				if ((fieldStatus != LyFormInterface.STATUS_OK)
						&& ((status == LyFormInterface.STATUS_OK) || (fieldStatus < status)))
					status = fieldStatus;
			}
		}

		if (_createFieldController != null)
		{
			fieldStatus = _createFieldController.control();
			if ((fieldStatus != LyFormInterface.STATUS_OK)
					&& ((status == LyFormInterface.STATUS_OK) || (fieldStatus < status)))
				status = fieldStatus;
		}

		return status;
	} // end method control
} // end class LyArrayTableFieldSetController
