/**
 * LyArrayStructFieldSetController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyArrayTableStructFieldSetController.java 28422 2010-11-03 12:03:38Z ehelmste $
 */

package leon.control.form;

import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.view.*;


/**
 * Field controler for modification of an array of struct.
 * @see LySetController
 */
public class LyArrayTableStructFieldSetController extends LyArrayTableFieldSetController
{
	/** File version */
	public static final String __VERSION =
		"$Id: LyArrayTableStructFieldSetController.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Constructor.
	 * @param parent parent set controller
	 * @param fieldInfo array field handled by this controller
	 */
	public LyArrayTableStructFieldSetController(LySetController parent,
		LyArrayFieldInfo fieldInfo)
	{
		super(parent, fieldInfo);
	}

	/**
	 * Get the field controller at the given index.
	 * @param index the index
	 * @return the field controller
	 */
	public LyFieldController getFieldController(int index)
	{
		LyFieldController fieldController = super.getFieldController(index);
		LyObject		  object = getObject();

		if ((index >= 0) && (object != null))
		{
			LyObjectId[] values = (LyObjectId[])object.getValue(_fieldInfo);

			if ((values != null) && (values.length > index))
			{
				LyObject structObject = values[index].getLogicalObject();

				((LyStructFieldSetController)fieldController).setStructObject(structObject);
			}
		}

		return fieldController;
	}

	/**
	 * @param fieldController
	 * @return the row corresponding to the controller
	 */
	protected LyRow getRow(LyFieldController fieldController)
	{
		LyFieldItem fieldItem = getFieldItem();
		LyTable     table = (LyTable)fieldItem;
		LyRows	    rows = table.getRows();
		int		    nr = ((rows == null) ? 0 : rows.getRowCount());

		if ((rows == null) || (nr == 0))
			return null;

		if (fieldController == _createFieldController)
			// Get last row
			return rows.getRow(nr - 1);

		int				  i, n = _fieldSetControllers.size();
		LyFieldSetController controller;

		for (i = 0; (i < n) && (nr > i); i++)
		{
			controller = _fieldSetControllers.get(i);
			if (controller == fieldController)
				return rows.getRow(i);
		}

		return null;
	}

	/**
	 * Called when the son form end
	 * @param fieldController a struct field controller
	 */
	protected void notifyFormResult(LyStructFieldSetController fieldController)
	{
		if (fieldController != _createFieldController)
			return;

		// Update row corresponding to given controller
		LyRow row = getRow(fieldController);

		if (row == null)
			return;

		String image = getImageFromPrefix("LY_VALIDATE_ROW", DEFAULT_IMAGE_FORMAT, 12);
		String tooltip = "LY_CREATE_OBJECT";

		row.setTooltip(tooltip);
		row.setImage(image);
	}
} // end class LyArrayTableStructFieldSetController
