/**
 * LyCompositionArrayFieldConsultController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyCompositionArrayFieldConsultController.java 28832 2010-12-08 08:27:37Z jlsaouli $
 */

package leon.control.form;

import leon.app.*;
import leon.app.behaviorinterface.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.view.*;


/**
 * Field controller for consultation of an array of composition relation.
 * @see LyConsultController
 */
public class LyCompositionArrayFieldConsultController
	extends LyMultiCompositionFieldConsultController
	implements LyGetArrayInterface
{
	/** File version */
	public static final String __VERSION =
		"$Id: LyCompositionArrayFieldConsultController.java 28832 2010-12-08 08:27:37Z jlsaouli $";

	/** Array field */
	protected LyArrayFieldInfo _array;

	/**
	 * Constructor.
	 * @param parent parent form controller
	 * @param array array field
	 * @param relation relation field 
	 */
	public LyCompositionArrayFieldConsultController(LyFormController parent,
		LyArrayFieldInfo array, LyRelationFieldInfo relation)
	{
		super(parent, relation);
		_array = array;
	}

	/**
	 * Accessor to the formField (View component) associated to the fieldInfo. The formField
	 * contains a caption (label of the fieldInfo), a fieldItem (value of the fieldInfo) and the
	 * constraints definition (how the component are shown in the view).
	 * @return the formField corresponding to the given fieldInfo
	 */
	public LyFormField getFormField()
	{
		if (_formField != null)
			return _formField;

		_formField = getFormField(_array);
		_component = _formField;
		return _formField;
	}

	/**
	 * @see leon.control.form.LyMultiCompositionFieldConsultController#buildAreaController(boolean)
	 */
	protected LyAreaController buildAreaController(boolean typeChanged)
	{
		if (_fieldType == null)
			_fieldType = getFieldType();

		LyConsultBehaviorInterface behavior = (LyConsultBehaviorInterface)getBehavior();
		LyAction				   action = behavior.getAction((LyConsultController)
				_formController, (LyRelationFieldInfo)_fieldInfo, _fieldType);

		if (action == null)
		{
            LyApplication application = getApplication();
			String        actionId = LyAction.ID_CONSULT_RELATION_TABLE;

			action = _fieldType.getActionWithId(actionId);
			if (action == null)
			{
                String  fieldTypeActionId = _fieldType.getId() + "." + actionId;
                
                action = application.getAction(fieldTypeActionId);
                if (action == null)           
                    action = new LyAction(fieldTypeActionId,
						application.getActionWithId(actionId, null), _fieldType, null);
			}
		}

		_areaController = new LyRelationTableController(getSession(), this, action,
				(LyRelationFieldInfo)_fieldInfo, _array, _fieldType, null);
		((LyFormInterface)_areaController).setEditable(false);
		_areaController.initialize();

		return _areaController;
	} // end method buildAreaController

	/**
	 * @see leon.control.form.LyFieldConsultController#updateFieldItem(java.lang.Object, java.lang.Object, boolean)
	 */
	public void updateFieldItem(Object newValue, Object oldValue, boolean propagate)
	{
		// Keep value
		_consultValue = _array.buildFieldValue(newValue);

		// Update field item
		LyFieldItem fieldItem = getFieldItem();

		fieldItem.setFieldValue(newValue);

		LyInfoLabel valueLabel = getInfoLabel(_array, LyInfoLabel.Type.VALUE);

		if (valueLabel != null)
		{
			super.updateFieldItem(newValue, oldValue, propagate);
			return;
		}

		updateValue(newValue, oldValue);
	}

	/**
	 * @see leon.control.form.LyMultiCompositionFieldConsultController#updateValue(java.lang.Object, java.lang.Object)
	 */
	public void updateValue(Object newValue, Object oldValue)
	{
		LyObject object = getObject();

		if (object == null)
			return;

		LyAreaController  areaController = getAreaController();
		LyObjectList	  previousContext = areaController.getContext();
		LySimpleWorkSpace context = new LySimpleWorkSpace(getSession(), object, true);

		if ((previousContext == null)
				|| !context.equals(previousContext))
			areaController.setContext(context);
		else if (!_array.equals(newValue, oldValue))
			areaController.reset();
	}

	/**
	 * Get the array fieldInfo associated to the controller.
	 * @return The array fieldInfo associated to the controller.
	 */
	public LyArrayFieldInfo getArray()
	{
		return _array;
	}
} // end class LyCompositionArrayFieldConsultController
