/**
 * LyCompositionFieldConsultController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyCompositionFieldConsultController.java 28832 2010-12-08 08:27:37Z jlsaouli $
 */

package leon.control.form;

import leon.app.*;
import leon.app.behaviorinterface.*;
import leon.control.*;
import leon.control.builder.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;


/**
 * Field controller for consultation of a simple composition relation.
 * @see LyConsultController
 */
public class LyCompositionFieldConsultController extends LyFieldConsultController
{
	/** File version */
	public static final String __VERSION =
		"$Id: LyCompositionFieldConsultController.java 28832 2010-12-08 08:27:37Z jlsaouli $";

	/**  */
	protected LyAreaController _areaController;

	/**
	 * Constructor. The controller automatically registers on the specified session.
	 * @param parent parent controller
	 * @param fieldInfo relation field handled by this controller
	 */
	public LyCompositionFieldConsultController(LyFormController parent,
		LyRelationFieldInfo fieldInfo)
	{
		super(parent, fieldInfo);
	}

	/**
	 * Update the value of a fieldInfo shown in the view. The method is overriden for relation
	 * fieldInfo (updateRelation) and file fieldInfo (updateFile)
	 * @param newValue  the new value of the field.
	 * @param oldValue  former value of the field,or null if unknown. This parameter is not used
	 *                  within generic code.
	 */
	protected void updateValue(Object newValue, Object oldValue)
	{
	    if (_areaController == null)
	        return;
	    
		LyObject newObject = ((newValue != null) ? ((LyObjectId)newValue).getLogicalObject() : null);
		LyObject oldObject = ((oldValue != null) ? ((LyObjectId)oldValue).getLogicalObject() : null);

		if ((newObject == oldObject)
				&& (!(_areaController instanceof LyFormController)
					|| (((LyFormController)_areaController).getObject() != null)))
			return;

		if (newObject != null)
		{
			if (newObject.getClassInfo() != _areaController.getClassInfo())
				_areaController = changeAreaFromTypeChange(newValue);
			else
				_areaController.setContext(new LySimpleObjectList(newObject));
		}
		else
			_areaController.setContext(null);
	}

	/**
	 * Gets the specific areaController for the given field info.
	 * @return Area controller corresponding to the given field info, null in most cases.
	 */
	public LyAreaController getAreaController()
	{
		if (_areaController != null)
			return _areaController;

		_areaController = buildAreaController(false);

		LyBuilder builder = _parent.getBuilder();

		if (builder instanceof LyFileBuilder)
		{
			// Check if the view was built with a file builder
			LyFormField formField = getFormField();

			if ((formField != null) && !_fieldInfo.fromFieldTag())
			{
				LyComponent component = formField.getFieldItem();

				if ((component instanceof LyArea)
						&& (((LyArea)component).getPanelCount() > 0))
				{
					LyEnvironment environment = getEnvironment();
					LyAction	  action = _areaController.getAction();

					environment.logWarning("BF AppEngine : action " + action
						+ " should have its own specific view if needed");
				}

				// Replace existing area by sub controller area
				LyArea area = (LyArea)_areaController.getComponent();

				formField.setFieldItem(area);
				_areaController.initialize();
			}
		}

		return _areaController;
	} // end method getAreaController

	/**
	 * Gets specific area controller for the controlled field info.
	 * @param typeChanged if a type change occurs
	 * @return an areaController
	 */
	protected LyAreaController buildAreaController(boolean typeChanged)
	{
		if (_fieldType == null)
		{
			_fieldType = getFieldType();
			if (_fieldType == null)
			    return null;
		}

		LyConsultBehaviorInterface behavior = (LyConsultBehaviorInterface)getBehavior();
		LyAction				   action = behavior.getAction((LyConsultController)
				_formController, (LyRelationFieldInfo)_fieldInfo, _fieldType);

		if (action == null)
		{
			action = _fieldType.getActionWithId(LyAction.ID_CONSULT);
			if (action == null)
				return null;
		}

		LyClassInfo  classInfo = getClassInfo();
		LyObject     object = getObject();
		LyObjectList list = null;

		if ((object != null)
				&& classInfo.contains(_fieldInfo))
		{
			LyRelationValue value = object.getRelationValue((LyRelationFieldInfo)_fieldInfo);

			if (value != null)
				list = value.getObjectList();
		}

		if (list == null)
			list = new LySimpleObjectList();

		LySession session = getSession();

		_areaController = (LyAreaController)session.invokeAction(_formController, action,
				_fieldType, list, false);
		if (_areaController != null)
			_areaController.setViewController(_formController.getViewController());

		_formController.addAreaController(_areaController);
		return _areaController;
	} // end method buildAreaController

	/**
	 * Method used to propagate the type change from other controller.
	 * @param  newValue       The new value of the field.
	 * @return The areaController expected.
	 */
	protected LyAreaController changeAreaFromTypeChange(Object newValue)
	{
		// if the type has changed
		LyRelationFieldInfo relationField = (LyRelationFieldInfo)_fieldInfo;

		if (relationField.fromFieldTag()
				&& ((((LyObjectId)newValue).getClassInfo()) != _areaController.getClassInfo()))
		{
			_fieldType = ((LyObjectId)newValue).getClassInfo();

			LyArea oldArea = (LyArea)_areaController.getComponent();
			LyArea parentArea = (LyArea)oldArea.getParent();

			_areaController.free(_formController);
			_areaController = buildAreaController(true);

			LyArea	    newArea = (LyArea)_areaController.getComponent();
			LyComponent areaChild = newArea.getChild(0);

			newArea.setScrolling(LyArea.Scrolling.NO);
			setAreaNoScroll(areaChild);

			newArea.setLabel(null);
			_areaController.initialize();

			parentArea.removePanel(oldArea);
			parentArea.addPanel(newArea);

			LyAction action = getAction();

			// update of the relation type
			if (!action.getRoot().getId().equals(LyAction.ID_CONSULT))
			{
				LyComponent component = parentArea.getParent();

				if ((component != null) && (component instanceof LyFormField))
				{
					LyRelation typeRelation = ((LyFormField)component).getRelation();

					if (typeRelation == null)
						return _areaController;

					int    i, nl = typeRelation.getLinkCount();
					String newClassInfoId;

					for (i = 0; i < nl; i++)
					{
						newClassInfoId = ((LyObjectId)newValue).getClassInfo().getId();
						if (typeRelation.getLink(i).getId().equals(newClassInfoId))
						{
							typeRelation.selectLink(i, true);
							break;
						}
					}
				}
			}
		}

		return _areaController;
	} // end method changeAreaFromTypeChange

	/**
	 * Gets the classInfo of the object pointed by the relation in the current form.
	 * @return The classInfo of the object pointed by the relation in the current form.
	 */
	public LyClassInfo getFieldType()
	{
		LyClassInfo		    classInfo = getClassInfo();
		LyRelationFieldInfo relation = (LyRelationFieldInfo)_fieldInfo;
		LyClassInfo		    fieldType = null;
		LyObject		    object = getObject();

		if ((object != null)
				&& classInfo.contains(relation))
		{
			LyRelationValue relationValue = object.getRelationValue(relation);

			if ((relationValue != null) && (relation != null))
			{
				LyObjectId  objectId = (LyObjectId)relationValue.getValue();
				LyClassInfo newClassInfo;

				if (objectId != null)
				{
					newClassInfo = objectId.getClassInfo();
					fieldType = newClassInfo;
				}
			}
		}

		if (fieldType == null)
		{
			LyInfoList<LyClassInfo> fieldTypes = relation.getClassInfos();
			int		   index = 0, nClassInfos = fieldTypes.getSize();

			if (index < nClassInfos)
			{
			    fieldType = fieldTypes.getInfo(index++);
			    
    			while ((fieldType != null)
    			        && fieldType.isAbstract()
    			        && (index < nClassInfos))
    				fieldType = fieldTypes.getInfo(index++);
			}
		}

		return fieldType;
	} // end method getFieldType
} // end class LyCompositionFieldConsultController
