/*
 * LyCompositionFieldUnitConsultController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyCompositionFieldUnitConsultController.java 34632 2012-03-14 10:30:05Z jlsaouli $
 */

package leon.control.form;

import leon.app.*;
import leon.app.behaviorinterface.*;
import leon.control.*;
import leon.control.form.builder.*;
import leon.data.*;
import leon.info.*;
import leon.info.misc.*;
import leon.misc.*;
import leon.view.*;


/**
 * Field controller for a modification of a simple composition relation.
 * @see LySetController
 */
public class LyCompositionFieldUnitConsultController extends LyFieldConsultController
{
	/** File version */
	public static final String __VERSION =
		"$Id: LyCompositionFieldUnitConsultController.java 34632 2012-03-14 10:30:05Z jlsaouli $";

	/** The internal area controller */
	protected LyAreaController _areaController;
	/** Indicates a from type change */
	protected boolean		   _fromTypeChange;
	/** Struct object */
	protected LyObject		   _structObject;
	
	/** New classInfo */
	protected LyClassInfo	   _newClassInfo;

	/**
	 * Constructor.
	 * @param parent parent consult controller
	 * @param fieldInfo relation field handled by this controller
	 */
	public LyCompositionFieldUnitConsultController(LyConsultController parent,
		LyRelationFieldInfo fieldInfo)
	{
		super(parent, fieldInfo);

		_areaController = null;
		_fromTypeChange = false;
		_newClassInfo = null;
		initStructObject();
	}

	/**
	 * Initialized the struct object
	 */
	protected void initStructObject()
	{
		if (!(_fieldInfo instanceof LyStructFieldInfo)
				|| (((LyStructFieldInfo)_fieldInfo).isMultiple()))
		{
			_structObject = null;
			return;
		}

		LyObject object = getObject();

		if ((object != null)
				&& object.getClassInfo().contains(_fieldInfo))
		{
			LyObjectId objectId = (LyObjectId)object.getValue(_fieldInfo);

			if (objectId != null)
				_structObject = objectId.getLogicalObject();
		}
	}

	/**
	 * Set the struct object
	 * @param structObject a struct object
	 */
	protected void setStructObject(LyObject structObject)
	{
		if (!(_fieldInfo instanceof LyStructFieldInfo)
				|| (((LyStructFieldInfo)_fieldInfo).isMultiple())
			/*|| (_structObject != null)*/)
			return;

		LyValueSet  values = ((structObject == null) ? null : structObject.getValues());
		LyValueSet  newValues = new LyValueSet();
		LyValue     value;
		LyFieldInfo fieldInfo;
		int		    i, n = ((values == null) ? 0 : values.getSize());

		for (i = 0; i < n; i++)
		{
			value = values.getFieldValue(i);
			fieldInfo = (LyFieldInfo)value.getField();
			if ((fieldInfo.hasMark(LyFieldInfo.MARK_ID)))
				continue;
			newValues.addValue(fieldInfo, value.getValue());
		}

		buildStructObject(structObject);
	}

	/**
	 * 
	 * @param newStructObject a struct object
	 * @return the struct object id
	 */
	protected LyObjectId buildStructObject(LyObject newStructObject)
	{
		if (_fieldType == null)
			_fieldType = getFieldType();

		if ((_structObject == null)
                || (_structObject.getClassInfo() != _fieldType))
			_structObject = newStructObject;

		return ((_structObject == null) ? null : _structObject.getObjectId());
	}

	/**
	 * Internal use : update the value of a fieldInfo in the form.
	 * @param newValue the new value of the field. This value may be null, a LyValue instance or the
	 *                 internal value associated to thge fieldInfo. See LyValue.getValue()
	 * @param oldValue the latest value of the field. This value may be null, a LyValue instance or
	 *                 the internal value associated to thge fieldInfo. See LyValue.getValue()
	 */
	protected void updateValue(Object newValue, Object oldValue)
	{
		LyObjectId  newObjectId = ((newValue == null) ? null : (LyObjectId)newValue);
		LyObject    newObject = ((newObjectId == null) ? null : newObjectId.getLogicalObject());
		LyClassInfo newClassInfo = ((newObject == null)
				? ((LyStructFieldInfo)_fieldInfo).getClassInfo() : newObject.getClassInfo());

		if (newClassInfo != _fieldType)
			changeAreaFromTypeChange(newValue);

		LyAreaController areaController = getAreaController();

		if ((_fieldInfo instanceof LyStructFieldInfo)
				&& !(((LyStructFieldInfo)_fieldInfo).isMultiple())
				&& (areaController == null)
				&& (_structObject == null))
		{
			LyObjectId objectId = (LyObjectId)newValue;

			if (objectId != null)
				_structObject = objectId.getLogicalObject();
		}

		LyFieldItem fieldItem = getFieldItem();

		if (areaController == null)
		{
			LyStructFieldInfo subStruct = (LyStructFieldInfo)newClassInfo.getParent();
			LyUnit			  unit = getDefaultFieldUnit(subStruct);
			LyFormat		  format = ((unit != null) ? unit.getFormat() : null);
			String			  label;

			if (format == null)
			{
				// Search for a value label
				LyInfoLabel valueLabel = getInfoLabel(_fieldInfo, LyInfoLabel.Type.VALUE);
				LyObject    object = getObject();

				if ((valueLabel != null) && (object != null))
					label = getLabel(object, _fieldInfo, newValue);
				else if ((valueLabel != null) && (_structObject != null))
				{
					// We haven't any source object (case of creation)
					// Can we compute label even so ?
					LyUnit	    pathUnit;
					LyPath	    path;
					LyFieldInfo target;
					LyClassInfo structClass = _structObject.getClassInfo();
					int		    i, n = valueLabel.getPartsCount();
					String[]    args = new String[n];
					Object	    value;

					for (i = 0; i < n; i++)
					{
						path = valueLabel.getPath(i);
						target = path.getTargetFieldInfo();
						if (structClass.contains(target))
						{
							pathUnit = valueLabel.getUnit(i);
							value = _structObject.getValue(target);
							value = target.convertFromFieldUnit(value, pathUnit);
							args[i] = target.toName(value);
						}
						else
							break;
					}

					if (i == n)
					{
						String		  str = valueLabel.getValue();
						LyEnvironment env = getEnvironment();

						label = env.getString(str, args);
					}
					else
						label = _fieldInfo.toString(newValue);
				}
				else
					label = _fieldInfo.toString(newValue);
			}
			else
				label = format.toString(_fieldInfo, newValue, unit);

			fieldItem.setLabel(label);
			fieldItem.setFieldValue(newValue);
			return;
		}

		LyRelationFieldInfo relationField = (LyRelationFieldInfo)_fieldInfo;
		LyObjectList	    context = null;
		LyObjectList	    previousContext = areaController.getContext();

		if (getObject() != null)
			context = new LySimpleObjectList(getObject());

		if (((context == null) && (previousContext != null))
				|| ((context != null) && (previousContext == null))
				|| ((context != null) && (previousContext != null)
					&& !context.equals(previousContext)))
			areaController.setContext(context);
		else if (!relationField.equals(newValue, oldValue))
		{
			if (!relationField.isMultiple())
				_areaController = changeAreaFromTypeChange(newValue);
			areaController.reset();
		}
	} // end method updateValue

	/**
	 * Method used to propagate the type change from other controller.
	 * @param  newValue       The new value of the field.
	 * @return The areaController expected.
	 */
	protected LyAreaController changeAreaFromTypeChange(Object newValue)
	{
		// if the type has changed
		if (newValue == null)
			return _areaController;

		_fromTypeChange = true;

		LyRelationFieldInfo relationField = (LyRelationFieldInfo)_fieldInfo;

		_newClassInfo = ((LyObjectId)newValue).getClassInfo();
		if (relationField.fromFieldTag()
				&& (_newClassInfo != _fieldType))
			typeChanged();

		return _areaController;
	}

	/**
	 * Internal method for propagating a type change. This methods is called when the type field has
	 * been changed by the user.
	 * <P>The LySetController additionaly impact the view using the setState method.</P>
	 */
	protected void typeChanged()
	{
		LyClassInfo oldClassInfo;

		oldClassInfo = _fieldType;

		if (_newClassInfo == oldClassInfo)
			return;

		LyObject	    parentObject = getObject();
		LyObject	    newObject = null;

		LyRelationValue value = ((parentObject == null)
				? null : parentObject.getRelationValue((LyStructFieldInfo)_fieldInfo));

		LyObjectId	    objectId = null;

		if ((value != null) && (value.getValueCount() > 1))
			objectId = value.getObjectId(0);

		newObject = ((objectId == null) ? null : objectId.getLogicalObject());

		_fieldType = _newClassInfo;
		_newClassInfo = null;

		LyFormFieldBuilder builder = (LyFormFieldBuilder)getBuilder();
		LyFormField		   formField = builder.buildFormField(false);

		_formField.setFieldItem(formField.getFieldItem());
		_formField.setFieldUnit(formField.getFieldUnit());
		_formField.setToolBar(formField.getToolBar());

		if ((getAreaController() != null) && (newObject != null))
			((LyConsultController)getAreaController()).updateObjectFromCompositionRelation(
				(LyStructFieldInfo)_fieldInfo);
		else if ((newObject == null) && (_areaController != null))
			newObject = ((LySetController)_areaController).getObject();

		setStructObject(newObject);

		_formField.resetContent();
		_formController.resetFormContent();
	} // end method typeChanged

	/**
	 * Gets the specific areaController for the given field info.
	 * @return Area controller corresponding to the given field info, null in most cases.
	 */
	public LyAreaController getAreaController()
	{
		if (_areaController != null)
		{
			// If the type has changed, removing the old areaController
			if (_areaController.getClassInfo() != _fieldType)
			{
				_fromTypeChange = true;
				_areaController.free(_formController);
				_formController.removeAreaController(_areaController);
				_areaController = null;
			}
			else
			{
				_fromTypeChange = false;
				return _areaController;
			}
		}

		if (_formController.isComposition(_fieldInfo))
			_areaController = buildAreaController(_fromTypeChange);

		return _areaController;
	}

	/**
	 * Build the area Controller
	 * @param typeChanged if the build occurs due to a type change
	 * @return an area controller
	 */
	protected LyAreaController buildAreaController(boolean typeChanged)
	{
		if (_fieldType == null)
			_fieldType = getFieldType();

		if ((_fieldInfo instanceof LyStructFieldInfo)
				&& (((LyStructFieldInfo)_fieldInfo).fromFieldTag())
				&& !(((LyStructFieldInfo)_fieldInfo).isMultiple()))
		{
			LyUnit	unit = getDefaultFieldUnit();

			if ((unit != null) && (unit.getFormat() != null))
			{
				_fromTypeChange = false;
				return null;
			}
		}

		LyClassInfo  classInfo = getClassInfo();
		LyObjectList list = null;
		LyObjectList context = null;
		LyObject     object = getObject();

		if ((object != null)
				&& classInfo.contains(_fieldInfo))
		{
			LyRelationValue value = object.getRelationValue((LyRelationFieldInfo)_fieldInfo);

			if (value != null)
				list = value.getObjectList();

			context = new LySimpleObjectList(object);
		}

		if (list == null)
			list = new LySimpleObjectList();

		LyConsultBehaviorInterface behavior = (LyConsultBehaviorInterface)getBehavior();
		LyAction				   action = behavior.getAction((LyConsultController)
				_formController, (LyRelationFieldInfo)_fieldInfo, _fieldType);

		if (action == null)
		{
			_fromTypeChange = false;
			action = _fieldType.getActionWithId(LyAction.ID_CONSULT);
			if (action == null)
				return null;
		}

		LySession session = getSession();

		_areaController = (LyAreaController)session.invokeAction(_formController, action,
				_fieldType, list, false);

		if (_areaController != null)
		{
			if (_areaController instanceof LyFormController)
				((LyFormController)_areaController).setCompositionRelation((LyRelationFieldInfo)
					_fieldInfo);
			_areaController.setViewController(_formController.getViewController());
			LyArea area = (LyArea)_areaController.getComponent();

			area.setScrolling(LyArea.Scrolling.NO);
			if (context != null)
				_areaController.setContext(context);
			
			_formController.addAreaController(_areaController);
			_fromTypeChange = false;
		}

		return _areaController;
	} // end method buildAreaController

	/**
	 * Gets the classInfo of the object pointed by the relation in the current form.
	 * @return The classInfo of the object pointed by the relation in the current form.
	 */
	public LyClassInfo getFieldType()
	{
		if (_fieldType != null)
			return _fieldType;

		LyClassInfo		    classInfo = getClassInfo();
		LyRelationFieldInfo relation = (LyRelationFieldInfo)_fieldInfo;
		LyClassInfo		    fieldType = null;
		LyObject		    object = getObject();

		if ((object != null)
				&& classInfo.contains(relation))
		{
			LyRelationValue relationValue = object.getRelationValue(relation);

			if ((relationValue != null) && (relation != null))
			{
				LyObjectId  objectId = (LyObjectId)relationValue.getValue();
				LyClassInfo newClassInfo;

				if (objectId != null)
				{
					newClassInfo = objectId.getClassInfo();
					fieldType = newClassInfo;
				}
			}
		}

		if (fieldType == null)
		{
			LyInfoList<LyClassInfo> fieldTypes = relation.getClassInfos();
			int		   index = 0, nClassInfos = fieldTypes.getSize();

			fieldType = fieldTypes.getInfo(index++);
			while ((fieldType.isAbstract()) && (index < nClassInfos))
				fieldType = fieldTypes.getInfo(index++);
		}

		return fieldType;
	} // end method getFieldType

	/**
	 * Sets formField. Called by LyFormFieldBuilder to avoid loops. Reserved for internal use.
	 * @param formField the form field to use
	 */
	public void setFormField(LyFormField formField)
	{
		if (_formField == formField)
			return;
		else if (_formField == null)
			super.setFormField(formField);
	}
} // end class LyCompositionFieldUnitConsultController
