/**
 * LyCompositionFieldSetController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyCompositionFieldUnitSetController.java 34634 2012-03-14 10:31:26Z jlsaouli $
 */

package leon.control.form;

import java.util.*;
import leon.app.*;
import leon.app.behaviorinterface.*;
import leon.control.*;
import leon.control.builder.*;
import leon.control.form.builder.*;
import leon.data.*;
import leon.info.*;
import leon.info.misc.*;
import leon.misc.*;
import leon.view.*;
import leon.view.event.*;
import leon.view.getinterface.*;


/**
 * Field controller for a modification of a simple composition relation.
 * @see LySetController
 */
public class LyCompositionFieldUnitSetController extends LyFieldSetController
	implements LyFormResultInterface,
		LyWorkSpaceListener
{
	/** File version */
	public static final String  __VERSION =
		"$Id: LyCompositionFieldUnitSetController.java 34634 2012-03-14 10:31:26Z jlsaouli $";

	/** internal area controller */
	protected LyAreaController  _areaController;
	/** List of old objects */
	protected Hashtable<LyClassInfo, LyObject>		    _oldObjects;
	/** Indicates if a unit change occurs */
	protected boolean		    _unitChange;
	/** Indicates if a type change occurs */
	protected boolean		    _typeChange;
	/** Indicates if a from type change occurs */
	protected boolean		    _fromTypeChange;
	/** Associared struct object */
	protected LyObject		    _structObject;
	/** origin object */
	protected LyObject		    _origin;
	/** temporary new object */
	protected LyObject		    _tempNewObject;
	/** list of objects */
	protected LySimpleWorkSpace _workspace;
	/** update status */
	protected boolean		    _update;

	/**
	 * Constructor.
	 * @param parent parent set controller
	 * @param fieldInfo relation field handled by this controller
	 */
	public LyCompositionFieldUnitSetController(LySetController parent,
		LyRelationFieldInfo fieldInfo)
	{
		super(parent, fieldInfo);

		_areaController = null;
		_oldObjects = new Hashtable<LyClassInfo, LyObject>();
		_unitChange = false;
		_typeChange = false;
		_fromTypeChange = false;
		_tempNewObject = null;
		_workspace = new LySimpleWorkSpace(getSession(), true);
		_workspace.addListener(this);
		_update = true;
		initStructObject();
	}

	/**
	 * Free this controller.
	 */
	public void free()
	{
		if (_areaController != null)
		{
			_areaController.clearViewCache();
			_areaController = null;
		}
		if (_workspace != null)
		{
			_workspace.free(this);
			_workspace = null;
		}

		super.free();
	}

	/**
	 * Clear view cache.
	 */
	public void clearViewCache()
	{
		if (_areaController != null)
			_areaController.clearViewCache();
		if (_workspace != null)
		{
			_workspace.free(this);
			_workspace = null;
		}

		super.clearViewCache();
	}

	/**
	 * Initialize the struct object
	 */
	protected void initStructObject()
	{
		if (!(_fieldInfo instanceof LyStructFieldInfo)
				|| (((LyStructFieldInfo)_fieldInfo).isMultiple()))
		{
			_structObject = null;
			return;
		}

		LyObject object = getModel();

		if ((object != null)
				&& object.getClassInfo().contains(_fieldInfo))
		{
			LyObjectId objectId = (LyObjectId)object.getValue(_fieldInfo);

			if (objectId != null)
			{
				_structObject = objectId.getLogicalObject();
				_workspace.addObject(_structObject);
				_origin = _structObject;
			}
		}
	}

	/**
	 * Set the struct object
	 * @param structObject a struct object
	 */
	protected void setStructObject(LyObject structObject)
	{
		if (!(_fieldInfo instanceof LyStructFieldInfo)
				|| ((LyStructFieldInfo)_fieldInfo).isMultiple())
			return;

		buildStructObject(structObject);
	}

	/**
	 * Build a struct object from a value set
	 * @param values a value set
	 * @return object id of the created object
	 */
	protected LyObjectId buildStructObject(LyValueSet values)
	{
		if (_fieldType == null)
			_fieldType = getFieldType();

		LyContext context = getControllerContext();

		if ((_structObject == null)
				|| (_structObject.isDeleted())
				|| (_structObject.getClassInfo() != _fieldType))
		{
			if (_structObject != null)
				_structObject = null;

			if (_workspace != null)
			{
				_workspace.free(this);
				_workspace = new LySimpleWorkSpace(getSession(), true);
				_workspace.addListener(this);
			}

			if (values != null)
			{
				_structObject = _fieldType.createInstance();
				_structObject.setValues(values);
				context.createObject(_structObject);
				_workspace.addObject(_structObject);
			}
		}
		else
			context.setValues(_structObject, values);

		return ((_structObject == null) ? null : _structObject.getObjectId());
	} // end method buildStructObject

	/**
	 * 
	 * @param newStructObject the new struct object
	 * @return the id of the struct object
	 */
	protected LyObjectId buildStructObject(LyObject newStructObject)
	{
		if (_fieldType == null)
			_fieldType = getFieldType();

		if ((_structObject == null)
				|| (_structObject.getClassInfo() != _fieldType)
				|| (_setController.isChangingObject()))
			_structObject = newStructObject;

		if (_workspace != null)
		{
			_workspace.free(this);
			_workspace = new LySimpleWorkSpace(getSession(), true);
			_workspace.addListener(this);
		}

		if (_structObject != null)
		{
			_workspace.addObject(_structObject);
			return _structObject.getObjectId();
		}
		else
			return null;
	}

	/**
	 * Invoked when an action occurs. At this level, the implementation manages<BR>
	 * DEMAND_VALIDATE demands. (demand.getType = LyActionDemand.TYPE_ACTION AND demand.getSource()
	 * = LyController.DEMAND_VALIDATE)<BR>
	 * DEMAND_APPLY demands. (demand.getType = LyActionDemand.TYPE_ACTION AND demand.getSource() =
	 * LyController.DEMAND_APPLY)<BR>
	 * Default action (double click)<BR>
	 * Default action (double click)<BR>
	 * VALUE_CHANGED demands. (demand.getType = LyActionDemand.TYPE_VALUE_CHANGED
	 * @param demand the action demand
	 * @see   LyController#demandPerformed
	 */
	public void actionDemand(LyActionDemand demand)
	{
		if (demand.getType() == LyActionDemand.TYPE_TYPE_CHANGED)
		{
			typeChanged();
			return;
		}
		else if (demand.getType() == LyActionDemand.TYPE_UNIT_CHANGED)
		{
			unitChanged(demand.getValue(), demand.getAdditionalValue());
			return;
		}

		super.actionDemand(demand);
	}

	/**
	 * Extract current value for the specified fieldInfo in the form.
	 * @param  unit      Unit of the value.
	 * @return current value, or null if not found or field is disabled.
	 */
	public LyValue getFieldValue(LyUnit unit)
	{
		LyFieldItem		  fieldItem = getFieldItem();
		String			  strValue = null;
		LyObjectId		  objectId;
		LyStructFieldInfo subStruct = ((_fieldType == null)
				? null : (LyStructFieldInfo)_fieldType.getParent());

		if (unit != null)
		{
			LyFormat format = unit.getFormat();

			if (format != null)
			{
				fieldItem = findFieldItem();
				LyValueSet values = null;

				strValue = ((LyGetLabelInterface)fieldItem).getLabel();
				if (strValue != null)
					values = (LyValueSet)format.parse(((subStruct == null) ? _fieldInfo
																		   : subStruct), strValue,
							unit);

				if (_structObject == null)
					objectId = buildStructObject(values);
				else
					objectId = _structObject.getObjectId();

				return _fieldInfo.buildFieldValue(objectId);
			}
			else if (_areaController instanceof LyFormInterface)
			{
				LyObjectList relations = _areaController.getContent();

				return new LyRelationValue((LyRelationFieldInfo)_fieldInfo, relations);
			}
		}

		if (_structObject != null)
		{
			objectId = _structObject.getObjectId();
			return _fieldInfo.buildFieldValue(objectId);
		}
		else
			return super.getFieldValue(unit);
	} // end method getFieldValue

	/**
	 * Extract current value for the specified fieldInfo in the form.
	 * @return current value, or null if not found or field is disabled.
	 */
	public LyValue getFieldValue()
	{
		LyUnit unit = getUnit();

		if ((_areaController instanceof LyFormInterface)
				&& ((unit == null) || (unit.getFormat() == null)))
		{
			LyObjectList relations = _areaController.getContent();

			return new LyRelationValue((LyRelationFieldInfo)_fieldInfo, relations);
		}
		else
			return super.getFieldValue();
	}

	/**
	 * @see leon.control.form.LyFieldSetController#updateFieldItem(java.lang.Object, java.lang.Object)
	 */
	protected Object updateFieldItem(Object newValue, Object oldValue)
	{
		LyUnit			  unit = getUnit();
		LyUnit			  fieldUnit = null;
		LyStructFieldInfo subStruct = null;
		LyObjectId		  newObjectId = ((newValue == null) ? null : (LyObjectId)newValue);
		LyObjectId		  oldObjectId = ((oldValue == null) ? null : (LyObjectId)oldValue);
		LyClassInfo		  newClassInfo = ((LyStructFieldInfo)_fieldInfo).getClassInfo();
		LyClassInfo		  oldClassInfo = ((LyStructFieldInfo)_fieldInfo).getClassInfo();

		if (newObjectId != null)
			newClassInfo = newObjectId.getClassInfo();
		if (oldObjectId != null)
			oldClassInfo = oldObjectId.getClassInfo();

		if ((newClassInfo != oldClassInfo)
				&& (newClassInfo != _fieldType))
		{
			fieldUnit = null;
			unit = null;
		}
		else
		{
			LyClassInfo subStructClass = _fieldType;

			subStruct = (LyStructFieldInfo)subStructClass.getParent();
			fieldUnit = getDefaultFieldUnit(subStruct);
		}

		LyFieldItem fieldItem = getFieldItem();

		if (((unit != null)
					&& (unit.hasEditionFormat()))
				|| ((unit == null)
					&& (fieldUnit != null)
					&& fieldUnit.hasEditionFormat()))
		{
			LyFormat format = (((unit != null) && unit.hasEditionFormat()) ? unit.getFormat()
																		   : fieldUnit.getFormat());
			String   label = format.toString(subStruct, newValue, unit);

			fieldItem.setLabel(label);
			fieldItem.setFieldValue(newValue);
		}

		updateValue(newValue, oldValue);
		// New value may have changed when updating
		newValue = getValue();

		return newValue;
	} // end method updateFieldItem

	/**
	 * Internal use : update the value of a fieldInfo in the form.
	 * @param newValue the new value of the field. This value may be null, a LyValue instance or the
	 *                 internal value associated to thge fieldInfo. See LyValue.getValue()
	 * @param oldValue the latest value of the field. This value may be null, a LyValue instance or
	 *                 the internal value associated to thge fieldInfo. See LyValue.getValue()
	 */
	protected void updateValue(Object newValue, Object oldValue)
	{
		LyObjectId  newObjectId = ((newValue == null) ? null : (LyObjectId)newValue);
		LyObject    newObject = ((newObjectId == null) ? null : newObjectId.getLogicalObject());
		LyClassInfo newClassInfo = ((newObject == null)
				? ((LyStructFieldInfo)_fieldInfo).getClassInfo() : newObject.getClassInfo());

		if ((newClassInfo != _fieldType) || (_setController.isChangingObject()))
			changeAreaFromTypeChange(newValue);

		LyAreaController areaController = getAreaController();

		if ((_fieldInfo instanceof LyStructFieldInfo)
				&& !(((LyStructFieldInfo)_fieldInfo).isMultiple())
				&& (areaController == null)
				&& (_structObject == null))
		{
			LyObjectId objectId = (LyObjectId)newValue;

			if (objectId != null)
				_structObject = objectId.getLogicalObject();
		}

		LyFieldItem fieldItem = getFieldItem();

		if (areaController == null)
		{
			LyUnit   unit = getUnit();
			LyFormat format = ((unit != null) ? unit.getFormat() : null);
			String   label;

			if (format == null)
			{
				// Search for a value label
				LyInfoLabel valueLabel = getInfoLabel(_fieldInfo, LyInfoLabel.Type.VALUE);
				LyObject    object = (isCreation() ? null : getObject());

				if ((valueLabel != null) && (object != null))
					label = getLabel(object, _fieldInfo, newValue);
				else if ((valueLabel != null) && (_structObject != null))
				{
					// We haven't any source object (case of creation)
					// Can we compute label even so ?
					LyUnit	    pathUnit;
					LyPath	    path;
					LyFieldInfo target;
					LyClassInfo structClass = _structObject.getClassInfo();
					int		    i, n = valueLabel.getPartsCount();
					String[]    args = new String[n];
					Object	    value;

					for (i = 0; i < n; i++)
					{
						path = valueLabel.getPath(i);
						target = path.getTargetFieldInfo();
						if (structClass.contains(target))
						{
							pathUnit = valueLabel.getUnit(i);
							value = _structObject.getValue(target);
							value = target.convertFromFieldUnit(value, pathUnit);
							args[i] = target.toName(value);
						}
						else
							break;
					}

					if (i == n)
					{
						String		  str = valueLabel.getValue();
						LyEnvironment env = getEnvironment();

						label = env.getString(str, args);
					}
					else
						label = _fieldInfo.toString(newValue);
				}
				else
					label = _fieldInfo.toString(newValue);
			}
			else
			{
				label = format.toString(_fieldInfo, newValue, unit);

				// Find the correct fieldItem
				fieldItem = findFieldItem();
			}

			fieldItem.setLabel(label);
			fieldItem.setFieldValue(newValue);
			return;
		}

		LyRelationFieldInfo    relationField = (LyRelationFieldInfo)_fieldInfo;
		LyObjectList		   context;
		LyObjectList		   previousContext = areaController.getContext();
		LySetBehaviorInterface behavior = _setController.getSetBehavior();

		context = behavior.getRelationContext(_setController, relationField);
		if ((context == null) && (getObject() != null))
			context = new LySimpleObjectList(getObject());

		if (((context == null) && (previousContext != null))
				|| ((context != null) && (previousContext == null))
				|| ((context != null) && (previousContext != null)
					&& !context.equals(previousContext)))
			areaController.setContext(context);
		else if (!relationField.equals(newValue, oldValue))
		{
			if (!relationField.isMultiple())
				_areaController = changeAreaFromTypeChange(newValue);
			areaController.reset();
		}
	} // end method updateValue

	private LyFieldItem findFieldItem()
	{

		LyFieldItem topField = getFieldItem();

		if (topField instanceof LyArea)
		{
			Vector<LyFormField> result = new Vector<LyFormField>();

			topField.getComponents(LyFormField.class, result);

			if (result.size() != 1)
				return topField;
            
			LyFormField formField = result.get(0);

			return formField.getFieldItem();
		}

		return topField;
	}

	/**
	 * Method used to propagate the type change from other controller.
	 * @param  newValue       The new value of the field.
	 * @return The areaController expected.
	 */
	protected LyAreaController changeAreaFromTypeChange(Object newValue)
	{
		// if the type has changed
		if (newValue == null)
		{
			if (_setController.isChangingObject())
				setStructObject(null);

			return _areaController;
		}

		_fromTypeChange = true;

		LyRelationFieldInfo relationField = (LyRelationFieldInfo)_fieldInfo;
		LyClassInfo		    newClassInfo = ((LyObjectId)newValue).getClassInfo();

		_tempNewObject = ((LyObjectId)newValue).getLogicalObject();

		if (relationField.fromFieldTag()
				&& ((newClassInfo != _fieldType)
					|| (_setController.isChangingObject())))
		{
			LyRelation types = _formField.getRelation();
			int		   i, n = ((types == null) ? 0 : types.getLinkCount());
			String     classId = newClassInfo.getId();

			for (i = 0; i < n; i++)
			{
				if (types.getLink(i).getId().equals(classId))
					types.selectLink(i, true);
				else
					types.selectLink(i, false);
			}
			typeChanged();
			_tempNewObject = null;
		}
		_fromTypeChange = false;
		return _areaController;
	} // end method changeAreaFromTypeChange

	/**
	 * Method called at the begining of the parent form validation to check if everything is ok
	 * before completing the edition.
	 * @return One of the status codes indicating if the validation was done (STATUS_OK), if an
	 *         error occured (STATUS_KO), or if nothing was done by the user (STATUS_IGNORE).
	 */
	public short startValidate()
	{
		if (_areaController instanceof LyFormInterface)
		{
			// If the field is optional, only create on need (if something changed)
			if (_areaController instanceof LyCreateController)
			{
				LySetController		   set = (LySetController)_formController;
				LySetBehaviorInterface behavior = set.getSetBehavior();
				boolean				   optional = behavior.isOptional(set, _fieldInfo);

				((LyCreateController)_areaController).setCreateIfModified(optional);
			}
			return ((LyFormInterface)_areaController).startValidate();
		}
		else
			return super.startValidate();
	}

	/**
	 * Method called at the end of the parent form validation to complete the edition.
	 * @return One of the status codes indicating if the validation was complete (STATUS_OK), if an
	 *         error occured (STATUS_KO), or if nothing was done by the user (STATUS_IGNORE).
	 */
	public short endValidate()
	{
		if (_areaController instanceof LyFormInterface)
			return ((LyFormInterface)_areaController).endValidate();
		else
			return super.endValidate();
	}

	/**
	 * Internal method for propagating a type change. This methods is called when the type field has
	 * been changed by the user.
	 * <P>The LySetController additionaly impact the view using the setState method.</P>
	 */
	protected void typeChanged()
	{
		_typeChange = true;
		LyApplication application = getApplication();
		LyRelation    typeRelation = _formField.getRelation();
		LyClassInfo   newClassInfo, oldClassInfo;

		newClassInfo = application.getClassInfo(typeRelation.getValue());
		oldClassInfo = _fieldType;

		if ((newClassInfo == oldClassInfo) && !_setController.isChangingObject())
			return;

		LyObject oldObject = null;
		LyObject newObject = null;

		if (!_fromTypeChange)
		{
			newObject = _oldObjects.get(newClassInfo);
			if (_areaController != null)
				oldObject = ((LyFormController)_areaController).getObject();
			else
				oldObject = _structObject;

			if (oldObject != null)
				_oldObjects.put(oldClassInfo, oldObject);
		}
		else
			newObject = _tempNewObject;

		_fieldType = newClassInfo;

		LyFormFieldBuilder builder = (LyFormFieldBuilder)getBuilder();

		LyContext		   context = getControllerContext();
		LyObject		   setObject = getObject();
		LyRelationValue    relationValue = ((setObject == null)
				? null : setObject.getRelationValue((LyRelationFieldInfo)_fieldInfo));

		LyFormField		   formField = builder.buildFormField(false);

		formField.setCaption(null); // caption is not needed
		_formField.setFieldItem(formField.getFieldItem());
		_formField.setFieldUnit(formField.getFieldUnit());
		_formField.setToolBar(formField.getToolBar());

		getAreaController();

		if ((_areaController != null) && (newObject != null))
			((LySetController)_areaController).setObject(newObject);
		else if ((newObject == null) && (_areaController != null))
			newObject = ((LySetController)_areaController).getObject();

		if (_areaController != null)
			_areaController.initialize();

		_oldObjects.remove(newClassInfo);
		setStructObject(newObject);

		if (!_fromTypeChange)
		{
			setFieldValue(((newObject == null) ? null : newObject.getObjectId()), true);
			valueChanged();
		}

		_formField.resetContent();
		_setController.resetFormContent();
		typeRelation.setFocus();

		// Object must be updated after building and updating view.
		if ((relationValue != null) && (relationValue.getValue() != null) && !_fromTypeChange)
		{
			LyObjectId objectId = relationValue.getObjectId(0);
			LyObject   object = objectId.getLogicalObject();

			if ((object == null) || (object.getClassInfo() != _fieldType))
				context.setValue(getObject(), _fieldInfo,
					((newObject == null) ? null : newObject.getObjectId()));
			else if (object != null)
				newObject = object;
		}
		else if ((!_fromTypeChange) && (setObject != null))
			context.setValue(getObject(), _fieldInfo,
				((newObject == null) ? null : newObject.getObjectId()));
	} // end method typeChanged

	/**
	 * Sets formField. Called by LyFormFieldBuilder to avoid loops. Reserved for internal use.
	 * @param formField the form field
	 */
	public void setFormField(LyFormField formField)
	{
		if (_formField == formField)
			return;
		else if (_formField == null)
			super.setFormField(formField);
	}

	/**
	 * Internal method for propagating a unit change. This methods is called when the field unit has
	 * been changed by the user.
	 * <P>The LySetController additionaly impact the view using the setValid method.</P>
	 * @param newUnitId The new fieldinfo unit.
	 * @param oldUnitId The old fieldinfo unit.
	 */
	public void unitChanged(String newUnitId, String oldUnitId)
	{
		_unitChange = true;

		if ((_formField == null) || !_formField.isEnabled())
			return;

		// the new unit id
		LyStructFieldInfo subStruct = (LyStructFieldInfo)_fieldType.getParent();
		LyUnits			  units = subStruct.getUnits();
		LyUnit			  oldUnit = ((oldUnitId == null) ? getUnit() : units.getUnit(oldUnitId));
		LyUnit			  newUnit = ((newUnitId == null) ? getUnit() : units.getUnit(newUnitId));

		// The units are the same
		if (newUnit == oldUnit)
			return;

		// Update the view layer
		LyFieldUnit formFieldUnit = _formField.getFieldUnit();
		LyRelation  relation = (LyRelation)formFieldUnit.getFieldItem();

		relation.selectLink(oldUnit.getId(), false);
		relation.selectLink(newUnit.getId(), true);

		// the field value
		LyFieldInfoValue fieldValue = (LyFieldInfoValue)getFieldValue(oldUnit);

		if (fieldValue == null)
			// An error occured when getting the value.
			return;

		Object  oldValue = fieldValue.getValue();
		Object  newValue = oldValue;
		boolean convert = (units.isConvert()
					|| newUnit.hasEditionFormat()
					|| oldUnit.hasEditionFormat());

		// Convert the value from the reference unit to the new unit.
		if (convert)
			newValue = subStruct.convertToFieldUnit(oldValue, newUnit);

		// Rebuild field item
		LyFieldSetBuilder builder = (LyFieldSetBuilder)getBuilder();

		LyFieldItem		  fieldItem = builder.buildFieldItem(_fieldInfo, newUnit);
		LyToolBar		  toolBar = null;

		if (getAreaController() == null)
			toolBar = builder.updateToolBar(_fieldInfo);

		_formField.setToolBar(toolBar);
		_formField.setFieldItem(fieldItem);
		updateFieldItem(newValue, oldValue, false);
		_formField.resetContent();

		// Reset the form
		_setController.resetFormContent();

		fieldItem = getFieldItem();

		if (fieldItem instanceof LyText)
		{
			LyText  text = (LyText)fieldItem;
			boolean editable = newUnit.hasEditionFormat();

			text.setEditable(editable);
		}
		else if ((getAreaController() != null) && (_structObject != null))
			((LySetController)_areaController).updateObject(_structObject.getValues());
	} // end method unitChanged

	/**
	 * @see leon.control.form.LyFieldSetController#valueChanged()
	 */
	protected void valueChanged()
	{
		LyUnit			  unit = getUnit();
		LyFieldItem		  fieldItem = getFieldItem();
		String			  strValue = null;
		LyStructFieldInfo subStruct = ((_fieldType == null)
				? null : (LyStructFieldInfo)_fieldType.getParent());

		if (unit != null)
		{
			LyFormat format = unit.getFormat();

			if (format != null)
			{
				LyValueSet values = null;

				strValue = ((LyGetLabelInterface)fieldItem).getLabel();
				//if (strValue != null)
				values = (LyValueSet)format.parse(((subStruct == null) ? _fieldInfo : subStruct),
						strValue, unit);

				buildStructObject(values);
				if (isFreed())
					return;
			}
		}

		super.valueChanged();
	} // end method valueChanged

	/**
	 * Get current unit for the specified fieldInfo in the form.
	 * @return current unit, or null if not found or field is disabled.
	 */
	public leon.info.LyUnit getUnit()
	{
		LyUnits units = _fieldInfo.getUnits();
		boolean subField = false;

		if ((_fieldInfo instanceof LyStructFieldInfo)
				&& !(((LyStructFieldInfo)_fieldInfo).isMultiple()))
		{
			subField = true;
			LyStructFieldInfo subStruct = (LyStructFieldInfo)_fieldType.getParent();

			units = subStruct.getUnits();
		}

		if (units == null)
			return null;

		if ((_formField == null) || !_formField.isEnabled())
			return null;

		LyFieldUnit formFieldUnit = _formField.getFieldUnit();

		if (formFieldUnit == null)
			return null;

		LyFieldItem unitItem = formFieldUnit.getFieldItem();
		String	    unitId = unitItem.getValue();
		LyUnit	    unit = units.getUnit(unitId);

		if (unit == null)
		{
			if (subField)
			{
				LyStructFieldInfo subStruct = (LyStructFieldInfo)_fieldType.getParent();

				unit = getDefaultFieldUnit(subStruct);
			}
			else
				unit = getDefaultFieldUnit();
		}

		return unit;
	} // end method getUnit

	/**
	 * Internal method to invoke on a relation a transverse action for a given classInfo.
	 * @param action    Transverse action.
	 * @param classInfo Class info on which the action is applied.
	 */
	public void invokeTransverseAction(LyAction action, LyClassInfo classInfo)
	{
		if (!(_fieldInfo instanceof LyStructFieldInfo)
				|| (((LyStructFieldInfo)_fieldInfo).isMultiple()))
		{
			super.invokeTransverseAction(action, classInfo);
			return;
		}

		LySimpleObjectList list = null;

		if (_areaController != null)
			super.invokeTransverseAction(action, classInfo);

		// Create an empty struct : this struct may be updated by a valueChanged
		// called after the invokeTransverseAction method.
		if (_structObject == null)
			buildStructObject(new LyValueSet());

		LyAction root = action.getRoot();
		String   rootActionId = root.getId();

		// Switch action on set
		if ((_structObject != null)
				&& (rootActionId == LyAction.ID_CREATE))
		{
			LyClassInfo structClassInfo = _structObject.getClassInfo();
			LyAction    structAction = structClassInfo.getActionWithId(LyAction.ID_SET);

			if (structAction != null)
				action = structAction;
		}

		if (_structObject != null)
			list = new LySimpleObjectList(_structObject);

		LyRelationFieldInfo relation = (LyRelationFieldInfo)_fieldInfo;
		LyAreaController    controller = (LyAreaController)invokeAction(action, classInfo, list,
				false);

		if (controller == null)
			return;

		if (controller instanceof LyTransverseInterface)
			((LyTransverseInterface)controller).setRelationParent(this, relation);

		if (relation.ignoresContext())
		{
			if (controller.getContext() != null)
				controller.setContext(null);

			controller.showView();
			return;
		}

		controller.showView();

		// XXX we can change title parameters after the initArea
		// Set the good image and the good title
		LyArea     area = (LyArea)controller.getComponent();
		LyTitleBar titleBar = area.getTitleBar();

		if (titleBar != null)
		{
			if (getEnvironment().testResource("LY_TITLEBAR_LEFT_IMAGE"))
			{
				// Set the good image
				if (rootActionId == LyAction.ID_CREATE)
					titleBar.setImage("LY_ACTION_CREATE");
				else if (rootActionId == LyAction.ID_SET)
					titleBar.setImage("LY_ACTION_SET");
			}

			// Set the good title
			String title = _fieldInfo.getName();

			titleBar.setLeftLabel(title);
			titleBar.setRightLabel(null);
		}
	} // end method invokeTransverseAction

	/**
	 * Internal use. When an object is created in a popup creation form for a relation, the child
	 * controller invokes this method to set the newly created object as the current value of the
	 * relation (if a new object is created we automatically select this value). This method is
	 * never used by the application : use setFieldValue instead
	 * @param object        newly created object. The new value of the relation fieldInfo.
	 * @param relation the relation
	 */
	public void notifyFormResult(LyObject object, LyRelationFieldInfo relation)
	{
		if ((_fieldInfo instanceof LyStructFieldInfo)
				&& (!((LyStructFieldInfo)_fieldInfo).isMultiple()))
		{
			LyFieldItem fieldItem = getFieldItem();

			if (_structObject == null)
				_structObject = object;
			else if (object != _structObject)
			{
				LyValueSet  values = object.getValues();
				LyValue     value;
				LyFieldInfo fieldInfo;
				int		    i, n = values.getSize();

				LyContext   context = getControllerContext();

				context.setLinkUndo(true);

				for (i = 0; i < n; i++)
				{
					value = values.getFieldValue(i);
					fieldInfo = (LyFieldInfo)value.getField();
					if (fieldInfo.hasMark(LyFieldInfo.MARK_ID))
						continue;

					context.setValue(_structObject, fieldInfo, value.getValue());
				}
				context.deleteObject(object);
				context.setObject(_structObject);

				context.setLinkUndo(false);
			}

			if (fieldItem instanceof LyText)
			{
				LyRelationValue value = new LyRelationValue((LyStructFieldInfo)_fieldInfo,
						_structObject);

				updateFieldItem(value.getValue(), null, true);
				valueChanged();
			}
		}
		
		_setController.setActiveField(_fieldInfo);
	} // end method notifyFormResult

	/**
	 * Gets the specific areaController for the given field info.
	 * @return Area controller corresponding to the given field info, null in most cases.
	 */
	public LyAreaController getAreaController()
	{
		if (_areaController != null)
		{
			// If the type has changed, removing the old areaController
			if (_areaController.getClassInfo() != _fieldType)
			{
				LyContext context = ((LySetController)_areaController).getControllerContext();

				context.commit();
				_areaController.free(_formController);
				_formController.removeAreaController(_areaController);
				_areaController = null;
				_typeChange = true;
			}
			else if (!_unitChange)
				return _areaController;
			else
			{
				LyContext context = ((LySetController)_areaController).getControllerContext();

				context.commit();
				_areaController.free(_formController);
				_formController.removeAreaController(_areaController);
				_areaController = null;
			}
		}

		if (_formController.isComposition(_fieldInfo))
		{
			_areaController = buildAreaController(_typeChange);

			LyBuilder builder = getBuilder();

			if ((builder instanceof LyFileBuilder)
					&& (_parentFieldController == null))
			{
				// Check if the view was built with a file builder
				LyFormField formField = getFormField();

				if ((formField != null) && !_fieldInfo.fromFieldTag())
				{
					LyComponent component = formField.getFieldItem();

					if ((component instanceof LyArea)
							&& (((LyArea)component).getPanelCount() > 0))
					{
						LyEnvironment environment = getEnvironment();
						LyAction	  action = _areaController.getAction();

						environment.logWarning("BF AppEngine : action " + action
							+ " should have its own specific view if needed");
					}

					// Replace existing area by sub controller area
					LyArea area = (LyArea)_areaController.getComponent();

					formField.setFieldItem(area);
					_areaController.initialize();
				}
			}
		}
		_unitChange = false;

		return _areaController;
	} // end method getAreaController

	/**
	 * Build an area controller
	 * @param typeChanged indicates if a type change occurs
	 * @return an area controller
	 */
	protected LyAreaController buildAreaController(boolean typeChanged)
	{
		if (_fieldType == null)
			_fieldType = getFieldType();

		if ((_fieldInfo instanceof LyStructFieldInfo)
				&& (((LyStructFieldInfo)_fieldInfo).fromFieldTag())
				&& !(((LyStructFieldInfo)_fieldInfo).isMultiple()))
		{
			LyStructFieldInfo subStruct = (LyStructFieldInfo)_fieldType.getParent();
			LyUnits			  units = subStruct.getUnits();
			LyUnit			  unit = null;

			if ((units != null) && (_formField != null))
			{
				LyFieldUnit formFieldUnit = _formField.getFieldUnit();

				if (formFieldUnit == null)
				{
					_typeChange = false;
					return null;
				}
				LyFieldItem unitItem = formFieldUnit.getFieldItem();
				String	    unitId = unitItem.getValue();

				if (unitId != null)
					unit = units.getUnit(unitId);
			}

			if (unit == null)
				unit = getDefaultFieldUnit();

			if ((unit != null) && (unit.getFormat() != null))
			{
				_typeChange = false;
				return null;
			}
		}

		LyClassInfo  classInfo = getClassInfo();
		LyObjectList list = null;
		LyObjectList context = null;
		LyObject     object = getObject();
		boolean		 fromSubType = false;

		if ((object != null)
				&& classInfo.contains(_fieldInfo))
		{
			if (_oldObjects.get(_fieldType) != null)
			{
				LyObject oldObject = _oldObjects.get(_fieldType);

				if (oldObject != null)
					list = new LySimpleObjectList(oldObject);
				fromSubType = true;
			}
			else if (_unitChange && (_structObject != null))
			{
				list = new LySimpleObjectList(_structObject);
				fromSubType = true;
			}
			else
			{
				LyRelationValue value = object.getRelationValue((LyRelationFieldInfo)_fieldInfo);

				if (value != null)
					list = value.getObjectList();
			}
			context = new LySimpleObjectList(object);
		}

		LySetBehaviorInterface behavior = (LySetBehaviorInterface)getBehavior();
		LyAction			   action = behavior.getAction(_setController,
				(LyRelationFieldInfo)_fieldInfo, _fieldType);

		if (action == null)
		{
			LyAction formAction = getAction();
			String   actionId = ((list != null) ? LyAction.ID_SET : formAction.getRoot().getId());

			if ((list == null) && (actionId != LyAction.ID_CONSULT))
				actionId = LyAction.ID_CREATE;

			// type has changed --> CREATION
			if ((typeChanged
						&& !fromSubType
						&& !_fromTypeChange)
					|| (list == null)
					|| (list.getSize() == 0))
			{
				_typeChange = false;
				actionId = LyAction.ID_CREATE;
				list = null;
			}

			action = _fieldType.getActionWithId(actionId);
			if (action == null)
			{
				_typeChange = false;
				return null;
			}
		}

		if ((list == null) && (action.getMin() > 0))
			list = new LySimpleObjectList();

		LySession session = getSession();

		// We must create the controller with this as parent to manage
		// correctly the internal context (see LySetController.getControllerContext).
		// The parent is set to the form when we call addAreaController.
		_areaController = (LyAreaController)session.invokeAction(this, action, _fieldType, list,
				false);

		if (_areaController != null)
		{
			if (_areaController instanceof LyFormController)
				((LyFormController)_areaController).setCompositionRelation((LyRelationFieldInfo)
					_fieldInfo);
			if (_areaController instanceof LySetController)
				((LySetController)_areaController).setContextInternal();
			_areaController.setViewController(_formController.getViewController());
			if (context != null)
				_areaController.setContext(context);
			
			_formController.addAreaController(_areaController);
			
			LyArea area = (LyArea)_areaController.getComponent();

			area.setScrolling(LyArea.Scrolling.NO);
			_typeChange = false;
		}

		return _areaController;
	} // end method buildAreaController

	/**
	 * Gets the classInfo of the object pointed by the relation in the current form.
	 * @return The classInfo of the object pointed by the relation in the current form.
	 */
	public LyClassInfo getFieldType()
	{
		if (_fieldType != null)
			return _fieldType;

		LyClassInfo		    classInfo = getClassInfo();
		LyRelationFieldInfo relation = (LyRelationFieldInfo)_fieldInfo;
		LyClassInfo		    fieldType = null;
		LyObject		    object = getObject();

		if ((object != null)
				&& classInfo.contains(relation))
		{
			LyRelationValue relationValue = object.getRelationValue(relation);

			if ((relationValue != null) && (relation != null))
			{
				LyObjectId  objectId = (LyObjectId)relationValue.getValue();
				LyClassInfo newClassInfo;

				if (objectId != null)
				{
					newClassInfo = objectId.getClassInfo();
					fieldType = newClassInfo;
				}
			}
		}

		if (fieldType == null)
		{
			LyInfoList<LyClassInfo> fieldTypes = relation.getClassInfos();
			int		   index = 0, nClassInfos = fieldTypes.getSize();

			fieldType = fieldTypes.getInfo(index++);
			while ((fieldType.isAbstract()) && (index < nClassInfos))
				fieldType = fieldTypes.getInfo(index++);
		}

		return fieldType;
	} // end method getFieldType

	/**
	 * Changes current field status to (non-)editable. If the field is not editable, the value is
	 * read-only but however submited with all other values. A disabled field is not submited with
	 * all other values and is just ignored.
	 * @param editable new fieldInfo status. (default is true or obtained from the view
	 *                  description)
	 */
	public void setEditable(boolean editable)
	{
		if ((_formField != null)
				&& (_formField.isEditable() != editable))
		{
			super.setEditable(editable);
			if ((_formField.isEditable() == editable)
					&& (_areaController instanceof LyFormInterface))
				((LyFormInterface)_areaController).setEditable(editable);
		}
	}

	/**
	 * Changes current field status to (non-)enabled. If the field is not editable, and not submited
	 * with all other values.
	 * @param enabled  new fieldInfo status. (default is true or obtained from the view
	 *                  description)
	 */
	public void setEnabled(boolean enabled)
	{
		if ((_formField != null)
				&& (_formField.isEnabled() != enabled))
		{
			super.setEnabled(enabled);
			if (_areaController instanceof LyFormInterface)
				((LyFormInterface)_areaController).setEnabled(enabled);
		}
	}

	/**
	 * Method called when all modifications are committed.
	 */
	public void commit()
	{
		if (_areaController instanceof LyFormInterface)
		{
			((LyFormInterface)_areaController).commit();

			if (_parent instanceof LyFormController)
			{
				LyObject parent = ((LyFormController)_parent).getObject();

				// Update the value in the relation for auto-increment
				if (parent != null)
				{
					LyValue value = getFieldValue();

					if (_parent instanceof LyFormInterface)
					{
						LyContext context = getControllerContext();

						context.setValue(parent, value);
					}
					else
					{
						parent.setFieldValue(value);
						parent.set(false);
					}
				}
			}

			if (_areaController instanceof LyFormController)
			{
				LyClassInfo classInfo = ((LyFormController)_areaController).getClassInfo();

				_oldObjects.remove(classInfo);
			}
		}

		// Suppress all oldObjects.
		if (!_oldObjects.isEmpty())
		{
			Enumeration<LyObject> e = _oldObjects.elements();
			LyObject    object;
			LyContext   context = getControllerContext();

			while (e.hasMoreElements())
			{
				object = (LyObject)e.nextElement();
				context.deleteObject(object);
			}
			_oldObjects.clear();
		}
		_workspace.free(this);
		_workspace = new LySimpleWorkSpace(getSession(), true);
		_workspace.addListener(this);
	} // end method commit

	/**
	 * Method called when editions is cancelled.
	 */
	public void cancel()
	{
		_structObject = null;

		// Do not suppress the original object.
		if (_areaController instanceof LyFormInterface)
		{
			((LyFormInterface)_areaController).cancel();
			if (_areaController instanceof LyFormController)
			{
				LyClassInfo classInfo = ((LyFormController)_areaController).getClassInfo();

				_oldObjects.remove(classInfo);
			}
		}

		// Suppress all oldObjects.
		if (!_oldObjects.isEmpty())
		{
			Enumeration<LyObject> e = _oldObjects.elements();
			LyObject    object;
			LyContext   context = getControllerContext();

			while (e.hasMoreElements())
			{
				object = (LyObject)e.nextElement();
				context.deleteObject(object);
			}
			_oldObjects.clear();
		}
		if (_workspace != null)
			_workspace.free(this);
		_workspace = new LySimpleWorkSpace(getSession(), true);
		_workspace.addListener(this);
	} // end method cancel

	/**
	 * Checks if the value for the fieldInfo has been modified.
	 * @return true if the value has been modified, false otherwise.
	 */
	public boolean checkIfModified()
	{
		if (_areaController instanceof LyFormInterface)
		{
			boolean optional = isOptional();

            if (_areaController instanceof LyCreateController)
            {
    			if (_setController instanceof LyCreateController)
    				optional |= ((LyCreateController)_setController).isCreateIfModified();
    			((LyCreateController)_areaController).setCreateIfModified(optional);
            }
            
			return ((LyFormInterface)_areaController).checkIfModified();
		}

		return super.checkIfModified();
	}

	/**
	 * Called when an object is added in the workspace.
	 * @param  event Event notifying object add.
	 * @return True if the event is used by the listener, false otherwise.
	 */
	public boolean objectAdded(LyWorkSpaceEvent event)
	{
		return true;
	}

	/**
	 * Called when an object is removed from the workspace.
	 * @param  event Event notifying object remove.
	 * @return True if the event is used by the listener, false otherwise.
	 */
	public boolean objectRemoved(LyWorkSpaceEvent event)
	{
		return true;
	}

	/**
	 * Called when an object has changed in the workspace. This call notifies a change on the object
	 * concerned by the event either because its values have changed or because the sort on the
	 * workspace has changed and consequently the object position within the list has changed. These
	 * two cases can easily be distinguished by accessing the values set contained in the event (if
	 * it's not null, this is an change on the object, not a modification of the sort).
	 * <p>Note that object position may also change if one of the modified fields is one of the
	 * fields specified in the list sort.</p>
	 * @param  event Event notifying object change.
	 * @return True if the event is used by the listener, false otherwise.
	 */
	public boolean objectChanged(LyWorkSpaceEvent event)
	{
		if (!_update)
			return true;

		_update = false;

		// Do not remove this call : to create a new context if necessary
		getControllerContext();

		LyObjectId objectId = event.getObject().getObjectId();

		updateFieldItem(objectId, objectId);
		_update = true;
		return true;
	}

	/**
	 * Controls current edition.
	 * @return The error status.
	 */
	public short control()
	{
		if (_areaController instanceof LyCreateController)
		{
			LySetController		   set = (LySetController)_formController;
			LySetBehaviorInterface behavior = set.getSetBehavior();
			boolean				   optional = behavior.isOptional(set, _fieldInfo);

			((LyCreateController)_areaController).setCreateIfModified(optional);
		}

		if (_areaController instanceof LyFormInterface)
			return ((LyFormInterface)_areaController).control();
		else
			return super.control();
	}

	/**
	 * Gets field item used for values input of the controlled field info.
	 * @return the field item
	 */
	protected LyFieldItem getFieldItem()
	{
		LyFormField formField = getFormField();
		LyFieldItem fieldItem = ((formField == null) ? null : formField.getFieldItem());
		LyUnit	    unit = getDefaultFieldUnit();

		if ((_fieldInfo instanceof LyStructFieldInfo)
				&& (((LyStructFieldInfo)_fieldInfo).fromFieldTag())
				&& !(((LyStructFieldInfo)_fieldInfo).isMultiple()))
		{
			LyClassInfo		  subStructClass = getFieldType();
			LyStructFieldInfo subStruct = (LyStructFieldInfo)subStructClass.getParent();

			unit = getDefaultFieldUnit(subStruct);
		}

		if (unit != null)
		{
			LyFormat format = unit.getFormat();

			if ((format != null)
					&& (fieldItem instanceof LyArea))
			{
				// Here fieldItem should be an area
				LyClassInfo classInfo = getClassInfo();

				formField = (LyFormField)formField.getComponent(classInfo.getId()
						+ LyFormController.FORM_FIELD_SEP + _fieldInfo.getId(), true);
				fieldItem = ((formField == null) ? null : formField.getFieldItem());
			}
		}

		return fieldItem;
	} // end method getFieldItem
} // end class LyCompositionFieldUnitSetController
