/**
 * LyFileFieldConsultController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyFileFieldConsultController.java 34972 2012-03-22 15:19:54Z jlsaouli $
 */

package leon.control.form;

import java.io.*;
import java.util.*;

import leon.app.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;
import leon.view.event.*;


/**
 * Field controller for consultation of a LyFileFieldInfo.
 * @see LyConsultController
 */
public class LyFileFieldConsultController extends LyFieldConsultController
{
	/** File version */
	public static final String __VERSION =
		"$Id: LyFileFieldConsultController.java 34972 2012-03-22 15:19:54Z jlsaouli $";

	/** LOCATION REF String */
	public static final String LOCATION_REF = "location://";
	
	/** BLOB REF String */
	public static final String BLOB_REF = "blob://";
	/**
	 * Constructor.
	 * @param parent parent controller
	 * @param fileFieldInfo file field handled by this controller
	 */
	public LyFileFieldConsultController(LyFormController parent,
		LyFileFieldInfo fileFieldInfo)
	{
		super(parent, fileFieldInfo);
	}

	/**
	 * Invoked when an action occurs. At this level, the implementation manages hypertext navigation
	 * on file fieldInfo. Default invokes showLocation on the view manager to open a popup view.
	 * @param demand demand parameters sent by the view component.
	 * @see   LyFormController#actionDemand
	 * @see   LyController#demandPerformed
	 */
	public void actionDemand(LyActionDemand demand)
	{
		String demandId = demand.getValue();

		if ((demand.getType() == LyActionDemand.TYPE_HREF)
				&& (demandId != null))
		{
			// The HRef link may be either a reference to an objet OR
			// a consultation of a file fieldInfo
			String FILE_REF = "file://";
			String HTTP_REF = "http://";
	        String HTTPS_REF = "https://";

			if (demandId.startsWith(FILE_REF)
					|| demandId.startsWith(HTTP_REF)
					|| demandId.startsWith(HTTPS_REF)
					|| demandId.startsWith(LOCATION_REF)
					|| demandId.startsWith(BLOB_REF))
			{
				LyFileFieldInfo fieldInfo = (LyFileFieldInfo)getFieldInfo();

				if ("blob".equals(fieldInfo.getAttachmentHandler())) // old attachments
				{
					// Fetch it !
					LyObject    object = getObject();
					LyFileValue fileValue = object.getFileValue(fieldInfo);

					if (fileValue.fetchAttachment(object))
						demandId = FILE_REF + fileValue.getLocator();
				}
				if ((fieldInfo.getBinding() != null
					&& "BLOB".equals(fieldInfo.getBinding().getPeerType())) // new blob type
					|| ("true".equals(fieldInfo.getData("BLOB"))))//
				{
					// Fetch it !
					LyObject    object = getObject();
					LyFileValue fileValue = object.getFileValue(fieldInfo);

					if (fileValue.getBlobContent(object))
					{

		         String locator = fileValue.getLocator();

		          LyViewManager vm = getViewManager();
		          String label = (String)fileValue.getValue();
		          if (vm.hasProperty("MANAGE_ATTACHMENT"))
		          {
		            // We have a BLOB, this blob is saved to a temporary file and we shall
		            // have an URL to this temporary location
		            String fileDir = vm.getDownloadAction(); // Should be : LyFilePartWriter.FILE_DIR;

		            if (fileDir  == null)
		            {
		              fileDir = vm.getTemporaryLocation();
		              demandId = getEnvironment().getString().expand(fileDir + "/" + label);
		              demandId = demandId.replace('\\', '/');
		            }
		            else
		            {
		              String id = getSession().storeTempFile(locator, label);
		              demandId = fileDir + id;
		            }
		          }
		          else
		          {
		            demandId = FILE_REF + fileValue.getLocator();
		          }
					}
					else
					{
					  //No attachment was found, the locator value is null
					 return;
					}
				}

				if (demandId.startsWith(LOCATION_REF))
					demandId = demandId.substring(LOCATION_REF.length());

				LyViewManager viewManager = getViewManager();
				viewManager.showLocation(getComponent(), demandId, true);
			}
			else
			{
				LyViewManager viewManager = getViewManager();
					viewManager.showLocation(getComponent(), demandId, true);
			}

		}
		else
			_formController.actionDemand(demand);
	} // end method actionDemand

	/**
	 * Update the value of a fieldInfo shown in the view.
	 * @param newValue the new value of the field.
	 * @param oldValue former value of the field,or null if unknown.
	 */
	public void updateValue(Object newValue, Object oldValue)
	{
		// Warning this part is duplicated LyFileFieldSetController
		LyFileFieldInfo	fileField = (LyFileFieldInfo)_fieldInfo;
		String			mimeTypes = fileField.getMimeTypes();
		LyFieldItem		fieldItem = getFieldItem();
		LyEnvironment	env = getEnvironment();

		if ((mimeTypes != null)
				&& mimeTypes.startsWith("image")
				&& !(fieldItem instanceof LyIcon))
			return;
		else if ((mimeTypes == null)
				&& !(fieldItem instanceof LyHRef))
			return;

		String   label;
		LyObject object = getObject();

		if (object != null)
			label = _formController.getLabel(object, fileField, newValue);
		else
			label = fileField.toName(newValue);

		int    width = fileField.getNCols();
		int    height = fileField.getNRows();

		// URL may or may not start with http.
		String url;

		if ((label != null) && (fileField.getAttachmentHandler() != null))
		{
			// XXX Should use the file attachment hanfdler instance here,
			// however, can access view/web packages from here.
			// XX Create interfaces.
			LyViewManager vm = getViewManager();

			if (vm.hasProperty("MANAGE_ATTACHMENT"))
			{
				String docURL = env.getEnv("LY_APP_DOC");
				String attachmentDir = fileField.getAttachmentDirectory();
				
				if (attachmentDir == null)
					attachmentDir = "files";
				
				String fileDir = docURL + "/" + attachmentDir; // Should be : LyFilePartWriter.FILE_DIR;

				url = env.getString().expand(fileDir + "/" + label);
				url = url.replace('\\', '/');
			}
			else
				url = null;
		}
		else if ((label != null)
					&& (((fileField.getBinding() != null) && "BLOB".equals(fileField.getBinding().getPeerType()) )
					|| "true".equals(fileField.getData("BLOB"))))
		{
			LyViewManager vm = getViewManager();
			
			int			  n1;
			LyFileValue fileValue = (LyFileValue)_formController.getFieldValue(fileField);
			String value = label;

			if (fileValue.getBlobContent(object))
			{
				//label = fileValue.getLocator();
				value = fileValue.getLocator();
				url  = LOCATION_REF+label;
			}
			else
			{
				url  = LOCATION_REF+label;
				label = label.replace('\\', '/');
				n1 = label.lastIndexOf('/');
				if (n1 > 0)
					label = label.substring(n1+1);
			}
		
			
			if (vm.hasProperty("MANAGE_ATTACHMENT"))
			{
				// We have a BLOB, this blob is saved to a temporary file and we shall
				// have an URL to this temporary location
				String fileDir = vm.getDownloadAction(); // Should be : LyFilePartWriter.FILE_DIR;
				File testFile = new File(value);
				
				if (!testFile.exists())
				{
					value= vm.getTemporaryDir() + File.separator + label;
					testFile = new File(value);
				}
				if (testFile.exists())
				{
					if (fileDir  == null)
					{
						fileDir = vm.getTemporaryLocation();
						url = env.getString().expand(fileDir + "/" + label);
						url = url.replace('\\', '/');
					}
					else
					{
						LySession session = getSession();
						
						String id = session.storeTempFile(testFile.getAbsolutePath(), label);
						url = fileDir + id;
					}
				}
				else
					url = null;
			}
		}
		else
			url = null;

		if (mimeTypes != null)
		{
			// XX May use various output depending on mime type : textarea, image...
			mimeTypes = mimeTypes.toLowerCase();

			if (mimeTypes.startsWith("image"))
			{
				LyIcon icon = (LyIcon)fieldItem;

				icon.setWidth(width);
				icon.setHeight(height);
				icon.setBorder(LyIcon.Border.SINGLE);

				if (label != null)
				{
					String id = url;

					if (id == null)
					{
						id = fileField.getUrl(env, label);

						if (id != null)
							id = id.replace('\\', '/');
					}
					else if (id.startsWith(LOCATION_REF))
						id = id.substring(LOCATION_REF.length());
					icon.setLabel(label);

					if (id != null)
						icon.setImage(id);
				}
				else
				{
					icon.setLabel("none");
					icon.setImage("LY_DEFAULT_FILE_IMAGE");
				}

				_formController.resetFormContent();
			}
			else if (mimeTypes.equals("text/x-url"))
			{
				LyHRef href = (LyHRef)fieldItem;

				href.setId(LOCATION_REF + label);
				href.setLabel(label);
			}
			else
			{
				// Handle file attachments
				// Clean up in display URL which may contain "__"+date+"__"
				// Link to an attachment.
				LyHRef href = (LyHRef)fieldItem;

				if (url == null)
					href.setId(fileField.getUrl(env, label));
				else
					href.setId(url);
				if ((label != null) && (fileField.getAttachmentHandler() != null))
				{
					int n1, n2;

					n1 = label.indexOf("__");
					if (n1 >= 0)
					{
						n2 = label.indexOf("__", n1 + 1);
						if (n2 >= 0)
							label = label.substring(0, n1) + label.substring(n2 + 2);
					}
				}

				href.setLabel(label);
			}
		}
		else
		{
			LyHRef		  href = (LyHRef)fieldItem;
			LyString	  string = env.getString();
			LyViewManager vm = getViewManager();
			LyFileFieldInfo fieldInfo = (LyFileFieldInfo)getFieldInfo();
			if ((fileField.getBinding() != null)
				&& "BLOB".equals(fileField.getBinding().getPeerType())
				&& vm.hasProperty("MANAGE_ATTACHMENT"))
			{
				href.setId(url);
			}
			else if("true".equals(fieldInfo.getData("BLOB")))
			{
				href.setId(BLOB_REF + label);
			}
			else
			{
			if (fileField.getDirectory() != null)
				href.setId("file://" + fileField.getUrl(env, label));
			else
				href.setId(fileField.getUrl(env, label));
			}
			label = string.expand(label);
			href.setLabel(label);
		}

		// Update the last modification time of the controller.
		setLastModified(new Date().getTime());
	} // end method updateValue
} // end class LyFileFieldConsultController
