/**
 * LyFileFieldSetController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyFileFieldSetController.java 41849 2012-12-26 14:29:55Z bangiole $
 */

package leon.control.form;

import java.util.*;
import leon.app.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.info.LyInfoCache.*;
import leon.misc.*;
import leon.view.*;
import leon.view.getinterface.*;


/**
 * Field controller for modification of a LyFileFielInfo.
 * @see LySetController
 */
public class LyFileFieldSetController extends LyFieldSetController
{
	/** File version */
	public static final String __VERSION =
		"$Id: LyFileFieldSetController.java 41849 2012-12-26 14:29:55Z bangiole $";

	/**
	 * Constructor.
	 * @param parent parent controller
	 * @param fieldInfo file field handled by this controller
	 */
	public LyFileFieldSetController(LySetController parent, LyFileFieldInfo fieldInfo)
	{
		super(parent, fieldInfo);
	}

	/**
	 * @see leon.control.form.LyFieldSetController#getStringValue()
	 */
	protected String getStringValue()
	{
		LyFieldItem     fieldItem = getFieldItem();
		String		    strValue = null;
		LyFileFieldInfo fileField = (LyFileFieldInfo)getFieldInfo();
		String		    directory = fileField.getDirectory();

		if (fieldItem instanceof LyGetLabelInterface)
			strValue = ((LyGetLabelInterface)fieldItem).getLabel();

		if (strValue == null)
			return null;

		// Try to match the begining of the value with an environment variable.
		LyEnvironment environment = getEnvironment();
		LyString	  string = environment.getString();
		String		  docDir, fieldDocDir = null;
		int			  l;

		// We must expand all strings before comparison to have the sames
		// file separators.
		strValue = string.expand(strValue, null);

		String strValueLower = strValue.toLowerCase();

		boolean isBlob = false;
		if ((_fieldInfo.getBinding() != null
				&& "BLOB".equals(_fieldInfo.getBinding().getPeerType())) // new blob type
				|| ("true".equals(_fieldInfo.getData("BLOB"))))//
			isBlob = true;
		docDir = string.expand(environment.getEnv("LY_APP_DOC_DIR"), null).toLowerCase();

		if (directory != null)
			fieldDocDir = string.expand(directory, null).toLowerCase();

		// Key word '_absolute' is used to indicate that value must not be changed.
		if (LyFileFieldInfo.VALUE_ABSOLUTE.equals(directory)
				|| environment.testResource("LY_FILE_KEEP_DIRECTORY"))
			return strValue;
		String tempDir = environment.getViewManager().getTemporaryDir();

		if ((fieldDocDir != null) && strValueLower.startsWith(fieldDocDir))
		{
			l = fieldDocDir.length();
			strValue = strValue.substring(l);
		}
		else if (isBlob && (tempDir != null) && strValueLower.startsWith(tempDir.toLowerCase()))
		{
			l = environment.getViewManager().getTemporaryDir().length();
			//strValue = strValue.substring(l + 1); // remove trailing File separator
		}
		else if (strValueLower.startsWith(docDir))
		{
			l = docDir.length();
			strValue = strValue.substring(l);
			strValue = "$LY_APP_DOC_DIR$" + strValue;
		}
		else
		{
			docDir = string.expand(environment.getEnv("LY_LEON_DOC_DIR"), null).toLowerCase();
			if (strValueLower.startsWith(docDir))
			{
				l = docDir.length();
				strValue = strValue.substring(l);
				strValue = "$LY_LEON_DOC_DIR$" + strValue;
			}
		}

		return strValue;
	} // end method getStringValue

	/**
	 * Internal use : update the value of a fieldInfo in the form.
	 * @param newValue the new value of the field. This value may be null, a LyValue instance or the
	 *                 internal value associated to thge fieldInfo. See LyValue.getValue()
	 * @param oldValue the latest value of the field. This value may be null, a LyValue instance or
	 *                 the internal value associated to thge fieldInfo. See LyValue.getValue()
	 */
	protected void updateValue(Object newValue, Object oldValue)
	{
		if (_setController.isConsult(_fieldInfo))
		{
			// Warning this part is a duplicate from LyFileFieldConsultController
			LyFileFieldInfo	fileField = (LyFileFieldInfo)_fieldInfo;
			String			mimeTypes = fileField.getMimeTypes();
			LyFieldItem		fieldItem = getFieldItem();
			LyEnvironment	env = getEnvironment();

			if ((mimeTypes != null)
					&& mimeTypes.startsWith("image")
					&& !(fieldItem instanceof LyIcon))
				return;
			else if ((mimeTypes == null)
					&& !(fieldItem instanceof LyHRef))
				return;

			String   label;
			LyObject object = getObject();

			if (object != null)
				label = _formController.getLabel(object, fileField, newValue);
			else
				label = fileField.toName(newValue);

			int    width = fileField.getNCols();
			int    height = fileField.getNRows();

			// URL may or may not start with http.
			String url;

			if ((label != null) && (fileField.getAttachmentHandler() != null))
			{
				// XXX Should use the file attachment hanfdler instance here,
				// however, can access view/web packages from here.
				// XX Create interfaces.
				LyViewManager vm = getViewManager();

				if (vm.hasProperty("MANAGE_ATTACHMENT"))
				{
					String docURL = env.getEnv("LY_APP_DOC");
					String fileDir = docURL + "/files"; // Should be : LyFilePartWriter.FILE_DIR;

					url = env.getString().expand(fileDir + "/" + label);
					url = url.replace('\\', '/');
				}
				else
					url = null;
			}
			else if ((label != null)
						&& (((fileField.getBinding() != null) && "BLOB".equals(fileField.getBinding().getPeerType()) )
						|| "true".equals(fileField.getData("BLOB"))))
			{
				LyViewManager vm = getViewManager();

				int			  n1;
				LyFileValue fileValue = (LyFileValue)_formController.getFieldValue(fileField);
				String value = label;

				if (fileValue.getBlobContent(object))
				{
					//label = fileValue.getLocator();
					value = fileValue.getLocator();
					url  = LyFileFieldConsultController.LOCATION_REF+label;
				}
				else
				{
					url  = LyFileFieldConsultController.LOCATION_REF+label;
					label = label.replace('\\', '/');
					n1 = label.lastIndexOf('/');
					if (n1 > 0)
						label = label.substring(n1+1);
				}
				if (vm.hasProperty("MANAGE_ATTACHMENT"))
				{
					// We have a BLOB, this blob is saved to a temporary file and we shall
					// have an URL to this temporary location
					String fileDir = vm.getDownloadAction(); // Should be : LyFilePartWriter.FILE_DIR;

					if (fileDir  == null)
					{
						fileDir = vm.getTemporaryLocation();
						url = env.getString().expand(fileDir + "/" + label);
						url = url.replace('\\', '/');
					}
					else
					{
						LySession session = getSession();

						String id = session.storeTempFile(value, label);
						url = fileDir + id;
					}
				}
			}
			else
				url = null;

			if (mimeTypes != null)
			{
				// XX May use various output depending on mime type : textarea, image...
				mimeTypes = mimeTypes.toLowerCase();

				if (mimeTypes.startsWith("image") && (fieldItem instanceof LyIcon))
				{
					LyIcon icon = (LyIcon)fieldItem;

					icon.setWidth(width);
					icon.setHeight(height);
					icon.setBorder(LyIcon.Border.SINGLE);

					if (label != null)
					{
						String id = url;

						if (id == null)
						{
							id = fileField.getUrl(env, label);

							if (id != null)
								id = id.replace('\\', '/');
						}
						else if (id.startsWith(LyFileFieldConsultController.LOCATION_REF))
							id = id.substring(LyFileFieldConsultController.LOCATION_REF.length());
						icon.setLabel(label);

						if (id != null)
							icon.setImage(id);
					}
					else
					{
						icon.setLabel("none");
						icon.setImage("LY_DEFAULT_FILE_IMAGE");
					}

					_formController.resetFormContent();
				}
				else if (mimeTypes.equals("text/x-url"))
				{
					LyHRef href = (LyHRef)fieldItem;

					href.setId(LyFileFieldConsultController.LOCATION_REF + label);
					href.setLabel(label);
				}
				else
				{
					// Handle file attachments
					// Clean up in display URL which may contain "__"+date+"__"
					// Link to an attachment.
					LyHRef href = (LyHRef)fieldItem;

					if (url == null)
						href.setId(fileField.getUrl(env, label));
					else
						href.setId(url);
					if ((label != null) && (fileField.getAttachmentHandler() != null))
					{
						int n1, n2;

						n1 = label.indexOf("__");
						if (n1 >= 0)
						{
							n2 = label.indexOf("__", n1 + 1);
							if (n2 >= 0)
								label = label.substring(0, n1) + label.substring(n2 + 2);
						}
					}

					href.setLabel(label);
				}
			}
			else
			{
				LyHRef		  href = (LyHRef)fieldItem;
				LyString	  string = env.getString();
				LyViewManager vm = getViewManager();
				LyFileFieldInfo fieldInfo = (LyFileFieldInfo)getFieldInfo();
				if ((fileField.getBinding() != null)
					&& "BLOB".equals(fileField.getBinding().getPeerType())
					&& vm.hasProperty("MANAGE_ATTACHMENT"))
				{
					href.setId(url);
				}
				else if("true".equals(fieldInfo.getData("BLOB")))
				{
					href.setId(LyFileFieldConsultController.BLOB_REF + label);
				}
				else
				{
				if (fileField.getDirectory() != null)
					href.setId("file://" + fileField.getUrl(env, label));
				else
					href.setId(fileField.getUrl(env, label));
				}
				label = string.expand(label);
				href.setLabel(label);
			}

			// Update the last modification time of the controller.
			setLastModified(new Date().getTime());
			return;
		}

		// Default method consists to update label
		LyFieldItem     fieldItem = getFieldItem();
		String		    label = _fieldInfo.toString(newValue);
		LyEnvironment   environment = getEnvironment();
		LyString	    string = environment.getString();
		LyFileFieldInfo fileField = (LyFileFieldInfo)_fieldInfo;
		String		    mimeTypes = fileField.getMimeTypes();

		label = string.expand(label);
        if ((label == null)
                && (fileField.getType() == LyFileFieldInfo.Type.IMAGE))
            label = (String)fileField.getDefaultValue();

		fieldItem.setLabel(label);
		fieldItem.setFieldValue(newValue);

		if (mimeTypes != null)
		{
			// May use various output depending on mime type : textarea, image...
			mimeTypes = mimeTypes.toLowerCase();

			if (mimeTypes.startsWith("image") && (fieldItem instanceof LyFile))
			{
				LyFile	file = (LyFile)fieldItem;

				// Some managers do not display file component as an icon but as a file selector.
				// Do not use the image size for this selector.
				if (getViewManager().hasProperty("MANAGE_IMAGE_FILE_SELECTOR"))
				{
					int		width = fileField.getNCols();
					int		height = fileField.getNRows();

					file.setNCols(width);
					file.setNRows(height);
					file.setBorder(LyFile.Border.SINGLE);
				}
			}
		}
	}

	/**
	 * Free this controller.
	 */
	public void free()
	{
		super.free();
	}

	/**
	 * Add a file to the delete on free list
	 * @param path the file path to add to the delete list
	 */
	public void addTempFilePath(String path)
	{
		if ((path != null)
				&& (getFieldInfo() != null)
				&& (getFieldInfo().getBinding() != null))
		{
			if ("BLOB".equals(getFieldInfo().getBinding().getPeerType())
					|| "CLOB".equals(getFieldInfo().getBinding().getPeerType()))
			{
				LyInfoCache cache = getClassInfo().getCachePolicy();
				if (cache.getType() != CachePolicy.FULL)
				{
					super.addTempFilePath(path);
				}

			}
		}
	}

} // end class LyFileFieldSetController
