/**
 * LyMultiCompositionFieldConsultController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyMultiCompositionFieldConsultController.java 28997 2010-12-23 10:17:15Z bmichel $
 */

package leon.control.form;

import java.lang.reflect.*;
import leon.app.*;
import leon.app.behaviorinterface.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;


/**
 * Field controller for consultation of a multiple composition relation.
 * @see LyConsultController
 */
public class LyMultiCompositionFieldConsultController extends LyCompositionFieldConsultController
{
	/** File version */
	public static final String __VERSION =
		"$Id: LyMultiCompositionFieldConsultController.java 28997 2010-12-23 10:17:15Z bmichel $";

	/**
	 * Constructor. The controller automatically registers on the specified session.
	 * @param parent parent controller
	 * @param fieldInfo relation field handled by this controller
	 */
	public LyMultiCompositionFieldConsultController(LyFormController parent,
		LyRelationFieldInfo fieldInfo)
	{
		super(parent, fieldInfo);
	}

	/**
	 * Free this controller.
	 */
	public void free()
	{
		if (_areaController != null)
		{
			_areaController.free(this);
			_areaController = null;
		}
		super.free();
	}

	/**
	 * Clear view cache.
	 */
	public void clearViewCache()
	{
		if (_areaController != null)
			_areaController.clearViewCache();
		super.clearViewCache();
	}

	/**
	 * @see leon.control.form.LyCompositionFieldConsultController#buildAreaController(boolean)
	 */
	protected LyAreaController buildAreaController(boolean typeChanged)
	{
		if (_fieldType == null)
			_fieldType = getFieldType();

		LyConsultBehaviorInterface behavior = (LyConsultBehaviorInterface)getBehavior();
		LyAction				   action = behavior.getAction((LyConsultController)
				_formController, (LyRelationFieldInfo)_fieldInfo, _fieldType);

		if (action == null)
		{
            LyApplication application = getApplication();
            String        actionId = LyAction.ID_CONSULT_RELATION_TABLE;

            action = _fieldType.getActionWithId(actionId);
			if (action == null)
			{
                String  fieldTypeActionId = _fieldType.getId() + "." + actionId;
                
                action = application.getAction(fieldTypeActionId);
                if (action == null)           
                    action = new LyAction(fieldTypeActionId,
						application.getActionWithId(actionId, null), _fieldType, null);
			}
		}

		if ((action.getControllerClass() != null)
				&& !action.getControllerClass().equals(LyRelationTableController.class))
		{
			LyEnvironment environment = getEnvironment();

			// check the controller has the right type
			Class<?>	controllerClass = action.getControllerClass();

			if (!LyRelationTableController.class.isAssignableFrom(controllerClass))
            {
                if (!LyFormInterface.class.isAssignableFrom(controllerClass))
    				throw new IllegalStateException("BF AppEngine : invalid controller "
    					+ controllerClass + " for action " + action
    					+ ", controller class must extend " + LyRelationTableController.class);
                
                try
                {
                    // Try default invoke
                    _areaController = (LyAreaController)_session.invokeAction(this, action, _fieldType, null, false);
                }
                catch (Exception e)
                {
                    environment.writeException(e);
                    return null;
                }
            }
            else
            {
    			// build parameters and types for reflexion
    			Class<?>[]  types = new Class[7];
    			Object[] parameters = new Object[7];
    
    			types[0] = LySession.class;
    			parameters[0] = getSession();
    
    			types[1] = LyFieldController.class;
    			parameters[1] = this;
    
    			types[2] = LyAction.class;
    			parameters[2] = action;
    
    			types[3] = LyRelationFieldInfo.class;
    			parameters[3] = _fieldInfo;
    
    			types[4] = LyArrayFieldInfo.class;
    			parameters[4] = null;
    
    			types[5] = LyClassInfo.class;
    			parameters[5] = _fieldType;
    
    			types[6] = LyObjectList.class;
    			parameters[6] = null;
    
    			//      Build controller
    			try
    			{
    				Constructor<?> constructor = controllerClass.getConstructor(types);
    
    				_areaController = (LyAreaController)constructor.newInstance(parameters);
    			}
    			catch (NoSuchMethodException e)
    			{
    				// Unknown controller
    				String message = "BF AppEngine : constructor with parameters ("
    					+ types[0] + ", " + types[1] + ", " + types[2] + ", "
    					+ types[3] + ", " + types[4] + ", " + types[5] + ", " + types[6];
    
    				message += ") not found on class " + controllerClass;
    
    				environment.logError(message);
    				environment.writeException(e);
    				return null;
    			}
    			catch (InvocationTargetException e)
    			{
    				environment.writeException(e.getTargetException());
    				return null;
    			}
    			catch (Exception e)
    			{
    				environment.writeException(e);
    				return null;
    			}
            }
		}

		if (_areaController == null)
			_areaController = new LyRelationTableController(getSession(), this, action,
					(LyRelationFieldInfo)_fieldInfo, null, _fieldType, null);

        if (_areaController instanceof LyFormInterface)
            ((LyFormInterface)_areaController).setEditable(false);
        
		_areaController.initialize();

		return _areaController;
	} // end method buildAreaController

	/**
	 * Gets the classInfo of the object pointed by the relation in the current form.
	 * @return The classInfo of the object pointed by the relation in the current form.
	 */
	public LyClassInfo getFieldType()
	{
		LyRelationFieldInfo relation = (LyRelationFieldInfo)_fieldInfo;

		if ((relation.fromFieldTag())
				&& (relation.getType() == LyRelationFieldInfo.Type.COMPOSITION))
			relation = (LyRelationFieldInfo)relation.getTemplate();

		LyClassInfo fieldType = null;
		LyInfoList<LyClassInfo>  classInfos = relation.getClassInfos();
		LyInfoList<LyClassInfo>  subClasses;
		int		    i, n = classInfos.getSize();

		for (i = 0; i < n; i++)
		{
			fieldType = classInfos.getInfo(i);
			subClasses = fieldType.getSubClasses(true);

			if (subClasses == null)
				subClasses = new LyInfoList<LyClassInfo>();
			subClasses.addInfo(fieldType);
			if (subClasses.contains(classInfos))
				break;
		}

		// if no parent classInfo found, get the first one
		if (fieldType == null)
			fieldType = classInfos.getInfo(0);

		return fieldType;
	} // end method getFieldType

	/**
	 * Internal use : update the value of a fieldInfo in the form.
	 * @param newValue the new value of the field. This value may be null, a LyValue instance or the
	 *                 internal value associated to thge fieldInfo. See LyValue.getValue()
	 * @param oldValue the latest value of the field. This value may be null, a LyValue instance or
	 *                 the internal value associated to thge fieldInfo. See LyValue.getValue()
	 */
	protected void updateValue(Object newValue, Object oldValue)
	{
		LyAreaController    areaController = getAreaController();
		LyRelationFieldInfo relationField = (LyRelationFieldInfo)_fieldInfo;
		LyObject		    object = getObject();
		LyObjectList	    context = ((object == null) ? null : new LySimpleObjectList(object));
		LyObjectList	    previousContext = areaController.getContext();

		if (((context == null) && (previousContext != null))
				|| ((context != null) && (previousContext == null))
				|| ((context != null) && (previousContext != null)
					&& !context.equals(previousContext)))
			areaController.setContext(context);
		else if (!relationField.equals(newValue, oldValue))
			areaController.reset();
	}
} // end class LyMultiCompositionFieldConsultController
