/**
 * LyMultiCompositionFieldSetController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyMultiCompositionFieldSetController.java 40730 2012-09-10 14:19:44Z bangiole $
 */

package leon.control.form;

import java.lang.reflect.*;
import java.util.*;

import leon.app.*;
import leon.app.behaviorinterface.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.info.save.*;
import leon.misc.*;


/**
 * Field controller for modification of a multiple composition relation.
 * @see LySetController
 */
public class LyMultiCompositionFieldSetController extends LyCompositionFieldSetController
{
	/** File version */
	public static final String __VERSION =
		"$Id: LyMultiCompositionFieldSetController.java 40730 2012-09-10 14:19:44Z bangiole $";

	/**
	 * Constructor.
	 * @param parent parent controller
	 * @param relation relation field handled by this controller
	 */
	public LyMultiCompositionFieldSetController(LySetController parent,
		LyRelationFieldInfo relation)
	{
		super(parent, relation);
	}

	/**
	 * Free this controller.
	 */
	public void free()
	{
		if (_areaController != null)
		{
			_areaController.free(this);
			_areaController = null;
		}
		super.free();
	}

	/**
	 * Clear view cache.
	 */
	public void clearViewCache()
	{
		if (_areaController != null)
			_areaController.clearViewCache();
		super.clearViewCache();
	}

	/**
	 * @see leon.control.form.LyCompositionFieldSetController#buildAreaController(boolean)
	 */
	protected LyAreaController buildAreaController(boolean typeChanged)
	{
		if (_fieldType == null)
			_fieldType = getFieldType();

		LyEnvironment environment = getEnvironment();

		if (_fieldType == null)
		{
			Exception e = new IllegalArgumentException("[LyMulticompositionFieldSetController] Impossible to build field type (null).");

			environment.logError(e.getMessage());
			environment.writeException(e);
			return null;

		}
		LySetBehaviorInterface behavior = (LySetBehaviorInterface)getBehavior();
		LyAction			   action = behavior.getAction(_setController,
				(LyRelationFieldInfo)_fieldInfo, _fieldType);

		if (action == null)
		{
            LyApplication application = getApplication();
            String        actionId = LyAction.ID_SET_RELATION_TABLE;

            action = _fieldType.getActionWithId(actionId);
            if (action == null)
            {
                String    fieldTypeActionId = _fieldType.getId() + "." + actionId;

                action = application.getAction(fieldTypeActionId);
                if (action == null)
                    action = new LyAction(fieldTypeActionId,
						application.getActionWithId(actionId, null), _fieldType, null);
            }
		}

		if ((action.getControllerClass() != null)
			&& !action.getControllerClass().equals(LyRelationTableController.class))
		{
			// check the controller has the right type
			Class<?>		  controllerClass = action.getControllerClass();

			if (!LyRelationTableController.class.isAssignableFrom(controllerClass))
            {
                if (!LyFormInterface.class.isAssignableFrom(controllerClass))
                    throw new IllegalStateException("BF AppEngine : invalid controller "
                        + controllerClass + " for action " + action
                        + ", controller class must extend " + LyRelationTableController.class);

                try
                {
                    // Try default invoke
                    _areaController = (LyAreaController)_session.invokeAction(this, action, _fieldType, null, false);
                }
                catch (Exception e)
                {
                    environment.writeException(e);
                    return null;
                }
            }
            else
            {
    			// build parameters and types for reflexion
    			Class<?>[]  types = new Class[7];
    			Object[] parameters = new Object[7];

    			types[0] = LySession.class;
    			parameters[0] = getSession();

    			types[1] = LyFieldController.class;
    			parameters[1] = this;

    			types[2] = LyAction.class;
    			parameters[2] = action;

    			types[3] = LyRelationFieldInfo.class;
    			parameters[3] = _fieldInfo;

    			types[4] = LyArrayFieldInfo.class;
    			parameters[4] = null;

    			types[5] = LyClassInfo.class;
    			parameters[5] = _fieldType;

    			types[6] = LyObjectList.class;
    			parameters[6] = null;

    			//      Build controller
    			try
    			{
    				Constructor<?> constructor = controllerClass.getConstructor(types);

    				_areaController = (LyAreaController)constructor.newInstance(parameters);
    			}
    			catch (NoSuchMethodException e)
    			{
    				// Unknown controller
    				String message = "BF AppEngine : constructor with parameters ("
    					+ types[0] + ", " + types[1] + ", " + types[2] + ", "
    					+ types[3] + ", " + types[4] + ", " + types[5] + ", " + types[6];

    				message += ") not found on class " + controllerClass;

                    environment.logWarning(message);
                    environment.writeException(e);

                    return null;
    			}
    			catch (InvocationTargetException e)
    			{
    				environment.writeException(e.getTargetException());
    				return null;
    			}
    			catch (Exception e)
    			{
    				environment.writeException(e);
    				return null;
    			}
            }
		}

		if (_areaController == null)
			_areaController = new LyRelationTableController(getSession(), this, action,
					(LyRelationFieldInfo)_fieldInfo, null, _fieldType, null);

		if (_fieldType.isAbstract())
			((LyRelationTableController)_areaController).forceActionsEnabled();

        if (_areaController instanceof LyFormInterface)
            ((LyFormInterface)_areaController).setEditable(true);

		_areaController.initialize();

		return _areaController;
	} // end method buildAreaController

	/**
	 * Gets the classInfo of the object pointed by the relation in the current form.
	 * @return The classInfo of the object pointed by the relation in the current form.
	 */
	public LyClassInfo getFieldType()
	{
		LyRelationFieldInfo relation = (LyRelationFieldInfo)_fieldInfo;

		if ((relation.fromFieldTag())
				&& (relation.getType() == LyRelationFieldInfo.Type.COMPOSITION))
			relation = (LyRelationFieldInfo)relation.getTemplate();

		LyClassInfo fieldType = null;
		LyInfoList<LyClassInfo>  classInfos = relation.getClassInfos();
		LyInfoList<LyClassInfo>  subClasses;
		int		    i, n = classInfos.getSize();

		for (i = 0; i < n; i++)
		{
			fieldType = classInfos.getInfo(i);
			subClasses = fieldType.getSubClasses(true);

			if (subClasses == null)
				subClasses = new LyInfoList<LyClassInfo>();
			subClasses.addInfo(fieldType);
			if (subClasses.contains(classInfos))
				break;
		}
		// if no parent classInfo found, get the first one
		if (fieldType == null)
			fieldType = classInfos.getInfo(0);

		return fieldType;
	} // end method getFieldType

	/**
	 * Method called when all modifications are committed.
	 */
	public void commit()
	{
		if (_areaController instanceof LyFormInterface)
			((LyFormInterface)_areaController).commit();
	}

	/**
	 * Method called when editions is cancelled.
	 */
	public void cancel()
	{
		if (_areaController instanceof LyFormInterface)
			((LyFormInterface)_areaController).cancel();
	}

	/**
	 * Controls current edition.
	 * @return The error status.
	 */
	public short control()
	{
		if (_areaController instanceof LyFormInterface)
			return ((LyFormInterface)_areaController).control();
		else
			return super.control();
	}

	/**
	 * Clears fieldInfo value : setting value to null.
	 */
	public void clear()
	{
	    LyObject	object = getObject();

	    ((LyRelationTableController)_areaController).cancel();
	    if (object != null)
	    {
	    	LyContext context = getControllerContext();
	        context.setValue(object, _fieldInfo, null);
	    }
	    setFieldValue(null, true);
	}

    /**
     * Save field input.
     */
    public LyFieldSave save(boolean full)
    {
        LyFieldSave              fieldSave = super.save(full);
        LyAreaController         areaController = getAreaController();
        Vector<LyFormSave>  formSaves = null;

        if (areaController instanceof LyRelationTableController)
            formSaves = ((LyRelationTableController)areaController).save(full);

        if ((fieldSave == null) && (formSaves == null))
            return null;
        else if (fieldSave == null)
            return new LyMultiCompositionFieldSave(_fieldInfo, formSaves);
        else
            return new LyMultiCompositionFieldSave(fieldSave, formSaves);
    }

    /**
     * Restore field input.
     */
    public boolean restore(LyFieldSave fieldSave)
    {
        if (!super.restore(fieldSave))
            return false;

        if (!(fieldSave instanceof LyMultiCompositionFieldSave))
            return false;

        LyAreaController    areaController = getAreaController();
        Vector<LyFormSave>  formSaves = ((LyMultiCompositionFieldSave)fieldSave)._formSaves;

        if (areaController instanceof LyRelationTableController)
            ((LyRelationTableController)areaController).restore(formSaves);

        return true;
    }
} // end class LyMultiCompositionFieldSetController
