/**
 * LyRelationAsLinkFieldSetController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyRelationAsLinkFieldSetController.java 41702 2012-12-04 15:04:43Z bangiole $
 */

package leon.control.form;

import leon.app.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;
import leon.view.event.*;


/**
 * Field controller for modification of a simple relation.
 * @see LySetController
 */
public class LyRelationAsLinkFieldSetController extends LyRelationFieldSetController
	implements LyFormResultInterface
{
	/** File version */
	public static final String __VERSION =
		"$Id: LyRelationAsLinkFieldSetController.java 41702 2012-12-04 15:04:43Z bangiole $";

	/**
	 * Constructor.
	 * @param parent the parent controller
	 * @param fieldInfo the relation field handled by this controller
	 */
	public LyRelationAsLinkFieldSetController(LySetController parent,
		LyRelationFieldInfo fieldInfo)
	{
		super(parent, fieldInfo);
	}

	/**
	 * Update the value of a fieldInfo shown in the view. The method is overriden for relation
	 * fieldInfo (updateRelation) and file fieldInfo (updateFile)
	 * @param newValue  the new value of the field.
	 * @param oldValue  former value of the field,or null if unknown. This parameter is not used
	 *                  within generic code.
	 */
	public void updateValue(Object newValue, Object oldValue)
	{
		LyRelationFieldInfo relationField = (LyRelationFieldInfo)_fieldInfo;
		LyFieldItem		    fieldItem = getFieldItem();
		boolean			    multiple = relationField.isMultiple();

		if (!multiple && !(fieldItem instanceof LyHRef))
			return;
		if (multiple && !(fieldItem instanceof LyHRefs))
			return;

		LyObject	  object = null;
		LyClassInfo   classInfo = null;
		LyHRef		  href = null;
		LyEnvironment environment = getEnvironment();
		LyAction	  action = null;
		boolean       addImage = (getEnvironment().getIntResource("LY_SHOW_RELATION_IMAGES") != 0);

		if (!multiple)
		{
			LyObjectId objectId = (LyObjectId)newValue;

			href = (LyHRef)fieldItem;

			if ((objectId == null)
					|| (environment.testResource("LY_HIDE_UNKNOWN_OBJECTS") && (objectId.getLogicalObject() == null)))
			{
				href.setId(null);
				href.setLabel(null);
				href.setImage(null);
				href.setSensitive(false);
			}
			else
			{
				object = objectId.getLogicalObject();
				if (object == null)
					classInfo = objectId.getClassInfo();
				else
					classInfo = object.getClassInfo();

				href.setId((object == null) ? objectId.toString() : object.getId());
				href.setLabel((object == null) ? objectId.toString()
											   : _formController.getName(object));
				if (addImage)
					href.setImage((object == null) ? getImage(classInfo) : getImage(object));

				if (object != null)
				{
					action = classInfo.getActionWithId(LyAction.ID_CONSULT);
					if (action == null)
						action = classInfo.getDefaultAction();
				}

				if ((object == null)
						|| (action == null)
						|| !_session.enableAction(action, this, new LySimpleObjectList(object),
							false)
						|| !environment.testResource("LY_FORM_TRANSLATE_RELATION_TO_LINK"))
					href.setSensitive(false);
				else
					href.setSensitive(true);
			}
		}
		else
		{
			LyObjectId[] objectIds = (LyObjectId[])newValue;
			LyHRefs		 hrefs = (LyHRefs)fieldItem;
			int			 i, n, pos = 0;

			objectIds = relationField.getOrderedIds(objectIds);

			// Remove objects figuring in old list and not in the new one
			n = hrefs.getHRefCount();
			for (i = n - 1; i >= 0; i--)
			{
				href = hrefs.getHRef(i);
				hrefs.removeHRef(href);
			}

			// Add remaining objects still figuring in new objects list
			n = ((objectIds == null) ? 0 : objectIds.length);
			for (i = 0; i < n; i++)
			{
				object = objectIds[i].getLogicalObject();

				if ((object == null) && environment.testResource("LY_HIDE_UNKNOWN_OBJECTS"))
					continue;

				if (object == null)
					classInfo = objectIds[i].getClassInfo();
				else
					classInfo = object.getClassInfo();

				href = new LyHRef();
				href.setId((object == null) ? objectIds[i].toString() : object.getId());
				href.setLabel((object == null) ? objectIds[i].toString()
											   : _formController.getName(object));
				if (addImage)
					href.setImage((object == null) ? getImage(classInfo) : getImage(object));

				if (object != null)
				{
					action = classInfo.getActionWithId(LyAction.ID_CONSULT);
					if (action == null)
						action = classInfo.getDefaultAction();
				}

				if ((object == null)
						|| (action == null)
						|| !_session.enableAction(action, this, new LySimpleObjectList(object),
							false)
						|| !environment.testResource("LY_FORM_TRANSLATE_RELATION_TO_LINK"))
					href.setSensitive(false);
				else
					href.setSensitive(true);

				hrefs.insertHRef(href, pos);
				pos++;
			}
		}
		checkTransverseToolbar();
		// Update the last modification time of the controller.
		setLastModified(System.currentTimeMillis());
	} // end method updateVal

	/**
	 * Internal use. When an object is created in a popup creation form for a relation, the child
	 * controller invokes this method to set the newly created object as the current value of the
	 * relation (if a new object is created we automatically select this value). This method is
	 * never used by the application : use setFieldValue instead
	 * @param object        newly created object. The new value of the relation fieldInfo.
	 * @param relationField the relation
	 */
	public void notifyFormResult(LyObject object, LyRelationFieldInfo relationField)
	{
		LyFieldItem fieldItem = _formField.getFieldItem();

		if ((!(fieldItem instanceof LyHRef)) && (!(fieldItem instanceof LyHRefs)))
			return;

		if (fieldItem instanceof LyHRef)
		{
			if (object == null)
				updateValue(null, null);
			else
				updateValue(object.getObjectId(), null);
		}
		else
		{
			if (object == null)
				updateValue(null, null);
			else
			{
				LyObjectId[]	oldValue = (LyObjectId[])getValue();
				int				newSize = (oldValue != null ? oldValue.length + 1 : 1);
				LyObjectId[]	newValue = new LyObjectId[newSize];

				if (oldValue != null)
				{
					for (int i = 0; i < oldValue.length; i++)
						newValue[i] = oldValue[i];
				}

				newValue[newValue.length - 1] = object.getObjectId();
				updateValue(newValue, oldValue);
			}
		}

		valueChanged();

		checkTransverseToolbar();
		_formController.setActiveField(relationField);
	} // end method notifyFormResult

	/**
	 * Extract current value for the specified fieldInfo in the form.
	 * @return current value, or null if not found or field is disabled.
	 */
	public String getStringValue()
	{
		// If a temporary value is set, return this value (needed for edit tables when cell
		// is no longer in edition mode).
		if ((getParent() instanceof LyEditTableSetController) && (_temporaryValue != null))
			return _temporaryValue;

		LyFieldItem   fieldItem = getFieldItem();
		String		  strValue = null;
		LyApplication application = getApplication();

		if (fieldItem instanceof LyHRef)
		{
			LyHRef 	href = (LyHRef)fieldItem;

			strValue = href.getId();
		}
		else if (fieldItem instanceof LyHRefs)
		{
			LyHRefs		 hrefs = (LyHRefs)fieldItem;
			int			 i, n;

			n = hrefs.getHRefCount();

			StringBuffer sb = null;

			for (i = 0; i < n; i++)
			{
				LyHRef 	href = hrefs.getHRef(i);

				if (sb == null)
					sb = new StringBuffer(href.getId());
				else
					sb.append(application.VALUE_SEP).append(href.getId());
			}

			if (sb != null)
				strValue = sb.toString();
		}

		return strValue;
	} // end method getStringValue

	/**
	 * Invoked when an action occurs. At this level, the implementation manages<BR>
	 * DEMAND_VALIDATE demands. (demand.getType = LyActionDemand.TYPE_ACTION AND demand.getSource()
	 * = LyController.DEMAND_VALIDATE)<BR>
	 * DEMAND_APPLY demands. (demand.getType = LyActionDemand.TYPE_ACTION AND demand.getSource() =
	 * LyController.DEMAND_APPLY)<BR>
	 * Default action (double click)<BR>
	 * Default action (double click)<BR>
	 * VALUE_CHANGED demands. (demand.getType = LyActionDemand.TYPE_VALUE_CHANGED
	 * @param demand the action demand
	 * @see   LyController#demandPerformed
	 */
	public void actionDemand(LyActionDemand demand)
	{
		String demandId = demand.getValue();

		if ((demand.getType() == LyActionDemand.TYPE_HREF)
				&& (demandId != null))
		{
			LyObjectId oid = getObjectId(demandId);
			LyObject   object = oid.getLogicalObject();

			if (object != null)
				consult(object);
			return;
		}

		super.actionDemand(demand);
	}
} // end class LyRelationFieldSetController
