/**
 * LyRelationFieldSetController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyRelationFieldSetController.java 33020 2011-11-02 16:51:58Z jlsaouli $
 */

package leon.control.form;

import java.util.*;

import leon.app.*;
import leon.app.behaviorinterface.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;


/**
 * Field controller for modification of a simple relation.
 *
 * @see LySetController
 */
public class LyRelationFieldFilterController extends LyRelationFieldController
	implements LyFormResultInterface
{
	/** File version */
	public static final String			  __VERSION = "$Id: LyRelationFieldSetController.java 33020 2011-11-02 16:51:58Z jlsaouli $";

	/**  */
	protected _LyRelationFilterDomainController _relationDomainController;

	/**
	 * Temporary value used when typed value does not match any link value (may happen with
	 * auto-completion mode).
	 */
	protected String					  _temporaryValue;

	/**
	 * Constructor.
	 * @param parent parent controller
	 * @param fieldInfo relation field handled by this controller
	 */
	public LyRelationFieldFilterController(LyAreaController parent,
		LyRelationFieldInfo fieldInfo)
	{
		super(parent, fieldInfo);
		_temporaryValue = null;
	}

	/**
	 * Gets name of an object in the context of the given area controller.
	 *
	 * @param  object Object for which its name is required.
	 * @return Name of the object.
	 */
	protected String getName(LyObject object)
	{
		return ((LyAreaController)_parent).getName(object);
	}

	/**
	 * Internal use : update the value of a fieldInfo in the form.
	 *
	 * @param newValue the new value of the field. This value may be null, a LyValue instance or the
	 *                 internal value associated to the fieldInfo. See LyValue.getValue()
	 * @param oldValue the latest value of the field. This value may be null, a LyValue instance or
	 *                 the internal value associated to the fieldInfo. See LyValue.getValue()
	 */
	protected void updateValue(Object newValue, Object oldValue)
	{
		LyRelationFieldInfo relationField = (LyRelationFieldInfo)_fieldInfo;
		LyFieldItem		    fieldItem = getFieldItem();

		if (!(fieldItem instanceof LyRelation)
				|| (fieldItem.getParent() == null))
			return;

		LyRelation relation = (LyRelation)fieldItem;
		boolean    wasIdle = isIdle();

		setIdle(true);
		relation.setIdle(true);
		Object		 relationValue;
		LyObjectList list = null, tempo_list;

		if ((newValue != null) && (newValue instanceof Vector<?>))
		{
			Vector<?> vect = (Vector<?>)newValue;

			for (int vv = 0; vv < vect.size(); vv++)
			{
				relationValue = vect.elementAt(vv);
				tempo_list = relationField.getObjects(relationValue);
				if ((tempo_list == null)
						|| (tempo_list.getSize() == 0))
					continue;
				if (list == null)
					list = tempo_list;
				else
					list.addList(tempo_list);
			}
		}
		else
			list = relationField.getObjects(newValue);

		// Get relation domain and selection
		LyFilterController     filterController = (LyFilterController)getParent();
		LyFilterBehaviorInterface behavior = (LyFilterBehaviorInterface)filterController.getBehavior();
		LyObjectList		   relationDomain = behavior.getFilterRelationDomain(filterController, relationField);
		LyObjectList		   domain = relationDomain;

		// FFT00251 : sort links as in the set and creation forms
		if (domain != null)
			domain.setSortByName(true);

		boolean		 multiple = relationField.isMultiple();

		// Update domain and selection
		LyObject     object;

		LyLink		 link;

		int			 pos;
		boolean		 selected;
		int			 nList = ((list == null) ? 0 : list.getSize());
		LyLink[]	 links = new LyLink[nList];
		int			 i, n = ((domain == null) ? 0 : domain.getSize());

		// Add images only if multiple classes
		int			 nClasses = relationField.getClassInfoCount();
		LyEnvironment environment = getEnvironment();
		boolean		 addImage = (nClasses
					>= environment.getIntResource("LY_SHOW_RELATION_IMAGES"));

		// first remove previous link
		int nbLink = relation.getLinkCount();

		for (i = 0; i < nbLink; i++)
		{
			relation.removeLink(relation.getLink(0));
		}

		if  (!multiple)
		{
	        String empty_value = "LY_RELATION_EMPTY_VALUE";
            String val = environment.translate(empty_value);

            if ((val != null) && !val.equals("_null"))
            {
                if ((val != null)
                        && val.equals("_null"))
                    empty_value = null;

                link = new LyLink("LY_NO_CHOICE", empty_value);
                relation.addLink(link);
            }
		}

		for (i = 0; i < n; i++)
		{
			object = domain.getObject(i);
			selected = ((list != null) && list.contains(object));

			if (relationField.hasMark(LyFieldInfo.MARK_TRANSLATE))
			    link = new LyLink(object.getId(), getName(object));
			else
			    link = new LyLink(object.getId(), "$$" + getName(object));

			if (selected)
			{
				pos = list.getIndex(object);
				links[pos] = link;
			}

			if (addImage)
				link.setImage(_parent.getImage(object));

			// FFT00252 : keep order in relation
			if (!selected)
				relation.addLink(link);
		}

		// FFT00252 : keep order in relation
		for (i = 0; i < nList; i++)
		{
			if (links[i] != null)
			{
				relation.addLink(links[i]);
				links[i].setSelected(true);
			}
		}


		if (relationDomain != null)
			// Create relation controller to maintain domain valid
			new _LyRelationFilterDomainController(relationDomain);

		relation.setIdle(false);
		setIdle(wasIdle);

		// Update the last modification time.
		setLastModified(new Date().getTime());
	} // end method updateValue



	/**
	 * Clears all cached data for this controller (especially the view and the component).
	 *
	 * <P>This method may be invoked for memory optimization, but the controller is still valid and
	 * may be used. (Unlike free) Further usage, needs to rebuild internal component & view which
	 * may be time consuming.</P>
	 */
	public void clearViewCache()
	{
		if (_relationDomainController != null)
		{
			_relationDomainController.free();
			_relationDomainController = null;
		}
		super.clearViewCache();
	}

	/**
	 * Free this controller.
	 *
	 */
	public void free()
	{
		clearViewCache();
		super.free();
	}

	/**
	 * Internal class to create a popup creation form for a relation fieldInfo.
	 */
	public class _LyRelationFilterDomainController
		implements LyDataSourceListener,
			LyWorkSpaceListener
	{
		LyRelationFieldInfo _relation;
		LyObjectList	    _domain;

		/**
		 * Constructor.
		 *
		 * @param domain        context domain to set the values in the popup form.
		 */
		protected _LyRelationFilterDomainController(LyObjectList domain)
		{
			_relation = (LyRelationFieldInfo)_fieldInfo;
			if (domain == null)
				_domain = null;
			else if (domain.isDynamic())
				_domain = domain;
			else
				_domain = new LySimpleObjectList(domain);

			if ((_domain != null) && _domain.isDynamic())
				((LyWorkSpace)_domain).addListener(this);
			else
			{
				// Listen to relation classes
				int			   i, n = _relation.getClassInfoCount();
				LyClassInfo    classInfo;
				LyObjectSource source;
				LySession	   session = getParent().getSession();

				for (i = 0; i < n; i++)
				{
					classInfo = _relation.getClassInfo(i);
					source = session.getObjectSource(classInfo);
					if (source != null)
						source.addListener(this);
				}
			}

			// Add this to relation controllers list
			if (_relationDomainController != null)
				// We already have a relation controller, free it
				_relationDomainController.free();

			_relationDomainController = this;
		}

		/**
		 * Free this controller.
		 */
		protected void free()
		{
			if ((_domain != null) && _domain.isDynamic())
				((LyWorkSpace)_domain).free(this);
			else
			{
				// Stop listening to relation sources
				int			   i, n = _relation.getClassInfoCount();
				LyClassInfo    classInfo;
				LyObjectSource source;
				LySession	   session = getParent().getSession();

				for (i = 0; i < n; i++)
				{
					classInfo = _relation.getClassInfo(i);
					source = session.getObjectSource(classInfo);
					if (source != null)
						source.removeListener(this);
				}
			}
		}

		/**
		 * Checks if this controller is idle.
		 * @return the idle status
		 */
		public boolean isIdle()
		{
			return LyRelationFieldFilterController.this.isIdle();
		}

		/**
		 * Called when an object is added in the workspace.
		 *
		 * @param  event Event notifying object add.
		 * @return True if the event is used by the listener, false otherwise.
		 */
		public boolean objectAdded(LyWorkSpaceEvent event)
		{
			if (isIdle())
				return false;

			LyObject object = event.getObject();
			int		 index = _domain.getIndex(object);

			addObject(object, index);
			return true;
		}

		/**
		 * Method for receiving an object creation.
		 *
		 * @param  event Event notifying the creation of an object.
		 * @return Boolean indicating if the event was used, false otherwise.
		 */
		public boolean objectCreated(LyDataSourceEvent event)
		{
			if (isIdle())
				return false;

			LyObject object = (LyObject)event.getObject();

			if ((_domain != null) && _domain.contains(object))
				return false;

			LyFilterController filterController = (LyFilterController)getParent();
			LyFilterBehaviorInterface behavior = (LyFilterBehaviorInterface)filterController.getBehavior();


			LyObjectList domain = behavior.getFilterRelationDomain(filterController, _relation);
			int			 oldSize, newSize;

			// If domain becomes null, we may have reached the select relation limit (given by
			// the LY_SELECT_RELATION_COUNT and LY_SELECT_BY_NAME_RELATION_COUNT resources).
			// In this case, add the object to the old domain to avoid getting the relation
			// without this object and without any select action.
			if ((_domain != null) && (domain == null))
			{
				((LySimpleObjectList)_domain).addObject(object);

				// Force the domain to be sorted in order to place the newly created object at the
				// right place.
				_domain.setSorted(true);
				addObject(object, _domain.getIndex(object));
				return true;
			}

			oldSize = ((_domain == null) ? 0 : _domain.getSize());
			newSize = ((domain == null) ? 0 : domain.getSize());

			int index = ((domain == null) ? -1 : domain.getIndex(object));

			if (index < 0)
				return false;
			else
			{
				if (oldSize != (newSize - 1))
				{
					// We must rebuild the relation.
					for (int i = 0; i < oldSize; i++)
					{
						object = _domain.getObject(i);
						if (!domain.contains(object))
							removeObject(object);
					}

					for (int i = 0; i < newSize; i++)
					{
						object = domain.getObject(i);
						if ((_domain != null)
								&& !_domain.contains(object))
							addObject(object, 0);
					}

					_domain = new LySimpleObjectList(domain);
				}
				else
				{
					// Update domain
					_domain = new LySimpleObjectList(domain);

					addObject(object, index);
				}
			}

			return true;
		} // end method objectCreated

		/**
		 * Adds the object at the given position.
		 *
		 * @param object Object to add to domain.
		 * @param index  Position where the object is inserted.
		 */
		void addObject(LyObject object, int index)
		{
			// Add a link for the new object
			LyRelation relation = (LyRelation)_formField.getFieldItem();
			LyLink     link = new LyLink(object.getId(), getLinkLabel(object));
			int		   offset = 0;

			// Add images only if multiple classes
			int     nClasses = _relation.getClassInfoCount();
			boolean addImage = (nClasses
						>= getEnvironment().getIntResource("LY_SHOW_RELATION_IMAGES"));
			int        nr = relation.getLinkCount();

			if (addImage)
				link.setImage(getImage(object));

			if (_relation.isMultiple())
				// FFT0172 : if the relation is multiple, the object should be
				// added at the end of the selection box.
				index = -1;
			else if ((relation.getLinkCount() > 0)
					&& relation.getLink(0).getId().equals("LY_NO_CHOICE"))
				offset = 1;

			if (index + offset >= nr)
			    relation.addLink(link);
			else
			    relation.insertLink(link, index + offset);

			// Update the last modification time of the set controller.
			getParent().setLastModified(new Date().getTime());
		} // end method addObject

		/**
		 * Called when an object is removed from the workspace.
		 *
		 * @param  event Event notifying object remove.
		 * @return True if the event is used by the listener, false otherwise.
		 */
		public boolean objectRemoved(LyWorkSpaceEvent event)
		{
			if (isIdle())
				return false;

			LyObject object = event.getObject();

			removeObject(object);
			return true;
		}

		/**
		 * Method for receiving an object deletion.
		 *
		 * @param  event Event notifying the deletion of an object.
		 * @return Boolean indicating if the event was used, false otherwise.
		 */
		public boolean objectDeleted(LyDataSourceEvent event)
		{
			if (isIdle())
				return false;

			LyObject object = (LyObject)event.getObject();

			if ((_domain == null) || !_domain.contains(object))
				return false;

			removeObject(object);
			((LySimpleObjectList)_domain).removeObject(object);
			return true;
		}

		/**
		 * Removes given object from relation domain.
		 *
		 * @param object Object to remove.
		 */
		void removeObject(LyObject object)
		{
			// Retrieve corresponding Link and remove it
			LyRelation relation = (LyRelation)_formField.getFieldItem();
			LyLink     link;
			int		   i, n = relation.getLinkCount();
			String     id;

			for (i = 0; i < n; i++)
			{
				link = relation.getLink(i);
				id = link.getId();
				if ((id != null) && id.equals(object.getId()))
				{
					// If the link is selected, unselected before --> avoid automatically selecting an other link
					// Flandres/FFT00448
					if (link.isSelected())
						relation.selectLink(id, false);
					relation.removeLink(link);
					break;
				}
			}

			// Update the last modification time of the set controller.
			getParent().setLastModified(new Date().getTime());
		}

		/**
		 * Called when an object has changed in the workspace. This call notifies a change on the
		 * object concerned by the event either because its values have changed or because the sort
		 * on the workspace has changed and consequently the object position within the list has
		 * changed. These two cases can easily be distinguished by accessing the values set
		 * contained in the event (if it's not null, this is an change on the object, not a
		 * modification of the sort).
		 *
		 * <p>Note that object position may also change if one of the modified fields is one of the
		 * fields specified in the list sort.</p>
		 *
		 * @param  event Event notifying object change.
		 * @return True if the event is used by the listener, false otherwise.
		 */
		public boolean objectChanged(LyWorkSpaceEvent event)
		{
			if (isIdle())
				return false;

			LyObject   object = event.getObject();
			LyValueSet values = event.getValues();
			int		   index = event.getIndex();
			int		   oldIndex = event.getIndex();

			if (index == oldIndex)
				updateObject(object, values);
			else
			{
				removeObject(object);
				addObject(object, index);
			}

			return true;
		}

		/**
		 * Method for receiving an object modification.
		 *
		 * @param  event Event notifying the modification of an object.
		 * @return Boolean indicating if the event was used, false otherwise.
		 */
		public boolean objectModified(LyDataSourceEvent event)
		{
			if (isIdle())
				return false;

			LyObject	object = (LyObject)event.getObject();

			if ((_domain == null) || !_domain.contains(object))
				return objectCreated(event);

			// We're only interested in name changes
			LyValueSet values = event.getValues();

			updateObject(object, values);
			return true;
		} // end method objectModified

		/**
		 * Updates the given object in relation domain.
		 *
		 * @param object Object that is changed.
		 * @param values Changes made on the object.
		 */
		void updateObject(LyObject object, LyValueSet values)
		{
			if (values == null)
				// Nothing to do.
				return;

			if (!values.hasMark(LyFieldInfo.MARK_ID)
					&& !values.hasMark(LyFieldInfo.MARK_NAME))
				// Nothing to do
				return;

			// Retrieve corresponding Link and change its name or its id
			LyApplication application = getApplication();
			LyRelation    relation = (LyRelation)_formField.getFieldItem();
			LyLink		  link, sortLink;
			int			  i, j, n = relation.getLinkCount();
			String		  id, currentLinkId;
			LyObject	  currentLinkObject;
			LyObjectId    currentLinkObjectId;
			int			  offset = (_relation.isMultiple() ? 0 : 1);

			if (values.hasMark(LyFieldInfo.MARK_ID))
			{
				LyObjectId objectId;
				LyLink     newLink;

				for (i = offset; i < n; i++)
				{
					link = relation.getLink(i);
					id = link.getId();
					if (id.equals("LY_NO_CHOICE"))
						continue;

					objectId = new LyObjectId(application, id);
					if (objectId.getLogicalObject() == null)
					{
						relation.removeLink(link);
						newLink = new LyLink(object.getId(), getLinkLabel(object));
						newLink.setImage(link.getImage());
						relation.insertLink(newLink, i);
						if (link.isSelected())
							relation.selectLink(i, true);

						// Update the last modification time of the set controller.
						getParent().setLastModified(new Date().getTime());
						break;
					}
				}
			}

			for (i = offset; i < n; i++)
			{
				link = relation.getLink(i);
				id = link.getId();

				if ((id != null) && id.equals(object.getId()))
				{
					if (link.isSelected() && _relation.isMultiple())
					{
						// Just change link label, not its position in selections list
						link.setLabel(getLinkLabel(object));

						// XXX Force redisplay, changing label is not enough
						relation.removeLink(link);
						relation.insertLink(link, i);
						break;
					}

					// We are going to change the link label, we have to change its place
					// firts we remove it from the relation...
					relation.removeLink(link);

					// we change the link Label
					link.setLabel(getLinkLabel(object));

					// then add it at the right place
					for (j = offset; j < (n - 1); j++)
					{
						sortLink = relation.getLink(j);
						currentLinkId = sortLink.getId();
						currentLinkObjectId = new LyObjectId(application, currentLinkId);
						currentLinkObject = currentLinkObjectId.getLogicalObject();
						if (currentLinkObject.compare(object, true) >= 0)
						{
							relation.insertLink(link, j);
							if (link.isSelected())
								relation.selectLink(j, true);
							break;
						}
					}

					if (j == (n - 1))
					{
						relation.addLink(link);
						if (link.isSelected())
							relation.selectLink(j, true);
					}
					break;
				}
			}

			if (i == n)
				// Object not found => full reset
				updateValue(getFieldValue(), null);

			// Update the last modification time of the set controller.
			getParent().setLastModified(new Date().getTime());
		} // end method updateObject
	} // end class _LyRelationDomainController

	/**
	 * Internal use. When an object is created in a popup creation form for a relation, the child
	 * controller invokes this method to set the newly created object as the current value of the
	 * relation (if a new object is created we automatically select this value). This method is
	 * never used by the application : use setFieldValue instead
	 *
	 * @param object        newly created object. The new value of the relation fieldInfo.
	 * @param relationField the relation
	 */
	public void notifyFormResult(LyObject object, LyRelationFieldInfo relationField)
	{
		_temporaryValue = null;

		LyFieldItem fieldItem = _formField.getFieldItem();

		if (!(fieldItem instanceof LyRelation))
			return;

		LyRelation relation = (LyRelation)fieldItem;
		LyLink     link;
		int		   i, n = relation.getLinkCount();
		String     id;
		boolean    multiple = relation.isMultiple();
		boolean    found = false;

		for (i = 0; i < n; i++)
		{
			link = relation.getLink(i);
			id = link.getId();

			if ((object != null)
					&& (id != null)
					&& id.equals(object.getId()))
			{
				link.setSelected(true);
				found = true;
				if (multiple)
					break;
			}
			else if ((!multiple) && (link.isSelected()))
				link.setSelected(false);
		}

		if (!found)
		{
			link = new LyLink(object.getId(), getLinkLabel(object));

			// Add images only if multiple classes
			int     nClasses = relationField.getClassInfoCount();
			boolean addImage = (nClasses
						>= getEnvironment().getIntResource("LY_SHOW_RELATION_IMAGES"));

			if (addImage)
				link.setImage(getImage(object));

			relation.addLink(link);
			link.setSelected(true);
		}
	} // end method notifyFormResult

	/**
	 * Gives the label for the link related to an object.<br>
	 * Add $$ to avoid translations except if the relation has translate mark.
	 *
	 * @param  object object displayed in a relation.
	 * @return a string.
	 */
	protected String getLinkLabel(LyObject object)
	{
		boolean   translate = _fieldInfo.hasMark(LyFieldInfo.MARK_TRANSLATE);

		if (translate)
			return _formController.getName(object);
		else
			return "$$" + _formController.getName(object);
	}
} // end class LyRelationFieldSetController
