/**
 * LyRelationFieldSetController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyRelationFieldSetController.java 40879 2012-09-20 07:36:55Z bangiole $
 */

package leon.control.form;

import java.util.*;
import leon.app.*;
import leon.app.behaviorinterface.*;
import leon.control.*;
import leon.control.form.builder.*;
import leon.data.*;
import leon.info.*;
import leon.view.*;
import leon.view.event.*;


/**
 * Field controller for modification of a simple relation.
 *
 * @see LySetController
 */
public class LyRelationFieldSetController extends LyFieldSetController
	implements LyFormResultInterface
{
	/** File version */
	public static final String			  __VERSION = "$Id: LyRelationFieldSetController.java 40879 2012-09-20 07:36:55Z bangiole $";

	/**  */
	protected _LyRelationDomainController _relationDomainController;

	/**
	 * Temporary value used when typed value does not match any link value (may happen with
	 * auto-completion mode).
	 */
	protected String					  _temporaryValue;

	/**
	 * Constructor.
	 * @param parent parent controller
	 * @param fieldInfo relation field handled by this controller
	 */
	public LyRelationFieldSetController(LySetController parent,
		LyRelationFieldInfo fieldInfo)
	{
		super(parent, fieldInfo);
		_temporaryValue = null;
	}

	/**
	 * Invoked when an action occurs. At this level, the implementation manages<BR>
	 * DEMAND_VALIDATE demands. (demand.getType = LyActionDemand.TYPE_ACTION AND demand.getSource()
	 * = LyController.DEMAND_VALIDATE)<BR>
	 * DEMAND_APPLY demands. (demand.getType = LyActionDemand.TYPE_ACTION AND demand.getSource() =
	 * LyController.DEMAND_APPLY)<BR>
	 * Default action (double click)<BR>
	 * Default action (double click)<BR>
	 * VALUE_CHANGED demands. (demand.getType = LyActionDemand.TYPE_VALUE_CHANGED
	 *
	 * @param demand the action demand
	 * @see   LyController#demandPerformed
	 */
	public void actionDemand(LyActionDemand demand)
	{
		if ((demand.getType() == LyActionDemand.TYPE_ACTION)
				&& (demand.getValue() != null)
				&& (demand.getValue().indexOf(Character.toString(LyObjectId.CLASS_SEP)) > 0))
		{
			// Consult object
			LyObjectId oid = new LyObjectId(getApplication(), demand.getValue());
			LyObject   object = (LyObject)oid.getObject();

			if (object != null)
				consult(object);
		}
		else if (demand.getType() == LyActionDemand.TYPE_VALUE_CHANGED)
		{
			String value = demand.getValue();

			_temporaryValue = null;
			checkTransverseToolbar();

			// Only for a simple relation check if given value match an existing object
			// (in auto-completion mode value may not match).
			if (!_fieldInfo.isMultiple()
					&& (value != null)
					&& (value.length() > 0)
					&& !"LY_NO_CHOICE".equals(value))
			{
				LyObjectId oid = null;

				try
				{
					oid = new LyObjectId(getApplication(), value);
				}
				catch (Exception e)
				{
				}

				LyObject object = ((oid == null) ? null : (LyObject)oid.getLogicalObject());

				if (object == null)
				{
					int	pos = value.indexOf(LyObjectId.CLASS_SEP);

					if (pos != -1)
					{
						LyError  error;
						LyObject parentObject = _setController.getObject();

						error = new LyError(LyObject.LEVEL_ERROR, getEnvironment().getMessage("LY_ER_FIELD_SET_VALUE",
								_fieldInfo.getName(), value), null, _fieldInfo, object, _fieldInfo);
						showError(parentObject, error);

	                    // Setting the form field to invalid to stop the process.
	                    setState(LyFormField.STATE_ERROR);
						return;
					}
					else
						_temporaryValue = value;
				}
			}
		}
		else if ((demand.getType() == LyActionDemand.TYPE_COMPLETE_VALUE)
				|| (demand.getType() == LyActionDemand.TYPE_EXPAND))
		{
			LySetBehaviorInterface behavior = _setController.getSetBehavior();
			LyRelationFieldInfo    relationFieldInfo = (LyRelationFieldInfo)_fieldInfo;

			// If completion is managed locally by the graphical component, do nothing.
			if (behavior.useLocalCompletion(_setController, relationFieldInfo))
				return;

			String		 prefix = demand.getValue();
			LyObjectList domain = behavior.getRelationDomain(_setController, relationFieldInfo,
					prefix, (demand.getType() == LyActionDemand.TYPE_EXPAND));
			LyFieldItem  fieldItem = getFieldItem();

			if (!(fieldItem instanceof LyRelation)
					|| (fieldItem.getParent() == null))
				return;

			LyRelation relation = (LyRelation)fieldItem;
			boolean    optional = _setController.isOptional(relationFieldInfo);
			LyLink     link;
			int		   i, n = relation.getLinkCount();
			String     selectedLinkId = null;
			for (i = n - 1; i >= 0; i--)
			{
				link = relation.getLink(i);
				relation.removeLink(link);
				if (link.isSelected())
				    selectedLinkId = link.getId();
			}

			String empty_value = "LY_RELATION_EMPTY_VALUE";
			String val = getEnvironment().translate(empty_value);

			if (((prefix == null)
						|| (prefix.length() == 0))
					&& !relationFieldInfo.isMultiple()
					&& (optional
						|| ((val != null) && !val.equals("_null"))))
			{
				if (optional
						&& (val != null)
						&& val.equals("_null"))
					empty_value = null;

				link = new LyLink("LY_NO_CHOICE", empty_value);
				relation.insertLink(link, 0);
			}

			n = ((domain == null) ? 0 : domain.getSize());

			// Add images only if multiple classes
			int		 nClasses = relationFieldInfo.getClassInfoCount();
			boolean  addImage = (nClasses
						>= getEnvironment().getIntResource("LY_SHOW_RELATION_IMAGES"));
			LyObject object;
			String   id, name, shortLabel;

			for (i = 0; i < n; i++)
			{
				object = domain.getObject(i);
				id = object.getId();
				name = getLinkLabel(object);
				shortLabel = getLinkShortLabel(object);
				link = new LyLink(id, name);
				link.setShortLabel(shortLabel);

				if (addImage)
					link.setImage(getImage(object));
				if (id.equals(selectedLinkId))
				    link.setSelected(true);
				relation.addLink(link);
			}

			return;
		}

		super.actionDemand(demand);
	} // end method actionDemand

	/**
	 * Extract current value for the specified fieldInfo in the form.
	 *
	 * @return current value, or null if not found or field is disabled.
	 */
	public String getStringValue()
	{
		// If a temporary value is set, return this value (needed for edit tables when cell
		// is no longer in edition mode).
		if (_temporaryValue != null)
			return _temporaryValue;

		LyFieldItem   fieldItem = getFieldItem();
		String		  strValue = null;
		LyApplication application = getApplication();

		if (fieldItem instanceof LyRelation)
		{
			LyRelation   relation = (LyRelation)fieldItem;
			int			 i, n = relation.getLinkCount();
			LyLink		 link;
			StringBuffer sb = null;

			for (i = 0; i < n; i++)
			{
				link = relation.getLink(i);

				if (link.isSelected()
						&& !link.getId().equals("LY_NO_CHOICE"))
				{
					if (sb == null)
						sb = new StringBuffer(link.getId());
					else
						sb.append(application.VALUE_SEP).append(link.getId());
				}
			}

			if (sb != null)
				strValue = sb.toString();
		}

		return strValue;
	} // end method getStringValue

	/**
	 * Internal use : update the value of a fieldInfo in the form.
	 *
	 * @param newValue the new value of the field. This value may be null, a LyValue instance or the
	 *                 internal value associated to the fieldInfo. See LyValue.getValue()
	 * @param oldValue the latest value of the field. This value may be null, a LyValue instance or
	 *                 the internal value associated to the fieldInfo. See LyValue.getValue()
	 */
	protected void updateValue(Object newValue, Object oldValue)
	{
        if (_setController.isConsult(_fieldInfo))
        {
            super.updateValue(newValue, oldValue);
            return;
        }

        if (_temporaryValue != null)
        {
            // Invalid auto-completion value notified by immediate update
            if ((newValue.toString() != null)
                    && _temporaryValue.equals(newValue.toString()))
                return; // Don't erase erroneous input

            _temporaryValue = null;
        }

		LyRelationFieldInfo relationField = (LyRelationFieldInfo)_fieldInfo;
		LyFieldItem		    fieldItem = getFieldItem();

		if (!(fieldItem instanceof LyRelation)
				|| (fieldItem.getParent() == null))
			return;

		LyRelation relation = (LyRelation)fieldItem;
		boolean    wasIdle = isIdle();

		setIdle(true);
		relation.setIdle(true);

		// Get relation domain and selection
		LyApplication		   application = getApplication();
		LySetBehaviorInterface behavior = _setController.getSetBehavior();
		LyObjectList		   relationDomain = behavior.getRelationDomain(_setController,
				relationField, null, true);
		LyObjectList		   domain = relationDomain;

		// Get the object identifiers ordered if needed
		if (newValue instanceof LyObjectId[])
			newValue = relationField.getOrderedIds((LyObjectId[])newValue);

		LyObjectList newList = relationField.getObjects(newValue);
		boolean		 multiple = relationField.isMultiple();
		boolean		 optional = _setController.isOptional(relationField);

		// Update domain and selection
		LyObject     object, linkObject;
		LyObjectId   objectId;
		LyLink		 link;
		String		 id, linkId, name, shortLabel;
		int			 i, n, j, nl, test = -1;
		boolean		 selected;

		// Add images only if multiple classes
		int			 nClasses = relationField.getClassInfoCount();
		boolean		 addImage = (nClasses
					>= getEnvironment().getIntResource("LY_SHOW_RELATION_IMAGES"));

		// Step 1 : Remove newValue objects from domain for multiple relations
		if (multiple && (domain != null))
		{
			LySort sort = domain.getSort();

			domain = new LySimpleObjectList(domain);
			domain.setSort(sort);

			n = ((newList == null) ? 0 : newList.getSize());
			for (i = 0; (i < n) && (domain.getSize() > 0); i++)
			{
				object = newList.getObject(i);
				if (domain.contains(object))
					((LySimpleObjectList)domain).removeObject(object);
			}

			if (sort == null)
				domain.setSortByName(true);
		}
		else if (!multiple
			&& (!_setController.isCreation()
				|| (_setController.getModel() == null)))
		{
		    // Complete domain if needed
		    if (domain == null)
		        domain = newList;
		    else if (newList != null)
		    {
				domain = new LySimpleObjectList(domain);
		        domain.addList(newList);
		    }
		}

		// Step 2 : Update domain (domain is sorted by object name)
		n = ((domain == null) ? 0 : domain.getSize());
		nl = relation.getLinkCount();

		boolean emptyLink = !multiple && (nl > 0)
				&& relation.getLink(0).getId().equals("LY_NO_CHOICE");
		int     offset = (emptyLink ? 1 : 0);
		int     ns = 0;

		for (i = 0, j = offset; i < n; i++)
		{
			object = domain.getObject(i);
			selected = ((multiple || (newList == null)) ? false : newList.contains(object));
			if (selected)
				ns++;
			id = object.getId();

			name = getLinkLabel(object);
			shortLabel = getLinkShortLabel(object);

			for (; j < nl; j++)
			{
				link = relation.getLink(j);
				linkId = link.getId();
				objectId = new LyObjectId(application, linkId);
				linkObject = objectId.getLogicalObject();
				test = object.compare(linkObject, true);
				if (test == 0)
				{
					linkId = link.getId();
					test = object.compare(linkObject, false);
					if (test == 0) // Object found
					{
						// If needed change selected state
						if (link.isSelected() != selected)
							relation.selectLink(i + offset, selected);
						j++;
						break;
					}
				}

				if (test > 0)
				{
					relation.removeLink(link);
					j--;
					nl--;
				}
				else if (test < 0)
					break;
			}
			// FFT0147 : the last item of the old list has been reached and selected (test=0),
			// compute the test with the current object
			if ((test == 0) && (j == nl))
			{
				objectId = new LyObjectId(application, relation.getLink(j - 1).getId());
				linkObject = objectId.getLogicalObject();
				test = object.compare(linkObject, true);
			}

			if ((test < 0) || ((test > 0) && (j == nl)))
			{
				link = new LyLink(id, name);
				link.setShortLabel(shortLabel);

				if (addImage)
					link.setImage(getImage(object));

				if (selected)
					link.setSelected(true);
				if ((i + offset) >= nl)
					relation.addLink(link);
				else
					relation.insertLink(link, i + offset);
				j++;
				nl++;
			}
		}
		while (j < nl)
		{
			link = relation.getLink(j);
			relation.removeLink(link);
			nl--;
		}

		// Add an empty link when needed
		if (!multiple && !emptyLink)
		{
			String empty_value = "LY_RELATION_EMPTY_VALUE";
			String val = getEnvironment().translate(empty_value);

			if (optional
					|| ((val != null) && !val.equals("_null")))
			{
				if (optional
						&& (val != null)
						&& val.equals("_null"))
					empty_value = null;

				link = new LyLink("LY_NO_CHOICE", empty_value);
				relation.insertLink(link, 0);
				emptyLink = true;
			}
		}

		// Step 3 : Add selections
		if (multiple && (newList != null))
		{
			for (i = 0, n = newList.getSize(); i < n; i++)
			{
				object = newList.getObject(i);
				if ((relationDomain == null) || !relationDomain.contains(object))
					continue;

				// Add link corresponding to the object
				link = new LyLink(object.getId(), getLinkLabel(object));
				link.setShortLabel(getLinkShortLabel(object));

				if (addImage)
					link.setImage(getImage(object));

				relation.addLink(link);

				// We must set the selection state after adding the link in the relation.
				relation.selectLink(nl + i, true);
			}

			if (domain != null)
				domain.addList(newList);
			else
				domain = newList;
		}
		else if (!multiple && (ns == 0))
		{
			// If there is only one possible choice and if the form
			// does'nt manage a composition relation we select the only choice.
			// We must not select it if the form managed a composition relation
			// because the default value is not this choice and we could not
			// perform any selection and so not create the composant object.
			if (emptyLink
					&& !optional
					&& (domain != null)
					&& (domain.getSize() == 1)
					&& (_setController.getCompositionRelation() == null))
				relation.selectLink(1, true);
			else if (domain != null)
				relation.selectLink(0, true);
		}
		else if (!multiple && emptyLink)
			// There is an empty link, we must verify that it is not selected.
			relation.selectLink(0, false);

		if (relationDomain != null)
			// Create relation controller to maintain domain valid
			new _LyRelationDomainController(relationDomain);

		// FFT01211 : update transverse action label if relation has a class field
		if (relationField.getClassField() != null)
		{
			LyInfoList<LyClassInfo> classes = behavior.getRelationClasses(_setController,
					relationField);

			if ((classes != null) && (classes.getSize() != 0))
			{
				LyFormField		  formField = (LyFormField)fieldItem.getParent();
				LyFieldSetBuilder builder = (LyFieldSetBuilder)getBuilder();
				if(!formField.isHidden())
					formField.setToolBar(builder.updateToolBar(relationField));
			}
		}
		checkTransverseToolbar();

		relation.setIdle(false);
		setIdle(wasIdle);

		// Update the last modification time.
		setLastModified(new Date().getTime());

		newValue = getValue(); // newValue may have changed when updating

		// Specific case of propagation on context fields
		if ((oldValue == null)
				&& (newValue == null)
				&& isOptional()
				&& (_fieldInfo.getDepends() != null))
			propagate(newValue);
	} // end method updateValue

	/**
	 * Overridden to update transverse toolbar.
	 * @param editable the editable status
	 */
	public void setEditable(boolean editable)
	{
		super.setEditable(editable);

		checkTransverseToolbar();
	}

	/**
	 * Check if toolbar should enable/disable some tools.
	 */
	protected void checkTransverseToolbar()
	{
		// Update transverse relation toolbar (enable / disable tools)
		LyFormField formField = getFormField();

		LyToolBar   toolbar = formField.getToolBar();

		if (toolbar != null)
		{
			// Add toolbar for transverse actions if needed
			LyInfoDataList<LyAction, LyClassInfo> actions = getTransverseActions();

			if (actions == null)
				return;

			HashMap<String, LyAction> actionsTable = new HashMap<String, LyAction>();

			// Get the related action (table for fast search)
			LyAction				  action;
			int						  i, na = actions.getSize();

			for (i = 0; i < na; i++)
			{
				action = actions.getInfo(i);
				actionsTable.put(action.getId(), action);
			}

			// Go through the toolbar to check if enabled
			LyObjectList	    objects = null;
			int				    nc = toolbar.getChildCount(), p;
			LyTool			    tool;
			String			    key;
			boolean			    enable;
			LySession		    session = getSession();
			LyRelationFieldInfo relation2;
			LyRelationValue     value;
			LyClassInfo		    classInfo;

			for (i = 0; i < nc; i++)
			{
				if (!(toolbar.getComponent(i) instanceof LyTool))
					continue;

				tool = (LyTool)toolbar.getComponent(i);

				if (!isEnabled())
				{
					tool.setEnabled(false);
					continue;
				}

				p = tool.getId().indexOf(LyController.ID_SEP);
				key = ((p > 0) ? tool.getId().substring(0, p) : tool.getId());
				action = actionsTable.get(key);

				if (!isEditable()
						&& (action != null)
						&& LyTransverseInterface.class.isAssignableFrom(
							action.getControllerClass()))
				{
					tool.setEnabled(false);
					continue;
				}

				// Check if still enabled
				if (action != null)
				{
					classInfo = actions.getData(action);
					enable = true;

					if (action.getMin() > 0)
					{
						// action applies on objects in the relation
						relation2 = (LyRelationFieldInfo)_fieldInfo;
						value = (LyRelationValue)relation2.buildFieldValue(getValue());
						objects = value.getObjectList();
					}
					else
						objects = null;

					enable = session.enableTransverseAction(action, classInfo, _setController,
							objects, false);

					tool.setEnabled(enable);
				}
			}
		}
	} // end method checkTransverseToolbar

	/**
	 * Clears all cached data for this controller (especially the view and the component).
	 *
	 * <P>This method may be invoked for memory optimization, but the controller is still valid and
	 * may be used. (Unlike free) Further usage, needs to rebuild internal component & view which
	 * may be time consuming.</P>
	 */
	public void clearViewCache()
	{
		if (_relationDomainController != null)
		{
			_relationDomainController.free();
			_relationDomainController = null;
		}
		super.clearViewCache();
	}

	/**
	 * Free this controller.
	 *
	 */
	public void free()
	{
		clearViewCache();
		super.free();
	}

	/**
	 * Internal class to create a popup creation form for a relation fieldInfo.
	 */
	protected class _LyRelationDomainController
		implements LyGuiListener,
			LyDataSourceListener,
			LyWorkSpaceListener
	{
		LyRelationFieldInfo _relation;
		LyObjectList	    _domain;

		/**
		 * Constructor.
		 *
		 * @param domain        context domain to set the values in the popup form.
		 */
		protected _LyRelationDomainController(LyObjectList domain)
		{
			_setController = (LySetController)getParent();
			_relation = (LyRelationFieldInfo)_fieldInfo;
			if (domain == null)
				_domain = null;
			else if (domain.isDynamic()) _domain = domain;
			else
				_domain = new LySimpleObjectList(domain);

			if ((_domain != null) && _domain.isDynamic())
				((LyWorkSpace)_domain).addListener(this);
			else
			{
				// Listen to relation classes
				int			   i, n = _relation.getClassInfoCount();
				LyClassInfo    classInfo;
				LyObjectSource source;
				LySession	   session = _setController.getSession();

				for (i = 0; i < n; i++)
				{
					classInfo = _relation.getClassInfo(i);
					source = session.getObjectSource(classInfo);
					if (source != null)
						source.addListener(this);
				}
			}

			// Add this to relation controllers list
			if (_relationDomainController != null)
				// We already have a relation controller, free it
				_relationDomainController.free();

			_relationDomainController = this;
		}

		/**
		 * Free this controller.
		 */
		protected void free()
		{
			if ((_domain != null) && _domain.isDynamic())
				((LyWorkSpace)_domain).free(this);
			else
			{
				// Stop listening to relation sources
				int			   i, n = _relation.getClassInfoCount();
				LyClassInfo    classInfo;
				LyObjectSource source;
				LySession	   session = _setController.getSession();

				for (i = 0; i < n; i++)
				{
					classInfo = _relation.getClassInfo(i);
					source = session.getObjectSource(classInfo);
					if (source != null)
						source.removeListener(this);
				}
			}
		}

		/**
		 * Checks if this controller is idle.
		 * @return the idle status
		 */
		public boolean isIdle()
		{
			return LyRelationFieldSetController.this.isIdle();
		}

		/**
		 * Called when an object is added in the workspace.
		 *
		 * @param  event Event notifying object add.
		 * @return True if the event is used by the listener, false otherwise.
		 */
		public boolean objectAdded(LyWorkSpaceEvent event)
		{
			if (isIdle())
				return false;

			LyObject object = event.getObject();
			int		 index = _domain.getIndex(object);

			addObject(object, index);
			return true;
		}

		/**
		 * Method for receiving an object creation.
		 *
		 * @param  event Event notifying the creation of an object.
		 * @return Boolean indicating if the event was used, false otherwise.
		 */
		public boolean objectCreated(LyDataSourceEvent event)
		{
			if (isIdle())
				return false;

			LyObject object = (LyObject)event.getObject();

			if ((_domain != null) && _domain.contains(object))
				return false;

			LySetBehaviorInterface behavior = _setController.getSetBehavior();

			// First check if the new object match the relation filter
			LyFilter filter = behavior.getRelationFilter(_setController, _relation, object.getClassInfo());

			if ((filter != null)
					&& !filter.match(object))
				return false;

			if (!behavior.enableRelationLink(_setController, _relation, object))
				return false;

			LyObjectList domain = behavior.getRelationDomain(_setController, _relation);
			int			 oldSize, newSize;

			// If domain becomes null, we may have reached the select relation limit (given by
			// the LY_SELECT_RELATION_COUNT and LY_SELECT_BY_NAME_RELATION_COUNT resources).
			// In this case, add the object to the old domain to avoid getting the relation
			// without this object and without any select action.
			if ((_domain != null) && (domain == null))
			{
				((LySimpleObjectList)_domain).addObject(object);

				// Force the domain to be sorted in order to place the newly created object at the
				// right place.
				_domain.setSorted(true);
				addObject(object, _domain.getIndex(object));
				return true;
			}

			oldSize = ((_domain == null) ? 0 : _domain.getSize());
			newSize = ((domain == null) ? 0 : domain.getSize());

			int index = ((domain == null) ? -1 : domain.getIndex(object));

			if (index < 0)
				return false;
			else
			{
				if (oldSize != (newSize - 1))
				{
					// We must rebuild the relation.
					for (int i = 0; i < oldSize; i++)
					{
						object = _domain.getObject(i);
						if (!domain.contains(object))
							removeObject(object);
					}

					for (int i = 0; i < newSize; i++)
					{
						object = domain.getObject(i);
						if ((_domain != null)
								&& !_domain.contains(object))
							addObject(object, 0);
					}

					_domain = new LySimpleObjectList(domain);
				}
				else
				{
					// Update domain
					_domain = new LySimpleObjectList(domain);

					addObject(object, index);
				}
			}

			return true;
		} // end method objectCreated

		/**
		 * Adds the object at the given position.
		 *
		 * @param object Object to add to domain.
		 * @param index  Position where the object is inserted.
		 */
		void addObject(LyObject object, int index)
		{
			// Add a link for the new object
			LyRelation relation = (LyRelation)_formField.getFieldItem();
			LyLink     link = new LyLink(object.getId(), getLinkLabel(object));
			int		   offset = 0;

			link.setShortLabel(getLinkShortLabel(object));

			// Add images only if multiple classes
			int     nClasses = _relation.getClassInfoCount();
			boolean addImage = (nClasses
						>= getEnvironment().getIntResource("LY_SHOW_RELATION_IMAGES"));
			int        nr = relation.getLinkCount();

			if (addImage)
				link.setImage(getImage(object));

			if (_relation.isMultiple())
				// FFT0172 : if the relation is multiple, the object should be
				// added at the end of the selection box.
				index = -1;
			else if ((relation.getLinkCount() > 0)
					&& relation.getLink(0).getId().equals("LY_NO_CHOICE"))
				offset = 1;

			if (index + offset >= nr)
			    relation.addLink(link);
			else
			    relation.insertLink(link, index + offset);

			// Update the last modification time of the set controller.
			_setController.setLastModified(new Date().getTime());
		} // end method addObject

		/**
		 * Called when an object is removed from the workspace.
		 *
		 * @param  event Event notifying object remove.
		 * @return True if the event is used by the listener, false otherwise.
		 */
		public boolean objectRemoved(LyWorkSpaceEvent event)
		{
			if (isIdle())
				return false;

			LyObject object = event.getObject();

			removeObject(object);
			return true;
		}

		/**
		 * Method for receiving an object deletion.
		 *
		 * @param  event Event notifying the deletion of an object.
		 * @return Boolean indicating if the event was used, false otherwise.
		 */
		public boolean objectDeleted(LyDataSourceEvent event)
		{
			if (isIdle())
				return false;

			LyObject object = (LyObject)event.getObject();

			if ((_domain == null) || !_domain.contains(object))
				return false;

			removeObject(object);
			((LySimpleObjectList)_domain).removeObject(object);
			return true;
		}

		/**
		 * Removes given object from relation domain.
		 *
		 * @param object Object to remove.
		 */
		void removeObject(LyObject object)
		{
			// Retrieve corresponding Link and remove it
			LyRelation relation = (LyRelation)_formField.getFieldItem();
			LyLink     link;
			int		   i, n = relation.getLinkCount();
			String     id;

			for (i = 0; i < n; i++)
			{
				link = relation.getLink(i);
				id = link.getId();
				if ((id != null) && id.equals(object.getId()))
				{
					// If the link is selected, unselected before --> avoid automatically selecting an other link
					// Flandres/FFT00448
					if (link.isSelected())
						relation.selectLink(id, false);
					relation.removeLink(link);
					break;
				}
			}

			// Update the last modification time of the set controller.
			_setController.setLastModified(new Date().getTime());
		}

		/**
		 * Called when an object has changed in the workspace. This call notifies a change on the
		 * object concerned by the event either because its values have changed or because the sort
		 * on the workspace has changed and consequently the object position within the list has
		 * changed. These two cases can easily be distinguished by accessing the values set
		 * contained in the event (if it's not null, this is an change on the object, not a
		 * modification of the sort).
		 *
		 * <p>Note that object position may also change if one of the modified fields is one of the
		 * fields specified in the list sort.</p>
		 *
		 * @param  event Event notifying object change.
		 * @return True if the event is used by the listener, false otherwise.
		 */
		public boolean objectChanged(LyWorkSpaceEvent event)
		{
			if (isIdle())
				return false;

			LyObject   object = event.getObject();
			LyValueSet values = event.getValues();
			int		   index = event.getIndex();
			int		   oldIndex = event.getIndex();

			if (index == oldIndex)
				updateObject(object, values);
			else
			{
				removeObject(object);
				addObject(object, index);
			}

			return true;
		}

		/**
		 * Method for receiving an object modification.
		 *
		 * @param  event Event notifying the modification of an object.
		 * @return Boolean indicating if the event was used, false otherwise.
		 */
		public boolean objectModified(LyDataSourceEvent event)
		{
			if (isIdle())
				return false;

			LyObject			   object = (LyObject)event.getObject();
			LyClassInfo			   classInfo = object.getClassInfo();
			LySetBehaviorInterface behavior = _setController.getSetBehavior();
			LyFilter			   filter = behavior.getRelationFilter(_setController, _relation,
					classInfo);

			if (filter != null)
			{
				if (((_domain == null) || !_domain.contains(object))
						&& filter.match(object))
					return objectCreated(event);

				if (((_domain != null) && _domain.contains(object))
						&& !filter.match(object))
					return objectDeleted(event);
			}

			boolean enabled = behavior.enableRelationLink(_setController, _relation, object);

			if ((_domain == null) || !_domain.contains(object))
			{
				if (!enabled)
					return false;
				else
					return objectCreated(event);
			}
			else if (!enabled)
				return objectDeleted(event);

			// We're only interested in name changes
			LyValueSet values = event.getValues();

			updateObject(object, values);
			return true;
		} // end method objectModified

		/**
		 * Updates the given object in relation domain.
		 *
		 * @param object Object that is changed.
		 * @param values Changes made on the object.
		 */
		void updateObject(LyObject object, LyValueSet values)
		{
			if (values == null)
				// Nothing to do.
				return;

			if (!values.hasMark(LyFieldInfo.MARK_ID)
					&& !values.hasMark(LyFieldInfo.MARK_NAME))
				// Nothing to do
				return;

			// Retrieve corresponding Link and change its name or its id
			LyApplication application = getApplication();
			LyRelation    relation = (LyRelation)_formField.getFieldItem();
			LyLink		  link, sortLink;
			int			  i, j, n = relation.getLinkCount();
			String		  id, currentLinkId;
			LyObject	  currentLinkObject;
			LyObjectId    currentLinkObjectId;
			int			  offset = (_relation.isMultiple() ? 0 : 1);

			if (values.hasMark(LyFieldInfo.MARK_ID))
			{
				LyObjectId objectId;
				LyLink     newLink;

				for (i = offset; i < n; i++)
				{
					link = relation.getLink(i);
					id = link.getId();
					if (id.equals("LY_NO_CHOICE"))
						continue;

					objectId = new LyObjectId(application, id);
					if (objectId.getLogicalObject() == null)
					{
						relation.removeLink(link);
						newLink = new LyLink(object.getId(), getLinkLabel(object));
						newLink.setShortLabel(getLinkShortLabel(object));
						newLink.setImage(link.getImage());
						relation.insertLink(newLink, i);
						if (link.isSelected())
							relation.selectLink(i, true);

						// Update the last modification time of the set controller.
						_setController.setLastModified(new Date().getTime());
						break;
					}
				}
			}

			for (i = offset; i < n; i++)
			{
				link = relation.getLink(i);
				id = link.getId();

				if ((id != null) && id.equals(object.getId()))
				{
					if (link.isSelected() && _relation.isMultiple())
					{
						// Just change link label, not its position in selections list
						link.setLabel(getLinkLabel(object));
						link.setShortLabel(getLinkShortLabel(object));

						// XXX Force redisplay, changing label is not enough
						relation.removeLink(link);
						relation.insertLink(link, i);
						break;
					}

					// We are going to change the link label, we have to change its place
					// firts we remove it from the relation...
					relation.removeLink(link);

					// we change the link Label
					link.setLabel(getLinkLabel(object));
					link.setShortLabel(getLinkShortLabel(object));

					// then add it at the right place
					for (j = offset; j < (n - 1); j++)
					{
						sortLink = relation.getLink(j);
						currentLinkId = sortLink.getId();
						currentLinkObjectId = new LyObjectId(application, currentLinkId);
						currentLinkObject = currentLinkObjectId.getLogicalObject();
						if (currentLinkObject.compare(object, true) >= 0)
						{
							relation.insertLink(link, j);
							if (link.isSelected())
								relation.selectLink(j, true);
							break;
						}
					}

					if (j == (n - 1))
					{
						relation.addLink(link);
						if (link.isSelected())
							relation.selectLink(j, true);
					}
					break;
				}
			}

			if (i == n)
				// Object not found => full reset
				updateValue(getValue(), null);

			// Update the last modification time of the set controller.
			_setController.setLastModified(new Date().getTime());
		} // end method updateObject
	} // end class _LyRelationDomainController

	/**
	 * Internal use. When an object is created in a popup creation form for a relation, the child
	 * controller invokes this method to set the newly created object as the current value of the
	 * relation (if a new object is created we automatically select this value). This method is
	 * never used by the application : use setFieldValue instead
	 *
	 * @param object        newly created object. The new value of the relation fieldInfo.
	 * @param relationField the relation
	 */
	public void notifyFormResult(LyObject object, LyRelationFieldInfo relationField)
	{
		_temporaryValue = null;

		LyFieldItem fieldItem = _formField.getFieldItem();

		if (!(fieldItem instanceof LyRelation))
			return;

		LyRelation relation = (LyRelation)fieldItem;
		LyLink     link;
		int		   i, n = relation.getLinkCount();
		String     id;
		boolean    multiple = relation.isMultiple();
		boolean    found = false;

		for (i = 0; i < n; i++)
		{
			link = relation.getLink(i);
			id = link.getId();

			if ((object != null)
					&& (id != null)
					&& id.equals(object.getId()))
			{
				link.setSelected(true);
				found = true;
				if (multiple)
					break;
			}
			else if ((!multiple) && (link.isSelected()))
				link.setSelected(false);
		}

		if (!found)
		{
			link = new LyLink(object.getId(), getLinkLabel(object));
			link.setShortLabel(getLinkShortLabel(object));

			// Add images only if multiple classes
			int     nClasses = relationField.getClassInfoCount();
			boolean addImage = (nClasses
						>= getEnvironment().getIntResource("LY_SHOW_RELATION_IMAGES"));

			if (addImage)
				link.setImage(getImage(object));

			relation.addLink(link);
			link.setSelected(true);
		}

		valueChanged();

		checkTransverseToolbar();
		_formController.setActiveField(relationField);
	} // end method notifyFormResult

	/**
	 * Internal method to invoke on a relation a transverse action for a given classInfo.
	 *
	 * @param action    Transverse action.
	 * @param classInfo Class info on which the action is applied.
	 */
	public void invokeTransverseAction(LyAction action, LyClassInfo classInfo)
	{
		LyRelationFieldInfo relation = (LyRelationFieldInfo)_fieldInfo;

		if (classInfo == null)
		{
			LySetBehaviorInterface  behavior = _setController.getSetBehavior();
			LyInfoList<LyClassInfo> classes = behavior.getRelationClasses(_setController, relation);

			if ((classes == null) || (classes.getSize() == 0))
				return;

			classInfo = classes.getInfo(0);
		}

		LyObjectList objects = null;

		if (action.getMin() > 0)
		{
			// action applies on objects in the relation
			LyRelationValue value = (LyRelationValue)relation.buildFieldValue(getValue());

			objects = value.getObjectList();
		}

		LySession		 session = getSession();
		LyAreaController controller = (LyAreaController)session.invokeAction(_setController, action,
				classInfo, objects, false);

		checkTransverseToolbar();

		if (controller == null)
			return;

		LyAreaController transverseController = controller;
		if ((!(controller instanceof LyTransverseInterface))
				&& (controller instanceof LyCompoundAreaController)
				&& (((LyCompoundAreaController)controller).getAreaController(0) instanceof LyTransverseInterface))
			transverseController = ((LyCompoundAreaController)controller).getAreaController(0);

		if (transverseController instanceof LyTransverseInterface)
		{
			if (_parentFieldController instanceof LyArrayFieldSetController)
				// Nested into an array field (array of Relation)
				((LyTransverseInterface)transverseController).setRelationParent((LyArrayFieldSetController)
					_parentFieldController, relation);
			else
				((LyTransverseInterface)transverseController).setRelationParent(_setController, relation);
		}

		if (controller instanceof LySetController)
		{
            LyContext.Position position = getInsertPosition();

            if (position != null)
            {
                LyContext   controllerContext = ((LySetController)controller).getControllerContext();

                controllerContext.setKeepOrder(true);
                controllerContext.setInsertPosition(position);
            }
		}

		if (((LyFormController)_parent).getParent() instanceof LyFormController)
		{
			LyFormController formController = (LyFormController)((LyFormController)_parent)
					.getParent();

			formController.openSubAreaController(controller, true);
		}
		else
			_setController.openSubAreaController(controller, true);

		// Context in content
		if (objects != null)
		{
			controller.showView();

			return;
		}

		if (relation.ignoresContext())
		{
			if (controller.getContext() != null)
				controller.setContext(null);

			controller.showView();

			return;
		}

		// Best effort to guess the context
		LySimpleObjectList  context = null;
		LyBehaviorInterface behavior = getBehavior();
		LyObject		    model = getObject();

		if ((model == null)
				&& (_formController instanceof LyCreateController))
			model = ((LyCreateController)_formController).getModel();

		if (controller instanceof LyFormController)
		{
			// Context of the relation
			if ((behavior instanceof LySetBehaviorInterface)
					&& (_formController instanceof LySetController))
			{
				LyObjectList relationContext = ((LySetBehaviorInterface)behavior)
						.getRelationContext((LySetController)_formController, relation);

				if ((relationContext != null) && (relationContext.getSize() > 0))
					context = new LySimpleObjectList(relationContext);
			}

			// Edited object
			if (model != null)
			{
				if (context == null)
					context = new LySimpleObjectList(model);
				else
					context.addObject(model);
			}

			// Current context
			if (getContext() != null)
			{
				if (context == null)
					context = new LySimpleObjectList(getContext());
				else
					context.addList(getContext());
			}
		}
		else
		{
			if ((behavior instanceof LySetBehaviorInterface)
					&& (_formController instanceof LySetController))
			{
				// Domain of the relation
				LyObjectList relationDomain = ((LySetBehaviorInterface)behavior).getRelationDomain(
						(LySetController)_formController, relation);

				if (relationDomain != null)
					context = new LySimpleObjectList(relationDomain);
				else
					context = new LySimpleObjectList();
			}
			else if (model != null)
			// Edited object
			context = new LySimpleObjectList(model);
			else if (getContext() != null)
				// Current context
				context = new LySimpleObjectList(getContext());
		}

		if (context != null)
			controller.setContext(context);

		controller.showView();
	} // end method invokeTransverseAction

	/**
	 * Gives the label for the link related to an object.<br>
	 * Add $$ to avoid translations except if the relation has translate mark.
	 *
	 * @param  object object displayed in a relation.
	 * @return a string.
	 */
	protected String getLinkLabel(LyObject object)
	{
		boolean   translate = _fieldInfo.hasMark(LyFieldInfo.MARK_TRANSLATE);

		if (translate)
			return _formController.getName(object);
		else
			return "$$" + _formController.getName(object);
	}

	/**
	 * Gives the short label for the link related to an object.<br>
     * Add $$ to avoid translations except if the relation has translate mark.
	 *
	 * @param  object object displayed in a relation.
	 * @return a string.
	 */
	protected String getLinkShortLabel(LyObject object)
	{
		LySetBehaviorInterface behavior = _setController.getSetBehavior();
		LyRelationFieldInfo    relationFieldInfo = (LyRelationFieldInfo)_fieldInfo;

		if (behavior != null)
		{
			String name = behavior.getLinkShortLabel(_setController, relationFieldInfo, object);

			if (name != null)
			{
				if (relationFieldInfo.hasMark(LyFieldInfo.MARK_TRANSLATE))
					return name;
				else
					return "$$" + name;
			}
		}

		return null;
	}
} // end class LyRelationFieldSetController
