/**
 * LyStructInArraySetController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyStructInArraySetController.java 34633 2012-03-14 10:30:44Z jlsaouli $
 */

package leon.control.form;

import leon.app.*;
import leon.app.behaviorinterface.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;


/**
 * Field controller for a modification of an array of composition relation.
 * @see LyFieldSetController
 */
public class LyStructInArraySetController extends LyCompositionFieldSetController
{
	/** File version */
	public static final String __VERSION =
		"$Id: LyStructInArraySetController.java 34633 2012-03-14 10:30:44Z jlsaouli $";

	/** The associated LyArrayFieldInfo */
	protected LyArrayFieldInfo _array;

	/**
	 * Constructor.
	 * @param parent The parent controller
	 * @param array the array field
	 * @param structRelation the struct field
	 */
	public LyStructInArraySetController(LySetController parent,
		LyArrayFieldInfo array, LyStructFieldInfo structRelation)
	{
		super(parent, structRelation);
		_array = array;
	}

	/**
	 * Internal use : update the value of a fieldInfo in the form.
	 * @param newValue the new value of the field. This value may be null, a LyValue instance or the
	 *                 internal value associated to thge fieldInfo. See LyValue.getValue()
	 * @param oldValue the latest value of the field. This value may be null, a LyValue instance or
	 *                 the internal value associated to thge fieldInfo. See LyValue.getValue()
	 */
	protected void updateValue(Object newValue, Object oldValue)
	{
		_first = false;
		if (getAreaController() instanceof LySetController)
		{
			LyObjectList content = ((LySetController)getAreaController()).getContent();

			LyObjectId   id = null;

			if ((content != null) && (content.getSize() > 0))
				id = content.getObject(0).getObjectId();

			if ((newValue != null)
					&& (newValue instanceof LyObjectId))
			{
				if (!newValue.equals(id))
					((LySetController)getAreaController()).setObject(((LyObjectId)newValue)
							.getLogicalObject());
				return;
			}
		}

		super.updateValue(newValue, oldValue);
	}

	/**
	 * Extract current value for the specified fieldInfo in the form.
	 * @return current value, or null if not found or field is disabled.
	 */
	public LyValue getFieldValue()
	{
		if (getAreaController() instanceof LySetController)
		{
			LyObjectList content = ((LySetController)getAreaController()).getContent();

			LyObjectId   id = null;

			if ((content != null) && (content.getSize() > 0))
				id = content.getObject(0).getObjectId();
			else if (getAreaController() instanceof LyCreateController)
			{
				LyObject obj = ((LyCreateController)getAreaController()).getLastCreatedObject();

				if (obj != null)
					id = obj.getObjectId();
			}
			return _fieldInfo.buildFieldValue(id);
		}
		return super.getFieldValue();
	}

	/**
	 * Free this field controller.
	 */
	public void free()
	{
		_array = null;

		super.free();
	}

	/**
	 * Method called when all modifications are committed.
	 */
	public void commit()
	{
		// Commit and cancel are done one level above in the ArrayFieldSetController.
		// at this level the value is the Struct value, sor the Array of Struct value.
		if (_areaController instanceof LyFormInterface)
			((LyFormInterface)_areaController).commit();
	}

	/**
	 * @see leon.control.form.LyCompositionFieldSetController#buildAreaController(boolean)
	 */
	protected LyAreaController buildAreaController(boolean typeChanged)
	{
		if (_internal)
		{
			// Do not call super.buildAreaController because we must have an empty
			// list to invoke action otherwise we have a multiset form and the nextObject
			// call asks for discarding modifications.
			if (_fieldType == null)
				_fieldType = getFieldType();

			LyObjectList list = new LySimpleObjectList();
			LyAction     action = getAreaControllerAction(list, typeChanged);
			LySession    session = getSession();

			// We must create the controller with this as parent to manage
			// correctly the internal context (see LySetController.getControllerContext).
			// The parent is set to the form when we call addAreaController.
			_areaController = (LyAreaController)session.invokeAction(this, action, _fieldType, list,
					false);

			if (_areaController != null)
			{
				if (_areaController instanceof LyFormController)
					((LyFormController)_areaController).setCompositionRelation((LyRelationFieldInfo)
						_fieldInfo);
				if (_areaController instanceof LySetController)
					((LySetController)_areaController).setContextInternal();
				_areaController.setViewController(_formController.getViewController());
				
				_formController.addAreaController(_areaController);
			}

			return _areaController;
		}
		else
		{
			if (_fieldType == null)
				_fieldType = getFieldType();

			//LyClassInfo           classInfo = getClassInfo();
			LyObjectList list = null;
			LySession    session = getSession();
			LyAction     action = getAreaControllerAction(list, typeChanged);

			_areaController = new LyCreateOrSetController(session, this, action, _fieldType, list);

			if (_areaController != null)
			{
				if (_areaController instanceof LyFormController)
					((LyFormController)_areaController).setCompositionRelation((LyRelationFieldInfo)
						_fieldInfo);
				if (_areaController instanceof LySetController)
					((LySetController)_areaController).setContextInternal();
				_areaController.setViewController(_formController.getViewController());
			}

			_formController.addAreaController(_areaController);
		}
		return _areaController;
	} // end method buildAreaController

	/**
	 * Internal use only.
	 * @param list list of objects
	 * @param typeChanged 
	 * @return an action
	 */
	protected LyAction getAreaControllerAction(LyObjectList list, boolean typeChanged)
	{
		if (_internal)
		{
			LySetBehaviorInterface behavior = (LySetBehaviorInterface)getBehavior();
			LyAction			   action = behavior.getAction(_setController,
					(LyRelationFieldInfo)_fieldInfo, _fieldType);

			if (action == null)
			{
			    LyFormController  formController = getFormController();
			    LyClassInfo       classInfo = formController.getClassInfo();
			        
				action = getApplication().getActionWithId(LyAction.ID_SET_EMPTY, classInfo);

				if (action != null)
					return action;
			}
			else
				return action;
		}
		return super.getAreaControllerAction(list, typeChanged);
	}
} // end class LyStructInArraySetController
