/**
 * LyTextFieldSetController.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyTextFieldSetController.java 28422 2010-11-03 12:03:38Z ehelmste $
 */

package leon.control.form;


import leon.app.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;
import leon.view.event.*;


/**
 * Default text field controller for consult action.
 * @see LyconsultController
 */
public class LyTextFieldConsultController extends LyFieldConsultController
{
	/** File version */
	public static final String __VERSION =
		"$Id: LyTextFieldSetController.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Id of the transverse tool for multilinguism. */
	public static final String     		LANG_TOOL = "_langTool";
	
	protected String					_currentLang;
	
	
	/**
	 * Constructor.
	 * @param parent the parent consult controller
	 * @param fieldInfo the fieldinfo handled by this controller
	 */
	public LyTextFieldConsultController(LyConsultController parent, LyTextFieldInfo fieldInfo)
	{
		super(parent, fieldInfo);
		
		if (((LyTextFieldInfo)_fieldInfo).isMultilingual())
			_currentLang = getSession().getEnvironment().getLanguage();
	}
	
	/**
	 * Invoked when an action occurs.
	 * At this level, the implementation manages multilingual inputs.
	 */
	public void actionDemand(LyActionDemand demand)
	{
		if (demand.getType() == LyActionDemand.TYPE_ACTION)
		{
			String   sourceId = demand.getSource().intern();
			String[] parameters = LyString.stringToList(sourceId, ID_SEP);
			
			if ((parameters.length > 1)
					&& parameters[0].equals(LANG_TOOL))
				changeLang(parameters[1]);
		}
	}
	
	/**
	 * Change lang to the given one.
	 */
	protected void changeLang(String lang)
	{
		_currentLang = lang;
		
		// Update field and associated tool
		LyFormField	formField = getFormField();
		LyToolBar	toolBar = formField.getToolBar();
		
		if (toolBar != null)
		{
			LyTool	tool = (LyTool)toolBar.getComponent(LANG_TOOL);
			
			if (tool != null)
				tool.setLabel(lang.toUpperCase());
		}
		
		LyValue			value = getFieldValue();
		String			stringValue = (String)value.getValue();
		LyTextFieldInfo	textFieldInfo = (LyTextFieldInfo)_fieldInfo;
		String			newValue = textFieldInfo.getLangValue(lang, stringValue);
		
		updateValue(newValue, value);
	}
	
	/**
	 * Internal use : update the value of a fieldInfo in the form.
	 * @param newValue the new value of the field.
	 * @param oldValue the latest value of the field.
	 */
	protected void updateValue(Object newValue, Object oldValue)
	{
		if (!(newValue instanceof String)
			|| !((LyTextFieldInfo)_fieldInfo).isMultilingual())
		{
			super.updateValue(newValue, oldValue);
			return;
		}
		
		LyTextFieldInfo	textFieldInfo = (LyTextFieldInfo)_fieldInfo;
		LyApplication	application = getApplication();
		String[][]		languages = application.getSupportedLanguages();
		int				i, nl = (languages == null ? 0 : languages.length);
		String			langValues = (String)newValue;
		String			langValue;
		
		for (i = 0; i < nl; i++)
		{
			langValue = textFieldInfo.getLangValue(languages[i][0], langValues);
			if (langValue != newValue)
			{
				if (languages[i][0].equals(_currentLang))
					newValue = langValue;
			}
		}
		
		super.updateValue(newValue, oldValue);
	}
} // end class LyTextFieldSetController
