/**
 * LyEditTableMultipleChoiceSetBuilder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyEditTableMultipleChoiceSetBuilder.java 30526 2011-04-14 13:24:53Z nhamon $
 */

package leon.control.form.builder;

import java.util.*;
import leon.app.behaviorinterface.*;
import leon.control.*;
import leon.control.form.*;
import leon.data.*;
import leon.info.*;
import leon.view.*;


/**
 * This builder creates a form field for a multiple choice in editable table.
 *
 * @see LyEditTableChoiceFieldSetController
 */
public class LyEditTableMultipleChoiceSetBuilder extends LyFieldSetBuilder
{
	/** File version */
	public static final String __VERSION =
		"$Id: LyEditTableMultipleChoiceSetBuilder.java 30526 2011-04-14 13:24:53Z nhamon $";

	/**
	 * Default constructor.
	 *
	 * @param parent parent controller
	 */
	public LyEditTableMultipleChoiceSetBuilder(LyController parent)
	{
		super(parent);
	}

	/**
	 * Builds the view component.
	 *
	 * @return the view Component
	 */
	public LyComponent build()
	{
		return buildInternalComponent();
	}

	/**
	 * Build a view component for the given field.
	 *
	 * @return the view component for the field
	 */
	protected LyComponent buildInternalComponent()
	{
		LyFieldSetController     fieldSetController = (LyFieldSetController)_parent;
		LyChoiceFieldInfo        choiceFieldInfo = (LyChoiceFieldInfo)fieldSetController.getFieldInfo();
		LyEditTableSetController tableSetController = (LyEditTableSetController)_parent.getParent();
		LyEditTableController    editTableController
		    = (LyEditTableController)tableSetController.getParent();
		LyObject				 object = tableSetController.getObject();
		boolean					 editable = editTableController.isEditable(object, choiceFieldInfo);

		if (choiceFieldInfo.isMultiple()
				&& !editable
				&& !tableSetController.isConsult(choiceFieldInfo))
			return null;

		LyFormField field;

		if (choiceFieldInfo.isMultiple() && !choiceFieldInfo.isDictionary())
		{
			short           i, n = choiceFieldInfo.getChoiceOptionCount();
			String          id;
			LyForm          form = new LyForm("dummy");
            LyChoiceValue   value = (object == null ? null : object.getChoiceValue(choiceFieldInfo));
            boolean         checked;

			for (i = 0; i < n; i++)
			{
				// Create the editable item
				id = getFieldId(fieldSetController, choiceFieldInfo, i);
                checked = (value == null ? false : value.isSelected(i));
				field = buildFormField(choiceFieldInfo, i, id, checked);
				if (field != null)
				{
					field.setEnabled(field.isEditable() && editable);
					form.addFormField(field);
				}
			}
			return form;
		}

		field = buildFormField(true);
		field.setCaption(null);
		field.setConstraint(null);
		field.setController(_parent.getId());

		return field;
	} // end method buildInternalComponent

	/**
	 * Utility method to build a field id for a multiple choice.
	.*
	 * @param controller the parent controller 
	 * @param  choiceFieldInfo the fieldInfo which is associated to the view component.
	 * @param  index     of the option being shown
	 * @return the formField corresponding to the given fieldInfo
	 */
	public String getFieldId(LyFieldSetController controller, LyChoiceFieldInfo choiceFieldInfo,
		short index)
	{
		String id = controller.getId();
		String sep = LyFormController.FORM_FIELD_SEP;

		id += sep + choiceFieldInfo.getId();
		id += sep + choiceFieldInfo.getChoiceOption(index).getId();
		return id;
	}

	/**
	 * Build a view component for the multiple choice field.
	 *
	 * @param  choiceField The field being modified.
	 * @param  index       Index of the option being modified.
     * @param  id          Id of the option.
     * @param  checked     Boolean indicating if option is checked.
	 * @return The view component for the field.
	 */
	LyFormField buildFormField(LyChoiceFieldInfo choiceField, short index, String id, boolean checked)
	{
		// Check marks ...
		LyInfoList<LyChoiceOption> tableOptions = choiceField.getTableOptions();
		LyChoiceOption			   choiceOption = choiceField.getChoiceOption(index);

		if ((tableOptions == null) || !tableOptions.contains(choiceOption))
			return null;

		// Now retrieve a SetController associated to this object
		LyEditTableSetController tableSetController = (LyEditTableSetController)_parent.getParent();
		LySetBehaviorInterface   behavior = tableSetController.getSetBehavior();
		Vector<LyChoiceOption>	 domain = behavior.getChoiceDomain(tableSetController, choiceField);
		LyChoice				 choice = new LyChoice();
		LyOption				 option;

		choice.setMultiple(true);

		option = new LyOption(choiceOption.getId(), "");
		if ((domain != null) && (!domain.contains(choiceOption)))
			option.setEnabled(false);

        option.setChecked(checked);
		choice.addOption(option);

		LyFormField field = new LyFormField(id);

		field.setFieldItem(choice);
		field.setController(tableSetController.getId());
		field.setNeedCheck(true);
		// field.setOptional(behavior.isOptional(_setController, fieldInfo));
		field.setEditable(behavior.isEditable(tableSetController, choiceField));
		field.setEnabled(behavior.isEnabled(tableSetController, choiceField));

		return field;
	} // end method buildFormField
} // end class LyEditTableMultipleChoiceSetBuilder
