/**
 * LyRelationTableBuilder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyRelationTableBuilder.java 41280 2012-10-19 13:07:36Z kchenal $
 */

package leon.control.form.builder;

import leon.control.LyController;
import leon.control.builder.LyEditTableBuilder;
import leon.control.builder.LySimpleTableBuilder;
import leon.control.form.LyMultiCompositionFieldConsultController;
import leon.control.form.LyRelationTableController;
import leon.info.LyAction;
import leon.info.LyArrayFieldInfo;
import leon.info.LyClassInfo;
import leon.info.LyRelationFieldInfo;
import leon.view.LyFormField;
import leon.view.LyHeader;
import leon.view.LyHeaders;
import leon.view.LyTable;
import leon.view.LyTool;
import leon.view.LyToolBar;
import leon.view.mobile.LyMobileViewManager;


/**
 * This builder creates a table view for a relation table controller It is based on the simple table
 * builder.
 *
 * @see LySimpleTableBuilder
 * @see LyRelationTableController
 */
public class LyRelationTableBuilder extends LyEditTableBuilder
{
	/** File version */
	public static final String __VERSION =
		"$Id: LyRelationTableBuilder.java 41280 2012-10-19 13:07:36Z kchenal $";

	/**
	 * Default constructor.
	 *
	 * @param parent parent controller
	 */
	public LyRelationTableBuilder(LyController parent)
	{
		super(parent);
	}

	/**
	 * Builds the toolbar for this area.
	 *
	 * @return the toolbar component
	 */
	protected LyToolBar buildToolBar()
	{
		LyRelationTableController parent = (LyRelationTableController)_parent;
		LyRelationFieldInfo		  relation = parent.getRelation();

		// If we don't already have a toolbar, build it
		if (!relation.isToolbarDisabled() || getViewManager().hasProperty(LyMobileViewManager.MOBILE_ONLY_PROPERTY))
			return super.buildToolBar();
		else
			return null;
	}

	/**
	 * Builds the table description.
	 *
	 * @return the table component
	 */
	protected LyTable buildTable()
	{
		LyRelationTableController parent = (LyRelationTableController)_parent;
		LyArrayFieldInfo		  array = parent.getArray();
		LyRelationFieldInfo		  relation = parent.getRelation();
		int						  nRows;
		LyTable					  table = super.buildTable();

		if (array != null)
		{
			nRows = array.getNRows();
			if (nRows <= 0)
			{
				nRows = array.getMaxCardinality();
				if (nRows <= 0)
					nRows = 3;
				else if (nRows > 10)
					nRows = 10;
			}
		}
		else
			nRows = relation.getNRows();
		table.setFooters(buildFooters());
		if (nRows > 0)
			table.setNRows(nRows);

		return table;
	} // end method buildTable

	/**
	 * Complete the toolbar.
	 *
	 * @param  toolBar the toolbar to complete
	 * @return the completed toolbar
	 */
	protected LyToolBar completeToolBar(LyToolBar toolBar)
	{
		// Add tools for ordering objects
		LyToolBar tools = super.completeToolBar(toolBar);

		if (!((LyRelationTableController)_parent).isOrdered())
			return tools;

		LyTool tool;

		if (tools == null)
		{
			tools = new LyToolBar();
			tools.setId("_toolBar");
			tools.setFill(LyToolBar.Fill.HORIZONTAL);
			toolBar.setColor("LY_DASHBOARD_HORIZONTAL_BACKGROUND");
		}
		else if (tools.getComponentCount() > 0)
			tools.addSep();

		tool = new LyTool(LyRelationTableController.DEMAND_MOVE_UP, "LY_TOOL_MOVE_UP");
		tool.setImage("LY_TOOL_MOVE_UP");
		tools.addTool(tool);

		tool = new LyTool(LyRelationTableController.DEMAND_MOVE_DOWN, "LY_TOOL_MOVE_DOWN");
		tool.setImage("LY_TOOL_MOVE_DOWN");
		tools.addTool(tool);

		return tools;
	} // end method completeToolBar

	/**
     * Complete the toolbar with edition tools.<br />
     * Default implementation adds two tools to validate and cancel the rows.
     * 
     * @param  toolBar the toolbar to complete
     * @return the completed toolbar
     */
    protected LyToolBar completeEditionTools(LyToolBar toolBar)
    {
        if (_parent.getParent() instanceof LyMultiCompositionFieldConsultController)
            return toolBar;
        else
            return super.completeEditionTools(toolBar);
    }
    
	/**
	 * Add extra tools if there is no menu.
	 *
	 * @param toolBar the toolbar to complete
	 */
	protected void completeToolBarForMenus(LyToolBar toolBar)
	{
		// No extra tools
	}
    
	/**
	 * Overridden to disable preference field on relation tables.
	 * @return null
	 */
    protected LyFormField buildPreferenceField()
    {
    	return null;
    }
    
	/**
	 * Builds the table header line description.
	 *
	 * @return the headers line
	 */
	protected LyHeaders buildHeaders()
	{
		LyRelationTableController parent = (LyRelationTableController)_parent;
		LyHeaders				  headers = super.buildHeaders();
		LyHeader				  header;
		int						  i, n = headers.getHeaderCount();

		if (parent.isOrdered())
		{
			for (i = 0; i < n; i++)
			{
				header = headers.getHeader(i);
				header.setSortable(false);
			}
		}

		return headers;
	}

	/**
	 * @see leon.control.builder.LySimpleTableBuilder#acceptSubAction(leon.info.LyAction, leon.info.LyClassInfo, leon.info.LyAction, leon.info.LyClassInfo)
	 */
	protected boolean acceptSubAction(LyAction action1, LyClassInfo classInfo1, LyAction action2,
		LyClassInfo classInfo2)
	{
		LyRelationTableController parent = (LyRelationTableController)_parent;

		if ((parent.getRelation() != null) && (parent.getRelation().getClassField() != null))
			return false; // do not add subactions : we have a field to indicate the class
		return super.acceptSubAction(action1, classInfo1, action2, classInfo2);
	}
	
    /**
     * @see leon.control.builder.LyPanelBuilder#completeRightToolBar(leon.view.LyToolBar)
     */
    protected LyToolBar completeRightToolBar(LyToolBar toolbar)
    {
        // Nothing to do
        return toolbar;
    }
} // end class LyRelationTableBuilder
