/*
 * LyAdvancedDataSourceEvent.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.data;

/**
 * This class implements an event of selection changed, visibility changed or state changed for an
 * object within a data source. The event is sent to 'registered listeners' : classes which
 * implements the LyAdvancedDataSourceListener interface and which were added as listener of the
 * dataSource.
 *
 * @see LyDataSource
 * @see LyAdvancedDataSourceListener
 */
public final class LyAdvancedDataSourceEvent extends LyDataSourceEvent
{
	/** File version */
	public static final String __VERSION =
		"$Id: LyAdvancedDataSourceEvent.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Old state of the object. For a selection changed event the values are 0 for unselected state
	 * and 1 for selected state. For a visibility changed event the values are 0 for hidden state
	 * and 1 for visible state. For a state change event, the values are one of the
	 * LyObject.LEVEL_XXX value defined in the LyObject class.
	 */
	protected short			   _oldState;

	/**
	 * Constructs a data source event of the given type with no associated values.
	 *
	 * @param source   Transmitter which created the event.
	 * @param type     Type of event which occured. One of the TYPE_XXX constants.
	 * @param object   Object concerned by this notification.
	 * @param oldState The old state value of the object.
	 */
	public LyAdvancedDataSourceEvent(LyDataSource source, short type, Object object,
		short oldState)
	{
		super(source, type, object, null);
		_oldState = oldState;
	}

	/**
	 * Gets the old state value of this event.
	 * @return the old state
	 */
	public short getOldState()
	{
		return _oldState;
	}
} // end class LyAdvancedDataSourceEvent
