/*
 * LyAdvancedDataSourceListener.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.data;

import java.util.*;


/**
 * This class extends LyDataSourceListener interface for receiving state changed and user events on
 * objects managed in a data source.
 *
 * <p>Three types of state changed events are throw : selection changed , visibility changed and
 * error changed.</p>
 *
 * @see LyDataSourceListener
 * @see LyDataSourceEvent
 */
public interface LyAdvancedDataSourceListener extends LyDataSourceListener
{
	/** File version */
	String __VERSION = "$Id: LyAdvancedDataSourceListener.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Method for receiving state changes on an object.
	 *
	 * @param  event Event notifying the state change of an object.
	 * @return True indicating if the event was used, false otherwise.
	 */
	boolean objectStateChanged(LyAdvancedDataSourceEvent event);

	/**
	 * Method for receiving user event on an object.
	 *
	 * @param  event Event notifying the user event of an object.
	 * @return True indicating if the event was used, false otherwise.
	 */
	boolean objectUserEvent(LyAdvancedDataSourceEvent event);

	/**
	 * Called when grouped events are sent.
	 *
	 * @param  events Sorted list of events.
	 * @return True if the events are used by the listener, false otherwise.
	 */
	boolean dataSourceEventsGrouped(ArrayList<LyDataSourceEvent> events);
}