/*
 * LyAdvancedWorkSpaceEvent.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.data;

/**
 * This class implements an event of adding/removing/changing an object within a workspace (dynamic
 * objects list). These events are sent to 'registered listeners' : classes which implements the
 * LyWorkSpaceListener interface and that were added as listeners on a workspace.
 *
 * @see LyWorkSpace
 * @see LyWorkSpaceListener
 */
public final class LyAdvancedWorkSpaceEvent extends LyWorkSpaceEvent
{
	/** File version */
	public static final String __VERSION =
		"$Id: LyAdvancedWorkSpaceEvent.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** The old state of this object for STATE_CHANGE event. */
	short					   _oldState;

	/** Original data source event for user events. */
	LyAdvancedDataSourceEvent  _origin;

	/**
	 * Constructs an event of the given type concerning the specified object without old position
	 * and associated values.
	 *
	 * @param source   Workspace that creates this event.
	 * @param type     Type of event that occured. One of the TYPE_XXX constants.
	 * @param object   Object concerned by the notification.
	 * @param index    Index of the object in the source workspace
	 * @param oldState Old value of the object state.
	 */
	public LyAdvancedWorkSpaceEvent(LyWorkSpace source, Type type, LyObject object, int index,
		short oldState)
	{
		super(source, type, object, index, -1, null);
		_oldState = oldState;
	}

	/**
	 * Constructs an event of the given type concerning the specified object without old position
	 * and associated values.
	 *
	 * @param source Workspace that creates this event.
	 * @param type   Type of event that occured. One of the TYPE_XXX constants.
	 * @param object Object concerned by the notification.
	 * @param index  Index of the object in the workspace.
	 * @param origin Origin of this event
	 */
	public LyAdvancedWorkSpaceEvent(LyWorkSpace source, Type type, LyObject object, int index,
		LyAdvancedDataSourceEvent origin)
	{
		this(source, type, object, index, (short)0);
		_origin = origin;
	}

	/**
	 * Gets the old state of this event.
	 * @return the old state
	 */
	public short getOldState()
	{
		return _oldState;
	}

	/**
	 * Gets the origin of this event.
	 * @return the origin of this event
	 */
	public LyAdvancedDataSourceEvent getOrigin()
	{
		return _origin;
	}
} // end class LyAdvancedWorkSpaceEvent
