/*
 * LyAdvancedWorkSpaceListener.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.data;

import java.util.*;


/**
 * This interface is the listener interface for receiving object state change events and object user
 * events on workspaces (dynamic objects list, LyWorkSpace).
 *
 * <p>State changed events are notified when :</p>
 *
 * <ul>
 *   <LI>an object is selected/unselected,</LI>
 *   <LI>an object is showed/hidden,</LI>
 *   <LI>an object error level is changed.</LI>
 * </ul>
 *
 * @see LyWorkSpace
 * @see LyAdvancedWorkSpaceEvent
 */
public interface LyAdvancedWorkSpaceListener extends LyWorkSpaceListener
{
	/** File version */
	String __VERSION = "$Id: LyAdvancedWorkSpaceListener.java 28997 2010-12-23 10:17:15Z bmichel $";

	/**
	 * Called when an object state is changed.
	 *
	 * @param  event Event notifying object state change.
	 * @return True if the event is used by the listener, false otherwise.
	 */
	boolean objectStateChanged(LyAdvancedWorkSpaceEvent event);

	/**
	 * Called when an object user event occured.
	 *
	 * @param  event Event notifying object user event.
	 * @return True if the event is used by the listener, false otherwise.
	 */
	boolean objectUserEvent(LyAdvancedWorkSpaceEvent event);

	/**
	 * Called when grouped events are sent.
	 *
	 * @param  events Sorted list of events.
	 * @return True if the events are used by the listener, false otherwise.
	 */
    boolean workSpaceEventsGrouped(ArrayList<LyWorkSpaceEvent> events);
}