/*
 * LyDataSourceListener.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.data;

/**
 * This class is the listener interface for receiving events on objects managed in a data source.
 *
 * <p>Three types of events are throw : creation , modification, deletion. A method corresponds to
 * each of these types.</p>
 *
 * @see LyDataSource
 * @see LyDataSourceEvent
 */
public interface LyDataSourceListener
{
	/** File version */
	String __VERSION = "$Id: LyDataSourceListener.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Method for receiving an object creation.
	 *
	 * @param  event Event notifying the creation of an object.
	 * @return True indicating if the event was used, false otherwise.
	 */
	boolean objectCreated(LyDataSourceEvent event);

	/**
	 * Method for receiving an object modification.
	 *
	 * @param  event Event notifying the modification of an object.
	 * @return True indicating if the event was used, false otherwise.
	 */
	boolean objectModified(LyDataSourceEvent event);

	/**
	 * Method for receiving an object deletion.
	 *
	 * @param  event Event notifying the deletion of an object.
	 * @return True indicating if the event was used, false otherwise.
	 */
	boolean objectDeleted(LyDataSourceEvent event);
}