/*
 * LyField.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.data;

/**
 * The field interface is used to represent a field of a LyClass.
 *
 * <p>A LyField is mainly defined by an identifier and a java class for storing effective values of
 * this LyField.</p>
 *
 * <p>This class is implemented by both physical fields ans logical fields (classes that both
 * extends LyPeerField or LyFieldInfo).</p>
 *
 * @see LyClass
 * @see leon.peer.LyPeerField
 * @see leon.info.LyFieldInfo
 */
public interface LyField
{
	/** File version */
	String __VERSION = "$Id: LyField.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Gets identifier of this field.
	 *
	 * @return Identifier of this field.
	 */
	String getId();

	/**
	 * Gets the java class representation of the value of this field.
	 *
	 * @return Java class used to represent values of this field.
	 */
	Class<?> getValueClass();

	/**
	 * Builds the appropriate field value (LyValue) corresponding to this field from an untyped
	 * value object. The returned value is an instance of an inherited class of LyValue that depends
	 * on the class of the given field.
	 *
	 * @param  value Untyped value.
	 * @return A field value for the given field that corresponds to the specified value.
	 */
	LyValue buildFieldValue(Object value);
}