/*
 * LyResponseListener.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.data;

/**
 * This interface allows to receive responses to demands adressed to data sources.
 *
 * @see LyDataSource
 * @see LyResponse
 * @see leon.peer.LyRequestEncoder
 */
public interface LyResponseListener
{
	/** File version */
	String __VERSION = "$Id: LyResponseListener.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Method for receiving a new object in a response. This method is called by LyResponse when a
	 * new objet is added in response and thus will be called several times for demands that return
	 * several objects.
	 *
	 * @param response  Response for which a new object is received.
	 * @param object    The new object returned in the response.
	 * @param oldValues When the demand was a modification of an object, this parameter may contains
	 *                  the list of values that were changed. In other cases, this parameter is
	 *                  null.
	 */
	void newResponse(LyResponse response, Object object, LyValueSet oldValues);

	/**
	 * Method for ending a response. This method is called by LyResponse when all results have been
	 * received to notify the response is terminated.
	 *
	 * @param response Response which end is notified.
	 */
	void endResponse(LyResponse response);
}