/*
 * LyObjectSource.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.data;

import java.util.*;
import leon.info.*;
import leon.peer.*;


/**
 * Constructor for a temporary object source : object source that can be freed.
 *
 * @see LyObjectSource
 */
public class LyTemporaryObjectSource extends LyObjectSource
{
	/** File version */
	public static final String __VERSION =
		"$Id: LyTemporaryObjectSource.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Indicates if the error events should be sent by this temporary object source Temporary
	 * objectsource may load temporary objects like import that should not be kept in 'general
	 * errors' list.
	 */
	protected boolean		   _handleErrorEvents;

	/**
	 * Constructs a temporary object source for a given class info.
	 *
	 * @param provider  Provider on which this data source is connected.
	 * @param classInfo Class info for which an object source is constructed.
	 */
	public LyTemporaryObjectSource(LyDataProvider provider, LyClassInfo classInfo)
	{
		super(provider, classInfo);
		_handleErrorEvents = true;
	}

	/**
	 * Free this temporary object source.
	 */
	public void free()
	{
		_application.removeDataSource(this);
		if (_dataSource != null)
			_application.removeDataSource(_dataSource);
	}

	/**
	 * Accessor on error events handling Indicates if the error events should be sent by this
	 * temporary object source Temporary objectsource may load temporary objects like import that
	 * should not be kept in 'general errors' list.
	 *
	 * @param status default error handling status is true
	 */
	public void setHandleErrorEvents(boolean status)
	{
		_handleErrorEvents = status;
	}

	/**
	 * Accessor on error events handling Indicates if the error events should be sent by this
	 * temporary object source Temporary objectsource may load temporary objects like import that
	 * should not be kept in 'general errors' list.
	 *
	 * @return status default error handling status is true
	 */
	public boolean isHandleErrorEvents()
	{
		return _handleErrorEvents;
	}

	/**
	 * Internal method to post an event to listeners.
	 *
	 * @param     event         Posted event.
	 * @param     skipListeners Listeners that were already notified and that must be skipped by
	 *                          this method to avoid a double notification of the same event.
	 * @return    True if the vent was sent and used by at least one listener, false otherwise.
	 * @exception IllegalArgumentException If the given event is null.
	 */
	protected boolean postEvent(LyDataSourceEvent event, Vector<LyDataSourceListener> skipListeners)
	{
		if ((!isHandleErrorEvents())
				&& ((event.getType() == LyDataSourceEvent.TYPE_ERROR)
					|| (event.getType() == LyDataSourceEvent.TYPE_ERROR_ADDED)
					|| (event.getType() == LyDataSourceEvent.TYPE_ERROR_REMOVED)))
			return true;

		return super.postEvent(event, skipListeners);
	}
} // end class LyTemporaryObjectSource
