/*
 * LyTextValue.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.data;

import leon.info.*;


/**
 * This class offers methods for constructing, accessing and modifying values corresponding to a
 * LyTextFieldInfo.
 *
 * @see LyTextFieldInfo
 * @see LyObject
 */
public final class LyTextValue extends LyFieldInfoValue
{
	/** File version */
	public static final String __VERSION =
		"$Id: LyTextValue.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Constructs an empty relation value for the given text field.
	 *
	 * @param textFieldInfo Text fieldInfo for which a value is created.
	 */
	public LyTextValue(LyTextFieldInfo textFieldInfo)
	{
		super(textFieldInfo);
	}

	/**
	 * Constructs a text value for the given text field from a string.
	 *
	 * @param textFieldInfo Text fieldInfo for which a value is created.
	 * @param value         String value.
	 * @see   LyRelationFieldInfo#parse
	 */
	public LyTextValue(LyTextFieldInfo textFieldInfo, String value)
	{
		super(textFieldInfo, value);
	}

	/**
	 * Constructs a text value for the given text field from an object. The given object is
	 * converted to a string using toString method.
	 *
	 * @param textFieldInfo Text fieldInfo for which a value is created.
	 * @param value         Value given as an object.
	 * @see   Object#toString
	 */
	public LyTextValue(LyTextFieldInfo textFieldInfo, Object value)
	{
		super(textFieldInfo, ((value == null) ? null : value.toString()));
	}
} // end class LyTextValue
