/*
 * LyWorkSpaceListener.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.data;

/**
 * This interface is the listener interface for receiving object events on workspaces (dynamic
 * objects list, LyWorkSpace).
 *
 * <p>Events are notified when :</p>
 *
 * <ul>
 *   <LI>an object is added in a workspace,</LI>
 *   <LI>an object is removed from a workspace,</LI>
 *   <LI>an object is changed within the workspace because its values have changed, or because the
 *     sort of the list has changed.</LI>
 * </ul>
 *
 * @see LyWorkSpace
 * @see LyWorkSpaceEvent
 */
public interface LyWorkSpaceListener
{
	/** File version */
	String __VERSION = "$Id: LyWorkSpaceListener.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Called when an object is added in the workspace.
	 *
	 * @param  event Event notifying object add.
	 * @return True if the event is used by the listener, false otherwise.
	 */
	boolean objectAdded(LyWorkSpaceEvent event);

	/**
	 * Called when an object is removed from the workspace.
	 *
	 * @param  event Event notifying object remove.
	 * @return True if the event is used by the listener, false otherwise.
	 */
	boolean objectRemoved(LyWorkSpaceEvent event);

	/**
	 * Called when an object has changed in the workspace. This call notifies a change on the object
	 * concerned by the event either because its values have changed or because the sort on the
	 * workspace has changed and consequently the object position within the list has changed. These
	 * two cases can easily be distinguished by accessing the values set contained in the event (if
	 * it's not null, this is an change on the object, not a modification of the sort).
	 *
	 * <p>Note that object position may also change if one of the modified fields is one of the
	 * fields specified in the list sort.</p>
	 *
	 * @param  event Event notifying object change.
	 * @return True if the event is used by the listener, false otherwise.
	 */
	boolean objectChanged(LyWorkSpaceEvent event);
}