/*
 * LyNoCache.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.data.cache;

import leon.data.*;
import leon.info.*;


/**
 * This class implements a cache that never contains objects !
 *
 * <p>Each time objects are requested the response is negative to force reload of objects on data
 * provider.</p>
 */
public class LyNoCache extends LyObjectCache
{
	/** File versioin */
	public static final String __VERSION =
		"$Id: LyNoCache.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Constructor of an empty cache.
	 *
	 * @param cacheClass classDescription associated with the dataSource.
	 * @param policy    Description of th epolicy of thsi cache.
	 */
	public LyNoCache(LyClass cacheClass, LyInfoCache policy)
	{
		super(cacheClass, policy);
	}

	
	/**
	 * @see leon.data.LyCache#checkObject(leon.data.LyObjectId)
	 */
	protected boolean checkObject(LyObjectId id)
	{
		Object object = _objects.get(id);

		if ((object != null)
				&& (object instanceof LyObject)
				&& (((LyObject)object).getPeer() == null))
			return true;

		return false;
	}

	/**
	 * @see leon.data.LyCache#checkObject(java.lang.Object)
	 */
	protected boolean checkObject(Object object)
	{
		if ((object instanceof LyObject)
				&& (((LyObject)object).getPeer() == null))
		{
			LyObjectId objectId = _class.getObjectId(object);

			if (_objects.get(objectId) == object)
				return true;
		}

		return false;
	}

	/**
	 * Tests whether this cache is completely loaded or not.
	 *
	 * @return True if the cache is loaded, false otherwise.
	 */
	public boolean isLoaded()
	{
		return false;
	}

	/**
	 * Registers an object id as corresponding to an unknown object. Once registered, the getObject
	 * method won't throw exceptions for this id.
	 *
	 * @param id Object id that is registered.
	 * @see   #getObject
	 */
	protected final void registerUnknownId(LyObjectId id)
	{
		if (_objects.containsKey(id))
			_objects.remove(id);
	}
	
	/**
	 * Setter for the allow loaded attribute.
	 * For a no cache, allowLoaded is forced to be false;
	 * @param allowLoaded the value of the allowLoaded attribute
	 */
	public void allowLoaded(boolean allowLoaded)
	{
		_allowLoaded = false;		
	}

} // end class LyNoCache
