/*
 * LyFieldInfoDaemon.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.data.daemon;

import leon.data.*;
import leon.info.*;


/**
 * Daemon for calculated relations.
 */
public abstract class LyFieldInfoDaemon extends LyDaemon
{
	/** File version */
	public static final String __VERSION =
		"$Id: LyFieldInfoDaemon.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Fieldinfo controlled by this daemon. */
	LyFieldInfo				   _fieldInfo;

	/**
	 * Default constructor.
	 * @param sourceClass the source class
	 * @param fieldInfo the field associated to the daemon
	 */
	protected LyFieldInfoDaemon(LyClassInfo sourceClass, LyFieldInfo fieldInfo)
	{
		super(sourceClass, fieldInfo);

		_fieldInfo = fieldInfo;
	}

	/**
	 * To get the value for the specified object.
	 *
	 * @param  object the object we want the value.
	 * @return the value of field info for the specified object.
	 */
	public abstract Object getValue(LyObject object);

	/**
	 * To free a calculated value for the specified object.
	 *
	 * @param object the object we want to free the value.
	 */
	public abstract void freeValue(LyObject object);

	/**
	 * Get the fieldInfo handled by this daemon
	 * @return a fieldInfo
	 */
	public LyFieldInfo getFieldInfo()
	{
		return _fieldInfo;
	}

	/**
	 * Sets the value in the object, and use LyContext and/or lock depending on the event Same as
	 * object.setValue(fieldInfo, value, operFlags) based on the event.
	 * @param object the object to modify
	 * @param fieldInfo the fieldInfo to change
	 * @param value the new value
	 * @param event the source event
	 */
	protected void setValue(LyObject object, LyFieldInfo fieldInfo, Object value,
		LyDataSourceEvent event)
	{
		setValue(object, fieldInfo, value, event,
            (event == null ? true : !event.isLocal())); // Don't commit if event is local
	}

	/**
	 * Sets the value in the object, and use LyContext and/or lock depending on the event Same as
	 * object.setValue(fieldInfo, value, operFlags) based on the event.
	 * @param object the object to modify
	 * @param fieldInfo the fieldinfo to change
	 * @param value the new value
	 * @param event the source event
	 * @param commit indicates if a commit must be performed
	 */
	protected void setValue(LyObject object, LyFieldInfo fieldInfo, Object value,
		LyDataSourceEvent event, boolean commit)
	{
		if ((event == null)
            || !event.isNotified())
		{
			// defaults from setValue(fieldInfo, value)
			object.setValue(fieldInfo, value);
			if (commit)
				object.set();
		}
		else
			// Notified from JMS : use no control and no context
			object.setLocalValue(fieldInfo, value);
	}
} // end class LyFieldInfoDaemon
