/*
 * LyClassInfoComparator.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.data.misc;

import leon.data.*;
import leon.info.*;
import leon.info.infointerface.*;


/**
 * Class used to compare two objects with their class info.
 */
public class LyClassInfoComparator
	implements LyComparatorInterface
{
	/** File version */
	public static final String __VERSION =
		"$Id: LyClassInfoComparator.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Method to compare two objects according a given sort criterion. The given objects are not
	 * null and are not equals.
	 *
	 * @param  criterion the sort criterion.
	 * @param  object1   the first object
	 * @param  object2   the second object
	 * @return O if the two values are equals, a positive value if value of object1 is "greater"
	 *         than value of object2, a negative value otherwise.
	 */
	public int compare(LySortCriterion criterion, LyObject object1, LyObject object2)
	{
		if (object1 == object2)
			return 0;
		if (object1 == null)
			return -1;
		if (object2 == null)
			return 1;

		LyClassInfo classInfo1 = object1.getClassInfo();
		LyClassInfo classInfo2 = object2.getClassInfo();
		String	    label1, label2;

		// Objects do not belong to the same class info
		if (!classInfo1.equals(classInfo2))
		{
			label1 = classInfo1.getName();
			label2 = classInfo2.getName();

			return (label1.compareTo(label2));
		}
		// If objects belong to the same class info, use sort by name on objects
		else
			return (object1.compare(object2, true));
	}
} // end class LyClassInfoComparator
