/*
 * LyAction.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.info;

import java.util.*;

import leon.info.infointerface.*;
import leon.info.misc.*;
import leon.misc.*;


/**
 * This class implements the description of actions available for the objects of the application.
 * <br>
 * Default actions are defined in a resource file (leon.xml) Generic actions have an id that starts
 * with underscore (_) Internal actions (not cloneable) have an id starting with 2 underscores (__)
 * <br>
 * Actions can have marks, these marks define properties for actions. Generic marks are defined as
 * public static, their names start with MARK_.<br>
 * Actions may require a minimum and a maximum number of objects. Additionally, a filter may be set
 * on objects to impose a condition on the objects.<br>
 * An action can have a shortcut that is used to quickly invoke the action when presented in a
 * menubar.<br>
 * An action may have parameters. A parameter is a pair (name, value) where name is a string and
 * value an object of the following classes : LyInfo, LyBitSet, String. Parameters are used to
 * parameter actions. Valid parameters for actions depend on each action.<br>
 * An action controller class is associated to an action. A controller is in charge of controlling
 * the view built when the action is executed.<br>
 * An action behavior class may be associated to an action. This behavior allows to customize action
 * processing.<br>
 * An action builder class can be associated to an action. It is in charge to build the view
 * corresponding to the action. The builder constructs an empty view (containing no data), this view
 * is updated by the action controller.<br>
 * An alternate way is to associate a specific view to the action. This view is a reference to an
 * XML file describing the view according to views.dtd. In that case the builder won't be used.<br>
 * An action can specify what are the other actions that can be accessed from the action view.<br>
 * Finally, an action can have a default action which is the action invoked when the user
 * double-clicks in the action view.<br>
 * User defined actions may "clone" default actions. If an action clones a generic action, the
 * generic implementation will be able to offer generic processing : checking validity, name,
 * resource, execute...
 *
 * <p>See also tag <ACTION> of leon.dtd</p>
 */
public class LyAction extends LyObjectInfo
{
	public static final String __version =
		"$Id: LyAction.java 41059 2012-10-08 10:25:23Z jlsaouli $";

	/** Id of generic consult action. */
	public static final String ID_CONSULT = "_consult".intern();

	/** Id of generic create action. */
	public static final String ID_CREATE = "_create".intern();

	/** Id of generic web service create action. */
	public static final String ID_CREATEWS = "_createws".intern();

	/** Id of generic clone action. */
	public static final String ID_CLONE = "_clone".intern();

	/** Id of generic action : COPY. */
	public static final String ID_COPY = "__copy".intern();

	/** Id of generic action : Paste. */
	public static final String ID_PASTE = "__paste".intern();

	/** Id of generic action : SET. */
	public static final String ID_SET = "_set".intern();

	/** Id of generic action : SETWS. */
	public static final String ID_SETWS = "_setws".intern();

	/** Id of generic action : DELETE. */
	public static final String ID_DELETE = "_delete".intern();

	/** Id of generic action : DELETEWS. */
	public static final String ID_DELETEWS = "_deletews".intern();

	/** Id of generic action : PRINT. */
	public static final String ID_PRINT = "_print".intern();

	/** Id of generic action : PRINT. */
	public static final String ID_PRINT_TABLE = "__printTable".intern();

	/** Id of generic action : (CHANGE) PASSWORD. */
	public static final String ID_PASSWORD = "_password".intern();

	/** Id of generic action : SAVE_AS. */
	public static final String ID_SAVE_AS = "_save_as".intern();

	/** Id of generic action : ACTIVATE. */
	public static final String ID_ACTIVATE = "_activate".intern();

	/** Id of generic action : IMPORT. */
	public static final String ID_IMPORT = "_import".intern();

	/** Id of generic action : EXPORT. */
	public static final String ID_EXPORT = "_export".intern();

	/** Id of generic action : XREF. */
	public static final String ID_XREF = "_xref".intern();

	/** Id of generic action : MAP. */
	public static final String ID_MAP = "_map".intern();

	/** Id of generic action : TABLE. */
	public static final String ID_TABLE = "_table".intern();
	
    /** Id of generic action : multiPageTable. */
    public static final String ID_MULTI_PAGE_TABLE = "_multiPageTable".intern();

	/** Id of generic action : TREE. */
	public static final String ID_TREE = "_tree".intern();

	/** Id of generic action : LOGIN. */
	public static final String ID_LOGIN = "_login".intern();

	/** Id of generic action : DASHBOARD. */
	public static final String ID_DASHBOARD = "_dashboard".intern();

	/** Id of generic action : EXIT. */
	public static final String ID_EXIT = "_exit".intern();

	/** Id of generic action : EXECUTE. */
	public static final String ID_EXEC = "_exec".intern();

	/** Id of generic action : MAIL. */
	public static final String ID_MAIL = "_mail".intern();

	/** Id of internal action : Waiting view. */
	public static final String ID_WAITING_VIEW = "_wait".intern();

	/** Id of internal action : TEXT. */
	public static final String ID_TEXT = "_text".intern();

	/** Id of internal action : MODIFY. */
	public static final String ID_MODIFY = "_modify".intern();

	/** Id of internal action : REPORTING. */
	public static final String ID_REPORTING = "_reporting".intern();

	/** Id of internal action : Previous page. */
	public static final String ID_PREVIOUS_PAGE = "__pagePrevious".intern();

	/** Id of internal action : Next page. */
	public static final String ID_NEXT_PAGE = "__pageNext".intern();

	/** Id of internal action : Context. */
	public static final String ID_CONTEXT = "__context".intern();

	/** Id of internal action : New filter. */
	public static final String ID_NEW_FILTER = "__filter_new".intern();

	/** Id of internal action : Set filter. */
    public static final String ID_SET_FILTER = "__filter_set_expression".intern();

	/** Id of generic action : Start Module. */
	public static final String ID_START_MODULE = "__startModule".intern();

	/** Id of generic action : Show. */
	public static final String ID_SHOW = "__show".intern();

	/** Id of generic action : Hide. */
	public static final String ID_HIDE = "__hide".intern();

	/** Id of generic action : Undo. */
	public static final String ID_UNDO = "__undo".intern();

	/** Id of generic action : Redo. */
	public static final String ID_REDO = "__redo".intern();

	/** Id of generic action : select. */
    public static final String ID_SELECT = "__select".intern();

    /** Id of generic action : selectByName. */
    public static final String ID_SELECT_BY_NAME = "__selectByName".intern();

    /** Id of generic action : selectFile. */
    public static final String ID_SELECT_FILE = "__selectFile".intern();

    /** Id of generic action : consultRelationTable. */
    public static final String ID_CONSULT_RELATION_TABLE = "__consultRelationTable".intern();

    /** Id of generic action : setRelationTable. */
    public static final String ID_SET_RELATION_TABLE = "__setRelationTable".intern();

    /** Id of generic action : fastFilter. */
    public static final String ID_FAST_FILTER = "__fastFilter".intern();

    /** Id of generic action : updateArrayValue. */
    public static final String ID_UPDATE_ARRAY_VALUE = "__update_array_value".intern();

    /** Id of generic action : moveUpTool. */
    public static final String ID_MOVE_UP_TOOL = "__moveUpTool".intern();

    /** Id of generic action : moveDownTool. */
    public static final String ID_MOVE_DOWN_TOOL = "__moveDownTool".intern();

    /** Id of generic action : choose. */
    public static final String ID_CHOOSE = "__choose".intern();

    /** Id of generic action : sortWizard. */
    public static final String ID_SORT_WIZARD = "__sortWizard".intern();

    /** Id of generic action : userInput. */
    public static final String ID_USER_INPUT = "__userInput".intern();

    /** Id of generic action : importTable. */
    public static final String ID_IMPORT_TABLE = "__importTable".intern();

    /** Id of generic action : exportPreview. */
    public static final String ID_EXPORT_PREVIEW = "__exportPreview".intern();

    /** Id of generic action : dateTool. */
    public static final String ID_DATE_TOOL = "__dateTool".intern();

    /** Id of generic action : chartFile. */
    public static final String ID_CHART_FILE = "__chartFile".intern();

    /** Id of generic action : deleteChart. */
    public static final String ID_DELETE_CHART = "__deleteChart".intern();

    /** Id of generic action : setEmpty. */
    public static final String ID_SET_EMPTY = "__setEmpty".intern();
    
    /** Id of generic action : gantt. */
    public static final String ID_GANTT = "_gantt".intern();
    
    /** Id of generic action : chart. */
    public static final String ID_CHART = "_chart".intern();
    
    /** Id of generic action : canvas. */
    public static final String ID_CANVAS = "_canvas".intern();

	/**
	 * Mark DEFAULT : indicates that this action is a the default action (action invoked on
	 * double-click in some views).
	 */
	public static final int    MARK_DEFAULT = 0;

	/** Mark TOOL : this action is enabled as a tool in toolbars. */
	public static final int    MARK_TOOL = 1;

	/** Mark FORM : this action is enabled in forms toolbars. */
	public static final int    MARK_FORM = 2;

	/**
	 * Mark TRANSVERSE : this action is enabled in relations toolbars in edition forms on relations
	 * that reference classInfos that contain this action.
	 */
	public static final int    MARK_TRANSVERSE = 3;

	/** Mark HIDDEN : this action is hidden (not shown to user). */
	public static final int    MARK_HIDDEN = 4;

	/**
	 * Mark DIALOG : Indicates whether this action opens a dialog view or not. The only effect is to
	 * concatenate "..." to action name.
	 */
	public static final int    MARK_DIALOG = 5;

	/**
	 * Mark NO_TITLEBAR : Specifies that the view corresponding to this action should not contain a
	 * titlebar.
	 */
	public static final int    MARK_NO_TITLEBAR = 6;

	/**
	 * Mark NO_TOOLBAR : Specifies that the view corresponding to this action should not contain a
	 * toolbar.
	 */
	public static final int    MARK_NO_TOOLBAR = 7;

	/**
	 * Mark NO_COMMANDS : Specifies that the view corresponding to this action should not contain a
	 * commands bar.
	 */
	public static final int    MARK_NO_COMMANDS = 8;

	/** Mark NO_TABS : Ignores fieldInfos tabs when adding fields in forms. */
	public static final int    MARK_NO_TABS = 9;

	/** Mark NO_MENU : Disable contextual menus. */
	public static final int    MARK_NO_MENU = 10;

	/**
	 * Mark STATUS : Specifies that this action is invoked when selecting the status mark for an
	 * object in a table view.
	 */
	public static final int    MARK_STATUS = 11;

	/** Mark NO_MENUBAR : Disable menubar. */
	public static final int    MARK_NO_MENUBAR = 12;

	/** Mark FRAME_VIEW : Action shown in frame (or in dialog). */
	public static final int    MARK_FRAME_VIEW = 13;

	/** Mark MODAL : Modal view (only used for dialogs). */
	public static final int    MARK_MODAL = 14;

	/** Mark NO_MESSAGES : No message bar. */
	public static final int    MARK_NO_MESSAGES = 15;

	/** Mark NO_DECORATION : No decoration. */
	public static final int    MARK_NO_DECORATION = 16;

	/** Mark NO_CONTEXT : No context. */
	public static final int    MARK_NO_CONTEXT = 17;

	/** Mark PORTLET : action available as portlet. */
	public static final int    MARK_PORTLET = 18;

	/** Mark WEBSERVICE : action available as webservice. */
	public static final int    MARK_WEBSERVICE = 19;

	/** Mark MOBILEONLY : action available on mobile device only. */
	public static final int    MARK_MOBILEONLY = 20;

	/** Mark NOTFORMOBILE : action available only on non mobile device. */
	public static final int    MARK_NOTFORMOBILE = 21;

	/** Mark POPUP : action displayed as popup. */
	public static final int    MARK_POPUP = 22;
	
	/** Mark MOBILEWEBVIEW : action displayed as a webview on mobile device. */
	public static final int    MARK_MOBILEWEBVIEW = 23;

	/** Parameter name : classInfo Used for actions that have a classInfo as parameter. */
	public static final String PARAM_CLASS = "_classInfo".intern();

	/** Parameter name : fieldInfo Used for actions that have a fieldInfo as parameter. */
	public static final String PARAM_FIELD = "_fieldInfo".intern();

	/** Parameter name : filter Used for actions that have a filter as parameter. */
	public static final String PARAM_FILTER = "_filter".intern();

	/** Parameter name : sort Used for actions that have a sort as parameter. */
	public static final String PARAM_SORT = "_sort".intern();

	/** Parameter name : fieldInfo marks Used for actions that have a sort as parameter. */
	public static final String PARAM_FIELD_MARKS = "_fieldMarks".intern();

	/** Action id separator used for composite action id. */
	public static final char   ACTION_ID_SEP = '.';

	/** Default parameter value : null value. */
	static final Object		   PARAM_NULL = new Object();

	/** Default parameter value : null value. */
	static final Boolean	   PARAM_REQUIRED = Boolean.TRUE;


	/**
	 * Gets mark index corresponding to the given mark name.
	 *
	 * @param  markName Mark name.
	 * @param  add indicates if the.mark must be added if it doesn't exit
	 * @return Mark value.
	 */
	public static int getMark(String markName, boolean add)
	{
		return LyInfoTokens.getMarkIndex(LyBitSet.Type.A_MARKS, markName, add);
	}

	/**
	 * Gets mark index corresponding to the given mark name.
	 *
	 * @param  markName Mark name.
	 * @return Mark value.
	 */
	public static int getMark(String markName)
	{
		return getMark(markName, false);
	}

	/**
	 * Get mark name for the given index of a mark.
	 *
	 * @param  markIndex Index of the mark.
	 * @return Mark name.
	 */
	public static String getMarkName(int markIndex)
	{
		return LyInfoTokens.getMarkName(LyBitSet.Type.A_MARKS, markIndex);
	}

	/** Controller java class. Constructed by reflection. */
	LyJavaName     			 _controllerClassName;

	/** Behavior java class. Constructed by reflection. */
	LyJavaName     			 _behaviorClassName;

	/** Builder java class. Constructed by reflection. */
	LyJavaName     			 _builderClassName;

	/** Xml view file (conforms to views.dtd) that describes the view to be used for this action. */
	String					 _view;

	/** Menu : name of the menu to which this action belongs to. Menu may be null (not in menus). */
	String					 _menu;

	/**
	 * Minimum number of objects which must be selected to enable the action or 0 if no minimum is
	 * required (-1 means don't care of selections).
	 */
	int						 _min;

	/**
	 * Maximum number of objects that can be selected to enable the action or -1 if no maximum is
	 * required.
	 */
	int						 _max;

	/**
	 * Filter associated with this action, and which may disable the action selected object do not
	 * match filter.
	 */
	LyFilter				 _filter;

	/** Action parameters declaration. */
	Vector<LyParameterDecl>  _parametersDecl;

	/** Action parameters. */
	Vector<LyParameter>		 _parameters;

	/**
	 * Actions that can be invoked from this action. These actions are added to actions menu and to
	 * the default toolbar built for this action.
	 */
	LyInfoList<LyAction>     _actions;

	/**
	 * Shortcut for the corresponding menu item. This allows to invoke the action from keyboard when
	 * presenteed in a menubar.
	 */
	String					 _shortcut;

	/** Default action. Action launched when user double-clicks on action view. */
	LyAction				 _defaultAction;

	/** Specific resources. */
	Vector<LyActionResource> _resources;

	/** Geometry for action view (used for fat client only). */
	LyGeometry				 _geometry;

	/** Specifies that this action is an action reference. */
	boolean					 _actionRef;

	/** Specifies the zone target for this action. */
	String					 _target;

	/** Specifies if this action is abstract or not. */
	boolean					 _abstract;

	/** Environment of the action. */
	LyEnvironment			 _parentEnvironment;

	/**
	 * Initialisation of action attributes.
	 */
	protected void init()
	{
		super.init();

		// Initialize class attributes.
		_controllerClassName = null;
		_behaviorClassName = null;
		_builderClassName = null;
		_view = null;
		_menu = null;
		_min = -1;
		_max = -1;
		_filter = null;
		_parametersDecl = null;
		_parameters = null;
		_actions = null;
		_shortcut = null;
		_defaultAction = null;
		_resources = null;
		_geometry = null;
		_actionRef = false;
		_target = null;
		_abstract = false;
		_parentEnvironment = null;
	}

	/**
	 * Init this instance attributes from its template.
	 *
	 * @param parts Indicates which parts of the template are copied.
	 */
	void initFromTemplate(int parts)
	{
	    parts &= ~_initParts;
		if (parts == 0)
			return;
		if (_template == null)
			return;

		super.initFromTemplate(parts);

		LyAction template = (LyAction)_template;

		if ((parts & PART_GUI) > 0)
		{
			if (_controllerClassName == null)
				_controllerClassName = template._controllerClassName;
			if (_behaviorClassName == null)
				_behaviorClassName = template._behaviorClassName;
			if (_builderClassName == null)
				_builderClassName = template._builderClassName;

			if (_view == null)
				_view = template._view;

            if  (_target == null)
                _target = template._target;

			_menu = template._menu;
			_min = template._min;
			_max = template._max;
			_filter = template._filter;
			_shortcut = template._shortcut;
			_defaultAction = template._defaultAction;
			_geometry = template._geometry;

			if (template._parametersDecl != null)
				_parametersDecl = new Vector<LyParameterDecl>(template._parametersDecl);
			if (template._parameters != null)
				_parameters = new Vector<LyParameter>(template._parameters);
			if (template._actions != null)
				_actions = new LyInfoList<LyAction>(template._actions);
			if (template._resources != null)
				_resources = new Vector<LyActionResource>(template._resources);
		}
	} // end method initFromTemplate

	/**
	 * Constructor of an external action.
	 *
	 * @param id Identifier of the action.
	 */
	LyAction(String id)
	{
		super(id, false);
	}

	/**
	 * Constructor of action.
	 *
	 * @param id              Id of the action.
	 * @param template        Reference of the action template.
	 * @param controllerClass Java class name of the action controller.
	 * @param behaviorClass   Java class name of the action behavior.
	 * @param builderClass    Java class name of the action builder.
	 * @param view            Path to XML view file.
	 * @param target          The target zone id.
	 */
	protected LyAction(String id, LyAction template, String controllerClassName,
		String behaviorClassName, String builderClassName, String view, String target)
	{
		this(id);
		construct(template, controllerClassName, behaviorClassName, builderClassName, view, target);
	}

	/**
	 * Construction of an action.
	 *
	 * @param template        Reference of the action template.
	 * @param controllerClass Java class name of the action controller.
	 * @param behaviorClass   Java class name of the action behavior.
	 * @param builderClass    Java class name of the action builder.
	 * @param view            Path to XML view file.
	 * @param target          The target zone id.
	 */
	protected void construct(LyAction template, String controllerClassName,
		String behaviorClassName, String builderClassName, String view, String target)
	{
		super.construct(template);

		// Sets controller class
		setControllerClassName(controllerClassName);

		// Sets behavior class
		setBehaviorClassName(behaviorClassName);

		// Sets builder class
		setBuilderClassName(builderClassName);

		// Sets specific view
        if (view != null)
            _view = view;

		// Sets the target
        if (target != null)
            _target = target;
	}

	/**
	 * Constructs a new action for an action that applies on a classInfo. This allows to instantiate
	 * an action for a given classInfo. The parameter named _classInfo with the given classInfo is
	 * added on this new action.
	 *
	 * @param template   Action template.
	 * @param classInfo  Class info on which the action is applied.
	 * @param parameters Action parameters.
	 */
	public LyAction(LyAction template, LyClassInfo classInfo, Hashtable<String, Object> parameters)
	{
		this(null, template, classInfo, parameters);
	}

	/**
	 * Constructs a new action for an action that applies on a classInfo. This allows to instantiate
	 * an action for a given classInfo. The parameter named _classInfo with the given classInfo is
	 * added on this new action.
	 *
	 * @param id         Id of the action.
	 * @param template   Action template.
	 * @param classInfo  Class info on which the action is applied.
	 * @param parameters Action parameters.
	 */
	public LyAction(String id, LyAction template, LyClassInfo classInfo,
		Hashtable<String, Object> parameters)
	{
		super(id, template);

		initFromTemplate();

		if (classInfo != null)
		{
            String actionName = ((LyAction)_template).getName();

            if ((actionName != null) && !actionName.equals("LY_ACTION_TABLE") && (!actionName.equals("LY_ACTION_MULTI_PAGE_TABLE")))
                setName(actionName + " : " + classInfo.getName());
            else
                setName(classInfo.getName());

    		if (_alias == null)
    			setAlias(classInfo.getAlias());
    		if (_group == null)
    			setGroup(classInfo.getGroup());
    		if (_filePrefix == null)
    			setFilePrefix(classInfo.getFilePrefix());

            setApplication(classInfo.getApplication());
            setParent(classInfo);

    		addParameterDecl(new LyParameterDecl(PARAM_CLASS, LyParameterDecl.Type.CLASS, true, null));
    		addParameter(new LyParameter(PARAM_CLASS, classInfo));
		}


		// Add extra parameters
		if (parameters != null)
		{
			Enumeration<String> keys = parameters.keys();
			String			    key;

			while (keys.hasMoreElements())
			{
				key = keys.nextElement();
				addParameter(new LyParameter(key, parameters.get(key)));
			}
		}

		// It's an action ref
		_actionRef = true;
	}

	/**
	 * Constructs a new action.
	 *
	 * @param id         Id of the action.
	 * @param template   Action template.
	 * @param parameters Action parameters.
	 */
	public LyAction(LyAction template, Hashtable<String, Object> parameters)
	{
		super(null, template);

		initFromTemplate();

		setApplication(template.getApplication());

		// Add extra parameters
		if (parameters != null)
		{
			Enumeration<String> keys = parameters.keys();
			String			    key;

			while (keys.hasMoreElements())
			{
				key = keys.nextElement();
				addParameter(new LyParameter(key, parameters.get(key)));
			}
		}

		// It's an action ref
		_actionRef = true;
	}
	
    /**
     * Method for freeing dynamic action.
     */
    public void free()
    {
    	if ((_application != null)
	    		&& !_application.isEdited()
	    		&& !_actionRef)
    		return;
    	
        super.free();
    }

	/**
     * Returns extra tags.
     */
    protected LyTag[] getExtraTags()
    {
        return new LyTag[] { _controllerClassName, _behaviorClassName, _builderClassName };
    }

	/**
	 * Final step at the end of the parsing.
	 */
	void check(LyObjectInfo parent)
	{
		super.check(parent);

		// Check for the marks :

		// FFT00066 : If the action has the mark TRANSVERSE and the min > 0,
		// we remove the mark TRANSVERSE.
		// Removed by FFT00437 SIR : actions with min > 0 are supported
		/*if (hasMark(MARK_TRANSVERSE) && (getMin() > 0))
		{
		        _marks.clear(MARK_TRANSVERSE);
		        _application.getEnvironment().logWarning("BF AppEngine : remove mark TRANSVERSE from the action "
		                + this + " which has a min > 0.");
		}*/

		if ((getMin() > 0) && (getMax() >= 0) && (getMax() < getMin()))
			_application.getEnvironment().logWarning(
				"BF AppEngine : max value must be greater than min value, (or specify -1 (unlimited)) on the action "
				+ this + " which has a min > 0.");

		// FFT 1040 : Allow forward references : at this stage the classInfo may not be valid
		LyClassInfo classInfo = getClassInfo();

		if ((classInfo != null)
				&& isActionRef())
		{
			String actionName = ((LyAction)_template).getName();

			if ((actionName != null) && !actionName.equals("LY_ACTION_TABLE"))
				setName(actionName + " : " + classInfo.getName());
			else
				setName(classInfo.getName());
			if (_group == null)
				setGroup(classInfo.getGroup());
			if ((_filePrefix == null) || (_filePrefix.equals(classInfo.getId())))
				setFilePrefix(classInfo.getFilePrefix());
		}

        // If action has parameter _preferredTarget
        String result = (String)this.getParameter("_preferredTarget", true);

        if (result != null)
        {
            _application.getEnvironment().logWarning(
                "BF AppEngine : parameter '_preferredTarget' defined on action " + this
                + " is deprecated. Replace it by the appropriate mark.");

            // we have to remove the parameter and the parameter decl
            LyParameter foundParam = null;

            for (LyParameter param : _parameters)
            {
                if (param.getName().equals("_preferredTarget"))
                {
                    foundParam = param;
                    break;
                }
            }

            if (foundParam != null)
                _parameters.remove(foundParam);

            LyParameterDecl foundParamDecl = null;

            for (LyParameterDecl param : _parametersDecl)
            {
                if (param.getName().equals("_preferredTarget"))
                {
                    foundParamDecl = param;
                    break;
                }
            }

            if (foundParamDecl != null)
                _parametersDecl.remove(foundParamDecl);

            if ("popup_view".equals(result))
            {
                if (_marks == null)
                    _marks = new BitSet();

                _marks.set(MARK_POPUP);
            }
        }
	} // end method check

	/**
	 * Lock this action.
	 */
	protected void lock()
	{
		if (!isLoaded())
			return;

		super.lock();

		// Turn lists to unwritable
		if (_actions != null)
			_actions.lock();
	}

	/**
	 * Gets controller class name.
	 *
	 * @return Java name of controller class (may be null).
	 */
	public String getControllerClassName()
	{
		return (_controllerClassName == null ? null : _controllerClassName.getJavaName());
	}

	/**
	 * Returns controller class.
	 *
	 * @return Java class that implements the controller of this action.
	 */

	public final Class<?> getControllerClass()
	{
	    if (_controllerClassName == null)
	        return null;

	    		// loading classes and displaying appropriate error messages
		Class<?> controllerClass = _controllerClassName.getJavaClass();

		if (controllerClass != null)
		    return controllerClass;

		LyEnvironment     environment = getEnvironment();

		environment.logError("BF AppEngine : unknown controller class " + _controllerClassName
			+ " on action " + _id);

		if (_template != null)
		    return ((LyAction)_template).getControllerClass();

		return null;
	} // end method getControllerClass

	/**
	 * Gets behavior class name.
	 *
	 * @return Java name of behavior class (may be null).
	 */
	public String getBehaviorClassName()
	{
		return (_behaviorClassName == null ? null : _behaviorClassName.getJavaName());
	}

	/**
	 * Return behavior class.
	 *
	 * @return Java class that implements the behavior of this action, null if no specific behavior
	 *         is associated to this action.
	 */

	public final Class<?> getBehaviorClass()
	{
	    if ((_behaviorClassName == null)
	        || (_application.isEdited()
	            && !_behaviorClassName.isGeneric()))
	    {
	        if (_template != null)
                return ((LyAction)_template).getBehaviorClass();
	        else
	            return null;
	    }

		Class<?> behaviorClass = _behaviorClassName.getJavaClass();

	    if (behaviorClass != null)
	        return behaviorClass;

	    LyEnvironment     environment = getEnvironment();

	    environment.logError("BF AppEngine : unknown behavior class " + _behaviorClassName
			+ " on action " + _id);

	    if (_template != null)
	        return ((LyAction)_template).getBehaviorClass();

		return null;
	} // end method getBehaviorClass

	/**
	 * Gets builder class name.
	 *
	 * @return Java name of builder class (may be null).
	 */
	public String getBuilderClassName()
	{
		return (_builderClassName == null ? null : _builderClassName.getJavaName());
	}

	/**
	 * Return builder class.
	 *
	 * @return Java class that implements the builder of this action. null if no specific builder is
	 *         defined.
	 */

	public final Class<?> getBuilderClass()
	{
	    if (_builderClassName == null)
	        return null;

		Class<?> builderClass = _builderClassName.getJavaClass();

		if (builderClass != null)
		    return builderClass;

		LyEnvironment environment = getEnvironment();

		environment.logError("BF AppEngine : unknown builder class " + _builderClassName
			+ " on action " + _id);

		if (_template != null)
		    return ((LyAction)_template).getBuilderClass();
		return null;
	}

	/**
	 * Return XML view file.
	 *
	 * @return Path to XML file that contains the desciption of the view that should be used for
	 *         this action, null if no specific builder is defined.
	 */
	public final String getView()
	{
		return _view;
	}

	/**
	 * Returns the target zone id for this action.
	 *
	 * @return The target zone id.
	 */
	public final String getTarget()
	{
		return _target;
	}

	/**
	 * Accessor to the menu name.
	 *
	 * @return the menu or null if action is not present in a menu.
	 */
	public final String getMenu()
	{
		return _menu;
	}

	/**
	 * Gets the minimum number of objects that must be selected to enable the action or 0 if no
	 * selection is accepted.
	 *
	 * @return Minimum number of objects, 0 means no selection required, 1 means that selections
	 *         have no effect for this action.
	 */
	public final int getMin()
	{
		return _min;
	}

	/**
	 * Gets the maximum number of objects which can be selected to enable the action or -1 if no
	 * maximum is specified.
	 *
	 * @return Maximum number of objects, -1 means no limit.
	 */
	public final int getMax()
	{
		return _max;
	}

	/**
	 * Gets the filter associated with this action, and which may disable the action if selected
	 * objects don't match the filter.
	 *
	 * @return Action filter, or null if no filter is specified.
	 */
	public final LyFilter getFilter()
	{
		return _filter;
	}

	/**
	 * Gets value corresponding to the given parameter name. Shortcut for the following method with
	 * <i>allowUnknowValue</i> set to false.
	 *
	 * @param  name Parameter name.
	 * @return Value of the parameter if found.
	 * @see    LyAction#getParameter
	 */
	public final Object getParameter(String name)
	{
		return getParameter(name, false);
	}

	/**
	 * Gets value corresponding to the given parameter.
	 *
	 * @param     name         Parameter name.
	 * @param     allowUnknown Parameter name.
	 * @return    Untyped value of the parameter if found. Returns null if no parameter with the
	 *            given name is found on this action and <i>allowUnkownValue</i> is set to true
	 *            otherwise exceptions are thrown.
	 * @exception IllegalStateException    If no parameters are registered on this action (never
	 *                                     thrown if <i>allowUnkownValue</i> is on).
	 * @exception IllegalArgumentException If the action doesn't have a parameter with the given
	 *                                     name. (never thrown if <i>allowUnkownValue</i> is on).
	 */
	public final Object getParameter(String name, boolean allowUnkownValue)
	{
		LyParameter parameter = getLyParameter(name, allowUnkownValue);

		if (parameter == null)
		    return null;

		Object    value = parameter.getValue();

        if ((value instanceof String)
            && (((String)value).charAt(0) == '$'))
        {
            LyEnvironment  environment = getEnvironment();
            String         strValue = environment.getResource(((String)value).substring(1));

            if ((strValue == null)
                    || (strValue.length() == 0))
                return value;
            else
                return strValue;
        }

		return value;
	}

	/**
	 * Gets value corresponding to the given parameter mark. The result is a BitSet. A type is given
	 * to known if marks apply to action, classInfo, fieldInfo or filter.
	 *
	 * @param  name the name of the parameter
	 * @param  type the type of expected bitset (for validation) Possible values are :
	 *              LyBitSet.Type.A_MARKS, TYPE_F_MARKS, TYPE_C_MARKS...
	 * @return the marks
	 */
	public BitSet getParameterMarks(String name, LyBitSet.Type type)
	{
		Object object = getParameter(name, false);

		if ((object != null) && (!(object instanceof LyBitSet)))
			throw new IllegalStateException("BF AppEngine : invalid parameter marks "
				+ name + " (parameter is not associated to a mark)");

		LyBitSet marks = (LyBitSet)object;

		if (marks == null)
			return null;

		if (marks.getType() != type)
			throw new IllegalStateException("BF AppEngine : invalid parameter marks type "
				+ type + " (excepted type is " + marks.getType() + ")");

		return marks.getBitSet();
	}

	/**
	 * Gets parameter corresponding to the given parameter name.
	 *
	 * @param     name         Parameter name.
	 * @param     allowUnknown Parameter name.
	 * @return    The parameter if found. Null if no parameter with the given name is found on this
	 *            action and <i>allowUnkownValue</i> is set to true otherwise exceptions are thrown.
	 * @exception IllegalStateException    If no parameters are registered on this action (never
	 *                                     thrown if <i>allowUnkownValue</i> is on).
	 * @exception IllegalArgumentException If the action doesn't have a parameter with the given
	 *                                     name. (never thrown if <i>allowUnkownValue</i> is on).
	 */
	public final LyParameter getLyParameter(String name, boolean allowUnkownValue)
	{
		LyParameter parameter = null;

		if (_parameters != null)
		{
			int		    i, n = _parameters.size();
			LyParameter param;

			name = name.intern();
			for (i = 0; (i < n) && (parameter == null); i++)
			{
				param = _parameters.elementAt(i);
				if (param.getName() == name)
					parameter = param;
			}
		}

		if ((parameter == null) && !allowUnkownValue)
		{
			LyParameterDecl parameterDecl = getParameterDecl(name);

			if (parameterDecl == null)
				throw new IllegalArgumentException("BF AppEngine : unknown parameter "
					+ name + " for action " + this);

			if (parameterDecl.isRequired())
				throw new IllegalArgumentException("BF AppEngine : parameter "
					+ name + " can't be null for action " + this);
		}

		return parameter;
	} // end method getLyParameter

	/**
	 * Gets a parameter declaration from a parameter name.
	 *
	 * @param  Name of the parameter.
	 * @return The parameter declaration if found, null otherwise.
	 */
	public LyParameterDecl getParameterDecl(String name)
	{
		if ((_parametersDecl == null)
				|| (name == null))
			return null;

		int			    i, n = _parametersDecl.size();
		LyParameterDecl decl;

		name = name.intern();
		for (i = 0; i < n; i++)
		{
			decl = _parametersDecl.elementAt(i);
			if (decl.getName() == name)
				return decl;
		}

		return null;
	}

	/**
	 * Gets all parameter declarations defined for one action. This method returns a copy of the
	 * table, this copy can freely be modified.
	 *
	 * @return Vector containing all parameter declarations, null if no parameter was declared for
	 *         this action.
	 */
	public final Vector<LyParameterDecl> getParametersDecl()
	{
		return ((_parametersDecl == null) ? null
			: new Vector<LyParameterDecl>(_parametersDecl));
	}

	/**
	 * Gets all parameters defined for one action. This method returns a copy of the table, this
	 * copy can freely be modified.
	 *
	 * @return Vector containing all parameters, null if no parameter was registered for this
	 *         action.
	 */
	public final Vector<LyParameter> getParameters()
	{
		return ((_parameters == null) ? null
            : new Vector<LyParameter>(_parameters));
	}


	/**
	 * Gets sub-actions list : extra registered actions that can be invoked from action view.
	 *
	 * @return List of these actions.
	 */
	public final LyInfoList<LyAction> getActions()
	{
		return _actions;
	}

	/**
	 * Gets sub-action at the given index in sub-actions list.
	 *
	 * @param  index Index of the searched sub-action.
	 * @return Sub-action at the given index, null if no sub-action is defined.
	 */
	public final LyAction getAction(int index)
	{
		return ((_actions == null) ? null : _actions.getInfo(index));
	}

	/**
	 * Gets number of sub-actions.
	 *
	 * @return Number of sub-actions of this action.
	 */
	public final int getActionCount()
	{
		return ((_actions == null) ? 0 : _actions.getSize());
	}

	/**
	 * Gets the shortcut associated with this action, and which appear in the corresponding menu
	 * item.
	 *
	 * @return Shortcut for the action.
	 */
	public final String getShortcut()
	{
		return _shortcut;
	}

	/**
	 * Gets the default action for this action. The default action is the one executed when the
	 * end-user double-clicks in action view.
	 *
	 * @return Default action for this action.
	 */
	public final LyAction getDefaultAction()
	{
		return _defaultAction;
	}

	/**
	 * Is the current action an action_ref.
	 *
	 * @return true if the action is an action_ref
	 */
	public final boolean isActionRef()
	{
		return _actionRef;
	}

	/**
	 * Sets the default action as an actionreference.
	 *
	 * @param actionRef action reference
	 */
	void setActionRef(boolean actionRef)
	{
		_actionRef = actionRef;
	}

	/**
	 * Gets action environment containing specific action resources.
	 *
	 * @param  parentEnvironnement Parent environment to which the returned environment has to be
	 *                             parented.
	 * @return A new environment parented to the given parent environment and containing the action
	 *         resources, if this action has specific resources. Otherwise, the parent environment
	 *         given as parameter is returned directly.
	 */
	public final LyEnvironment getEnvironment(LyEnvironment parentEnvironment)
	{
		if (_parentEnvironment != null)
			parentEnvironment = _parentEnvironment;
		if (_resources == null)
			return parentEnvironment;

		LyEnvironment    environment = new LyEnvironment(null, parentEnvironment, false);
		int				 i, n = _resources.size();
		LyActionResource resource;

		for (i = 0; i < n; i++)
		{
			resource = _resources.elementAt(i);
			environment.putResource(resource.getType(), resource.getKey(), resource.getValue());
		}

		return environment;
	}

	/**
	 * Method used to check if this action is invokable or not (i.e. all its required parameters
	 * have a value)
	 *
	 * @param  checkClassInfo True if classInfo parameter is checked or ignored.
	 * @return True if this action can be invoked, false otherwise.
	 */
	public boolean canBeInvoked(boolean checkClassInfo)
	{
		if (_parametersDecl == null)
			return true;

		LyParameterDecl parameterDecl;
		Enumeration<LyParameterDecl>     elements = _parametersDecl.elements();
		String		    name;
		int			    l;

		while (elements.hasMoreElements())
		{
			parameterDecl = elements.nextElement();
			if (!parameterDecl.isRequired())
				continue;

			name = parameterDecl.getName();
			if (!checkClassInfo && name.equals(PARAM_CLASS))
				continue;

			l = name.length();
			if (name.charAt(l - 1) == '#')
			{
				name = name.substring(0, l - 1);
				if (getParameter(name, true) == null)
				{
					if (name.equals(PARAM_FIELD))
						name += '1';
					else
						name += '0';
				}
			}
			if (getParameter(name, true) == null)
				return false;
		}

		return true;
	} // end method canBeInvoked

	/**
	 * Method used to check if this action is viewable or not (i.e. the action is executed in a
	 * view).
	 *
	 * @return True if this action is viewable, false otherwise.
	 */

	public boolean isViewable()
	{
		load();

        LyEnvironment       environment = getEnvironment();

        if (environment.isDebugLevel(LyEnvironment.DEBUG_ALL))
        {
            Class<?> controllerClass = getControllerClass();

            if ((controllerClass == null)
                    || LyViewableInterface.class.isAssignableFrom(controllerClass))
                return false;
            else
                return true;
        }
        else
            return (_controllerClassName != null);
	}

	/**
	 * Get All resources defined for this action.
	 *
	 * @return a vector with all resources
	 */
	public Vector<LyActionResource> getResources()
	{
		return (_resources == null ? null : new Vector<LyActionResource>(_resources));
	}

	/**
	 * Gets geometry information for action view (used with fat client only).
	 */
	public LyGeometry getGeometry()
	{
		return _geometry;
	}

	/**
	 * Checks if this action is abstract or not.
	 *
	 * @return True if abstract, false otherwise.
	 */
	public final boolean isAbstract()
	{
		return _abstract;
	}

	/**
	 * Gets the 'root' template for this action. This action if no template for this action,
	 * otherwise calls getRoot on the template until the farthest template is found.
	 *
	 * @return Root template if any, this otherwise.
	 */
	public final LyAction getRoot()
	{
		LyAction template = (LyAction)getTemplate();

		return ((template == null) ? this : template.getRoot());
	}

	/**
	 * Returns class info on which the action is applied if specified.
	 *
	 * @return The class info on which the action is applied (may be null).
	 */
	public LyClassInfo getClassInfo()
	{
		return (LyClassInfo)getParameter(PARAM_CLASS, true);
	}

	/**
	 * Gets list of fields concerned by the action if specified.
	 *
	 * @param  classInfo Class info on which fields are searched if not given on action.
	 * @param  userFieldMarks Indicates if field marks are taken into account or not.
	 * @return Info list containing fields concerned by the action, null if not specified.
	 */
	public LyInfoList<LyFieldInfo> getFieldInfos(LyClassInfo classInfo, boolean useFieldMarks)
	{
		LyClassInfo actionClassInfo = getClassInfo();

		if ((actionClassInfo != null)
				&& (classInfo != null)
				&& (actionClassInfo != classInfo))
			return null;

		if (classInfo == null)
			classInfo = actionClassInfo;

		LyInfoList<LyFieldInfo> fields = null;
		LyFieldInfo			    fieldInfo = (LyFieldInfo)getParameter("_fieldInfo", true);
		int					    np = 2;

		if (fieldInfo == null)
		    fieldInfo = (LyFieldInfo)getParameter("_fieldInfo1", true);

		while (fieldInfo != null)
		{
			if (fields == null)
				fields = new LyInfoList<LyFieldInfo>();
			if ((classInfo == null)
					|| classInfo.contains(fieldInfo))
				fields.addInfo(fieldInfo);

			fieldInfo = (LyFieldInfo)getParameter("_fieldInfo" + np, true);
			np++;
		}

		if ((fields == null)
		    && useFieldMarks)
		{
		    BitSet    fieldMarks = getParameterMarks("_fieldMarks", LyBitSet.Type.F_MARKS);

		    if ((classInfo != null)
                    && (fieldMarks != null))
                fields = classInfo.getFieldInfos(fieldMarks);

		    if ((fieldMarks != null)
                && (fieldMarks.get(LyFieldInfo.MARK_CREATE)
                    || fieldMarks.get(LyFieldInfo.MARK_SET)))
		    {
    			LyInfoList<LyAction> actions = new LyInfoList<LyAction>();
    			LyAction             action = this;
    			int                  i, mark;
    			String               extraMarkName, markName
    			    = (fieldMarks.get(LyFieldInfo.MARK_CREATE) ? "create" : "set");

    			do
    			{
    			    actions.addInfo(action);
    			    action = (LyAction)action.getTemplate();
    			}
    			while (action != null);

    			for (i = actions.getSize() - 1; i >= 0; i--)
    			{
    			    action = actions.getInfo(i);
    			    extraMarkName = "__" + action.getId() + '.' + markName;

    			    mark = LyFieldInfo.getMark(extraMarkName);
    			    if (mark >= 0)
    			        if (fields == null)
    			            fields = classInfo.getFieldInfos(mark);
    			        else
    			            fields.addList(classInfo.getFieldInfos(mark));

    			    mark = LyFieldInfo.getMark(extraMarkName + "=false");
        			if ((fields != null)
        			        && (mark >= 0))
        			    fields = fields.getInfos(mark, true);
    			}
		    }
		}

		return fields;
	} // end method getFieldInfos

	/**
	 * Sets the name of the menu to which this action belongs to.
	 *
	 * @param tab The menu name or null if not present in a menu.
	 */
	final void setMenu(String menu)
	{
		_menu = ((menu == null) ? null : menu.intern());
	}

	/**
	 * Sets the minimum number of objects which must be selected to enable the action.
	 *
	 * @param minSelection Minimum number of objects required. (0 = no selection required, -1 =
	 *                     don't care about selections).
	 */
	final void setMin(int minSelection)
	{
		_min = minSelection;
	}

	/**
	 * Sets the maximum number of objects that can be selected to enable the action.
	 *
	 * @param maxSelection Maximum number of objects accepted. (0 = no selection accepted, -1 = no
	 *                     limit).
	 */
	final void setMax(int maxSelection)
	{
		_max = maxSelection;
	}

	/**
	 * Sets the filter associated with this action.
	 *
	 * @param filter Filter for this action.
	 */
	final void setFilter(LyFilter filter)
	{
		_filter = filter;
	}

	/**
	 * Sets the shortcut associated with this action, and which appear in the corresponding menu
	 * item.
	 *
	 * @param shortcut Menu shortcut for this action.
	 */
	final void setShortcut(String shortcut)
	{
		_shortcut = shortcut;
	}

	/**
	 * Adds a parameter declaration for this action. If a parameter is already registered with the
	 * same name, its value is overriden.
	 *
	 * @param parameterDecl the parameter declaration to add.
	 */
	final void addParameterDecl(LyParameterDecl parameterDecl)
	{
		if (_parametersDecl == null)
			_parametersDecl = new Vector<LyParameterDecl>(1);
		else
		{
			int			    i, n = _parametersDecl.size();
			LyParameterDecl decl;
			String		    name = parameterDecl.getName();

			for (i = 0; i < n; i++)
			{
				decl = _parametersDecl.elementAt(i);
				if (decl.getName() == name)
				{
					_parametersDecl.setElementAt(decl, i);
					parameterDecl.setParent(this);
					return;
				}
			}
		}

		_parametersDecl.add(parameterDecl);
		parameterDecl.setParent(this);
	}

	/**
	 * Adds a parameter for this action. If a parameter is already registered for the same name, its
	 * value is overriden.
	 *
	 * @param name  Parameter name.
	 * @param value Parameter value.
	 */
	final void addParameter(LyParameter parameter)
	{
		// Check first if the parameter was declared
		Class<?>		valueClass = null;
		String		    name = parameter.getName();
		LyParameterDecl parameterDecl = getParameterDecl(name);

		if ((name != null) && name.startsWith("A_MARKS."))
		{
			// Special case : if parameter name starts with "A_MARKS.", an action mark must be set or unset.
			// Value must be one of the Boolean class constants (Boolean.TRUE or Boolean.FALSE).
			BitSet	marks = getMarks();
			int		mark = getMark(name.substring("A_MARKS.".length()), false);
			Object	value = parameter.getValue();

			if ((value instanceof Boolean) && (mark >= 0))
			{
				if ((Boolean)value)
					marks.set(mark);
				else
					marks.clear(mark);

				setMarks(marks);
			}
			
			return;
		}
		
		if (parameterDecl == null)
		{
			// Do we have a declared parameter that ends with a # ?
			int i, n = name.length();

			for (i = n; Character.isDigit(name.charAt(i - 1)) || (name.charAt(i - 1) == '*'); i--)
				;

			if (i > 0)
			{
				parameterDecl = getParameterDecl(name.substring(0, i) + '#');
				if (parameterDecl == null)
					parameterDecl = getParameterDecl(name.substring(0, i) + "##");
			}
		}

		if (parameterDecl != null)
			valueClass = parameterDecl.getValueClass();

		if (valueClass == null)
		{
			LyObjectInfo    template = _template;
			boolean		    raiseException = true;
            LyParameterDecl templateParameterDecl;

			while (template != null)
			{
				if ((!LyInclude._allowLoad
						&& (template.getParent() instanceof LyInclude))
					|| (template._initParts == 0))
				{
                    templateParameterDecl =  ((LyAction)template).getParameterDecl(parameter.getName());
					parameterDecl = new LyParameterDecl(parameter.getName(),
                        (templateParameterDecl != null ? templateParameterDecl.getType() : null), true, null);
                    if (templateParameterDecl != null)
                        valueClass = templateParameterDecl.getValueClass();

                    if (_application != null)
                        _application.getEnvironment().logWarning("BF AppEngine : parameter " + name
                                + " may have not been declared on action " + this);
					raiseException = false;
					break;
				}
				else
					template = template.getTemplate();
			}

			if (raiseException)
				throw new IllegalArgumentException("BF AppEngine : parameter " + name
					+ " not declared on action " + this);
		}

		Object value = parameter.getValue();

		// Check now if the value has the right type
		if ((value != null)
                && (valueClass != null)
				&& !valueClass.isAssignableFrom(value.getClass()))
			throw new IllegalArgumentException("BF AppEngine : invalid value for parameter "
				+ name + " on action " + this + " (expected : " + valueClass
				+ ", received : " + value.getClass() + ")");

		if (_parameters == null)
			_parameters = new Vector<LyParameter>(1);
		else
		{
			int		    i, n = _parameters.size();
			LyParameter param;

			name = name.intern();
			for (i = 0; i < n; i++)
			{
				param = _parameters.elementAt(i);
				if (param.getName() == name)
				{
					_parameters.setElementAt(parameter, i);
				    parameter.setParent(this);
					return;
				}
			}
		}

		_parameters.add(parameter);
	    parameter.setParent(this);
	} // end method addParameter

	/**
	 * Add an sub-action for this action at the given position.
	 *
	 * @param action   Sub-action to add to this action.
	 * @param position Position where the sub-action has to be added (-1 means at the end of the list).
	 */
	final void addAction(LyAction action, int position)
	{
		if (_actions == null)
			_actions = new LyInfoList<LyAction>();

		_actions.addInfo(action, position);
	}

	/**
	 * Removes the specified sub-action from the sub-actions list.
	 *
	 * @param action Action to remove.
	 */
	final void removeAction(LyAction action)
	{
		if (_actions != null)
			_actions.removeInfo(action);
	}

	/**
	 * Resets sub-actions list (clear all sub-actions of this action).
	 */
	final void resetActions()
	{
		if (_actions != null)
			_actions.resetInfos();
	}

	/**
	 * Sets the default action for this action.
	 *
	 * @param action Default action for this action.
	 */
	final void setDefaultAction(LyAction action)
	{
		_defaultAction = action;
	}

	/**
	 * Sets the target of this action.
	 *
	 * @param target the name of the zone where the action is displayed
	 */
	final void setTarget(String target)
	{
		_target = target;
	}

	/**
	 * Sets the behavior class name of this action.
	 *
	 * @param behavior the behavior java class (without extention)
	 */
	final void setBehavior(String behavior)
	{
		_behaviorClassName = (behavior == null ? null : new LyJavaName(this, behavior));
	}

	/**
	 * Adds a specific resource for this action.
	 *
	 * @param resource the resource to add
	 */
	final void addResource(LyActionResource resource)
	{
		if (_resources == null)
			_resources = new Vector<LyActionResource>(1);

		_resources.addElement(resource);
	}

	/**
	 * Sets geometry information for action view.
	 */
	final void setGeometry(LyGeometry geometry)
	{
		_geometry = geometry;
	}

	/**
	 * Specifies if this action is abstract or not.
	 *
	 * @param abstractAction True if abstract, false otherwise.
	 */
	final void setAbstract(boolean abstractAction)
	{
		_abstract = abstractAction;
	}

	/**
	 * Check if a specific mark is set on this action from its name.
	 *
	 * @param  markName Name of the mark tested for this action.
	 * @return True if the mark is set, false otherwise.
	 */
	public final boolean hasMark(String markName)
	{
		return hasMark(getMark(markName));
	}

	/**
	 * Sets action parent environment (internal use).
	 *
	 * @param parentEnvironnement Parent environment to which the returned environment has to be
	 *                            parented.
	 */
	public final void setParentEnvironment(LyEnvironment parentEnvironment)
	{
		_parentEnvironment = parentEnvironment;
	}

    /**
     * Sets XML view file.
     *
     * @param view Path to XML file that contains the desciption of the view that should be used for
     *         this action, null if no specific builder is defined.
     */
    protected void setView(String view)
    {
        _view = view;
    }

    /**
     * Sets parameters for this action.
     */
    protected void setParameters(Vector<LyParameter> parameters)
    {
        _parameters = null;

        int     i, n = (parameters == null ? 0 : parameters.size());

        for (i = 0; i < n; i++)
            addParameter(parameters.get(i));
    }

    /**
     * Sets parameter declarations for this action.
     */
    protected void setParametersDecl(Vector<LyParameterDecl> parametersDecl)
    {
        _parametersDecl = null;

        int     i, n = (parametersDecl == null ? 0 : parametersDecl.size());

        for (i = 0; i < n; i++)
            addParameterDecl(parametersDecl.get(i));
    }

    /**
     * Sets buider class name.
     *
     * @param builderClassName Java name of builder class (may be null).
     */
    protected void setBuilderClassName(String builderClassName)
    {
        _builderClassName = (builderClassName == null ? null
            : new LyJavaName(this, builderClassName));
    }

    /**
     * Sets controller class name.
     *
     * @param controllerClassName Java name of controller class (may be null).
     */
    protected void setControllerClassName(String controllerClassName)
    {
        _controllerClassName = (controllerClassName == null ? null
            : new LyJavaName(this, controllerClassName));
    }

    /**
     * Sets behavior class name.
     *
     * @param behaviorClassName Java name of behavior class (may be null).
     */
    protected void setBehaviorClassName(String behaviorClassName)
    {
        _behaviorClassName = (behaviorClassName == null ? null
            : new LyJavaName(this, behaviorClassName));
    }

    /**
     * Sets resources defined for this action.
     */
    protected void setResources(Vector<LyActionResource> resources)
    {
        _resources = (resources == null ? null : new Vector<LyActionResource>(resources));
    }

    /**
     * Sets actions defined for this action.
     */
    protected void setActions(LyInfoList<LyAction> actions)
    {
        int     i, n = (actions == null ? 0 : actions.getSize());

        _actions = null;
        for (i = 0; i < n; i++)
            addAction(actions.getInfo(i), -1);
    }

    /**
     * Accessor to the generic marks attached to this instance.
     */
    public BitSet getGenericMarks()
    {
        return (new LyBitSet(LyBitSet.Type.A_MARKS, _marks)).getGenericMarks();
    }

    /**
     * Sets the generic marks attached to this instance.
     */
    protected void setGenericMarks(BitSet marks)
    {
        _marks = (new LyBitSet(LyBitSet.Type.A_MARKS, _marks)).setGenericMarks(marks);
    }

    /**
     * Accessor to the marks attached to this instance (presented as an array of strings).
     */
    public String[] getSpecificMarks()
    {
        return (new LyBitSet(LyBitSet.Type.A_MARKS, _marks)).getSpecificMarks();
    }

    /**
     * Sets the specific marks attached to this instance (presented as an array of strings).
     */
    protected void setSpecificMarks(String[] marks)
    {
        _marks = (new LyBitSet(LyBitSet.Type.A_MARKS, _marks)).setSpecificMarks(marks);
    }
} // end class LyAction
