/*
 * LyFilterElement.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.info;

import java.io.*;
import leon.data.*;


/**
 * Common interface for LyFilter, LyFilterRelation and LyFilterExpression FilterElements may be
 * added into a filter.
 *
 * @see LyFilterExpression
 * @see LyFilter
 * @see LyFilterRelation
 */
public interface LyFilterElement
{
	String __VERSION = "$Id: LyFilterElement.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Indicates whether the specified object matches the filter criterion.
	 *
	 * @param  object Object for which the filter criterion is checked.
	 * @return True if the objects matches criterion, false otherwise.
	 */
	boolean match(Object object);

	/**
	 * Indicates whether the specified values matches the filter criterion.
	 *
	 * @param  values List of values for which the filter criterion is checked.
	 * @return True if the values matches criterion, false otherwise.
	 */
	boolean match(LyValueSet values);

	/**
	 * Writes XML corresponding to this filter element.
	 *
	 * @param pw PrintWriter on which the XML is written.
	 */
	void writeXml(PrintWriter pw);

	/**
	 * Returns a displayable string of this element.
	 *
	 * @return A displayable string representation of this element.
	 */
	String getName();
}