/**
 * LyInfoTokens.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.info;

import java.text.*;
import java.util.*;
import leon.info.location.*;
import leon.info.misc.*;
import leon.info.save.*;
import leon.misc.*;


/**
 * This class is a utility class used to support static attributes and methods related to data
 * description parser. It gives the list of all known XML tags and their correlated attributes, and
 * also the lists of supported objectInfo tags. It offers utility methods to convert string tokens
 * into known objectInfo types.
 *
 * @see     LyInfoParser
 * @version $Id: LyInfoTokens.java 41407 2012-10-30 16:17:49Z bangiole $
 * @author  Lyria
 */
public final class LyInfoTokens
{
	public static final String __VERSION =
		"$Id: LyInfoTokens.java 41407 2012-10-30 16:17:49Z bangiole $";

	/** List of all XML tags. */
	static final String		   TAG_A_MARKS = "A_MARKS".intern();
	static final String		   TAG_ALIAS = "ALIAS".intern();
	static final String		   TAG_ACTION = "ACTION".intern();
	static final String		   TAG_ACTION_REF = "ACTION_REF".intern();
	static final String		   TAG_ACTIONS = "ACTIONS".intern();
	static final String		   TAG_ARG = "ARG".intern();
	static final String		   TAG_ARRAY = "ARRAY".intern();
	static final String		   TAG_ARRAY_DAEMON = "ARRAY_DAEMON".intern();
	static final String		   TAG_BEAN = "BEAN".intern();
	static final String		   TAG_BIND_MODEL = "BIND_MODEL".intern();
	static final String		   TAG_C_BIND = "C_BIND".intern();
	static final String		   TAG_C_BINDS = "C_BINDS".intern();
	static final String		   TAG_C_BIND_INFO = "C_BIND_INFO".intern();
	static final String		   TAG_C_BIND_LDAP = "C_BIND_LDAP".intern();
	static final String		   TAG_C_MARKS = "C_MARKS".intern();
	static final String		   TAG_CACHE = "CACHE".intern();
	static final String		   TAG_CHART = "CHART".intern();
	static final String		   TAG_CHARTYPE = "CHARTYPE".intern();
	static final String		   TAG_CHARTYPE_REF = "CHARTYPE_REF".intern();
	static final String		   TAG_CHOICE = "CHOICE".intern();
	static final String		   TAG_CLASS = "CLASS".intern();
	static final String		   TAG_CLASS_REF = "CLASS_REF".intern();
	static final String		   TAG_CLASS_FIELD = "CLASS_FIELD".intern();
	static final String		   TAG_CLASSES = "CLASSES".intern();
	static final String        TAG_COMPOSITION_SAVE = "COMPOSITION_SAVE".intern();
	static final String		   TAG_COMPOUND_ACTION = "COMPOUND_ACTION".intern();
	static final String		   TAG_CONSTRAINT = "CONSTRAINT".intern();
	static final String		   TAG_CONTEXT_FIELD = "CONTEXT_FIELD".intern();
	static final String		   TAG_CONTROL = "CONTROL".intern();
	static final String		   TAG_CONTROL_REF = "CONTROL_REF".intern();
	static final String		   TAG_DATA = "DATA".intern();
	static final String		   TAG_DATA_MODEL = "DATA_MODEL".intern();
	static final String		   TAG_DATE = "DATE".intern();
	static final String		   TAG_DECIMALS = "DECIMALS".intern();
	static final String		   TAG_DEFAULT = "DEFAULT".intern();
	static final String		   TAG_ENCODING = "ENCODING".intern();
	static final String		   TAG_EXP = "EXP".intern();
	static final String		   TAG_EXTERN = "EXTERN".intern();
	static final String		   TAG_F_BIND = "F_BIND".intern();
	static final String		   TAG_F_BINDS = "F_BINDS".intern();
	static final String		   TAG_F_BIND_INFO = "F_BIND_INFO".intern();
	static final String		   TAG_F_BIND_LDAP = "F_BIND_LDAP".intern();
	static final String		   TAG_FILE = "FILE".intern();
	static final String        TAG_F_MARK = "F_MARK".intern();
	static final String		   TAG_F_MARKS = "F_MARKS".intern();
	static final String		   TAG_FIELD = "FIELD".intern();
	static final String		   TAG_FIELD_REF = "FIELD_REF".intern();
	static final String        TAG_FIELD_SAVE = "FIELD_SAVE".intern();
	static final String		   TAG_FIELDS = "FIELDS".intern();
	static final String		   TAG_FILTER = "FILTER".intern();
	static final String		   TAG_FILTER_CRITERIA = "FILTER_CRITERIA".intern();
	static final String		   TAG_FILTER_CRITERIA_REF = "FILTER_CRITERIA_REF".intern();
	static final String		   TAG_FILTER_RELATION = "FILTER_RELATION".intern();
	static final String		   TAG_FILTER_REF = "FILTER_REF".intern();
	static final String        TAG_FORM_SAVE = "FORM_SAVE".intern();
	static final String		   TAG_FORMAT = "FORMAT".intern();
	static final String		   TAG_FUNCTION = "FUNCTION".intern();
	static final String		   TAG_FORMULA = "FORMULA".intern();
	static final String		   TAG_GEOMETRY = "GEOMETRY".intern();
	static final String		   TAG_GROUP = "GROUP".intern();
    static final String        TAG_GROUP_REF = "GROUP_REF".intern();
	static final String		   TAG_GUI_MODEL = "GUI_MODEL".intern();
	static final String		   TAG_IMPORT = "IMPORT".intern();
	static final String		   TAG_INCLUDE = "INCLUDE".intern();
	static final String		   TAG_INFO = "INFO".intern();
	static final String		   TAG_JOIN = "JOIN".intern();
	static final String		   TAG_JOIN_DAEMON = "JOIN_DAEMON".intern();
	static final String		   TAG_L_FILE = "L_FILE".intern();
	static final String		   TAG_L_GEN = "L_GEN".intern();
	static final String		   TAG_L_LDAP = "L_LDAP".intern();
	static final String		   TAG_L_SQL = "L_SQL".intern();
	static final String		   TAG_LABEL = "LABEL".intern();
	static final String		   TAG_MARK = "MARK".intern();
	static final String		   TAG_MAX = "MAX".intern();
	static final String		   TAG_MENU = "MENU".intern();
	static final String		   TAG_MIN = "MIN".intern();
	static final String		   TAG_MODEL = "MODEL".intern();
	static final String        TAG_MULTI_COMPOSITION_SAVE = "MULTI_COMPOSITION_SAVE".intern();
	static final String		   TAG_NAME = "NAME".intern();
	static final String		   TAG_NCHARS = "NCHARS".intern();
	static final String		   TAG_NCOLS = "NCOLS".intern();
	static final String		   TAG_NOTIFIER = "NOTIFIER".intern();
	static final String		   TAG_NOTIFIER_REF = "NOTIFIER_REF".intern();
	static final String		   TAG_N_DEFAULT = "N_DEFAULT".intern();
	static final String		   TAG_NROWS = "NROWS".intern();
	static final String		   TAG_NUMBER = "NUMBER".intern();
	static final String		   TAG_OPTION = "OPTION".intern();
	static final String		   TAG_OPTIONS = "OPTIONS".intern();
	static final String		   TAG_PARAMETER = "PARAMETER".intern();
	static final String		   TAG_PARAMETER_DECL = "PARAMETER_DECL".intern();
	static final String		   TAG_PATH = "PATH".intern();
	static final String		   TAG_PATH_REF = "PATH_REF".intern();
	static final String		   TAG_PITCH = "PITCH".intern();
	static final String		   TAG_PRECISION = "PRECISION".intern();
	static final String		   TAG_PREFERENCE = "PREFERENCE".intern();
	static final String		   TAG_PREFIX = "PREFIX".intern();
	static final String		   TAG_PROJECT = "PROJECT".intern();
	static final String		   TAG_REGEX = "REGEX".intern();
	static final String		   TAG_REGEX_REF = "REGEX_REF".intern();
	static final String		   TAG_RELATION = "RELATION".intern();
	static final String		   TAG_RESOURCE = "RESOURCE".intern();
	static final String		   TAG_REVERSE = "REVERSE".intern();
	static final String		   TAG_ROUTE = "ROUTE".intern();
	static final String		   TAG_ROUTE_REF = "ROUTE_REF".intern();
	static final String		   TAG_ROUTES = "ROUTES".intern();
	static final String		   TAG_RULE = "RULE".intern();
	static final String		   TAG_RULE_REF = "RULE_REF".intern();
	static final String		   TAG_SHORTCUT = "SHORTCUT".intern();
	static final String		   TAG_SORT = "SORT".intern();
	static final String		   TAG_SORT_CRITERIA = "SORT_CRITERIA".intern();
	static final String		   TAG_SORT_CRITERIA_REF = "SORT_CRITERIA_REF".intern();
	static final String		   TAG_SORT_REF = "SORT_REF".intern();
	static final String		   TAG_STEP = "STEP".intern();
	static final String		   TAG_STICKER = "STICKER".intern();
	static final String		   TAG_STRUCT = "STRUCT".intern();
	static final String		   TAG_TAB = "TAB".intern();
    static final String        TAG_TAB_REF = "TAB_REF".intern();
	static final String		   TAG_TAB_ACTION = "TAB_ACTION".intern();
	static final String		   TAG_TEXT = "TEXT".intern();
	static final String		   TAG_TIP = "TIP".intern();
	static final String		   TAG_TREEMAP_CONFIGURATION = "TREEMAP_CONFIGURATION".intern();
	static final String		   TAG_UNION = "UNION".intern();
	static final String		   TAG_UNIT = "UNIT".intern();
	static final String		   TAG_UNIT_REF = "UNIT_REF".intern();
	static final String		   TAG_UNITS = "UNITS".intern();
	static final String		   TAG_UNITS_REF = "UNITS_REF".intern();
	static final String		   TAG_YFIELD = "YFIELD".intern();

	/** Attribute names used within XML tags. */
	static final String		   ATT_ABSTRACT = "abstract";
	static final String		   ATT_ACTION = "action";
	static final String		   ATT_AGGREGATION_FIELD = "aggregationField";
	static final String		   ATT_AGGREGATION_TYPE = "aggregationType";
	static final String		   ATT_ALIAS = "alias";
	static final String		   ATT_APP_BEHAVIOR = "applicationBehavior";
	static final String		   ATT_APP_ID = "applicationId";
	static final String		   ATT_ATTACHMENT_HANDLER = "attachmentHandler";
	static final String		   ATT_ATTACHMENT_DIRECTORY = "attachmentDirectory";
	static final String		   ATT_BASE_OBJECT_NAME = "baseObjectName";
	static final String		   ATT_BEHAVIOR = "behavior";
	static final String		   ATT_BIDIRECTIONAL = "bidirectional";
	static final String        ATT_BINARY = "binary";
	static final String		   ATT_BORDER_SIZES = "borderSizes";
	static final String		   ATT_BUILDER = "builder";
	static final String		   ATT_CALCULATED = "calculated";
	static final String		   ATT_CASE_SENSITIVE = "caseSensitive";
	static final String		   ATT_CBIND = "cbind";
	static final String		   ATT_CLASS = "class";
	static final String		   ATT_CLASS_BEHAVIOR = "classBehavior";
	static final String		   ATT_CLASS_NAME = "className";
	static final String		   ATT_CLIENT_CONTEXT = "clientContext";
	static final String		   ATT_CMIN = "cMin";
	static final String		   ATT_CMAX = "cMax";
	static final String		   ATT_COL_SPAN = "colSpan";
	static final String		   ATT_COL_WEIGHT = "colWeight";
	static final String		   ATT_COLOR_FIELD = "colorField";
	static final String		   ATT_COLUMN_SIZES = "columnSizes";
	static final String		   ATT_CONDITION = "condition";
	static final String		   ATT_CONVERT = "convert";
	static final String		   ATT_CONTEXT = "context";
	static final String		   ATT_CONTENT = "content";
	static final String		   ATT_CONTROLLER = "controller";
	static final String		   ATT_DBNAME = "dbName";
	static final String		   ATT_DEFAULT = "default";
	static final String		   ATT_DEFAULT_ACTION = "defaultAction";
	static final String		   ATT_DEFAULT_FOR_MOBILE = "defaultForMobile";
	static final String		   ATT_DIRECTION = "direction";
	static final String		   ATT_DIRECTORY = "directory";
	static final String		   ATT_DISABLE = "disable";
	static final String		   ATT_DOMAIN = "domain";
	static final String		   ATT_DRIVER = "driver";
	static final String		   ATT_EDITABLE = "editable";
	static final String		   ATT_EXTENDS = "extends";
	static final String		   ATT_FACTOR = "factor";
	static final String		   ATT_FASTVIEW = "fastView";
	static final String		   ATT_FDN_FORMAT = "fdnFormat";
	static final String		   ATT_FIELD = "field";
	static final String		   ATT_FIELD_SEP = "fieldSep";
	static final String		   ATT_FILL = "fill";
	static final String		   ATT_FILTER = "filter";
	static final String		   ATT_FILTER_ACTIVE = "filterActive";
	static final String		   ATT_FILTER_SHOW_ALL = "filterShowAll";
	static final String		   ATT_FIXED_COLUMN = "fixedColumn";
	static final String		   ATT_FOR_PORTLET = "forPortlet";
	static final String		   ATT_FORMAT = "format";
    static final String        ATT_GROUP_BY = "groupBy";
	static final String		   ATT_HEIGHT = "height";
	static final String		   ATT_HIDDEN = "hidden";
	static final String		   ATT_HIERARCHY_FIELDS = "hierarchyFields";
	static final String		   ATT_HIERARCHY_LEVELS = "hierarchyLevels";
	static final String		   ATT_H_ALIGN = "hAlign";
	static final String		   ATT_ID = "id";
	static final String		   ATT_IDREF = "idref";
	static final String		   ATT_IDREFS = "idrefs";
	static final String		   ATT_INCREASING = "increasing";
	static final String		   ATT_JAVA_ENCODER = "javaEncoder";
	static final String		   ATT_JAVA_NAME = "javaName";
	static final String		   ATT_JAVA_TYPE = "javaType";
	static final String		   ATT_PEER_TYPE = "peerType";
	static final String		   ATT_JOIN_CLASS = "joinClass";
	static final String		   ATT_LABEL = "label";
	static final String		   ATT_LABEL_STYLES = "labelStyles";
	static final String		   ATT_LAST_FILE_NAME = "lastFileName";
	static final String        ATT_LDAP_CLASS_FILTER = "classFilter";
	static final String		   ATT_LISTEN = "listen";
	static final String		   ATT_LOAD = "load";
	static final String		   ATT_LOCATION = "location";
	static final String		   ATT_LEVEL = "level";
	static final String        ATT_MARK = "mark";
	static final String		   ATT_MAX_COLOR = "maxColor";
	static final String        ATT_MESSAGE = "message";
	static final String		   ATT_METADATA = "metadata";
	static final String		   ATT_MIME_TYPES = "mimeTypes";
	static final String		   ATT_MIN_COLOR = "minColor";
	static final String		   ATT_MIN_WIDTH = "minWidth";
	static final String		   ATT_MIN_HEIGHT = "minHeight";
	static final String		   ATT_MODEL = "template";
	static final String		   ATT_MODULO = "modulo";
	static final String		   ATT_MULTIPLE = "multiple";
	static final String		   ATT_MULTILINGUAL = "multilingual";
	static final String		   ATT_NAME = "name";
	static final String		   ATT_NB_CONNECTION = "nbConnection";
	static final String		   ATT_NB_OBJECTS = "nbObjects";
	static final String		   ATT_NCOLS = "nCols";
	static final String		   ATT_NOCONTEXT = "noContext";
	static final String		   ATT_NOT = "not";
	static final String		   ATT_NOTOOLBAR = "noToolbar";
	static final String        ATT_OBJECT_ID = "objectId";
	static final String		   ATT_OFFSET = "offset";
	static final String        ATT_OLD_VALUE = "oldValue";
	static final String		   ATT_ON_CHECK = "onCheck";
	static final String		   ATT_OPERATION = "operation";
	static final String		   ATT_OPTION = "option";
	static final String		   ATT_ORDERED = "ordered";
	static final String		   ATT_PARENT = "parent";
	static final String		   ATT_PART = "part";
	static final String		   ATT_PASSWORD = "password";
	static final String		   ATT_PERSISTENT = "persistent";
	static final String		   ATT_PITCH = "pitch";
	static final String		   ATT_POSITION = "position";
	static final String		   ATT_PREFIX = "prefix";
	static final String		   ATT_PRINT = "print";
	static final String		   ATT_PRIORITY = "priority";
	static final String		   ATT_PROPERTIES = "properties";
	static final String		   ATT_PROVIDER_VALUE = "providerValue";
	static final String        ATT_RDN = "rdn";
	static final String		   ATT_READ_ONLY = "readOnly".intern();
	static final String		   ATT_REFERENCE = "reference";
	static final String		   ATT_REMAINDER = "remainder";
	static final String		   ATT_REQUIRED = "required";
	static final String		   ATT_RELATION = "relation";
	static final String        ATT_REVERSE = "reverse";
	static final String		   ATT_REVERSE_RELATION = "reverseRelation";
	static final String		   ATT_ROOT_ACTION = "rootAction";
	static final String		   ATT_ROW_SPAN = "rowSpan";
	static final String		   ATT_ROW_WEIGHT = "rowWeight";
	static final String		   ATT_SCHEMA = "schema";
    static final String        ATT_SECURITY_TYPE = "securityType";
	static final String		   ATT_SESSION_BEHAVIOR = "sessionBehavior";
	static final String		   ATT_SHOW_AS = "showAs";
	static final String		   ATT_SIZE_FIELD = "sizeField";
	static final String		   ATT_SIZES = "sizes";
	static final String		   ATT_SORT_BY = "sortBy";
	static final String		   ATT_SOURCE = "source";
	static final String		   ATT_SOURCE_RELATION = "sourceRelation";
    static final String        ATT_STATE = "state";
    static final String        ATT_STRICT = "strict";
	static final String		   ATT_SUB_TYPE = "subType";
	static final String		   ATT_SUB_TYPES = "subTypes";
	static final String		   ATT_STYLE = "style";
	static final String		   ATT_TAB = "tab";
	static final String		   ATT_TABLE = "table";
	static final String		   ATT_TARGET = "target";
	static final String		   ATT_TARGET_RELATION = "targetRelation";
	static final String		   ATT_THRESHOLD_COLORS = "thresholdColors";
	static final String		   ATT_TIP = "tip";
	static final String		   ATT_TRIM = "trim";
	static final String		   ATT_TYPE = "type";
	static final String		   ATT_UNKNOWN = "unknown";
	static final String		   ATT_URL = "url";
	static final String		   ATT_USER = "user";
	static final String		   ATT_USER_DATA = "userData";
	static final String		   ATT_VALUE = "value";
	static final String		   ATT_VALUE2 = "value2";
	static final String		   ATT_VALUES = "values";
	static final String		   ATT_VALUE_SEP = "valueSep";
	static final String		   ATT__VERSION = "version";
	static final String		   ATT_VIEW = "view";
	static final String		   ATT_VIRTUAL = "virtual";
	static final String		   ATT_V_ALIGN = "vAlign";
	static final String		   ATT_WIDTH = "width";
	static final String		   ATT_WITHTIMEZONE = "withTimeZone";
	static final String		   ATT_X = "x";
	static final String		   ATT_XML_FILE = "xmlFile";
	static final String		   ATT_Y = "y";

	// chart attributes
	static final String		   ATT_CLASS_INFO_ID = "classInfoId";
	static final String		   ATT_X_FIELD = "xField";
	static final String		   ATT_GROUP_FIELD = "groupField";
	static final String		   ATT_TYPE_Y = "typeVarY";
	static final String		   ATT_TYPE_X = "typeVarX";
	static final String		   ATT_GRAPH_TYPE = "graphType";
	static final String		   ATT_X_VAL = "xVal";
	static final String		   ATT_Y_VAL = "yVal";
	static final String		   ATT_X_INTS_BEGIN = "xIntsBegin";
	static final String		   ATT_X_INTS_LENGTH = "xIntsLength";
	static final String		   ATT_X_INTS_UNIT = "xIntsUnit";
	static final String		   ATT_Y_INTS_BEGIN = "yIntsBegin";
	static final String		   ATT_Y_INTS_LENGTH = "yIntsLength";
	static final String		   ATT_Y_INTS_UNIT = "yIntsUnit";
	static final String		   ATT_PIE_BY = "pieBy";
	static final String		   ATT_TITLE = "title";
	static final String		   ATT_X_TITLE = "xTitle";
	static final String		   ATT_Y_TITLE = "yTitle";
	static final String		   ATT_X_AXIS_MIN = "xAxisMin";
	static final String		   ATT_X_AXIS_MAX = "xAxisMax";
	static final String		   ATT_X_AXIS_WIDTH = "xAxisWidth";
	static final String		   ATT_Y_AXIS_MIN = "yAxisMin";
	static final String		   ATT_Y_AXIS_MAX = "yAxisMax";
	static final String		   ATT_Y_AXIS_WIDTH = "yAxisWidth";
	static final String		   ATT_X_VERT_LABELS = "xVerticalLabels";
	static final String		   ATT_X_GRID = "xGrid";
	static final String		   ATT_Y_GRID = "yGrid";
	static final String		   ATT_3D_EFFECT = "d3Effect";
	static final String		   ATT_SHOW_LEGEND = "showLegend";
	static final String		   ATT_SHOW_TIPS = "showTips";
	static final String		   ATT_RATIOS = "ratios";

	/**
	 * Operations supported by list tags when copied from a template : ACTIONS, FIELDS, CLASSES,
	 * BRIDGES, OPTIONS. RESET : Reset template list and add given elements, ADD : Add given
	 * elements to template list. CLEAR : Clear given elements from template list.
	 */
	static final String		   OPER_LIST_RESET = "RESET";
	static final String		   OPER_LIST_ADD = "ADD";
	static final String		   OPER_LIST_REMOVE = "REMOVE";

	/**
	 * Operations supported by marks when copied from a template : A_MARKS, C_MARKS, F_MARKS. RESET
	 * : Reset template marks and add given marks, OR : Add given marks to template marks. AND : And
	 * given marks with template marks. XOR : Xor given marks with template marks. CLEAR : Clear
	 * given marks from template marks.
	 */
	static final String		   OPER_BS_RESET = "RESET";
	static final String		   OPER_BS_OR = "OR";
	static final String		   OPER_BS_AND = "AND";
	static final String		   OPER_BS_XOR = "XOR";
	static final String		   OPER_BS_CLEAR = "CLEAR";

	/** List of supported action marks. */
	public static String[]     actionMarks =
	{
		// WARNING : Must remain consistent with static finals in LyAction
		"default".intern(),
		"tool".intern(),
		"form".intern(),
		"transverse".intern(),
		"hidden".intern(),
		"dialog".intern(),
		"noTitlebar".intern(),
		"noToolbar".intern(),
		"noCommands".intern(),
		"noTabs".intern(),
		"noMenu".intern(),
		"status".intern(),
		"noMenubar".intern(),
		"frameView".intern(),
		"modal".intern(),
		"noMessages".intern(),
		"noDecoration".intern(),
		"noContext".intern(),
		"portlet".intern(),
		"webservice".intern(),
		"mobileOnly".intern(),
		"notForMobile".intern(),
		"popup".intern(),
		"mobileWebview".intern()
	};

	/** List of supported classInfo marks. */
	public static String[]     classInfoMarks =
	{
		// WARNING : Must remain consistent with static finals in LyClassInfo
		"local".intern(),
		"load".intern(),
	};

	/** List of supported fieldInfo marks. */
	public static String[]     fieldInfoMarks =
	{
		// WARNING : Must remain consistent with static finals in LyFieldInfo
		"id".intern(),
		"name".intern(),
		"notNull".intern(),
		"create".intern(),
		"set".intern(),
		"optional".intern(),
		"unique".intern(),
		"secret".intern(),
		"private".intern(),
		"table".intern(),
		"status".intern(),
		"sort".intern(),
		"filter".intern(),
		"find".intern(),
		"local".intern(),
		"consult".intern(),
		"createConsult".intern(),
		"setConsult".intern(),
		"needPost".intern(),
		"chart".intern(),
		"readOnly".intern(),
		"disable".intern(),
		"main".intern(),
		"hidden".intern(),
		"load".intern(),
		"providerValue".intern(),
		"appId".intern(),
		"_providerValueBind".intern(),
		"complexTable".intern(),
		"search".intern(),
		"autoCompletion".intern(),
		"translate".intern(),
		"noCaption".intern(),
	};

	/** List of specific actions marks. */
	static Vector<String>	   extraActionMarks;

	/** List of specific classInfo marks. */
	static Vector<String>	   extraClassInfoMarks;

	/** List of specific fieldInfo marks. */
	static Vector<String>	   extraFieldInfoMarks;

    /** Indexes of last extra marks. */
    public static int    LAST_EXTRA_ACTION_MARKS = 0;

    public static int    LAST_EXTRA_CLASS_MARKS = 0;

    public static int    LAST_EXTRA_FIELD_MARKS = 0;


	/**
	 * Utility method to get filter type from a token string.
	 *
	 * @param     type The token.
	 * @return    The corresponding type if it exists.
	 * @exception IllegalArgumentException if the given token doesn't correspond to any known filter
	 *                                     type.
	 */
	static LyFilter.Type getFilterType(String type)
	{
        if (type == null)
            return null;

        LyFilter.Type  result = Enum.valueOf(LyFilter.Type.class, type);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid filter type " + type);
	}

	/**
	 * Utility method to get filter exp condition from a token string.
	 *
	 * @param     type The token.
	 * @return    The corresponding condition if it exists.
	 * @exception IllegalArgumentException if the given token doesn't correspond to any known filter
	 *                                     exp. condition.
	 */
	static LyFilterExpression.Condition getFilterExpCondition(String condition)
	{
        if (condition == null)
            return null;

        LyFilterExpression.Condition  result = Enum.valueOf(LyFilterExpression.Condition.class, condition);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid filter exp. condition " + condition);
	}

	/**
	 * Utility method to get sort criterion of a choice field info from a token string.
	 *
	 * @param     sortBy The token.
	 * @return    The corresponding sort criterion if it exists.
	 * @exception IllegalArgumentException if the given token doesn't correspond to any known choice
	 *                                     sort type.
	 */
	static LyChoiceFieldInfo.SortBy getChoiceSortBy(String sortBy)
	{
        if (sortBy == null)
            return null;

        LyChoiceFieldInfo.SortBy  result = Enum.valueOf(LyChoiceFieldInfo.SortBy.class, sortBy);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid choice sort criterion " + sortBy);
    }

	/**
	 * Utility method to get a number field info type from a token string.
	 *
	 * @param     type The token.
	 * @return    The corresponding type if it exists.
	 * @exception IllegalArgumentException if the given token doesn't correspond to any known number
	 *                                     field info type.
	 */
	static LyChoiceFieldInfo.Type getChoiceFieldInfoType(String type)
	{
        if (type == null)
            return null;

        LyChoiceFieldInfo.Type  result = Enum.valueOf(LyChoiceFieldInfo.Type.class, type);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid choice field type " + type);
	}

	/**
	 * Utility method to get a date field info type from a token string.
	 *
	 * @param     type The token.
	 * @return    The corresponding type if it exists.
	 * @exception IllegalArgumentException if the given token doesn't correspond to any known date
	 *                                     field info type.
	 */
	static LyDateFieldInfo.Type getDateFieldInfoType(String type)
	{
        if (type == null)
            return null;

        LyDateFieldInfo.Type  result = Enum.valueOf(LyDateFieldInfo.Type.class, type);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid date field type " + type);
	}

	/**
	 * Utility method to get a location file type from a token string.
	 *
	 * @param     type The token.
	 * @return    The corresponding type if it exists.
	 * @exception IllegalArgumentException if the given token doesn't correspond to any known
	 *                                     location file type.
	 */
	static LyFileLocation.Type getFileLocationType(String type)
	{
        if (type == null)
            return null;

        LyFileLocation.Type  result = Enum.valueOf(LyFileLocation.Type.class, type);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid location file type " + type);
	}

	/**
	 * Utility method to get a number field info type from a token string.
	 *
	 * @param     type The token.
	 * @return    The corresponding type if it exists.
	 * @exception IllegalArgumentException if the given token doesn't correspond to any known number
	 *                                     field info type.
	 */
	static LyNumberFieldInfo.Type getNumberFieldInfoType(String type)
	{
        if (type == null)
            return null;

        LyNumberFieldInfo.Type  result = Enum.valueOf(LyNumberFieldInfo.Type.class, type);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid number field type " + type);
	}

	/**
	 * Utility method to get a number field info function from a token string.
	 *
	 * @param     type The token.
	 * @return    The corresponding function if it exists.
	 * @exception IllegalArgumentException if the given token doesn't correspond to any known number
	 *                                     field info function.
	 */
	static LyNumberFieldInfo.Function getNumberFieldInfoFunction(String function)
	{
        if (function == null)
            return null;

        LyNumberFieldInfo.Function  result = Enum.valueOf(LyNumberFieldInfo.Function.class, function);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid number field function " + function);
	}

	/**
	 * Utility method to get a relation field info direction from a token string.
	 *
	 * @param     type The token.
	 * @return    The corresponding direction if it exists.
	 * @exception IllegalArgumentException if the given token doesn't correspond to any known
	 *                                     relation field info direction.
	 */
	static LyRelationFieldInfo.Direction getRelationFieldInfoDirection(String direction)
	{
        if (direction == null)
            return null;

        LyRelationFieldInfo.Direction  result = Enum.valueOf(LyRelationFieldInfo.Direction.class, direction);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid relation field function "
			+ direction);
	}

	/**
	 * Utility method to get a tab action type from a token string.
	 *
	 * @param     type The token.
	 * @return    The corresponding tab action type if it exists.
	 * @exception IllegalArgumentException if the given token doesn't correspond to any known tab
	 *                                     action type.
	 */
	static LyTabAction.Type getTabActionType(String type)
	{
        if (type == null)
            return null;

        LyTabAction.Type  result = Enum.valueOf(LyTabAction.Type.class, type);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid tab action type " + type);
	}

	/**
	 * Utility method to get a treemap configuration aggregation type from a token string.
	 *
	 * @param     aggregationType The token.
	 * @return    The corresponding aggregation type if it exists.
	 * @exception IllegalArgumentException if the given token doesn't correspond to any known
	 *                                     treemap configuration aggregation type.
	 */
	static LyTreemapConfiguration.Aggregation getTreemapConfigurationAggregationType(String aggregationType)
	{
        if (aggregationType == null)
            return null;

        LyTreemapConfiguration.Aggregation  result
            = Enum.valueOf(LyTreemapConfiguration.Aggregation.class, aggregationType);

       if (result != null)
           return result;

		throw new IllegalArgumentException(
			"BF AppEngine : Invalid treemap configuration aggregation type " + aggregationType);
	}

	/**
	 * Utility method to get the list of threshold colors of a treemap configuration from a token
	 * string.
	 *
	 * @param  aggregationType The token.
	 * @return The corresponding list of threshold colors.
	 */
	static String[] getTreemapConfigurationThresholdColors(String thresholdColors)
	{
        if (thresholdColors == null)
            return null;

		String[] colors = LyString.stringToList(thresholdColors, ' ');

		return colors;
	}

	/**
	 * Utility method to get a relation field info type from a token string.
	 *
	 * @param     type The token.
	 * @return    The corresponding type if it exists.
	 * @exception IllegalArgumentException if the given token doesn't correspond to any known
	 *                                     relation field info type.
	 */
	static LyRelationFieldInfo.Type getRelationFieldInfoType(String type)
	{
        if (type == null)
            return null;

        LyRelationFieldInfo.Type  result = Enum.valueOf(LyRelationFieldInfo.Type.class, type);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid relation field type " + type);
	}

	/**
	 * Utility method to get a relation field info presentation from a token string.
	 *
	 * @param     type The token.
	 * @return    The corresponding presentation if it exists.
	 * @exception IllegalArgumentException if the given token doesn't correspond to any known
	 *                                     relation field info presentation.
	 */
	static LyRelationFieldInfo.ShowAs getRelationFieldInfoShowAs(String showAs)
	{
        if (showAs == null)
            return null;

        LyRelationFieldInfo.ShowAs  result = Enum.valueOf(LyRelationFieldInfo.ShowAs.class, showAs);

        if (result != null)
            return result;

        throw new IllegalArgumentException("BF AppEngine : Invalid relation show_as value " + showAs);
	}

	/**
     * Utility method to get a text field info type from a token string.
     *
     * @param     type The token.
     * @return    The corresponding type if it exists.
     * @exception IllegalArgumentException if the given token doesn't correspond to any known
     *                                     text field info type.
     */
    static LyTextFieldInfo.Type getTextFieldInfoType(String type)
    {
        if (type == null)
            return null;

        LyTextFieldInfo.Type  result = Enum.valueOf(LyTextFieldInfo.Type.class, type);

        if (result != null)
            return result;

        throw new IllegalArgumentException("BF AppEngine : Invalid text field type " + type);
    }

	/**
	 * Utility method to get class cache policy.
	 *
	 * @param     type The token.
	 * @return    The corresponding class cache policy if it exists.
	 * @exception IllegalArgumentException if the given token doesn't correspond to any known class
	 *                                     cache policy.
	 */
	public static LyInfoCache.CachePolicy getClassInfoCachePolicy(String type)
	{
        if (type == null)
            return null;

        LyInfoCache.CachePolicy  result = Enum.valueOf(LyInfoCache.CachePolicy.class, type);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid class cache policy " + type);
	}

	/**
	 * Utility method to get compound action style.
	 *
	 * @param     style The token.
	 * @return    The corresponding style if it exists.
	 * @exception IllegalArgumentException if the given token doesn't correspond to any known style
	 *                                     for compound action.
	 */
	static LyCompoundAction.Style getCompoundActionStyle(String style)
	{
        if (style == null)
            return null;

        LyCompoundAction.Style  result = Enum.valueOf(LyCompoundAction.Style.class, style);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid compound action style " + style);
	}

	/**
	 * Utility method to get compound action context.
	 *
	 * @param     style The token.
	 * @return    The corresponding context if it exists.
	 * @exception IllegalArgumentException if the given token doesn't correspond to any known
	 *                                     context for compound action.
	 */
	static LyCompoundAction.Context getCompoundActionContext(String context)
	{
        if (context == null)
            return null;

        LyCompoundAction.Context  result = Enum.valueOf(LyCompoundAction.Context.class, context);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid compound action context " + context);
	}

	/**
	 * Utility method to get compound action client context.
	 *
	 * @param     style The token.
	 * @return    The corresponding client context if it exists.
	 * @exception IllegalArgumentException if the given token doesn't correspond to any known client
	 *                                     context for compound action.
	 */
	static LyCompoundAction.ClientContext getCompoundActionClientContext(String context)
	{
        if (context == null)
            return null;

        LyCompoundAction.ClientContext  result = Enum.valueOf(LyCompoundAction.ClientContext.class, context);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid compound action client context "
			+ context);
	}

	/**
	 * Utility method to get option rule operation from a token string.
	 *
	 * @param     operation The token.
	 * @return    The corresponding operation if it exists.
	 * @exception IllegalArgumentException If the given token doesn't correspond to any known rule
	 *                                     operation.
	 */
	static LyRule.Operation getRuleOperation(String operation)
	{
        if (operation == null)
            return null;

        LyRule.Operation  result = Enum.valueOf(LyRule.Operation.class, operation);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid rule operation " + operation);
	}

	/**
	 * Utility method to get resource type from a token string.
	 *
	 * @param     type The token.
	 * @return    The corresponding resource type if it exists.
	 * @exception IllegalArgumentException If the given token doesn't correspond to any known
	 *                                     resource type.
	 */
	static LyEnvironment.ResourceType getResourceType(String type)
	{
        if (type == null)
            return null;

        LyEnvironment.ResourceType  result = Enum.valueOf(LyEnvironment.ResourceType.class, type);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid resource type " + type);
	}

	/**
	 * Utility method to get graph type from a token string.
	 *
	 * @param     type The token.
	 * @return    The corresponding chart type if it exists.
	 * @exception IllegalArgumentException If the given token doesn't correspond to any known chart
	 *                                     type.
	 */
	public static LyChartInfo.GraphType getChartInfoGraphType(String type)
	{
        if (type == null)
            return null;

        LyChartInfo.GraphType  result = Enum.valueOf(LyChartInfo.GraphType.class, type);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid chart type " + type);
	}

	/**
	 * Utility method to get chart value type from a token string.
	 *
	 * @param     type The token.
	 * @return    The corresponding values type for a chart if it exists (like REAL, COUNT_INTERVAL,
	 *            etc.).
	 * @exception IllegalArgumentException If the given token doesn't correspond to any known values
	 *                                     grouping for chart.
	 */
	public static LyChartInfo.ValType getChartInfoValType(String type)
	{
        if (type == null)
            return null;

        LyChartInfo.ValType  result = Enum.valueOf(LyChartInfo.ValType.class, type);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid grouping values type for the chart "
			+ type);
	}

	/**
	 * Utility method to get chart type from a token string.
	 *
	 * @param     type The token.
	 * @return    The corresponding variable type for a chart if it exists (number, text or date).
	 * @exception IllegalArgumentException If the given token doesn't correspond to any known
	 *                                     variable type for a chart.
	 */
	static LyChartInfo.Type getChartInfoTypeVar(String type)
	{
        if (type == null)
            return null;

        LyChartInfo.Type  result = Enum.valueOf(LyChartInfo.Type.class, type);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid type of variable for the chart "
			+ type);
	}

	/**
	 * Utility method to get the type of parameter decl from a token string.
	 *
	 * @param     type The token.
	 * @return    The corresponding type if it exists.
	 * @exception IllegalArgumentException If the given token doesn't correspond to any known type.
	 */
	static LyParameterDecl.Type getParameterDeclType(String type)
	{
        if (type == null)
            return null;

        LyParameterDecl.Type  result = Enum.valueOf(LyParameterDecl.Type.class, type);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid parameter decl type " + type);
	} // end method getParameterValueClass

    /**
     * Utility method to get field constraint type.
     *
     * @param     type The token.
     * @return    The corresponding constraint value if it exists.
     * @exception IllegalArgumentException If the given token doesn't correspond to any known
     *                                     horizontal alignment.
     */
    static LyFieldConstraint.Type getConstraintType(String type)
    {
        if (type == null)
            return null;

        LyFieldConstraint.Type  result = Enum.valueOf(LyFieldConstraint.Type.class, type);

        if (result != null)
            return result;

        throw new IllegalArgumentException("BF AppEngine : Invalid constraint type " + type);
    }

	/**
	 * Utility method to get horizontal alignment of a field.
	 *
	 * @param     hAlign The token.
	 * @return    The corresponding alignment value if it exists.
	 * @exception IllegalArgumentException If the given token doesn't correspond to any known
	 *                                     horizontal alignment.
	 */
	static LyFieldConstraint.HAlign getConstraintHAlign(String hAlign)
	{
        if (hAlign == null)
            return null;

        LyFieldConstraint.HAlign  result = Enum.valueOf(LyFieldConstraint.HAlign.class, hAlign);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid constraint horizontal alignement "
			+ hAlign);
	}

	/**
	 * Utility method to get vertical alignment of a field.
	 *
	 * @param     vAlign The token.
	 * @return    The corresponding alignment value if it exists.
	 * @exception IllegalArgumentException If the given token doesn't correspond to any known
	 *                                     vertical alignment.
	 */
	static LyFieldConstraint.VAlign getConstraintVAlign(String vAlign)
	{
        if (vAlign == null)
            return null;

        LyFieldConstraint.VAlign  result = Enum.valueOf(LyFieldConstraint.VAlign.class, vAlign);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid constraint vertical alignement "
			+ vAlign);
	}

	/**
	 * Utility method to get filling of a field.
	 *
	 * @param     fill The token.
	 * @return    The corresponding filling value if it exists.
	 * @exception IllegalArgumentException If the given token doesn't correspond to any known
	 *                                     filling.
	 */
	static LyFieldConstraint.Fill getConstraintFill(String fill)
	{
        if (fill == null)
            return null;

        LyFieldConstraint.Fill  result = Enum.valueOf(LyFieldConstraint.Fill.class, fill);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid constraint filling " + fill);
	}

	/**
	 * Finds a bit corresponding to a project specific mark to set within a bitset. For each type of
	 * specific mark, the first time this method is called with a specific mark, a position is given
	 * for it and is returned for further calls.
	 *
	 * @param     type     Type mark (actions : A_MARKS, classes : C_MARKS, fields : F_MARKS).
	 * @param     markName Name of the project specific mark.
	 * @param     add      Indicates if the mark must be added if not found.
	 * @return    The position corresponding to the specific mark.
	 * @exception IllegalArgumentException If the type doesn't correspond to any supported marks
	 *                                     type.
	 */
	public static int getMarkIndex(LyBitSet.Type type, String markName, boolean add)
	{
		// Check marks type, then identify standard marks list and project specific marks list
		String[]	   standardMarks;
		Vector<String> extraMarks;

		if (type == LyBitSet.Type.A_MARKS) // Action marks
		{
			standardMarks = actionMarks;
			if (extraActionMarks == null)
				extraActionMarks = new Vector<String>();
			extraMarks = extraActionMarks;
		}
		else if (type == LyBitSet.Type.C_MARKS) // Class marks
		{
			standardMarks = classInfoMarks;
			if (extraClassInfoMarks == null)
				extraClassInfoMarks = new Vector<String>();
			extraMarks = extraClassInfoMarks;
		}
		else if (type == LyBitSet.Type.F_MARKS) // Field marks
		{
			standardMarks = fieldInfoMarks;
			if (extraFieldInfoMarks == null)
				extraFieldInfoMarks = new Vector<String>();
			extraMarks = extraFieldInfoMarks;
		}
		else // Error unknown marks type
			throw new IllegalArgumentException("BF AppEngine : invalid mark type " + type);

		// Look inside generic marks
		int i = 0, index = -1, n = standardMarks.length;

		markName = markName.intern();
		while ((index < 0) && (i < n))
		{
			if (standardMarks[i] == markName)
				index = i;
			i++;
		}

		// Look inside specific marks
		if (index < 0)
		{
			// Look if this mark is already registered
			index = extraMarks.indexOf(markName);

			// If it's not, register it in corresponding extra marks list.
			if (index < 0)
			    if (add)
    			{
    				extraMarks.addElement(markName);
    				index = extraMarks.size() - 1;
    			}
			    else
			        return -1;

			// Return the position of this mark shifted of all predefined marks.
			index += n;
		}

		return index;
	} // end method getMarkIndex

	/**
	 * Finds the mark name for a defined mark (standard or specific).
	 *
	 * @param     type      Type mark (actions : A_MARKS, classes : C_MARKS, fields : F_MARKS).
	 * @param     markIndex Bit corresponding to a specific mark of the project.
	 * @return    The position corresponding to the specific mark.
	 * @exception IllegalArgumentException If the type doesn't correspond to any supported marks
	 *                                     type.
	 */
	public static String getMarkName(LyBitSet.Type type, int markIndex)
	{
		if (markIndex < 0)
			return null;
		// Check marks type, then identify standard marks list and project specific marks list
		String[]         standardMarks;
		Vector<String>   extraMarks;

		if (type == LyBitSet.Type.A_MARKS) // Action marks
		{
			standardMarks = actionMarks;
			extraMarks = extraActionMarks;
		}
		else if (type == LyBitSet.Type.C_MARKS) // Class marks
		{
			standardMarks = classInfoMarks;
			extraMarks = extraClassInfoMarks;
		}
		else if (type == LyBitSet.Type.F_MARKS) // Field marks
		{
			standardMarks = fieldInfoMarks;
			extraMarks = extraFieldInfoMarks;
		}
		else // Error unknown marks type
			throw new IllegalArgumentException("BF AppEngine : invalid mark type " + type);

		String markName = null;
		// search the mark inside the standard marks
		if (markIndex < standardMarks.length)
			return standardMarks[markIndex];
		// search the mark inside the extra marks
		int index = markIndex - standardMarks.length;

		if ((extraMarks != null) && (index < extraMarks.size()))
			return extraMarks.elementAt(index);

		return markName;
	} // end method getMarkName

	/**
	 * Gets the list of supported Action marks, these are the marks that defines a generic behavior
	 * for the actions.
	 *
	 * @return Table of supported action marks names.
	 */
	public static String[] getActionMarks()
	{
		return actionMarks.clone();
	}

    /**
     * Gets the list of extra Action marks.
     *
     * @return Table of extra action marks names.
     */
    public static String[] getExtraActionMarks()
    {
        return (extraActionMarks == null ? null : extraActionMarks.toArray(new String[extraActionMarks.size()]));
    }

	/**
	 * Gets the list of supported classInfo marks, these are the marks that defines a generic
	 * behavior for the class infos.
	 *
	 * @return Table of supported class info marks names.
	 */
	public static String[] getClassInfoMarks()
	{
		return classInfoMarks.clone();
	}

    /**
     * Gets the list of extra classInfo marks.
     *
     * @return Table of extra classInfo marks names.
     */
    public static String[] getExtraClassInfoMarks()
    {
        return (extraClassInfoMarks == null ? null : extraClassInfoMarks.toArray(new String[extraClassInfoMarks.size()]));
    }

	/**
	 * Gets the list of supported fieldInfo marks, these are the marks that defines a generic
	 * behavior for the field infos.
	 *
	 * @return Table of supported field info marks names.
	 */
	public static String[] getFieldInfoMarks()
	{
		return fieldInfoMarks.clone();
	}

    /**
     * Gets the list of extra fieldInfo marks.
     *
     * @return Table of extra fieldInfo marks names.
     */
    public static String[] getExtraFieldInfoMarks()
    {
        return (extraFieldInfoMarks == null ? null : extraFieldInfoMarks.toArray(new String[extraFieldInfoMarks.size()]));
    }

	/**
	 * Utility method to get action id corresponding to a bean use case.
	 *
	 * @param     action The token.
	 * @return    The corresponding action id if it exists.
	 * @exception IllegalArgumentException If the given token doesn't correspond to any known id
	 *                                     type.
	 */
	static LyFieldBean.Type getFieldBeanActionType(String action)
	{
        if (action == null)
            return null;

        LyFieldBean.Type  result = Enum.valueOf(LyFieldBean.Type.class, action);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid bean action " + action);
	}

	/**
	 * Utility method to get level of a control.
	 *
	 * @param     level The token.
	 * @return    The corresponding level id if it exists.
	 * @exception IllegalArgumentException If the given token doesn't correspond to any known id
	 *                                     level.
	 */
	static LyControl.Level getControlLevel(String level)
	{
        if (level == null)
            return null;

        LyControl.Level  result = Enum.valueOf(LyControl.Level.class, level);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid control level " + level);
	}

	/**
	 * Utility method to get type of a label.
	 *
	 * @param     type The token.
	 * @return    The corresponding type if it exists.
	 * @exception IllegalArgumentException If the given token doesn't correspond to any known id
	 *                                     type.
	 */
	static LyInfoLabel.Type getInfoLabelType(String type)
	{
        if (type == null)
            return null;

        LyInfoLabel.Type  result = Enum.valueOf(LyInfoLabel.Type.class, type);

        if (result != null)
            return result;

		throw new IllegalArgumentException("BF AppEngine : Invalid label type " + type);
	}

    /**
     * Utility method to get the state of a group.
     *
     * @param     state The token.
     * @return    The corresponding state if it exists.
     * @exception IllegalArgumentException If the given token doesn't correspond to any known id
     *                                     state.
     */
    static LyFieldGroup.State getFieldGroupState(String state)
    {
        if (state == null)
            return null;

        LyFieldGroup.State  result = Enum.valueOf(LyFieldGroup.State.class, state);

        if (result != null)
            return result;

        throw new IllegalArgumentException("BF AppEngine : Invalid field group state " + state);
    }

    /**
     * Utility method to get the type of a form save.
     *
     * @param     type The token.
     * @return    The corresponding type if it exists.
     * @exception IllegalArgumentException If the given token doesn't correspond to any known id
     *                                     type.
     */
    static LyFormSave.Type getFormSaveType(String type)
    {
        if (type == null)
            return null;

        LyFormSave.Type  result = Enum.valueOf(LyFormSave.Type.class, type);

        if (result != null)
            return result;

        throw new IllegalArgumentException("Leonardi : Invalid form save type " + type);
    }

	/**
	 * Utility method to get a preference presentation from a token string.
	 *
	 * @param     type The token.
	 * @return    The corresponding presentation if it exists.
	 * @exception IllegalArgumentException if the given token doesn't correspond to any known
	 *                                     preference presentation.
	 */
	static LyPreference.ShowAs getPreferenceShowAs(String showAs)
	{
        if (showAs == null)
            return null;

        LyPreference.ShowAs  result = Enum.valueOf(LyPreference.ShowAs.class, showAs);

        if (result != null)
            return result;

        throw new IllegalArgumentException("BF AppEngine : Invalid relation show_as value " + showAs);
	}

    /**
     * Builds a number of appropriate type from a number.
     *
     * @param     type  Type of the number to build (see LyNumberFieldInfos for valid types).
     * @param     number The number.
     * @return    returns a number of the wished class.
     */
    static Number parseNumber(LyNumberFieldInfo.Type type, Number number)
    {
        if (number == null)
            return null;

        if (type == LyNumberFieldInfo.Type.INT)
            return new Integer(number.intValue());
        else if (type == LyNumberFieldInfo.Type.SHORT)
            return new Short(number.shortValue());
        else if (type == LyNumberFieldInfo.Type.LONG)
            return new Long(number.longValue());
        else if (type == LyNumberFieldInfo.Type.DOUBLE)
            return new Double(number.doubleValue());
        else if (type == LyNumberFieldInfo.Type.FLOAT)
            return new Float(number.floatValue());
        else
            return number;
    }

	/**
	 * Builds a number of appropriate type from a string token.
	 *
	 * @param     type  Type of the number to build (see LyNumberFieldInfos for valid types).
	 * @param     token The token.
	 * @return    returns a number of the wished class.
	 * @exception IllegalArgumentException If the given token cannot be parsed as a number of the
	 *                                     given type.
	 * @exception IllegalArgumentException If the type doesn't correspond to any supported tags
	 *                                     type.
	 */
	static Number parseNumber(LyNumberFieldInfo.Type type, String token)
	{
		try
		{
			if (type == LyNumberFieldInfo.Type.INT)
				return new Integer(token);
			else if (type == LyNumberFieldInfo.Type.SHORT)
				return new Short(token);
			else if (type == LyNumberFieldInfo.Type.LONG)
				return new Long(token);
			else if (type == LyNumberFieldInfo.Type.DOUBLE)
				return new Double(token);
			else if (type == LyNumberFieldInfo.Type.FLOAT)
				return new Float(token);
		}
		catch (NumberFormatException e)
		{
			throw new IllegalArgumentException("BF AppEngine :  invalid number format " + token);
		}

		throw new IllegalArgumentException("BF AppEngine : invalid number type " + type);
	}

	/**
	 * Builds an int from a string token.
	 *
	 * @param     token The token.
	 * @return    returns an int corresponding to the given token.
	 * @exception IllegalArgumentException If the given token cannot be parsed as an int.
	 * @see       LyTokens#buildNumber
	 */
	static int parseInt(String token)
	{
		if ((token == null) || (token.length() == 0))
			return -1;

		try
		{
			return Integer.parseInt(token);
		}
		catch (NumberFormatException e)
		{
			throw new IllegalArgumentException("BF AppEngine :  invalid int format " + token);
		}
	}

	/**
	 * Builds a date from a string token.
	 *
	 * @param     token The token.
	 * @return    returns date
	 * @exception IllegalArgumentException If the given token cannot be parsed as a date.
	 */
	static Date parseDate(String token)
	{
		if ((token == null) || (token.length() == 0))
			return null;

		try
		{
			SimpleDateFormat format = new SimpleDateFormat();

			return format.parse(token);
		}
		catch (ParseException e)
		{
			throw new IllegalArgumentException("BF AppEngine : invalid date format " + token);
		}
	}

    /**
     * Registers extra marks indexes.
     */
    public static void registerExtraMarksIndexes()
    {
    	if (LAST_EXTRA_ACTION_MARKS == 0)
    		LAST_EXTRA_ACTION_MARKS = (extraActionMarks == null ? -1 : extraActionMarks.size());
    	if (LAST_EXTRA_CLASS_MARKS == 0)
    		LAST_EXTRA_CLASS_MARKS = (extraClassInfoMarks == null ? -1 : extraClassInfoMarks.size());
    	if (LAST_EXTRA_FIELD_MARKS == 0)
    		LAST_EXTRA_FIELD_MARKS = (extraFieldInfoMarks == null ? -1 : extraFieldInfoMarks.size());
    }

    /**
     * Clears extra marks indexes.
     */
    public static void clearExtraMarksIndexes()
    {
    	LAST_EXTRA_ACTION_MARKS = 0;
		LAST_EXTRA_CLASS_MARKS = 0;
		LAST_EXTRA_FIELD_MARKS = 0;
    }
} // end class LyInfoTokens
