package leon.info;

import java.util.*;

import leon.info.binding.*;
import leon.info.misc.*;

public abstract class LyMetaModelCopyPasteExtension extends LyMetaModel
{
	// Common
	private void completeInfoFromSource(LyInfo source, LyInfo target, LyInfo parent)
	{
		// assign parent
		target.setParent(parent);
	}

	private void completeObjectInFoFromSource(LyObjectInfo source, LyObjectInfo target, LyInfo parent)
	{
		completeInfoFromSource(source, target, parent);

		target.setName(source.getName());
		target.setAlias(source.getAlias());
		target.setGroup(source.getGroup());
		target.setFilePrefix(source.getFilePrefix());
		target.setTip(source.getTip());
		target.setUserData(source.getUserData());

		if (source.getMarks() != null)
			target.setMarks((BitSet)source.getMarks().clone());

		LyData sourceData, copy;
		Vector<LyData> copiedDatas = null;
		Vector<LyData> datas = source.getDataList();
		int i, n = (datas == null ? 0 : datas.size());

		for (i = 0; i < n; i++)
		{
			sourceData = datas.elementAt(i);
			copy = new LyData(sourceData.getName(), sourceData.getValue());
			if (copiedDatas == null)
				copiedDatas = new Vector<LyData>(n);

			copiedDatas.add(copy);
		}

		if (copiedDatas != null)
			target.setDataList(copiedDatas);
	}


	// ClassInfo
	/**
	 * Clone the selected class and add it to the given parent.
	 *
	 * @param source the classinfo to clone
	 * @param parent the parent element of the new classinfo
	 */
	protected LyClassInfo copyPaste(LyClassInfo source, LyInfo parent)
	{
		LyClassInfo result = newClassInfo((String)null, source.getSuperClass(), (LyClassInfo)source.getTemplate(), source.getJavaName(),
				source.getBehaviorClassName(), source.isAbstract(), source.isVirtual(), null);

		completeObjectInFoFromSource(source, result, parent);

		setBinding(result, duplicateClassBinding(result.getBinding()));

		// duplicate fields
		LyInfoList<LyFieldInfo> fields = source.getFieldInfos();
		LyFieldInfo field, fieldCopy;
		int i, n = (fields == null ? 0: fields.getSize());

		for (i = 0; i < n; i++)
		{
			field = fields.getInfo(i);

			if (field.getParent() != source)
			{
				result.addFieldInfo(field, -1);
			}
			else
			{
				fieldCopy = copyPaste(field, result);
				result.addFieldInfo(fieldCopy, -1);
			}
		}

		// duplicate actions
		LyInfoList<LyAction> actions = source.getActions();
		LyAction action, actionCopy;

		n = (actions == null ? 0: actions.getSize());

		for (i = 0; i < n; i++)
		{
			action = actions.getInfo(i);

			if (action.getParent() != source)
			{
				result.addAction(action, -1);
			}
			else
			{
				actionCopy = copyPaste(action, result);
				result.addAction(actionCopy, -1);
			}
		}

		// Clone the cache
		LyInfoCache currentCache = source.getCachePolicy();
		if (currentCache != null)
		{
			String userData = currentCache.getUserData();
			String cacheName = currentCache.getCachePolicyName();

			LyInfoCache cache = new LyInfoCache(cacheName, userData);
			setCachePolicy(result, cache);
		}
		return result;
	}

	private LyClassBinding duplicateClassBinding(LyClassBinding source)
	{
		if (source == null)
			return null;

		LyClassBinding result = null;
		if (source instanceof LyUnionBinding)
		{
			result = new LyUnionBinding();
			LyClassBinding b1 = ((LyUnionBinding)source).getClassBinding1();
			LyClassBinding b2 = ((LyUnionBinding)source).getClassBinding2();
			((LyUnionBinding)result).addClassBinding(duplicateClassBinding(b1));
			((LyUnionBinding)result).addClassBinding(duplicateClassBinding(b2));

			Vector<LyUnion> unions = ((LyUnionBinding)source).getUnions();
			int i, n = (unions == null ? 0 : unions.size());
			LyUnion union, copy;

			for (i = 0; i < n; i++)
			{
				union = unions.get(i);
				copy = new LyUnion(union.getValue1(), union.getValue2());
				((LyUnionBinding)result).addUnion(copy);
			}
		}
		else if (source instanceof LyClassInfoBinding)
		{
			result = new LyClassInfoBinding(source.getBinding());
		}
		else if (source instanceof LyLdapClassBinding)
		{
			LyLdapClassBinding ldap = (LyLdapClassBinding)source;

			// TODO should we clone the filter???
			result = new LyLdapClassBinding(source.getBinding(), source.getLocation(), source.getFilter());
			((LyLdapClassBinding)result).setFdnFormat(ldap.getFdnFormat());
			((LyLdapClassBinding)result).setRdn(ldap.getRdn());
			((LyLdapClassBinding)result).setClassFilter(ldap.getClassFilter());
		}
		else
		{
			// TODO should we clone the filter???
			result = new LyClassBinding(source.getBinding(),
					source.getLocation(), source.getFilter());
		}

		return result;
	}

	// Fields
	private LyFieldBinding duplicateFieldBinding(LyFieldBinding source)
	{
		if (source == null)
			return null;

		LyFieldBinding result;
		if (source instanceof LyFieldInfoBinding)
		{
			result = new LyFieldInfoBinding(source.getBinding());
		}
		else if (source instanceof LyLdapFieldBinding)
		{
			LyLdapFieldBinding sourceLdap = (LyLdapFieldBinding)source;
			result = new LyLdapFieldBinding(source.getBinding(),
					sourceLdap.isParent(), sourceLdap.isBinary());
		}
		else
		{
			result = new LyFieldBinding(source.getBinding(), source.getDefault(),
					source.getCBind(), source.isId(), source.getJavaType(), source.getJavaEncoder(),
					source.getPeerType());
		}
		return result;
	}

	private LyFieldConstraint duplicateFieldConstraint(LyFieldConstraint source)
	{
		LyFieldConstraint copy = new LyFieldConstraint(source.getType(), source.getColSpan(),
				source.getRowSpan(), source.isRemainder(), source.getColWeight(), source.getRowWeight(),
				source.getFill(), source.getHAlign(), source.getVAlign());

		copy.setAction(source.getAction());
		return copy;
	}
	private void completeFieldInfoFromSource(LyFieldInfo source, LyFieldInfo target, LyInfo parent)
	{
		completeObjectInFoFromSource(source, target, parent);

		target.setBeans(source.getBeans());

		target.setBinding(duplicateFieldBinding(source.getBinding()));

		target.setBehaviorClassName(source.getBehaviorClassName());
		target.setCalculated(source.isCalculated());

		Vector<LyFieldConstraint> constraints = source.getConstraints();
		int i, n = (constraints == null ? 0 : constraints.size());

		for (i = 0; i < n; i++)
		{
			target.addConstraint(duplicateFieldConstraint(constraints.elementAt(i)));
		}

		target.setControls(source.getControls());
		target.setDefaultValue(source.getDefaultValue());
		target.setFieldGroup(source.getFieldGroup()); // TODO duplicate???
		target.setFieldTab(source.getFieldTab()); // TODO duplicate ??
		target.setFieldTag(source.fromFieldTag());
		target.setFieldUnit(source.getFieldUnit());
		target.setFormat(source.getFormat());
		target.setFormula(source.getFormula());
		target.setGroup(source.getGroup());

		// TODO check field import...
		// todo check labels;
		// todo check path

		target.setJavaName(source.getJavaName());

		target.setSubTypes(source.getSubTypes());
		target.setUnits(source.getUnits());
	}

	/**
	 * Clone the selected field and add it to the given parent.
	 *
	 * @param source the field to clone
	 * @param parent the parent element of the new classinfo
	 */
	protected LyFieldInfo copyPaste(LyFieldInfo source, LyInfo parent)
	{
		if (source instanceof LyTextFieldInfo)
		{
			return duplicateTextFieldInfo((LyTextFieldInfo)source, parent);
		}
		else if (source instanceof LyNumberFieldInfo)
		{
			return duplicateNumberFieldInfo((LyNumberFieldInfo)source, parent);
		}
		else if (source instanceof LyDateFieldInfo)
		{
			return duplicateDateFieldInfo((LyDateFieldInfo)source, parent);
		}
		else if (source instanceof LyChoiceFieldInfo)
		{
			return duplicateChoiceFieldInfo((LyChoiceFieldInfo)source, parent);
		}
		else if (source instanceof LyFileFieldInfo)
		{
			return duplicateFileFieldInfo((LyFileFieldInfo)source, parent);
		}
		else if (source instanceof LyArrayFieldInfo)
		{
			return duplicateArrayFieldInfo((LyArrayFieldInfo)source, parent);
		}
		else if (source instanceof LyStructFieldInfo)
		{
			return duplicateStructFieldInfo((LyStructFieldInfo)source, parent);
		}
		else if (source instanceof LyRelationFieldInfo)
		{
			return duplicateRelationFieldInfo((LyRelationFieldInfo)source, parent);
		}

		return null;
	}

	private LyTextFieldInfo duplicateTextFieldInfo(LyTextFieldInfo source, LyInfo parent)
	{
		LyTextFieldInfo result = newTextFieldInfo(null, source.getJavaName(),
				(LyTextFieldInfo)source.getTemplate(), source.isMultiple(), source.getType());
		completeFieldInfoFromSource(source, result, parent);
		result.setMultilingual(source.isMultilingual());
		result.setCharTypes(source.getCharTypes());
		result.setEncoding(source.getEncoding());
		result.setMaxChars(source.getMaxChars());
		result.setMinChars(source.getMinChars());
		result.setNCols(source.getNCols());
		result.setNRows(source.getNRows());
		result.setRegExp(source.getRegExp());
		return result;
	}

	private LyNumberFieldInfo duplicateNumberFieldInfo(LyNumberFieldInfo source, LyInfo parent)
	{
		LyNumberFieldInfo result = newNumberFieldInfo(null, source.getJavaName(), (LyNumberFieldInfo)source.getTemplate(),
				 source.getType(), source.isModulo());

		completeFieldInfoFromSource(source, result, parent);

		result.setDecimals(source.getDecimals());
		result.setFunction(source.getFunction());
		result.setMax(source.getMax());
		result.setMin(source.getMin());
		result.setPitch(source.getPitch());
		result.setPrecision(source.getPrecision());
		return result;
	}

	private LyDateFieldInfo duplicateDateFieldInfo(LyDateFieldInfo source, LyInfo parent)
	{
		LyDateFieldInfo result = newDateFieldInfo(null, source.getJavaName(),
				(LyDateFieldInfo)source.getTemplate(), source.getType());

		completeFieldInfoFromSource(source, result, parent);
		result.setWithTimeZone(source.isWithTimeZone());
		return result;
	}

	private LyChoiceOptionGroup duplicateChoiceOptionGroup(LyChoiceOptionGroup source, LyInfo parent)
	{
		LyChoiceOptionGroup result = newChoiceOptionGroup(null, source.getName(), source.getAlias(), null);
		result.setValue(source.getValue());
		completeInfoFromSource(source, result, parent);
		// duplicate items
		LyInfoList<LyInfo> items = source.getChoiceItems();
		LyInfo sourceItem, targetItem;
		int i, n = (items == null ? 0 : items.getSize());
		for (i = 0; i < n ; i++)
		{
			sourceItem = items.getInfo(i);

			if (sourceItem.getParent() != source)
				targetItem = sourceItem; // reference -> no clone
			else if (sourceItem instanceof LyChoiceOptionGroup)
			{
				targetItem = duplicateChoiceOptionGroup((LyChoiceOptionGroup)sourceItem, result);
			}
			else // LyChoiceOption
			{
				targetItem = duplicateChoiceOption((LyChoiceOption)sourceItem, result);
			}
			result.addChoiceItem((LyChoiceItem)targetItem);
		}
		return result;
	}

	private LyChoiceOption duplicateChoiceOption(LyChoiceOption source, LyInfo parent)
	{
		LyChoiceOption result = newChoiceOption(null, source.getName(), source.getAlias(), source.getValue(),
				source.isDefault(), source.isDisable(), source.isUnknown(), source.hasTableMark());
		completeInfoFromSource(source, result, parent);
		return result;
	}

	private LyChoiceFieldInfo duplicateChoiceFieldInfo(LyChoiceFieldInfo source, LyInfo parent)
	{
		LyChoiceFieldInfo result = newChoiceFieldInfo(null, source.getJavaName(),
				(LyChoiceFieldInfo)source.getTemplate(), source.isMultiple(), source.getSortBy(), source.getType(),
				source.isNot());

		completeFieldInfoFromSource(source, result, parent);

		// duplicate groups
		LyInfoList<LyChoiceOptionGroup> groups = source.getChoiceOptionGroups();
		LyChoiceOptionGroup sourceGroup, targetGroup;
		int i, n = (groups == null ? 0 : groups.getSize());

		for (i = 0; i < n ; i++)
		{
			sourceGroup = groups.getInfo(i);

			if (sourceGroup.getParent() != source)
				targetGroup = sourceGroup; // reference -> no clone
			else
			{
				targetGroup = duplicateChoiceOptionGroup(sourceGroup, result);
			}

			result.addChoiceOptionGroup(targetGroup);
		}

		// duplicate option
		LyInfoList<LyChoiceOption> options = source.getChoiceOptions();
		LyChoiceOption sourceOption, targetOption;

		n = (options == null ? 0 : options.getSize());

		for (i = 0; i < n ; i++)
		{
			sourceOption = options.getInfo(i);

			if (sourceOption.getParent() != source)
				targetOption = sourceOption; // reference -> no clone
			else
			{
				targetOption = duplicateChoiceOption(sourceOption, result);
			}

			result.addChoiceOption(targetOption, -1);
		}

		result.setNCols(source.getNCols());
		return result;
	}

	private LyFileFieldInfo duplicateFileFieldInfo(LyFileFieldInfo source, LyInfo parent)
	{
		LyFileFieldInfo result = newFileFieldInfo(null, source.getJavaName(),
				(LyFileFieldInfo)source.getTemplate(),
				source.getMimeTypes(), source.getAttachmentHandler(),
				source.getAttachmentDirectory(), source.getDirectory(), source.getFilter());

		completeFieldInfoFromSource(source, result, parent);

		return result;
	}

	private LyRelationFieldInfo duplicateRelationFieldInfo(LyRelationFieldInfo source, LyInfo parent)
	{
		LyRelationFieldInfo result = newRelationFieldInfo(null, source.getJavaName(), source.getExtendedRelation(),
				(LyRelationFieldInfo)source.getTemplate(), source.isMultiple(), source.isOrdered(), source.getDirection(), source.getType(),
				source.getUserData(), source.ignoresContext(), source.isToolbarDisabled(), source.getShowAs());

		completeFieldInfoFromSource(source, result, parent);

		return result;
	}

	private LyStructFieldInfo duplicateStructFieldInfo(LyStructFieldInfo source, LyInfo parent)
	{
		//completeFieldInfoFromSource(source, result, parent);
		return null;
	}

	private LyArrayFieldInfo duplicateArrayFieldInfo(LyArrayFieldInfo source, LyInfo parent)
	{
		//completeFieldInfoFromSource(source, result, parent);
		return null;
	}

	// Actions
	/**
	 * Clone the selected action and add it to the given parent.
	 *
	 * @param source the action to clone
	 * @param parent the parent element of the new action
	 */
	protected LyAction copyPaste(LyAction source, LyInfo parent)
	{
		LyAction result = null;
		if (source instanceof LyTabAction)
			result = duplicateTabAction((LyTabAction)source, parent);
		else if (source instanceof LyCompoundAction)
			result = duplicateCompoundAction((LyCompoundAction)source, parent);
		else
			result = duplicateAction(source, parent);

		completeObjectInFoFromSource(source, result, parent);

		result.setAbstract(source.isAbstract());
		result.setMenu(source.getMenu());
		result.setMax(source.getMax());
		result.setMin(source.getMin());

		result.setGeometry(null);

		result.setParameters(null);
		result.setParametersDecl(null);
		result.setFilter(null);
		// clone the list of actions...
		//result.setActions(actions)

		return null;
	}

	private void completeAction(LyAction source, LyAction target, LyInfo parent)
	{
		LyInfoList<LyAction> actions = source.getActions();
		int i, n = (actions == null ? 0 : actions.getSize());
		LyAction action, copy;
		for (i = 0; i < n; i++)
		{
			action = actions.getInfo(i);

			if (action.getParent() != source)
				copy = action;
			else
				copy = copyPaste(action, target);

			target.addAction(copy, -1);
		}
	}

	private LyAction duplicateAction(LyAction source, LyInfo parent)
	{
		LyAction result = newAction(null, (LyAction)source.getTemplate(), source.getControllerClassName(),
				source.getBehaviorClassName(), source.getBuilderClassName(), source.getView(), source.getTarget());
		completeObjectInFoFromSource(source, result, parent);
		completeAction(source, result, parent);
		return result;
	}

	private LyTabAction duplicateTabAction(LyTabAction source, LyInfo parent)
	{
		LyTabAction result = newTabAction(null, (LyTabAction)source.getTemplate(), source.getControllerClassName(),
				source.getBehaviorClassName(), source.getBuilderClassName(), source.getClientContextIndex(),
				source.getTarget(), source.getType());

		completeObjectInFoFromSource(source, result, parent);
		completeAction(source, result, parent);

		LyInfoList<LyAction> actions = source.getTabActions();
		int i, n = (actions == null ? 0 : actions.getSize());
		LyAction action, copy;
		for (i = 0; i < n; i++)
		{
			action = actions.getInfo(i);

			if (action.getParent() != source)
				copy = action;
			else
				copy = copyPaste(action, result);

			result.addTabAction(copy);
		}
		return result;
	}

	private LyCompoundAction duplicateCompoundAction(LyCompoundAction source, LyInfo parent)
	{
		LyCompoundAction result = newCompoundAction(null, (LyCompoundAction)source.getTemplate(),
				source.getControllerClassName(), source.getBehaviorClassName(), source.getBuilderClassName(),
				source.getStyle(), source.getSizes(), source.getContext(), source.getClientContext(),
				source.getTarget());
		completeObjectInFoFromSource(source, result, parent);
		completeAction(source, result, parent);

		LyInfoList<LyAction> actions = source.getSubActions();
		int i, n = (actions == null ? 0 : actions.getSize());
		LyAction action, copy;
		for (i = 0; i < n; i++)
		{
			action = actions.getInfo(i);

			if (action.getParent() != source)
				copy = action;
			else
				copy = copyPaste(action, result);

			result.addSubAction(copy);
		}
		return result;
	}

	// project
	/**
	 * Clone the selected project and add it to the given parent.
	 *
	 * @param source the project to clone
	 * @param parent the parent element of the new classinfo
	 */
	protected LyProject copyPaste(LyProject source, LyInfo parent)
	{
		return null;
	}
}
