/*
 * LyRegExp.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.info;

import java.util.*;
import java.util.regex.*;

import leon.misc.*;

/**
 * This class defines a regular expression associated to a text.
 */
public class LyRegExp extends LyInfo
{
	public static final String __VERSION = "$Id: LyRegExp.java 28422 2010-11-03 12:03:38Z ehelmste $";

    /** All formats used by jakarta-oro. */
    protected static Hashtable<String, Pattern>     _formats = null;
    
    /**
     * Tests if a string match a regular expression.
     * @param  environment Environment where the test occured.
     * @param  format      The regular expression.
     * @param  value       The string that must be match the regula expression.
     * @return true if the given string match the regular expression, false otherwise.
     */
    public static boolean matchFormat(LyEnvironment environment, String format, String value)
    {
        if ((format == null) || (value == null))
            return true;

        Matcher   matcher = null;
        Pattern   pattern = null;
        String    key = format.intern();

        pattern = ((_formats == null) ? null : _formats.get(key));

        if (pattern == null)
        {
            try
            {
                pattern = getPattern(key);
                if (_formats == null)
                    _formats = new Hashtable<String, Pattern>();
                _formats.put(key, pattern);
            }
            catch (PatternSyntaxException e)
            {
                environment.logError("Invalid format : \"" + key + "\"\n" + e.getMessage());
                return false;
            }
        }

        matcher = pattern.matcher(value);
        return matcher.matches();
    } // end method matchFormat

    /**
     * Gets the pattern of the given format.
     * @param  format the format to analyse
     * @return the found pattern into the given format
     * @throws PatternSyntaxException if the patter of the format is not well formed
     */
    public static Pattern getPattern(String format)
        throws PatternSyntaxException
    {
        Pattern       pattern = Pattern.compile(format);

        return pattern;
    }
    
    
	/** Value of the unit. */
	String					   _value;

	/* Constructor */

	/**
	 * Constructor of a regular expression.
	 *
	 * @param id    Id of the regexp.
	 * @param value Value of the regexp.
	 */
	LyRegExp(String id, String value)
	{
		this(id);
		construct(value);
	}

	/**
	 * Constructor of a regular expression.
	 *
	 * @param id Id of the regexp.
	 */
	LyRegExp(String id)
	{
		super(id, false);
	}

	/**
	 * Constructor of a regular expression.
	 *
	 * @param id    Id of the regexp.
	 * @param value Value of the regexp.
	 */
	void construct(String value)
	{
		super.construct();

		setValue(value);
	}

	/**
	 * Gets value of the regexp.
	 */
	public final String getValue()
	{
		load();
		return _value;
	}
    
    /**
     * Sets value of the regexp.
     */
    protected void setValue(String value)
    {
        _value = ((value == null) ? null : value.intern());
    }
} // end class LyRegExp
