/*
 * LyClassInfo.java
 * Copyright (c) 2000-2010, W4.
 * All rights reserved.
 */

package leon.info;

/**
 * Class used to represents the internal class corresponding to a struct.
 */
class LyStructClassInfo extends LyClassInfo
{
    /**
     * Struct field info for which the class info is constructed.
     */
    protected LyStructFieldInfo     _structFieldInfo;
    
    /**
     * Constructor.
     * @param structFieldInfo   Struct field info for which the class info is constructed.
     * @param superClass        Super class of the struct class info.
     * @param template          Template of the struct class info. 
     * @param javaName          Java name of the struct class info.
     * @param abstractClass     Boolean indicating if the constructed class is abstract or not.
     */
    LyStructClassInfo(LyStructFieldInfo structFieldInfo, LyClassInfo superClass, LyClassInfo template,
        String javaName, boolean abstractClass)
    {
        super("__" + structFieldInfo.getId() + ".structClass", superClass,
            template, javaName, null, abstractClass, false, null);
        
        _structFieldInfo = structFieldInfo;
        
        setParent(structFieldInfo);
        setApplication(structFieldInfo._application);
    }
    
    /**
     * Checks if this classInfo results from a struct definition.
     *
     * @return True if this classInfo was created from a struct definition.
     */
    public boolean fromStruct()
    {
        return true;
    }
}
