/*
 * LyBinding.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.info.binding;

import leon.info.*;

/**
 * Container for the binding information found in the data description.
 */
public class LyBinding extends LyTag
	implements Cloneable
{
	public static final String __VERSION =
		"$Id: LyBinding.java 34925 2012-03-22 10:01:51Z jlsaouli $";

	/** Binding (mean to access information from a data provider). */
	String					   _binding;
    
    /** Boolean indicating if the binding is the default one generated by leon. */
    boolean                    _defaultBinding;
       

	/**
	 * Constructor.
	 * @param binding content of the binding information
	 */
	public LyBinding(String binding)
	{
		setBinding(binding);
        _defaultBinding = false;
	}

	/**
	 * Reads the binding information. Generic or specific connection classes use the binding
	 * information to fetch data from a data provider. For instance the binding info may be a CLASS
	 * ID in a DBMS.
	 * @return the binding information.
	 */
	public String getBinding()
	{		
		return _binding;
	}

	/**
	 * Sets the binding information. Generic or specific connection classes use the binding
	 * information to fetch data from a data provider. For instance the binding info may be a CLASS
	 * ID in a DBMS.
	 * @param binding the new binding information.
	 */
	void setBinding(String binding)
	{
		_binding = ((binding == null) ? null : binding.intern());
	}
    
    /**
     * Sets this binding as the default binding generated by leon or not.
     * @param defaultBinding True if default binding, false otherwise.
     */
    public void setDefaultBinding(boolean defaultBinding)
    {
        _defaultBinding = defaultBinding;
    }
    
    /**
     * Checks if this binding is the default binding generated by leon or not.
     * @return True if default binding, false otherwise.
     */
    public boolean isDefaultBinding()
    {
        return _defaultBinding;
    }
} // end class LyBinding
