/*
 * LyEncoderInterface.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.info.binding;

import leon.info.*;

public interface LyEncoderInterface
{
    /**
     * Builds a logical value for the given fieldInfo from a physical (peer) value.
     *
     * @param     fieldInfo  Field info for which the peer value is decoded.
     * @param     object     The peer value.
     * @param     valueClass Expected class of the logical value.
     * @return    The logical value that was built (instance of the valueClass).
     */
    Object decodeValue(LyFieldInfo fieldInfo, Object value, Class<?> valueClass);
    
    /**
     * Returns an object of the given class from a value of the given fieldInfo.
     * The purpose of this method is to support transformations from logical to physical representation.
     *
     * @param     fieldInfo   Field info for which the logical value is encoded.
     * @param     value       Value of the fieldInfo. This value is an instance of the class
     *                        returned by getValueClass method.
     * @param     objectClass Expected class of the physical value.
     */
    Object encodeValue(LyFieldInfo fieldInfo, Object value, Class<?> objectClass);
    
    /**
     * Returns a table of objects for the given classes from a value of the given fieldInfo.
     * The purpose of this method is to support transformations from logical to physical representation
     * when multiple bindings are used.
     *
     * @param     fieldInfo     Field info for which the logical values are encoded.
     * @param     value         Value of this fieldInfo. This value is an instance of the class
     *                          returned by getValueClass method.
     * @param     objectClasses Expected classes of the physical values.
     */
    Object[] encodeValues(LyFieldInfo fieldInfo, Object value, Class<?>[] objectClasses);
}
