/*
 * LyFieldsBinding.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.info.binding;

import java.util.*;

import leon.app.*;
import leon.info.*;
import leon.misc.*;


/**
 * Container for the binding information found in the data description for fieldInfo.
 */
public final class LyFieldsBinding extends LyFieldBinding
{
	public static final String __VERSION =
		"$Id: LyFieldsBinding.java 28997 2010-12-23 10:17:15Z bmichel $";

	/** Fields corresponding to this binding. */
	Vector<LyFieldBinding>		_bindings;

	/**
	 * Constructor.
	 */
	public LyFieldsBinding()
	{
		super(null, null, null, false);
	}

	/**
	 * Returns the field binding at the given index.
	 */
	public LyFieldBinding getFieldBinding(int index)
	{
		if (_bindings == null)
			return null;

		return _bindings.elementAt(index);
	}

	/**
	 * Returns the number of field bindings.
	 */
	public int getFieldBindingCount()
	{
		return ((_bindings == null) ? 0 : _bindings.size());
	}

	/**
	 * Gets the list of field bindings.
	 * @return the vector of all the field bindings. This is a copy of the internal vector that
	 *         contains these field bindings.
	 */
	public Vector<LyFieldBinding> getFieldBindings()
	{
		return (_bindings == null ? null : new Vector<LyFieldBinding>(_bindings));
	}

	/**
	 * Sets the fields corresponding to this binding.
	 */
	public void addFieldBinding(LyFieldBinding fieldBinding)
	{
		if (_bindings == null)
			_bindings = new Vector<LyFieldBinding>(2);

		_bindings.addElement(fieldBinding);
	}
	
	/**
     * Returns a table of objects for the given classes from a value of the given fieldInfo.
     * The purpose of this method is to support transformations from logical to physical representation
     * when multiple bindings are used.
     *
     * @param     value         Value of this fieldInfo. This value is an instance of the class
     *                          returned by getValueClass method.
     * @param     objectClasses Expected classes of the physical values..
     */
	public Object[] encodeValues(LyFieldInfo fieldInfo, Object value, Class<?>[] objectClasses)
    {
        LyApplication   application = fieldInfo.getApplication();
        Class<?>        encoderClass = getJavaEncoderClass(application);
        
        if (encoderClass != null)
        {
            try
            {
                LyEncoderInterface encoder = (LyEncoderInterface)encoderClass.newInstance();

                return encoder.encodeValues(fieldInfo, value, objectClasses);
            }
            catch (Exception e)
            {
                LyEnvironment   environment = application.getEnvironment();
                
                environment.writeException(e);
            }
        }
        
        return null;
    }
    
    /**
     * Sets the list of field bindings.
     */
    protected void setFieldBindings(Vector<LyFieldBinding> bindings)
    {
        _bindings = (bindings == null ? null : new Vector<LyFieldBinding>(bindings));
    }
} // end class LyFieldsBinding
