/*
 * LyArrayDaemon.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.info.daemon;

import leon.info.*;
import leon.info.binding.*;


/**
 * This class holds informations for array fieldinfos.
 * <p>See also tag <ARRAY_DAEMON> of leon.dtd.</p>
 * @see leon.data.daemon.LyArrayFieldInfoDaemon
 */
public final class LyArrayDaemon extends LyInfoDaemon
{
	public static final String __VERSION =
		"$Id: LyArrayDaemon.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** The cbind info of this daemon. */
	protected LyClassBinding   _classBinding;

	/** The fbind info of this daemon for the foreign key. */
	protected LyFieldBinding   _fkBinding;

	/** The fbind info of this daemon for the value field. */
	protected LyFieldBinding   _valueBinding;

	/** The fbind info of this daemon for the index field. */
	protected LyFieldBinding   _indexBinding;

	/** The fbind info of this daemon for the id field (may be null). */
	protected LyFieldBinding   _idBinding;

	/** The class info for array objects. */
	protected LyClassInfo	   _arrayClassInfo;

	/**
	 * Default constructor.
	 */
	public LyArrayDaemon()
	{
		super();

		_classBinding = null;
		_fkBinding = null;
		_valueBinding = null;
		_indexBinding = null;
		_idBinding = null;
	}

	/**
	 * Constructor with all parameters.
	 */
	public LyArrayDaemon(LyClassBinding classBinding, LyFieldBinding fkBinding,
		LyFieldBinding valueBinding, LyFieldBinding indexBinding,
		LyFieldBinding idBinding)
	{
		_classBinding = classBinding;
		_fkBinding = fkBinding;
		_valueBinding = valueBinding;
		_indexBinding = indexBinding;
		_idBinding = idBinding;
	}

	/**
	 * Accessor to the class binding.
	 * @return The classBinding
	 */
	public LyClassBinding getClassBinding()
	{
		return _classBinding;
	}

	/**
	 * Accessor to the foregin key field binding.
	 * @return The foreign key field binding
	 */
	public LyFieldBinding getFkBinding()
	{
		return _fkBinding;
	}

	/**
	 * Accessor to the value field binding.
	 * @return The value field binding
	 */
	public LyFieldBinding getValueBinding()
	{
		return _valueBinding;
	}

	/**
	 * Accessor to the index field binding.
	 * @return The index field binding
	 */
	public LyFieldBinding getIndexBinding()
	{
		return _indexBinding;
	}

	/**
	 * Accessor to the id field binding.
	 * @return The id field binding
	 */
	public LyFieldBinding getIdBinding()
	{
		return _idBinding;
	}

	/**
	 * Accessor to the array classInfo.
	 * @return The array class info.
	 */
	public LyClassInfo getArrayClassInfo()
	{
		return _arrayClassInfo;
	}

	/**
	 * Modifier of the class binding.
	 * @param classBinding class binding
	 */
	public void setClassBinding(LyClassBinding classBinding)
	{
		_classBinding = classBinding;
	}

	/**
	 * Modifier of the foreign key field binding.
	 * @param fieldBinding field binding
	 */
	public void setFkBinding(LyFieldBinding fieldBinding)
	{
		_fkBinding = fieldBinding;
	}

	/**
	 * Modifier of the value field binding.
	 * @param fieldBinding field binding
	 */
	public void setValueBinding(LyFieldBinding fieldBinding)
	{
		_valueBinding = fieldBinding;
	}

	/**
	 * Modifier of the index field binding.
	 * @param fieldBinding field binding
	 */
	public void setIndexBinding(LyFieldBinding fieldBinding)
	{
		_indexBinding = fieldBinding;
	}

	/**
	 * Modifier of the id field binding.
	 * @param fieldBinding field binding
	 */
	public void setIdBinding(LyFieldBinding fieldBinding)
	{
		_idBinding = fieldBinding;
	}

	/**
	 * Modifier of the class info representing array objects.
	 * @param classInfo tha array class info.
	 */
	public void setArrayClassInfo(LyClassInfo classInfo)
	{
		_arrayClassInfo = classInfo;
	}

	/**
	 * To add a field binding definition to this daemon. Fields binding must be added in the right
	 * order (fk, value, idx, id).
	 * @param fieldBinding the field binding to add.
	 */
	public void addFieldBinding(LyFieldBinding fieldBinding)
	{
		if (_fkBinding == null)
			setFkBinding(fieldBinding);
		else if (_valueBinding == null)
			setValueBinding(fieldBinding);
		else if (_indexBinding == null)
			setIndexBinding(fieldBinding);
		else if (_idBinding == null)
			setIdBinding(fieldBinding);
	}
} // end class LyArrayDaemon
