/**
 * LyFormatterInterface.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyFormatterInterface.java 28422 2010-11-03 12:03:38Z ehelmste $
 */

package leon.info.infointerface;

import leon.info.*;


/**
 * Interface implemented by classes that format field values.
 * @author Lyria
 */
public interface LyFormatterInterface
{
	String __VERSION = "$Id: LyFormatterInterface.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Method to format a value to an edition string with the given unit.
	 * @param  fieldInfo the fieldInfo for which a value is formatted
	 * @param  value     the value to format
	 * @param  unit      the destination unit of the string value
	 * @return The edition string corresponding to the given value and unit.
	 */
	String toString(LyFieldInfo fieldInfo, Object value, LyUnit unit);

	/**
	 * Method to format a value to a displayable string with the given unit.
	 * @param  fieldInfo the fieldInfo for which a value is formatted
	 * @param  value     the value to format
	 * @param  unit      the destination unit of the string value
	 * @return The displayable string corresponding to the given value and unit.
	 */
	String toName(LyFieldInfo fieldInfo, Object value, LyUnit unit);

	/**
	 * Method to format a value to a displayable short string with the given unit.
	 * @param  fieldInfo the fieldInfo for which a value is formatted
	 * @param  value     the value to format
	 * @param  unit      the destination unit of the string value
	 * @return The short displayable string corresponding to the given value and unit.
	 */
	String toShortName(LyFieldInfo fieldInfo, Object value, LyUnit unit);

	/**
	 * Method to parse a string value with a given unit.
	 * @param  fieldInfo the fieldInfo for which the string value is parsed.
	 * @param  value     the string value to parse
	 * @param  unit      the unit of the string value
	 * @return The value corresponding to the given string and unit.
	 */
	Object parse(LyFieldInfo fieldInfo, String value, LyUnit unit);
}