/*
 * LyLocation.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.info.location;

/**
 * This class implements a data file location.
 */
public class LyFileLocation extends LyLocation
{
	public static final String __VERSION =
		"$Id: LyFileLocation.java 35251 2012-04-17 15:32:36Z jlsaouli $";

    /** Location types. */
    public enum Type
    {
    	// XML file.
    	XML,
    	// INFO file.
    	INFO,
    	// CSV file.
    	CSV
    }

	/** Default separator used between fields. */
	public static final String FIELD_SEP = "|".intern();

	/** Default separator used between values. */
	public static final String VALUE_SEP = "~".intern();

	/** To specify the type of the file location. */
	protected Type			   _type;

	/** Value separator (for multiple fields). */
	protected String		   _valueSep;

	/** Field separator. */
	protected String		   _fieldSep;

	/**
	 * Init method.
	 */
	protected void init()
	{
		super.init();

		_type = Type.INFO;
		_valueSep = VALUE_SEP;
		_fieldSep = FIELD_SEP;
	}

    /**
     * Constructor.
     * @param id       identifier of the location
     */
    public LyFileLocation(String id)
    {
        this(id, null, Type.INFO, null, null);
    }
    
	/**
	 * Constructor.
	 * @param id       identifier of the location
	 * @param url      url of the provider
	 * @param type     type of file
	 * @param fieldSep Field separator
	 * @param valueSep Value separator
	 * @deprecated replaced by LyFileLocation(String, String, Type, String, String)
	 */
	public LyFileLocation(String id, String url, String type, String fieldSep, String valueSep)
	{
		this(id, url, Type.valueOf(type), fieldSep, valueSep);
	}
	
	/**
	 * Constructor.
	 * @param id       identifier of the location
	 * @param url      url of the provider
	 * @param type     type of file
	 * @param fieldSep Field separator
	 * @param valueSep Value separator
	 */
	public LyFileLocation(String id, String url, Type type, String fieldSep, String valueSep)
	{
		super(id, url);

		setType(type);
		setFieldSep(fieldSep);
		setValueSep(valueSep);
	}

	/**
	 * Accessor to the url of this location.
	 * @return the type of this location
	 */
	public Type getType()
	{
		return _type;
	}

	/**
	 * Accessor to the field separator of this location.
	 * @return the field separator of this location
	 */
	public String getFieldSep()
	{
		return _fieldSep;
	}

	/**
	 * Accessor to the value separator of this location.
	 * @return the value separator of this location
	 */
	public String getValueSep()
	{
		return _valueSep;
	}

	/**
	 * Accessor of the persistent mode of this location.
	 * @return persistent mode is always false for a file location.
	 */
	public boolean isPersistent()
	{
		return false;
	}
    
    /**
     * Sets the type of this location.
     * @param type the type of this location
     */
    protected void setType(Type type)
    {
        _type = (type == null ? Type.INFO : type);
    }

    /**
     * Sets the field separator of this location.
     * @param fieldSep the field separator of this location
     */
    protected void setFieldSep(String fieldSep)
    {
        _fieldSep = fieldSep;
    }

    /**
     * Sets the value separator of this location.
     * @param valueSep the value separator of this location
     */
    protected void setValueSep(String valueSep)
    {
        _valueSep = valueSep;
    }
} // end class LyFileLocation
