/*
 * LyLocation.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.info.location;

import leon.app.*;
import leon.info.*;
import leon.info.notifier.*;
import leon.peer.*;


/**
 * This class implements a data location.
 */
public class LyLocation extends LyInfo
{
	public static final String __VERSION =
		"$Id: LyLocation.java 29369 2011-01-26 14:36:22Z jlsaouli $";

	/** Default id for a location. */
	public static String	   DEFAULT_LOCATION = "DefaultLocation".intern();

	/** Id of info location. */
	public static String	   ID_L_INFO = "_info".intern();

	/** Id of union location. */
	public static String	   ID_L_UNION = "_union".intern();

	/** Id of extend location. */
	public static String	   ID_L_EXTEND = "_extend".intern();

	/** To specify the url of the data provider. */
	protected String		   _url;

	/**
	 * Reference to the Notifier used within this connector location. May be null if no notification
	 * mechanism is required.
	 */
	protected LyNotifier	   _notifier;

	/**
	 * Init method.
	 * @see LyInfo#init
	 */
	protected void init()
	{
		super.init();

		_url = null;
	}

    /**
     * Constructor.
     * @param id  identifier of the location
     */
    public LyLocation(String id)
    {
        this(id, null);
    }
    
	/**
	 * Constructor.
	 * @param id  identifier of the location
	 * @param url url of the provider
	 */
	public LyLocation(String id, String url)
	{
		super(id);

		_url = url;
	}

	/**
	 * This method allows to test the configuration of a classInfo for the given location This
	 * method is called during the check phase of the parsing.
	 * @param classInfo the classInfo to check
	 */
	public void check(LyClassInfo classInfo)
	{
	}

	/**
	 * Accessor to the url of this location.
	 * @return the url of this location
	 */
	public String getUrl()
	{
		return _url;
	}

	/**
	 * Accessor to the Notifier used within this connector location.
	 * @return notifier reference. May be null if no notification mechanism is required.
	 */
	public LyNotifier getNotifier()
	{
		return _notifier;
	}

	/**
	 * Accessor to the Notifier used within this connector location.
	 * @param notifier the notifier reference. May be null if no notification mechanism is required.
	 */
	public void setNotifier(LyNotifier notifier)
	{
		_notifier = notifier;
		if (notifier != null)
		    _notifier.setLocation(this);
	}

	/**
	 * Accessor of the persistent mode of this location.
	 * @return persistent mode.
	 */
	public boolean isPersistent()
	{
		return true;
	}

	/**
	 * To set the target of the step.
	 * @param classInfo the new target.
	 */
	void setUrl(String url)
	{
		_url = url;
	}
	
	/**
     * Sets the parent of this info.
     *
     * @param the parent of this info
     */
    protected void setParent(LyInfo parent)
    {
        super.setParent(parent);
        
        LyApplication   application = getApplication();
        
        if (application.isValid())
        {
            LyDataProvider  provider = LyDataProviderFactory.getProvider(application, this);
            
            application.addDataProvider(provider);
        }
    }
} // end class LyLocation
