/*
 * LyActionResource.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.info.misc;

import leon.misc.*;
import leon.info.*;


/**
 * This class is used to store resource on action A resource is composed of a key (equivalent to a
 * key in a properties file) a type (RESOURCE, STRING, MESSAGE, COLOR, FONT, FILE, IMAGE) a value.
 */
public class LyActionResource extends LyTag
{
	public static final String			 __VERSION =
		"$Id: LyActionResource.java 28422 2010-11-03 12:03:38Z ehelmste $";

	protected LyEnvironment.ResourceType _type;
	protected String					 _key;
	protected String					 _value;

    /**
     * Constructor.
     */
    protected LyActionResource()
    {
        this(null, null, null);
    }
    
	/**
	 * Constructor.
	 */
	public LyActionResource(LyEnvironment.ResourceType type, String key, String value)
	{
		setType(type);
		setKey(key);
		setValue(value);
	}

	/**
	 * Accessor for type.
	 */
	public LyEnvironment.ResourceType getType()
	{
		return _type;
	}

	/**
	 * Accessor for key.
	 */
	public String getKey()
	{
		return _key;
	}

	/**
	 * Accessor for value.
	 */
	public String getValue()
	{
		return _value;
	}

	/**
	 * Return a string corresponding to this resource.
	 *
	 * @return a string representing this value
	 */
	public String toString()
	{
		String result = "key = " + _key + " value= " + _value + " type = " + _type;

		return result;
	}

	/**
	 * Check if this resource is equal to the given one.
	 */
	public boolean equals(Object resource)
	{
		return (resource instanceof LyActionResource)
			&& (_type == ((LyActionResource)resource)._type)
			&& (_key == ((LyActionResource)resource)._key)
			&& (_value == ((LyActionResource)resource)._value);
	}
    
    /**
     * Sets type.
     */
    protected void setType(LyEnvironment.ResourceType type)
    {
        _type = (type == null ? LyEnvironment.ResourceType.RESOURCE : type);
    }

    /**
     * Sets key.
     */
    protected void setKey(String key)
    {
        _key = ((key == null) ? null : key.intern());
    }

    /**
     * Sets value.
     */
    protected void setValue(String value)
    {
        _value = ((value == null) ? null : value.intern());
    }
} // end class LyActionResource
