/*
 * LyArgument.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.info.misc;

import leon.info.*;

/**
 * This class is used to store an argument fo a control associated to a class or to a field. An
 * argument is composed of a name and a value
 */
public class LyArgument extends LyTag
{
	public static final String __VERSION =
		"$Id: LyArgument.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Name of the argument. */
	protected String		   _name;

	/** Value of the argument. */
	protected Object		   _value;

    /**
     * Default constructor.
     */
    protected LyArgument()
    {  
    }
    
	/**
	 * Constructor.
	 */
	public LyArgument(String name, Object value)
	{
		_name = ((name == null) ? null : name.intern());
		if (value instanceof String)
			_value = ((String)value).intern();
		else
			_value = value;
	}

	/**
	 * Accessor for argument name.
	 * @return Name of this argument.
	 */
	public String getName()
	{
		return _name;
	}

    /**
     * Sets argument name.
     * @param name Name of this argument.
     */
    protected void setName(String name)
    {
        _name = name;
    }

	/**
	 * Accessor for argument value.
	 * @return Value of this argument.
	 */
	public Object getValue()
	{
		return _value;
	}

	/**
	 * Sets argument value.
	 * @param value Value of this argument.
	 */
	public void setValue(Object newValue)
	{
		_value = newValue;
	}

	/**
	 * Compare two arguments by value.
	 * @param  object object to compare to
	 * @return true if equals by value
	 */
	public boolean equals(Object object)
	{
		if (this == object)
			return true;

		// Make by value comparizon
		if (object instanceof LyArgument)
		{
			LyArgument compare = (LyArgument)object;

			// check name
			if ((_name == compare._name)
					|| ((_name != null) && (_name.equals(compare.getName()))))
			{
				// check value
				if (((_value == null) && (compare.getValue() == null))
						|| ((_value != null) && (_value.equals(compare.getValue()))))
					return true;
			}
		}
		return false;
	}
} // end class LyArgument
