/*
 * LyFieldGroup.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.info.misc;

import leon.info.*;

/**
 * This class defines a field group.
 */
public class LyFieldGroup extends LyInfo
{
	public static final String			  __version =
		"$Id: LyFieldGroup.java 30064 2011-03-15 08:58:34Z jlsaouli $";

    /** Group states. */
    public enum State
    {
        OPENED,
        CLOSED,
        FIXED
    }
    
    
    /** Name of the field group. */
    String              _name;

	/** The prefix of the field group. */
	String				_prefix;

    /** Constraint information used for form layout. */
    LyFieldConstraint   _constraint;
    
    /** State of the group. */
    State               _state = State.FIXED;
    

	/* Constructor */
    
	/**
	 * Constructor of a field group.
	 *
	 * @param id Id of the field group.
	 * @param name Name of the field group.
	 */
	public LyFieldGroup(String id, String name, State state)
	{
        super(id);
        
        setName(name);
        setPrefix(null);
        setConstraint(null);
        setState(state);
	}

    /**
     * Constructor of a field group.
     *
     * @param name Name of the field group.
     */
    public LyFieldGroup(String name)
    {
        this(null, name, State.FIXED);
    }
    
	/**
	 * Gets name of the field group.
	 */
	public final String getName()
	{
		return (_name == null ? getId() : _name);
	}

	/**
	 * Accessor to the file prefix for this field group.
	 *
	 * @return File prefix of the field group. If the file prefix is null, this method returns the id.
	 */
	public final String getPrefix()
	{
		return (_prefix == null ? null : _prefix);
	}

	/**
	 * Sets the prefix corresponding to this field group.
	 *
	 * @param prefix Prefix for this field group.
	 */
	public void setPrefix(String prefix)
	{
		_prefix = (prefix == null ? null : prefix.intern());
	}

    /**
     * Gets field constraint used in forms layouts.
     *
     * @return Field constraint of this fieldInfo, null if no constraint was defined.
     */
    public LyFieldConstraint getConstraint()
    {
        return _constraint;
    }
    
    /**
     * Sets field constraint used in forms layouts.
     *
     * @param constraint Field constraint of this fieldInfo.
     * @see   #setCaptionConstraint
     */
    public void setConstraint(LyFieldConstraint constraint)
    {
        _constraint = constraint;
    }
    
    /**
     * Gets state of the group.
     *
     * @return One of the STATE_XXX constants indicating the state of the group.
     */
    public State getState()
    {
        return _state;
    }
    
    /**
     * Sets name of the field group.
     */
    public void setName(String name)
    {
        _name = (name == null ? null : name.intern());
    }
    
    /**
     * Sets the parent of this info.
     *
     * @param the parent of this info
     */
    public void setParent(LyInfo parent)
    {
        super.setParent(parent);
        
        if (parent instanceof LyProject)
            addToParentProject();
    }
 
    /**
     * Sets state of the group.
     *
     * @param state One of the STATE_XXX constants indicating the state of the group.
     */
    protected void setState(State state)
    {
        _state = (state == null ? State.FIXED : state);
    }
} // end class LyFieldGroup
