/*
 * LyFieldTab.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.info.misc;

import leon.info.*;

/**
 * This class defines a field tab.
 */
public class LyFieldTab extends LyInfo
{
	public static final String			  __version =
		"$Id: LyFieldTab.java 30064 2011-03-15 08:58:34Z jlsaouli $";

    /** Name of the field tab. */
    String              _name;

    /** The prefix of the field group. */
    String              _prefix;
    
    
	/* Constructor */
   
	/**
	 * Constructor of a field tab.
	 *
	 * @param id Id of the field tab.
	 * @param name Name of the field tab.
	 */
	public LyFieldTab(String id, String name)
	{
		super(id);
		
        setName(name);
        setPrefix(null);
	}
    
    /**
     * Constructor of a field tab.
     *
     * @param name Name of the field tab.
     */
    public LyFieldTab(String name)
    {
        this(null, name);
    }

	/**
	 * Gets name of the field tab.
	 */
	public final String getName()
	{
		return (_name == null ? getId() : _name);
	}
    
    /**
     * Accessor to the file prefix for this field group.
     *
     * @return File prefix of the field group. If the file prefix is null, this method returns the id.
     */
    public final String getPrefix()
    {
        return (_prefix == null ? null : _prefix);
    }

    /**
     * Sets name of the field group.
     */
    public void setName(String name)
    {
        _name = (name == null ? null : name.intern());
    }
    
    /**
     * Sets the prefix corresponding to this field group.
     *
     * @param prefix Prefix for this field group.
     */
    public void setPrefix(String prefix)
    {
        _prefix = (prefix == null ? null : prefix.intern());
    }

    /**
     * Sets the parent of this info.
     *
     * @param the parent of this info
     */
    public void setParent(LyInfo parent)
    {
        super.setParent(parent);
        
        if (parent instanceof LyProject)
            addToParentProject();
    }
} // end class LyFieldTab
