/*
 * LyGeometry.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.info.misc;

import leon.info.*;

/**
 * This class defines graphical characteristics of an action view.
 */
public class LyGeometry extends LyTag
{
	public static final String __VERSION =
		"$Id: LyGeometry.java 34734 2012-03-16 09:13:13Z jlsaouli $";

	/** X position. */
	protected int			   _x;

	/** Y position. */
	protected int			   _y;

	/** Width. */
	protected int			   _width;

	/** Height. */
	protected int			   _height;

	/** Min width. */
	protected int			   _minWidth;

	/** Min height. */
	protected int			   _minHeight;

    /**
     * Constructor.
     */
    protected LyGeometry()
    {
    }
    
	/**
	 * Constructor.
	 */
	public LyGeometry(int x, int y, int width, int height, int minWidth, int minHeight)
	{
		_x = x;
		_y = y;
		_width = width;
		_height = height;
		_minWidth = minWidth;
		_minHeight = minHeight;
	}

	/**
	 * Accessor for x position.
	 */
	public int getX()
	{
		return _x;
	}

	/**
	 * Accessor for y position.
	 */
	public int getY()
	{
		return _y;
	}

	/**
	 * Accessor for width.
	 */
	public int getWidth()
	{
		return _width;
	}

	/**
	 * Accessor for height.
	 */
	public int getHeight()
	{
		return _height;
	}

	/**
	 * Accessor for minimum width.
	 */
	public int getMinWidth()
	{
		return _minWidth;
	}

	/**
	 * Accessor for minimum height.
	 */
	public int getMinHeight()
	{
		return _minHeight;
	}

	/**
	 * Check if this geometry is equal to the given one.
	 */
	public boolean equals(Object geometry)
	{
		if (!(geometry instanceof LyGeometry))
			return false;
		return equals((LyGeometry)geometry);
	}
	
	/**
	 * Check if this geometry is equal to the given one.
	 */
	public boolean equals(LyGeometry geometry)
	{
		return (geometry != null)
			&& (_x == geometry._x)
			&& (_y == geometry._y)
			&& (_width == geometry._width)
			&& (_height == geometry._height)
			&& (_minWidth == geometry._minWidth)
			&& (_minHeight == geometry._minHeight);
	}
    
    /**
     * Sets the x position.
     */
    protected void setX(int x)
    {
        _x = x;
    }

    /**
     * Sets the y position.
     */
    protected void setY(int y)
    {
        _y = y;
    }

    /**
     * Sets the width.
     */
    protected void setWidth(int width)
    {
        _width = width;
    }

    /**
     * Sets the height.
     */
    protected void setHeight(int height)
    {
        _height = height;
    }

    /**
     * Sets the minimum width.
     */
    protected void setMinWidth(int minWidth)
    {
        _minWidth = minWidth;
    }

    /**
     * Sets the minimum height.
     */
    protected void setMinHeight(int minHeight)
    {
        _minHeight = minHeight;
    }
} // end class LyGeometry
