/*
 * InfoJava.java
 * Copyright (c) 2006, Lyria.
 * All rights reserved.
 * Version: $Id: InfoJava.java,v 1.2 2010/07/01 07:20:14 jls Exp $
 */

package leon.info.misc;

import leon.app.*;
import leon.info.*;

/**
 * This class describes a java Class used by an application.
 */
public class LyJavaName extends LyTag
{
    /**
     * Checks if the given java class is generic or not.
     * @return True if generic, false if specific. 
     */
    public static boolean isGeneric(String javaName)
    {
        if (javaName == null)
            return false;
        
        return (javaName.startsWith("leon.") && !javaName.startsWith("leon.contrib"))
            || javaName.startsWith("eu.w4.ae.");
    }

    
    /** Java name.* */
    protected String   _javaName;
    
	/** extern identifier.* */
	protected boolean  _external;
	
	/** Java class.*/
	protected Class<?> _javaClass;

    /**
     * Default constructor.
     */
    public LyJavaName()
    {
    }
    
	/**
	 * The constructor: store the id.
	 */
	public LyJavaName(LyInfo info, String javaName)
	{
		this(info, javaName, false);
	}

	/**
	 * The constructor: store the javaName and extern property.
	 */
	public LyJavaName(LyInfo info, String javaName, boolean external)
	{
		_javaName = (javaName == null ? null : javaName.intern());
		_external = external;
		
		setParent(info);
	}

	/**
     * Gives the internal identifier.
     */
    public String getJavaName()
    {
        return _javaName;
    }
    
    /**
     * Indicates if Java is external or not.
     */
	public boolean isExternal()
	{
		return _external;
	}

    /**
     * Sets the internal identifier.
     */
    protected void setJavaName(String javaName)
    {
        if (javaName != null)
            _javaName = javaName;
        _javaClass = null;
    }
    
    /**
     * Sets extern property.
     */
    protected void setExternal(boolean external)
    {
        _external = external;
    }
    
    /**
     * This method enables to find a class whose name matches the java name. 
     * @return  The loaded class or null if it could not be found.
     */
    public Class<?> getJavaClass()
    {
        if (_javaClass != null)
            return _javaClass;
        
        LyApplication   application = getApplication();
        
        _javaClass = application.loadClass(_javaName);
        return _javaClass;
    }
    
    /**
     * Checks if the associated java class is generic or not.
     * @return True if generic, false if specific. 
     */
    public boolean isGeneric()
    {
    	return isGeneric(_javaName);
    }
} // end class MetaJava
