/*
 * LyParameter.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.info.misc;

import java.util.*;
import leon.info.*;

/**
 * This class is used to store parameter on action and generic location A parameter is composed of a
 * name and a value.
 */
public class LyParameter extends LyTag
{
	public static final String __VERSION =
		"$Id: LyParameter.java 31544 2011-06-24 13:09:36Z bangiole $";

	protected String		   _name;
    
	protected Object		   _value;


    /**
     * Constructor.
     */
    public LyParameter(String name)
    {
        this(name, null);
    }
    
    /**
	 * Constructor.
	 */
	public LyParameter(String name, Object value)
	{
        setName(name);
        setValue(value);
	}

	/**
	 * Accessor for type.
	 */
	public String getName()
	{
		return _name;
	}

	/**
	 * Accessor for key.
	 */
	public Object getValue()
	{
		return _value;
	}

	/**
	 * compare two parameters by value.
	 * @param  object object to compare to
	 * @return true if equals by value
	 */
	public boolean equals(Object object)
	{
		if (this == object)
			return true;
		
		// Make by value comparison
		if (object instanceof LyParameter)
		{
			LyParameter compare = (LyParameter)object;

			// check name
			if ((_name == compare._name)
					|| ((_name != null) && (_name.equals(compare.getName()))))
			{
				// check value
				if (((_value == null) && (compare.getValue() == null))
						|| ((_value != null) && (_value.equals(compare.getValue()))))
					return true;
			}
		}
		return false;
	}
    
    /**
     * Sets name.
     */
    protected void setName(String  name)
    {
        _name = ((name == null) ? null : name.intern());
    }
    
    /**
     * Sets value.
     */
    public void setValue(Object value)
    {
        _value = value;
    }
    
    /**
     * Gets value as String.
     */
    protected String getStringValue()
    {
        return (_value instanceof String ? (String)_value : null);
    }
    
    /**
     * Gets value as Number.
     */
    protected Number getNumberValue()
    {
        return (_value instanceof Number ? (Number)_value : null);
    }
    
    /**
     * Gets value as Boolean.
     */
    protected Boolean getBooleanValue()
    {
        return (_value instanceof Boolean ? (Boolean)_value : null);
    }
    
    /**
     * Gets value as action marks.
     */
    protected BitSet getAMarksValue()
    {
        return ((_value instanceof LyBitSet)
            && (((LyBitSet)_value).getType() == LyBitSet.Type.A_MARKS) ? ((LyBitSet)_value).getGenericMarks() : null);
    }
    
    /**
     * Gets value as action specific marks.
     */
    protected String[] getASpecificMarksValue()
    {
        return ((_value instanceof LyBitSet)
            && (((LyBitSet)_value).getType() == LyBitSet.Type.A_MARKS) ? ((LyBitSet)_value).getSpecificMarks() : null);
    }
    
    /**
     * Gets value as field marks.
     */
    protected BitSet getFMarksValue()
    {
        return ((_value instanceof LyBitSet)
            && (((LyBitSet)_value).getType() == LyBitSet.Type.F_MARKS) ? ((LyBitSet)_value).getGenericMarks() : null);
    }
    
    /**
     * Gets value as field specific marks.
     */
    protected String[] getFSpecificMarksValue()
    {
        return ((_value instanceof LyBitSet)
            && (((LyBitSet)_value).getType() == LyBitSet.Type.F_MARKS) ? ((LyBitSet)_value).getSpecificMarks() : null);
    }
    
    /**
     * Gets value as Info.
     */
    public LyInfo getInfoValue()
    {
        return (_value instanceof LyInfo ? (LyInfo)_value : null);
    }
    
    /**
     * Gets value as Sort.
     */
    public LySort getSortValue()
    {
        return (_value instanceof LySort ? (LySort)_value : null);
    }
    
    /**
     * Gets value as Filter.
     */
    public LyFilter getFilterValue()
    {
        return (_value instanceof LyFilter ? (LyFilter)_value : null);
    }
    
    /**
     * Gets value as classInfo.
     */
    public LyClassInfo getClassInfoValue()
    {
        return (_value instanceof LyClassInfo ? (LyClassInfo)_value : null);
    }
    
    /**
     * Gets value as chartInfo.
     */
    public LyChartInfo getChartInfoValue()
    {
        return (_value instanceof LyChartInfo ? (LyChartInfo)_value : null);
    }
    
    /**
     * Sets value as String.
     */
    protected void setStringValue(String value)
    {
    	setValue(value);
    }
    
    /**
     * Gets value as Number.
     */
    protected void setNumberValue(Number value)
    {
    	setValue(value == null ? null : value.toString());
    }
    
    /**
     * Gets value as Boolean.
     */
    protected void setBooleanValue(Boolean value)
    {
        setValue(value == null ? null : value.toString());
    }
    
    /**
     * Sets value as action marks.
     */
    protected void setAMarksValue(BitSet marks)
    {
    	setValue(new LyBitSet(LyBitSet.Type.A_MARKS, marks));
    }
    
    /**
     * Sets value as action specific marks.
     */
    protected void setASpecificMarksValue(String[] specificMarks)
    {
        setValue(new LyBitSet(LyBitSet.Type.A_MARKS, specificMarks));
    }
    
    /**
     * Sets value as field marks.
     */
    protected void setFMarksValue(BitSet marks)
    {
        setValue(new LyBitSet(LyBitSet.Type.F_MARKS, marks));
    }
    
    /**
     * Sets value as field specific marks.
     */
    protected void setFSpecificMarksValue(String[] specificMarks)
    {
        setValue(new LyBitSet(LyBitSet.Type.F_MARKS, specificMarks));
    }
    
    /**
     * Sets value as Info.
     */
    protected void setInfoValue(LyInfo info)
    {
    	setValue(info);
    }
    
    /**
     * Sets value as Sort.
     */
    protected void setSortValue(LySort sort)
    {
    	setValue(sort);
    }
    
    /**
     * Sets value as Filter.
     */
    protected void setFilterValue(LyFilter filter)
    {
    	setValue(filter);
    }
    
    /**
     * Sets value as classInfo.
     */
    protected void setClassInfoValue(LyClassInfo classInfo)
    {
    	setValue(classInfo);
    }
    
    /**
     * Sets value as chartInfo.
     */
    protected void setChartInfoValue(LyChartInfo chartInfo)
    {
    	setValue(chartInfo);
    }
} // end class LyParameter
