/*
 * LyParameterDecl.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.info.misc;

import leon.info.*;

/**
 * This class is used to store parameter declaration on action A declaration is composed of a name a
 * type (see leon.dtd) a valueclass (corresponding to the type) It can be optional or not.
 */
public class LyParameterDecl extends LyTag
{
	public static final String __VERSION =
		"$Id: LyParameterDecl.java 34729 2012-03-16 07:48:27Z jlsaouli $";

	/** The name of the parameter. */
	protected String		   _name;

	/**
	 * Gives the type of the parameter : a string, action marks, field marks, filter marks, a
	 * filter, a single reference to a LyInfo object or a multiple reference to LyInfo objects.
	 */
	protected Type		       _type;

	/** Indicates if the parameter is required or not. */
	protected boolean		   _required;

	/** Domain of values accepted on the parameter. */
	protected String			_domain;

    /** Parameter types. */
    public enum Type
    {
    	A_MARKS(leon.info.LyBitSet.class),
        F_MARKS(leon.info.LyBitSet.class),
        FILTER(leon.info.LyFilter.class),
        SORT(leon.info.LySort.class),
        CLASS(leon.info.LyClassInfo.class),
        FIELD(leon.info.LyFieldInfo.class),
        ACTION(leon.info.LyAction.class),
        CHART(leon.info.LyChartInfo.class),
        INFOS(leon.info.LyInfoList.class),
        STRING(java.lang.String.class),
        FILE(java.lang.String.class),
        NUMBER(java.lang.String.class),
        CHOICE(java.lang.String.class),
        BOOLEAN(java.lang.String.class),
        CHOICES(java.lang.String.class);
        
		Class<?>   _typeClass;
        
		Type(Class<?> typeClass)
        {
            _typeClass = typeClass;
        }
        
		Class<?> getTypeClass()
        {
            return _typeClass;
        }
    }

    /**
     * Default constructor.
     */
    public LyParameterDecl(String name)
    {
        this(name, null, false, null);
    }
    
	/**
	 * Constructor.
	 */
	public LyParameterDecl(String name, Type type, boolean required, String domain)
	{
		setName(name);
		setType(type);
		setRequired(required);
		setDomain(domain);
	}
    
	/**
	 * Accessor for Name.
	 */
	public String getName()
	{
		return _name;
	}

	/**
	 * Accessor for Type.
	 */
	public Type getType()
	{
		return _type;
	}

	/**
	 * Accessor for required.
	 */
	public boolean isRequired()
	{
		return _required;
	}

	/**
	 * Gets value class.
	 */
	public Class<?> getValueClass()
	{
		return _type.getTypeClass();
	}

	/**
	 * Gets domain.
	 */
	public String getDomain()
	{
		return _domain;
	}

	/**
	 * Check if this parameter declaration is equal to the given one.
	 */
	public boolean equals(Object decl)
	{
		if (!(decl instanceof LyParameterDecl))
			return false;
		return equals((LyParameterDecl)decl);
	}
	
	/**
	 * Check if this parameter declaration is equal to the given one.
	 */
	public boolean equals(LyParameterDecl decl)
	{
		return (decl != null)
			&& (_name == decl._name)
			&& (_type == decl._type)
			&& (_required == decl._required)
			&& (_domain == decl._domain);
	}
    
    /**
     * Sets name.
     */
    protected void setName(String  name)
    {
        _name = ((name == null) ? null : name.intern());
    }

    /**
     * Sets type.
     */
    protected void setType(Type type)
    {
        _type = (type == null ? Type.STRING : type);
    }

    /**
     * Sets required property.
     */
    protected void setRequired(boolean required)
    {
        _required = required;
    }

    /**
     * Sets domain.
     */
    protected void setDomain(String domain)
    {
        _domain = ((domain == null) ? null : domain.intern());
    }
} // end class LyParameterDecl
