/*
 * LyWsFormula.java
 * Copyright (c) 2000-2011, W4.
 * All rights reserved.
 */

package leon.info.misc;

import leon.app.LyApplication;
import leon.data.LyObject;
import leon.data.LyValueSet;
import leon.info.LyChoiceFieldInfo;
import leon.info.LyFieldInfo;
import leon.info.LyInfo;
import leon.misc.LyEnvironment;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;


/**
 *
 * Class used to compute a formula on a LyFieldInfo using an external Web service.
 * @author cradafy
 *
 */
public class LyWSFormula extends LyFormula
{
    public static final Object CACHE_LOCK = new Object();
    public static String       _EXPECTED_SYNTAX =
        "webservice://<service_name>.<op_name>(<attributeId1>=<id_field1>,<attributeId2>=<id_field1>, ...)[attributeOnResponse]";
    public static String       _NO_INSTANCE_COMPLEX = "NO_INSTANCE_COMPLEX";
    public static String       _STATIC_VALUE = "STATIC_VALUE";
    public static String       _WS_PREFIX = "webservice://";
    public static String       _WS_OPERATION_RETURN = "get_return";
    protected long             CACHE_EXPIRY = 15000; // in ms , should be a parameter
    protected Map<String, LyWSCallInfo> _compiledExpression;
    protected Map<String, Object>       _valuesCache;
    protected Map<String, Long>         _valuesCacheTime;

    /**
     * Default constructor. Invoked by newInstance when checking expressions
     * from studio.
     */
    protected LyWSFormula()
    {
    }

    /**
     * Constructs a formula for a given info.
     */
    public LyWSFormula(LyInfo info, String expression)
    {
        super(info, expression);
    }

    /**
     * compute input attribute to input parameters
     * @param application
     * @param inputAttribute
     * @param wsClassInfo
     * @return
     */
    protected LyWSCallInfo computeInputAttributes(LyApplication application, String inputAttribute,
        LyWSCallInfo wsClassInfo)
    {
        Map<String, Map<String, Object>> listAttributes = null;
        Map<String, Object>              listFields = null;

        if (inputAttribute.length() > 0)
        {
            boolean isNewInstanceClass = isNewInstanceClass(inputAttribute);

            if (isNewInstanceClass)
            {
                int n5 = inputAttribute.indexOf("=");
                int n6 = inputAttribute.indexOf("{");

                //token
                String attributeId = inputAttribute.substring(0, n5);
                String typeComplexId = inputAttribute.substring(n5 + 1, n6);

                inputAttribute = inputAttribute.substring(n6);

                List<String> listInstanceClsParam = listInputParameters(inputAttribute.substring(
                            1));

                for (String token1 : listInstanceClsParam)
                {
                    int n3 = token1.indexOf("$");
                    int n4 = 0;

                    if (token1.endsWith("}}"))
                        n4 = token1.lastIndexOf("}}");
                    else
                        n4 = token1.lastIndexOf("}");

                    if (n3 > 0)
                    {
                        LyInfo fieldInfo = application.getInfo(token1.substring(n3 + 2, n4).trim());

                        if (fieldInfo instanceof LyFieldInfo)
                        {
                            String attributeTypeId = token1.substring(0, n3 - 1).trim();

                            if (listFields == null)
                                listFields = new HashMap<String, Object>();

                            listFields.put(typeComplexId + "{" + attributeTypeId,
                                (LyFieldInfo) fieldInfo);

                        }
                        else
                            throw new IllegalArgumentException(
                                "BF AppEngine : syntax of expression is not valid expecting : "
                                + _EXPECTED_SYNTAX);
                    }
                    else
                        throw new IllegalArgumentException(
                            "BF AppEngine : syntax of expression is not valid expecting : "
                            + _EXPECTED_SYNTAX);
                }

                if (listAttributes == null)
                    listAttributes = new HashMap<String, Map<String, Object>>();

                listAttributes.put(attributeId + "{" + typeComplexId, listFields);

            }
            else
            {
                int n3 = inputAttribute.indexOf("$");
                int n4 = 0;

                if (inputAttribute.endsWith("}}"))
                    n4 = inputAttribute.lastIndexOf("}}");
                else
                    n4 = inputAttribute.lastIndexOf("}");

                String attributeId = null;

                if (n3 > 0)
                {
                    LyInfo fieldInfo = application.getInfo(inputAttribute.substring(n3 + 2, n4)
                            .trim());

                    if (fieldInfo instanceof LyFieldInfo)
                    {

                        if (listFields == null)
                            listFields = new HashMap<String, Object>();

                        attributeId = inputAttribute.substring(0, n3 - 1).trim();
                        listFields.put(_NO_INSTANCE_COMPLEX, (LyFieldInfo) fieldInfo);
                    }
                    else
                        throw new IllegalArgumentException(
                            "BF AppEngine : syntax of expression is not valid expecting : "
                            + _EXPECTED_SYNTAX);
                }
                else
                {
                	// try to handle static attribute
                	n3 = inputAttribute.indexOf("=");
                	if (n3 < 0)
                		throw new IllegalArgumentException(
                				"BF AppEngine : syntax of expression is not valid expecting : "
                				+ _EXPECTED_SYNTAX);

                	String value = inputAttribute.substring(n3 +1);
                	if (listFields == null)
                        listFields = new HashMap<String, Object>();
                	attributeId = inputAttribute.substring(0, n3);
                	listFields.put(_STATIC_VALUE, value);
                }

                if (listAttributes == null)
                    listAttributes = new HashMap<String, Map<String, Object>>();

                listAttributes.put(attributeId, listFields);
            }
        }


        if (listAttributes != null)
        {

            if (wsClassInfo == null)
                wsClassInfo = new LyWSCallInfo();

            if ((wsClassInfo.getInputOperationAttributes() != null)
                    && (wsClassInfo.getInputOperationAttributes().size() > 0))
                listAttributes.putAll(wsClassInfo.getInputOperationAttributes());

            wsClassInfo.setInputOperationAttributes(listAttributes);
        }

        return wsClassInfo;
    }

    /**
     * This method construct a object LyWSClassInfo that contains the
     * stubObject; the operation; the attributes and the value of each attribute
     *
     * @param expression
     * @param fieldNames
     * @return an LyWSCallInfo
     */
    protected LyWSCallInfo compileExpression(String expression, List<String> fieldNames)
    {
        LyApplication application = _info.getApplication();
        if (application.isEdited())
            return null;
        LyWSCallInfo wsClassInfo = null;

        // Create an expression object
        if ((expression != null) && expression.startsWith(_WS_PREFIX))
        {

            if (wsClassInfo == null)
                wsClassInfo = new LyWSCallInfo();

            // parse service
            wsClassInfo = parseServiceWs(expression);

            // parse operation
            wsClassInfo = parseOperationWs(expression, wsClassInfo);

            // input attribute operation
            String fieldInfoIdsStr = getInputExpressionWs(expression);

            // input attribute operation
            List<String> inputParams = listInputParameters(fieldInfoIdsStr);

            for (String token : inputParams)
                // compute input attributes
                wsClassInfo = computeInputAttributes(application, token, wsClassInfo);

            // return attribute
            wsClassInfo = getReturedAttributeName(expression, wsClassInfo);
        }

        if (_compiledExpression == null)
            _compiledExpression = new HashMap<String, LyWSCallInfo>();

        _compiledExpression.put(getInfo().getId(), wsClassInfo);

        return wsClassInfo;
    }

    /**
     * compute the type object value
     * @param value
     * @return
     */
    protected Object computeTypeValue(LyEnvironment environment, Object stubObj, String attributeId,
        String typeComplex, Object value, boolean isStaticClass)
    {
        Object obj = null;

        if (!isStaticClass)
        {
            String classTypeName = stubObj.getClass().getName() + "$" + upperCaseFirst(typeComplex);
            Object objParameterClass = getObjectClass(classTypeName);

            if (objParameterClass == null)
                return obj;

            String              methodName = "set" + upperCaseFirst(attributeId);
            Map<Object, Method> hashM = getMethodByName(objParameterClass.getClass(), methodName, value);

            boolean useStringClass = false;

            if (hashM == null)
            {
                hashM = getMethodByName(objParameterClass.getClass(), methodName, String.class);
                useStringClass = true;
            }

            if (hashM == null)
                environment.logError("LyWSFormula Error : Operation method '" + methodName
                    + "' was not found for class '"
                    + objParameterClass.getClass().getCanonicalName() + "'");
            else
            {
                Iterator<Object> opValueIter = hashM.keySet().iterator();

                while (opValueIter.hasNext())
                {
                    Object opValue = opValueIter.next();

                    if (opValue instanceof java.lang.String)
                    {
                        if (useStringClass && (opValue != null))
                            opValue = opValue.toString();
                    }

                    invokeOperationMethod(hashM.get(opValue), objParameterClass, opValue);
                }
            }

            obj = objParameterClass;
            //TO DO
            // set parameter value
        }
        else
        {
            // intern class static
            String		classTypeName = stubObj.getClass().getName() + "$" + upperCaseFirst(typeComplex);
            Class<?>	clsParameter = getClassByName(classTypeName);

            if (clsParameter == null)
                return obj;

            Field field = null;

            try
            {
                field = clsParameter.getDeclaredField(value.toString());
            }
            catch (SecurityException ex)
            {
                environment.logError("An error for security field in class\n" + ex.getMessage()
                    + "\nClassName=" + clsParameter.getName());
            }
            catch (NoSuchFieldException ex)
            {

                environment.logError("An error of field not available in class\n" + ex.getMessage()
                    + "\nClassName=" + clsParameter.getName());
            }

            if (field == null)
                return obj;


            // get the object in the field
            try
            {
                obj = field.get(value);
            }
            catch (IllegalArgumentException ex)
            {
                environment.logError("An error of argument exception in the field on the class\n"
                    + ex.getMessage() + "\nClassName=" + clsParameter.getName());

            }
            catch (IllegalAccessException ex)
            {
                environment.logError("An error of illegal exception access in the class\n"
                    + ex.getMessage() + "\nClassName=" + clsParameter.getName());
            }
        }

        return obj;
    }

    /**
     * if it's a new instance class
     * @param s
     * @return
     */
    protected boolean isNewInstanceClass(String s)
    {

        if ((s != null) && (s.length() > 0))
        {
            LyBlock block = new LyBlock();

            for (char c = 0; c < s.length(); c++)
            {
                char ch = s.charAt(c);

                if (isCharBlockContains(block.begin, ch))
                    block.depth++;
            }

            if (block.depth > 1)
                return true;
        }

        return false;
    }

    /**
     * invoke operation method by attribute id
     * @param environment
     * @param operationObj
     * @param value
     * @param attributeId
     */
    protected void invokeOperationMethod(LyEnvironment environment, Object operationObj,
        Object value, String attributeId)
    {
        String              methodName = "set" + upperCaseFirst(attributeId);
        Map<Object, Method> hashM = getMethodByName(operationObj.getClass(), methodName, value);
        boolean             useStringClass = false;

        if (hashM == null)
        {
            hashM = getMethodByName(operationObj.getClass(), methodName, String.class);
            useStringClass = true;
        }

        if (hashM == null)
            environment.logError("LyWSFormula Error : Operation method '" + methodName
                + "' was not found for class '" + operationObj.getClass().getCanonicalName() + "'");
        else
        {
            Iterator<Object> opValueIter = hashM.keySet().iterator();

            while (opValueIter.hasNext())
            {
                Object opValue = opValueIter.next();

                if (opValue instanceof java.lang.String)
                {

                    if (useStringClass && (opValue != null))
                        opValue = opValue.toString();
                }

                invokeOperationMethod(hashM.get(opValue), operationObj, opValue);
            }
        }
    }

    /**
     * true if return value is array
     * @param expression
     * @return
     */
    protected boolean isReturnValueArray(String expression)
    {
        boolean test = false;

        int n3 = expression.indexOf("[");
        int n4 = expression.indexOf("]");

        if ((n3 > 0) && (n4 > 0))
            test = true;

        return test;
    }

    /**
     * if the expression is a simple type
     * @param value
     * @return
     */
    protected boolean isSimpleType(String value)
    {
        boolean test = false;
        int     n = value.indexOf("="); // complex type static
        int     n1 = value.indexOf("{"); // complex type new instance

        if (n < 0)
        {
            test = true;

            if (n1 < 0)
                test = true;
            else
                test = false;
        }
        else
            test = false;

        return test;
    }

    /**
     * if the complex type is a static class
     * @param value
     * @return
     */
    protected boolean isComplexTypeStaticClass(String attributeId)
    {
        boolean test = false;

        if ((attributeId != null) && (attributeId.length() > 0))
        {
            int n = attributeId.indexOf("=");

            if (n > 0)
                return true;

        }

        return test;
    }

    /**
     * test if c contains in ch
     * @param ch
     * @param c
     * @return
     */
    protected boolean isCharBlockContains(char[] ch, char c)
    {

        if ((ch != null) && (ch.length > 0))
        {

            for (int i = 0; i < ch.length; i++)
            {

                if (c == ch[i])
                    return true;
            }
        }

        return false;
    }

    /**
     * invoke method by operation object method has not an object return
     *
     * @param m
     * @param operation
     * @param value
     */
    protected void invokeOperationMethod(Method m, Object operation, Object value)
    {

        try
        {
            m.invoke(operation, value);
        }
        catch (IllegalArgumentException e)
        {
            LyEnvironment environment = getInfo().getEnvironment();
            environment.logError("An error for illegal argument expression object class \n"
                + e.getMessage());
        }
        catch (IllegalAccessException e)
        {
            LyEnvironment environment = getInfo().getEnvironment();
            environment.logError("An error for access object class \n" + e.getMessage());
        }
        catch (InvocationTargetException e)
        {
            LyEnvironment environment = getInfo().getEnvironment();
            environment.logError("An error for invoke target object class \n" + e.getMessage());
        }

        return;
    }

    /**
     * get object response operation
     * @param environment
     * @param stubObj
     * @param operationObj
     * @param wsCallInfo
     * @return
     */
    protected Object getCallOperationResponse(LyEnvironment environment, Object stubObj,
        Object operationObj, LyWSCallInfo wsCallInfo)
    {
        Object returnedValue = null;
        Object objResp = getObjectClass(operationObj.getClass().getName() + "Response");

        String operationName = lowerCaseFirst(wsCallInfo.getOperationName());
        // call method return object
        Map<Object, Method> hashm1 = getMethodByName(stubObj.getClass(), operationName,
                operationObj);

        Iterator<Object> opValueIter1 = hashm1.keySet().iterator();
        Object           deliveryResp = null;

        while (opValueIter1.hasNext())
        {
            Object opValue = opValueIter1.next();
            deliveryResp = getInvokeMethodReturn(hashm1.get(opValue), stubObj, operationObj);
        }

        Method m2 = null;

        try
        {

            try
            {
                m2 = objResp.getClass().getMethod(_WS_OPERATION_RETURN);
            }
            catch (NoSuchMethodException e)
            {
            }

            if (m2 == null)
            {
                String methodResultName = "get" + wsCallInfo.getOperationName() + "Result";

                try
                {
                    m2 = objResp.getClass().getMethod(methodResultName);
                }
                catch (NoSuchMethodException e)
                {
                    environment.logError(
                        "LyWSFormula could nor find method to fetch returned value : class="
                        + objResp.getClass().getCanonicalName() + " methods are "
                        + _WS_OPERATION_RETURN + " or " + methodResultName + "\n" + e.getMessage());
                }
            }

            if (m2 != null)
                returnedValue = m2.invoke(deliveryResp);
        }
        catch (SecurityException e)
        {
            environment.logError("An error for security object class " + e.getMessage());
        }
        catch (IllegalArgumentException e)
        {
            environment.logError("An error for illegal argument expression object class \n"
                + e.getMessage());
        }
        catch (IllegalAccessException e)
        {
            environment.logError("An error for access object class \n" + e.getMessage());
        }
        catch (InvocationTargetException e)
        {
            environment.logError("An error for invoke target object class \n" + e.getMessage());
        }

        return returnedValue;

    }


    /**
     * get the object by attribute response
     * @param environment
     * @param returnedValue
     * @param wsCallInfo
     * @return
     */
    protected Object getReturnedAttributeResponse(LyEnvironment environment, Object returnedValue,
        LyWSCallInfo wsCallInfo)
    {

        if ((returnedValue != null) && (wsCallInfo.getReturnedAttributeName() != null)
                && (wsCallInfo.getReturnedAttributeName().length() > 0))
        {

            try
            {
                String returnedAttributeName = wsCallInfo.getReturnedAttributeName();

                if (!isReturnValueArray(returnedAttributeName))
                {
                    String methodName = "get"
                        + upperCaseFirst(wsCallInfo.getReturnedAttributeName());
                    Method m3 = returnedValue.getClass().getMethod(methodName);

                    if (m3 != null)
                        returnedValue = m3.invoke(returnedValue);
                }
                else
                {
                    StringTokenizer st2 = new StringTokenizer(returnedAttributeName, ".");

                    while (st2.hasMoreTokens())
                    {
                        String token2 = st2.nextToken().trim();

                        int n = token2.indexOf("[");
                        int n1 = token2.lastIndexOf("]");
                        int nbObject = -1;

                        String returnAttribute2 = "";
                        String methodName = "";

                        if (n > 0)
                        {
                            nbObject = Integer.parseInt(token2.substring(n + 1, n1));
                            returnAttribute2 = token2.substring(0, n);

                            // get array result
                            methodName = "get" + upperCaseFirst(returnAttribute2) + "Result";

                            Method m3 = returnedValue.getClass().getMethod(methodName);

                            if (m3 != null)
                                returnedValue = m3.invoke(returnedValue);

                            // get array[] object
                            methodName = "get" + upperCaseFirst(returnAttribute2);
                        }
                        else
                            methodName = "get" + upperCaseFirst(token2);

                        Method m4 = returnedValue.getClass().getMethod(methodName);

                        if (m4 != null)
                        {

                            if (m4 != null)
                            {
                                returnedValue = m4.invoke(returnedValue);

                                if (nbObject > -1)
                                {

                                    if (returnedValue instanceof Object[])
                                    {
                                        Object[] returnedValues = (Object[]) returnedValue;
                                        returnedValue = returnedValues[nbObject];
                                    }
                                }
                            }
                        }
                        else
                            throw new IllegalArgumentException(
                                "BF AppEngine : An error for method object class \n : " + methodName
                                + " of " + returnedValue.toString());
                    }
                }
            }
            catch (Exception e)
            {
                environment.logWarning("LyWSFormula : " + e.getMessage());
            }
        }

        return returnedValue;

    }

    /**
     * Evaluate the formula Return depends on the fieldInto type All values are
     * contained in the LyObject
     *
     * @return an Object
     */
    protected Object getValue(LyObject instance, LyValueSet values)
    {
        Object returnedValue = null;

        if (_compiledExpression != null)
        {
            LyWSCallInfo wsCallInfo = _compiledExpression.get(getInfo().getId());

            Map<String, Map<String, Object>> listInputOperationAttributes = wsCallInfo
                .getInputOperationAttributes();

            // try from cache
            StringBuffer KEY_CACHE = new StringBuffer();

            if (listInputOperationAttributes != null)
            {

                for (String attributeId : listInputOperationAttributes.keySet())
                {
                    Map<String, Object> listAttributes = listInputOperationAttributes.get(
                            attributeId);

                    if (listAttributes != null)
                    {

                        for (String clsInstanceId : listAttributes.keySet())
                        {

                        	Object fieldVal = listAttributes.get(clsInstanceId);
                        	Object      value;

                        	if (fieldVal instanceof LyFieldInfo)
                        	{
                        	LyFieldInfo fi = (LyFieldInfo)fieldVal;


                            if ((values != null) && values.contains(fi))
                                value = values.getFieldValue(fi).getValue();
                            else if (instance != null)
                                value = instance.getValue(fi);
                            else
                                return null; // Impossible to compute
                        	}
                        	else
                        		value = fieldVal;

                        	if (KEY_CACHE.length() > 0)
                                KEY_CACHE.append('');

                            KEY_CACHE.append((value == null) ? "null" : value);
                        }
                    }
                }

                if (listInputOperationAttributes.size() == 0)
                    KEY_CACHE.append("No_Arguments");
            }
            else
                KEY_CACHE.append("No_Arguments");

            returnedValue = getFromCache(KEY_CACHE.toString());

            // get value if in cache
            if (returnedValue != null)
                return returnedValue;

            LyEnvironment environment = getInfo().getEnvironment();
            Object        stubObj = wsCallInfo.getStubObj();
            Object        operationObj = wsCallInfo.getOpObject();

            if (listInputOperationAttributes != null)
            {

                for (String attributeId : listInputOperationAttributes.keySet())
                {
                    Map<String, Object> listAttributes = listInputOperationAttributes.get(
                            attributeId);

                    if (listAttributes != null)
                    {

                        // mode not static
                        if (listAttributes.keySet().size() > 1)
                        {
                            Object value;
                            int    n = attributeId.lastIndexOf("{");
                            value = getInstanceValueCls(environment, stubObj, listAttributes,
                                    instance, values);
                            invokeOperationMethod(environment, operationObj, value,
                                attributeId.substring(0, n));
                        }
                        else
                            // mode static or simple
                            for (String clsInstanceId : listAttributes.keySet())
                            {
                            	Object      value;
                            	Object      fieldVal = listAttributes.get(clsInstanceId);
                            	if (fieldVal instanceof LyFieldInfo)
                            	{
                            		LyFieldInfo fi = (LyFieldInfo)fieldVal;

	                                if ((values != null) && values.contains(fi))
	                                    value = values.getFieldValue(fi).getValue();
	                                else if ((instance != null) && (fi instanceof LyChoiceFieldInfo))
	                                {
	                                    value = instance.getValue(fi);

	                                    if (value instanceof Short)
	                                        value = ((LyChoiceFieldInfo) fi).getChoiceItem(Short
	                                                .parseShort(value.toString())).getValue();
	                                }
	                                else if (instance != null)
	                                    value = instance.getValue(fi);
	                                else
	                                    return null; // Impossible to compute
                            	}
                            	else
                            		value = fieldVal;

                                boolean simpleType = isSimpleType(attributeId);

                                if (!simpleType)
                                {
                                    boolean complexTypeStaticCls = isComplexTypeStaticClass(
                                            attributeId);

                                    // static class
                                    if (complexTypeStaticCls)
                                    {
                                        int    nComplexType = attributeId.lastIndexOf("=");
                                        String complexStaticType = attributeId.substring(
                                                nComplexType + 1);
                                        attributeId = attributeId.substring(0, nComplexType);

                                        // compute type object value
                                        value = computeTypeValue(environment, stubObj, attributeId,
                                                complexStaticType, value, true);
                                    }
                                }

                                invokeOperationMethod(environment, operationObj, value,
                                    attributeId);
                            }
                    }
                }
            }

            // call return operation response
            returnedValue = getCallOperationResponse(environment, stubObj, operationObj,
                    wsCallInfo);

            // get the attribute response
            returnedValue = getReturnedAttributeResponse(environment, returnedValue, wsCallInfo);

            if (returnedValue != null)
            {
                LyFieldInfo fieldInfo = (LyFieldInfo) getInfo();
                returnedValue = fieldInfo.buildFieldValue(returnedValue).getValue();
            }

            if (returnedValue != null)
                putCache(KEY_CACHE.toString(), returnedValue);

        }

        return returnedValue;

    }

    /**
     * get a return attribute name
     * @param s
     * @param wsClassInfo
     * @return
     */
    protected LyWSCallInfo getReturedAttributeName(String s, LyWSCallInfo wsClassInfo)
    {

        if (wsClassInfo == null)
            wsClassInfo = new LyWSCallInfo();

        int n3 = s.indexOf("[", 3);
        int n4 = s.lastIndexOf("]");

        if ((n3 > 0) && (n4 > n3))
        {
            String returnedAttributeName = s.substring(n3 + 1, n4).trim();
            wsClassInfo.setReturnedAttributeName(returnedAttributeName);
        }

        return wsClassInfo;
    }

    /**
     * get the input attribute expression
     * @param s
     * @return
     */
    protected String getInputExpressionWs(String s)
    {
        int n1 = s.indexOf("(");

        if (n1 > 0)
        {
            int n2 = s.indexOf(")", n1);

            if (n2 > 0)
                return s.substring(n1 + 1, n2).trim();
            else
                throw new IllegalArgumentException(
                    "BF AppEngine : syntax of expression is not valid expecting : "
                    + _EXPECTED_SYNTAX);
        }
        else
            throw new IllegalArgumentException(
                "BF AppEngine : syntax of expression is not valid expecting : " + _EXPECTED_SYNTAX);
    }

    /**
     * Evaluate the formula Return depends on the fieldInto type All values are
     * contained in the LyObject
     *
     * @return an Object
     */
    public Object getValue(LyObject instance)
    {
        return getValue(instance, null);
    }

    /**
     * Evaluate the formula Return depends on the fieldInto type The value set
     * must contains all the values necessary to compute the formula
     */
    public Object getValue(LyValueSet values)
    {
        return getValue((LyObject) null, values);
    } // end method getValue

    /**
     * This method permit to get an object service stub by service Name
     *
     * @param stubClass
     * @return an Object
     */
    public Object getServiceStubObject(String serviceName)
    {
        Object        stub = null;
        Class<?>      stubClass = null;
        LyEnvironment environment = getInfo().getEnvironment();

        try
        {
            stubClass = Class.forName(serviceName + "Stub");
        }
        catch (ClassNotFoundException ex)
        {
            environment.logError("LyWSFormula : An error for retreiving Web Service stub class \n"
                + ex.getMessage() + "\nServiceName=" + serviceName + "\nStub class name : "
                + (serviceName + "Stub"));
        }

        if (stubClass != null)
        {

            try
            {
                stub = stubClass.newInstance();
            }
            catch (InstantiationException e)
            {
                environment.logError("An error for instantiate stub object from class "
                    + stubClass.getName() + "\n" + e.getMessage());
            }
            catch (IllegalAccessException e)
            {
                environment.logError("An error for instantiate object from stub class \n"
                    + e.getMessage());
            }
        }

        return stub;

    }

    /**
     * get class define by className
     * @param className
     * @return
     */
    public Class<?> getClassByName(String className)
    {
        Class<?>      cls = null;
        LyEnvironment environment = getInfo().getEnvironment();

        // create a class by className
        try
        {
            cls = Class.forName(className);
        }
        catch (ClassNotFoundException ex)
        {
            // TODO Auto-generated catch block
            environment.logError("An error for retreiving class name \n" + ex.getMessage()
                + "\nClassName=" + className);
        }

        return cls;

    }

    /**
     * This method can do get the object by your className
     *
     * @param cls
     * @return an Object
     */
    public Object getObjectClass(String className)
    {
        Object        obj = null;
        Class<?>      cls = null;
        LyEnvironment environment = getInfo().getEnvironment();

        // create a class by className
        try
        {
            cls = Class.forName(className);
        }
        catch (ClassNotFoundException ex)
        {
            // TODO Auto-generated catch block
            environment.logError("An error for retreiving class name \n" + ex.getMessage()
                + "\nClassName=" + className);
        }

        // create a instance object class
        if (cls != null)
        {

            try
            {
                obj = cls.newInstance();
            }
            catch (InstantiationException e)
            {
                environment.logError("An error for instantiate class name object from class "
                    + cls.getName() + "\n" + e.getMessage());
            }
            catch (IllegalAccessException e)
            {
                environment.logError("An error for instantiate object from class name \n"
                    + e.getMessage());
            }
        }

        return obj;

    }

    /**
     * compute type not compatible
     * @param cls
     * @param name
     * @param value
     * @return
     */
    protected Map<Object, Method> getArrayType(Class<?> cls, String name, Object value)
    {
        Map<Object, Method> hashMethod = null;
        Method              m = null;
        List<Class<?>>      list = new ArrayList<Class<?>>();

        // primitif type
        list.add(double.class);
        list.add(int.class);
        list.add(float.class);
        // object type primitif
        list.add(getClassByName("java.math.BigDecimal"));
        list.add(getClassByName("java.math.BigInteger"));
        list.add(getClassByName("java.lang.Double"));
        list.add(getClassByName("java.lang.Float"));
        list.add(getClassByName("java.lang.String"));

        boolean validType = false;

        for (int i = 0; (i < list.size()) && (!validType); i++)
        {
            try
            {
                validType = true;

                Class<?>	 classes = list.get(i);

                m = cls.getMethod(name, classes);

                if (hashMethod == null)
                    hashMethod = new HashMap<Object, Method>();

                if ("double".equals(classes.getName()))
                    hashMethod.put(Double.valueOf(value.toString()), m);
                else if ("int".equals(classes.getName()))
                    hashMethod.put(Integer.valueOf(value.toString()), m);
                else if ("float".equals(classes.getName()))
                    hashMethod.put(Float.valueOf(value.toString()), m);
                else if ("java.math.BigDecimal".equals(classes.getName()))
                    hashMethod.put(new java.math.BigDecimal(value.toString()), m);
                else if ("java.math.BigInteger".equals(classes.getName()))
                    hashMethod.put(new java.math.BigInteger(value.toString()), m);
                else if ("java.lang.Double".equals(classes.getName()))
                    hashMethod.put(new java.lang.Double(value.toString()), m);
                else if ("java.lang.Float".equals(classes.getName()))
                    hashMethod.put(new java.lang.Float(value.toString()), m);
                else if ("java.lang.String".equals(classes.getName()))
                    hashMethod.put(new java.lang.String(value.toString()), m);
            }
            catch (SecurityException e)
            {
                LyEnvironment environment = getInfo().getEnvironment();
                environment.logError("An error for security object class " + e.getMessage());
            }
            catch (NoSuchMethodException e)
            {
                validType = false;
            }
        }

        if (m == null)
            throw new IllegalArgumentException(
                "BF AppEngine : An error for method object class \n : " + value.toString());

        return hashMethod;
    }

    /**
     * This method can do to get a method by operation name in class
     *
     * @param cls
     * @param name
     * @param valueClass
     * @return
     */
    public Map<Object, Method> getMethodByName(Class<?> cls, String name, Object value)
    {
        Map<Object, Method> hashMethod = null;
        Method              m = null;

        try
        {
            m = cls.getMethod(name, value.getClass());

            if (hashMethod == null)
                hashMethod = new HashMap<Object, Method>();

            hashMethod.put(value, m);
        }
        catch (SecurityException e)
        {
            LyEnvironment environment = getInfo().getEnvironment();
            environment.logError("An error for security object class " + e.getMessage());
        }
        catch (NoSuchMethodException e)
        {
            hashMethod = getArrayType(cls, name, value);
        }

        return hashMethod;
    }

    /**
     * get the value object
     * if the class is new instance
     * @param environment
     * @param stubObj
     * @param listAttributes
     * @param instance
     * @param values
     * @return
     */
    protected Object getInstanceValueCls(LyEnvironment environment, Object stubObj,
        Map<String, Object> listAttributes, LyObject instance, LyValueSet values)
    {
        Object obj = null;

        Object objParameterClass = null;

        for (String clsInstanceId1 : listAttributes.keySet())
        {
        	Object      value;
        	Object      fieldVal = listAttributes.get(clsInstanceId1);
        	if (fieldVal instanceof LyFieldInfo)
            {
        		LyFieldInfo fi = (LyFieldInfo)fieldVal;

	            if ((values != null) && values.contains(fi))
	                value = values.getFieldValue(fi).getValue();
	            else if ((instance != null) && (fi instanceof LyChoiceFieldInfo))
	            {
	                value = instance.getValue(fi);

	                if (value instanceof Short)
	                    value = ((LyChoiceFieldInfo) fi).getChoiceItem(Short.parseShort(
	                                value.toString())).getValue();

	                //value = ((LyChoiceFieldInfo)fi).getChoiceItem(value.toString());
	            }
	            else if (instance != null)
	                value = instance.getValue(fi);
	            else
	                return null; // Impossible to compute
            }
        	else
        		value = fieldVal;

            int    n = clsInstanceId1.lastIndexOf("{");
            String attributeComplexType = clsInstanceId1.substring(n + 1);
            String complexType = clsInstanceId1.substring(0, n);

            String classTypeName = stubObj.getClass().getName() + "$" + upperCaseFirst(complexType);

            if (objParameterClass == null)
                objParameterClass = getObjectClass(classTypeName);

            if (objParameterClass == null)
                return obj;

            String              methodName = "set" + upperCaseFirst(attributeComplexType);
            Map<Object, Method> hashM = getMethodByName(objParameterClass.getClass(), methodName, value);

            boolean useStringClass = false;

            if (hashM == null)
            {
                hashM = getMethodByName(objParameterClass.getClass(), methodName, String.class);
                useStringClass = true;
            }

            if (hashM == null)
                environment.logError("LyWSFormula Error : Operation method '" + methodName
                    + "' was not found for class '"
                    + objParameterClass.getClass().getCanonicalName() + "'");
            else
            {
                Iterator<Object> opValueIter = hashM.keySet().iterator();

                while (opValueIter.hasNext())
                {
                    Object opValue = opValueIter.next();

                    if (opValue instanceof java.lang.String)
                    {

                        if (useStringClass && (opValue != null))
                            opValue = opValue.toString();
                    }

                    invokeOperationMethod(hashM.get(opValue), objParameterClass, opValue);
                }
            }

            obj = objParameterClass;
        }

        return obj;
    }

    /**
     * invoke method by operation object method has'nt an object return
     *
     * @param m
     * @param operation
     * @param value
     * @return
     */
    public Object getInvokeMethodReturn(Method m, Object operation, Object value)
    {
        Object obj = null;

        try
        {
            obj = m.invoke(operation, value);
        }
        catch (IllegalArgumentException e)
        {
            LyEnvironment environment = getInfo().getEnvironment();
            environment.logError("An error for illegal argument expression object class \n"
                + operation.getClass().getName() + " value object class: " + value.getClass() + "\n"
                + e.getMessage());
        }
        catch (IllegalAccessException e)
        {
            LyEnvironment environment = getInfo().getEnvironment();
            environment.logError("An error for access object class \n"
                + operation.getClass().getName() + " value object class: " + value.getClass() + "\n"
                + e.getMessage());
        }
        catch (InvocationTargetException e)
        {
            LyEnvironment environment = getInfo().getEnvironment();
            environment.logError("An error for invoke target object class \n"
                + operation.getClass().getName() + " value object class: " + value.getClass() + "\n"
                + e.getMessage());
        }

        return obj;
    }

    /**
     * get value in cache object
     * @param keyCache
     * @return
     */
    public Object getFromCache(String keyCache)
    {

        synchronized (CACHE_LOCK)
        {
            Object returnedValue = null;

            if ((_valuesCache != null) && (_valuesCacheTime != null) && (keyCache != null))
            {
                returnedValue = _valuesCache.get(keyCache);

                if (returnedValue != null)
                {
                    // check if obsolete
                    Long time = _valuesCacheTime.get(keyCache);

                    if ((time == null) || ((System.currentTimeMillis() - time) > CACHE_EXPIRY))
                    {
                        returnedValue = null;
                        _valuesCache.remove(keyCache);
                        _valuesCacheTime.remove(keyCache);
                    }
                }
            }

            return returnedValue;
        }
    }

    /**
     * list of the parameter
     * @param s
     * @return
     */
    protected List<String> listInputParameters(String s)
    {
        List<String> listInputParams = null;

        LyBlock block = new LyBlock();

        while (s.length() > 0)
        {
            StringBuffer sb = split(s, ',', block);

            if (listInputParams == null)
                listInputParams = new ArrayList<String>();

            if (sb.toString().endsWith(","))
                listInputParams.add(sb.toString().substring(0, sb.toString().length() - 1));
            else
                listInputParams.add(sb.toString());

            s = s.substring(sb.toString().length());
        }

        return listInputParams;
    }

    /**
     * compute a String to lower case the first begin character
     *
     * @param paramStr
     * @return a String
     */
    protected String lowerCaseFirst(String paramStr)
    {
        String resultParam = paramStr;

        if ((paramStr != null) && (paramStr.length() > 0))
            resultParam = paramStr.substring(0, 1).toLowerCase() + paramStr.substring(1);

        return resultParam;

    }

    /**
     * parse the service name
     * @param s
     * @return
     */
    protected LyWSCallInfo parseServiceWs(String s)
    {
        LyWSCallInfo wsClassInfo = null;

        if ((s != null) && s.startsWith(_WS_PREFIX))
        {
            s = s.substring(_WS_PREFIX.length());

            // create service name ws
            int n = s.indexOf("(");

            if (n < 0)
                throw new IllegalArgumentException(
                    "BF AppEngine : syntax of expression is not valid expecting : "
                    + _EXPECTED_SYNTAX + " for fieldInfo : " + getInfo().getId());

            String s1 = s.substring(0, n);
            int    n1 = s1.lastIndexOf(".");

            if (n1 < 0)
                throw new IllegalArgumentException(
                    "BF AppEngine : syntax of expression is not valid expecting : "
                    + _EXPECTED_SYNTAX + " for fieldInfo : " + getInfo().getId());

            if (wsClassInfo == null)
                wsClassInfo = new LyWSCallInfo();

            wsClassInfo.setServiceName(s1.substring(0, n1));

            // create a stub object
            Object objStub = getServiceStubObject(wsClassInfo.getServiceName());

            if (objStub == null)
                throw new IllegalArgumentException(
                    "BF AppEngine : Can not load Stub class for service name "
                    + wsClassInfo.getServiceName() + "(" + wsClassInfo.getServiceName()
                    + "Stub) check formula or CLASSPATH for fieldInfo : " + getInfo().getId());

            // set the stub object
            wsClassInfo.setStubObj(objStub);

        }
        else
            throw new IllegalArgumentException(
                "BF AppEngine : syntax of expression is not valid expecting : " + _EXPECTED_SYNTAX
                + " for fieldInfo : " + getInfo().getId());

        return wsClassInfo;
    }

    /**
     * parse operation by the current ws
     * @param s
     * @param wsClassInfo
     * @param n
     * @return
     */
    protected LyWSCallInfo parseOperationWs(String s, LyWSCallInfo wsClassInfo)
    {
        int n = s.indexOf("(");

        if (wsClassInfo == null)
            wsClassInfo = new LyWSCallInfo();

        // create a operation name
        String operationName = s.substring(_WS_PREFIX.length()
                + wsClassInfo.getServiceName().length() + 1, n);

        wsClassInfo.setOperationName(operationName);

        // object operation
        if (wsClassInfo.getStubObj() == null)
            throw new NullPointerException("BF AppEngine : no service object defined : "
                + " for serviceName : " + wsClassInfo.getServiceName());

        Object objOperation = getObjectClass(wsClassInfo.getStubObj().getClass().getName() + "$"
                + upperCaseFirst(operationName));

        wsClassInfo.setOpObject(objOperation);

        return wsClassInfo;
    }


    /**
     * add value in cache
     * @param keyCache
     * @param returnedValue
     */
    public void putCache(String keyCache, Object returnedValue)
    {
        synchronized (CACHE_LOCK)
        {
            if ((keyCache != null) && (returnedValue != null))
            {
                if (_valuesCache == null)
                    _valuesCache = new HashMap<String, Object>();

                _valuesCache.put(keyCache, returnedValue);

                if (_valuesCacheTime == null)
                    _valuesCacheTime = new HashMap<String, Long>();

                _valuesCacheTime.put(keyCache, System.currentTimeMillis());
            }
        }
    }

    /**
     * remove value in cache
     * @param keyCache
     */
    public void removeFromCache(String keyCache)
    {
        synchronized (CACHE_LOCK)
        {
            if ((_valuesCache != null) && (keyCache != null))
                _valuesCache.remove(keyCache);

            if ((_valuesCacheTime != null) && (keyCache != null))
                _valuesCacheTime.remove(keyCache);
        }
    }

    /**
     * split a string syntax
     * @param s
     * @param charset
     * @param block
     * @return
     */
    protected StringBuffer split(String s, char charset, LyBlock block)
    {
        StringBuffer sb = new StringBuffer();

        for (char c = 0; c < s.length(); c++)
        {
            char ch = s.charAt(c);

            sb.append(ch);

            if (isCharBlockContains(block.begin, ch))
                block.depth++;

            if (isCharBlockContains(block.end, ch))
                block.depth--;

            if ((charset == ch) && (block.depth == 0))
                return sb;

        }

        return sb;
    }

    /**
     * compute a String to upper case the first begin character
     *
     * @param paramStr
     * @return
     */
    protected String upperCaseFirst(String paramStr)
    {
        String resultParam = paramStr;

        if ((paramStr != null) && (paramStr.length() > 0))
            resultParam = paramStr.substring(0, 1).toUpperCase() + paramStr.substring(1);

        return resultParam;
    }

    /**
     *
     * define the type charset useful expression
     * @author cradafy
     *
     */
    class LyBlock
    {
        char[] begin = { '{', '[', '(' };
        char[] end = { '}', ']', ')' };
        int    depth = 0;
    }


    /**
     * define the classInfo webservice that you have a stubObject; the operation
     * object and the other input attributes
     *
     * @author cradafy
     *
     */
    class LyWSCallInfo
    {
        String                                _serviceName;
        Object                                _stubObj;
        String                                _operationName;
        Object                                _opObject;
        Map<String, Map<String, Object>> _inputOperationAttributes;
        String 								  _returnedAttributeName;

        public void setServiceName(String serviceName)
        {
            this._serviceName = serviceName;
        }

        public void setStubObj(Object stubObj)
        {
            this._stubObj = stubObj;
        }

        public void setOpObject(Object opObject)
        {
            this._opObject = opObject;
        }

        public void setOperationName(String opName)
        {
            this._operationName = opName;
        }

        public void setInputOperationAttributes(
            Map<String, Map<String, Object>> inputOperationAttributes)
        {
            this._inputOperationAttributes = inputOperationAttributes;
        }

        public String getServiceName()
        {
            return _serviceName;
        }

        public Object getStubObj()
        {
            return _stubObj;
        }

        public Object getOpObject()
        {
            return _opObject;
        }

        public String getOperationName()
        {
            return _operationName;
        }

        public String getReturnedAttributeName()
        {
            return _returnedAttributeName;
        }

        public void setReturnedAttributeName(String attName)
        {
            this._returnedAttributeName = attName;
        }

        public Map<String, Map<String, Object>> getInputOperationAttributes()
        {
            return _inputOperationAttributes;
        }


    }
} // end class LyFormula
