/*
 * LyFieldSave.java
 */

package leon.info.save;

import leon.app.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;

/**
 * Save of a field controller.
 */
public class LyFieldSave
{
    public static String    UNKNOWN_VALUE = "_unknown_";
    
    public LyFieldInfo      _fieldInfo;
    public Object		    _newValue;
    public Object		    _oldValue;
	
    protected LyFieldSave()
    {
    }
    
    public LyFieldSave(LyFieldInfo fieldInfo, Object newValue)
    {
        this (fieldInfo, newValue, UNKNOWN_VALUE);
    }
    
	public LyFieldSave(LyFieldInfo fieldInfo, Object newValue, Object oldValue)
	{
	    _fieldInfo = fieldInfo;
	    _newValue = newValue;
	    _oldValue = oldValue;
	}
	
    public LyFieldSave(LyFieldInfo fieldInfo, String newValue, String oldValue)
    {
        _fieldInfo = fieldInfo;
        _newValue = decode(newValue);
        if ((oldValue != null)
                && oldValue.equals(UNKNOWN_VALUE))
            _oldValue = UNKNOWN_VALUE;
        else
            _oldValue = decode(oldValue);
    }
	
    public String saveAsString()
    {
        StringBuffer    buffer = new StringBuffer();
        
        buffer.append("<FIELD_SAVE field='" + _fieldInfo.getId() + "'");
        if (_newValue != null)
            buffer.append(" value='" + encode(_newValue) + "'");
        if (_oldValue == UNKNOWN_VALUE)
            buffer.append(" oldValue='" + UNKNOWN_VALUE + "'"); 
        else if (_oldValue != null)
            buffer.append(" oldValue='" + encode(_oldValue) + "'"); 
        buffer.append("/>");
        return buffer.toString();
    }
    
    protected String encode(Object value)
    {
        String  strValue = null;
        
        if ((_fieldInfo instanceof LyRelationFieldInfo)
            && (value instanceof LyObjectId[]))
        {
            LyObjectId[] values = (LyObjectId[])value;
            LyObjectId   ojid;
            int          i, n = values.length;
            String       str;

            for (i = 0; i < n; i++)
            {
                ojid = values[i];
                str = ojid.toString();
                if (strValue == null)
                    strValue = str;
                else
                    strValue += LyApplication.VISIBLE_VALUE_SEP + str;
            }
        }
        else if (_fieldInfo.isMultiple())
        {
            strValue = _fieldInfo.toString(value);
            if (strValue != null)
            {
                LyApplication   application = _fieldInfo.getApplication();
                
                strValue = strValue.replace(application.VALUE_SEP, LyApplication.VISIBLE_VALUE_SEP);
            }
        }
        else if (_fieldInfo instanceof LyArrayFieldInfo)
        {
            strValue = _fieldInfo.toString(value);
            if (strValue != null)                
                strValue = strValue.replace(LyApplication.ARRAY_SEP, LyApplication.VISIBLE_ARRAY_SEP);
        }
        else
            strValue = _fieldInfo.toString(value);
        
        return LyString.getXmlString(strValue);
    }
    
    protected Object decode(String strValue)
    {
        return _fieldInfo.parse(strValue);
    }
}