/*
 * LyFormSave.java
 */

package leon.info.save;

import leon.app.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;

import java.io.*;
import java.util.*;

/**
 * Save of a field controller
 */
public class LyFormSave
{
    public static LyFormSave fromString(LyApplication application, String save)
    {
        if (save == null)
            return null;
        
        // Parse XML Result
        LyEnvironment   environment = application.getEnvironment();
        
        try
        {
            String       dir = environment.getEnv("LY_TEMP_DIR");
            File         dirFile = new File(dir);
            File         tmpFile = File.createTempFile("_save", ".xml", dirFile);
            OutputStream out = new FileOutputStream(tmpFile);
            
            if (!save.startsWith("<?xml"))
            {
                out.write("<?xml version='1.0' encoding='UTF-8'?>\n".getBytes("UTF-8"));
                out.write("<!DOCTYPE FORM_SAVE SYSTEM 'leon/conf/dtd/leon_gui.dtd'>\n".getBytes("UTF-8"));
            }
            
            out.write(save.getBytes("UTF-8"));
            out.close();
            
            LyInfoParser    infoParser = new LyInfoParser(application, tmpFile.getAbsolutePath(), LyInfoParser.Type.GUI, false);
            Object          root = infoParser.getRootElement();
            
            return (root instanceof LyFormSave ? (LyFormSave)root : null);
        }
        catch (Exception e)
        {
            environment.writeException(e);
            return null;
        }
    }
    
    public enum Type { CREATE, SET, DELETE };
    
    public Type                     _type;
    public LyClassInfo              _classInfo;
    public LyObjectId               _objectId;
    public Vector<LyFieldSave>	    _fieldSaves;
    public HashMap<String, String>  _datas;
	
    // Default constructor
    protected LyFormSave()
    {
    }
    
    // Creation
    public LyFormSave(Type type, LyClassInfo classInfo, LyObjectId objectId, Vector<LyFieldSave> fieldSaves)
    {
        _type = type;
        _classInfo = classInfo;
        _objectId = objectId;
        _fieldSaves = fieldSaves;
        _datas = null;
    }
    
    // Creation
    public LyFormSave(LyClassInfo classInfo, Vector<LyFieldSave> fieldSaves)
    {
        this(Type.CREATE, classInfo, null, fieldSaves);
    }
    
    // Set
    public LyFormSave(LyObject object, Vector<LyFieldSave> fieldSaves)
    {
        this(Type.SET, object.getClassInfo(), object.getObjectId(), fieldSaves);
    }
    
    // Delete
    public LyFormSave(LyObject object)
    {
        this(object.getClassInfo(), object.getObjectId());
    }
    
    // Delete
    public LyFormSave(LyClassInfo classInfo, LyObjectId objectId)
    {
        this(Type.DELETE, classInfo, objectId, null);
    }
    
    public void addFieldSave(LyFieldSave fieldSave)
    {
        if (_fieldSaves == null)
            _fieldSaves = new Vector<LyFieldSave>(1);
        _fieldSaves.add(fieldSave);
    }
    
    public boolean contains(LyFieldInfo fieldInfo)
    {
        if (_fieldSaves == null)
            return false;
        
        int         i, n = _fieldSaves.size();
        LyFieldSave fieldSave;
        
        for (i = 0; i < n; i++)
        {
            fieldSave = _fieldSaves.get(i);
            if (fieldSave._fieldInfo == fieldInfo)
                return true;
        }
        
        return false;
    }
    
    public String saveAsString()
    {
        StringBuffer    buffer = new StringBuffer();
        
        buffer.append("<FORM_SAVE type='" + _type + "'");
        buffer.append(" class='" + _classInfo.getId() + "'");
        if (_objectId != null)
            buffer.append(" objectId='" + encode(_objectId) + "'");

        buffer.append(">\n");

        if (_fieldSaves != null)
        {
            int         i, n = _fieldSaves.size();
            LyFieldSave fieldSave;
            
            for (i = 0; i < n; i++)
            {
                fieldSave = _fieldSaves.get(i);
                buffer.append(fieldSave.saveAsString());
                buffer.append("\n");
            }
        }
        
        if (_datas != null)
        {
            Set<String>         keys = _datas.keySet();
            Iterator<String>    iterator = keys.iterator();
            String              key, value;
            
            while (iterator.hasNext())
            {
                key = iterator.next();
                value = _datas.get(key);
                buffer.append("<DATA name=\"" + LyString.getXmlString(key)
                    + "\" value=\"" + LyString.getXmlString(value) + "\"/>\n");
            }
        }

        buffer.append("</FORM_SAVE>");

        return buffer.toString();
    }
    
    protected String encode(LyObjectId objectId)
    {
        String  str = _objectId.toString();
        
        str = str.replace(LyObjectId.FIELD_SEP, LyObjectId.VISIBLE_FIELD_SEP);
        str = str.replace(LyObjectId.VALUE_SEP, LyObjectId.VISIBLE_VALUE_SEP);
        return str;
    }
    
    public void putData(String key, String value)
    {
        if (_datas == null)
            _datas = new HashMap<String, String>();
        _datas.put(key, value);
    }
    
    public String removeData(String key)
    {
        if (_datas == null)
            return null;
        else
            return _datas.remove(key);
    }
    
    public String getData(String key)
    {
        if (_datas == null)
            return null;
        else
            return _datas.get(key);
    }
}