/*
 * LyFiles.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.misc;

import java.util.*;


/**
 * This class is used to manage the resources file of an application. It is based on the
 * LyResourceBundle mechanism.
 * @version $Id: LyFiles.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public class LyFiles extends LyResourceBundle
{

	public static final String __VERSION = "$Id: LyFiles.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Constructor with a given environment. Calls {@link #LyFiles(LyEnvironment, String)
	 * LyFiles(environment, null)} constructor.
	 * @param environment The environment of the files bundle.
	 */
	public LyFiles(LyEnvironment environment)
	{
		this(environment, null);
	}

	/**
	 * Constructor with a given environment and name. Calls the {@link #LyFiles(LyEnvironment,
	 * String, Locale) LyFiles(environment, name, environment.getLocale())} constructor.
	 * @param environment The environment of the files bundle.
	 * @param name        The name of the files bundle.
	 */
	public LyFiles(LyEnvironment environment, String name)
	{
		this(environment, name, environment.getLocale(), environment.getSkin());
	}

	/**
	 * Constructor with a given environment and name. Calls the {@link #LyFiles(LyEnvironment,
	 * String, Locale) LyFiles(environment, name, environment.getLocale())} constructor.
	 * @param environment The environment of the files bundle.
	 * @param name        The name of the files bundle.
	 * @param skin        The name of the skin
	 */
	public LyFiles(LyEnvironment environment, String name, String skin)
	{
		this(environment, name, environment.getLocale(), skin);
	}
	
	/**
	 * Constructor with a given environment, name and locale. Calls the super implementation.
	 * @param environment The environment of the strings bundle.
	 * @param name        The name of the strings bundle.
	 * @param locale      The locale used by the strings bundle.
	 */
	protected LyFiles(LyEnvironment environment, String name, Locale locale)
	{
		this(environment, name, locale, environment.getSkin());
	}
	
	/**
	 * Constructor with a given environment, name and locale. Calls the super implementation.
	 * @param environment The environment of the files bundle.
	 * @param name        The name of the files bundle.
	 * @param locale      The locale used by the files bundle.
	 * @param skin        The name of the skin
	 */
	protected LyFiles(LyEnvironment environment, String name, Locale locale, String skin)
	{
		super(environment, (name == null ? "files" : name), locale, skin);
	}

	/**
	 * If the bundle environment is not null, builds a parent files bundle.
	 * @return The parent files bundle for this files bundle, or null if the environment is null.
	 */
	protected LyResourceBundle buildParentBundle()
	{
		if (_environment == null)
			return null;

		return new LyFiles(_environment.getParent(), _name, _locale, _skin);
	}

	/**
	 * Gets the path corresponding to the given key.
	 * @param  key The path key.
	 * @return The path corresponding to the given key if found, the key otherwise.
	 */
	public String getPath(String key)
	{
		String value = get(key);

		// If not found return the key
		if (value == null)
			return key;

		String path = _environment.getString().expand(value, ".");

		return path;
	}

	/**
	 * Gets the URL corresponding to the given key.
	 * @param  key The key of the URL.
	 * @return The URL corresponding to the given key if found, the key otherwise.
	 */
	public String getUrl(String key)
	{
		String value = get(key);

		// If not found return the key
		if (value == null)
			return key;

		String url = _environment.getString().expandUrl(value, "/");

		return url;
	}

	/**
	 * Gets the String corresponding to the given key. Start with the application ResourceBundle and
	 * if not found, search within the parent ResourceBundle.
	 * @param  key The key to search
	 * @return The value corresponding to the given key if found, null otherwise.
	 */
	public String get(String key)
	{
		String value = super.get(key);

		if ((value != null) && (value.length() == 0))
			value = null;
		return value;
	}
} // end class LyFiles
