/*
 * LyDemandPlayer.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.misc;


/**
 * Interface used to take demands control in test modes.
 */
public interface LyPlayer
{

	String __VERSION = "$Id: LyPlayer.java 28448 2010-11-04 11:11:36Z jlsaouli $";

    /** Marker for new objects created when recording scenario. */
    public static final String    NEW_MARKER = "NEW.";
    
	/**
	 * Used in test mode to play next demand in test file.
	 * @return true if the next demand exists and is played, false otherwise
	 */
	boolean playNext();

	/**
	 * Gets delay between each event.
	 * @return the delay
	 */
	int getDelay();

	/**
	 * Free this player.
	 */
	void free();

	/**
	 * Indicates if this player is playing a scenario.
	 * @return true if this player is playing a scenario, false otherwise
	 */
	boolean isPlaying();

	/**
	 * Indicates if this player is recording a scenario.
	 * @return true if this player is recording a scenario, false otherwise
	 */
	boolean isRecording();

	/**
	 * Adds a pair key-value to the session.
	 * @param key   The key that must be stored.
	 * @param value The value that must be stored with the key.
     * @param name  Boolean indicating if the stored value corresponds to an object name.
	 * @see   #getValue
	 */
	void addValue(String key, String value, boolean name);

	/**
	 * Retrieves a value from a registered key.
	 * @param  key The key that must be used.
	 * @return The value stored with the key or null if the key is not registered.
     * @param name  Boolean indicating if the stored value corresponds to an object name.
	 * @see    #addValue
	 */
	String getValue(String key, boolean name);
    
    /**
     * Method called when an exception occured during the session proccessing.
     *
     * @param t       The throwable instance that stopped the processing.
     */
    void handleException(Throwable t);
}