/*
 * LyRegExpUtils.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved.
 */

package leon.misc;

import java.util.*;
import java.util.regex.*;


/**
 * This class implements some basic methods used for checking regular expressions.
 */
public class LyRegExpUtils
{
	public static final String __VERSION =
		"$Id: LyRegExpUtils.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** All formats. */
	protected static Hashtable<String, Pattern>     _formats = null;

    
    /**
     * Tests if a string match a regular expression.
     * @param  environment Environment where the test occured.
     * @param  format      The regular expression.
     * @param  value       The string that must be match the regula expression.
     * @return true if the given string match the regular expression, false otherwise.
     */
    public static boolean matchFormat(LyEnvironment environment, String format, String value)
    {
        if ((format == null) || (value == null))
            return true;

        Matcher   matcher = null;
        Pattern   pattern = null;
        String    key = format.intern();

        pattern = ((_formats == null) ? null : _formats.get(key));

        if (pattern == null)
        {
            try
            {
                pattern = getPattern(key);
                if (_formats == null)
                    _formats = new Hashtable<String, Pattern>();
                _formats.put(key, pattern);
            }
            catch (PatternSyntaxException e)
            {
                environment.logError("Invalid format : \"" + key + "\"\n" + e.getMessage());
                return false;
            }
        }

        matcher = pattern.matcher(value);
        return matcher.matches();
    } // end method matchFormat

    /**
     * Gets the pattern of the given format.
     * @param  format the format to analyse
     * @return the found pattern into the given format
     * @throws PatternSyntaxException if the patter of the format is not well formed
     */
    public static Pattern getPattern(String format)
        throws PatternSyntaxException
    {
        Pattern       pattern = Pattern.compile(format);

        return pattern;
    }
} // end class LyRegExpUtils
