/*
 * LyResources.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.misc;

import java.util.*;


/**
 * This class is used to manage resources of an application. These resources allow the user to
 * customize an application. (e.g. fading on titlebars, use of compression for generated web files,
 * PDF generation allowed...). It is based on the mechanism of LyResourceBundle.
 * @version $Id: LyResources.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public class LyResources extends LyResourceBundle
{

	public static final String __VERSION =
		"$Id: LyResources.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Constructor with a given environment.Calls {@link #LyResources(LyEnvironment, String)
	 * LyResources(environment, null)}.
	 * @param environment The environment of the resources bundle.
	 */
	public LyResources(LyEnvironment environment)
	{
		this(environment, null);
	}

	/**
	 * Constructor with a given environment and name. Calls {@link #LyResources(LyEnvironment,
	 * String, Locale) LyResources(environment, name, environment.getLocale())}.
	 * @param environment The environment of the resources bundle.
	 * @param name        The name of the resources bundle.
	 */
	public LyResources(LyEnvironment environment, String name)
	{
		this(environment, name, environment.getLocale(), environment.getSkin());
	}

	/**
	 * Constructor with a given environment and name. Calls {@link #LyResources(LyEnvironment,
	 * String, Locale) LyResources(environment, name, environment.getLocale())}.
	 * @param environment The environment of the resources bundle.
	 * @param name        The name of the resources bundle.
	 * @param skin        The name of the skin
	 */
	public LyResources(LyEnvironment environment, String name, String skin)
	{
		this(environment, name, environment.getLocale(), skin);
	}
	
	/**
	 * Constructor with a given environment, name and locale. Calls the super implementation.
	 * @param environment The environment of the strings bundle.
	 * @param name        The name of the strings bundle.
	 * @param locale      The locale used by the strings bundle.
	 */
	protected LyResources(LyEnvironment environment, String name, Locale locale)
	{
		this(environment, name, locale, environment.getSkin());
	}
	
	/**
	 * Constructor with a given environment, name and locale. Calls the super implementation.
	 * @param environment The environment of the resources bundle.
	 * @param name        The name of the resources bundle.
	 * @param locale      The locale used by the resources bundle.
	 * @param skin        The name of the skin
	 */
	protected LyResources(LyEnvironment environment, String name, Locale locale, String skin)
	{
		super(environment, (name == null ? "resources" : name), locale, skin);
	}

	/**
	 * Gets the resource string corresponding to the given key.
	 * @param  key The key of the string.
	 * @return The string corresponding to the key if found, the key otherwise.
	 */
	public String getString(String key)
	{
		return super.get(key);
	}

	/**
	 * Builds a parent resources bundle.
	 * @return The parent resources bundle for this resources bundle.
	 */
	protected LyResourceBundle buildParentBundle()
	{
		if (_environment == null)
			return null;

		return new LyResources(_environment.getParent(), _name, _locale, _skin);
	}
} // end class LyResources
