/**
 * LyThreadData.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.misc;

import java.lang.ref.WeakReference;
import java.util.*;


/**
 * This class is used to store and retrieve datas associated to the current thread.
 */
public class LyThreadData<T>
{
	public static final String	__VERSION = "$Id: LyThreadData.java 16363 2009-02-03 16:04:52Z jls $";

	/** Table of datas corresponding to Threads. */
	protected Hashtable<String, WeakReference<T>>     _threadDatas;

    /**
     * Default constructor.
     */
    public LyThreadData()
    {
    }

	/**
	 * Gets the current object associated to the given key for the current thread.
	 *
     * @param key The key for which the instance is looked for.
	 * @return Instance of the current object (may be null) associated to the given key
     *         if it was not set in the current thread.
	 */
	public T getData(String key)
	{
		if (_threadDatas == null)
			return null;
		
    	WeakReference<T>	dataRef = _threadDatas.get(getThreadKey(key));
    	
		return (dataRef == null ? null : dataRef.get());
	}

	/**
	 * Sets the object associated to the given key for the current thread.
	 *
     * @param key The key for which the instance is stored.
	 * @param context Object corresponding to the given key for the current thread.
     *                If null, unset instance associated to the key.
     * @return Previous value associated to the given key.
	 */
	public T putData(String key, T data)
	{
		if (data == null)
		    return removeData(key);
		else
		{
			if (_threadDatas == null)
				_threadDatas = new Hashtable<String, WeakReference<T>>(1);
						
			_threadDatas.put(getThreadKey(key), new WeakReference<T>(data));
			return data;
		}
	} // end method setCurrentContext

    /**
     * Sets the object associated to the given key for the current thread.
     *
     * @param key The key for which the instance is stored.
     * @param context Object corresponding to the given key for the current thread.
     *                If null, unset instance associated to the key.
     */
    public T removeData(String key)
    {
        if (_threadDatas == null)
            return null;
        else
        {
        	WeakReference<T>	dataRef = _threadDatas.remove(getThreadKey(key));
        	
        	return (dataRef == null ? null : dataRef.get());
        }
    } // end method setCurrentContext
    
    /**
     * Remove all keys associated to given object.
     * @param data The data to be removed
     */
    public void removeKeys(T data)
    {
        if (data == null)
            return;

        // Clean up threads/sessions table
        if (_threadDatas != null)
        {
            Enumeration<String> keys = _threadDatas.keys();
            String              key;
            WeakReference<T>	dataRef;

            while (keys.hasMoreElements())
            {
                key = keys.nextElement();
                dataRef = _threadDatas.get(key);
                if ((dataRef != null)
                		&& (dataRef.get() == data))
                    _threadDatas.remove(key);
            }
        }
    }
    
    /**
     * Gets unique key string for the given key object and current thread.
     * @param key The key object.
     * @return Unique key string.
     */
    protected static String getThreadKey(String key)
    {
        return key + Thread.currentThread().getId();
    }
} // end class LyApplication
