/*
 * LyEventConnection.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.notifier;

import java.util.ArrayList;

import leon.misc.LyEnvironment;

/**
 * This class is used to send events.
 *
 * @version $Id: LyEventConnection.java 34838 2012-03-19 11:01:55Z jlsaouli $
 * @author  Lyria
 * @see     LyEventEncoder
 */
public class LyEventConnection
{
	public static final String __VERSION =
		"$Id: LyEventConnection.java 34838 2012-03-19 11:01:55Z jlsaouli $";

	protected LyEventNotifier  _notifier;
	/**  */
	protected _LyEventQueueThread     _eventQueueThread = null;

	/**
	 * Default constructor Initialization is done with the init method.
	 */
	public LyEventConnection(LyEventNotifier notifier)
	{
		init();
		_notifier = notifier;
	}

	/**
	 * Initialisation of the data of the instance.
	 */
	protected void init()
	{
		_notifier = null;
	}

	/**
	 */
	public boolean close()
	{
		stopEventQueue();
		_eventQueueThread = null;

		return true;
	}
	
	/**
	 * Create a queue to post event in the same thread
	 * @param r
	 */
	public void addEvent(Runnable r)
	{
		if (_eventQueueThread != null)
			_eventQueueThread.addEvent(r);
		else
		{
			_eventQueueThread = new _LyEventQueueThread();
			_eventQueueThread.addEvent(r);
			_eventQueueThread.start();
		}
	}
	
	/**
	 * To stop the event queue thread.
	 */
	protected void stopEventQueue()
	{
		if (_eventQueueThread != null)
			_eventQueueThread.stopQueue();
	}


	/**
	 * Internal class used to manage events.
	 */
	protected class _LyEventQueueThread extends Thread
	{
		protected ArrayList<Runnable> _events = null;
		protected boolean             _stop = false;

		public _LyEventQueueThread()
		{
			super("BF AppEngine : Notifier Event queue");

			setDaemon(true);
			_events = new ArrayList<Runnable>();
		}

		public void addEvent(Runnable r)
		{
			_events.add(r);
		}

		public void stopQueue()
		{
			_stop = true;
		}

		/**
		 * Main loop.
		 */
		public void run()
		{
			LyEnvironment environment = _notifier.getEnvironment();
			Runnable	  r;

			environment.logInfo("[JMS EVENT QUEUE] : run");
			while (!_stop)
			{
				try
				{
					while (!(_events.isEmpty()))
					{
						r = _events.remove(0);
						r.run();
					}
					sleep(500);
				}
				catch (Exception e)
				{
					if (environment.isDebugLevel(LyEnvironment.DEBUG_ERROR))
						environment.writeException(e);
				}
			}
			
			environment.logInfo("[JMS EVENT QUEUE] : stop");
			_events = null; // cleanup
		}
	} // end class _LyEventQueueThread

} // end class LyEventConnection
