/*
 * LyNotifierFactory.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.notifier;

import java.lang.reflect.*;
import leon.app.*;
import leon.info.notifier.*;
import leon.misc.*;


/**
 * This class is used by LyApplication to create instances of LyEventNotifier defined by LyNotifier.
 *
 * @version $Id: LyNotifierFactory.java 31200 2011-05-26 14:14:22Z gtachon $
 * @author  Lyria
 * @see     leon.app.LyApplication
 * @see     leon.info.notifier.LyNotifier
 */
public class LyNotifierFactory
{
	public static final String __VERSION =
		"$Id: LyNotifierFactory.java 31200 2011-05-26 14:14:22Z gtachon $";

	/**
	 * Create a new notifier instance or access to existing one.
	 */
    public static LyEventNotifier getNotifier(LyApplication application,
		LyNotifier notifierDesc)
	{
		LyEventNotifier notifier = null;
		LyEnvironment   environment = application.getEnvironment();
		String		    javaName = notifierDesc.getJavaName();

		if (javaName != null)
		{
			// For compatibility reason, translate old leonardi packages to w4 packages
            if (javaName.indexOf("leon.notifier.jms") >=0)
            	javaName = javaName.replace("leon.notifier", "eu.w4.ae.notifier");	
            else if (javaName.equalsIgnoreCase("jms"))
				javaName = "eu.w4.ae.notifier.jms.LyJmsEventNotifier";
			else if (javaName.equalsIgnoreCase("link"))
				javaName = "eu.w4.ae.notifier.link.LyLinkEventNotifier";

			Class<?>[]  types = new Class[2];
			Object[] parameters = new Object[2];

			types[0] = LyApplication.class;
			parameters[0] = application;

			types[1] = LyNotifier.class;
			parameters[1] = notifierDesc;

			// Build provider
			@SuppressWarnings("unchecked")
			Class<LyEventNotifier>	    notifierClass = (Class<LyEventNotifier>)application.loadClass(javaName,
					"BF AppEngine : unknown notifier class " + javaName + " in LyNotifierFactory");;
			Constructor<LyEventNotifier> constructor;

			if (notifierClass == null)
				return null;

			try
			{
				constructor = notifierClass.getConstructor(types);
				notifier = constructor.newInstance(parameters);
			}
			catch (InvocationTargetException e)
			{
				environment.writeException(e.getTargetException());
				return null;
			}
			catch (NoSuchMethodException e)
			{
				// Unknown controller
				environment.logError("BF AppEngine : constructor with parameters ("
					+ types[0] + ", " + types[1] + ") not found on class " + notifierClass);
				environment.writeException(e);
				return null;
			}
			catch (Exception e)
			{
				environment.writeException(e);
				return null;
			}
		}

		return notifier;
	} // end method getNotifier
} // end class LyNotifierFactory
