package leon.parser;

import java.util.*;

public abstract class LyAbstractViewNodeFactory
{
	/** List of all XML tags. */
	protected static final String TAG_AREA = "AREA".intern();
	protected static final String TAG_CELL = "CELL".intern();
	protected static final String TAG_CAPTION = "CAPTION".intern();
	protected static final String TAG_CHOICE = "CHOICE".intern();
	protected static final String TAG_CHART = "CHART".intern();
	protected static final String TAG_COLUMN = "COLUMN".intern();
	protected static final String TAG_INTERVALS = "INTERVALS".intern();
	protected static final String TAG_AXIS = "AXIS".intern();
	protected static final String TAG_CHARTYPE = "CHARTYPE".intern();
	protected static final String TAG_CHART_VALUE = "CHART_VALUE".intern();
	protected static final String TAG_COLOR = "COLOR".intern();
	protected static final String TAG_COLOR_VALUE = "COLOR_VALUE".intern();
	protected static final String TAG_COMMAND = "COMMAND".intern();
	protected static final String TAG_COMMANDS = "COMMANDS".intern();
	protected static final String TAG_CONSTRAINT = "CONSTRAINT".intern();
	protected static final String TAG_DATA = "DATA".intern();
	protected static final String TAG_DATE = "DATE".intern();
	protected static final String TAG_DAY = "DAY".intern();
    protected static final String TAG_DEFINITION = "DEFINITION".intern();
    protected static final String TAG_DESCRIPTION = "DESCRIPTION".intern();
	protected static final String TAG_DIALOG = "DIALOG".intern();
    protected static final String TAG_EXTERN = "EXTERN".intern();
	protected static final String TAG_FIELD = "FIELD".intern();
	protected static final String TAG_FILE = "FILE".intern();
	protected static final String TAG_FONT = "FONT".intern();
	protected static final String TAG_FONT_VALUE = "FONT_VALUE".intern();
	protected static final String TAG_FOOTER = "FOOTER".intern();
	protected static final String TAG_FOOTERS = "FOOTERS".intern();
	protected static final String TAG_FORM = "FORM".intern();
	protected static final String TAG_FRAME = "FRAME".intern();
	protected static final String TAG_GAUGE = "GAUGE".intern();
	protected static final String TAG_ICON = "ICON".intern();
	protected static final String TAG_HEADER = "HEADER".intern();
	protected static final String TAG_HEADERS = "HEADERS".intern();
	protected static final String TAG_HREF = "HREF".intern();
	protected static final String TAG_HREFS = "HREFS".intern();
	protected static final String TAG_ITEM = "ITEM".intern();
	protected static final String TAG_INDEX = "INDEX".intern();
	protected static final String TAG_LABEL = "LABEL".intern();
	protected static final String TAG_LAYOUT = "LAYOUT".intern();
	protected static final String TAG_LINE = "LINE".intern();
	protected static final String TAG_LINK = "LINK".intern();
	protected static final String TAG_LIST = "LIST".intern();
	protected static final String TAG_LIST_OPTION = "LIST_OPTION".intern();
	protected static final String TAG_MAP = "MAP".intern();
	protected static final String TAG_MAP_IMAGE = "MAP_IMAGE".intern();
	protected static final String TAG_MAP_LINK = "MAP_LINK".intern();
	protected static final String TAG_MAP_NODE = "MAP_NODE".intern();
	protected static final String TAG_MAP_REGION = "MAP_REGION".intern();
	protected static final String TAG_MAP_CONTENT = "MAP_CONTENT".intern();
	protected static final String TAG_MENU = "MENU".intern();
	protected static final String TAG_MENUBAR = "MENUBAR".intern();
	protected static final String TAG_MESSAGE = "MESSAGE".intern();
	protected static final String TAG_MESSAGES = "MESSAGES".intern();
	protected static final String TAG_NOTICE = "NOTICE".intern();
	protected static final String TAG_NUMBER = "NUMBER".intern();
	protected static final String TAG_OPTION = "OPTION".intern();
	protected static final String TAG_PAGE = "PAGE".intern();
	protected static final String TAG_PAGES = "PAGES".intern();
	protected static final String TAG_POINT = "POINT".intern();
	protected static final String TAG_POLYGON = "POLYGON".intern();
	protected static final String TAG_RELATION = "RELATION".intern();
	protected static final String TAG_ROW = "ROW".intern();
	protected static final String TAG_ROWS = "ROWS".intern();
	protected static final String TAG_SELECTBAR = "SELECTBAR".intern();
	protected static final String TAG_SELECTITEM = "SELECTITEM".intern();
	protected static final String TAG_SEP = "SEP".intern();
	protected static final String TAG_SPLITTER = "SPLITTER".intern();
	protected static final String TAG_SUBINDEX = "SUBINDEX".intern();
	protected static final String TAG_TABLE = "TABLE".intern();
	protected static final String TAG_TAB = "TAB".intern();
	protected static final String TAG_TABS = "TABS".intern();
	protected static final String TAG_TEXT = "TEXT".intern();
	protected static final String TAG_TIP = "TIP".intern();
	protected static final String TAG_TITLEBAR = "TITLEBAR".intern();
	protected static final String TAG_TOOL = "TOOL".intern();
	protected static final String TAG_TOOLBAR = "TOOLBAR".intern();
	protected static final String TAG_TREE = "TREE".intern();
	protected static final String TAG_TREE_NODE = "TREE_NODE".intern();
	protected static final String TAG_UNIT = "UNIT".intern();
    protected static final String TAG_VIEWS = "VIEWS".intern();

    protected static final String ATT_ID = "id".intern();
	protected static final String ATT_NAME = "name".intern();
	protected static final String ATT_VALUE = "value".intern();
    protected static final String ATT_TEMPLATE = "template".intern();
    
	protected static Hashtable<String, String> _classNames;
	protected static Hashtable<String, String> _tags;

	static
	{
		_classNames = new Hashtable<String, String>();
		_tags = new Hashtable<String, String>();
	}
	
	/**
	 * Utility method to get class name from tag.
	 *
	 * @param   tag  The tag name
	 *
	 * @return  The corresponding class name.
	 */
	public static String getClassName(String tag)
	{
		if (tag == TAG_SEP)
			return null;

		String className = _classNames.get(tag);

		if (className != null)
			return className;

		// First, some exceptions in naming
		tag = tag.intern();

		if (tag == TAG_FIELD)
			className = "leon.view.LyFormField";
		else if (tag == TAG_MENUBAR)
			className = "leon.view.LyMenuBar";
		else if (tag == TAG_TOOLBAR)
			className = "leon.view.LyToolBar";
		else if (tag == TAG_TITLEBAR)
			className = "leon.view.LyTitleBar";
		else if (tag == TAG_SELECTBAR)
			className = "leon.view.LySelectBar";
		else if (tag == TAG_SELECTITEM)
			className = "leon.view.LySelectItem";
		else if (tag == TAG_SUBINDEX)
			className = "leon.view.LySubIndex";
		else if (tag == TAG_HREF)
			className = "leon.view.LyHRef";
		else if (tag == TAG_HREFS)
			className = "leon.view.LyHRefs";
		else if (tag == TAG_UNIT)
			className = "leon.view.LyFieldUnit";
		else if (tag == TAG_CHARTYPE)
			className = "leon.view.LyCharType";
		else
		{
			// General case
			StringBuffer buffer = new StringBuffer();
			int			 i, nc = tag.length();
			char		 c;
			boolean		 upperCase = true;

			for (i = 0; i < nc; i++)
			{
				c = tag.charAt(i);
				if (c == '_')
					upperCase = true;
				else if (upperCase)
				{
					buffer.append(c);
					upperCase = false;
				}
				else
					buffer.append(Character.toLowerCase(c));
			}

			className = "leon.view.Ly" + buffer.toString();
		}

		_classNames.put(tag, className);
		return className;
	} // end method getClassName
}