/*
 * LyEntityResolver.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.parser;

import org.xml.sax.*;
import java.io.*;
import leon.misc.*;


/**
 * This class looks for entities in the classpath when the parser can't find them locally.
 * @version $Id: LyEntityResolver.java 34806 2012-03-19 07:58:39Z jlsaouli $
 * @author  Lyria
 */
public class LyEntityResolver extends LySimpleEntityResolver
{
	public static final String __VERSION =
		"$Id: LyEntityResolver.java 34806 2012-03-19 07:58:39Z jlsaouli $";

	/** The name of the info directory. */
	public static String	   INFO_DIRECTORY = "info/";

	/** Name of the file that describes common types. */
	protected static String    COMMON_TYPES = "commonTypes.xsd";

	/** The current class loader. */
	protected ClassLoader	   _classLoader;

	/**
	 * Default constructor. Calls the super implementation with the environment parameter, then sets
	 * the current class loader with the given value.
	 * @param environment the current environment
	 * @param classLoader the current class loader
	 */
	public LyEntityResolver(LyEnvironment environment, ClassLoader classLoader)
	{
		super(environment);

		_classLoader = classLoader;
	}

	/**
	 * Builds an inputsource from the specified entity and the current environment.
	 * @param  systemId entity identifier
	 * @return an input source if it can be buildt, null otherwise
	 */
	public InputSource getInputSource(String systemId)
	{
		if (systemId == null)
			return null;

		// Set grammar type when found
		if ((systemId.endsWith(".dtd") || systemId.endsWith(".xsd"))
				&& !systemId.endsWith(LyEntityResolver.COMMON_TYPES))
		{
			int pos = systemId.lastIndexOf('/');

			_environment.putEnv("_LY_XML_GRAMMAR",
				systemId.substring(pos + 1, systemId.length() - 4));
		}

		// Skip if protocol is specified
		if (systemId.toLowerCase().startsWith("http:"))
			return null;

		systemId = systemId.replace('\\', '/');

		// Shall we check info directory ? Yes if info is not part of the
		// systemId
		boolean     tryInfoDir = (systemId.indexOf(INFO_DIRECTORY) < 0);

		// Try to find the public identifier in the classpath
		InputStream aStream;
		int		    n = 0;

		String	    aStreamSystemId = systemId;

		aStream = _environment.getResourceAsStream(aStreamSystemId, _classLoader);

		// Try with a subpart of the system id
		while ((aStream == null) && (n >= 0))
		{
			// Try with the end of systemId;
			n = systemId.indexOf('/', n + 1);

			if (n >= 0)
			{
				aStreamSystemId = systemId.substring(n + 1);
				
				if (aStreamSystemId.startsWith("//"))
				{
					aStreamSystemId = systemId.substring(n + 2);
					n++;
				}

				aStream = _environment.getResourceAsStream(aStreamSystemId, _classLoader);

				if ((tryInfoDir) && (aStream == null))
				{
					aStreamSystemId = INFO_DIRECTORY + systemId.substring(n + 1);
					aStream = _environment.getResourceAsStream(aStreamSystemId, _classLoader);
				}
			}
		}

		// If we could find it, return
		// Otherwise, try generic access.
		if (aStream != null)
		{
			// Set systemId to the InputSource to avoid NPE in Xerces parser.
			InputSource result = new InputSource(aStream);

			result.setSystemId(aStreamSystemId);

			_environment.logTrace("Resolve entity " + systemId + " with " + aStreamSystemId);
			return result;
		}

		// Can't resolve
		return null;
	} // end method getInputSource
} // end class LyEntityResolver
