/*
 * LySimpleEntityResolver.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.parser;

import org.xml.sax.*;
import leon.misc.*;


/**
 * This class implements a simple entity resolver. It does not return any entity but register the
 * system id for leon.dtd.
 * @version $Id: LySimpleEntityResolver.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public class LySimpleEntityResolver
	implements EntityResolver
{
	public static final String __VERSION =
		"$Id: LySimpleEntityResolver.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** The current environment. */
	protected LyEnvironment    _environment;

	/**
	 * Default constructor. Calls {@link #LySimpleEntityResolver(LyEnvironment) this(null)}
	 */
	public LySimpleEntityResolver()
	{
		this(null);
	}

	/**
	 * Default constructor with an environment.
	 * @param environment the resolver environment
	 */
	public LySimpleEntityResolver(LyEnvironment environment)
	{
		_environment = environment;
	}

	/**
	 * Allows the application to resolve external entities.
	 * @param publicId the public identifier of the external entity being referenced, or null if
	 *                 none was supplied
	 * @param systemId the system identifier of the external entity being referenced
	 */
	public InputSource resolveEntity(String publicId, String systemId)
	{
		return getInputSource(systemId);
	}

	/**
	 * Gets an inputsource from a specified URL. This method does nothing and always return null
	 * @param  systemId identifier of the entity
	 * @return null
	 */
	public InputSource getInputSource(String systemId)
	{
		return null;
	}
} // end class LySimpleEntityResolver
