/*
 * LyElementFactory.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.parser;

import org.xml.sax.*;


/**
 * This class is used to build parsed XML structure (object tree) from specified tag. For each tag,
 * the parser invokes the factory to create object instances (builElement) and to complete the
 * created instance at the end of the tag parsing (endElement).
 * @see     LyXmlNode
 * @version $Id: LyXMLElementFactory.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public interface LyXMLElementFactory
{
	String __VERSION = "$Id: LyXMLElementFactory.java 28422 2010-11-03 12:03:38Z ehelmste $";

    /**
     * Builds the object corresponding to the given node tag. This method is invoked by SAXParser at
     * the begening of each new tag.
     * @param  uri        the Namespace URI
     * @param  localName  the Namespace local name
     * @param  qName      the qualified (prefixed) name
     * @param  attributes the SAX2 attributes
     * @param  parent     the parent node (created by {@link #buildElement(String, String, String,
     *                    Attributes)})
     * @return the built element
     * @throws SAXException if the result could not have been built
     */
    Object buildElement(String uri, String localName, String qName, Attributes attributes, Object parent)
        throws SAXException;

	/**
	 * Complete the current node object instance at the end of the tag parsing. The result depends
	 * on the node tag. This method is invoked by SAXParser at the begening of each new tag.
	 * @param     uri       the Namespace URI
	 * @param     localName the Namespace local name
	 * @param     qName     the qualified (prefixed) name
	 * @param     parent    the parent node (created by {@link #buildElement(String, String, String,
	 *                      Attributes)})
	 * @param     current   the current node (created by {@link #buildElement(String, String,
	 *                      String, Attributes)})
	 * @exception SAXException if the result could not have been built
	 */
	void endElement(String uri, String localName, String qName, Object parent, Object current)
		throws SAXException;

}