/*
 * LyImportFileConnection.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.peer.file;

import java.util.*;
import leon.app.*;
import leon.data.*;
import leon.info.*;


/**
 * This class is used as a connection for import.
 * @version $Id: LyImportFileConnection.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 * @see     leon.peer.LyConnection
 * @see     LyFileConnection
 */
public class LyImportFileConnection extends LyFileConnection
{
	public static final String __VERSION =
		"$Id: LyImportFileConnection.java 28422 2010-11-03 12:03:38Z ehelmste $";


	/**  */
	protected String		   _filePath;

	/**
	 * Decoding type. One of the following:
	 * <ul>
	 *   <li>{@link LyFileRequestEncoder#RAW_ENCODING}</li>
	 *   <li>{@link LyFileRequestEncoder#TEXT_ENCODING}</li>
	 *   <li>{@link LyFileRequestEncoder#SPREADSHEET_ENCODING}</li>
	 *   <li>{@link LyFileRequestEncoder#EXPORT_ENCODING}</li>
	 *   <li>{@link LyFileRequestEncoder#XML_EXPORT_ENCODING}</li>
	 * </ul>
	 */
	protected int			   _decodingType;

	/** The list of fields contained in the file. */
	protected Vector<String>		   _fileFieldIds;

	/**
	 * Logical class info associated with this file connection. May be needed for spreadsheet
	 * import.
	 */
	protected LyClassInfo	   _classInfo;

	/** The index of the first line of the imported file containing real data. */
	protected int			   _startLine;


	/**
	 * Constructor.
	 * @param classInfo ClassInfo used by the request encoder.
	 */
	public LyImportFileConnection(LyApplication application, String filePath)
	{
		super(application, null);

		_filePath = filePath;
		_decodingType = LyFileRequestEncoder.RAW_ENCODING;
		_fileFieldIds = null;
		_startLine = 0;
	}


	/**
	 * Sets the decoding type.
	 * @param decodingType one of the following:
	 *                     <ul>
	 *                       <li>{@link LyFileRequestEncoder#RAW_ENCODING}</li>
	 *                       <li>{@link LyFileRequestEncoder#TEXT_ENCODING}</li>
	 *                       <li>{@link LyFileRequestEncoder#SPREADSHEET_ENCODING}</li>
	 *                       <li>{@link LyFileRequestEncoder#EXPORT_ENCODING}</li>
	 *                       <li>{@link LyFileRequestEncoder#XML_EXPORT_ENCODING}</li>
	 *                     </ul>
	 */
	public void setDecodingType(int decodingType)
	{
		_decodingType = decodingType;
	}

	/**
	 * Sets the list of the field identifers contained in the file.
	 * @param fieldIds the list of the field identifers contained in the file.
	 */
	public void setFileFieldIds(Vector<String> fieldIds)
	{
		_fileFieldIds = fieldIds;
	}

	/**
	 * Sets the logical class associated with this connection.
	 * @param classInfo the new logical class associated with this connection.
	 */
	public void setLogicalClass(LyClassInfo classInfo)
	{
		_classInfo = classInfo;
	}

	/**
	 * Sets the start line.
	 * @param startLine the index of the first line of the imported file containing real data.
	 */
	public void setStartLine(int startLine)
	{
		_startLine = startLine;
	}

	/**
	 * Return the path of the file containing the data.
	 */
	protected String getFilePath(LyClass peerClass)
	{
		return _filePath;
	}

	/**
	 * Return the decoding type.
	 */
	protected int getDecodingType()
	{
		return _decodingType;
	}

	/**
	 * Return the list of field identifiers contained in the file.
	 */
	protected Vector<String> getFileFieldIds()
	{
		return _fileFieldIds;
	}

	/**
	 * Return the logical class associated with this connection.
	 */
	protected LyClassInfo getLogicalClass()
	{
		return _classInfo;
	}

	/**
	 * Return the start line.
	 * @return the index of the first line of the imported file containing real data.
	 */
	public int getStartLine()
	{
		return _startLine;
	}
} // end class LyImportFileConnection
