/*
 * LyAttributeField.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.peer.java;

import java.lang.reflect.*;
import leon.peer.*;


/**
 * The implementation of Lyfield to describe attributes of a LyJavaClass.
 * @version $Id: LyAttributeField.java 28997 2010-12-23 10:17:15Z bmichel $
 * @author  Lyria
 * @see     LyJavaClass
 * @see     leon.data.LyField
 */
public class LyAttributeField extends LyPeerField
{
	public static final String __VERSION =
		"$Id: LyAttributeField.java 28997 2010-12-23 10:17:15Z bmichel $";

	protected String		   _name;
	protected Method		   _getMethod;
	protected Method		   _setMethod;
	
	protected Class<?>		   _valueClass;

	/**
	 * Constructor.
	 */
	public LyAttributeField(String name)
	{
		_name = name;
		_getMethod = null;
		_setMethod = null;
		_valueClass = null;
	}

	/**
	 * Gets the identifier of this field.
	 * @return the identifier of this field.
	 */
	public String getId()
	{
		return _name;
	}

	/**
	 * Gets the java class representation of the value of this field.
	 * @return the java class representation of this field.
	 */
	public Class<?> getValueClass()
	{
		return _valueClass;
	}

	/**
	 * Sets the java class representation of the value of this field.
	 * @param valueClass the java class representation of this field.
	 */
	public void setValueClass(Class<?> valueClass)
	{
		_valueClass = valueClass;
	}

	/**
	 * Accessor to the get method for this field.
	 * @param  javaClass the javaClass
	 * @return the get method
	 */
	public String getGetMethodName(LyJavaClass javaClass)
	{
		return "get" + Character.toUpperCase(_name.charAt(0)) + _name.substring(1);
	}
    
	/**
	 * Accessor to the get method for this field.
	 * @param  javaClass the javaClass
	 * @return the get method
	 */
	
	public Method getGetMethod(LyJavaClass javaClass)
	{
		if (_getMethod != null)
            return _getMethod;
        
		String methodName =  getGetMethodName(javaClass);
		Class<?>  objectClass = javaClass.getObjectClass();

		do
        {
		    try
			{
				_getMethod = objectClass.getDeclaredMethod(methodName);
                if (_getMethod != null)
                    return _getMethod;
			}
			catch (Exception e)
			{
				// Get method not found
			}
            
            objectClass = objectClass.getSuperclass();
        }
        while (objectClass != null);

		return null;
	}

    /**
     * Accessor to the set method for this field.
     * @param  javaClass the javaClass
     * @return the set method
     */
    public String getSetMethodName(LyJavaClass javaClass)
    {
        return "set" + Character.toUpperCase(_name.charAt(0)) + _name.substring(1);
    }
    
	/**
	 * Accessor to the set method for this field.
	 * @param  javaClass the javaClass
	 * @return the set method
	 */
	public Method getSetMethod(LyJavaClass javaClass)
	{
        if (_setMethod != null)
            return _setMethod;
        
        String  methodName = getSetMethodName(javaClass);
        Class<?>   objectClass = javaClass.getObjectClass();
        Class<?>[] setParameters = new Class[] { _valueClass };

        do
        {
            try
            {
                _setMethod = objectClass.getDeclaredMethod(methodName, setParameters);
                if (_setMethod != null)
                    return _setMethod;
            }
            catch (Exception e)
            {
				// Set method not found
            }
            
            objectClass = objectClass.getSuperclass();
        }
        while (objectClass != null);
        
        return null;
	}
} // end class LyAttributeField
