/*
 * LyJavaDataProvider.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.peer.java;

import java.lang.reflect.*;
import leon.app.*;
import leon.data.*;
import leon.info.*;
import leon.info.binding.*;
import leon.info.location.*;
import leon.peer.*;


/**
 * This class is used by DataSources to transmit their requests to an instance of
 * JavaRequestEncoder.
 * @see LyDataProvider
 * @see LyJavaRequestEncoder
 * @see LyJavaConnection
 */
public class LyJavaDataProvider extends LyPeerDataProvider
{
	public static final String __VERSION =
		"$Id: LyJavaDataProvider.java 28997 2010-12-23 10:17:15Z bmichel $";

	/**
	 * Default constructor Initialization is done with the init method.
	 */
	public LyJavaDataProvider(LyApplication application, LyLocation location)
	{
		super(application, location);
	}

	/**
	 * This class is a factory to create a LyConnection instance.
	 */
	protected LyConnection createConnection()
	{
		return null;
	}

	/**
	 * Create a specific request encoder for the specified dataSource.
	 */
	public LyRequestEncoder getRequestEncoder(LyDataSource dataSource)
	{
		return null;
	}

	/**
	 * Create a peer class with the specified id for the class binding.
	 * @param  id           Identifier of the new peer class.
	 * @param  classBinding Class binding for which a peer class is created.
	 * @return The new peer class.
	 */
	protected LyPeerClass createPeerClass(String id, LyClassBinding classBinding)
	{
		String binding = classBinding.getBinding();

		return new LyJavaClass(id, binding);
	}

	/**
	 * Create a peer field with the specified id for the field info of the given class binding.
	 * @param  id           Identifier of the new peer field.
	 * @param  fieldInfo    Field info for which a peer field is created.
	 * @param  classBinding Class binding to which the field info belongs to.
	 * @return The new peer field.
	 */
	protected LyPeerField createPeerField(String id, LyFieldInfo fieldInfo,
		LyClassBinding classBinding)
	{
		LyAttributeField attributeField;
		LyJavaClass		 javaClass = (LyJavaClass)classBinding.getPeerClass();
		Class<?>		 valueClass;

		attributeField = new LyAttributeField(id);
		valueClass = getValueClass(javaClass, attributeField, fieldInfo);
		attributeField.setValueClass(valueClass);

		return attributeField;
	}

	/**
	 */
	protected Class<?> getValueClass(LyJavaClass javaClass, LyAttributeField attributeField,
		LyFieldInfo fieldInfo)
	{
		Class<?> valueClass = attributeField.getValueClass();

		if (valueClass != null)
			return valueClass;

		Method getMethod = attributeField.getGetMethod(javaClass);

		if (getMethod == null)
			return fieldInfo.getValueClass();

		return getMethod.getReturnType();
	}
} // end class LyJavaDataProvider
