/*
 * LyJavaRequestEncoder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.peer.java;

import java.util.*;
import leon.data.*;
import leon.info.*;
import leon.peer.*;


/**
 * This class is used to send requests to a java connection and to receive its answers.
 * @version $Id: LyJavaRequestEncoder.java 29745 2011-02-21 11:16:01Z jlsaouli $
 * @author  Lyria
 * @see     LyRequestEncoder
 * @see     LyJavaConnection
 * @see     LyJavaDataProvider
 */
public class LyJavaRequestEncoder extends LyRequestEncoder
{
	public static final String __VERSION =
		"$Id: LyJavaRequestEncoder.java 29745 2011-02-21 11:16:01Z jlsaouli $";

	/**
	 * Constructor.
	 * @param provider
	 * @param dataSource
	 */
	public LyJavaRequestEncoder(LyDataProvider provider, LyDataSource dataSource)
	{
		super(provider, dataSource);
	}

	/**
	 * Set the request encoder response. Inherited method of LyRequestEncoder to set properties to
	 * the response.
	 * @param the response
	 */
	public void setResponse(LyResponse response)
	{
		_response = response;

		// Set properties of the response.
		_response.setSorted(false);
		_response.setFiltered(false);
		_response.setCounted(false);
		_response.setDistinctValues(false);
	}

	/**
	 * Get an object.
	 * @param  objectId object identifier
	 * @param  fields   the fields to load (if null, load all)
	 * @return true if the request was correctly sent to the external data provider.
	 */
	public boolean getObject(LyObjectId objectId, Vector<LyField> fields)
	{
		// Set the response in running mode
		_response.setStatus(LyResponse.Status.RUNNING);

		// Encoding the request.
		// TODO

		// Get a valid connection.
		LyJavaConnection connection = (LyJavaConnection)_provider.getConnection();

		// Send the request to the connection
		// TODO

		// Decode the values
		// TODO

		// Release the connection.
		_provider.releaseConnection(connection);

		// Add the object to the response
		// TODO

		return true;
	}

	/**
	 * Get a set of objects.
	 * @param  fields the fields to load (if null, load all)
	 * @param  sort   indicates sorting criterion, or null if default sort is used.
	 * @param  filter indicates filtering criterion, or null if there is no filter.
	 * @param  n      maximum number of objects to get.
	 * @return true if the request was correctly sent to the external data provider.
	 */
	public boolean getObjects(Vector<LyField> fields, LySort sort, LyFilter filter, int n)
	{
		// Set the response in running mode
		_response.setStatus(LyResponse.Status.RUNNING);

		// Encoding the request.
		// TODO

		// Get a valid connection.
		LyJavaConnection connection = (LyJavaConnection)_provider.getConnection();

		// Send the request to the connection
		// TODO

		// Decode the values
		// TODO
		Vector<Object>	objects = new Vector<Object>();

		// Release the connection.
		_provider.releaseConnection(connection);

		int    i, nb = objects.size();
		Object object;

		for (i = 0; i < nb; i++)
		{
			object = objects.elementAt(i);
			_dataSource.receiveResponse(_response, LyResponse.Status.RUNNING, object, null);
		}

		_dataSource.receiveResponse(_response, LyResponse.Status.COMPLETED, null, null);

		return true;
	} // end method getObjects

	/**
	 * Return the number of objects from the data provider.
	 * @return true if the request was correctly sent to the external data provider.
	 */
	public boolean getSize()
	{
		return getSize(null);
	}

	/**
	 * Return the number of objects from the data provider .
	 * @param  filter indicates filtering criterion, or null if there is no filter.
	 * @return true if the request was correctly sent to the external data provider.
	 */
	public boolean getSize(LyFilter filter)
	{
		// Set the response in running mode
		_response.setStatus(LyResponse.Status.RUNNING);

		// Encoding the request.
		// TODO

		// Get a valid connection.
		LyJavaConnection connection = (LyJavaConnection)_provider.getConnection();

		// Send the request to the connection
		// TODO

		// Decode the value
		// TODO
		Integer			 size = new Integer(-1);

		// Release the connection.
		_provider.releaseConnection(connection);

		// Add the size returned to the response
		_dataSource.receiveResponse(_response, LyResponse.Status.COMPLETED, size, null);

		return true;
	}

	/**
	 *  Methods for deletion *
	 */
	/**
	 * To delete an object.
	 * @param  object the object to delete
	 * @return true if the request was correctly sent to the external data provider.
	 */
	public boolean deleteObject(Object object)
	{
		Vector<Object> objects = new Vector<Object>(1);

		objects.addElement(object);
		return deleteObjects(objects);
	}

	/**
	 * To delete an object.
	 * @param  object the object to delete
	 * @return true if the request was correctly sent to the external data provider.
	 */
	public boolean deleteObjects(Vector<Object> objects)
	{
		// Set the response in running mode
		_response.setStatus(LyResponse.Status.RUNNING);

		// Encoding the request.
		// TODO

		// Get a valid connection
		LyJavaConnection connection = (LyJavaConnection)_provider.getConnection();

		// Send the request to the connection
		// TODO

		int				 i, n = objects.size();
		Object			 object;

		// Release the connection.
		_provider.releaseConnection(connection);

		for (i = 0; i < n; i++)
		{
			object = objects.elementAt(i);
			_dataSource.receiveResponse(_response, LyResponse.Status.RUNNING, object, null);
		}
		_dataSource.receiveResponse(_response, LyResponse.Status.COMPLETED, null, null);

		return true;
	} // end method deleteObjects

	/**
	 *  Methods for creation *
	 */
	/**
	 * To create a new object.
	 * @param  values the values of the new object
	 * @return true if the request was correctly sent to the external data provider.
	 */
	public boolean createObject(Object object)
	{
		// Set the response in running mode
		_response.setStatus(LyResponse.Status.RUNNING);

		// Encoding the request.
		// TODO

		// Get a valid connection
		LyJavaConnection connection = (LyJavaConnection)_provider.getConnection();

		// Send the request to the connection
		// TODO
		LyValueSet oldValues = validate(object);

		// Release the connection.
		_provider.releaseConnection(connection);

		_dataSource.receiveResponse(_response, LyResponse.Status.COMPLETED, object, oldValues);

		return true;
	}

	/**
	 *  Methods for modification *
	 */
	/**
	 * To set an object.
	 * @param  object the object to set
	 * @param  values the new values of the object
	 * @return true if the request was correctly sent to the external data provider.
	 */
	public boolean setObject(Object object)
	{
		// Set the response in running mode
		_response.setStatus(LyResponse.Status.RUNNING);

		// Encoding the request.
		// TODO

		// Get a valid connection
		LyJavaConnection connection = (LyJavaConnection)_provider.getConnection();

		// Send the request to the connection
		// TODO
		LyValueSet oldValues = validate(object);

		// Release the connection.
		_provider.releaseConnection(connection);

		_dataSource.receiveResponse(_response, LyResponse.Status.COMPLETED, object, oldValues);
		return true;
	}

	/**
	 * To set a set of objects.
	 * @param  objects the objects to set
	 * @param  values  the new values of the objects
	 * @return true if the request was correctly sent to the external data provider.
	 */
	public boolean setObjects(Vector<Object> objects)
	{
		throw new LyUnsupportedOperationException("BF AppEngine : "
			+ getEnvironment().getMessage("LY_ER_UNSUPPORTED_OPERATION", "LySqlRequestEncoder",
				"setObjects"));
	}
	
    /**
     * To validate the new values of given object. This call commits all changes made on
     * this object through setValue method.
     * @param  object Peer object that is validated.
     * @return Previous values for changed fields.
     */
    public LyValueSet validate(Object object)
    {
        return null; // Don't know how to get old values
    }
} // end class LyJavaRequestEncoder
