/*
 * LyLocalDataProvider.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.peer.local;

import java.util.*;
import leon.app.*;
import leon.data.*;
import leon.info.*;
import leon.info.location.*;
import leon.peer.*;


/**
 * This class represents the data provider used for local class infos.
 * @see LyClassInfo
 */
public final class LyLocalDataProvider extends LyDataProvider
{
	public static final String __VERSION =
		"$Id: LyLocalDataProvider.java 28422 2010-11-03 12:03:38Z ehelmste $";


	/** Internal counter to compute long or double ids. */
	protected long			   _counter;


	/**
	 * Constructs a local data provider.
	 * @param application current application instance
	 * @param location    this value is created in LyApplication.createProviders()
	 */
	public LyLocalDataProvider(LyApplication application, LyLocation location)
	{
		super(application, location);
		Date date = new Date();

		_counter = date.getTime();
	}


	/**
	 * This method is used to build all the class peers for all the classInfo. It must be
	 * specialized by subclasses. Basically, this method associates the dataProvider with local
	 * classes. A class is local if there is no binding.
	 * @param  classInfos Classes to validate.
	 * @return True if the validation was successful.
	 */
	public boolean validate(LyInfoList<LyClassInfo> classInfos)
	{
		LyClassInfo classInfo;
		int		    i, n = classInfos.getSize();

		for (i = 0; i < n; i++)
		{
			classInfo = classInfos.getInfo(i);

			if (((classInfo.getBinding() == null) || (classInfo.isLocal()))
					&& !classInfo.isAbstract()
					&& !classInfo.isVirtual())
				// The class is local and is not a super class.
				new LyObjectSource(this, classInfo);
		}

		return super.validate(classInfos);
	}

	/**
	 * Create a specific request encoder for the specified dataSource.
	 * @param  dataSource Data source for which is request encoder is requested.
	 * @return A LyLocalRequestEncoder for the specified dataSource.
	 */
	public LyRequestEncoder getRequestEncoder(LyDataSource dataSource)
	{
		return new LyLocalRequestEncoder(this, dataSource);
	}

	/**
	 * This class is a factory to create a LyConnection instance. Current implementation only return
	 * null.
	 * @return a new Connection instance. The connection instance is used in the specific LyRequest
	 *         encoder implementation.
	 */
	protected LyConnection createConnection()
	{
		return null;
	}

	/**
	 * Returns the next computed value for an id.
	 * @return the next free id.
	 */
	public long getNextId()
	{
		return _counter++;
	}

	/**
	 * This method is called when the application is going out of sleeping mode Default
	 * implementation call create the connection pool and wakes up the notifier.
	 * @see #sleep()
	 */
	public boolean wakeUp()
	{
		_lostConnection = false;
		return true;
	}
} // end class LyLocalDataProvider
