/*
 * LyMetaDataColumnInformation.java Copyright (c) 2000-2009, Lyria-W4. All rights
 * reserved.
 */

package leon.peer.sql;

import java.util.*;


/**
 */
public class LyMetaDataColumnInformation extends LyMetaDataInformation
{
	public static final String __VERSION =
		"$Id: LyMetaDataColumnInformation.java 31330 2011-06-07 08:59:33Z bangiole $";

	/**  */
	public static final short  NO_KEY = -2;

	/**  */
	short					   _keyOrder;

	/**  */
	String					   _tableName;

	/**  */
	short					   _dataType;

	/**  */
	String					   _typeName;

	/**  */
	int						   _columnSize;

	/**  */
	int						   _decimalDigits;

	/**  */
	int						   _position;

	/**  */
	int						   _octetLength;

	/**  */
	String					   _nullable;

	/**  */
	String					   _isNullable;

	/**  */
	String					   _def;

	/**  */
	String					   _dataTypeString;

	/**  */
	boolean					   _isPrimaryKey;

	/**  */
	boolean					   _isImportedKey;

	/**  */
	String					   _importedTableId;
	
	/**  */
	String					   _importedColumnId;
	
	/**  */
	String					   _importedKeyName;

	/**  */
	boolean					   _isExportedKey;

	/**  */
	String					   _exportedTableId;

	/**  */
	String					   _status;
	
	/** */
	Vector<_LyMetaDataColumnForeignKeyInformation>	_fkeys;

	/**
	 * Default Constructor.
	 * @param name    Name of this information.
	 * @param remarks Comments associated to this information.
	 */
	public LyMetaDataColumnInformation(String name, String remarks)
	{
		super(name, remarks);
		_tableName = "";
		_dataType = 0;
		_typeName = "";
		_columnSize = 0;
		_decimalDigits = 0;
		_nullable = "Null";
		_isNullable = "";
		_def = "";
		_dataTypeString = "";
		_keyOrder = -1;
		_status = "_";
		_isPrimaryKey = false;
		_isImportedKey = false;
		_isExportedKey = false;
		_octetLength = 0;
		_position = -1;
	}

	/**
	 * Accessor to the table name.
	 * @return Table name of this column.
	 */
	public String getTableName()
	{
		return _tableName;
	}

	/**
	 * Modifier of the table name of this column.
	 * @param tableName The table name of this column.
	 */
	public void setTableName(String tableName)
	{
		_tableName = tableName;
	}

	/**
	 * Accessor to the type of this column.
	 * @return The type of this column.
	 * @see    java.sql.Types
	 */
	public short getDataType()
	{
		return _dataType;
	}

	/**
	 * Modifier of the type of this column.
	 * @param dataType The type of this column.
	 * @see   java.sql.Types
	 */
	public void setDataType(short dataType)
	{
		_dataType = dataType;
		_dataTypeString = null;
	}

	/**
	 * Accessor to the string representation of the type of this column.
	 * @return The string representation of the type of this column.
	 */
	public String getDataTypeString()
	{
		if (_dataTypeString == null)
		{
			_dataTypeString = convertType(_dataType, _decimalDigits);
		}
		return _dataTypeString;
	}

	/**
	 * Accessor to the name of the type of this column.
	 * @return The name of the type of this column.
	 */
	public String getTypeName()
	{
		return _typeName;
	}

	/**
	 * Modifier of the name of the type of this column.
	 * @param typeName The name of the type of this column.
	 */
	public void setTypeName(String typeName)
	{
		_typeName = typeName;
	}

	/**
	 * Accessor to the size of this column.
	 * @return The size of this column.
	 */
	public int getColumnSize()
	{
		return _columnSize;
	}

	/**
	 * Modifier of the size of this column.
	 * @param columnSize The size of this column.
	 */
	public void setColumnSize(int columnSize)
	{
		_columnSize = columnSize;
	}

	/**
	 * Accessor to the number of decimals of this column.
	 * @return The number of decimals of this column.
	 */
	public int getDecimalDigits()
	{
		return _decimalDigits;
	}

	/**
	 * Modifier of the number of decimals of this column.
	 * @param decimalDigits The number of decimals of this column.
	 */
	public void setDecimalDigits(int decimalDigits)
	{
		_decimalDigits = decimalDigits;
	}

	/**
	 * Accessor to the nullable attribute.
	 * @return The nullable attribute.
	 */
	public String getNullable()
	{
		return _nullable;
	}

	/**
	 * Modifier of the nullable attribute.
	 * @param nullable The nullable attribute.
	 */
	public void setNullable(int nullable)
	{
		if (nullable == 0)
			_nullable = "Not Null";
		else
			_nullable = "Null";
	}

	/**
	 * Accessor to the isNullable attribute.
	 * @return The isNullable attribute.
	 */
	public String getIsNullable()
	{
		return _isNullable;
	}

	/**
	 * Modifier of the isNullbable attribute.
	 * @param isNullable The isNullbale attribute.
	 */
	public void setIsNullable(String isNullable)
	{
		_isNullable = isNullable;
	}

	/**
	 * Accessor to def attribute.
	 * @return The def attribute.
	 */
	public String getDef()
	{
		return _def;
	}

	/**
	 * Modifier of the def attribute.
	 * @param def The def attribute.
	 */
	public void setDef(String def)
	{
		_def = def;
	}

	/**
	 * Accessor to the position of this column.
	 * @return The position of this column.
	 */
	public int getPosition()
	{
		return _position;
	}

	/**
	 * Modifier of the position of this column.
	 * @param position The position of this column.
	 */
	public void setPosition(int position)
	{
		_position = position;
	}

	/**
	 * Accesor to the length of this column in bytes.
	 * @return The length of this column in bytes.
	 */
	public int getOctetLength()
	{
		return _octetLength;
	}

	/**
	 * Modifier of the length of this column in bytes.
	 * @param octetLength The length of this column in bytes.
	 */
	public void setOctetLength(int octetLength)
	{
		_octetLength = octetLength;
	}

	/**
	 * Accessor to key order of this column.
	 * @return the key order of this column.
	 */
	public short getKeyOrder()
	{
		return _keyOrder;
	}

	/**
	 * Modifier of the key order of this column.
	 * @param keyOrder the key order.
	 */
	public void setKeyOrder(short keyOrder)
	{
		_keyOrder = keyOrder;
	}

	/**
	 * Accessor to the primary key atttribute of this column.
	 * @return The primary key
	 */
	public short getPrimaryKey()
	{
		return _keyOrder;
	}

	/**
	 * Tests if this column is primary key or not.
	 * @return true if this column is primary key, false otherwise.
	 */
	public boolean isPrimaryKey()
	{
		return _isPrimaryKey;
	}

	/**
	 * To compute if this column is primary key or not.
	 */
	public void setIsPrimaryKey()
	{

		if ((_status != null) && (!_status.equals("_")))
			_status = "PK" + " " + _status;
		else
			_status = "PK";

		_isPrimaryKey = true;
	}
	
	/**
	 * To compute if this column is primary key or not.
	 */
	public void setIsPrimaryKey(boolean auto_increment)
	{
		if(auto_increment)
		{
			if ((_status != null) && (!_status.equals("_")))
				_status = "PK IDENTITY" + " " + _status;
			else
				_status = "PK IDENTITY";
			
			_isPrimaryKey = true;
		}
		else
			setIsPrimaryKey();
	}

	/**
	 * Modifier of the imported key attribute.
	 * @param importedTableId The table from which the key is imported.
	 */
	public void setIsImportedKey(String importedKeyName, String importedTableId, String importedColumnId)
	{	
		if(_fkeys == null)
		{
			_fkeys = new Vector<_LyMetaDataColumnForeignKeyInformation>();
		}
		
		if(importedKeyName == null || importedKeyName.trim().length() == 0)
			importedKeyName = "FK_"+importedTableId.toUpperCase()+"_"+importedColumnId.toUpperCase();
		
		_LyMetaDataColumnForeignKeyInformation fkey = 
			new _LyMetaDataColumnForeignKeyInformation(importedKeyName, importedTableId, importedColumnId);
		
		_fkeys.add(fkey);
			
		_importedTableId = importedTableId;
		_importedColumnId = importedColumnId;
		_importedKeyName = importedKeyName;
		
		if ((_status != null) && (!_status.equals("_")))
			_status += " FK " + importedKeyName + " (" + importedTableId + "/" + importedColumnId + ")";
		else
			_status = "FK " + importedKeyName + " (" + importedTableId + "/" + importedColumnId + ")";
		
		_isImportedKey = true;
	}

	/**
	 * Accessor to the table id of the imported key.
	 * @return The table id of the imported key.
	 */
	public String getImportedKeyTable()
	{
		return _importedTableId;
	}
	
	/**
	 * Accessor to the column id of the imported key.
	 * @return The column id of the imported key.
	 */
	public String getImportedKeyColumn()
	{
		return _importedColumnId;
	}
	
	/**
	 * Accessor to the name of the imported key.
	 * @return The name of the imported key.
	 */
	public String getImportedKeyName()
	{
		return _importedKeyName;
	}

	/**
	 * Test if this column is an imported key or not.
	 * @return true if this column is an imported key, false otherwise.
	 */
	public boolean isImportedKey()
	{
		return _isImportedKey;
	}

	/**
	 * Modifier of the exported table id of this column.
	 * @param exportedTableId The exported table id of this column.
	 */
	public void setIsExportedKey(String exportedTableId)
	{
		_exportedTableId = exportedTableId;
		//_status = "EK";
		_isExportedKey = true;
	}

	public Vector<_LyMetaDataColumnForeignKeyInformation> getForeignKeys()
	{
		return _fkeys;
	}
	
	/**
	 * Accessor to the exported table id of this column.
	 * @return The exported table id of this column.
	 */
	public String getExportedKeyTable()
	{
		return _exportedTableId;
	}

	/**
	 * Test if this column is an exported key or not.
	 * @return true if this column is an exported key, false otherwise.
	 */
	public boolean isExportedKey()
	{
		return _isExportedKey;
	}

	/**
	 * Accessor to the status of this column.
	 * @return The status of this column.
	 */
	public String getStatus()
	{
		return _status;
	}

	/**
	 * Compute a string representation of this column information.
	 * @return A string representation of this column information.
	 */
	public String toString()
	{
		StringBuffer buf = new StringBuffer();

		buf.append("\n\t\t\t Column  _name : " + getName());
		buf.append("\n\t\t\t Column  _remarks : " + getRemarks());
		buf.append("\n\t\t\t Column  _tableName : " + getTableName());
		buf.append("\n\t\t\t Column  _dataType : " + getDataType());
		buf.append("\n\t\t\t Column  _typeName : " + getTypeName());
		buf.append("\n\t\t\t Column  _columnSize : " + getColumnSize());
		buf.append("\n\t\t\t Column  _decimalDigits() : " + getDecimalDigits());
		buf.append("\n\t\t\t Column  _nullable() : " + getNullable());
		buf.append("\n\t\t\t Column  _isNullable() : " + getIsNullable());
		buf.append("\n\t\t\t Column  _def() : " + getDef());
		buf.append("\n\t\t\t Column  _keyOrder : " + getKeyOrder());
		buf.append("\n\t\t\t Column  _isPrimaryKey : " + isPrimaryKey());
		buf.append("\n\t\t\t Column  __position : " + getPosition());
		buf.append("\n\t\t\t Column  _octetLength : " + getOctetLength());

		buf.append("\n\n");
		return buf.toString();
	}

	/**
	 * Internal method to convert a SQL type in a string representation.
	 * @param  dataType a SQL typr
	 * @return A string representation of the given SQL type.
	 * @see    java.sql.Types
	 */
	private String convertType(short dataType, int decimalDigit)
	{
		switch (dataType)
		{
		//not exist if java < 1.2
		//case(java.sql.Types.ARRAY) :
		//      return "ARRAY";
		case (java.sql.Types.BIGINT):
			return "BIGINT";
		case (java.sql.Types.BINARY):
			return "BINARY";
		case (java.sql.Types.BIT):
			return "BIT";
		//not exist if java < 1.2
		//case(java.sql.Types.BLOB) :
		//      return "BLOB";
		case (java.sql.Types.CHAR):
			return "CHAR";
		//not exist if java < 1.2
		//case(java.sql.Types.CLOB) :
		//      return "CLOB";
		case (java.sql.Types.DATE):
			return "DATE";
		case (java.sql.Types.DECIMAL):
			return "DECIMAL";
		//not exist if java < 1.2
		//case(java.sql.Types.DISTINCT) :
		//      return "DISTINCT";
		case (java.sql.Types.DOUBLE):
			return "DOUBLE";
		case (java.sql.Types.FLOAT):
			return "FLOAT";
		case (java.sql.Types.INTEGER):
			return "INT";
		//not exist if java < 1.2
		//case(java.sql.Types.JAVA_OBJECT) :
		//      return "JAVA_OBJECT";
		case (java.sql.Types.LONGVARBINARY):
			return "LONGVARBINARY";
		case (java.sql.Types.LONGVARCHAR):
			return "LONGVARCHAR";
		case (java.sql.Types.NULL):
			return "NULL";
		case (java.sql.Types.NUMERIC):
		{
			if (decimalDigit > 0)
				return "NUMERIC";
			return "INT";
		}
		case (java.sql.Types.OTHER):
			return "OTHER";
		case (java.sql.Types.REAL):
			return "FLOAT";
		//not exist if java < 1.2
		//case(java.sql.Types.REF) :
		//      return "REF";
		case (java.sql.Types.SMALLINT):
			return "SHORT";
		//not exist if java < 1.2
		//case(java.sql.Types.STRUCT) :
		//      return "STRUCT";
		case (java.sql.Types.TIME):
			return "TIME";
		case (java.sql.Types.TIMESTAMP):
			return "TIMESTAMP";
		case (java.sql.Types.TINYINT):
			return "SHORT";
		case (java.sql.Types.VARBINARY):
			return "VARBINARY";
		case (java.sql.Types.VARCHAR):
			return "VARCHAR";
		default:
			return null;
		}
	} // end method convertType
	
	public class _LyMetaDataColumnForeignKeyInformation
	{
		public String	_fkName;
		public String	_importedTableName;
		public String	_importedColName;
		
		public	_LyMetaDataColumnForeignKeyInformation(String fkName, String importedTableName, String importedColName)
		{
			_fkName = fkName;
			_importedTableName = importedTableName;
			_importedColName = importedColName;
		}
	}
} // end class LyMetaDataColumnInformation
