/*
 * LyMetaDataInformation.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.peer.sql;

import java.sql.*;
import java.util.*;


/**
 */
public class LyMetaDataInformation
{
	public static final String					  __VERSION =
		"$Id: LyMetaDataInformation.java 28997 2010-12-23 10:17:15Z bmichel $";

	/**  */
	DatabaseMetaData							  _metaData;

	/**  */
	String										  _productName;

	/**  */
	String										  _productVersion;

	/**  */
	String										  _driverName;

	/**  */
	String										  _driverVersion;

	/**  */
	String										  _sqlKeywords;

	/**  */
	int											  _driverMajorVersion;

	/**  */
	int											  _driverMinorVersion;

	/**  */
	int											  _maxConnections;

	/**  */
	int											  _defaultTransactionIsolation;

	/** all MetaDataTableInformations for this base. */
	Hashtable<String, LyMetaDataTableInformation> _tables;

	/** The name of the information. */
	protected String							  _name;

	/** A comment associated to this information. */
	protected String							  _remarks;

	/**
	 * Default Constructor.
	 *
	 * @param name    Name of this information
	 * @param remarks Comments associated to this information.
	 */
	public LyMetaDataInformation(String name, String remarks)
	{
		_name = name;
		_remarks = remarks;
	}

	/**
	 * Constructor.
	 *
	 * @param metaData     MetaData informations.
	 * @param databaseName Name of the database.
	 */
	public LyMetaDataInformation(DatabaseMetaData metaData, String databaseName)
	{
		_name = databaseName;
		_remarks = "";
		_metaData = metaData;
		init();
	}

	/**
	 * Internal method to initialize an instance.
	 */
	private void init()
	{
		try
		{
			_productName = _metaData.getDatabaseProductName();
			_productVersion = _metaData.getDatabaseProductVersion();
			_driverName = _metaData.getDriverName();
			_driverVersion = _metaData.getDriverVersion();
			_sqlKeywords = _metaData.getSQLKeywords();
			_driverMajorVersion = _metaData.getDriverMajorVersion();
			_driverMinorVersion = _metaData.getDriverMinorVersion();
			_maxConnections = _metaData.getMaxConnections();
			_defaultTransactionIsolation = _metaData.getDefaultTransactionIsolation();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	/**
	 * Accessor to the name.
	 *
	 * @return The name of this information.
	 */
	public String getName()
	{
		return _name;
	}

	/**
	 * Accessor to the comments.
	 *
	 * @return The comments associated to this information.
	 */
	public String getRemarks()
	{
		return _remarks;
	}

	/**
	 * Accessor to the product name.
	 *
	 * @return The product name of the database.
	 */
	public String getProductName()
	{
		return _productName;
	}

	/**
	 * Accessor to the product version.
	 *
	 * @return The product version of the database.
	 */
	public String getProductVersion()
	{
		return _productVersion;
	}

	/**
	 * Accessor to the driver name.
	 *
	 * @return The driver name.
	 */
	public String getDriverName()
	{
		return _driverName;
	}

	/**
	 * Accessor to the driver version.
	 *
	 * @return The driver version.
	 */
	public String getDriverVersion()
	{
		return _driverVersion;
	}

	/**
	 * Accessor to the SQL keywords.
	 *
	 * @return The SQL keywords.
	 */
	public String getSqlKeyWords()
	{
		return _sqlKeywords;
	}

	/**
	 * To set the list of the tables of this database.
	 *
	 * @param tables The tables description list of this database.
	 */
	public void setTables(Hashtable<String, LyMetaDataTableInformation> tables)
	{
		_tables = tables;
	}

	/**
	 * @return A string representation of this database information.
	 */
	public String toString()
	{
		StringBuffer buf = new StringBuffer();

		buf.append("\n\t Base  _name : " + getName());
		buf.append("\n\t Base  _remarks : " + getRemarks());
		buf.append("\n\t Base  _productName : " + _productName);
		buf.append("\n\t Base  _productVersion : " + _productVersion);
		buf.append("\n\t Base  _driverName : " + _driverName);
		buf.append("\n\t Base  _driverVersion : " + _driverVersion);
		buf.append("\n\t Base  _sqlKeywords : " + _sqlKeywords);
		buf.append("\n\t Base  _driverMajorVersion : " + _driverMajorVersion);
		buf.append("\n\t Base  _driverMinorVersion : " + _driverMinorVersion);
		buf.append("\n\t Base  _maxConnections : " + _maxConnections);
		buf.append("\n\t Base  _defaultTransactionIsolation : " + _defaultTransactionIsolation);
		buf.append("\n");
		return buf.toString();
	}

	/**
	 * To get table descriptions of this database. Keys are the name of the tables.
	 *
	 * @return A list of the table description.
	 */
	
	public Hashtable<String, LyMetaDataTableInformation> getTables()
	{
		return _tables;
	}

} // end class LyMetaDataInformation
