/*
 * LyMetaDataTableInformation.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.peer.sql;

import java.util.*;


/**
 */
public class LyMetaDataTableInformation extends LyMetaDataInformation
{
	public static final String								 __VERSION =
		"$Id: LyMetaDataTableInformation.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**  */
	protected Hashtable<String, LyMetaDataColumnInformation> _columns;

	/** A comment associated to this information. */
	protected String										 _type;

	/**
	 * Default Constructor.
	 *
	 * @param name    Name of the information
	 * @param remarks Comments associated to this information
	 */
	public LyMetaDataTableInformation(String name, String remarks)
	{
		super(name, remarks);

		_columns = new Hashtable<String, LyMetaDataColumnInformation>();
	}

	/**
	 * Add a column information to this table information.
	 *
	 * @param column the column information.
	 */
	public void addColumn(LyMetaDataColumnInformation column)
	{
		String name = column.getName().toLowerCase();

		_columns.put(name, column);
	}

	/**
	 * Get a column information by this name.
	 *
	 * @param  name The name of the searched column information.
	 * @return The column information corresponding to the given name if exists, null otherwise.
	 */
	public LyMetaDataColumnInformation getColumn(String name)
	{
		if (name == null)
			return null;
		return _columns.get(name.toLowerCase());
	}

	/**
	 * Get all column informations of this table information.
	 *
	 * @return An enumeration of the column informations of this table.
	 */
	public Enumeration<LyMetaDataColumnInformation> getColumns()
	{
		return _columns.elements();
	}

	/**
	 * Get the hashtable of the column informations. Keys are the name of the columns.
	 *
	 * @return The hashtable of the columns.
	 */
	public Hashtable<String, LyMetaDataColumnInformation> getColumnsHashTable()
	{
		return _columns;
	}

	/**
	 * Accessor to the type of the table.
	 *
	 * @return the type of the table.
	 */
	public String getType()
	{
		return _type;
	}

	/**
	 * Modifier of the type of the table.
	 *
	 * @param type the type of the table.
	 */
	public void setType(String type)
	{
		_type = type;
	}

} // end class LyMetaDataTableInformation
