/*
 * LySqlColumnField.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.peer.sql;

import java.sql.*;
import leon.peer.util.*;


/**
 * This class provides the SQL implementation of LyField.
 */
public class LySqlColumnField extends LyColumnField
{
	public static final String __VERSION =
		"$Id: LySqlColumnField.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** The SQL type of the column to reprenset a date. The values are defined in java.sql.Types. */
	protected int			   _dateType;
	
	/** The binding of the class wich contains this field, may be null if not specified **/
	protected String		   _classBinding;
	
	/**
	 * Default Constructor.
	 */
	public LySqlColumnField()
	{
		super();

		_dateType = Types.NULL;
		_classBinding =null;
	}

	/**
	 * To set the date type of this field.
	 * @param type the date type of this field.
	 */
	public void setDateType(int type)
	{
		_dateType = type;
	}

	/**
	 * To get the date type of this field.
	 * @return the date type of this field.
	 */
	public int getDateType()
	{
		return _dateType;
	}

	/**
	 * To get the CLASS binding -table on top of this field binding-.
	 * @return specified the class binding (table name) in RDBMS or null if not specified
	 */
	public String getClassBinding()
	{
		return _classBinding;
	}

	/**
	 * To set the CLASS binding -table on top of this field binding-.
	 * @param specified the class binding (table name) in RDBMS
	 */
	public void setClassBinding(String classBinding)
	{
		_classBinding = classBinding;
	}

	/**
	 * Gets qualified identifier of this field. Defaults return getId()
	 * @return Identifier of this field.
	 */
	public String getFullId()
	{
		if (_classBinding == null)
			return getId();
		else
			return (new StringBuffer(_classBinding).append('.').append(getId())).toString();
	}

	/**
	 * Returns this peer field as a String.
	 * @return This peer field returned as a string.
	 */
	public String toString()
	{
		return super.toString() + " _dateType = " + _dateType + (_classBinding != null ? " _classBinding = " + _classBinding : "");
	}
} // end class LySqlColumnField
