/*
 * LySqlRequestEncoder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.peer.sql;

import java.util.*;
import leon.misc.*;


/**
 * Store debug info about the building of the sql request a task is a step in the building of the
 * sql request.
 *
 * @author nbo
 */
public class LySqlRequestInfoDebugTask
{
	public static final short				  TASK_STATUS_IN_PROGRESS = 1;
	public static final short				  TASK_STATUS_COMPLETED = 2;
	public static final String				  NEW_LINE_TOKEN = "\t\t\t\t\n";

	public static int						  _counter_seq = 0;
	protected List<LySqlRequestInfoDebugTask> _children;
	protected StringBuffer					  _info;
	protected StringBuffer					  _sqlFragment;
	protected short							  _status = TASK_STATUS_IN_PROGRESS;
	protected LySqlRequestInfoDebugTask		  _parent;
	protected int							  _countInfo = -1;
	protected int							  _counter = -1;

	public LySqlRequestInfoDebugTask()
	{
		_counter = _counter_seq++;
	}

	/**
	 * ad.
	 *
	 * @param task
	 */
	public void addTask(LySqlRequestInfoDebugTask task)
	{
		if (task == null)
			return;
		if (_children == null)
			_children = new ArrayList<LySqlRequestInfoDebugTask>();
		_children.add(task);
		task._parent = this;
	}

	public void setStatus(short status)
	{
		_status = status;
	}

	/**
	 * Add degug information about the task.
	 *
	 * @param info any textual information for debug
	 */
	public void addInfo(String info)
	{
		if (info == null)
			return;

		if (_info == null)
			_info = new StringBuffer();
		else
			_info.append(NEW_LINE_TOKEN);
		_info.append(++_countInfo).append(" : ");
		_info.append(info);
	}

	/**
	 * Add SQL description for debug.
	 *
	 * @param sql new sql fragment built by the process
	 */
	public void addSql(String sql)
	{
		if (sql == null)
			return;

		if (_sqlFragment == null)
			_sqlFragment = new StringBuffer();
		else
			_sqlFragment.append(NEW_LINE_TOKEN);
		_sqlFragment.append(_countInfo).append(" : ");
		_sqlFragment.append(sql);
	}

	/**
	 * return parent task.
	 *
	 * @return
	 */
	public LySqlRequestInfoDebugTask getParentTask()
	{
		return _parent;
	}

	/**
	 * return root task.
	 *
	 * @return
	 */
	public LySqlRequestInfoDebugTask getRootTask()
	{
		if (_parent == null)
			return this;
		else
			return _parent.getRootTask();
	}

	protected String getDebugInfo(int indent)
	{
		StringBuffer indentBuf = new StringBuffer();

		for (int s = 0; s < indent; s++)
			indentBuf.append("   ");

		StringBuffer sb = new StringBuffer();

		sb.append(indentBuf).append("LySqlRequestInfoDebugTask [").append(_counter).append("]\n");
		if (_info != null)
		{
			sb.append(indentBuf).append("[INFO] :\n");
			sb.append(indentBuf).append(LyString.replace(_info.toString(), NEW_LINE_TOKEN,
					"\n" + indentBuf.toString()));
			sb.append('\n');
		}
		if (_sqlFragment != null)
		{
			sb.append(indentBuf).append("[SQL Fragments] :\n");
			sb.append(indentBuf).append(LyString.replace(_sqlFragment.toString(), NEW_LINE_TOKEN,
					"\n" + indentBuf.toString()));
			sb.append('\n');
		}

		if (_children != null)
		{
			sb.append(indentBuf).append("[CHILDREN] :\n");
			for (Iterator<LySqlRequestInfoDebugTask> iter = _children.iterator(); iter.hasNext();)
			{
				LySqlRequestInfoDebugTask task = iter.next();

				sb.append(task.getDebugInfo(indent + 1));
				sb.append(indentBuf).append("   -----------\n");
			}

		}

		return sb.toString();
	} // end method getDebugInfo

	public int getCounter()
	{
		return _counter;
	}

	public String toString()
	{
		return getDebugInfo(0);
	}
} // end class LySqlRequestInfoDebugTask
