/*
 * LySqlSpecificEncoder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.peer.sql;

import java.sql.*;
import leon.data.*;


/**
 * This class is used to encode and decode specific values for a SQL location. Instances of this
 * class must be register in LySqlDataProvider with the method addSpecificEncoder(LyField,
 * LySqlSpecificEncoder) in the application behavior method registerSpecificEncoders.
 * @version $Id: LySqlSpecificEncoder.java 29006 2010-12-23 17:44:46Z bmichel $
 * @author  Lyria
 * @see     LySqlRequestEncoder
 * @see     LySqlConnection
 * @see     LySqlDataProvider
 * @see     LyDataApplicationBehavior
 */
public interface LySqlSpecificEncoder
{
	String __VERSION = "$Id: LySqlSpecificEncoder.java 29006 2010-12-23 17:44:46Z bmichel $";

	/**
	 * To encode a specific value for the specified peer field.
	 * @param  field The field for which we want to encode the value.
	 * @param  value The value to encode. The class of this object depends on internal
	 *               representation of values for the field. It will be java.util.Date, Object[] or
	 *               String.
	 * @return The string representation of the value that will be inserted in the sql request sent
	 *         to the RDBMS.
	 */
	String getSqlValue(LyField field, Object value);

	/**
	 * To decode a value given by the RDBMS.
	 * @param  rs         The ResultSet that contains the value to decode.
	 * @param  field      The field for which we want to decode the value.
	 * @param  index      The position of the column in the ResultSet.
	 * @param  valueClass The class of the result.
	 * @return The object of the given class representing the decoded value.
	 */
	Object getPeerValue(ResultSet rs, LyField field, int index, Class<?> valueClass)
		throws LySqlUnsupportedConversionException,
			SQLException;
}