/*
 * LyAccessConnection.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.peer.sql.access;

import java.sql.*;
import java.util.*;
import javax.sql.*;
import leon.app.*;
import leon.peer.sql.*;


/**
 * This class is used as a connection for MySql relational database.
 * @version $Id: LyAccessConnection.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 * @see     LySqlConnection
 */
public class LyAccessConnection extends LySqlConnection
{
	public static final String __VERSION =
		"$Id: LyAccessConnection.java 28422 2010-11-03 12:03:38Z ehelmste $";

	protected String		   _databaseName;

	/**
	 * Constructor.
	 * @param application  The application of this connection
	 * @param name         Name of the connection
	 * @param dataSource   JDBC DataSource providing the JDBC connection.
	 * @param databaseName Name of the database.
	 */
	public LyAccessConnection(LyApplication application, String name,
		DataSource dataSource, String databaseName)
	{
		super(application, name, dataSource);
		_databaseName = databaseName;
	}

	/**
	 * Initialisation of the data of the instance.
	 */
	protected void init()
	{
		super.init();
		_databaseName = null;
	}

	/**
	 * To get properties of the connection before opening it.
	 * @return Properties of this connection used to open it.
	 */
	public Properties getProperties()
	{
		Properties properties = super.getProperties();

		if (_databaseName != null)
			properties.put("DBNAME", _databaseName);

		return properties;
	}

	/**
	 * Open the physical connection to the database.
	 * @param  username user id.
	 * @param  password user password.
	 * @return true : connection established, false : connection refused
	 */
	public boolean open(String username, String password)
	{
		return open(username, password, null);
	}

	/**
	 * Open the physical connection to the database.
	 * @param  username   user id.
	 * @param  password   user password.
	 * @param  driverName The driver class name.
	 * @return true : connection established, false : connection refused
	 */
	public boolean open(String username, String password, String driverName)
	{
		boolean result = super.open(username, password, driverName);

		if ((result) && (_dataSource != null))
		{
			try
			{
				Connection connection = getConnection();
				
				connection.setCatalog(_databaseName);
			}
			catch (SQLException e)
			{
				handleError(e);
				result = false;
			}
		}

		return result;
	}
} // end class LyAccessConnection
