/*
 * LyAccessDataProvider.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.peer.sql.access;

import java.sql.*;
import java.util.*;

import leon.app.*;
import leon.data.*;
import leon.info.location.*;
import leon.misc.*;
import leon.peer.*;
import leon.peer.sql.*;


/**
 * This class is used by DataSources to transmit their requests to an instance of
 * MySqlRequestEncoder.
 * @version $Id: LyAccessDataProvider.java 32077 2011-08-25 07:59:20Z bangiole $
 * @author  Lyria
 * @see     LyDataProvider
 * @see     LySqlDataProvider
 * @see     LyAccessRequestEncoder
 * @see     LyAccessConnection
 */
public class LyAccessDataProvider extends LySqlDataProvider
{
	public static final String __VERSION =
		"$Id: LyAccessDataProvider.java 32077 2011-08-25 07:59:20Z bangiole $";

	/**
	 * Default constructor. Initialization is done with the init method.
	 * @param application the application of this provider.
	 * @param location    the info location defining this provider.
	 */
	public LyAccessDataProvider(LyApplication application, LyLocation location)
	{
		super(application, location);
	}

	/**
	 * Create a new connection for this data provider.
	 * @return the created connection
	 */
	protected LyConnection createConnection()
	{
		LySqlConnection connection;

		connection = new LyAccessConnection(_application, _connectionName, _dataSource,
				_databaseName);

		// Openning the connection.
		boolean connectionOpened;

		if (getEnvironment().testResource("LY_NO_DRIVER_MANAGER"))
			connectionOpened = connection.open(_username, _password, _driverName);
		else
			connectionOpened = connection.open(_username, _password);

		if (!connectionOpened)
			return null;

		return connection;
	}

	/**
	 * To get the name of the catalog to use with metadata.
	 * @return the name of the catalog.
	 */
	protected String getCatalogPattern()
	{
		return null;
	}

	/**
	 * To get the name of the schema to use with metadata.
	 * @return the name of the schema.
	 */
	protected String getSchemaPattern()
	{
		return null;
	}

	/**
	 * Create a specific request encoder for the specified dataSource.
	 * @param  dataSource reference to the dataSource associated to this request encoder.
	 * @return the specific LyRequestEncoder for the specified dataSource.
	 */
	public LyRequestEncoder getRequestEncoder(LyDataSource dataSource)
	{
		LyAccessRequestEncoder encoder = new LyAccessRequestEncoder(this, dataSource, _valueSep);

		encoder.setTrim(_trim);
		return encoder;
	}

	@Override
	protected List<Object[]> getPrimaryKeys(DatabaseMetaData metaData, String catalog,
			String schema, String tableName)
	{
		LyEnvironment	environment = _application.getEnvironment();
		String			name;
		Short			keyseq;
		ResultSet		rs = null;
		Object[]		result;
		List<Object[]>	resultList = null;
		
		try
		{
			rs = metaData.getIndexInfo(catalog,schema, tableName, true,true);
			resultList = new ArrayList<Object[]>();
			
			while( rs.next())
			{
				String idx = rs.getString("INDEX_NAME");
				if( idx != null)
				{
					//Note: index "PrimaryKey" is Access DB specific
					//		      other db server has diff. index syntax.
					if( idx.equalsIgnoreCase("PrimaryKey"))
					{
						name = rs.getString("COLUMN_NAME");
						if (environment.isDebugLevel(LyEnvironment.DEBUG_PEER))
							environment.logOut("\t\t" + name);
						keyseq = rs.getShort("ORDINAL_POSITION");
						result = new Object[2];
						result[0] = name;
						result[1] = keyseq;
						resultList.add(result);
					}
				}
			}
		}
		catch (Exception ex)
		{
			if (environment.isDebugLevel(LyEnvironment.DEBUG_DBMS))
				environment.writeException(ex);
		}
		finally
		{
			if (rs != null)
			{
				try
				{
					rs.close();
				}
				catch (SQLException e)
				{
				}
				rs = null;
			}
		}
		return resultList;
	}
	
	
} // end class LyAccessDataProvider
