/**
  * LyHsqldbDataProvider.java
  * Copyright (c) 2000-2009, Lyria-W4.
  * All rights reserved.
  */

package leon.peer.sql.hsqldb;

import leon.app.*;
import leon.data.*;
import leon.info.location.*;
import leon.peer.*;
import leon.peer.sql.*;

/**
 * This class is used by DataSources to transmit their requests to an instance of SqlRequestEncoder.
 * @version $Id: LyHsqldbDataProvider.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 * @see     LyDataProvider
 * @see     LySqlDataProvider
 * @see     LySqlRequestEncoder
 */
public class LyHsqldbDataProvider extends LySqlDataProvider
{
	/**
	 * Default constructor Initialization is done with the init method.
	 * @param application the application of this provider
	 * @param location    the info definition of this provider
	 */
	public LyHsqldbDataProvider(LyApplication application, LyLocation location)
	{
		super(application, location);

	}
	
	/**
	 * Get an encoder for the given dataSource and this provider.
	 * @param  dataSource the dataSource
	 * @return the request encoder for the specified datasource.
	 */
	public LyRequestEncoder getRequestEncoder(LyDataSource dataSource)
	{
		LyHsqldbRequestEncoder encoder = new LyHsqldbRequestEncoder(this, dataSource, _valueSep);

		encoder.setTrim(_trim);
		return encoder;
	}
	
	/**
	 * This method is a factory to create a LyConnection instance.
	 * @return a new Connection instance. The connection instance is used in the specific LyRequest
	 *         encoder implementation.
	 * @see    #getConnection
	 * @see    #releaseConnection
	 */
	protected LyConnection createConnection()
	{
		LySqlConnection connection;

		connection = new LyHsqldbConnection(_application, _connectionName, _dataSource);

		// Openning the connection.
		boolean connectionOpened;

		if (getEnvironment().testResource("LY_NO_DRIVER_MANAGER"))
			connectionOpened = connection.open(_username, _password, _driverName);
		else
			connectionOpened = connection.open(_username, _password);

		if (!connectionOpened)
			return null;

		return connection;
	}
}
