/**
  * LyHsqldbRequestEncoder.java
  * Copyright (c) 2000-2009, Lyria-W4.
  * All rights reserved.
  */

package leon.peer.sql.hsqldb;

import java.sql.*;
import java.util.*;

import leon.data.*;
import leon.misc.*;
import leon.peer.*;
import leon.peer.sql.*;
import leon.peer.util.*;

/**
 * This class is used to send requests to a database connection and to receive its answers.
 *
 * <p>It uses an LySqlRequestEncoder to encode and decode the values of requests and answers.</p>
 *
 * @version $Id: LyHsqldbRequestEncoder.java 28997 2010-12-23 10:17:15Z bmichel $
 * @author  Lyria
 * @see     LySqlRequestEncoder
 */
public class LyHsqldbRequestEncoder extends LySqlRequestEncoder
{
	/**
	 * Constructor.
	 * @param classInfo ClassInfo used by the sql request encoder.
	 */
	public LyHsqldbRequestEncoder(LyDataProvider provider, LyDataSource dataSource, String valueSep)
	{
		super(provider, dataSource, valueSep);
	}
	
	/**
	 * Initialisation of the data of the instance.
	 */
	protected void init()
	{
		super.init();
		if (getEnvironment().testResource("LY_DBMS_QUOTE_TABLES_AND_COLUMNS"))
		{
		//	QUOTE_CHAR_LEFT = "\"";
	//		QUOTE_CHAR_RIGHT = "\"";
		}
	}
	
	
	/**
	 * This method is used after an insert request to complete the object if some values or the
	 * identifier is set by the database.
	 */
	protected void completeObject(Object object, LySqlConnection connection)
	{
		LyEnvironment environment = getEnvironment();

		if (object == null)
			throw new IllegalArgumentException("BF AppEngine : "
				+ environment.getMessage("LY_ER_CANT_BE_NULL", object));
		if (connection == null)
			throw new IllegalArgumentException("BF AppEngine : "
				+ environment.getMessage("LY_ER_CANT_BE_NULL", connection));

		// If the object has no identifier, it may be an identity.
		LyValueSet   newValues = ((LyRowObject)object).getNewValues();
		LyTableClass tableClass = ((LyRowObject)object).getTableClass();
		Vector<LyField>	idFields = tableClass.getIdFields();

		if (idFields.size() != 1)
			// If there are several id fields, they can't be identity.
			return;

		LyColumnField columnField = (LyColumnField)idFields.firstElement();

		if ((newValues != null) && (newValues.contains(columnField)))
			// The identifier was not set by the database.
			return;

		// Create the request.
		StringBuffer request_text = new StringBuffer();

		request_text.append("CALL IDENTITY()");

		// Send the request to the connection
		ResultSet rs = connection.sendQueryRequest(request_text.toString());

		if (rs == null)
			// An error occured while sending the request to the database
			environment.logError("An error occured while sending the request\n"
				+ request_text + "\nto the database.");
		else
		{
			// The column in the database is an integer.
			try
			{
				rs.next();
				Object value = getPeerValue(rs, columnField, 1, columnField.getValueClass());

				((LyRowObject)object).setValue(columnField, value);
			}
			catch (Exception e)
			{
				environment.writeException(e);
			}
			finally
			{
				try
				{
					rs.close();
				}
				catch (SQLException sqle)
				{
					if (environment.isDebugLevel(LyEnvironment.DEBUG_DBMS)
							&& environment.isDebugLevel(LyEnvironment.DEBUG_TRACE))
						environment.writeException(sqle);
				}
			}
		}
	} // end method completeObject
}
