/*
 * LyMySqlConnection.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.peer.sql.mysql;

import java.sql.*;
import java.util.*;
import javax.sql.*;
import leon.app.*;
import leon.data.*;
import leon.misc.*;
import leon.peer.sql.*;


/**
 * This class is used as a connection for MySql relational database.
 * @version $Id: LyMySqlConnection.java 40675 2012-09-05 16:26:11Z jlsaouli $
 * @author  Lyria
 * @see     LySqlConnection
 */
public class LyMySqlConnection extends LySqlConnection
{
	public static final String __VERSION =
		"$Id: LyMySqlConnection.java 40675 2012-09-05 16:26:11Z jlsaouli $";

	/**  */
	protected String		   _databaseName;

	/**
	 * Constructor.
	 * @param application  The application of this connection
	 * @param name         Name of the connection
	 * @param databaseName Name of the database.
	 */
	public LyMySqlConnection(LyApplication application, String name, String databaseName)
	{
		this(application, name, null, databaseName);
	}

	/**
	 * Constructor.
	 * @param application  Application of the data provider that manages this connection.
	 * @param name         Name of the database.
	 * @param dataSource   JDBC DataSource providing the JDBC connection.
	 * @param databaseName Name of the database.
	 */
	public LyMySqlConnection(LyApplication application, String name,
		DataSource dataSource, String databaseName)
	{
		super(application, name, dataSource);
		_databaseName = databaseName;
	}

	/**
	 * Initialisation of the data of the instance.
	 */
	protected void init()
	{
		super.init();
		_databaseName = null;
	}

	/**
	 * To get properties of the connection before opening it.
	 * @return Properties of this connection used to open it.
	 */
	public Properties getProperties()
	{
		Properties properties = super.getProperties();

		if (_databaseName != null)
			properties.put("DBNAME", _databaseName);

		// add useUnicode and charset if needed
		if (!_name.contains("useUnicode")
				&& LyString.DEFAULT_CHARSET.equalsIgnoreCase("UTF-8"))
			properties.put("useUnicode", "true");
		
		if (!_name.contains("characterEncoding"))
				properties.put("characterEncoding", LyString.DEFAULT_CHARSET);
		return properties;
	}

	/**
	 * Open the physical connection to the database.
	 * @param  username user id.
	 * @param  password user password.
	 * @return true : connection established, false : connection refused
	 */
	public boolean open(String username, String password)
	{
		return open(username, password, null);
	}

	/**
	 * Open the physical connection to the database.
	 * @param  username   user id.
	 * @param  password   user password.
	 * @param  driverName The driver class name.
	 * @return true : connection established, false : connection refused
	 */
	public boolean open(String username, String password, String driverName)
	{
		boolean result = super.open(username, password, driverName);

		if ((result) && (_dataSource != null))
		{
			try
			{
				Connection connection = getConnection();
				
				if ((_databaseName != null) && (_databaseName.length() > 0))
					connection.setCatalog(_databaseName);
			}
			catch (SQLException e)
			{
				handleError(e);
				result = false;
			}
		}

		return result;
	}
	
	protected void setLimit(int limit)
    {
		int		maxLimit = LyPageWorkSpace.MAX_LIMIT;
		
    	if (limit > maxLimit)
    		limit %= maxLimit;
    	
    	super.setLimit(limit);
    }
} // end class LyMySqlConnection
