/*
 * LyMySqlDataProvider.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.peer.sql.mysql;

import leon.app.*;
import leon.data.*;
import leon.info.location.*;
import leon.peer.*;
import leon.peer.sql.*;


/**
 * This class is used by DataSources to transmit their requests to an instance of
 * MySqlRequestEncoder.
 * @version $Id: LyMySqlDataProvider.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 * @see     LyDataProvider
 * @see     LySqlDataProvider
 * @see     LyMySqlRequestEncoder
 * @see     LyMySqlConnection
 */
public class LyMySqlDataProvider extends LySqlDataProvider
{
	public static final String __VERSION =
		"$Id: LyMySqlDataProvider.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Default constructor Initialization is done with the init method.
	 * @param application the application of this provider
	 * @param location    the info definition of this provider
	 */
	public LyMySqlDataProvider(LyApplication application, LyLocation location)
	{
		super(application, location);

	}

	/**
	 * Create a new connection for this data provider.
	 * @return the created connection
	 */
	protected LyConnection createConnection()
	{
		LySqlConnection connection;

		connection = new LyMySqlConnection(_application, _connectionName, _dataSource,
				_databaseName);

		// Openning the connection.
		boolean connectionOpened;

		if (getEnvironment().testResource("LY_NO_DRIVER_MANAGER"))
			connectionOpened = connection.open(_username, _password, _driverName);
		else
			connectionOpened = connection.open(_username, _password);

		if (!connectionOpened)
			return null;

		return connection;
	}

	/**
	 * Get an encoder for the given dataSource and this provider.
	 * @param  dataSource the dataSource
	 * @return the request encoder for the specified datasource.
	 */
	public LyRequestEncoder getRequestEncoder(LyDataSource dataSource)
	{
		LyMySqlRequestEncoder encoder = new LyMySqlRequestEncoder(this, dataSource, _valueSep);

		encoder.setTrim(_trim);
		return encoder;
	}

	/**
	 * Tells if the current SQL data provider refers to a database that supports the boolean type.
	 * @return true
	 */
	protected boolean booleanTypeSupported()
	{
		return true;
	}
} // end class LyMySqlDataProvider
