/*
 * LyJoinRequestEncoder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.peer.union;

import java.util.*;
import leon.data.*;
import leon.info.*;


/**
 * This interface defines methods to encode a request made on a data source and decode results to
 * give a response for data providers supporting join requests.
 * @see LyJoinDataProvider
 * @see LyUnionRequestEncoder
 */
public interface LyJoinRequestEncoder
{
	String __VERSION = "$Id: LyJoinRequestEncoder.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Load a number of objects matching a given filter and sorted by a specified sort.
	 * @param  encoder The union encoder demanding the load of the objects.
	 * @param  fields  Fields to load (if null, load all fields).
	 * @param  sort    Gives sorting criterion, or null if default sort is used.
	 * @param  filter  Specifies filtering criterion, or null if there is no filter.
	 * @param  n       Maximum number of objects to get (-1 means no limit).
	 * @return A collection of LyValueSet corresponding to the values of the loaded objects or null
	 *         if there is an error. false otherwise.
	 */
	Collection<LyValueSet> getObjects(LyUnionRequestEncoder encoder, Vector<LyField> fields, LySort sort,
		LyFilter filter, int n);

	/**
	 * Return the number of objects from the data provider .
	 * @param  encoder The union encoder demanding the number of the objects.
	 * @param  filter  indicates filtering criterion, or null if there is no filter.
	 * @return The count of objects matching the filter or null if there is an error.
	 */
	Integer getSize(LyUnionRequestEncoder encoder, LyFilter filter);

	/**
	 * Return a new instance of the specified class. The encoder must build the object with the same
	 * method as it loads the object itself.
	 * @param  encoder   The union encoder demanding the creation
	 * @param  peerClass the physical class of the created object.
	 * @return a new instance of of a physical object of the specified class.
	 */
	Object buildObject(LyUnionRequestEncoder encoder, LyClass peerClass);
}