/*
 * LyPeerFileValue.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.peer.util;

/**
 * This class implements a peer storage info to fetch a FileValue from a provider. Usually, this is
 * a reference to a BLOB entry in a database.
 */
public class LyPeerFileValue
{
	public static final String __VERSION =
		"$Id: LyPeerFileValue.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Peer name of the file. */
	protected String		   _name;

	/** Peer locator to the file (value depends on the data provider implementation). */
	protected String		   _locator;

	/**
	 * Constructs a new object.
	 */
	public LyPeerFileValue()
	{
	}

	/**
	 * Gets the peer name of the file.
	 * @return the file name
	 */
	public String getName()
	{
		return _name;
	}

	/**
	 * Sets the peer name of the file.
	 * @param name the file name
	 */
	public void setName(String name)
	{
		_name = name;
	}

	/**
	 * Gets the locator of the file. This is a mean to fetch the file from the data provider
	 * @return the locator (reference to the file or content in the data provider)
	 */
	public String getLocator()
	{
		return _locator;
	}

	/**
	 * Sets the locator of the file. This is a mean to fetch the file from the data provider
	 * @return the locator (reference to the file or content in the data provider)
	 */
	public void setLocator(String locator)
	{
		// examples
		// Format is LyDataProvider://<provider_id>//<object_id>//<field_id>
		// or directly a pth to a file when localy stored
		_locator = locator;
	}
} // end class LyPeerFileValue
