/*
 * W4BpmRequestEncoder.java
 * Copyright (c) 2000-2011, W4.
 * All rights reserved.
 */
package leon.peer.w4;

import eu.w4.bpm.*;

import java.util.*;

import leon.data.*;
import leon.info.*;
import leon.misc.*;
import leon.peer.*;
import leon.peer.util.*;


public abstract class W4BpmRequestEncoder extends LyRequestEncoder
{
    public static final String LY_ER_GET_OBJECT_MESSAGE = "LY_ER_GET_OBJECT";

    public static final String DATA_NAME_W4_ACTIVITY = "w4activity";

    public static final String C_BIND_W4_TASK_NAME = "w4_task";

    public static final String C_BIND_W4_WORKCASE_NAME = "w4_workcase";

    public static final String W4_ACTOR_C_BIND_NAME = "w4_actor";

    public static final String W4_ROLE_C_BIND_NAME = "w4_role";

    public static final String W4_DOMAIN_C_BIND_NAME = "w4_domain";

    public static final String W4_EVENT_C_BIND_NAME = "w4_event";

    public static final String W4_PROCESS_C_BIND_NAME = "w4_process";

    public static final String W4_WORKCASE_METRICS_C_BIND_NAME = "w4_workcase_metrics";

    public static final String FLD_W4_TASK_STATE = "w4_task_state";

    public static final String FLD_TASK_W4_END_DATE = "w4_task_end_date";

    public static final String CLS_COMMENT = "w4_comment_cls";

    public static final String FLD_W4_COMMENT_INDEX = "w4_comment_index";

    public static final String FLD_W4_COMMENT_WORKCASE = "w4_comment_workcase";

    public static final String FLD_W4_COMMENT_DATE = "w4_comment_date";

    public static final String FLD_W4_COMMENT_AUTHOR = "w4_comment_author";

    public static final String FLD_W4_COMMENT_TITLE = "w4_comment_title";

    public static final String FLD_W4_COMMENT_CONTENT = "w4_comment_content";

    public static final String CLS_WORKCASE_VARIABLE = "w4_workcase_variable_cls";

    public static final String FLD_W4_WORKCASE_VARIABLE_WORKCASE = "w4_workcase_variable_workcase";

    public static final String FLD_W4_WORKCASE_VARIABLE_INDEX = "w4_workcase_variable_index";

    public static final String FLD_W4_WORKCASE_VARIABLE_NAME = "w4_workcase_variable_name";

    public static final String FLD_W4_WORKCASE_VARIABLE_TYPE = "w4_workcase_variable_type";

    public static final String FLD_W4_WORKCASE_VARIABLE_VALUE = "w4_workcase_variable_value";

    public static final String FLD_W4_WORKCASE_COMMENTS = "w4_workcase_comments";

    public static final String FLD_W4_WORKCASE_STATE = "w4_workcase_state";

    public static final String CLS_TASK_VARIABLE = "w4_task_variable_cls";

    public static final String CLS_TASK = "w4_task_cls";

    public static final String FLD_W4_TASK_VARIABLES = "w4_task_variables";

    public static final String FLD_W4_TASK_VARIABLE_TASK = "w4_task_variable_task";

    public static final String FLD_W4_TASK_VARIABLE_INDEX = "w4_task_variable_index";

    public static final String FLD_W4_TASK_VARIABLE_NAME = "w4_task_variable_name";

    public static final String FLD_W4_TASK_VARIABLE_TYPE = "w4_task_variable_type";

    public static final String FLD_W4_TASK_VARIABLE_VALUE = "w4_task_variable_value";

    public static final String FLD_W4_TASK_WORKCASE = "w4_task_workcase";

    public static final String FLD_W4_TASK_CREATION_DATE = "w4_task_creation_date";

    public static final String FLD_W4_TASK_END_DATE = "w4_task_end_date";

    public static final String FLD_W4_WORKCASE_STEPS = "w4_workcase_steps";

    public static final String FLD_W4_NEW_TASK_WC_INTERNAL_ID = "w4_new_task_workcase_internal_id";

    public static final String FLD_RELATION_W4_WORKCASE_PROCESS = "getProcessId";

    public static final String CLS_WORKCASE_METRICS = "w4_workcase_metrics_cls";

    public static final String FLD_W4_WORKCASE_METRIC_INDEX = "w4_workcase_metric_index";

    public static final String FLD_W4_WORKCASE_METRIC_WORKCASE = "w4_workcase_metric_workcase";

    public static final String FLD_W4_WORKCASE_METRIC_STATE = "w4_workcase_metric_state";

    public static final String FLD_W4_WORKCASE_METRIC_DAYS = "w4_workcase_metric_days";

    public static final String FLD_W4_WORKCASE_METRIC_HOURS = "w4_workcase_metric_hours";

    public static final String FLD_W4_WORKCASE_METRIC_MINUTES = "w4_workcase_metric_minutes";

    public static final char VARIABLE_SEP = '#';

    public W4BpmRequestEncoder(LyDataProvider provider, LyDataSource dataSource)
    {
        super(provider, dataSource);
    }

    public abstract Exception getLastError();

    public abstract void setResponse(LyResponse response);

    public abstract boolean createObject(Object object);

    public abstract boolean deleteObject(Object object);

    public abstract boolean deleteObjects(Vector<Object> objects);

    public abstract boolean getObject(LyObjectId objectId, Vector<LyField> fields);

    public abstract boolean getObjects(Vector<LyField> fields, LySort sort, LyFilter filter, int n);

    public abstract void reassignTask(W4BpmConnection w4connection, LyObject taskObj, LyObject reassignObj,
        boolean preserveData) throws Exception;

    public abstract boolean getSize(LyFilter filter);

    public abstract boolean setObject(Object object);

    public abstract boolean lock(LyObject object, boolean lock);

    public abstract boolean submit(LyObject object, BPMVariableMap varMap);

    public abstract boolean save(LyObject object, BPMVariableMap varMap);

    public abstract LyObject startGetNewWorkcase(String workcaseId);

    public abstract LyObject startGetNewWorkcase(String workcaseId, String domain);

    public abstract LyObject startGetNewWorkcase(String workcaseId, String domain, String workcaseName);

    public abstract boolean isStartGetNewWorkcaseEnabledInDomain(String workcaseId);

    public abstract boolean isStartGetNewWorkcaseEnabled(String workcaseId);

    public abstract LyClassInfo findTaskClassInfo(String activityId);

    public abstract Vector<LyRowObject> createRole(Object object) throws Exception;

    public abstract Vector<LyRowObject> createDomain(Object object) throws Exception;

    public abstract Vector<LyRowObject> createActor(Object object) throws Exception;

    public abstract Vector<LyRowObject> createTask(Object object) throws Exception;

    public abstract Vector<LyRowObject> createWorkcase(Object object) throws Exception;

    public abstract void deleteActor(Object object) throws Exception;

    public abstract void deleteRole(Object object) throws Exception;

    public abstract void deleteDomain(Object object) throws Exception;

    public abstract void deleteWorkcases(List<Object> objects) throws Exception;

    public abstract Vector<LyRowObject> setActor(Object object) throws Exception;

    public abstract Vector<LyRowObject> setRole(Object object) throws Exception;

    public abstract LyValueSet setDomain(Object object) throws Exception;

    public abstract Vector<LyRowObject> setWorkcase(Object object) throws Exception;

    public abstract Vector<LyRowObject> setTask(Object object) throws Exception;

    public abstract boolean addComment(LyObject comment);

    public abstract boolean setVariable(LyObject variable, boolean workcase);

    public abstract Object getWorkcaseVariableMap(W4BpmConnection connection, LyObjectId workcaseId,
        LyObject variableObj) throws Exception;

    public abstract Object getTaskVariableMap(W4BpmConnection connection, LyObjectId taskId, LyObject variableObj)
        throws Exception;

    public abstract String getBPMExceptionMessage(Exception e, LyEnvironment env);

    public abstract boolean cancelObjects(W4BpmConnection w4connection, LyObjectList objects) throws Exception;

    public abstract Object getBPMId(LyObjectId lyId);
}
