/**
 * LyImportXmlConnection.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current version: $Id: LyImportXmlConnection.java 28422 2010-11-03 12:03:38Z ehelmste $
 */
package leon.peer.xml;

import java.io.*;
import leon.app.*;
import leon.data.*;
import leon.misc.*;


/**
 * This class is used as a connection for import.
 * @author Lyria
 * @see    leon.peer.LyConnection
 * @see    LyXmlConnection
 */
public class LyImportXmlConnection extends LyXmlConnection
{
	public static final String __VERSION =
		"$Id: LyImportXmlConnection.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**  */
	protected String		   _filePath;

	/**  */
	protected String		   _buffer;

	/**
	 * Constructor.
	 * @param classInfo ClassInfo used by the request encoder.
	 */
	public LyImportXmlConnection(LyApplication application, String filePath)
	{
		super(application);

		_filePath = filePath;
		_buffer = null;
	}

	/**
	 * Get the reader on the peer class data source.
	 * @param  The peer class.
	 * @return A reader on the peer class data source.
	 */
	public Reader getReader(LyClass peerClass)
	{
		if (_buffer != null)
			return new StringReader(_buffer);
		else
		{
			LyEnvironment environment = _application.getEnvironment();
			FileReader    reader = null;
			File		  file = new File(_filePath);

			try
			{
				reader = new FileReader(file);
			}
			catch (FileNotFoundException notFound)
			{
				setError(environment.getMessage("LY_ER_GET_INPUT_STREAM", _filePath));
				return null;
			}
			return reader;
		}
	}

	/**
	 */
	public void setBuffer(String buffer)
	{
		_buffer = buffer;
	}

	/**
	 * Return the path of the file containing the data.
	 * @param The class.
	 */
	protected String getFilePath(LyClass peerClass)
	{
		return _filePath;
	}
} // end class LyImportXmlConnection
