/*
 * LyXmlFileConnection.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.peer.xml.file;

import java.io.*;
import java.util.*;
import leon.app.*;
import leon.data.*;
import leon.misc.*;
import leon.peer.*;
import leon.peer.xml.*;


/**
 * This class is used as a connection for XML file provider.
 * @version $Id: LyXmlFileConnection.java 33045 2011-11-03 15:40:50Z bangiole $
 * @author  Lyria
 * @see     LyXmlConnection
 */
public class LyXmlFileConnection extends LyXmlConnection
{
	public static final String __VERSION =
		"$Id: LyXmlFileConnection.java 33045 2011-11-03 15:40:50Z bangiole $";

	String					   _xmlDirectory;

	/**
	 * Constructor.
	 */
	public LyXmlFileConnection(LyApplication application, String xmlDirectory)
	{
		super(application);
		_xmlDirectory = xmlDirectory;
	}

	/**
	 */
	public Reader getReader(LyClass peerClass)
	{
		String		  filename = ((LyXmlClass)peerClass).getName() + ".xml";
		LyEnvironment environment = _application.getEnvironment();

		filename = _xmlDirectory + "/" + filename;

		// Check if file does Exists
		String path = environment.getString().expand(filename);

		File   f = new File(path);

		if (!f.exists())
		{
			PrintWriter pw = openFile(peerClass);

			closeFile(pw);
		}

		InputStream input = environment.getInputStream(filename);

		if (input == null)
			return null; // File not found
		else
			return new InputStreamReader(input);
	} // end method getReader

	/**
	 * Write lines in the file containing objects data.
	 * @return true if the operation was successful.
	 */
	public boolean writeXml(LyClass peerClass, Vector<String> lines)
	{
		// Get the path of file and directory
		String		  filename = ((LyPeerClass)peerClass).getName() + ".xml";
		LyEnvironment environment = _application.getEnvironment();

		String		  dirPath = environment.getString().expand(_xmlDirectory);

		filename = _xmlDirectory + "/" + filename;

		filename = environment.getString().expand(filename);

		File file = new File(filename);
		File directory = new File(dirPath);

		// Try to create the directory if needed
		// TO DO : check if applet mode
		if (!directory.exists())
		{
			try
			{
				environment.logInfo("Creating directory :" + dirPath);
				directory.mkdirs();
			}
			catch (SecurityException e)
			{
				environment.logError("Unable to create directory :" + dirPath);
				setError(environment.getMessage("LY_ER_CREATE_DIR", dirPath));
				return false;
			}
		}
		else if (file.exists())
		{
			// Create bak file
			File bakFile = new File(filename + ".bak");

			if (bakFile.exists())
				bakFile.delete();

			if (file.renameTo(bakFile))
				environment.logInfo("File :" + filename + " renamed to .bak");
		}

		// Open the file
		FileOutputStream output;

		try
		{
			output = new FileOutputStream(file);
		}
		catch (Exception e)
		{
			environment.logError("Unable to open file :" + filename);
			setError(environment.getMessage("LY_ER_OPEN_FILE", filename.toString()));
			return false;
		}

		environment.logInfo("Saving file :" + filename);

		// Write the file
		PrintWriter pw = new PrintWriter(output);

		// Write each line
		int		    i, nl = lines.size();
		String	    line;

		pw.println("<?xml version='1.0' encoding='" + LyString.DEFAULT_CHARSET + "'?>");
		pw.println("<DATA>");
		for (i = 0; i < nl; i++)
		{
			line = (String)lines.elementAt(i);
			pw.println(line);
		}
		pw.println("</DATA>");
		pw.close();
		return true;
	} // end method writeXml

	/**
	 * Open the file of peerClass and write the begining of file.
	 * @return the PrintWriter of file of peerClass.
	 */
	public PrintWriter openFile(LyClass peerClass)
	{
		// Get the path of file and directory
		String		  filename = ((LyPeerClass)peerClass).getName() + ".xml";
		LyEnvironment environment = _application.getEnvironment();

		String		  dirPath = environment.getString().expand(_xmlDirectory);

		filename = _xmlDirectory + "/" + filename;

		filename = environment.getString().expand(filename);

		File file = new File(filename);
		File directory = new File(dirPath);

		// Try to create the directory if needed
		// TO DO : check if applet mode
		if (!directory.exists())
		{
			try
			{
				environment.logInfo("Creating directory :" + dirPath);
				directory.mkdirs();
			}
			catch (SecurityException e)
			{
				environment.logError("Unable to create directory :" + dirPath);
				setError(environment.getMessage("LY_ER_CREATE_DIR", dirPath));
				return null;
			}
		}
		else if (file.exists())
		{
			// Create bak file
			File bakFile = new File(filename + ".bak");

			if (bakFile.exists())
				bakFile.delete();

			if (file.renameTo(bakFile))
				environment.logInfo("File :" + filename + " renamed to .bak");
		}

		// Open the file
		FileOutputStream output;

		try
		{
			output = new FileOutputStream(file);
		}
		catch (Exception e)
		{
			environment.logError("Unable to open file :" + filename);
			setError(environment.getMessage("LY_ER_OPEN_FILE", filename.toString()));
			return null;
		}

		environment.logInfo("Saving file :" + filename);

		// Write the file
		PrintWriter pw = new PrintWriter(output);

		pw.println("<?xml version='1.0' encoding='" + LyString.DEFAULT_CHARSET + "'?>");
		pw.println("<DATA>");

		return pw;
	} // end method openFile

	/**
	 * Write a line in the file containing objects data.
	 * @return true if the operation was successful.
	 */
	public void writeLine(String line, PrintWriter pw)
	{
		pw.println(line);
		pw.flush();
	}

	/**
	 * Close the PrintWriter and write the end of file.
	 * @return true if the operation was successful.
	 */
	public void closeFile(PrintWriter pw)
	{
		pw.println("</DATA>");
		pw.close();
	}

} // end class LyXmlFileConnection
