/*
 * LyXmlFileDataProvider.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.peer.xml.file;

import java.util.*;
import leon.app.*;
import leon.data.*;
import leon.info.location.*;
import leon.misc.*;
import leon.peer.*;
import leon.peer.xml.*;


/**
 * This class is used by DataSources to transmit their requests to an instance of RequestEncoder.
 * @version $Id: LyXmlFileDataProvider.java 35214 2012-04-12 13:12:20Z jlsaouli $
 * @author  Lyria
 * @see     LyXmlRequestEncoder
 * @see     LyXmlFileConnection
 */
public class LyXmlFileDataProvider extends LyXmlDataProvider
{
	public static final String __VERSION =
		"$Id: LyXmlFileDataProvider.java 35214 2012-04-12 13:12:20Z jlsaouli $";

	String					   _xmlDirectory;

	/**
	 * Default constructor Initialization is done with the init method.
	 */
	public LyXmlFileDataProvider(LyApplication application, LyLocation location)
	{
		super(application, location);
	}

	/**
	 * Sets location on provider.
	 * @param location New location for the provider.
	 */
	public void setLocation(LyLocation location)
	{
		_xmlDirectory = location.getUrl();

		super.setLocation(location);
	}

	/**
	 * Initialisation of the data of the instance.
	 */
	protected void init()
	{
		super.init();
		_xmlDirectory = null;
	}

	/**
	 */
	protected LyConnection createConnection()
	{
		return new LyXmlFileConnection(_application, _xmlDirectory);
	}

	/**
	 * Return the file data provider.
	 * @return the file data provider.
	 */
	protected LyDataProvider getSaveFileDataProvider()
	{
		//FFT00504
		return this;
	}

	/**
	 * Saves given peer data source.
	 */
	protected boolean save(LyDataSource dataSource, LyDataProvider provider)
	{
		LyClass			    peerClass = dataSource.getDataClass();
		LyXmlRequestEncoder encoder;
		LyResponse		    response;
		LyCache			    cache;

		cache = dataSource.getCache();
		encoder = (LyXmlRequestEncoder)provider.getRequestEncoder(dataSource);
		response = new LyResponse(LyResponse.Operation.SAVE, peerClass);
		encoder.setResponse(response);

		Vector<Object> objects = cache.getObjects();

		sort(objects, peerClass);
		return encoder.saveObjects(objects, null, LyXmlRequestEncoder.TYPE_DEFAULT);
	}

	/**
	 * Closes all connections from the connection pool.
	 * @return true if the request completed successfully.
	 * @see    #createConnections(int)
	 */
	public boolean close()
	{
		LyEnvironment environment = getEnvironment();

		if (environment.testEnv("LY_SAVE_FILES"))
		    // If LY_SAVE_FILES == true we let parent do the job.
			return super.close();

		if (environment.testEnv("_LY_SAVE_DATA")
				&& _application.isValid())
			save(false);

		return super.close();
	}
} // end class LyXmlFileDataProvider
