// LyDateScript.js
// Copyright (c) 2000-2009, Lyria-W4.
// All rights reserved.
// @version	$Id: LyDateScript.js 35339 2012-04-23 14:46:31Z smaupin $

var dateForm;
var field;
var dateField;
var timeField;
var initialDay;
var initialMonth;
var initialYear;
var currentDate;
var yearLeftImage;
var monthLeftImage;
var monthRightImage;
var yearRightImage;
var dateLang;
var dateType;
var previousValues;
var calendarFrameId;
var dateSelectorEvent = false;

var yearTmp;

// This function displays the date selector
// @param form the form containing the date field
// @param field the date/time hidden field name
// @param fieldDate the date field name
// @param fieldTime the time field name
// @param form the language
// @param form the image path for year decrement
// @param form the image path for month decrement
// @param form the image path for month increment
// @param form the image path for year increment
// @param type the type of the date field : it's an integer and must be the same
// as constants defined in LyDate.java.
function ShowDateSelector(frameId, scriptPath, form, fieldName, fieldDateName,
	fieldTimeName, lang, yearLeft, monthLeft, monthRight, yearRight, type)
{
	if (typeof form == "undefined")
		form = this.document.forms[0];

	// Initialize values
	yearLeftImage = yearLeft;
	monthLeftImage = monthLeft;
	monthRightImage = monthRight;
	yearRightImage = yearRight;
	dateLang = lang;
	dateForm = form;
	field = fieldName;
	dateField = fieldDateName;
	timeField = fieldTimeName;
	dateType = type;
	yearTmp = null;
	calendarFrameId = frameId;

	var value = form.elements[dateField].value;

	// If not english, swap day and month
//	if (dateLang != "en_US" && dateLang != "en_EN")
//		value = DateLangSwapDayMonth(value);

	if (value.length <= 0)
		currentDate = new Date();
	else
	{
		currentDate = parseDate(value, form.elements[dateField]);

		if (isNaN(currentDate))
			currentDate = new Date();
	}

	initialDay = currentDate.getDate();
	initialMonth = currentDate.getMonth();
	initialYear = currentDate.getFullYear();

	SetFrameSrc(frameId, scriptPath + '/LyDateSelector.html');
	CloseLastDivShown();
	SwitchDivVisibility(frameId);
	SetDivSize(frameId, 155, 155);
}

// This function hides the date selector immediatly
function HideDateSelector()
{
	CloseLastDivShown();
	SetDivVisibility(calendarFrameId, false);
	calendarFrameId = null;
}

// This function updates the date field.
function DateSelectorEvent()
{
	dateSelectorEvent = true;

	UpdateDate(dateForm, field, dateField, timeField, dateType, dateLang, 'false', 'true', 10);
	HideDateSelector();

	dateSelectorEvent= false;
}

// This function swap the day and the month of the given value.
function DateLangSwapDayMonth(value)
{
	if (value.length <= 5)
		return value;

	var n1 = value.indexOf('/', 0);
	var n2 = value.indexOf('/', n1 + 1);

	if (n1>0 && n2 > n1+1)
	{
		var newDate = value.substring(n1 + 1, n2 + 1) + value.substring(0, n1) +
			value.substring(n2);

		value = newDate;
	}

	return value;
}

// This function add a 0 before the number if single-figure number.
function GetTwoFiguresNumber(number)
{
	if (number.length >= 2)
		return number;

	return (number < 10) ? '0' + number : number;
}

// This function converts a year for being Y2K compliant.
function ConvertY2K(number)
{
 	if (number < 20)
		number += 2000;
 	if (number < 100)
		number += 1900;

	return number;
}

function TransformCurrentDate(dateField, optional)
{
	var str = '';

	if (currentDate == null)
		currentDate = new Date();

	str = formatDate(currentDate, dateField);

//	if (dateLang == "en_US" || dateLang == "en_EN")
//		str = DateLangSwapDayMonth(str);

	if (dateField != null)
	{
		if ((dateField.value != "") ||
		    ((optional != "true") && (dateField.value == "")))
		{
			dateField.value = str;
		}
	}
}

// Format a date using a given field and its format
function formatDate(date, dateField)
{
	var dayString = GetTwoFiguresNumber(currentDate.getDate());
	var monthString = GetTwoFiguresNumber(currentDate.getMonth() + 1);
	var yearString;

	if (dateSelectorEvent)
	{
		yearString = "" + yearTmp;

		while (yearString.length < 4)
			yearString = "0" + yearString;

		yearTmp = null;
	}
	else
		yearString = currentDate.getFullYear();

	var dateFormat = dateField.getAttribute("dateFormat");
	var str = "";
	var pos;
	var currentChar;
	var daySet = false, monthSet = false, yearSet = false;

	for (pos = 0; pos < dateFormat.length; pos++)
	{
		currentChar = dateFormat.substring(pos, pos + 1);

		if (currentChar == 'd')
		{
			if (!daySet)
			{
				str = str + dayString;
				daySet = true;
			}
		}
		else if (currentChar == 'M')
		{
			if (!monthSet)
			{
				str = str + monthString;
				monthSet = true;
			}
		}
		else if (currentChar == 'y')
		{
			if (!yearSet)
			{
				str = str + yearString;
				yearSet = true;
			}
		}
		else
			str = str + currentChar;
	}

	return str;
}

// Parse a date using a given field and its format
function parseDate(value, dateField)
{
	var dateFormat = dateField.getAttribute("dateFormat");

	if ((value == null) || (typeof value == "undefined") || value.length != dateFormat.length)
		return new Date();

	var pos;
	var currentValueChar, currentFormatChar;
	var dayString = "", monthString = "", yearString = "";

	for (pos = 0; pos < value.length; pos++)
	{
		currentValueChar = value.substring(pos, pos + 1);
		currentFormatChar = dateFormat.substring(pos, pos + 1);

		if (currentFormatChar == 'd')
			dayString = dayString + currentValueChar;
		else if (currentFormatChar == 'M')
			monthString = monthString + currentValueChar;
		else if (currentFormatChar == 'y')
			yearString = yearString + currentValueChar;
	}

	return new Date(monthString + "/" + dayString + "/" + yearString);
}

// This function resets the internal date. Mandatory when this date was created
// by the popup date selector to "close" the links to the popup window.
function ResetDate()
{
	currentDate = new Date(currentDate.getYear(),
		currentDate.getMonth(), currentDate.getDate());

	if (currentDate == null
		|| isNaN(currentDate))
			currentDate = new Date();
}

function UpdateDateOnBlur(form, fieldName, dateFieldName, timeFieldName, type,
	lang, optional, format, error, updateFieldValue)
{
	if (typeof form == "undefined")
		form = this.document.forms[0];

	var dateField = form.elements[dateFieldName];
	var timeField = form.elements[timeFieldName];
	var check = CheckFormatField(form, dateFieldName, format, error, 'true');
	var updateField = (typeof updateFieldValue == "undefined") || (updateFieldValue != false);

	if (check == "false")
	{
		// update the date field with the previous value
		var previousValue = GetPreviousValue(dateFieldName);

		if (previousValue != "")
		{
			if (updateField)
				dateField.value = previousValue;

			UpdateDate(form, fieldName, dateFieldName, timeFieldName, type, lang, optional, "false", null, updateField);
		}
		else if (optional == "true")
		{
			if (updateField)
				dateField.value = "";

			UpdateDate(form, fieldName, dateFieldName, timeFieldName, type, lang, optional, "false", null, updateField);
		}
		else 	// update the date with transformation of the current date
			UpdateDate(form, fieldName, dateFieldName, timeFieldName, type, lang, "false", "true", null, updateField);
	}
	else
	{
		var transform = "false";

		// if the new value is null and the field not optional : revert to previous value
		//var dateField = form.elements[dateFieldName];
		if ((optional != "true") && (dateField.value == ""))
			dateField.value = GetPreviousValue(dateFieldName);
		else
			StorePreviousValue(dateField); // update the previous value

		// if is optional and the date is null, set time field to null

		if ((optional == "true") && (timeField != null))
		{
			if (updateField)
			{
				if (dateField.value == "")
					timeField.value = "";
				else if (timeField.value == "")
					timeField.value = GetDefaultTime(type);
			}
		}
		else if ((optional != "true") && (timeField != null))
		{
		 	if (updateField && (timeField.value == "") && (dateField.value != "") )
		 	{
				timeField.value = GetDefaultTime(type);
			}
		}

		var length = dateField.value.length;

		// update the date
		UpdateDate(form, fieldName, dateFieldName, timeFieldName, type, lang, optional, transform, length, updateField);
	}
}

function UpdateTimeOnBlur(form, fieldName, dateFieldName, timeFieldName, type,
	lang, optional, format, error, updateFieldValue)
{
	var timeField = form.elements[timeFieldName];
	var dateField = form.elements[dateFieldName];
	var updateField = (typeof updateFieldValue == "undefined") || (updateFieldValue != false);

	if (CheckFormatField(form, timeFieldName, format, error, 'true') == "false")
	{
		// update the date field with the previous value
		var previousValue = GetPreviousValue(timeFieldName);

		var transform = "false";

		if (previousValue != "")
		{
			if (updateField)
				timeField.value = previousValue;
		}
		else if (optional == "true")
		{
			if (updateField)
				timeField.value = "";
		}
		else 	// update the date with transformation of the current date
		{
//			timeField.value = GetCurrentTime(type);
			transform = "true";
		}

		UpdateDate(form, fieldName, dateFieldName, timeFieldName, type, lang, optional, transform, null, updateField);
	}
	else
	{
		var transform = "false";

		if ((optional != "true") && (timeField.value == "") && updateField)
			timeField.value = GetPreviousValue(timeFieldName);
		else
			StorePreviousValue(timeField); // update the previous value


		// if the field is optional if the time value is empty : empty the date.
		if ( (optional == "true") && (dateField != null) )
		{

			if (timeField.value == "")
			{
				if (updateField)
					dateField.value = "";
			}
			else
			{
				if ((dateField != null) && (dateField.value == ""))
				{
					optional = "false";
					transform = "true";
				}
			}
		}
		else if (timeField.value != null)
		{
			// Obligatory: if the timeField is not null, the date field should not be null!
		 	if ( (dateField != null) && (dateField.value == "") && (timeField.value.length != 0))
		 		transform = "true";
		}
		UpdateDate(form, fieldName, dateFieldName, timeFieldName, type, lang, optional, transform, null, updateField);
	}
}

// This function updates a field with a date field and a time field.
function UpdateDate(form, fieldName, dateFieldName, timeFieldName, type, lang, optional, transform, length, updateField)
{
	var field = form.elements[fieldName];
	var dateField = form.elements[dateFieldName];
	var timeField = form.elements[timeFieldName];
	var timeAmPmField = form.elements[timeFieldName + "_ampm"];
	var value = "";

	if (currentDate == null)
		currentDate = new Date();
	dateLang = lang;

	if (typeof updateField == "undefined")
		updateField = true;

	if ((transform != "false") && updateField)
		TransformCurrentDate(dateField, optional);

	// check if the value of the date is Y2K compliant
	// and modify it if needed.
	if ((dateField != null) && updateField)
	{
		var dateValue = dateField.value;
		if (dateValue != null)
		{
			var yearStr = dateValue.substring(dateValue.lastIndexOf("/") + 1);
			var d = new Date("01/01/" + yearStr);

			yearStr = removeLeadingZeros(yearStr);
			var year = parseInt(yearStr);

			if ((year < 1000) && (length != null) && (length < 10)) // case of a date lower than 1000
			{
				// If lower than 30 consider it is 20xx
				if (year < 10)
					year = "200" + year;
				else if (year < 30)
					year = "20" + year;
				// If lower than 100 but greater than 10 it is 19xx
				else if (year < 100)
					year = "19" + year;
				// If lower than 1000 it is 0xxx
				else if (year < 1000)
					year = "0" + year;

				// create the new value for the field
				var str = dateValue.substring(0,dateValue.lastIndexOf("/"));
				str = str+"/"+year;
				dateField.value = str;
				StorePreviousValue(dateField);
			}
		}
	}

	if ((transform == "true") && updateField)
	{
		if ((timeField != null) && (timeField.value == "")) {
			if ((timeAmPmField != null) && (typeof timeAmPmField != "undefined")) {
				timeField.value = GetAmPmDefaultTime(type);
				setSelectedIndex(timeAmPmField, isAmPm());
			} else {
				timeField.value = GetDefaultTime(type);
			}
		}
	}


	if (field != null)
	{
		if (dateField != null)
			value = dateField.value;

		if (timeField != null)
		{
			// If date and time are not null, add a space between the values
			if ((value.length != 0) && (timeField.value.length != 0))
				value += " ";

			value += timeField.value;

			if ((timeAmPmField != null) && (typeof timeAmPmField != "undefined") && timeField.value) {
				value += " " + isAmPm();
				setSelectedIndex(timeAmPmField, isAmPm());
			}
				

			if (!updateField && (optional != "true"))
			{
				if ((value.length != 0) && (timeField.value.length != 0))
					value += " ";
				if ((timeAmPmField != null) && (typeof timeAmPmField != "undefined")) {
					value += GetAmPmDefaultTime(type) + " " + isAmPm();
					setSelectedIndex(timeAmPmField, isAmPm());
				} else {
					value += GetDefaultTime(type);
				}
			}
		}
	}

	// Notify new value only if it has changed
	field.value = value;

	var checkDateField = form.elements['_checkDate' + fieldName];

	if ((checkDateField != null) && updateField)
	{
		var checkDateValue = checkDateField.value;

		FieldChange(form, field, checkDateValue);
	}
}

function setSelectedIndex(selectField, value) {
	if (selectField) {
		for (var i = 0; i< selectField.options.length; i++) { 
			if (selectField.options[i].value == value) {
				selectField.options[i].selected = true;
				break;
			}
		}
	}
	return;
}


// This function returns the current time
function GetCurrentTime(type)
{
	var date = new Date();
	var hours;
	var minutes;
	var seconds;
	var result;

	hours = date.getHours();
	minutes = date.getMinutes();
	seconds = date.getSeconds();

	if (hours < 10)
		hours = '0' + hours;
	if (minutes < 10)
		minutes = '0' + minutes;
	if (seconds < 10)
		seconds = '0' + seconds;

	if ((type == "DATE_TIME") || (type == "TIME"))
		// DATE_TIME or TIME
		result = hours + ":" + minutes + ":" + seconds;
	else if ((type == "DATE_HOUR_MIN") || (type == "HOUR_MIN"))
		// DATE_HOUR_MIN or HOUR_MIN
		result = hours + ":" + minutes;
	else
		// HOUR
		result = hours;

	return result;
}


function isAmPm() {
	var now = new Date();
	var hour = now.getHours();
	var ap = "AM";
	if (hour > 12) {
		ap = "PM";
	}
	return ap;
}

function GetAmPmDefaultTime(type) {
	var now = new Date();
	var hour = now.getHours();
	var minute = now.getMinutes();
	var second = now.getSeconds();
	var ap = "AM";
	if (hour > 11) {
		ap = "PM";
	}
	if (hour > 12) {
		hour = hour - 12;
	}
	if (hour == 0) {
		hour = 12;
	}
	if (hour < 10) {
		hour = "0" + hour;
	}
	if (minute < 10) {
		minute = "0" + minute;
	}
	if (second < 10) {
		second = "0" + second;
	}

	if ((type == "DATE_TIME") || (type == "TIME"))
		// DATE_TIME or TIME
		result = hour + ":" + minute + ":" + second + " " + ap;
	else if ((type == "DATE_HOUR_MIN") || (type == "HOUR_MIN"))
		// DATE_HOUR_MIN or HOUR_MIN
		result = hour + ":" + minute + " " + ap;
	else
		// HOUR
		result = hour + " " + ap;
	return result;
}


// This function returns the current time
function GetDefaultTime(type)
{
	var date = new Date();
	var hours;
	var minutes;
	var seconds;
	var result;

	hours = date.getHours();
	minutes = date.getMinutes();
	seconds = date.getSeconds();

	if (hours < 10)
		hours = "0" + hours;

	if (minutes < 10)
		minutes = "0" + minutes;

	if (seconds < 10)
		seconds = "0" + seconds;

	if ((type == "DATE_TIME") || (type == "TIME"))
		// DATE_TIME or TIME
		result = hours + ":" + minutes + ":" + seconds;
	else if ((type == "DATE_HOUR_MIN") || (type == "HOUR_MIN"))
		// DATE_HOUR_MIN or HOUR_MIN
		result = hours + ":" + minutes;
	else
		// HOUR
		result = hours;

	return result;
}

// This function stores the previous value of a date input
function StorePreviousValue(dateInput)
{
	if (typeof previousValues == "undefined")
		previousValues = new Array();
	previousValues[dateInput.name] = dateInput.value;
}

// retrieves the previous value for a given date field
function GetPreviousValue(dateInputName)
{
	return previousValues[dateInputName];
}
