/*
 * LyAxis.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;
import java.io.*;
import leon.misc.LyString;


/**
 * The LyAxis class is a generated Java implementation that corresponds to the element 
 * AXIS of the descriptive grammar of the views (views.dtd).
 * <B>&lt;AXIS&gt; : Axis Setting</B>
 * An AXIS element allows defining the limits of the X and Y axis, as well as the 
 * difference between the graduations. This element is useful only in the case 
 * where the type of graph is not a pie, and for the variables of number or date 
 * type.
 * @see leon.view.LyComponent
 * See also tag <A href="../../conf/dtd/view/AXIS.html">AXIS</A> of grammar leon_views.dtd.
 */
public class LyAxis extends LyComponent
{
	public static final String			__VERSION = "$Id$";


	/* Attributes */

	/**
	 * Minimum value on the abscissa axis.
	 */
	protected String			_xBegin;

	/**
	 * Maximum value on the abscissa axis
	 */
	protected String			_xEnd;

	/**
	 * Difference of the graduations on the abscissa axis.
	 */
	protected String			_xWidth;

	/**
	 * Minimum value on the ordinate axis.
	 */
	protected String			_yBegin;

	/**
	 * Maximum value on the ordinate axis.
	 */
	protected String			_yEnd;

	/**
	 * Difference of the graduations on the ordinate axis.
	 */
	protected String			_yWidth;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_xBegin = null;
		_xEnd = null;
		_xWidth = null;
		_yBegin = null;
		_yEnd = null;
		_yWidth = null;
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyAxis	model = (LyAxis)component;

		_model = component;
		_xBegin = model._xBegin;
		_xEnd = model._xEnd;
		_xWidth = model._xWidth;
		_yBegin = model._yBegin;
		_yEnd = model._yEnd;
		_yWidth = model._yWidth;
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	public LyAxis()
	{
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyAxis(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_xBegin = node.getString("xBegin");
		_xEnd = node.getString("xEnd");
		_xWidth = node.getString("xWidth");
		_yBegin = node.getString("yBegin");
		_yEnd = node.getString("yEnd");
		_yWidth = node.getString("yWidth");
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyAxis clone = new LyAxis();

		clone.initFromModel(this);
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of xBegin.
	 * @return value of xBegin.
	 */
	public String getXBegin()
	{
		return _xBegin;
	}

	/**
	 * Gets value of xEnd.
	 * @return value of xEnd.
	 */
	public String getXEnd()
	{
		return _xEnd;
	}

	/**
	 * Gets value of xWidth.
	 * @return value of xWidth.
	 */
	public String getXWidth()
	{
		return _xWidth;
	}

	/**
	 * Gets value of yBegin.
	 * @return value of yBegin.
	 */
	public String getYBegin()
	{
		return _yBegin;
	}

	/**
	 * Gets value of yEnd.
	 * @return value of yEnd.
	 */
	public String getYEnd()
	{
		return _yEnd;
	}

	/**
	 * Gets value of yWidth.
	 * @return value of yWidth.
	 */
	public String getYWidth()
	{
		return _yWidth;
	}


	/* Modifiers */

	/**
	 * Initializes attribute xBegin to the given value.
	 * @param xBegin New value of xBegin.
	 */
	public void setXBegin(String xBegin)
	{
		if ((xBegin == _xBegin)
			|| ((xBegin != null)
				&& (_xBegin != null)
				&& xBegin.equals(_xBegin)))
			return;

		_xBegin = xBegin;
	}

	/**
	 * Initializes attribute xEnd to the given value.
	 * @param xEnd New value of xEnd.
	 */
	public void setXEnd(String xEnd)
	{
		if ((xEnd == _xEnd)
			|| ((xEnd != null)
				&& (_xEnd != null)
				&& xEnd.equals(_xEnd)))
			return;

		_xEnd = xEnd;
	}

	/**
	 * Initializes attribute xWidth to the given value.
	 * @param xWidth New value of xWidth.
	 */
	public void setXWidth(String xWidth)
	{
		if ((xWidth == _xWidth)
			|| ((xWidth != null)
				&& (_xWidth != null)
				&& xWidth.equals(_xWidth)))
			return;

		_xWidth = xWidth;
	}

	/**
	 * Initializes attribute yBegin to the given value.
	 * @param yBegin New value of yBegin.
	 */
	public void setYBegin(String yBegin)
	{
		if ((yBegin == _yBegin)
			|| ((yBegin != null)
				&& (_yBegin != null)
				&& yBegin.equals(_yBegin)))
			return;

		_yBegin = yBegin;
	}

	/**
	 * Initializes attribute yEnd to the given value.
	 * @param yEnd New value of yEnd.
	 */
	public void setYEnd(String yEnd)
	{
		if ((yEnd == _yEnd)
			|| ((yEnd != null)
				&& (_yEnd != null)
				&& yEnd.equals(_yEnd)))
			return;

		_yEnd = yEnd;
	}

	/**
	 * Initializes attribute yWidth to the given value.
	 * @param yWidth New value of yWidth.
	 */
	public void setYWidth(String yWidth)
	{
		if ((yWidth == _yWidth)
			|| ((yWidth != null)
				&& (_yWidth != null)
				&& yWidth.equals(_yWidth)))
			return;

		_yWidth = yWidth;
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<AXIS");
		if (_xBegin != null)
			pw.print(" xBegin='" + LyString.getXmlString(_xBegin) + "'");
		if (_xEnd != null)
			pw.print(" xEnd='" + LyString.getXmlString(_xEnd) + "'");
		if (_xWidth != null)
			pw.print(" xWidth='" + LyString.getXmlString(_xWidth) + "'");
		if (_yBegin != null)
			pw.print(" yBegin='" + LyString.getXmlString(_yBegin) + "'");
		if (_yEnd != null)
			pw.print(" yEnd='" + LyString.getXmlString(_yEnd) + "'");
		if (_yWidth != null)
			pw.print(" yWidth='" + LyString.getXmlString(_yWidth) + "'");
		pw.println("/>");
	}
}
