/*
 * LyBoolean.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;
import java.io.*;
import leon.view.getinterface.*;
import leon.view.setinterface.*;
import leon.misc.LyString;


/**
 * The LyBoolean class is a generated Java implementation that corresponds to the element 
 * BOOLEAN of the descriptive grammar of the views (views.dtd).
 * <B>&lt;BOOLEAN&gt; : Input Field of Boolean Type</B>
 * An input field of boolean type allows displaying a graphical component with a 
 * three state checkbox:
 * not specified (empty),
 * true (checked with a green sign),
 * false (with a red cross).
 * It is possible to specify a label for each of these states and to make a 
 * negation of the boolean value.
 * @see leon.view.LyFieldItem
 * See also tag <A href="../../conf/dtd/view/BOOLEAN.html">BOOLEAN</A> of grammar leon_views.dtd.
 */
public class LyBoolean extends LyFieldItem
	implements  LyGetIdInterface,
				LySetStatusInterface
{
	public static final String			__VERSION = "$Id$";


	/** Possible values for attribute 'status'. */
	public enum Status
	{
		CHECKED,
		UNCHECKED,
		EMPTY;
	}


	/* Static methods */

	/**
	 * Utility method to get status value from a string.
	 * @param token Token string
	 * @return Status value corresponding to this token
	 */
	public static final Status getStatus(String token)
	{
		if (token == null)
			return null;
		else if (token.equalsIgnoreCase("CHECKED")
				|| token.equals("0"))
			return Status.CHECKED;
		else if (token.equalsIgnoreCase("UNCHECKED")
				|| token.equals("1"))
			return Status.UNCHECKED;
		else if (token.equalsIgnoreCase("EMPTY")
				|| token.equals("2"))
			return Status.EMPTY;
		else
			return Status.EMPTY;
	}


	/* Attributes */

	/**
	 * Identifier of the component.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyBoolean			_template;

	/**
	 * Label of the field for CHECKED state.
	 */
	protected String			_labelChecked;

	/**
	 * Image used when checked.
	 */
	protected String			_imageChecked;

	/**
	 * Label of the field for UNCHECKED state.
	 */
	protected String			_labelUnchecked;

	/**
	 * Image used when unchecked.
	 */
	protected String			_imageUnchecked;

	/**
	 * Label of the field for EMPTY state.
	 */
	protected String			_labelEmpty;

	/**
	 * Image used when null (the boolean is neither checked, nor unchecked).
	 */
	protected String			_imageEmpty;

	/**
	 * Indicates if the field value is the negation of its boolean value:
	 * if its state is CHECKED, its value is false;
	 * if its state is UNCHECKED, its value is true;
	 * if its state is EMPTY, its value is the one of the EMPTY state.
	 */
	protected boolean			_not;

	/**
	 * Indicates the status of the boolean field:
	 * CHECKED if the field is checked with a green sign;
	 * UNCHECKED if the field is checked with a red cross;
	 * EMPTY if the field is not specified.
	 */
	protected Status			_status;

	/**
	 * Allows specifying the name of a Java class that replaces the default graphical 
	 * component.
	 */
	protected String			_bean;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	/**
	 * Size of the images used for displaying values of the boolean.
	 */
	protected int				_nCols;

	/**
	 * BOOLEAN.backgroundImage
	 */
	protected String			_backgroundImage;

	/**
	 * BOOLEAN.hooverImage
	 */
	protected String			_hooverImage;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_id = null;
		_template = null;
		_labelChecked = null;
		_imageChecked = getStringValue("BOOLEAN.imageChecked", "$ALL.imageChecked");
		_labelUnchecked = null;
		_imageUnchecked = getStringValue("BOOLEAN.imageUnchecked", "$ALL.imageUnchecked");
		_labelEmpty = null;
		_imageEmpty = getStringValue("BOOLEAN.imageEmpty", "$ALL.imageEmpty");
		_not = getBooleanValue("BOOLEAN.not", false);
		_status = Status.EMPTY;
		_bean = null;
		_font = getStringValue("BOOLEAN.font", "$ALL.font");
		_color = getStringValue("BOOLEAN.color", "$ALL.color");
		_fontColor = getStringValue("BOOLEAN.fontColor", "$ALL.fontColor");
		_nCols = getIntValue("BOOLEAN.nCols", -1);
		_backgroundImage = getStringValue("BOOLEAN.backgroundImage", "$ALL.backgroundImage");
		_hooverImage = getStringValue("BOOLEAN.hooverImage", "$ALL.hooverImage");
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyBoolean	model = (LyBoolean)component;

		_model = component;
		_labelChecked = model._labelChecked;
		_imageChecked = model._imageChecked;
		_labelUnchecked = model._labelUnchecked;
		_imageUnchecked = model._imageUnchecked;
		_labelEmpty = model._labelEmpty;
		_imageEmpty = model._imageEmpty;
		_not = model._not;
		_status = model._status;
		_bean = model._bean;
		_font = model._font;
		_color = model._color;
		_fontColor = model._fontColor;
		_nCols = model._nCols;
		_backgroundImage = model._backgroundImage;
		_hooverImage = model._hooverImage;
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	public LyBoolean()
	{
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyBoolean(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_id = node.getString("id");
		_template = (LyBoolean)node.getComponent("template");

		if (_template != null)
			initFromModel(_template);

		if (node.isSpecified("labelChecked"))
			_labelChecked = node.getString("labelChecked");
		if (node.isSpecified("imageChecked"))
			_imageChecked = node.getString("imageChecked");
		if (node.isSpecified("labelUnchecked"))
			_labelUnchecked = node.getString("labelUnchecked");
		if (node.isSpecified("imageUnchecked"))
			_imageUnchecked = node.getString("imageUnchecked");
		if (node.isSpecified("labelEmpty"))
			_labelEmpty = node.getString("labelEmpty");
		if (node.isSpecified("imageEmpty"))
			_imageEmpty = node.getString("imageEmpty");
		if (node.isSpecified("not"))
			_not = node.getBoolean("not");
		if (node.isSpecified("status"))
			_status = getStatus(node.getString("status"));
		if (node.isSpecified("bean"))
			_bean = node.getString("bean");
		if (node.isSpecified("font"))
			_font = node.getString("font");
		if (node.isSpecified("color"))
			_color = node.getString("color");
		if (node.isSpecified("fontColor"))
			_fontColor = node.getString("fontColor");
		if (node.isSpecified("nCols"))
			_nCols = node.getInt("nCols");
		if (node.isSpecified("backgroundImage"))
			_backgroundImage = node.getString("backgroundImage");
		if (node.isSpecified("hooverImage"))
			_hooverImage = node.getString("hooverImage");
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyBoolean clone = new LyBoolean();

		clone.initFromModel(this);
		clone._template = this;
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of id.
	 * @return value of id.
	 */
	public String getId()
	{
		return _id;
	}

	/**
	 * Gets value of template.
	 * @return value of template.
	 */
	public LyBoolean getTemplate()
	{
		return _template;
	}

	/**
	 * Gets value of labelChecked.
	 * @return value of labelChecked.
	 */
	public String getLabelChecked()
	{
		return _labelChecked;
	}

	/**
	 * Gets value of imageChecked.
	 * @return value of imageChecked.
	 */
	public String getImageChecked()
	{
		return _imageChecked;
	}

	/**
	 * Checks whether the imageChecked attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultImageChecked()
	{
		return (_imageChecked == getStringValue("BOOLEAN.imageChecked"));
	}

	/**
	 * Gets value of labelUnchecked.
	 * @return value of labelUnchecked.
	 */
	public String getLabelUnchecked()
	{
		return _labelUnchecked;
	}

	/**
	 * Gets value of imageUnchecked.
	 * @return value of imageUnchecked.
	 */
	public String getImageUnchecked()
	{
		return _imageUnchecked;
	}

	/**
	 * Checks whether the imageUnchecked attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultImageUnchecked()
	{
		return (_imageUnchecked == getStringValue("BOOLEAN.imageUnchecked"));
	}

	/**
	 * Gets value of labelEmpty.
	 * @return value of labelEmpty.
	 */
	public String getLabelEmpty()
	{
		return _labelEmpty;
	}

	/**
	 * Gets value of imageEmpty.
	 * @return value of imageEmpty.
	 */
	public String getImageEmpty()
	{
		return _imageEmpty;
	}

	/**
	 * Checks whether the imageEmpty attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultImageEmpty()
	{
		return (_imageEmpty == getStringValue("BOOLEAN.imageEmpty"));
	}

	/**
	 * Checks if value of attribute not is set.
	 * @return true if not is set, false otherwise.
	 */
	public boolean isNot()
	{
		return _not;
	}

	/**
	 * Gets value of status.
	 * @return value of status.
	 */
	public Status getStatus()
	{
		return _status;
	}

	/**
	 * Gets value of bean.
	 * @return value of bean.
	 */
	public String getBean()
	{
		return _bean;
	}

	/**
	 * Gets value of font.
	 * @return value of font.
	 */
	public String getFont()
	{
		return _font;
	}

	/**
	 * Checks whether the font attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFont()
	{
		return (_font == getStringValue("BOOLEAN.font"));
	}

	/**
	 * Gets value of color.
	 * @return value of color.
	 */
	public String getColor()
	{
		return _color;
	}

	/**
	 * Checks whether the color attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultColor()
	{
		return (_color == getStringValue("BOOLEAN.color"));
	}

	/**
	 * Gets value of fontColor.
	 * @return value of fontColor.
	 */
	public String getFontColor()
	{
		return _fontColor;
	}

	/**
	 * Checks whether the fontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFontColor()
	{
		return (_fontColor == getStringValue("BOOLEAN.fontColor"));
	}

	/**
	 * Gets value of nCols.
	 * @return value of nCols.
	 */
	public int getNCols()
	{
		return _nCols;
	}

	/**
	 * Gets value of backgroundImage.
	 * @return value of backgroundImage.
	 */
	public String getBackgroundImage()
	{
		return _backgroundImage;
	}

	/**
	 * Checks whether the backgroundImage attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultBackgroundImage()
	{
		return (_backgroundImage == getStringValue("BOOLEAN.backgroundImage"));
	}

	/**
	 * Gets value of hooverImage.
	 * @return value of hooverImage.
	 */
	public String getHooverImage()
	{
		return _hooverImage;
	}

	/**
	 * Checks whether the hooverImage attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultHooverImage()
	{
		return (_hooverImage == getStringValue("BOOLEAN.hooverImage"));
	}


	/* Modifiers */

	/**
	 * Initializes attribute id to the given value.
	 * @param id New value of id.
	 */
	public void setId(String id)
	{
		if ((id == _id)
			|| ((id != null)
				&& (_id != null)
				&& id.equals(_id)))
			return;

		_id = (id != null ? id.intern() : null);
	}

	/**
	 * Initializes attribute template to the given value.
	 * @param template New value of template.
	 */
	public void setTemplate(LyBoolean template)
	{
		if (template == null)
			return;
		if (_template != null)
			throw new IllegalStateException("BF AppEngine : attribute template was already set.");

		initFromModel(template);
		_template = template;
	}

	/**
	 * Initializes attribute labelChecked to the given value.
	 * @param labelChecked New value of labelChecked.
	 */
	public void setLabelChecked(String labelChecked)
	{
		if ((labelChecked == _labelChecked)
			|| ((labelChecked != null)
				&& (_labelChecked != null)
				&& labelChecked.equals(_labelChecked)))
			return;

		_labelChecked = labelChecked;
	}

	/**
	 * Initializes attribute imageChecked to the given value.
	 * @param imageChecked New value of imageChecked.
	 */
	public void setImageChecked(String imageChecked)
	{
		if ((imageChecked == _imageChecked)
			|| ((imageChecked != null)
				&& (_imageChecked != null)
				&& imageChecked.equals(_imageChecked)))
			return;

		_imageChecked = imageChecked;
	}

	/**
	 * Initializes attribute labelUnchecked to the given value.
	 * @param labelUnchecked New value of labelUnchecked.
	 */
	public void setLabelUnchecked(String labelUnchecked)
	{
		if ((labelUnchecked == _labelUnchecked)
			|| ((labelUnchecked != null)
				&& (_labelUnchecked != null)
				&& labelUnchecked.equals(_labelUnchecked)))
			return;

		_labelUnchecked = labelUnchecked;
	}

	/**
	 * Initializes attribute imageUnchecked to the given value.
	 * @param imageUnchecked New value of imageUnchecked.
	 */
	public void setImageUnchecked(String imageUnchecked)
	{
		if ((imageUnchecked == _imageUnchecked)
			|| ((imageUnchecked != null)
				&& (_imageUnchecked != null)
				&& imageUnchecked.equals(_imageUnchecked)))
			return;

		_imageUnchecked = imageUnchecked;
	}

	/**
	 * Initializes attribute labelEmpty to the given value.
	 * @param labelEmpty New value of labelEmpty.
	 */
	public void setLabelEmpty(String labelEmpty)
	{
		if ((labelEmpty == _labelEmpty)
			|| ((labelEmpty != null)
				&& (_labelEmpty != null)
				&& labelEmpty.equals(_labelEmpty)))
			return;

		_labelEmpty = labelEmpty;
	}

	/**
	 * Initializes attribute imageEmpty to the given value.
	 * @param imageEmpty New value of imageEmpty.
	 */
	public void setImageEmpty(String imageEmpty)
	{
		if ((imageEmpty == _imageEmpty)
			|| ((imageEmpty != null)
				&& (_imageEmpty != null)
				&& imageEmpty.equals(_imageEmpty)))
			return;

		_imageEmpty = imageEmpty;
	}

	/**
	 * Initializes attribute not to the given value.
	 * @param not Boolean indicating if attribute not is set or unset.
	 */
	public void setNot(boolean not)
	{
		if (not == _not)
			return;
		_not = not;
	}

	/**
	 * Sets attribute status to the given value.
	 * @param status New value of status.
	 * @exception IllegalArgumentException if status parameter is null.
	 */
	public void setStatus(Status status)
	{
		if (status == _status)
			return;
		if (status == null)
			throw new IllegalArgumentException("BF AppEngine : invalid value for attribute status");

		_status = status;
		if (_internalView instanceof LySetStatusInterface)
			((LySetStatusInterface)_internalView).setStatus(status);
	}

	/**
	 * Initializes attribute bean to the given value.
	 * @param bean New value of bean.
	 */
	public void setBean(String bean)
	{
		if ((bean == _bean)
			|| ((bean != null)
				&& (_bean != null)
				&& bean.equals(_bean)))
			return;

		_bean = bean;
	}

	/**
	 * Initializes attribute font to the given value.
	 * @param font New value of font.
	 */
	public void setFont(String font)
	{
		if ((font == _font)
			|| ((font != null)
				&& (_font != null)
				&& font.equals(_font)))
			return;

		_font = font;
	}

	/**
	 * Initializes attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color)
	{
		if ((color == _color)
			|| ((color != null)
				&& (_color != null)
				&& color.equals(_color)))
			return;

		_color = color;
	}

	/**
	 * Initializes attribute fontColor to the given value.
	 * @param fontColor New value of fontColor.
	 */
	public void setFontColor(String fontColor)
	{
		if ((fontColor == _fontColor)
			|| ((fontColor != null)
				&& (_fontColor != null)
				&& fontColor.equals(_fontColor)))
			return;

		_fontColor = fontColor;
	}

	/**
	 * Initializes attribute nCols to the given value.
	 * @param nCols New value of nCols.
	 */
	public void setNCols(int nCols)
	{
		if (nCols == _nCols)
			return;
		_nCols = nCols;
	}

	/**
	 * Initializes attribute backgroundImage to the given value.
	 * @param backgroundImage New value of backgroundImage.
	 */
	public void setBackgroundImage(String backgroundImage)
	{
		if ((backgroundImage == _backgroundImage)
			|| ((backgroundImage != null)
				&& (_backgroundImage != null)
				&& backgroundImage.equals(_backgroundImage)))
			return;

		_backgroundImage = backgroundImage;
	}

	/**
	 * Initializes attribute hooverImage to the given value.
	 * @param hooverImage New value of hooverImage.
	 */
	public void setHooverImage(String hooverImage)
	{
		if ((hooverImage == _hooverImage)
			|| ((hooverImage != null)
				&& (_hooverImage != null)
				&& hooverImage.equals(_hooverImage)))
			return;

		_hooverImage = hooverImage;
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<BOOLEAN");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && (_labelChecked != null))
				|| ((_template != null) && (_labelChecked != _template._labelChecked)))
			pw.print(" labelChecked='" + LyString.getXmlString(_labelChecked) + "'");
		if (((_template == null) && (_imageChecked != getStringValue("BOOLEAN.imageChecked", "$ALL.imageChecked")))
				|| ((_template != null) && (_imageChecked != _template._imageChecked)))
			pw.print(" imageChecked='" + LyString.getXmlString(_imageChecked) + "'");
		if (((_template == null) && (_labelUnchecked != null))
				|| ((_template != null) && (_labelUnchecked != _template._labelUnchecked)))
			pw.print(" labelUnchecked='" + LyString.getXmlString(_labelUnchecked) + "'");
		if (((_template == null) && (_imageUnchecked != getStringValue("BOOLEAN.imageUnchecked", "$ALL.imageUnchecked")))
				|| ((_template != null) && (_imageUnchecked != _template._imageUnchecked)))
			pw.print(" imageUnchecked='" + LyString.getXmlString(_imageUnchecked) + "'");
		if (((_template == null) && (_labelEmpty != null))
				|| ((_template != null) && (_labelEmpty != _template._labelEmpty)))
			pw.print(" labelEmpty='" + LyString.getXmlString(_labelEmpty) + "'");
		if (((_template == null) && (_imageEmpty != getStringValue("BOOLEAN.imageEmpty", "$ALL.imageEmpty")))
				|| ((_template != null) && (_imageEmpty != _template._imageEmpty)))
			pw.print(" imageEmpty='" + LyString.getXmlString(_imageEmpty) + "'");
		if (((_template == null) && _not)
				|| ((_template != null) && (_not != _template._not)))
			pw.print(" not='" + _not + "'");
		if ((_template == null)
			|| (_status != _template._status))
		{
			if (_status == Status.CHECKED)
				pw.print(" status='CHECKED'");
			else if (_status == Status.UNCHECKED)
				pw.print(" status='UNCHECKED'");
			else if ((_status == Status.EMPTY) && (_template != null))
				pw.print(" status='EMPTY'");
		}
		if (((_template == null) && (_bean != null))
				|| ((_template != null) && (_bean != _template._bean)))
			pw.print(" bean='" + LyString.getXmlString(_bean) + "'");
		if (((_template == null) && (_font != getStringValue("BOOLEAN.font", "$ALL.font")))
				|| ((_template != null) && (_font != _template._font)))
			pw.print(" font='" + LyString.getXmlString(_font) + "'");
		if (((_template == null) && (_color != getStringValue("BOOLEAN.color", "$ALL.color")))
				|| ((_template != null) && (_color != _template._color)))
			pw.print(" color='" + LyString.getXmlString(_color) + "'");
		if (((_template == null) && (_fontColor != getStringValue("BOOLEAN.fontColor", "$ALL.fontColor")))
				|| ((_template != null) && (_fontColor != _template._fontColor)))
			pw.print(" fontColor='" + LyString.getXmlString(_fontColor) + "'");
		if (((_template == null) && (_nCols != getIntValue("BOOLEAN.nCols", -1)))
				|| ((_template != null) && (_nCols != _template._nCols)))
			pw.print(" nCols='" + _nCols + "'");
		if (((_template == null) && (_backgroundImage != getStringValue("BOOLEAN.backgroundImage", "$ALL.backgroundImage")))
				|| ((_template != null) && (_backgroundImage != _template._backgroundImage)))
			pw.print(" backgroundImage='" + LyString.getXmlString(_backgroundImage) + "'");
		if (((_template == null) && (_hooverImage != getStringValue("BOOLEAN.hooverImage", "$ALL.hooverImage")))
				|| ((_template != null) && (_hooverImage != _template._hooverImage)))
			pw.print(" hooverImage='" + LyString.getXmlString(_hooverImage) + "'");
		pw.println("/>");
	}
}
