/*
 * LyCanvas.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved. Current Version: $Id: LyCanvas.java,v 6.5 2007/12/05
 * 09:01:20 jls Exp $.
 */

package leon.view;

import leon.misc.*;
import leon.view.event.*;

import java.lang.reflect.*;
import java.util.*;


/**
 * The LyCanvas class implements generic behavior for the CANVAS components as described in views.dt
 * regardless of the requested target (win, swing, html...) It represents CLASS_CANVAS.
 *
 * @see  LyComponent
 */
public class LyCanvas extends LyCanvasBase
{
	public static final String __VERSION = "$Id: LyCanvas.java 33969 2012-01-30 09:12:52Z nhamon $";

    /**
     * Default constructor.
     */
    protected LyCanvas()
    {
    }
    
	/**
	 * Constructor with all required attributes.
	 */
	public LyCanvas(String javaName)
	{
		super(javaName);
	}

	/* Protected methods for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 *
	 * @param  node  Node from which the component is constructed.
	 *
	 * @see    LyViewNodeFactory#buildElement
	 */
	protected LyCanvas(LyViewNode node)
	{
		super(node);
	}

	/**
	 * Get internal view implementation.
	 *
	 * @param  build  Indicates if this representation should be built if it doesn't exist.
	 */
	public LyViewComponent getInternalView(boolean build)
	{
		if ((_internalView != null) || !build)
			return _internalView;

		if (_javaName == null)
		{
			LyArea area = new LyArea();

			area.setEnvironment(getEnvironment());

			_internalView = area.getInternalView(true);

			return _internalView;
		}

		Class<?>[]  types = new Class[1];
		Object[]    parameters = new Object[1];

		types[0] = LyCanvas.class;
		parameters[0] = this;

		try
		{
			Class<?>	   viewClass = Class.forName(_javaName);
			Constructor<?> constructor = viewClass.getConstructor(types);

			_internalView = (LyViewComponent)constructor.newInstance(parameters);
		}
		catch (Exception e)
		{
			LyEnvironment environment = getEnvironment();

			environment.writeException(e);
			throw new IllegalArgumentException("BF AppEngine : can't build internal view for Canvas, specific view class : "
				+ _javaName);
		}

		return _internalView;
	} // end method getInternalView

	/**
	 * Utility method to post a demand to controller registered on this canvas.
	 *
	 * @param  type             Demand type (see LyActionDemand.TYPE_XXX).
	 * @param  source           Id of the demand.
	 * @param  parameterValues  Table of parameter values.
	 */
	public void postDemand(int type, String source, Hashtable<String, String> parameterValues)
	{
		LyViewManager  viewManager = getViewManager();
		LyActionDemand demand = new LyActionDemand(LyActionDemand.TYPE_ACTION, source);

		if (parameterValues != null)
			demand.setParameterValues(new _LyCanvasParameterValues(parameterValues));

		viewManager.processDemand(getController(), demand);
	}

	/**
	 * Utility method to post a TYPE_ACTION demand to controller registered on this canvas.
	 *
	 * @param  source           Id of the demand.
	 * @param  parameterValues  Table of parameter values.
	 */
	public final void postDemand(String source, Hashtable<String, String> parameterValues)
	{
		postDemand(LyActionDemand.TYPE_ACTION, source, parameterValues);
	}

	/**
	 * Utility method to post a TYPE_ACTION demand to controller registered on this canvas.
	 *
	 * @param  source  Id of the demand.
	 */
	public final void postDemand(String source)
	{
		postDemand(LyActionDemand.TYPE_ACTION, source, null);
	}

	/**
	 * Parameter values for canvas.
	 */
	class _LyCanvasParameterValues
		implements LyParameterValues
	{
		Hashtable<String, String> _parameterValues;

		_LyCanvasParameterValues(Hashtable<String, String> parameterValues)
		{
			_parameterValues = parameterValues;
		}

		public String getParameter(String param)
		{
			return (String)_parameterValues.get(param);
		}

		public String[] getParameterNames()
		{
			Enumeration<String> names = _parameterValues.keys();
			int		            i, n = _parameterValues.size();
			String[]            result = new String[n];

			for (i = 0; names.hasMoreElements(); i++)
				result[i] = names.nextElement();

			return result;
		}

		public String[] getParameterValues(String param)
		{
			Enumeration<String> values = _parameterValues.elements();
			int		            i, n = _parameterValues.size();
			String[]            result = new String[n];

			for (i = 0; values.hasMoreElements(); i++)
				result[i] = values.nextElement();

			return result;
		}

		public void updateViewFromValues(LyComponent component)
		{
			updateViewFromValues(component, false);
		}

		public void updateViewFromValues(LyComponent component, boolean needValueChange)
		{
			updateViewFromValues(component, null, false);
		}

		public void updateViewFromValues(LyComponent component, LyComponent priorityComponent, boolean needValueChange)
		{
		}
	} // end class _LyCanvasParameterValues
} // end class LyCanvas
