/*
 * LyCell.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;

import java.util.*;
import java.io.*;
import leon.view.graphic.*;
import leon.view.getinterface.*;
import leon.view.setinterface.*;
import leon.misc.LyString;


/**
 * The LyCell class is a generated Java implementation that corresponds to the element 
 * CELL of the descriptive grammar of the views (views.dtd).
 * <B>&lt;CELL&gt; : Cell</B>
 * A cell may contain sub-cells to allow the construction of complex tables.
 * @see leon.view.LyComponent
 * See also tag <A href="../../conf/dtd/view/CELL.html">CELL</A> of grammar leon_views.dtd.
 */
public class LyCell extends LyComponent
	implements  LyGetIdInterface,
				LyGetLabelInterface,
				LySetLabelInterface,
				LySetEditableInterface,
				LySetSelectedInterface,
				LySetImageInterface,
				LySetColorInterface
{
	public static final String			__VERSION = "$Id$";


	/** Possible values for attribute 'align'. */


	/** Possible values for attribute 'type'. */
	public enum Type
	{
		NORMAL,
		RICH;
	}


	/* Static methods */

	/**
	 * Utility method to get type value from a string.
	 * @param token Token string
	 * @return Type value corresponding to this token
	 */
	public static final Type getType(String token)
	{
		if (token == null)
			return null;
		else if (token.equalsIgnoreCase("NORMAL")
				|| token.equals("0"))
			return Type.NORMAL;
		else if (token.equalsIgnoreCase("RICH")
				|| token.equals("1"))
			return Type.RICH;
		else
			return Type.NORMAL;
	}


	/* Attributes */

	/**
	 * Identifier of the component.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyCell			_template;

	/**
	 * Text of the cell.
	 */
	protected String			_label;

	/**
	 * Identifier used when initiating the action that is associated to the cell.
	 */
	protected String			_objectId;

	/**
	 * Indicates if the content of the cell is an hyperlink.
	 */
	protected String			_hLink;

	/**
	 * Indicates whether the cell is editable or not.
	 */
	protected boolean			_editable;

	/**
	 * Indicates whether the cell is selected or not.
	 */
	protected boolean			_selected;

	/**
	 * Text alignment of the label of the cell (left, right or centered).
	 */
	protected LyHeader.Align			_align;

	/**
	 * Image of the cell.
	 */
	protected String			_image;

	/**
	 * Text of the tooltip for the cell.
	 */
	protected String			_tooltip;

	/**
	 * Color of the top border of the cell.
	 */
	protected String			_topBorderColor;

	/**
	 * Color of the bottom border of the cell.
	 */
	protected String			_bottomBorderColor;

	/**
	 * Color of the left border of the cell.
	 */
	protected String			_leftBorderColor;

	/**
	 * Color of the right border of the cell.
	 */
	protected String			_rightBorderColor;

	/**
	 * Image placed in the top-left corner of the cell.
	 */
	protected String			_topLeftImage;

	/**
	 * Image placed in the top-right corner of the cell.
	 */
	protected String			_topRightImage;

	/**
	 * Image placed in the bottom-left corner of the cell.
	 */
	protected String			_bottomLeftImage;

	/**
	 * Image placed in the bottom-right corner of the cell.
	 */
	protected String			_bottomRightImage;

	/**
	 * Image of the cell.
	 */
	protected String			_font;

	/**
	 * Background color of the cell.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	/**
	 * Type of the cell, either normal or rich text.
	 */
	protected Type				_type;


	/* Sub components */

	/**
	 * Sub-cells of the cell.
	 */
	protected ArrayList<LyCell>		_cells;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_id = null;
		_template = null;
		_label = null;
		_objectId = null;
		_hLink = null;
		_editable = getBooleanValue("CELL.editable", false);
		_selected = getBooleanValue("CELL.selected", false);
		_align = LyHeader.Align.LEFT;
		_image = getStringValue("CELL.image", "$ALL.image");
		_tooltip = null;
		_topBorderColor = getStringValue("CELL.topBorderColor", "$ALL.topBorderColor");
		_bottomBorderColor = getStringValue("CELL.bottomBorderColor", "$ALL.bottomBorderColor");
		_leftBorderColor = getStringValue("CELL.leftBorderColor", "$ALL.leftBorderColor");
		_rightBorderColor = getStringValue("CELL.rightBorderColor", "$ALL.rightBorderColor");
		_topLeftImage = getStringValue("CELL.topLeftImage", "$ALL.topLeftImage");
		_topRightImage = getStringValue("CELL.topRightImage", "$ALL.topRightImage");
		_bottomLeftImage = getStringValue("CELL.bottomLeftImage", "$ALL.bottomLeftImage");
		_bottomRightImage = getStringValue("CELL.bottomRightImage", "$ALL.bottomRightImage");
		_font = getStringValue("CELL.font", "$ALL.font");
		_color = getStringValue("CELL.color", "$ALL.color");
		_fontColor = getStringValue("CELL.fontColor", "$ALL.fontColor");
		_type = Type.NORMAL;

		// Initialize sub components
		_cells = null;
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyCell	model = (LyCell)component;

		_model = component;
		_label = model._label;
		_objectId = model._objectId;
		_hLink = model._hLink;
		_editable = model._editable;
		_selected = model._selected;
		_align = model._align;
		_image = model._image;
		_tooltip = model._tooltip;
		_topBorderColor = model._topBorderColor;
		_bottomBorderColor = model._bottomBorderColor;
		_leftBorderColor = model._leftBorderColor;
		_rightBorderColor = model._rightBorderColor;
		_topLeftImage = model._topLeftImage;
		_topRightImage = model._topRightImage;
		_bottomLeftImage = model._bottomLeftImage;
		_bottomRightImage = model._bottomRightImage;
		_font = model._font;
		_color = model._color;
		_fontColor = model._fontColor;
		_type = model._type;

		if (REWRITING_MODE)
			return;

		// Copy sub-components
		LyCell	cells;
		int			i, n;

		n = (model._cells == null ? 0 : model._cells.size());
		for (i = 0; i < n; i++)
		{
			cells = model._cells.get(i);
			if (cells != null)
				cells = (LyCell)cells.cloneComponent();
			addChild(cells);
		}
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	public LyCell()
	{
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyCell(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_id = node.getString("id");
		_template = (LyCell)node.getComponent("template");

		if (_template != null)
			initFromModel(_template);

		if (node.isSpecified("label"))
			_label = node.getString("label");
		if (node.isSpecified("objectId"))
			_objectId = node.getString("objectId");
		if (node.isSpecified("hLink"))
			_hLink = node.getString("hLink");
		if (node.isSpecified("editable"))
			_editable = node.getBoolean("editable");
		if (node.isSpecified("selected"))
			_selected = node.getBoolean("selected");
		if (node.isSpecified("align"))
			_align = LyHeader.getAlign(node.getString("align"));
		if (node.isSpecified("image"))
			_image = node.getString("image");
		if (node.isSpecified("tooltip"))
			_tooltip = node.getString("tooltip");
		if (node.isSpecified("topBorderColor"))
			_topBorderColor = node.getString("topBorderColor");
		if (node.isSpecified("bottomBorderColor"))
			_bottomBorderColor = node.getString("bottomBorderColor");
		if (node.isSpecified("leftBorderColor"))
			_leftBorderColor = node.getString("leftBorderColor");
		if (node.isSpecified("rightBorderColor"))
			_rightBorderColor = node.getString("rightBorderColor");
		if (node.isSpecified("topLeftImage"))
			_topLeftImage = node.getString("topLeftImage");
		if (node.isSpecified("topRightImage"))
			_topRightImage = node.getString("topRightImage");
		if (node.isSpecified("bottomLeftImage"))
			_bottomLeftImage = node.getString("bottomLeftImage");
		if (node.isSpecified("bottomRightImage"))
			_bottomRightImage = node.getString("bottomRightImage");
		if (node.isSpecified("font"))
			_font = node.getString("font");
		if (node.isSpecified("color"))
			_color = node.getString("color");
		if (node.isSpecified("fontColor"))
			_fontColor = node.getString("fontColor");
		if (node.isSpecified("type"))
			_type = getType(node.getString("type"));
	}

	/**
	 * End of construction : completion with child component.
	 * @param child Sub child component.
	 * @see leon.view.LyComponent#addChild
	 */
	protected void addChild(LyComponent child)
	{
		// Add sub-components
		if (child instanceof LyCell)
			addCell((LyCell)child);
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyCell clone = new LyCell();

		clone.initFromModel(this);
		clone._template = this;
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of id.
	 * @return value of id.
	 */
	public String getId()
	{
		return _id;
	}

	/**
	 * Gets value of template.
	 * @return value of template.
	 */
	public LyCell getTemplate()
	{
		return _template;
	}

	/**
	 * Gets value of label.
	 * @return value of label.
	 */
	public String getLabel()
	{
		return _label;
	}

	/**
	 * Gets value of objectId.
	 * @return value of objectId.
	 */
	public String getObjectId()
	{
		return _objectId;
	}

	/**
	 * Gets value of hLink.
	 * @return value of hLink.
	 */
	public String getHLink()
	{
		return _hLink;
	}

	/**
	 * Checks if value of attribute editable is set.
	 * @return true if editable is set, false otherwise.
	 */
	public boolean isEditable()
	{
		return _editable;
	}

	/**
	 * Checks if value of attribute selected is set.
	 * @return true if selected is set, false otherwise.
	 */
	public boolean isSelected()
	{
		return _selected;
	}

	/**
	 * Gets value of align.
	 * @return value of align.
	 */
	public LyHeader.Align getAlign()
	{
		return _align;
	}

	/**
	 * Gets value of image.
	 * @return value of image.
	 */
	public String getImage()
	{
		return _image;
	}

	/**
	 * Checks whether the image attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultImage()
	{
		return (_image == getStringValue("CELL.image"));
	}

	/**
	 * Gets value of tooltip.
	 * @return value of tooltip.
	 */
	public String getTooltip()
	{
		return _tooltip;
	}

	/**
	 * Gets value of topBorderColor.
	 * @return value of topBorderColor.
	 */
	public String getTopBorderColor()
	{
		return _topBorderColor;
	}

	/**
	 * Checks whether the topBorderColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultTopBorderColor()
	{
		return (_topBorderColor == getStringValue("CELL.topBorderColor"));
	}

	/**
	 * Gets value of bottomBorderColor.
	 * @return value of bottomBorderColor.
	 */
	public String getBottomBorderColor()
	{
		return _bottomBorderColor;
	}

	/**
	 * Checks whether the bottomBorderColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultBottomBorderColor()
	{
		return (_bottomBorderColor == getStringValue("CELL.bottomBorderColor"));
	}

	/**
	 * Gets value of leftBorderColor.
	 * @return value of leftBorderColor.
	 */
	public String getLeftBorderColor()
	{
		return _leftBorderColor;
	}

	/**
	 * Checks whether the leftBorderColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultLeftBorderColor()
	{
		return (_leftBorderColor == getStringValue("CELL.leftBorderColor"));
	}

	/**
	 * Gets value of rightBorderColor.
	 * @return value of rightBorderColor.
	 */
	public String getRightBorderColor()
	{
		return _rightBorderColor;
	}

	/**
	 * Checks whether the rightBorderColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultRightBorderColor()
	{
		return (_rightBorderColor == getStringValue("CELL.rightBorderColor"));
	}

	/**
	 * Gets value of topLeftImage.
	 * @return value of topLeftImage.
	 */
	public String getTopLeftImage()
	{
		return _topLeftImage;
	}

	/**
	 * Checks whether the topLeftImage attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultTopLeftImage()
	{
		return (_topLeftImage == getStringValue("CELL.topLeftImage"));
	}

	/**
	 * Gets value of topRightImage.
	 * @return value of topRightImage.
	 */
	public String getTopRightImage()
	{
		return _topRightImage;
	}

	/**
	 * Checks whether the topRightImage attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultTopRightImage()
	{
		return (_topRightImage == getStringValue("CELL.topRightImage"));
	}

	/**
	 * Gets value of bottomLeftImage.
	 * @return value of bottomLeftImage.
	 */
	public String getBottomLeftImage()
	{
		return _bottomLeftImage;
	}

	/**
	 * Checks whether the bottomLeftImage attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultBottomLeftImage()
	{
		return (_bottomLeftImage == getStringValue("CELL.bottomLeftImage"));
	}

	/**
	 * Gets value of bottomRightImage.
	 * @return value of bottomRightImage.
	 */
	public String getBottomRightImage()
	{
		return _bottomRightImage;
	}

	/**
	 * Checks whether the bottomRightImage attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultBottomRightImage()
	{
		return (_bottomRightImage == getStringValue("CELL.bottomRightImage"));
	}

	/**
	 * Gets value of font.
	 * @return value of font.
	 */
	public String getFont()
	{
		return _font;
	}

	/**
	 * Checks whether the font attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFont()
	{
		return (_font == getStringValue("CELL.font"));
	}

	/**
	 * Gets value of color.
	 * @return value of color.
	 */
	public String getColor()
	{
		return _color;
	}

	/**
	 * Checks whether the color attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultColor()
	{
		return (_color == getStringValue("CELL.color"));
	}

	/**
	 * Gets value of fontColor.
	 * @return value of fontColor.
	 */
	public String getFontColor()
	{
		return _fontColor;
	}

	/**
	 * Checks whether the fontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFontColor()
	{
		return (_fontColor == getStringValue("CELL.fontColor"));
	}

	/**
	 * Gets value of type.
	 * @return value of type.
	 */
	public Type getType()
	{
		return _type;
	}

	/**
	 * Get sub-component Cell of index i.
	 * @param i the index of the Cell to get.
	 * @return sub-component Cell of index i.
	 */
	public LyCell getCell(int i)
	{
		if (_cells == null)
			return null;
		if ((i < 0) || (i >= _cells.size()))
			return null;
		return _cells.get(i);
	}

	/**
	 * Get sub-component Cell count.
	 * @return sub-component Cell count.
	 */
	public int getCellCount()
	{
		if (_cells == null)
			return 0;
		return _cells.size();
	}


	/* Modifiers */

	/**
	 * Initializes attribute id to the given value.
	 * @param id New value of id.
	 */
	public void setId(String id)
	{
		if ((id == _id)
			|| ((id != null)
				&& (_id != null)
				&& id.equals(_id)))
			return;

		_id = (id != null ? id.intern() : null);
	}

	/**
	 * Initializes attribute template to the given value.
	 * @param template New value of template.
	 */
	public void setTemplate(LyCell template)
	{
		if (template == null)
			return;
		if (_template != null)
			throw new IllegalStateException("BF AppEngine : attribute template was already set.");

		initFromModel(template);
		_template = template;
	}

	/**
	 * Sets attribute label to the given value.
	 * @param label New value of label.
	 */
	public void setLabel(String label)
	{
		if ((label == _label)
			|| ((label != null)
				&& (_label != null)
				&& label.equals(_label)))
			return;

		_label = label;
		if (_internalView != null)
			((LyGraphicalCell)_internalView).setLabel(label);
	}

	/**
	 * Sets attribute objectId to the given value.
	 * @param objectId New value of objectId.
	 */
	public void setObjectId(String objectId)
	{
		if ((objectId == _objectId)
			|| ((objectId != null)
				&& (_objectId != null)
				&& objectId.equals(_objectId)))
			return;

		_objectId = objectId;
		if (_internalView != null)
			((LyGraphicalCell)_internalView).setObjectId(objectId);
	}

	/**
	 * Initializes attribute hLink to the given value.
	 * @param hLink New value of hLink.
	 */
	public void setHLink(String hLink)
	{
		if ((hLink == _hLink)
			|| ((hLink != null)
				&& (_hLink != null)
				&& hLink.equals(_hLink)))
			return;

		_hLink = hLink;
	}

	/**
	 * Sets attribute editable to the given value.
	 * @param editable Boolean indicating if attribute editable is set or unset.
	 */
	public void setEditable(boolean editable)
	{
		if (editable == _editable)
			return;
		_editable = editable;
		if (_internalView != null)
			((LyGraphicalCell)_internalView).setEditable(editable);
	}

	/**
	 * Sets attribute selected to the given value.
	 * @param selected Boolean indicating if attribute selected is set or unset.
	 */
	public void setSelected(boolean selected)
	{
		if (selected == _selected)
			return;
		_selected = selected;
		if (_internalView != null)
			((LyGraphicalCell)_internalView).setSelected(selected);
	}

	/**
	 * Initializes attribute align to the given value.
	 * @param align New value of align.
	 * @exception IllegalArgumentException if align parameter is null.
	 */
	public void setAlign(LyHeader.Align align)
	{
		if (align == _align)
			return;
		if (align == null)
			throw new IllegalArgumentException("BF AppEngine : invalid value for attribute align");

		_align = align;
	}

	/**
	 * Sets attribute image to the given value.
	 * @param image New value of image.
	 */
	public void setImage(String image)
	{
		if ((image == _image)
			|| ((image != null)
				&& (_image != null)
				&& image.equals(_image)))
			return;

		_image = image;
		if (_internalView != null)
			((LyGraphicalCell)_internalView).setImage(image);
	}

	/**
	 * Sets attribute tooltip to the given value.
	 * @param tooltip New value of tooltip.
	 */
	public void setTooltip(String tooltip)
	{
		if ((tooltip == _tooltip)
			|| ((tooltip != null)
				&& (_tooltip != null)
				&& tooltip.equals(_tooltip)))
			return;

		_tooltip = tooltip;
		if (_internalView != null)
			((LyGraphicalCell)_internalView).setTooltip(tooltip);
	}

	/**
	 * Sets attribute topBorderColor to the given value.
	 * @param topBorderColor New value of topBorderColor.
	 */
	public void setTopBorderColor(String topBorderColor)
	{
		if ((topBorderColor == _topBorderColor)
			|| ((topBorderColor != null)
				&& (_topBorderColor != null)
				&& topBorderColor.equals(_topBorderColor)))
			return;

		_topBorderColor = topBorderColor;
		if (_internalView != null)
			((LyGraphicalCell)_internalView).setTopBorderColor(topBorderColor);
	}

	/**
	 * Sets attribute bottomBorderColor to the given value.
	 * @param bottomBorderColor New value of bottomBorderColor.
	 */
	public void setBottomBorderColor(String bottomBorderColor)
	{
		if ((bottomBorderColor == _bottomBorderColor)
			|| ((bottomBorderColor != null)
				&& (_bottomBorderColor != null)
				&& bottomBorderColor.equals(_bottomBorderColor)))
			return;

		_bottomBorderColor = bottomBorderColor;
		if (_internalView != null)
			((LyGraphicalCell)_internalView).setBottomBorderColor(bottomBorderColor);
	}

	/**
	 * Sets attribute leftBorderColor to the given value.
	 * @param leftBorderColor New value of leftBorderColor.
	 */
	public void setLeftBorderColor(String leftBorderColor)
	{
		if ((leftBorderColor == _leftBorderColor)
			|| ((leftBorderColor != null)
				&& (_leftBorderColor != null)
				&& leftBorderColor.equals(_leftBorderColor)))
			return;

		_leftBorderColor = leftBorderColor;
		if (_internalView != null)
			((LyGraphicalCell)_internalView).setLeftBorderColor(leftBorderColor);
	}

	/**
	 * Sets attribute rightBorderColor to the given value.
	 * @param rightBorderColor New value of rightBorderColor.
	 */
	public void setRightBorderColor(String rightBorderColor)
	{
		if ((rightBorderColor == _rightBorderColor)
			|| ((rightBorderColor != null)
				&& (_rightBorderColor != null)
				&& rightBorderColor.equals(_rightBorderColor)))
			return;

		_rightBorderColor = rightBorderColor;
		if (_internalView != null)
			((LyGraphicalCell)_internalView).setRightBorderColor(rightBorderColor);
	}

	/**
	 * Sets attribute topLeftImage to the given value.
	 * @param topLeftImage New value of topLeftImage.
	 */
	public void setTopLeftImage(String topLeftImage)
	{
		if ((topLeftImage == _topLeftImage)
			|| ((topLeftImage != null)
				&& (_topLeftImage != null)
				&& topLeftImage.equals(_topLeftImage)))
			return;

		_topLeftImage = topLeftImage;
		if (_internalView != null)
			((LyGraphicalCell)_internalView).setTopLeftImage(topLeftImage);
	}

	/**
	 * Sets attribute topRightImage to the given value.
	 * @param topRightImage New value of topRightImage.
	 */
	public void setTopRightImage(String topRightImage)
	{
		if ((topRightImage == _topRightImage)
			|| ((topRightImage != null)
				&& (_topRightImage != null)
				&& topRightImage.equals(_topRightImage)))
			return;

		_topRightImage = topRightImage;
		if (_internalView != null)
			((LyGraphicalCell)_internalView).setTopRightImage(topRightImage);
	}

	/**
	 * Sets attribute bottomLeftImage to the given value.
	 * @param bottomLeftImage New value of bottomLeftImage.
	 */
	public void setBottomLeftImage(String bottomLeftImage)
	{
		if ((bottomLeftImage == _bottomLeftImage)
			|| ((bottomLeftImage != null)
				&& (_bottomLeftImage != null)
				&& bottomLeftImage.equals(_bottomLeftImage)))
			return;

		_bottomLeftImage = bottomLeftImage;
		if (_internalView != null)
			((LyGraphicalCell)_internalView).setBottomLeftImage(bottomLeftImage);
	}

	/**
	 * Sets attribute bottomRightImage to the given value.
	 * @param bottomRightImage New value of bottomRightImage.
	 */
	public void setBottomRightImage(String bottomRightImage)
	{
		if ((bottomRightImage == _bottomRightImage)
			|| ((bottomRightImage != null)
				&& (_bottomRightImage != null)
				&& bottomRightImage.equals(_bottomRightImage)))
			return;

		_bottomRightImage = bottomRightImage;
		if (_internalView != null)
			((LyGraphicalCell)_internalView).setBottomRightImage(bottomRightImage);
	}

	/**
	 * Initializes attribute font to the given value.
	 * @param font New value of font.
	 */
	public void setFont(String font)
	{
		if ((font == _font)
			|| ((font != null)
				&& (_font != null)
				&& font.equals(_font)))
			return;

		_font = font;
	}

	/**
	 * Sets attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color)
	{
		if ((color == _color)
			|| ((color != null)
				&& (_color != null)
				&& color.equals(_color)))
			return;

		_color = color;
		if (_internalView != null)
			((LyGraphicalCell)_internalView).setColor(color);
	}

	/**
	 * Initializes attribute fontColor to the given value.
	 * @param fontColor New value of fontColor.
	 */
	public void setFontColor(String fontColor)
	{
		if ((fontColor == _fontColor)
			|| ((fontColor != null)
				&& (_fontColor != null)
				&& fontColor.equals(_fontColor)))
			return;

		_fontColor = fontColor;
	}

	/**
	 * Initializes attribute type to the given value.
	 * @param type New value of type.
	 * @exception IllegalArgumentException if type parameter is null.
	 */
	public void setType(Type type)
	{
		if (type == _type)
			return;
		if (type == null)
			throw new IllegalArgumentException("BF AppEngine : invalid value for attribute type");

		_type = type;
	}

	/**
	 * Adds a sub-component cell.
	 * @param cell The cell to add to cells list.
	 */
	public void addCell(LyCell cell)
	{
		insertCell(cell, -1);
	}

	/**
	 * Inserts a sub-component cell at given position.
	 * @param cell The cell to add to cells list.
	 * @param pos	Position where the sub-component is inserted.
	 */
	public void insertCell(LyCell cell, int pos)
	{
		if (_cells == null)
		{
			_cells = new ArrayList<LyCell>(1);
			_children = _cells;
		}
		else if (_cells.contains(cell))
			return;
		if (pos >= 0)
			_cells.add(pos,cell);
		if (cell != null)
			cell.setParent(this);
		if (_internalView != null)
			((LyGraphicalCell)_internalView).addCell(cell, pos);
	}

	/**
	 * Removes a sub-component cell.
	 * @param cell The cell to remove to cells list.
	 * @exception IllegalStateException if cells list is empty.
	 * @exception IllegalArgumentException if cells list doesn't contain cell.
	 */
	public void removeCell(LyCell cell)
	{
		if (_cells == null)
			throw new IllegalStateException("BF AppEngine : cells list is empty");
		if (!_cells.contains(cell))
			throw new IllegalArgumentException("BF AppEngine : cells list doesn't contain cell");

		if (_internalView != null)
			((LyGraphicalCell)_internalView).removeCell(cell);
		cell.setParent(null);
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<CELL");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template._label)))
			pw.print(" label='" + LyString.getXmlString(_label) + "'");
		if (((_template == null) && (_objectId != null))
				|| ((_template != null) && (_objectId != _template._objectId)))
			pw.print(" objectId='" + LyString.getXmlString(_objectId) + "'");
		if (((_template == null) && (_hLink != null))
				|| ((_template != null) && (_hLink != _template._hLink)))
			pw.print(" hLink='" + LyString.getXmlString(_hLink) + "'");
		if (((_template == null) && _editable)
				|| ((_template != null) && (_editable != _template._editable)))
			pw.print(" editable='" + _editable + "'");
		if (((_template == null) && _selected)
				|| ((_template != null) && (_selected != _template._selected)))
			pw.print(" selected='" + _selected + "'");
		if ((_template == null)
			|| (_align != _template._align))
		{
			if ((_align == LyHeader.Align.LEFT) && (_template != null))
				pw.print(" align='LEFT'");
			else if (_align == LyHeader.Align.RIGHT)
				pw.print(" align='RIGHT'");
			else if (_align == LyHeader.Align.CENTER)
				pw.print(" align='CENTER'");
		}
		if (((_template == null) && (_image != getStringValue("CELL.image", "$ALL.image")))
				|| ((_template != null) && (_image != _template._image)))
			pw.print(" image='" + LyString.getXmlString(_image) + "'");
		if (((_template == null) && (_tooltip != null))
				|| ((_template != null) && (_tooltip != _template._tooltip)))
			pw.print(" tooltip='" + LyString.getXmlString(_tooltip) + "'");
		if (((_template == null) && (_topBorderColor != getStringValue("CELL.topBorderColor", "$ALL.topBorderColor")))
				|| ((_template != null) && (_topBorderColor != _template._topBorderColor)))
			pw.print(" topBorderColor='" + LyString.getXmlString(_topBorderColor) + "'");
		if (((_template == null) && (_bottomBorderColor != getStringValue("CELL.bottomBorderColor", "$ALL.bottomBorderColor")))
				|| ((_template != null) && (_bottomBorderColor != _template._bottomBorderColor)))
			pw.print(" bottomBorderColor='" + LyString.getXmlString(_bottomBorderColor) + "'");
		if (((_template == null) && (_leftBorderColor != getStringValue("CELL.leftBorderColor", "$ALL.leftBorderColor")))
				|| ((_template != null) && (_leftBorderColor != _template._leftBorderColor)))
			pw.print(" leftBorderColor='" + LyString.getXmlString(_leftBorderColor) + "'");
		if (((_template == null) && (_rightBorderColor != getStringValue("CELL.rightBorderColor", "$ALL.rightBorderColor")))
				|| ((_template != null) && (_rightBorderColor != _template._rightBorderColor)))
			pw.print(" rightBorderColor='" + LyString.getXmlString(_rightBorderColor) + "'");
		if (((_template == null) && (_topLeftImage != getStringValue("CELL.topLeftImage", "$ALL.topLeftImage")))
				|| ((_template != null) && (_topLeftImage != _template._topLeftImage)))
			pw.print(" topLeftImage='" + LyString.getXmlString(_topLeftImage) + "'");
		if (((_template == null) && (_topRightImage != getStringValue("CELL.topRightImage", "$ALL.topRightImage")))
				|| ((_template != null) && (_topRightImage != _template._topRightImage)))
			pw.print(" topRightImage='" + LyString.getXmlString(_topRightImage) + "'");
		if (((_template == null) && (_bottomLeftImage != getStringValue("CELL.bottomLeftImage", "$ALL.bottomLeftImage")))
				|| ((_template != null) && (_bottomLeftImage != _template._bottomLeftImage)))
			pw.print(" bottomLeftImage='" + LyString.getXmlString(_bottomLeftImage) + "'");
		if (((_template == null) && (_bottomRightImage != getStringValue("CELL.bottomRightImage", "$ALL.bottomRightImage")))
				|| ((_template != null) && (_bottomRightImage != _template._bottomRightImage)))
			pw.print(" bottomRightImage='" + LyString.getXmlString(_bottomRightImage) + "'");
		if (((_template == null) && (_font != getStringValue("CELL.font", "$ALL.font")))
				|| ((_template != null) && (_font != _template._font)))
			pw.print(" font='" + LyString.getXmlString(_font) + "'");
		if (((_template == null) && (_color != getStringValue("CELL.color", "$ALL.color")))
				|| ((_template != null) && (_color != _template._color)))
			pw.print(" color='" + LyString.getXmlString(_color) + "'");
		if (((_template == null) && (_fontColor != getStringValue("CELL.fontColor", "$ALL.fontColor")))
				|| ((_template != null) && (_fontColor != _template._fontColor)))
			pw.print(" fontColor='" + LyString.getXmlString(_fontColor) + "'");
		if ((_template == null)
			|| (_type != _template._type))
		{
			if ((_type == Type.NORMAL) && (_template != null))
				pw.print(" type='NORMAL'");
			else if (_type == Type.RICH)
				pw.print(" type='RICH'");
		}

		if ((getChildCount() == 0) && (_datas == null))
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		LyComponent	cells;
		int			i, n;

		n = (_cells == null ? 0 : _cells.size());
		for (i = 0; i < n; i++)
		{
			cells = _cells.get(i);
			cells.writeXml(pw, debug);
		}


		pw.println("</CELL>");
	}
}
