/*
 * LyCharType.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;
import java.io.*;
import leon.view.getinterface.*;
import leon.misc.LyString;


/**
 * The LyCharType class is a generated Java implementation that corresponds to the element 
 * CHARTYPE of the descriptive grammar of the views (views.dtd).
 * <B>&lt;CHARTYPE&gt; : Character Type</B>
 * A character type may be used to specify the characters allowed in a input field 
 * of TEXT type. If no CHARTYPE is indicated for the TEXT field, this field accepts
 * all the default character types.
 * @see leon.view.LyComponent
 * See also tag <A href="../../conf/dtd/view/CHARTYPE.html">CHARTYPE</A> of grammar leon_views.dtd.
 */
public class LyCharType extends LyComponent
	implements  LyGetValueInterface
{
	public static final String			__VERSION = "$Id$";


	/* Attributes */

	/**
	 * Value of the character type
	 */
	protected String			_value;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_value = null;
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyCharType	model = (LyCharType)component;

		_model = component;
		_value = model._value;
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	protected LyCharType()
	{
	}

	/**
	 * Constructor with all required attributes.
	 * @param value Value of the character type
	 */
	public LyCharType(String value)
	{
		// Fill up required attributes with parameters
		_value = value;
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyCharType(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_value = node.getString("value");
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyCharType clone = new LyCharType();

		clone.initFromModel(this);
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of value.
	 * @return value of value.
	 */
	public String getValue()
	{
		return _value;
	}


	/* Modifiers */

	/**
	 * Initializes attribute value to the given value.
	 * @param value New value of value.
	 * @exception IllegalArgumentException if value is null.
	 */
	public void setValue(String value)
	{
		if (value == null)
			throw new IllegalArgumentException("BF AppEngine : attribute value is required (can't be null)");

		if ((value == _value)
			|| ((value != null)
				&& (_value != null)
				&& value.equals(_value)))
			return;

		_value = value;
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<CHARTYPE");
		pw.print(" value='" + LyString.getXmlString(_value) + "'");
		pw.println("/>");
	}
}
