/*
 * LyChart.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved.
 */

package leon.view;

import leon.view.getinterface.*;


/**
 * The LyChart class implements generic behavior for the CHART components as described in views.dtd
 * regardless of the requested target (win, swing, html...). It represents CLASS_CHART.
 *
 * @author   Lyria
 * @version  $Id: LyChart.java 33165 2011-11-16 08:07:09Z nhamon $
 * @see      LyComponent
 */
public class LyChart extends LyChartBase
{
	public static final String __VERSION = "$Id: LyChart.java 33165 2011-11-16 08:07:09Z nhamon $";

	/** Indicates if the real y values have been computed. */
	protected boolean _yValuesComputed;

	/** Indicates if data must be inverted. */
	protected boolean _invertData;

	
	/**
	 * Constructor with all required attributes.
	 */
	public LyChart()
	{
		super();

		_yValuesComputed = false;
		_invertData = false;
	}

	/* Protected methods for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 *
	 * @param  node  Node from which the component is constructed.
	 *
	 * @see    LyViewNodeFactory#buildElement
	 */
	protected LyChart(LyViewNode node)
	{
		super(node);
	}

	/**
	 * Retrieve the scrollable attribute.
	 *
	 * @return  true if the component is scrollable, else false
	 */
	public boolean isScrollable()
	{
		return false;
	}

	/**
	 * Save the image of the chart.
	 *
	 * @param  tmpDir     temporary directory
	 * @param  imageFile  the PNG File
	 * @param  pw         the print writer where imagemap must be generated
	 */
	public void saveImg(String tmpDir, String imageFile)
	{
		// Retrieve internal view
		if (_internalView instanceof LyGetChartConverterInterface)
		{
			LyChartConverter chartConv = ((LyGetChartConverterInterface)_internalView).getChartConverter();

			chartConv.saveImg(tmpDir, imageFile, null, false);
		}
	}

	/**
	 * Indicates if the real y values have been computed.
	 *
	 * @return  true if the real y values have been computed, false otherwise.
	 */
	public boolean isYValuesComputed()
	{
		return _yValuesComputed;
	}

	/**
	 * Set if the real y values have been computed.
	 *
	 * @param  yValuesComputed  true if the real y values have been computed, false otherwise.
	 */
	public void setYValuesComputed(boolean yValuesComputed)
	{
		_yValuesComputed = yValuesComputed;
	}

	/**
	 * Indicates if data must be inverted.
	 *
	 * @return  true if data must be inverted, false otherwise.
	 */
	public boolean isInvertData()
	{
		return _invertData;
	}

	/**
	 * Set if data must be inverted.
	 *
	 * @param  invertData  true if data must be inverted, false otherwise.
	 */
	public void setInvertData(boolean invertData)
	{
		_invertData = invertData;
	}
} // end class LyChart
