/*
 * LyChartConverter.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved.
 */

package leon.view;

import leon.misc.*;

import java.text.*;
import java.util.*;


/**
 * This class implements the Chart object (as described in the dtd) for the viewer.
 *
 * @version  $Id: LyChartConverter.java 41110 2012-10-10 12:20:53Z jlsaouli $
 * @author   Lyria
 * @see      LyChart
 */
public abstract class LyChartConverter
{
	public static final String __VERSION = "$Id: LyChartConverter.java 41110 2012-10-10 12:20:53Z jlsaouli $";

	/** Width of the window. */
	public static int DEFAULT_WIDTH = 660;

	/** Height of the window. */
	public static int DEFAULT_HEIGHT = 420;

	/** The datas which are necessary to build the chart. */
	protected LyChart _chart;

	/** Datas. */
	protected Vector<Vector<Object>> _data = new Vector<Vector<Object>>();

	/** X axis Graduation Labels. */
	protected String[] _points;

	/** Data Series Labels. */
	protected Vector<String> _seriesLabel = new Vector<String>();

	/** Default Date Type. */
	protected short _defaultDateType = LyEnvironment.TYPE_DATE_TIME;

	/** the max of the last X interval, chartable (if needed). */
	protected Object _lastBoundInterval;

	/** Calendar constructed only if there are intervals of dates. */
	protected Calendar _calendar = null;

	/** Grid of intervals for x constructed only if there are intervals of dates. */
	protected Vector<Long> _gridX = new Vector<Long>();

	/** Grid of intervals for y constructed only if there are intervals of dates. */
	protected Vector<Long> _gridY = new Vector<Long>();

	/** Specific legend labels. */
	protected Hashtable<String, String> _specificLegendValues = new Hashtable<String, String>();

	/** Indicates if the legend must be positionned on the right of the chart. */
	protected boolean _legendRightPosition = false;

	/** Indicates if sensitive areas must be generated on the chart. */
	protected boolean _generateSensitiveAreas = false;

	/**
	 * Constructor.
	 */
	public LyChartConverter()
	{
	}

	/**
	 * Constructor.
	 *
	 * @param  chart  the chart created by LyChartController
	 *
	 * @see    leon.control.LyChartController
	 */
	public LyChartConverter(LyChart chart)
	{
		_chart = chart;

		initialize();
	}

	/**
	 * Initialization method.
	 */
	public void initialize()
	{
	}

	/**
	 * Affect a chart to this view.
	 *
	 * @param  chart  the chart created by LyChartController
	 *
	 * @see    leon.control.LyChartController
	 */
	public void setChart(LyChart chart)
	{
		_chart = chart;
	}

	/**
	 * Sets the content of this view.
	 */
	public void setContent()
	{
		reset();
	}

	/**
	 * Reset the display from the underlying XML data Main method to construct the chart.
	 */
	public abstract void reset();

	/**
	 * Set the specific legend values.
	 *
	 * @param  values
	 */
	public void setSpecificLegendValue(Hashtable<String, String> values)
	{
		_specificLegendValues = values;
	}

	/**
	 * Sets the legendRightPosition parameter.
	 *
	 * @param  rightPosition  the new legendRightPosition value
	 */
	public void setLegendRightPosition(boolean rightPosition)
	{
		_legendRightPosition = rightPosition;
	}

	/**
	 * Sets the generateSensitiveAreas parameter.
	 *
	 * @param  generate  the new generateSensitiveAreas value
	 */
	public void generateSensitiveAreas(boolean generate)
	{
		_generateSensitiveAreas = generate;
	}

	/**
	 * Returns the generateSensitiveAreas parameter.
	 */
	public boolean generateSensitiveAreas()
	{
		return _generateSensitiveAreas;
	}

	/**
	 * Create final data series : interval grouping, sum or count, labels, etc.
	 *
	 * @param  xSerie         the x serie of values to represent with a chart
	 * @param  ySerie         the y serie of values to represent with a chart
	 * @param  unknownValues  true if unknown values are autorized, false if they have to be
	 *                        replaced by zeros
	 */
	protected void createChartableData(Vector<String> xSerie, Vector<String> ySerie, boolean unknownValues)
	{
		// don't show the legend if there is only one Y variable and type of chart is plot or scatter plot
/*if (!( (_chart.getColumnCount() == 1) &&
 *                      ((_chart.getGraphType() == _chart.GraphType.SCATTER_PLOT) || (_chart.getGraphType() ==
 * _chart.GraphType.SCATTER_PLOT)) ))                     showLegend = true;
 */
		// Replace x values by the middle of X intervals if needed
		// and idem for Y
		setIntervals();

		// if a count or a sum is to be made on Y values, authorize unknown values (if not, the count would be false)
		// else, and if unknown values are not authorized, set zeros where unknown values.
		if (_chart.getYVal() != LyChart.YVal.Y_REAL)
			countSumY();
		else if (!unknownValues)
			setZeros();

		// invert Coordinates for one Pie by Y variable,
		// and for percentiles when there is only one Y variable, one X variable and sum Y values, and graph type is not pie
		if (_chart.isInvertData())
/*|| ((_chart.getGraphType() != LyChart.GraphType.PIE)
 *                                              && (_chart.isRatios())                                             &&
 * (_chart.getColumnCount() == 1)                                             && !_chart.isNoXField()
 *                          && (LyChart.YVal.Y_SUM_REAL == _chart.getYVal())))*/
			_chart = invertChart(_chart);

		LyColumn	   col;
		Vector<Object> row;
		int			   i, j, nColumns = _chart.getColumnCount();

		// (X,Y) attributes
		for (j = 0; j < nColumns; j++)
		{
			// Each (X,Y) attribute
			col = _chart.getColumn(j);

			// Sort X column if necessary
			row = new Vector<Object>();
			col = sortX(col, _chart.getXType());

			for (i = 0; i < col.getChartValueCount(); i++)
				row.addElement(getChartableValue(col.getChartValue(i).getXValue(), _chart.getXType(), xSerie));

			_data.addElement(row);

			row = new Vector<Object>();

			// Create a list of values
			for (i = 0; i < col.getChartValueCount(); i++)
				row.addElement(getChartableValue(col.getChartValue(i).getYValue(), _chart.getYType(), ySerie));

			// add Y serie
			_data.addElement(row);

			// add Y serie label
			_seriesLabel.addElement(col.getLabel());
		}

		// add the X graduation LABELS
		_points = buildXLabels(xSerie, _chart.getXVal(), _chart.getXType(), _chart.getXDateType());

		// Convert data to Percentiles when needed
		if (_chart.isRatios())
			makePercentiles();
	} // end method createChartableData

	/**
	 * Build the labels corresponding to X.
	 *
	 * @param   xSerie     Vector of the real values of the series.
	 * @param   xVal       type of study for X : real values or intervals
	 * @param   xType      type X values : text, numeric or date
	 * @param   xDateType  type X dates (if needed): date, time, etc.
	 *
	 * @return  the X graduation labels
	 */
	public String[] buildXLabels(Vector<String> xSerie, LyChart.XVal xVal, LyChart.Type xType, String xDateType)
	{
		return buildXLabels(xSerie, xVal, xType, xDateType, true);
	}

	/**
	 * Build the labels corresponding to X.
	 *
	 * @param   xSerie            Vector of the real values of the series.
	 * @param   xVal              type of study for X : real values or intervals
	 * @param   xType             type X values : text, numeric or date
	 * @param   xDateType         type X dates (if needed): date, time, etc.
	 * @param   addBlankForLabel  indicates if blank must be added before ending line.
	 *
	 * @return  the X graduation labels
	 */
	public String[] buildXLabels(Vector<String> xSerie, LyChart.XVal xVal, LyChart.Type xType, String xDateType,
		boolean addBlankForLabel)
	{
		String[] result = new String[xSerie.size()];

		// in case of intervals (and if the type of x variable has not been put to TEXT by invertChart method) :
		// construct an expression like "a to b", or :
		// for dates : if the intervals borders are from first days of months
		// and if their width are one month, write names of months
		// if the intervals borders are from first days of years
		// and if their width are one year, write number of years
		if ((xVal == LyChart.XVal.X_INTERVAL)
				&& (xType != LyChart.Type.CHARTTEXT))
		{
			Date borderDate = new Date();
			// set the calendar to the border of intervals
			if (xType == LyChart.Type.CHARTDATE)
			{
				double begin = _chart.getIntervals().getXBegin();
				// construct the calendar if it has not been constructed yet
				if (_calendar == null)
					_calendar = _chart.getEnvironment().calendarFactory();

				borderDate = new Date(new Double(begin).longValue());
				_calendar.setTime(borderDate);
			}

			// first case : name of months, number of years
			if ((xType == LyChart.Type.CHARTDATE)
					&& (_chart.getIntervals().getXTimeUnit() == LyIntervals.TimeUnit.MONTHS)
					&& (_chart.getIntervals().getXWidth() == 1)
					&& (_calendar.get(Calendar.DAY_OF_MONTH) == 1))
			{
				String dateFormat = _chart.getEnvironment().getStringDateFormat(LyTools.intFromString(xDateType));

				for (int i = 0; i < xSerie.size(); i++)
				{
					Date myDate = LyDateFormatter.parse(xSerie.elementAt(i), dateFormat);

					_calendar.setTime(myDate);
					// retrieve the names of the months
					DateFormatSymbols dateFormatSymbols = new DateFormatSymbols(_chart.getEnvironment().getLocale());
					String[]		  months = dateFormatSymbols.getMonths();
					// label = month + year
					result[i] = months[_calendar.get(Calendar.MONTH)] + " "
						+ new Integer(_calendar.get(Calendar.YEAR)).toString();
				}

				_calendar.setTime(borderDate); // restablish value not to go in the second case
			}

			// second case : number of years
			else if ((xType == LyChart.Type.CHARTDATE)
					&& (_chart.getIntervals().getXTimeUnit() == LyIntervals.TimeUnit.YEARS)
					&& (_chart.getIntervals().getXWidth() == 1)
					&& (_calendar.get(Calendar.DAY_OF_YEAR) == 1))
			{
				String dateFormat = _chart.getEnvironment().getStringDateFormat(LyTools.intFromString(xDateType));

				for (int i = 0; i < xSerie.size(); i++)
				{
					Date myDate = LyDateFormatter.parse(xSerie.elementAt(i), dateFormat);

					_calendar.setTime(myDate);
					// label = year
					result[i] = new Integer(_calendar.get(Calendar.YEAR)).toString();
				}
			}
			// other cases : "a to b"
			else
			{
				LyEnvironment environment = _chart.getEnvironment();
				String		  labelTo = environment.translate("LY_CHART_TO_INTERVALS");
				int			  i, n = xSerie.size();
				String		  dateFormat = environment.getStringDateFormat(_defaultDateType);
				Date		  date = null;
				String		  xLabel, newLabel;

				for (i = 0; i < (n - 1); i++)
				{
					xLabel = xSerie.elementAt(i);

					try
					{
						date = LyDateFormatter.parse(xLabel, dateFormat);
					}
					catch (RuntimeException e)
					{
						date = LyDateFormatter.parse(xLabel,
								environment.getStringDateFormat(LyTools.shortFromString(xDateType)));
					}

					newLabel = LyDateFormatter.format(date,
							environment.getStringDateFormat(LyTools.shortFromString(xDateType)));

					result[i] = getMultiLineBegin() + newLabel + getMultiLineSep() + labelTo + " "
						+ getNextPoint(xSerie.elementAt(i), xType, null, _chart.getIntervals().getXWidth(),
							_chart.getIntervals().getXTimeUnit(), _gridX, xDateType) + getMultiLineEnd();
				}

				xLabel = (n > 1 ? xSerie.elementAt(n - 1) : null);

				try
				{
					date = LyDateFormatter.parse(xLabel, dateFormat);
				}
				catch (RuntimeException e)
				{
					date = LyDateFormatter.parse(xLabel,
							environment.getStringDateFormat(LyTools.shortFromString(xDateType)));
				}

				if (date != null)
				{
					newLabel = LyDateFormatter.format(date,
							environment.getStringDateFormat(LyTools.shortFromString(xDateType)));
	
					result[n - 1] = getMultiLineBegin() + newLabel
						+ getMultiLineSep() + labelTo + " " + _lastBoundInterval
						+ getMultiLineEnd();
				}
			}
		}
		else
		{
			for (int i = 0; i < xSerie.size(); i++)
				result[i] = xSerie.elementAt(i);
		}

		return result;
	} // end method buildXLabels

	/**
	 * To give a correct value to the JCChart object. If the type of value is number, no change. If
	 * the type of value is text or date, give a number corresponding to it.
	 *
	 * @return  the real values of the series;
	 *
	 * @param   value  index of the current value in the series (numeric);
	 * @param   type   Graphic Type of LyChart;
	 * @param   serie  Vector of the real values of the series.
	 */
	protected Object getChartableValue(String value, LyChart.Type type, Vector<String> serie)
	{
		if (type == LyChart.Type.CHARTNUMBER)
		{
			// each element of serie is different
			if ((serie != null) && !serie.contains(value))
				serie.addElement(value);

			return value;
		}
		else if (type == LyChart.Type.CHARTTEXT)
		{
			// each element of series is different
			if (serie != null)
			{
				if (!serie.contains(value))
					serie.addElement(value);

				return new Integer(serie.indexOf(value)).toString();
			}
			else
				return null;
		}
		else if (type == LyChart.Type.CHARTDATE)
		{
			// give a numeric value representing the date, and fill Vector series
			LyEnvironment environment = _chart.getEnvironment();
			String		  format = environment.getStringDateFormat(_defaultDateType);
			DateFormat    dateFormat = new SimpleDateFormat(format);
			Date		  myDate = parseDate(value, dateFormat);

			if (null == myDate)
				return null;

			long time = myDate.getTime();
			// the label is from the type of date given by the LyChart object
			String dateType = null;

			if (type == LyChart.Type.CHARTDATE)
				dateType = _chart.getXDateType();
			else
				dateType = _chart.getYDateType();

			value = makeDate(value, dateType);
			if ((serie != null) && !serie.contains(value))
				serie.addElement(value);

			return Long.toString(time);
		}
		else
			return null;
	} // end method getChartableValue

	/**
	 * To sort the X vector of values, and the Y values as them.
	 *
	 * @param   initial  : the values before the sort
	 * @param   type     : type of x values : number, text or date
	 *
	 * @return  the sorted column in ascendant order
	 */
	protected LyColumn sortX(LyColumn initial, LyChart.Type type)
	{
		if ((type != LyChart.Type.CHARTNUMBER)
				&& (type != LyChart.Type.CHARTDATE))
			return initial;

		LyColumn	  finalX = initial;
		boolean		  compare = false;
		LyEnvironment environment = _chart.getEnvironment();
		String		  format = environment.getStringDateFormat(_defaultDateType);
		DateFormat    dateFormat = new SimpleDateFormat(format);
		int			  i, j, n = initial.getChartValueCount();
		String		  xi, xj, yi, yj;
		Object[]	  xvalues = new Object[n];
		Object		  xvalue;

		// Parse values
		for (i = 0; i < n; i++)
		{
			xi = finalX.getChartValue(i).getXValue();
			if (type == LyChart.Type.CHARTDATE)
				xvalues[i] = parseDate(xi, dateFormat);
			else
				xvalues[i] = LyTools.DoubleFromString(xi);
		}

		// Sort the datas if numeric or date
		for (i = 0; i < (n - 1); i++)
		{
			xi = finalX.getChartValue(i).getXValue();
			if (xi == null)
				continue;

			for (j = i + 1; j < n; j++)
			{
				xj = finalX.getChartValue(j).getXValue();
				if (xj == null)
					continue;

				if (type == LyChart.Type.CHARTDATE)
					compare = ((Date)xvalues[i]).after((Date)xvalues[j]);
				else
					compare = (((Double)xvalues[i]).compareTo((Double)xvalues[j]) > 0);

				if (compare)
				{
					// exchange X values
					finalX.getChartValue(i).setXValue(xj);
					finalX.getChartValue(j).setXValue(xi);
					xi = xj;

					xvalue = xvalues[i];
					xvalues[i] = xvalues[j];
					xvalues[j] = xvalue;

					// exchange Y values
					yi = finalX.getChartValue(i).getYValue();
					yj = finalX.getChartValue(j).getYValue();
					finalX.getChartValue(i).setYValue(yj);
					finalX.getChartValue(j).setYValue(yi);
				}
			}
		}

		return finalX;
	} // end method sortX

	/**
	 * Invert coordinates, to construct one pie by Y variable.
	 *
	 * @param   chart  the initial chart
	 *
	 * @return  the chart inverting X and Y
	 */
	protected LyChart invertChart(LyChart chart)
	{
		LyChart		 oldChart = new LyChart();
		LyColumn     col = null;
		LyChartValue chartValue = null;
		int			 i, nc = chart.getColumnCount();
		LyColumn 	 sortedCol;

		// fill oldChart columns with chart columns, and clear chart columns
		for (i = 0; i < nc; i++)
		{
			sortedCol = sortX(chart.getColumn(i), _chart.getXType());
			oldChart.addColumn(sortedCol);
		}

		for (i = nc - 1; i >= 0; i--)
			chart.removeColumn(chart.getColumn(i));

		// fill the new chart columns
		// we suppose that all the columns have the same size (see countSumY() method)
		int		j, nv = (nc > 0 ? oldChart.getColumn(0).getChartValueCount() : 0);
		String 	formatedDate;
		
		for (i = 0; i < nv; i++)
		{
			col = new LyColumn();

			for (j = 0; j < nc; j++)
			{
				chartValue = new LyChartValue(oldChart.getColumn(j).getChartValue(i).getYValue());
				chartValue.setXValue(oldChart.getColumn(j).getLabel());
				col.addChartValue(chartValue);
			}
			
			// Create Y columns labels.
			// They may be modified at the end of this method if original X is by intervals
			if (LyChart.Type.CHARTDATE != _chart.getXType())
				col.setLabel(oldChart.getColumn(0).getChartValue(i).getXValue());
			else
			{
				formatedDate = makeDate(oldChart.getColumn(0).getChartValue(i).getXValue(),
						_chart.getXDateType());
				col.setLabel(formatedDate);
			}

			chart.addColumn(col);
		}

		// If the original X is by intervals, the new Y variables labels have to be intervals
		if (_chart.getXVal() == LyChart.XVal.X_INTERVAL)
		{
			// make a Vector containing the Y columns labels we have just created
			Vector<String> simpleLabels = new Vector<String>();

			for (i = 0; i < chart.getColumnCount(); i++)
				simpleLabels.addElement(chart.getColumn(i).getLabel());

			// build the new interval labels
			String[] intervalLabels = buildXLabels(simpleLabels, _chart.getXVal(), _chart.getXType(),
					_chart.getXDateType());
			// affect them to the columns
			for (i = 0; i < intervalLabels.length; i++)
				chart.getColumn(i).setLabel(intervalLabels[i]);
		}

		// As we have put the Y original columns labels in the X values, the type of X must be TEXT
		_chart.setXType(LyChart.Type.CHARTTEXT);

		oldChart = null;
		return chart;
	} // end method invertChart

	/**
	 * Parses a dateString for the given dateFormat and raise a Warning message as needed.
	 *
	 * @param   dateString  the date String we want to parse
	 * @param   dateFormat  The dateFormat instance used for parsing
	 *
	 * @return  the date if the parsing succeeded, false otherwise
	 */
	protected Date parseDate(String dateString, DateFormat dateFormat)
	{
		// WARNING : Should be kept consistent with method in LyDateFieldInfo
		if ((dateString == null) || (dateFormat == null))
			return null;

		try
		{
			return (dateFormat.parse(dateString));
		}
		catch (ParseException e)
		{
			// Raise a Warning message
			LyEnvironment environment = _chart.getEnvironment();

			environment.logWarning("Invalid date format in LyChartConverter "
				+ " (display format) :" + dateString + " : " + e.getMessage());
		}

		return null;
	}

	/**
	 * Makes a date label with the right type.
	 *
	 * @param   value     The complete expression of the date with the type Type.DATE_TIME
	 * @param   dateType  The type of the date to return
	 *
	 * @return  The date with the type dateType.
	 *
	 * @see     LyDate
	 */
	protected String makeDate(String value, String dateType)
	{
		LyEnvironment environment = _chart.getEnvironment();
		String		  format = environment.getStringDateFormat(_defaultDateType);
		DateFormat    dateFormat = new SimpleDateFormat(format);
		Date		  myDate = parseDate(value, dateFormat);
		
		String		  result = LyDateFormatter.format(myDate,
				environment.getStringDateFormat(LyTools.shortFromString(dateType)));

		return result;
	}

	/**
	 * Replace all unknown values by zeros. JClass Chart uses only array types of source for BAR,
	 * PIE so X series must be the same for all Y series.
	 */
	protected void setZeros()
	{
		Vector<String> allValues = new Vector<String>();
		LyColumn	   Col = null;
		boolean		   isPresent = false;

		// first fill vector allValues with all X values
		allValues = allXValues();

		// then add zeros when unknown values
		for (int j = 0; j < _chart.getColumnCount(); j++)
		{
			Col = _chart.getColumn(j);
			if (Col != null)
			{
				for (int k = 0; k < allValues.size(); k++)
				{
					isPresent = false;
					for (int i = 0; i < Col.getChartValueCount(); i++)
					{
						if ((null != allValues.elementAt(k)) && (null != Col.getChartValue(i))
								&& (null != Col.getChartValue(i).getXValue())
								&& (allValues.elementAt(k).compareTo(Col.getChartValue(i).getXValue()) == 0))
							isPresent = true;
					}

					if (!isPresent)
					{
						LyChartValue chartValue = new LyChartValue("0");

						chartValue.setXValue(allValues.elementAt(k));
						Col.addChartValue(chartValue);
					}
				}
			}
		}
	} // end method setZeros

	/**
	 * Fill a Vector with all X values.
	 *
	 * @return  A Vector with all X values.
	 */
	protected Vector<String> allXValues()
	{
		Vector<String> allValues = new Vector<String>();
		LyColumn	   col = null;
		int			   i, j, n = _chart.getColumnCount(), n2;
		String		   xValue;

		for (i = 0; i < n; i++)
		{
			col = _chart.getColumn(i);

			if (col != null)
			{
				n2 = col.getChartValueCount();

				for (j = 0; j < n2; j++)
				{
					xValue = col.getChartValue(j).getXValue();
					if ((xValue != null) && !(allValues.contains(xValue)))
						allValues.addElement(xValue);
				}
			}
		}

		return allValues;
	}

	/**
	 * Fill a Vector with all Y values.
	 *
	 * @return  A Vector with all Y values.
	 */
	protected Vector<String> allYValues()
	{
		Vector<String> allValues = new Vector<String>();
		LyColumn	   col = null;
		int			   c, nc, v, nv, i, iMin, iMax;
		String		   yValue, cursor;
		double		   value, cursorValue;
		LyEnvironment  environment = _chart.getEnvironment();
		String		   format = environment.getStringDateFormat(_defaultDateType);
		DateFormat     dateFormat = new SimpleDateFormat(format);
		LyChart.Type   type = _chart.getYType();

		for (c = 0, nc = _chart.getColumnCount(); c < nc; c++)
		{
			col = _chart.getColumn(c);
			if (col == null)
				continue;
			
			for (v = 0, nv = col.getChartValueCount(); v < nv; v++)
			{
				yValue = col.getChartValue(v).getYValue();
				
				// Sort the datas if numeric or date
				if ((type == LyChart.Type.CHARTNUMBER)
					|| (type == LyChart.Type.CHARTDATE))
				{
					if (type == LyChart.Type.CHARTDATE)
						value = parseDate(yValue, dateFormat).getTime();
					else
						value = LyTools.doubleFromString(yValue);
					
					// Add the object at the right position
					iMin = 0;
					iMax = allValues.size() - 1;
	
					while ((iMin>= 0)
						&& (iMax >= iMin))
					{
						i = (iMin + iMax) / 2;
						cursor = allValues.elementAt(i);
	
						if (type == LyChart.Type.CHARTDATE)
							cursorValue = parseDate(cursor, dateFormat).getTime();
						else
							cursorValue = LyTools.doubleFromString(cursor);
						
						if (value == cursorValue)
							iMin = -1 - i;
						else if (value > cursorValue)
							iMin = i + 1;
						else
							iMax = i - 1;
					}
					
					if (iMin >= 0)
						allValues.insertElementAt(yValue, iMin);
				}
				else if (!allValues.contains(yValue))
					allValues.addElement(yValue);
			}
		}
		
		return allValues;
	}

	/**
	 * If needed : replace all x values by the beginning of x intervals, replace all y values by the
	 * middle of y intervals.
	 */
	public void setIntervals()
	{
		LyColumn	  Col = null;
		Long		  oldDateValue = null, newDateValue = null;
		Double		  oldNumValue = null, newNumValue = null;
		LyEnvironment environment = _chart.getEnvironment();
		String		  format = environment.getStringDateFormat(_defaultDateType);
		DateFormat    dateFormat = new SimpleDateFormat(format);
		double		  lastValue = 0; // the max of the last X interval (if needed)

		String xDateType = _chart.getXDateType();

		if ("DATE_TIME".equals(xDateType))
			xDateType = "0";
		else if ("DATE_HOUR_MIN".equals(xDateType))
			xDateType = "1";
		else if ("DATE".equals(xDateType))
			xDateType = "2";
		else if ("TIME".equals(xDateType))
			xDateType = "3";
		else if ("HOUR_MIN".equals(xDateType))
			xDateType = "4";

		_chart.setXDateType(xDateType);

		if (_chart.getIntervals() != null)
		{

			// if X represents dates : make a grid of values for the intervals.
			if ((_chart.getXVal() == LyChart.XVal.X_INTERVAL)
					&& (_chart.getXType() == LyChart.Type.CHARTDATE))
			{
				Vector<String> allValues = allXValues();

				_gridX = makeDateGrid(allValues, _chart.getIntervals().getXBegin(), _chart.getIntervals().getXWidth(),
						_chart.getIntervals().getXTimeUnit(), true);
			}

			// if Y represents dates : make a grid of values for the intervals.
			if ((_chart.getYVal() == LyChart.YVal.Y_COUNT_INTERVAL)
					&& (_chart.getYType() == LyChart.Type.CHARTDATE))
			{
				Vector<String> allValues = allYValues();

				_gridY = makeDateGrid(allValues, _chart.getIntervals().getYBegin(), _chart.getIntervals().getYWidth(),
						_chart.getIntervals().getYTimeUnit(), false);
			}
			
			for (int j = 0; j < _chart.getColumnCount(); j++)
			{
				Col = _chart.getColumn(j);
				if (Col != null)
				{
					for (int i = 0; i < Col.getChartValueCount(); i++)
					{
						// intervals for X
						if ((_chart.getXVal() == LyChart.XVal.X_INTERVAL)
								&& (_chart.getIntervals().getXWidth() != 0)
								&& (_chart.getXType() != LyChart.Type.CHARTTEXT))
						{
							if (_chart.getXType() == LyChart.Type.CHARTDATE)
							{
								// search the right interval in the grid
								Date myDate = parseDate(Col.getChartValue(i).getXValue(), dateFormat);

								if (null != myDate) // can appear when absent values
								{
									oldDateValue = new Long(myDate.getTime());
									int k = 1;
									// because the milliseconds are aleatory, we should compare dates with an error of 999 milliseconds
									while ((k < (_gridX.size() - 1))
											&& ((oldDateValue.longValue() + 999)
												>= _gridX.elementAt(k).longValue()))
										k++;

									newDateValue = _gridX.elementAt(k - 1);
									Col.getChartValue(i).setXValue(dateFormat.format(
											new Date(newDateValue.longValue())));

									_lastBoundInterval = LyDateFormatter.format(new Date(
												_gridX.lastElement().longValue()),
											environment.getStringDateFormat(
												LyTools.shortFromString(_chart.getXDateType())));
								}
							}
							else if (_chart.getXType() == LyChart.Type.CHARTNUMBER)
							{
								oldNumValue = LyTools.DoubleFromString(Col.getChartValue(i).getYValue());

								// the begining of the interval concerned
								if (null != oldNumValue) // can appear in case of COUNT and Y type is choice
								{
									newNumValue = new Double(_chart.getIntervals().getXBegin()
											+ (_chart.getIntervals().getXWidth()
													* Math.floor(
														(oldNumValue.doubleValue()
															- _chart.getIntervals().getXBegin())
														/ _chart.getIntervals().getXWidth())));
									Col.getChartValue(i).setXValue(newNumValue.toString());

									if (lastValue
											< (newNumValue.doubleValue()
												+ _chart.getIntervals().getXWidth()))
										lastValue = newNumValue.doubleValue()
											+ _chart.getIntervals().getXWidth();

									_lastBoundInterval = getChartableValue(new Double(lastValue).toString(),
											_chart.getXType(), null);
								}
								else
								{
									newNumValue = null;
									Col.getChartValue(i).setXValue(null);
								}
							}
						}

						// intervals for Y
						if ((_chart.getYVal() == LyChart.YVal.Y_COUNT_INTERVAL)
								&& (_chart.getIntervals().getYWidth() != 0)
								&& (_chart.getYType() != LyChart.Type.CHARTTEXT))
						{
							if (_chart.getYType() == LyChart.Type.CHARTDATE)
							{
								// search the right interval in the grid
								Date myDate = parseDate(Col.getChartValue(i).getYValue(), dateFormat);

								if (null != myDate) // can appear when absent values
								{
									oldDateValue = new Long(myDate.getTime());
									int k = 1;
									// because the milliseconds are aleatory, we should compare dates with an error of 999 milliseconds
									while ((k < _gridY.size())
											&& ((oldDateValue.longValue() + 999) >= _gridY.elementAt(k)))
										k++;

									newDateValue = _gridY.elementAt(k - 1);
									Col.getChartValue(i).setYValue(dateFormat.format(
											new Date(newDateValue.longValue())));
								}
							}
							else if (_chart.getYType() == LyChart.Type.CHARTNUMBER)
							{
								oldNumValue = LyTools.DoubleFromString(Col.getChartValue(i).getYValue());

								// the middle of the interval concerned
								newNumValue = new Double(_chart.getIntervals().getYBegin()
										+ (_chart.getIntervals().getYWidth()
												* Math.floor(
													(oldNumValue.doubleValue()
														- _chart.getIntervals().getYBegin())
													/ _chart.getIntervals().getYWidth())));

								Col.getChartValue(i).setYValue(newNumValue.toString());
							}
						}
					}
				}
			}
		}
	} // end method setIntervals

	/**
	 * If intervals are required and if the variable represents dates : make a grid of values (in
	 * milliseconds) for the intervals.
	 *
	 * @param   allValues  the vector of values used to find the min and the max
	 * @param   begin      the border of an interval
	 * @param   width      the number of time unities in an interval
	 * @param   timeUnit   the unit of time for the width of intervals
	 * @param   isX        is true if the treatment concerns X, and false if it concerns Y
	 *
	 * @return  The truncated date with the type dateType
	 */
	protected Vector<Long> makeDateGrid(Vector<String> allValues, double begin, float width,
		LyIntervals.TimeUnit timeUnit, boolean isX)
	{
		LyEnvironment environment = _chart.getEnvironment();
		String		  format = environment.getStringDateFormat(_defaultDateType);
		DateFormat    dateFormat = new SimpleDateFormat(format);
		Vector<Long>  grid = new Vector<Long>();

		// construct the calendar if it has not been constructed yet
		if (_calendar == null)
			_calendar = environment.calendarFactory();

		if ((allValues != null) && (allValues.size() > 0))
		{
			// First search the min and the max dates in the vector allValues
			Date dateMin = parseDate(allValues.elementAt(0), dateFormat);
			Date dateMax = parseDate(allValues.elementAt(0), dateFormat);
			Date dateValue;

			for (int i = 1; i < allValues.size(); i++)
			{
				try
				{
					dateValue = parseDate(allValues.elementAt(i), dateFormat);
					if (dateValue.before(dateMin))
						dateMin = dateValue;

					if (dateValue.after(dateMax))
						dateMax = dateValue;
				}
				catch (Exception e) // can arrive when absent values
				{
				}
			}

			// recover the time unit for the increments in the calendar
			int field;

			if (isX)
			{
				switch (timeUnit)
				{

				case YEARS:
					field = Calendar.YEAR;
					break;

				case MONTHS:
					field = Calendar.MONTH;
					break;

				case DAYS:
					field = Calendar.DATE;
					break;

				case HOURS:
					field = Calendar.HOUR;
					break;

				case MINS:
					field = Calendar.MINUTE;
					break;

				case SECS:
					field = Calendar.SECOND;
					break;

				default:
					field = Calendar.YEAR;
				}
			}
			else
			{
				switch (timeUnit)
				{

				case YEARS:
					field = Calendar.YEAR;
					break;

				case MONTHS:
					field = Calendar.MONTH;
					break;

				case DAYS:
					field = Calendar.DATE;
					break;

				case HOURS:
					field = Calendar.HOUR;
					break;

				case MINS:
					field = Calendar.MINUTE;
					break;

				case SECS:
					field = Calendar.SECOND;
					break;

				default:
					field = Calendar.YEAR;
				}
			}

			// then build the grid of intervals

			// add values after begin in the grid
			_calendar.setTime(new Date(new Double(begin).longValue()));
			// because the milliseconds are aleatory, we should compare dates with an error of 999 milliseconds
			while ((_calendar.getTime().getTime() - 999) <= (dateMax.getTime()))
			{
				grid.addElement(new Long(_calendar.getTime().getTime()));
				_calendar.add(field, new Float(width).intValue());
			}

			grid.addElement(new Long(_calendar.getTime().getTime()));

			// insert values before begin at the beginning of the grid
			// then the grid is allready sorted
			_calendar.setTime(new Date(new Double(begin).longValue()));
			_calendar.add(field, new Float(-width).intValue());
			// because the milliseconds are aleatory, we should compare dates with an error of 999 milliseconds
			while ((_calendar.getTime().getTime() + 999) > (dateMin.getTime()))
			{
				grid.insertElementAt(new Long(_calendar.getTime().getTime()), 0);
				_calendar.add(field, new Float(-width).intValue());
			}

			grid.insertElementAt(new Long(_calendar.getTime().getTime()), 0);
		}

		// shouldn't be useful
		if (grid.size() == 0)
			grid.addElement(new Long(0));

		return grid;
	} // end method makeDateGrid

	/**
	 * If intervals are required : retrieves the max value of an interval, given the min value.
	 *
	 * @param   values    the min value of the interval concerned
	 * @param   type      the type of values treated (number, date)
	 * @param   width     the number of time unities in an interval
	 * @param   timeUnit  the unit of time for the width of intervals (exists only for date values)
	 * @param   grid      the grid of intervals (created only for date values)
	 * @param   dateType  the format of the dates (exists only for date values)
	 *
	 * @return  The max value of the interval concerned
	 */
	protected String getNextPoint(String value, LyChart.Type xtype, LyChart.Type ytype, float width,
		LyIntervals.TimeUnit timeUnit, Vector<Long> grid, String dateType)
	{
		LyEnvironment environment = _chart.getEnvironment();

		if ((xtype == LyChart.Type.CHARTNUMBER)
				|| (ytype == LyChart.Type.CHARTNUMBER))
		{
			Double maxValue = new Double(width + LyTools.doubleFromString(value));

			return maxValue.toString();
		}
		else if ((xtype == LyChart.Type.CHARTDATE)
				|| (ytype == LyChart.Type.CHARTDATE))
		{
			String dateFormat = environment.getStringDateFormat(_defaultDateType);
			Date   minDate = null;

			try
			{
				minDate = LyDateFormatter.parse(value, dateFormat);
			}
			catch (RuntimeException e) // arrive when the format is not the default format (for x intervals by exemple)
			{
				minDate = LyDateFormatter.parse(value,
						environment.getStringDateFormat(LyTools.shortFromString(dateType)));
			}

			if (null == minDate)
				return null;

			long minTime = minDate.getTime();
			long maxTime = 0;
			int  i = 0;

			while ((i < (grid.size() - 1))
					&& (minTime >= grid.elementAt(i).longValue()))
			{
				maxTime = grid.elementAt(i + 1).longValue();
				i++;
			}

			Date   maxDate = new Date(maxTime);
			String result = LyDateFormatter.format(maxDate,
					environment.getStringDateFormat(LyTools.shortFromString(dateType)));

			return result;
		}
		else
			return null;
	} // end method getNextPoint

	/**
	 * If needed : Replace Y values by their count, for each X value. Keep only different X values.
	 * If Y is counted, Y type of values become NUMBER
	 */
	protected void countSumY()
	{
		if (_chart.isYValuesComputed())
			return;

		_chart.setYValuesComputed(true);

		LyEnvironment  environment = _chart.getEnvironment();
		LyColumn	   oldCol = null;
		LyColumn	   newCol = null;
		Vector<String> allValues = new Vector<String>();
		int			   k = 0;
		Double		   oldCount = new Double(0);
		Double		   newCount = new Double(0);
		Vector<String> allYValues = new Vector<String>();
		int			   n = 0;
		int			   nbCol;

		// nbCol = initial number of Y variables
		nbCol = _chart.getColumnCount();

		// fill vector allValues with all X values
		allValues = allXValues();

		// if count is demanded, fill vector allYValues with all Y values
		if ((_chart.getYVal() == LyChart.YVal.Y_COUNT_REAL)
				|| (_chart.getYVal() == LyChart.YVal.Y_COUNT_INTERVAL))
		{
			allYValues = allYValues();
			n = allYValues.size();
		}
		else
			n = 1;

		int    i, nv;
		int    j, nc = _chart.getColumnCount();
		String str;

		for (j = 0; j < nc; j++)
		{
			oldCol = _chart.getColumn(j);

			if (oldCol != null)
			{
				for (int m = 0; m < n; m++)
				{
					// fill newCol with all X Values, and zeros for Y Values
					newCol = new LyColumn();
					for (i = 0, nv = allValues.size(); i < nv; i++)
					{
						// particular case when COUNT, y is a choice and a choice option doesn't appear in the data (see LyGraphicalChartBuilder, fillChartColumns)
						str = allValues.elementAt(i);
						if (str != null)
						{
							LyChartValue myChartV = new LyChartValue("0");

							myChartV.setXValue(str);
							newCol.addChartValue(myChartV);
						}
					}

					// set label : the same if no count is demanded (m = 0),
					// else Y value if there is only one Y variable,
					// else "Y variable label = Y value".
					// where Y value can be an interval ("... to ...")
					if (allYValues.size() == 0)
						newCol.setLabel(oldCol.getLabel());
					else
					{
						String myLabel = allYValues.elementAt(m);

						// construct the string "... to ..." , or :
						// for dates : if the intervals borders are from first days of months
						// and if their width are one month, write names of months
						// if the intervals borders are from first days of years
						// and if their width are one year, write number of years

						if (_chart.getYVal() == LyChart.YVal.Y_COUNT_INTERVAL)
						{
							String labelTo = environment.translate("LY_CHART_TO_INTERVALS");
							String formatedDate = null;
							Date   borderDate;

							// set the calendar to the border of intervals
							if (_chart.getYType() == LyChart.Type.CHARTDATE)
							{
								double begin = _chart.getIntervals().getYBegin();
								// construct the calendar if it has not been constructed yet
								if (_calendar == null)
									_calendar = environment.calendarFactory();

								borderDate = new Date(new Double(begin).longValue());
								_calendar.setTime(borderDate);

								// the date to write on the graphic
								formatedDate = makeDate(myLabel, _chart.getYDateType());
							}

							// first case : name of months
							if ((_chart.getYType() == LyChart.Type.CHARTDATE)
									&& (_chart.getIntervals().getYTimeUnit()
										== LyIntervals.TimeUnit.MONTHS)
									&& (_chart.getIntervals().getYWidth() == 1)
									&& (_calendar.get(Calendar.DAY_OF_MONTH) == 1))
							{
								String dateFormat = environment.getStringDateFormat(LyTools.intFromString(
											_chart.getYDateType()));
								Date   myDate = LyDateFormatter.parse(formatedDate, dateFormat);

								_calendar.setTime(myDate);
								// retrieve the names of the months
								DateFormatSymbols dateFormatSymbols = new DateFormatSymbols(environment.getLocale());
								String[]		  months = dateFormatSymbols.getMonths();
								// label = month + year
								myLabel = months[_calendar.get(Calendar.MONTH)] + " "
									+ new Integer(_calendar.get(Calendar.YEAR)).toString();
								borderDate = new Date();
								_calendar.setTime(borderDate); // restablish value not to go in the second case
							}
							// second case : number of years
							else if ((_chart.getYType() == LyChart.Type.CHARTDATE)
									&& (_chart.getIntervals().getYTimeUnit()
										== LyIntervals.TimeUnit.YEARS)
									&& (_chart.getIntervals().getYWidth() == 1)
									&& (_calendar.get(Calendar.DAY_OF_YEAR) == 1))
							{
								String dateFormat = environment.getStringDateFormat(LyTools.intFromString(
											_chart.getYDateType()));
								Date   myDate = LyDateFormatter.parse(formatedDate, dateFormat);

								_calendar.setTime(myDate);
								// label =  year
								myLabel = new Integer(_calendar.get(Calendar.YEAR)).toString();
							}
							// other cases : "a to b"
							else if (_chart.getYType() == LyChart.Type.CHARTDATE)
							{
								// type "a to b"
								if (nbCol == 1)
									myLabel = getMultiLineBegin() + formatedDate + getMultiLineSep()
										+ labelTo + " "
										+ this.getNextPoint(myLabel, null, _chart.getYType(),
											_chart.getIntervals().getYWidth(), _chart.getIntervals().getYTimeUnit(),
											_gridY, _chart.getYDateType())
										+ getMultiLineEnd();
								else // type "colName = a to b"
									myLabel = formatedDate + " " + labelTo + " "
										+ this.getNextPoint(myLabel, null, _chart.getYType(),
											_chart.getIntervals().getYWidth(), _chart.getIntervals().getYTimeUnit(),
											_gridY, _chart.getYDateType());
							}
							else
								myLabel = myLabel + " " + labelTo + " "
									+ this.getNextPoint(myLabel, null, _chart.getYType(),
										_chart.getIntervals().getYWidth(), _chart.getIntervals().getYTimeUnit(), _gridY,
										_chart.getYDateType());
						}

						if (nbCol == 1)
							newCol.setLabel(myLabel);
						else
							newCol.setLabel(oldCol.getLabel() + " = " + myLabel);
					}

					// for each X value, count or sum Y values
					LyChartValue ovi, nvk;

					for (i = 0; i < oldCol.getChartValueCount(); i++)
					{
						ovi = oldCol.getChartValue(i);
						k = allValues.indexOf(ovi.getXValue());
						nvk = newCol.getChartValue(k);
						// particular case when COUNT, y is a choice and a choice option doesn't appear in the data (see LyGraphicalChartBuilder, fillChartColumns)
						if (nvk != null)
						{
							oldCount = LyTools.DoubleFromString(nvk.getYValue());

							// case count values : increment by 1 when the value is equal to allYValues(m)
							// case sum   : add value
							newCount = oldCount;
							if (((_chart.getYVal() == LyChart.YVal.Y_COUNT_REAL)
										|| (_chart.getYVal() == LyChart.YVal.Y_COUNT_INTERVAL))
									&& ((ovi.getYValue()).compareTo(allYValues.elementAt(m)) == 0))
								newCount = new Double(oldCount.doubleValue() + 1);
							else if (_chart.getYVal() == LyChart.YVal.Y_SUM_REAL)
								newCount = new Double(oldCount.doubleValue()
										+ LyTools.doubleFromString(ovi.getYValue()));

							nvk.setYValue(newCount.toString());
						}
					}
					// update the current Column (if it is the first) or add column
					if (m == 0)
						_chart.removeColumn(oldCol);
					else
						j++; // to avoid an infinite loop

					_chart.insertColumn(newCol, j);
				}
			}
		}
		// if Y values are counted, the type of Y values become numeric
		if ((_chart.getYVal() == LyChart.YVal.Y_COUNT_REAL)
				|| (_chart.getYVal() == LyChart.YVal.Y_COUNT_INTERVAL))
			_chart.setYType(LyChart.Type.CHARTNUMBER);
	} // end method countSumY

	/**
	 * If needed : Replace Y values by their percentile value, for each X value.
	 */
	protected void makePercentiles()
	{
		// Construct a vector containing all the X Values, with only different values
		// X series are all pair vectors of _data
		Vector<Object> allXValues = new Vector<Object>();
		int			   i, j;

		if (null != _data)
		{
			for (i = 0; i < _data.size(); i = i + 2)
			{
				for (j = 0; j < (_data.elementAt(i)).size(); j++)
				{
					if (!allXValues.contains(_data.elementAt(i).elementAt(j)))
						allXValues.addElement(_data.elementAt(i).elementAt(j));
				}
			}

			// yTotals is a vector that contains Y totals, corresponding to X values in allXValues Vector
			Vector<Double> yTotals = new Vector<Double>();
			int			   k; // indices in allXValues and yTotals

			// initialize yTotals with zeros
			for (i = 0; i < allXValues.size(); i++)
				yTotals.addElement(new Double(0));

			// sum the corresponding Y values to fill yTotals Vector (at the same indice than X value),
			// Y series are all impair vectors of _data
			for (i = 1; i < _data.size(); i = i + 2)
			{
				for (j = 0; j < (_data.elementAt(i)).size(); j++)
				{
					// retrieve the index of x value in allXValues
					if (null != _data.elementAt(i - 1))
					{
						k = allXValues.indexOf((_data.elementAt(i - 1)).elementAt(j));

						try
						{
							yTotals.setElementAt(new Double(
									(yTotals.elementAt(k).doubleValue())
									+ LyTools.doubleFromString((_data.elementAt(i)).elementAt(j).toString())), k);
						}
						catch (Exception e) // Y value in _data is not numeric, shouldn't happen
						{
						}
					}
				}
			}

			if (_chart.getGraphType() != LyChart.GraphType.PIE)
			{
				double total = 0;

				for (i = 0; i < yTotals.size(); i++)
					total += yTotals.elementAt(i).doubleValue();

				for (i = 0; i < yTotals.size(); i++)
					yTotals.setElementAt(new Double(total), i);
			}

			// transform the Y values as percentiles of yTotal values, with allXValues Vector indices
			for (i = 1; i < _data.size(); i = i + 2)
			{
				for (j = 0; j < _data.elementAt(i).size(); j++)
				{
					// retrieve the index of x value in allXValues
					if (null != _data.elementAt(i - 1))
					{
						k = allXValues.indexOf(_data.elementAt(i - 1).elementAt(j));

						try
						{
							double oldValue = LyTools.doubleFromString(_data.elementAt(i).elementAt(j).toString());
							double newValue0 = oldValue
									/ (yTotals.elementAt(j).doubleValue()) * 100;
							// keep only two decimals for the percentile
							Double newValue = new Double(LyTools.checkDecimals(newValue0, 2));

							_data.elementAt(i).setElementAt(newValue, j);
						}
						catch (Exception e) // Y value in _data is not numeric, shouldn't happen
						{
						}
					}
				}
			}
		}
	} // end method makePercentiles

	/**
	 * Build the customized point labels (instead of autoLabels).
	 *
	 * @return  a Vector of String[]. Each String array correspond to a points serie.
	 */
	protected Vector<String[]> buildPointLabels()
	{
		Vector<Object>   xVector;
		Vector<Object>   yVector;
		Vector<String[]> result = new Vector<String[]>();
		String			 label;
		String			 xLabel;
		String			 yLabel;
		String[]		 labels;
		LyEnvironment    environment = _chart.getEnvironment();

		for (int i = 0; i < (_data.size() - 1); i = i + 2)
		{
			xVector = _data.elementAt(i);
			yVector = _data.elementAt(i + 1);
			labels = new String[xVector.size()];

			for (int j = 0; j < xVector.size(); j++)
			{
				// create each (x, y) label
				xLabel = null;
				yLabel = null;

				// Y LABEL PART :
				// First case : count
				if ((_chart.getYVal() == LyChart.YVal.Y_COUNT_REAL)
						|| (_chart.getYVal() == LyChart.YVal.Y_COUNT_INTERVAL))
					yLabel = yVector.elementAt(j).toString();

				// Second case : sum or real numeric values : keep the right format
				else if ((_chart.getYVal() == LyChart.YVal.Y_SUM_REAL)
						|| (_chart.getYType() == LyChart.Type.CHARTNUMBER))
				{
					if (!_chart.isRatios())
					{
						NumberFormat numberFormat = environment.getNumberFormat(_chart.getYFormat());

						yLabel = numberFormat.format(LyTools.DoubleFromString(yVector.elementAt(j).toString())
									.doubleValue());
					}
					else
						yLabel = yVector.elementAt(j).toString();
				}

				// Third case : date
				else if (_chart.getYType() == LyChart.Type.CHARTDATE)
				{
					if (null != yVector.elementAt(j))
					{
						Date date = new Date(LyTools.DoubleFromString(yVector.elementAt(j).toString()).longValue());

						yLabel = LyDateFormatter.format(date,
								environment.getStringDateFormat(LyTools.shortFromString(_chart.getYDateType())));
					}
				}

				// Add Percentile sign when needed
				if ((_chart.isRatios()) && (null != yLabel))
					yLabel = yLabel + " %";

				// X LABEL PART :
				// Only for real values, because we suppose that intervals a visible on the X axis
				// First case : real numeric values : keep the right format
				if ((_chart.getXVal() == LyChart.XVal.X_REAL)
						&& (_chart.getXType() == LyChart.Type.CHARTNUMBER))
				{
					NumberFormat numberFormat = environment.getNumberFormat(_chart.getXFormat());

					if (null != xVector.elementAt(j))
						xLabel = numberFormat.format(LyTools.DoubleFromString(xVector.elementAt(j).toString())
									.doubleValue());
					else
						xLabel = null;
				}

				// Second case : real date values
				else if ((_chart.getXVal() == LyChart.XVal.X_REAL)
						&& (_chart.getXType() == LyChart.Type.CHARTDATE))
				{
					if (null != xVector.elementAt(j))
					{
						Date date = new Date(LyTools.DoubleFromString(xVector.elementAt(j).toString()).longValue());

						xLabel = LyDateFormatter.format(date,
								environment.getStringDateFormat(LyTools.shortFromString(_chart.getXDateType())));
					}
				}

				// for Bars ot Pies, show only the Y value on the label
				if ((_chart.getGraphType() == LyChart.GraphType.BAR)
						|| (_chart.getGraphType() == LyChart.GraphType.STACK_BAR)
						|| (_chart.getGraphType() == LyChart.GraphType.PIE))
					xLabel = null;

				// group x and y labels
				if ((null != xLabel) && (null != yLabel))
					label = " " + xLabel + ", " + yLabel + " ";
				else if (null != xLabel)
					label = " " + xLabel + " ";
				else if (null != yLabel)
					label = " " + yLabel + " ";
				else
					label = "";

				labels[j] = label;
			}

			result.addElement(labels);
		}

		return result;
	} // end method buildPointLabels

	/**
	 * Save the image of the chart.
	 *
	 * @param  tmpDir        temporary directory
	 * @param  imageFile     the PNG File
	 * @param  buffer        the buffer where imagemap must be generated
	 * @param  deleteOnExit  indicates whether the file must be deleted on exiting the application
	 */
	public abstract void saveImg(String tmpDir, String imageFile, StringBuffer buffer, boolean deleteOnExit);

	/**
	 * Retrieve the beginning of the label when it has more than one line.
	 *
	 * @return  for example : null for String, and <html>... for HTML
	 */
	public abstract String getMultiLineBegin();

	/**
	 * Retrieve the end of the label when it has more than one line.
	 *
	 * @return  for example : null for String, and ... for HTML
	 */
	public abstract String getMultiLineEnd();

	/**
	 * Retrieve the line separator of the label when it has more than one line.
	 *
	 * @return  for example : \n for String, and /p> <p for HTML
	 */
	public abstract String getMultiLineSep();
} // end class LyChartConverter
