/*
 * LyChartValue.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;
import java.io.*;
import leon.misc.LyString;


/**
 * The LyChartValue class is a generated Java implementation that corresponds to the element 
 * CHART_VALUE of the descriptive grammar of the views (views.dtd).
 * <B>&lt;CHART_VALUE&gt; : Statistic Values</B>
 * A CHART_VALUE element is a (x, y) pair where y is a value (a real or a sum or a 
 * count) of a variable to analyze and x is either a value of the variable 
 * according to which y is studied (if there is one) or the name of the variable to
 * study.
 * @see leon.view.LyComponent
 * See also tag <A href="../../conf/dtd/view/CHART_VALUE.html">CHART_VALUE</A> of grammar leon_views.dtd.
 */
public class LyChartValue extends LyComponent
{
	public static final String			__VERSION = "$Id$";


	/* Attributes */

	/**
	 * A value of the variable according to which the graph is drawn, or if there is 
	 * none, the name of the variable to study.
	 */
	protected String			_xValue;

	/**
	 * A value of the variable to study corresponding to the current COLUMN element.
	 */
	protected String			_yValue;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_xValue = null;
		_yValue = null;
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyChartValue	model = (LyChartValue)component;

		_model = component;
		_xValue = model._xValue;
		_yValue = model._yValue;
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	protected LyChartValue()
	{
	}

	/**
	 * Constructor with all required attributes.
	 * @param yValue A value of the variable to study corresponding to the current COLUMN element.
	 */
	public LyChartValue(String yValue)
	{
		// Fill up required attributes with parameters
		_yValue = yValue;
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyChartValue(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_xValue = node.getString("xValue");
		_yValue = node.getString("yValue");
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyChartValue clone = new LyChartValue();

		clone.initFromModel(this);
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of xValue.
	 * @return value of xValue.
	 */
	public String getXValue()
	{
		return _xValue;
	}

	/**
	 * Gets value of yValue.
	 * @return value of yValue.
	 */
	public String getYValue()
	{
		return _yValue;
	}


	/* Modifiers */

	/**
	 * Initializes attribute xValue to the given value.
	 * @param xValue New value of xValue.
	 */
	public void setXValue(String xValue)
	{
		if ((xValue == _xValue)
			|| ((xValue != null)
				&& (_xValue != null)
				&& xValue.equals(_xValue)))
			return;

		_xValue = xValue;
	}

	/**
	 * Initializes attribute yValue to the given value.
	 * @param yValue New value of yValue.
	 * @exception IllegalArgumentException if yValue is null.
	 */
	public void setYValue(String yValue)
	{
		if (yValue == null)
			throw new IllegalArgumentException("BF AppEngine : attribute yValue is required (can't be null)");

		if ((yValue == _yValue)
			|| ((yValue != null)
				&& (_yValue != null)
				&& yValue.equals(_yValue)))
			return;

		_yValue = yValue;
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<CHART_VALUE");
		if (_xValue != null)
			pw.print(" xValue='" + LyString.getXmlString(_xValue) + "'");
		pw.print(" yValue='" + LyString.getXmlString(_yValue) + "'");
		pw.println("/>");
	}
}
