/*
 * LyChoice.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved.
 */

package leon.view;

import leon.view.beaninterface.*;

import java.util.*;


/**
 * The LyChoice class implements generic behavior for the CHOICE components as described in
 * views.dtd regardless of the requested target (win, swing, html...). It represents a choice.
 *
 * @author   Lyria
 * @version  $Id: LyChoice.java 30893 2011-05-10 14:19:45Z jlsaouli $
 * @see      LyComponent
 */
public class LyChoice extends LyChoiceBase
{
	public static final String __VERSION = "$Id: LyChoice.java 30893 2011-05-10 14:19:45Z jlsaouli $";

	/**
	 * Constructor with all required attributes.
	 */
	public LyChoice()
	{
		super();
	}

	protected LyChoice(LyViewNode node)
	{
		super(node);
	}

	public void enableOption(LyOption option, boolean enable)
	{
		enableOption(getPosition(option), enable);
	}

	public void enableOption(int i, boolean enable)
	{
		if (i < 0)
			return;
		if (i >= getOptionCount())
			return;

		LyOption option = getOption(i);

        if (option.isEnabled() != enable)
        {
    		option.setEnabled(enable);
    
    		if (_internalView instanceof LyChoiceFieldBeanInterface)
    			((LyChoiceFieldBeanInterface)_internalView).enableOption(i, enable);
        }
	}

	public void checkOption(LyOption option, boolean check)
	{
		checkOption(getPosition(option), check);
	}

	public void checkOption(int i, boolean check)
	{
		if (i < 0)
			return;
		if (i >= getOptionCount())
			return;

		LyOption option = getOption(i);

        if  (option.isChecked() != check)
        {
    		option.setChecked(check);
    
    		if (_internalView instanceof LyChoiceFieldBeanInterface)
    			((LyChoiceFieldBeanInterface)_internalView).checkOption(i, check);
        }
	}

	/**
	 * Retrieves position of the given option.
	 */
	public int getPosition(LyOption option)
	{
		return _options.indexOf(option);
	}

	/**
	 * Retrieves an option from its id.
	 */
	public LyOption getOption(String id)
	{
		return (LyOption)getComponent(id);
	}

	/**
	 * Accessor to internal list of options (protected) Should not be used directly.
	 *
	 * @return  the internal list of options
	 */
	protected ArrayList<LyOption> getInternalOptions()
	{
        if (_options == null)
        {
            _options = new ArrayList<LyOption>(1);
            _children = _options;
        }
        
		return _options;
	}
    
    /**
     * Removes given option from choice.
     * @param option Option to be removed.
     */
    public void removeOption(LyOption option)
    {
        _options.remove(option);
        option.free();
    }
} // end class LyChoice
