/*
 * LyCommands.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;

import java.util.*;
import java.io.*;
import leon.view.getinterface.*;
import leon.misc.LyString;


/**
 * The LyCommands class is a generated Java implementation that corresponds to the element 
 * COMMANDS of the descriptive grammar of the views (views.dtd).
 * <B>&lt;COMMANDS&gt; : Command Bar</B>
 * A command bar allows displaying a set of command buttons (Validate, Cancel, 
 * Close, etc.).
 * This bar is generally located at the bottom of a page or window.
 * @see leon.view.LyComponent
 * See also tag <A href="../../conf/dtd/view/COMMANDS.html">COMMANDS</A> of grammar leon_views.dtd.
 */
public class LyCommands extends LyComponent
	implements  LyGetIdInterface,
				LyGetConstraintInterface
{
	public static final String			__VERSION = "$Id$";


	/** Possible values for attribute 'border'. */


	/** Possible values for attribute 'align'. */
	public enum Align
	{
		LEFT,
		RIGHT,
		CENTER;
	}


	/* Static methods */

	/**
	 * Utility method to get align value from a string.
	 * @param token Token string
	 * @return Align value corresponding to this token
	 */
	public static final Align getAlign(String token)
	{
		if (token == null)
			return null;
		else if (token.equalsIgnoreCase("LEFT")
				|| token.equals("0"))
			return Align.LEFT;
		else if (token.equalsIgnoreCase("RIGHT")
				|| token.equals("1"))
			return Align.RIGHT;
		else if (token.equalsIgnoreCase("CENTER")
				|| token.equals("2"))
			return Align.CENTER;
		else
			return Align.CENTER;
	}


	/* Attributes */

	/**
	 * Identifier of the command bar.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyCommands			_template;

	/**
	 * Style of the border used for the command bar.
	 */
	protected LyArea.Border			_border;

	/**
	 * Indicates the alignment of the buttons in the command bar (left, right, or 
	 * centered).
	 * The default value is center.
	 */
	protected Align				_align;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;


	/* Sub components */

	/**
	 * Commands contained in the command bar.
	 */
	protected ArrayList<LyCommand>		_commands;

	/**
	 * Positioning constraint of the comman bar.
	 */
	protected LyConstraint		_constraint;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_id = null;
		_template = null;
		_border = LyArea.Border.IN;
		_align = Align.CENTER;
		_font = getStringValue("COMMANDS.font", "$ALL.font");
		_color = getStringValue("COMMANDS.color", "$ALL.color");
		_fontColor = getStringValue("COMMANDS.fontColor", "$ALL.fontColor");

		// Initialize sub components
		_commands = null;
		_constraint = null;
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyCommands	model = (LyCommands)component;

		_model = component;
		_border = model._border;
		_align = model._align;
		_font = model._font;
		_color = model._color;
		_fontColor = model._fontColor;

		if (REWRITING_MODE)
			return;

		// Copy sub-components
		LyCommand	commands;
		int			i, n;

		n = (model._commands == null ? 0 : model._commands.size());
		for (i = 0; i < n; i++)
		{
			commands = model._commands.get(i);
			if (commands != null)
				commands = (LyCommand)commands.cloneComponent();
			addChild(commands);
		}
		if (model._constraint != null)
			addChild(model._constraint.cloneComponent());
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	public LyCommands()
	{
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyCommands(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_id = node.getString("id");
		_template = (LyCommands)node.getComponent("template");

		if (_template != null)
			initFromModel(_template);

		if (node.isSpecified("border"))
			_border = LyArea.getBorder(node.getString("border"));
		if (node.isSpecified("align"))
			_align = getAlign(node.getString("align"));
		if (node.isSpecified("font"))
			_font = node.getString("font");
		if (node.isSpecified("color"))
			_color = node.getString("color");
		if (node.isSpecified("fontColor"))
			_fontColor = node.getString("fontColor");
	}

	/**
	 * End of construction : completion with child component.
	 * @param child Sub child component.
	 * @see leon.view.LyComponent#addChild
	 */
	protected void addChild(LyComponent child)
	{
		// Add sub-components
		if (child instanceof LyCommand)
			addCommand((LyCommand)child);
		else if (child == null)
			addSep();
		else if (child instanceof LyConstraint)
			setConstraint((LyConstraint)child);
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyCommands clone = new LyCommands();

		clone.initFromModel(this);
		clone._template = this;
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of id.
	 * @return value of id.
	 */
	public String getId()
	{
		return _id;
	}

	/**
	 * Gets value of template.
	 * @return value of template.
	 */
	public LyCommands getTemplate()
	{
		return _template;
	}

	/**
	 * Gets value of border.
	 * @return value of border.
	 */
	public LyArea.Border getBorder()
	{
		return _border;
	}

	/**
	 * Gets value of align.
	 * @return value of align.
	 */
	public Align getAlign()
	{
		return _align;
	}

	/**
	 * Gets value of font.
	 * @return value of font.
	 */
	public String getFont()
	{
		return _font;
	}

	/**
	 * Checks whether the font attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFont()
	{
		return (_font == getStringValue("COMMANDS.font"));
	}

	/**
	 * Gets value of color.
	 * @return value of color.
	 */
	public String getColor()
	{
		return _color;
	}

	/**
	 * Checks whether the color attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultColor()
	{
		return (_color == getStringValue("COMMANDS.color"));
	}

	/**
	 * Gets value of fontColor.
	 * @return value of fontColor.
	 */
	public String getFontColor()
	{
		return _fontColor;
	}

	/**
	 * Checks whether the fontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFontColor()
	{
		return (_fontColor == getStringValue("COMMANDS.fontColor"));
	}

	/**
	 * Get sub-component Command of index i.
	 * @param i the index of the Command to get.
	 * @return sub-component Command of index i.
	 */
	public LyCommand getCommand(int i)
	{
		if (_commands == null)
			return null;
		if ((i < 0) || (i >= _commands.size()))
			return null;
		return _commands.get(i);
	}

	/**
	 * Get sub-component Command count.
	 * @return sub-component Command count.
	 */
	public int getCommandCount()
	{
		if (_commands == null)
			return 0;
		return _commands.size();
	}

	/**
	 * Get sub-component Constraint.
	 * @return sub-component Constraint.
	 */
	public LyConstraint getConstraint()
	{
		return _constraint;
	}


	/* Modifiers */

	/**
	 * Initializes attribute id to the given value.
	 * @param id New value of id.
	 */
	public void setId(String id)
	{
		if ((id == _id)
			|| ((id != null)
				&& (_id != null)
				&& id.equals(_id)))
			return;

		_id = (id != null ? id.intern() : null);
	}

	/**
	 * Initializes attribute template to the given value.
	 * @param template New value of template.
	 */
	public void setTemplate(LyCommands template)
	{
		if (template == null)
			return;
		if (_template != null)
			throw new IllegalStateException("BF AppEngine : attribute template was already set.");

		initFromModel(template);
		_template = template;
	}

	/**
	 * Initializes attribute border to the given value.
	 * @param border New value of border.
	 * @exception IllegalArgumentException if border parameter is null.
	 */
	public void setBorder(LyArea.Border border)
	{
		if (border == _border)
			return;
		if (border == null)
			throw new IllegalArgumentException("BF AppEngine : invalid value for attribute border");

		_border = border;
	}

	/**
	 * Initializes attribute align to the given value.
	 * @param align New value of align.
	 * @exception IllegalArgumentException if align parameter is null.
	 */
	public void setAlign(Align align)
	{
		if (align == _align)
			return;
		if (align == null)
			throw new IllegalArgumentException("BF AppEngine : invalid value for attribute align");

		_align = align;
	}

	/**
	 * Initializes attribute font to the given value.
	 * @param font New value of font.
	 */
	public void setFont(String font)
	{
		if ((font == _font)
			|| ((font != null)
				&& (_font != null)
				&& font.equals(_font)))
			return;

		_font = font;
	}

	/**
	 * Initializes attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color)
	{
		if ((color == _color)
			|| ((color != null)
				&& (_color != null)
				&& color.equals(_color)))
			return;

		_color = color;
	}

	/**
	 * Initializes attribute fontColor to the given value.
	 * @param fontColor New value of fontColor.
	 */
	public void setFontColor(String fontColor)
	{
		if ((fontColor == _fontColor)
			|| ((fontColor != null)
				&& (_fontColor != null)
				&& fontColor.equals(_fontColor)))
			return;

		_fontColor = fontColor;
	}

	/**
	 * Adds a sub-component command.
	 * @param command The command to add to commands list.
	 */
	public void addCommand(LyCommand command)
	{
		insertCommand(command, -1);
	}

	/**
	 * Inserts a sub-component command at given position.
	 * @param command The command to add to commands list.
	 * @param pos	Position where the sub-component is inserted.
	 */
	public void insertCommand(LyCommand command, int pos)
	{
		if (_commands == null)
			_commands = new ArrayList<LyCommand>(1);
		else if (_commands.contains(command))
			return;
		if (pos >= 0)
			_commands.add(pos,command);
		else
			_commands.add(command);
		if (command != null)
			command.setParent(this);
	}

	/**
	 * Adds a separator.
	 */
	public void addSep()
	{
		insertSep(-1);
	}

	/**
	 * Inserts a separator at the given position.
	 * @param pos	Position where the separator is inserted.
	 */
	public void insertSep(int pos)
	{
		LyCommand sep = new LyCommand(SEP_ID, null);
		insertCommand(sep, pos);
	}

	/**
	 * Sets sub-component constraint.
	 * @param constraint New value of constraint
	 */
	public void setConstraint(LyConstraint constraint)
	{
		if (_constraint != null)
			_constraint.setParent(null);
		_constraint = constraint;
		if (constraint != null)
			constraint.setParent(this);
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<COMMANDS");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if ((_template == null)
			|| (_border != _template._border))
		{
			if (_border == LyArea.Border.NONE)
				pw.print(" border='NONE'");
			else if ((_border == LyArea.Border.IN) && (_template != null))
				pw.print(" border='IN'");
			else if (_border == LyArea.Border.OUT)
				pw.print(" border='OUT'");
			else if (_border == LyArea.Border.SOLID)
				pw.print(" border='SOLID'");
			else if (_border == LyArea.Border.RAISED)
				pw.print(" border='RAISED'");
			else if (_border == LyArea.Border.LOWERED)
				pw.print(" border='LOWERED'");
		}
		if ((_template == null)
			|| (_align != _template._align))
		{
			if (_align == Align.LEFT)
				pw.print(" align='LEFT'");
			else if (_align == Align.RIGHT)
				pw.print(" align='RIGHT'");
			else if ((_align == Align.CENTER) && (_template != null))
				pw.print(" align='CENTER'");
		}
		if (((_template == null) && (_font != getStringValue("COMMANDS.font", "$ALL.font")))
				|| ((_template != null) && (_font != _template._font)))
			pw.print(" font='" + LyString.getXmlString(_font) + "'");
		if (((_template == null) && (_color != getStringValue("COMMANDS.color", "$ALL.color")))
				|| ((_template != null) && (_color != _template._color)))
			pw.print(" color='" + LyString.getXmlString(_color) + "'");
		if (((_template == null) && (_fontColor != getStringValue("COMMANDS.fontColor", "$ALL.fontColor")))
				|| ((_template != null) && (_fontColor != _template._fontColor)))
			pw.print(" fontColor='" + LyString.getXmlString(_fontColor) + "'");

		if ((getChildCount() == 0) && (_datas == null))
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		LyComponent	commands;
		int			i, n;

		n = (_commands == null ? 0 : _commands.size());
		for (i = 0; i < n; i++)
		{
			commands = _commands.get(i);
			if ((commands == null)
					|| (commands.getId() == SEP_ID))
				pw.println("<SEP/>");
			else
				commands.writeXml(pw, debug);
		}

		if (_constraint != null)
			_constraint.writeXml(pw, debug);

		pw.println("</COMMANDS>");
	}
}
