/*
 * LyConstraintBase.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;
import java.io.*;


/**
 * The LyConstraintBase class is a generated Java implementation that corresponds to the element 
 * CONSTRAINT of the descriptive grammar of the views (views.dtd).
 * <B>&lt;CONSTRAINT&gt; : Positioning Constraint</B>
 * A constraint allows specifying for a given graphical component the way it will 
 * be placed inside its parent (i.e. the graphical component that contains it).
 * @see leon.view.LyComponent
 * @see leon.view.LyLyConstraint
 * See also tag <A href="../../conf/dtd/view/CONSTRAINT.html">CONSTRAINT</A> of grammar leon_views.dtd.
 */
public abstract class LyConstraintBase extends LyComponent
{
	public static final String			__VERSION = "$Id$";


	/** Possible values for attribute 'fill'. */
	public enum Fill
	{
		HORIZONTAL,
		VERTICAL,
		BOTH,
		NONE,
		DEFAULT;
	}


	/** Possible values for attribute 'hAlign'. */
	public enum HAlign
	{
		LEFT,
		CENTER,
		RIGHT,
		DEFAULT;
	}


	/** Possible values for attribute 'vAlign'. */
	public enum VAlign
	{
		TOP,
		MIDDLE,
		BOTTOM,
		DEFAULT;
	}


	/* Static methods */

	/**
	 * Utility method to get fill value from a string.
	 * @param token Token string
	 * @return Fill value corresponding to this token
	 */
	public static final Fill getFill(String token)
	{
		if (token == null)
			return null;
		else if (token.equalsIgnoreCase("HORIZONTAL")
				|| token.equals("0"))
			return Fill.HORIZONTAL;
		else if (token.equalsIgnoreCase("VERTICAL")
				|| token.equals("1"))
			return Fill.VERTICAL;
		else if (token.equalsIgnoreCase("BOTH")
				|| token.equals("2"))
			return Fill.BOTH;
		else if (token.equalsIgnoreCase("NONE")
				|| token.equals("3"))
			return Fill.NONE;
		else if (token.equalsIgnoreCase("DEFAULT")
				|| token.equals("4"))
			return Fill.DEFAULT;
		else
			return Fill.DEFAULT;
	}

	/**
	 * Utility method to get hAlign value from a string.
	 * @param token Token string
	 * @return HAlign value corresponding to this token
	 */
	public static final HAlign getHAlign(String token)
	{
		if (token == null)
			return null;
		else if (token.equalsIgnoreCase("LEFT")
				|| token.equals("0"))
			return HAlign.LEFT;
		else if (token.equalsIgnoreCase("CENTER")
				|| token.equals("1"))
			return HAlign.CENTER;
		else if (token.equalsIgnoreCase("RIGHT")
				|| token.equals("2"))
			return HAlign.RIGHT;
		else if (token.equalsIgnoreCase("DEFAULT")
				|| token.equals("3"))
			return HAlign.DEFAULT;
		else
			return HAlign.DEFAULT;
	}

	/**
	 * Utility method to get vAlign value from a string.
	 * @param token Token string
	 * @return VAlign value corresponding to this token
	 */
	public static final VAlign getVAlign(String token)
	{
		if (token == null)
			return null;
		else if (token.equalsIgnoreCase("TOP")
				|| token.equals("0"))
			return VAlign.TOP;
		else if (token.equalsIgnoreCase("MIDDLE")
				|| token.equals("1"))
			return VAlign.MIDDLE;
		else if (token.equalsIgnoreCase("BOTTOM")
				|| token.equals("2"))
			return VAlign.BOTTOM;
		else if (token.equalsIgnoreCase("DEFAULT")
				|| token.equals("3"))
			return VAlign.DEFAULT;
		else
			return VAlign.DEFAULT;
	}


	/* Attributes */

	/**
	 * Allows specifying the width occupied by the component inside its parent in 
	 * comparison with the other components.
	 * The expected value is an integer value.
	 * There is a special value, remainder, which allows specifying that the component 
	 * must occupy the whole space remaining available in width.
	 * A negative or null value is equivalent to the remainder value.
	 * The value -2 corresponds to using the default size.
	 */
	protected int				_colSpan;

	/**
	 * Allows specifying the height occupied by the component inside its parent in 
	 * comparison with the other components.
	 * The expected value is an integer value.
	 * There is a special value, remainder, which allows specifying that the component 
	 * must occupy the whole space remaining available in height.
	 * A negative or null value is equivalent to the remainder value.
	 * The value -2 corresponds to using the default size.
	 */
	protected int				_rowSpan;

	/**
	 * Allows specifying that the component occupies the whole space remaining in 
	 * width.
	 */
	protected boolean			_remainder;

	/**
	 * Allows specifying the horizontal position of the component.
	 * The DEFAULT value lets the viewer decide the way the component will be placed.
	 */
	protected Fill				_fill;

	/**
	 * Allows specifying the horizontal position of the component.
	 * The DEFAULT value lets the viewer decide the way the component will be placed.
	 */
	protected HAlign			_hAlign;

	/**
	 * Allows specifying the vertical position of the component.
	 * The DEFAULT value lets the viewer decide the way the component will be placed.
	 */
	protected VAlign			_vAlign;

	/**
	 * Allows specifying the margin that should be inserted around the component in the
	 * space that is reserved to it.
	 * The expected value is a positive integer value corresponding to a number of 
	 * pixels.
	 * A negative value is equivalent to the value0.
	 */
	protected int				_cellPadding;

	/**
	 * Allows specifying the margin that should be inserted at the left of the 
	 * component in the space that is reserved to it.
	 * The expected value is a positive integer value corresponding to a number of 
	 * pixels.
	 * A negative value is equivalent to the value0.
	 */
	protected int				_leftPadding;

	/**
	 * Allows specifying the margin that should be inserted at the right of the 
	 * component in the space that is reserved to it.
	 * The expected value is a positive integer value corresponding to a number of 
	 * pixels.
	 * A negative value is equivalent to the value0.
	 */
	protected int				_rightPadding;

	/**
	 * Allows specifying the margin that should be inserted at the top of the component
	 * in the space that is reserved to it.
	 * The expected value is a positive integer value corresponding to a number of 
	 * pixels.
	 * A negative value is equivalent to the value0.
	 */
	protected int				_topPadding;

	/**
	 * Allows specifying the margin that should be inserted at the bottom of the 
	 * component in the space that is reserved to it.
	 * The expected value is a positive integer value corresponding to a number of 
	 * pixels.
	 * A negative value is equivalent to the value0.
	 */
	protected int				_bottomPadding;

	/**
	 * Allows specifying the margin that should be inserted between the component and 
	 * the components that are around it.
	 * The expected value is a positive integer value corresponding to a number of 
	 * pixels.
	 * A negative value is equivalent to the value0.
	 */
	protected int				_cellSpacing;

	/**
	 * Allows specifying the percentage of horizontal enlargement of the component if 
	 * its parent has enough free space available.
	 * The expected value is a positive integer value corresponding to a weight (the 
	 * weights of the different components contained in the parent allow to perform a 
	 * barycentric distribution of the space that is allocated to each component).
	 * A negative value is equivalent to the value 0.
	 */
	protected double			_colWeight;

	/**
	 * Allows specifying the percentage of vertical enlargement of the component if its
	 * parent has enough free space available.
	 * The expected value is a positive integer value corresponding to a weight (the 
	 * weights of the different components contained in the parent allow performing a 
	 * barycentric distribution of the space that is allocated to each component).
	 * A negative value is equivalent to the value 0.
	 */
	protected double			_rowWeight;

	/**
	 * Gives the height (number of columns) on which the component is shown (given in 
	 * number of characters).
	 */
	protected int				_nCols;

	/**
	 * Gives the width (number of rows) on which the component is shown (given in 
	 * number of characters).
	 */
	protected int				_nRows;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_colSpan = getIntValue("CONSTRAINT.colSpan", -2);
		_rowSpan = getIntValue("CONSTRAINT.rowSpan", -2);
		_remainder = getBooleanValue("CONSTRAINT.remainder", false);
		_fill = Fill.DEFAULT;
		_hAlign = HAlign.DEFAULT;
		_vAlign = VAlign.DEFAULT;
		_cellPadding = getIntValue("CONSTRAINT.cellPadding", -2);
		_leftPadding = getIntValue("CONSTRAINT.leftPadding", -2);
		_rightPadding = getIntValue("CONSTRAINT.rightPadding", -2);
		_topPadding = getIntValue("CONSTRAINT.topPadding", -2);
		_bottomPadding = getIntValue("CONSTRAINT.bottomPadding", -2);
		_cellSpacing = getIntValue("CONSTRAINT.cellSpacing", -2);
		_colWeight = getDoubleValue("CONSTRAINT.colWeight", -2);
		_rowWeight = getDoubleValue("CONSTRAINT.rowWeight", -2);
		_nCols = getIntValue("CONSTRAINT.nCols", -1);
		_nRows = getIntValue("CONSTRAINT.nRows", -1);
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyConstraint	model = (LyConstraint)component;

		_model = component;
		_colSpan = model._colSpan;
		_rowSpan = model._rowSpan;
		_remainder = model._remainder;
		_fill = model._fill;
		_hAlign = model._hAlign;
		_vAlign = model._vAlign;
		_cellPadding = model._cellPadding;
		_leftPadding = model._leftPadding;
		_rightPadding = model._rightPadding;
		_topPadding = model._topPadding;
		_bottomPadding = model._bottomPadding;
		_cellSpacing = model._cellSpacing;
		_colWeight = model._colWeight;
		_rowWeight = model._rowWeight;
		_nCols = model._nCols;
		_nRows = model._nRows;
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	public LyConstraintBase()
	{
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyConstraintBase(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_colSpan = node.getInt("colSpan");
		_rowSpan = node.getInt("rowSpan");
		_remainder = node.getBoolean("remainder");
		_fill = getFill(node.getString("fill"));
		_hAlign = getHAlign(node.getString("hAlign"));
		_vAlign = getVAlign(node.getString("vAlign"));
		_cellPadding = node.getInt("cellPadding");
		_leftPadding = node.getInt("leftPadding");
		_rightPadding = node.getInt("rightPadding");
		_topPadding = node.getInt("topPadding");
		_bottomPadding = node.getInt("bottomPadding");
		_cellSpacing = node.getInt("cellSpacing");
		_colWeight = node.getDouble("colWeight");
		_rowWeight = node.getDouble("rowWeight");
		_nCols = node.getInt("nCols");
		_nRows = node.getInt("nRows");
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyConstraint clone = new LyConstraint();

		clone.initFromModel(this);
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of colSpan.
	 * @return value of colSpan.
	 */
	public int getColSpan()
	{
		return _colSpan;
	}

	/**
	 * Gets value of rowSpan.
	 * @return value of rowSpan.
	 */
	public int getRowSpan()
	{
		return _rowSpan;
	}

	/**
	 * Checks if value of attribute remainder is set.
	 * @return true if remainder is set, false otherwise.
	 */
	public boolean isRemainder()
	{
		return _remainder;
	}

	/**
	 * Gets value of fill.
	 * @return value of fill.
	 */
	public Fill getFill()
	{
		return _fill;
	}

	/**
	 * Gets value of hAlign.
	 * @return value of hAlign.
	 */
	public HAlign getHAlign()
	{
		return _hAlign;
	}

	/**
	 * Gets value of vAlign.
	 * @return value of vAlign.
	 */
	public VAlign getVAlign()
	{
		return _vAlign;
	}

	/**
	 * Gets value of cellPadding.
	 * @return value of cellPadding.
	 */
	public int getCellPadding()
	{
		return _cellPadding;
	}

	/**
	 * Gets value of leftPadding.
	 * @return value of leftPadding.
	 */
	public int getLeftPadding()
	{
		return _leftPadding;
	}

	/**
	 * Gets value of rightPadding.
	 * @return value of rightPadding.
	 */
	public int getRightPadding()
	{
		return _rightPadding;
	}

	/**
	 * Gets value of topPadding.
	 * @return value of topPadding.
	 */
	public int getTopPadding()
	{
		return _topPadding;
	}

	/**
	 * Gets value of bottomPadding.
	 * @return value of bottomPadding.
	 */
	public int getBottomPadding()
	{
		return _bottomPadding;
	}

	/**
	 * Gets value of cellSpacing.
	 * @return value of cellSpacing.
	 */
	public int getCellSpacing()
	{
		return _cellSpacing;
	}

	/**
	 * Gets value of colWeight.
	 * @return value of colWeight.
	 */
	public double getColWeight()
	{
		return _colWeight;
	}

	/**
	 * Gets value of rowWeight.
	 * @return value of rowWeight.
	 */
	public double getRowWeight()
	{
		return _rowWeight;
	}

	/**
	 * Gets value of nCols.
	 * @return value of nCols.
	 */
	public int getNCols()
	{
		return _nCols;
	}

	/**
	 * Gets value of nRows.
	 * @return value of nRows.
	 */
	public int getNRows()
	{
		return _nRows;
	}


	/* Modifiers */

	/**
	 * Initializes attribute colSpan to the given value.
	 * @param colSpan New value of colSpan.
	 */
	public void setColSpan(int colSpan)
	{
		if (colSpan == _colSpan)
			return;
		_colSpan = colSpan;
	}

	/**
	 * Initializes attribute rowSpan to the given value.
	 * @param rowSpan New value of rowSpan.
	 */
	public void setRowSpan(int rowSpan)
	{
		if (rowSpan == _rowSpan)
			return;
		_rowSpan = rowSpan;
	}

	/**
	 * Initializes attribute remainder to the given value.
	 * @param remainder Boolean indicating if attribute remainder is set or unset.
	 */
	public void setRemainder(boolean remainder)
	{
		if (remainder == _remainder)
			return;
		_remainder = remainder;
	}

	/**
	 * Initializes attribute fill to the given value.
	 * @param fill New value of fill.
	 * @exception IllegalArgumentException if fill parameter is null.
	 */
	public void setFill(Fill fill)
	{
		if (fill == _fill)
			return;
		if (fill == null)
			throw new IllegalArgumentException("BF AppEngine : invalid value for attribute fill");

		_fill = fill;
	}

	/**
	 * Initializes attribute hAlign to the given value.
	 * @param hAlign New value of hAlign.
	 * @exception IllegalArgumentException if hAlign parameter is null.
	 */
	public void setHAlign(HAlign hAlign)
	{
		if (hAlign == _hAlign)
			return;
		if (hAlign == null)
			throw new IllegalArgumentException("BF AppEngine : invalid value for attribute hAlign");

		_hAlign = hAlign;
	}

	/**
	 * Initializes attribute vAlign to the given value.
	 * @param vAlign New value of vAlign.
	 * @exception IllegalArgumentException if vAlign parameter is null.
	 */
	public void setVAlign(VAlign vAlign)
	{
		if (vAlign == _vAlign)
			return;
		if (vAlign == null)
			throw new IllegalArgumentException("BF AppEngine : invalid value for attribute vAlign");

		_vAlign = vAlign;
	}

	/**
	 * Initializes attribute cellPadding to the given value.
	 * @param cellPadding New value of cellPadding.
	 */
	public void setCellPadding(int cellPadding)
	{
		if (cellPadding == _cellPadding)
			return;
		_cellPadding = cellPadding;
	}

	/**
	 * Initializes attribute leftPadding to the given value.
	 * @param leftPadding New value of leftPadding.
	 */
	public void setLeftPadding(int leftPadding)
	{
		if (leftPadding == _leftPadding)
			return;
		_leftPadding = leftPadding;
	}

	/**
	 * Initializes attribute rightPadding to the given value.
	 * @param rightPadding New value of rightPadding.
	 */
	public void setRightPadding(int rightPadding)
	{
		if (rightPadding == _rightPadding)
			return;
		_rightPadding = rightPadding;
	}

	/**
	 * Initializes attribute topPadding to the given value.
	 * @param topPadding New value of topPadding.
	 */
	public void setTopPadding(int topPadding)
	{
		if (topPadding == _topPadding)
			return;
		_topPadding = topPadding;
	}

	/**
	 * Initializes attribute bottomPadding to the given value.
	 * @param bottomPadding New value of bottomPadding.
	 */
	public void setBottomPadding(int bottomPadding)
	{
		if (bottomPadding == _bottomPadding)
			return;
		_bottomPadding = bottomPadding;
	}

	/**
	 * Initializes attribute cellSpacing to the given value.
	 * @param cellSpacing New value of cellSpacing.
	 */
	public void setCellSpacing(int cellSpacing)
	{
		if (cellSpacing == _cellSpacing)
			return;
		_cellSpacing = cellSpacing;
	}

	/**
	 * Initializes attribute colWeight to the given value.
	 * @param colWeight New value of colWeight.
	 */
	public void setColWeight(double colWeight)
	{
		if (colWeight == _colWeight)
			return;
		_colWeight = colWeight;
	}

	/**
	 * Initializes attribute rowWeight to the given value.
	 * @param rowWeight New value of rowWeight.
	 */
	public void setRowWeight(double rowWeight)
	{
		if (rowWeight == _rowWeight)
			return;
		_rowWeight = rowWeight;
	}

	/**
	 * Initializes attribute nCols to the given value.
	 * @param nCols New value of nCols.
	 */
	public void setNCols(int nCols)
	{
		if (nCols == _nCols)
			return;
		_nCols = nCols;
	}

	/**
	 * Initializes attribute nRows to the given value.
	 * @param nRows New value of nRows.
	 */
	public void setNRows(int nRows)
	{
		if (nRows == _nRows)
			return;
		_nRows = nRows;
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<CONSTRAINT");
		if (_colSpan != getIntValue("CONSTRAINT.colSpan", -2))
			pw.print(" colSpan='" + _colSpan + "'");
		if (_rowSpan != getIntValue("CONSTRAINT.rowSpan", -2))
			pw.print(" rowSpan='" + _rowSpan + "'");
		if (_remainder)
			pw.print(" remainder='" + _remainder + "'");
		if (_fill == Fill.HORIZONTAL)
			pw.print(" fill='HORIZONTAL'");
		else if (_fill == Fill.VERTICAL)
			pw.print(" fill='VERTICAL'");
		else if (_fill == Fill.BOTH)
			pw.print(" fill='BOTH'");
		else if (_fill == Fill.NONE)
			pw.print(" fill='NONE'");
		if (_hAlign == HAlign.LEFT)
			pw.print(" hAlign='LEFT'");
		else if (_hAlign == HAlign.CENTER)
			pw.print(" hAlign='CENTER'");
		else if (_hAlign == HAlign.RIGHT)
			pw.print(" hAlign='RIGHT'");
		if (_vAlign == VAlign.TOP)
			pw.print(" vAlign='TOP'");
		else if (_vAlign == VAlign.MIDDLE)
			pw.print(" vAlign='MIDDLE'");
		else if (_vAlign == VAlign.BOTTOM)
			pw.print(" vAlign='BOTTOM'");
		if (_cellPadding != getIntValue("CONSTRAINT.cellPadding", -2))
			pw.print(" cellPadding='" + _cellPadding + "'");
		if (_leftPadding != getIntValue("CONSTRAINT.leftPadding", -2))
			pw.print(" leftPadding='" + _leftPadding + "'");
		if (_rightPadding != getIntValue("CONSTRAINT.rightPadding", -2))
			pw.print(" rightPadding='" + _rightPadding + "'");
		if (_topPadding != getIntValue("CONSTRAINT.topPadding", -2))
			pw.print(" topPadding='" + _topPadding + "'");
		if (_bottomPadding != getIntValue("CONSTRAINT.bottomPadding", -2))
			pw.print(" bottomPadding='" + _bottomPadding + "'");
		if (_cellSpacing != getIntValue("CONSTRAINT.cellSpacing", -2))
			pw.print(" cellSpacing='" + _cellSpacing + "'");
		if (_colWeight != getDoubleValue("CONSTRAINT.colWeight", -2))
			pw.print(" colWeight='" + _colWeight + "'");
		if (_rowWeight != getDoubleValue("CONSTRAINT.rowWeight", -2))
			pw.print(" rowWeight='" + _rowWeight + "'");
		if (_nCols != getIntValue("CONSTRAINT.nCols", -1))
			pw.print(" nCols='" + _nCols + "'");
		if (_nRows != getIntValue("CONSTRAINT.nRows", -1))
			pw.print(" nRows='" + _nRows + "'");
		pw.println("/>");
	}
}
