/*
 * LyDate.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved.
 */

package leon.view;

import leon.misc.*;


/**
 * The LyDate class is a generated Java implementation that corresponds to the element DATE of the
 * descriptive grammar of the views (views.dtd). <B>&lt;DATE&gt; : Input Field of Date Type</B> An
 * input field of Date type allows displaying a graphical component allowing to input or to consult
 * date and/or hour. It does not comprise any element. It is possible to specify the type of the
 * date as well as the display format.
 *
 * @see  leon.view.LyFieldItem See also tag <A href="../../conf/dtd/view/DATE.html">DATE</A> of
 *       grammar leon_views.dtd.
 */
public class LyDate extends LyDateBase
{
	/** Indicates if we use arrows for formatted date inputs. */
	public static boolean	SHOW_ARROWS = true;

	/** minimum size for date component in edit table */
	public static int      EDITTABLE_DATE_MIN_SIZE = 20;

	/* Static methods */

	/**
	 * Utility method to get environment date type from date type.
	 *
	 * @param   token  Token string
	 *
	 * @return  Type value corresponding to this token
	 */
	public static final short getType(Type type)
	{
		if (type == null)
			return -1;
		else if (type == Type.DATE_TIME)
			return LyEnvironment.TYPE_DATE_TIME;
		else if (type == Type.DATE_HOUR_MIN)
			return LyEnvironment.TYPE_DATE_HOUR_MIN;
		else if (type == Type.DATE)
			return LyEnvironment.TYPE_DATE;
		else if (type == Type.TIME)
			return LyEnvironment.TYPE_TIME;
		else if (type == Type.HOUR_MIN)
			return LyEnvironment.TYPE_HOUR_MIN;
		else
			return -1;
	}

	/**
	 * Utility method to get date type from environment date type.
	 *
	 * @param   type  Environment date type.
	 *
	 * @return  Type value corresponding to given date type.
	 */
	public static final Type getType(int type)
	{
		if (type == LyEnvironment.TYPE_DATE_TIME)
			return Type.DATE_TIME;
		else if (type == LyEnvironment.TYPE_DATE_HOUR_MIN)
			return Type.DATE_HOUR_MIN;
		else if (type == LyEnvironment.TYPE_DATE)
			return Type.DATE;
		else if (type == LyEnvironment.TYPE_TIME)
			return Type.TIME;
		else if (type == LyEnvironment.TYPE_HOUR_MIN)
			return Type.HOUR_MIN;
		else
			return null;
	}

	boolean _showArrows;

	/**
	 * Default constructor.
	 */
	public LyDate()
	{
		super();
	}

	/* Protected methods for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 *
	 * @param  node  Node from which the component is constructed.
	 *
	 * @see    leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyDate(LyViewNode node)
	{
		super(node);
	}

	/**
	 * Initialisation of the data structure.
	 *
	 * @see  leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();
		_showArrows = true;
	}

	/**
	 * Initializes attribute showArrows to the given value.
	 *
	 * @param  showArrows  Boolean indicating if attribute showArrows is set or unset.
	 */
	public void showArrows(boolean showArrows)
	{
		if (showArrows == _showArrows)
			return;

		_showArrows = showArrows;
	}

	/**
	 * Checks if value of attribute showArrows is set.
	 *
	 * @return  true if showArrows is set, false otherwise.
	 */
	public boolean isShowArrows()
	{
		return _showArrows;
	}
} // end class LyDate
