/*
 * LyDateBase.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;
import java.io.*;
import leon.view.getinterface.*;
import leon.view.setinterface.*;
import leon.misc.LyString;


/**
 * The LyDateBase class is a generated Java implementation that corresponds to the element 
 * DATE of the descriptive grammar of the views (views.dtd).
 * <B>&lt;DATE&gt; : Input Field of Date Type</B>
 * An input field of Date type allows displaying a graphical component allowing to 
 * input or to consult date and/or hour.
 * It does not comprise any element.
 * It is possible to specify the type of the date as well as the display format.
 * @see leon.view.LyFieldItem
 * @see leon.view.LyLyDate
 * See also tag <A href="../../conf/dtd/view/DATE.html">DATE</A> of grammar leon_views.dtd.
 */
public abstract class LyDateBase extends LyFieldItem
	implements  LyGetIdInterface,
				LyGetLabelInterface,
				LySetLabelInterface
{
	public static final String			__VERSION = "$Id$";


	/** Possible values for attribute 'type'. */
	public enum Type
	{
		DATE_TIME,
		DATE_HOUR_MIN,
		DATE,
		TIME,
		HOUR_MIN;
	}


	/* Static methods */

	/**
	 * Utility method to get type value from a string.
	 * @param token Token string
	 * @return Type value corresponding to this token
	 */
	public static final Type getType(String token)
	{
		if (token == null)
			return null;
		else if (token.equalsIgnoreCase("DATE_TIME")
				|| token.equals("0"))
			return Type.DATE_TIME;
		else if (token.equalsIgnoreCase("DATE_HOUR_MIN")
				|| token.equals("1"))
			return Type.DATE_HOUR_MIN;
		else if (token.equalsIgnoreCase("DATE")
				|| token.equals("2"))
			return Type.DATE;
		else if (token.equalsIgnoreCase("TIME")
				|| token.equals("3"))
			return Type.TIME;
		else if (token.equalsIgnoreCase("HOUR_MIN")
				|| token.equals("4"))
			return Type.HOUR_MIN;
		else
			return Type.DATE_TIME;
	}


	/* Attributes */

	/**
	 * Identifier of the component.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyDate			_template;

	/**
	 * Value displayed for the date.
	 */
	protected String			_label;

	/**
	 * Type.
	 * This type determines the components managed in the date field.
	 * The following types are managed:
	 * DATE_TIME: Month-day-year-hour-minutes-seconds
	 * DATE_HOUR_MIN: Month-day -year-hour-minutes
	 * DATE: Day-month-year
	 * TIME: Hour-minutes-seconds
	 * HOUR_MIN: Minutes-seconds
	 * HOUR: Hour
	 */
	protected Type				_type;

	/**
	 * Display format.
	 * The syntax of the display format is the one specified by the 
	 * java.text.SimpleDateFormat class (see the corresponding Java documentation). The
	 * default display format for US English is: MM/dd/yyyy hh:mm:ss.
	 */
	protected String			_format;

	/**
	 * Allows specifying the name of a Java class that replaces the default graphical 
	 * component.
	 */
	protected String			_bean;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	/**
	 * Gives the number of columns of the date (given in number of characters).
	 */
	protected int				_nCols;

	/**
	 * DATE.disabledColor
	 */
	protected String			_disabledColor;

	/**
	 * DATE.arrows
	 */
	protected boolean			_arrows;

	/**
	 * DATE.arrowUp
	 */
	protected String			_arrowUp;

	/**
	 * DATE.arrowDown
	 */
	protected String			_arrowDown;

	/**
	 * DATE.arrowDownBig
	 */
	protected String			_arrowDownBig;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_id = null;
		_template = null;
		_label = null;
		_type = Type.DATE_TIME;
		_format = null;
		_bean = null;
		_font = getStringValue("DATE.font", "$ALL.font");
		_color = getStringValue("DATE.color", "$ALL.color");
		_fontColor = getStringValue("DATE.fontColor", "$ALL.fontColor");
		_nCols = getIntValue("DATE.nCols", -1);
		_disabledColor = getStringValue("DATE.disabledColor", "$ALL.disabledColor");
		_arrows = getBooleanValue("DATE.arrows", true);
		_arrowUp = getStringValue("DATE.arrowUp", "$ALL.arrowUp");
		_arrowDown = getStringValue("DATE.arrowDown", "$ALL.arrowDown");
		_arrowDownBig = getStringValue("DATE.arrowDownBig", "$ALL.arrowDownBig");
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyDate	model = (LyDate)component;

		_model = component;
		_label = model._label;
		_type = model._type;
		_format = model._format;
		_bean = model._bean;
		_font = model._font;
		_color = model._color;
		_fontColor = model._fontColor;
		_nCols = model._nCols;
		_disabledColor = model._disabledColor;
		_arrows = model._arrows;
		_arrowUp = model._arrowUp;
		_arrowDown = model._arrowDown;
		_arrowDownBig = model._arrowDownBig;
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	public LyDateBase()
	{
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyDateBase(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_id = node.getString("id");
		_template = (LyDate)node.getComponent("template");

		if (_template != null)
			initFromModel(_template);

		if (node.isSpecified("label"))
			_label = node.getString("label");
		if (node.isSpecified("type"))
			_type = getType(node.getString("type"));
		if (node.isSpecified("format"))
			_format = node.getString("format");
		if (node.isSpecified("bean"))
			_bean = node.getString("bean");
		if (node.isSpecified("font"))
			_font = node.getString("font");
		if (node.isSpecified("color"))
			_color = node.getString("color");
		if (node.isSpecified("fontColor"))
			_fontColor = node.getString("fontColor");
		if (node.isSpecified("nCols"))
			_nCols = node.getInt("nCols");
		if (node.isSpecified("disabledColor"))
			_disabledColor = node.getString("disabledColor");
		if (node.isSpecified("arrows"))
			_arrows = node.getBoolean("arrows");
		if (node.isSpecified("arrowUp"))
			_arrowUp = node.getString("arrowUp");
		if (node.isSpecified("arrowDown"))
			_arrowDown = node.getString("arrowDown");
		if (node.isSpecified("arrowDownBig"))
			_arrowDownBig = node.getString("arrowDownBig");
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyDate clone = new LyDate();

		clone.initFromModel(this);
		clone._template = (LyDate)this;
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of id.
	 * @return value of id.
	 */
	public String getId()
	{
		return _id;
	}

	/**
	 * Gets value of template.
	 * @return value of template.
	 */
	public LyDate getTemplate()
	{
		return _template;
	}

	/**
	 * Gets value of label.
	 * @return value of label.
	 */
	public String getLabel()
	{
		return _label;
	}

	/**
	 * Gets value of type.
	 * @return value of type.
	 */
	public Type getType()
	{
		return _type;
	}

	/**
	 * Gets value of format.
	 * @return value of format.
	 */
	public String getFormat()
	{
		return _format;
	}

	/**
	 * Gets value of bean.
	 * @return value of bean.
	 */
	public String getBean()
	{
		return _bean;
	}

	/**
	 * Gets value of font.
	 * @return value of font.
	 */
	public String getFont()
	{
		return _font;
	}

	/**
	 * Checks whether the font attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFont()
	{
		return (_font == getStringValue("DATE.font"));
	}

	/**
	 * Gets value of color.
	 * @return value of color.
	 */
	public String getColor()
	{
		return _color;
	}

	/**
	 * Checks whether the color attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultColor()
	{
		return (_color == getStringValue("DATE.color"));
	}

	/**
	 * Gets value of fontColor.
	 * @return value of fontColor.
	 */
	public String getFontColor()
	{
		return _fontColor;
	}

	/**
	 * Checks whether the fontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFontColor()
	{
		return (_fontColor == getStringValue("DATE.fontColor"));
	}

	/**
	 * Gets value of nCols.
	 * @return value of nCols.
	 */
	public int getNCols()
	{
		return _nCols;
	}

	/**
	 * Gets value of disabledColor.
	 * @return value of disabledColor.
	 */
	public String getDisabledColor()
	{
		return _disabledColor;
	}

	/**
	 * Checks whether the disabledColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultDisabledColor()
	{
		return (_disabledColor == getStringValue("DATE.disabledColor"));
	}

	/**
	 * Checks if value of attribute arrows is set.
	 * @return true if arrows is set, false otherwise.
	 */
	public boolean isArrows()
	{
		return _arrows;
	}

	/**
	 * Gets value of arrowUp.
	 * @return value of arrowUp.
	 */
	public String getArrowUp()
	{
		return _arrowUp;
	}

	/**
	 * Checks whether the arrowUp attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultArrowUp()
	{
		return (_arrowUp == getStringValue("DATE.arrowUp"));
	}

	/**
	 * Gets value of arrowDown.
	 * @return value of arrowDown.
	 */
	public String getArrowDown()
	{
		return _arrowDown;
	}

	/**
	 * Checks whether the arrowDown attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultArrowDown()
	{
		return (_arrowDown == getStringValue("DATE.arrowDown"));
	}

	/**
	 * Gets value of arrowDownBig.
	 * @return value of arrowDownBig.
	 */
	public String getArrowDownBig()
	{
		return _arrowDownBig;
	}

	/**
	 * Checks whether the arrowDownBig attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultArrowDownBig()
	{
		return (_arrowDownBig == getStringValue("DATE.arrowDownBig"));
	}


	/* Modifiers */

	/**
	 * Initializes attribute id to the given value.
	 * @param id New value of id.
	 */
	public void setId(String id)
	{
		if ((id == _id)
			|| ((id != null)
				&& (_id != null)
				&& id.equals(_id)))
			return;

		_id = (id != null ? id.intern() : null);
	}

	/**
	 * Initializes attribute template to the given value.
	 * @param template New value of template.
	 */
	public void setTemplate(LyDate template)
	{
		if (template == null)
			return;
		if (_template != null)
			throw new IllegalStateException("BF AppEngine : attribute template was already set.");

		initFromModel(template);
		_template = template;
	}

	/**
	 * Sets attribute label to the given value.
	 * @param label New value of label.
	 */
	public void setLabel(String label)
	{
		if ((label == _label)
			|| ((label != null)
				&& (_label != null)
				&& label.equals(_label)))
			return;

		_label = label;
		if (_internalView instanceof LySetLabelInterface)
			((LySetLabelInterface)_internalView).setLabel(label);
	}

	/**
	 * Initializes attribute type to the given value.
	 * @param type New value of type.
	 * @exception IllegalArgumentException if type parameter is null.
	 */
	public void setType(Type type)
	{
		if (type == _type)
			return;
		if (type == null)
			throw new IllegalArgumentException("BF AppEngine : invalid value for attribute type");

		_type = type;
	}

	/**
	 * Initializes attribute format to the given value.
	 * @param format New value of format.
	 */
	public void setFormat(String format)
	{
		if ((format == _format)
			|| ((format != null)
				&& (_format != null)
				&& format.equals(_format)))
			return;

		_format = format;
	}

	/**
	 * Initializes attribute bean to the given value.
	 * @param bean New value of bean.
	 */
	public void setBean(String bean)
	{
		if ((bean == _bean)
			|| ((bean != null)
				&& (_bean != null)
				&& bean.equals(_bean)))
			return;

		_bean = bean;
	}

	/**
	 * Initializes attribute font to the given value.
	 * @param font New value of font.
	 */
	public void setFont(String font)
	{
		if ((font == _font)
			|| ((font != null)
				&& (_font != null)
				&& font.equals(_font)))
			return;

		_font = font;
	}

	/**
	 * Initializes attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color)
	{
		if ((color == _color)
			|| ((color != null)
				&& (_color != null)
				&& color.equals(_color)))
			return;

		_color = color;
	}

	/**
	 * Initializes attribute fontColor to the given value.
	 * @param fontColor New value of fontColor.
	 */
	public void setFontColor(String fontColor)
	{
		if ((fontColor == _fontColor)
			|| ((fontColor != null)
				&& (_fontColor != null)
				&& fontColor.equals(_fontColor)))
			return;

		_fontColor = fontColor;
	}

	/**
	 * Initializes attribute nCols to the given value.
	 * @param nCols New value of nCols.
	 */
	public void setNCols(int nCols)
	{
		if (nCols == _nCols)
			return;
		_nCols = nCols;
	}

	/**
	 * Initializes attribute disabledColor to the given value.
	 * @param disabledColor New value of disabledColor.
	 */
	public void setDisabledColor(String disabledColor)
	{
		if ((disabledColor == _disabledColor)
			|| ((disabledColor != null)
				&& (_disabledColor != null)
				&& disabledColor.equals(_disabledColor)))
			return;

		_disabledColor = disabledColor;
	}

	/**
	 * Initializes attribute arrows to the given value.
	 * @param arrows Boolean indicating if attribute arrows is set or unset.
	 */
	public void setArrows(boolean arrows)
	{
		if (arrows == _arrows)
			return;
		_arrows = arrows;
	}

	/**
	 * Initializes attribute arrowUp to the given value.
	 * @param arrowUp New value of arrowUp.
	 */
	public void setArrowUp(String arrowUp)
	{
		if ((arrowUp == _arrowUp)
			|| ((arrowUp != null)
				&& (_arrowUp != null)
				&& arrowUp.equals(_arrowUp)))
			return;

		_arrowUp = arrowUp;
	}

	/**
	 * Initializes attribute arrowDown to the given value.
	 * @param arrowDown New value of arrowDown.
	 */
	public void setArrowDown(String arrowDown)
	{
		if ((arrowDown == _arrowDown)
			|| ((arrowDown != null)
				&& (_arrowDown != null)
				&& arrowDown.equals(_arrowDown)))
			return;

		_arrowDown = arrowDown;
	}

	/**
	 * Initializes attribute arrowDownBig to the given value.
	 * @param arrowDownBig New value of arrowDownBig.
	 */
	public void setArrowDownBig(String arrowDownBig)
	{
		if ((arrowDownBig == _arrowDownBig)
			|| ((arrowDownBig != null)
				&& (_arrowDownBig != null)
				&& arrowDownBig.equals(_arrowDownBig)))
			return;

		_arrowDownBig = arrowDownBig;
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<DATE");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template._label)))
			pw.print(" label='" + LyString.getXmlString(_label) + "'");
		if ((_template == null)
			|| (_type != _template._type))
		{
			if ((_type == Type.DATE_TIME) && (_template != null))
				pw.print(" type='DATE_TIME'");
			else if (_type == Type.DATE_HOUR_MIN)
				pw.print(" type='DATE_HOUR_MIN'");
			else if (_type == Type.DATE)
				pw.print(" type='DATE'");
			else if (_type == Type.TIME)
				pw.print(" type='TIME'");
			else if (_type == Type.HOUR_MIN)
				pw.print(" type='HOUR_MIN'");
		}
		if (((_template == null) && (_format != null))
				|| ((_template != null) && (_format != _template._format)))
			pw.print(" format='" + LyString.getXmlString(_format) + "'");
		if (((_template == null) && (_bean != null))
				|| ((_template != null) && (_bean != _template._bean)))
			pw.print(" bean='" + LyString.getXmlString(_bean) + "'");
		if (((_template == null) && (_font != getStringValue("DATE.font", "$ALL.font")))
				|| ((_template != null) && (_font != _template._font)))
			pw.print(" font='" + LyString.getXmlString(_font) + "'");
		if (((_template == null) && (_color != getStringValue("DATE.color", "$ALL.color")))
				|| ((_template != null) && (_color != _template._color)))
			pw.print(" color='" + LyString.getXmlString(_color) + "'");
		if (((_template == null) && (_fontColor != getStringValue("DATE.fontColor", "$ALL.fontColor")))
				|| ((_template != null) && (_fontColor != _template._fontColor)))
			pw.print(" fontColor='" + LyString.getXmlString(_fontColor) + "'");
		if (((_template == null) && (_nCols != getIntValue("DATE.nCols", -1)))
				|| ((_template != null) && (_nCols != _template._nCols)))
			pw.print(" nCols='" + _nCols + "'");
		if (((_template == null) && (_disabledColor != getStringValue("DATE.disabledColor", "$ALL.disabledColor")))
				|| ((_template != null) && (_disabledColor != _template._disabledColor)))
			pw.print(" disabledColor='" + LyString.getXmlString(_disabledColor) + "'");
		if (((_template == null) && !_arrows)
				|| ((_template != null) && (_arrows != _template._arrows)))
			pw.print(" arrows='" + _arrows + "'");
		if (((_template == null) && (_arrowUp != getStringValue("DATE.arrowUp", "$ALL.arrowUp")))
				|| ((_template != null) && (_arrowUp != _template._arrowUp)))
			pw.print(" arrowUp='" + LyString.getXmlString(_arrowUp) + "'");
		if (((_template == null) && (_arrowDown != getStringValue("DATE.arrowDown", "$ALL.arrowDown")))
				|| ((_template != null) && (_arrowDown != _template._arrowDown)))
			pw.print(" arrowDown='" + LyString.getXmlString(_arrowDown) + "'");
		if (((_template == null) && (_arrowDownBig != getStringValue("DATE.arrowDownBig", "$ALL.arrowDownBig")))
				|| ((_template != null) && (_arrowDownBig != _template._arrowDownBig)))
			pw.print(" arrowDownBig='" + LyString.getXmlString(_arrowDownBig) + "'");
		pw.println("/>");
	}
}
