/*
 * LyDay.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;
import java.io.*;
import leon.view.graphic.*;
import leon.view.getinterface.*;
import leon.view.setinterface.*;
import leon.misc.LyString;


/**
 * The LyDay class is a generated Java implementation that corresponds to the element 
 * DAY of the descriptive grammar of the views (views.dtd).
 * <B>&lt;DAY&gt; : Day</B>
 * A day allows specifying on a Gantt planning the special days (days off for 
 * example). Those days may be of three types: days of week (for example Saturday 
 * and Sunday), days of year (for example may the first) and special days (for 
 * example 24th December 2006).
 * @see leon.view.LyComponent
 * See also tag <A href="../../conf/dtd/view/DAY.html">DAY</A> of grammar leon_views.dtd.
 */
public class LyDay extends LyComponent
	implements  LyGetValueInterface,
				LySetValueInterface,
				LySetColorInterface
{
	public static final String			__VERSION = "$Id$";


	/** Possible values for attribute 'type'. */
	public enum Type
	{
		DAY_OF_WEEK,
		DAY_OF_YEAR,
		DAY;
	}


	/** Possible values for attribute 'dayOfWeek'. */
	public enum DayOfWeek
	{
		MONDAY,
		TUESDAY,
		WEDNESDAY,
		THURSDAY,
		FRIDAY,
		SATURDAY,
		SUNDAY;
	}


	/* Static methods */

	/**
	 * Utility method to get type value from a string.
	 * @param token Token string
	 * @return Type value corresponding to this token
	 */
	public static final Type getType(String token)
	{
		if (token == null)
			return null;
		else if (token.equalsIgnoreCase("DAY_OF_WEEK")
				|| token.equals("0"))
			return Type.DAY_OF_WEEK;
		else if (token.equalsIgnoreCase("DAY_OF_YEAR")
				|| token.equals("1"))
			return Type.DAY_OF_YEAR;
		else if (token.equalsIgnoreCase("DAY")
				|| token.equals("2"))
			return Type.DAY;
		else
			return Type.DAY_OF_WEEK;
	}

	/**
	 * Utility method to get dayOfWeek value from a string.
	 * @param token Token string
	 * @return DayOfWeek value corresponding to this token
	 */
	public static final DayOfWeek getDayOfWeek(String token)
	{
		if (token == null)
			return null;
		else if (token.equalsIgnoreCase("MONDAY")
				|| token.equals("0"))
			return DayOfWeek.MONDAY;
		else if (token.equalsIgnoreCase("TUESDAY")
				|| token.equals("1"))
			return DayOfWeek.TUESDAY;
		else if (token.equalsIgnoreCase("WEDNESDAY")
				|| token.equals("2"))
			return DayOfWeek.WEDNESDAY;
		else if (token.equalsIgnoreCase("THURSDAY")
				|| token.equals("3"))
			return DayOfWeek.THURSDAY;
		else if (token.equalsIgnoreCase("FRIDAY")
				|| token.equals("4"))
			return DayOfWeek.FRIDAY;
		else if (token.equalsIgnoreCase("SATURDAY")
				|| token.equals("5"))
			return DayOfWeek.SATURDAY;
		else if (token.equalsIgnoreCase("SUNDAY")
				|| token.equals("6"))
			return DayOfWeek.SUNDAY;
		else
			return DayOfWeek.SUNDAY;
	}


	/* Attributes */

	/**
	 * Indicates the type of the day:
	 * DAY_OF_WEEK: day of week (for example Sunday)
	 * DAY_OF_YEAR: day of year (for example may the first)
	 * DAY: special day (for example 24th December 2006).
	 */
	protected Type				_type;

	/**
	 * Day of week. Only used for the DAY_OF_WEEK type
	 * MONDAY: monday
	 * TUESDAY: tuesday
	 * WEDNESDAY: wednesday
	 * THURSDAY: thursday
	 * FRIDAY: friday
	 * SATURDAY: saturday
	 * SUNDAY: sunday
	 */
	protected DayOfWeek			_dayOfWeek;

	/**
	 * Value of the data. Only used for the DAY_OF_YEAR ou DAY types. Represents the 
	 * day date respecting the date format used in the Gantt planning.
	 */
	protected String			_value;

	/**
	 * Color used to represent the day on the planning.
	 */
	protected String			_color;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_type = Type.DAY_OF_WEEK;
		_dayOfWeek = DayOfWeek.SUNDAY;
		_value = null;
		_color = getStringValue("DAY.color", "$ALL.color");
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyDay	model = (LyDay)component;

		_model = component;
		_type = model._type;
		_dayOfWeek = model._dayOfWeek;
		_value = model._value;
		_color = model._color;
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	public LyDay()
	{
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyDay(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_type = getType(node.getString("type"));
		_dayOfWeek = getDayOfWeek(node.getString("dayOfWeek"));
		_value = node.getString("value");
		_color = node.getString("color");
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyDay clone = new LyDay();

		clone.initFromModel(this);
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of type.
	 * @return value of type.
	 */
	public Type getType()
	{
		return _type;
	}

	/**
	 * Gets value of dayOfWeek.
	 * @return value of dayOfWeek.
	 */
	public DayOfWeek getDayOfWeek()
	{
		return _dayOfWeek;
	}

	/**
	 * Gets value of value.
	 * @return value of value.
	 */
	public String getValue()
	{
		return _value;
	}

	/**
	 * Gets value of color.
	 * @return value of color.
	 */
	public String getColor()
	{
		return _color;
	}

	/**
	 * Checks whether the color attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultColor()
	{
		return (_color == getStringValue("DAY.color"));
	}


	/* Modifiers */

	/**
	 * Sets attribute type to the given value.
	 * @param type New value of type.
	 * @exception IllegalArgumentException if type parameter is null.
	 */
	public void setType(Type type)
	{
		if (type == _type)
			return;
		if (type == null)
			throw new IllegalArgumentException("BF AppEngine : invalid value for attribute type");

		_type = type;
		if (_internalView != null)
			((LyGraphicalDay)_internalView).setType(type);
	}

	/**
	 * Sets attribute dayOfWeek to the given value.
	 * @param dayOfWeek New value of dayOfWeek.
	 * @exception IllegalArgumentException if dayOfWeek parameter is null.
	 */
	public void setDayOfWeek(DayOfWeek dayOfWeek)
	{
		if (dayOfWeek == _dayOfWeek)
			return;
		if (dayOfWeek == null)
			throw new IllegalArgumentException("BF AppEngine : invalid value for attribute dayOfWeek");

		_dayOfWeek = dayOfWeek;
		if (_internalView != null)
			((LyGraphicalDay)_internalView).setDayOfWeek(dayOfWeek);
	}

	/**
	 * Sets attribute value to the given value.
	 * @param value New value of value.
	 */
	public void setValue(String value)
	{
		if ((value == _value)
			|| ((value != null)
				&& (_value != null)
				&& value.equals(_value)))
			return;

		_value = value;
		if (_internalView != null)
			((LyGraphicalDay)_internalView).setValue(value);
	}

	/**
	 * Sets attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color)
	{
		if ((color == _color)
			|| ((color != null)
				&& (_color != null)
				&& color.equals(_color)))
			return;

		_color = color;
		if (_internalView != null)
			((LyGraphicalDay)_internalView).setColor(color);
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<DAY");
		if (_type == Type.DAY_OF_YEAR)
			pw.print(" type='DAY_OF_YEAR'");
		else if (_type == Type.DAY)
			pw.print(" type='DAY'");
		if (_dayOfWeek == DayOfWeek.MONDAY)
			pw.print(" dayOfWeek='MONDAY'");
		else if (_dayOfWeek == DayOfWeek.TUESDAY)
			pw.print(" dayOfWeek='TUESDAY'");
		else if (_dayOfWeek == DayOfWeek.WEDNESDAY)
			pw.print(" dayOfWeek='WEDNESDAY'");
		else if (_dayOfWeek == DayOfWeek.THURSDAY)
			pw.print(" dayOfWeek='THURSDAY'");
		else if (_dayOfWeek == DayOfWeek.FRIDAY)
			pw.print(" dayOfWeek='FRIDAY'");
		else if (_dayOfWeek == DayOfWeek.SATURDAY)
			pw.print(" dayOfWeek='SATURDAY'");
		if (_value != null)
			pw.print(" value='" + LyString.getXmlString(_value) + "'");
		if (_color != getStringValue("DAY.color", "$ALL.color"))
			pw.print(" color='" + LyString.getXmlString(_color) + "'");
		pw.println("/>");
	}
}
