/*
 * LyDictionaryBase.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;
import java.io.*;
import leon.view.getinterface.*;
import leon.view.setinterface.*;
import leon.misc.LyString;


/**
 * The LyDictionaryBase class is a generated Java implementation that corresponds to the element 
 * DICTIONARY of the descriptive grammar of the views (views.dtd).
 * <B>&lt;DICTIONARY&gt; : Input Field of Dictionary Type</B>
 * An input field of dictionary type allows displaying a graphical component 
 * containing a combo with a tree.
 * @see leon.view.LyFieldItem
 * @see leon.view.LyLyDictionary
 * See also tag <A href="../../conf/dtd/view/DICTIONARY.html">DICTIONARY</A> of grammar leon_views.dtd.
 */
public abstract class LyDictionaryBase extends LyFieldItem
	implements  LyGetIdInterface
{
	public static final String			__VERSION = "$Id$";


	/** Possible values for attribute 'selectionType'. */
	public enum SelectionType
	{
		ALL,
		LEAVES_ONLY;
	}


	/* Static methods */

	/**
	 * Utility method to get selectionType value from a string.
	 * @param token Token string
	 * @return SelectionType value corresponding to this token
	 */
	public static final SelectionType getSelectionType(String token)
	{
		if (token == null)
			return null;
		else if (token.equalsIgnoreCase("ALL")
				|| token.equals("0"))
			return SelectionType.ALL;
		else if (token.equalsIgnoreCase("LEAVES_ONLY")
				|| token.equals("1"))
			return SelectionType.LEAVES_ONLY;
		else
			return SelectionType.LEAVES_ONLY;
	}


	/* Attributes */

	/**
	 * Identifier of the component.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyDictionary			_template;

	/**
	 * Indicates whether the dictionary is multiple or not. If it is multiple, 
	 * checkboxes are displayed with each leaf.
	 */
	protected boolean			_multiple;

	/**
	 * Allows specifying the name of a Java class that replaces the default graphical 
	 * component.
	 */
	protected String			_bean;

	/**
	 * Gives the number of columns of the dictionary (given in number of characters).
	 */
	protected int				_nCols;

	/**
	 * Gives the number of lines on which the hyperlinks are shown.
	 */
	protected int				_nRows;

	/**
	 * Selection typr of the nodes of the dictionary. The following types are managed:
	 * ALL All the nodes are selectable
	 * LEAVES_ONLY Only leaf nodes are selectable
	 */
	protected SelectionType			_selectionType;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	/**
	 * DICTIONARY.imageSelector
	 */
	protected String			_imageSelector;

	/**
	 * DICTIONARY.enabledColor
	 */
	protected String			_enabledColor;

	/**
	 * DICTIONARY.disabledColor
	 */
	protected String			_disabledColor;


	/* Sub components */

	/**
	 * Root entry of the dictionary.
	 */
	protected LyEntry			_entry;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_id = null;
		_template = null;
		_multiple = getBooleanValue("DICTIONARY.multiple", false);
		_bean = null;
		_nCols = getIntValue("DICTIONARY.nCols", -1);
		_nRows = getIntValue("DICTIONARY.nRows", -1);
		_selectionType = SelectionType.LEAVES_ONLY;
		_font = getStringValue("DICTIONARY.font", "$ALL.font");
		_color = getStringValue("DICTIONARY.color", "$ALL.color");
		_fontColor = getStringValue("DICTIONARY.fontColor", "$ALL.fontColor");
		_imageSelector = getStringValue("DICTIONARY.imageSelector", "$ALL.imageSelector");
		_enabledColor = getStringValue("DICTIONARY.enabledColor", "$ALL.enabledColor");
		_disabledColor = getStringValue("DICTIONARY.disabledColor", "$ALL.disabledColor");

		// Initialize sub components
		_entry = null;
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyDictionary	model = (LyDictionary)component;

		_model = component;
		_multiple = model._multiple;
		_bean = model._bean;
		_nCols = model._nCols;
		_nRows = model._nRows;
		_selectionType = model._selectionType;
		_font = model._font;
		_color = model._color;
		_fontColor = model._fontColor;
		_imageSelector = model._imageSelector;
		_enabledColor = model._enabledColor;
		_disabledColor = model._disabledColor;

		if (REWRITING_MODE)
			return;

		// Copy sub-components
		if (model._entry != null)
			addChild(model._entry.cloneComponent());
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	public LyDictionaryBase()
	{
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyDictionaryBase(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_id = node.getString("id");
		_template = (LyDictionary)node.getComponent("template");

		if (_template != null)
			initFromModel(_template);

		if (node.isSpecified("multiple"))
			_multiple = node.getBoolean("multiple");
		if (node.isSpecified("bean"))
			_bean = node.getString("bean");
		if (node.isSpecified("nCols"))
			_nCols = node.getInt("nCols");
		if (node.isSpecified("nRows"))
			_nRows = node.getInt("nRows");
		if (node.isSpecified("selectionType"))
			_selectionType = getSelectionType(node.getString("selectionType"));
		if (node.isSpecified("font"))
			_font = node.getString("font");
		if (node.isSpecified("color"))
			_color = node.getString("color");
		if (node.isSpecified("fontColor"))
			_fontColor = node.getString("fontColor");
		if (node.isSpecified("imageSelector"))
			_imageSelector = node.getString("imageSelector");
		if (node.isSpecified("enabledColor"))
			_enabledColor = node.getString("enabledColor");
		if (node.isSpecified("disabledColor"))
			_disabledColor = node.getString("disabledColor");
	}

	/**
	 * End of construction : completion with child component.
	 * @param child Sub child component.
	 * @see leon.view.LyComponent#addChild
	 */
	protected void addChild(LyComponent child)
	{
		// Add sub-components
		if (child instanceof LyEntry)
			setEntry((LyEntry)child);
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyDictionary clone = new LyDictionary();

		clone.initFromModel(this);
		clone._template = (LyDictionary)this;
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of id.
	 * @return value of id.
	 */
	public String getId()
	{
		return _id;
	}

	/**
	 * Gets value of template.
	 * @return value of template.
	 */
	public LyDictionary getTemplate()
	{
		return _template;
	}

	/**
	 * Checks if value of attribute multiple is set.
	 * @return true if multiple is set, false otherwise.
	 */
	public boolean isMultiple()
	{
		return _multiple;
	}

	/**
	 * Gets value of bean.
	 * @return value of bean.
	 */
	public String getBean()
	{
		return _bean;
	}

	/**
	 * Gets value of nCols.
	 * @return value of nCols.
	 */
	public int getNCols()
	{
		return _nCols;
	}

	/**
	 * Gets value of nRows.
	 * @return value of nRows.
	 */
	public int getNRows()
	{
		return _nRows;
	}

	/**
	 * Gets value of selectionType.
	 * @return value of selectionType.
	 */
	public SelectionType getSelectionType()
	{
		return _selectionType;
	}

	/**
	 * Gets value of font.
	 * @return value of font.
	 */
	public String getFont()
	{
		return _font;
	}

	/**
	 * Checks whether the font attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFont()
	{
		return (_font == getStringValue("DICTIONARY.font"));
	}

	/**
	 * Gets value of color.
	 * @return value of color.
	 */
	public String getColor()
	{
		return _color;
	}

	/**
	 * Checks whether the color attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultColor()
	{
		return (_color == getStringValue("DICTIONARY.color"));
	}

	/**
	 * Gets value of fontColor.
	 * @return value of fontColor.
	 */
	public String getFontColor()
	{
		return _fontColor;
	}

	/**
	 * Checks whether the fontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFontColor()
	{
		return (_fontColor == getStringValue("DICTIONARY.fontColor"));
	}

	/**
	 * Gets value of imageSelector.
	 * @return value of imageSelector.
	 */
	public String getImageSelector()
	{
		return _imageSelector;
	}

	/**
	 * Checks whether the imageSelector attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultImageSelector()
	{
		return (_imageSelector == getStringValue("DICTIONARY.imageSelector"));
	}

	/**
	 * Gets value of enabledColor.
	 * @return value of enabledColor.
	 */
	public String getEnabledColor()
	{
		return _enabledColor;
	}

	/**
	 * Checks whether the enabledColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultEnabledColor()
	{
		return (_enabledColor == getStringValue("DICTIONARY.enabledColor"));
	}

	/**
	 * Gets value of disabledColor.
	 * @return value of disabledColor.
	 */
	public String getDisabledColor()
	{
		return _disabledColor;
	}

	/**
	 * Checks whether the disabledColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultDisabledColor()
	{
		return (_disabledColor == getStringValue("DICTIONARY.disabledColor"));
	}

	/**
	 * Get sub-component Entry.
	 * @return sub-component Entry.
	 */
	public LyEntry getEntry()
	{
		return _entry;
	}


	/* Modifiers */

	/**
	 * Initializes attribute id to the given value.
	 * @param id New value of id.
	 */
	public void setId(String id)
	{
		if ((id == _id)
			|| ((id != null)
				&& (_id != null)
				&& id.equals(_id)))
			return;

		_id = (id != null ? id.intern() : null);
	}

	/**
	 * Initializes attribute template to the given value.
	 * @param template New value of template.
	 */
	public void setTemplate(LyDictionary template)
	{
		if (template == null)
			return;
		if (_template != null)
			throw new IllegalStateException("BF AppEngine : attribute template was already set.");

		initFromModel(template);
		_template = template;
	}

	/**
	 * Initializes attribute multiple to the given value.
	 * @param multiple Boolean indicating if attribute multiple is set or unset.
	 */
	public void setMultiple(boolean multiple)
	{
		if (multiple == _multiple)
			return;
		_multiple = multiple;
	}

	/**
	 * Initializes attribute bean to the given value.
	 * @param bean New value of bean.
	 */
	public void setBean(String bean)
	{
		if ((bean == _bean)
			|| ((bean != null)
				&& (_bean != null)
				&& bean.equals(_bean)))
			return;

		_bean = bean;
	}

	/**
	 * Initializes attribute nCols to the given value.
	 * @param nCols New value of nCols.
	 */
	public void setNCols(int nCols)
	{
		if (nCols == _nCols)
			return;
		_nCols = nCols;
	}

	/**
	 * Initializes attribute nRows to the given value.
	 * @param nRows New value of nRows.
	 */
	public void setNRows(int nRows)
	{
		if (nRows == _nRows)
			return;
		_nRows = nRows;
	}

	/**
	 * Initializes attribute selectionType to the given value.
	 * @param selectionType New value of selectionType.
	 * @exception IllegalArgumentException if selectionType parameter is null.
	 */
	public void setSelectionType(SelectionType selectionType)
	{
		if (selectionType == _selectionType)
			return;
		if (selectionType == null)
			throw new IllegalArgumentException("BF AppEngine : invalid value for attribute selectionType");

		_selectionType = selectionType;
	}

	/**
	 * Initializes attribute font to the given value.
	 * @param font New value of font.
	 */
	public void setFont(String font)
	{
		if ((font == _font)
			|| ((font != null)
				&& (_font != null)
				&& font.equals(_font)))
			return;

		_font = font;
	}

	/**
	 * Initializes attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color)
	{
		if ((color == _color)
			|| ((color != null)
				&& (_color != null)
				&& color.equals(_color)))
			return;

		_color = color;
	}

	/**
	 * Initializes attribute fontColor to the given value.
	 * @param fontColor New value of fontColor.
	 */
	public void setFontColor(String fontColor)
	{
		if ((fontColor == _fontColor)
			|| ((fontColor != null)
				&& (_fontColor != null)
				&& fontColor.equals(_fontColor)))
			return;

		_fontColor = fontColor;
	}

	/**
	 * Initializes attribute imageSelector to the given value.
	 * @param imageSelector New value of imageSelector.
	 */
	public void setImageSelector(String imageSelector)
	{
		if ((imageSelector == _imageSelector)
			|| ((imageSelector != null)
				&& (_imageSelector != null)
				&& imageSelector.equals(_imageSelector)))
			return;

		_imageSelector = imageSelector;
	}

	/**
	 * Initializes attribute enabledColor to the given value.
	 * @param enabledColor New value of enabledColor.
	 */
	public void setEnabledColor(String enabledColor)
	{
		if ((enabledColor == _enabledColor)
			|| ((enabledColor != null)
				&& (_enabledColor != null)
				&& enabledColor.equals(_enabledColor)))
			return;

		_enabledColor = enabledColor;
	}

	/**
	 * Initializes attribute disabledColor to the given value.
	 * @param disabledColor New value of disabledColor.
	 */
	public void setDisabledColor(String disabledColor)
	{
		if ((disabledColor == _disabledColor)
			|| ((disabledColor != null)
				&& (_disabledColor != null)
				&& disabledColor.equals(_disabledColor)))
			return;

		_disabledColor = disabledColor;
	}

	/**
	 * Sets sub-component entry.
	 * @param entry New value of entry
	 */
	public void setEntry(LyEntry entry)
	{
		if (_entry != null)
			_entry.setParent(null);
		_entry = entry;
		if (entry != null)
			entry.setParent(this);
		if (_internalView instanceof LySetEntryInterface)
			((LySetEntryInterface)_internalView).setEntry(entry);
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<DICTIONARY");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && _multiple)
				|| ((_template != null) && (_multiple != _template._multiple)))
			pw.print(" multiple='" + _multiple + "'");
		if (((_template == null) && (_bean != null))
				|| ((_template != null) && (_bean != _template._bean)))
			pw.print(" bean='" + LyString.getXmlString(_bean) + "'");
		if (((_template == null) && (_nCols != getIntValue("DICTIONARY.nCols", -1)))
				|| ((_template != null) && (_nCols != _template._nCols)))
			pw.print(" nCols='" + _nCols + "'");
		if (((_template == null) && (_nRows != getIntValue("DICTIONARY.nRows", -1)))
				|| ((_template != null) && (_nRows != _template._nRows)))
			pw.print(" nRows='" + _nRows + "'");
		if ((_template == null)
			|| (_selectionType != _template._selectionType))
		{
			if (_selectionType == SelectionType.ALL)
				pw.print(" selectionType='ALL'");
			else if ((_selectionType == SelectionType.LEAVES_ONLY) && (_template != null))
				pw.print(" selectionType='LEAVES_ONLY'");
		}
		if (((_template == null) && (_font != getStringValue("DICTIONARY.font", "$ALL.font")))
				|| ((_template != null) && (_font != _template._font)))
			pw.print(" font='" + LyString.getXmlString(_font) + "'");
		if (((_template == null) && (_color != getStringValue("DICTIONARY.color", "$ALL.color")))
				|| ((_template != null) && (_color != _template._color)))
			pw.print(" color='" + LyString.getXmlString(_color) + "'");
		if (((_template == null) && (_fontColor != getStringValue("DICTIONARY.fontColor", "$ALL.fontColor")))
				|| ((_template != null) && (_fontColor != _template._fontColor)))
			pw.print(" fontColor='" + LyString.getXmlString(_fontColor) + "'");
		if (((_template == null) && (_imageSelector != getStringValue("DICTIONARY.imageSelector", "$ALL.imageSelector")))
				|| ((_template != null) && (_imageSelector != _template._imageSelector)))
			pw.print(" imageSelector='" + LyString.getXmlString(_imageSelector) + "'");
		if (((_template == null) && (_enabledColor != getStringValue("DICTIONARY.enabledColor", "$ALL.enabledColor")))
				|| ((_template != null) && (_enabledColor != _template._enabledColor)))
			pw.print(" enabledColor='" + LyString.getXmlString(_enabledColor) + "'");
		if (((_template == null) && (_disabledColor != getStringValue("DICTIONARY.disabledColor", "$ALL.disabledColor")))
				|| ((_template != null) && (_disabledColor != _template._disabledColor)))
			pw.print(" disabledColor='" + LyString.getXmlString(_disabledColor) + "'");

		if ((getChildCount() == 0) && (_datas == null))
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		if (_entry != null)
			_entry.writeXml(pw, debug);

		pw.println("</DICTIONARY>");
	}
}
