/*
 * LyEntry.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;

import java.util.*;
import java.io.*;
import leon.view.graphic.*;
import leon.view.getinterface.*;
import leon.view.setinterface.*;
import leon.misc.LyString;


/**
 * The LyEntry class is a generated Java implementation that corresponds to the element 
 * ENTRY of the descriptive grammar of the views (views.dtd).
 * <B>&lt;ENTRY&gt; : Dictionary Entry</B>
 * An entry represents a node or a leaf of a dictionary. An entry may contain 
 * several sub-entries.
 * @see leon.view.LyComponent
 * See also tag <A href="../../conf/dtd/view/ENTRY.html">ENTRY</A> of grammar leon_views.dtd.
 */
public class LyEntry extends LyComponent
	implements  LyGetIdInterface,
				LyGetLabelInterface,
				LySetValidityInterface,
				LySetImageInterface
{
	public static final String			__VERSION = "$Id$";


	/** Possible values for attribute 'validity'. */


	/* Attributes */

	/**
	 * Identifier of the entry. It represents the internal value of the application 
	 * associated to the entry.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyEntry			_template;

	/**
	 * Label of the entry.
	 */
	protected String			_label;

	/**
	 * Indicates whether the box representing the entry is checked or not.
	 */
	protected boolean			_checked;

	/**
	 * Indicates whether the entry is open or not (only if it is a node).
	 */
	protected boolean			_open;

	/**
	 * Indicates if the entry is valid, invalid or partially valid. By default the 
	 * entry is valid.
	 */
	protected LyTreeNode.Validity			_validity;

	/**
	 * Image of the entry.
	 */
	protected String			_image;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	/**
	 * ENTRY.selectedBackgroundColor
	 */
	protected String			_selectedBackgroundColor;

	/**
	 * ENTRY.selectedFontColor
	 */
	protected String			_selectedFontColor;

	/**
	 * ENTRY.inactiveSelectedBackgroundColor
	 */
	protected String			_inactiveSelectedBackgroundColor;

	/**
	 * ENTRY.emptyCheckboxImage
	 */
	protected String			_emptyCheckboxImage;

	/**
	 * ENTRY.selectedCheckboxImage
	 */
	protected String			_selectedCheckboxImage;


	/* Sub components */

	/**
	 * Sub-entries.
	 */
	protected ArrayList<LyEntry>		_entries;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_id = null;
		_template = null;
		_label = null;
		_checked = getBooleanValue("ENTRY.checked", false);
		_open = getBooleanValue("ENTRY.open", false);
		_validity = LyTreeNode.Validity.VALID;
		_image = getStringValue("ENTRY.image", "$ALL.image");
		_font = getStringValue("ENTRY.font", "$ALL.font");
		_color = getStringValue("ENTRY.color", "$ALL.color");
		_fontColor = getStringValue("ENTRY.fontColor", "$ALL.fontColor");
		_selectedBackgroundColor = getStringValue("ENTRY.selectedBackgroundColor", "$ALL.selectedBackgroundColor");
		_selectedFontColor = getStringValue("ENTRY.selectedFontColor", "$ALL.selectedFontColor");
		_inactiveSelectedBackgroundColor = getStringValue("ENTRY.inactiveSelectedBackgroundColor", "$ALL.inactiveSelectedBackgroundColor");
		_emptyCheckboxImage = getStringValue("ENTRY.emptyCheckboxImage", "$ALL.emptyCheckboxImage");
		_selectedCheckboxImage = getStringValue("ENTRY.selectedCheckboxImage", "$ALL.selectedCheckboxImage");

		// Initialize sub components
		_entries = null;
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyEntry	model = (LyEntry)component;

		_model = component;
		_label = model._label;
		_checked = model._checked;
		_open = model._open;
		_validity = model._validity;
		_image = model._image;
		_font = model._font;
		_color = model._color;
		_fontColor = model._fontColor;
		_selectedBackgroundColor = model._selectedBackgroundColor;
		_selectedFontColor = model._selectedFontColor;
		_inactiveSelectedBackgroundColor = model._inactiveSelectedBackgroundColor;
		_emptyCheckboxImage = model._emptyCheckboxImage;
		_selectedCheckboxImage = model._selectedCheckboxImage;

		if (REWRITING_MODE)
			return;

		// Copy sub-components
		LyEntry	entries;
		int			i, n;

		n = (model._entries == null ? 0 : model._entries.size());
		for (i = 0; i < n; i++)
		{
			entries = model._entries.get(i);
			if (entries != null)
				entries = (LyEntry)entries.cloneComponent();
			addChild(entries);
		}
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	protected LyEntry()
	{
		_id = getInternalId();
	}

	/**
	 * Constructor with all required attributes.
	 * @param id Identifier of the entry. It represents the internal value of the application associated to the entry.
	 * @param label Label of the entry.
	 */
	public LyEntry(String id, String label)
	{
		// Fill up required attributes with parameters
		_id = (id != null ? id.intern() : null);
		_label = label;
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyEntry(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_id = node.getString("id");
		_template = (LyEntry)node.getComponent("template");

		if (_template != null)
			initFromModel(_template);

		if (node.isSpecified("label"))
			_label = node.getString("label");
		if (node.isSpecified("checked"))
			_checked = node.getBoolean("checked");
		if (node.isSpecified("open"))
			_open = node.getBoolean("open");
		if (node.isSpecified("validity"))
			_validity = LyTreeNode.getValidity(node.getString("validity"));
		if (node.isSpecified("image"))
			_image = node.getString("image");
		if (node.isSpecified("font"))
			_font = node.getString("font");
		if (node.isSpecified("color"))
			_color = node.getString("color");
		if (node.isSpecified("fontColor"))
			_fontColor = node.getString("fontColor");
		if (node.isSpecified("selectedBackgroundColor"))
			_selectedBackgroundColor = node.getString("selectedBackgroundColor");
		if (node.isSpecified("selectedFontColor"))
			_selectedFontColor = node.getString("selectedFontColor");
		if (node.isSpecified("inactiveSelectedBackgroundColor"))
			_inactiveSelectedBackgroundColor = node.getString("inactiveSelectedBackgroundColor");
		if (node.isSpecified("emptyCheckboxImage"))
			_emptyCheckboxImage = node.getString("emptyCheckboxImage");
		if (node.isSpecified("selectedCheckboxImage"))
			_selectedCheckboxImage = node.getString("selectedCheckboxImage");
	}

	/**
	 * End of construction : completion with child component.
	 * @param child Sub child component.
	 * @see leon.view.LyComponent#addChild
	 */
	protected void addChild(LyComponent child)
	{
		// Add sub-components
		if (child instanceof LyEntry)
			addEntry((LyEntry)child);
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyEntry clone = new LyEntry();

		clone.initFromModel(this);
		clone._template = this;
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of id.
	 * @return value of id.
	 */
	public String getId()
	{
		return _id;
	}

	/**
	 * Gets value of template.
	 * @return value of template.
	 */
	public LyEntry getTemplate()
	{
		return _template;
	}

	/**
	 * Gets value of label.
	 * @return value of label.
	 */
	public String getLabel()
	{
		return _label;
	}

	/**
	 * Checks if value of attribute checked is set.
	 * @return true if checked is set, false otherwise.
	 */
	public boolean isChecked()
	{
		return _checked;
	}

	/**
	 * Checks if value of attribute open is set.
	 * @return true if open is set, false otherwise.
	 */
	public boolean isOpen()
	{
		return _open;
	}

	/**
	 * Gets value of validity.
	 * @return value of validity.
	 */
	public LyTreeNode.Validity getValidity()
	{
		return _validity;
	}

	/**
	 * Gets value of image.
	 * @return value of image.
	 */
	public String getImage()
	{
		return _image;
	}

	/**
	 * Checks whether the image attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultImage()
	{
		return (_image == getStringValue("ENTRY.image"));
	}

	/**
	 * Gets value of font.
	 * @return value of font.
	 */
	public String getFont()
	{
		return _font;
	}

	/**
	 * Checks whether the font attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFont()
	{
		return (_font == getStringValue("ENTRY.font"));
	}

	/**
	 * Gets value of color.
	 * @return value of color.
	 */
	public String getColor()
	{
		return _color;
	}

	/**
	 * Checks whether the color attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultColor()
	{
		return (_color == getStringValue("ENTRY.color"));
	}

	/**
	 * Gets value of fontColor.
	 * @return value of fontColor.
	 */
	public String getFontColor()
	{
		return _fontColor;
	}

	/**
	 * Checks whether the fontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFontColor()
	{
		return (_fontColor == getStringValue("ENTRY.fontColor"));
	}

	/**
	 * Gets value of selectedBackgroundColor.
	 * @return value of selectedBackgroundColor.
	 */
	public String getSelectedBackgroundColor()
	{
		return _selectedBackgroundColor;
	}

	/**
	 * Checks whether the selectedBackgroundColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultSelectedBackgroundColor()
	{
		return (_selectedBackgroundColor == getStringValue("ENTRY.selectedBackgroundColor"));
	}

	/**
	 * Gets value of selectedFontColor.
	 * @return value of selectedFontColor.
	 */
	public String getSelectedFontColor()
	{
		return _selectedFontColor;
	}

	/**
	 * Checks whether the selectedFontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultSelectedFontColor()
	{
		return (_selectedFontColor == getStringValue("ENTRY.selectedFontColor"));
	}

	/**
	 * Gets value of inactiveSelectedBackgroundColor.
	 * @return value of inactiveSelectedBackgroundColor.
	 */
	public String getInactiveSelectedBackgroundColor()
	{
		return _inactiveSelectedBackgroundColor;
	}

	/**
	 * Checks whether the inactiveSelectedBackgroundColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultInactiveSelectedBackgroundColor()
	{
		return (_inactiveSelectedBackgroundColor == getStringValue("ENTRY.inactiveSelectedBackgroundColor"));
	}

	/**
	 * Gets value of emptyCheckboxImage.
	 * @return value of emptyCheckboxImage.
	 */
	public String getEmptyCheckboxImage()
	{
		return _emptyCheckboxImage;
	}

	/**
	 * Checks whether the emptyCheckboxImage attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultEmptyCheckboxImage()
	{
		return (_emptyCheckboxImage == getStringValue("ENTRY.emptyCheckboxImage"));
	}

	/**
	 * Gets value of selectedCheckboxImage.
	 * @return value of selectedCheckboxImage.
	 */
	public String getSelectedCheckboxImage()
	{
		return _selectedCheckboxImage;
	}

	/**
	 * Checks whether the selectedCheckboxImage attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultSelectedCheckboxImage()
	{
		return (_selectedCheckboxImage == getStringValue("ENTRY.selectedCheckboxImage"));
	}

	/**
	 * Get sub-component Entry of index i.
	 * @param i the index of the Entry to get.
	 * @return sub-component Entry of index i.
	 */
	public LyEntry getEntry(int i)
	{
		if (_entries == null)
			return null;
		if ((i < 0) || (i >= _entries.size()))
			return null;
		return _entries.get(i);
	}

	/**
	 * Get sub-component Entry count.
	 * @return sub-component Entry count.
	 */
	public int getEntryCount()
	{
		if (_entries == null)
			return 0;
		return _entries.size();
	}


	/* Modifiers */

	/**
	 * Initializes attribute template to the given value.
	 * @param template New value of template.
	 */
	public void setTemplate(LyEntry template)
	{
		if (template == null)
			return;
		if (_template != null)
			throw new IllegalStateException("BF AppEngine : attribute template was already set.");

		initFromModel(template);
		_template = template;
	}

	/**
	 * Initializes attribute label to the given value.
	 * @param label New value of label.
	 * @exception IllegalArgumentException if label is null.
	 */
	public void setLabel(String label)
	{
		if (label == null)
			throw new IllegalArgumentException("BF AppEngine : attribute label is required (can't be null)");

		if ((label == _label)
			|| ((label != null)
				&& (_label != null)
				&& label.equals(_label)))
			return;

		_label = label;
	}

	/**
	 * Sets attribute checked to the given value.
	 * @param checked Boolean indicating if attribute checked is set or unset.
	 */
	public void setChecked(boolean checked)
	{
		if (checked == _checked)
			return;
		_checked = checked;
		if (_internalView != null)
			((LyGraphicalEntry)_internalView).setChecked(checked);
	}

	/**
	 * Sets attribute open to the given value.
	 * @param open Boolean indicating if attribute open is set or unset.
	 */
	public void setOpen(boolean open)
	{
		if (open == _open)
			return;
		_open = open;
		if (_internalView != null)
			((LyGraphicalEntry)_internalView).setOpen(open);
	}

	/**
	 * Sets attribute validity to the given value.
	 * @param validity New value of validity.
	 * @exception IllegalArgumentException if validity parameter is null.
	 */
	public void setValidity(LyTreeNode.Validity validity)
	{
		if (validity == _validity)
			return;
		if (validity == null)
			throw new IllegalArgumentException("BF AppEngine : invalid value for attribute validity");

		_validity = validity;
		if (_internalView != null)
			((LyGraphicalEntry)_internalView).setValidity(validity);
	}

	/**
	 * Sets attribute image to the given value.
	 * @param image New value of image.
	 */
	public void setImage(String image)
	{
		if ((image == _image)
			|| ((image != null)
				&& (_image != null)
				&& image.equals(_image)))
			return;

		_image = image;
		if (_internalView != null)
			((LyGraphicalEntry)_internalView).setImage(image);
	}

	/**
	 * Initializes attribute font to the given value.
	 * @param font New value of font.
	 */
	public void setFont(String font)
	{
		if ((font == _font)
			|| ((font != null)
				&& (_font != null)
				&& font.equals(_font)))
			return;

		_font = font;
	}

	/**
	 * Initializes attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color)
	{
		if ((color == _color)
			|| ((color != null)
				&& (_color != null)
				&& color.equals(_color)))
			return;

		_color = color;
	}

	/**
	 * Initializes attribute fontColor to the given value.
	 * @param fontColor New value of fontColor.
	 */
	public void setFontColor(String fontColor)
	{
		if ((fontColor == _fontColor)
			|| ((fontColor != null)
				&& (_fontColor != null)
				&& fontColor.equals(_fontColor)))
			return;

		_fontColor = fontColor;
	}

	/**
	 * Initializes attribute selectedBackgroundColor to the given value.
	 * @param selectedBackgroundColor New value of selectedBackgroundColor.
	 */
	public void setSelectedBackgroundColor(String selectedBackgroundColor)
	{
		if ((selectedBackgroundColor == _selectedBackgroundColor)
			|| ((selectedBackgroundColor != null)
				&& (_selectedBackgroundColor != null)
				&& selectedBackgroundColor.equals(_selectedBackgroundColor)))
			return;

		_selectedBackgroundColor = selectedBackgroundColor;
	}

	/**
	 * Initializes attribute selectedFontColor to the given value.
	 * @param selectedFontColor New value of selectedFontColor.
	 */
	public void setSelectedFontColor(String selectedFontColor)
	{
		if ((selectedFontColor == _selectedFontColor)
			|| ((selectedFontColor != null)
				&& (_selectedFontColor != null)
				&& selectedFontColor.equals(_selectedFontColor)))
			return;

		_selectedFontColor = selectedFontColor;
	}

	/**
	 * Initializes attribute inactiveSelectedBackgroundColor to the given value.
	 * @param inactiveSelectedBackgroundColor New value of inactiveSelectedBackgroundColor.
	 */
	public void setInactiveSelectedBackgroundColor(String inactiveSelectedBackgroundColor)
	{
		if ((inactiveSelectedBackgroundColor == _inactiveSelectedBackgroundColor)
			|| ((inactiveSelectedBackgroundColor != null)
				&& (_inactiveSelectedBackgroundColor != null)
				&& inactiveSelectedBackgroundColor.equals(_inactiveSelectedBackgroundColor)))
			return;

		_inactiveSelectedBackgroundColor = inactiveSelectedBackgroundColor;
	}

	/**
	 * Initializes attribute emptyCheckboxImage to the given value.
	 * @param emptyCheckboxImage New value of emptyCheckboxImage.
	 */
	public void setEmptyCheckboxImage(String emptyCheckboxImage)
	{
		if ((emptyCheckboxImage == _emptyCheckboxImage)
			|| ((emptyCheckboxImage != null)
				&& (_emptyCheckboxImage != null)
				&& emptyCheckboxImage.equals(_emptyCheckboxImage)))
			return;

		_emptyCheckboxImage = emptyCheckboxImage;
	}

	/**
	 * Initializes attribute selectedCheckboxImage to the given value.
	 * @param selectedCheckboxImage New value of selectedCheckboxImage.
	 */
	public void setSelectedCheckboxImage(String selectedCheckboxImage)
	{
		if ((selectedCheckboxImage == _selectedCheckboxImage)
			|| ((selectedCheckboxImage != null)
				&& (_selectedCheckboxImage != null)
				&& selectedCheckboxImage.equals(_selectedCheckboxImage)))
			return;

		_selectedCheckboxImage = selectedCheckboxImage;
	}

	/**
	 * Adds a sub-component entry.
	 * @param entry The entry to add to entries list.
	 */
	public void addEntry(LyEntry entry)
	{
		insertEntry(entry, -1);
	}

	/**
	 * Inserts a sub-component entry at given position.
	 * @param entry The entry to add to entries list.
	 * @param pos	Position where the sub-component is inserted.
	 */
	public void insertEntry(LyEntry entry, int pos)
	{
		if (_entries == null)
		{
			_entries = new ArrayList<LyEntry>(1);
			_children = _entries;
		}
		else if (_entries.contains(entry))
			return;
		if (pos >= 0)
			_entries.add(pos,entry);
		if (entry != null)
			entry.setParent(this);
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<ENTRY");
		pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		pw.print(" label='" + LyString.getXmlString(_label) + "'");
		if (((_template == null) && _checked)
				|| ((_template != null) && (_checked != _template._checked)))
			pw.print(" checked='" + _checked + "'");
		if (((_template == null) && _open)
				|| ((_template != null) && (_open != _template._open)))
			pw.print(" open='" + _open + "'");
		if ((_template == null)
			|| (_validity != _template._validity))
		{
			if ((_validity == LyTreeNode.Validity.VALID) && (_template != null))
				pw.print(" validity='VALID'");
			else if (_validity == LyTreeNode.Validity.INVALID)
				pw.print(" validity='INVALID'");
			else if (_validity == LyTreeNode.Validity.PARTIALLY)
				pw.print(" validity='PARTIALLY'");
		}
		if (((_template == null) && (_image != getStringValue("ENTRY.image", "$ALL.image")))
				|| ((_template != null) && (_image != _template._image)))
			pw.print(" image='" + LyString.getXmlString(_image) + "'");
		if (((_template == null) && (_font != getStringValue("ENTRY.font", "$ALL.font")))
				|| ((_template != null) && (_font != _template._font)))
			pw.print(" font='" + LyString.getXmlString(_font) + "'");
		if (((_template == null) && (_color != getStringValue("ENTRY.color", "$ALL.color")))
				|| ((_template != null) && (_color != _template._color)))
			pw.print(" color='" + LyString.getXmlString(_color) + "'");
		if (((_template == null) && (_fontColor != getStringValue("ENTRY.fontColor", "$ALL.fontColor")))
				|| ((_template != null) && (_fontColor != _template._fontColor)))
			pw.print(" fontColor='" + LyString.getXmlString(_fontColor) + "'");
		if (((_template == null) && (_selectedBackgroundColor != getStringValue("ENTRY.selectedBackgroundColor", "$ALL.selectedBackgroundColor")))
				|| ((_template != null) && (_selectedBackgroundColor != _template._selectedBackgroundColor)))
			pw.print(" selectedBackgroundColor='" + LyString.getXmlString(_selectedBackgroundColor) + "'");
		if (((_template == null) && (_selectedFontColor != getStringValue("ENTRY.selectedFontColor", "$ALL.selectedFontColor")))
				|| ((_template != null) && (_selectedFontColor != _template._selectedFontColor)))
			pw.print(" selectedFontColor='" + LyString.getXmlString(_selectedFontColor) + "'");
		if (((_template == null) && (_inactiveSelectedBackgroundColor != getStringValue("ENTRY.inactiveSelectedBackgroundColor", "$ALL.inactiveSelectedBackgroundColor")))
				|| ((_template != null) && (_inactiveSelectedBackgroundColor != _template._inactiveSelectedBackgroundColor)))
			pw.print(" inactiveSelectedBackgroundColor='" + LyString.getXmlString(_inactiveSelectedBackgroundColor) + "'");
		if (((_template == null) && (_emptyCheckboxImage != getStringValue("ENTRY.emptyCheckboxImage", "$ALL.emptyCheckboxImage")))
				|| ((_template != null) && (_emptyCheckboxImage != _template._emptyCheckboxImage)))
			pw.print(" emptyCheckboxImage='" + LyString.getXmlString(_emptyCheckboxImage) + "'");
		if (((_template == null) && (_selectedCheckboxImage != getStringValue("ENTRY.selectedCheckboxImage", "$ALL.selectedCheckboxImage")))
				|| ((_template != null) && (_selectedCheckboxImage != _template._selectedCheckboxImage)))
			pw.print(" selectedCheckboxImage='" + LyString.getXmlString(_selectedCheckboxImage) + "'");

		if ((getChildCount() == 0) && (_datas == null))
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		LyComponent	entries;
		int			i, n;

		n = (_entries == null ? 0 : _entries.size());
		for (i = 0; i < n; i++)
		{
			entries = _entries.get(i);
			entries.writeXml(pw, debug);
		}


		pw.println("</ENTRY>");
	}
}
