/*
 * LyFieldUnit.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;
import java.io.*;
import leon.view.getinterface.*;
import leon.misc.LyString;


/**
 * The LyFieldUnit class is a generated Java implementation that corresponds to the element 
 * UNIT of the descriptive grammar of the views (views.dtd).
 * <B>&lt;UNIT&gt; : Unit</B>
 * A unit allows adding to a field a combo for selecting a unit or a unit label.
 * @see leon.view.LyFieldItem
 * See also tag <A href="../../conf/dtd/view/UNIT.html">UNIT</A> of grammar leon_views.dtd.
 */
public class LyFieldUnit extends LyFieldItem
	implements  LyGetIdInterface,
				LyGetConstraintInterface
{
	public static final String			__VERSION = "$Id$";


	/* Attributes */

	/**
	 * Identifier of the component.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyFieldUnit			_template;


	/* Sub components */

	/**
	 * The field item of a unit is either a relation or a label.
	 */
	protected LyFieldItem		_fieldItem;

	/**
	 * Positioning constraint of the unit.
	 */
	protected LyConstraint		_constraint;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_id = null;
		_template = null;

		// Initialize sub components
		_fieldItem = null;
		_constraint = null;
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyFieldUnit	model = (LyFieldUnit)component;

		_model = component;

		if (REWRITING_MODE)
			return;

		// Copy sub-components
		if (model._fieldItem != null)
			addChild(model._fieldItem.cloneComponent());
		if (model._constraint != null)
			addChild(model._constraint.cloneComponent());
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	public LyFieldUnit()
	{
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyFieldUnit(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_id = node.getString("id");
		_template = (LyFieldUnit)node.getComponent("template");

		if (_template != null)
			initFromModel(_template);

	}

	/**
	 * End of construction : completion with child component.
	 * @param child Sub child component.
	 * @see leon.view.LyComponent#addChild
	 */
	protected void addChild(LyComponent child)
	{
		// Add sub-components
		if (child instanceof LyRelation)
			setFieldItem((LyFieldItem)child);
		else if (child instanceof LyLabel)
			setFieldItem((LyFieldItem)child);
		else if (child instanceof LyConstraint)
			setConstraint((LyConstraint)child);
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyFieldUnit clone = new LyFieldUnit();

		clone.initFromModel(this);
		clone._template = this;
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of id.
	 * @return value of id.
	 */
	public String getId()
	{
		return _id;
	}

	/**
	 * Gets value of template.
	 * @return value of template.
	 */
	public LyFieldUnit getTemplate()
	{
		return _template;
	}

	/**
	 * Get sub-component FieldItem.
	 * @return sub-component FieldItem.
	 */
	public LyFieldItem getFieldItem()
	{
		return _fieldItem;
	}

	/**
	 * Get sub-component Constraint.
	 * @return sub-component Constraint.
	 */
	public LyConstraint getConstraint()
	{
		return _constraint;
	}


	/* Modifiers */

	/**
	 * Initializes attribute id to the given value.
	 * @param id New value of id.
	 */
	public void setId(String id)
	{
		if ((id == _id)
			|| ((id != null)
				&& (_id != null)
				&& id.equals(_id)))
			return;

		_id = (id != null ? id.intern() : null);
	}

	/**
	 * Initializes attribute template to the given value.
	 * @param template New value of template.
	 */
	public void setTemplate(LyFieldUnit template)
	{
		if (template == null)
			return;
		if (_template != null)
			throw new IllegalStateException("BF AppEngine : attribute template was already set.");

		initFromModel(template);
		_template = template;
	}

	/**
	 * Sets sub-component fieldItem.
	 * @param fieldItem New value of fieldItem
	 */
	public void setFieldItem(LyFieldItem fieldItem)
	{
		if (_fieldItem != null)
			_fieldItem.setParent(null);
		_fieldItem = fieldItem;
		if (fieldItem != null)
			fieldItem.setParent(this);
	}

	/**
	 * Sets sub-component constraint.
	 * @param constraint New value of constraint
	 */
	public void setConstraint(LyConstraint constraint)
	{
		if (_constraint != null)
			_constraint.setParent(null);
		_constraint = constraint;
		if (constraint != null)
			constraint.setParent(this);
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<UNIT");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");

		if ((getChildCount() == 0) && (_datas == null))
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		if (_fieldItem != null)
			_fieldItem.writeXml(pw, debug);
		if (_constraint != null)
			_constraint.writeXml(pw, debug);

		pw.println("</UNIT>");
	}
}
