/*
 * LyFooter.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;
import java.io.*;
import leon.view.graphic.*;
import leon.view.getinterface.*;
import leon.view.setinterface.*;
import leon.misc.LyString;


/**
 * The LyFooter class is a generated Java implementation that corresponds to the element 
 * FOOTER of the descriptive grammar of the views (views.dtd).
 * <B>&lt;FOOTER&gt; : List Footer Cell</B>
 * A header is the bottom label of column in a table.
 * @see leon.view.LyComponent
 * See also tag <A href="../../conf/dtd/view/FOOTER.html">FOOTER</A> of grammar leon_views.dtd.
 */
public class LyFooter extends LyComponent
	implements  LyGetIdInterface,
				LyGetLabelInterface,
				LySetLabelInterface
{
	public static final String			__VERSION = "$Id$";


	/** Possible values for attribute 'align'. */


	/* Attributes */

	/**
	 * Identifier of the footer.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyFooter			_template;

	/**
	 * Label of the footer.
	 */
	protected String			_label;

	/**
	 * Text of the tooltip for the footer.
	 */
	protected String			_tooltip;

	/**
	 * Text alignment of the footer label (left, right or centered).
	 */
	protected LyHeader.Align			_align;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	/**
	 * Indicates if the label of the footer can be wrapped over several lines.
	 * The 'true' value indicates that the text can be wrapped.
	 */
	protected boolean			_wrap;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_id = null;
		_template = null;
		_label = null;
		_tooltip = null;
		_align = LyHeader.Align.LEFT;
		_font = getStringValue("FOOTER.font", "$ALL.font");
		_color = getStringValue("FOOTER.color", "$ALL.color");
		_fontColor = getStringValue("FOOTER.fontColor", "$ALL.fontColor");
		_wrap = getBooleanValue("FOOTER.wrap", false);
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyFooter	model = (LyFooter)component;

		_model = component;
		_label = model._label;
		_tooltip = model._tooltip;
		_align = model._align;
		_font = model._font;
		_color = model._color;
		_fontColor = model._fontColor;
		_wrap = model._wrap;
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	public LyFooter()
	{
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyFooter(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_id = node.getString("id");
		_template = (LyFooter)node.getComponent("template");

		if (_template != null)
			initFromModel(_template);

		if (node.isSpecified("label"))
			_label = node.getString("label");
		if (node.isSpecified("tooltip"))
			_tooltip = node.getString("tooltip");
		if (node.isSpecified("align"))
			_align = LyHeader.getAlign(node.getString("align"));
		if (node.isSpecified("font"))
			_font = node.getString("font");
		if (node.isSpecified("color"))
			_color = node.getString("color");
		if (node.isSpecified("fontColor"))
			_fontColor = node.getString("fontColor");
		if (node.isSpecified("wrap"))
			_wrap = node.getBoolean("wrap");
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyFooter clone = new LyFooter();

		clone.initFromModel(this);
		clone._template = this;
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of id.
	 * @return value of id.
	 */
	public String getId()
	{
		return _id;
	}

	/**
	 * Gets value of template.
	 * @return value of template.
	 */
	public LyFooter getTemplate()
	{
		return _template;
	}

	/**
	 * Gets value of label.
	 * @return value of label.
	 */
	public String getLabel()
	{
		return _label;
	}

	/**
	 * Gets value of tooltip.
	 * @return value of tooltip.
	 */
	public String getTooltip()
	{
		return _tooltip;
	}

	/**
	 * Gets value of align.
	 * @return value of align.
	 */
	public LyHeader.Align getAlign()
	{
		return _align;
	}

	/**
	 * Gets value of font.
	 * @return value of font.
	 */
	public String getFont()
	{
		return _font;
	}

	/**
	 * Checks whether the font attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFont()
	{
		return (_font == getStringValue("FOOTER.font"));
	}

	/**
	 * Gets value of color.
	 * @return value of color.
	 */
	public String getColor()
	{
		return _color;
	}

	/**
	 * Checks whether the color attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultColor()
	{
		return (_color == getStringValue("FOOTER.color"));
	}

	/**
	 * Gets value of fontColor.
	 * @return value of fontColor.
	 */
	public String getFontColor()
	{
		return _fontColor;
	}

	/**
	 * Checks whether the fontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFontColor()
	{
		return (_fontColor == getStringValue("FOOTER.fontColor"));
	}

	/**
	 * Checks if value of attribute wrap is set.
	 * @return true if wrap is set, false otherwise.
	 */
	public boolean isWrap()
	{
		return _wrap;
	}


	/* Modifiers */

	/**
	 * Initializes attribute id to the given value.
	 * @param id New value of id.
	 */
	public void setId(String id)
	{
		if ((id == _id)
			|| ((id != null)
				&& (_id != null)
				&& id.equals(_id)))
			return;

		_id = (id != null ? id.intern() : null);
	}

	/**
	 * Initializes attribute template to the given value.
	 * @param template New value of template.
	 */
	public void setTemplate(LyFooter template)
	{
		if (template == null)
			return;
		if (_template != null)
			throw new IllegalStateException("BF AppEngine : attribute template was already set.");

		initFromModel(template);
		_template = template;
	}

	/**
	 * Sets attribute label to the given value.
	 * @param label New value of label.
	 */
	public void setLabel(String label)
	{
		if ((label == _label)
			|| ((label != null)
				&& (_label != null)
				&& label.equals(_label)))
			return;

		_label = label;
		if (_internalView != null)
			((LyGraphicalFooter)_internalView).setLabel(label);
	}

	/**
	 * Initializes attribute tooltip to the given value.
	 * @param tooltip New value of tooltip.
	 */
	public void setTooltip(String tooltip)
	{
		if ((tooltip == _tooltip)
			|| ((tooltip != null)
				&& (_tooltip != null)
				&& tooltip.equals(_tooltip)))
			return;

		_tooltip = tooltip;
	}

	/**
	 * Initializes attribute align to the given value.
	 * @param align New value of align.
	 * @exception IllegalArgumentException if align parameter is null.
	 */
	public void setAlign(LyHeader.Align align)
	{
		if (align == _align)
			return;
		if (align == null)
			throw new IllegalArgumentException("BF AppEngine : invalid value for attribute align");

		_align = align;
	}

	/**
	 * Initializes attribute font to the given value.
	 * @param font New value of font.
	 */
	public void setFont(String font)
	{
		if ((font == _font)
			|| ((font != null)
				&& (_font != null)
				&& font.equals(_font)))
			return;

		_font = font;
	}

	/**
	 * Initializes attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color)
	{
		if ((color == _color)
			|| ((color != null)
				&& (_color != null)
				&& color.equals(_color)))
			return;

		_color = color;
	}

	/**
	 * Initializes attribute fontColor to the given value.
	 * @param fontColor New value of fontColor.
	 */
	public void setFontColor(String fontColor)
	{
		if ((fontColor == _fontColor)
			|| ((fontColor != null)
				&& (_fontColor != null)
				&& fontColor.equals(_fontColor)))
			return;

		_fontColor = fontColor;
	}

	/**
	 * Initializes attribute wrap to the given value.
	 * @param wrap Boolean indicating if attribute wrap is set or unset.
	 */
	public void setWrap(boolean wrap)
	{
		if (wrap == _wrap)
			return;
		_wrap = wrap;
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<FOOTER");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template._label)))
			pw.print(" label='" + LyString.getXmlString(_label) + "'");
		if (((_template == null) && (_tooltip != null))
				|| ((_template != null) && (_tooltip != _template._tooltip)))
			pw.print(" tooltip='" + LyString.getXmlString(_tooltip) + "'");
		if ((_template == null)
			|| (_align != _template._align))
		{
			if ((_align == LyHeader.Align.LEFT) && (_template != null))
				pw.print(" align='LEFT'");
			else if (_align == LyHeader.Align.RIGHT)
				pw.print(" align='RIGHT'");
			else if (_align == LyHeader.Align.CENTER)
				pw.print(" align='CENTER'");
		}
		if (((_template == null) && (_font != getStringValue("FOOTER.font", "$ALL.font")))
				|| ((_template != null) && (_font != _template._font)))
			pw.print(" font='" + LyString.getXmlString(_font) + "'");
		if (((_template == null) && (_color != getStringValue("FOOTER.color", "$ALL.color")))
				|| ((_template != null) && (_color != _template._color)))
			pw.print(" color='" + LyString.getXmlString(_color) + "'");
		if (((_template == null) && (_fontColor != getStringValue("FOOTER.fontColor", "$ALL.fontColor")))
				|| ((_template != null) && (_fontColor != _template._fontColor)))
			pw.print(" fontColor='" + LyString.getXmlString(_fontColor) + "'");
		if (((_template == null) && _wrap)
				|| ((_template != null) && (_wrap != _template._wrap)))
			pw.print(" wrap='" + _wrap + "'");
		pw.println("/>");
	}
}
