/*
 * LyFooters.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;

import java.util.*;
import java.io.*;
import leon.view.getinterface.*;
import leon.misc.LyString;


/**
 * The LyFooters class is a generated Java implementation that corresponds to the element 
 * FOOTERS of the descriptive grammar of the views (views.dtd).
 * <B>&lt;FOOTERS&gt; : Footers List</B>
 * A footers list allows to display footers at the bottom of a table.
 * A footers list does not contain any element: this is only a non-empty list of 
 * footers.
 * @see leon.view.LyComponent
 * See also tag <A href="../../conf/dtd/view/FOOTERS.html">FOOTERS</A> of grammar leon_views.dtd.
 */
public class LyFooters extends LyComponent
	implements  LyGetIdInterface
{
	public static final String			__VERSION = "$Id$";


	/* Attributes */

	/**
	 * Identifier of the component.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyFooters			_template;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;


	/* Sub components */

	/**
	 * Footer contained in the footers list.
	 */
	protected ArrayList<LyFooter>		_footers;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_id = null;
		_template = null;
		_font = getStringValue("FOOTERS.font", "$ALL.font");
		_color = getStringValue("FOOTERS.color", "$ALL.color");
		_fontColor = getStringValue("FOOTERS.fontColor", "$ALL.fontColor");

		// Initialize sub components
		_footers = null;
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyFooters	model = (LyFooters)component;

		_model = component;
		_font = model._font;
		_color = model._color;
		_fontColor = model._fontColor;

		if (REWRITING_MODE)
			return;

		// Copy sub-components
		LyFooter	footers;
		int			i, n;

		n = (model._footers == null ? 0 : model._footers.size());
		for (i = 0; i < n; i++)
		{
			footers = model._footers.get(i);
			if (footers != null)
				footers = (LyFooter)footers.cloneComponent();
			addChild(footers);
		}
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	public LyFooters()
	{
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyFooters(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_id = node.getString("id");
		_template = (LyFooters)node.getComponent("template");

		if (_template != null)
			initFromModel(_template);

		if (node.isSpecified("font"))
			_font = node.getString("font");
		if (node.isSpecified("color"))
			_color = node.getString("color");
		if (node.isSpecified("fontColor"))
			_fontColor = node.getString("fontColor");
	}

	/**
	 * End of construction : completion with child component.
	 * @param child Sub child component.
	 * @see leon.view.LyComponent#addChild
	 */
	protected void addChild(LyComponent child)
	{
		// Add sub-components
		if (child instanceof LyFooter)
			addFooter((LyFooter)child);
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyFooters clone = new LyFooters();

		clone.initFromModel(this);
		clone._template = this;
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of id.
	 * @return value of id.
	 */
	public String getId()
	{
		return _id;
	}

	/**
	 * Gets value of template.
	 * @return value of template.
	 */
	public LyFooters getTemplate()
	{
		return _template;
	}

	/**
	 * Gets value of font.
	 * @return value of font.
	 */
	public String getFont()
	{
		return _font;
	}

	/**
	 * Checks whether the font attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFont()
	{
		return (_font == getStringValue("FOOTERS.font"));
	}

	/**
	 * Gets value of color.
	 * @return value of color.
	 */
	public String getColor()
	{
		return _color;
	}

	/**
	 * Checks whether the color attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultColor()
	{
		return (_color == getStringValue("FOOTERS.color"));
	}

	/**
	 * Gets value of fontColor.
	 * @return value of fontColor.
	 */
	public String getFontColor()
	{
		return _fontColor;
	}

	/**
	 * Checks whether the fontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFontColor()
	{
		return (_fontColor == getStringValue("FOOTERS.fontColor"));
	}

	/**
	 * Get sub-component Footer of index i.
	 * @param i the index of the Footer to get.
	 * @return sub-component Footer of index i.
	 */
	public LyFooter getFooter(int i)
	{
		if (_footers == null)
			return null;
		if ((i < 0) || (i >= _footers.size()))
			return null;
		return _footers.get(i);
	}

	/**
	 * Get sub-component Footer count.
	 * @return sub-component Footer count.
	 */
	public int getFooterCount()
	{
		if (_footers == null)
			return 0;
		return _footers.size();
	}


	/* Modifiers */

	/**
	 * Initializes attribute id to the given value.
	 * @param id New value of id.
	 */
	public void setId(String id)
	{
		if ((id == _id)
			|| ((id != null)
				&& (_id != null)
				&& id.equals(_id)))
			return;

		_id = (id != null ? id.intern() : null);
	}

	/**
	 * Initializes attribute template to the given value.
	 * @param template New value of template.
	 */
	public void setTemplate(LyFooters template)
	{
		if (template == null)
			return;
		if (_template != null)
			throw new IllegalStateException("BF AppEngine : attribute template was already set.");

		initFromModel(template);
		_template = template;
	}

	/**
	 * Initializes attribute font to the given value.
	 * @param font New value of font.
	 */
	public void setFont(String font)
	{
		if ((font == _font)
			|| ((font != null)
				&& (_font != null)
				&& font.equals(_font)))
			return;

		_font = font;
	}

	/**
	 * Initializes attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color)
	{
		if ((color == _color)
			|| ((color != null)
				&& (_color != null)
				&& color.equals(_color)))
			return;

		_color = color;
	}

	/**
	 * Initializes attribute fontColor to the given value.
	 * @param fontColor New value of fontColor.
	 */
	public void setFontColor(String fontColor)
	{
		if ((fontColor == _fontColor)
			|| ((fontColor != null)
				&& (_fontColor != null)
				&& fontColor.equals(_fontColor)))
			return;

		_fontColor = fontColor;
	}

	/**
	 * Adds a sub-component footer.
	 * @param footer The footer to add to footers list.
	 */
	public void addFooter(LyFooter footer)
	{
		insertFooter(footer, -1);
	}

	/**
	 * Inserts a sub-component footer at given position.
	 * @param footer The footer to add to footers list.
	 * @param pos	Position where the sub-component is inserted.
	 */
	public void insertFooter(LyFooter footer, int pos)
	{
		if (_footers == null)
		{
			_footers = new ArrayList<LyFooter>(1);
			_children = _footers;
		}
		else if (_footers.contains(footer))
			return;
		if (pos >= 0)
			_footers.add(pos,footer);
		if (footer != null)
			footer.setParent(this);
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<FOOTERS");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && (_font != getStringValue("FOOTERS.font", "$ALL.font")))
				|| ((_template != null) && (_font != _template._font)))
			pw.print(" font='" + LyString.getXmlString(_font) + "'");
		if (((_template == null) && (_color != getStringValue("FOOTERS.color", "$ALL.color")))
				|| ((_template != null) && (_color != _template._color)))
			pw.print(" color='" + LyString.getXmlString(_color) + "'");
		if (((_template == null) && (_fontColor != getStringValue("FOOTERS.fontColor", "$ALL.fontColor")))
				|| ((_template != null) && (_fontColor != _template._fontColor)))
			pw.print(" fontColor='" + LyString.getXmlString(_fontColor) + "'");

		if ((getChildCount() == 0) && (_datas == null))
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		LyComponent	footers;
		int			i, n;

		n = (_footers == null ? 0 : _footers.size());
		for (i = 0; i < n; i++)
		{
			footers = _footers.get(i);
			footers.writeXml(pw, debug);
		}


		pw.println("</FOOTERS>");
	}
}
