/*
 * LyForm.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved.
 */

package leon.view;

import leon.view.setinterface.*;


/**
 * The LyForm class implements generic behavior for the FORM components as described in views.dtd
 * regardless of the requested target (win, swing, html...). It represents a form.
 *
 * @author   Lyria
 * @version  $Id: LyForm.java 30840 2011-05-06 16:47:19Z nhamon $
 * @see      LyComponent
 */
public class LyForm extends LyFormBase
{
	public static final String __VERSION = "$Id: LyForm.java 30840 2011-05-06 16:47:19Z nhamon $";

	/** Name of the form. */
	String _name;

	/**
     * Default constructor.
     */
    protected LyForm()
    {
    }
    
	/**
	 * Constructor with all required attributes.
	 */
	public LyForm(String id)
	{
		super(id);
	}

	/**
	 * Construction from a LyViewNode.
	 */
	protected LyForm(LyViewNode node)
	{
		super(node);
	}

	/**
	 * Retrieve the scrollable attribute.
	 */
	public boolean isScrollable()
	{
		return false;
	}

	/**
	 * Gets name of the form.
	 */
	public String getName()
	{
		// FFT00146 : use one single form for a same web frame
		if (_name == null)
			_name = getRootController();

		return _name;
	}

	/**
	 * Sets name of the form.
	 */
	public void setName(String name)
	{
		_name = name;
	}

	/**
	 * Removes FormField of index i.
	 *
	 * @param  i  the index of the FormField to remove.
	 */
	public void removeFormField(int i)
	{
		if ((i < 0) || (i >= _formFields.size()))
			return;

		LyFormField	formField = getFormField(i);
		
		if (formField != null)
			formField.setParent(null);
		
		_formFields.remove(i);
	}

	/**
	 * Reset content of the form.
	 */
	public void resetContent()
	{
		resetContent(false);
	}

	/**
	 * Reset content of the form.
	 *
	 * @param  full  indicates whether old components must be removed from graphical view.
	 */
	public void resetContent(boolean full)
	{
		Object internal = getInternalView(false);

		if (internal instanceof LyResetContentInterface)
			((LyResetContentInterface)internal).resetContent(full);
	}
} // end class LyForm
