/*
 * LyGauge.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;
import java.io.*;
import leon.view.graphic.*;
import leon.view.getinterface.*;
import leon.view.setinterface.*;
import leon.misc.LyString;


/**
 * The LyGauge class is a generated Java implementation that corresponds to the element 
 * GAUGE of the descriptive grammar of the views (views.dtd).
 * <B>&lt;GAUGE&gt; : Progress Bar</B>
 * A progress bar is used to indicate to the end-user the progression of a process 
 * that takes a long time.
 * @see leon.view.LyFieldItem
 * See also tag <A href="../../conf/dtd/view/GAUGE.html">GAUGE</A> of grammar leon_views.dtd.
 */
public class LyGauge extends LyFieldItem
	implements  LyGetIdInterface,
				LyGetLabelInterface,
				LySetLabelInterface,
				LyGetControllerInterface,
				LySetControllerInterface
{
	public static final String			__VERSION = "$Id$";


	/* Attributes */

	/**
	 * Identifier of the progress bar.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyGauge			_template;

	/**
	 * Label of the progress bar.
	 */
	protected String			_label;

	/**
	 * Reference of the controller that manages the progress bar (i.e. handles user 
	 * demands and refresh content).
	 */
	protected String			_controller;

	/**
	 * Minimum value used for the progress bar.
	 * The expected value is an integer value.
	 */
	protected int				_nMin;

	/**
	 * Maximum value used for the progress bar.
	 * The expected value is an integer value.
	 */
	protected int				_nMax;

	/**
	 * Current value of the progress bar.
	 * The expected value is an integer value.
	 * If the value is lower than the minimum value, the minimum value is taken as the 
	 * current value.
	 * If the value is greater than the maximum value, the maximum value is taken as 
	 * current value.
	 */
	protected int				_nValue;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Color of the progress bar of the gauge.
	 */
	protected String			_progressColor;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	/**
	 * Gives the width of the progress bar.
	 */
	protected int				_nCols;

	/**
	 * Gives the height of the progress bar.
	 */
	protected int				_nRows;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_id = null;
		_template = null;
		_label = null;
		_controller = null;
		_nMin = getIntValue("GAUGE.nMin", -1);
		_nMax = getIntValue("GAUGE.nMax", -1);
		_nValue = getIntValue("GAUGE.nValue", -1);
		_font = getStringValue("GAUGE.font", "$ALL.font");
		_color = getStringValue("GAUGE.color", "$ALL.color");
		_progressColor = getStringValue("GAUGE.progressColor", "$ALL.progressColor");
		_fontColor = getStringValue("GAUGE.fontColor", "$ALL.fontColor");
		_nCols = getIntValue("GAUGE.nCols", -1);
		_nRows = getIntValue("GAUGE.nRows", -1);
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyGauge	model = (LyGauge)component;

		_model = component;
		_label = model._label;
		_controller = model._controller;
		_nMin = model._nMin;
		_nMax = model._nMax;
		_nValue = model._nValue;
		_font = model._font;
		_color = model._color;
		_progressColor = model._progressColor;
		_fontColor = model._fontColor;
		_nCols = model._nCols;
		_nRows = model._nRows;
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	protected LyGauge()
	{
		_id = getInternalId();
	}

	/**
	 * Constructor with all required attributes.
	 * @param id Identifier of the progress bar.
	 */
	public LyGauge(String id)
	{
		// Fill up required attributes with parameters
		_id = (id != null ? id.intern() : null);
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyGauge(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_id = node.getString("id");
		_template = (LyGauge)node.getComponent("template");

		if (_template != null)
			initFromModel(_template);

		if (node.isSpecified("label"))
			_label = node.getString("label");
		if (node.isSpecified("controller"))
			_controller = node.getString("controller");
		if (node.isSpecified("nMin"))
			_nMin = node.getInt("nMin");
		if (node.isSpecified("nMax"))
			_nMax = node.getInt("nMax");
		if (node.isSpecified("nValue"))
			_nValue = node.getInt("nValue");
		if (node.isSpecified("font"))
			_font = node.getString("font");
		if (node.isSpecified("color"))
			_color = node.getString("color");
		if (node.isSpecified("progressColor"))
			_progressColor = node.getString("progressColor");
		if (node.isSpecified("fontColor"))
			_fontColor = node.getString("fontColor");
		if (node.isSpecified("nCols"))
			_nCols = node.getInt("nCols");
		if (node.isSpecified("nRows"))
			_nRows = node.getInt("nRows");
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyGauge clone = new LyGauge();

		clone.initFromModel(this);
		clone._template = this;
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of id.
	 * @return value of id.
	 */
	public String getId()
	{
		return _id;
	}

	/**
	 * Gets value of template.
	 * @return value of template.
	 */
	public LyGauge getTemplate()
	{
		return _template;
	}

	/**
	 * Gets value of label.
	 * @return value of label.
	 */
	public String getLabel()
	{
		return _label;
	}

	/**
	 * Gets value of controller.
	 * @return value of controller.
	 */
	public String getController()
	{
		return _controller;
	}

	/**
	 * Gets value of nMin.
	 * @return value of nMin.
	 */
	public int getNMin()
	{
		return _nMin;
	}

	/**
	 * Gets value of nMax.
	 * @return value of nMax.
	 */
	public int getNMax()
	{
		return _nMax;
	}

	/**
	 * Gets value of nValue.
	 * @return value of nValue.
	 */
	public int getNValue()
	{
		return _nValue;
	}

	/**
	 * Gets value of font.
	 * @return value of font.
	 */
	public String getFont()
	{
		return _font;
	}

	/**
	 * Checks whether the font attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFont()
	{
		return (_font == getStringValue("GAUGE.font"));
	}

	/**
	 * Gets value of color.
	 * @return value of color.
	 */
	public String getColor()
	{
		return _color;
	}

	/**
	 * Checks whether the color attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultColor()
	{
		return (_color == getStringValue("GAUGE.color"));
	}

	/**
	 * Gets value of progressColor.
	 * @return value of progressColor.
	 */
	public String getProgressColor()
	{
		return _progressColor;
	}

	/**
	 * Checks whether the progressColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultProgressColor()
	{
		return (_progressColor == getStringValue("GAUGE.progressColor"));
	}

	/**
	 * Gets value of fontColor.
	 * @return value of fontColor.
	 */
	public String getFontColor()
	{
		return _fontColor;
	}

	/**
	 * Checks whether the fontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFontColor()
	{
		return (_fontColor == getStringValue("GAUGE.fontColor"));
	}

	/**
	 * Gets value of nCols.
	 * @return value of nCols.
	 */
	public int getNCols()
	{
		return _nCols;
	}

	/**
	 * Gets value of nRows.
	 * @return value of nRows.
	 */
	public int getNRows()
	{
		return _nRows;
	}


	/* Modifiers */

	/**
	 * Initializes attribute template to the given value.
	 * @param template New value of template.
	 */
	public void setTemplate(LyGauge template)
	{
		if (template == null)
			return;
		if (_template != null)
			throw new IllegalStateException("BF AppEngine : attribute template was already set.");

		initFromModel(template);
		_template = template;
	}

	/**
	 * Sets attribute label to the given value.
	 * @param label New value of label.
	 */
	public void setLabel(String label)
	{
		if ((label == _label)
			|| ((label != null)
				&& (_label != null)
				&& label.equals(_label)))
			return;

		_label = label;
		if (_internalView != null)
			((LyGraphicalGauge)_internalView).setLabel(label);
	}

	/**
	 * Initializes attribute controller to the given value.
	 * @param controller New value of controller.
	 */
	public void setController(String controller)
	{
		if ((controller == _controller)
			|| ((controller != null)
				&& (_controller != null)
				&& controller.equals(_controller)))
			return;

		_controller = controller;
	}

	/**
	 * Initializes attribute nMin to the given value.
	 * @param nMin New value of nMin.
	 */
	public void setNMin(int nMin)
	{
		if (nMin == _nMin)
			return;
		_nMin = nMin;
	}

	/**
	 * Initializes attribute nMax to the given value.
	 * @param nMax New value of nMax.
	 */
	public void setNMax(int nMax)
	{
		if (nMax == _nMax)
			return;
		_nMax = nMax;
	}

	/**
	 * Sets attribute nValue to the given value.
	 * @param nValue New value of nValue.
	 */
	public void setNValue(int nValue)
	{
		if (nValue == _nValue)
			return;
		_nValue = nValue;
		if (_internalView != null)
			((LyGraphicalGauge)_internalView).setNValue(nValue);
	}

	/**
	 * Initializes attribute font to the given value.
	 * @param font New value of font.
	 */
	public void setFont(String font)
	{
		if ((font == _font)
			|| ((font != null)
				&& (_font != null)
				&& font.equals(_font)))
			return;

		_font = font;
	}

	/**
	 * Initializes attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color)
	{
		if ((color == _color)
			|| ((color != null)
				&& (_color != null)
				&& color.equals(_color)))
			return;

		_color = color;
	}

	/**
	 * Initializes attribute progressColor to the given value.
	 * @param progressColor New value of progressColor.
	 */
	public void setProgressColor(String progressColor)
	{
		if ((progressColor == _progressColor)
			|| ((progressColor != null)
				&& (_progressColor != null)
				&& progressColor.equals(_progressColor)))
			return;

		_progressColor = progressColor;
	}

	/**
	 * Initializes attribute fontColor to the given value.
	 * @param fontColor New value of fontColor.
	 */
	public void setFontColor(String fontColor)
	{
		if ((fontColor == _fontColor)
			|| ((fontColor != null)
				&& (_fontColor != null)
				&& fontColor.equals(_fontColor)))
			return;

		_fontColor = fontColor;
	}

	/**
	 * Initializes attribute nCols to the given value.
	 * @param nCols New value of nCols.
	 */
	public void setNCols(int nCols)
	{
		if (nCols == _nCols)
			return;
		_nCols = nCols;
	}

	/**
	 * Initializes attribute nRows to the given value.
	 * @param nRows New value of nRows.
	 */
	public void setNRows(int nRows)
	{
		if (nRows == _nRows)
			return;
		_nRows = nRows;
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<GAUGE");
		pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template._label)))
			pw.print(" label='" + LyString.getXmlString(_label) + "'");
		if (_controller != null)
			pw.print(" controller='" + LyString.getXmlString(_controller) + "'");
		if (((_template == null) && (_nMin != getIntValue("GAUGE.nMin", -1)))
				|| ((_template != null) && (_nMin != _template._nMin)))
			pw.print(" nMin='" + _nMin + "'");
		if (((_template == null) && (_nMax != getIntValue("GAUGE.nMax", -1)))
				|| ((_template != null) && (_nMax != _template._nMax)))
			pw.print(" nMax='" + _nMax + "'");
		if (((_template == null) && (_nValue != getIntValue("GAUGE.nValue", -1)))
				|| ((_template != null) && (_nValue != _template._nValue)))
			pw.print(" nValue='" + _nValue + "'");
		if (((_template == null) && (_font != getStringValue("GAUGE.font", "$ALL.font")))
				|| ((_template != null) && (_font != _template._font)))
			pw.print(" font='" + LyString.getXmlString(_font) + "'");
		if (((_template == null) && (_color != getStringValue("GAUGE.color", "$ALL.color")))
				|| ((_template != null) && (_color != _template._color)))
			pw.print(" color='" + LyString.getXmlString(_color) + "'");
		if (((_template == null) && (_progressColor != getStringValue("GAUGE.progressColor", "$ALL.progressColor")))
				|| ((_template != null) && (_progressColor != _template._progressColor)))
			pw.print(" progressColor='" + LyString.getXmlString(_progressColor) + "'");
		if (((_template == null) && (_fontColor != getStringValue("GAUGE.fontColor", "$ALL.fontColor")))
				|| ((_template != null) && (_fontColor != _template._fontColor)))
			pw.print(" fontColor='" + LyString.getXmlString(_fontColor) + "'");
		if (((_template == null) && (_nCols != getIntValue("GAUGE.nCols", -1)))
				|| ((_template != null) && (_nCols != _template._nCols)))
			pw.print(" nCols='" + _nCols + "'");
		if (((_template == null) && (_nRows != getIntValue("GAUGE.nRows", -1)))
				|| ((_template != null) && (_nRows != _template._nRows)))
			pw.print(" nRows='" + _nRows + "'");
		pw.println("/>");
	}
}
