/*
 * LyHRef.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;
import java.io.*;
import leon.view.getinterface.*;
import leon.view.setinterface.*;
import leon.misc.LyString;


/**
 * The LyHRef class is a generated Java implementation that corresponds to the element 
 * HREF of the descriptive grammar of the views (views.dtd).
 * <B>&lt;HREF&gt; : Input Field of Hyperlink Type</B>
 * An input field of Hyperlink type allows displaying a graphical component 
 * containing an hyperlink.
 * This link allows launching an action by a simple click.
 * It does not contain any element.
 * @see leon.view.LyFieldItem
 * See also tag <A href="../../conf/dtd/view/HREF.html">HREF</A> of grammar leon_views.dtd.
 */
public class LyHRef extends LyFieldItem
	implements  LyGetIdInterface,
				LyGetLabelInterface,
				LySetLabelInterface,
				LySetImageInterface,
				LySetVisitedInterface,
				LySetSensitiveInterface
{
	public static final String			__VERSION = "$Id$";


	/* Attributes */

	/**
	 * Identifier associated to the link.
	 * It is used when launching the action that is associated to the hyperlink.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyHRef			_template;

	/**
	 * Label of the hyperlink.
	 */
	protected String			_label;

	/**
	 * Image of the hyperlink.
	 * The image is displayed beside the link and functions as the link (clicking the 
	 * image is equivalent to clicking the link).
	 */
	protected String			_image;

	/**
	 * Indicates if the link has already been clicked.
	 * A link that has already be clicked appears in a different color.
	 */
	protected boolean			_visited;

	/**
	 * Indicates if the link can be clicked.
	 */
	protected boolean			_sensitive;

	/**
	 * Allows specifying the name of a Java class that replaces the default graphical 
	 * component.
	 */
	protected String			_bean;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	/**
	 * Font color used when hyperlink is disabled.
	 */
	protected String			_disabledFontColor;

	/**
	 * Font color used when hyperlink is visited.
	 */
	protected String			_visitedFontColor;

	/**
	 * HREF.highlightColor
	 */
	protected String			_highlightColor;

	/**
	 * Gives the height (number of columns) on which the component is shown (given in 
	 * number of characters).
	 */
	protected int				_nCols;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_id = null;
		_template = null;
		_label = null;
		_image = getStringValue("HREF.image", "$ALL.image");
		_visited = getBooleanValue("HREF.visited", false);
		_sensitive = getBooleanValue("HREF.sensitive", true);
		_bean = null;
		_font = getStringValue("HREF.font", "$ALL.font");
		_color = getStringValue("HREF.color", "$ALL.color");
		_fontColor = getStringValue("HREF.fontColor", "$ALL.fontColor");
		_disabledFontColor = getStringValue("HREF.disabledFontColor", "$ALL.disabledFontColor");
		_visitedFontColor = getStringValue("HREF.visitedFontColor", "$ALL.visitedFontColor");
		_highlightColor = getStringValue("HREF.highlightColor", "$ALL.highlightColor");
		_nCols = getIntValue("HREF.nCols", -1);
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyHRef	model = (LyHRef)component;

		_model = component;
		_label = model._label;
		_image = model._image;
		_visited = model._visited;
		_sensitive = model._sensitive;
		_bean = model._bean;
		_font = model._font;
		_color = model._color;
		_fontColor = model._fontColor;
		_disabledFontColor = model._disabledFontColor;
		_visitedFontColor = model._visitedFontColor;
		_highlightColor = model._highlightColor;
		_nCols = model._nCols;
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	public LyHRef()
	{
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyHRef(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_id = node.getString("id");
		_template = (LyHRef)node.getComponent("template");

		if (_template != null)
			initFromModel(_template);

		if (node.isSpecified("label"))
			_label = node.getString("label");
		if (node.isSpecified("image"))
			_image = node.getString("image");
		if (node.isSpecified("visited"))
			_visited = node.getBoolean("visited");
		if (node.isSpecified("sensitive"))
			_sensitive = node.getBoolean("sensitive");
		if (node.isSpecified("bean"))
			_bean = node.getString("bean");
		if (node.isSpecified("font"))
			_font = node.getString("font");
		if (node.isSpecified("color"))
			_color = node.getString("color");
		if (node.isSpecified("fontColor"))
			_fontColor = node.getString("fontColor");
		if (node.isSpecified("disabledFontColor"))
			_disabledFontColor = node.getString("disabledFontColor");
		if (node.isSpecified("visitedFontColor"))
			_visitedFontColor = node.getString("visitedFontColor");
		if (node.isSpecified("highlightColor"))
			_highlightColor = node.getString("highlightColor");
		if (node.isSpecified("nCols"))
			_nCols = node.getInt("nCols");
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyHRef clone = new LyHRef();

		clone.initFromModel(this);
		clone._template = this;
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of id.
	 * @return value of id.
	 */
	public String getId()
	{
		return _id;
	}

	/**
	 * Gets value of template.
	 * @return value of template.
	 */
	public LyHRef getTemplate()
	{
		return _template;
	}

	/**
	 * Gets value of label.
	 * @return value of label.
	 */
	public String getLabel()
	{
		return _label;
	}

	/**
	 * Gets value of image.
	 * @return value of image.
	 */
	public String getImage()
	{
		return _image;
	}

	/**
	 * Checks whether the image attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultImage()
	{
		return (_image == getStringValue("HREF.image"));
	}

	/**
	 * Checks if value of attribute visited is set.
	 * @return true if visited is set, false otherwise.
	 */
	public boolean isVisited()
	{
		return _visited;
	}

	/**
	 * Checks if value of attribute sensitive is set.
	 * @return true if sensitive is set, false otherwise.
	 */
	public boolean isSensitive()
	{
		return _sensitive;
	}

	/**
	 * Gets value of bean.
	 * @return value of bean.
	 */
	public String getBean()
	{
		return _bean;
	}

	/**
	 * Gets value of font.
	 * @return value of font.
	 */
	public String getFont()
	{
		return _font;
	}

	/**
	 * Checks whether the font attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFont()
	{
		return (_font == getStringValue("HREF.font"));
	}

	/**
	 * Gets value of color.
	 * @return value of color.
	 */
	public String getColor()
	{
		return _color;
	}

	/**
	 * Checks whether the color attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultColor()
	{
		return (_color == getStringValue("HREF.color"));
	}

	/**
	 * Gets value of fontColor.
	 * @return value of fontColor.
	 */
	public String getFontColor()
	{
		return _fontColor;
	}

	/**
	 * Checks whether the fontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFontColor()
	{
		return (_fontColor == getStringValue("HREF.fontColor"));
	}

	/**
	 * Gets value of disabledFontColor.
	 * @return value of disabledFontColor.
	 */
	public String getDisabledFontColor()
	{
		return _disabledFontColor;
	}

	/**
	 * Checks whether the disabledFontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultDisabledFontColor()
	{
		return (_disabledFontColor == getStringValue("HREF.disabledFontColor"));
	}

	/**
	 * Gets value of visitedFontColor.
	 * @return value of visitedFontColor.
	 */
	public String getVisitedFontColor()
	{
		return _visitedFontColor;
	}

	/**
	 * Checks whether the visitedFontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultVisitedFontColor()
	{
		return (_visitedFontColor == getStringValue("HREF.visitedFontColor"));
	}

	/**
	 * Gets value of highlightColor.
	 * @return value of highlightColor.
	 */
	public String getHighlightColor()
	{
		return _highlightColor;
	}

	/**
	 * Checks whether the highlightColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultHighlightColor()
	{
		return (_highlightColor == getStringValue("HREF.highlightColor"));
	}

	/**
	 * Gets value of nCols.
	 * @return value of nCols.
	 */
	public int getNCols()
	{
		return _nCols;
	}


	/* Modifiers */

	/**
	 * Initializes attribute id to the given value.
	 * @param id New value of id.
	 */
	public void setId(String id)
	{
		if ((id == _id)
			|| ((id != null)
				&& (_id != null)
				&& id.equals(_id)))
			return;

		_id = (id != null ? id.intern() : null);
	}

	/**
	 * Initializes attribute template to the given value.
	 * @param template New value of template.
	 */
	public void setTemplate(LyHRef template)
	{
		if (template == null)
			return;
		if (_template != null)
			throw new IllegalStateException("BF AppEngine : attribute template was already set.");

		initFromModel(template);
		_template = template;
	}

	/**
	 * Sets attribute label to the given value.
	 * @param label New value of label.
	 */
	public void setLabel(String label)
	{
		if ((label == _label)
			|| ((label != null)
				&& (_label != null)
				&& label.equals(_label)))
			return;

		_label = label;
		if (_internalView instanceof LySetLabelInterface)
			((LySetLabelInterface)_internalView).setLabel(label);
	}

	/**
	 * Sets attribute image to the given value.
	 * @param image New value of image.
	 */
	public void setImage(String image)
	{
		if ((image == _image)
			|| ((image != null)
				&& (_image != null)
				&& image.equals(_image)))
			return;

		_image = image;
		if (_internalView instanceof LySetImageInterface)
			((LySetImageInterface)_internalView).setImage(image);
	}

	/**
	 * Sets attribute visited to the given value.
	 * @param visited Boolean indicating if attribute visited is set or unset.
	 */
	public void setVisited(boolean visited)
	{
		if (visited == _visited)
			return;
		_visited = visited;
		if (_internalView instanceof LySetVisitedInterface)
			((LySetVisitedInterface)_internalView).setVisited(visited);
	}

	/**
	 * Sets attribute sensitive to the given value.
	 * @param sensitive Boolean indicating if attribute sensitive is set or unset.
	 */
	public void setSensitive(boolean sensitive)
	{
		if (sensitive == _sensitive)
			return;
		_sensitive = sensitive;
		if (_internalView instanceof LySetSensitiveInterface)
			((LySetSensitiveInterface)_internalView).setSensitive(sensitive);
	}

	/**
	 * Initializes attribute bean to the given value.
	 * @param bean New value of bean.
	 */
	public void setBean(String bean)
	{
		if ((bean == _bean)
			|| ((bean != null)
				&& (_bean != null)
				&& bean.equals(_bean)))
			return;

		_bean = bean;
	}

	/**
	 * Initializes attribute font to the given value.
	 * @param font New value of font.
	 */
	public void setFont(String font)
	{
		if ((font == _font)
			|| ((font != null)
				&& (_font != null)
				&& font.equals(_font)))
			return;

		_font = font;
	}

	/**
	 * Initializes attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color)
	{
		if ((color == _color)
			|| ((color != null)
				&& (_color != null)
				&& color.equals(_color)))
			return;

		_color = color;
	}

	/**
	 * Initializes attribute fontColor to the given value.
	 * @param fontColor New value of fontColor.
	 */
	public void setFontColor(String fontColor)
	{
		if ((fontColor == _fontColor)
			|| ((fontColor != null)
				&& (_fontColor != null)
				&& fontColor.equals(_fontColor)))
			return;

		_fontColor = fontColor;
	}

	/**
	 * Initializes attribute disabledFontColor to the given value.
	 * @param disabledFontColor New value of disabledFontColor.
	 */
	public void setDisabledFontColor(String disabledFontColor)
	{
		if ((disabledFontColor == _disabledFontColor)
			|| ((disabledFontColor != null)
				&& (_disabledFontColor != null)
				&& disabledFontColor.equals(_disabledFontColor)))
			return;

		_disabledFontColor = disabledFontColor;
	}

	/**
	 * Initializes attribute visitedFontColor to the given value.
	 * @param visitedFontColor New value of visitedFontColor.
	 */
	public void setVisitedFontColor(String visitedFontColor)
	{
		if ((visitedFontColor == _visitedFontColor)
			|| ((visitedFontColor != null)
				&& (_visitedFontColor != null)
				&& visitedFontColor.equals(_visitedFontColor)))
			return;

		_visitedFontColor = visitedFontColor;
	}

	/**
	 * Initializes attribute highlightColor to the given value.
	 * @param highlightColor New value of highlightColor.
	 */
	public void setHighlightColor(String highlightColor)
	{
		if ((highlightColor == _highlightColor)
			|| ((highlightColor != null)
				&& (_highlightColor != null)
				&& highlightColor.equals(_highlightColor)))
			return;

		_highlightColor = highlightColor;
	}

	/**
	 * Initializes attribute nCols to the given value.
	 * @param nCols New value of nCols.
	 */
	public void setNCols(int nCols)
	{
		if (nCols == _nCols)
			return;
		_nCols = nCols;
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<HREF");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template._label)))
			pw.print(" label='" + LyString.getXmlString(_label) + "'");
		if (((_template == null) && (_image != getStringValue("HREF.image", "$ALL.image")))
				|| ((_template != null) && (_image != _template._image)))
			pw.print(" image='" + LyString.getXmlString(_image) + "'");
		if (((_template == null) && _visited)
				|| ((_template != null) && (_visited != _template._visited)))
			pw.print(" visited='" + _visited + "'");
		if (((_template == null) && !_sensitive)
				|| ((_template != null) && (_sensitive != _template._sensitive)))
			pw.print(" sensitive='" + _sensitive + "'");
		if (((_template == null) && (_bean != null))
				|| ((_template != null) && (_bean != _template._bean)))
			pw.print(" bean='" + LyString.getXmlString(_bean) + "'");
		if (((_template == null) && (_font != getStringValue("HREF.font", "$ALL.font")))
				|| ((_template != null) && (_font != _template._font)))
			pw.print(" font='" + LyString.getXmlString(_font) + "'");
		if (((_template == null) && (_color != getStringValue("HREF.color", "$ALL.color")))
				|| ((_template != null) && (_color != _template._color)))
			pw.print(" color='" + LyString.getXmlString(_color) + "'");
		if (((_template == null) && (_fontColor != getStringValue("HREF.fontColor", "$ALL.fontColor")))
				|| ((_template != null) && (_fontColor != _template._fontColor)))
			pw.print(" fontColor='" + LyString.getXmlString(_fontColor) + "'");
		if (((_template == null) && (_disabledFontColor != getStringValue("HREF.disabledFontColor", "$ALL.disabledFontColor")))
				|| ((_template != null) && (_disabledFontColor != _template._disabledFontColor)))
			pw.print(" disabledFontColor='" + LyString.getXmlString(_disabledFontColor) + "'");
		if (((_template == null) && (_visitedFontColor != getStringValue("HREF.visitedFontColor", "$ALL.visitedFontColor")))
				|| ((_template != null) && (_visitedFontColor != _template._visitedFontColor)))
			pw.print(" visitedFontColor='" + LyString.getXmlString(_visitedFontColor) + "'");
		if (((_template == null) && (_highlightColor != getStringValue("HREF.highlightColor", "$ALL.highlightColor")))
				|| ((_template != null) && (_highlightColor != _template._highlightColor)))
			pw.print(" highlightColor='" + LyString.getXmlString(_highlightColor) + "'");
		if (((_template == null) && (_nCols != getIntValue("HREF.nCols", -1)))
				|| ((_template != null) && (_nCols != _template._nCols)))
			pw.print(" nCols='" + _nCols + "'");
		pw.println("/>");
	}
}
