/*
 * LyHRefs.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;

import java.util.*;
import java.io.*;
import leon.view.getinterface.*;
import leon.view.setinterface.*;
import leon.misc.LyString;


/**
 * The LyHRefs class is a generated Java implementation that corresponds to the element 
 * HREFS of the descriptive grammar of the views (views.dtd).
 * <B>&lt;HREFS&gt; : List of Hyperlinks</B>
 * An input field of Hyperlinks type allows displaying a graphical component 
 * containing a set of hyperlinks.
 * Such a component is a simple list of hyperlinks such as described below.
 * It does not contain any attribute.
 * @see leon.view.LyFieldItem
 * See also tag <A href="../../conf/dtd/view/HREFS.html">HREFS</A> of grammar leon_views.dtd.
 */
public class LyHRefs extends LyFieldItem
	implements  LyGetIdInterface
{
	public static final String			__VERSION = "$Id$";


	/* Attributes */

	/**
	 * Identifier of the component.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyHRefs			_template;

	/**
	 * Allows specifying the name of a Java class that replaces the default graphical 
	 * component.
	 */
	protected String			_bean;

	/**
	 * Indicates if a border must be drawn around the hyperlinks.
	 */
	protected boolean			_border;

	/**
	 * Gives the height (number of columns) on which the lines are shown (given in 
	 * number of characters).
	 */
	protected int				_nCols;

	/**
	 * Gives the width (number of rows) on which the lines are shown (given in number 
	 * of characters).
	 */
	protected int				_nRows;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	/**
	 * Font color used when hyperlinks are disabled.
	 */
	protected String			_disabledFontColor;

	/**
	 * Font color used when hyperlinks are visited.
	 */
	protected String			_visitedFontColor;

	/**
	 * HREFS.highlightColor
	 */
	protected String			_highlightColor;


	/* Sub components */

	/**
	 * List of hyperlinks.
	 */
	protected ArrayList<LyHRef>		_hRefs;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_id = null;
		_template = null;
		_bean = null;
		_border = getBooleanValue("HREFS.border", true);
		_nCols = getIntValue("HREFS.nCols", -1);
		_nRows = getIntValue("HREFS.nRows", -1);
		_font = getStringValue("HREFS.font", "$ALL.font");
		_color = getStringValue("HREFS.color", "$ALL.color");
		_fontColor = getStringValue("HREFS.fontColor", "$ALL.fontColor");
		_disabledFontColor = getStringValue("HREFS.disabledFontColor", "$ALL.disabledFontColor");
		_visitedFontColor = getStringValue("HREFS.visitedFontColor", "$ALL.visitedFontColor");
		_highlightColor = getStringValue("HREFS.highlightColor", "$ALL.highlightColor");

		// Initialize sub components
		_hRefs = null;
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyHRefs	model = (LyHRefs)component;

		_model = component;
		_bean = model._bean;
		_border = model._border;
		_nCols = model._nCols;
		_nRows = model._nRows;
		_font = model._font;
		_color = model._color;
		_fontColor = model._fontColor;
		_disabledFontColor = model._disabledFontColor;
		_visitedFontColor = model._visitedFontColor;
		_highlightColor = model._highlightColor;

		if (REWRITING_MODE)
			return;

		// Copy sub-components
		LyHRef	hRefs;
		int			i, n;

		n = (model._hRefs == null ? 0 : model._hRefs.size());
		for (i = 0; i < n; i++)
		{
			hRefs = model._hRefs.get(i);
			if (hRefs != null)
				hRefs = (LyHRef)hRefs.cloneComponent();
			addChild(hRefs);
		}
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	public LyHRefs()
	{
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyHRefs(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_id = node.getString("id");
		_template = (LyHRefs)node.getComponent("template");

		if (_template != null)
			initFromModel(_template);

		if (node.isSpecified("bean"))
			_bean = node.getString("bean");
		if (node.isSpecified("border"))
			_border = node.getBoolean("border");
		if (node.isSpecified("nCols"))
			_nCols = node.getInt("nCols");
		if (node.isSpecified("nRows"))
			_nRows = node.getInt("nRows");
		if (node.isSpecified("font"))
			_font = node.getString("font");
		if (node.isSpecified("color"))
			_color = node.getString("color");
		if (node.isSpecified("fontColor"))
			_fontColor = node.getString("fontColor");
		if (node.isSpecified("disabledFontColor"))
			_disabledFontColor = node.getString("disabledFontColor");
		if (node.isSpecified("visitedFontColor"))
			_visitedFontColor = node.getString("visitedFontColor");
		if (node.isSpecified("highlightColor"))
			_highlightColor = node.getString("highlightColor");
	}

	/**
	 * End of construction : completion with child component.
	 * @param child Sub child component.
	 * @see leon.view.LyComponent#addChild
	 */
	protected void addChild(LyComponent child)
	{
		// Add sub-components
		if (child instanceof LyHRef)
			addHRef((LyHRef)child);
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyHRefs clone = new LyHRefs();

		clone.initFromModel(this);
		clone._template = this;
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of id.
	 * @return value of id.
	 */
	public String getId()
	{
		return _id;
	}

	/**
	 * Gets value of template.
	 * @return value of template.
	 */
	public LyHRefs getTemplate()
	{
		return _template;
	}

	/**
	 * Gets value of bean.
	 * @return value of bean.
	 */
	public String getBean()
	{
		return _bean;
	}

	/**
	 * Checks if value of attribute border is set.
	 * @return true if border is set, false otherwise.
	 */
	public boolean isBorder()
	{
		return _border;
	}

	/**
	 * Gets value of nCols.
	 * @return value of nCols.
	 */
	public int getNCols()
	{
		return _nCols;
	}

	/**
	 * Gets value of nRows.
	 * @return value of nRows.
	 */
	public int getNRows()
	{
		return _nRows;
	}

	/**
	 * Gets value of font.
	 * @return value of font.
	 */
	public String getFont()
	{
		return _font;
	}

	/**
	 * Checks whether the font attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFont()
	{
		return (_font == getStringValue("HREFS.font"));
	}

	/**
	 * Gets value of color.
	 * @return value of color.
	 */
	public String getColor()
	{
		return _color;
	}

	/**
	 * Checks whether the color attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultColor()
	{
		return (_color == getStringValue("HREFS.color"));
	}

	/**
	 * Gets value of fontColor.
	 * @return value of fontColor.
	 */
	public String getFontColor()
	{
		return _fontColor;
	}

	/**
	 * Checks whether the fontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFontColor()
	{
		return (_fontColor == getStringValue("HREFS.fontColor"));
	}

	/**
	 * Gets value of disabledFontColor.
	 * @return value of disabledFontColor.
	 */
	public String getDisabledFontColor()
	{
		return _disabledFontColor;
	}

	/**
	 * Checks whether the disabledFontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultDisabledFontColor()
	{
		return (_disabledFontColor == getStringValue("HREFS.disabledFontColor"));
	}

	/**
	 * Gets value of visitedFontColor.
	 * @return value of visitedFontColor.
	 */
	public String getVisitedFontColor()
	{
		return _visitedFontColor;
	}

	/**
	 * Checks whether the visitedFontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultVisitedFontColor()
	{
		return (_visitedFontColor == getStringValue("HREFS.visitedFontColor"));
	}

	/**
	 * Gets value of highlightColor.
	 * @return value of highlightColor.
	 */
	public String getHighlightColor()
	{
		return _highlightColor;
	}

	/**
	 * Checks whether the highlightColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultHighlightColor()
	{
		return (_highlightColor == getStringValue("HREFS.highlightColor"));
	}

	/**
	 * Get sub-component HRef of index i.
	 * @param i the index of the HRef to get.
	 * @return sub-component HRef of index i.
	 */
	public LyHRef getHRef(int i)
	{
		if (_hRefs == null)
			return null;
		if ((i < 0) || (i >= _hRefs.size()))
			return null;
		return _hRefs.get(i);
	}

	/**
	 * Get sub-component HRef count.
	 * @return sub-component HRef count.
	 */
	public int getHRefCount()
	{
		if (_hRefs == null)
			return 0;
		return _hRefs.size();
	}


	/* Modifiers */

	/**
	 * Initializes attribute id to the given value.
	 * @param id New value of id.
	 */
	public void setId(String id)
	{
		if ((id == _id)
			|| ((id != null)
				&& (_id != null)
				&& id.equals(_id)))
			return;

		_id = (id != null ? id.intern() : null);
	}

	/**
	 * Initializes attribute template to the given value.
	 * @param template New value of template.
	 */
	public void setTemplate(LyHRefs template)
	{
		if (template == null)
			return;
		if (_template != null)
			throw new IllegalStateException("BF AppEngine : attribute template was already set.");

		initFromModel(template);
		_template = template;
	}

	/**
	 * Initializes attribute bean to the given value.
	 * @param bean New value of bean.
	 */
	public void setBean(String bean)
	{
		if ((bean == _bean)
			|| ((bean != null)
				&& (_bean != null)
				&& bean.equals(_bean)))
			return;

		_bean = bean;
	}

	/**
	 * Initializes attribute border to the given value.
	 * @param border Boolean indicating if attribute border is set or unset.
	 */
	public void setBorder(boolean border)
	{
		if (border == _border)
			return;
		_border = border;
	}

	/**
	 * Initializes attribute nCols to the given value.
	 * @param nCols New value of nCols.
	 */
	public void setNCols(int nCols)
	{
		if (nCols == _nCols)
			return;
		_nCols = nCols;
	}

	/**
	 * Initializes attribute nRows to the given value.
	 * @param nRows New value of nRows.
	 */
	public void setNRows(int nRows)
	{
		if (nRows == _nRows)
			return;
		_nRows = nRows;
	}

	/**
	 * Initializes attribute font to the given value.
	 * @param font New value of font.
	 */
	public void setFont(String font)
	{
		if ((font == _font)
			|| ((font != null)
				&& (_font != null)
				&& font.equals(_font)))
			return;

		_font = font;
	}

	/**
	 * Initializes attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color)
	{
		if ((color == _color)
			|| ((color != null)
				&& (_color != null)
				&& color.equals(_color)))
			return;

		_color = color;
	}

	/**
	 * Initializes attribute fontColor to the given value.
	 * @param fontColor New value of fontColor.
	 */
	public void setFontColor(String fontColor)
	{
		if ((fontColor == _fontColor)
			|| ((fontColor != null)
				&& (_fontColor != null)
				&& fontColor.equals(_fontColor)))
			return;

		_fontColor = fontColor;
	}

	/**
	 * Initializes attribute disabledFontColor to the given value.
	 * @param disabledFontColor New value of disabledFontColor.
	 */
	public void setDisabledFontColor(String disabledFontColor)
	{
		if ((disabledFontColor == _disabledFontColor)
			|| ((disabledFontColor != null)
				&& (_disabledFontColor != null)
				&& disabledFontColor.equals(_disabledFontColor)))
			return;

		_disabledFontColor = disabledFontColor;
	}

	/**
	 * Initializes attribute visitedFontColor to the given value.
	 * @param visitedFontColor New value of visitedFontColor.
	 */
	public void setVisitedFontColor(String visitedFontColor)
	{
		if ((visitedFontColor == _visitedFontColor)
			|| ((visitedFontColor != null)
				&& (_visitedFontColor != null)
				&& visitedFontColor.equals(_visitedFontColor)))
			return;

		_visitedFontColor = visitedFontColor;
	}

	/**
	 * Initializes attribute highlightColor to the given value.
	 * @param highlightColor New value of highlightColor.
	 */
	public void setHighlightColor(String highlightColor)
	{
		if ((highlightColor == _highlightColor)
			|| ((highlightColor != null)
				&& (_highlightColor != null)
				&& highlightColor.equals(_highlightColor)))
			return;

		_highlightColor = highlightColor;
	}

	/**
	 * Adds a sub-component hRef.
	 * @param hRef The hRef to add to hRefs list.
	 */
	public void addHRef(LyHRef hRef)
	{
		insertHRef(hRef, -1);
	}

	/**
	 * Inserts a sub-component hRef at given position.
	 * @param hRef The hRef to add to hRefs list.
	 * @param pos	Position where the sub-component is inserted.
	 */
	public void insertHRef(LyHRef hRef, int pos)
	{
		if (_hRefs == null)
		{
			_hRefs = new ArrayList<LyHRef>(1);
			_children = _hRefs;
		}
		else if (_hRefs.contains(hRef))
			return;
		if (pos >= 0)
			_hRefs.add(pos,hRef);
		if (hRef != null)
			hRef.setParent(this);
		if (_internalView instanceof LySetHRefsInterface)
			((LySetHRefsInterface)_internalView).addHRef(hRef, pos);
	}

	/**
	 * Removes a sub-component hRef.
	 * @param hRef The hRef to remove to hRefs list.
	 * @exception IllegalStateException if hRefs list is empty.
	 * @exception IllegalArgumentException if hRefs list doesn't contain hRef.
	 */
	public void removeHRef(LyHRef hRef)
	{
		if (_hRefs == null)
			throw new IllegalStateException("BF AppEngine : hRefs list is empty");
		if (!_hRefs.contains(hRef))
			throw new IllegalArgumentException("BF AppEngine : hRefs list doesn't contain hRef");

		if (_internalView instanceof LySetHRefsInterface)
			((LySetHRefsInterface)_internalView).removeHRef(hRef);
		hRef.setParent(null);
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<HREFS");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && (_bean != null))
				|| ((_template != null) && (_bean != _template._bean)))
			pw.print(" bean='" + LyString.getXmlString(_bean) + "'");
		if (((_template == null) && !_border)
				|| ((_template != null) && (_border != _template._border)))
			pw.print(" border='" + _border + "'");
		if (((_template == null) && (_nCols != getIntValue("HREFS.nCols", -1)))
				|| ((_template != null) && (_nCols != _template._nCols)))
			pw.print(" nCols='" + _nCols + "'");
		if (((_template == null) && (_nRows != getIntValue("HREFS.nRows", -1)))
				|| ((_template != null) && (_nRows != _template._nRows)))
			pw.print(" nRows='" + _nRows + "'");
		if (((_template == null) && (_font != getStringValue("HREFS.font", "$ALL.font")))
				|| ((_template != null) && (_font != _template._font)))
			pw.print(" font='" + LyString.getXmlString(_font) + "'");
		if (((_template == null) && (_color != getStringValue("HREFS.color", "$ALL.color")))
				|| ((_template != null) && (_color != _template._color)))
			pw.print(" color='" + LyString.getXmlString(_color) + "'");
		if (((_template == null) && (_fontColor != getStringValue("HREFS.fontColor", "$ALL.fontColor")))
				|| ((_template != null) && (_fontColor != _template._fontColor)))
			pw.print(" fontColor='" + LyString.getXmlString(_fontColor) + "'");
		if (((_template == null) && (_disabledFontColor != getStringValue("HREFS.disabledFontColor", "$ALL.disabledFontColor")))
				|| ((_template != null) && (_disabledFontColor != _template._disabledFontColor)))
			pw.print(" disabledFontColor='" + LyString.getXmlString(_disabledFontColor) + "'");
		if (((_template == null) && (_visitedFontColor != getStringValue("HREFS.visitedFontColor", "$ALL.visitedFontColor")))
				|| ((_template != null) && (_visitedFontColor != _template._visitedFontColor)))
			pw.print(" visitedFontColor='" + LyString.getXmlString(_visitedFontColor) + "'");
		if (((_template == null) && (_highlightColor != getStringValue("HREFS.highlightColor", "$ALL.highlightColor")))
				|| ((_template != null) && (_highlightColor != _template._highlightColor)))
			pw.print(" highlightColor='" + LyString.getXmlString(_highlightColor) + "'");

		if ((getChildCount() == 0) && (_datas == null))
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		LyComponent	hRefs;
		int			i, n;

		n = (_hRefs == null ? 0 : _hRefs.size());
		for (i = 0; i < n; i++)
		{
			hRefs = _hRefs.get(i);
			hRefs.writeXml(pw, debug);
		}


		pw.println("</HREFS>");
	}
}
