/*
 * LyIcon.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;
import java.io.*;
import leon.view.getinterface.*;
import leon.view.setinterface.*;
import leon.misc.LyString;


/**
 * The LyIcon class is a generated Java implementation that corresponds to the element 
 * ICON of the descriptive grammar of the views (views.dtd).
 * <B>&lt;ICON&gt; : Icon</B>
 * An icon allows defining an image to which a text can be associated.
 * @see leon.view.LyFieldItem
 * See also tag <A href="../../conf/dtd/view/ICON.html">ICON</A> of grammar leon_views.dtd.
 */
public class LyIcon extends LyFieldItem
	implements  LyGetIdInterface,
				LyGetLabelInterface,
				LySetImageInterface,
				LyGetConstraintInterface
{
	public static final String			__VERSION = "$Id$";


	/** Possible values for attribute 'border'. */
	public enum Border
	{
		NONE,
		SINGLE,
		DOUBLE,
		RAISED,
		LOWERED;
	}


	/* Static methods */

	/**
	 * Utility method to get border value from a string.
	 * @param token Token string
	 * @return Border value corresponding to this token
	 */
	public static final Border getBorder(String token)
	{
		if (token == null)
			return null;
		else if (token.equalsIgnoreCase("NONE")
				|| token.equals("0"))
			return Border.NONE;
		else if (token.equalsIgnoreCase("SINGLE")
				|| token.equals("1"))
			return Border.SINGLE;
		else if (token.equalsIgnoreCase("DOUBLE")
				|| token.equals("2"))
			return Border.DOUBLE;
		else if (token.equalsIgnoreCase("RAISED")
				|| token.equals("3"))
			return Border.RAISED;
		else if (token.equalsIgnoreCase("LOWERED")
				|| token.equals("4"))
			return Border.LOWERED;
		else
			return Border.NONE;
	}


	/* Attributes */

	/**
	 * Identifier of the component.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyIcon			_template;

	/**
	 * Text to display with the image
	 */
	protected String			_label;

	/**
	 * Image
	 */
	protected String			_image;

	/**
	 * Width of the image
	 */
	protected int				_width;

	/**
	 * Height of the image
	 */
	protected int				_height;

	/**
	 * Allows defining whether a border will be displayed around the image or not:
	 * NONE: No border (default value)
	 * SINGLE: Simple border
	 * DOUBLE: Double border
	 * RAISED: Border simulating a raised 3D frame
	 * LOWERED: Border simulating a lowered 3D frame
	 */
	protected Border			_border;

	/**
	 * Allows specifying the name of a Java class that replaces the default graphical 
	 * component.
	 */
	protected String			_bean;


	/* Sub components */

	/**
	 * Positioning constraint of the icon.
	 */
	protected LyConstraint		_constraint;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_id = null;
		_template = null;
		_label = null;
		_image = getStringValue("ICON.image", "$ALL.image");
		_width = getIntValue("ICON.width", -1);
		_height = getIntValue("ICON.height", -1);
		_border = Border.NONE;
		_bean = null;

		// Initialize sub components
		_constraint = null;
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyIcon	model = (LyIcon)component;

		_model = component;
		_label = model._label;
		_image = model._image;
		_width = model._width;
		_height = model._height;
		_border = model._border;
		_bean = model._bean;

		if (REWRITING_MODE)
			return;

		// Copy sub-components
		if (model._constraint != null)
			addChild(model._constraint.cloneComponent());
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	protected LyIcon()
	{
	}

	/**
	 * Constructor with all required attributes.
	 * @param label Text to display with the image
	 * @param image Image
	 */
	public LyIcon(String label, String image)
	{
		// Fill up required attributes with parameters
		_label = label;
		_image = image;
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyIcon(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_id = node.getString("id");
		_template = (LyIcon)node.getComponent("template");

		if (_template != null)
			initFromModel(_template);

		if (node.isSpecified("label"))
			_label = node.getString("label");
		if (node.isSpecified("image"))
			_image = node.getString("image");
		if (node.isSpecified("width"))
			_width = node.getInt("width");
		if (node.isSpecified("height"))
			_height = node.getInt("height");
		if (node.isSpecified("border"))
			_border = getBorder(node.getString("border"));
		if (node.isSpecified("bean"))
			_bean = node.getString("bean");
	}

	/**
	 * End of construction : completion with child component.
	 * @param child Sub child component.
	 * @see leon.view.LyComponent#addChild
	 */
	protected void addChild(LyComponent child)
	{
		// Add sub-components
		if (child instanceof LyConstraint)
			setConstraint((LyConstraint)child);
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyIcon clone = new LyIcon();

		clone.initFromModel(this);
		clone._template = this;
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of id.
	 * @return value of id.
	 */
	public String getId()
	{
		return _id;
	}

	/**
	 * Gets value of template.
	 * @return value of template.
	 */
	public LyIcon getTemplate()
	{
		return _template;
	}

	/**
	 * Gets value of label.
	 * @return value of label.
	 */
	public String getLabel()
	{
		return _label;
	}

	/**
	 * Gets value of image.
	 * @return value of image.
	 */
	public String getImage()
	{
		return _image;
	}

	/**
	 * Checks whether the image attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultImage()
	{
		return (_image == getStringValue("ICON.image"));
	}

	/**
	 * Gets value of width.
	 * @return value of width.
	 */
	public int getWidth()
	{
		return _width;
	}

	/**
	 * Gets value of height.
	 * @return value of height.
	 */
	public int getHeight()
	{
		return _height;
	}

	/**
	 * Gets value of border.
	 * @return value of border.
	 */
	public Border getBorder()
	{
		return _border;
	}

	/**
	 * Gets value of bean.
	 * @return value of bean.
	 */
	public String getBean()
	{
		return _bean;
	}

	/**
	 * Get sub-component Constraint.
	 * @return sub-component Constraint.
	 */
	public LyConstraint getConstraint()
	{
		return _constraint;
	}


	/* Modifiers */

	/**
	 * Initializes attribute id to the given value.
	 * @param id New value of id.
	 */
	public void setId(String id)
	{
		if ((id == _id)
			|| ((id != null)
				&& (_id != null)
				&& id.equals(_id)))
			return;

		_id = (id != null ? id.intern() : null);
	}

	/**
	 * Initializes attribute template to the given value.
	 * @param template New value of template.
	 */
	public void setTemplate(LyIcon template)
	{
		if (template == null)
			return;
		if (_template != null)
			throw new IllegalStateException("BF AppEngine : attribute template was already set.");

		initFromModel(template);
		_template = template;
	}

	/**
	 * Initializes attribute label to the given value.
	 * @param label New value of label.
	 * @exception IllegalArgumentException if label is null.
	 */
	public void setLabel(String label)
	{
		if (label == null)
			throw new IllegalArgumentException("BF AppEngine : attribute label is required (can't be null)");

		if ((label == _label)
			|| ((label != null)
				&& (_label != null)
				&& label.equals(_label)))
			return;

		_label = label;
	}

	/**
	 * Sets attribute image to the given value.
	 * @param image New value of image.
	 * @exception IllegalArgumentException if image is null.
	 */
	public void setImage(String image)
	{
		if (image == null)
			throw new IllegalArgumentException("BF AppEngine : attribute image is required (can't be null)");

		if ((image == _image)
			|| ((image != null)
				&& (_image != null)
				&& image.equals(_image)))
			return;

		_image = image;
		if (_internalView instanceof LySetImageInterface)
			((LySetImageInterface)_internalView).setImage(image);
	}

	/**
	 * Initializes attribute width to the given value.
	 * @param width New value of width.
	 */
	public void setWidth(int width)
	{
		if (width == _width)
			return;
		_width = width;
	}

	/**
	 * Initializes attribute height to the given value.
	 * @param height New value of height.
	 */
	public void setHeight(int height)
	{
		if (height == _height)
			return;
		_height = height;
	}

	/**
	 * Initializes attribute border to the given value.
	 * @param border New value of border.
	 * @exception IllegalArgumentException if border parameter is null.
	 */
	public void setBorder(Border border)
	{
		if (border == _border)
			return;
		if (border == null)
			throw new IllegalArgumentException("BF AppEngine : invalid value for attribute border");

		_border = border;
	}

	/**
	 * Initializes attribute bean to the given value.
	 * @param bean New value of bean.
	 */
	public void setBean(String bean)
	{
		if ((bean == _bean)
			|| ((bean != null)
				&& (_bean != null)
				&& bean.equals(_bean)))
			return;

		_bean = bean;
	}

	/**
	 * Sets sub-component constraint.
	 * @param constraint New value of constraint
	 */
	public void setConstraint(LyConstraint constraint)
	{
		if (_constraint != null)
			_constraint.setParent(null);
		_constraint = constraint;
		if (constraint != null)
			constraint.setParent(this);
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<ICON");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		pw.print(" label='" + LyString.getXmlString(_label) + "'");
		pw.print(" image='" + LyString.getXmlString(_image) + "'");
		if (((_template == null) && (_width != getIntValue("ICON.width", -1)))
				|| ((_template != null) && (_width != _template._width)))
			pw.print(" width='" + _width + "'");
		if (((_template == null) && (_height != getIntValue("ICON.height", -1)))
				|| ((_template != null) && (_height != _template._height)))
			pw.print(" height='" + _height + "'");
		if ((_template == null)
			|| (_border != _template._border))
		{
			if ((_border == Border.NONE) && (_template != null))
				pw.print(" border='NONE'");
			else if (_border == Border.SINGLE)
				pw.print(" border='SINGLE'");
			else if (_border == Border.DOUBLE)
				pw.print(" border='DOUBLE'");
			else if (_border == Border.RAISED)
				pw.print(" border='RAISED'");
			else if (_border == Border.LOWERED)
				pw.print(" border='LOWERED'");
		}
		if (((_template == null) && (_bean != null))
				|| ((_template != null) && (_bean != _template._bean)))
			pw.print(" bean='" + LyString.getXmlString(_bean) + "'");

		if ((getChildCount() == 0) && (_datas == null))
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		if (_constraint != null)
			_constraint.writeXml(pw, debug);

		pw.println("</ICON>");
	}
}
