/*
 * LyImage.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;
import java.io.*;
import leon.view.graphic.*;
import leon.view.getinterface.*;
import leon.view.setinterface.*;
import leon.misc.LyString;


/**
 * The LyImage class is a generated Java implementation that corresponds to the element 
 * IMAGE of the descriptive grammar of the views (views.dtd).
 * <B>&lt;IMAGE&gt; : Image</B>
 * An image is used to have several icons associated to a tree node.
 * @see leon.view.LyComponent
 * See also tag <A href="../../conf/dtd/view/IMAGE.html">IMAGE</A> of grammar leon_views.dtd.
 */
public class LyImage extends LyComponent
	implements  LyGetValueInterface,
				LySetValueInterface
{
	public static final String			__VERSION = "$Id$";


	/* Attributes */

	/**
	 * The image.
	 */
	protected String			_value;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_value = null;
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyImage	model = (LyImage)component;

		_model = component;
		_value = model._value;
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	protected LyImage()
	{
	}

	/**
	 * Constructor with all required attributes.
	 * @param value The image.
	 */
	public LyImage(String value)
	{
		// Fill up required attributes with parameters
		_value = value;
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyImage(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_value = node.getString("value");
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyImage clone = new LyImage();

		clone.initFromModel(this);
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of value.
	 * @return value of value.
	 */
	public String getValue()
	{
		return _value;
	}


	/* Modifiers */

	/**
	 * Sets attribute value to the given value.
	 * @param value New value of value.
	 * @exception IllegalArgumentException if value is null.
	 */
	public void setValue(String value)
	{
		if (value == null)
			throw new IllegalArgumentException("BF AppEngine : attribute value is required (can't be null)");

		if ((value == _value)
			|| ((value != null)
				&& (_value != null)
				&& value.equals(_value)))
			return;

		_value = value;
		if (_internalView != null)
			((LyGraphicalImage)_internalView).setValue(value);
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<IMAGE");
		pw.print(" value='" + LyString.getXmlString(_value) + "'");
		pw.println("/>");
	}
}
