/*
 * LyIntervals.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;
import java.io.*;


/**
 * The LyIntervals class is a generated Java implementation that corresponds to the element 
 * INTERVALS of the descriptive grammar of the views (views.dtd).
 * <B>&lt;INTERVALS&gt; : Time unit for the length of the intervals of the variables to study, in the case of dates: YEARS | MONTHS | DAYS | HOURS | MINS | SECS By default, the unit is YEARS</B>
 * An INTERVALS element allows defining the series of intervals of the variables to
 * study and of the variable according to which they are studied.
 * This is useful only if the xVal attribute of CHART is X_INTERVALS or if the yVal
 * attribute of CHART is Y_COUNT_INTERVALS.
 * @see leon.view.LyComponent
 * See also tag <A href="../../conf/dtd/view/INTERVALS.html">INTERVALS</A> of grammar leon_views.dtd.
 */
public class LyIntervals extends LyComponent
{
	public static final String			__VERSION = "$Id$";


	/** Possible values for attribute 'xTimeUnit'. */
	public enum TimeUnit
	{
		YEARS,
		MONTHS,
		DAYS,
		HOURS,
		MINS,
		SECS;
	}


	/** Possible values for attribute 'yTimeUnit'. */


	/* Static methods */

	/**
	 * Utility method to get xTimeUnit value from a string.
	 * @param token Token string
	 * @return TimeUnit value corresponding to this token
	 */
	public static final TimeUnit getTimeUnit(String token)
	{
		if (token == null)
			return null;
		else if (token.equalsIgnoreCase("YEARS")
				|| token.equals("0"))
			return TimeUnit.YEARS;
		else if (token.equalsIgnoreCase("MONTHS")
				|| token.equals("1"))
			return TimeUnit.MONTHS;
		else if (token.equalsIgnoreCase("DAYS")
				|| token.equals("2"))
			return TimeUnit.DAYS;
		else if (token.equalsIgnoreCase("HOURS")
				|| token.equals("3"))
			return TimeUnit.HOURS;
		else if (token.equalsIgnoreCase("MINS")
				|| token.equals("4"))
			return TimeUnit.MINS;
		else if (token.equalsIgnoreCase("SECS")
				|| token.equals("5"))
			return TimeUnit.SECS;
		else
			return TimeUnit.YEARS;
	}


	/* Attributes */

	/**
	 * Limit of one (any) of the intervals of the variable X.
	 */
	protected double			_xBegin;

	/**
	 * Length of the intervals in X.
	 */
	protected float				_xWidth;

	/**
	 * Time unit for the length of the intervals in X, in the case of dates: YEARS | 
	 * MONTHS | DAYS | HOURS | MINS | SECS
	 */
	protected TimeUnit			_xTimeUnit;

	/**
	 * By default, the unit is YEARS
	 */
	protected double			_yBegin;

	/**
	 * Limit of one (any) of the intervals of the variables to study.
	 */
	protected float				_yWidth;

	/**
	 * Length of the intervals of the variables to study.
	 */
	protected TimeUnit			_yTimeUnit;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_xBegin = getDoubleValue("INTERVALS.xBegin", 0);
		_xWidth = getFloatValue("INTERVALS.xWidth", 0F);
		_xTimeUnit = TimeUnit.YEARS;
		_yBegin = getDoubleValue("INTERVALS.yBegin", 0);
		_yWidth = getFloatValue("INTERVALS.yWidth", 0F);
		_yTimeUnit = TimeUnit.YEARS;
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyIntervals	model = (LyIntervals)component;

		_model = component;
		_xBegin = model._xBegin;
		_xWidth = model._xWidth;
		_xTimeUnit = model._xTimeUnit;
		_yBegin = model._yBegin;
		_yWidth = model._yWidth;
		_yTimeUnit = model._yTimeUnit;
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	public LyIntervals()
	{
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyIntervals(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_xBegin = node.getDouble("xBegin");
		_xWidth = node.getFloat("xWidth");
		_xTimeUnit = getTimeUnit(node.getString("xTimeUnit"));
		_yBegin = node.getDouble("yBegin");
		_yWidth = node.getFloat("yWidth");
		_yTimeUnit = getTimeUnit(node.getString("yTimeUnit"));
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyIntervals clone = new LyIntervals();

		clone.initFromModel(this);
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of xBegin.
	 * @return value of xBegin.
	 */
	public double getXBegin()
	{
		return _xBegin;
	}

	/**
	 * Gets value of xWidth.
	 * @return value of xWidth.
	 */
	public float getXWidth()
	{
		return _xWidth;
	}

	/**
	 * Gets value of xTimeUnit.
	 * @return value of xTimeUnit.
	 */
	public TimeUnit getXTimeUnit()
	{
		return _xTimeUnit;
	}

	/**
	 * Gets value of yBegin.
	 * @return value of yBegin.
	 */
	public double getYBegin()
	{
		return _yBegin;
	}

	/**
	 * Gets value of yWidth.
	 * @return value of yWidth.
	 */
	public float getYWidth()
	{
		return _yWidth;
	}

	/**
	 * Gets value of yTimeUnit.
	 * @return value of yTimeUnit.
	 */
	public TimeUnit getYTimeUnit()
	{
		return _yTimeUnit;
	}


	/* Modifiers */

	/**
	 * Initializes attribute xBegin to the given value.
	 * @param xBegin New value of xBegin.
	 */
	public void setXBegin(double xBegin)
	{
		if (xBegin == _xBegin)
			return;
		_xBegin = xBegin;
	}

	/**
	 * Initializes attribute xWidth to the given value.
	 * @param xWidth New value of xWidth.
	 */
	public void setXWidth(float xWidth)
	{
		if (xWidth == _xWidth)
			return;
		_xWidth = xWidth;
	}

	/**
	 * Initializes attribute xTimeUnit to the given value.
	 * @param xTimeUnit New value of xTimeUnit.
	 * @exception IllegalArgumentException if xTimeUnit parameter is null.
	 */
	public void setXTimeUnit(TimeUnit xTimeUnit)
	{
		if (xTimeUnit == _xTimeUnit)
			return;
		if (xTimeUnit == null)
			throw new IllegalArgumentException("BF AppEngine : invalid value for attribute xTimeUnit");

		_xTimeUnit = xTimeUnit;
	}

	/**
	 * Initializes attribute yBegin to the given value.
	 * @param yBegin New value of yBegin.
	 */
	public void setYBegin(double yBegin)
	{
		if (yBegin == _yBegin)
			return;
		_yBegin = yBegin;
	}

	/**
	 * Initializes attribute yWidth to the given value.
	 * @param yWidth New value of yWidth.
	 */
	public void setYWidth(float yWidth)
	{
		if (yWidth == _yWidth)
			return;
		_yWidth = yWidth;
	}

	/**
	 * Initializes attribute yTimeUnit to the given value.
	 * @param yTimeUnit New value of yTimeUnit.
	 * @exception IllegalArgumentException if yTimeUnit parameter is null.
	 */
	public void setYTimeUnit(TimeUnit yTimeUnit)
	{
		if (yTimeUnit == _yTimeUnit)
			return;
		if (yTimeUnit == null)
			throw new IllegalArgumentException("BF AppEngine : invalid value for attribute yTimeUnit");

		_yTimeUnit = yTimeUnit;
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<INTERVALS");
		if (_xBegin != getDoubleValue("INTERVALS.xBegin", 0))
			pw.print(" xBegin='" + _xBegin + "'");
		if (_xWidth != getFloatValue("INTERVALS.xWidth", 0F))
			pw.print(" xWidth='" + _xWidth + "'");
		if (_xTimeUnit == TimeUnit.MONTHS)
			pw.print(" xTimeUnit='MONTHS'");
		else if (_xTimeUnit == TimeUnit.DAYS)
			pw.print(" xTimeUnit='DAYS'");
		else if (_xTimeUnit == TimeUnit.HOURS)
			pw.print(" xTimeUnit='HOURS'");
		else if (_xTimeUnit == TimeUnit.MINS)
			pw.print(" xTimeUnit='MINS'");
		else if (_xTimeUnit == TimeUnit.SECS)
			pw.print(" xTimeUnit='SECS'");
		if (_yBegin != getDoubleValue("INTERVALS.yBegin", 0))
			pw.print(" yBegin='" + _yBegin + "'");
		if (_yWidth != getFloatValue("INTERVALS.yWidth", 0F))
			pw.print(" yWidth='" + _yWidth + "'");
		if (_yTimeUnit == TimeUnit.MONTHS)
			pw.print(" yTimeUnit='MONTHS'");
		else if (_yTimeUnit == TimeUnit.DAYS)
			pw.print(" yTimeUnit='DAYS'");
		else if (_yTimeUnit == TimeUnit.HOURS)
			pw.print(" yTimeUnit='HOURS'");
		else if (_yTimeUnit == TimeUnit.MINS)
			pw.print(" yTimeUnit='MINS'");
		else if (_yTimeUnit == TimeUnit.SECS)
			pw.print(" yTimeUnit='SECS'");
		pw.println("/>");
	}
}
