/*
 * LyLine.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;

import java.util.*;
import java.io.*;
import leon.view.graphic.*;
import leon.view.getinterface.*;
import leon.view.setinterface.*;
import leon.misc.LyString;


/**
 * The LyLine class is a generated Java implementation that corresponds to the element 
 * LINE of the descriptive grammar of the views (views.dtd).
 * <B>&lt;LINE&gt; : Line of a planning</B>
 * This element represents a line of a gantt.
 * @see leon.view.LyComponent
 * See also tag <A href="../../conf/dtd/view/LINE.html">LINE</A> of grammar leon_views.dtd.
 */
public class LyLine extends LyComponent
	implements  LyGetIdInterface,
				LyGetLabelInterface,
				LySetLabelInterface,
				LySetImageInterface,
				LySetSelectedInterface,
				LySetHiddenInterface,
				LySetFontInterface,
				LySetColorInterface
{
	public static final String			__VERSION = "$Id$";


	/* Attributes */

	/**
	 * Identifier of the line.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyLine			_template;

	/**
	 * Label of the line in the tree view.
	 */
	protected String			_label;

	/**
	 * Image used to represent the line in the tree view.
	 */
	protected String			_image;

	/**
	 * Tooltip of the line.
	 */
	protected String			_tooltip;

	/**
	 * Indicates whether the line is selected or not.
	 */
	protected boolean			_selected;

	/**
	 * Indicates whether the line is open or not in the tree view (only if line has 
	 * sub-lines).
	 */
	protected boolean			_open;

	/**
	 * Indicates whether the line is hidden or not.
	 */
	protected boolean			_hidden;

	/**
	 * Font used to represent the line in the tree view.
	 */
	protected String			_font;

	/**
	 * Background used to represent the line in the tree view.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	/**
	 * Color used to represent the line in the planning view (only if it has 
	 * sub-lines).
	 */
	protected String			_superLineColor;

	/**
	 * Label of the line in the planning view (only if it has sub-lines).
	 */
	protected String			_superLineLabel;

	/**
	 * Indicates whether the line is hidden or not in the planning view (only if it is 
	 * a super line ie if it has sub-lines).
	 */
	protected boolean			_superLineHidden;

	/**
	 * Indicates whether the line may be hidden by the button?Hide?
	 */
	protected boolean			_allowHiding;


	/* Sub components */

	/**
	 * Components of line : either intervals or sub-lines.
	 */
	protected ArrayList<LyComponent>		_components;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_id = null;
		_template = null;
		_label = null;
		_image = null;
		_tooltip = null;
		_selected = getBooleanValue("LINE.selected", false);
		_open = getBooleanValue("LINE.open", false);
		_hidden = getBooleanValue("LINE.hidden", false);
		_font = getStringValue("LINE.font", "$ALL.font");
		_color = getStringValue("LINE.color", "$ALL.color");
		_fontColor = getStringValue("LINE.fontColor", "$ALL.fontColor");
		_superLineColor = getStringValue("LINE.superLineColor", "$ALL.superLineColor");
		_superLineLabel = null;
		_superLineHidden = getBooleanValue("LINE.superLineHidden", false);
		_allowHiding = getBooleanValue("LINE.allowHiding", false);

		// Initialize sub components
		_components = new ArrayList<LyComponent>(1);
		_children = _components;
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyLine	model = (LyLine)component;

		_model = component;
		_label = model._label;
		_image = model._image;
		_tooltip = model._tooltip;
		_selected = model._selected;
		_open = model._open;
		_hidden = model._hidden;
		_font = model._font;
		_color = model._color;
		_fontColor = model._fontColor;
		_superLineColor = model._superLineColor;
		_superLineLabel = model._superLineLabel;
		_superLineHidden = model._superLineHidden;
		_allowHiding = model._allowHiding;

		if (REWRITING_MODE)
			return;

		// Copy sub-components
		LyComponent	components;
		int			i, n;

		n = model._components.size();
		for (i = 0; i < n; i++)
		{
			components = model._components.get(i);
			if (components != null)
				components = components.cloneComponent();
			addChild(components);
		}
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	public LyLine()
	{
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyLine(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_id = node.getString("id");
		_template = (LyLine)node.getComponent("template");

		if (_template != null)
			initFromModel(_template);

		if (node.isSpecified("label"))
			_label = node.getString("label");
		if (node.isSpecified("image"))
			_image = node.getString("image");
		if (node.isSpecified("tooltip"))
			_tooltip = node.getString("tooltip");
		if (node.isSpecified("selected"))
			_selected = node.getBoolean("selected");
		if (node.isSpecified("open"))
			_open = node.getBoolean("open");
		if (node.isSpecified("hidden"))
			_hidden = node.getBoolean("hidden");
		if (node.isSpecified("font"))
			_font = node.getString("font");
		if (node.isSpecified("color"))
			_color = node.getString("color");
		if (node.isSpecified("fontColor"))
			_fontColor = node.getString("fontColor");
		if (node.isSpecified("superLineColor"))
			_superLineColor = node.getString("superLineColor");
		if (node.isSpecified("superLineLabel"))
			_superLineLabel = node.getString("superLineLabel");
		if (node.isSpecified("superLineHidden"))
			_superLineHidden = node.getBoolean("superLineHidden");
		if (node.isSpecified("allowHiding"))
			_allowHiding = node.getBoolean("allowHiding");
	}

	/**
	 * End of construction : completion with child component.
	 * @param child Sub child component.
	 * @see leon.view.LyComponent#addChild
	 */
	protected void addChild(LyComponent child)
	{
		// Add sub-components
		if (child instanceof LyInterval)
			addInterval((LyInterval)child);
		else if (child instanceof LyLine)
			addLine((LyLine)child);
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyLine clone = new LyLine();

		clone.initFromModel(this);
		clone._template = this;
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of id.
	 * @return value of id.
	 */
	public String getId()
	{
		return _id;
	}

	/**
	 * Gets value of template.
	 * @return value of template.
	 */
	public LyLine getTemplate()
	{
		return _template;
	}

	/**
	 * Gets value of label.
	 * @return value of label.
	 */
	public String getLabel()
	{
		return _label;
	}

	/**
	 * Gets value of image.
	 * @return value of image.
	 */
	public String getImage()
	{
		return _image;
	}

	/**
	 * Gets value of tooltip.
	 * @return value of tooltip.
	 */
	public String getTooltip()
	{
		return _tooltip;
	}

	/**
	 * Checks if value of attribute selected is set.
	 * @return true if selected is set, false otherwise.
	 */
	public boolean isSelected()
	{
		return _selected;
	}

	/**
	 * Checks if value of attribute open is set.
	 * @return true if open is set, false otherwise.
	 */
	public boolean isOpen()
	{
		return _open;
	}

	/**
	 * Checks if value of attribute hidden is set.
	 * @return true if hidden is set, false otherwise.
	 */
	public boolean isHidden()
	{
		return _hidden;
	}

	/**
	 * Gets value of font.
	 * @return value of font.
	 */
	public String getFont()
	{
		return _font;
	}

	/**
	 * Checks whether the font attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFont()
	{
		return (_font == getStringValue("LINE.font"));
	}

	/**
	 * Gets value of color.
	 * @return value of color.
	 */
	public String getColor()
	{
		return _color;
	}

	/**
	 * Checks whether the color attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultColor()
	{
		return (_color == getStringValue("LINE.color"));
	}

	/**
	 * Gets value of fontColor.
	 * @return value of fontColor.
	 */
	public String getFontColor()
	{
		return _fontColor;
	}

	/**
	 * Checks whether the fontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFontColor()
	{
		return (_fontColor == getStringValue("LINE.fontColor"));
	}

	/**
	 * Gets value of superLineColor.
	 * @return value of superLineColor.
	 */
	public String getSuperLineColor()
	{
		return _superLineColor;
	}

	/**
	 * Checks whether the superLineColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultSuperLineColor()
	{
		return (_superLineColor == getStringValue("LINE.superLineColor"));
	}

	/**
	 * Gets value of superLineLabel.
	 * @return value of superLineLabel.
	 */
	public String getSuperLineLabel()
	{
		return _superLineLabel;
	}

	/**
	 * Checks if value of attribute superLineHidden is set.
	 * @return true if superLineHidden is set, false otherwise.
	 */
	public boolean isSuperLineHidden()
	{
		return _superLineHidden;
	}

	/**
	 * Checks if value of attribute allowHiding is set.
	 * @return true if allowHiding is set, false otherwise.
	 */
	public boolean isAllowHiding()
	{
		return _allowHiding;
	}

	/**
	 * Get sub-component Component of index i.
	 * @param i the index of the Component to get.
	 * @return sub-component Component of index i.
	 */
	public LyComponent getComponent(int i)
	{
		if ((i < 0) || (i >= _components.size()))
			return null;
		return _components.get(i);
	}

	/**
	 * Get sub-component Component count.
	 * @return sub-component Component count.
	 */
	public int getComponentCount()
	{
		return _components.size();
	}


	/* Modifiers */

	/**
	 * Initializes attribute id to the given value.
	 * @param id New value of id.
	 */
	public void setId(String id)
	{
		if ((id == _id)
			|| ((id != null)
				&& (_id != null)
				&& id.equals(_id)))
			return;

		_id = (id != null ? id.intern() : null);
	}

	/**
	 * Initializes attribute template to the given value.
	 * @param template New value of template.
	 */
	public void setTemplate(LyLine template)
	{
		if (template == null)
			return;
		if (_template != null)
			throw new IllegalStateException("BF AppEngine : attribute template was already set.");

		initFromModel(template);
		_template = template;
	}

	/**
	 * Sets attribute label to the given value.
	 * @param label New value of label.
	 */
	public void setLabel(String label)
	{
		if ((label == _label)
			|| ((label != null)
				&& (_label != null)
				&& label.equals(_label)))
			return;

		_label = label;
		if (_internalView != null)
			((LyGraphicalLine)_internalView).setLabel(label);
	}

	/**
	 * Sets attribute image to the given value.
	 * @param image New value of image.
	 */
	public void setImage(String image)
	{
		if ((image == _image)
			|| ((image != null)
				&& (_image != null)
				&& image.equals(_image)))
			return;

		_image = image;
		if (_internalView != null)
			((LyGraphicalLine)_internalView).setImage(image);
	}

	/**
	 * Sets attribute tooltip to the given value.
	 * @param tooltip New value of tooltip.
	 */
	public void setTooltip(String tooltip)
	{
		if ((tooltip == _tooltip)
			|| ((tooltip != null)
				&& (_tooltip != null)
				&& tooltip.equals(_tooltip)))
			return;

		_tooltip = tooltip;
		if (_internalView != null)
			((LyGraphicalLine)_internalView).setTooltip(tooltip);
	}

	/**
	 * Sets attribute selected to the given value.
	 * @param selected Boolean indicating if attribute selected is set or unset.
	 */
	public void setSelected(boolean selected)
	{
		if (selected == _selected)
			return;
		_selected = selected;
		if (_internalView != null)
			((LyGraphicalLine)_internalView).setSelected(selected);
	}

	/**
	 * Sets attribute open to the given value.
	 * @param open Boolean indicating if attribute open is set or unset.
	 */
	public void setOpen(boolean open)
	{
		if (open == _open)
			return;
		_open = open;
		if (_internalView != null)
			((LyGraphicalLine)_internalView).setOpen(open);
	}

	/**
	 * Sets attribute hidden to the given value.
	 * @param hidden Boolean indicating if attribute hidden is set or unset.
	 */
	public void setHidden(boolean hidden)
	{
		if (hidden == _hidden)
			return;
		_hidden = hidden;
		if (_internalView != null)
			((LyGraphicalLine)_internalView).setHidden(hidden);
	}

	/**
	 * Sets attribute font to the given value.
	 * @param font New value of font.
	 */
	public void setFont(String font)
	{
		if ((font == _font)
			|| ((font != null)
				&& (_font != null)
				&& font.equals(_font)))
			return;

		_font = font;
		if (_internalView != null)
			((LyGraphicalLine)_internalView).setFont(font);
	}

	/**
	 * Sets attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color)
	{
		if ((color == _color)
			|| ((color != null)
				&& (_color != null)
				&& color.equals(_color)))
			return;

		_color = color;
		if (_internalView != null)
			((LyGraphicalLine)_internalView).setColor(color);
	}

	/**
	 * Initializes attribute fontColor to the given value.
	 * @param fontColor New value of fontColor.
	 */
	public void setFontColor(String fontColor)
	{
		if ((fontColor == _fontColor)
			|| ((fontColor != null)
				&& (_fontColor != null)
				&& fontColor.equals(_fontColor)))
			return;

		_fontColor = fontColor;
	}

	/**
	 * Sets attribute superLineColor to the given value.
	 * @param superLineColor New value of superLineColor.
	 */
	public void setSuperLineColor(String superLineColor)
	{
		if ((superLineColor == _superLineColor)
			|| ((superLineColor != null)
				&& (_superLineColor != null)
				&& superLineColor.equals(_superLineColor)))
			return;

		_superLineColor = superLineColor;
		if (_internalView != null)
			((LyGraphicalLine)_internalView).setSuperLineColor(superLineColor);
	}

	/**
	 * Sets attribute superLineLabel to the given value.
	 * @param superLineLabel New value of superLineLabel.
	 */
	public void setSuperLineLabel(String superLineLabel)
	{
		if ((superLineLabel == _superLineLabel)
			|| ((superLineLabel != null)
				&& (_superLineLabel != null)
				&& superLineLabel.equals(_superLineLabel)))
			return;

		_superLineLabel = superLineLabel;
		if (_internalView != null)
			((LyGraphicalLine)_internalView).setSuperLineLabel(superLineLabel);
	}

	/**
	 * Sets attribute superLineHidden to the given value.
	 * @param superLineHidden Boolean indicating if attribute superLineHidden is set or unset.
	 */
	public void setSuperLineHidden(boolean superLineHidden)
	{
		if (superLineHidden == _superLineHidden)
			return;
		_superLineHidden = superLineHidden;
		if (_internalView != null)
			((LyGraphicalLine)_internalView).setSuperLineHidden(superLineHidden);
	}

	/**
	 * Initializes attribute allowHiding to the given value.
	 * @param allowHiding Boolean indicating if attribute allowHiding is set or unset.
	 */
	public void setAllowHiding(boolean allowHiding)
	{
		if (allowHiding == _allowHiding)
			return;
		_allowHiding = allowHiding;
	}

	/**
	 * Adds a sub-component interval.
	 * @param interval The interval to add to components list.
	 */
	public void addInterval(LyInterval interval)
	{
		insertInterval(interval, -1);
	}

	/**
	 * Inserts a sub-component interval at given position.
	 * @param interval The interval to add to components list.
	 * @param pos	Position where the sub-component is inserted.
	 */
	public void insertInterval(LyInterval interval, int pos)
	{
		if (pos >= 0)
			_components.add(pos,interval);
		if (interval != null)
			interval.setParent(this);
		if (_internalView != null)
			((LyGraphicalLine)_internalView).addComponent(interval, pos);
	}

	/**
	 * Removes a sub-component interval.
	 * @param interval The interval to remove to components list.
	 * @exception IllegalStateException if components list is empty.
	 * @exception IllegalArgumentException if components list doesn't contain interval.
	 */
	public void removeInterval(LyInterval interval)
	{
		if (!_components.contains(interval))
			throw new IllegalArgumentException("BF AppEngine : components list doesn't contain interval");

		if (_internalView != null)
			((LyGraphicalLine)_internalView).removeComponent(interval);
		interval.setParent(null);
	}

	/**
	 * Adds a sub-component line.
	 * @param line The line to add to components list.
	 */
	public void addLine(LyLine line)
	{
		insertLine(line, -1);
	}

	/**
	 * Inserts a sub-component line at given position.
	 * @param line The line to add to components list.
	 * @param pos	Position where the sub-component is inserted.
	 */
	public void insertLine(LyLine line, int pos)
	{
		if (pos >= 0)
			_components.add(pos,line);
		if (line != null)
			line.setParent(this);
		if (_internalView != null)
			((LyGraphicalLine)_internalView).addComponent(line, pos);
	}

	/**
	 * Removes a sub-component line.
	 * @param line The line to remove to components list.
	 * @exception IllegalStateException if components list is empty.
	 * @exception IllegalArgumentException if components list doesn't contain line.
	 */
	public void removeLine(LyLine line)
	{
		if (!_components.contains(line))
			throw new IllegalArgumentException("BF AppEngine : components list doesn't contain line");

		if (_internalView != null)
			((LyGraphicalLine)_internalView).removeComponent(line);
		line.setParent(null);
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<LINE");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template._label)))
			pw.print(" label='" + LyString.getXmlString(_label) + "'");
		if (((_template == null) && (_image != null))
				|| ((_template != null) && (_image != _template._image)))
			pw.print(" image='" + LyString.getXmlString(_image) + "'");
		if (((_template == null) && (_tooltip != null))
				|| ((_template != null) && (_tooltip != _template._tooltip)))
			pw.print(" tooltip='" + LyString.getXmlString(_tooltip) + "'");
		if (((_template == null) && _selected)
				|| ((_template != null) && (_selected != _template._selected)))
			pw.print(" selected='" + _selected + "'");
		if (((_template == null) && _open)
				|| ((_template != null) && (_open != _template._open)))
			pw.print(" open='" + _open + "'");
		if (((_template == null) && _hidden)
				|| ((_template != null) && (_hidden != _template._hidden)))
			pw.print(" hidden='" + _hidden + "'");
		if (((_template == null) && (_font != getStringValue("LINE.font", "$ALL.font")))
				|| ((_template != null) && (_font != _template._font)))
			pw.print(" font='" + LyString.getXmlString(_font) + "'");
		if (((_template == null) && (_color != getStringValue("LINE.color", "$ALL.color")))
				|| ((_template != null) && (_color != _template._color)))
			pw.print(" color='" + LyString.getXmlString(_color) + "'");
		if (((_template == null) && (_fontColor != getStringValue("LINE.fontColor", "$ALL.fontColor")))
				|| ((_template != null) && (_fontColor != _template._fontColor)))
			pw.print(" fontColor='" + LyString.getXmlString(_fontColor) + "'");
		if (((_template == null) && (_superLineColor != getStringValue("LINE.superLineColor", "$ALL.superLineColor")))
				|| ((_template != null) && (_superLineColor != _template._superLineColor)))
			pw.print(" superLineColor='" + LyString.getXmlString(_superLineColor) + "'");
		if (((_template == null) && (_superLineLabel != null))
				|| ((_template != null) && (_superLineLabel != _template._superLineLabel)))
			pw.print(" superLineLabel='" + LyString.getXmlString(_superLineLabel) + "'");
		if (((_template == null) && _superLineHidden)
				|| ((_template != null) && (_superLineHidden != _template._superLineHidden)))
			pw.print(" superLineHidden='" + _superLineHidden + "'");
		if (((_template == null) && _allowHiding)
				|| ((_template != null) && (_allowHiding != _template._allowHiding)))
			pw.print(" allowHiding='" + _allowHiding + "'");

		if ((getChildCount() == 0) && (_datas == null))
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		LyComponent	components;
		int			i, n;

		n = _components.size();
		for (i = 0; i < n; i++)
		{
			components = _components.get(i);
			components.writeXml(pw, debug);
		}


		pw.println("</LINE>");
	}
}
