/*
 * LyLink.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;
import java.io.*;
import leon.view.graphic.*;
import leon.view.getinterface.*;
import leon.view.setinterface.*;
import leon.misc.LyString;


/**
 * The LyLink class is a generated Java implementation that corresponds to the element 
 * LINK of the descriptive grammar of the views (views.dtd).
 * <B>&lt;LINK&gt; : Link</B>
 * A link represents a possible value in an input field of Relation type.
 * @see leon.view.LyComponent
 * See also tag <A href="../../conf/dtd/view/LINK.html">LINK</A> of grammar leon_views.dtd.
 */
public class LyLink extends LyComponent
	implements  LyGetIdInterface,
				LyGetLabelInterface,
				LySetSelectedInterface
{
	public static final String			__VERSION = "$Id$";


	/* Attributes */

	/**
	 * Identifier associated to the link.
	 * It represents the internal value of the application that is associated to the 
	 * link.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyLink			_template;

	/**
	 * Short label of the link.
	 */
	protected String			_shortLabel;

	/**
	 * Label of the link.
	 */
	protected String			_label;

	/**
	 * Indicates whether the link is selected or not.
	 */
	protected boolean			_selected;

	/**
	 * Image associated to the link.
	 */
	protected String			_image;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_id = null;
		_template = null;
		_shortLabel = null;
		_label = null;
		_selected = getBooleanValue("LINK.selected", false);
		_image = getStringValue("LINK.image", "$ALL.image");
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyLink	model = (LyLink)component;

		_model = component;
		_shortLabel = model._shortLabel;
		_label = model._label;
		_selected = model._selected;
		_image = model._image;
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	protected LyLink()
	{
		_id = getInternalId();
	}

	/**
	 * Constructor with all required attributes.
	 * @param id Identifier associated to the link.<BR>It represents the internal value of the application that is associated to the link.
	 * @param label Label of the link.
	 */
	public LyLink(String id, String label)
	{
		// Fill up required attributes with parameters
		_id = (id != null ? id.intern() : null);
		_label = label;
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyLink(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_id = node.getString("id");
		_template = (LyLink)node.getComponent("template");

		if (_template != null)
			initFromModel(_template);

		if (node.isSpecified("shortLabel"))
			_shortLabel = node.getString("shortLabel");
		if (node.isSpecified("label"))
			_label = node.getString("label");
		if (node.isSpecified("selected"))
			_selected = node.getBoolean("selected");
		if (node.isSpecified("image"))
			_image = node.getString("image");
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyLink clone = new LyLink();

		clone.initFromModel(this);
		clone._template = this;
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of id.
	 * @return value of id.
	 */
	public String getId()
	{
		return _id;
	}

	/**
	 * Gets value of template.
	 * @return value of template.
	 */
	public LyLink getTemplate()
	{
		return _template;
	}

	/**
	 * Gets value of shortLabel.
	 * @return value of shortLabel.
	 */
	public String getShortLabel()
	{
		return _shortLabel;
	}

	/**
	 * Gets value of label.
	 * @return value of label.
	 */
	public String getLabel()
	{
		return _label;
	}

	/**
	 * Checks if value of attribute selected is set.
	 * @return true if selected is set, false otherwise.
	 */
	public boolean isSelected()
	{
		return _selected;
	}

	/**
	 * Gets value of image.
	 * @return value of image.
	 */
	public String getImage()
	{
		return _image;
	}

	/**
	 * Checks whether the image attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultImage()
	{
		return (_image == getStringValue("LINK.image"));
	}


	/* Modifiers */

	/**
	 * Initializes attribute template to the given value.
	 * @param template New value of template.
	 */
	public void setTemplate(LyLink template)
	{
		if (template == null)
			return;
		if (_template != null)
			throw new IllegalStateException("BF AppEngine : attribute template was already set.");

		initFromModel(template);
		_template = template;
	}

	/**
	 * Initializes attribute shortLabel to the given value.
	 * @param shortLabel New value of shortLabel.
	 */
	public void setShortLabel(String shortLabel)
	{
		if ((shortLabel == _shortLabel)
			|| ((shortLabel != null)
				&& (_shortLabel != null)
				&& shortLabel.equals(_shortLabel)))
			return;

		_shortLabel = shortLabel;
	}

	/**
	 * Initializes attribute label to the given value.
	 * @param label New value of label.
	 * @exception IllegalArgumentException if label is null.
	 */
	public void setLabel(String label)
	{
		if (label == null)
			throw new IllegalArgumentException("BF AppEngine : attribute label is required (can't be null)");

		if ((label == _label)
			|| ((label != null)
				&& (_label != null)
				&& label.equals(_label)))
			return;

		_label = label;
	}

	/**
	 * Sets attribute selected to the given value.
	 * @param selected Boolean indicating if attribute selected is set or unset.
	 */
	public void setSelected(boolean selected)
	{
		if (selected == _selected)
			return;
		_selected = selected;
		if (_internalView != null)
			((LyGraphicalLink)_internalView).setSelected(selected);
	}

	/**
	 * Initializes attribute image to the given value.
	 * @param image New value of image.
	 */
	public void setImage(String image)
	{
		if ((image == _image)
			|| ((image != null)
				&& (_image != null)
				&& image.equals(_image)))
			return;

		_image = image;
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<LINK");
		pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && (_shortLabel != null))
				|| ((_template != null) && (_shortLabel != _template._shortLabel)))
			pw.print(" shortLabel='" + LyString.getXmlString(_shortLabel) + "'");
		pw.print(" label='" + LyString.getXmlString(_label) + "'");
		if (((_template == null) && _selected)
				|| ((_template != null) && (_selected != _template._selected)))
			pw.print(" selected='" + _selected + "'");
		if (((_template == null) && (_image != getStringValue("LINK.image", "$ALL.image")))
				|| ((_template != null) && (_image != _template._image)))
			pw.print(" image='" + LyString.getXmlString(_image) + "'");
		pw.println("/>");
	}
}
