/*
 * LyListOption.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;
import java.io.*;
import leon.view.graphic.*;
import leon.view.getinterface.*;
import leon.view.setinterface.*;
import leon.misc.LyString;


/**
 * The LyListOption class is a generated Java implementation that corresponds to the element 
 * LIST_OPTION of the descriptive grammar of the views (views.dtd).
 * <B>&lt;LIST_OPTION&gt; : List option</B>
 * A list option represents a possible value in an input field of List type.
 * @see leon.view.LyComponent
 * See also tag <A href="../../conf/dtd/view/LIST_OPTION.html">LIST_OPTION</A> of grammar leon_views.dtd.
 */
public class LyListOption extends LyComponent
	implements  LyGetIdInterface,
				LyGetLabelInterface,
				LySetSelectedInterface
{
	public static final String			__VERSION = "$Id$";


	/* Attributes */

	/**
	 * Identifier associated to the list option.
	 * It represents the internal value of the application that is associated to the 
	 * option.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyListOption			_template;

	/**
	 * Label of the list option.
	 */
	protected String			_label;

	/**
	 * Image of the list option.
	 */
	protected String			_image;

	/**
	 * Indicates whether the list option is selected or not.
	 */
	protected boolean			_selected;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_id = null;
		_template = null;
		_label = null;
		_image = getStringValue("LIST_OPTION.image", "$ALL.image");
		_selected = getBooleanValue("LIST_OPTION.selected", false);
		_font = getStringValue("LIST_OPTION.font", "$ALL.font");
		_color = getStringValue("LIST_OPTION.color", "$ALL.color");
		_fontColor = getStringValue("LIST_OPTION.fontColor", "$ALL.fontColor");
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyListOption	model = (LyListOption)component;

		_model = component;
		_label = model._label;
		_image = model._image;
		_selected = model._selected;
		_font = model._font;
		_color = model._color;
		_fontColor = model._fontColor;
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	protected LyListOption()
	{
		_id = getInternalId();
	}

	/**
	 * Constructor with all required attributes.
	 * @param id Identifier associated to the list option.<BR>It represents the internal value of the application that is associated to the option.
	 * @param label Label of the list option.
	 */
	public LyListOption(String id, String label)
	{
		// Fill up required attributes with parameters
		_id = (id != null ? id.intern() : null);
		_label = label;
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyListOption(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_id = node.getString("id");
		_template = (LyListOption)node.getComponent("template");

		if (_template != null)
			initFromModel(_template);

		if (node.isSpecified("label"))
			_label = node.getString("label");
		if (node.isSpecified("image"))
			_image = node.getString("image");
		if (node.isSpecified("selected"))
			_selected = node.getBoolean("selected");
		if (node.isSpecified("font"))
			_font = node.getString("font");
		if (node.isSpecified("color"))
			_color = node.getString("color");
		if (node.isSpecified("fontColor"))
			_fontColor = node.getString("fontColor");
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyListOption clone = new LyListOption();

		clone.initFromModel(this);
		clone._template = this;
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of id.
	 * @return value of id.
	 */
	public String getId()
	{
		return _id;
	}

	/**
	 * Gets value of template.
	 * @return value of template.
	 */
	public LyListOption getTemplate()
	{
		return _template;
	}

	/**
	 * Gets value of label.
	 * @return value of label.
	 */
	public String getLabel()
	{
		return _label;
	}

	/**
	 * Gets value of image.
	 * @return value of image.
	 */
	public String getImage()
	{
		return _image;
	}

	/**
	 * Checks whether the image attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultImage()
	{
		return (_image == getStringValue("LIST_OPTION.image"));
	}

	/**
	 * Checks if value of attribute selected is set.
	 * @return true if selected is set, false otherwise.
	 */
	public boolean isSelected()
	{
		return _selected;
	}

	/**
	 * Gets value of font.
	 * @return value of font.
	 */
	public String getFont()
	{
		return _font;
	}

	/**
	 * Checks whether the font attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFont()
	{
		return (_font == getStringValue("LIST_OPTION.font"));
	}

	/**
	 * Gets value of color.
	 * @return value of color.
	 */
	public String getColor()
	{
		return _color;
	}

	/**
	 * Checks whether the color attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultColor()
	{
		return (_color == getStringValue("LIST_OPTION.color"));
	}

	/**
	 * Gets value of fontColor.
	 * @return value of fontColor.
	 */
	public String getFontColor()
	{
		return _fontColor;
	}

	/**
	 * Checks whether the fontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFontColor()
	{
		return (_fontColor == getStringValue("LIST_OPTION.fontColor"));
	}


	/* Modifiers */

	/**
	 * Initializes attribute template to the given value.
	 * @param template New value of template.
	 */
	public void setTemplate(LyListOption template)
	{
		if (template == null)
			return;
		if (_template != null)
			throw new IllegalStateException("BF AppEngine : attribute template was already set.");

		initFromModel(template);
		_template = template;
	}

	/**
	 * Initializes attribute label to the given value.
	 * @param label New value of label.
	 * @exception IllegalArgumentException if label is null.
	 */
	public void setLabel(String label)
	{
		if (label == null)
			throw new IllegalArgumentException("BF AppEngine : attribute label is required (can't be null)");

		if ((label == _label)
			|| ((label != null)
				&& (_label != null)
				&& label.equals(_label)))
			return;

		_label = label;
	}

	/**
	 * Initializes attribute image to the given value.
	 * @param image New value of image.
	 */
	public void setImage(String image)
	{
		if ((image == _image)
			|| ((image != null)
				&& (_image != null)
				&& image.equals(_image)))
			return;

		_image = image;
	}

	/**
	 * Sets attribute selected to the given value.
	 * @param selected Boolean indicating if attribute selected is set or unset.
	 */
	public void setSelected(boolean selected)
	{
		if (selected == _selected)
			return;
		_selected = selected;
		if (_internalView != null)
			((LyGraphicalListOption)_internalView).setSelected(selected);
	}

	/**
	 * Initializes attribute font to the given value.
	 * @param font New value of font.
	 */
	public void setFont(String font)
	{
		if ((font == _font)
			|| ((font != null)
				&& (_font != null)
				&& font.equals(_font)))
			return;

		_font = font;
	}

	/**
	 * Initializes attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color)
	{
		if ((color == _color)
			|| ((color != null)
				&& (_color != null)
				&& color.equals(_color)))
			return;

		_color = color;
	}

	/**
	 * Initializes attribute fontColor to the given value.
	 * @param fontColor New value of fontColor.
	 */
	public void setFontColor(String fontColor)
	{
		if ((fontColor == _fontColor)
			|| ((fontColor != null)
				&& (_fontColor != null)
				&& fontColor.equals(_fontColor)))
			return;

		_fontColor = fontColor;
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<LIST_OPTION");
		pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		pw.print(" label='" + LyString.getXmlString(_label) + "'");
		if (((_template == null) && (_image != getStringValue("LIST_OPTION.image", "$ALL.image")))
				|| ((_template != null) && (_image != _template._image)))
			pw.print(" image='" + LyString.getXmlString(_image) + "'");
		if (((_template == null) && _selected)
				|| ((_template != null) && (_selected != _template._selected)))
			pw.print(" selected='" + _selected + "'");
		if (((_template == null) && (_font != getStringValue("LIST_OPTION.font", "$ALL.font")))
				|| ((_template != null) && (_font != _template._font)))
			pw.print(" font='" + LyString.getXmlString(_font) + "'");
		if (((_template == null) && (_color != getStringValue("LIST_OPTION.color", "$ALL.color")))
				|| ((_template != null) && (_color != _template._color)))
			pw.print(" color='" + LyString.getXmlString(_color) + "'");
		if (((_template == null) && (_fontColor != getStringValue("LIST_OPTION.fontColor", "$ALL.fontColor")))
				|| ((_template != null) && (_fontColor != _template._fontColor)))
			pw.print(" fontColor='" + LyString.getXmlString(_fontColor) + "'");
		pw.println("/>");
	}
}
