/*
 * LyMapContent.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;

import java.util.*;
import java.io.*;
import leon.view.graphic.*;


/**
 * The LyMapContent class is a generated Java implementation that corresponds to the element 
 * MAP_CONTENT of the descriptive grammar of the views (views.dtd).
 * <B>&lt;MAP_CONTENT&gt; : Map content</B>
 * A map content allows grouping together in a same group various elements that can
 * be placed on a map (nodes, links, images, regions).
 * @see leon.view.LyComponent
 * See also tag <A href="../../conf/dtd/view/MAP_CONTENT.html">MAP_CONTENT</A> of grammar leon_views.dtd.
 */
public class LyMapContent extends LyComponent
{
	public static final String			__VERSION = "$Id$";


	/* Sub components */

	/**
	 * Components of the map content. These components can be map nodes, map links or 
	 * map regions.
	 */
	protected ArrayList<LyComponent>		_components;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize sub components
		_components = null;
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;


		if (REWRITING_MODE)
			return;

		// Copy sub-components
		LyComponent	components;
		LyMapContent	model = (LyMapContent)component;

		int			i, n;

		n = (model._components == null ? 0 : model._components.size());
		for (i = 0; i < n; i++)
		{
			components = model._components.get(i);
			if (components != null)
				components = components.cloneComponent();
			addChild(components);
		}
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	public LyMapContent()
	{
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyMapContent(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
	}

	/**
	 * End of construction : completion with child component.
	 * @param child Sub child component.
	 * @see leon.view.LyComponent#addChild
	 */
	protected void addChild(LyComponent child)
	{
		// Add sub-components
		if (child instanceof LyMapNode)
			addMapNode((LyMapNode)child);
		else if (child instanceof LyMapLink)
			addMapLink((LyMapLink)child);
		else if (child instanceof LyMapRegion)
			addMapRegion((LyMapRegion)child);
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyMapContent clone = new LyMapContent();

		clone.initFromModel(this);
		return clone;
	}


	/* Accessors */

	/**
	 * Get sub-component Component of index i.
	 * @param i the index of the Component to get.
	 * @return sub-component Component of index i.
	 */
	public LyComponent getComponent(int i)
	{
		if (_components == null)
			return null;
		if ((i < 0) || (i >= _components.size()))
			return null;
		return _components.get(i);
	}

	/**
	 * Get sub-component Component count.
	 * @return sub-component Component count.
	 */
	public int getComponentCount()
	{
		if (_components == null)
			return 0;
		return _components.size();
	}


	/* Modifiers */

	/**
	 * Adds a sub-component mapNode.
	 * @param mapNode The mapNode to add to components list.
	 */
	public void addMapNode(LyMapNode mapNode)
	{
		insertMapNode(mapNode, -1);
	}

	/**
	 * Inserts a sub-component mapNode at given position.
	 * @param mapNode The mapNode to add to components list.
	 * @param pos	Position where the sub-component is inserted.
	 */
	public void insertMapNode(LyMapNode mapNode, int pos)
	{
		if (_components == null)
		{
			_components = new ArrayList<LyComponent>(1);
			_children = _components;
		}
		else if (_components.contains(mapNode))
			return;
		if (pos >= 0)
			_components.add(pos,mapNode);
		if (mapNode != null)
			mapNode.setParent(this);
		if (_internalView != null)
			((LyGraphicalMapContent)_internalView).addComponent(mapNode, pos);
	}

	/**
	 * Removes a sub-component mapNode.
	 * @param mapNode The mapNode to remove to components list.
	 * @exception IllegalStateException if components list is empty.
	 * @exception IllegalArgumentException if components list doesn't contain mapNode.
	 */
	public void removeMapNode(LyMapNode mapNode)
	{
		if (_components == null)
			throw new IllegalStateException("BF AppEngine : components list is empty");
		if (!_components.contains(mapNode))
			throw new IllegalArgumentException("BF AppEngine : components list doesn't contain mapNode");

		if (_internalView != null)
			((LyGraphicalMapContent)_internalView).removeComponent(mapNode);
		mapNode.setParent(null);
	}

	/**
	 * Adds a sub-component mapLink.
	 * @param mapLink The mapLink to add to components list.
	 */
	public void addMapLink(LyMapLink mapLink)
	{
		insertMapLink(mapLink, -1);
	}

	/**
	 * Inserts a sub-component mapLink at given position.
	 * @param mapLink The mapLink to add to components list.
	 * @param pos	Position where the sub-component is inserted.
	 */
	public void insertMapLink(LyMapLink mapLink, int pos)
	{
		if (_components == null)
		{
			_components = new ArrayList<LyComponent>(1);
			_children = _components;
		}
		else if (_components.contains(mapLink))
			return;
		if (pos >= 0)
			_components.add(pos,mapLink);
		if (mapLink != null)
			mapLink.setParent(this);
		if (_internalView != null)
			((LyGraphicalMapContent)_internalView).addComponent(mapLink, pos);
	}

	/**
	 * Removes a sub-component mapLink.
	 * @param mapLink The mapLink to remove to components list.
	 * @exception IllegalStateException if components list is empty.
	 * @exception IllegalArgumentException if components list doesn't contain mapLink.
	 */
	public void removeMapLink(LyMapLink mapLink)
	{
		if (_components == null)
			throw new IllegalStateException("BF AppEngine : components list is empty");
		if (!_components.contains(mapLink))
			throw new IllegalArgumentException("BF AppEngine : components list doesn't contain mapLink");

		if (_internalView != null)
			((LyGraphicalMapContent)_internalView).removeComponent(mapLink);
		mapLink.setParent(null);
	}

	/**
	 * Adds a sub-component mapRegion.
	 * @param mapRegion The mapRegion to add to components list.
	 */
	public void addMapRegion(LyMapRegion mapRegion)
	{
		insertMapRegion(mapRegion, -1);
	}

	/**
	 * Inserts a sub-component mapRegion at given position.
	 * @param mapRegion The mapRegion to add to components list.
	 * @param pos	Position where the sub-component is inserted.
	 */
	public void insertMapRegion(LyMapRegion mapRegion, int pos)
	{
		if (_components == null)
		{
			_components = new ArrayList<LyComponent>(1);
			_children = _components;
		}
		else if (_components.contains(mapRegion))
			return;
		if (pos >= 0)
			_components.add(pos,mapRegion);
		if (mapRegion != null)
			mapRegion.setParent(this);
		if (_internalView != null)
			((LyGraphicalMapContent)_internalView).addComponent(mapRegion, pos);
	}

	/**
	 * Removes a sub-component mapRegion.
	 * @param mapRegion The mapRegion to remove to components list.
	 * @exception IllegalStateException if components list is empty.
	 * @exception IllegalArgumentException if components list doesn't contain mapRegion.
	 */
	public void removeMapRegion(LyMapRegion mapRegion)
	{
		if (_components == null)
			throw new IllegalStateException("BF AppEngine : components list is empty");
		if (!_components.contains(mapRegion))
			throw new IllegalArgumentException("BF AppEngine : components list doesn't contain mapRegion");

		if (_internalView != null)
			((LyGraphicalMapContent)_internalView).removeComponent(mapRegion);
		mapRegion.setParent(null);
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag
		pw.println("<MAP_CONTENT>");

		// Write sub-components
		LyComponent	components;
		int			i, n;

		n = (_components == null ? 0 : _components.size());
		for (i = 0; i < n; i++)
		{
			components = _components.get(i);
			components.writeXml(pw, debug);
		}


		pw.println("</MAP_CONTENT>");
	}
}
