/*
 * LyMapRegion.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved.
 */

package leon.view;

import leon.misc.*;
import leon.view.common.*;

import java.util.*;


/**
 * This class represents a MapRegion component.
 *
 * @author   Lyria
 * @version  $Id: LyMapRegion.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @see      LyComponent
 */
public class LyMapRegion extends LyMapRegionBase
{
	public static final String __VERSION = "$Id: LyMapRegion.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Constructor with all required attributes.
	 */
	public LyMapRegion()
	{
	}

	/**
	 * Construction from a LyViewNode.
	 *
	 * @param  node  Node from which the component is constructed
	 *
	 * @see    LyViewNodeFactory#buildElement
	 */
	protected LyMapRegion(LyViewNode node)
	{
		super(node);
	}

	/**
	 * Create the polygons for a map region with the file attribute.
	 */
	public void loadPolygonsFile(LyEnvironment environment)
	{
		String file = getFile();

		// Check if a file is given
		if ((file == null) || (file.length() == 0))
			return;

		Vector<LyPolygon> polygons = new Vector<LyPolygon>();
		int				  i, n;
		LyPolygon		  polygon;

		// Check which type of file is given
		if (file.endsWith(".xml"))
			polygons = buildPolygonsFromXml(environment, file);
		else if (file.endsWith(".dxf"))
			polygons = buildPolygonsFromDxf(environment, file);
		else
			return;

		n = polygons.size();

		// Add the polygons to the region and reset the file attribute
		for (i = 0; i < n; i++)
		{
			polygon = polygons.elementAt(i);
			addPolygon(polygon);
		}

		setFile("");
	} // end method loadPolygonsFile

	/**
	 * Builds a polygon object from an xml file.
	 */
	protected Vector<LyPolygon> buildPolygonsFromXml(LyEnvironment environment, String file)
	{
		LyViewParser	  parser = new LyViewParser(environment, file);
		LyMapRegion		  mapRegion = (LyMapRegion)parser.getRootElement();
		Vector<LyPolygon> polygons = new Vector<LyPolygon>();
		int				  i, n = mapRegion.getPolygonCount();

		for (i = 0; i < n; i++)
			polygons.addElement(mapRegion.getPolygon(i));

		return polygons;
	}

	/**
	 * Builds a polygon object from an dxf file.
	 */
	protected Vector<LyPolygon> buildPolygonsFromDxf(LyEnvironment environment, String file)
	{
		LyDxfParser parser = new LyDxfParser();

		return parser.parseFile(environment, file);
	}
} // end class LyMapRegion
