/*
 * LyMenu.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved.
 */

package leon.view;

import leon.view.graphic.*;

/**
 * The LyMenu class is a generated Java implementation that corresponds to the element MENU of the
 * descriptive grammar of the views (views.dtd). <B>&lt;MENU&gt; : Menu</B> A menu contains a set of
 * menu, sub-menus entries and separators.
 *
 * @see  leon.view.LyComponent See also tag <A href="../../conf/dtd/view/MENU.html">MENU</A> of
 *       grammar leon_views.dtd.
 */
public class LyMenu extends LyMenuBase
{
	public static final String __VERSION = "$Id: LyMenu.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
     * Default constructor.
     */
    protected LyMenu()
    {
    }
    
	/**
	 * Constructor with all required attributes.
	 *
	 * @param  label  Text of the menu.
	 */
	public LyMenu(String label)
	{
		super(label);
	}

	/* Protected methods for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 *
	 * @param  node  Node from which the component is constructed.
	 *
	 * @see    leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyMenu(LyViewNode node)
	{
		super(node);
	}

	/**
	 * Inserts a sub-component item at given position.
	 *
	 * @param  item  The item to add to components list.
	 * @param  pos   Position where the sub-component is inserted.
	 */
	public void insertItem(LyItem item, int pos)
	{
		super.insertItem(item, pos);

		LyComponent parent = getParent();

		while ((parent != null) && !(parent instanceof LyMenuBar))
			parent = parent.getParent();

		if (parent instanceof LyMenuBar)
			((LyMenuBar)parent).addItemToCache(item);
	}
	
	/**
     * Sets attribute enabled to the given value.
     * @param enabled Boolean indicating if attribute enabled is set or unset.
     */
    public void setEnabled(boolean enabled)
    {
        if (enabled == _enabled)
            return;
        
        super.setEnabled(enabled);
        
        int             i, n = this.getComponentCount();
        LyComponent     component;
        LyViewComponent internalView;
        
        for (i = 0; i < n; i++)
        {
            component = getComponent(i);
            internalView = component.getInternalView();
            if ((component instanceof LyItem)
                    && (internalView instanceof LyGraphicalItem))
                ((LyGraphicalItem)internalView).setEnabled(enabled && ((LyItem)component).isEnabled());
            else if ((component instanceof LyMenu)
                    && (internalView instanceof LyGraphicalMenu))
                ((LyGraphicalMenu)internalView).setEnabled(enabled && ((LyMenu)component).isEnabled());
        }
    }
} // end class LyMenu
