/*
 * LyMessages.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;

import java.util.*;
import java.io.*;
import leon.view.graphic.*;
import leon.view.getinterface.*;
import leon.misc.LyString;


/**
 * The LyMessages class is a generated Java implementation that corresponds to the element 
 * MESSAGES of the descriptive grammar of the views (views.dtd).
 * <B>&lt;MESSAGES&gt; : Message bar</B>
 * A message bar can be used in a main window to provide the user with information 
 * about the displayed data (number of objects, printing in progress...).
 * @see leon.view.LyComponent
 * See also tag <A href="../../conf/dtd/view/MESSAGES.html">MESSAGES</A> of grammar leon_views.dtd.
 */
public class LyMessages extends LyComponent
	implements  LyGetIdInterface,
				LyGetConstraintInterface
{
	public static final String			__VERSION = "$Id$";


	/** Possible values for attribute 'border'. */
	public enum Border
	{
		NONE,
		SOLID,
		RAISED,
		LOWERED,
		IN,
		OUT;
	}


	/* Static methods */

	/**
	 * Utility method to get border value from a string.
	 * @param token Token string
	 * @return Border value corresponding to this token
	 */
	public static final Border getBorder(String token)
	{
		if (token == null)
			return null;
		else if (token.equalsIgnoreCase("NONE")
				|| token.equals("0"))
			return Border.NONE;
		else if (token.equalsIgnoreCase("SOLID")
				|| token.equals("1"))
			return Border.SOLID;
		else if (token.equalsIgnoreCase("RAISED")
				|| token.equals("2"))
			return Border.RAISED;
		else if (token.equalsIgnoreCase("LOWERED")
				|| token.equals("3"))
			return Border.LOWERED;
		else if (token.equalsIgnoreCase("IN")
				|| token.equals("4"))
			return Border.IN;
		else if (token.equalsIgnoreCase("OUT")
				|| token.equals("5"))
			return Border.OUT;
		else
			return Border.LOWERED;
	}


	/* Attributes */

	/**
	 * Identifier of the message bar.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyMessages			_template;

	/**
	 * Style of the border used for the message bar. The following borders are 
	 * supported:
	 * NONE No border.
	 * SOLID Solid border.
	 * RAISED Border having the appearance of a raised 3D button.
	 * LOWERED Border having the appearance of a lowered 3D button.
	 * IN Border having a 3D lowered appearance.
	 * OUT Border having a 3D raised appearance.
	 */
	protected Border			_border;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;


	/* Sub components */

	/**
	 * Messages contained in the message bar.
	 */
	protected ArrayList<LyMessage>		_messages;

	/**
	 * Positioning constraint of the message bar.
	 */
	protected LyConstraint		_constraint;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_id = null;
		_template = null;
		_border = Border.LOWERED;
		_font = getStringValue("MESSAGES.font", "$ALL.font");
		_color = getStringValue("MESSAGES.color", "$ALL.color");
		_fontColor = getStringValue("MESSAGES.fontColor", "$ALL.fontColor");

		// Initialize sub components
		_messages = null;
		_constraint = null;
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyMessages	model = (LyMessages)component;

		_model = component;
		_border = model._border;
		_font = model._font;
		_color = model._color;
		_fontColor = model._fontColor;

		if (REWRITING_MODE)
			return;

		// Copy sub-components
		LyMessage	messages;
		int			i, n;

		n = (model._messages == null ? 0 : model._messages.size());
		for (i = 0; i < n; i++)
		{
			messages = model._messages.get(i);
			if (messages != null)
				messages = (LyMessage)messages.cloneComponent();
			addChild(messages);
		}
		if (model._constraint != null)
			addChild(model._constraint.cloneComponent());
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	public LyMessages()
	{
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyMessages(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_id = node.getString("id");
		_template = (LyMessages)node.getComponent("template");

		if (_template != null)
			initFromModel(_template);

		if (node.isSpecified("border"))
			_border = getBorder(node.getString("border"));
		if (node.isSpecified("font"))
			_font = node.getString("font");
		if (node.isSpecified("color"))
			_color = node.getString("color");
		if (node.isSpecified("fontColor"))
			_fontColor = node.getString("fontColor");
	}

	/**
	 * End of construction : completion with child component.
	 * @param child Sub child component.
	 * @see leon.view.LyComponent#addChild
	 */
	protected void addChild(LyComponent child)
	{
		// Add sub-components
		if (child instanceof LyMessage)
			addMessage((LyMessage)child);
		else if (child == null)
			addSep();
		else if (child instanceof LyConstraint)
			setConstraint((LyConstraint)child);
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyMessages clone = new LyMessages();

		clone.initFromModel(this);
		clone._template = this;
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of id.
	 * @return value of id.
	 */
	public String getId()
	{
		return _id;
	}

	/**
	 * Gets value of template.
	 * @return value of template.
	 */
	public LyMessages getTemplate()
	{
		return _template;
	}

	/**
	 * Gets value of border.
	 * @return value of border.
	 */
	public Border getBorder()
	{
		return _border;
	}

	/**
	 * Gets value of font.
	 * @return value of font.
	 */
	public String getFont()
	{
		return _font;
	}

	/**
	 * Checks whether the font attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFont()
	{
		return (_font == getStringValue("MESSAGES.font"));
	}

	/**
	 * Gets value of color.
	 * @return value of color.
	 */
	public String getColor()
	{
		return _color;
	}

	/**
	 * Checks whether the color attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultColor()
	{
		return (_color == getStringValue("MESSAGES.color"));
	}

	/**
	 * Gets value of fontColor.
	 * @return value of fontColor.
	 */
	public String getFontColor()
	{
		return _fontColor;
	}

	/**
	 * Checks whether the fontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFontColor()
	{
		return (_fontColor == getStringValue("MESSAGES.fontColor"));
	}

	/**
	 * Get sub-component Message of index i.
	 * @param i the index of the Message to get.
	 * @return sub-component Message of index i.
	 */
	public LyMessage getMessage(int i)
	{
		if (_messages == null)
			return null;
		if ((i < 0) || (i >= _messages.size()))
			return null;
		return _messages.get(i);
	}

	/**
	 * Get sub-component Message count.
	 * @return sub-component Message count.
	 */
	public int getMessageCount()
	{
		if (_messages == null)
			return 0;
		return _messages.size();
	}

	/**
	 * Get sub-component Constraint.
	 * @return sub-component Constraint.
	 */
	public LyConstraint getConstraint()
	{
		return _constraint;
	}


	/* Modifiers */

	/**
	 * Initializes attribute id to the given value.
	 * @param id New value of id.
	 */
	public void setId(String id)
	{
		if ((id == _id)
			|| ((id != null)
				&& (_id != null)
				&& id.equals(_id)))
			return;

		_id = (id != null ? id.intern() : null);
	}

	/**
	 * Initializes attribute template to the given value.
	 * @param template New value of template.
	 */
	public void setTemplate(LyMessages template)
	{
		if (template == null)
			return;
		if (_template != null)
			throw new IllegalStateException("BF AppEngine : attribute template was already set.");

		initFromModel(template);
		_template = template;
	}

	/**
	 * Initializes attribute border to the given value.
	 * @param border New value of border.
	 * @exception IllegalArgumentException if border parameter is null.
	 */
	public void setBorder(Border border)
	{
		if (border == _border)
			return;
		if (border == null)
			throw new IllegalArgumentException("BF AppEngine : invalid value for attribute border");

		_border = border;
	}

	/**
	 * Initializes attribute font to the given value.
	 * @param font New value of font.
	 */
	public void setFont(String font)
	{
		if ((font == _font)
			|| ((font != null)
				&& (_font != null)
				&& font.equals(_font)))
			return;

		_font = font;
	}

	/**
	 * Initializes attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color)
	{
		if ((color == _color)
			|| ((color != null)
				&& (_color != null)
				&& color.equals(_color)))
			return;

		_color = color;
	}

	/**
	 * Initializes attribute fontColor to the given value.
	 * @param fontColor New value of fontColor.
	 */
	public void setFontColor(String fontColor)
	{
		if ((fontColor == _fontColor)
			|| ((fontColor != null)
				&& (_fontColor != null)
				&& fontColor.equals(_fontColor)))
			return;

		_fontColor = fontColor;
	}

	/**
	 * Adds a sub-component message.
	 * @param message The message to add to messages list.
	 */
	public void addMessage(LyMessage message)
	{
		insertMessage(message, -1);
	}

	/**
	 * Inserts a sub-component message at given position.
	 * @param message The message to add to messages list.
	 * @param pos	Position where the sub-component is inserted.
	 */
	public void insertMessage(LyMessage message, int pos)
	{
		if (_messages == null)
			_messages = new ArrayList<LyMessage>(1);
		else if (_messages.contains(message))
			return;
		if (pos >= 0)
			_messages.add(pos,message);
		else
			_messages.add(message);
		if (message != null)
			message.setParent(this);
		if (_internalView != null)
			((LyGraphicalMessages)_internalView).addMessage(message, pos);
	}

	/**
	 * Removes a sub-component message.
	 * @param message The message to remove to messages list.
	 * @exception IllegalStateException if messages list is empty.
	 * @exception IllegalArgumentException if messages list doesn't contain message.
	 */
	public void removeMessage(LyMessage message)
	{
		if (_messages == null)
			throw new IllegalStateException("BF AppEngine : messages list is empty");
		if (!_messages.contains(message))
			throw new IllegalArgumentException("BF AppEngine : messages list doesn't contain message");

		if (_internalView != null)
			((LyGraphicalMessages)_internalView).removeMessage(message);
		_messages.remove(message);
		message.setParent(null);
	}

	/**
	 * Adds a separator.
	 */
	public void addSep()
	{
		insertSep(-1);
	}

	/**
	 * Inserts a separator at the given position.
	 * @param pos	Position where the separator is inserted.
	 */
	public void insertSep(int pos)
	{
		LyMessage sep = new LyMessage(SEP_ID, null);
		insertMessage(sep, pos);
	}

	/**
	 * Removes a separator at the given position.
	 * @param pos	Position where the separator is removed.
	 * @exception IllegalStateException if messages list is empty.
	 * @exception IllegalArgumentException if messages list doesn't contain a separator at the given position.
	 */
	public void removeSep(int pos)
	{
		if (_messages == null)
			throw new IllegalStateException("BF AppEngine : messages list is empty");
		if ((pos < 0)
				|| (pos >= _messages.size())
				|| ((_messages.get(pos) != null)
					&& (_messages.get(pos).getId() != SEP_ID)))
			throw new IllegalArgumentException("BF AppEngine : messages list doesn't contain a separator at pos " + pos);

		_messages.remove(pos);
	}

	/**
	 * Sets sub-component constraint.
	 * @param constraint New value of constraint
	 */
	public void setConstraint(LyConstraint constraint)
	{
		if (_constraint != null)
			_constraint.setParent(null);
		_constraint = constraint;
		if (constraint != null)
			constraint.setParent(this);
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<MESSAGES");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if ((_template == null)
			|| (_border != _template._border))
		{
			if (_border == Border.NONE)
				pw.print(" border='NONE'");
			else if (_border == Border.SOLID)
				pw.print(" border='SOLID'");
			else if (_border == Border.RAISED)
				pw.print(" border='RAISED'");
			else if ((_border == Border.LOWERED) && (_template != null))
				pw.print(" border='LOWERED'");
			else if (_border == Border.IN)
				pw.print(" border='IN'");
			else if (_border == Border.OUT)
				pw.print(" border='OUT'");
		}
		if (((_template == null) && (_font != getStringValue("MESSAGES.font", "$ALL.font")))
				|| ((_template != null) && (_font != _template._font)))
			pw.print(" font='" + LyString.getXmlString(_font) + "'");
		if (((_template == null) && (_color != getStringValue("MESSAGES.color", "$ALL.color")))
				|| ((_template != null) && (_color != _template._color)))
			pw.print(" color='" + LyString.getXmlString(_color) + "'");
		if (((_template == null) && (_fontColor != getStringValue("MESSAGES.fontColor", "$ALL.fontColor")))
				|| ((_template != null) && (_fontColor != _template._fontColor)))
			pw.print(" fontColor='" + LyString.getXmlString(_fontColor) + "'");

		if ((getChildCount() == 0) && (_datas == null))
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		LyComponent	messages;
		int			i, n;

		n = (_messages == null ? 0 : _messages.size());
		for (i = 0; i < n; i++)
		{
			messages = _messages.get(i);
			if ((messages == null)
					|| (messages.getId() == SEP_ID))
				pw.println("<SEP/>");
			else
				messages.writeXml(pw, debug);
		}

		if (_constraint != null)
			_constraint.writeXml(pw, debug);

		pw.println("</MESSAGES>");
	}
}
