/*
 * LyNotice.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;

import java.util.*;
import java.io.*;
import leon.view.getinterface.*;
import leon.view.setinterface.*;
import leon.misc.LyString;


/**
 * The LyNotice class is a generated Java implementation that corresponds to the element 
 * NOTICE of the descriptive grammar of the views (views.dtd).
 * <B>&lt;NOTICE&gt; : Information Panel</B>
 * An information panel allows displaying an insert having an informative purpose.
 * It is possible to assign an icon, a label and a progress bar to an information 
 * panel.
 * @see leon.view.LyPanel
 * See also tag <A href="../../conf/dtd/view/NOTICE.html">NOTICE</A> of grammar leon_views.dtd.
 */
public class LyNotice extends LyPanel
	implements  LyGetIdInterface,
				LyGetControllerInterface,
				LySetControllerInterface,
				LyGetConstraintInterface
{
	public static final String			__VERSION = "$Id$";


	/** Possible values for attribute 'style'. */
	public enum Style
	{
		STANDARD,
		STICKER;
	}


	/* Static methods */

	/**
	 * Utility method to get style value from a string.
	 * @param token Token string
	 * @return Style value corresponding to this token
	 */
	public static final Style getStyle(String token)
	{
		if (token == null)
			return null;
		else if (token.equalsIgnoreCase("STANDARD")
				|| token.equals("0"))
			return Style.STANDARD;
		else if (token.equalsIgnoreCase("STICKER")
				|| token.equals("1"))
			return Style.STICKER;
		else
			return Style.STANDARD;
	}


	/* Attributes */

	/**
	 * Identifier of the information panel.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyNotice			_template;

	/**
	 * Reference of the controller that manages the notice panel (i.e. handles user 
	 * demands and refresh content).
	 */
	protected String			_controller;

	/**
	 * Number of columns used to display the label.
	 */
	protected int				_nCols;

	/**
	 * Number of rows used to display the label.
	 */
	protected int				_nRows;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	/**
	 * Style of the notice.
	 * The following styles are managed:
	 * STANDARD Standard notice.
	 * STICKER Notice is displayed as a sticker.
	 */
	protected Style				_style;


	/* Sub components */

	/**
	 * Icon
	 * associated to the information panel.
	 */
	protected LyIcon			_icon;

	/**
	 * Label associated to the information panel.
	 */
	protected LyLabel			_label;

	/**
	 * Progress bar associated to the information panel.
	 */
	protected LyGauge			_gauge;

	/**
	 * Positioning constraint of the notice panel.
	 */
	protected LyConstraint		_constraint;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_id = null;
		_template = null;
		_controller = null;
		_nCols = getIntValue("NOTICE.nCols", -1);
		_nRows = getIntValue("NOTICE.nRows", -1);
		_font = getStringValue("NOTICE.font", "$ALL.font");
		_color = getStringValue("NOTICE.color", "$ALL.color");
		_fontColor = getStringValue("NOTICE.fontColor", "$ALL.fontColor");
		_style = Style.STANDARD;

		// Initialize sub components
		_icon = null;
		_label = null;
		_gauge = null;
		_constraint = null;
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyNotice	model = (LyNotice)component;

		_model = component;
		_controller = model._controller;
		_nCols = model._nCols;
		_nRows = model._nRows;
		_font = model._font;
		_color = model._color;
		_fontColor = model._fontColor;
		_style = model._style;

		if (REWRITING_MODE)
			return;

		// Copy sub-components
		if (model._icon != null)
			addChild(model._icon.cloneComponent());
		if (model._label != null)
			addChild(model._label.cloneComponent());
		if (model._gauge != null)
			addChild(model._gauge.cloneComponent());
		if (model._constraint != null)
			addChild(model._constraint.cloneComponent());
		if (model._datas != null)
			_datas = new Hashtable<String, String>(model._datas);
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	protected LyNotice()
	{
		_id = getInternalId();
	}

	/**
	 * Constructor with all required attributes.
	 * @param id Identifier of the information panel.
	 */
	public LyNotice(String id)
	{
		// Fill up required attributes with parameters
		_id = (id != null ? id.intern() : null);
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyNotice(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_id = node.getString("id");
		_template = (LyNotice)node.getComponent("template");

		if (_template != null)
			initFromModel(_template);

		if (node.isSpecified("controller"))
			_controller = node.getString("controller");
		if (node.isSpecified("nCols"))
			_nCols = node.getInt("nCols");
		if (node.isSpecified("nRows"))
			_nRows = node.getInt("nRows");
		if (node.isSpecified("font"))
			_font = node.getString("font");
		if (node.isSpecified("color"))
			_color = node.getString("color");
		if (node.isSpecified("fontColor"))
			_fontColor = node.getString("fontColor");
		if (node.isSpecified("style"))
			_style = getStyle(node.getString("style"));
	}

	/**
	 * End of construction : completion with child component.
	 * @param child Sub child component.
	 * @see leon.view.LyComponent#addChild
	 */
	protected void addChild(LyComponent child)
	{
		// Add sub-components
		if (child instanceof LyIcon)
			setIcon((LyIcon)child);
		else if (child instanceof LyLabel)
			setLabel((LyLabel)child);
		else if (child instanceof LyGauge)
			setGauge((LyGauge)child);
		else if (child instanceof LyConstraint)
			setConstraint((LyConstraint)child);
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyNotice clone = new LyNotice();

		clone.initFromModel(this);
		clone._template = this;
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of id.
	 * @return value of id.
	 */
	public String getId()
	{
		return _id;
	}

	/**
	 * Gets value of template.
	 * @return value of template.
	 */
	public LyNotice getTemplate()
	{
		return _template;
	}

	/**
	 * Gets value of controller.
	 * @return value of controller.
	 */
	public String getController()
	{
		return _controller;
	}

	/**
	 * Gets value of nCols.
	 * @return value of nCols.
	 */
	public int getNCols()
	{
		return _nCols;
	}

	/**
	 * Gets value of nRows.
	 * @return value of nRows.
	 */
	public int getNRows()
	{
		return _nRows;
	}

	/**
	 * Gets value of font.
	 * @return value of font.
	 */
	public String getFont()
	{
		return _font;
	}

	/**
	 * Checks whether the font attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFont()
	{
		return (_font == getStringValue("NOTICE.font"));
	}

	/**
	 * Gets value of color.
	 * @return value of color.
	 */
	public String getColor()
	{
		return _color;
	}

	/**
	 * Checks whether the color attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultColor()
	{
		return (_color == getStringValue("NOTICE.color"));
	}

	/**
	 * Gets value of fontColor.
	 * @return value of fontColor.
	 */
	public String getFontColor()
	{
		return _fontColor;
	}

	/**
	 * Checks whether the fontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFontColor()
	{
		return (_fontColor == getStringValue("NOTICE.fontColor"));
	}

	/**
	 * Gets value of style.
	 * @return value of style.
	 */
	public Style getStyle()
	{
		return _style;
	}

	/**
	 * Get sub-component Icon.
	 * @return sub-component Icon.
	 */
	public LyIcon getIcon()
	{
		return _icon;
	}

	/**
	 * Get sub-component Label.
	 * @return sub-component Label.
	 */
	public LyLabel getLabel()
	{
		return _label;
	}

	/**
	 * Get sub-component Gauge.
	 * @return sub-component Gauge.
	 */
	public LyGauge getGauge()
	{
		return _gauge;
	}

	/**
	 * Get sub-component Constraint.
	 * @return sub-component Constraint.
	 */
	public LyConstraint getConstraint()
	{
		return _constraint;
	}


	/* Modifiers */

	/**
	 * Initializes attribute template to the given value.
	 * @param template New value of template.
	 */
	public void setTemplate(LyNotice template)
	{
		if (template == null)
			return;
		if (_template != null)
			throw new IllegalStateException("BF AppEngine : attribute template was already set.");

		initFromModel(template);
		_template = template;
	}

	/**
	 * Initializes attribute controller to the given value.
	 * @param controller New value of controller.
	 */
	public void setController(String controller)
	{
		if ((controller == _controller)
			|| ((controller != null)
				&& (_controller != null)
				&& controller.equals(_controller)))
			return;

		_controller = controller;
	}

	/**
	 * Initializes attribute nCols to the given value.
	 * @param nCols New value of nCols.
	 */
	public void setNCols(int nCols)
	{
		if (nCols == _nCols)
			return;
		_nCols = nCols;
	}

	/**
	 * Initializes attribute nRows to the given value.
	 * @param nRows New value of nRows.
	 */
	public void setNRows(int nRows)
	{
		if (nRows == _nRows)
			return;
		_nRows = nRows;
	}

	/**
	 * Initializes attribute font to the given value.
	 * @param font New value of font.
	 */
	public void setFont(String font)
	{
		if ((font == _font)
			|| ((font != null)
				&& (_font != null)
				&& font.equals(_font)))
			return;

		_font = font;
	}

	/**
	 * Initializes attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color)
	{
		if ((color == _color)
			|| ((color != null)
				&& (_color != null)
				&& color.equals(_color)))
			return;

		_color = color;
	}

	/**
	 * Initializes attribute fontColor to the given value.
	 * @param fontColor New value of fontColor.
	 */
	public void setFontColor(String fontColor)
	{
		if ((fontColor == _fontColor)
			|| ((fontColor != null)
				&& (_fontColor != null)
				&& fontColor.equals(_fontColor)))
			return;

		_fontColor = fontColor;
	}

	/**
	 * Initializes attribute style to the given value.
	 * @param style New value of style.
	 * @exception IllegalArgumentException if style parameter is null.
	 */
	public void setStyle(Style style)
	{
		if (style == _style)
			return;
		if (style == null)
			throw new IllegalArgumentException("BF AppEngine : invalid value for attribute style");

		_style = style;
	}

	/**
	 * Sets sub-component icon.
	 * @param icon New value of icon
	 */
	public void setIcon(LyIcon icon)
	{
		if (_icon != null)
			_icon.setParent(null);
		_icon = icon;
		if (icon != null)
			icon.setParent(this);
	}

	/**
	 * Sets sub-component label.
	 * @param label New value of label
	 */
	public void setLabel(LyLabel label)
	{
		if (_label != null)
			_label.setParent(null);
		_label = label;
		if (label != null)
			label.setParent(this);
	}

	/**
	 * Sets sub-component gauge.
	 * @param gauge New value of gauge
	 */
	public void setGauge(LyGauge gauge)
	{
		if (_gauge != null)
			_gauge.setParent(null);
		_gauge = gauge;
		if (gauge != null)
			gauge.setParent(this);
	}

	/**
	 * Sets sub-component constraint.
	 * @param constraint New value of constraint
	 */
	public void setConstraint(LyConstraint constraint)
	{
		if (_constraint != null)
			_constraint.setParent(null);
		_constraint = constraint;
		if (constraint != null)
			constraint.setParent(this);
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<NOTICE");
		pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (_controller != null)
			pw.print(" controller='" + LyString.getXmlString(_controller) + "'");
		if (((_template == null) && (_nCols != getIntValue("NOTICE.nCols", -1)))
				|| ((_template != null) && (_nCols != _template._nCols)))
			pw.print(" nCols='" + _nCols + "'");
		if (((_template == null) && (_nRows != getIntValue("NOTICE.nRows", -1)))
				|| ((_template != null) && (_nRows != _template._nRows)))
			pw.print(" nRows='" + _nRows + "'");
		if (((_template == null) && (_font != getStringValue("NOTICE.font", "$ALL.font")))
				|| ((_template != null) && (_font != _template._font)))
			pw.print(" font='" + LyString.getXmlString(_font) + "'");
		if (((_template == null) && (_color != getStringValue("NOTICE.color", "$ALL.color")))
				|| ((_template != null) && (_color != _template._color)))
			pw.print(" color='" + LyString.getXmlString(_color) + "'");
		if (((_template == null) && (_fontColor != getStringValue("NOTICE.fontColor", "$ALL.fontColor")))
				|| ((_template != null) && (_fontColor != _template._fontColor)))
			pw.print(" fontColor='" + LyString.getXmlString(_fontColor) + "'");
		if ((_template == null)
			|| (_style != _template._style))
		{
			if ((_style == Style.STANDARD) && (_template != null))
				pw.print(" style='STANDARD'");
			else if (_style == Style.STICKER)
				pw.print(" style='STICKER'");
		}

		if ((getChildCount() == 0) && (_datas == null))
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		if (_icon != null)
			_icon.writeXml(pw, debug);
		if (_label != null)
			_label.writeXml(pw, debug);
		if (_gauge != null)
			_gauge.writeXml(pw, debug);
		if (_constraint != null)
			_constraint.writeXml(pw, debug);

		writeDatas(pw);

		pw.println("</NOTICE>");
	}
}
