/*
 * LyNumber.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved.
 */

package leon.view;

/**
 * The LyNumber class is a generated Java implementation that corresponds to the element NUMBER of
 * the descriptive grammar of the views (views.dtd). <B>&lt;NUMBER&gt; : Input Field of Number
 * Type</B> An input field of Number type allows displaying a graphical component allowing to input
 * or to consult a number value. It does not comprise any element. It is possible to specify the
 * minimum and the maximum value, the incrementation step, the number of decimals and the unit.
 *
 * @see  leon.view.LyFieldItem See also tag <A href="../../conf/dtd/view/NUMBER.html">NUMBER</A> of
 *       grammar leon_views.dtd.
 */
public class LyNumber extends LyNumberBase
{
	/** Indicates if we use arrows for number inputs. */
	public static boolean		SHOW_ARROWS = true;

	boolean _showArrows;

	/**
	 * Default constructor.
	 */
	public LyNumber()
	{
		super();
	}

	/* Protected methods for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 *
	 * @param  node  Node from which the component is constructed.
	 *
	 * @see    leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyNumber(LyViewNode node)
	{
		super(node);
	}

	/**
	 * Initialisation of the data structure.
	 *
	 * @see  leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();
		_showArrows = true;
	}

	/**
	 * Initializes attribute showArrows to the given value.
	 *
	 * @param  showArrows  Boolean indicating if attribute showArrows is set or unset.
	 */
	public void showArrows(boolean showArrows)
	{
		if (showArrows == _showArrows)
			return;

		_showArrows = showArrows;
	}

	/**
	 * Checks if value of attribute showArrows is set.
	 *
	 * @return  true if showArrows is set, false otherwise.
	 */
	public boolean isShowArrows()
	{
		return _showArrows;
	}


	/**
     * Get number of chars for LyNumber input.
     * @return number of chars for LyNumber input
     */
    public int getMaxChars()
    {
        int maxChars = 1;
        double var = 1;
        int i = 0;
        boolean noMin = (getVMin() == Double.NEGATIVE_INFINITY);
        boolean noMax = (getVMax()  == Double.POSITIVE_INFINITY);

        if (!noMax && !noMin)
        {
            // Get the max between min and max (abs values)
            double absMax = Math.max(Math.abs(getVMax() ), Math.abs(getVMin()));

            for (i = 0; (var <= absMax) && (i < 20); i++)
                var *= 10;

            maxChars = i + 1
                    + ((getDecimals() == 0) ? 0 : ((getDecimals() < 0) ? 10 : (getDecimals() + 1)));

            if (maxChars > 25)
                maxChars = 25;
        }
        else if (getDecimals() == 0)
            maxChars = 10;
        else
            maxChars = 20;

        return maxChars;
    }


} // end class LyNumber
