/*
 * LyPanel.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved.
 */

package leon.view;

import leon.view.getinterface.*;
import leon.view.setinterface.*;

import java.io.*;


/**
 * This class implements the generic behavior of the following classes : LyTable, LyTree, LyMap.
 *
 * @version  $Id: LyPanel.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author   Lyria
 */
public abstract class LyPanel extends LyFieldItem
	implements LySetWaitingInterface,
		LyPdfInterface
{
	public static final String __VERSION = "$Id: LyPanel.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** (Optional) Last modification date, or -1 if not specified. */
	protected long _lastModified = -1;

	/**
	 * Default constructor.
	 */
	public LyPanel()
	{
		super();
	}

	/**
	 * Sets or unset the idle mode on the component/node.
	 *
	 * @param  status  new idle status (LySetIdleInterface interface)
	 */
	public void setIdle(boolean status)
	{
		Object internal = getInternalView(false);

		if (internal instanceof LySetIdleInterface)
			((LySetIdleInterface)internal).setIdle(status);
	}

	/**
	 * Sets or unset the idle mode on the component/node.
	 *
	 * @param  status  new idle status (LySetIdleInterface interface)
	 */
	public boolean isIdle()
	{
		Object internal = getInternalView(false);

		if (internal instanceof LyIsIdleInterface)
			return ((LyIsIdleInterface)internal).isIdle();

		return false;
	}

	/**
	 * Checks if the panel is scrollable or not.
	 *
	 * @return  True if scrollable, false otherwise.
	 */
	public boolean isScrollable()
	{
		return true;
	}

	/**
	 * Accessor to the waiting status.
	 *
	 * @see  LySetWaitingInterface#setWaiting
	 */
	public void setWaiting(boolean status)
	{
		Object internal = getInternalView(false);

		if (internal instanceof LySetWaitingInterface)
			((LySetWaitingInterface)internal).setWaiting(status);
	}

	/**
	 * Affectation of the popup menu.
	 */
	public void setPopupMenu(LyPopupMenu popup)
	{
		if (_internalView instanceof LySetPopupMenuInterface)
			((LySetPopupMenuInterface)_internalView).setPopupMenu(popup);
	}

	/**
	 * Print the content of the panel. Result depends on the nature of the panel and the display
	 * (win, web ...)
	 */
	public void print(LyPrintListenerInterface listener)
	{
		Object internal = getInternalView(false);

		if (internal instanceof LyPrintInterface)
			((LyPrintInterface)internal).print(listener);
	}

	/**
	 * Generate the content of the panel in a PDF document. Result depends on the nature of the
	 * panel and the display (win, web ...)
	 */
	public void generatePdf(String fileName) throws IOException
	{
		Object internal = getInternalView(false);

		if (internal instanceof LyPdfInterface)
			((LyPdfInterface)internal).generatePdf(fileName);
	}

	/**
	 * Set the last modified date info (if any).
	 *
	 * @param  lastModified  last modification date or -1 means info not available, or not
	 *                       specified.
	 */
	public void setLastModified(long lastModified)
	{
		_lastModified = lastModified;
	}

	/**
	 * Get the last modified date info (if any).
	 *
	 * @return  last modification date or -1 means info not available, or not specified.
	 */
	public long getLastModified()
	{
		return (_lastModified);
	}

} // end class LyPanel
