/*
 * LyPeriod.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;
import java.io.*;
import leon.view.graphic.*;
import leon.misc.LyString;


/**
 * The LyPeriod class is a generated Java implementation that corresponds to the element 
 * PERIOD of the descriptive grammar of the views (views.dtd).
 * <B>&lt;PERIOD&gt; : Period</B>
 * A period allows dividing an interval of a Gantt planning into several sub-parts.
 * It is described by a begin date and an end date.
 * @see leon.view.LyComponent
 * See also tag <A href="../../conf/dtd/view/PERIOD.html">PERIOD</A> of grammar leon_views.dtd.
 */
public class LyPeriod extends LyComponent
{
	public static final String			__VERSION = "$Id$";


	/* Attributes */

	/**
	 * Begin date (respecting date format of the Gantt planning).
	 */
	protected String			_beginDate;

	/**
	 * End date (respecting date format of the Gantt planning).
	 */
	protected String			_endDate;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_beginDate = null;
		_endDate = null;
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyPeriod	model = (LyPeriod)component;

		_model = component;
		_beginDate = model._beginDate;
		_endDate = model._endDate;
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	protected LyPeriod()
	{
	}

	/**
	 * Constructor with all required attributes.
	 * @param beginDate Begin date (respecting date format of the Gantt planning).
	 * @param endDate End date (respecting date format of the Gantt planning).
	 */
	public LyPeriod(String beginDate, String endDate)
	{
		// Fill up required attributes with parameters
		_beginDate = beginDate;
		_endDate = endDate;
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyPeriod(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_beginDate = node.getString("beginDate");
		_endDate = node.getString("endDate");
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyPeriod clone = new LyPeriod();

		clone.initFromModel(this);
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of beginDate.
	 * @return value of beginDate.
	 */
	public String getBeginDate()
	{
		return _beginDate;
	}

	/**
	 * Gets value of endDate.
	 * @return value of endDate.
	 */
	public String getEndDate()
	{
		return _endDate;
	}


	/* Modifiers */

	/**
	 * Sets attribute beginDate to the given value.
	 * @param beginDate New value of beginDate.
	 * @exception IllegalArgumentException if beginDate is null.
	 */
	public void setBeginDate(String beginDate)
	{
		if (beginDate == null)
			throw new IllegalArgumentException("BF AppEngine : attribute beginDate is required (can't be null)");

		if ((beginDate == _beginDate)
			|| ((beginDate != null)
				&& (_beginDate != null)
				&& beginDate.equals(_beginDate)))
			return;

		_beginDate = beginDate;
		if (_internalView != null)
			((LyGraphicalPeriod)_internalView).setBeginDate(beginDate);
	}

	/**
	 * Sets attribute endDate to the given value.
	 * @param endDate New value of endDate.
	 * @exception IllegalArgumentException if endDate is null.
	 */
	public void setEndDate(String endDate)
	{
		if (endDate == null)
			throw new IllegalArgumentException("BF AppEngine : attribute endDate is required (can't be null)");

		if ((endDate == _endDate)
			|| ((endDate != null)
				&& (_endDate != null)
				&& endDate.equals(_endDate)))
			return;

		_endDate = endDate;
		if (_internalView != null)
			((LyGraphicalPeriod)_internalView).setEndDate(endDate);
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<PERIOD");
		pw.print(" beginDate='" + LyString.getXmlString(_beginDate) + "'");
		pw.print(" endDate='" + LyString.getXmlString(_endDate) + "'");
		pw.println("/>");
	}
}
