/*
 * LyPoint.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved.
 */

package leon.view;

/**
 * This class represents a Point component.
 *
 * @author   Lyria
 * @version  $Id: LyPoint.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @see      LyComponent
 */
public class LyPoint extends LyPointBase
{
	public static final String __VERSION = "$Id: LyPoint.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Constructor with all required attributes.
	 */
	public LyPoint()
	{
	}

	/**
	 * Constructor with all attributes.
	 */
	public LyPoint(float x, float y, float z)
	{
		this();

		_x = x;
		_y = y;
		_z = z;
	}

	/* Protected methods for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 *
	 * @param  node  node from which the component is constructed.
	 *
	 * @see    LyViewNodeFactory#buildElement
	 */
	protected LyPoint(LyViewNode node)
	{
		super(node);
	}

	/* Specifics methods */

	/**
	 * Multiply the coordonates by the given factor.
	 *
	 * @param  f  multiplying factor.
	 */
	public void scale(float f)
	{
		if (f != 1F)
		{
			_x *= f;
			_y *= f;
			_z *= f;
		}
	}

	/**
	 * Translation of the node.
	 *
	 * @param  dx  x coordonate.
	 * @param  dy  y coordonate.
	 * @param  dz  z coordonate.
	 */
	public void translate(float dx, float dy, float dz)
	{
		_x += dx;
		_y += dy;
		_z += dz;
	}

	@Override public String toString()
	{
		return "(" + _x + ", " + _y + ", " + _z + ")";
	}
} // end class LyPoint
