/*
 * LyPointBase.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;
import java.io.*;


/**
 * The LyPointBase class is a generated Java implementation that corresponds to the element 
 * POINT of the descriptive grammar of the views (views.dtd).
 * <B>&lt;POINT&gt; : Point</B>
 * A point is used to define the drawing of a link in a map when a specific drawing
 * is needed.
 * @see leon.view.LyComponent
 * @see leon.view.LyLyPoint
 * See also tag <A href="../../conf/dtd/view/POINT.html">POINT</A> of grammar leon_views.dtd.
 */
public abstract class LyPointBase extends LyComponent
{
	public static final String			__VERSION = "$Id$";


	/* Attributes */

	/**
	 * Abscissa of the point.
	 */
	protected float				_x;

	/**
	 * Ordinate of the point.
	 */
	protected float				_y;

	/**
	 * Altitude of the point.
	 */
	protected float				_z;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_x = getFloatValue("POINT.x", 0F);
		_y = getFloatValue("POINT.y", 0F);
		_z = getFloatValue("POINT.z", 0F);
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyPoint	model = (LyPoint)component;

		_model = component;
		_x = model._x;
		_y = model._y;
		_z = model._z;
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	public LyPointBase()
	{
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyPointBase(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_x = node.getFloat("x");
		_y = node.getFloat("y");
		_z = node.getFloat("z");
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyPoint clone = new LyPoint();

		clone.initFromModel(this);
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of x.
	 * @return value of x.
	 */
	public float getX()
	{
		return _x;
	}

	/**
	 * Gets value of y.
	 * @return value of y.
	 */
	public float getY()
	{
		return _y;
	}

	/**
	 * Gets value of z.
	 * @return value of z.
	 */
	public float getZ()
	{
		return _z;
	}


	/* Modifiers */

	/**
	 * Initializes attribute x to the given value.
	 * @param x New value of x.
	 */
	public void setX(float x)
	{
		if (x == _x)
			return;
		_x = x;
	}

	/**
	 * Initializes attribute y to the given value.
	 * @param y New value of y.
	 */
	public void setY(float y)
	{
		if (y == _y)
			return;
		_y = y;
	}

	/**
	 * Initializes attribute z to the given value.
	 * @param z New value of z.
	 */
	public void setZ(float z)
	{
		if (z == _z)
			return;
		_z = z;
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<POINT");
		if (_x != getFloatValue("POINT.x", 0F))
			pw.print(" x='" + _x + "'");
		if (_y != getFloatValue("POINT.y", 0F))
			pw.print(" y='" + _y + "'");
		if (_z != getFloatValue("POINT.z", 0F))
			pw.print(" z='" + _z + "'");
		pw.println("/>");
	}
}
