/*
 * LyPopupMenu.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved.
 */

package leon.view;

import leon.misc.*;
import leon.view.graphic.*;


/**
 * The LyPopupMenu class implements generic behavior for the POPUP_MENU components as described in
 * views.dtd regardless of the requested target (win, swing, html...). It represents popup menu.
 *
 * @author   Lyria
 * @version  $Id: LyPopupMenu.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @see      LyComponent
 */
public class LyPopupMenu extends LyPopupMenuBase
{
	public static final String __VERSION = "$Id: LyPopupMenu.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Environment. */
	protected LyEnvironment _environment;

	/**
	 * Constructor with all required attributes.
	 */
	public LyPopupMenu()
	{
		super();
	}

	/**
	 * Construction from a LyViewNode.
	 *
	 * @param  node  Node from which the component is constructed.
	 *
	 * @see    leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyPopupMenu(LyViewNode node)
	{
		super(node);
	}

	/**
	 * Set Environment : popup have no parent.
	 */
	public void setEnvironment(LyEnvironment environment)
	{
		_environment = environment;
	}

	/**
	 * Get Environment : popup have no parent.
	 */
	public LyEnvironment getEnvironment()
	{
		return (_environment != null ? _environment : super.getEnvironment());
	}

	/**
	 * Removes a sub-component item.
	 *
	 * @param      item  The item to remove to components list.
	 *
	 * @exception  IllegalStateException     if components list is empty.
	 * @exception  IllegalArgumentException  if components list doesn't contain item.
	 */
	public void removeItem(LyItem item)
	{
		if (!_components.contains(item))
			throw new IllegalArgumentException("BF AppEngine : components list doesn't contain item");

		if (_internalView != null)
			((LyGraphicalPopupMenu)_internalView).removeComponent(item);

		_components.remove(item);
		item._parent = null;
        
        int     n = getComponentCount();
        
        if (n > 0)
        {
            LyComponent     lastComp = getComponent(n - 1);

            if ((lastComp == null)
                    || lastComp.getId().equals(SEP_ID))
                removeSep(n - 1);
        }
	}

	/**
	 * Removes a separator at the given position.
	 *
	 * @param      pos  Position where the separator is removed.
	 *
	 * @exception  IllegalStateException     if components list is empty.
	 * @exception  IllegalArgumentException  if components list doesn't contain a separator at the
	 *                                       given position.
	 */
	public void removeSep(int pos)
	{
		if ((pos < 0)
				|| (pos >= _components.size())
				|| ((_components.get(pos) != null)
				    &&  !_components.get(pos).getId().equals(SEP_ID)))
			throw new IllegalArgumentException("BF AppEngine : components list doesn't contain a separator at pos " + pos);

		_components.remove(pos);
	}

	/**
	 * Removes a sub-component menu.
	 *
	 * @param      menu  The menu to remove to components list.
	 *
	 * @exception  IllegalStateException     if components list is empty.
	 * @exception  IllegalArgumentException  if components list doesn't contain menu.
	 */
	public void removeMenu(LyMenu menu)
	{
		if (!_components.contains(menu))
			throw new IllegalArgumentException("BF AppEngine : components list doesn't contain menu");

		if (_internalView instanceof LyGraphicalMenu)
			((LyGraphicalMenu)_internalView).removeComponent(menu);

		_components.remove(menu);
		menu._parent = null;
	}
} // end class LyPopupMenu
