/*
 * LyRelation.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved.
 */

package leon.view;

import leon.view.beaninterface.*;
import leon.view.setinterface.*;


/**
 * The LyRelation class implements generic behavior for the RELATION components as described in
 * views.dtd regardless of the requested target (win, swing, html...). It represents a relation.
 *
 * @author   Lyria
 * @version  $Id: LyRelation.java 40925 2012-09-25 15:26:17Z smaupin $
 * @see      LyRelationBase
 */
public class LyRelation extends LyRelationBase
{
	public static final String __VERSION = "$Id: LyRelation.java 40925 2012-09-25 15:26:17Z smaupin $";

	/** Width of the list for this relation. */
	protected int _listWidth;

	/** Minimum width of the relation. */
	protected int _minWidth;

	/** Maximum width of the relation. */
	protected int _maxWidth;

	/** default width value in edit table. */
    public static int      DEFAULT_WIDTH_EDITTABLE = 20;

	/**
	 * Constructor with all required attributes.
	 */
	public LyRelation()
	{
		super();
		_listWidth = -1;
		_minWidth = -1;
		_maxWidth = -1;
	}

	protected LyRelation(LyViewNode node)
	{
		super(node);
		_listWidth = -1;
	}

	public void selectLink(String id, boolean select)
	{
		if (id == null)
			return;

		int    nLink = getLinkCount();
		LyLink link;

		for (int j = 0; j < nLink; j++)
		{
			link = getLink(j);
			if (id.equals(link.getId()))
			{
				selectLink(j, select);
				return;
			}
		}
	}

	public void selectLink(int i, boolean select)
	{
		if (i >= getLinkCount())
			return;

		LyLink link = getLink(i);

		link.setSelected(select);

		if (_internalView instanceof LyRelationFieldBeanInterface)
			((LyRelationFieldBeanInterface)_internalView).selectLink(i, select);

		// Ensure other links are unselected if not multiple
		if ((select) && (!isMultiple()))
		{
			int j, n = getLinkCount();

			for (j = 0; j < n; j++)
			{
				link = getLink(j);

				if ((j != i) && (link.isSelected()))
				{
					link.setSelected(false);

					if (_internalView instanceof LyRelationFieldBeanInterface)
						((LyRelationFieldBeanInterface)_internalView).selectLink(j, false);
				}
			}
		}
	} // end method selectLink

	public void setIdle(boolean idle)
	{
		if (_internalView instanceof LySetIdleInterface)
			((LySetIdleInterface)_internalView).setIdle(idle);
	}

	public void setFocus()
	{
		if (_internalView instanceof LySetFocusInterface)
			((LySetFocusInterface)_internalView).setFocus();
	}

	/**
	 * Sets the width of the list for this relation.
	 *
	 * @param  width  the width of the list for this relation.
	 */
	public void setListWidth(int width)
	{
		_listWidth = width;
	}

	/**
	 * Returns the width of the list for this relation.
	 *
	 * @return  the width of the list for this relation if set, -1 otherwise.
	 */
	public int getListWidth()
	{
		return _listWidth;
	}

	/**
	 * Sets the minimum width of this relation.
	 *
	 * @param  width  the minimum width of this relation.
	 */
	public void setMinWidth(int width)
	{
		_minWidth = width;
	}

	/**
	 * Returns the minimum width of this relation.
	 *
	 * @return  the minimum width of this relation if set, the value of the LY_COMBO_MIN_WIDTH
	 *          resource otherwise.
	 */
	public int getMinWidth()
	{
		if (_minWidth > 0)
			return _minWidth;

		if (!_multiple)
			return getEnvironment().getIntResource("LY_COMBO_MIN_WIDTH");
		else
			return getEnvironment().getIntResource("LY_MULTIPLE_RELATION_MIN_WIDTH");
	}

	/**
	 * Sets the maximum width of this relation.
	 *
	 * @param  width  the maximum width of this relation.
	 */
	public void setMaxWidth(int width)
	{
		_maxWidth = width;
	}

	/**
	 * Returns the maximum width of this relation.
	 *
	 * @return  the maximum width of this relation if set, the value of the LY_COMBO_MAX_WIDTH
	 *          resource otherwise.
	 */
	public int getMaxWidth()
	{
		if (_maxWidth > 0)
			return _maxWidth;

		if (!_multiple)
			return getEnvironment().getIntResource("LY_COMBO_MAX_WIDTH");
		else
			return getEnvironment().getIntResource("LY_MULTIPLE_RELATION_MAX_WIDTH");
	}
} // end class LyRelation
