/*
 * LyRelationBase.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;

import java.util.*;
import java.io.*;
import leon.view.getinterface.*;
import leon.view.setinterface.*;
import leon.misc.LyString;


/**
 * The LyRelationBase class is a generated Java implementation that corresponds to the element 
 * RELATION of the descriptive grammar of the views (views.dtd).
 * <B>&lt;RELATION&gt; : Input Field of Relation Type</B>
 * An input field of Relation type allows choosing one or more values among a list 
 * of values.
 * This list offers generally application objects belonging to a same class and is 
 * used to establish a relation between an object and one or more objects of this 
 * class.
 * It is possible to specify whether the relation is multiple or not.
 * @see leon.view.LyFieldItem
 * @see leon.view.LyLyRelation
 * See also tag <A href="../../conf/dtd/view/RELATION.html">RELATION</A> of grammar leon_views.dtd.
 */
public abstract class LyRelationBase extends LyFieldItem
	implements  LyGetIdInterface
{
	public static final String			__VERSION = "$Id$";


	/* Attributes */

	/**
	 * Identifier of the component.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyRelation			_template;

	/**
	 * Indicates whether the relation is multiple or not.
	 */
	protected boolean			_multiple;

	/**
	 * Indicates whether the relation is ordered or not.
	 */
	protected boolean			_ordered;

	/**
	 * Allows specifying the name of a Java class that replaces the default graphical 
	 * component.
	 */
	protected String			_bean;

	/**
	 * Gives the number of columns on which the relation is presented.
	 */
	protected int				_nCols;

	/**
	 * Gives the number of lines on which the relation is presented.
	 */
	protected int				_nRows;

	/**
	 * Indicates whether the relation allows auto-completion or not.
	 * 0 means that auto-completion is not used, -1 means that auto-completion is used 
	 * with no limit for displayed values,
	 * any other value indicates the maximum number of values displayed at the same 
	 * time.
	 */
	protected int				_autoCompletionMax;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * RELATION.enabledColor
	 */
	protected String			_enabledColor;

	/**
	 * RELATION.disabledColor
	 */
	protected String			_disabledColor;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	/**
	 * Image used for the selection button.
	 */
	protected String			_selectImage;

	/**
	 * Image used for the selection button.
	 */
	protected String			_deselectImage;

	/**
	 * Image used for the full selection button.
	 */
	protected String			_selectAllImage;

	/**
	 * Image used for the full deselection button.
	 */
	protected String			_deselectAllImage;

	/**
	 * Image of the button used to move down a value within the selection.
	 */
	protected String			_moveDownImage;

	/**
	 * Image of the button used to move up a value within the selection.
	 */
	protected String			_moveUpImage;

	/**
	 * RELATION.arrowDownBig
	 */
	protected String			_arrowDownBig;

	/**
	 * RELATION.selectedColor
	 */
	protected String			_selectedColor;

	/**
	 * RELATION.inactiveSelectedColor
	 */
	protected String			_inactiveSelectedColor;

	/**
	 * RELATION.selectedFontColor
	 */
	protected String			_selectedFontColor;

	/**
	 * RELATION.multiSelectorSelectImage
	 */
	protected String			_multiSelectorSelectImage;

	/**
	 * RELATION.multiSelectorUnselectImage
	 */
	protected String			_multiSelectorUnselectImage;


	/* Sub components */

	/**
	 * List of the links offered for this relation.
	 */
	protected ArrayList<LyLink>		_links;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_id = null;
		_template = null;
		_multiple = getBooleanValue("RELATION.multiple", false);
		_ordered = getBooleanValue("RELATION.ordered", false);
		_bean = null;
		_nCols = getIntValue("RELATION.nCols", -1);
		_nRows = getIntValue("RELATION.nRows", -1);
		_autoCompletionMax = getIntValue("RELATION.autoCompletionMax", 0);
		_font = getStringValue("RELATION.font", "$ALL.font");
		_enabledColor = getStringValue("RELATION.enabledColor", "$ALL.enabledColor");
		_disabledColor = getStringValue("RELATION.disabledColor", "$ALL.disabledColor");
		_fontColor = getStringValue("RELATION.fontColor", "$ALL.fontColor");
		_selectImage = getStringValue("RELATION.selectImage", "$ALL.selectImage");
		_deselectImage = getStringValue("RELATION.deselectImage", "$ALL.deselectImage");
		_selectAllImage = getStringValue("RELATION.selectAllImage", "$ALL.selectAllImage");
		_deselectAllImage = getStringValue("RELATION.deselectAllImage", "$ALL.deselectAllImage");
		_moveDownImage = getStringValue("RELATION.moveDownImage", "$ALL.moveDownImage");
		_moveUpImage = getStringValue("RELATION.moveUpImage", "$ALL.moveUpImage");
		_arrowDownBig = getStringValue("RELATION.arrowDownBig", "$ALL.arrowDownBig");
		_selectedColor = getStringValue("RELATION.selectedColor", "$ALL.selectedColor");
		_inactiveSelectedColor = getStringValue("RELATION.inactiveSelectedColor", "$ALL.inactiveSelectedColor");
		_selectedFontColor = getStringValue("RELATION.selectedFontColor", "$ALL.selectedFontColor");
		_multiSelectorSelectImage = getStringValue("RELATION.multiSelectorSelectImage", "$ALL.multiSelectorSelectImage");
		_multiSelectorUnselectImage = getStringValue("RELATION.multiSelectorUnselectImage", "$ALL.multiSelectorUnselectImage");

		// Initialize sub components
		_links = null;
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyRelation	model = (LyRelation)component;

		_model = component;
		_multiple = model._multiple;
		_ordered = model._ordered;
		_bean = model._bean;
		_nCols = model._nCols;
		_nRows = model._nRows;
		_autoCompletionMax = model._autoCompletionMax;
		_font = model._font;
		_enabledColor = model._enabledColor;
		_disabledColor = model._disabledColor;
		_fontColor = model._fontColor;
		_selectImage = model._selectImage;
		_deselectImage = model._deselectImage;
		_selectAllImage = model._selectAllImage;
		_deselectAllImage = model._deselectAllImage;
		_moveDownImage = model._moveDownImage;
		_moveUpImage = model._moveUpImage;
		_arrowDownBig = model._arrowDownBig;
		_selectedColor = model._selectedColor;
		_inactiveSelectedColor = model._inactiveSelectedColor;
		_selectedFontColor = model._selectedFontColor;
		_multiSelectorSelectImage = model._multiSelectorSelectImage;
		_multiSelectorUnselectImage = model._multiSelectorUnselectImage;

		if (REWRITING_MODE)
			return;

		// Copy sub-components
		LyLink	links;
		int			i, n;

		n = (model._links == null ? 0 : model._links.size());
		for (i = 0; i < n; i++)
		{
			links = model._links.get(i);
			if (links != null)
				links = (LyLink)links.cloneComponent();
			addChild(links);
		}
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	public LyRelationBase()
	{
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyRelationBase(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_id = node.getString("id");
		_template = (LyRelation)node.getComponent("template");

		if (_template != null)
			initFromModel(_template);

		if (node.isSpecified("multiple"))
			_multiple = node.getBoolean("multiple");
		if (node.isSpecified("ordered"))
			_ordered = node.getBoolean("ordered");
		if (node.isSpecified("bean"))
			_bean = node.getString("bean");
		if (node.isSpecified("nCols"))
			_nCols = node.getInt("nCols");
		if (node.isSpecified("nRows"))
			_nRows = node.getInt("nRows");
		if (node.isSpecified("autoCompletionMax"))
			_autoCompletionMax = node.getInt("autoCompletionMax");
		if (node.isSpecified("font"))
			_font = node.getString("font");
		if (node.isSpecified("enabledColor"))
			_enabledColor = node.getString("enabledColor");
		if (node.isSpecified("disabledColor"))
			_disabledColor = node.getString("disabledColor");
		if (node.isSpecified("fontColor"))
			_fontColor = node.getString("fontColor");
		if (node.isSpecified("selectImage"))
			_selectImage = node.getString("selectImage");
		if (node.isSpecified("deselectImage"))
			_deselectImage = node.getString("deselectImage");
		if (node.isSpecified("selectAllImage"))
			_selectAllImage = node.getString("selectAllImage");
		if (node.isSpecified("deselectAllImage"))
			_deselectAllImage = node.getString("deselectAllImage");
		if (node.isSpecified("moveDownImage"))
			_moveDownImage = node.getString("moveDownImage");
		if (node.isSpecified("moveUpImage"))
			_moveUpImage = node.getString("moveUpImage");
		if (node.isSpecified("arrowDownBig"))
			_arrowDownBig = node.getString("arrowDownBig");
		if (node.isSpecified("selectedColor"))
			_selectedColor = node.getString("selectedColor");
		if (node.isSpecified("inactiveSelectedColor"))
			_inactiveSelectedColor = node.getString("inactiveSelectedColor");
		if (node.isSpecified("selectedFontColor"))
			_selectedFontColor = node.getString("selectedFontColor");
		if (node.isSpecified("multiSelectorSelectImage"))
			_multiSelectorSelectImage = node.getString("multiSelectorSelectImage");
		if (node.isSpecified("multiSelectorUnselectImage"))
			_multiSelectorUnselectImage = node.getString("multiSelectorUnselectImage");
	}

	/**
	 * End of construction : completion with child component.
	 * @param child Sub child component.
	 * @see leon.view.LyComponent#addChild
	 */
	protected void addChild(LyComponent child)
	{
		// Add sub-components
		if (child instanceof LyLink)
			addLink((LyLink)child);
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyRelation clone = new LyRelation();

		clone.initFromModel(this);
		clone._template = (LyRelation)this;
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of id.
	 * @return value of id.
	 */
	public String getId()
	{
		return _id;
	}

	/**
	 * Gets value of template.
	 * @return value of template.
	 */
	public LyRelation getTemplate()
	{
		return _template;
	}

	/**
	 * Checks if value of attribute multiple is set.
	 * @return true if multiple is set, false otherwise.
	 */
	public boolean isMultiple()
	{
		return _multiple;
	}

	/**
	 * Checks if value of attribute ordered is set.
	 * @return true if ordered is set, false otherwise.
	 */
	public boolean isOrdered()
	{
		return _ordered;
	}

	/**
	 * Gets value of bean.
	 * @return value of bean.
	 */
	public String getBean()
	{
		return _bean;
	}

	/**
	 * Gets value of nCols.
	 * @return value of nCols.
	 */
	public int getNCols()
	{
		return _nCols;
	}

	/**
	 * Gets value of nRows.
	 * @return value of nRows.
	 */
	public int getNRows()
	{
		return _nRows;
	}

	/**
	 * Gets value of autoCompletionMax.
	 * @return value of autoCompletionMax.
	 */
	public int getAutoCompletionMax()
	{
		return _autoCompletionMax;
	}

	/**
	 * Gets value of font.
	 * @return value of font.
	 */
	public String getFont()
	{
		return _font;
	}

	/**
	 * Checks whether the font attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFont()
	{
		return (_font == getStringValue("RELATION.font"));
	}

	/**
	 * Gets value of enabledColor.
	 * @return value of enabledColor.
	 */
	public String getEnabledColor()
	{
		return _enabledColor;
	}

	/**
	 * Checks whether the enabledColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultEnabledColor()
	{
		return (_enabledColor == getStringValue("RELATION.enabledColor"));
	}

	/**
	 * Gets value of disabledColor.
	 * @return value of disabledColor.
	 */
	public String getDisabledColor()
	{
		return _disabledColor;
	}

	/**
	 * Checks whether the disabledColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultDisabledColor()
	{
		return (_disabledColor == getStringValue("RELATION.disabledColor"));
	}

	/**
	 * Gets value of fontColor.
	 * @return value of fontColor.
	 */
	public String getFontColor()
	{
		return _fontColor;
	}

	/**
	 * Checks whether the fontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFontColor()
	{
		return (_fontColor == getStringValue("RELATION.fontColor"));
	}

	/**
	 * Gets value of selectImage.
	 * @return value of selectImage.
	 */
	public String getSelectImage()
	{
		return _selectImage;
	}

	/**
	 * Checks whether the selectImage attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultSelectImage()
	{
		return (_selectImage == getStringValue("RELATION.selectImage"));
	}

	/**
	 * Gets value of deselectImage.
	 * @return value of deselectImage.
	 */
	public String getDeselectImage()
	{
		return _deselectImage;
	}

	/**
	 * Checks whether the deselectImage attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultDeselectImage()
	{
		return (_deselectImage == getStringValue("RELATION.deselectImage"));
	}

	/**
	 * Gets value of selectAllImage.
	 * @return value of selectAllImage.
	 */
	public String getSelectAllImage()
	{
		return _selectAllImage;
	}

	/**
	 * Checks whether the selectAllImage attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultSelectAllImage()
	{
		return (_selectAllImage == getStringValue("RELATION.selectAllImage"));
	}

	/**
	 * Gets value of deselectAllImage.
	 * @return value of deselectAllImage.
	 */
	public String getDeselectAllImage()
	{
		return _deselectAllImage;
	}

	/**
	 * Checks whether the deselectAllImage attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultDeselectAllImage()
	{
		return (_deselectAllImage == getStringValue("RELATION.deselectAllImage"));
	}

	/**
	 * Gets value of moveDownImage.
	 * @return value of moveDownImage.
	 */
	public String getMoveDownImage()
	{
		return _moveDownImage;
	}

	/**
	 * Checks whether the moveDownImage attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultMoveDownImage()
	{
		return (_moveDownImage == getStringValue("RELATION.moveDownImage"));
	}

	/**
	 * Gets value of moveUpImage.
	 * @return value of moveUpImage.
	 */
	public String getMoveUpImage()
	{
		return _moveUpImage;
	}

	/**
	 * Checks whether the moveUpImage attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultMoveUpImage()
	{
		return (_moveUpImage == getStringValue("RELATION.moveUpImage"));
	}

	/**
	 * Gets value of arrowDownBig.
	 * @return value of arrowDownBig.
	 */
	public String getArrowDownBig()
	{
		return _arrowDownBig;
	}

	/**
	 * Checks whether the arrowDownBig attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultArrowDownBig()
	{
		return (_arrowDownBig == getStringValue("RELATION.arrowDownBig"));
	}

	/**
	 * Gets value of selectedColor.
	 * @return value of selectedColor.
	 */
	public String getSelectedColor()
	{
		return _selectedColor;
	}

	/**
	 * Checks whether the selectedColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultSelectedColor()
	{
		return (_selectedColor == getStringValue("RELATION.selectedColor"));
	}

	/**
	 * Gets value of inactiveSelectedColor.
	 * @return value of inactiveSelectedColor.
	 */
	public String getInactiveSelectedColor()
	{
		return _inactiveSelectedColor;
	}

	/**
	 * Checks whether the inactiveSelectedColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultInactiveSelectedColor()
	{
		return (_inactiveSelectedColor == getStringValue("RELATION.inactiveSelectedColor"));
	}

	/**
	 * Gets value of selectedFontColor.
	 * @return value of selectedFontColor.
	 */
	public String getSelectedFontColor()
	{
		return _selectedFontColor;
	}

	/**
	 * Checks whether the selectedFontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultSelectedFontColor()
	{
		return (_selectedFontColor == getStringValue("RELATION.selectedFontColor"));
	}

	/**
	 * Gets value of multiSelectorSelectImage.
	 * @return value of multiSelectorSelectImage.
	 */
	public String getMultiSelectorSelectImage()
	{
		return _multiSelectorSelectImage;
	}

	/**
	 * Checks whether the multiSelectorSelectImage attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultMultiSelectorSelectImage()
	{
		return (_multiSelectorSelectImage == getStringValue("RELATION.multiSelectorSelectImage"));
	}

	/**
	 * Gets value of multiSelectorUnselectImage.
	 * @return value of multiSelectorUnselectImage.
	 */
	public String getMultiSelectorUnselectImage()
	{
		return _multiSelectorUnselectImage;
	}

	/**
	 * Checks whether the multiSelectorUnselectImage attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultMultiSelectorUnselectImage()
	{
		return (_multiSelectorUnselectImage == getStringValue("RELATION.multiSelectorUnselectImage"));
	}

	/**
	 * Get sub-component Link of index i.
	 * @param i the index of the Link to get.
	 * @return sub-component Link of index i.
	 */
	public LyLink getLink(int i)
	{
		if (_links == null)
			return null;
		if ((i < 0) || (i >= _links.size()))
			return null;
		return _links.get(i);
	}

	/**
	 * Get sub-component Link count.
	 * @return sub-component Link count.
	 */
	public int getLinkCount()
	{
		if (_links == null)
			return 0;
		return _links.size();
	}


	/* Modifiers */

	/**
	 * Initializes attribute id to the given value.
	 * @param id New value of id.
	 */
	public void setId(String id)
	{
		if ((id == _id)
			|| ((id != null)
				&& (_id != null)
				&& id.equals(_id)))
			return;

		_id = (id != null ? id.intern() : null);
	}

	/**
	 * Initializes attribute template to the given value.
	 * @param template New value of template.
	 */
	public void setTemplate(LyRelation template)
	{
		if (template == null)
			return;
		if (_template != null)
			throw new IllegalStateException("BF AppEngine : attribute template was already set.");

		initFromModel(template);
		_template = template;
	}

	/**
	 * Initializes attribute multiple to the given value.
	 * @param multiple Boolean indicating if attribute multiple is set or unset.
	 */
	public void setMultiple(boolean multiple)
	{
		if (multiple == _multiple)
			return;
		_multiple = multiple;
	}

	/**
	 * Initializes attribute ordered to the given value.
	 * @param ordered Boolean indicating if attribute ordered is set or unset.
	 */
	public void setOrdered(boolean ordered)
	{
		if (ordered == _ordered)
			return;
		_ordered = ordered;
	}

	/**
	 * Initializes attribute bean to the given value.
	 * @param bean New value of bean.
	 */
	public void setBean(String bean)
	{
		if ((bean == _bean)
			|| ((bean != null)
				&& (_bean != null)
				&& bean.equals(_bean)))
			return;

		_bean = bean;
	}

	/**
	 * Initializes attribute nCols to the given value.
	 * @param nCols New value of nCols.
	 */
	public void setNCols(int nCols)
	{
		if (nCols == _nCols)
			return;
		_nCols = nCols;
	}

	/**
	 * Initializes attribute nRows to the given value.
	 * @param nRows New value of nRows.
	 */
	public void setNRows(int nRows)
	{
		if (nRows == _nRows)
			return;
		_nRows = nRows;
	}

	/**
	 * Initializes attribute autoCompletionMax to the given value.
	 * @param autoCompletionMax New value of autoCompletionMax.
	 */
	public void setAutoCompletionMax(int autoCompletionMax)
	{
		if (autoCompletionMax == _autoCompletionMax)
			return;
		_autoCompletionMax = autoCompletionMax;
	}

	/**
	 * Initializes attribute font to the given value.
	 * @param font New value of font.
	 */
	public void setFont(String font)
	{
		if ((font == _font)
			|| ((font != null)
				&& (_font != null)
				&& font.equals(_font)))
			return;

		_font = font;
	}

	/**
	 * Initializes attribute enabledColor to the given value.
	 * @param enabledColor New value of enabledColor.
	 */
	public void setEnabledColor(String enabledColor)
	{
		if ((enabledColor == _enabledColor)
			|| ((enabledColor != null)
				&& (_enabledColor != null)
				&& enabledColor.equals(_enabledColor)))
			return;

		_enabledColor = enabledColor;
	}

	/**
	 * Initializes attribute disabledColor to the given value.
	 * @param disabledColor New value of disabledColor.
	 */
	public void setDisabledColor(String disabledColor)
	{
		if ((disabledColor == _disabledColor)
			|| ((disabledColor != null)
				&& (_disabledColor != null)
				&& disabledColor.equals(_disabledColor)))
			return;

		_disabledColor = disabledColor;
	}

	/**
	 * Initializes attribute fontColor to the given value.
	 * @param fontColor New value of fontColor.
	 */
	public void setFontColor(String fontColor)
	{
		if ((fontColor == _fontColor)
			|| ((fontColor != null)
				&& (_fontColor != null)
				&& fontColor.equals(_fontColor)))
			return;

		_fontColor = fontColor;
	}

	/**
	 * Initializes attribute selectImage to the given value.
	 * @param selectImage New value of selectImage.
	 */
	public void setSelectImage(String selectImage)
	{
		if ((selectImage == _selectImage)
			|| ((selectImage != null)
				&& (_selectImage != null)
				&& selectImage.equals(_selectImage)))
			return;

		_selectImage = selectImage;
	}

	/**
	 * Initializes attribute deselectImage to the given value.
	 * @param deselectImage New value of deselectImage.
	 */
	public void setDeselectImage(String deselectImage)
	{
		if ((deselectImage == _deselectImage)
			|| ((deselectImage != null)
				&& (_deselectImage != null)
				&& deselectImage.equals(_deselectImage)))
			return;

		_deselectImage = deselectImage;
	}

	/**
	 * Initializes attribute selectAllImage to the given value.
	 * @param selectAllImage New value of selectAllImage.
	 */
	public void setSelectAllImage(String selectAllImage)
	{
		if ((selectAllImage == _selectAllImage)
			|| ((selectAllImage != null)
				&& (_selectAllImage != null)
				&& selectAllImage.equals(_selectAllImage)))
			return;

		_selectAllImage = selectAllImage;
	}

	/**
	 * Initializes attribute deselectAllImage to the given value.
	 * @param deselectAllImage New value of deselectAllImage.
	 */
	public void setDeselectAllImage(String deselectAllImage)
	{
		if ((deselectAllImage == _deselectAllImage)
			|| ((deselectAllImage != null)
				&& (_deselectAllImage != null)
				&& deselectAllImage.equals(_deselectAllImage)))
			return;

		_deselectAllImage = deselectAllImage;
	}

	/**
	 * Initializes attribute moveDownImage to the given value.
	 * @param moveDownImage New value of moveDownImage.
	 */
	public void setMoveDownImage(String moveDownImage)
	{
		if ((moveDownImage == _moveDownImage)
			|| ((moveDownImage != null)
				&& (_moveDownImage != null)
				&& moveDownImage.equals(_moveDownImage)))
			return;

		_moveDownImage = moveDownImage;
	}

	/**
	 * Initializes attribute moveUpImage to the given value.
	 * @param moveUpImage New value of moveUpImage.
	 */
	public void setMoveUpImage(String moveUpImage)
	{
		if ((moveUpImage == _moveUpImage)
			|| ((moveUpImage != null)
				&& (_moveUpImage != null)
				&& moveUpImage.equals(_moveUpImage)))
			return;

		_moveUpImage = moveUpImage;
	}

	/**
	 * Initializes attribute arrowDownBig to the given value.
	 * @param arrowDownBig New value of arrowDownBig.
	 */
	public void setArrowDownBig(String arrowDownBig)
	{
		if ((arrowDownBig == _arrowDownBig)
			|| ((arrowDownBig != null)
				&& (_arrowDownBig != null)
				&& arrowDownBig.equals(_arrowDownBig)))
			return;

		_arrowDownBig = arrowDownBig;
	}

	/**
	 * Initializes attribute selectedColor to the given value.
	 * @param selectedColor New value of selectedColor.
	 */
	public void setSelectedColor(String selectedColor)
	{
		if ((selectedColor == _selectedColor)
			|| ((selectedColor != null)
				&& (_selectedColor != null)
				&& selectedColor.equals(_selectedColor)))
			return;

		_selectedColor = selectedColor;
	}

	/**
	 * Initializes attribute inactiveSelectedColor to the given value.
	 * @param inactiveSelectedColor New value of inactiveSelectedColor.
	 */
	public void setInactiveSelectedColor(String inactiveSelectedColor)
	{
		if ((inactiveSelectedColor == _inactiveSelectedColor)
			|| ((inactiveSelectedColor != null)
				&& (_inactiveSelectedColor != null)
				&& inactiveSelectedColor.equals(_inactiveSelectedColor)))
			return;

		_inactiveSelectedColor = inactiveSelectedColor;
	}

	/**
	 * Initializes attribute selectedFontColor to the given value.
	 * @param selectedFontColor New value of selectedFontColor.
	 */
	public void setSelectedFontColor(String selectedFontColor)
	{
		if ((selectedFontColor == _selectedFontColor)
			|| ((selectedFontColor != null)
				&& (_selectedFontColor != null)
				&& selectedFontColor.equals(_selectedFontColor)))
			return;

		_selectedFontColor = selectedFontColor;
	}

	/**
	 * Initializes attribute multiSelectorSelectImage to the given value.
	 * @param multiSelectorSelectImage New value of multiSelectorSelectImage.
	 */
	public void setMultiSelectorSelectImage(String multiSelectorSelectImage)
	{
		if ((multiSelectorSelectImage == _multiSelectorSelectImage)
			|| ((multiSelectorSelectImage != null)
				&& (_multiSelectorSelectImage != null)
				&& multiSelectorSelectImage.equals(_multiSelectorSelectImage)))
			return;

		_multiSelectorSelectImage = multiSelectorSelectImage;
	}

	/**
	 * Initializes attribute multiSelectorUnselectImage to the given value.
	 * @param multiSelectorUnselectImage New value of multiSelectorUnselectImage.
	 */
	public void setMultiSelectorUnselectImage(String multiSelectorUnselectImage)
	{
		if ((multiSelectorUnselectImage == _multiSelectorUnselectImage)
			|| ((multiSelectorUnselectImage != null)
				&& (_multiSelectorUnselectImage != null)
				&& multiSelectorUnselectImage.equals(_multiSelectorUnselectImage)))
			return;

		_multiSelectorUnselectImage = multiSelectorUnselectImage;
	}

	/**
	 * Adds a sub-component link.
	 * @param link The link to add to links list.
	 */
	public void addLink(LyLink link)
	{
		insertLink(link, -1);
	}

	/**
	 * Inserts a sub-component link at given position.
	 * @param link The link to add to links list.
	 * @param pos	Position where the sub-component is inserted.
	 */
	public void insertLink(LyLink link, int pos)
	{
		if (_links == null)
		{
			_links = new ArrayList<LyLink>(1);
			_children = _links;
		}
		else if (_links.contains(link))
			return;
		if (pos >= 0)
			_links.add(pos,link);
		if (link != null)
			link.setParent(this);
		if (_internalView instanceof LySetLinksInterface)
			((LySetLinksInterface)_internalView).addLink(link, pos);
	}

	/**
	 * Removes a sub-component link.
	 * @param link The link to remove to links list.
	 * @exception IllegalStateException if links list is empty.
	 * @exception IllegalArgumentException if links list doesn't contain link.
	 */
	public void removeLink(LyLink link)
	{
		if (_links == null)
			throw new IllegalStateException("BF AppEngine : links list is empty");
		if (!_links.contains(link))
			throw new IllegalArgumentException("BF AppEngine : links list doesn't contain link");

		if (_internalView instanceof LySetLinksInterface)
			((LySetLinksInterface)_internalView).removeLink(link);
		link.setParent(null);
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<RELATION");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && _multiple)
				|| ((_template != null) && (_multiple != _template._multiple)))
			pw.print(" multiple='" + _multiple + "'");
		if (((_template == null) && _ordered)
				|| ((_template != null) && (_ordered != _template._ordered)))
			pw.print(" ordered='" + _ordered + "'");
		if (((_template == null) && (_bean != null))
				|| ((_template != null) && (_bean != _template._bean)))
			pw.print(" bean='" + LyString.getXmlString(_bean) + "'");
		if (((_template == null) && (_nCols != getIntValue("RELATION.nCols", -1)))
				|| ((_template != null) && (_nCols != _template._nCols)))
			pw.print(" nCols='" + _nCols + "'");
		if (((_template == null) && (_nRows != getIntValue("RELATION.nRows", -1)))
				|| ((_template != null) && (_nRows != _template._nRows)))
			pw.print(" nRows='" + _nRows + "'");
		if (((_template == null) && (_autoCompletionMax != getIntValue("RELATION.autoCompletionMax", 0)))
				|| ((_template != null) && (_autoCompletionMax != _template._autoCompletionMax)))
			pw.print(" autoCompletionMax='" + _autoCompletionMax + "'");
		if (((_template == null) && (_font != getStringValue("RELATION.font", "$ALL.font")))
				|| ((_template != null) && (_font != _template._font)))
			pw.print(" font='" + LyString.getXmlString(_font) + "'");
		if (((_template == null) && (_enabledColor != getStringValue("RELATION.enabledColor", "$ALL.enabledColor")))
				|| ((_template != null) && (_enabledColor != _template._enabledColor)))
			pw.print(" enabledColor='" + LyString.getXmlString(_enabledColor) + "'");
		if (((_template == null) && (_disabledColor != getStringValue("RELATION.disabledColor", "$ALL.disabledColor")))
				|| ((_template != null) && (_disabledColor != _template._disabledColor)))
			pw.print(" disabledColor='" + LyString.getXmlString(_disabledColor) + "'");
		if (((_template == null) && (_fontColor != getStringValue("RELATION.fontColor", "$ALL.fontColor")))
				|| ((_template != null) && (_fontColor != _template._fontColor)))
			pw.print(" fontColor='" + LyString.getXmlString(_fontColor) + "'");
		if (((_template == null) && (_selectImage != getStringValue("RELATION.selectImage", "$ALL.selectImage")))
				|| ((_template != null) && (_selectImage != _template._selectImage)))
			pw.print(" selectImage='" + LyString.getXmlString(_selectImage) + "'");
		if (((_template == null) && (_deselectImage != getStringValue("RELATION.deselectImage", "$ALL.deselectImage")))
				|| ((_template != null) && (_deselectImage != _template._deselectImage)))
			pw.print(" deselectImage='" + LyString.getXmlString(_deselectImage) + "'");
		if (((_template == null) && (_selectAllImage != getStringValue("RELATION.selectAllImage", "$ALL.selectAllImage")))
				|| ((_template != null) && (_selectAllImage != _template._selectAllImage)))
			pw.print(" selectAllImage='" + LyString.getXmlString(_selectAllImage) + "'");
		if (((_template == null) && (_deselectAllImage != getStringValue("RELATION.deselectAllImage", "$ALL.deselectAllImage")))
				|| ((_template != null) && (_deselectAllImage != _template._deselectAllImage)))
			pw.print(" deselectAllImage='" + LyString.getXmlString(_deselectAllImage) + "'");
		if (((_template == null) && (_moveDownImage != getStringValue("RELATION.moveDownImage", "$ALL.moveDownImage")))
				|| ((_template != null) && (_moveDownImage != _template._moveDownImage)))
			pw.print(" moveDownImage='" + LyString.getXmlString(_moveDownImage) + "'");
		if (((_template == null) && (_moveUpImage != getStringValue("RELATION.moveUpImage", "$ALL.moveUpImage")))
				|| ((_template != null) && (_moveUpImage != _template._moveUpImage)))
			pw.print(" moveUpImage='" + LyString.getXmlString(_moveUpImage) + "'");
		if (((_template == null) && (_arrowDownBig != getStringValue("RELATION.arrowDownBig", "$ALL.arrowDownBig")))
				|| ((_template != null) && (_arrowDownBig != _template._arrowDownBig)))
			pw.print(" arrowDownBig='" + LyString.getXmlString(_arrowDownBig) + "'");
		if (((_template == null) && (_selectedColor != getStringValue("RELATION.selectedColor", "$ALL.selectedColor")))
				|| ((_template != null) && (_selectedColor != _template._selectedColor)))
			pw.print(" selectedColor='" + LyString.getXmlString(_selectedColor) + "'");
		if (((_template == null) && (_inactiveSelectedColor != getStringValue("RELATION.inactiveSelectedColor", "$ALL.inactiveSelectedColor")))
				|| ((_template != null) && (_inactiveSelectedColor != _template._inactiveSelectedColor)))
			pw.print(" inactiveSelectedColor='" + LyString.getXmlString(_inactiveSelectedColor) + "'");
		if (((_template == null) && (_selectedFontColor != getStringValue("RELATION.selectedFontColor", "$ALL.selectedFontColor")))
				|| ((_template != null) && (_selectedFontColor != _template._selectedFontColor)))
			pw.print(" selectedFontColor='" + LyString.getXmlString(_selectedFontColor) + "'");
		if (((_template == null) && (_multiSelectorSelectImage != getStringValue("RELATION.multiSelectorSelectImage", "$ALL.multiSelectorSelectImage")))
				|| ((_template != null) && (_multiSelectorSelectImage != _template._multiSelectorSelectImage)))
			pw.print(" multiSelectorSelectImage='" + LyString.getXmlString(_multiSelectorSelectImage) + "'");
		if (((_template == null) && (_multiSelectorUnselectImage != getStringValue("RELATION.multiSelectorUnselectImage", "$ALL.multiSelectorUnselectImage")))
				|| ((_template != null) && (_multiSelectorUnselectImage != _template._multiSelectorUnselectImage)))
			pw.print(" multiSelectorUnselectImage='" + LyString.getXmlString(_multiSelectorUnselectImage) + "'");

		if ((getChildCount() == 0) && (_datas == null))
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		LyComponent	links;
		int			i, n;

		n = (_links == null ? 0 : _links.size());
		for (i = 0; i < n; i++)
		{
			links = _links.get(i);
			links.writeXml(pw, debug);
		}


		pw.println("</RELATION>");
	}
}
