/*
 * LyRow.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved. Current Version: $Id: LyRow.java,v 6.8 2007/12/05 09:01:22
 * jls Exp $.
 */

package leon.view;

/**
 * The LyRow class implements generic behavior for the ROW components as described in views.dtd
 * regardless of the requested target (win, swing, html...). It represents a row.
 *
 * @see  leon.view.LyComponent
 */
public class LyRow extends LyRowBase
{
	public static final String __VERSION = "$Id: LyRow.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Indicates if this component is selected or not. */
	boolean _hierarchical;

	/* Initialization */

	/**
	 * Initialisation of the data structure.
	 *
	 * @see  leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_hierarchical = false;
	}

	/**
	 * Constructor with all required attributes.
	 */
	public LyRow()
	{
		super();
	}

	/* Protected methods for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 *
	 * @param  node  Node from which the component is constructed.
	 *
	 * @see    leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyRow(LyViewNode node)
	{
		super(node);
	}

	/**
	 * Sets sub-component rows.
	 *
	 * @param  rows  New value of rows
	 */
	public void setRows(LyRows rows)
	{
		if (_rows != null)
			_rows.setParent(null);

		_rows = rows;
		if (rows != null)
		{
			setHierarchical(true);
			rows.setParent(this);
		}
	}

	protected void setHierarchical(boolean hierarchical)
	{
		if (_hierarchical == hierarchical)
			return;

		_hierarchical = hierarchical;

		if (_rows != null)
			_rows.setHierarchical(hierarchical);

		if (_parent instanceof LyRows)
			((LyRows)_parent).setHierarchical(hierarchical);
	}

	/**
	 * Initializes attribute id to the given value.
	 *
	 * @param  id  New value of id.
	 */
	public void setId(String id)
	{
		if ((id == _id)
				|| ((id != null)
					&& (_id != null)
					&& id.equals(_id)))
			return;

		if (_parent instanceof LyRows)
		{
			if (id != null)
				((LyRows)_parent)._rowsById.put(id, this);

			if (_id != null)
				((LyRows)_parent)._rowsById.remove(_id);
		}

		super.setId(id);
	}

	/**
	 * Sets attribute hidden to the given value.
	 *
	 * @param  hidden  Boolean indicating if attribute hidden is set or unset.
	 */
	public void setHidden(boolean hidden)
	{
		if ((hidden != _hidden) && (_rows != null))
		{
			if (hidden)
				_rows.addHiddenRow();
			else
				_rows.removeHiddenRow();
		}

		super.setHidden(hidden);
	}

} // end class LyRow
