/*
 * LyRowsBase.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;

import java.util.*;
import java.io.*;


/**
 * The LyRowsBase class is a generated Java implementation that corresponds to the element 
 * ROWS of the descriptive grammar of the views (views.dtd).
 * <B>&lt;ROWS&gt; : List of table rows</B>
 * This element allows displaying rows within a table.
 * It does not contain any attribute: this is only a non-empty list of rows.
 * @see leon.view.LyComponent
 * @see leon.view.LyLyRows
 * See also tag <A href="../../conf/dtd/view/ROWS.html">ROWS</A> of grammar leon_views.dtd.
 */
public abstract class LyRowsBase extends LyComponent
{
	public static final String			__VERSION = "$Id$";


	/* Sub components */

	/**
	 * Rows contained in the rows list.
	 */
	protected ArrayList<LyRow>		_rows;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize sub components
		_rows = null;
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;


		if (REWRITING_MODE)
			return;

		// Copy sub-components
		LyRow	rows;
		LyRows	model = (LyRows)component;

		int			i, n;

		n = (model._rows == null ? 0 : model._rows.size());
		for (i = 0; i < n; i++)
		{
			rows = model._rows.get(i);
			if (rows != null)
				rows = (LyRow)rows.cloneComponent();
			addChild(rows);
		}
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	public LyRowsBase()
	{
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyRowsBase(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
	}

	/**
	 * End of construction : completion with child component.
	 * @param child Sub child component.
	 * @see leon.view.LyComponent#addChild
	 */
	protected void addChild(LyComponent child)
	{
		// Add sub-components
		if (child instanceof LyRow)
			addRow((LyRow)child);
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyRows clone = new LyRows();

		clone.initFromModel(this);
		return clone;
	}


	/* Accessors */

	/**
	 * Get sub-component Row of index i.
	 * @param i the index of the Row to get.
	 * @return sub-component Row of index i.
	 */
	public LyRow getRow(int i)
	{
		if (_rows == null)
			return null;
		if ((i < 0) || (i >= _rows.size()))
			return null;
		return _rows.get(i);
	}

	/**
	 * Get sub-component Row count.
	 * @return sub-component Row count.
	 */
	public int getRowCount()
	{
		if (_rows == null)
			return 0;
		return _rows.size();
	}


	/* Modifiers */

	/**
	 * Adds a sub-component row.
	 * @param row The row to add to rows list.
	 */
	public void addRow(LyRow row)
	{
		insertRow(row, -1);
	}

	/**
	 * Inserts a sub-component row at given position.
	 * @param row The row to add to rows list.
	 * @param pos	Position where the sub-component is inserted.
	 */
	public void insertRow(LyRow row, int pos)
	{
		if (_rows == null)
		{
			_rows = new ArrayList<LyRow>(1);
			_children = _rows;
		}
		else if (_rows.contains(row))
			return;
		if (pos >= 0)
			_rows.add(pos,row);
		if (row != null)
			row.setParent(this);
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag
		pw.println("<ROWS>");

		// Write sub-components
		LyComponent	rows;
		int			i, n;

		n = (_rows == null ? 0 : _rows.size());
		for (i = 0; i < n; i++)
		{
			rows = _rows.get(i);
			rows.writeXml(pw, debug);
		}


		pw.println("</ROWS>");
	}
}
