/*
 * LySelectItem.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;
import java.io.*;
import leon.view.graphic.*;
import leon.view.getinterface.*;
import leon.view.setinterface.*;
import leon.misc.LyString;


/**
 * The LySelectItem class is a generated Java implementation that corresponds to the element 
 * SELECTITEM of the descriptive grammar of the views (views.dtd).
 * <B>&lt;SELECTITEM&gt; : Selection item</B>
 * A selection item allows pointing to a page of data that can be reached by simple
 * selection.
 * @see leon.view.LyComponent
 * See also tag <A href="../../conf/dtd/view/SELECTITEM.html">SELECTITEM</A> of grammar leon_views.dtd.
 */
public class LySelectItem extends LyComponent
	implements  LyGetIdInterface,
				LyGetLabelInterface,
				LySetLabelInterface,
				LySetImageInterface,
				LySetSelectedInterface,
				LySetEnabledInterface
{
	public static final String			__VERSION = "$Id$";


	/* Attributes */

	/**
	 * Identifier of the navigation item.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LySelectItem			_template;

	/**
	 * Displayable label of the navigation item.
	 */
	protected String			_label;

	/**
	 * Tooltip associated to the navigation item.
	 */
	protected String			_tooltip;

	/**
	 * Overview label associated to the navigation item.
	 */
	protected String			_overview;

	/**
	 * Image associated to the navigation item.
	 */
	protected String			_image;

	/**
	 * Indicates whether the navigation item is selected or not (current item).
	 * Only the first one that is selected in a navigation bar is taken into account.
	 */
	protected boolean			_selected;

	/**
	 * Indicates whether the navigation item is enabled or not.
	 */
	protected boolean			_enabled;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_id = null;
		_template = null;
		_label = null;
		_tooltip = null;
		_overview = null;
		_image = getStringValue("SELECTITEM.image", "$ALL.image");
		_selected = getBooleanValue("SELECTITEM.selected", false);
		_enabled = getBooleanValue("SELECTITEM.enabled", true);
		_color = getStringValue("SELECTITEM.color", "$ALL.color");
		_font = getStringValue("SELECTITEM.font", "$ALL.font");
		_fontColor = getStringValue("SELECTITEM.fontColor", "$ALL.fontColor");
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LySelectItem	model = (LySelectItem)component;

		_model = component;
		_label = model._label;
		_tooltip = model._tooltip;
		_overview = model._overview;
		_image = model._image;
		_selected = model._selected;
		_enabled = model._enabled;
		_color = model._color;
		_font = model._font;
		_fontColor = model._fontColor;
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	protected LySelectItem()
	{
		_id = getInternalId();
	}

	/**
	 * Constructor with all required attributes.
	 * @param id Identifier of the navigation item.
	 * @param label Displayable label of the navigation item.
	 */
	public LySelectItem(String id, String label)
	{
		// Fill up required attributes with parameters
		_id = (id != null ? id.intern() : null);
		_label = label;
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LySelectItem(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_id = node.getString("id");
		_template = (LySelectItem)node.getComponent("template");

		if (_template != null)
			initFromModel(_template);

		if (node.isSpecified("label"))
			_label = node.getString("label");
		if (node.isSpecified("tooltip"))
			_tooltip = node.getString("tooltip");
		if (node.isSpecified("overview"))
			_overview = node.getString("overview");
		if (node.isSpecified("image"))
			_image = node.getString("image");
		if (node.isSpecified("selected"))
			_selected = node.getBoolean("selected");
		if (node.isSpecified("enabled"))
			_enabled = node.getBoolean("enabled");
		if (node.isSpecified("color"))
			_color = node.getString("color");
		if (node.isSpecified("font"))
			_font = node.getString("font");
		if (node.isSpecified("fontColor"))
			_fontColor = node.getString("fontColor");
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LySelectItem clone = new LySelectItem();

		clone.initFromModel(this);
		clone._template = this;
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of id.
	 * @return value of id.
	 */
	public String getId()
	{
		return _id;
	}

	/**
	 * Gets value of template.
	 * @return value of template.
	 */
	public LySelectItem getTemplate()
	{
		return _template;
	}

	/**
	 * Gets value of label.
	 * @return value of label.
	 */
	public String getLabel()
	{
		return _label;
	}

	/**
	 * Gets value of tooltip.
	 * @return value of tooltip.
	 */
	public String getTooltip()
	{
		return _tooltip;
	}

	/**
	 * Gets value of overview.
	 * @return value of overview.
	 */
	public String getOverview()
	{
		return _overview;
	}

	/**
	 * Gets value of image.
	 * @return value of image.
	 */
	public String getImage()
	{
		return _image;
	}

	/**
	 * Checks whether the image attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultImage()
	{
		return (_image == getStringValue("SELECTITEM.image"));
	}

	/**
	 * Checks if value of attribute selected is set.
	 * @return true if selected is set, false otherwise.
	 */
	public boolean isSelected()
	{
		return _selected;
	}

	/**
	 * Checks if value of attribute enabled is set.
	 * @return true if enabled is set, false otherwise.
	 */
	public boolean isEnabled()
	{
		return _enabled;
	}

	/**
	 * Gets value of color.
	 * @return value of color.
	 */
	public String getColor()
	{
		return _color;
	}

	/**
	 * Checks whether the color attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultColor()
	{
		return (_color == getStringValue("SELECTITEM.color"));
	}

	/**
	 * Gets value of font.
	 * @return value of font.
	 */
	public String getFont()
	{
		return _font;
	}

	/**
	 * Checks whether the font attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFont()
	{
		return (_font == getStringValue("SELECTITEM.font"));
	}

	/**
	 * Gets value of fontColor.
	 * @return value of fontColor.
	 */
	public String getFontColor()
	{
		return _fontColor;
	}

	/**
	 * Checks whether the fontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFontColor()
	{
		return (_fontColor == getStringValue("SELECTITEM.fontColor"));
	}


	/* Modifiers */

	/**
	 * Initializes attribute template to the given value.
	 * @param template New value of template.
	 */
	public void setTemplate(LySelectItem template)
	{
		if (template == null)
			return;
		if (_template != null)
			throw new IllegalStateException("BF AppEngine : attribute template was already set.");

		initFromModel(template);
		_template = template;
	}

	/**
	 * Sets attribute label to the given value.
	 * @param label New value of label.
	 * @exception IllegalArgumentException if label is null.
	 */
	public void setLabel(String label)
	{
		if (label == null)
			throw new IllegalArgumentException("BF AppEngine : attribute label is required (can't be null)");

		if ((label == _label)
			|| ((label != null)
				&& (_label != null)
				&& label.equals(_label)))
			return;

		_label = label;
		if (_internalView != null)
			((LyGraphicalSelectItem)_internalView).setLabel(label);
	}

	/**
	 * Initializes attribute tooltip to the given value.
	 * @param tooltip New value of tooltip.
	 */
	public void setTooltip(String tooltip)
	{
		if ((tooltip == _tooltip)
			|| ((tooltip != null)
				&& (_tooltip != null)
				&& tooltip.equals(_tooltip)))
			return;

		_tooltip = tooltip;
	}

	/**
	 * Initializes attribute overview to the given value.
	 * @param overview New value of overview.
	 */
	public void setOverview(String overview)
	{
		if ((overview == _overview)
			|| ((overview != null)
				&& (_overview != null)
				&& overview.equals(_overview)))
			return;

		_overview = overview;
	}

	/**
	 * Sets attribute image to the given value.
	 * @param image New value of image.
	 */
	public void setImage(String image)
	{
		if ((image == _image)
			|| ((image != null)
				&& (_image != null)
				&& image.equals(_image)))
			return;

		_image = image;
		if (_internalView != null)
			((LyGraphicalSelectItem)_internalView).setImage(image);
	}

	/**
	 * Sets attribute selected to the given value.
	 * @param selected Boolean indicating if attribute selected is set or unset.
	 */
	public void setSelected(boolean selected)
	{
		if (selected == _selected)
			return;
		_selected = selected;
		if (_internalView != null)
			((LyGraphicalSelectItem)_internalView).setSelected(selected);
	}

	/**
	 * Sets attribute enabled to the given value.
	 * @param enabled Boolean indicating if attribute enabled is set or unset.
	 */
	public void setEnabled(boolean enabled)
	{
		if (enabled == _enabled)
			return;
		_enabled = enabled;
		if (_internalView != null)
			((LyGraphicalSelectItem)_internalView).setEnabled(enabled);
	}

	/**
	 * Initializes attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color)
	{
		if ((color == _color)
			|| ((color != null)
				&& (_color != null)
				&& color.equals(_color)))
			return;

		_color = color;
	}

	/**
	 * Initializes attribute font to the given value.
	 * @param font New value of font.
	 */
	public void setFont(String font)
	{
		if ((font == _font)
			|| ((font != null)
				&& (_font != null)
				&& font.equals(_font)))
			return;

		_font = font;
	}

	/**
	 * Initializes attribute fontColor to the given value.
	 * @param fontColor New value of fontColor.
	 */
	public void setFontColor(String fontColor)
	{
		if ((fontColor == _fontColor)
			|| ((fontColor != null)
				&& (_fontColor != null)
				&& fontColor.equals(_fontColor)))
			return;

		_fontColor = fontColor;
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<SELECTITEM");
		pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		pw.print(" label='" + LyString.getXmlString(_label) + "'");
		if (((_template == null) && (_tooltip != null))
				|| ((_template != null) && (_tooltip != _template._tooltip)))
			pw.print(" tooltip='" + LyString.getXmlString(_tooltip) + "'");
		if (((_template == null) && (_overview != null))
				|| ((_template != null) && (_overview != _template._overview)))
			pw.print(" overview='" + LyString.getXmlString(_overview) + "'");
		if (((_template == null) && (_image != getStringValue("SELECTITEM.image", "$ALL.image")))
				|| ((_template != null) && (_image != _template._image)))
			pw.print(" image='" + LyString.getXmlString(_image) + "'");
		if (((_template == null) && _selected)
				|| ((_template != null) && (_selected != _template._selected)))
			pw.print(" selected='" + _selected + "'");
		if (((_template == null) && !_enabled)
				|| ((_template != null) && (_enabled != _template._enabled)))
			pw.print(" enabled='" + _enabled + "'");
		if (((_template == null) && (_color != getStringValue("SELECTITEM.color", "$ALL.color")))
				|| ((_template != null) && (_color != _template._color)))
			pw.print(" color='" + LyString.getXmlString(_color) + "'");
		if (((_template == null) && (_font != getStringValue("SELECTITEM.font", "$ALL.font")))
				|| ((_template != null) && (_font != _template._font)))
			pw.print(" font='" + LyString.getXmlString(_font) + "'");
		if (((_template == null) && (_fontColor != getStringValue("SELECTITEM.fontColor", "$ALL.fontColor")))
				|| ((_template != null) && (_fontColor != _template._fontColor)))
			pw.print(" fontColor='" + LyString.getXmlString(_fontColor) + "'");
		pw.println("/>");
	}
}
