/*
 * LyTab.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved. Current Version: $Id: LyTab.java,v 6.10 2008/07/21
 * 09:36:45 nha Exp $.
 */

package leon.view;

import leon.view.getinterface.*;
import leon.view.graphic.*;
import leon.view.setinterface.*;


/**
 * The LyTab class implements generic behavior for the TAB components as described in views.dtd
 * regardless of the requested target (win, swing, html...). It represents a tab.
 *
 * @see  leon.view.LyComponent
 */
public class LyTab extends LyTabBase
	implements LyGetLabelInterface
{
	public static final String __VERSION = "$Id: LyTab.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Target which should be submitted when tag is clicked (internal use for tabactions). */
	String _targetFormId;

	/**
	 * Constructor with all required attributes.
	 */
	public LyTab()
	{
	}

	/**
	 * Construction from a LyViewNode.
	 *
	 * @param  node  Node from which the component is constructed.
	 *
	 * @see    leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyTab(LyViewNode node)
	{
		super(node);
	}

	/**
	 * Sets attribute enabled to the given value.
	 *
	 * @param  enabled  Boolean indicating if attribute enabled is set or unset.
	 */
	public void setEnabled(boolean enabled)
	{
		super.setEnabled(enabled);

		if (_parent == null)
			return;

		if (_parent._internalView == null)
			return;

		Object internal = _parent.getInternalView();

		if (internal instanceof LySetTabInterface)
			((LySetTabInterface)_parent.getInternalView()).setEnabled(_label, enabled);
	}

	/**
	 * Initializes attribute label to the given value.
	 *
	 * @param  label  New value of label.
	 */
	public void setLabel(String label)
	{
		super.setLabel(label);

		if (_parent == null)
			return;

		if (_parent._internalView == null)
			return;

		Object internal = _parent.getInternalView();

		if (internal instanceof LySetTabInterface)
			((LySetTabInterface)_parent.getInternalView()).setLabel(_label, label);
	}

	/**
	 * Gets value of targetFormId.
	 *
	 * @return  value of targetFormId.
	 */
	public String getTargetFormId()
	{
		return _targetFormId;
	}

	/**
	 * Initializes attribute targetFormId to the given value.
	 *
	 * @param  targetFormId  New value of targetFormId.
	 */
	public void setTargetFormId(String targetFormId)
	{
		if ((targetFormId == _targetFormId)
				|| ((targetFormId != null)
					&& (_targetFormId != null)
					&& targetFormId.equals(_targetFormId)))
			return;

		_targetFormId = targetFormId;
	}

	/**
	 * Returns the position and size of current component relatively to given component.
	 *
	 * @param   parentComponent  the component used to compute relative position of current
	 *                           component. If null, position of current component is computed with
	 *                           the top-left corner of the screen as the origin.
	 *
	 * @return  an array of 4 integers (x position, y position, width, height) representing the
	 *          position and size of current component.
	 */
	public int[] getBounds(LyComponent parentComponent)
	{
		LyTabs tabs = getParentTabs();

		if (tabs != null)
		{
			LyViewComponent   internal = tabs.getInternalView(false);
			LyGraphicalObject object = (LyGraphicalObject)internal;
			int[]			  parentBounds = tabs.getBounds(parentComponent);
			int[]			  bounds = object.getChildrenBounds(this);

			if ((bounds == null) || (parentBounds == null))
				return null;

			return new int[] { bounds[0] + parentBounds[0], bounds[1] + parentBounds[1], bounds[2], bounds[3] };
		}

		return null;
	}
} // end class LyTab
