/*
 * LyTitleBarBase.java
 * Copyright (c) 2000-2010, Lyria-W4.
 * All rights reserved.
 */

package leon.view;
import java.io.*;
import leon.view.graphic.*;
import leon.view.getinterface.*;
import leon.view.setinterface.*;
import leon.misc.LyString;


/**
 * The LyTitleBarBase class is a generated Java implementation that corresponds to the element 
 * TITLEBAR of the descriptive grammar of the views (views.dtd).
 * <B>&lt;TITLEBAR&gt; : Title bar</B>
 * A title bar allows displaying a colored horizontal bar containing images and 
 * texts.
 * The text comprises three independent parts and each one of them is optional: a 
 * left, a central and a right part.
 * The color of the title bar is defined in the resources of the application.
 * @see leon.view.LyComponent
 * @see leon.view.LyLyTitleBar
 * See also tag <A href="../../conf/dtd/view/TITLEBAR.html">TITLEBAR</A> of grammar leon_views.dtd.
 */
public abstract class LyTitleBarBase extends LyComponent
	implements  LyGetIdInterface,
				LySetImageInterface,
				LyGetConstraintInterface
{
	public static final String			__VERSION = "$Id$";


	/* Attributes */

	/**
	 * Identifier of the title bar.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyTitleBar			_template;

	/**
	 * Image of the title bar.
	 */
	protected String			_image;

	/**
	 * Text of the tooltip over the image
	 */
	protected String			_imageToolTip;

	/**
	 * Indicates whether the image displayed in the title bar is sensitive or not.
	 */
	protected boolean			_imageSensitive;

	/**
	 * Right image of the title bar.
	 */
	protected String			_rightImage;

	/**
	 * Text of the tooltip over the image on the right
	 */
	protected String			_rightImageToolTip;

	/**
	 * Left label of the title bar.
	 */
	protected String			_leftLabel;

	/**
	 * Central label of the title bar.
	 */
	protected String			_centerLabel;

	/**
	 * Right label of the title bar.
	 */
	protected String			_rightLabel;

	/**
	 * Indicates whether the label on the right is sensitive or not (presented as an 
	 * hyperlink).
	 */
	protected boolean			_rightLabelSensitive;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;


	/* Sub components */

	/**
	 * Toolbar displayed on the right of the title bar.
	 */
	protected LyToolBar			_toolBar;

	/**
	 * Positioning constraint of the title bar.
	 */
	protected LyConstraint		_constraint;


	/* Initialization */

	/**
	 * Initialization of the data structure.
	 * @see leon.view.LyComponent#init
	 */
	protected void init()
	{
		super.init();

		// Initialize attributes
		_id = null;
		_template = null;
		_image = getStringValue("TITLEBAR.image", "$ALL.image");
		_imageToolTip = null;
		_imageSensitive = getBooleanValue("TITLEBAR.imageSensitive", false);
		_rightImage = getStringValue("TITLEBAR.rightImage", "$ALL.rightImage");
		_rightImageToolTip = null;
		_leftLabel = null;
		_centerLabel = null;
		_rightLabel = null;
		_rightLabelSensitive = getBooleanValue("TITLEBAR.rightLabelSensitive", false);
		_font = getStringValue("TITLEBAR.font", "$ALL.font");
		_color = getStringValue("TITLEBAR.color", "$ALL.color");
		_fontColor = getStringValue("TITLEBAR.fontColor", "$ALL.fontColor");

		// Initialize sub components
		_toolBar = null;
		_constraint = null;
	}

	/**
	 * Method for initializing a component from a model.
	 */
	protected void initFromModel(LyComponent component)
	{
		if ((component == null)
				|| (_model != null)) // Already initialized from its model
			return;

		LyTitleBar	model = (LyTitleBar)component;

		_model = component;
		_image = model._image;
		_imageToolTip = model._imageToolTip;
		_imageSensitive = model._imageSensitive;
		_rightImage = model._rightImage;
		_rightImageToolTip = model._rightImageToolTip;
		_leftLabel = model._leftLabel;
		_centerLabel = model._centerLabel;
		_rightLabel = model._rightLabel;
		_rightLabelSensitive = model._rightLabelSensitive;
		_font = model._font;
		_color = model._color;
		_fontColor = model._fontColor;

		if (REWRITING_MODE)
			return;

		// Copy sub-components
		if (model._toolBar != null)
			addChild(model._toolBar.cloneComponent());
		if (model._constraint != null)
			addChild(model._constraint.cloneComponent());
	}


	/* Constructors */

	/**
	 * Default constructor.
	 */
	public LyTitleBarBase()
	{
	}


	/* Protected method for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 * @param node	Node from which the component is constructed.
	 * @see leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyTitleBarBase(LyViewNode node)
	{
		initFromViewNode(node);
	}

	/**
	 * Inits from a LyViewNode.
	 * @param node    Node from which the component is initialized.
	 */
	protected void initFromViewNode(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		_id = node.getString("id");
		_template = (LyTitleBar)node.getComponent("template");

		if (_template != null)
			initFromModel(_template);

		if (node.isSpecified("image"))
			_image = node.getString("image");
		if (node.isSpecified("imageToolTip"))
			_imageToolTip = node.getString("imageToolTip");
		if (node.isSpecified("imageSensitive"))
			_imageSensitive = node.getBoolean("imageSensitive");
		if (node.isSpecified("rightImage"))
			_rightImage = node.getString("rightImage");
		if (node.isSpecified("rightImageToolTip"))
			_rightImageToolTip = node.getString("rightImageToolTip");
		if (node.isSpecified("leftLabel"))
			_leftLabel = node.getString("leftLabel");
		if (node.isSpecified("centerLabel"))
			_centerLabel = node.getString("centerLabel");
		if (node.isSpecified("rightLabel"))
			_rightLabel = node.getString("rightLabel");
		if (node.isSpecified("rightLabelSensitive"))
			_rightLabelSensitive = node.getBoolean("rightLabelSensitive");
		if (node.isSpecified("font"))
			_font = node.getString("font");
		if (node.isSpecified("color"))
			_color = node.getString("color");
		if (node.isSpecified("fontColor"))
			_fontColor = node.getString("fontColor");
	}

	/**
	 * End of construction : completion with child component.
	 * @param child Sub child component.
	 * @see leon.view.LyComponent#addChild
	 */
	protected void addChild(LyComponent child)
	{
		// Add sub-components
		if (child instanceof LyToolBar)
			setToolBar((LyToolBar)child);
		else if (child instanceof LyConstraint)
			setConstraint((LyConstraint)child);
	}

	/**
	 * Method for cloning a component.
	 */
	public LyComponent cloneComponent()
	{
		LyTitleBar clone = new LyTitleBar();

		clone.initFromModel(this);
		clone._template = (LyTitleBar)this;
		return clone;
	}


	/* Accessors */

	/**
	 * Gets value of id.
	 * @return value of id.
	 */
	public String getId()
	{
		return _id;
	}

	/**
	 * Gets value of template.
	 * @return value of template.
	 */
	public LyTitleBar getTemplate()
	{
		return _template;
	}

	/**
	 * Gets value of image.
	 * @return value of image.
	 */
	public String getImage()
	{
		return _image;
	}

	/**
	 * Checks whether the image attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultImage()
	{
		return (_image == getStringValue("TITLEBAR.image"));
	}

	/**
	 * Gets value of imageToolTip.
	 * @return value of imageToolTip.
	 */
	public String getImageToolTip()
	{
		return _imageToolTip;
	}

	/**
	 * Checks if value of attribute imageSensitive is set.
	 * @return true if imageSensitive is set, false otherwise.
	 */
	public boolean isImageSensitive()
	{
		return _imageSensitive;
	}

	/**
	 * Gets value of rightImage.
	 * @return value of rightImage.
	 */
	public String getRightImage()
	{
		return _rightImage;
	}

	/**
	 * Checks whether the rightImage attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultRightImage()
	{
		return (_rightImage == getStringValue("TITLEBAR.rightImage"));
	}

	/**
	 * Gets value of rightImageToolTip.
	 * @return value of rightImageToolTip.
	 */
	public String getRightImageToolTip()
	{
		return _rightImageToolTip;
	}

	/**
	 * Gets value of leftLabel.
	 * @return value of leftLabel.
	 */
	public String getLeftLabel()
	{
		return _leftLabel;
	}

	/**
	 * Gets value of centerLabel.
	 * @return value of centerLabel.
	 */
	public String getCenterLabel()
	{
		return _centerLabel;
	}

	/**
	 * Gets value of rightLabel.
	 * @return value of rightLabel.
	 */
	public String getRightLabel()
	{
		return _rightLabel;
	}

	/**
	 * Checks if value of attribute rightLabelSensitive is set.
	 * @return true if rightLabelSensitive is set, false otherwise.
	 */
	public boolean isRightLabelSensitive()
	{
		return _rightLabelSensitive;
	}

	/**
	 * Gets value of font.
	 * @return value of font.
	 */
	public String getFont()
	{
		return _font;
	}

	/**
	 * Checks whether the font attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFont()
	{
		return (_font == getStringValue("TITLEBAR.font"));
	}

	/**
	 * Gets value of color.
	 * @return value of color.
	 */
	public String getColor()
	{
		return _color;
	}

	/**
	 * Checks whether the color attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultColor()
	{
		return (_color == getStringValue("TITLEBAR.color"));
	}

	/**
	 * Gets value of fontColor.
	 * @return value of fontColor.
	 */
	public String getFontColor()
	{
		return _fontColor;
	}

	/**
	 * Checks whether the fontColor attribute is valuated to the default value.
	 * @return True if default, false otherwise.
	 */
	public boolean isDefaultFontColor()
	{
		return (_fontColor == getStringValue("TITLEBAR.fontColor"));
	}

	/**
	 * Get sub-component ToolBar.
	 * @return sub-component ToolBar.
	 */
	public LyToolBar getToolBar()
	{
		return _toolBar;
	}

	/**
	 * Get sub-component Constraint.
	 * @return sub-component Constraint.
	 */
	public LyConstraint getConstraint()
	{
		return _constraint;
	}


	/* Modifiers */

	/**
	 * Initializes attribute id to the given value.
	 * @param id New value of id.
	 */
	public void setId(String id)
	{
		if ((id == _id)
			|| ((id != null)
				&& (_id != null)
				&& id.equals(_id)))
			return;

		_id = (id != null ? id.intern() : null);
	}

	/**
	 * Initializes attribute template to the given value.
	 * @param template New value of template.
	 */
	public void setTemplate(LyTitleBar template)
	{
		if (template == null)
			return;
		if (_template != null)
			throw new IllegalStateException("BF AppEngine : attribute template was already set.");

		initFromModel(template);
		_template = template;
	}

	/**
	 * Sets attribute image to the given value.
	 * @param image New value of image.
	 */
	public void setImage(String image)
	{
		if ((image == _image)
			|| ((image != null)
				&& (_image != null)
				&& image.equals(_image)))
			return;

		_image = image;
		if (_internalView != null)
			((LyGraphicalTitleBar)_internalView).setImage(image);
	}

	/**
	 * Sets attribute imageToolTip to the given value.
	 * @param imageToolTip New value of imageToolTip.
	 */
	public void setImageToolTip(String imageToolTip)
	{
		if ((imageToolTip == _imageToolTip)
			|| ((imageToolTip != null)
				&& (_imageToolTip != null)
				&& imageToolTip.equals(_imageToolTip)))
			return;

		_imageToolTip = imageToolTip;
		if (_internalView != null)
			((LyGraphicalTitleBar)_internalView).setImageToolTip(imageToolTip);
	}

	/**
	 * Initializes attribute imageSensitive to the given value.
	 * @param imageSensitive Boolean indicating if attribute imageSensitive is set or unset.
	 */
	public void setImageSensitive(boolean imageSensitive)
	{
		if (imageSensitive == _imageSensitive)
			return;
		_imageSensitive = imageSensitive;
	}

	/**
	 * Sets attribute rightImage to the given value.
	 * @param rightImage New value of rightImage.
	 */
	public void setRightImage(String rightImage)
	{
		if ((rightImage == _rightImage)
			|| ((rightImage != null)
				&& (_rightImage != null)
				&& rightImage.equals(_rightImage)))
			return;

		_rightImage = rightImage;
		if (_internalView != null)
			((LyGraphicalTitleBar)_internalView).setRightImage(rightImage);
	}

	/**
	 * Sets attribute rightImageToolTip to the given value.
	 * @param rightImageToolTip New value of rightImageToolTip.
	 */
	public void setRightImageToolTip(String rightImageToolTip)
	{
		if ((rightImageToolTip == _rightImageToolTip)
			|| ((rightImageToolTip != null)
				&& (_rightImageToolTip != null)
				&& rightImageToolTip.equals(_rightImageToolTip)))
			return;

		_rightImageToolTip = rightImageToolTip;
		if (_internalView != null)
			((LyGraphicalTitleBar)_internalView).setRightImageToolTip(rightImageToolTip);
	}

	/**
	 * Sets attribute leftLabel to the given value.
	 * @param leftLabel New value of leftLabel.
	 */
	public void setLeftLabel(String leftLabel)
	{
		if ((leftLabel == _leftLabel)
			|| ((leftLabel != null)
				&& (_leftLabel != null)
				&& leftLabel.equals(_leftLabel)))
			return;

		_leftLabel = leftLabel;
		if (_internalView != null)
			((LyGraphicalTitleBar)_internalView).setLeftLabel(leftLabel);
	}

	/**
	 * Sets attribute centerLabel to the given value.
	 * @param centerLabel New value of centerLabel.
	 */
	public void setCenterLabel(String centerLabel)
	{
		if ((centerLabel == _centerLabel)
			|| ((centerLabel != null)
				&& (_centerLabel != null)
				&& centerLabel.equals(_centerLabel)))
			return;

		_centerLabel = centerLabel;
		if (_internalView != null)
			((LyGraphicalTitleBar)_internalView).setCenterLabel(centerLabel);
	}

	/**
	 * Sets attribute rightLabel to the given value.
	 * @param rightLabel New value of rightLabel.
	 */
	public void setRightLabel(String rightLabel)
	{
		if ((rightLabel == _rightLabel)
			|| ((rightLabel != null)
				&& (_rightLabel != null)
				&& rightLabel.equals(_rightLabel)))
			return;

		_rightLabel = rightLabel;
		if (_internalView != null)
			((LyGraphicalTitleBar)_internalView).setRightLabel(rightLabel);
	}

	/**
	 * Initializes attribute rightLabelSensitive to the given value.
	 * @param rightLabelSensitive Boolean indicating if attribute rightLabelSensitive is set or unset.
	 */
	public void setRightLabelSensitive(boolean rightLabelSensitive)
	{
		if (rightLabelSensitive == _rightLabelSensitive)
			return;
		_rightLabelSensitive = rightLabelSensitive;
	}

	/**
	 * Initializes attribute font to the given value.
	 * @param font New value of font.
	 */
	public void setFont(String font)
	{
		if ((font == _font)
			|| ((font != null)
				&& (_font != null)
				&& font.equals(_font)))
			return;

		_font = font;
	}

	/**
	 * Initializes attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color)
	{
		if ((color == _color)
			|| ((color != null)
				&& (_color != null)
				&& color.equals(_color)))
			return;

		_color = color;
	}

	/**
	 * Initializes attribute fontColor to the given value.
	 * @param fontColor New value of fontColor.
	 */
	public void setFontColor(String fontColor)
	{
		if ((fontColor == _fontColor)
			|| ((fontColor != null)
				&& (_fontColor != null)
				&& fontColor.equals(_fontColor)))
			return;

		_fontColor = fontColor;
	}

	/**
	 * Sets sub-component toolBar.
	 * @param toolBar New value of toolBar
	 */
	public void setToolBar(LyToolBar toolBar)
	{
		if (_toolBar != null)
			_toolBar.setParent(null);
		_toolBar = toolBar;
		if (toolBar != null)
			toolBar.setParent(this);
	}

	/**
	 * Sets sub-component constraint.
	 * @param constraint New value of constraint
	 */
	public void setConstraint(LyConstraint constraint)
	{
		if (_constraint != null)
			_constraint.setParent(null);
		_constraint = constraint;
		if (constraint != null)
			constraint.setParent(this);
	}


	/* XML Writing */

	/**
	 * Method to rewrite XML corresponding to this component.
	 * @param pw	the printWriter on which the XML is written.
	 * @param debug	true if debug mode (full dump).
	 */
	public void writeXml(PrintWriter pw, boolean debug)
	{
		// Write tag with its attributes
		pw.print("<TITLEBAR");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && (_image != getStringValue("TITLEBAR.image", "$ALL.image")))
				|| ((_template != null) && (_image != _template._image)))
			pw.print(" image='" + LyString.getXmlString(_image) + "'");
		if (((_template == null) && (_imageToolTip != null))
				|| ((_template != null) && (_imageToolTip != _template._imageToolTip)))
			pw.print(" imageToolTip='" + LyString.getXmlString(_imageToolTip) + "'");
		if (((_template == null) && _imageSensitive)
				|| ((_template != null) && (_imageSensitive != _template._imageSensitive)))
			pw.print(" imageSensitive='" + _imageSensitive + "'");
		if (((_template == null) && (_rightImage != getStringValue("TITLEBAR.rightImage", "$ALL.rightImage")))
				|| ((_template != null) && (_rightImage != _template._rightImage)))
			pw.print(" rightImage='" + LyString.getXmlString(_rightImage) + "'");
		if (((_template == null) && (_rightImageToolTip != null))
				|| ((_template != null) && (_rightImageToolTip != _template._rightImageToolTip)))
			pw.print(" rightImageToolTip='" + LyString.getXmlString(_rightImageToolTip) + "'");
		if (((_template == null) && (_leftLabel != null))
				|| ((_template != null) && (_leftLabel != _template._leftLabel)))
			pw.print(" leftLabel='" + LyString.getXmlString(_leftLabel) + "'");
		if (((_template == null) && (_centerLabel != null))
				|| ((_template != null) && (_centerLabel != _template._centerLabel)))
			pw.print(" centerLabel='" + LyString.getXmlString(_centerLabel) + "'");
		if (((_template == null) && (_rightLabel != null))
				|| ((_template != null) && (_rightLabel != _template._rightLabel)))
			pw.print(" rightLabel='" + LyString.getXmlString(_rightLabel) + "'");
		if (((_template == null) && _rightLabelSensitive)
				|| ((_template != null) && (_rightLabelSensitive != _template._rightLabelSensitive)))
			pw.print(" rightLabelSensitive='" + _rightLabelSensitive + "'");
		if (((_template == null) && (_font != getStringValue("TITLEBAR.font", "$ALL.font")))
				|| ((_template != null) && (_font != _template._font)))
			pw.print(" font='" + LyString.getXmlString(_font) + "'");
		if (((_template == null) && (_color != getStringValue("TITLEBAR.color", "$ALL.color")))
				|| ((_template != null) && (_color != _template._color)))
			pw.print(" color='" + LyString.getXmlString(_color) + "'");
		if (((_template == null) && (_fontColor != getStringValue("TITLEBAR.fontColor", "$ALL.fontColor")))
				|| ((_template != null) && (_fontColor != _template._fontColor)))
			pw.print(" fontColor='" + LyString.getXmlString(_fontColor) + "'");

		if ((getChildCount() == 0) && (_datas == null))
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		if (_toolBar != null)
			_toolBar.writeXml(pw, debug);
		if (_constraint != null)
			_constraint.writeXml(pw, debug);

		pw.println("</TITLEBAR>");
	}
}
