/*
 * LyTool.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved. Current Version: $Id: LyTool.java,v 6.8 2008/08/21
 * 13:12:27 nha Exp $.
 */

package leon.view;

import leon.view.graphic.*;


/**
 * The LyTool class implements generic behavior for the TOOL components as described in views.dtd
 * regardless of the requested target (win, swing, html...). It represents a tool.
 *
 * @see  leon.view.LyComponent
 */
public class LyTool extends LyToolBase
{
	public static final String __VERSION = "$Id: LyTool.java 35025 2012-03-23 16:43:07Z jlsaouli $";

	/**
     * Default constructor.
     */
    protected LyTool()
    {
    }
    
	/**
	 * Constructor with all required attributes.
	 */
	public LyTool(String id, String label)
	{
		// Fill up required attributes with parameters
		super(id, label);
		_width = -1;
		_height = -1;
	}

	/* Protected methods for construction from parser  */

	/**
	 * Construction from a LyViewNode.
	 *
	 * @param  node  Node from which the component is constructed.
	 *
	 * @see    leon.view.LyViewNodeFactory#buildElement
	 */
	protected LyTool(LyViewNode node)
	{
		// Retrieve attributes from node attributes
		super(node);
		_width = -1;
		_height = -1;
	}

	/**
	 * Sets attribute enabled to the given value.
	 *
	 * @param  enabled  Boolean indicating if attribute enabled is set or unset.
	 */
	public void setEnabled(boolean enabled)
	{
		super.setEnabled(enabled);

		// If tool contains an area, notify internal view even if state did not change.
		// May be useful in web mode to refresh javascript code describing area.
		if ((getComponent() != null)
				&& (_internalView != null))
			((LyGraphicalTool)_internalView).setEnabled(enabled);
	} // end method setEnabled
} // end class LyTool
