/**
 * LyViewNode.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved.
 */

package leon.view;

import leon.misc.*;
import leon.parser.*;

import org.xml.sax.*;
import org.xml.sax.ext.*;


/**
 * This class is used to offer temporary storage to LyComponent instances. We chose not to extend
 * ElementNode directly from LyComponent to save memory. In user execution mode, these instances can
 * be freed after loading because they are totally unuseful at this time.
 *
 * @see      LyXmlNode
 * @see      LyComponent
 * @version  $Id: LyViewNode.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author   Lyria
 */
public class LyViewNode extends LyXmlNode
{
	public static final String __VERSION = "$Id: LyViewNode.java 28422 2010-11-03 12:03:38Z ehelmste $";

	protected Attributes        _attributes;
	protected LyViewNodeFactory _factory;

	/**
	 * Constructor.
	 */
	public LyViewNode(Attributes attributes, LyViewNodeFactory factory)
	{
		_attributes = attributes;
		_factory = factory;
	}

	/**
	 * Overriden to fetch attribute name.
	 */
	public String getAttribute(String attributeName)
	{
		return _attributes.getValue(attributeName);
	}

	/**
	 * Access to the environment.
	 */
	public LyEnvironment getEnvironment()
	{
		return _factory.getEnvironment();
	}

	/**
	 * Utility method to get the value of an attribute as an int.
	 *
	 * @see  LyXmlNode#getInt
	 */
	public int getInt(String attributeName, int defaultValue)
	{
		String value = getString(attributeName);

		if ((value != null) && value.equals("remainder"))
			return LyConstraint.REMAINDER;
		else
			return super.getInt(attributeName, defaultValue);
	}
    
    /**
     * Utility method to get the value of an attribute as an LyComponent.
     */
    public LyComponent getComponent(String attributeName)
    {
        String value = getString(attributeName);
        
        return _factory.findComponent(value);
    }
   
    /**
     * Checks if the attribute with given name is specified  or not.
     */
    public boolean isSpecified(String attributeName)
    {        
        if (_attributes instanceof Attributes2)
        {
            int     index = _attributes.getIndex(attributeName);
            
            if (index < 0)
                return false;
            else
                return ((Attributes2)_attributes).isSpecified(index);
        }
        else
            return true;
    }
} // end class LyViewNode
