/**
 * LyViewParser.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved.
 */

package leon.view;

import java.io.*;

import leon.misc.*;
import leon.parser.*;


/**
 * This class represents our Xml view parser. It's the entry point to parse Xml view. Our parser is
 * configured to build LyComponent nodes instead of standard ElementNode nodes.
 *
 * @see      LySAXParser
 * @see      LyComponent
 * @version  $Id: LyViewParser.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author   Lyria
 */
public class LyViewParser extends LySAXParser
{
	public static final String __VERSION = "$Id: LyViewParser.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** The environment of the document. */
	protected LyEnvironment _environment;

    /**
     * Constructs an Xml Info Document.
     *
     * @param  xmlFile  Path of the file that contains the Xml info.
     */
    public LyViewParser(LyEnvironment environment, String xmlFile)
    {
        _environment = environment;
        
        // Set parser environment as current one
        LyEnvironment oldEnvironment = LyEnvironment.setCurrentEnvironment(_environment);
        
        try
        {
            LyViewNodeFactory factory = new LyViewNodeFactory(environment);
    
            // Load the document.
            loadDocument(xmlFile, factory, null, environment);
        }
        finally
        {
            // Remove current environment
            LyEnvironment.setCurrentEnvironment(oldEnvironment);
        }
    }

    /**
     * Constructs an Xml Info Document.
     *
     * @param  is  Input stream that contains the XML info
     */
    public LyViewParser(LyEnvironment environment, InputStream is)
    {
        _environment = environment;

        LyViewNodeFactory factory = new LyViewNodeFactory(environment);

        // Load the document.
        loadDocument(is, factory, null, environment);
    }

	/**
	 */
	public LyEnvironment getEnvironment()
	{
		return _environment;
	}
} // end class LyViewParser
