/*
 * Ly4DMatrix.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.common;

import leon.view.*;


/**
 * This class defines a 4x4 matrix.
 *
 * @version $Id: Ly4DMatrix.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public class Ly4DMatrix
{
	public static final String __VERSION =
		"$Id: Ly4DMatrix.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** xx value. */
	public float			   _xx;

	/** xy value. */
	public float			   _xy;

	/** xz value. */
	public float			   _xz;

	/** xw value. */
	public float			   _xw;

	/** yx value. */
	public float			   _yx;

	/** yy value. */
	public float			   _yy;

	/** yz value. */
	public float			   _yz;

	/** yw value. */
	public float			   _yw;

	/** zx value. */
	public float			   _zx;

	/** zy value. */
	public float			   _zy;

	/** zz value. */
	public float			   _zz;

	/** zw value. */
	public float			   _zw;

	/** wx value. */
	public float			   _wx;

	/** wy value. */
	public float			   _wy;

	/** wz value. */
	public float			   _wz;

	/** ww value. */
	public float			   _ww;

	/**
	 * Constructor.
	 */
	public Ly4DMatrix()
	{
		this(1F, 0F, 0F, 0F, 0F, 1F, 0F, 0F, 0F, 0F, 1F, 0F, 0F, 0F, 0F, 1F);
	}

	/**
	 * Constructor.
	 *
	 * @param xx xx value
	 * @param xy xy value
	 * @param xz xz value
	 * @param xw xw value
	 * @param yx yx value
	 * @param yy yy value
	 * @param yz yz value
	 * @param yw yw value
	 * @param zx zx value
	 * @param zy zy value
	 * @param zz zz value
	 * @param zw zw value
	 * @param wx wx value
	 * @param wy wy value
	 * @param wz wz value
	 * @param ww ww value
	 */
	public Ly4DMatrix(float xx, float xy, float xz, float xw,
		float yx, float yy, float yz, float yw,
		float zx, float zy, float zz, float zw,
		float wx, float wy, float wz, float ww)
	{
		init();

		setValues(xx, xy, xz, xw, yx, yy, yz, yw, zx, zy, zz, zw, wx, wy, wz, ww);
	}

	/**
	 * Init method.
	 */
	public void init()
	{
		_xx = 0F;
		_xy = 0F;
		_xz = 0F;
		_xw = 0F;
		_yx = 0F;
		_yy = 0F;
		_yz = 0F;
		_yw = 0F;
		_zx = 0F;
		_zy = 0F;
		_zz = 0F;
		_zw = 0F;
		_wx = 0F;
		_wy = 0F;
		_wz = 0F;
		_ww = 0F;
	}

	/**
	 * Translate the origin of the matrix.
	 *
	 * @param tx translation in x
	 * @param ty translation in y
	 * @param tz translation in z
	 */
	public void translate(float tx, float ty, float tz)
	{
		_xw += (tx * _xx) + (ty * _xy) + (tz * _xz);
		_yw += (tx * _yx) + (ty * _yy) + (tz * _yz);
		_zw += (tx * _zx) + (ty * _zy) + (tz * _zz);
		_ww += (tx * _wx) + (ty * _wy) + (tz * _wz);
	}

	/**
	 * Returns the result of matrix multiplication for the given 3D point.
	 *
	 * @param  point the point to transform.
	 * @return the result of the multiplication.
	 */
	public LyPoint getMultiplicationBy(LyPoint point)
	{
		LyPoint result = new LyPoint((_xx * point.getX()) + (_xy * point.getY())
				+ (_xz * point.getZ()) + _xw,
				(_yx * point.getX()) + (_yy * point.getY()) + (_yz * point.getZ()) + _yw,
				(_zx * point.getX()) + (_zy * point.getY()) + (_zz * point.getZ()) + _zw);

		result.scale(1f
				/ ((_wx * point.getX()) + (_wy * point.getY()) + (_wz * point.getZ()) + _ww));

		return result;
	}

	/**
	 * Multiply the current matrix by the given matrix (this*matrix).
	 *
	 * @param matrix the matrix to multiply by.
	 */
	public void multiply(Ly4DMatrix matrix)
	{
		float tmp1, tmp2, tmp3, tmp4;

		tmp1 = (_xx * matrix._xx) + (_xy * matrix._yx) + (_xz * matrix._zx) + (_xw * matrix._wx);
		tmp2 = (_xx * matrix._xy) + (_xy * matrix._yy) + (_xz * matrix._zy) + (_xw * matrix._wy);
		tmp3 = (_xx * matrix._xz) + (_xy * matrix._yz) + (_xz * matrix._zz) + (_xw * matrix._wz);
		tmp4 = (_xx * matrix._xw) + (_xy * matrix._yw) + (_xz * matrix._zw) + (_xw * matrix._ww);

		_xx = tmp1;
		_xy = tmp2;
		_xz = tmp3;
		_xw = tmp4;

		tmp1 = (_yx * matrix._xx) + (_yy * matrix._yx) + (_yz * matrix._zx) + (_yw * matrix._wx);
		tmp2 = (_yx * matrix._xy) + (_yy * matrix._yy) + (_yz * matrix._zy) + (_yw * matrix._wy);
		tmp3 = (_yx * matrix._xz) + (_yy * matrix._yz) + (_yz * matrix._zz) + (_yw * matrix._wz);
		tmp4 = (_yx * matrix._xw) + (_yy * matrix._yw) + (_yz * matrix._zw) + (_yw * matrix._ww);

		_yx = tmp1;
		_yy = tmp2;
		_yz = tmp3;
		_yw = tmp4;

		tmp1 = (_zx * matrix._xx) + (_zy * matrix._yx) + (_zz * matrix._zx) + (_zw * matrix._wx);
		tmp2 = (_zx * matrix._xy) + (_zy * matrix._yy) + (_zz * matrix._zy) + (_zw * matrix._wy);
		tmp3 = (_zx * matrix._xz) + (_zy * matrix._yz) + (_zz * matrix._zz) + (_zw * matrix._wz);
		tmp4 = (_zx * matrix._xw) + (_zy * matrix._yw) + (_zz * matrix._zw) + (_zw * matrix._ww);

		_zx = tmp1;
		_zy = tmp2;
		_zz = tmp3;
		_zw = tmp4;

		tmp1 = (_wx * matrix._xx) + (_wy * matrix._yx) + (_wz * matrix._zx) + (_ww * matrix._wx);
		tmp2 = (_wx * matrix._xy) + (_wy * matrix._yy) + (_wz * matrix._zy) + (_ww * matrix._wy);
		tmp3 = (_wx * matrix._xz) + (_wy * matrix._yz) + (_wz * matrix._zz) + (_ww * matrix._wz);
		tmp4 = (_wx * matrix._xw) + (_wy * matrix._yw) + (_wz * matrix._zw) + (_ww * matrix._ww);

		_wx = tmp1;
		_wy = tmp2;
		_wz = tmp3;
		_ww = tmp4;
	} // end method multiply

	/**
	 * Add the current matrix by the given matrix (this + matrix).
	 *
	 * @param matrix the matrix to add.
	 */
	public void add(Ly4DMatrix matrix)
	{
		_xx += matrix._xx;
		_xy += matrix._xy;
		_xz += matrix._xz;
		_xw += matrix._xw;

		_yx += matrix._yx;
		_yy += matrix._yy;
		_yz += matrix._yz;
		_yw += matrix._yw;

		_zx += matrix._zx;
		_zy += matrix._zy;
		_zz += matrix._zz;
		_zw += matrix._zw;

		_wx += matrix._wx;
		_wy += matrix._wy;
		_wz += matrix._wz;
		_ww += matrix._ww;
	}

	/**
	 * Returns the result of matrix multiplication (this*matrix).
	 *
	 * @param  matrix the matrix to multiply by.
	 * @return the result of the multiplication.
	 */
	public Ly4DMatrix getMultiplicationBy(Ly4DMatrix matrix)
	{
		return new Ly4DMatrix((_xx * matrix._xx) + (_xy * matrix._yx) + (_xz * matrix._zx)
			+ (_xw * matrix._wx),
			(_xx * matrix._xy) + (_xy * matrix._yy) + (_xz * matrix._zy) + (_xw * matrix._wy),
			(_xx * matrix._xz) + (_xy * matrix._yz) + (_xz * matrix._zz) + (_xw * matrix._wz),
			(_xx * matrix._xw) + (_xy * matrix._yw) + (_xz * matrix._zw) + (_xw * matrix._ww),
			(_yx * matrix._xx) + (_yy * matrix._yx) + (_yz * matrix._zx) + (_yw * matrix._wx),
			(_yx * matrix._xy) + (_yy * matrix._yy) + (_yz * matrix._zy) + (_yw * matrix._wy),
			(_yx * matrix._xz) + (_yy * matrix._yz) + (_yz * matrix._zz) + (_yw * matrix._wz),
			(_yx * matrix._xw) + (_yy * matrix._yw) + (_yz * matrix._zw) + (_yw * matrix._ww),
			(_zx * matrix._xx) + (_zy * matrix._yx) + (_zz * matrix._zx) + (_zw * matrix._wx),
			(_zx * matrix._xy) + (_zy * matrix._yy) + (_zz * matrix._zy) + (_zw * matrix._wy),
			(_zx * matrix._xz) + (_zy * matrix._yz) + (_zz * matrix._zz) + (_zw * matrix._wz),
			(_zx * matrix._xw) + (_zy * matrix._yw) + (_zz * matrix._zw) + (_zw * matrix._ww),
			(_wx * matrix._xx) + (_wy * matrix._yx) + (_wz * matrix._zx) + (_ww * matrix._wx),
			(_wx * matrix._xy) + (_wy * matrix._yy) + (_wz * matrix._zy) + (_ww * matrix._wy),
			(_wx * matrix._xz) + (_wy * matrix._yz) + (_wz * matrix._zz) + (_ww * matrix._wz),
			(_wx * matrix._xw) + (_wy * matrix._yw) + (_wz * matrix._zw) + (_ww * matrix._ww));
	}

	/**
	 * Returns the transponized matrix.
	 *
	 * @return the result of the transponization.
	 */
	public Ly4DMatrix getTransponize()
	{
		return new Ly4DMatrix(_xx, _yx, _zx, _wx, _xy, _yy, _zy, _wy, _xz, _yz, _zz, _wz, _xw, _yw,
			_zw, _ww);
	}

	/**
	 * Rotate alpha radians about the x axis.
	 *
	 * @param alpha the rotation angle in radians
	 */
	public void rotateX(double alpha)
	{
		if (alpha == 0.0)
			return;

		double cos = Math.cos(alpha);
		double sin = Math.sin(alpha);

		float  tmp1, tmp2;

		tmp1 = (float)((_yx * cos) - (_zx * sin));
		tmp2 = (float)((_yx * sin) + (_zx * cos));
		_yx = tmp1;
		_zx = tmp2;

		tmp1 = (float)((_yy * cos) - (_zy * sin));
		tmp2 = (float)((_yy * sin) + (_zy * cos));
		_yy = tmp1;
		_zy = tmp2;

		tmp1 = (float)((_yz * cos) - (_zz * sin));
		tmp2 = (float)((_yz * sin) + (_zz * cos));
		_yz = tmp1;
		_zz = tmp2;

		tmp1 = (float)((_yw * cos) - (_zw * sin));
		tmp2 = (float)((_yw * sin) + (_zw * cos));
		_yw = tmp1;
		_zw = tmp2;
	} // end method rotateX

	/**
	 * Rotate alpha radians about the y axis.
	 *
	 * @param alpha the rotation angle in radians
	 */
	public void rotateY(double alpha)
	{
		if (alpha == 0.0)
			return;

		double cos = Math.cos(alpha);
		double sin = -Math.sin(alpha);

		float  tmp1, tmp2;

		tmp1 = (float)((_xx * cos) - (_zx * sin));
		tmp2 = (float)((_xx * sin) + (_zx * cos));
		_xx = tmp1;
		_zx = tmp2;

		tmp1 = (float)((_xy * cos) - (_zy * sin));
		tmp2 = (float)((_xy * sin) + (_zy * cos));
		_xy = tmp1;
		_zy = tmp2;

		tmp1 = (float)((_xz * cos) - (_zz * sin));
		tmp2 = (float)((_xz * sin) + (_zz * cos));
		_xz = tmp1;
		_zz = tmp2;

		tmp1 = (float)((_xw * cos) - (_zw * sin));
		tmp2 = (float)((_xw * sin) + (_zw * cos));
		_xw = tmp1;
		_zw = tmp2;
	} // end method rotateY

	/**
	 * Rotate alpha radians about the z axis.
	 *
	 * @param alpha the rotation angle in radians
	 */
	public void rotateZ(double alpha)
	{
		if (alpha == 0.0)
			return;

		double cos = Math.cos(alpha);
		double sin = Math.sin(alpha);

		float  tmp1, tmp2;

		tmp1 = (float)((_xx * cos) - (_yx * sin));
		tmp2 = (float)((_xx * sin) + (_yx * cos));
		_xx = tmp1;
		_yx = tmp2;

		tmp1 = (float)((_xy * cos) - (_yy * sin));
		tmp2 = (float)((_xy * sin) + (_yy * cos));
		_xy = tmp1;
		_yy = tmp2;

		tmp1 = (float)((_xz * cos) - (_yz * sin));
		tmp2 = (float)((_xz * sin) + (_yz * cos));
		_xz = tmp1;
		_yz = tmp2;

		tmp1 = (float)((_xw * cos) - (_yw * sin));
		tmp2 = (float)((_xw * sin) + (_yw * cos));
		_xw = tmp1;
		_yw = tmp2;
	} // end method rotateZ

	/**
	 * Returns a string representation of the matrix.
	 */
	public String toString()
	{
		StringBuffer buffer = new StringBuffer();

		buffer.append("[[");
		buffer.append(_xx).append(",").append(_xy).append(",").append(_xz).append(",").append(_xw)
			.append("]");
		buffer.append('\n');
		buffer.append("  ");
		buffer.append(_yx).append(",").append(_yy).append(",").append(_yz).append(",").append(_yw)
			.append("]");
		buffer.append('\n');
		buffer.append("  ");
		buffer.append(_zx).append(",").append(_zy).append(",").append(_zz).append(",").append(_zw)
			.append("]");
		buffer.append('\n');
		buffer.append("  ");
		buffer.append(_wx).append(",").append(_wy).append(",").append(_wz).append(",").append(_ww)
			.append("]]");

		return (buffer.toString());
	}

	/**
	 * Sets the values of the matrix.
	 *
	 * @param xx xx value
	 * @param xy xy value
	 * @param xz xz value
	 * @param xw xw value
	 * @param yx yx value
	 * @param yy yy value
	 * @param yz yz value
	 * @param yw yw value
	 * @param zx zx value
	 * @param zy zy value
	 * @param zz zz value
	 * @param zw zw value
	 * @param wx wx value
	 * @param wy wy value
	 * @param wz wz value
	 * @param ww ww value
	 */
	public void setValues(float xx, float xy, float xz, float xw,
		float yx, float yy, float yz, float yw,
		float zx, float zy, float zz, float zw,
		float wx, float wy, float wz, float ww)
	{
		_xx = xx;
		_xy = xy;
		_xz = xz;
		_xw = xw;
		_yx = yx;
		_yy = yy;
		_yz = yz;
		_yw = yw;
		_zx = zx;
		_zy = zy;
		_zz = zz;
		_zw = zw;
		_wx = wx;
		_wy = wy;
		_wz = wz;
		_ww = ww;
	}
} // end class Ly4DMatrix
