/*
 * LyDxfParser.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.common;

import de.escape.quincunx.dxf.*;
import de.escape.quincunx.dxf.reader.*;
import de.escape.quincunx.trafo.*;
import java.awt.*;
import java.util.*;
import leon.misc.*;
import leon.view.*;


/**
 * This class parses and converts dxf files into LyPolygon objects.
 *
 * @version $Id: LyDxfParser.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public class LyDxfParser
{
	public static final String __VERSION =
		"$Id: LyDxfParser.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Constructor.
	 */
	public LyDxfParser()
	{
	}

	/**
	 * Parses a dxf file and return a vector of LyPolygon.
	 */
	public Vector<LyPolygon> parseFile(LyEnvironment environment, String file)
	{
		if (file == null)
			return null;

		LyString stringEnv = new LyString(environment);

		file = stringEnv.expand(file, ".");

		DxfFile			  dxfFile;
		DrawableConverter converter;
		DrawSet			  drawset;
		Vector<DrawAble>  drawables;
		Vector<LyPolygon> polygons = null;

		try
		{
			environment.logInfo("Start parsing DXF file " + file);

			// Create the representation of dxf file
			dxfFile = new DxfFile(file);
			converter = new DrawableConverter();

			// Convert the data
			dxfFile.convert(converter);
			drawset = converter.getModel();
			drawables = getDrawables(drawset);
			// Transform the drawables in LyPolygons
			polygons = transformDrawables(drawables);

			environment.logInfo("End of parsing for DXF file");
		}
		catch (Exception e)
		{
			environment.logError(e.toString());
		}

		return polygons;
	} // end method parseFile

	/**
	 * Returns the set of drawables contained in a drawset.
	 */
	protected Vector<DrawAble> getDrawables(DrawSet drawset)
	{
		int		           i, n = drawset.nrDrawables;
		Vector<DrawAble>   drawables = new Vector<DrawAble>();
		DrawAble           drawable;

		for (i = 0; i < n; i++)
		{
			drawable = drawset.drawables[i];
			drawables.addElement(drawable);
		}

		return drawables;
	}

	/**
	 * Transform a set of drawables in a set of LyPolygons.
	 */
	protected Vector<LyPolygon> transformDrawables(Vector<DrawAble> drawables)
	{
		int		  i, j, n = drawables.size();
		DrawAble  drawable;
		DrawLines drawlines;
		Point3D[] points;
		Point3D   point3D;
		int		  nPoints;
		LyPoint   point;
		LyPolygon polygon;
		Vector<LyPolygon>    polygons = new Vector<LyPolygon>(n);
		Color     graphicalColor;
		String    colorCode;

		for (i = 0; i < n; i++)
		{
			drawable = drawables.elementAt(i);

			if (drawable instanceof DrawLines)
			{
				drawlines = (DrawLines)drawable;
				points = drawlines.line;
				nPoints = drawlines.nrPoints;

				polygon = new LyPolygon();

				for (j = 0; j < nPoints; j++)
				{
					point3D = points[j];

					point = new LyPoint(point3D.x, -point3D.y, point3D.z);
					polygon.addPoint(point);
				}

				graphicalColor = DxfColorModel.getNormalColors().getColor(drawable.color);

				if (graphicalColor != null)
				{
					colorCode = graphicalColor.getRed() + " "
						+ graphicalColor.getGreen() + " "
						+ graphicalColor.getBlue();
					polygon.setColor(colorCode);
				}

				polygons.addElement(polygon);
			}
		}

		return polygons;
	} // end method transformDrawables
} // end class LyDxfParser
