/*
 * LyActionEvent.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.event;

/**
 * The listener interface for receiving action events.
 *
 * @version $Id: LyActionEvent.java 28997 2010-12-23 10:17:15Z bmichel $
 * @author  Lyria
 */
public class LyActionEvent extends LyEvent
{
	/**
	 * 
	 */
	private static final long serialVersionUID = -3819356062280693588L;

	public static final String __VERSION =
		"$Id: LyActionEvent.java 28997 2010-12-23 10:17:15Z bmichel $";

	// value of the action event (optional)
	protected String		   _value;
	// Name/identifier of the source or field which was the container of the action
	protected String		   _sourceId;
	// type of this action event : see action demands types.
	protected int			   _type;
	// Parameter of the event (coordinates ...)
	protected Object		   _parameter;
	// Additional value (could be for instance the old value but depends ont the type of the event)
	protected String		   _additionalValue;

	/**
	 * Constructor.
	 */
	public LyActionEvent(Object source)
	{
		this(source, null, null, LyActionDemand.TYPE_ACTION);
	}

	/**
	 * Constructor.
	 */
	public LyActionEvent(Object source, String sourceId, String value, int actionType)
	{
		super(source);

		_value = value;
		_sourceId = sourceId;
		_type = actionType;
	}

	/**
	 * Value associated to this event (optional).
	 */
	public String getValue()
	{
		return _value;
	}

	/**
	 * Value associated to this event (optional).
	 */
	public void setValue(String value)
	{
		_value = value;
	}

	/**
	 * Accessor to the additional value associated to this action. The meaning of the value depends
	 * on the type of the event.
	 *
	 * @return the value
	 */
	public String getAdditionalValue()
	{
		return (_additionalValue);
	}

	/**
	 * Accessor to the additional value associated to this action. The meaning of the value depends
	 * on the type of the event.
	 *
	 * @param value the value
	 */
	public void setAdditionalValue(String value)
	{
		_additionalValue = value;
	}

	/**
	 * The id of the field which contains the sourve of the action.
	 */
	public String getSourceId()
	{
		return _sourceId;
	}

	/**
	 * The id of the field which contains the sourve of the action.
	 */
	public void setSourceId(String sourceId)
	{
		_sourceId = sourceId;
	}

	/**
	 * Some components may return various actions and the type depends on the nature of the source.
	 * Usually, generic classes don not have to handle a specific implementation for this type.
	 *
	 * @return the action type, or -1 if none specified.
	 */
	public int getType()
	{
		return (_type);
	}

	/**
	 * Accessor to the action parameter. Nature depends on the type of the action. For instance,
	 * ACTION_POPUP events return clic Point
	 *
	 * @return parameter
	 */
	public Object getParameter()
	{
		return _parameter;
	}

	/**
	 * Accessor to the action parameter.
	 *
	 * @param parameter freely set parameter which depends on the type of the action
	 */
	public void setParameter(Object parameter)
	{
		_parameter = parameter;
	}
} // end class LyActionEvent
