/*
 * LyDemandListener.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.event;

/**
 * An interface implemented by controllers.to receive event notifications from the view components.
 * Typically, a demand is associated to a user interaction on the GUI.
 *
 * <P>Current demand types are : action, selection menu and help.</P>
 *
 * @see     LyDemand
 * @see     LyActionDemand
 * @see     LySelectionDemand
 * @see     LyMenuDemand
 * @see     LyHelpDemand
 * @version $Id: LyDemandListener.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public interface LyDemandListener
{
	String __VERSION = "$Id: LyDemandListener.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Indicates the identification of the component which receives the event. This identification
	 * must be unique for all registred listeners.
	 *
	 * @return the component identifier.
	 */
	String getId();

	/**
	 * Invoked when a notification is sent to this listener. The listener should make the
	 * interpretation of the event, and update the view description if necessary.
	 *
	 * @param demand demand parameters set by the view component.
	 * @see   leon.view.event.LyDemandListener interface
	 */
	void demandPerformed(LyDemand demand);
}