/*
 * LyDemandTransmitter.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.event;

/**
 * Client may register on components which implement this interface to receive notifications from
 * the component.
 *
 * @version $Id: LyDemandTransmitter.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public interface LyDemandTransmitter
{
	String __VERSION = "$Id: LyDemandTransmitter.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Adds the specified demand listener to receive demands (events) from this component.
	 * (LyDemandTransmitter interface)
	 *
	 * @param listener client for the demand
	 */
	void addDemandListener(LyDemandListener listener);

	/**
	 * Removes the specified demand listener to receive demands from this component.
	 * (LyDemandTransmitter interface)
	 *
	 * @param listener client for the demand
	 */
	void removeDemandListener(LyDemandListener listener);

	/**
	 * Processes demands on this component. (LyDemandTransmitter interface)
	 */
	void processDemand(LyDemand demand);

	/**
	 * Processes demands on this component.
	 *
	 * @param controllerId the demand is sent to the specified listener (LyDemandTransmitter
	 *                     interface)
	 */
	void processDemand(String listenerId, LyDemand demand);

	/**
	 * Retrieve a LyDemandListener by its ID.
	 *
	 * @param listenerId which has to be retrieve
	 */
	LyDemandListener getListenerById(String listenerId);

	/**
	 * Free the demand listener.
	 */
	void free();

	/**
	 * Used in test mode to play next demand in test file.
	 */
	boolean playNext();

	/**
	 * Gets delay between plays in test mode.
	 */
	int getDelay();
}