/*
 * LyDragDropDemand.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.event;

/**
 * This class implements an action event created from the presentation layer, which is sent to the
 * associated controller and sent to its controller using RMI.
 *
 * @version $Id: LyDragDropDemand.java 28997 2010-12-23 10:17:15Z bmichel $
 * @author  Lyria
 * @see     LyDemand
 * @see     LySelectionDemand
 * @see     LyHelpDemand
 * @see     LyMenuDemand
 */
public class LyDragDropDemand extends LyDemand
{
	/**
	 * 
	 */
	private static final long serialVersionUID = -1370466390331637496L;

	public static final String __VERSION =
		"$Id: LyDragDropDemand.java 28997 2010-12-23 10:17:15Z bmichel $";

	/** This event indicates a drag start request. */
	public static final short  TYPE_DRAG_START = 501;

	/** This event indicates a drop request. The data will be dropped. */
	public static final short  TYPE_DROP = 502;

	/** This event indicates a finished drag request. */
	public static final short  TYPE_DRAG_FINISHED = 503;

	/* Internal types */

	/** This event indicates a drag set data request. */
	public static final short  TYPE_DRAG_SET_DATA = 551;

	/** This event indicates a drop accept request. The drop will be about to be performed. */
	public static final short  TYPE_DROP_ACCEPT = 552;

	/**
	 * This event indicates a drag enter request. The cursor will be entered the drop target
	 * boundaries.
	 */
	public static final short  TYPE_DRAG_ENTER = 553;

	/**
	 * This event indicates a drag leave request. The cursor will have left the drop target
	 * boundaries OR the drop will have been cancelled OR the data will be about to be dropped.
	 */
	public static final short  TYPE_DRAG_LEAVE = 554;

	/**
	 * This event indicates a drag operation changed request. The operation will be performed has
	 * changed (usually due to the user changing the selected modifier key(s) while dragging).
	 */
	public static final short  TYPE_DRAG_OPERATION_CHANGED = 555;

	/** This event indicates a drag over request. The cursor will be moving over the drop target. */
	public static final short  TYPE_DRAG_OVER = 556;

	/** Optional value associated to this action event. */
	protected String		   _value;

	/** Mouse position associated to a drag / drop demand. */
	protected int[]			   _pos;

	/**
	 * Constructor.
	 */
	public LyDragDropDemand(int demandType, String sourceId)
	{
		super(demandType, sourceId);

		_value = null;
		_pos = null;
	}

	/**
	 * Accessor to the value associated to this action. The meaning of the value depends on the type
	 * of the event.
	 *
	 * @return the value
	 */
	public String getValue()
	{
		return _value;
	}

	/**
	 * Accessor to the value associated to this action. The meaning of the value depends on the type
	 * of the event.
	 *
	 * @param value the value
	 */
	public void setValue(String value)
	{
		_value = value;
	}

	/**
	 * Convert the demand type to a string (print and dump).
	 */
	protected String getDemandType(int typ)
	{
		switch (typ)
		{
		case TYPE_DRAG_START:
			return "TYPE_DRAG_START";

		case TYPE_DRAG_FINISHED:
			return "TYPE_DRAG_FINISHED";

		case TYPE_DROP:
			return "TYPE_DROP";

		case TYPE_DRAG_SET_DATA:
			return "TYPE_SET_DATA";

		case TYPE_DROP_ACCEPT:
			return "TYPE_DROP_ACCEPT";

		case TYPE_DRAG_ENTER:
			return "TYPE_DRAG_ENTER";

		case TYPE_DRAG_LEAVE:
			return "TYPE_DRAG_LEAVE";

		case TYPE_DRAG_OPERATION_CHANGED:
			return "TYPE_DRAG_OPERATION_CHANGED";

		case TYPE_DRAG_OVER:
			return "TYPE_DRAG_OVER";

		default:
			return super.getDemandType(typ);
		}
	} // end method getDemandType

	/**
	 * Accessor to the position identifier.
	 */
	public int[] getPosition()
	{
		return _pos;
	}

	/**
	 * Modifier of the position identifier.
	 */
	public void setPosition(int[] p)
	{
		_pos = p;
	}
} // end class LyDragDropDemand
