/*
 * LyEvent.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.event;

import java.util.*;


/**
 * The generic class for all graphical events.
 *
 * @version $Id: LyEvent.java 28997 2010-12-23 10:17:15Z bmichel $
 * @author  Lyria
 */
public class LyEvent extends EventObject
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 4808919180708508691L;

	public static final String __VERSION = "$Id: LyEvent.java 28997 2010-12-23 10:17:15Z bmichel $";

	/** Indicates wether this event has been completely processed. */
	protected boolean		   _consumed;

	// Target indexes
	protected int[]			   _index_list;

	/** Sub-index of this event. */
	protected int			   _sub_index;

	public LyEvent(Object source)
	{
		super(source);

		_consumed = false;
		_index_list = null;
		_sub_index = -1;
	}

	/**
	 * Indicates wether this event has been completely processed. a Consumed event should not
	 * propagate.
	 */
	public boolean isConsumed()
	{
		return (_consumed);
	}

	/**
	 * Indicates that this event has been completely processed.
	 */
	public void consume()
	{
		_consumed = true;
	}

	/**
	 * Accessor to the index of the (un)selected item.
	 */
	public void addIndex(int idx)
	{
		int pos = 0;

		// check size
		if (_index_list != null)
		{
			boolean spaceLeft = false;

			for (pos = 0; (pos < _index_list.length) && (!spaceLeft); pos++)
			{
				if (_index_list[pos] == -1)
					spaceLeft = true;
			}

			if (!spaceLeft)
			{
				// increase size
				int   newSize = (_index_list.length * 2) + 5;
				int[] tmpArray = new int[newSize];

				// copy current
				for (int i = 0; i < _index_list.length; i++)
					tmpArray[i] = _index_list[i];
				//prepare spaceLeft
				for (int i = _index_list.length; i < tmpArray.length; i++)
					tmpArray[i] = -1;
				// done
				_index_list = tmpArray;
			}
		}
		else
			_index_list = new int[1];

		if (pos < _index_list.length)
			_index_list[pos] = idx;
	} // end method addIndex

	/**
	 * Accessor to the index of the (un)selected item return -1 if not found.
	 */
	public int getIndex(int pos)
	{
		if (_index_list != null)
		{
			if ((pos >= 0) && (pos < _index_list.length))
				return (_index_list[pos]);
		}
		return -1;
	}

	/**
	 * Sub-index associated to this event (optional).
	 */
	public int getSubIndex()
	{
		return _sub_index;
	}

	/**
	 * Value associated to this event (optional).
	 */
	public void setSubIndex(int sub_index)
	{
		_sub_index = sub_index;
	}
} // end class LyEvent
