/*
 * LyHelpDemand.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.event;

/**
 * This class implements a menu event created from the presentation layer, which is sent to the
 * associated controller and sent to its controller using RMI.
 *
 * @version $Id: LyMenuDemand.java 34704 2012-03-15 10:19:07Z jlsaouli $
 * @author  Lyria
 * @see     LyDemand
 * @see     LyActionDemand
 * @see     LySelectionDemand
 * @see     LyMenuDemand
 */
public class LyMenuDemand extends LyDemand
{
	/**
	 * 
	 */
	private static final long serialVersionUID = -8961643852494874562L;

	public static final String __VERSION =
		"$Id: LyMenuDemand.java 34704 2012-03-15 10:19:07Z jlsaouli $";

	/** Request for a popup menu. */
	public static final short  TYPE_POPUP = 201;

	/** Request for a extended popup menu. */
	public static final short  TYPE_EXTEND_POPUP = 202;

	// item associated to the request
	protected String		   _item;

	// item position associated to a popup menu
	protected int[]			   _pos;

	/**
	 * Constructor.
	 */
	public LyMenuDemand(int demandType, String sourceId)
	{
		super(demandType, sourceId);
	}

	/**
	 * Accessor to the context associated to this request.
	 *
	 * @return the context value if any
	 */
	public String getItem()
	{
		return (_item);
	}

	/**
	 * Accessor to the value associated to this action.
	 *
	 * @param context the context
	 */
	public void setItem(String item)
	{
		_item = item;
	}

	/**
	 * accessor to the position identifier.
	 */
	public int[] getPosition()
	{
		return _pos;
	}

	/**
	 * accessor to the position identifier.
	 */
	public void setPosition(int[] p)
	{
		_pos = p;
	}

	/**
	 * Convert the demand type to a string (print and dump).
	 */
	protected String getDemandType(int typ)
	{
		switch (typ)
		{
		case TYPE_POPUP:
			return "TYPE_POPUP";
		case TYPE_EXTEND_POPUP:
			return "TYPE_EXTEND_POPUP";
		default:
			return super.getDemandType(typ);
		}
	}

	/**
	 * Dump as String.
	 */
	public String toString()
	{
		StringBuffer sb = new StringBuffer();

		sb.append(super.toString());
		sb.append("- position  : " + String.valueOf(_pos));
		sb.append('\n');
		sb.append("- item : " + _item);
		sb.append('\n');

		return sb.toString();
	}
} // end class LyMenuDemand
