/*
 * LySelectionEvent.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.event;

/**
 * The listener interface for receiving selection events.
 *
 * @version $Id: LySelectionEvent.java 33376 2011-12-01 15:57:34Z nhamon $
 * @author  Lyria
 */
public class LySelectionEvent extends LyEvent
{
	/**
	 * 
	 */
	private static final long serialVersionUID = -4881377816611876910L;

	public static final String __VERSION =
		"$Id: LySelectionEvent.java 33376 2011-12-01 15:57:34Z nhamon $";

	// identifier of a selected item
	protected Object		   _item;
	// selection status
	protected boolean		   _selected;
	// number of selected items
	protected int			   _selectionSize;
	// ectend former selection
	protected boolean		   _extend;
	// position of the selection
	protected int[]			   _pos;

	/**
	 * Constructor.
	 */
	public LySelectionEvent(Object source, Object item, boolean selected)
	{
		super(source);

		_item = item;
		_selected = selected;
		_selectionSize = -1;
		_extend = false;
	}

	/**
	 * Accessor to the (un)selected item identifier.
	 */
	public Object getItem()
	{
		return _item;
	}

	public void setItem(Object item)
	{
		_item = item;
	}

	/**
	 * Accessor to the number of selections.
	 */
	public void setSelectionSize(int size)
	{
		_selectionSize = size;
	}

	/**
	 * Accessor to the number of selections.
	 */
	public int getSelectionSize()
	{
		return (_selectionSize);
	}

	/**
	 * Accessor to the new selection status.
	 */
	public boolean isSelected()
	{
		return _selected;
	}

	/**
	 * Accessor to the new extension status.
	 *
	 * @return true if the selection extends previous notification
	 */
	public boolean isExtend()
	{
		return _extend;
	}

	/**
	 * Accessor to the new extension status. Default is false.
	 *
	 * @param extend true if the selection extends previous notification.
	 */
	public void setExtend(boolean extend)
	{
		_extend = extend;
	}

	/**
	 * Accessor to the position identifier.
	 */
	public int[] getPosition()
	{
		return _pos;
	}

	/**
	 * Accessor to the position identifier.
	 */
	public void setPosition(int[] p)
	{
		_pos = p;
	}
} // end class LySelectionEvent
