/*
 * LySelectionListener.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.event;

import java.util.*;


/**
 * Client may register on components which implement this interface to receive notifications from
 * the component.
 *
 * @version $Id: LySimpleActionTransmitter.java 29006 2010-12-23 17:44:46Z bmichel $
 * @author  Lyria
 */
public class LySimpleActionTransmitter
	implements LyActionTransmitter
{
	public static final String __VERSION =
		"$Id: LySimpleActionTransmitter.java 29006 2010-12-23 17:44:46Z bmichel $";

	/** List of listeners for action events. */
	protected Vector<LyActionListener>		   _actionListeners;

	/**
	 * Constructor.
	 */
	public LySimpleActionTransmitter()
	{
	}

	/**
	 * Adds the specified action listener to receive action events from this component.
	 * (LyActionTransmitter interface)
	 *
	 * @param listener client for the action event
	 */
	public synchronized void addActionListener(LyActionListener listener)
	{
		if (_actionListeners == null)
			_actionListeners = new Vector<LyActionListener>(1);
		if (_actionListeners.contains(listener))
			// Don't register listener twice
			return;

		_actionListeners.addElement(listener);
	}

	/**
	 * Removes the specified action listener to receive action events from this component.
	 * (LyActionTransmitter interface)
	 *
	 * @param listener client for the action event
	 */
	public synchronized void removeActionListener(LyActionListener listener)
	{
		if ((_actionListeners != null) && (listener != null))
			_actionListeners.removeElement(listener);
	}

	/**
	 * Processes events on this component.
	 */
	public void processActionEvent(LyActionEvent event)
	{
		// check parameter and event validity
		if ((_actionListeners == null)
				|| (event == null)
				|| (event.isConsumed()))
			return;

		LyActionListener listener;
		int				 i, n = _actionListeners.size();

		if (n == 0)
			return;

		// Duplicate list to avoid issues if a listener is removed
		Vector<LyActionListener> listeners = null;

		synchronized (_actionListeners)
		{
			listeners = new Vector<LyActionListener>(_actionListeners);
		}

		n = listeners.size();

		for (i = 0; i < n; i++)
		{
			listener = listeners.elementAt(i);
			listener.actionPerformed(event);
		}
	} // end method processActionEvent
} // end class LySimpleActionTransmitter
