/**
 * LySimpleBeanAdapter.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.event;

import leon.view.*;
import leon.view.beaninterface.*;


/**
 * This class implements a simple adapter for bean components used for field items.
 *
 * @version $Id: LySimpleBeanAdapter.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 * @see     LyFieldBeanInterface
 */
public class LySimpleBeanAdapter
	implements LyEventTransmitter,
		LyFieldBeanInterface
{
	public static final String		 __VERSION =
		"$Id: LySimpleBeanAdapter.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Corresponding field item of the view package. */
	protected LyFieldItem			 _fieldItem;

	/** Action transmitter. */
	protected LyActionTransmitter    _actionTransmitter;

	/** Selection transmitter. */
	protected LySelectionTransmitter _selectionTransmitter;

	public LySimpleBeanAdapter()
	{
	}

	/**
	 * Implementation of LyFieldBeanInterface.
	 */
	public void setViewComponent(LyFieldItem fieldItem)
	{
		_fieldItem = fieldItem;
	}

	/**
	 * Returns the field item corresponding to this bean.
	 */
	public LyFieldItem getViewComponent()
	{
		return _fieldItem;
	}

	/**
	 * Implementation of LyEventTransmitter.
	 */
	public LyActionTransmitter getActionTransmitter()
	{
		if (_actionTransmitter == null)
			_actionTransmitter = new LySimpleActionTransmitter();

		return (_actionTransmitter);
	}

	/**
	 * Implementation of LyEventTransmitter.
	 */
	public LySelectionTransmitter getSelectionTransmitter()
	{
		if (_selectionTransmitter == null)
			_selectionTransmitter = new LySimpleSelectionTransmitter();

		return (_selectionTransmitter);
	}

	/**
	 * Implementation of LyFieldBeanInterface.
	 */
	public void setLabel(String label)
	{
		// Must be implemented in subclasses
	}

	/**
	 * Send a value changed event.
	 */
	protected void valueChanged(String value)
	{
		if (_actionTransmitter != null)
			_actionTransmitter.processActionEvent(new LyActionEvent(this, null, value,
					LyActionDemand.TYPE_VALUE_CHANGED));
	}
} // end class LySimpleBeanAdapter
