/*
 * LySimpleSelectionTransmitter.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.event;

import java.util.*;


/**
 * Client may register on components which implement this interface to receive notifications from
 * the component.
 *
 * @version $Id: LySimpleSelectionTransmitter.java 29006 2010-12-23 17:44:46Z bmichel $
 * @author  Lyria
 */
public class LySimpleSelectionTransmitter
	implements LySelectionTransmitter
{
	public static final String __VERSION =
		"$Id: LySimpleSelectionTransmitter.java 29006 2010-12-23 17:44:46Z bmichel $";

	/** List of listeners for Selection events. */
	protected Vector<LySelectionListener>		   _selectionListeners;

	/**
	 * Constructor.
	 */
	public LySimpleSelectionTransmitter()
	{
	}

	/**
	 * Adds the specified selection listener to receive selection events from this component.
	 * (LyEventTransmitter interface)
	 *
	 * @param listener client for the action event
	 */
	public synchronized void addSelectionListener(LySelectionListener listener)
	{
		if (_selectionListeners == null)
			_selectionListeners = new Vector<LySelectionListener>(1);

		_selectionListeners.addElement(listener);
	}

	/**
	 * Removes the specified selection listener from this component. (LySelectionTransmitter
	 * interface)
	 *
	 * @param listener client for the action event
	 */
	public synchronized void removeSelectionListener(LySelectionListener listener)
	{
		if ((_selectionListeners != null) && (listener != null))
			_selectionListeners.removeElement(listener);
	}

	/**
	 * Processes events on this component.
	 */
	public void processSelectionEvent(LySelectionEvent selectionEvent)
	{
		if ((_selectionListeners == null)
				|| (selectionEvent == null)
				|| (selectionEvent.isConsumed()))
			return;

		LySelectionListener listener;
		int				    i, n = _selectionListeners.size();

		if (n == 0)
			return;

		// Duplicate list to avoid issues if a listener is removed
		Vector<LySelectionListener> listeners = null;

		synchronized (_selectionListeners)
		{
			listeners = new Vector<LySelectionListener>(_selectionListeners);
		}

		n = listeners.size();
		for (i = 0; i < n; i++)
		{
			listener = listeners.elementAt(i);
			listener.selectionPerformed(selectionEvent);
		}
	} // end method processSelectionEvent
} // end class LySimpleSelectionTransmitter
