/*
 * LyDisableImageFilter.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.filters;

import java.awt.*;


/**
 * This class is used to create disabled images. The algorithm uses the background color every two
 * pixels in the created image.
 * @version $Id: LyDisableImageFilter.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public class LyDisableImageFilter extends LyImageFilter
{
	public static final String __VERSION =
		"$Id: LyDisableImageFilter.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Name of the filter used for keys in table of images. */
	public static String	   DISABLE_FILTER_NAME = "_disableFilter";

	/**
	 * Constructor with no parameter. Calls the super implementation.
	 */
	public LyDisableImageFilter()
	{
		super();
	}

	/**
	 * Gets the filtered image associated to the given image and background color. It it is not
	 * found, creates it.
	 * @param  image      The image that must be filtered.
	 * @return The filtered image.
	 */
	public static Image getFilteredImage(Image image)
	{
		if (image == null)
			return null;

		// Creation of a unique key : imSource, color
		StringBuffer key = new StringBuffer(10);

		key.append(image.hashCode());
		key.append('-');
		key.append(DISABLE_FILTER_NAME);

		Image result = null;

		if (_images != null)
			result = (Image)_images.get(key.toString());

		// If image not found, create a new one
		if (result == null)
		{
			int[]     pixels = getPixels(image);
			Dimension imageDim = getImageSize(image);

			pixels = disable(pixels, imageDim.width, imageDim.height);
			result = createImage(pixels, imageDim.width, imageDim.height);

			putCache(key.toString(), result);
		}

		return result;
	} // end method getFilteredImage

	/**
	 * Creates an array of pixels representing the filtered data for the given array of pixels.
	 * @param  sourcePixels The source pixels that must be filtered.
	 * @param  width        The width of the image represented by the given array of pixels.
	 * @param  height       The height of the image represented by the given array of pixels.
	 * @param  background   The color that must be used every two pixels.
	 * @return The array of pixels that represents the filtered given array of pixels.
	 */
	protected static int[] disable(int[] sourcePixels, int width, int height)
	{
		int[] pixels = new int[width * height];
		int   t, c, r, g, b, gray, index;

		for (index = 0; index < (width * height); index++)
		{
			c = sourcePixels[index];
			t = ((c & 0xff000000));
			r = (c & 0xff0000) >> 16;
			g = (c & 0xff00) >> 8;
			b = c & 0xff;
			gray = ((r * 3) + (g * 4) + (b * 2)) / 9;
			
			gray = Math.max(150, ((r * 2) + (g * 5) + (b * 1)) / 8);
			
			gray = Math.min(255, gray + 25);

			pixels[index] = (t | (gray << 16) | (gray << 8) | gray);
		}

		return pixels;
	}

} // end class LyDisableImageFilter
