/*
 * LyGraphicalCell.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.graphic;

import leon.view.*;
import leon.view.setinterface.*;

/**
 * The LyGraphicalCell interface represents the abstract graphical component
 * for the cell of views.dtd regardless of the target (swing, swt, dhtml...).
 * @see leon.view.LyCell
 */
public interface LyGraphicalCell
	extends LyGraphicalComponent,
			LySetLabelInterface,
			LySetEditableInterface,
			LySetSelectedInterface,
			LySetImageInterface,
			LySetColorInterface
{
	public static final String			__VERSION = "$Id$";


	/* Methods for updating graphical components */

	/**
	 * Updates attribute label to the given value.
	 * @param label New value of label.
	 */
	public void setLabel(String label);

	/**
	 * Updates attribute objectId to the given value.
	 * @param objectId New value of objectId.
	 */
	public void setObjectId(String objectId);

	/**
	 * Updates attribute editable to the given value.
	 * @param editable Boolean indicating if attribute editable is set or unset.
	 */
	public void setEditable(boolean editable);

	/**
	 * Updates attribute selected to the given value.
	 * @param selected Boolean indicating if attribute selected is set or unset.
	 */
	public void setSelected(boolean selected);

	/**
	 * Updates attribute image to the given value.
	 * @param image New value of image.
	 */
	public void setImage(String image);

	/**
	 * Updates attribute tooltip to the given value.
	 * @param tooltip New value of tooltip.
	 */
	public void setTooltip(String tooltip);

	/**
	 * Updates attribute topBorderColor to the given value.
	 * @param topBorderColor New value of topBorderColor.
	 */
	public void setTopBorderColor(String topBorderColor);

	/**
	 * Updates attribute bottomBorderColor to the given value.
	 * @param bottomBorderColor New value of bottomBorderColor.
	 */
	public void setBottomBorderColor(String bottomBorderColor);

	/**
	 * Updates attribute leftBorderColor to the given value.
	 * @param leftBorderColor New value of leftBorderColor.
	 */
	public void setLeftBorderColor(String leftBorderColor);

	/**
	 * Updates attribute rightBorderColor to the given value.
	 * @param rightBorderColor New value of rightBorderColor.
	 */
	public void setRightBorderColor(String rightBorderColor);

	/**
	 * Updates attribute topLeftImage to the given value.
	 * @param topLeftImage New value of topLeftImage.
	 */
	public void setTopLeftImage(String topLeftImage);

	/**
	 * Updates attribute topRightImage to the given value.
	 * @param topRightImage New value of topRightImage.
	 */
	public void setTopRightImage(String topRightImage);

	/**
	 * Updates attribute bottomLeftImage to the given value.
	 * @param bottomLeftImage New value of bottomLeftImage.
	 */
	public void setBottomLeftImage(String bottomLeftImage);

	/**
	 * Updates attribute bottomRightImage to the given value.
	 * @param bottomRightImage New value of bottomRightImage.
	 */
	public void setBottomRightImage(String bottomRightImage);

	/**
	 * Updates attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color);

	/**
	 * Inserts a sub-component cells at given position.
	 * @param cells The cells to add to cells list.
	 * @param pos	Position where the sub-component is inserted.
	 */
	public void addCell(LyCell cells, int pos);

	/**
	 * Removes a sub-component cells.
	 * @param cells The cells to remove to cells list.
	 */
	public void removeCell(LyCell cells);
}