/*
 * LyGraphicalLine.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.graphic;

import leon.view.*;
import leon.view.setinterface.*;

/**
 * The LyGraphicalLine interface represents the abstract graphical component
 * for the line of views.dtd regardless of the target (swing, swt, dhtml...).
 * @see leon.view.LyLine
 */
public interface LyGraphicalLine
	extends LyGraphicalComponent,
			LySetLabelInterface,
			LySetImageInterface,
			LySetSelectedInterface,
			LySetHiddenInterface,
			LySetFontInterface,
			LySetColorInterface
{
	public static final String			__VERSION = "$Id$";


	/* Methods for updating graphical components */

	/**
	 * Updates attribute label to the given value.
	 * @param label New value of label.
	 */
	public void setLabel(String label);

	/**
	 * Updates attribute image to the given value.
	 * @param image New value of image.
	 */
	public void setImage(String image);

	/**
	 * Updates attribute tooltip to the given value.
	 * @param tooltip New value of tooltip.
	 */
	public void setTooltip(String tooltip);

	/**
	 * Updates attribute selected to the given value.
	 * @param selected Boolean indicating if attribute selected is set or unset.
	 */
	public void setSelected(boolean selected);

	/**
	 * Updates attribute open to the given value.
	 * @param open Boolean indicating if attribute open is set or unset.
	 */
	public void setOpen(boolean open);

	/**
	 * Updates attribute hidden to the given value.
	 * @param hidden Boolean indicating if attribute hidden is set or unset.
	 */
	public void setHidden(boolean hidden);

	/**
	 * Updates attribute font to the given value.
	 * @param font New value of font.
	 */
	public void setFont(String font);

	/**
	 * Updates attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color);

	/**
	 * Updates attribute superLineColor to the given value.
	 * @param superLineColor New value of superLineColor.
	 */
	public void setSuperLineColor(String superLineColor);

	/**
	 * Updates attribute superLineLabel to the given value.
	 * @param superLineLabel New value of superLineLabel.
	 */
	public void setSuperLineLabel(String superLineLabel);

	/**
	 * Updates attribute superLineHidden to the given value.
	 * @param superLineHidden Boolean indicating if attribute superLineHidden is set or unset.
	 */
	public void setSuperLineHidden(boolean superLineHidden);

	/**
	 * Inserts a sub-component components at given position.
	 * @param components The components to add to components list.
	 * @param pos	Position where the sub-component is inserted.
	 */
	public void addComponent(LyComponent components, int pos);

	/**
	 * Removes a sub-component components.
	 * @param components The components to remove to components list.
	 */
	public void removeComponent(LyComponent components);
}