/*
 * LyGraphicalMap.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.graphic;

import leon.view.*;
import leon.view.setinterface.*;

/**
 * The LyGraphicalMap interface represents the abstract graphical component
 * for the map of views.dtd regardless of the target (swing, swt, dhtml...).
 * @see leon.view.LyMap
 */
public interface LyGraphicalMap
	extends LyGraphicalPanel,
			LySetLabelInterface,
			LySetViewerInterface
{
	public static final String			__VERSION = "$Id$";


	/* Methods for updating graphical components */

	/**
	 * Updates attribute label to the given value.
	 * @param label New value of label.
	 */
	public void setLabel(String label);

	/**
	 * Updates attribute viewer to the given value.
	 * @param viewer Boolean indicating if attribute viewer is set or unset.
	 */
	public void setViewer(boolean viewer);

	/**
	 * Updates attribute viewerStatus to the given value.
	 * @param viewerStatus New value of viewerStatus.
	 */
	public void setViewerStatus(LyMap.ViewerStatus viewerStatus);

	/**
	 * Updates attribute zoom to the given value.
	 * @param zoom Boolean indicating if attribute zoom is set or unset.
	 */
	public void setZoom(boolean zoom);

	/**
	 * Updates attribute zoomStatus to the given value.
	 * @param zoomStatus New value of zoomStatus.
	 */
	public void setZoomStatus(LyMap.ZoomStatus zoomStatus);

	/**
	 * Updates attribute zoomFactor to the given value.
	 * @param zoomFactor New value of zoomFactor.
	 */
	public void setZoomFactor(float zoomFactor);

	/**
	 * Updates attribute translation to the given value.
	 * @param translation Boolean indicating if attribute translation is set or unset.
	 */
	public void setTranslation(boolean translation);

	/**
	 * Updates attribute translationStatus to the given value.
	 * @param translationStatus New value of translationStatus.
	 */
	public void setTranslationStatus(LyMap.TranslationStatus translationStatus);

	/**
	 * Updates attribute spatial to the given value.
	 * @param spatial Boolean indicating if attribute spatial is set or unset.
	 */
	public void setSpatial(boolean spatial);

	/**
	 * Inserts a sub-component components at given position.
	 * @param components The components to add to components list.
	 * @param pos	Position where the sub-component is inserted.
	 */
	public void addComponent(LyComponent components, int pos);

	/**
	 * Removes a sub-component components.
	 * @param components The components to remove to components list.
	 */
	public void removeComponent(LyComponent components);

	/**
	 * Sets sub-component mapContent.
	 * @param mapContent New value of mapContent
	 */
	public void setMapContent(LyMapContent mapContent);

	/**
	 * Sets sub-component layout.
	 * @param layout New value of layout
	 */
	public void setLayout(LyLayout layout);
}