/*
 * LyGraphicalMapLink.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.graphic;

import leon.view.*;
import leon.view.setinterface.*;

/**
 * The LyGraphicalMapLink interface represents the abstract graphical component
 * for the mapLink of views.dtd regardless of the target (swing, swt, dhtml...).
 * @see leon.view.LyMapLink
 */
public interface LyGraphicalMapLink
	extends LyGraphicalComponent,
			LySetLabelInterface,
			LySetSelectedInterface,
			LySetHiddenInterface
{
	public static final String			__VERSION = "$Id$";


	/* Methods for updating graphical components */

	/**
	 * Updates attribute label to the given value.
	 * @param label New value of label.
	 */
	public void setLabel(String label);

	/**
	 * Updates attribute startLabel to the given value.
	 * @param startLabel New value of startLabel.
	 */
	public void setStartLabel(String startLabel);

	/**
	 * Updates attribute endLabel to the given value.
	 * @param endLabel New value of endLabel.
	 */
	public void setEndLabel(String endLabel);

	/**
	 * Updates attribute selected to the given value.
	 * @param selected Boolean indicating if attribute selected is set or unset.
	 */
	public void setSelected(boolean selected);

	/**
	 * Updates attribute colorStart to the given value.
	 * @param colorStart New value of colorStart.
	 */
	public void setColorStart(String colorStart);

	/**
	 * Updates attribute colorEnd to the given value.
	 * @param colorEnd New value of colorEnd.
	 */
	public void setColorEnd(String colorEnd);

	/**
	 * Updates attribute hidden to the given value.
	 * @param hidden Boolean indicating if attribute hidden is set or unset.
	 */
	public void setHidden(boolean hidden);

	/**
	 * Inserts a sub-component points at given position.
	 * @param points The points to add to points list.
	 * @param pos	Position where the sub-component is inserted.
	 */
	public void addPoint(LyPoint points, int pos);

	/**
	 * Removes a sub-component points.
	 * @param points The points to remove to points list.
	 */
	public void removePoint(LyPoint points);
}