/*
 * LyGraphicalRow.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.graphic;

import leon.view.*;
import leon.view.setinterface.*;

/**
 * The LyGraphicalRow interface represents the abstract graphical component
 * for the row of views.dtd regardless of the target (swing, swt, dhtml...).
 * @see leon.view.LyRow
 */
public interface LyGraphicalRow
	extends LyGraphicalComponent,
			LySetImageInterface,
			LySetLabelInterface,
			LySetHiddenInterface,
			LySetColorInterface
{
	public static final String			__VERSION = "$Id$";


	/* Methods for updating graphical components */

	/**
	 * Updates attribute open to the given value.
	 * @param open Boolean indicating if attribute open is set or unset.
	 */
	public void setOpen(boolean open);

	/**
	 * Updates attribute image to the given value.
	 * @param image New value of image.
	 */
	public void setImage(String image);

	/**
	 * Updates attribute label to the given value.
	 * @param label New value of label.
	 */
	public void setLabel(String label);

	/**
	 * Updates attribute index to the given value.
	 * @param index New value of index.
	 */
	public void setIndex(String index);

	/**
	 * Updates attribute tooltip to the given value.
	 * @param tooltip New value of tooltip.
	 */
	public void setTooltip(String tooltip);

	/**
	 * Updates attribute hidden to the given value.
	 * @param hidden Boolean indicating if attribute hidden is set or unset.
	 */
	public void setHidden(boolean hidden);

	/**
	 * Updates attribute borderHeight to the given value.
	 * @param borderHeight New value of borderHeight.
	 */
	public void setBorderHeight(int borderHeight);

	/**
	 * Updates attribute borderColor to the given value.
	 * @param borderColor New value of borderColor.
	 */
	public void setBorderColor(String borderColor);

	/**
	 * Updates attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color);

	/**
	 * Inserts a sub-component components at given position.
	 * @param components The components to add to components list.
	 * @param pos	Position where the sub-component is inserted.
	 */
	public void addComponent(LyComponent components, int pos);

	/**
	 * Removes a sub-component components.
	 * @param components The components to remove to components list.
	 */
	public void removeComponent(LyComponent components);
}