/*
 * LyGraphicalTreeNode.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.graphic;

import leon.view.*;
import leon.view.setinterface.*;

/**
 * The LyGraphicalTreeNode interface represents the abstract graphical component
 * for the treeNode of views.dtd regardless of the target (swing, swt, dhtml...).
 * @see leon.view.LyTreeNode
 */
public interface LyGraphicalTreeNode
	extends LyGraphicalComponent,
			LySetLabelInterface,
			LySetSelectedInterface,
			LySetHiddenInterface,
			LySetImageInterface,
			LySetValidityInterface,
			LySetFontInterface,
			LySetColorInterface
{
	public static final String			__VERSION = "$Id$";


	/* Methods for updating graphical components */

	/**
	 * Updates attribute label to the given value.
	 * @param label New value of label.
	 */
	public void setLabel(String label);

	/**
	 * Updates attribute tooltip to the given value.
	 * @param tooltip New value of tooltip.
	 */
	public void setTooltip(String tooltip);

	/**
	 * Updates attribute selected to the given value.
	 * @param selected Boolean indicating if attribute selected is set or unset.
	 */
	public void setSelected(boolean selected);

	/**
	 * Updates attribute open to the given value.
	 * @param open Boolean indicating if attribute open is set or unset.
	 */
	public void setOpen(boolean open);

	/**
	 * Updates attribute hidden to the given value.
	 * @param hidden Boolean indicating if attribute hidden is set or unset.
	 */
	public void setHidden(boolean hidden);

	/**
	 * Updates attribute expandable to the given value.
	 * @param expandable Boolean indicating if attribute expandable is set or unset.
	 */
	public void setExpandable(boolean expandable);

	/**
	 * Updates attribute image to the given value.
	 * @param image New value of image.
	 */
	public void setImage(String image);

	/**
	 * Updates attribute validity to the given value.
	 * @param validity New value of validity.
	 */
	public void setValidity(LyTreeNode.Validity validity);

	/**
	 * Updates attribute font to the given value.
	 * @param font New value of font.
	 */
	public void setFont(String font);

	/**
	 * Updates attribute color to the given value.
	 * @param color New value of color.
	 */
	public void setColor(String color);

	/**
	 * Inserts a sub-component images at given position.
	 * @param images The images to add to images list.
	 * @param pos	Position where the sub-component is inserted.
	 */
	public void addImage(LyImage images, int pos);

	/**
	 * Removes a sub-component images.
	 * @param images The images to remove to images list.
	 */
	public void removeImage(LyImage images);

	/**
	 * Inserts a sub-component treeNodes at given position.
	 * @param treeNodes The treeNodes to add to treeNodes list.
	 * @param pos	Position where the sub-component is inserted.
	 */
	public void addTreeNode(LyTreeNode treeNodes, int pos);

	/**
	 * Removes a sub-component treeNodes.
	 * @param treeNodes The treeNodes to remove to treeNodes list.
	 */
	public void removeTreeNode(LyTreeNode treeNodes);
}