package leon.view.mobile;

import java.io.*;
import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;

public class LyMobileArea extends LyMobileObject
	implements LyGraphicalArea
{
    /** The area generic component. */
    protected LyArea						_area;
    
    protected String 						_id;
    
    protected LyArea 						_template;
    
    protected String 						_xmlFile;
    
    protected String 						_label;
    
    protected String 						_controller;
    
    protected LyTitleBar 					_titleBar;
    
    protected ArrayList<LyToolBar> 			_toolBars;
    
	protected ArrayList<LyPanel>			_panels;
	
	protected LySelectBar 					_selectBar;
	
	protected LyCommands 					_commands;
	
	/**
	 * Font used in this area.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Font color of the component. 
	 */
	protected String			_fontColor;
    
	public LyMobileArea(LyArea area)
	{
        super(area);
        
        _area = area;
        
		_toolBars = new ArrayList<LyToolBar>();
		_panels = new ArrayList<LyPanel>();		
		
		_font = getFontValue(getStringValue("MOBILE_AREA.font"));

		_color = getStringValue("MOBILE_AREA.color");

		_fontColor = getStringValue("MOBILE_AREA.fontColor");
        

	}
	
	public void init()
	{
		
        _id = _area.getId();
		_template = _area.getTemplate();
		_xmlFile = _area.getXmlFile();	

		if(_area.getLabel() != null && !_area.getLabel().equals(""))
		{
			_label = translate(_area.getLabel());
		}
		else
		{
			_label = null;
		}
		_controller = _area.getController();
		_titleBar = _area.getTitleBar();

		_selectBar = _area.getSelectBar();
		_commands = _area.getCommands();
		
		_toolBars.clear();
		_panels.clear();
		
		for(int i = 0; i < _area.getToolBarCount(); i++)
		{
			_toolBars.add(_area.getToolBar(i));
		}
		for(int i = 0; i < _area.getPanelCount(); i++)
		{
			_panels.add(_area.getPanel(i));
		}
	}

	
	public void writeXml(PrintWriter pw)
	{
		init();
		pw.print("<AREA");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
			pw.print(" label='" + LyString.getXmlStringStrict(_label) + "'");
		if (_controller != null)
			pw.print(" controller='" + LyString.getXmlStringStrict(_controller) + "'");

		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + getColorValue(_color) + "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");
		if ((_xmlFile != null)
			|| (_area.getChildCount() == 0))
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");
		
		if (_titleBar != null)
			((LyMobileTitleBar)_titleBar.getInternalView()).writeXml(pw);
		LyComponent	toolBars;
		int			i, n;

		n = (_toolBars == null ? 0 : _toolBars.size());
		for (i = 0; i < n; i++)
		{
			toolBars = _toolBars.get(i);
			((LyMobileToolBar)toolBars.getInternalView(true)).writeXml(pw);
		}

		LyComponent	panels;

		n = (_panels == null ? 0 : _panels.size());
		for (i = 0; i < n; i++)
		{
			panels = _panels.get(i);
			((LyMobileObject)panels.getInternalView(true)).writeXml(pw);
		}

		if (_selectBar != null)
			((LyMobileSelectBar)_selectBar.getInternalView()).writeXml(pw);
		if (_commands != null)
			((LyMobileCommands)_commands.getInternalView()).writeXml(pw);

		pw.println("</AREA>");
	}
	
	public boolean printJsonAttributeSeparator(boolean first, PrintWriter pw)
	{
		if(!first)
		{
			pw.println(",");
			return first;
		}
		
		return false;
	}
	
	public void writeJson(PrintWriter pw, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		pw.println("{\"AREA\": [");
		
		if (_id != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");
		}
			
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"label\": \"" + LyString.getJsonString(_label) + "\"}");
		}

		if (_controller != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"controller\": \"" + LyString.getJsonString(_controller) + "\"}");
		}

		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
		}
		
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"color\": \"" + getColorValue(_color) + "\"}");
		}
		
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");
		}
		
		if ((_xmlFile != null)
			|| (_area.getChildCount() == 0))
		{
			pw.println("]");
			return;
		}
		
		if (_titleBar != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			((LyMobileTitleBar)_titleBar.getInternalView()).writeJson(pw, sessionId);
		}
			
		LyComponent	toolBars;
		int			i, n;

		n = (_toolBars == null ? 0 : _toolBars.size());
		for (i = 0; i < n; i++)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			toolBars = _toolBars.get(i);
			((LyMobileToolBar)toolBars.getInternalView(true)).writeJson(pw, sessionId);
		}

		LyComponent	panels;

		n = (_panels == null ? 0 : _panels.size());
		for (i = 0; i < n; i++)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			panels = _panels.get(i);
			LyViewComponent component = panels.getInternalView();
			if(component instanceof LyWebObject)
			{
				((LyWebObject)panels.getInternalView(true)).writeJson(pw, sessionId);
			}
			else
			{
				((LyMobileObject)panels.getInternalView(true)).writeJson(pw, sessionId);
			}			
		}

		if (_selectBar != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			((LyMobileSelectBar)_selectBar.getInternalView()).writeJson(pw, sessionId);
		}
			
		if (_commands != null)
		{			
			((LyMobileCommands)_commands.getInternalView()).writeJson(pw, isFirstElement, sessionId);
			if(isFirstElement)
			{
				isFirstElement = false;
			}
		}
			

		pw.println("]}");
	}

	
	public void setRedirect(String redirect)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void setRefresh(int refresh)
	{
		// TODO Auto-generated method stub
	}

	
	public void setLayoutInfo(String layoutInfo)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void setOpen(boolean open)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void addPanel(LyPanel panels, int pos)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void removePanel(LyPanel panels)
	{
		// TODO Auto-generated method stub
		
	}
}
