package leon.view.mobile;

import java.io.*;

import leon.misc.*;
import leon.view.*;
import leon.view.LyBoolean.Status;
import leon.view.graphic.*;

public class LyMobileBoolean extends LyMobileObject
	implements LyGraphicalBoolean
{
	/** The boolean generic component. */
    protected LyBoolean			_boolean;
    
    protected String			_id;

	protected LyBoolean			_template;

	protected String			_labelChecked;

	protected String			_labelUnchecked;

	protected String			_labelEmpty;
	
	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;
	
    
	public LyMobileBoolean(LyBoolean bool)
	{
		super(bool);
		
		_boolean = bool;
		
		_font = getFontValue(getStringValue("MOBILE_BOOLEAN.font"));

		_color = getStringValue("MOBILE_BOOLEAN.color");

		_fontColor = getStringValue("MOBILE_BOOLEAN.fontColor");
		
	}
	
	public void init()
	{
		_id = _boolean.getId();
		_template = _boolean.getTemplate();
		_labelChecked = _boolean.getLabelChecked();
		_labelUnchecked = _boolean.getLabelUnchecked();
		_labelEmpty = _boolean.getLabelEmpty();
	}

	
	public void writeXml(PrintWriter pw)
	{
		// Write tag with its attributes
		init();
		pw.print("<BOOLEAN");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && (_labelChecked != null))
				|| ((_template != null) && (_labelChecked != _template.getLabelChecked())))
			pw.print(" labelChecked='" + LyString.getXmlStringStrict(_labelChecked) + "'");
		if (((_template == null) && (_labelUnchecked != null))
				|| ((_template != null) && (_labelUnchecked != _template.getLabelUnchecked())))
			pw.print(" labelUnchecked='" + LyString.getXmlStringStrict(_labelUnchecked) + "'");
		if (((_template == null) && (_labelEmpty != null))
				|| ((_template != null) && (_labelEmpty != _template.getLabelEmpty())))
			pw.print(" labelEmpty='" + LyString.getXmlStringStrict(_labelEmpty) + "'");
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + getColorValue(_color) + "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");
		pw.println("/>");

	}
	
	public boolean printJsonAttributeSeparator(boolean first, PrintWriter pw)
	{
		if(!first)
		{
			pw.println(",");
			return first;
		}
		
		return false;
	}
	
	public void writeJson(PrintWriter pw, String sessionId)
	{
		boolean isFirstElement = true;
		init();		
		pw.println("{\"BOOLEAN\": [");
		
		if (_id != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");
		}
			
		if (((_template == null) && (_labelChecked != null))
				|| ((_template != null) && (_labelChecked != _template.getLabelChecked())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"labelChecked\": \"" + LyString.getJsonString(_labelChecked) + "\"}");
		}
		
		if (((_template == null) && (_labelUnchecked != null))
				|| ((_template != null) && (_labelUnchecked != _template.getLabelUnchecked())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"labelUnchecked\": \"" + LyString.getJsonString(_labelUnchecked) + "\"}");
		}
		
		if (((_template == null) && (_labelEmpty != null))
				|| ((_template != null) && (_labelEmpty != _template.getLabelEmpty())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"labelEmpty\": \"" + LyString.getJsonString(_labelEmpty) + "\"}");
		}

		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
		}
		
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"color\": \"" + getColorValue(_color) + "\"}");
		}
		
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");
		}
		pw.println("]}");
	}

	
	public void setStatus(Status status)
	{
		// TODO Auto-generated method stub
		
	}

}
