package leon.view.mobile;

import java.io.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;

public class LyMobileCaption extends LyMobileObject
	implements LyGraphicalCaption
{
	/** The Caption generic component. */
    protected LyCaption						_caption;
    
    protected String 						_id;
    
    protected LyCaption 					_template;
        
    protected String 						_label;
	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;
    
	/**
	 * Indicates whether the text of the legend should be followed by a colon or not.
	 */
	protected boolean			_colon;

	public LyMobileCaption(LyCaption caption)
	{
		super(caption);
        
        _caption = caption;
		
		_font = getFontValue(getStringValue("MOBILE_CAPTION.font"));

		_color = getStringValue("MOBILE_CAPTION.color");

		_fontColor = getStringValue("MOBILE_CAPTION.fontColor");
       
	}
	
	public void init()
	{
		_id = _caption.getId();
		_template = _caption.getTemplate();
		if(_caption.getLabel() != null && !_caption.getLabel().equals(""))
		{
			_label = translate(_caption.getLabel());
		}
		else
		{
			_label = null;
		}
		_colon = _caption.isColon();
	}

	
	public void writeXml(PrintWriter pw)
	{
		// Write tag with its attributes
		init();
		pw.print("<CAPTION");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
			pw.print(" label='" + LyString.getXmlStringStrict(_label) + "'");
		if (((_template == null) && !_colon)
				|| ((_template != null) && (_colon != _template.isColon())))
			pw.print(" colon='" + _colon + "'");
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + getColorValue(_color) + "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");
		
		pw.println("/>");

	}
	
	public void writeJson(PrintWriter pw, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		pw.println("{\"CAPTION\": [");
		
		if (_id != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");
		}
			
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"label\": \"" + LyString.getJsonString(_label) + "\"}");
		}

		if (((_template == null) && !_colon)
				|| ((_template != null) && (_colon != _template.isColon())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"colon\": \"" + _colon + "\"}");
		}

		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
		}
		
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"color\": \"" + getColorValue(_color) + "\"}");
		}
		
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");
		}
		pw.println("]}");
	}

	
	public void setLabel(String label)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void setEnabled(boolean enabled)
	{
		// TODO Auto-generated method stub
		
	}

}
