package leon.view.mobile;

import java.io.*;
import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;

public class LyMobileCell extends LyMobileTableObject
implements LyGraphicalCell
{
	/** The cell generic component. */
	protected LyCell                      	_cell;

	protected String						_id;

	protected LyCell						_template;

	protected String						_label;

	protected boolean						_editable;

	protected String						_image;

	protected ArrayList<LyCell>				_cells;

	protected String						_rootUrl;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	public LyMobileCell(LyCell cell)
	{
		super(cell);

		_cell = cell;

		_cells = new ArrayList<LyCell>();
		_rootUrl = ((LyMobileViewManager) getViewManager()).getRealRootURL();

		_font = getFontValue(getStringValue("MOBILE_CELL.font"));

		_color = getStringValue("MOBILE_CELL.color");

		_fontColor = getStringValue("MOBILE_CELL.fontColor");
	}

	public void init()
	{
		_id = _cell.getId();
		_template = _cell.getTemplate();
		_label = _cell.getLabel();
		_editable = _cell.isEditable();
		_image = _cell.getImage();

		_cells.clear();

		for(int i = 0; i < _cell.getCellCount(); i++)
		{
			_cells.add(_cell.getCell(i));
		}		
	}

	public void writeXml(PrintWriter pw)
	{
		writeXml(pw, false, 1);
	}


	public void writeXml(PrintWriter pw, boolean repeat, int order)
	{
		init();
		if(_cells == null || _cells.size() == 0)
		{
			pw.print("<CELL");
			if (_id != null)
				pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");

			pw.print(" order='"+order+"'");
			if (((_template == null) && (_label != null))
					|| ((_template != null) && (_label != _template.getLabel())))
				pw.print(" label='" + LyString.getXmlStringStrict(_label) + "'");

			if (((_template == null) && (_image != getStringValue("CELL.image", "$ALL.image")))
					|| ((_template != null) && (_image != _template.getImage())))
				pw.print(" image='" + _rootUrl+ getImageUrl(_image) + "'");


			if (((_template == null) && (_font != null))
					|| ((_template != null) && (_font != _template.getFont())))
				pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
			if (((_template == null) && (_color != null))
					|| ((_template != null) && (_color != _template.getColor())))
				pw.print(" color='" + getColorValue(_color) + "'");
			if (((_template == null) && (_fontColor != null))
					|| ((_template != null) && (_fontColor != _template.getFontColor())))
				pw.print(" fontColor='" + getColorValue(_fontColor) + "'");


			if (_cell.getChildCount() == 0)
			{
				pw.println("/>");
				return;
			}
			else
				pw.println(">");

		}
		// Write tag with its attributes



		// Write sub-components
		LyComponent	cells;
		int			i, n;

		n = (_cells == null ? 0 : _cells.size());
		for (i = 0; i < n; i++)
		{
			cells = _cells.get(i);

			((LyMobileCell)cells.getInternalView()).writeXml(pw, true, i+1);


		}

		if(_cells == null || _cells.size() == 0)
		{
			pw.println("</CELL>");
		}

	}
	
	public void writeJson(PrintWriter pw, String sessionId)
	{
		writeJson(pw, false, 1, sessionId);
	}

	public void writeJson(PrintWriter pw, boolean repeat, int order, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		if(_cells == null || _cells.size() == 0)
		{
			pw.println("{\"CELL\": [");

			if (_id != null)
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");
			}

			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"order\": \"" + order+ "\"}");

			if (((_template == null) && (_label != null))
					|| ((_template != null) && (_label != _template.getLabel())))
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"label\": \"" + LyString.getJsonString(_label) + "\"}");
			}

			if (((_template == null) && (_image != getStringValue("CELL.image", "$ALL.image")))
					|| ((_template != null) && (_image != _template.getImage())))
			{
				String imageUrl = getImageUrl(_image);
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);	
				pw.print("{\"image\": \"" + 
						(imageUrl.startsWith(_rootUrl) ? imageUrl : _rootUrl+imageUrl)  + "\"}");
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);	
				pw.print("{\"sessionId\": \"" + sessionId + "\"}");
			}

			if (((_template == null) && (_font != null))
					|| ((_template != null) && (_font != _template.getFont())))
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
			}

			if (((_template == null) && (_color != null))
					|| ((_template != null) && (_color != _template.getColor())))
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"color\": \"" + getColorValue(_color) + "\"}");
			}

			if (((_template == null) && (_fontColor != null))
					|| ((_template != null) && (_fontColor != _template.getFontColor())))
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");
			}

			if (_cell.getChildCount() == 0)
			{
				pw.println("]}");
				return;
			}			
		}

		// Write sub-components
		LyComponent	cells;
		int			i, n;

		n = (_cells == null ? 0 : _cells.size());
		for (i = 0; i < n; i++)
		{
			cells = _cells.get(i);
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			((LyMobileCell)cells.getInternalView()).writeJson(pw, true, i+1, sessionId);


		}

		if(_cells == null || _cells.size() == 0)
		{
			pw.println("]}");
		}

		
	}

	public void setLabel(String label)
	{
		// TODO Auto-generated method stub

	}

	public void setObjectId(String objectId)
	{
		// TODO Auto-generated method stub

	}

	public void setEditable(boolean editable)
	{
		// TODO Auto-generated method stub

	}

	public void setSelected(boolean selected)
	{
		// TODO Auto-generated method stub

	}

	public void setImage(String image)
	{
		// TODO Auto-generated method stub

	}

	public void setColor(String color)
	{
		// TODO Auto-generated method stub

	}

	public void setTooltip(String tooltip)
	{
		// TODO Auto-generated method stub

	}

	public void setTopBorderColor(String topBorderColor)
	{
		// TODO Auto-generated method stub

	}

	public void setBottomBorderColor(String bottomBorderColor)
	{
		// TODO Auto-generated method stub

	}

	public void setLeftBorderColor(String leftBorderColor)
	{
		// TODO Auto-generated method stub

	}

	public void setRightBorderColor(String rightBorderColor)
	{
		// TODO Auto-generated method stub

	}

	public void setTopLeftImage(String topLeftImage)
	{
		// TODO Auto-generated method stub

	}

	public void setTopRightImage(String topRightImage)
	{
		// TODO Auto-generated method stub

	}

	public void setBottomLeftImage(String bottomLeftImage)
	{
		// TODO Auto-generated method stub

	}

	public void setBottomRightImage(String bottomRightImage)
	{
		// TODO Auto-generated method stub

	}

	public void addCell(LyCell cells, int pos)
	{
		// TODO Auto-generated method stub

	}

	public void removeCell(LyCell cells)
	{
		// TODO Auto-generated method stub

	}

	public int writeXml(PrintWriter pw, int nb)
	{
		init();
		pw.print("<CELL");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
			pw.print(" label='" + LyString.getXmlStringStrict(_label) + "'");


		if (((_template == null) && (_font != getStringValue("$ALL.font")))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_color != getStringValue("CELL.color", "$ALL.color")))
				|| ((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + getColorValue(_color) + "'");
		if (((_template == null) && (_fontColor != getStringValue("CELL.fontColor", "$ALL.fontColor")))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");

		if (_cell.getChildCount() == 0)
		{
			pw.println("/>");
			return nb-1;
		}
		else
			pw.println(">");

		// Write sub-components
		LyComponent	cells;
		int			i, n;

		n = (_cells == null ? 0 : _cells.size());
		for (i = 0; i < n; i++)
		{
			if(nb > 0)
			{
				cells = _cells.get(i);
				((LyMobileCell)cells.getInternalView()).writeXml(pw, nb);
			}
		}


		pw.println("</CELL>");
		return nb-1;
	}

}
