package leon.view.mobile;

import java.io.*;
import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.common.*;
import leon.view.getinterface.*;
import leon.view.graphic.*;

public class LyMobileChart extends LyMobileObject
	implements LyGraphicalChart,
	LyGetChartConverterInterface
{
	/** The chart generic component. */
    protected LyChart						_chart;
    
	/** The generic chart converter for this component. */
	protected LyCommonFreeChartConverter _chartConv;
	
	/** Width of the generated image. */
	protected int						 _width;

	/** Height of the generated image. */
	protected int						_height;
	
	protected String					_path;
    
	public LyMobileChart(LyChart chart)
	{
		super(chart);

		_chart = chart;
		init();
		
		
    }

	
	public void writeXml(PrintWriter pw)
	{
		init();
		pw.print("<CHART");
		pw.print(" image='" + _path + "'");
		pw.println("/>");
		free();
	}
	
	public void writeJson(PrintWriter pw, String sessionId)
	{
		init();
		pw.println("{\"CHART\": [");
		pw.print("{\"image\": \"" + _path + "\"}");
		pw.println("]}");
		free();
	}

	
	public void addColumn(LyColumn columns, int pos)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void removeColumn(LyColumn columns)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void init()
	{
		LyEnvironment   environment = _chart.getEnvironment();
		String chartId = _chart.getId();
		String dateStamp = "";
		
		if ((chartId != null) && (chartId.endsWith("_chart")))
			dateStamp = "_" + (new Date()).getTime();
        
        _width = environment.getIntResource("LY_CHART_WIDTH");
        if (_width < 0)
            _width = LyCommonFreeChartConverter.DEFAULT_CHART_WIDTH;
        _height = environment.getIntResource("LY_CHART_HEIGHT"); 
        if (_height < 0)
            _height = LyCommonFreeChartConverter.DEFAULT_CHART_HEIGHT;
        
        _chartConv = new LyCommonFreeChartConverter(_chart);
        _chartConv.getFreeChart();
        
        LyViewManager viewManager = getViewManager();
		String        tmpDir = viewManager.getTemporaryDir();
		getEnvironment().logFatal("TmpDir : "+tmpDir);
		String 		  url = ((LyMobileViewManager) viewManager).getRealRootURL();
		getEnvironment().logFatal("RootUrl : "+url);
		String 		  tmpLoc = viewManager.getTemporaryLocation();
		getEnvironment().logFatal("tmpLoc : "+tmpLoc);
		String        imageFile = tmpDir + "/chart_" + chartId + dateStamp;
		//_path = "chart_" + chartId + dateStamp+".png";
		_path = url+tmpLoc+"/chart_" + chartId + dateStamp+".png";
		getEnvironment().logFatal("Path : "+ _path);
		StringBuffer  buffer = new StringBuffer();
      
		saveChartFile(tmpDir, imageFile, buffer);
		
	}
	
	public void free()
	{
		_chartConv = null;
	}
	
	/**
	 * Save the file to the http directory.
	 *
	 * @param tmpDir    temporary directory for the image/svg
	 * @param imageFile name of the file
	 * @param buffer    the buffer where imagemap must be generated
	 */
	public void saveChartFile(String tmpDir, String imageFile, StringBuffer buffer)
	{
		if ((tmpDir == null) || (imageFile == null))
			return;

		// add the extension
		imageFile += ".png";

        _chartConv.setDimension(_width, _height);
		_chartConv.saveImg(tmpDir, imageFile, buffer, true);
	}


	public LyChartConverter getChartConverter()
	{
		return _chartConv;
	}


	public int[] getComponentDimension()
	{
		return new int[] {_width, _height};
	}

}
