package leon.view.mobile;

import java.io.*;
import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;

public class LyMobileChoice extends LyMobileObject
implements LyGraphicalChoice
{
	/** The choice generic component. */
	protected LyChoice					_choice;

	protected String					_id;

	protected LyChoice					_template;

	protected boolean					_multiple;

	protected ArrayList<LyOption>		_options;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	public LyMobileChoice(LyChoice choice)
	{
		super(choice);

		_choice = choice;

		_options = new ArrayList<LyOption>();

		_font = getFontValue(getStringValue("MOBILE_CHOICE.font"));

		_color = getStringValue("MOBILE_CHOICE.color");

		_fontColor = getStringValue("MOBILE_CHOICE.fontColor");

	}



	public void writeXml(PrintWriter pw)
	{
		// Write tag with its attributes
		init();

		pw.print("<CHOICE");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && _multiple)
				|| ((_template != null) && (_multiple != _template.isMultiple())))
			pw.print(" multiple='" + _multiple + "'");

		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + getColorValue(_color) + "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");

		if ((_choice.getChildCount() == 0))
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		LyComponent	options;
		int			i, n;

		n = (_options == null ? 0 : _options.size());
		for (i = 0; i < n; i++)
		{
			options = _options.get(i);
			((LyMobileOption)options.getInternalView()).writeXml(pw);
		}


		pw.println("</CHOICE>");

	}

	public void writeJson(PrintWriter pw, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		pw.println("{\"CHOICE\": [");

		if (_id != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");
		}

		if (((_template == null) && _multiple)
				|| ((_template != null) && (_multiple != _template.isMultiple())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"multiple\": \"" + _multiple + "\"}");
		}

		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
		}

		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"color\": \"" + getColorValue(_color) + "\"}");
		}

		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");
		}

		if (_choice.getChildCount() == 0)
		{
			pw.println("]}");
			return;
		}

		// Write sub-components
		LyComponent	options;
		int			i, n;

		n = (_options == null ? 0 : _options.size());
		for (i = 0; i < n; i++)
		{
			options = _options.get(i);
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			((LyMobileOption)options.getInternalView()).writeJson(pw, sessionId);
		}


		pw.println("]}");
	}

	public void init()
	{

		_id = _choice.getId();
		_template = _choice.getTemplate();
		_multiple = _choice.isMultiple();
		_options.clear();

		for(int i = 0; i < _choice.getOptionCount(); i++)
		{
			_options.add(_choice.getOption(i));
		}		
	}

}
