package leon.view.mobile;

import java.io.*;

import leon.misc.*;
import leon.view.*;
import leon.view.LyCommand.Action;
import leon.view.graphic.*;

public class LyMobileCommand extends LyMobileObject
implements LyGraphicalCommand
{
	/** The commands generic component. */
	protected LyCommand						_command;

	protected String 						_id;

	protected LyCommand 					_template;

	protected String 						_label;

	protected Action 						_action;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	public LyMobileCommand(LyCommand command)
	{
		super(command);

		_command = command;

		_font = getFontValue(getStringValue("MOBILE_COMMAND.font"));

		_color = getStringValue("MOBILE_COMMAND.color");

		_fontColor = getStringValue("MOBILE_COMMAND.fontColor");

	}


	public void writeXml(PrintWriter pw)
	{
		// Write tag with its attributes
		init();
		if(_label.equals("LY_APPLY"))
		{
			return;
		}
		else
		{
			if(!_command.getLabel().equals(""))
			{
				_label = getString(_label);
			}
		}
		pw.print("<COMMAND");
		pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		pw.print(" label='" + LyString.getXmlStringStrict(_label) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if ((_template == null)
				|| (_action != _template.getAction()))
		{
			if (_action == Action.CANCEL)
				pw.print(" action='CANCEL'");
			else if (_action == Action.SUBMIT)
				pw.print(" action='SUBMIT'");
			else if (_action == Action.RESET)
				pw.print(" action='RESET'");
			else if (_action == Action.HELP)
				pw.print(" action='HELP'");
			else if ((_action == Action.OTHER) && (_template != null))
				pw.print(" action='OTHER'");
			else if (_action == Action.CLOSE)
				pw.print(" action='CLOSE'");
		}

		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + getColorValue(_color) + "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");

		pw.println("/>");

	}

	public void writeJson(PrintWriter pw, String sessionId)
	{		
		writeJson(pw, false, sessionId);
	}
	
	public void writeJson(PrintWriter pw, boolean print, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		if(_label.equals("LY_APPLY"))
		{
			return;
		}
		else
		{
			if(!_command.getLabel().equals(""))
			{
				_label = getString(_label);
			}
		}
		if(print)
		{
			pw.println(",");
		}
		pw.println("{\"COMMAND\": [");
		isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
		pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");
		isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
		pw.print("{\"label\": \"" + LyString.getJsonString(_label) + "\"}");

		if ((_template == null)
				|| (_action != _template.getAction()))
		{
			
			if (_action == Action.CANCEL)
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"action\": \"CANCEL\"}");
			}
			else if (_action == Action.SUBMIT)
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"action\": \"SUBMIT\"}");
			}
			else if (_action == Action.RESET)
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"action\": \"RESET\"}");
			}
			else if (_action == Action.HELP)
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"action\": \"HELP\"}");
			}
			else if ((_action == Action.OTHER) && (_template != null))
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"action\": \"OTHER\"}");
			}
			else if (_action == Action.CLOSE)
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"action\": \"CLOSE\"}");
			}
		}

		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
		}

		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"color\": \"" + getColorValue(_color) + "\"}");
		}

		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");
		}

		pw.println("]}");
	}

	public void init()
	{
		_id = _command.getId();
		_template = _command.getTemplate();
		_label = _command.getLabel();
		_action = _command.getAction();
	}


	public void setLabel(String label)
	{
		// TODO Auto-generated method stub

	}


	public void setEnabled(boolean enabled)
	{
		// TODO Auto-generated method stub

	}

}
