package leon.view.mobile;

import java.io.*;

import leon.misc.*;
import leon.view.*;
import leon.view.LyDateBase.*;
import leon.view.graphic.*;
import leon.view.web.mobile.*;

public class LyMobileDate extends LyMobileObject
implements LyGraphicalDate
{
	/** The date generic component. */
	protected LyDate			_date;

	protected String			_id;

	protected LyDate			_template;

	protected String			_label;

	protected Type				_type;

	protected String			_format;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	public LyMobileDate(LyDate date)
	{
		super(date);

		_date = date;

		_font = getFontValue(getStringValue("MOBILE_DATE.font"));

		_color = getStringValue("MOBILE_DATE.color");

		_fontColor = getStringValue("MOBILE_DATE.fontColor");
	}


	public void writeXml(PrintWriter pw)
	{
		init();
		// Write tag with its attributes
		pw.print("<DATE");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
			pw.print(" label='" + LyString.getXmlStringStrict(_label) + "'");

		if (_type == Type.DATE_TIME)
			pw.print(" type='DATE_TIME'");
		else if (_type == Type.DATE_HOUR_MIN)
			pw.print(" type='DATE_HOUR_MIN'");
		else if (_type == Type.DATE)
			pw.print(" type='DATE'");
		else if (_type == Type.TIME)
			pw.print(" type='TIME'");
		else if (_type == Type.HOUR_MIN)
			pw.print(" type='HOUR_MIN'");

		if (((_template == null) && (_format != null))
				|| ((_template != null) && (_format != _template.getFormat())))
			pw.print(" format='" + LyString.getXmlStringStrict(_format) + "'");

		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + getColorValue(_color) + "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");
		pw.println("/>");

	}
	
	public void writeJson(PrintWriter pw, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		pw.println("{\"DATE\": [");
		
		if (_id != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");
		}
			
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"label\": \"" + LyString.getJsonString(_label) + "\"}");
		}
		
		if(_type != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			if (_type == Type.DATE_TIME)
				pw.print("{\"type\": \"DATE_TIME\"}");
			else if (_type == Type.DATE_HOUR_MIN)
				pw.print("{\"type\": \"DATE_HOUR_MIN\"}");
			else if (_type == Type.DATE)
				pw.print("{\"type\": \"DATE\"}");
			else if (_type == Type.TIME)
				pw.print("{\"type\": \"TIME\"}");
			else if (_type == Type.HOUR_MIN)
				pw.print("{\"type\": \"HOUR_MIN\"}");
		}
		

		if (((_template == null) && (_format != null))
				|| ((_template != null) && (_format != _template.getFormat())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"format\": \"" + LyString.getJsonString(_format) + "\"}");
		}
		
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
		}
		
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"color\": \"" + getColorValue(_color) + "\"}");
		}
		
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");
		}
		pw.println("]}");
	}


	public void init()
	{
		_id = _date.getId();
		_template = _date.getTemplate();
		_label = _date.getLabel();
		_type = _date.getType();
		if(_type == null)
		{
			_type = Type.DATE_TIME;
		}
		_format = _date.getFormat();	
		if(_format == null)
		{
			_format = LyMobileConstants.DEFAULT_LYDATE_FORMAT;
		}
	}


	public void setLabel(String label)
	{
		// TODO Auto-generated method stub

	}

}
