package leon.view.mobile;

import java.io.*;

import leon.misc.*;
import leon.view.*;
import leon.view.LyFile.*;
import leon.view.graphic.*;

public class LyMobileFile extends LyMobileObject
implements LyGraphicalFile
{
	/** The file generic component. */
	protected LyFile			_file;

	protected String			_id;

	protected LyFile			_template;

	protected String			_label;

	protected Type				_type;

	protected String			_attachmentDirectory;
	
	/**
	 * Text font used on the component.
	 */
	protected String			_font;
	/**
	 * Background of the file selector when enabled.
	 */
	protected String			_enabledColor;

	/**
	 * Background of the file selector when disabled.
	 */
	protected String			_disabledColor;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	public LyMobileFile(LyFile file)
	{
		super(file);

		_file = file;
		
		_font = getFontValue(getStringValue("MOBILE_FILE.font"));

		_enabledColor = getStringValue("MOBILE_FILE.enabledColor");
		
		_disabledColor = getStringValue("MOBILE_FILE.disabledColor");

		_fontColor = getStringValue("MOBILE_FILE.fontColor");
	}

	public void init()
	{
		_id = _file.getId();
		_template = _file.getTemplate();
		_label = _file.getLabel();
		_type = _file.getType();
		_attachmentDirectory = _file.getAttachmentDirectory();		
	}


	public void writeXml(PrintWriter pw)
	{
		init();
		pw.print("<FILE");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
			pw.print(" label='" + LyString.getXmlStringStrict(_label) + "'");
		if ((_template == null)
				|| (_type != _template.getType()))
		{
			if ((_type == Type.TEXT) && (_template != null))
				pw.print(" type='TEXT'");
			else if (_type == Type.IMAGE)
				pw.print(" type='IMAGE'");
			else if (_type == Type.AUDIO)
				pw.print(" type='AUDIO'");
			else if (_type == Type.VIDEO)
				pw.print(" type='VIDEO'");
			else if (_type == Type.DIRECTORY)
				pw.print(" type='DIRECTORY'");
		}
		if (_attachmentDirectory != null)
			pw.print(" attachmentDirectory='" + LyString.getXmlStringStrict(_attachmentDirectory) + "'");
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_enabledColor != null))
				|| ((_template != null) && (_enabledColor != _template.getEnabledColor())))
			pw.print(" enabledColor='" + getColorValue(_enabledColor) + "'");
		if (((_template == null) && (_disabledColor != null))
				|| ((_template != null) && (_disabledColor != _template.getDisabledColor())))
			pw.print(" disabledColor='" + getColorValue(_disabledColor) + "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");
		
		pw.println("/>");
	}
	
	public void writeJson(PrintWriter pw, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		pw.println("{\"FILE\": [");
		
		if (_id != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");
		}
			
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"label\": \"" + LyString.getJsonString(_label) + "\"}");
		}

		if ((_template == null)
				|| (_type != _template.getType()))
		{			
			if ((_type == Type.TEXT) && (_template != null))
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"type\": \"TEXT\"}");
			}
			else if (_type == Type.IMAGE)
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"type\": \"IMAGE\"}");
			}
			else if (_type == Type.AUDIO)
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"type\": \"AUDIO\"}");
			}
			else if (_type == Type.VIDEO)
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"type\": \"VIDEO\"}");
			}
			else if (_type == Type.DIRECTORY)
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"type\": \"DIRECTORY\"}");
			}
				
		}

		if (_attachmentDirectory != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"attachmentDirectory\": \"" + LyString.getJsonString(_attachmentDirectory) + "\"}");
		}
		
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
		}
		
		if (((_template == null) && (_enabledColor != null))
				|| ((_template != null) && (_enabledColor != _template.getEnabledColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"enabledColor\": \"" + getColorValue(_enabledColor) + "\"}");
		}
		
		if (((_template == null) && (_disabledColor != null))
				|| ((_template != null) && (_disabledColor != _template.getDisabledColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"disabledColor\": \"" + getColorValue(_disabledColor) + "\"}");
		}
		
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");
		}
		
		pw.println("]}");
	}


	public void setLabel(String label)
	{
		// TODO Auto-generated method stub

	}

}
