package leon.view.mobile;

import java.io.*;
import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;

public class LyMobileFooters extends LyMobileTableObject
	implements LyGraphicalFooters
{
    /** The footers generic component. */
    protected LyFooters                 _footer;
    
    protected String					_id;

	protected LyFooters					_template;

	protected String					_font;

	protected String					_color;

	protected String					_fontColor;

	protected ArrayList<LyFooter>		_footers;
	
	protected Hashtable<String, String> _datas;
	
	public LyMobileFooters(LyFooters footers)
	{
		super(footers);
		
		_footer = footers;
		
		_footers = new ArrayList<LyFooter>();
	}

	@Override
	public void writeXml(PrintWriter pw)
	{
		// Write tag with its attributes
		init();
		pw.print("<FOOTERS");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && (_font != getStringValue("FOOTERS.font", "$ALL.font")))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_color != getStringValue("FOOTERS.color", "$ALL.color")))
				|| ((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + LyString.getXmlStringStrict(_color) + "'");
		if (((_template == null) && (_fontColor != getStringValue("FOOTERS.fontColor", "$ALL.fontColor")))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + LyString.getXmlStringStrict(_fontColor) + "'");

		if ((_footer.getChildCount() == 0) && (_datas == null))
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		LyComponent	footers;
		int			i, n;

		n = (_footers == null ? 0 : _footers.size());
		for (i = 0; i < n; i++)
		{
			footers = _footers.get(i);
			((LyMobileFooter)footers.getInternalView()).writeXml(pw);
		}


		pw.println("</FOOTERS>");

	}
	
	public void writeJson(PrintWriter pw, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		pw.println("\"FOOTERS\": {");
		
		if (_id != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("\"id\": \"" + LyString.getJsonString(_id) + "\"");
		}

		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("\"font\": \"" + LyString.getJsonString(_font) + "\"");
		}
		
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("\"color\": \"" + getColorValue(_color) + "\"");
		}
		
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("\"fontColor\": \"" + getColorValue(_fontColor) + "\"");
		}
		
		if ((_footer.getChildCount() == 0) && (_datas == null))
		{
			pw.println("}");
			return;
		}

		// Write sub-components
		LyComponent	footers;
		int			i, n;

		n = (_footers == null ? 0 : _footers.size());
		for (i = 0; i < n; i++)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			footers = _footers.get(i);
			((LyMobileFooter)footers.getInternalView()).writeJson(pw, sessionId);
		}
		
		pw.println("}");
	}

	@Override
	public void init()
	{
		_id = _footer.getId();
		_template = _footer.getTemplate();
		_font = _footer.getFont();
		_color = _footer.getColor();
		_fontColor = _footer.getFontColor();

		_footers.clear();
		
		for(int i = 0; i < _footer.getFooterCount(); i++)
		{
			_footers.add(_footer.getFooter(i));
		}
		
		_datas = _footer.getDatas();
		
	}

	@Override
	public int writeXml(PrintWriter pw, int nb)
	{
		init();
		pw.print("<FOOTERS");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && (_font != getStringValue("FOOTERS.font", "$ALL.font")))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_color != getStringValue("FOOTERS.color", "$ALL.color")))
				|| ((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + LyString.getXmlStringStrict(_color) + "'");
		if (((_template == null) && (_fontColor != getStringValue("FOOTERS.fontColor", "$ALL.fontColor")))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + LyString.getXmlStringStrict(_fontColor) + "'");

		if ((_footer.getChildCount() == 0) && (_datas == null))
		{
			pw.println("/>");
			return nb;
		}
		else
			pw.println(">");

		// Write sub-components
		LyComponent	footers;
		int			i, n;

		n = (_footers == null ? 0 : _footers.size());
		for (i = 0; i < n; i++)
		{
			if(nb > 0)
			{
				footers = _footers.get(i);
				nb = ((LyMobileFooter)footers.getInternalView()).writeXml(pw, nb);
			}
			
		}
		pw.println("</FOOTERS>");
		return nb;
	}

}
