package leon.view.mobile;

import java.io.*;
import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;

public class LyMobileForm extends LyMobileObject
implements LyGraphicalForm
{
	/** The form generic component. */
	protected LyForm						_form;

	protected String						_id;

	protected LyForm						_template;

	protected String						_controller;

	protected ArrayList<LyFormField>		_formFields;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	/**
	 * Color used to display a field on warning.
	 */
	protected String			_warningColor;

	/**
	 * Color used to display a field on error.
	 */
	protected String			_errorColor;

	public LyMobileForm(LyForm form)
	{
		super(form);

		_form = form;

		_formFields = new ArrayList<LyFormField>();	

		_font = getFontValue(getStringValue("MOBILE_FORM.font"));

		_color = getStringValue("MOBILE_FORM.color");

		_fontColor = getStringValue("MOBILE_FORM.fontColor");

		_warningColor = getStringValue("MOBILE_FORM.warningColor");

		_errorColor = getStringValue("MOBILE_FORM.errorColor");
	}


	public void writeXml(PrintWriter pw)
	{
		// Write tag with its attributes
		init();
		pw.print("<FORM");
		pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		//pw.print(" label='" + LyString.getXmlStringStrict(translate(_id)) + "'");
		/*int index = _id.indexOf("::");
		if(index != -1)
		{
			String label = _id.substring(index + 2);
			label = getString(label);
			pw.print(" label='" + LyString.getXmlStringStrict(label) + "'");
		}*/


		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + getColorValue(_color) + "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");
		if (((_template == null) && (_warningColor != null))
				|| ((_template != null) && (_warningColor != _template.getWarningColor())))
			pw.print(" warningColor='" + getColorValue(_warningColor) + "'");
		if (((_template == null) && (_errorColor != null))
				|| ((_template != null) && (_errorColor != _template.getErrorColor())))
			pw.print(" errorColor='" + getColorValue(_errorColor) + "'");

		if (_controller != null)
			pw.print(" controller='" + LyString.getXmlStringStrict(_controller) + "'");

		if (_form.getChildCount() == 0)
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		LyComponent	formFields;
		int			i, n;

		n = (_formFields == null ? 0 : _formFields.size());
		for (i = 0; i < n; i++)
		{
			formFields = _formFields.get(i);
			((LyMobileFormField)formFields.getInternalView()).writeXml(pw);
		}

		pw.println("</FORM>");

	}

	public void writeJson(PrintWriter pw, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		pw.println("{\"FORM\": [");
		isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
		pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");

		if (_controller != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"controller\": \"" + LyString.getJsonString(_controller) + "\"}");
		}

		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
		}

		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"color\": \"" + getColorValue(_color) + "\"}");
		}

		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");
		}

		if (((_template == null) && (_warningColor != null))
				|| ((_template != null) && (_warningColor != _template.getWarningColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"warningColor\": \"" + getColorValue(_warningColor) + "\"}");
		}

		if (((_template == null) && (_errorColor != null))
				|| ((_template != null) && (_errorColor != _template.getErrorColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"errorColor\": \"" + getColorValue(_errorColor) + "\"}");
		}

		if (_form.getChildCount() == 0)
		{
			pw.println("]}");
			return;
		}

		// Write sub-components
		LyComponent	formFields;
		int			i, n;

		n = (_formFields == null ? 0 : _formFields.size());
		for (i = 0; i < n; i++)
		{
			formFields = _formFields.get(i);
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			((LyMobileFormField)formFields.getInternalView()).writeJson(pw, sessionId);
		}

		pw.println("]}");
}


	public void init()
	{
		_id = _form.getId();
		_template = _form.getTemplate();
		_controller = _form.getController();

		_formFields.clear();

		for(int i = 0; i < _form.getFormFieldCount(); i++)
		{
			_formFields.add(_form.getFormField(i));
		}

	}


	public void setEditable(boolean editable)
	{
		// TODO Auto-generated method stub

	}

}
