package leon.view.mobile;

import java.io.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;

public class LyMobileFormField extends LyMobileTableObject
implements LyGraphicalFormField
{
	/** The FormfieldBase generic component. */
	protected LyFormField				_field;
	/** Possible values for attribute 'state'. */
	public static final short				STATE_NORMAL = 0;
	public static final short				STATE_ERROR = 1;
	public static final short				STATE_WARNING = 2;

	protected String 						_id;

	protected LyFormField 					_template;

	protected boolean 						_editable;

	protected boolean 						_enabled;

	protected boolean 						_optional;

	protected boolean 						_needCheck;

	protected boolean 						_hidden;

	protected String 						_controller;

	protected LyCaption 					_caption;

	protected LyFieldItem 					_fieldItem;

	protected LyToolBar 					_toolBar;

	protected LyRelation 					_relation;

	/**
	 * State of the field. The state of the field has one of the following values:
	 * NORMAL: Normal state,
	 * WARNING: warning state (non-blocking state)
	 * ERROR: error state (blocking state).
	 */
	protected short				_state;

	public LyMobileFormField(LyFormField field)
	{
		super(field);

		_field = field;

	}


	public void writeXml(PrintWriter pw)
	{
		// Write tag with its attributes
		init();
		pw.print("<FIELD");
		pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && !_editable)
				|| ((_template != null) && (_editable != _template.isEditable())))
			pw.print(" editable='" + _editable + "'");
		if (((_template == null) && !_optional)
				|| ((_template != null) && (_optional != _template.isOptional())))
			pw.print(" optional='" + _optional + "'");
		if (((_template == null) && _needCheck)
				|| ((_template != null) && (_needCheck != _template.isNeedCheck())))
			pw.print(" needCheck='" + _needCheck + "'");

		if (((_template == null) && _hidden)
				|| ((_template != null) && (_hidden != _template.isHidden())))
			pw.print(" hidden='" + _hidden + "'");
		if (_controller != null)
			pw.print(" controller='" + LyString.getXmlStringStrict(_controller) + "'");

		if ((_template == null)
				|| (_state != _template.getState()))
		{
			if ((_state == STATE_NORMAL) && (_template != null))
				pw.print(" state='NORMAL'");
			else if (_state == STATE_ERROR)
				pw.print(" state='ERROR'");
			else if (_state == STATE_WARNING)
				pw.print(" state='WARNING'");
		}

		if (_field.getChildCount() == 0)
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");


		// Write sub-components
		if (_caption != null)
			((LyMobileCaption)_caption.getInternalView()).writeXml(pw);
		if (_fieldItem != null)
			((LyMobileObject)_fieldItem.getInternalView()).writeXml(pw);
		if (_toolBar != null)
			((LyMobileToolBar)_toolBar.getInternalView()).writeXml(pw);
		if (_relation != null)
			((LyMobileRelation)_relation.getInternalView()).writeXml(pw);

		pw.println("</FIELD>");

	}
	
	public void writeJson(PrintWriter pw, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		pw.println("{\"FIELD\": [");
		
		isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
		pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");
		
		if (!_editable || !_enabled)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"editable\": \"" + false + "\"}");
		}
		
		if (((_template == null) && !_optional)
				|| ((_template != null) && (_optional != _template.isOptional())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"optional\": \"" + _optional + "\"}");
		}
		

		isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
		pw.print("{\"needCheck\": \"" + _needCheck + "\"}");

		
		if (((_template == null) && _hidden)
				|| ((_template != null) && (_hidden != _template.isHidden())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"hidden\": \"" + _hidden + "\"}");
		}

		if (_controller != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"controller\": \"" + LyString.getJsonString(_controller) + "\"}");
		}

		if ((_template == null)
				|| (_state != _template.getState()))
		{
			if ((_state == STATE_NORMAL) && (_template != null))
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"state\": \"NORMAL\"}");
			}				
			else if (_state == STATE_ERROR)
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"state\": \"ERROR\"}");
			}				
			else if (_state == STATE_WARNING)
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"state\": \"WARNING\"}");
			}				
		}
		
		if (_field.getChildCount() == 0)
		{
			pw.println("]}");
			return;
		}

		// Write sub-components
		if (_caption != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			((LyMobileCaption)_caption.getInternalView()).writeJson(pw, sessionId);
		}			
		if (_fieldItem != null)
		{
			if(_fieldItem.getInternalView() instanceof LyMobileCommands)
			{
				((LyMobileCommands)_fieldItem.getInternalView()).writeJson(pw, isFirstElement, sessionId);
				if(isFirstElement)
				{
					isFirstElement = false;
				}
			}
			else
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				((LyMobileObject)_fieldItem.getInternalView()).writeJson(pw, sessionId);
			}
			
		}
		if (_toolBar != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			((LyMobileToolBar)_toolBar.getInternalView()).writeJson(pw, sessionId);
		}
		if (_relation != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			((LyMobileRelation)_relation.getInternalView()).writeJson(pw, sessionId);
		}		

		pw.println("]}");
	}


	public void init()
	{
		_id = _field.getId();
		_template = _field.getTemplate();
		_editable = _field.isEditable();
		_enabled = _field.isEnabled();
		_optional = _field.isOptional();
		_needCheck = _field.isNeedCheck();
		_hidden = _field.isHidden();
		_controller = _field.getController();
		_caption = _field.getCaption();
		_fieldItem = _field.getFieldItem();
		_toolBar = _field.getToolBar();
		_relation = _field.getRelation();
		_state = _field.getState();

	}


	public void setEditable(boolean editable)
	{
		// TODO Auto-generated method stub

	}


	public void setOptional(boolean optional)
	{
		// TODO Auto-generated method stub

	}


	public void setEnabled(boolean enabled)
	{
		// TODO Auto-generated method stub

	}


	public void setChecked(boolean checked)
	{
		// TODO Auto-generated method stub

	}


	public void setState(short state)
	{
		// TODO Auto-generated method stub

	}


	public void setMessage(String message)
	{
		// TODO Auto-generated method stub

	}


	public void setHidden(boolean hidden)
	{
		// TODO Auto-generated method stub

	}


	public void setToolBar(LyToolBar toolBar)
	{
		// TODO Auto-generated method stub

	}


	public int writeXml(PrintWriter pw, int nb)
	{
		writeXml(pw);
		return nb;
	}

}
