package leon.view.mobile;

import java.io.*;
import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;

public class LyMobileHeaders extends LyMobileTableObject
implements LyGraphicalHeaders
{
	/** The headers generic component. */
	protected LyHeaders                     _headers;

	protected String						_id;

	protected LyHeaders						_template;

	protected String						_label;

	protected LyHeader.Align				_align;

	protected String						_tooltip;

	protected String						_overview;

	protected String						_font;

	protected String						_sortableFont;

	protected String						_color;

	protected String						_fontColor;

	protected ArrayList<LyComponent>		_components;

	protected Hashtable<String, String> 	_datas;

	public LyMobileHeaders(LyHeaders headers)
	{
		super(headers);

		_headers = headers;

		_components = new ArrayList<LyComponent>();

		_font = getFontValue(getStringValue("MOBILE_HEADERS.font"));

		_color = getStringValue("MOBILE_HEADERS.color");

		_fontColor = getStringValue("MOBILE_HEADERS.fontColor");

	}

	@Override
	public void writeXml(PrintWriter pw)
	{
		writeXml(pw, false);

	}

	public void writeJson(PrintWriter pw, String sessionId)
	{
		writeJson(pw, false, sessionId);
	}

	public void writeXml(PrintWriter pw, boolean ok)
	{
		// Write tag with its attributes
		init();

		if(!ok)
		{
			pw.print("<HEADERS");
			/*if (_id != null)
				pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
			if (_template != null)
				pw.print(" template='" + _template.getId() + "'");
			if (((_template == null) && (_label != null))
					|| ((_template != null) && (_label != _template.getLabel())))
				pw.print(" label='" + LyString.getXmlStringStrict(_label) + "'");
			if ((_template == null)
				|| (_align != _template.getAlign()))
			{
				if (_align == LyHeader.Align.LEFT)
					pw.print(" align='LEFT'");
				else if (_align == LyHeader.Align.RIGHT)
					pw.print(" align='RIGHT'");
				else if ((_align == LyHeader.Align.CENTER) && (_template != null))
					pw.print(" align='CENTER'");
			}
			if (((_template == null) && (_tooltip != null))
					|| ((_template != null) && (_tooltip != _template.getTooltip())))
				pw.print(" tooltip='" + LyString.getXmlStringStrict(_tooltip) + "'");
			if (((_template == null) && (_overview != null))
					|| ((_template != null) && (_overview != _template.getOverview())))
				pw.print(" overview='" + LyString.getXmlStringStrict(_overview) + "'");*/
			if (((_template == null) && (_font != null))
					|| ((_template != null) && (_font != _template.getFont())))
				pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
			/*if (((_template == null) && (_sortableFont != getStringValue("HEADERS.sortableFont", "$ALL.sortableFont")))
					|| ((_template != null) && (_sortableFont != _template.getSortableFont())))
				pw.print(" sortableFont='" + LyString.getXmlStringStrict(_sortableFont) + "'");*/
			if (((_template == null) && (_color != null))
					|| ((_template != null) && (_color != _template.getColor())))
				pw.print(" color='" + getColorValue(_color) + "'");
			if (((_template == null) && (_fontColor != null))
					|| ((_template != null) && (_fontColor != _template.getFontColor())))
				pw.print(" fontColor='" + getColorValue(_fontColor) + "'");

			if ((_headers.getChildCount() == 0) && (_datas == null))
			{
				pw.println("/>");
				return;
			}
			else
				pw.println(">");
		}


		// Write sub-components
		LyComponent	components;
		int			i, n;

		n = (_components == null ? 0 : _components.size());
		for (i = 0; i < n; i++)
		{

			components = _components.get(i);
			if(components instanceof LyHeaders)
			{
				((LyMobileHeaders)components.getInternalView()).writeXml(pw, true);
			}
			else
			{
				((LyMobileObject)components.getInternalView()).writeXml(pw);
			}
		}
		if(!ok)
		{
			pw.println("</HEADERS>");			
		}


	}

	public void writeJson(PrintWriter pw, boolean ok, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		if(!ok)
		{
			pw.println("{\"HEADERS\": [");

			if (((_template == null) && (_font != null))
					|| ((_template != null) && (_font != _template.getFont())))
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
			}

			if (((_template == null) && (_color != null))
					|| ((_template != null) && (_color != _template.getColor())))
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"color\": \"" + getColorValue(_color) + "\"}");
			}

			if (((_template == null) && (_fontColor != null))
					|| ((_template != null) && (_fontColor != _template.getFontColor())))
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");
			}

			if ((_headers.getChildCount() == 0) && (_datas == null))
			{
				pw.println("]}");
				return;
			}
		}

		// Write sub-components
		LyComponent	components;
		int			i, n;

		n = (_components == null ? 0 : _components.size());
		
		if(LyMobileViewManager.MAX_HEADERS > 0 && LyMobileViewManager.MAX_HEADERS < n)
		{
			n = LyMobileViewManager.MAX_HEADERS;
		}
		
		
		for (i = 0; i < n; i++)
		{			
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			components = _components.get(i);
			if(components instanceof LyHeaders)
			{
				((LyMobileHeaders)components.getInternalView()).writeJson(pw, true, sessionId);
			}
			else
			{
				((LyMobileObject)components.getInternalView()).writeJson(pw, sessionId);
			}
		}
		if(!ok)
		{
			pw.println("]}");			
		}
	}

	@Override
	public void init()
	{

		_id = _headers.getId();
		_template = _headers.getTemplate();
		_label = _headers.getLabel();
		_align = _headers.getAlign();
		_tooltip = _headers.getTooltip();
		_overview = _headers.getOverview();
		_datas = _headers.getDatas();

		_components.clear();

		for(int i = 0; i < _headers.getComponentCount(); i++)
		{
			_components.add(_headers.getComponent(i));
		}

	}

	@Override
	public int writeXml(PrintWriter pw, int nb)
	{
		init();
		pw.print("<HEADERS");

		if ((_headers.getChildCount() == 0) && (_datas == null))
		{
			pw.println("/>");
			return nb;
		}
		else
			pw.println(">");

		// Write sub-components
		LyComponent	components;
		int			i, n;

		n = (_components == null ? 0 : _components.size());
		for (i = 0; i < n; i++)
		{
			if(nb > 0)
			{
				components = _components.get(i);
				nb = ((LyMobileTableObject)components.getInternalView()).writeXml(pw,nb);
			}

		}


		pw.println("</HEADERS>");
		return nb;

	}

}
