package leon.view.mobile;

import java.io.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;

public class LyMobileIcon extends LyMobileObject
	implements LyGraphicalIcon
{
	/** The icon generic component. */
    protected LyIcon			_icon;
    
	/**
	 * Identifier of the component.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyIcon			_template;

	/**
	 * Text to display with the image
	 */
	protected String			_label;

	/**
	 * Image
	 */
	protected String			_image;

	protected String						_rootUrl;
    
	public LyMobileIcon(LyIcon icon)
	{
		super(icon);
		
		_icon = icon;
		
		_rootUrl = ((LyMobileViewManager) getViewManager()).getRealRootURL();
	}

	
	public void writeXml(PrintWriter pw)
	{
		init();
		pw.print("<ICON");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		pw.print(" label='" + LyString.getXmlStringStrict(_label) + "'");
		
		pw.print(" image='" + getViewManager().getRootURL()+getImageUrl(_image) + "'");
		
		pw.println("/>");
	}
	
	public void writeJson(PrintWriter pw, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		pw.println("{\"ICON\": [");
		
		if (_id != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");
		}
		
		if(_label != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"label\": \"" + LyString.getJsonString(_label) + "\"}");			
		}
		String imageUrl = getImageUrl(_image);
		isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);	
		pw.print("{\"image\": \"" + 
				(imageUrl.startsWith(_rootUrl) ? imageUrl : _rootUrl+imageUrl)  + "\"}");
		
		pw.println("]}");
	}
	
	public void init()
	{
		_id = _icon.getId();

		_template = _icon.getTemplate();

		_label = _icon.getLabel();

		_image = _icon.getImage();
		
	}

	
	public void setImage(String image)
	{
		// TODO Auto-generated method stub
		
	}

}
