package leon.view.mobile;

import java.io.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;

public class LyMobileLabel extends LyMobileObject
	implements LyGraphicalLabel
{
	/** The label generic component. */
    protected LyLabel	_label;
    
    protected String 	_id;
    
    protected LyLabel 	_template;
    
    protected String	_lab;
    
    /**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;
	

	/**
	 * LABEL.highlightColor
	 */
	protected String			_highlightColor;
	
	protected int				_nRows;
    

	public LyMobileLabel(LyLabel label)
	{
		super(label);
        
        _label = label;
		
		_font = getFontValue(getStringValue("MOBILE_LABEL.font"));

		_color = getStringValue("MOBILE_LABEL.color");

		_fontColor = getStringValue("MOBILE_LABEL.fontColor");
		
		_highlightColor = getStringValue("MOBILE_LABEL.highlightColor");

	}

	
	public void writeXml(PrintWriter pw)
	{
		// Write tag with its attributes
		init();
		pw.print("<LABEL");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (((_template == null) && (_lab != null))
				|| ((_template != null) && (_lab != _template.getLabel())))
			pw.print(" label='" + LyString.getXmlStringStrict(_lab) + "'");
		
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlString(_font) + "'");
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + getColorValue(_color) + "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");
		if (((_template == null) && (_highlightColor != null))
				|| ((_template != null) && (_highlightColor != _template.getHighlightColor())))
			pw.print(" highlightColor='" + getColorValue(_highlightColor) + "'");
		pw.println("/>");
	}
	
	public void writeJson(PrintWriter pw, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		pw.println("{\"LABEL\": [");
		
		if (_id != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");
		}
			
		if (((_template == null) && (_lab != null))
				|| ((_template != null) && (_lab != _template.getLabel())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"label\": \"" + LyString.getJsonString(_lab) + "\"}");
		}


		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
		}
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"color\": \"" + getColorValue(_color) + "\"}");			
		}
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");				
		}
		if (((_template == null) && (_highlightColor != null))
				|| ((_template != null) && (_highlightColor != _template.getHighlightColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"highlightColor\": \"" + getColorValue(_highlightColor) + "\"}");				
		}
		
		if (_nRows > 0)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"nRows\": \"" + _nRows + "\"}");				
		}
		pw.println("]}");
	}

	
	public void init()
	{
		_id = _label.getId();
		_template = _label.getTemplate();
		if(_label.getLabel() != null && !_label.getLabel().equals(""))
		{
			_lab = getEnvironment().translate(_label.getLabel());
		}
		else
		{
			_lab = null;
		}	
		
		_nRows = _label.getNRows();
	}

	
	public void setLabel(String label)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void setImage(String image)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void setFont(String font)
	{
		// TODO Auto-generated method stub
		
	}

}
