package leon.view.mobile;

import java.io.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;

public class LyMobileNumber extends LyMobileObject
	implements LyGraphicalNumber
{

	/** The number generic component. */
    protected LyNumber			_number;
    
    protected String			_id;

	protected LyNumber			_template;

	protected String			_label;

	protected String			_format;
	
	/**
	 * Text font used on the component.
	 */
	protected String			_font;
	/**
	 * Background of the file selector when enabled.
	 */
	protected String			_enabledColor;

	/**
	 * Background of the file selector when disabled.
	 */
	protected String			_disabledColor;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;
	
	
	public LyMobileNumber(LyNumber number)
	{
		super(number);

		_number = number;		
		
		_font = getFontValue(getStringValue("MOBILE_NUMBER.font"));

		_enabledColor = getStringValue("MOBILE_NUMBER.enabledColor");

		_disabledColor = getStringValue("MOBILE_NUMBER.disabledColor");

		_fontColor = getStringValue("MOBILE_NUMBER.fontColor");

	}

    /**
     * Utility method to retrieves the value of an attribute from a key corresponding to an entry in a values file.
     */
    protected double getDoubleValue(String str, double defaultValue)
    {
        String  value = getValue(str);
        
        return (value == null  ? defaultValue : LyTools.doubleFromString(value));
    }
    
	
	public void writeXml(PrintWriter pw)
	{
		// Write tag with its attributes
		init();
		pw.print("<NUMBER");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
			pw.print(" label='" + LyString.getXmlStringStrict(_label) + "'");
		if (((_template == null) && (_format != null))
				|| ((_template != null) && (_format != _template.getFormat())))
			pw.print(" format='" + LyString.getXmlStringStrict(_format) + "'");		
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_enabledColor != null))
				|| ((_template != null) && (_enabledColor != _template.getEnabledColor())))
			pw.print(" enabledColor='" + getColorValue(_enabledColor) + "'");
		if (((_template == null) && (_disabledColor != null))
				|| ((_template != null) && (_disabledColor != _template.getDisabledColor())))
			pw.print(" disabledColor='" + getColorValue(_disabledColor) + "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");
		pw.println("/>");

	}
	
	public void writeJson(PrintWriter pw, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		pw.println("{\"NUMBER\": [");
		
		if (_id != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");
		}	
		
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"label\": \"" + LyString.getJsonString(_label) + "\"}");
		}	
		if (((_template == null) && (_format != null))
				|| ((_template != null) && (_format != _template.getFormat())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"format\": \"" + LyString.getJsonString(_format) + "\"}");
		}	
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
		}	
		if (((_template == null) && (_enabledColor != null))
				|| ((_template != null) && (_enabledColor != _template.getEnabledColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"enabledColor\": \"" + getColorValue(_enabledColor) + "\"}");
		}	
		if (((_template == null) && (_disabledColor != null))
				|| ((_template != null) && (_disabledColor != _template.getDisabledColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"disabledColor\": \"" + getColorValue(_disabledColor) + "\"}");
		}	
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");
		}	
		pw.println("]}");
	}

	
	public void init()
	{		
		_id = _number.getId();
		_template = _number.getTemplate();
		_label = _number.getLabel();
		_format = _number.getFormat();				
	}

	
	public void setLabel(String label)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void setFormat(String format)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void setVMin(double vMin)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void setVMax(double vMax)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void setPitch(double pitch)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void setDecimals(int decimals)
	{
		// TODO Auto-generated method stub
		
	}

}
