package leon.view.mobile;

import java.io.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;

/**
 * The LyMobileObject class implements root object for all Mobile components.
 */
public abstract class LyMobileObject extends LyGraphicalObject
implements LyMobileComponent
{
	/** Id for sep components. */
	public static final String    	SEP_ID = "_sep".intern();

	public static final String		WEB_COMPONENT_ID = "/component.htm?_componentId=".intern();
	public static final String		WEB_CONTROLLER_ID = "&_controllerId=".intern();
	public static final String		WEB_SESSION_ID = "&_sessionId=".intern();

	/**
	 * Utility method to retrieves the value of an attribute from a key corresponding to an entry in a values file.
	 */
	protected int getIntValue(String str, int defaultValue)
	{
		String  value = getValue(str);

		return (value == null  ? defaultValue : LyTools.intFromString(value));
	}

	/**
	 * Utility method to retrieves the value of an attribute from a key corresponding to an entry in a values file.
	 */
	protected float getFloatValue(String str, float defaultValue)
	{
		String  value = getValue(str);

		return (value == null  ? defaultValue : LyTools.floatFromString(value));
	}

	/**
	 * Utility method to retrieves the value of an attribute from a key corresponding to an entry in a values file.
	 */
	protected double getDoubleValue(String str, double defaultValue)
	{
		String  value = getValue(str);

		return (value == null  ? defaultValue : LyTools.doubleFromString(value));
	}

	/**
	 * Utility method to retrieves the value of an attribute from a key corresponding to an entry in a values file.
	 */
	protected String getValue(String str)
	{        
		String  value = getStringValue(str);

		if (value == null)
			return null;

		LyEnvironment   env = getEnvironment();

		if (env == null)
			return str;

		value = env.getResource(value);
		return value;
	}

	/**
	 * Utility method to retrieves the value of an attribute from a key corresponding to an entry in a values file.
	 */
	protected String getStringValue(String str)
	{
		return getStringValue(str, null);
	}

	/**
	 * Boolean used for activating raw XML writing of components
	 * This boolean inhibits attributes initialization from properties and initFromModel.
	 * Reserved for internal use.
	 */
	public static boolean   REWRITING_MODE = false;

	/**
	 * Utility method to retrieves the value of an attribute from a key corresponding to an entry in a values file.
	 */ 
	protected String getStringValue(String str, String defaultValue)
	{
		if (REWRITING_MODE)
		{
			if ((defaultValue != null)
					&& (defaultValue.length() > 0)
					&& (defaultValue.charAt(0) == '$'))
				return null;

			return defaultValue;
		}

		LyEnvironment   env = getEnvironment();
		String          value = (env == null ? str : env.getViewAttValue(str));

		if (value == str)
			value = defaultValue;
		if ((value != null)
				&& (value.length() > 0)
				&& (value.charAt(0) == '$'))
			value = getStringValue(value.substring(1));
		if (value == null)
			value = env.getString(str);
		if (value == str) // if the value is still equals to str, put value to null...
			value = null;

		return value;
	}

	public LyMobileObject(LyComponent viewComponent)
	{
		super(viewComponent);
		// TODO Auto-generated constructor stub
	}

	public String getUrl(String image)
	{
		String img = LyString.getXmlStringStrict(image);
		String root = ((LyMobileViewManager) getEnvironment().getViewManager(true)).getRealRootURL();
		return root.substring(0,root.lastIndexOf('/'))+
				getEnvironment().getImageUrl(img);		
	}

	public String getImageUrl(String image)
	{
		String imageUrl;
		String docRoot = expandUrl(getEnv("LY_APP_DOC"));
		if (image.startsWith(docRoot))
			imageUrl = expandUrl(image);
		else if (image.startsWith("file"))
			imageUrl = LyString.encodeFileURL(image);
		else if (image.startsWith("http") || image.startsWith("ftp"))
			imageUrl = image;
		else
			imageUrl = getEnvironment().getImageUrl(image);
		
		/*if(imageUrl.indexOf("download.htm?_") != -1)
		{
			imageUrl = getDirForDownloadImage(imageUrl);
		}*/

		int index = imageUrl.indexOf(' ');
		if(index != -1)
		{
			imageUrl = imageUrl.substring(0, index);
		}
		return imageUrl;
	}
	
	public String getDirForDownloadImage(String url)
	{
		String fileId;
		
		if(url.indexOf("=") == -1)
			return null;
		if(url.indexOf("(") == -1)
		{
			fileId = url.substring(url.indexOf("=")+1);
		}
		else
		{
			fileId = url.substring(url.indexOf("=")+1,
					url.lastIndexOf("("));
		}
				
		return getViewManager().getTemporaryDir()+"\\"+fileId;
	}

	/**
	 * Replaces within a given String some $properties$ found in the system environment. This method
	 * is looking for macros between '$' and replaces these macros by a property if any (such as
	 * $LY_APP_DIR$/data/test.html).
	 *
	 * @param  text The original string, before any substitution.
	 * @return The string after all substitutions.
	 */
	public final String expandUrl(String text)
	{
		return getEnvironment().getString().expandUrl(text);
	}


	/**
	 * Utility method to retrieves the value of an attribute from a key corresponding to an entry in a values file.
	 */ 
	protected String getFontValue(String str)
	{
		if(str == null)
		{
			return null;
		}

		LyFont font = getEnvironment().getLyFont(str);
		if(str.equals(font.getFontName()))
		{
			return null;
		}
		return font.getValue();
	}

	/**
	 * Utility method to retrieves the value of an attribute from a key corresponding to an entry in a values file.
	 */ 
	protected String getColorValue(String str)
	{
		if(str == null)
		{
			return null;
		}

		LyColor color = getEnvironment().getLyColor(str);

		if(color == null)
		{
			String[] tab;
			int		 red = 0, green = 0, blue = 0;
			tab = LyString.stringToList(str, ' ');
			if (tab.length != 3)
				return null;

			try
			{
				red = Integer.parseInt(tab[0]);
				green = Integer.parseInt(tab[1]);
				blue = Integer.parseInt(tab[2]);
			}
			catch (NumberFormatException e)
			{
			}

			new LyColor(red, green, blue);

			return LyString.getXmlStringStrict(LyColors.getHexString(red, green, blue));
		}
		else
		{
			return LyString.getXmlStringStrict(LyColors.getHexString(color));
		}
	}
	

	
	public boolean printJsonAttributeSeparator(boolean first, PrintWriter pw)
	{
		if(!first)
		{
			pw.println(",");
			return first;
		}
		
		return false;
	}
	
	
	public void createWebview(PrintWriter pw, String sessionId)
	{
		LyComponent component = getViewComponent();
		String path = getViewManager().getRootURL()+WEB_COMPONENT_ID+component.getId()
				+WEB_CONTROLLER_ID+component.getRootController()+WEB_SESSION_ID+sessionId;
		
		pw.println("{\"WEBVIEW\": [");
		pw.print("{\"link\": \"" + path + "\"}");
		pw.println("]}");
	}
	


}
