package leon.view.mobile;

import java.io.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;

public class LyMobileOption extends LyMobileObject
	implements LyGraphicalOption
{
	/** The option generic component. */
    protected LyOption			_option;
   
    protected String			_id;

	protected LyOption			_template;

	protected String			_label;

	protected boolean			_checked;

	protected boolean			_enabled;
    /**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;
    
	public LyMobileOption(LyOption option)
	{
		super(option);
		
		_option = option;
		
		_font = getFontValue(getStringValue("MOBILE_OPTION.font"));

		_color = getStringValue("MOBILE_OPTION.color");

		_fontColor = getStringValue("MOBILE_OPTION.fontColor");		
	}

	
	public void writeXml(PrintWriter pw)
	{
		// Write tag with its attributes
		init();
		pw.print("<OPTION");
		pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
			pw.print(" label='" + getString(LyString.getXmlStringStrict(_label)) + "'");		
		if (((_template == null) && _checked)
				|| ((_template != null) && (_checked != _template.isChecked())))
			pw.print(" checked='" + _checked + "'");
		if (((_template == null) && !_enabled)
				|| ((_template != null) && (_enabled != _template.isEnabled())))
			pw.print(" enabled='" + _enabled + "'");
		
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + getColorValue(_color) + "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");
		pw.println("/>");

	}
	
	public void writeJson(PrintWriter pw, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		pw.println("{\"OPTION\": [");

		isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
		pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");

		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"label\": \"" + getString(LyString.getJsonString(_label)) + "\"}");
		}	
		if (((_template == null) && _checked)
				|| ((_template != null) && (_checked != _template.isChecked())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"checked\": \"" + _checked + "\"}");
		}
		if (((_template == null) && !_enabled)
				|| ((_template != null) && (_enabled != _template.isEnabled())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"enabled\": \"" + _enabled + "\"}");
		}		
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
		}
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"color\": \"" + getColorValue(_color) + "\"}");
		}
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");
		}
		pw.println("]}");

	}

	
	public void init()
	{
		_id = _option.getId();
		_template = _option.getTemplate();
		_label = _option.getLabel();
		_checked = _option.isChecked();
		_enabled = _option.isEnabled();	
	}

	
	public void setChecked(boolean checked)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void setEnabled(boolean enabled)
	{
		// TODO Auto-generated method stub
		
	}

}
