package leon.view.mobile;

import java.io.*;
import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;

public class LyMobileRelation extends LyMobileObject
	implements LyGraphicalRelation
{
	/** The relation generic component. */
	protected LyRelation 					_relation;
	
	protected String 						_id;
	
	protected LyRelation 					_template;
	
	protected boolean 						_multiple;
	
	protected ArrayList<LyLink> 			_links;
	
	/**
	 * Text font used on the component.
	 */
	protected String			_font;
	/**
	 * Background of the file selector when enabled.
	 */
	protected String			_enabledColor;

	/**
	 * Background of the file selector when disabled.
	 */
	protected String			_disabledColor;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	public LyMobileRelation(LyRelation relation)
	{
		super(relation);

		_relation = relation;
		
		_links = new ArrayList<LyLink>();
		
		_font = getFontValue(getStringValue("MOBILE_RELATION.font"));

		_enabledColor = getStringValue("MOBILE_RELATION.enabledColor");

		_disabledColor = getStringValue("MOBILE_RELATION.disabledColor");

		_fontColor = getStringValue("MOBILE_RELATION.fontColor");
		
		
	}

	
	public void writeXml(PrintWriter pw)
	{		
		// Write tag with its attributes
		init();
		pw.print("<RELATION");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");

		if (((_template == null) && _multiple)
				|| ((_template != null) && (_multiple != _template.isMultiple())))
			pw.print(" multiple='" + _multiple + "'");
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_enabledColor != null))
				|| ((_template != null) && (_enabledColor != _template.getEnabledColor())))
			pw.print(" enabledColor='" + getColorValue(_enabledColor) + "'");
		if (((_template == null) && (_disabledColor != null))
				|| ((_template != null) && (_disabledColor != _template.getDisabledColor())))
			pw.print(" disabledColor='" + getColorValue(_disabledColor) + "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");

		if (_relation.getChildCount() == 0)
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		LyComponent	links;
		int			i, n;

		n = (_links == null ? 0 : _links.size());
		for (i = 0; i < n; i++)
		{
			links = _links.get(i);
			((LyMobileLink)links.getInternalView()).writeXml(pw);
		}

		pw.println("</RELATION>");

	}
	
	public void writeJson(PrintWriter pw, String sessionId)
	{		
		boolean isFirstElement = true;
		init();
		pw.println("{\"RELATION\": [");
		
		if (_id != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");
		}

		if (((_template == null) && _multiple)
				|| ((_template != null) && (_multiple != _template.isMultiple())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"multiple\": \"" + _multiple + "\"}");
		}
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
		}
		if (((_template == null) && (_enabledColor != null))
				|| ((_template != null) && (_enabledColor != _template.getEnabledColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"enabledColor\": \"" + getColorValue(_enabledColor) + "\"}");
		}
		if (((_template == null) && (_disabledColor != null))
				|| ((_template != null) && (_disabledColor != _template.getDisabledColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"disabledColor\": \"" + getColorValue(_disabledColor) + "\"}");
		}
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");
		}

		if (_relation.getChildCount() == 0)
		{
			pw.println("]}");
			return;
		}

		// Write sub-components
		LyComponent	links;
		int			i, n;

		n = (_links == null ? 0 : _links.size());
		for (i = 0; i < n; i++)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			links = _links.get(i);
			((LyMobileLink)links.getInternalView()).writeJson(pw, sessionId);
		}

		pw.println("]}");

	}

	
	public void init()
	{
		_id = _relation.getId();
		_template = _relation.getTemplate();
		_multiple = _relation.isMultiple();
		
		_links.clear();

		for(int j = 0; j < _relation.getLinkCount(); j++)
		{
			_links.add(_relation.getLink(j));
		}
		
		_font = _relation.getFont();

		_enabledColor = _relation.getEnabledColor();

		_disabledColor = _relation.getDisabledColor();

		_fontColor = _relation.getFontColor();
		
	}

	
	public void addLink(LyLink links, int pos)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void removeLink(LyLink links)
	{
		// TODO Auto-generated method stub
		
	}

}
