package leon.view.mobile;

import java.io.*;
import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;

public class LyMobileRow extends LyMobileTableObject
implements LyGraphicalRow
{
	/** The row generic component. */
	protected LyRow                       	_row;

	protected String						_id;

	protected LyRow							_template;

	protected String						_image;

	protected String						_label;

	protected String						_tooltip;

	protected ArrayList<LyComponent>		_components;

	protected LyRows						_rows;

	protected String						_rootUrl;

	protected String						_index;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;


	public LyMobileRow(LyRow row)
	{
		super(row);

		_row = row;

		_components = new ArrayList<LyComponent>();
		_rootUrl = ((LyMobileViewManager) getViewManager()).getRealRootURL();		
		
		_font = getFontValue(getStringValue("MOBILE_ROW.font"));

		_color = getStringValue("MOBILE_ROW.color");

		_fontColor = getStringValue("MOBILE_ROW.fontColor");		

	}


	public void writeXml(PrintWriter pw)
	{
		init();
		pw.print("<ROW");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlStringStrict(_id)+ "'");
		if (((_template == null) && (_image != getStringValue("ROW.image", "$ALL.image")))
				|| ((_template != null) && (_image != _template.getImage())))
			pw.print(" image='" + _rootUrl+getEnvironment().getImageUrl(_image) + "'");
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
			pw.print(" label='" + LyString.getXmlStringStrict(_label) + "'");
		if (((_template == null) && (_tooltip != null))
				|| ((_template != null) && (_tooltip != _template.getTooltip())))
			pw.print(" tooltip='" + LyString.getXmlStringStrict(_tooltip) + "'");
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + getColorValue(_color) + "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");

		if (_row.getChildCount() == 0)
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		LyComponent	components;
		int			i, n;

		n = (_components == null ? 0 : _components.size());
		for (i = 0; i < n; i++)
		{
			components = _components.get(i);
			((LyMobileObject)components.getInternalView()).writeXml(pw);
		}

		/*if (_rows != null)
			((LyMobileRows)_rows.getInternalView()).writeXml(pw);*/

		pw.println("</ROW>");

	}
	
	public void writeJson(PrintWriter pw, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		
		if (_rows != null)
		{
			((LyMobileRows)_rows.getInternalView()).writeJson(pw, sessionId);
			return;
		}
		
		pw.println("{\"ROW\": [");
		
		if (_id != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"id\": \"" + LyString.getJsonString(_id).replace("\\", "\\\\") + "\"}");
		}
		
		if (_index != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"index\": \"" + LyString.getJsonString(_index) + "\"}");
		}
		
		if (((_template == null) && (_image != getStringValue("ROW.image", "$ALL.image")))
				|| ((_template != null) && (_image != _template.getImage())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"image\": \"" + _rootUrl+getEnvironment().getImageUrl(_image) + "\"}");
		}
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"label\": \"" + LyString.getJsonString(_label) + "\"}");
		}
		if (((_template == null) && (_tooltip != null))
				|| ((_template != null) && (_tooltip != _template.getTooltip())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"tooltip\": \"" + LyString.getJsonString(_tooltip) + "\"}");
		}
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
		}
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"color\": \"" + getColorValue(_color) + "\"}");
		}
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");
		}

		if (_row.getChildCount() == 0)
		{
			pw.println("]}");
			return;
		}

		// Write sub-components
		LyComponent	components;
		int			i, n;

		n = (_components == null ? 0 : _components.size());
		
		if(LyMobileViewManager.MAX_HEADERS > 0 && LyMobileViewManager.MAX_HEADERS < n)
		{
			n = LyMobileViewManager.MAX_HEADERS;
		}
		
		int depart = 0;
		
		if(_image != null && !_image.equals(""))
		{
			n++;
			depart++;
		}
		
		for (i = depart; i < n; i++)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			components = _components.get(i);
			((LyMobileObject)components.getInternalView()).writeJson(pw, sessionId);
		}

		pw.println("]}");

	}


	public void init()
	{
		_id = _row.getId();
		_template = _row.getTemplate();
		_image = _row.getImage();
		_label = _row.getLabel();
		_tooltip = _row.getTooltip();
		_index = _row.getIndex();

		_components.clear();

		for(int i = 0; i < _row.getComponentCount(); i++)
		{
			_components.add(_row.getComponent(i));
		}
		_rows = _row.getRows();

	}


	public void setOpen(boolean open)
	{
		// TODO Auto-generated method stub

	}


	public void setImage(String image)
	{
		// TODO Auto-generated method stub

	}


	public void setLabel(String label)
	{
		// TODO Auto-generated method stub

	}

	public void setIndex(String index)
	{
		// TODO Auto-generated method stub

	}

	public void setTooltip(String tooltip)
	{
		// TODO Auto-generated method stub

	}


	public void setHidden(boolean hidden)
	{
		// TODO Auto-generated method stub

	}


	public void setBorderHeight(int borderHeight)
	{
		// TODO Auto-generated method stub

	}


	public void setBorderColor(String borderColor)
	{
		// TODO Auto-generated method stub

	}


	public void addComponent(LyComponent components, int pos)
	{
		// TODO Auto-generated method stub

	}


	public void removeComponent(LyComponent components)
	{
		// TODO Auto-generated method stub

	}


	public int writeXml(PrintWriter pw, int nb)
	{
		init();
		pw.print("<ROW");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");

		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
			pw.print(" label='" + LyString.getXmlStringStrict(_label) + "'");
		if (((_template == null) && (_tooltip != null))
				|| ((_template != null) && (_tooltip != _template.getTooltip())))
			pw.print(" tooltip='" + LyString.getXmlStringStrict(_tooltip) + "'");
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + getColorValue(_color) + "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");


		if (_row.getChildCount() == 0)
		{
			pw.println("/>");
			return nb;
		}
		else
			pw.println(">");

		// Write sub-components
		LyComponent	components;
		int			i, n;

		n = (_components == null ? 0 : _components.size());
		for (i = 0; i < n; i++)
		{
			if(nb > 0)
			{
				components = _components.get(i);
				nb = ((LyMobileTableObject)components.getInternalView()).writeXml(pw, nb);
			}

		}

		if (_rows != null)
			((LyMobileRows)_rows.getInternalView()).writeXml(pw, nb);

		pw.println("</ROW>");
		return nb;
	}

	public void setColor(String color)
	{
		// TODO Auto-generated method stub

	}

}
