package leon.view.mobile;

import java.io.*;
import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;

public class LyMobileSelectBar extends LyMobileObject
	implements LyGraphicalSelectBar
{
	/** The selectBar generic component. */
    protected LySelectBar				_selectBar;
    
    protected ArrayList<LyComponent> 	_components;

    protected LyFormField 				_formField;
    
    protected String					_id;

	protected LySelectBar				_template;

	protected String					_label;

	protected String					_controller;
	
	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;
    
	public LyMobileSelectBar(LySelectBar selectBar)
	{
        super(selectBar);
        
        _selectBar = selectBar;
        
        _components = new ArrayList<LyComponent>();
		
		_font = getFontValue(getStringValue("MOBILE_SELECTBAR.font"));

		_color = getStringValue("MOBILE_SELECTBAR.color");

		_fontColor = getStringValue("MOBILE_SELECTBAR.fontColor");	

	}
	
	
	public void writeXml(PrintWriter pw)
	{   
		// Write sub-components
		init();
		pw.print("<SELECTBAR");
		pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
			pw.print(" label='" + LyString.getXmlStringStrict(_label) + "'");

		if (_controller != null)
			pw.print(" controller='" + LyString.getXmlStringStrict(_controller) + "'");
		
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_color != null))
				||((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + getColorValue(_color)+ "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");

		LyComponent	components;
		int	i, n, cpt, page;
		cpt = 0;
		page = 0;
		n = (_components == null ? 0 : _components.size());
		for (i = 0; i < n; i++)
		{
			components = _components.get(i);
			if(components instanceof LySelectItem)
			{
				if(((LySelectItem) components).isSelected())
				{
					page = i+1;
				}
				cpt++;
			}			
		}
		pw.print(" maxPage='" + cpt + "'");
		pw.print(" actualPage='" + page + "'");
		if ((_selectBar.getChildCount() == 0) && (_selectBar.getDatas() == null))
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		
		/*
		if (_formField != null)
			((LyMobileFormField)_formField.getInternalView()).writeXml(pw);*/
		
		pw.println("</SELECTBAR>");

	}
	
	public void writeJson(PrintWriter pw, String sessionId)
	{   
		// Write sub-components
		boolean isFirstElement = true;
		init();
		pw.println("{\"SELECTBAR\": [");
		
		if (_id != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");
		}
			
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"label\": \"" + LyString.getJsonString(_label) + "\"}");
		}

		if (_controller != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"controller\": \"" + LyString.getJsonString(_controller) + "\"}");
		}
		
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
		}
		if (((_template == null) && (_color != null))
				||((_template != null) && (_color != _template.getColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"color\": \"" + getColorValue(_color) + "\"}");
		}
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");
		}

		LyComponent	components;
		int	i, n, cpt, page;
		cpt = 0;
		page = 0;
		n = (_components == null ? 0 : _components.size());
		for (i = 0; i < n; i++)
		{
			
			components = _components.get(i);
			if(components instanceof LySelectItem)
			{
				if(((LySelectItem) components).isSelected())
				{
					page = i+1;
				}
				cpt++;
			}			
		}
		isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
		pw.print("{\"maxPage\": \"" + cpt + "\"}");
		isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
		pw.print("{\"actualPage\": \"" + page + "\"}");
		if ((_selectBar.getChildCount() == 0) && (_selectBar.getDatas() == null))
		{
			pw.println("]}");
			return;
		}

		
		/*
		if (_formField != null)
			((LyMobileFormField)_formField.getInternalView()).writeXml(pw);*/
		
		pw.println("]}");

	}

	
	public void init()
	{
		_components.clear();
		
		for(int j = 0; j < _selectBar.getComponentCount(); j++)
		{
			_components.add(_selectBar.getComponent(j));
		}
		
		_formField = _selectBar.getFormField();
		
	    _id = _selectBar.getId();

		_template = _selectBar.getTemplate();

		_label = _selectBar.getLabel();

		_controller = _selectBar.getController();
		
	}

	
	public void setLabel(String label)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void setStartItem(int startItem)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void addComponent(LyComponent components, int pos)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void removeComponent(LyComponent components)
	{
		// TODO Auto-generated method stub
		
	}

}
