package leon.view.mobile;

import java.io.*;
import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.LySplitter.ActivePart;
import leon.view.LySplitter.Style;
import leon.view.graphic.*;

public class LyMobileSplitter extends LyMobileObject
implements LyGraphicalSplitter
{
	/** The splitter generic component. */
	protected LySplitter		_splitter;

	/**
	 * Identifier of the component.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LySplitter			_template;

	/**
	 * Indicates whether the panels should be placed horizontally or vertically.
	 * HORIZONTAL: Indicates that the panels are placed horizontally and that the 
	 * separation bar is vertical.
	 * VERTICAL: Indicates that the panels are placed vertically and that the 
	 * separation bar is horizontal.
	 */
	protected Style				_style;

	protected ActivePart			_activePart;

	/* Sub components */

	/**
	 * List of additional panels of the container.
	 * At least one is needed in order to have at least two panels.
	 */
	protected ArrayList<LyPanel>		_panels;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	public LyMobileSplitter(LySplitter splitter)
	{
		super(splitter);

		_splitter = splitter;

		_panels = new ArrayList<LyPanel>();

		_color = getStringValue("MOBILE_SPLITTER.color");
	}


	public void writeXml(PrintWriter pw)
	{
		init();
		// Write tag with its attributes
		pw.print("<SPLITTER");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlString(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if ((_template == null)
				|| (_activePart != _template.getActivePart()))
		{
			if ((_activePart == ActivePart.SECOND))
				pw.print(" selected='2'");
			else if (_activePart == ActivePart.FIRST && _template != null)
				pw.print(" selected='1'");
		}
		if ((_template == null)
				|| (_style != _template.getStyle()))
		{
			if ((_style == Style.HORIZONTAL) && (_template != null))
				pw.print(" style='HORIZONTAL'");
			else if (_style == Style.VERTICAL)
				pw.print(" style='VERTICAL'");
		}

		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + getColorValue(_color) + "'");

		if (_splitter.getChildCount() == 0)
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		LyComponent	panels;
		int			i, n, start;
		if(_activePart == ActivePart.SECOND)
		{
			start = 1;
			n = 2;
		}
		else
		{
			start = 0;
			n = 1;
		}

		if(_panels == null)
		{
			n = start;
		}
		//n = (_panels == null ? 0 : _panels.size());
		for (i = start; i < n; i++)
		{
			panels = _panels.get(i);
			((LyMobileObject) panels.getInternalView()).writeXml(pw);
		}

		pw.println("</SPLITTER>");
	}

	public void writeJson(PrintWriter pw, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		pw.println("{\"SPLITTER\": [");

		if (_id != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");
		}

		if ((_template == null)
				|| (_activePart != _template.getActivePart()))
		{
			if ((_activePart == ActivePart.SECOND))
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"selected\": \"2\"}");				
			}
			else if (_activePart == ActivePart.FIRST && _template != null)
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				pw.print("{\"selected\": \"1\"}");				
			}
		}

		if (_style == Style.VERTICAL)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"style\": \"VERTICAL\"}");				
		}
		else 
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"style\": \"HORIZONTAL\"}");
		}

		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"color\": \"" + getColorValue(_color) + "\"}");
		}

		if (_splitter.getChildCount() == 0)
		{
			pw.println("]}");
			return;
		}

		// Write sub-components
		LyComponent	panels;
		int			i, n, start;
		if(_activePart == ActivePart.SECOND)
		{
			start = 1;
			n = 2;
		}
		else
		{
			start = 0;
			n = 1;
		}

		if(_panels == null)
		{
			n = start;
		}
		//n = (_panels == null ? 0 : _panels.size());
		for (i = start; i < n; i++)
		{
			panels = _panels.get(i);
			if(panels.getInternalView() instanceof LyMobileCommands)
			{
				((LyMobileCommands)panels.getInternalView()).writeJson(pw, isFirstElement, sessionId);
				if(isFirstElement)
				{
					isFirstElement = false;
				}
			}
			else
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				((LyMobileObject) panels.getInternalView()).writeJson(pw, sessionId);
			}
		}

		pw.println("]}");
	}


	public void init()
	{
		_id = _splitter.getId();

		_template = _splitter.getTemplate();

		_style = _splitter.getStyle();

		_activePart = _splitter.getActivePart();

		_panels.clear();

		for (int i = 0; i < _splitter.getPanelCount(); i++)
		{
			_panels.add(_splitter.getPanel(i));
		}		
	}


	public void setSizes(String sizes)
	{
		// TODO Auto-generated method stub

	}


	public void setSelected(boolean status)
	{
		// TODO Auto-generated method stub

	}


	public void setActivePart(ActivePart activePart)
	{
		// TODO Auto-generated method stub

	}


}
