package leon.view.mobile;

import java.io.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;

public class LyMobileTab extends LyMobileObject
	implements LyGraphicalTab
{
	/** The tab generic component. */
    protected LyTab						_tab;
	protected String					_id;
	protected LyTab						_template;
	protected String					_label;
	protected LyPanel					_panel;
	
	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;
    
	public LyMobileTab(LyTab tab)
	{
		super(tab);
		
		_tab = tab;
		
		_font = getFontValue(getStringValue("MOBILE_TAB.font"));

		_color = getStringValue("MOBILE_TAB.color");

		_fontColor = getStringValue("MOBILE_TAB.fontColor");	
		
	}

	
	public void writeXml(PrintWriter pw, boolean print, int index)
	{
		// Write tag with its attributes
		init();
		pw.print("<TAB");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
		{
			pw.print(" label='" + getString(LyString.getXmlStringStrict(_label)) + "'");
			pw.print(" labelNet='" + LyString.getXmlStringStrict(_label) + "'");
		}
		pw.print(" index='" + index + "'");	
		
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + getColorValue(_color) + "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");

		
		if (_tab.getChildCount() == 0)
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		if (_panel != null && print)
			((LyMobileObject)_panel.getInternalView()).writeXml(pw);

		pw.println("</TAB>");

	}
	
	public void writeJson(PrintWriter pw, boolean print, int index, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		pw.println("{\"TAB\": [");

		if (_id != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");
		}
		
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"label\": \"" + getString(LyString.getJsonString(_label)) + "\"}");
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"labelNet\": \"" + LyString.getJsonString(_label) + "\"}");
		}
		
		isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
		pw.print("{\"index\": \"" + index + "\"}");
		
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
		}
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"color\": \"" + getColorValue(_color) + "\"}");
		}
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");
		}

		
		if (_tab.getChildCount() == 0)
		{
			pw.println("]}");
			return;
		}

		// Write sub-components
		if (_panel != null && print)
		{
			if(_panel.getInternalView() instanceof LyMobileCommands)
			{
				((LyMobileCommands)_panel.getInternalView()).writeJson(pw, isFirstElement, sessionId);
				if(isFirstElement)
				{
					isFirstElement = false;
				}
			}
			else
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				LyViewComponent component = _panel.getInternalView();
				if(component instanceof LyWebObject)
				{
					((LyWebObject)_panel.getInternalView(true)).writeJson(pw, sessionId);
				}
				else
				{
					((LyMobileObject)_panel.getInternalView(true)).writeJson(pw, sessionId);
				}
			}			
		}		

		pw.println("]}");

	}


	
	public void init()
	{
		_id = _tab.getId();
		_template = _tab.getTemplate();
		_label = _tab.getLabel();
		_panel = _tab.getPanel();
		
	}

	
	public void setEnabled(boolean enabled)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void setColor(String color)
	{
		// TODO Auto-generated method stub
		
	}


	public String get_id()
	{
		return _id;
	}


	public String get_label()
	{
		return _label;
	}


	public void writeXml(PrintWriter pw)
	{
		writeXml(pw, true, 0);		
	}

	public void writeJson(PrintWriter pw, String sessionId)
	{
		writeJson(pw, true, 0, sessionId);		
	}

}
