package leon.view.mobile;

import java.io.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;

public class LyMobileTable extends LyMobileObject
implements LyGraphicalTable
{
	/** The Table generic component. */
	protected LyTable						_table;

	protected String						_id;

	protected LyTable						_template;

	protected LyHeaders						_headers;

	protected LyRows						_rows;

	protected LyFooters						_footers;
	/**
	 * Reference of the controller that manages the table (i.e. handles user demands 
	 * and refresh content).
	 */
	protected String			_controller;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	public LyMobileTable(LyTable table)
	{
		super(table);

		_table = table; 

		_font = getFontValue(getStringValue("MOBILE_TABLE.font"));

		_color = getStringValue("MOBILE_TABLE.color");

		_fontColor = getStringValue("MOBILE_TABLE.fontColor");	
	}


	public void writeXml(PrintWriter pw)
	{
		// Write tag with its attributes
		init();
		pw.print("<TABLE");
		pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + getColorValue(_color) + "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");

		if (_controller != null)
			pw.print(" controller='" + LyString.getXmlString(_controller) + "'");


		if (_table.getChildCount() == 0)
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		if (_headers != null)
			//((LyMobileHeaders)_headers.getInternalView()).writeXml(pw,2);
			((LyMobileHeaders)_headers.getInternalView()).writeXml(pw);
		if (_rows != null)
			//((LyMobileRows)_rows.getInternalView()).writeXml(pw,2);
			((LyMobileRows)_rows.getInternalView()).writeXml(pw);
		if (_footers != null)
			//((LyMobileFooters)_footers.getInternalView()).writeXml(pw,2);
			((LyMobileFooters)_footers.getInternalView()).writeXml(pw);

		pw.println("</TABLE>");

	}

	public void writeJson(PrintWriter pw, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		pw.println("{\"TABLE\": [");

		isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
		pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");


		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
		}
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"color\": \"" + getColorValue(_color) + "\"}");
		}
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");
		}

		if (_controller != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"controller\": \"" + LyString.getJsonString(_controller) + "\"}");
		}


		if (_table.getChildCount() == 0)
		{
			pw.println("]}");
			return;
		}

		// Write sub-components
		if (_headers != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			((LyMobileHeaders)_headers.getInternalView()).writeJson(pw, sessionId);			
		}
		if (_rows != null)
		{
			((LyMobileRows)_rows.getInternalView()).writeJson(pw, sessionId, true);			
		}
		/*if (_footers != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			((LyMobileFooters)_footers.getInternalView()).writeJson(pw);			
		}*/
		pw.println("]}");
	}


	public void init()
	{
		_id = _table.getId();		
		_template = _table.getTemplate();
		_headers = _table.getHeaders();
		_rows = _table.getRows();
		_footers = _table.getFooters();
		_controller = _table.getController();
	}


	public void setEnabled(boolean enabled)
	{
		// TODO Auto-generated method stub

	}


	public void setHeaders(LyHeaders headers)
	{
		// TODO Auto-generated method stub

	}


	public void setFooters(LyFooters footers)
	{
		// TODO Auto-generated method stub

	}


	public void setNRows(int nRows)
	{
		// TODO Auto-generated method stub

	}

}
