package leon.view.mobile;

import java.io.*;
import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;

public class LyMobileTabs extends LyMobileObject
	implements LyGraphicalTabs
{
	/** The tabs generic component. */
    protected LyTabs						_tabs;
    
	protected String						_id;
	
	protected LyTabs						_template;
	
	protected String						_selectedTab;
	
	protected ArrayList<LyTab>				_tab;
	
	/**
	 * Text font used on the component.
	 */
	protected String			_font;

	/**
	 * Background color of the component.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	/**
	 * Color of the selected tab.
	 */
	protected String			_selectedColor;
    
	public LyMobileTabs(LyTabs tabs)
	{
		super(tabs);
		
		_tabs = tabs;
		
		_tab = new ArrayList<LyTab>();
		
		_font = getFontValue(getStringValue("MOBILE_TABS.font"));

		_color = getStringValue("MOBILE_TABS.color");

		_fontColor = getStringValue("MOBILE_TABS.fontColor");	
		
		_selectedColor = getStringValue("MOBILE_TABS.selectedColor");	
		
	}

	
	public void writeXml(PrintWriter pw)
	{
		// Write tag with its attributes
		init();
		pw.print("<TABS");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (((_template == null) && (_selectedTab != null))
				|| ((_template != null) && (_selectedTab != _template.getSelectedTab())))
			pw.print(" selectedTab='" + LyString.getXmlStringStrict(_selectedTab) + "'");
		
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + getColorValue(_color) + "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");
		
		if (((_template == null) && (_selectedColor != null))
				|| ((_template != null) && (_selectedColor != _template.getSelectedColor())))
			pw.print(" selectedColor='" + getColorValue(_selectedColor) + "'");

		if (_tabs.getChildCount() == 0)
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		LyMobileTab	tabs;
		int			i, n;

		n = (_tab == null ? 0 : _tab.size());
		boolean first = true;
		for (i = 0; i < n; i++)
		{
			tabs = (LyMobileTab) _tab.get(i).getInternalView();
			tabs.init();

			if(tabs.get_label() == null || tabs.get_label().equals(""))
			{
				tabs.writeXml(pw, true, i);
			}
			else if(_selectedTab == null)
			{
				if(first)
				{
					tabs.writeXml(pw, true, i);
					first = false;
				}
				else
				{
					tabs.writeXml(pw, false, i);
				}
				
			}
			else if(_selectedTab.equals(tabs.get_id()) || _selectedTab.equals(tabs.get_label()))
			{
				tabs.writeXml(pw, true, i);
			}
			else
			{
				tabs.writeXml(pw, false, i);
			}
		}
		
		pw.println("</TABS>");
	}
	
	public void writeJson(PrintWriter pw, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		pw.println("{\"TABS\": [");
		
		if (_id != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");
		}
				
		if (((_template == null) && (_selectedTab != null))
				|| ((_template != null) && (_selectedTab != _template.getSelectedTab())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"selectedTab\": \"" + LyString.getJsonString(_selectedTab) + "\"}");
		}
				
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
		}
		
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"color\": \"" + getColorValue(_color) + "\"}");
		}

		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");
		}
		
		if (((_template == null) && (_selectedColor != null))
				|| ((_template != null) && (_selectedColor != _template.getSelectedColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"selectedColor\": \"" + getColorValue(_selectedColor) + "\"}");
		}

		if (_tabs.getChildCount() == 0)
		{
			pw.println("]}");
			return;
		}

		// Write sub-components
		LyMobileTab	tabs;
		int			i, n;

		n = (_tab == null ? 0 : _tab.size());
		boolean first = true;
		for (i = 0; i < n; i++)
		{
			tabs = (LyMobileTab) _tab.get(i).getInternalView();
			tabs.init();
			if(tabs.get_label() == null || tabs.get_label().equals(""))
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				tabs.writeJson(pw, true, i, sessionId);
			}
			else if(_selectedTab == null)
			{
				if(first)
				{
					isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
					tabs.writeJson(pw, true, i, sessionId);
					first = false;
				}
				else
				{
					isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
					tabs.writeJson(pw, false, i, sessionId);
				}
				
			}
			else if(_selectedTab.equals(tabs.get_id()) || _selectedTab.equals(tabs.get_label()))
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				tabs.writeJson(pw, true, i, sessionId);
			}
			else
			{
				isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
				tabs.writeJson(pw, false, i, sessionId);
			}
		}
		
		pw.println("]}");
	}

	
	public void init()
	{
		_id = _tabs.getId();
		_template = _tabs.getTemplate();
		_selectedTab = _tabs.getSelectedTab();
		
		_tab.clear();
		
		for(int i = 0; i < _tabs.getTabCount(); i++)
		{
			_tab.add(_tabs.getTab(i));
		}
		
	}

	
	public void setSelectedTab(String selectedTab)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void addTab(LyTab tabs, int pos)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void removeTab(LyTab tabs)
	{
		// TODO Auto-generated method stub
		
	}

}
