package leon.view.mobile;

import java.io.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;

public class LyMobileText extends LyMobileObject
implements LyGraphicalText
{
	/** The Text generic component. */
	protected LyText						_text;  

	protected String 						_id;

	protected LyText 						_template;

	protected String 						_label;

	protected boolean 						_secret;

	protected String 						_format;

	protected boolean 						_editable;

	/**
	 * Text font used on the component.
	 */
	protected String			_font;
	/**
	 * Background of the file selector when enabled.
	 */
	protected String			_enabledColor;

	/**
	 * Background of the file selector when disabled.
	 */
	protected String			_disabledColor;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;
	
	protected int				_nRows;

	public LyMobileText(LyText text)
	{
		super(text);

		_text = text;

		_font = getFontValue(getStringValue("MOBILE_TEXT.font"));

		_enabledColor = getStringValue("MOBILE_TEXT.enabledColor");

		_disabledColor = getStringValue("MOBILE_TEXT.disabledColor");

		_fontColor = getStringValue("MOBILE_TEXT.fontColor");

	}


	public void writeXml(PrintWriter pw)
	{
		// Write tag with its attributes
		init();
		pw.print("<TEXT");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
			pw.print(" label='" + LyString.getXmlStringStrict(_label) + "'");
		if (((_template == null) && _secret)
				|| ((_template != null) && (_secret != _template.isSecret())))
			pw.print(" secret='" + _secret + "'");
		if (((_template == null) && (_format != null))
				|| ((_template != null) && (_format != _template.getFormat())))
			pw.print(" format='" + LyString.getXmlStringStrict(_format) + "'");
		if (((_template == null) && !_editable)
				|| ((_template != null) && (_editable != _template.isEditable())))
			pw.print(" editable='" + _editable + "'");
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_enabledColor != null))
				|| ((_template != null) && (_enabledColor != _template.getEnabledColor())))
			pw.print(" enabledColor='" + getColorValue(_enabledColor) + "'");
		if (((_template == null) && (_disabledColor != null))
				|| ((_template != null) && (_disabledColor != _template.getDisabledColor())))
			pw.print(" disabledColor='" + getColorValue(_disabledColor) + "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");

		pw.println("/>");

	}

	public void writeJson(PrintWriter pw, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		pw.println("{\"TEXT\": [");

		if (_id != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");
		}
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"label\": \"" + LyString.getJsonString(_label) + "\"}");
		}
		if (((_template == null) && _secret)
				|| ((_template != null) && (_secret != _template.isSecret())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"secret\": \"" + _secret + "\"}");
		}
		if (((_template == null) && (_format != null))
				|| ((_template != null) && (_format != _template.getFormat())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"format\": \"" + LyString.getJsonString(_format) + "\"}");
		}
		if (((_template == null) && !_editable)
				|| ((_template != null) && (_editable != _template.isEditable())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"editable\": \"" + _editable + "\"}");
		}
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
		}
		if (((_template == null) && (_enabledColor != null))
				|| ((_template != null) && (_enabledColor != _template.getEnabledColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"enabledColor\": \"" + getColorValue(_enabledColor) + "\"}");
		}
		if (((_template == null) && (_disabledColor != null))
				|| ((_template != null) && (_disabledColor != _template.getDisabledColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"disabledColor\": \"" + getColorValue(_disabledColor) + "\"}");
		}
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");
		}
		
		if (_nRows > 0)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"nRows\": \"" + _nRows + "\"}");				
		}

		pw.println("]}");

	}


	public void init()
	{

		_id = _text.getId();
		_template = _text.getTemplate();
		_label = _text.getLabel();
		_secret = _text.isSecret();
		_format = _text.getFormat();
		_editable = _text.isEditable();
		_nRows = _text.getNRows();

	}


	public void setLabel(String label)
	{
		// TODO Auto-generated method stub

	}

}
