package leon.view.mobile;

import java.io.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;

public class LyMobileTool extends LyMobileObject
	implements LyGraphicalTool
{
	/** The tool generic component. */
    protected LyTool                  		_tool;
    
    protected String 						_id;
    
    protected LyTool 						_template;
    
    protected String 						_label;
    
    protected String 						_image;
    
    protected String 						_tooltip;
    
    protected boolean 						_enabled;
    
    protected LyComponent 					_component;

	protected int							_min;
    
    protected String 						_font;
    
    protected String 						_color;
    
    protected String 						_fontColor;
    
	/**
	 * TOOL.clickColor
	 */
	protected String			_clickColor;

	protected String						_rootUrl;
	
    
	public LyMobileTool(LyTool tool)
	{
		super(tool);
        
        _tool = tool;
		
		_font = getFontValue(getStringValue("MOBILE_TOOL.font"));

		_color = getStringValue("MOBILE_TOOL.color");

		_fontColor = getStringValue("MOBILE_TOOL.fontColor");	
		
		_clickColor = getStringValue("MOBILE_TOOL.clickColor");
		
		_rootUrl = ((LyMobileViewManager) getViewManager()).getRealRootURL();

	}
	
	
	public void writeXml(PrintWriter pw)
	{		
		// Write tag with its attributes
		init();
		if(_id == null || _id.startsWith("_closeAll") 
				|| _id.startsWith("_find") || _id.startsWith("_sort") || _id.startsWith("_choose") || _id.startsWith("_save") 
				|| _id.startsWith("_deletePref") || _id.startsWith("_about") || _id.startsWith("_exit") )
			return;
		
		pw.print("<TOOL");
		pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		pw.print(" label='" + LyString.getXmlStringStrict(_label) + "'");
		if (((_template == null) && (_tooltip != null))
				|| ((_template != null) && (_tooltip != _template.getTooltip())))
			pw.print(" tooltip='" + LyString.getXmlStringStrict(_tooltip) + "'");
		if (_min <= 0)
		{
			pw.print(" select='false'");
		}
		else if(_min > 0)
		{
			pw.print(" select='true'");
		}
			
		if (((_template == null) && (_image != getStringValue("TOOL.image", "$ALL.image")))
				|| ((_template != null) && (_image != _template.getImage())))
			pw.print(" image='" + getViewManager().getRootURL()+getImageUrl(_image) + "'");
		
		if (((_template == null) && !_enabled)
				|| ((_template != null) && (_enabled != _template.isEnabled())))
			pw.print(" enabled='" + _enabled + "'");
		
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + getColorValue(_color) + "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");
		
		if (((_template == null) && (_clickColor != null))
				|| ((_template != null) && (_clickColor != _template.getClickColor())))
			pw.print(" highlightColor='" + getColorValue(_clickColor) + "'");

		if (_tool.getChildCount() == 0)
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		
		// Write sub-components
		//TODO : Ne tient pas compte de POPUP_MENU
		if (_component != null && _component instanceof LyArea)
			((LyMobileArea)_component.getInternalView()).writeXml(pw);

		pw.println("</TOOL>");

	}

	public void writeJson(PrintWriter pw, String sessionId)
	{		
		writeJson(pw, false, sessionId);
	}
	
	public void writeJson(PrintWriter pw, boolean print, String sessionId)
	{		
		// Write tag with its attributes
		boolean isFirstElement = true;
		init();
		
		if(_id == null || _id.startsWith("_closeAll") 
				|| _id.startsWith("_find")
				|| _id.startsWith("_sort") || _id.startsWith("_choose") || _id.startsWith("_save") 
				|| _id.startsWith("_deletePref") || _id.startsWith("_about") || _id.startsWith("_exit") )
			return;
		
		if(print)
		{
			pw.println(",");
		}
		
		pw.println("{\"TOOL\": [");

		isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
		pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");
		isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
		pw.print("{\"label\": \"" + LyString.getJsonString(_label) + "\"}");

		if (((_template == null) && (_tooltip != null))
				|| ((_template != null) && (_tooltip != _template.getTooltip())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"tooltip\": \"" + LyString.getJsonString(_tooltip) + "\"}");
		}
		if (_min <= 0)
		{

			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"select\": \"false\"}");
		}
		else if(_min > 0)
		{

			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"select\": \"true\"}");
		}
			
		if (((_template == null) && (_image != getStringValue("TOOL.image", "$ALL.image")))
				|| ((_template != null) && (_image != _template.getImage())))
		{			
			String imageUrl = getImageUrl(_image);
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);	
			pw.print("{\"image\": \"" + 
					(imageUrl.startsWith(_rootUrl) ? imageUrl : _rootUrl+imageUrl)  + "\"}");
		}
		
		if (((_template == null) && !_enabled)
				|| ((_template != null) && (_enabled != _template.isEnabled())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"enabled\": \""+_enabled+"\"}");
		}
		
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
		}
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"color\": \"" + getColorValue(_color) + "\"}");
		}
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");
		}
		
		if (((_template == null) && (_clickColor != null))
				|| ((_template != null) && (_clickColor != _template.getClickColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"highlightColor\": \"" + getColorValue(_clickColor) + "\"}");
		}

		if (_tool.getChildCount() == 0)
		{
			pw.println("]}");
			return;
		}
		
		// Write sub-components
		//TODO : Ne tient pas compte de POPUP_MENU
		if (_component != null && _component instanceof LyArea)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			((LyMobileArea)_component.getInternalView()).writeJson(pw, sessionId);
		}

		pw.println("]}");

	}
	
	public void init()
	{
		_id = _tool.getId();
		_template = _tool.getTemplate();
		_tooltip = _tool.getTooltip();
		if(_tool.getLabel() != null && !_tool.getLabel().equals(""))
		{
			_label = getEnvironment().translate(_tool.getLabel());
		}
		else
		{
			if(_tooltip != null && !_tooltip.equals(""))
			{
				_label = getEnvironment().translate(_tooltip);
			}
			else
			{
				_label = null;
			}
			
		}
		_enabled = _tool.isEnabled();
		_image = _tool.getImage();
		_component = _tool.getComponent();
		LyMobileViewManager manager = (LyMobileViewManager) getViewManager();
		String actionId;
		int index = _id.indexOf(':');
		if(index != -1)
		{
			actionId = _id.substring(0, index);
		}
		else
		{
			actionId = _id;
		}
		_min = manager.getMin(actionId);	
	}

	
	public void setLabel(String label)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void setTooltip(String tooltip)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void setImage(String image)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void setPressed(boolean pressed)
	{
		// TODO Auto-generated method stub
		
	}

	
	public void setEnabled(boolean enabled)
	{
		// TODO Auto-generated method stub
		
	}

}
