package leon.view.mobile;

import java.io.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;

public class LyMobileTree extends LyMobileObject
implements LyGraphicalTree
{
	/** The Tree generic component. */
	protected LyTree						_tree;

	/**
	 * Identifier of the tree.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyTree			_template;

	/**
	 * Label of the tree.
	 */
	protected String			_label;

	/**
	 * Reference of the controller that manages the tree (i.e. handles user demands and
	 * refresh content).
	 */
	protected String			_controller;

	/**
	 * Root node of the tree.
	 */
	protected LyTreeNode		_treeNode;

	protected String			_rootUrl;

	/**
	 * Font of the tree.
	 */
	protected String			_font;

	/**
	 * Background color of the tree.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	/**
	 * Color used for selected nodes.
	 */
	protected String			_selectedColor;

	/**
	 * Color used for labels of selected nodes.
	 */
	protected String			_selectedFontColor;


	public LyMobileTree(LyTree tree)
	{
		super(tree);

		_tree = tree;

		_rootUrl = ((LyMobileViewManager) getViewManager()).getRealRootURL();

		_font = getFontValue(getStringValue("MOBILE_TREE.font"));

		_selectedColor = getStringValue("MOBILE_TREE.selectedColor");

		_selectedFontColor = getStringValue("MOBILE_TREE.selectedFontColor");

		_fontColor = getStringValue("MOBILE_TREE.fontColor");

		_color = getStringValue("MOBILE_TREE.color");
	}


	public void writeXml(PrintWriter pw)
	{
		init();
		pw.print("<TREE");
		pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
			pw.print(" label='" + LyString.getXmlStringStrict(_label) + "'");
		if (_controller != null)
			pw.print(" controller='" + LyString.getXmlStringStrict(_controller) + "'");

		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + getColorValue(_color) + "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");
		if (((_template == null) && (_selectedColor != null))
				|| ((_template != null) && (_selectedColor != _template.getSelectedColor())))
			pw.print(" selectedColor='" + getColorValue(_selectedColor) + "'");
		if (((_template == null) && (_selectedFontColor != null))
				|| ((_template != null) && (_selectedFontColor != _template.getSelectedFontColor())))
			pw.print(" selectedFontColor='" + getColorValue(_selectedFontColor) + "'");


		if (_tree.getChildCount() == 0)
		{
			pw.println("/>");
			return;
		}
		else
			pw.println(">");

		// Write sub-components
		if (_treeNode != null)
			((LyMobileTreeNode)_treeNode.getInternalView()).writeXml(pw);

		pw.println("</TREE>");

	}

	public void writeJson(PrintWriter pw, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		pw.println("{\"TREE\": [");

		isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
		pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");		
		
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"label\": \"" + LyString.getJsonString(_label) + "\"}");
		}
		if (_controller != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"controller\": \"" + LyString.getJsonString(_controller) + "\"}");
		}

		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
		}
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"color\": \"" + getColorValue(_color) + "\"}");
		}
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"fontColor\": \"" + getColorValue(_fontColor)+ "\"}");
		}
		if (((_template == null) && (_selectedColor != null))
				|| ((_template != null) && (_selectedColor != _template.getSelectedColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"selectedColor\": \"" + getColorValue(_selectedColor) + "\"}");
		}
		if (((_template == null) && (_selectedFontColor != null))
				|| ((_template != null) && (_selectedFontColor != _template.getSelectedFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"selectedFontColor\": \"" + getColorValue(_selectedFontColor)  + "\"}");
		}

		if (_tree.getChildCount() == 0)
		{
			pw.println("]}");
			return;
		}

		// Write sub-components
		if (_treeNode != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			((LyMobileTreeNode)_treeNode.getInternalView()).writeJson(pw, sessionId);
		}
			

		pw.println("]}");

	}


	public void init()
	{
		_id = _tree.getId();

		_template = _tree.getTemplate();

		_label = _tree.getLabel();

		_controller = _tree.getController();

		_treeNode = _tree.getTreeNode();
	}


	public void setLabel(String label)
	{
		// TODO Auto-generated method stub

	}


	public void setActiveNode(String activeNode)
	{
		// TODO Auto-generated method stub

	}


	public void setTreeNode(LyTreeNode treeNode)
	{
		// TODO Auto-generated method stub

	}

}
