package leon.view.mobile;

import java.io.*;
import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.LyTreeNode.*;
import leon.view.graphic.*;

public class LyMobileTreeNode extends LyMobileObject implements
LyGraphicalTreeNode
{
	/** The TreeNode generic component. */
	protected LyTreeNode	_treeNode;

	/**
	 * Identifier of the node of the tree.
	 */
	protected String			_id;

	/**
	 * Template of the component.
	 */
	protected LyTreeNode			_template;
	
	/**
	 * Indicates if the node is selected.
	 * This attribute is not used for the NONE selection mode.
	 */
	protected boolean			_selected;
	
	/**
	 * Specifies if the tree node is expandable or not (can be opened).
	 */
	protected boolean			_expandable;
	
	/**
	 * Image associated to the node.
	 */
	protected String			_image;
	
	protected boolean			_open;

	/**
	 * Label of the node of the tree.
	 */
	protected String			_label;

	/**
	 * TREE_NODE.openDirImage
	 */
	protected String			_openDirImage;

	/**
	 * TREE_NODE.closedDirImage
	 */
	protected String			_closedDirImage;

	/**
	 * Sub-nodes of the tree node.
	 */
	protected ArrayList<LyTreeNode>		_treeNodes;

	protected String						_rootUrl;
	
	/**
	 * Font of the node.
	 */
	protected String			_font;

	/**
	 * Background color of the node.
	 */
	protected String			_color;

	/**
	 * Text color used on the component.
	 */
	protected String			_fontColor;

	/**
	 * TREE_NODE.selectedBackgroundColor
	 */
	protected String			_selectedBackgroundColor;
	/**
	 * TREE_NODE.selectedFontColor
	 */
	protected String			_selectedFontColor;
	
	public LyMobileTreeNode(LyTreeNode treeNode)
	{
		super(treeNode);

		_treeNode = treeNode;
		
		_rootUrl = ((LyMobileViewManager) getViewManager()).getRealRootURL();
		
		_treeNodes = new ArrayList<LyTreeNode>();
		
		_font = getFontValue(getStringValue("MOBILE_TREE_NODE.font"));

		_selectedBackgroundColor = getStringValue("MOBILE_TREE_NODE.selectedBackgroundColor");

		_selectedFontColor = getStringValue("MOBILE_TREE_NODE.selectedFontColor");

		_fontColor = getStringValue("MOBILE_TREE_NODE.fontColor");
		
		_color = getStringValue("MOBILE_TREE_NODE.color");
	}

	public void writeXml(PrintWriter pw)
	{
		init();
		// Write tag with its attributes
		pw.print("<TREE_NODE");
		if (_id != null)
			pw.print(" id='" + LyString.getXmlStringStrict(_id) + "'");
		if (_template != null)
			pw.print(" template='" + _template.getId() + "'");
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
			pw.print(" label='" + LyString.getXmlString(_label) + "'");
		if (((_template == null) && (_openDirImage != getStringValue("TREE_NODE.openDirImage", "$ALL.openDirImage")))
				|| ((_template != null) && (_openDirImage != _template.getOpenDirImage())))
			pw.print(" openDirImage='" + _rootUrl + getImageUrl(_closedDirImage) + "'");
		if (((_template == null) && (_closedDirImage != getStringValue("TREE_NODE.closedDirImage", "$ALL.closedDirImage")))
				|| ((_template != null) && (_closedDirImage != _template.getClosedDirImage())))
			pw.print(" closedDirImage='" + _rootUrl + getImageUrl(_closedDirImage) + "'");
		if (((_template == null) && _selected)
				|| ((_template != null) && (_selected != _template.isSelected())))
			pw.print(" selected='" + _selected + "'");
		
		if (((_template == null) && _expandable)
				|| ((_template != null) && (_expandable != _template.isExpandable())))
			pw.print(" expandable='" + _expandable + "'");
		
		if (((_template == null) && (_image != null))
				|| ((_template != null) && (_image != _template.getImage())))
			pw.print(" image='" + _rootUrl + getImageUrl(_image) + "'");
		
		if (((_template == null) && _open)
				|| ((_template != null) && (_open != _template.isOpen())))
			pw.print(" open='" + _open + "'");
		
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
			pw.print(" font='" + LyString.getXmlStringStrict(_font) + "'");
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
			pw.print(" color='" + getColorValue(_color) + "'");
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
			pw.print(" fontColor='" + getColorValue(_fontColor) + "'");
		if (((_template == null) && (_selectedBackgroundColor != null))
				|| ((_template != null) && (_selectedBackgroundColor != _template.getSelectedBackgroundColor())))
			pw.print(" selectedBackgroundColor='" + getColorValue(_selectedBackgroundColor) + "'");
		if (((_template == null) && (_selectedFontColor != null))
				|| ((_template != null) && (_selectedFontColor != _template.getSelectedFontColor())))
			pw.print(" selectedFontColor='" + getColorValue(_selectedFontColor) + "'");


		if ((_treeNode.getChildCount() == 0) )
		{
			pw.println("/>");
			return;
		}
		else
		{
			pw.print(" children='true'");
			pw.println(">");
		}
			

		int			i, n;

		LyComponent	treeNodes;

		n = (_treeNodes == null ? 0 : _treeNodes.size());
		for (i = 0; i < n; i++)
		{
			treeNodes = _treeNodes.get(i);
			((LyMobileTreeNode)treeNodes.getInternalView()).writeXml(pw);
		}


		pw.println("</TREE_NODE>");

	}
	
	public void writeJson(PrintWriter pw, String sessionId)
	{
		boolean isFirstElement = true;
		init();
		pw.println("{\"TREE_NODE\": [");
		
		if (_id != null)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"id\": \"" + LyString.getJsonString(_id) + "\"}");
		}
		
		if (((_template == null) && (_label != null))
				|| ((_template != null) && (_label != _template.getLabel())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"label\": \"" + LyString.getJsonString(_label) + "\"}");
		}
		if (((_template == null) && (_openDirImage != getStringValue("TREE_NODE.openDirImage", "$ALL.openDirImage")))
				|| ((_template != null) && (_openDirImage != _template.getOpenDirImage())))
		{
			String imageUrl = getImageUrl(_openDirImage);
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);	
			pw.print("{\"openDirImage\": \"" + 
					(imageUrl.startsWith(_rootUrl) ? imageUrl : _rootUrl+imageUrl)  + "\"}");
		}
		if (((_template == null) && (_closedDirImage != getStringValue("TREE_NODE.closedDirImage", "$ALL.closedDirImage")))
				|| ((_template != null) && (_closedDirImage != _template.getClosedDirImage())))
		{
			String imageUrl = getImageUrl(_closedDirImage);
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);	
			pw.print("{\"closedDirImage\": \"" + 
					(imageUrl.startsWith(_rootUrl) ? imageUrl : _rootUrl+imageUrl)  + "\"}");
		}
		if (((_template == null) && _selected)
				|| ((_template != null) && (_selected != _template.isSelected())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"selected\": \"" + _selected + "\"}");
		}
		
		if (((_template == null) && _expandable)
				|| ((_template != null) && (_expandable != _template.isExpandable())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"expandable\": \"" + _expandable + "\"}");	
		}
		
		if (((_template == null) && (_image != null))
				|| ((_template != null) && (_image != _template.getImage())))
		{
			String imageUrl = getImageUrl(_image);
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);	
			pw.print("{\"image\": \"" + 
					(imageUrl.startsWith(_rootUrl) ? imageUrl : _rootUrl+imageUrl)  + "\"}");
		}
		
		if (((_template == null) && _open)
				|| ((_template != null) && (_open != _template.isOpen())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"open\": \"" + _open + "\"}");
		}
		
		if (((_template == null) && (_font != null))
				|| ((_template != null) && (_font != _template.getFont())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"font\": \"" + LyString.getJsonString(_font) + "\"}");
		}
		if (((_template == null) && (_color != null))
				|| ((_template != null) && (_color != _template.getColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"color\": \"" + getColorValue(_color) + "\"}");
		}
		if (((_template == null) && (_fontColor != null))
				|| ((_template != null) && (_fontColor != _template.getFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"fontColor\": \"" + getColorValue(_fontColor) + "\"}");
		}
		if (((_template == null) && (_selectedBackgroundColor != null))
				|| ((_template != null) && (_selectedBackgroundColor != _template.getSelectedBackgroundColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"selectedBackgroundColor\": \"" + getColorValue(_selectedBackgroundColor)  + "\"}");
		}
		if (((_template == null) && (_selectedFontColor != null))
				|| ((_template != null) && (_selectedFontColor != _template.getSelectedFontColor())))
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"selectedFontColor\": \"" + getColorValue(_selectedFontColor) + "\"}");
		}


		if ((_treeNode.getChildCount() == 0) )
		{
			pw.println("]}");
			return;
		}
		else
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			pw.print("{\"children\": \"true\"}");
		}
			

		int			i, n;

		LyComponent	treeNodes;

		n = (_treeNodes == null ? 0 : _treeNodes.size());
		for (i = 0; i < n; i++)
		{
			isFirstElement = printJsonAttributeSeparator(isFirstElement, pw);
			treeNodes = _treeNodes.get(i);
			((LyMobileTreeNode)treeNodes.getInternalView()).writeJson(pw, sessionId);
		}


		pw.println("]}");

	}

	public void init()
	{
		_id = _treeNode.getId();

		_template = _treeNode.getTemplate();

		_label = _treeNode.getLabel();

		_openDirImage = _treeNode.getOpenDirImage();

		_closedDirImage = _treeNode.getClosedDirImage();
		
		_image = _treeNode.getImage();
		
		_expandable = _treeNode.isExpandable();
		
		_selected = _treeNode.isSelected();
		
		_open = _treeNode.isOpen();

		_treeNodes.clear();

		for(int i = 0; i < _treeNode.getTreeNodeCount(); i++)
		{
			_treeNodes.add(_treeNode.getTreeNode(i));
		}
	}

	public void setLabel(String label)
	{
		// TODO Auto-generated method stub

	}

	public void setTooltip(String tooltip)
	{
		// TODO Auto-generated method stub

	}

	public void setSelected(boolean selected)
	{
		// TODO Auto-generated method stub

	}

	public void setOpen(boolean open)
	{
		// TODO Auto-generated method stub

	}

	public void setHidden(boolean hidden)
	{
		// TODO Auto-generated method stub

	}

	public void setExpandable(boolean expandable)
	{
		// TODO Auto-generated method stub

	}

	public void setImage(String image)
	{
		// TODO Auto-generated method stub

	}

	public void setValidity(Validity validity)
	{
		// TODO Auto-generated method stub

	}

	public void setFont(String font)
	{
		// TODO Auto-generated method stub

	}

	public void setColor(String color)
	{
		// TODO Auto-generated method stub

	}

	public void addImage(LyImage images, int pos)
	{
		// TODO Auto-generated method stub

	}

	public void removeImage(LyImage images)
	{
		// TODO Auto-generated method stub

	}

	public void addTreeNode(LyTreeNode treeNodes, int pos)
	{
		// TODO Auto-generated method stub

	}

	public void removeTreeNode(LyTreeNode treeNodes)
	{
		// TODO Auto-generated method stub

	}

}
