package leon.view.mobile;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Frame;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import leon.app.LySession;
import leon.info.LyAction;
import leon.misc.LyColor;
import leon.misc.LyEnvironment;
import leon.misc.LyFont;
import leon.view.LyArea;
import leon.view.LyComponent;
import leon.view.LyDialog;
import leon.view.LyFieldItem;
import leon.view.LyItem;
import leon.view.LyLayout;
import leon.view.LyMap;
import leon.view.LyMapRegion;
import leon.view.LyMenu;
import leon.view.LyMenuBar;
import leon.view.LyPopupMenu;
import leon.view.LyViewComponent;
import leon.view.LyViewManager;
import leon.view.LyViewNodeFactory;
import leon.view.common.LyImageUtils;
import leon.view.web.LyWebComponent;
import leon.view.web.LyWebRequestInfo;
import leon.view.web.LyWebViewManager;
import leon.view.web.mobile.LyMobileConstants;
import leon.view.web.mobile.LyMobileDecodeRequest;

public class LyMobileViewManager extends LyWebViewManager
{
	public static int		MAX_HEADERS = 2;
	public final static String MOBILE_ONLY_PROPERTY = "MOBILEONLY";
	
	/** The context url **/
	private String			   _contextUrl;
	
	/** The wait counter. */
    public static int          WAIT_COUNTER = 50;
    
    protected String			_url;
    
    protected String			_applicationName;
    
    /** For knowing if we have just swapped the sessions **/
    protected boolean			_sessionSwapped;
    
	@Override
	protected void setProperties()
	{
		super.setProperties();
		removeProperty("NOTFORMOBILE");
		setProperty("DISPLAY_GOOGLE_MAP");
		setProperty(MOBILE_ONLY_PROPERTY);
		setProperty("MOBILEWEBVIEW");
	}

	/**
     * Default Frame used in some parts of the code when a default AWT component is needed (for
     * example to create an image).
     */
    public static Frame        _frame;
    
    private LySession		   _session;
    
    /**
     * Creates and gets the default frame.
     *
     * @return The default frame.
     */
    public static Frame getDefaultFrame()
    {
        if (_frame == null)
        {
            _frame = new Frame();
            // ensure that the peer is ready.
            // a peer is required for some methods such as createImage()
            // see also : LyEnvironment.getImage()
            if (LyEnvironment.isOperatingSystem(LyEnvironment.OS_WINDOWS))
                _frame.addNotify();
        }

        return _frame;
    }
    
    LyWebViewManager		_webviewViewManager;
    
    /*
	 * (non-Javadoc)
	 *
	 * @see leon.view.LyViewManager#buildInternalView(leon.view.LyComponent)
	 */
    @Override
	public LyViewComponent buildInternalView(LyComponent component)
	{
		// Check if field item and if a bean must be used
		if (component instanceof LyFieldItem)
		{
			LyFieldItem item = (LyFieldItem)component;

			if (item.getBean() != null)
			{
				LyViewComponent bean = item.buildBean();

				if (bean instanceof LyWebComponent)
					return bean;
				else if (bean != null)
					_environment.logWarning("Can't use bean " + bean.getClass().getName()
						+ " because it's not a LyWebComponent");
			}
		}

		if (_beans == null)
			// try to load view files
			loadBeans();

		if (_beans != null)
		{
			String tag = LyViewNodeFactory.getTag(component.getClass().getName());

			// Special case for treemaps in HTML mode (using DHTML) : retrieve
			// TREEMAP and TREEMAP_REGION tags
			if ((component instanceof LyMap) || (component instanceof LyMapRegion))
			{
				LyComponent parent = component;

				while (!(parent instanceof LyMap) && (parent != null))
					parent = parent.getParent();

				if (parent instanceof LyMap)
				{
					LyMap    map = (LyMap)parent;
					LyLayout layout = map.getLayout();

					if ((layout != null) && (layout.getStyle() == LyLayout.Style.TREEMAP))
						tag = "TREE" + tag;
				}
			}
			
			if ((component.getParent() != null && 
					component.getParent().getData(LyMobileConstants.WEBVIEW_MOBILE_KEY) != null)
					|| component.getData(LyMobileConstants.WEBVIEW_MOBILE_KEY) != null)
			{
				component.addData(LyMobileConstants.WEBVIEW_MOBILE_KEY, LyMobileConstants.TRUE);
				
				if (_webviewViewManager == null)
				{
					LyEnvironment	env = new LyEnvironment(null, _session.getEnvironment(), true);					
					_webviewViewManager = (LyWebViewManager)env.getViewManager(false);
					_webviewViewManager.setRootURL(getRootURL());
				}
				
				if(component instanceof LyArea)
				{
					Vector<LyArea> subAreas = new Vector<LyArea>();
					
					component.getComponents(LyArea.class, subAreas);
					
					for (LyArea subArea : subAreas)
						subArea.setEnvironment(_webviewViewManager.getEnvironment());
					
					((LyArea) component).setEnvironment(_webviewViewManager.getEnvironment());
				}
				else if(component.getParentArea() != null)
				{
					component.getParentArea().setEnvironment(_webviewViewManager.getEnvironment());
				}
				else if(component instanceof LyDialog && ((LyDialog) component).getArea() != null)
				{
					((LyDialog) component).getArea().setEnvironment(_webviewViewManager.getEnvironment());
					((LyDialog) component).setEnvironment(_webviewViewManager.getEnvironment());
					Vector<LyArea> subAreas = new Vector<LyArea>();
					
					component.getComponents(LyArea.class, subAreas);
					
					for (LyArea subArea : subAreas)
						subArea.setEnvironment(_webviewViewManager.getEnvironment());
				}
				
				
				LyViewComponent	result = _webviewViewManager.buildInternalView(component);
				
				return result;
			}

			String beanClassName = _beans.getString(tag);

			if (beanClassName != null)
			{
				if (beanClassName.length() == 0)
					return null;
				else
					return buildInternalView(beanClassName, component);
			}

			// Try to find bean class name on the super class.
			Class<?>  superClass = component.getClass().getSuperclass();

			while (superClass != null)
			{
				String parentTag = LyViewNodeFactory.getTag(superClass.getName());
				String parentBeanClassName = _beans.getString(parentTag);

				if (parentBeanClassName != null)
				{
					if (parentBeanClassName.length() == 0)
						return null;
					else
						return buildInternalView(parentBeanClassName, component);
				}

				superClass = superClass.getSuperclass();
			}
		}

		if ((component instanceof LyItem) || (component instanceof LyMenu)
				|| (component instanceof LyMenuBar) || (component instanceof LyPopupMenu))
			return null;

		return buildInternalView("leon.view.mobile", "LyMobile", component);
	} // end method buildInternalView
	
	public LyMobileViewManager(LyEnvironment environment)
	{
		super(environment);
		_beans = null;
		_url = null;
		_sessionSwapped = false;
	}

	@Override
	protected void loadBeans()
	{
		loadBeans("views_mobile");
	}

	@Override
	public String getDownloadAction()
	{
		String actionUrl = getRealRootURL() + "/" +LyMobileConstants.PATH_ACTION_DOWNLOAD;
		return actionUrl + "?_file=";
	}
	
	public String getRealRootURL()
	{
		String endUrl = getEnvironment().getEnv("LY_APP_DOC");
        String rootUrl = _root_url;

        if (rootUrl.lastIndexOf(endUrl) != -1)
            rootUrl = rootUrl.substring(0, rootUrl.lastIndexOf(endUrl)+1);

        int index = rootUrl.lastIndexOf('/');
        
        if (index > 6)
            rootUrl = rootUrl.substring(0, index);

        return rootUrl;
	}
		
	/**
	 * Get the partial URL for a context relative url.
	 *
	 * @param  url             A context relative URL.
	 * @param  parameters      The parameters used to build the query String.
	 * @param  encodeAmpersand Tells if the "&" character should be encoded.
	 * @return an absolute URL (scheme + server + port + context path + url + "?" + param1=value1 +
	 *         "&" + param2=value2, etc...).
	 * @throws MalformedURLException if the URL is not well formed
	 */
	public String getPartialURL(String url, Map<String, String> parameters, boolean encodeAmpersand)
		throws MalformedURLException
	{
		String		 partialUrl = _contextUrl + url;
		StringBuffer result = new StringBuffer(partialUrl);

		// add the query string if needed
		if ((parameters != null) && (parameters.size() > 0))
		{
			result.append("?"); // add the query separator

			Map.Entry<String, String> entry;

			for (Iterator<Map.Entry<String, String>> it = parameters.entrySet().iterator(); it.hasNext();)
			{
				entry = it.next();
				result.append(entry.getKey()).append('=').append(entry.getValue());

				// add the parameter separator
				if (it.hasNext())
				{
					if (encodeAmpersand)
						result.append("&#38;");
					else
						result.append('&');
				}
			}
		}

		return result.toString();
	} // end method getAbsoluteURL

	/**
     * Copy a given text to the system clipboard.
     *
     * @param text the text to be copied to the clipboard.
     */
    public void copyToClipboard(String text)
    {
        // Nothing to do in web mode
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.misc.LyViewManagerInterface#getColor(leon.misc.LyColor)
     */
    public Object getColor(LyColor color)
    {
        if (color == null)
            return null;

        return new Color(color.getRGB());
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.misc.LyViewManagerInterface#getFont(leon.misc.LyFont)
     */
    public Object getFont(LyFont font)
    {
        if (font == null)
            return getDefaultFrame().getFont();

        int style = 0;

        if (font.isBold())
            style |= Font.BOLD;
        if (font.isItalic())
            style |= Font.ITALIC;

        return new Font(font.getFontName(), style, font.getSize());
    }

    
    /**
     * (non-Javadoc).
     *
     * @see leon.misc.LyViewManagerInterface#getStringWidth(leon.misc.LyFont, java.lang.String)
     */
    public int getStringWidth(LyFont font, String text)
    {
        Font        f = (Font)getFont(font);
        FontMetrics fm = getDefaultFrame().getFontMetrics(f);

        return fm.stringWidth(text);
    }

    /**
     * Construct a default image.
     *
     * @see LyImageUtils#getDefaultImage()
     * @see leon.misc.LyViewManagerInterface#getDefaultImage()
     */
    public Object getDefaultImage()
    {
        return LyImageUtils.getDefaultImage();
    }

    /**
     * Gets images sizes.
     *
     * @see LyImageUtils#getSizes(Image)
     * @see leon.misc.LyViewManagerInterface#getSizes(Object)
     */
    public int[] getSizes(Object image)
    {
        return LyImageUtils.getSizes((Image)image);
    }

    /**
     * Creates a scaled version of the given image.
     *
     * @see LyImageUtils#getScaledInstance(Image, int, int)
     * @see leon.misc.LyViewManagerInterface#getScaledInstance(Object, int, int)
     */
    public Object getScaledInstance(Object image, int width, int height)
    {
        return LyImageUtils.getScaledInstance((Image)image, width, height);
    }

    /**
     * Waits for the image loading if the headless mode is null or set to true.
     *
     * @see LyImageUtils#waitForLoading(Component, Image)
     * @see leon.misc.LyViewManagerInterface#waitForLoading(Object, Object)
     */
    public void waitForLoading(Object component, Object image)
        throws InterruptedException
    {
        // In headless mode, waitForLoading can't be used because the method
        // creates a new Frame
        Image  pic = (Image)image;
        int    counter = 0;

        if (GraphicsEnvironment.isHeadless())
        {
             while ((pic.getWidth(null) < 0) && (counter < WAIT_COUNTER))
             {
                 Thread.sleep(20);
                 counter++;
             }
        }
        else
            LyImageUtils.waitForLoading((Component)component, (Image)image);
    }

    /**
     * Retrieves an image from a Stream.
     *
     * @see LyImageUtils#loadImageFromStream(Component, InputStream)
     * @see leon.misc.LyViewManagerInterface#loadImageFromStream(Object, InputStream)
     */
    public Object loadImageFromStream(Object component, InputStream resource)
        throws IOException
    {
        return LyImageUtils.loadImageFromStream((Component)component, resource);
    }

    /**
     * Compare two images pixel by pixel.
     *
     * @see LyImageUtils#isSameImage(Image, Image)
     * @see leon.view.LyViewManager#isSameImage(java.lang.Object, java.lang.Object)
     */
    public boolean isSameImage(Object image1, Object image2)
        throws InterruptedException
    {
        return LyImageUtils.isSameImage((Image)image1, (Image)image2);
    }

    /**
     * Free the resources of the image if not in the cache.
     *
     * @see LyImageUtils#freeImage(Image)
     * @see leon.misc.LyViewManagerInterface#freeImage(java.lang.Object)
     */
    public void freeImage(Object image)
    {
        LyImageUtils.freeImage((Image)image);
    }
    
    public void setSession(LySession session)
    {
    	_session = session;
    	
    	if (_webviewViewManager == null)
		{
			LyEnvironment	env = new LyEnvironment(null, _session.getEnvironment(), true);
			
			_webviewViewManager = (LyWebViewManager)env.getViewManager(false);
		}
    }
    
    @Override
    public void setRootURL(String rootUrl)
    {
    	super.setRootURL(rootUrl);
    	if(_webviewViewManager != null)
    	{
    		_webviewViewManager.setRootURL(rootUrl);
    	}
    }
    
    public LyWebViewManager getWebViewManager()
    {
    	return _webviewViewManager;
    }
   
    
    public int getMin(String actionId)
    {
    	if(_session == null)
    	{
    		return -1;
    	}
    	else
    	{
    		LyAction action = _session.getApplication().getAction(actionId);
    		if(action == null)
    		{
    			return -1;
    		}
    		return action.getMin();
    	}
    }
	
	/**
	 * Return the current web request info object.
	 */
	protected LyWebRequestInfo getWebRequestInfo()
	{
		return null;
	}
	
	/**
	 * Swap two sessions.
	 * @param newSession the new session
	 * @param oldSession the old session
	 * @param applicationId the application id
	 */
	public void swapSession(Object newSession, Object oldSession, String applicationId)
	{
		// we need to swap some specific configuration: -> _root_url
		LySession oldLySession = (LySession)oldSession;
		LyViewManager viewManager = oldLySession.getViewManager();
		if (viewManager instanceof LyMobileViewManager)
		{
			String newRoot = getRootURL();
			((LyMobileViewManager)viewManager).setRootURL(newRoot);
			((LyMobileViewManager)viewManager).set_sessionSwapped(true);
		}
		LyMobileDecodeRequest.getInstance().swapSession(newSession, oldSession, applicationId);
		((LySession)newSession).close();
	}
	
	/*
	 * (non-Javadoc)
	 *
	 * @see leon.view.LyViewManager#showLocation(leon.view.LyComponent,
	 *      java.lang.String, boolean)
	 */
	public void showLocation(LyComponent source, String location, boolean newWindow, Hashtable<Short, String> parameters)
	{
		String docDir = _environment.getEnv("LY_APP_DOC_DIR");
		// case a file being consulted with an hyperlink.
		String fileRef = "file://";

		if ((location.startsWith(docDir)) || (location.startsWith(fileRef+docDir)))
		{
			String appDoc = _environment.getEnv("LY_APP_DOC");

			// use a relative URL to the document, not file://
			if ((location.startsWith(docDir)))
			{
				location = location.replace('\\', '/');

				location = appDoc+"/"+location.substring(docDir.length()+1);
			}
			else if ((location.startsWith(fileRef+docDir)))
			{
				location = location.replace('\\', '/');

				location = appDoc+"/"+location.substring(fileRef.length()+docDir.length()+1);
			}
		}
		else
		{
			// use a  file:// reference to the document
				location = location.replace('\\', '/');
			if (location.startsWith(fileRef))
				location = location.substring(fileRef.length());
		}
		
		if(location.startsWith("http://"))
		{
			setUrl(location);
			return;
		}
		
		if(!location.startsWith("/"))
		{
			location = _applicationName+'/'+location;
		}

		setUrl(getRealRootURL()+location);
	}
	
	public void setUrl(String url)
	{
		_url = url;
	}

	public String getUrl()
	{
		String res = _url;
		_url = null;
		return res;
	}
	
	public boolean hasUrl()
	{
		if(_url == null)
		{
			return false;
		}
		return true;
	}
	
	public void setApplicationName(String name)
	{
		_applicationName = name;
	}

	public String getApplicationName()
	{
		return _applicationName;
	}

	public boolean is_sessionSwapped()
	{
		return _sessionSwapped;
	}

	public void set_sessionSwapped(boolean _sessionSwapped)
	{
		this._sessionSwapped = _sessionSwapped;
	}
	
	//FFT5040
	/**
	 * Generates the HTML code for a <code>_LyWebMessage</code> instance.
	 */
	/*protected String generateMessage(_LyWebMessage message)
	{
		LyEnvironment   environment = getEnvironment();
		StringBuffer	script = new StringBuffer();
		String			title = LyString.getJsonString(message._title);
		String			text = LyString.getJsonString(message._text);
		boolean 		firstElement = true;
		
		script.append("{\"MESSAGE\": [");

		if (message._commands.get(LyViewManager.COMMAND_OK))
		{
			firstElement = false;
			script.append("{\"BUTTON\": [");
			script.append("{\"id\": \"" + LyViewManager.COMMAND_OK + "\"},");
			script.append("{\"label\": \"" + LyString.getJsonString(environment.translate("LY_OK")) + "\"}]}");

		}
		if (message._commands.get(LyViewManager.COMMAND_YES))
		{
			if(!firstElement)
			{
				script.append(",");
			}
			else
			{
				firstElement = false;
			}
			script.append("{\"BUTTON\": [");
			script.append("{\"id\": \"" + LyViewManager.COMMAND_YES + "\"},");
			script.append("{\"label\": \"" + LyString.getJsonString(environment.translate("LY_YES")) + "\"}]}");	
		}
		if (message._commands.get(LyViewManager.COMMAND_ALL))
		{
			if(!firstElement)
			{
				script.append(",");
			}
			else
			{
				firstElement = false;
			}
			script.append("{\"BUTTON\": [");
			script.append("{\"id\": \"" + LyViewManager.COMMAND_ALL + "\"},");
			script.append("{\"label\": \"" + LyString.getJsonString(environment.translate("LY_ALL")) + "\"}]}");
		}
		if (message._commands.get(LyViewManager.COMMAND_NONE))
		{
			if(!firstElement)
			{
				script.append(",");
			}
			else
			{
				firstElement = false;
			}
			script.append("{\"BUTTON\": [");
			script.append("{\"id\": \"" + LyViewManager.COMMAND_NONE + "\"},");
			script.append("{\"label\": \"" + LyString.getJsonString(environment.translate("LY_NONE")) + "\"}]}");
		}
		if (message._commands.get(LyViewManager.COMMAND_NO))
		{
			if(!firstElement)
			{
				script.append(",");
			}
			else
			{
				firstElement = false;
			}
			script.append("{\"BUTTON\": [");
			script.append("{\"id\": \"" + LyViewManager.COMMAND_NO + "\"},");
			script.append("{\"label\": \"" + LyString.getJsonString(environment.translate("LY_NO")) + "\"}]}");

		}
		if (message._commands.get(LyViewManager.COMMAND_CANCEL))
		{
			if(!firstElement)
			{
				script.append(",");
			}
			else
			{
				firstElement = false;
			}
			script.append("{\"BUTTON\": [");
			script.append("{\"id\": \"" + LyViewManager.COMMAND_CANCEL + "\"},");
			script.append("{\"label\": \"" + LyString.getJsonString(environment.translate("LY_CANCEL")) + "\"}]}");
		}
		if (message._commands.get(LyViewManager.COMMAND_HELP))
		{
			if(!firstElement)
			{
				script.append(",");
			}
			else
			{
				firstElement = false;
			}
			script.append("{\"BUTTON\": [");
			script.append("{\"id\": \"" + LyViewManager.COMMAND_HELP + "\"},");
			script.append("{\"label\": \"" + LyString.getJsonString(environment.translate("LY_HELP")) + "\"}]}");
		}

		if(!firstElement)
		{
			script.append(",");
		}
		else
		{
			firstElement = false;
		}

		script.append("{\"title\": \"" + LyString.getJsonString(title) + "\"},");
		script.append("{\"id\": \"" + LyString.getJsonString(message._infoId) + "\"},");
		script.append("{\"text\": \"" + LyString.getJsonString(text) + "\"}]}");

		return script.toString();
	}*/
	
	
}
