/**
 * LyPrintManager.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.print;

import java.util.*;

import leon.data.*;
import leon.misc.*;


/**
 */
public abstract class LyPrintManager
{
	public static final String     __VERSION =
		"$Id: LyPrintManager.java 28997 2010-12-23 10:17:15Z bmichel $";

	/** Reduce the font size in printing compared to screen. */
	public static int			   PRINT_TABLE_FONT_REDUCTION = 2;

	public static int			   LEFT_MARGIN = 40;
	public static int			   TOP_MARGIN = 20;
	public static int			   RIGHT_MARGIN = 40;
	public static int			   BOTTOM_MARGIN = 40;

	/** Dump the result to the screen, not to printer. */
	public static boolean		   DEBUG_TO_SCREEN = false;
	public static int			   PAGE_WIDTH = 630;
	public static int			   PAGE_HEIGHT = 888;

	protected LyEnvironment		   _environment;

	protected LyPrintDataInterface _printDataInterface;

	protected Properties		   _properties;

	protected boolean			   _inFile;

	protected String			   _supportName;

	protected String			   _format;

	/**
	 * Default constructor.
	 */
	public LyPrintManager(LyEnvironment environment,
		LyPrintDataInterface printDataInterface, Properties properties)
	{
		_environment = environment;
		_printDataInterface = printDataInterface;
		_properties = (properties == null ? new Properties() : properties);
	}

	/**
	 * Checks if printing is done in a file.
	 *
	 * @return True if printing is done in a file, false otherwise.
	 */
	public boolean isInFile()
	{
		return _inFile;
	}

	/**
	 * Gets support name (either printer name or file name if printing is done in a file).
	 *
	 * @return Support name.
	 */
	public String getSupportName()
	{
		return _supportName;
	}

	/**
	 * Gets printing format.
	 *
	 * @return Print format.
	 */
	public String getFormat()
	{
		return _format;
	}

	/**
	 * Gets environment;
	 *
	 * @return Environment of the print manager.
	 */
	public LyEnvironment getEnvironment()
	{
		return _environment;
	}

	/**
	 * Tests if job was cencelled.
	 *
	 * @return True if cancelled, false otherwise.
	 */
	public abstract boolean wasCancelled();

	/**
	 * End of printing.
	 */
	public abstract void end();

	/**
	 * @param  object        Printed object.
	 * @param  fields        List of printed fields
	 * @param  fontReduction Font reduction factor.
	 * @param  startPage     Page number.
	 * @return New page count after printing.
	 */
	public abstract int print(Object object, Vector<Object> fields, int fontReduction,
		int startPage);

	/**
	 * Print an objects list with its given list of fields.
	 *
	 * @param objects       List of printed objects.
	 * @param fields        List of printed fields
	 * @param fontReduction Font reduction factor.
	 * @param startPage     Page number.
	 */
	public abstract void print(Vector<LyObject> objects, Vector<Object> fields, int fontReduction);
} // end class LyPrintManager
