/*
 * LyMessageWindowInterface.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.setinterface;

import java.util.*;
import leon.view.event.*;


/**
 * This interface is used to implement a message window.
 */
public interface LyMessageWindowInterface
{
	String __VERSION = "$Id: LyMessageWindowInterface.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Accessor to the client Data.
	 *
	 * @return the client data
	 */
	Object getClientData();

	/**
	 * Set the client data.
	 *
	 * @param clientData
	 */
	void setClientData(Object clientData);

	/**
	 * Show the message window.
	 *
	 * @param status    the visible status of the message box
	 * @param newThread true if the window must be started in a new thread
	 */
	void setVisible(boolean status, boolean newThread);

	/**
	 * Set the icon of the message box.
	 *
	 * @param icon
	 */
	void setIcon(String icon);

	/**
	 * Set the message displayed in the message box.
	 *
	 * @param message
	 */
	void setMessage(String message);

	/**
	 * Set the title of the message box.
	 *
	 * @param title
	 */
	void setTitle(String title);

	/**
	 * Add a listener on event of the message box.
	 *
	 * @param listener
	 */
	void addActionListener(LyActionListener listener);

	/**
	 * Get the result of the message box (which button was pressed..).
	 *
	 * @return the result of the question
	 * @see    leon.view.LyViewManager
	 */
	int getCommandResult();

	/**
	 * Set the commands (buttons) that are present in the message box.
	 *
	 * @param commands
	 * @see   leon.view.LyViewManager
	 */
	void setCommands(BitSet commands);

	/**
	 * Set the background color of the message box.
	 *
	 * @param bgColor
	 */
	void setBackgroundColor(String bgColor);

	/**
	 * Set the modal state of the message window.
	 *
	 * @param modal
	 */
	void setModal(boolean modal);

	/**
	 * Initialize the window This method is called when all properties are set.
	 */
	void initialize();

	/**
	 * Free the window. This method is called to hide and free the window.
	 */
	void free();
}