/*
 * LySetTableInterface.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.setinterface;

import leon.view.*;


/**
 * Abstract access to a method. Hides the specific implementation of the view (win, swing, html...)
 * A value, and may have its value set.
 *
 * @version $Id: LySetTableInterface.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public interface LySetTableInterface
{
	String __VERSION = "$Id: LySetTableInterface.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Adds the given row at the end of the current table.
	 *
	 * @param row the row to add
	 */
	void addRow(LyRow row);

	/**
	 * Adds the given row at the given position in the current table.
	 *
	 * @param row the row to add
	 * @param idx the position where the row has to be added
	 */
	void addRow(LyRow row, int idx);

	/**
	 * Removes the given row from the current table.
	 *
	 * @param row the row to remove
	 */
	void removeRow(LyRow row);

	/**
	 * Removes the row at the given index from the current table.
	 *
	 * @param idx the index of the row to remove
	 */
	void removeRow(int idx);

	/**
	 * Swaps the rows at the specified indexes.
	 *
	 * @param idx1 the position of the first row to swap
	 * @param idx2 the position of the second row to swap
	 */
	void swapRows(int idx1, int idx2);

	/**
	 * Selects or unselects the specified row in the table.
	 *
	 * @param row    the row to select or unselect
	 * @param select true if the row has to be selected, false otherwise
	 */
	void selectRow(LyRow row, boolean select);

	/**
	 * Selects or unselects the row at the given index of the table.
	 *
	 * @param idx    index of the row to select or unselect
	 * @param select true if the row has to be selected, false otherwise
	 */
	void selectRow(int idx, boolean select);

	/**
	 * Selects or unselects all objects of the table.
	 *
	 * @param select     true if all objects have to be selected, false otherwise
	 * @param updateView indicates if the view must be updated or not.
	 */
	void selectAll(boolean select, boolean updateView);

	/**
	 * Scrolls to make visible to the user the row at the given index .
	 *
	 * @param idx the index of the row to show
	 */
	void showRow(int idx);

	/**
	 * Hides or show the row at the given index.
	 *
	 * @param idx  the index of the row to show or hide
	 * @param hide indicates if the row has to be shown or hided
	 */
	void hideRow(int idx, boolean hide);

	/**
	 * Freezes the column at the given index in the view.
	 *
	 * @param idx the index of the column to freeze
	 */
	void freezeColumn(int idx);

	/**
	 * Updates the given row.
	 *
	 * @param row the row to update
	 */
	void updateRow(LyRow row);
}