/**
 * LyWebApplication.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved.
 */

package leon.view.web;

import leon.app.*;
import leon.misc.*;


/**
 * The instance of an application for web viewers. This instance is created by {@link
 * leon.view.web.LyServlet} and is used to manage a lock to avoid freeing the application before the
 * end of a HttpRequest.
 *
 * @version $Id: LyWebApplication.java 31874 2011-08-03 10:03:44Z kchenal $
 * @author  Lyria
 * @see     leon.app.LyApplication
 */
public class LyWebApplication extends LyApplication
{
	public static final String __VERSION =
		"$Id: LyWebApplication.java 31874 2011-08-03 10:03:44Z kchenal $";

	/** Counter of lock request. */
	int						   _lock;

	/** Flag to know if we must free the application when the last lock is freed. */
	boolean					   _free = false;

	/**
	 * Constructor with a given id and configuration file.
	 *
	 * @param id         The identifier of the application.
	 * @param configFile The configuration file of the application.
	 * @param path       The initial path for the application .
	 */
	public LyWebApplication(String id, String configFile, String path)
	{
		super(id, configFile, path, null);
	}
	
	/**
	 * Constructor with a given id and configuration file.
	 *
	 * @param id         The identifier of the application.
	 * @param configFile The configuration file of the application.
	 * @param path       The initial path for the application .
	 */
	public LyWebApplication(String id, String configFile, String path, boolean isMobile)
	{
		super(null, null, id, configFile, path, null, isMobile);
	}

	/**
	 * Indicates whether the application is valid or not.
	 *
	 * @return true if the application is valid, false otherwise.
	 */
	public int lock()
	{
		_lock++;
		return _lock;
	}

	/**
	 * Indicates whether the application is valid or not.
	 *
	 * @return true if the application is valid, false otherwise.
	 */
	public int unlock()
	{
		_lock--;

		if (_lock < 0)
		{
			LyEnvironment env = getEnvironment();
			if ((env != null) && env.isDebugLevel(LyEnvironment.DEBUG_WARNING))
				env.logWarning("[LyWebApplication] Problem with lock/unlock");
		}
		
		if (_free && (_lock <= 0))
			free();

		return _lock;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.app.LyApplication#free()
	 */
	protected synchronized void free()
	{
		if (_lock <= 0)
			super.free();
		else
			_free = true;
	}
} // end class LyWebApplication
