/*
 * LyWebConstants.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Version: $Id: LyWebConstants.java 35391 2012-04-27 15:19:50Z nhamon $
 */
package leon.view.web;

/**
 * Type : leon.view.web.struts.LyWebConstants<br>
 * This class defines all the static constants used in the WEB view managers.<br>
 * Are defined here:
 *
 * <ul>
 *   <li>All the Leonardi request parameters;</li>
 *   <li>All the frame elements;</li>
 *   <li>Some other usefull constants for debug.</li>
 * </ul>
 *
 * @version $Id: LyWebConstants.java 35391 2012-04-27 15:19:50Z nhamon $
 * @author  Lyria S.A.
 */
public abstract class LyWebConstants
{
	public static final String __VERSION =
		"$Id: LyWebConstants.java 35391 2012-04-27 15:19:50Z nhamon $";

	/** The request parameter used to define an action in the HTTP request.<br>
	 */
	public static final String ACTION = "_action".intern();

	/** The <code>about</code> action.<br>
	 */
	public static final String ACTION_ABOUT = "_about".intern();

	/** The exit action.<br>
	 */
	public static final String ACTION_EXIT = "_exit".intern();

	/** The cancel command in a form.<br>
	 */
	public static final String ACTION_FORM_CANCEL = "_cancel".intern();

	/** The relaod page action.<br>
	 */
	public static final String ACTION_RELOAD_PAGE = "_reloadPage".intern();

	/** The show root page action.<br>
	 */
	public static final String ACTION_ROOT_PAGE = "_rootPage".intern();

	/**
	 * The request parameter used to define the ajax mode in the HTTP request.<br>
	 * Possible values are <code>true</code> or <code>false</code>.
	 */
	public static final String AJAX = "_ajax".intern();

	/**
	 * The request parameter used to define an answer in the HTTP request.<br>
	 * Possible values are <code>true</code> or <code>false</code>.
	 */
	public static final String ANSWER = "_answer".intern();

	/**
	 * The request parameter used to define if an action is explicitly called in the HTTP request.
	 * <br>
	 * Possible values are a valid action ID or null in case of graphical event.
	 */
	public static final String CALL_ACTION = "_call_action".intern();

	/**
	 * The request parameter used to define if an action is explicitly called in the HTTP request.
	 * <br>
	 * Possible values are a valid action ID or null in case of graphical event.
	 */
	public static final String CALL_ACTION_NO_GUI = "_call_action_no_gui".intern();

	/**
	 * Internal request parameter : indicates that the hidden frame has been set and CALL_ACTION can
	 * be invoked.
	 */
	public static final String CALL_ACTION_READY = "_call_action_ready".intern();

	/** ClassInfo parameter when using URL Navigation (optional). */
	public static final String CLASSINFO = "_classInfo".intern();

	/**
	 * Specify an identifier for the objects in the context for URL Navigation : this parameter is
	 * optional AND multiple. Is may be used simultaneously with OBJECTNAME.
	 */
	public static final String OBJECTID = "_objectId".intern();

	/**
	 * Specify the name for the objects in the contextt for URL Navigation : this parameter is optional
	 * AND multiple. Is may be used simultaneously with OBJECTID.
	 */
	public static final String OBJECTNAME = "_objectName".intern();

	/** Display up page parameter when using URL Navigation (optional). */
	public static final String UP_PAGE = "_upPage".intern();

	/** Specify to exit the call action mode an restore a new leonardi session (regular mode). */
	public static final String NEW_SESSION = "_new_session".intern();

	/**
	 * The request parameter used to define a command in the HTTP request.<br>
	 * Possible values are <code>true</code> or <code>false</code>.
	 */
	public static final String COMMAND = "_command".intern();

	/** The command button HTML name. */
	public static final String COMMAND_ITEM = "LyCommandItem.".intern();

	/**
	 * The <code>Content-Type</code> value <code>multipart/form-data</code>.<br>
	 * Content-Type is an attribute of the FORM HTML element who specifies the content type used to
	 * encode the form data set for submission to the server.
	 */
	public static final String CONTENT_MULTIPART = "multipart/form-data".intern();

	/**
	 * The request parameter used to define the identifier of the Leonardi controller involved in
	 * the HTTP request.
	 */
	public static final String CONTROLLER = "_controller".intern();

	/** Constant duplicated from class LyController. The value has changed. */
	public static final String CONTROLLER_ID_SEP = "::".intern();

	/** Parameter used to significate that the frame must be closed. */
	public static final String FRAME_CONTROLLER_DEMAND_CLOSE = "_close".intern();
	
	/** Parameter used to significate that the frame must be closed. */
	public static final String FILE = "_file".intern();

	/**
	 * The request parameter used to define the session default action in the HTTP request.<br>
	 * Possible values are a valid action id or null.
	 */
	public static final String DEFAULTACTION = "_defaultAction".intern();

	/**
	 * The request parameter used to define the extension of a sort or a selection.<br>
	 * Possible values are <code>true</code> or <code>false</code>.
	 */
	public static final String EXTEND = "_extend".intern();

	/**
	 * The request parameter used to define the path name of a resource to forward.<br>
	 * The pathname must begin with a "/" and is interpreted as relative to the current context
	 * root.
	 */
	public static final String FORWARD = "_forward".intern();

	/** The request parameter used to define a frame in the HTTP request. */
	public static final String FRAME = "_frame".intern();

	/** The name of the header frame in the Leonardi frameset. */
	public static final String HEADER_FRAME = "pilot_header_frame".intern();

	/**
	 * The request parameter used to define the hidden properties in the HTTP request. Possible
	 * values are <code>true</code> or <code>false</code>
	 */
	public static final String HIDDEN = "_hidden".intern();

	/** The name of a hidden frame in the Leonardi frameset. */
	public static final String HIDDEN_FRAME = "hidden_frame".intern();

	/** The request parameter used to define the index of a graphical event in the HTTP request. */
	public static final String INDEX = "_index".intern();

	/** The request parameter used to define the application language in the HTTP request. */
	public static final String LANGUAGE = "_language".intern();

	/** The name of the main frame in the Leonardi frameset. */
	public static final String MAIN_FRAME = "main_frame".intern();

	/**
	 * The request parameter used to define a message in the HTTP request.<br>
	 * The value is a string representing the message.
	 */
	public static final String MESSAGE = "_message".intern();

	/** The name of the navigation frame in the Leonardi frameset. */
	public static final String NAV_FRAME = "navigation_frame".intern();

	/** A key to get the appropriate page writer id. */
	public static final String PAGE = "_page".intern();

	/** The name of the pilot frame in the Leonardi frameset. */
	public static final String PILOT_FRAME = "pilot_frame".intern();

	/** Used within URLs to introduce the popup menu action. */
	public static final String POPUP_MENU = "_popupMenu".intern();

	/** Used within URLs to introduce the extend popup menu action. */
	public static final String POPUP_MENU_EXTEND = "_popupMenuExtend".intern();

	/** The name of a pop up view in the Leonardi frameset. */
	public static final String POPUP_VIEW = "popup_view".intern();

	/** The request parameter used to define the x position in the HTTP request. */
	public static final String POSITION_X = "_positionX".intern();

	/** The request parameter used to define the y position in the HTTP request. */
	public static final String POSITION_Y = "_positionY".intern();

	/**
	 * The key of the request attribute (R.A.) that contains the url for the application doc
	 * (LY_APP_DOC) directory.
	 */
	public static final String RA_LY_APP_DOC_URL = "appRootURL".intern();

	/**
	 * The key of the request attribute (R.A.) that contains the url for the leonardi doc
	 * (LY_LEON_DOC) directory.
	 */
	public static final String RA_LY_LEON_DOC_URL = "leonardiRootURL".intern();

	/**
	 * The key of the request attribute (R.A.) that contains the url for the leonardi doc external
	 * (_LY_LEON_DOC_EXTERNAL) directory.
	 */
	public static final String RA_LY_LEON_DOC_EXTERNAL_URL = "leonardiExternalURL".intern();

	/** The key of the request attribute (R.A.) that contains the root url of the view manager. */
	public static final String RA_LY_ROOT_URL = "servletRootURL".intern();

	/** The key of the request attribute (R.A.) that contains the user Leonardi session. */
	public static final String RA_LY_SESSION = "leonardiSession".intern();

	/**
	 * The key of the request attribute (R.A.) that contains the url for the web skin file.
	 */
	public static final String RA_LY_SKIN_URL = "skinRootURL".intern();

	/**
	 * The key of the request attribute (R.A.) that contains the application name.
	 */
	public static final String RA_LY_APP_NAME = "appName".intern();

	/**
	 * The request parameter used to define the refresh attribute in the HTTP request. Possible
	 * values are <code>true</code> or <code>false</code>.
	 */
	public static final String REFRESH = "_refresh".intern();

	/**
	 * The request parameter used to define the reload rate in the HTTP request. Value is a {@link
	 * java.util.Date}
	 */
	public static final String RELOAD = "_reload".intern();

	/**
	 * The request parameter used to indicates if frames needs reload in the HTTP request.<br>
	 * The parameter values are <code>true</code> or <code>false</code>.<br>
	 */
	public static final String RELOAD_FRAMES = "reload_frames".intern();

	/** The select all action. */
	public static final String SELECT_ALL = "_selectAll".intern();

	/** The select action. */
	public static final String SELECT_ITEM = "_select".intern();

	/** The request parameter used to define the application skin in the HTTP request. */
	public static final String SKIN = "_skin".intern();

	/**
	 * The request parameter used to define the source component in the User Interface that
	 * initiated an HTTP request.<br>
	 */
	public static final String SOURCE = "_source".intern();

	// CLEANCODE : SELECT_PAGE is not used
//      /**/
//      public static final String      SELECT_PAGE                                             = "_select_page".intern();

	// CLEANCODE : SELECTION is not used
//      /**/
//      public static final String      SELECTION                                               = "_selection".intern();

	/** Used to introduce values of sort parameter in URLs. */
	public static final String SORT = "_sort".intern();

	/**
	 * Used within the <b>name</b> attribute of &lt;input&gt; HTML tags that represents selection
	 * items.
	 */
	public static final String SELECTION_ITEM = "LySelectionItem.".intern();

	/**
	 * Used within URLs to indicate value of parameters defined with the {@link #SOURCE} constant.
	 */
	public static final String STATUS = "_status".intern();

	/** Separator used for sub-index values. */
	public static final String SUB_INDEX_SEPARATOR = "_subIndexSeparator_".intern();

	/** Used within the <b>name</b> attribute of an &lt;input&gt; HTML tags. */
	public static final String SUBMITTED = "_submitted".intern();
	
	/** Used to create multiple Leonardi sessions for a unique Http session. */
	public final static String TARGET_PARAM = "target_session";

	/** Used within the <b>name</b> attribute of an &lt;input&gt; HTML tags. */
	public static final String TARGET_CONTROLLER = "_target_controller".intern();

	/** The name of the target frame in the Leonardi frameset. */
	public static final String TARGET_FRAME = "target_frame".intern();

	/** The name of the parameter of a part of the framset to be refreshed. */
	public static final String TARGET_WINDOW = "_target_window".intern();

    /** The name of the parameter used to check if servlet is still alive. */
    public static final String TEST_ALIVE = "_test_alive".intern();

	/** The name of the top frame. */
	public static final String TOP_FRAME = "top_frame".intern();

	/** The request parameter used to define the type of a graphical event in the user interface. */
	public static final String TYPE = "_type".intern();

	/** Used within URLs to introduce the unselect all action. */
	public static final String UNSELECT_ALL = "_unselectAll".intern();

	/** Used within URLs to introduce the unselect action. */
	public static final String UNSELECT_ITEM = "_unselect".intern();

	/** Used to indicates the manipulated frame is the up frame. */
	public static final String UP_FRAME = "up_frame".intern();

	/** Used by upload mechanism. */
	public static final String UPLOAD = "_upload".intern();

	/** Used to introduce values of a value attribute in URLs. */
	public static final String VALUE = "_value".intern();

	/** Key used to store the Web dashboard controller identifier into the Leonardi session. */
	public static final String WEB_DASHBOARD = "_dashboard".intern();

	/**
	 * A possible value associated to the key {@link #WEB_DASHBOARD} that indicates there is not any
	 * dashboard.
	 */
	public static final String WEB_NO_DASHBOARD = "NO_DASHBOARD".intern();

	/** The identifier of the popup used for tools are. */
	public static final String TOOL_POPUP = "_toolPopup".intern();

	/** Delay for pages expiration. */
	public static int		   DELAY = 5000;

	/** The total number of requests receied : used for debug. */
	public static int		   REQU_COUNT = 0;
	
	/** The time to wait in milliseconds before wait shadow is shown. */
	public static int		   WAIT_SHADOW_DELAY = 1000;
	
	/** The time to wait in milliseconds before wait panel is shown. */
	public static int		   WAIT_PANEL_DELAY = 1000;

	/**
	 * Page counter to dump written HTML pages to disk : -1 means no DUMP, -2 means check
	 * environment, 0 meand start dump Activate this with debug mode DUMP_HTML.
	 */
	public static int		   WRITTEN_PAGES_COUNTER = -2;

	/** Name of the javascript file for the actions. */
	public static final String JS_ACTIONS = "LyActionScript.js";

	/** Name of the javascript file for the messages. */
	public static final String JS_MESSAGE = "LyMessageScript.js";

	/** Name of the javascript file for the forms. */
	public static final String JS_FORM = "LyFormScript.js";

	/** Name of the javascript file for the pages. */
	public static final String JS_PAGE = "LyPageScript.js";

	/** Name of the javascript file for strings. */
	public static final String JS_STRING = "LyStringScript.js";

	/** Name of the javascript file for trees. */
	public static final String JS_TREE = "LyTreeScript.js";

	/** Name of the javascript file for utils functions. */
	public static final String JS_UTILS = "LyUtilsScript.js";

	/**
	 * Id of the portlet instance in Business Visual (BV mode only)
	 */
	public static final String KEY_PORTLET_VIEW_ID = "LEONARDI_PORTLET_VIEW_ID";

	/**
	 * Callback url (for AJAX) of the portlet instance in Business Visual (BV mode only)
	 */
	public static final String KEY_PORTLET_ACTION_URL = "LEONARDI_PORTLET_ACTION_URL";
	
	/** The request parameter used to define the type of access. */
	public static final String MOBILE = "_mobile".intern();

} // end class LyWebConstants
