/*
 * LyWebDecoration.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved.
 */

package leon.view.web;

import java.io.*;
import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.web.struts.*;


/**
 * This class defines static parts of the web decoration switch the choosen skin. It must be
 * inherited in order to generate HTML pages.
 *
 * @version $Id: LyWebDecoration.java 29006 2010-12-23 17:44:46Z bmichel $
 * @author  Lyria
 */
public class LyWebDecoration
{
	public static final String	    __VERSION =
		"$Id: LyWebDecoration.java 29006 2010-12-23 17:44:46Z bmichel $";

	/** Decoration code relative to the window begining. */
	public static final int		    WINDOW_BEGIN = 0;

	/** Decoration code relative to the window end. */
	public static final int		    WINDOW_END = 1;

	/** Decoration code relative to the window left side. */
	public static final int		    WINDOW_LEFT = 2;

	/** Decoration code relative to the window right side. */
	public static final int		    WINDOW_RIGHT = 3;

	/** Decoration code relative to the window top. */
	public static final int		    WINDOW_TOP = 4;

	/** Decoration code relative to the window bottom. */
	public static final int		    WINDOW_BOTTOM = 5;

	/** Decoration code relative to the left window's header side. */
	public static final int		    WINDOW_HEADER_LEFT = 6;

	/** Decoration code relative to the right window's header side. */
	public static final int		    WINDOW_HEADER_RIGHT = 7;

	/** Decoration code relative to the header bar begining. */
	public static final int		    HEADER_BAR_BEGIN = 8;

	/** Decoration code relative to the header bar end. */
	public static final int		    HEADER_BAR_END = 9;

	/** Decoration code relative to the content begining. */
	public static final int		    CONTENT_BEGIN = 10;

	/** Decoration code relative to the content begining with full height. */
	public static final int		    CONTENT_BEGIN_FULL_HEIGHT = 11;

	/** Decoration code relative to the top of the content. */
	public static final int		    CONTENT_TOP = 12;

	/** Decoration code relative to the left of the content. */
	public static final int		    CONTENT_LEFT = 13;

	/** Decoration code relative to the right of the content. */
	public static final int		    CONTENT_RIGHT = 14;

	/** Decoration code relative to the content footer. */
	public static final int		    CONTENT_FOOTER = 15;

	/** Decoration code relative to the content end. */
	public static final int		    CONTENT_END = 16;

	/** Decoration code relative to the form field begining. */
	public static final int		    FORM_FIELDS_BEGIN = 17;

	/** Decoration code relative to the form field end. */
	public static final int		    FORM_FIELDS_END = 18;

	/** Decoration code relative to the toolbar label. */
	public static final int		    TOOLBAR_LABEL = 19;

	/** Decoration code relative to the toolbar end of the label. */
	public static final int		    TOOLBAR_LABEL_END = 20;

	/** Decoration code relative to the window begining without a full height. */
	public static final int		    WINDOW_BEGIN_NOT_FULL_HEIGHT = 21;

	/** Decoration code relative open zone. */
	public static final int		    OPEN_ZONE_1 = 22;

	/** Decoration code relative close zone. */
	public static final int		    CLOSE_ZONE = 23;

	/** Decoration code relative inner window begining. */
	public static final int		    INNER_WINDOW_BEGIN = 24;

	/** Decoration code relative to area border begining. */
	public static final int		    AREA_BORDER_BEGIN = 25;

	/** Decoration code relative to area border begining without a full width. */
	public static final int		    AREA_BORDER_BEGIN_NOT_FULL_WIDTH = 26;

	/** Decoration code relative to area border begining without a full height. */
	public static final int		    AREA_BORDER_BEGIN_NOT_FULL_HEIGHT = 27;

	/** Decoration code relative to area border begining without a full width and height. */
	public static final int		    AREA_BORDER_BEGIN_NOT_FULL_WIDTH_AND_HEIGHT = 28;

	/** Decoration code relative to area border left side. */
	public static final int		    AREA_BORDER_LEFT = 29;

	/** Decoration code relative to area border left side without a full width. */
	public static final int		    AREA_BORDER_LEFT_NOT_FULL_WIDTH = 30;

	/** Decoration code relative to area border left side without a full height. */
	public static final int		    AREA_BORDER_LEFT_NOT_FULL_HEIGHT = 31;

	/** Decoration code relative to area border left side without a full width and height. */
	public static final int		    AREA_BORDER_LEFT_NOT_FULL_WIDTH_AND_HEIGHT = 32;

	/** Decoration code relative to area border right side. */
	public static final int		    AREA_BORDER_RIGHT = 33;

	/** Decoration code relative to area border end. */
	public static final int		    AREA_BORDER_END = 34;

	/** Decoration code relative to vertical toolbar begining. */
	public static final int		    VERTICAL_TOOLBAR_BEGIN = 35;

	/** Decoration code relative to vertical toolbar left side. */
	public static final int		    VERTICAL_TOOLBAR_LEFT = 36;

	/** Decoration code relative to vertical toolbar right side. */
	public static final int		    VERTICAL_TOOLBAR_RIGHT = 37;

	/** Decoration code relative to vertical toolbar end. */
	public static final int		    VERTICAL_TOOLBAR_END = 38;

	/** Decoration code relative to horizontal toolbar begining. */
	public static final int		    HORIZONTAL_TOOLBAR_BEGIN = 39;

	/** List of CSS files generated from their model. */
	protected static Hashtable<String, String> _generatedSkinFiles = new Hashtable<String, String>();

	/**
	 * Method used to get code of the related closing decoration.
	 *
	 * @param  whatToShow the code of the opening decoration
	 * @return the closing decoration code or -1 if no match found
	 */
	public static int getClosingDecoration(int whatToShow)
	{
		switch (whatToShow)
		{
		case WINDOW_BEGIN:
			return WINDOW_END;
		case WINDOW_LEFT:
			return WINDOW_RIGHT;
		case WINDOW_TOP:
			return WINDOW_BOTTOM;
		case WINDOW_HEADER_LEFT:
			return WINDOW_HEADER_RIGHT;
		case HEADER_BAR_BEGIN:
			return HEADER_BAR_END;
		case CONTENT_BEGIN:
			return CONTENT_END;
		case CONTENT_BEGIN_FULL_HEIGHT:
			return CONTENT_END;
		case CONTENT_TOP:
			return CONTENT_FOOTER;
		case CONTENT_LEFT:
			return CONTENT_RIGHT;
		case FORM_FIELDS_BEGIN:
			return FORM_FIELDS_END;
		case OPEN_ZONE_1:
			return CLOSE_ZONE;

		default:
			return -1; // No match
		}
	} // end method getClosingDecoration

	/**
	 * Method used to get code of the related opening decoration.
	 *
	 * @param  whatToShow the code of the closing decoration
	 * @return the closing decoration code or -1 if no match found
	 */
	public static int getOpeningDecoration(int whatToShow)
	{
		switch (whatToShow)
		{
		case WINDOW_END:
			return WINDOW_BEGIN;
		case WINDOW_RIGHT:
			return WINDOW_LEFT;
		case WINDOW_BOTTOM:
			return WINDOW_TOP;
		case WINDOW_HEADER_RIGHT:
			return WINDOW_HEADER_LEFT;
		case HEADER_BAR_END:
			return HEADER_BAR_BEGIN;
		case CONTENT_END:
			return CONTENT_BEGIN;
		case CONTENT_FOOTER:
			return CONTENT_TOP;
		case CONTENT_RIGHT:
			return CONTENT_LEFT;
		case FORM_FIELDS_END:
			return FORM_FIELDS_BEGIN;
		case CLOSE_ZONE:
			return OPEN_ZONE_1;

		default:
			return -1; // No match
		}
	} // end method getOpeningDecoration

	/** Current skin CSS file. */
	protected String	    _skinFile;

	/** URL of specific CSS file. */
	protected String	    _specificCSSFile;

	/** The application environment. */
	protected LyEnvironment _environment;

	/**
	 * Default protected constructor. Use getInstance() to acces to this decoration. Sets the
	 * environment.
	 *
	 * @param environment the application environment
	 */
	public LyWebDecoration(LyEnvironment environment)
	{
		setEnvironment(environment);
	}

	/**
	 * Sets the current environment.
	 *
	 * @param environment the environment
	 */
	public void setEnvironment(LyEnvironment environment)
	{
		String skinFile = environment.getFile("LY_CSS_FILE");

		if ((skinFile != null) && (skinFile.trim().length() == 0))
			skinFile = null;

		if (skinFile != null)
			skinFile = environment.getString().expand(skinFile, "/");

		LyWebViewManager viewManager = (LyWebViewManager)environment.getViewManager();

		if (viewManager.getBrowser() != LyWebViewManager.BROWSER_MSIE)
			skinFile = skinFile.replace('\\', '/');

		_specificCSSFile = environment.getUrl("LY_SPECIFIC_CSS_FILE");

		if ("LY_SPECIFIC_CSS_FILE".equals(_specificCSSFile))
			_specificCSSFile = null;

		setSkinFile(skinFile, environment);
	}

	/**
	 * Sets the skin and initialize the environment.
	 *
	 * @param skinFile    the skin
	 * @param environment the environment
	 * @see   #getSkinFile()
	 */
	public void setSkinFile(String skinFile, LyEnvironment environment)
	{
		_skinFile = skinFile;
		init(environment);
	}

	/**
	 * Initialization: gets the skin and init window and content top & bottom height and right &
	 * left width.
	 *
	 * @param environment the application environment
	 */
	protected void init(LyEnvironment environment)
	{
		_environment = environment;

		if (_skinFile == null)
			return;

		synchronized (_generatedSkinFiles)
		{
			if (!_generatedSkinFiles.containsKey(_skinFile))
				generateCssFile(_skinFile);

			if (_specificCSSFile == null)
				return;
			
			if (!_generatedSkinFiles.containsKey(_specificCSSFile))
				generateCssFile(_specificCSSFile);
		}
	} // end method init

	/**
	 * Generate a skin css file from its model. Model may contain color, image and font resources.
	 */
	protected synchronized void generateCssFile(String skinFile)
	{
		// Retrieve CSS file path from the CSS file URL
		String leonDocDir = _environment.getString().expand("$LY_LEON_DOC_DIR$");
		String leonDoc = _environment.getString().expand("$LY_LEON_DOC$");
		String appDocDir = _environment.getString().expand("$LY_APP_DOC_DIR$");
		String appDoc = _environment.getString().expand("$LY_APP_DOC$");
		String modelFilePath = skinFile;

		// FFT V3.3 0012 : use URL patterns
		modelFilePath = modelFilePath.replace('\\', '/');
		leonDoc = leonDoc.replace('\\', '/');
		appDoc = appDoc.replace('\\', '/');

		// Check if skin file is in one of leon or app dir
		if (modelFilePath.startsWith(leonDoc))
			modelFilePath = leonDocDir + modelFilePath.substring(leonDoc.length());
		else if (modelFilePath.startsWith(appDoc))
			modelFilePath = appDocDir + modelFilePath.substring(appDoc.length());

		//FFT V3.3 0012 : use File patterns now
		modelFilePath = modelFilePath.replace('/', File.separatorChar);

		File modelFile = new File(modelFilePath);

		// If model file does not exist, display an error message
		if (!modelFile.exists())
		{
			_environment.logWarning("The css file '" + modelFilePath
				+ "' could not be overwritten with applicative resources.");
			return;
		}

		try
		{
			InputStream  in = new BufferedInputStream(new FileInputStream(modelFilePath));
			int			 nRead;
			StringBuffer cssModelContentBuffer = new StringBuffer();
			char		 startChar = '$', sepChar = ':';
			String		 colorType = "COLOR", imageType = "IMAGE", fontType = "FONT";

			// Read the content of the model file
			do
			{
				nRead = in.read();

				if (nRead >= 0)
					cssModelContentBuffer.append((char)nRead);
			}
			while (nRead >= 0);

			in.close();

			Writer		 wr = new BufferedWriter(new FileWriter(modelFilePath));
			PrintWriter  p = new PrintWriter(wr);

			// Replace all the occurences corresponding to Leonardi resources
			String		 cssModelContent = cssModelContentBuffer.toString();
			String		 typeString, valueString;
			StringBuffer cssContent = new StringBuffer();
			int			 beginPos, sepPos, endPos;

			while ((beginPos = cssModelContent.indexOf(startChar)) >= 0)
			{
				// Retrieve both type and value of the resource
				sepPos = cssModelContent.indexOf(sepChar, beginPos);
				endPos = cssModelContent.indexOf(startChar, sepPos);
				typeString = cssModelContent.substring(beginPos + 1, sepPos);
				valueString = cssModelContent.substring(sepPos + 1, endPos);

				// Append part of the buffer before the declaration of the resource
				cssContent.append(cssModelContent.substring(0, beginPos));

				// Convert resource with its value from environment properties
				if (colorType.equals(typeString))
				{
					LyColor color = _environment.getLyColor(valueString);

					if (color != null)
					{
						cssContent.append("#");
						cssContent.append(LyColors.getHexString(color));
					}
				}
				else if (imageType.equals(typeString))
					cssContent.append(_environment.getImageUrl(valueString));
				else if (fontType.equals(typeString))
				{
					LyFont		 font = _environment.getLyFont(valueString);
					StringBuffer buffer = new StringBuffer();

					buffer.append("font-family:").append(font.getFontName()).append(";");
					buffer.append("font-size:").append(font.getSize()).append("px;");

					if (font.isBold())
						buffer.append("font-weight:bold;");

					if (font.isItalic())
						buffer.append("font-style:italic;");

					cssContent.append(buffer.toString());
				}

				cssModelContent = cssModelContent.substring(endPos + 1);
			}

			cssContent.append(cssModelContent);

			p.print(cssContent);
			p.close();
			wr.close();

			// Add the generated file to the static list of generated CSS files
			_generatedSkinFiles.put(skinFile, modelFilePath);
		}
		catch (Exception e)
		{
			_environment.logError("The css file '" + modelFilePath
				+ "' was not generated correctly (error : " + e.getMessage() + ").");
		}
	} // end method generateCssFile

	/**
	 * Method used to set the decoration (skin) into the specified writer.
	 *
	 * @param pw         writer to write piece of decoration to
	 * @param whatToShow what part of the decoration shall be written (see static values)
	 */
	public void writePageDecoration(LyWebPageWriter pw, int whatToShow)
	{
		writePageDecoration(pw, whatToShow, null);
	}

	/**
	 * Method used to set the decoration (skin) into the specified writer.
	 *
	 * @param pw         writer to write piece of decoration to
	 * @param whatToShow what part of the decoration shall be written (see static values)
	 * @param param      parameter of the decoration to be produced
	 */
	public void writePageDecoration(LyWebPageWriter pw, int whatToShow, Object param)
	{
		if (!needPageDecoration(pw, whatToShow))
			return;
		pw.append(getPageDecoration(whatToShow, param));
	}

	/**
	 * Method used to set the decoration (skin) into the specified writer.
	 *
	 * @param  pw         writer to write piece of decoration to
	 * @param  whatToShow what part of the decoration shall be written (see static values)
	 * @throws IOException
	 */
	public void writePageDecoration(Writer pw, int whatToShow)
		throws IOException
	{
		pw.write(getPageDecoration(whatToShow));
	}

	/**
	 * Method used to check if we may apply this decoration or not (avoid duplicate border).
	 *
	 * @param  pw         writer to write piece of decoration to
	 * @param  whatToShow what part of the decoration shall be written (see static values)
	 * @return true if Ok to add decoration
	 */
	public boolean needPageDecoration(LyWebPageWriter pw, int whatToShow)
	{
		int opening = getOpeningDecoration(whatToShow);
		int closing = getClosingDecoration(whatToShow);

		if (closing >= 0)
		{
			opening = getOpeningDecoration(closing); // Because we may have
			// several opening for
			// the same closing
			String key = "ADDED_PAGE_DECORATION_" + opening;

			// Opening new branch
			if (pw.getProperty(key) != null)
			{
				String countStr = pw.getProperty(key);

				int    count = LyTools.intFromString(countStr);

				if (count > 0)
				{
					// Increment counter
					count++;
					pw.setProperty(key, Integer.toString(count));
				}
				// CLEANCODE : s is never read
				//else
				//{
				//      String s = "error"; // Should never happen
				//}
				return false;
			}
			else
			{
				pw.setProperty(key, "1"); // First instance
				return true;
			}
		}
		else if (opening >= 0)
		{
			String key = "ADDED_PAGE_DECORATION_" + opening;

			// Opening new branch
			if (pw.getProperty(key) != null)
			{
				String countStr = pw.getProperty(key);

				int    count = LyTools.intFromString(countStr);

				if (count > 1)
				{
					// Decrement counter
					count--;
					pw.setProperty(key, Integer.toString(count));
					return false;
				}
				else
				{
					pw.setProperty(key, null); // No more counter for opening
					// deco
					return true;
				}
			}
			// CLEANCODE : s is never read
			//else
			//{
			//      String s = "error"; // Should never happen
			//}
		}

		return true;
	} // end method needPageDecoration

	/**
	 * Method used to produce the decoration (skin) as a String.
	 *
	 * @param  whatToShow decoration code of the part that shall be written (see static values)
	 * @return the HTML result
	 */
	public String getPageDecoration(int whatToShow)
	{
		return getPageDecoration(whatToShow, null);
	}

	/**
	 * Method used to produce the decoration (skin) as a String.
	 *
	 * @param  whatToShow decoration code of the part that shall be written (see static values)
	 * @param  param      parameter of the decoration to be produced
	 * @return the HTML result
	 */
	public String getPageDecoration(int whatToShow, Object param)
	{
		if (_skinFile == null)
			return "";

		StringBuffer sb = new StringBuffer();

		if (whatToShow == WINDOW_BEGIN) // Header bar begin
			sb.append(
				"<!-- skin window begin -->\n<table width='100%' height='100%' border='0' cellspacing='0' cellpadding='0'>\n");

		if (whatToShow == INNER_WINDOW_BEGIN) // Header bar begin
			sb.append(
				"<!-- skin window begin -->\n<table width='100%' border='0' cellspacing='0' cellpadding='0'>\n");
		if (whatToShow == WINDOW_BEGIN_NOT_FULL_HEIGHT)
			sb.append(
				"<!-- skin window begin not full height-->\n<table width='100%' border='0' cellspacing='0' cellpadding='0'>\n");
		if (whatToShow == WINDOW_LEFT) // Header bar begin
			whatToShow = WINDOW_HEADER_LEFT;
		if (whatToShow == WINDOW_RIGHT) // Header bar end
			whatToShow = WINDOW_HEADER_RIGHT;

		if (whatToShow == WINDOW_HEADER_LEFT) // Header bar begin
			sb.append("<tr><td valign='top'>\n");
		if (whatToShow == WINDOW_HEADER_RIGHT) // Header bar end
		{
			sb.append("</td>\n");
			sb.append("</tr>\n");
		}

		if (whatToShow == WINDOW_END) // Header bar end
			sb.append("</table>\n<!-- skin window end -->\n");

		if (whatToShow == HEADER_BAR_BEGIN) // Header bar begin
			sb.append("<table border='0' width='100%' cellspacing='0' cellpadding='0'><tr><td>\n");
		if (whatToShow == HEADER_BAR_END) // Header bar end
			sb.append("</td></tr></table>\n");
		if (whatToShow == CONTENT_BEGIN_FULL_HEIGHT) // content begin
			// BORDER
			sb.append(
				"<!-- skin content begin full height -->\n<table width='100%' height='100%' border='0' cellspacing='0' cellpadding='0'>\n");
		if (whatToShow == CONTENT_BEGIN) // content begin
			// BORDER
			sb.append(
				"<!-- skin content begin -->\n<table width='100%' border='0' cellspacing='0' cellpadding='0'>\n");
		if (whatToShow == CONTENT_TOP) // content top line
		{
			String contentTopLeft = _environment.getImageUrl("LY_CONTENT_TOP_LEFT");
			String contentTop = _environment.getImageUrl("LY_CONTENT_TOP");
			String contentTopRight = _environment.getImageUrl("LY_CONTENT_TOP_RIGHT");

			sb.append("<tr height='1'>\n<td><img src='").append(contentTopLeft).append(
				"'/></td>\n");
			sb.append("<td width='100%' style='background:url(").append(contentTop).append(
				")'></td>\n");
			sb.append("<td><img src='").append(contentTopRight).append("'/></td>\n");
			sb.append("</tr>\n");
		}

		if (whatToShow == CONTENT_LEFT) // content left
		{
			String contentLeft = _environment.getImageUrl("LY_CONTENT_LEFT");

			sb.append("<tr>\n<td style='background:url(").append(contentLeft).append(");'/>\n");
			sb.append("<td width='100%' vAlign='top'>\n");
		}

		if (whatToShow == CONTENT_RIGHT) // content right
		{
			String contentRight = _environment.getImageUrl("LY_CONTENT_RIGHT");

			sb.append("</td>\n");
			sb.append("<td style='background:url(").append(contentRight).append(");'/>\n");
			sb.append("</tr>\n");
		}
		if (whatToShow == CONTENT_FOOTER) // content base line
		{
			String contentBottomLeft = _environment.getImageUrl("LY_CONTENT_BOTTOM_LEFT");
			String contentBottom = _environment.getImageUrl("LY_CONTENT_BOTTOM");
			String contentBottomRight = _environment.getImageUrl("LY_CONTENT_BOTTOM_RIGHT");

			sb.append("<tr height='1'>\n<td><img src='").append(contentBottomLeft).append(
				"'/></td>\n");
			sb.append("<td width='100%' style='background:url(").append(contentBottom).append(
				")'></td>\n");
			sb.append("<td><img src='").append(contentBottomRight).append("'/></td>\n");
		}
		if (whatToShow == CONTENT_END) // content end
			sb.append("</tr>\n</table>\n<!-- skin content end -->\n");

		LyWebViewManager viewManager = (LyWebViewManager)_environment.getViewManager();

		if ((whatToShow == OPEN_ZONE_1) && (viewManager.writeDivs()))
			sb.append("<div id=\"zone1\">\n");

		if (whatToShow == FORM_FIELDS_BEGIN)
		{
			String className = "formBorder";

			if ((param instanceof LyWebPageWriter)
					&& LyWebConstants.POPUP_VIEW.equals(((LyWebPageWriter)param).getWindowName()))
				className = "popupFormBorder";

			sb.append(
				"<!-- skin form fields begin -->\n<table class='formOutsideBorder' border='0' ");
			sb.append(
				"cellpadding='0' cellspacing='0' width='100%'>\n<tr>\n<td valign='top' float='center' align='center'>\n");
			sb.append("<table class='" + className + "' border='0' ");
			sb.append("cellpadding='0' cellspacing='0'>\n<tr>\n<td valign='top'>\n");
		}

		if ((whatToShow == CLOSE_ZONE) && (viewManager.writeDivs()))
			sb.append("</div>\n");

		if (whatToShow == FORM_FIELDS_END)
			sb.append("</td>\n</tr></table>\n</td>\n</tr></table>\n<!-- skin form field end -->\n");

		if (whatToShow == TOOLBAR_LABEL)
			sb.append("<td><table><tr>\n");

		if (whatToShow == TOOLBAR_LABEL_END)
			sb.append("</tr>\n</table></td>\n");

		if ((whatToShow == AREA_BORDER_BEGIN)
				|| (whatToShow == AREA_BORDER_BEGIN_NOT_FULL_WIDTH)
				|| (whatToShow == AREA_BORDER_BEGIN_NOT_FULL_HEIGHT)
				|| (whatToShow == AREA_BORDER_BEGIN_NOT_FULL_WIDTH_AND_HEIGHT)
				|| (whatToShow == HORIZONTAL_TOOLBAR_BEGIN))
		{
			String	borderTop = _environment.getImageUrl("LY_AREA_BORDER_TOP");
			String	borderTopClosed = _environment.getImageUrl("LY_AREA_BORDER_TOP_CLOSED");
			String	borderLabel = _environment.getImages().getImageUrl("LY_AREA_BORDER_LABEL", false, false);
			String	borderLabelClosed = _environment.getImages().getImageUrl("LY_AREA_BORDER_LABEL_CLOSED", false, false);
			String	borderTopLeft = _environment.getImageUrl("LY_AREA_BORDER_TOP_LEFT");
			String	borderTopRight = _environment.getImageUrl("LY_AREA_BORDER_TOP_RIGHT");
			String	borderTopNoLabel = _environment.getImageUrl("LY_AREA_BORDER_TOP_NO_LABEL");
			String	borderTopLeftNoLabel = _environment.getImageUrl("LY_AREA_BORDER_TOP_LEFT_NO_LABEL");
			String	borderTopRightNoLabel = _environment.getImageUrl("LY_AREA_BORDER_TOP_RIGHT_NO_LABEL");
			String	borderLabelLeft = _environment.getImages().getImageUrl("LY_AREA_BORDER_LABEL_LEFT", false, false);
			String	borderLabelRight = _environment.getImages().getImageUrl("LY_AREA_BORDER_LABEL_RIGHT", false, false);
			String	borderLabelLeftClosed = _environment.getImages().getImageUrl("LY_AREA_BORDER_LABEL_LEFT_CLOSED", false, false);
			String	borderLabelRightClosed = _environment.getImages().getImageUrl("LY_AREA_BORDER_LABEL_RIGHT_CLOSED", false, false);
			String	borderTopLeftClosed = _environment.getImageUrl("LY_AREA_BORDER_TOP_LEFT_CLOSED");
			String	borderTopRightClosed = _environment.getImageUrl("LY_AREA_BORDER_TOP_RIGHT_CLOSED");
			String	label = null;
			String	onClick = null;
			String  imgSrc = null;
			boolean isOpened = true;
			String  td1Id = "";
			String  td2Id = "";
			String  td3Id = "";
			String  imgId = "";
			String  imgLeftId = "";
			String  imgRightId = "";
			String  imgLabelLeftId = "";
			String  imgLabelRightId = "";
			
			if (param instanceof String[])
			{
				String buttonClose = _environment.getImageUrl("LY_AREA_BUTTON_CLOSE");
				String buttonOpen = _environment.getImageUrl("LY_AREA_BUTTON_OPEN");

				label = ((String[])param)[0];
				onClick = ((String[])param)[1];
				isOpened = ((String[])param)[2].equalsIgnoreCase("true");
				td1Id = "td1" + ((String[])param)[3];
				td2Id = "td2" + ((String[])param)[3];
				td3Id = "td3" + ((String[])param)[3];
				imgId = "imgOpen" + ((String[])param)[3];
				imgLeftId = "imgLeftOpen" + ((String[])param)[3];
				imgRightId = "imgRightOpen" + ((String[])param)[3];
				imgLabelLeftId = "imgLabelLeftOpen" + ((String[])param)[3];
				imgLabelRightId = "imgLabelRightOpen" + ((String[])param)[3];
				imgSrc = (isOpened ? buttonClose : buttonOpen);
			}
			else
				label = (String)param;
			
			String widthString = "";
			String heightString = "";
			
			if ((whatToShow == AREA_BORDER_BEGIN) || (whatToShow == AREA_BORDER_BEGIN_NOT_FULL_HEIGHT))
				widthString = " width='100%'";
			
			if ((whatToShow == AREA_BORDER_BEGIN) || (whatToShow == AREA_BORDER_BEGIN_NOT_FULL_WIDTH))
				heightString = " height='100%'";
			
			if (!isOpened)
				heightString = "";
			
			sb.append("<table border='0' cellspacing='0' cellpadding='0'" + widthString + heightString + ">");
			sb.append("<tr height='1'>\n");
			sb.append("<td width='1' valign='top' float='left' align='left'><img id='").append(imgLeftId).append("' src='");
			
			if (!isOpened)
			{
				borderTop = borderTopClosed;
				borderLabel = borderLabelClosed;
				borderLabelLeft = borderLabelLeftClosed;
				borderLabelRight = borderLabelRightClosed;
			}
			
			if (label == null)
			{
				borderTop = borderTopNoLabel;
				borderTopLeft = borderTopLeftNoLabel;
				borderTopRight = borderTopRightNoLabel;
			}
			
			if (isOpened)
				sb.append(borderTopLeft);
			else
				sb.append(borderTopLeftClosed);
				
			sb.append("'/></td>\n");
			
			if (label != null)
			{
				sb.append("<td nowrap='yes' height='1' colspan='2' valign='top'>");
				sb.append("<table border='0' cellspacing='0' cellpadding='0' width='100%' height='10'>");
				sb.append("<tr>");
				
				if ((borderLabelLeft != null) && (whatToShow != HORIZONTAL_TOOLBAR_BEGIN))
					sb.append("<td width='1' nowrap='yes' valign='top'><img id='").append(imgLabelLeftId).append("' src='").append(borderLabelLeft).append("'/></td>");
				
				sb.append("<td width='1%' nowrap='yes' id='" + td1Id + "'");
				
				if (whatToShow != HORIZONTAL_TOOLBAR_BEGIN)
				{
					sb.append(" class='areaGroupLabel'");
					
					if (borderLabel != null)
						sb.append(" style='background:url(").append(borderLabel).append(");background-repeat:repeat-x;'");
				}
				
				sb.append(" valign='middle'>");
				sb.append(label);
				sb.append("</td>");
								
				if ((borderLabelRight != null) && (whatToShow != HORIZONTAL_TOOLBAR_BEGIN))
					sb.append("<td width='1' nowrap='yes' valign='top'><img id='").append(imgLabelRightId).append("' src='").append(borderLabelRight).append("'/></td>");
				
				sb.append("<td id='" + td2Id + "' width='99%' style='background:url(").append(borderTop).append(");background-repeat:repeat-x;'></td>");
				sb.append("</tr></table></td>");
				sb.append("\n");
			}
			else
				sb.append("<td id='" + td2Id + "' height='1' colspan='2' width='20' style='background:url(").append(
					borderTop).append(");background-repeat:repeat-x;'></td>\n");
			
			sb.append("<td id='" + td3Id + "' height='1' valign='top' float='right' align='right' style='background:url(").append(
					borderTop).append(");background-repeat:repeat-x;'>");
			
			if (onClick != null)
			{
				sb.append("<img id='").append(imgId).append("' onclick=\"").append(onClick).append("\" class='verticalToolbarTop'");
				sb.append(" src='").append(imgSrc).append("'/>");
			}
			
			sb.append("</td>\n");
			sb.append("<td width='1' height='1' valign='top' float='right' align='right'><img id='").append(imgRightId).append("' src='");
			
			if (isOpened)
				sb.append(borderTopRight);
			else
				sb.append(borderTopRightClosed);
				
			sb.append("'/></td>\n");
			sb.append("</tr>\n");
		}

		if ((whatToShow == AREA_BORDER_LEFT)
				|| (whatToShow == AREA_BORDER_LEFT_NOT_FULL_WIDTH)
				|| (whatToShow == AREA_BORDER_LEFT_NOT_FULL_HEIGHT)
				|| (whatToShow == AREA_BORDER_LEFT_NOT_FULL_WIDTH_AND_HEIGHT))
		{
			String	borderLeft = _environment.getImageUrl("LY_AREA_BORDER_LEFT");
			String	trId = "";
			boolean	isOpened = true;
			String  widthString = " width='20'";
			String  heightString = "";
			
			if ((whatToShow == AREA_BORDER_LEFT) || (whatToShow == AREA_BORDER_LEFT_NOT_FULL_HEIGHT))
				widthString = "";
			
			if ((whatToShow == AREA_BORDER_LEFT) || (whatToShow == AREA_BORDER_LEFT_NOT_FULL_WIDTH))
				heightString = " height='99%'";
			
			if (param instanceof String[])
			{
				isOpened = ((String[])param)[0].equalsIgnoreCase("true");
				trId = " id='borderLeftTR" + ((String[])param)[1] + "'";
			}
			
			if (!isOpened)
				heightString = "";
			
			sb.append("<tr" + heightString);
			sb.append(trId);
			
			if (!isOpened)
				sb.append(" style='display:none;'");

			sb.append(">\n");
			sb.append("<td style='background:url(").append(borderLeft).append(")'></td>\n");
			sb.append("<td colspan='3'").append(widthString).append(" vAlign='top' class='areaGroup'>\n");
		}

		if (whatToShow == AREA_BORDER_RIGHT)
		{
			String borderRight = _environment.getImageUrl("LY_AREA_BORDER_RIGHT");

			sb.append("</td>\n");
			sb.append("<td style='background:url(").append(borderRight).append(")'></td>\n");
			sb.append("</tr>\n");
		}

		if (whatToShow == AREA_BORDER_END)
		{
			String borderBottom = _environment.getImageUrl("LY_AREA_BORDER_BOTTOM");
			String borderBottomLeft = _environment.getImageUrl("LY_AREA_BORDER_BOTTOM_LEFT");
			String borderBottomRight = _environment.getImageUrl("LY_AREA_BORDER_BOTTOM_RIGHT");
			String	trId = "";
			boolean	isOpened = true;
			
			if (param instanceof String[])
			{
				isOpened = ((String[])param)[0].equalsIgnoreCase("true");
				trId = " id='borderBottomTR" + ((String[])param)[1] + "'";
			}

			sb.append("<tr height='1'").append(trId);
			
			if (!isOpened)
				sb.append(" style='display:none;'");

			sb.append(">\n");
			sb.append("<td float='right' align='right' height='1'><img src='").append(borderBottomLeft).append(
				"'/></td>\n");
			sb.append("<td colspan='3' height='1' style='background:url(").append(borderBottom)
				.append(")'></td>");
			sb.append("<td float='right' align='right' height='1'><img src='").append(borderBottomRight).append(
				"'/></td>\n");
			sb.append("</tr>\n");
			sb.append("</table>\n");
		}

		if (whatToShow == VERTICAL_TOOLBAR_BEGIN)
		{
			String  buttonClose = _environment.getImageUrl("LY_VERTICAL_TOOLBAR_BUTTON_CLOSE");
			String  buttonOpen = _environment.getImageUrl("LY_VERTICAL_TOOLBAR_BUTTON_OPEN");
			String  buttonCloseOver = _environment.getImageUrl(
					"LY_VERTICAL_TOOLBAR_BUTTON_CLOSE_OVER");
			String  buttonOpenOver = _environment.getImageUrl(
					"LY_VERTICAL_TOOLBAR_BUTTON_OPEN_OVER");
			String  topLeft = _environment.getImageUrl("LY_VERTICAL_TOOLBAR_TOP_LEFT");
			String  topRight = _environment.getImageUrl("LY_VERTICAL_TOOLBAR_TOP_RIGHT");
			String	topRightExternal = _environment.getImages().getImageUrl("LY_VERTICAL_TOOLBAR_TOP_RIGHT_EXTERNAL", false, false);
			String  label = null;
			String  onClick = null;
			String  imgSrc = null;
			String  imgOverSrc = null;
			boolean isOpened = false;

			if (param instanceof String[])
			{
				label = ((String[])param)[0];
				onClick = ((String[])param)[1];
				isOpened = ((String[])param)[2].equalsIgnoreCase("true");
				imgSrc = (isOpened ? buttonClose : buttonOpen);
				imgOverSrc = (isOpened ? buttonCloseOver : buttonOpenOver);
			}
			else
				label = (String)param;

			sb.append("<table border='0' cellspacing='0' cellpadding='0' width='100%'>");

			if (onClick != null)
			{
				sb.append("<tr class='verticalToolbarTop' onclick=\"" + onClick + "\"");
				sb.append(" onMouseOver=\"SetImageSrc('" + onClick.hashCode() + "_button', '");
				sb.append(imgOverSrc).append("');\"");
				sb.append(" onMouseOut=\"SetImageSrc('" + onClick.hashCode() + "_button', '");
				sb.append(imgSrc).append("');\"");
				sb.append(">\n");
			}
			else
				sb.append("<tr class='verticalToolbarTop'>\n");

			sb.append("<td width='1' float='left' align='left'><img src='").append(topLeft).append("'/></td>\n");
			
			if (viewManager.getBrowser() == LyWebViewManager.BROWSER_CHROME)
				sb.append("<td nowrap='yes' class='verticalToolbarTopContent'>");
			else
				sb.append("<td width='99%' nowrap='yes' class='verticalToolbarTopContent'>");

			if (label != null)
				sb.append(label);

			sb.append("</td>\n");
			sb.append("<td width='1' float='right' align='right' class='verticalToolbarButton'>");

			if (onClick != null)
			{
				sb.append("<img id='" + onClick.hashCode() + "_button'");
				sb.append(" src='").append(imgSrc).append("'/>");
			}

			sb.append("</td>\n");
			sb.append("<td width='1' valign='bottom' float='right' align='right'><img src='").append(topRight).append("'/></td>\n");
			sb.append("<td width='1' valign='bottom' float='right' align='left'>");
			
			if (topRightExternal != null)
				sb.append("<img src='").append(topRightExternal).append("'/>");
			
			sb.append("</td>\n");
			sb.append("</tr>\n");
		}

		if (whatToShow == VERTICAL_TOOLBAR_LEFT)
		{
			sb.append("<tr>\n");
			sb.append("<td colspan='4' class='verticalToolbarContent'>\n");
		}

		if (whatToShow == VERTICAL_TOOLBAR_RIGHT)
		{
			sb.append("</td>\n");
			sb.append("<td width='1' valign='bottom' float='right' align='right' class='verticalToolbarRightExternal'></td>\n");
			sb.append("</tr>\n");
		}

		if (whatToShow == VERTICAL_TOOLBAR_END)
		{
			String	bottomLeftExternal = _environment.getImages().getImageUrl("LY_VERTICAL_TOOLBAR_BOTTOM_LEFT_EXTERNAL", false, false);
			String	bottomRightExternal = _environment.getImages().getImageUrl("LY_VERTICAL_TOOLBAR_BOTTOM_RIGHT_EXTERNAL", false, false);

			sb.append("<tr>\n");
			sb.append("<td float='right' align='right'>");
			
			if (bottomLeftExternal != null)
				sb.append("<img src='").append(bottomLeftExternal).append("'/>");
			
			sb.append("</td>\n");
			sb.append("<td colspan='3' class='verticalToolbarBottomExternal'></td>\n");
			sb.append("<td float='left' align='left'>");
			
			if (bottomRightExternal != null)
				sb.append("<img src='").append(bottomRightExternal).append("'/>");

			sb.append("</td>\n");
			sb.append("</tr>\n");
			sb.append("</table>\n");
		}
		
		return sb.toString();
	} // end method getPageDecoration

	/**
	 * Indicates if we may use the default background from ressources (or if it is managed by the
	 * skin).
	 *
	 * @return usage status
	 */
	public boolean isUsingDefaultBackground()
	{
		if (_skinFile == null)
			return true;
		else
			return false;
	}

	/**
	 * Gets the current skin.
	 *
	 * @return the current skin
	 * @see    #setSkin(String, LyEnvironment)
	 */
	public String getSkinFile()
	{
		return _skinFile;
	}
	
	/**
	 * Gets the path of current skin.
	 *
	 * @return the path of current skin
	 */
	public String getSkinPath()
	{
		if (_generatedSkinFiles.containsKey(_skinFile))
			return _generatedSkinFiles.get(_skinFile);
		
		return null;
	}

	/**
	 * Gets the url of the specific CSS file.
	 *
	 * @return the url of the specific CSS file or null if no specific CSS file is used.
	 */
	public String getSpecificCSSFile()
	{
		return _specificCSSFile;
	}

	/**
	 * Returns the CSS class string to be used for body tag of a page writer.
	 *
	 * @param  writer the page writer generating the body tag.
	 * @return the CSS class string or null if no special class string has to be used.
	 */
	public String getBodyClassString(LyWebPageWriter writer)
	{
		LyComponent viewComponent = writer.getViewComponent();

		if ((viewComponent != null)
				&& ((viewComponent.getParentTab() != null)
					|| (viewComponent instanceof LyTab)
					|| (viewComponent instanceof LyTabs)))
			return "class='tabBody'";

		if (LyWebConstants.POPUP_VIEW.equals(writer.getWindowName()))
			return "class='popupBody'";

		return null;
	}

	/**
	 * Returns the CSS class name to be used for given component and page writer.
	 *
	 * @param  component the component used to generate HTML code.
	 * @param  pw        current page writer.
	 * @return the name of the CSS class to be used for given component.
	 */
	public String getClassName(LyComponent component, LyWebPageWriter pw)
	{
		return getClassName(component, -1, pw);
	}

	/**
	 * Returns the CSS class name to be used for given component and page writer.<br />
	 * This method may be overridden in order to change the CCS class name for a specific component.
	 *
	 * @param  component the component used to generate HTML code.
	 * @param  type      the type of the class to be generated. Depends on the class of the
	 *                   component.
	 * @param  pw        current page writer.
	 * @return the name of the CSS class to be used for given component.
	 */
	public String getClassName(LyComponent component, int type, LyWebPageWriter pw)
	{
		if (component == null)
			return null;

		LyViewComponent internal = component.getInternalView();
		String		    className = null;

		// Area component
		if (component instanceof LyArea)
		{
			if (type == LyStrutsArea.DECO_TIP_PARAMETER)
			{
				String activeName = pw.getActiveName();

				className = "tip";

				if (LyWebConstants.PILOT_FRAME.equals(activeName))
					className = "dashboardTip";
			}
			else
			{
				if (component.getParentArea() == null)
				{
					LyArea area = (LyArea)component;

					// Css for an area containing tabs
					if ((area.getPanelCount() == 1) && (area.getPanel(0) instanceof LyTabs))
						className = "topAreaWithTabs";
					else
						className = "topArea";
				}
				else
					className = "";
			}
		}
		// Toolbar component
		else if (component instanceof LyToolBar)
		{
			LyStrutsToolBar webToolbar = (LyStrutsToolBar)internal;

			className = "toolbarLabel";

			if (!webToolbar.isHorizontal())
				className = "verticalToolbarLabel";
		}
		// Tool component
		else if (component instanceof LyTool)
			className = "tool";
		// Boolean component
		else if (component instanceof LyBoolean)
			className = "label";
		else if (component instanceof LyCell)
			className = "hyperlink";
		// Command component
		else if (component instanceof LyCommand)
		{
			LyStrutsCommand dhtmlCommand = (LyStrutsCommand)component.getInternalView();
			
			if ((dhtmlCommand != null) && !dhtmlCommand.isEnabled())
				className = "commandLabelDisabled";
			else
				className = "commandLabel";
		}
		// Commands component
		else if (component instanceof LyCommands)
		{
			if (LyWebConstants.POPUP_VIEW.equals(pw.getWindowName()))
				className = "popupCommandBar";
			else
				className = "commandBar";
		}
		// Footer component
		else if (component instanceof LyFooter)
		{
			LyFooter	   footer = (LyFooter)component;
			LyStrutsFooter webFooter = (LyStrutsFooter)footer.getInternalView();
			LyFooters      footers = (LyFooters)footer.getParent();
			LyTable		   table = (LyTable)footers.getParent();
			LyHeaders      headers = table.getHeaders();

			if (headers != null)
			{
				int		 index = webFooter.getFooterIndex();
				LyHeader header = headers.getHeader(index);

				if (header.getSort() >= 0)
					className = "footerSorted";
				else
					className = "footer";
			}
			else
				className = "footer";
		}
		// Form component
		else if (component instanceof LyForm)
		{
			if (type == LyStrutsForm.DECO_FORM_PARAMETER)
			{
				if (LyWebConstants.POPUP_VIEW.equals(pw.getWindowName()))
					className = "popupFormBorder";
				else
					className = "formBorder";
			}
			else
				className = "skinFieldCaption";
		}
		// Form field component
		else if (component instanceof LyFormField)
		{
			if (type == LyStrutsForm.DECO_CAPTION_PARAMETER)
				className = "skinFieldCaption";
			else
				className = "skinFieldValue";
		}
		// Caption component
		else if (component instanceof LyCaption)
		{
			LyCaption   caption = (LyCaption)component;
			LyFormField field = (LyFormField)caption.getParent();

			if (field.getParent() instanceof LySelectBar)
				className = "selectBarItem";
			else
			{
				className = "lbl";

				if (!field.isOptional())
					className += "Obl";

				if (field.getState() == LyFormField.STATE_ERROR)
					className += "Err";
				else if (field.getState() == LyFormField.STATE_WARNING)
					className += "Warn";
			}
		}
		// Header component
		else if (component instanceof LyHeader)
		{
			LyHeader header = (LyHeader)component;
			boolean  sortable = header.isSortable();

			className = "header";

			if (sortable)
				className = "sortableheader";

			if (header.getSort() >= 0)
				className = "headerSorted";
		}
		// HRef component
		else if (component instanceof LyHRef)
		{
			LyHRef hRef = (LyHRef)component;

			className = "href";

			if (LyWebConstants.TOOL_POPUP.equals(pw.getActiveName()))
				className = "popupHref";

			if (!hRef.isSensitive())
				className += "Disabled";
			else if (hRef.isVisited())
				className += "Visited";
		}
		// Item component
		else if (component instanceof LyItem)
		{
			if (!((LyItem)component).isEnabled())
				className = "disabledTool";
		}
		// Label component
		else if (component instanceof LyLabel)
			className = "label";
		// Menu bar component
		else if (component instanceof LyMenuBar)
			className = "menuBar";
		// Message component
		else if (component instanceof LyMessage)
			className = "message";
		// Messages component
		else if (component instanceof LyMessages)
		{
			if (type == LyStrutsMessages.DECO_BORDER_PARAMETER)
				className = "messagesBorder";
			else
				className = "messages";
		}
		// Option component
		else if (component instanceof LyOption)
			className = "label";
		// Select bar component
		else if (component instanceof LySelectBar)
		{
			LyArea area = ((LySelectBar)component).getParentArea();

			if (type == LyStrutsSelectBar.DECO_SELECTITEM_PARAMETER)
				className = "selectBarItem";
			else
			{
				if ((area != null) && (area.getParentArea() != null))
				{
					if (type == LyStrutsSelectBar.DECO_BORDER_PARAMETER)
						className = "selectBarBorder";
					else
						className = "selectBar";
				}
				else
				{
					if (type == LyStrutsSelectBar.DECO_BORDER_PARAMETER)
						className = "topSelectBarBorder";
					else
						className = "topSelectBar";
				}
			}
		}
		// Select item component
		else if (component instanceof LySelectItem)
		{
			if (((LySelectItem)component).isSelected())
				className = "selectBarItemSelected";
			else
				className = "selectBarItem";
		}
		// Splitter component
		else if (component instanceof LySplitter)
		{
			boolean isVertical = ((LySplitter)component).getStyle() == LySplitter.Style.VERTICAL;

			if (isVertical)
				className = "verticalSplitter";
			else
				className = "horizontalSplitter";
		}
		// Table component
		else if (component instanceof LyTable)
		{
			if (component.getParentForm() != null)
				className = "formTable";
			else
				className = "mainTable";
		}
		// Tabs component
		else if (component instanceof LyTabs)
		{
			if (type == LyStrutsTabs.DECO_LABEL_PARAMETER)
				className = "tabLabel";
			else if (type == LyStrutsTabs.DECO_DISABLED_LABEL_PARAMETER)
				className = "disabledTabLabel";
			else if (type == LyStrutsTabs.DECO_SELECTED_LABEL_PARAMETER)
				className = "selectedTabLabel";
			else if (type == LyStrutsTabs.DECO_DISABLED_SELECTED_LABEL_PARAMETER)
				className = "selectedDisabledTabLabel";
			else if (type == LyStrutsTabs.DECO_TAB_CONTENT_PARAMETER)
			{
				LyView	view = component.getParentView();
					
				if (view instanceof LyFrame)
					className = "frameTabContent";
				else
					className = "tabContent";
			}
			else if (type == LyStrutsTabs.DECO_HEADERS_PARAMETER)
			{
				LyArea parentArea = ((LyTabs)component).getParentArea();

				if ((parentArea != null) && (parentArea.getToolBar() != null))
					className = "filledTabHeaders";
				else
					className = "tabHeaders";
			}
		}
		// Titlebar component
		else if (component instanceof LyTitleBar)
		{
			if (type == LyStrutsTitleBar.DECO_LABEL_PARAMETER)
				className = "titleLabel";
			else
			{
				String activeName = pw.getActiveName();

				className = "titlebar";

				if (LyWebConstants.PILOT_FRAME.equals(activeName))
					className = "dashboardTitlebar";
				else if (pw.isPrintMode() || LyWebConstants.POPUP_VIEW.equals(pw.getWindowName()))
					className = "popupTitlebar";
				else
				{
					LyArea area = ((LyTitleBar)component).getParentArea();

					if ((area != null) && (area.getParentArea() == null))
						className = "topTitlebar";
				}
			}
		}
		// Tree component
		else if (component instanceof LyTree)
			className = "tree";

		return className;
	} // end method getClassName

} // end class LyWebDecoration
