/*
 * LyWebRequestInfo.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web;

import javax.servlet.http.*;


/**
 * Keep information about which servlet, which request and which info is in progress.
 *
 * @version $Id: LyWebRequestInfo.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public class LyWebRequestInfo
{
	public static final String    __VERSION =
		"$Id: LyWebRequestInfo.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** The number of the request. Used for debug issues. */
	protected int				  _requestCount;

	/** The servlet in use. */
	protected HttpServlet		  _servlet;

	/** The request. */
	protected HttpServletRequest  _request;

	/** The response. */
	protected HttpServletResponse _response;

	/**
	 * Default constructor.
	 *
	 * @param requestCount number of this request (debug)
	 * @param servlet      current servlet
	 * @param request      current request
	 * @param response     current response
	 */
	public LyWebRequestInfo(int requestCount, HttpServlet servlet, HttpServletRequest request,
		HttpServletResponse response)
	{
		_requestCount = requestCount;
		_servlet = servlet;
		_request = request;
		_response = response;
	}

	/**
	 * Gets the current request.
	 *
	 * @return the request
	 */
	public HttpServletRequest getRequest()
	{
		return _request;
	}

	/**
	 * Gets the current response.
	 *
	 * @return the response
	 */
	public HttpServletResponse getResponse()
	{
		return _response;
	}

	/**
	 * Gets the current servlet.
	 *
	 * @return the servlet
	 */
	public HttpServlet getServlet()
	{
		return _servlet;
	}

	/**
	 * Gets the current request count.
	 *
	 * @return the request count
	 */
	public int getRequestCount()
	{
		return _requestCount;
	}

} // end class LyWebRequestInfo
