/**
 * LySession.java Copyright (c) 2000-2009,
 * Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.jquery;

import java.io.Serializable;

import javax.servlet.http.*;

import leon.app.*;
import leon.view.*;
import leon.view.mobile.*;
import leon.view.web.*;
import leon.view.web.jquery.controller.*;


/**
 * The instance of session for JQuery viewer.
 *
 * @see     leon.app.LySession
 */
public class LyJQSession extends LyWebSession
	implements Serializable
{
	/**
	 * Comment for <code>serialVersionUID</code>
	 */
	private static final long serialVersionUID = 397778904466430763L;
	
	public static final String __VERSION =
		"$Id: LyWebSession.java 31632 2011-07-08 08:30:48Z nhamon $";
	
    /**
     * Constructor of web session.
     *
     * @param application     The application owner of the session.
     * @param registerSession Indicates if the session must be registered on the application
     *                        (default is true), or if this is a temporary session which should be
     *                        ignored.
     */
    public LyJQSession(LyApplication application, boolean registerSession)
    {
        super(application, registerSession);
    }

    /**
     * Gets the current request.
     *
     * @return the request
     */
    public HttpServletRequest getCurrentRequest()
    {
        LyWebRequestInfo    info = LyJQServlet.getWebRequestInfo();
        
        return (info == null ? null : info.getRequest());
    }

    /**
     * Gets the current response.
     *
     * @return the response
     */
    public HttpServletResponse getCurrentResponse()
    {
        LyWebRequestInfo    info = LyJQServlet.getWebRequestInfo();
        
        return (info == null ? null : info.getResponse());
    }
    
    @Override
    public void close()
    {
    	// Remove http session from session values
    	setHttpSession(null, null);
    	
    	LyViewManager viewManager = getViewManager();
    	
    	if(!viewManager.hasProperty(LyMobileViewManager.MOBILE_ONLY_PROPERTY))
    	{        	
        	((LyJQViewManager)viewManager).setSessionClosing();
    	}
    	
    	super.close();
    }
    
    /**
     * Binds an http session with current Leonardi session.
     */
    public void setHttpSession(HttpSession httpSession, String httpSessionKey)
    {
    	HttpSession	previousHttpSession = (HttpSession)getValue("HTTP_SESSION");

    	addValue("HTTP_SESSION", httpSession);
    	
    	if (httpSession != null)
    	{
    		httpSession.setAttribute(httpSessionKey, this);
    		LyJQController._sessions.put(httpSessionKey, this);
    	}
    	else if (previousHttpSession != null)
    	{
    		String	previousHttpSessionKey = LyJQController.getSessionKey(previousHttpSession, null);

    		LyJQController._sessions.remove(previousHttpSessionKey);
    	}
    }
    
    /**
     * Returns the http session binded with current Leonardi session.
     */
    public HttpSession getHttpSession()
    {
    	return (HttpSession)getValue("HTTP_SESSION");
    }
} // end class LyJQuerySession
