/*
 * LyWeb2ViewManager.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.jquery;

import java.awt.*;
import java.io.*;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

import leon.app.*;
import leon.data.*;
import leon.misc.*;
import leon.view.*;
import leon.view.common.*;
import leon.view.setinterface.*;
import leon.view.web.*;
import leon.view.web.jquery.common.*;
import leon.view.web.jquery.component.*;
import leon.view.web.jquery.component.area.*;
import leon.view.web.jquery.component.form.field.dictionary.*;
import leon.view.web.jquery.component.form.field.href.*;
import leon.view.web.jquery.component.form.field.label.*;
import leon.view.web.jquery.component.form.field.relation.LyJQRelation;
import leon.view.web.jquery.component.icon.*;
import leon.view.web.jquery.component.map.*;
import leon.view.web.jquery.component.menubar.*;
import leon.view.web.jquery.component.message.*;
import leon.view.web.jquery.component.selectBar.*;
import leon.view.web.jquery.component.selectItem.*;
import leon.view.web.jquery.component.table.*;
import leon.view.web.jquery.component.titleBar.*;
import leon.view.web.jquery.component.toolbar.*;
import leon.view.web.jquery.component.toolbar.tool.*;
import leon.view.web.jquery.component.tree.*;
import leon.view.web.jquery.controller.*;

import org.springframework.context.*;
import org.springframework.web.context.support.*;

/**
 * Communication control layer --> View Will implement remote update notifications.
 * @version $Revision: -1M $
 * @author atalbi, last modified by $Author: (local) $
 */
public class LyJQViewManager extends LyWebViewManager
{

	/** The current servlet context : the configuration of the WEB application. */
	protected ServletContext				_context;

	protected static String					JQUERY_INTERNAL_VIEW			= "leon.view.web.jquery";

	protected static String					ADD_PANEL_TEMPLATE				= "ajax/area/addPanelTemplate.vm";

	protected static String					SHOW_SHARING_BOX_TEMPLATE		= "ajax/showSharingBox.vm";

	protected static String					AREA_PANELS_CONTAINER_TEMPLATE	= "area/lyAreaPanelsContainerTemplate.vm";

	protected static String					NEW_WINDOW_TEMPLATE				= "ajax/newWindowScript.vm";

    /** Indicates whether an absolute URL must be used as a root url. */
    public static boolean					USE_ABSOLUTE_ROOT_URL = true;

	protected final String					POPUP_MENU_DIV_NAME				= "popupMenuContainer";

	protected final String					POPUP_MENU_VELO_VAR_NAME		= "lyPopupMenu";

	protected final String					POPUP_MENU_TEMPLATE				= "popupMenu/lyPopupMenuTemplate.vm";

	protected final String					HIDE_DIALOG_TEMPLATE			= "ajax/hideDialogBox.vm";

	protected final String					SHOW_DIALOG_TEMPLATE			= "ajax/showDialogBox.vm";

	/**
	 * Components to reload, the map key is the id of the component and the value represents the object
	 **/
	protected HashMap<String, LyComponent>	_componentsToRender				= new HashMap<String, LyComponent>();

	/* action response update scripts */

	protected StringBuffer				_updateScript					= new StringBuffer();

	protected HttpServletRequest			_request;

	protected boolean						_isPrintMode					= false;

	protected boolean						_isAutoCompleteEvent			= false;

	protected LyComponent					_currentTreeMap					= null;

	protected LyTemplatesUtilsBean			_veloTemplatesManager;

	protected LyView						_currentPrintView					= null;

	protected boolean						_requestInProgress				= false;

	protected long							_stopPushTime					= -1;

	protected Hashtable<HttpSession, LyJQSession>	_sharingSessions;

	protected String								_sharedSessionUserName;

	protected boolean						_isPortlet				= false;

	protected boolean						_sessionClosing			= false;

	/**
	 * Constructor.
	 * @param environment the application environment
	 */
	public LyJQViewManager(LyEnvironment environment)
	{
		super(environment);
		_beans = null;
		_sharedSessionUserName = null;
	}

	/**
	 * @see leon.misc.LyViewManagerInterface#free()
	 */
	public void free()
	{
		// Generic free
		super.free();
	}

	/**
	 * Returns the name of the resource to be used for default page.
	 */
	public String getDefaultPageResource()
	{
		return "LY_JQUERY_DEFAULT_PAGE";
	}

	/**
	 * Returns the url of the directory that contains the Javascript files.
	 * @return the url of the directory that contains the Javascript files.
	 */
	public String getScriptsDir()
	{
		String scriptRoot = _environment.getEnv(LyJQConstants.LY_LEON_DOC) + "/scripts/jquery";
		return scriptRoot;
	}

	/**
	 * Returns the path of the directory that contains the Javascript files.
	 * @return the path of the directory that contains the Javascript files.
	 */
	public String getScriptsDocDir()
	{
		String scriptRoot = _environment.getEnv(LyJQConstants.LY_LEON_DOC_DIR) + "/scripts/jquery";
		return scriptRoot;
	}

	/**
	 * Swap two sessions.
	 * @param newSession the new session
	 * @param oldSession the old session
	 * @param applicationId the application id
	 */
	public void swapSession(Object newSession, Object oldSession, String applicationId)
	{
		LyJQSession		jqSession = (LyJQSession)oldSession;
		LyJQSession		newJqSession = (LyJQSession)newSession;
		HttpSession		newHttpSession = newJqSession.getHttpSession();
		String			newHttpSessionKey = LyJQController.getSessionKey(newHttpSession, LyJQController.DEFAULT_SESSION);

		((LySession)newSession).close();
		jqSession.setHttpSession(newHttpSession, newHttpSessionKey);
	}

	/**
	 * Share a session with other sessions.
	 *
	 * @param  sharedSession   the shared session
	 * @param  sharingSessions the list of sessions sharing the shared session
	 * @param  applicationId   the application id
	 */
	public void shareSession(Object sharedSession, Vector<Object> sharingSessions, String applicationId)
	{
		LyJQSession		jqSharedSession = (LyJQSession)sharedSession;
		LyObject		user = jqSharedSession.getUser();

		if (user != null)
			_sharedSessionUserName = user.getName();

		HashMap<String, Object>	params = new HashMap<String, Object>();
		LyTemplatesUtilsBean	templateManager = getVeloTemplatesBean();

		params.put("show", Boolean.TRUE);
		params.put("sharingMessage", getEnvironment().getString("LY_SHARING_IN_PROGRESS", _sharedSessionUserName));
		addToUpdateScript(templateManager.mergeTemplate(SHOW_SHARING_BOX_TEMPLATE, params));

		LyJQViewManager	jqSharedManager = (LyJQViewManager)jqSharedSession.getViewManager();
		HttpSession		sharedHttpSession = jqSharedSession.getHttpSession();
		String			cookieTheme = (String)sharedHttpSession.getAttribute(LyJQConstants.COOKIE_THEME);
		String			sharedHttpSessionKey = LyJQController.getSessionKey(sharedHttpSession, LyJQController.DEFAULT_SESSION);
		int				i, n = sharingSessions.size();

		for (i = 0; i < n; i++)
		{
			LyJQSession		sharingSession = (LyJQSession)sharingSessions.elementAt(i);
			LyJQViewManager	jqSharingManager = (LyJQViewManager)sharingSession.getViewManager();
			HttpSession		sharingHttpSession = sharingSession.getHttpSession();
			String			sharingHttpSessionKey = LyJQController.getSessionKey(sharingHttpSession, LyJQController.DEFAULT_SESSION);

			jqSharedSession.setHttpSession(sharingHttpSession, sharingHttpSessionKey);
			jqSharedManager.addSharingManager(sharingHttpSession, sharingSession);
			sharingHttpSession.setAttribute("SHARED_THEME", cookieTheme);
			jqSharingManager.redirectToHomePage();
		}

		// If push mode is not activated, force a refresh of the home page to activate push frame
		if (!getEnvironment().testResource("LY_WEB_PUSH_MODE"))
			jqSharedManager.redirectToHomePage();

		jqSharedSession.setHttpSession(sharedHttpSession, sharedHttpSessionKey);
	}

	/**
	 * Stop the sharing process of current session.
	 */
	public void stopSharingSession()
	{
		if (_sharingSessions != null)
		{
			HashMap<String, Object>	params = new HashMap<String, Object>();
			LyTemplatesUtilsBean	templateManager = getVeloTemplatesBean();

			params.put("show", Boolean.FALSE);
			addToUpdateScript(templateManager.mergeTemplate(SHOW_SHARING_BOX_TEMPLATE, params));

			// If push mode is not activated, force a refresh of the home page to disable push frame
			if (!getEnvironment().testResource("LY_WEB_PUSH_MODE"))
				redirectToHomePage();

			for (HttpSession sharingHttpSession : _sharingSessions.keySet())
			{
				LyJQSession		sharingSession = _sharingSessions.get(sharingHttpSession);
				LyJQViewManager	sharingManager = (LyJQViewManager)sharingSession.getViewManager();
				String			sharingHttpSessionKey = LyJQController.getSessionKey(sharingHttpSession, LyJQController.DEFAULT_SESSION);

				sharingSession.setHttpSession(sharingHttpSession, sharingHttpSessionKey);
				sharingHttpSession.setAttribute("SHARED_THEME", null);
				sharingManager.redirectToHomePage();
				sharingManager.showWarning(null, sharingManager.getEnvironment().getMessage("LY_STOP_SHARING_SESSION"));
			}

			_sharingSessions.clear();
			_sharingSessions = null;
			_sharedSessionUserName = null;
			showWarning(null, getEnvironment().getMessage("LY_STOP_SHARING_SESSION"));
		}
	}

	/**
	 * Register a view manager sharing current view manager session.
	 *
	 * @param sharingHttpSession http session of the sharing view manager
	 * @param sharingManager     view manager sharing current view manager session
	 */
    public void addSharingManager(HttpSession sharingHttpSession, LyJQSession sharingSession)
    {
    	if (_sharingSessions == null)
    		_sharingSessions = new Hashtable<HttpSession, LyJQSession>();

    	_sharingSessions.put(sharingHttpSession, sharingSession);
    }

    /**
     * Return the view manager sharing current view manager session from a given http session.
     *
     * @param sharingHttpSession http session corresponding to the sharing view manager
     * @return the sharing view manager if it exists, false otherwise
     */
    public LyJQViewManager getSharingManager(HttpSession sharingHttpSession)
    {
    	if ((_sharingSessions == null) || _sharingSessions.isEmpty() || (sharingHttpSession == null))
    		return null;

    	LyJQSession	sharingSession = _sharingSessions.get(sharingHttpSession);

    	return (sharingSession != null ? (LyJQViewManager)sharingSession.getViewManager() : null);
    }

    /**
     * Indicates whether current view manager is being shared.
     *
     * @return true if current view manager is being shared, false otherwise.
     */
    public boolean isShared()
    {
    	return ((_sharingSessions != null) && (_sharingSessions.size() > 0));
    }

    /**
     * Indicates whether current view manager is sharing another view manager.
     *
     * @return true if current view manager is sharing another view manager, false otherwise.
     */
    public boolean isSharing()
    {
    	LyWebRequestInfo	webRequestInfo = getWebRequestInfo();
    	HttpSession			httpSession = (webRequestInfo != null ? webRequestInfo.getRequest().getSession() : null);

    	return (getSharingManager(httpSession) != null);
    }

    /**
     * Returns the message that must be displayed in the sharing box.
     */
    public String getSharedSessionMessage()
    {
		LyWebRequestInfo	requestInfo = getWebRequestInfo();
		HttpSession			httpSession = (requestInfo != null ? requestInfo.getRequest().getSession() : null);
		LyJQViewManager		sharingManager = getSharingManager(httpSession);
		LyEnvironment		environment = getEnvironment();

		if (sharingManager != null)
			environment = sharingManager.getEnvironment();

		if (sharingManager == null || (sharingManager == this))
			return environment.getString("LY_CURRENT_SHARING_IN_PROGRESS");
		else
			return environment.getString("LY_SHARING_IN_PROGRESS", _sharedSessionUserName);
    }

    public void redirectToHomePage()
    {
    	redirectToHomePage(null);
    }

    /**
     * Sends a Javascript script to redirect to home page.
     */
    public void redirectToHomePage(LyView closedView)
    {
		String					homeUrl = getRootURL() + "/" + LyJQConstants.PATH_ACTION_HOME;
		Map<String, Object>		params = new HashMap<String, Object>();

		homeUrl = LyJQController.completeUrlParams(homeUrl, null, this);

		if (getMobileStartURL() != null)
    	{
    		Vector<LyView>	views = getViewList();

    		if ((closedView != null) && (views != null) && views.contains(closedView))
    			views.remove(closedView);

    		if ((views == null) || views.isEmpty())
    			homeUrl = getMobileStartURL();
    	}

		LyTemplatesUtilsBean	templateManager = getVeloTemplatesBean();

		params.put("redirectUrl", homeUrl);
		addToUpdateScript(templateManager.mergeTemplate("ajax/redirectionScript.vm", params));
    }

	/**
	 * (non-Javadoc).
	 * @see leon.view.LyViewManager#setProperties()
	 */
	protected void setProperties()
	{
		super.setProperties();

		setProperty("MENU");
		setProperty("PARTIAL_MENU");
		setProperty("DOUBLE_CLICK");
		setProperty("POPUP_VIEW");
		setProperty("DHTML_NO_SCROLLING");
		setProperty("LOCAL_MAP_SELECTION");
		setProperty("DYNAMIC");
		setProperty("MULTI_FRAME_FORM");
		setProperty("ALLOW_TABLE_HEADERS_UPDATE");
		setProperty("DISPLAY_RICH_TEXT");
		setProperty("DISPLAY_GOOGLE_MAP");
		setProperty("MANAGE_STICKERS");
		setProperty("ALLOW_FORM_RESIZE");
		setProperty("HANDLE_SOCIAL_BEHAVIOR");
		setProperty("FORCE_POPUP_VIEW");
		setProperty("MANAGE_IMAGE_FILE_SELECTOR");

		removeProperty("ASK_MULTIPLE_OBJECTS_DELETION");
		removeProperty("LAYOUT_MANAGER");
	}

	/**
	 * (non-Javadoc).
	 * @see leon.view.web.LyWebViewManager#loadBeans()
	 */
	protected void loadBeans()
	{
		// Add WEB2 beans
		loadBeans("views_web2");
	}

	/**
	 * (non-Javadoc).
	 * @see leon.view.LyViewManager#buildInternalView(leon.view.LyComponent)
	 */
	public LyViewComponent buildInternalView(LyComponent component)
	{
		LyViewComponent internalView = super.buildInternalView(component);

		if (internalView != null)
			return internalView;

		if (_beans != null)
		{
			String tag = component.getTag();
			String beanClassName = _beans.getString(tag);

			if (beanClassName != null)
				return internalView;
		}

		if ((component instanceof LyItem) || (component instanceof LyMenu)
				|| (component instanceof LyMenuBar) || (component instanceof LyPopupMenu))
			return null;

		if (_beans != null)
		{
			String tag = component.getTag();
			String beanClassName = _beans.getString(tag);

			if (beanClassName != null)
				return internalView;
		}

		if (component instanceof LyPopupMenu)
			return null;

		return buildInternalView(JQUERY_INTERNAL_VIEW, "LyJQ", component);
	} // end method buildInternalView

	/**
	 * (non-Javadoc).
	 * @see leon.misc.LyViewManagerInterface#getColor(leon.misc.LyColor)
	 */
	public Object getColor(LyColor color)
	{
		if (color == null)
			return null;

		return new Color(color.getRGB());
	}

	/**
	 * (non-Javadoc).
	 * @see leon.misc.LyViewManagerInterface#getFont(leon.misc.LyFont)
	 */
	public Object getFont(LyFont font)
	{
		if (font == null)
			return LyJQDrawingArea.getDefaultFrame().getFont();

		int style = 0;

		if (font.isBold())
			style |= Font.BOLD;
		if (font.isItalic())
			style |= Font.ITALIC;

		return new Font(font.getFontName(), style, font.getSize());
	}

	/**
	 * (non-Javadoc).
	 * @see leon.misc.LyViewManagerInterface#getStringWidth(leon.misc.LyFont, java.lang.String)
	 */
	public int getStringWidth(LyFont font, String text)
	{
		return 0;
	}

	/**
	 * Construct a default image.
	 * @see LyImageUtils#getDefaultImage()
	 * @see leon.misc.LyViewManagerInterface#getDefaultImage()
	 */
	public Object getDefaultImage()
	{
		return LyImageUtils.getDefaultImage();
	}

	/**
	 * Creates a scaled version of the given image.
	 * @see LyImageUtils#getScaledInstance(Image, int, int)
	 * @see leon.misc.LyViewManagerInterface#getScaledInstance(Object, int, int)
	 */
	public Object getScaledInstance(Object image, int width, int height)
	{
		return LyImageUtils.getScaledInstance((Image) image, width, height);
	}

	/**
	 * Waits for the image loading if the headless mode is null or set to true.
	 * @see LyImageUtils#waitForLoading(Component, Image)
	 * @see leon.misc.LyViewManagerInterface#waitForLoading(Object, Object)
	 */
	public void waitForLoading(Object component, Object image) throws InterruptedException
	{
		// empty
	}

	/**
	 * Compare two images pixel by pixel.
	 * @see LyImageUtils#isSameImage(Image, Image)
	 * @see leon.view.LyViewManager#isSameImage(java.lang.Object, java.lang.Object)
	 */
	public boolean isSameImage(Object image1, Object image2) throws InterruptedException
	{
		return LyImageUtils.isSameImage((Image) image1, (Image) image2);
	}

	/**
	 * Free the resources of the image if not in the cache.
	 * @see LyImageUtils#freeImage(Image)
	 * @see leon.misc.LyViewManagerInterface#freeImage(java.lang.Object)
	 */
	public void freeImage(Object image)
	{
		LyImageUtils.freeImage((Image) image);
	}

	/**
	 * (non-Javadoc).
	 * @see leon.misc.LyViewManagerInterface#exit(int)
	 */
	public void exit(int code)
	{
		// Do some cleaning
		LyImageUtils.clean();
		super.exit(code);
	}

	/**
	 * Copy a given text to the system clipboard.
	 * @param text the text to be copied to the clipboard.
	 */
	public void copyToClipboard(String text)
	{
		// Nothing to do in web mode
	}

	/**
	 * Check if this view manager is compatible with the given view manager
	 * @param otherViewManager the other view manager
	 * @return true if both view manager are compatible
	 */
	public boolean areViewManagerCompatible(LyViewManager otherViewManager)
	{
		boolean result = super.areViewManagerCompatible(otherViewManager);

		if (!result)
			return result;
		if (this.getBrowser() == ((LyJQViewManager) otherViewManager).getBrowser())
			return true;

		return false;
	}

	public String getDownloadAction()
	{
		String	downloadUrl = getRootURL() + "/" + LyJQConstants.PATH_ACTION_DOWNLOAD;

		return downloadUrl + "?_file=";
	}

	/**
	 * @return the _context
	 */
	public ServletContext getServletContext()
	{
		return _context;
	}

	/**
	 * @param _context the _context to set
	 */
	public void setServletContext(ServletContext _context)
	{
		this._context = _context;
	}

	/**
	 * Sets the popup menu with the given value.
	 * @param popup the new popup menu
	 */
	public void setPopupMenu(LyPopupMenu popup)
	{
		LyTemplatesUtilsBean templateManager = getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(POPUP_MENU_VELO_VAR_NAME, popup);

		String popupMenuDivContent = templateManager.mergeTemplate(POPUP_MENU_TEMPLATE, params);
		popupMenuDivContent = TemplateUtils.getFormattedJs(popupMenuDivContent);

		params = new HashMap<String, Object>();
		params.put(LyJQConstants.PARENT_ID_PARAM, POPUP_MENU_DIV_NAME);
		params.put(LyJQConstants.DIV_TO_APPEND_PARAM, popupMenuDivContent);

		addToUpdateScript(templateManager.mergeTemplate(LyJQConstants.APPEND_SCRIPT_TEMPLATE,
				params));
		params = null;
		popupMenuDivContent = null;

	}

	public void setServletRequest(HttpServletRequest request)
	{
		_request = request;
	}

	public boolean isSelectedTab(LyTabs tabs, String tabId)
	{
		String selectedTab = (tabs.getSelectedTab() != null ? tabs.getSelectedTab() : tabs
				.getChild(0).getId());
		return selectedTab.equals(tabId);
	}

	/**
	 * get reload components scripts
	 * @return javascript to reload modified components
	 */
	public StringBuffer getReloadDivScripts(Map<String, LyComponent> components)
	{
		StringBuffer			updateSc = new StringBuffer();
		LyComponent				component;
		String					data;
		String					newItem;
		Map<String, Object>		params;
		LyTemplatesUtilsBean	templateManager = getVeloTemplatesBean();
		LyComponent				parent;
		LyJQInternalView		view;
		LyTabs					parentTabs;
		Set<String>				keys = components.keySet();

		for (String idComponent : keys)
		{
			component = components.get(idComponent);
			params = new HashMap<String, Object>();
			view = null;

			if (component != null)
			{
				view = (LyJQInternalView) component.getInternalView();
				parentTabs = component.getParentTabs();

				if ((view != null)
						&& ((parentTabs == null)
								|| isSelectedTab(parentTabs, component.getParentTab().getLabel())
								|| (components.get(parentTabs.getInternalId()) == null)))
				{
					if (component.getClass() == LyTable.class)
					{
						params = new HashMap<String, Object>();
						params.put("componentId", ((LyJQTable)view).getId());
						params.put("table", component);
						data = templateManager.mergeTemplate("table/lyTableTemplate.vm", params);
						data = TemplateUtils.getFormattedJs(data);
						params.put("content", data);
						updateSc.append(templateManager.mergeTemplate(LyJQConstants.REPLACE_SCRIPT_TEMPLATE, params));
					}
					else if (component.getClass() == LySelectItem.class)
					{
						if ((component.getParent() == null) || (components.get(component.getParent().getInternalId()) == null))
						{
							params.put("item", component);
							newItem = templateManager.mergeTemplate("selectBar/lySelectItemTemplate.vm", params);
							newItem = TemplateUtils.getFormattedJs(newItem);
							params = new HashMap<String, Object>();
							params.put("content", newItem);
							params.put("componentId", ((LyJQSelectItem) view).getId());
							updateSc.append(templateManager.mergeTemplate("ajax/replaceWith.vm", params));
						}
					}
					else if (component.getClass() == LySelectBar.class)
					{
						params.put("selectBar", ((LySelectBar) component));
						newItem = templateManager.mergeTemplate("selectBar/lySelectBarTemplate.vm", params);
						newItem = TemplateUtils.getFormattedJs(newItem);
						params = new HashMap<String, Object>();
						params.put("content", newItem);
						params.put("componentId", ((LyJQSelectBar) view).getId());
						updateSc.append(templateManager.mergeTemplate("ajax/replaceWith.vm", params));
					}
					else if (component.getClass() == LyTool.class)
					{
						params.put("lyTool", component);
						params.put("callToolMacro", true);
						parent = component;

						while (!(parent instanceof LyToolBar || parent instanceof LyFrame))
							parent = parent.getParent();

						if (parent instanceof LyToolBar)
							params.put("lyToolBar", parent);

						data = templateManager.mergeTemplate("toolbar/lyToolBarTemplate.vm", params);
						data = TemplateUtils.getFormattedJs(data);
						params = new HashMap<String, Object>();
						params.put("content", data);
						params.put("componentId", ((LyJQTool) view).getId());
						updateSc.append(templateManager.mergeTemplate("ajax/replaceWith.vm", params));
					}
					else if (component.getClass() == LyToolBar.class)
					{
						params.put("lyToolBar", component);
						data = templateManager.mergeTemplate("toolbar/lyToolBarTemplate.vm", params);
						data = TemplateUtils.getFormattedJs(data);
						params = new HashMap<String, Object>();
						params.put("content", data);
						params.put("componentId", ((LyJQToolbar) view).getId());
						updateSc.append(templateManager.mergeTemplate("ajax/replaceWith.vm", params));
					}
					else if (component.getClass() == LyFormField.class)
					{
						updateSc.append(view.getReloadScript());
					}
					else if (component instanceof LyPanel && component.getParent() != null)
					{
						params = new HashMap<String, Object>();
						params.put("panelToAdd", component);
						data = templateManager.mergeTemplate(AREA_PANELS_CONTAINER_TEMPLATE, params);
						data = TemplateUtils.getFormattedJs(data);
						params = new HashMap<String, Object>();

						if (component.getParent() != null)
						{
							params.put("panelsContainerId", "panelsContainer_content_" + ((LyJQArea) component.getParent().getInternalView()).getId());
							params.put("appendDiv", data);
							updateSc.append(templateManager.mergeTemplate(ADD_PANEL_TEMPLATE, params));
						}
					}
					else if (component.getClass() == LyMessages.class)
					{
						updateSc.append(((LyJQMessages) view).getReloadScript());
					}
					else if (component.getClass() == LyGantt.class)
					{
						updateSc.append(((LyJQMessages) view).getReloadScript());
					}
					else if (component.getClass() == LyLabel.class)
					{
						params = new HashMap<String, Object>();
						params.put("label", component);
						data = templateManager.mergeTemplate("form/field/label/lyLabelTemplate.vm", params);
						data = TemplateUtils.getFormattedJs(data);
						params = new HashMap<String, Object>();
						params.put("content", data);
						params.put("componentId", ((LyJQLabel) view).getId());
						updateSc.append(templateManager.mergeTemplate("ajax/replaceWith.vm", params));
					}
					else if (component.getClass() == LyTree.class)
					{
						updateSc.append(((LyJQTree) view).getReloadScript());
					}
					else if (component.getClass() == LyMenuBar.class)
					{
						updateSc.append(((LyJQMenuBar) view).getReloadScript());
					}
					else if (component.getClass() == LyHRefs.class)
					{
						updateSc.append(((LyJQHRefs) view).getReloadScript());
					}
					else if (component.getClass() == LyIcon.class)
					{
						updateSc.append(((LyJQIcon) view).getReloadScript());

					}
					else if (component.getClass() == LyTitleBar.class)
					{
						updateSc.append(((LyJQTitleBar) view).getReloadScript());
					}
					else if (component.getClass() == LyRelation.class)
					{
						updateSc.append(((LyJQRelation) view).getReloadScript());
					}
					else if (component.getClass() == LyDictionary.class)
					{
						updateSc.append(((LyJQDictionary) view).getReloadScript());
					}
					else if (component.getClass() == LyList.class)
					{
						updateSc.append(view.getReloadScript());
					}
					else if (component.getClass() == LyMap.class)
					{
						if (view != null)
						{
							LyJQMap lyJqMap = (LyJQMap) component.getInternalView();
							updateSc.append(lyJqMap.getReloadScript());
						}
					}
				}
			}

		}

		params = null;

		return updateSc;
	}

	public LyTemplatesUtilsBean getVeloTemplatesBean()
	{
		if ((_veloTemplatesManager == null) && (_context != null))
		{
			ApplicationContext appContext = WebApplicationContextUtils.getWebApplicationContext(_context);
			_veloTemplatesManager = (LyTemplatesUtilsBean) appContext.getBean("veloTemplates");
		}
		return _veloTemplatesManager;
	}

	public void emptyRefreshScripts()
	{
		_updateScript = null;
		_componentsToRender = null;
		_updateScript = new StringBuffer();
		_componentsToRender = new HashMap<String, LyComponent>();

	}

	public int[] getSizes(Object image)
	{
		return LyImageUtils.getSizes((Image) image);
	}

	public void addToUpdateScript(String updateScript)
	{
		synchronized (this)
		{
			_updateScript.append(updateScript);
		}
	}

	public void addToReloadComponents(LyComponent object)
	{
		synchronized (this)
		{
			_componentsToRender.put(object.getInternalId(), object);
		}
	}

	public String getUpdateScripts(boolean emptyScripts)
	{
		LyWebRequestInfo			requestInfo = getWebRequestInfo();
		HttpSession					httpSession = (requestInfo != null ? requestInfo.getRequest().getSession() : null);
		LyJQViewManager				sharingManager = getSharingManager(httpSession);

		if (sharingManager != null)
			return "";

		Map<String, LyComponent>	components = null;
		String						message = null;
		StringBuffer				updateScript = null;

		synchronized (this)
		{
			components = new HashMap<String, LyComponent>(_componentsToRender);
			message = getMessage(false, !emptyScripts);
			updateScript = new StringBuffer(_updateScript.toString());

			if (emptyScripts)
			{
				setMessage(null);
				emptyRefreshScripts();
			}
		}

		StringBuffer				reloadDivScripts = getReloadDivScripts(components);
		StringBuffer				scripts = updateScript.append(reloadDivScripts);

		if (message != null)
			scripts.append(message);

		if (isShared() && (scripts.length() > 0))
		{
			// Add a script to close popup menu when sharing session
			LyTemplatesUtilsBean	templateManager = getVeloTemplatesBean();
			String					hidePopupMenucript = templateManager.mergeTemplate("ajax/hidePopupMenu.vm", null);

			scripts.insert(0, hidePopupMenucript);
		}

		if (emptyScripts)
		{
			// Check if current view manager is being shared
			if ((scripts.length() > 0)
					&& (_sharingSessions != null) && !_sharingSessions.isEmpty())
			{
				Collection<LyJQSession>		sharingSessions = _sharingSessions.values();
				Iterator<LyJQSession>		sharingSessionsIterator = sharingSessions.iterator();
				LyJQSession					sharingSession;
				int							pos = scripts.indexOf("jQuery('#sharingMessage').html");

				// Prevent sharing box replacement in sharing sessions (to keep user language)
				if (pos >= 0)
					scripts = scripts.replace(pos + 18, pos + 18, "_none");

				// Add computed update scripts to every sharing view manager
				while (sharingSessionsIterator.hasNext())
				{
					sharingSession = sharingSessionsIterator.next();
					sharingManager = (LyJQViewManager)sharingSession.getViewManager();
					sharingManager.addToUpdateScript(scripts.toString());
				}
			}
		}

		_isPrintMode = false;

		return scripts.toString();
	}

	public void setRequestInProgress(boolean inProgress)
	{
		_requestInProgress = inProgress;
	}

	public boolean isRequestInProgress()
	{
		return _requestInProgress;
	}

	public void setStopPush()
	{
		LyWebRequestInfo	requestInfo = getWebRequestInfo();
		HttpSession			httpSession = (requestInfo != null ? requestInfo.getRequest().getSession() : null);
		LyJQViewManager		sharingManager = getSharingManager(httpSession);

		if (sharingManager != null)
			sharingManager.setStopPush();
		else
			_stopPushTime = System.currentTimeMillis();
	}

	public void setSessionClosing()
	{
		_sessionClosing = true;
	}

	public boolean isSessionClosing()
	{
		return _sessionClosing;
	}

	public long getStopPushTime()
	{
		long	stopPushTime = _stopPushTime;

		_stopPushTime = -1;

		return stopPushTime;
	}

	public Vector<LyView> getViewList()
	{
		return _viewList;
	}

	/**
	 * Return the JQuery theme registered in the cookies.
	 */
	public String getCookieTheme(HttpServletRequest request)
	{
		HttpSession	session = request.getSession();
		String		sharedCookie = (String)session.getAttribute("SHARED_THEME");

		if (sharedCookie != null)
			return sharedCookie;

		Cookie[]	cookies = request.getCookies();

		if ((cookies != null) && (cookies.length > 0))
		{
			for (int i = 0; i < cookies.length; i++)
			{
				if ("ly-current-theme".equals(cookies[i].getName())
						&& (cookies[i].getValue() != null)
						&& (cookies[i].getValue().length() > 0))
				{
					// Check if theme is available
					if (isThemeAvailable(cookies[i].getValue()))
						return cookies[i].getValue();
					else
					{
						LyWebRequestInfo	info = getWebRequestInfo();

						if ((info != null) && (_environment != null))
						{
							HttpServletResponse		response = info.getResponse();
							Cookie					cookie = new Cookie("ly-current-theme", _environment.getSkin());

							response.addCookie(cookie);
						}

						return null;
					}
				}
			}
		}

		return null;
	}

	/**
     * Returns the list of available themes.
     */
    public String getThemes()
    {
        StringBuffer themes = new StringBuffer();
        themes.append(_environment.getResource("LY_JQUERY_THEMES"));
        themes.append(",");
        themes.append(_environment.getResource("LY_JQUERY_THEMES_2"));
        themes.append(",");
        themes.append(_environment.getResource("LY_JQUERY_THEMES_3"));
        themes.append(",");
        themes.append(_environment.getResource("LY_JQUERY_THEMES_4"));

        if (themes != null && !"".equals(themes.toString()))
        {
            return getAvailableThemes(themes.toString().split(","));
        }
        return "";
    }


    class SkinJarFileFilter implements FilenameFilter
    {
        public boolean accept(File dir, String name)
        {
            return (name.startsWith("skin_") && name.endsWith(".jar"));
        }
    }

    /**
     * Returns the list of available custom themes.
     */
    public String getCustomThemes()
    {
        String themes = _environment.getResource("LY_JQUERY_CUSTOM_THEMES");

        String skinJarPath = _environment.getEnv("LY_APP_DIR") + "/WEB-INF/lib/";

        File skinDir = new File(skinJarPath);
        if (skinDir.exists() && skinDir.isDirectory())
        {
            if (themes == null)
                themes = "";
            String[] skinJars = skinDir.list(new SkinJarFileFilter());
            for (int i = 0; i < skinJars.length; i++)
            {

                String skinFileName = skinJars[i];
                String skinName = skinFileName.substring(skinFileName.lastIndexOf("_") + 1, skinFileName.lastIndexOf("."));

                if (!themes.contains(skinName))
                {
                    themes = themes + "," + skinName;
                }
            }
        }

        if (themes != null && !"".equals(themes))
        {
            return getAvailableThemes(themes.split(","));
        }
        return "";
    }

	/**
	 * Returns the list of available themes.
	 */
	protected String getAvailableThemes(String[] themes)
	{
		int				i, n = themes.length;
		StringBuffer	buffer = new StringBuffer();

		for (i = 0; i < n; i++)
		{
			if (isThemeAvailable(themes[i]))
			{
				if (buffer.length() > 0)
					buffer.append(", ");

				buffer.append("'" + themes[i] + "'");
			}
		}

		buffer.insert(0, "[");
		buffer.append("]");

		return buffer.toString();
	}

	/**
	 * Checks whether a theme is available or not.
	 *
	 * @param theme the name of the theme
	 * @return true if theme is available, false otherwise
	 */
	protected boolean isThemeAvailable(String theme)
	{
		if (_environment != null)
		{
			String	themePath = _environment.getEnv(LyJQConstants.LY_LEON_DOC_DIR) + "/css/jquery-ui/themes/" + theme;
			File	themeDir = new File(themePath);

			if (!themeDir.exists() || !themeDir.isDirectory())
				return false;
		}

		return true;
	}

	public Object loadImageFromStream(Object component, InputStream resource) throws IOException
	{
		return LyImageUtils.loadImageFromStream((Component) component, resource);
	}

	/**
	 * Answer the interface by calling the answer associated to message associated to the given ID.
	 *
	 * @param  infoId the id of the answer message
	 * @param  result true if the user chose "yes" or "ok", false otherwise
	 * @return the answer interface
	 */
	public LyAnswerInterface answerMessage(String infoId, boolean result)
	{
		LyTemplatesUtilsBean	templateManager = getVeloTemplatesBean();
		String					dialog = templateManager.mergeTemplate(HIDE_DIALOG_TEMPLATE, null);

		addToUpdateScript(dialog);

		LyAnswerInterface		answerInterface = super.answerMessage(infoId, result);

		return answerInterface;
	}

	/**
	 * Answer the interface by calling the answer associated to message associated to the given ID.
	 *
	 * @param  infoId the id of the answer message
	 * @param  result true if the user chossed "yes" or "ok", false otherwise
	 * @return the answer interface
	 */
	public LyAnswerInterface answerMessage(String infoId, short result)
	{
		LyTemplatesUtilsBean	templateManager = getVeloTemplatesBean();
		String					dialog = templateManager.mergeTemplate(HIDE_DIALOG_TEMPLATE, null);

		addToUpdateScript(dialog);

		LyAnswerInterface		answerInterface = super.answerMessage(infoId, result);

		return answerInterface;
	}

	private String getNewWidowScript(String parameters, int type)
	{
		Map<String, Object>	params = new HashMap<String, Object>();

		params.put("type", type);
		params.put("parameters", parameters);

		LyTemplatesUtilsBean	templateManager = getVeloTemplatesBean();
		String					newWindowScript = templateManager.mergeTemplate(NEW_WINDOW_TEMPLATE, params);

		return newWindowScript;
	}

	/**
	 * Generates the HTML code for a <code>_LyWebMessage</code> instance.
	 */
	protected String generateMessage(_LyWebMessage message)
	{
		if ((message._type == TYPE_SHOW_FILE)
				|| (message._type == TYPE_LOCATION)
				|| (message._type == TYPE_LOCATION_NEW_WINDOW))
		{
			String	text = LyWebViewManager.encodeString(message._text);
			String	params = getNewWindowScriptParameters(text, message._parameters);

			return getNewWidowScript(params, message._type);
		}
		else
		{
			Map<String, Object>	params = new HashMap<String, Object>();
			boolean				onlyOneChoice = false;

			if ((message._commands == null) || (message._commands.size() == 0))
			{
				onlyOneChoice = true;
				message._commands = new BitSet();
				message._commands.set(LyViewManager.COMMAND_OK);
			}

			String	text = LyString.replace(message._text, "\r\n", "<br>");

			text = LyString.replace(text, "\n", "<br>");
			params.put("msg",TemplateUtils.getFormattedJs(text));
			params.put("dialogIcon", message._icon);
			params.put("dialogTitle", message._title);
			params.put("dialogCommands", message._commands);
			params.put("environnement", getEnvironment());
			params.put("infoId", message._infoId);
			params.put("onlyOneChoice", onlyOneChoice);

			LyTemplatesUtilsBean templateManager = null;
			templateManager = getVeloTemplatesBean();
			String dialog = templateManager.mergeTemplate(SHOW_DIALOG_TEMPLATE, params);

			return dialog;
		}
	}

	/**
	 * Returns the type of message that must be used to show a PDF preview.
	 */
	protected int getShowPDFPreviewType()
	{
		return TYPE_LOCATION;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see leon.view.LyViewManager#showLocation(leon.view.LyComponent,
	 *      java.lang.String, boolean, Hashtable<Short, String>)
	 */
	public void showLocation(LyComponent source, String location, boolean newWindow, Hashtable<Short, String> parameters)
	{
		if ((location.startsWith(LyWebConstants.TARGET_PARAM)))
		{
			// Add root url with home path before target param
			String	name = LyString.replace(location, "=", "");

			name = LyString.replace(name, ".", "");
			location = getRootURL() + "/" + LyJQConstants.PATH_ACTION_HOME + '?' + location;

			if (parameters == null)
				parameters = new Hashtable<Short, String>(1);

			parameters.put(SHOW_LOCATION_PARAMETER_NAME, name);

			// Register newly opened window in the list of slave windows
			String	jsParams = getNewWindowScriptParameters(LyWebViewManager.encodeString(location), parameters);
			String	newWindowScript = "RegisterNewWindow(" + jsParams + ");";

			addNewWindowScript(newWindowScript);
			addToUpdateScript("RegisterNewWindow(" + jsParams + ");");
		}
		else
			super.showLocation(source, location, newWindow, parameters);
	}

	public String getNewWindowScriptsString()
	{
		Vector<String>	newWindowScripts = getNewWindowScripts();

		if ((newWindowScripts != null) && (newWindowScripts.size() > 0))
		{
			StringBuffer	buffer = new StringBuffer();

			for (int i = 0; i < newWindowScripts.size(); i++)
			{
				buffer.append(newWindowScripts.get(i));
			}

			return buffer.toString();
		}

		return null;
	}

    /**
     * Returns the name of the Java class that must be used to instanciate a new slave session.
     *
     * @return null if default session class must be used, a full java path otherwise.
     */
    public String getSlaveSessionClassName()
    {
    	return "leon.view.web.jquery.LyJQSession";
    }

	/**
	 * set request print mode
	 * @param isPrintMode
	 */
	public void setPrintMode(boolean isPrintMode)
	{
		_isPrintMode = isPrintMode;
	}

	public boolean isPrintMode()
	{
		return _isPrintMode;
	}

	public LyComponent getCurrentTreeMapView()
	{
		return _currentTreeMap;
	}

	public void setCurrentTreeMapView(LyComponent view)
	{
		_currentTreeMap = view;
	}

	public void setCurrentPrintView(LyView currentView)
	{
		_currentPrintView = currentView;

	}

	public LyView getCurrentPrintView()
	{
		return _currentPrintView;
	}

	public boolean isComponentToRefresh(String compId)
	{
		return _componentsToRender.get(compId) != null;
	}

	/**
	 * Return the current web request info object.
	 */
	protected LyWebRequestInfo getWebRequestInfo()
	{
		return LyJQServlet.getWebRequestInfo();
	}

	public LyJQPage instanciatePageWriter(LyComponent component, boolean writeDivOnly)
	{
		return new LyJQPage(component, writeDivOnly);
	}

	/**
	 * @return the _isAutoCompleteEvent
	 */
	public boolean isAutoCompleteEvent() {
		return _isAutoCompleteEvent;
	}

	/**
	 * @param _isAutoCompleteEvent the _isAutoCompleteEvent to set
	 */
	public void setAutoCompleteEvent(boolean _isAutoCompleteEvent) {
		this._isAutoCompleteEvent = _isAutoCompleteEvent;
	}

	/**
	 * Indicates whether file components must be customized in web browser.
	 * Customization is not used with IE since there are security issues with IE file selector:
	 * use standard file selector
	 *
	 * @return true if a customized file selector must be used, false otherwise
	 */
	public boolean useCustomizedFilecomponent()
	{
		return (getBrowser() != LyWebViewManager.BROWSER_MSIE);
	}

    /**
     * Indicates whether RTL (right to left) mode is activated
     * @return true is RTL mode is activated, false otherwise.
     */
    public boolean isRtlMode()
    {
		return _environment.getLanguage().equals("ar");
    }

    @Override
    public boolean isPortletMode()
    {
    	return _isPortlet;
    }

    /***
     *
     */
    public boolean getIsPortlet()
    {
    	return _isPortlet;
    }
    public void setIsPortlet(boolean isportlet)
    {
    	_isPortlet=isportlet;
    }
}