/*
 * LyWeb2Area.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.jquery.component.area;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import leon.view.LyArea;
import leon.view.LyPanel;
import leon.view.LyToolBar;
import leon.view.graphic.LyGraphicalArea;
import leon.view.setinterface.LySetBorderPanelInterface;
import leon.view.setinterface.LySetVisibleInterface;
import leon.view.web.LyWebPageWriter;
import leon.view.web.jquery.LyJQContainer;
import leon.view.web.jquery.common.TemplateUtils;
import leon.view.web.jquery.component.titleBar.LyJQTitleBar;
import leon.view.web.jquery.controller.LyTemplatesUtilsBean;

/**
 * Type : leon.view.web.web2.frame.LyWeb2Area
 * @version $Revision: -1M $
 * @author ychaari, last modified by $Author: (local) $
 */
public class LyJQArea extends LyJQContainer	implements
											LyGraphicalArea,
											LySetVisibleInterface,
											LySetBorderPanelInterface
{
	public static final String	LAYOUT_INFO_SEP					= " ";

	protected LyArea			_area;

	protected List<String>		_layoutInfoList					= new ArrayList<String>();

	protected static String		PANEL_PARAM						= "panel";

	protected static String		PANEL_TEMPLATE					= "panel/lyPanelTemplate.vm";

	protected static String		ADD_BORDER_PANEL_TEMPLATE		= "ajax/area/addBorderPanelTemplate.vm";

	protected static String		REMOVE_BORDER_PANEL_TEMPLATE	= "ajax/area/removeBorderPanelTemplate.vm";

	protected static String		REMOVE_PANEL_TEMPLATE			= "ajax/area/removePanelTemplate.vm";

	protected static String		CONTENT_PARAM					= "panelDiv";

	protected static String		PANEL_TD_PARAM					= "panelTd";

	protected static String		TR_TO_DISPLAY_PARAM				= "trToDisplay";

	protected static String		TDS_DISPLAY_PARAM				= "tdsToDisplay";

	protected static String		TD_TO_CLEAR						= "tdToClear";

	protected static String		TR_TO_HIDE						= "trToHide";

	protected static String		TDS_TO_HIDE						= "tdsToHide";

	protected static String		CENTER_EAST						= "centerEast_";

	protected static String		TOP_CENTER						= "topCenter_";

	protected static String		NORTH_BORDER					= "northBorder_";

	protected static String		CENTER_WEST						= "centerWest_";

	protected static String		TOP_WEST						= "topWest_";

	protected static String		TOP_EAST						= "topEast_";

	protected static String		BOTTOM_EAST						= "bottomEast_";

	protected static String		BOTTOM_CENTER					= "bottomCenter_";

	protected static String		SOUTH_BORDER					= "southBorder_";

	/**
	 * Constructor
	 * @param area
	 */
	public LyJQArea(LyArea area)
	{
		super(area);
		_area = area;
	}

	/**
	 * get area panels
	 * @return panels list
	 */
	public List<LyPanel> getPanels()
	{
		List<LyPanel> panels = new ArrayList<LyPanel>();
		LyPanel panel;
		for (int panelIndex = 0; panelIndex < _area.getPanelCount(); panelIndex++)
		{
			panel = _area.getPanel(panelIndex);
			panels.add(panel);
		}
		return panels;
	}

	/**
	 * getNorthPanel
	 */
	public void computePanelsPosition()
	{
		String layoutInfo = _area.getLayoutInfo();
		String[] layoutPanes = layoutInfo.split(LAYOUT_INFO_SEP);

		for (int index = 0; index < layoutPanes.length; index++)
		{
			_layoutInfoList.add(index, layoutPanes[index]);
		}
	}

	/**
	 * getColumnNumber
	 */
	public int getColumnNumber()
	{
		_area.computePanelsPositions();
		return _area.getGridSize()[0];

	}

	/**
	 * getNorthPanel
	 */
	public LyPanel getNorthPanel()
	{
		int northPanelIndex = _layoutInfoList.indexOf("N");
		List<LyPanel> panels = getPanels();
		if (northPanelIndex != -1 && northPanelIndex < panels.size())
		{
			return panels.get(northPanelIndex);
		}
		return null;
	}

	/**
	 * getEastPanel
	 */
	public LyPanel getEastPanel()
	{
		int eastPanelIndex = _layoutInfoList.indexOf("E");
		List<LyPanel> panels = getPanels();
		if (eastPanelIndex != -1 && eastPanelIndex < panels.size())
		{
			return panels.get(eastPanelIndex);
		}
		return null;
	}

	/**
	 * getCenterPanel
	 */
	public LyPanel getCenterPanel()
	{
		int centerPanelIndex = _layoutInfoList.indexOf("C");
		List<LyPanel> panels = getPanels();
		if (centerPanelIndex != -1 && centerPanelIndex < panels.size())
		{
			return panels.get(centerPanelIndex);
		}
		return null;
	}

	/**
	 * getWestPanel
	 */
	public LyPanel getWestPanel()
	{
		int westPanelIndex = _layoutInfoList.indexOf("W");
		List<LyPanel> panels = getPanels();
		if (westPanelIndex != -1 && westPanelIndex < panels.size())
		{
			return panels.get(westPanelIndex);
		}
		return null;
	}

	/**
	 * getSouthPanel
	 */
	public LyPanel getSouthPanel()
	{
		int southPanelIndex = _layoutInfoList.indexOf("S");
		List<LyPanel> panels = getPanels();
		if (southPanelIndex != -1 && southPanelIndex < panels.size())
		{
			return panels.get(southPanelIndex);
		}
		return null;
	}

	/**
	 * Get panel by layoutInfo if layoutInfo=null return the last panel used in the CARD layout
	 */
	public LyPanel getPanel(String layoutInfo)
	{
		List<LyPanel> panels = getPanels();
		if (!panels.isEmpty())
		{
			if (layoutInfo == null || layoutInfo.equals(""))
			{
				return panels.get(panels.size() - 1);
			}
			for (LyPanel currentPanel : panels)
			{
				if (currentPanel.getId().equals(layoutInfo))
					return currentPanel;
			}
		}
		return null;

	}

	/**
	 * Getting LyWeb2TitleBar
	 * @return LyWeb2TitleBar
	 */
	public LyJQTitleBar getTitleBar()
	{
		if (_area.getTitleBar() != null)
			return (LyJQTitleBar) _area.getTitleBar().getInternalView();
		else
			return null;
	}

	/**
	 * @return the northBorderPanel
	 */
	public LyPanel getNorthBorderPanel()
	{
		return _area.getBorderPanel(LyArea.NORTH);
	}

	/**
	 * @return the southBorderPanel
	 */
	public LyPanel getSouthBorderPanel()
	{
		return _area.getBorderPanel(LyArea.SOUTH);
	}

	/**
	 * @return the eastBorderPanel
	 */
	public LyPanel getEastBorderPanel()
	{
		return _area.getBorderPanel(LyArea.EAST);
	}

	/**
	 * @return the westBorderPanel
	 */
	public LyPanel getWestBorderPanel()
	{
		return _area.getBorderPanel(LyArea.WEST);
	}

	/**
	 * @return the bottomLeftToolbar
	 */
	public LyToolBar getBottomLeftToolbar()
	{
		return _area.getBottomLeftToolBar();
	}

	/**
	 * @return the bottomRightToolbar
	 */
	public LyToolBar getBottomRightToolbar()
	{
		return _area.getBottomRightToolBar();
	}

	public void writeHtml(LyWebPageWriter pw)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();
		
		params.put("lyArea", _area);
		
		String	result = templateManager.mergeTemplate("area/lyAreaTemplate.vm", params);
		
		pw.append(result);
	}

	public void addPanel(LyPanel panel, int pos)
	{
		_viewManager.addToReloadComponents(panel);
	}

	public void removePanel(LyPanel panel)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		HashMap<String, Object> params = new HashMap<String, Object>();
		params.put("panelsContainerId", "panelsContainer_content_"
				+ ((LyJQArea) panel.getParent().getInternalView()).getId());
		_viewManager
				.addToUpdateScript(templateManager.mergeTemplate(REMOVE_PANEL_TEMPLATE, params));
		params = null;
	}

	public void setLayoutInfo(String layoutInfo)
	{
		// Nop
	}

	public void setOpen(boolean open)
	{
		// Nop
	}

	public void setRedirect(String redirect)
	{
		// Nop
	}

	public void setRefresh(int refresh)
	{
		// Nop
	}

	public void setBorderPanel(LyPanel panel, short position)
	{
		if ((panel != null) && (position != -1))
		{
			LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
			Map<String, Object>		params = new HashMap<String, Object>();

			params.put(PANEL_PARAM, panel);

			String panelDiv = templateManager.mergeTemplate(PANEL_TEMPLATE, params);
			
			panelDiv = TemplateUtils.getFormattedJs(panelDiv);
			params = new HashMap<String, Object>();
			params.put(CONTENT_PARAM, panelDiv);

			String			panelTdParam = null;
			String			trToDisplay = null;
			List<String>	tdsToDisplay = new ArrayList<String>();
			
			if (position == LyArea.NORTH)
			{
				panelTdParam = TOP_CENTER + getId();
				trToDisplay = NORTH_BORDER + getId();
				tdsToDisplay.add(panelTdParam);
				
				if (getWestBorderPanel() != null)
					tdsToDisplay.add(TOP_WEST + getId());

				if (getEastBorderPanel() != null)
					tdsToDisplay.add(TOP_EAST + getId());
			}
			else if (position == LyArea.WEST)
			{
				panelTdParam = CENTER_WEST + getId();
				tdsToDisplay.add(panelTdParam);

				if (getNorthBorderPanel() != null)
					tdsToDisplay.add(TOP_WEST + getId());

				if (getSouthBorderPanel() != null)
					tdsToDisplay.add("bottomWest_" + getId());
			}
			else if (position == LyArea.EAST)
			{
				panelTdParam = CENTER_EAST + getId();
				tdsToDisplay.add(panelTdParam);

				if (getNorthBorderPanel() != null)
					tdsToDisplay.add(TOP_EAST + getId());

				if (getSouthBorderPanel() != null)
					tdsToDisplay.add(BOTTOM_EAST + getId());
			}
			else if (position == LyArea.SOUTH)
			{
				panelTdParam = BOTTOM_CENTER + getId();
				tdsToDisplay.add(panelTdParam);
				trToDisplay = SOUTH_BORDER + getId();
				
				if (getWestBorderPanel() != null)
					tdsToDisplay.add("bottomWest_" + getId());

				if (getEastBorderPanel() != null)
					tdsToDisplay.add(BOTTOM_EAST + getId());
			}

			params.put(PANEL_TD_PARAM, panelTdParam);
			params.put(TR_TO_DISPLAY_PARAM, trToDisplay);
			params.put(TDS_DISPLAY_PARAM, tdsToDisplay);
			_viewManager.addToUpdateScript(templateManager.mergeTemplate(ADD_BORDER_PANEL_TEMPLATE, params));
			params = null;
		}
		else if (panel == null)
		{
			LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
			Map<String, Object>		params = new HashMap<String, Object>();
			String					trToHide = null;
			String					tdToClear = "";
			List<String>			tdsToHide = new ArrayList<String>();

			if (position == LyArea.NORTH)
			{
				tdToClear = TOP_CENTER + getId();
				trToHide = NORTH_BORDER + getId();
				tdsToHide.add(tdToClear);
			}
			else if (position == LyArea.WEST)
			{
				tdToClear = CENTER_WEST + getId();
				tdsToHide.add(tdToClear);

				if (getNorthBorderPanel() != null)
					tdsToHide.add(TOP_WEST + getId());

				if (getSouthBorderPanel() != null)
					tdsToHide.add("bottomWest_" + getId());
			}
			else if (position == LyArea.EAST)
			{
				tdToClear = CENTER_EAST + getId();
				tdsToHide.add(tdToClear);

				if (getNorthBorderPanel() != null)
					tdsToHide.add(TOP_EAST + getId());

				if (getSouthBorderPanel() != null)
					tdsToHide.add(BOTTOM_EAST + getId());
			}
			else if (position == LyArea.SOUTH)
			{
				tdToClear = BOTTOM_CENTER + getId();
				tdsToHide.add(tdToClear);
				trToHide = SOUTH_BORDER + getId();
			}

			params.put(TD_TO_CLEAR, tdToClear);
			params.put(TR_TO_HIDE, trToHide);
			params.put(TDS_TO_HIDE, tdsToHide);
			_viewManager.addToUpdateScript(templateManager.mergeTemplate(REMOVE_BORDER_PANEL_TEMPLATE, params));
			params = null;
		}

	}

	public boolean hasSpecificBackgroundImage()
	{
		String	bgImage = _area.getEnvironment().getImageUrl("LY_AREA_BACKGROUND_IMAGE");
		LyArea	parentArea = _area.getParentArea();
		String	parentBgImage = ((parentArea == null)
				? null : parentArea.getEnvironment().getImageUrl("LY_AREA_BACKGROUND_IMAGE"));

		return (bgImage != null) && (parentBgImage != null) && !bgImage.equals(parentBgImage);
	}

	public void setVisible(boolean visible)
	{
		String	areaId = getId();
		
		if (visible)
			_viewManager.addToUpdateScript("jQuery('#" + areaId + "').show();");
		else
			_viewManager.addToUpdateScript("jQuery('#" + areaId + "').hide();");
	}

	@Override
	public String getReloadScript()
	{
		return null;
	}
}