package leon.view.web.jquery.component.cell;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import leon.misc.LyEnvironment;
import leon.misc.LyString;
import leon.view.*;
import leon.view.graphic.LyGraphicalCell;
import leon.view.web.LyWebPageWriter;
import leon.view.web.jquery.LyJQConstants;
import leon.view.web.jquery.component.LyJQInternalView;
import leon.view.web.jquery.component.table.LyJQTable;
import leon.view.web.jquery.controller.LyTemplatesUtilsBean;

/**
 * LyCell internal view
 * @author bsmichet
 */
public class LyJQCell extends LyJQInternalView	implements
												LyGraphicalCell
{
	/** Cell component. */
	protected LyCell			_cell;

	protected LyTable			_parentTable;

	protected final String		CELL_LABEL_TEMPLATE				= "table/actions/cellLabel.vm";

	protected final String		CELL_EDITABLE_TEMPLATE			= "table/actions/cellEditable.vm";

	protected final String		CELL_SELECTED_TEMPLATE			= "table/actions/cellSelected.vm";

	protected final String		CELL_IMAGE_TEMPLATE				= "table/actions/cellImage.vm";

	protected final String		CELL_TIP_TEMPLATE				= "table/actions/cellTooltip.vm";

	protected final String		CELL_BORDER_COLOR_TEMPLATE		= "table/actions/cellBorderColor.vm";

	protected final String		CELL_POS_IMAGE_TEMPLATE			= "table/actions/cellPosImage.vm";

	protected final String		TABLE_ID_TEMPLATE_PARAM			= "tableId";

	protected final String		CELL_ID_TEMPLATE_PARAM			= "cellId";

	protected final String		CELL_LABEL_PARAM				= "cellLabel";

	protected final String		IS_CELL_EDITABLE_PARAM			= "cellIsEditable";

	protected final String		GRID_PARAM						= "grid";

	protected final String		IMAGE_PARAM						= "image";

	protected final String		IS_CELL_SELECTED_PARM			= "cellIsSelected";

	protected final String		CELL_TOOLTIP_PARM				= "cellTooltip";

	protected final String		CELL_TOP_BORDER_COLOR_PARAM		= "cellTopBorderColor";

	protected final String		CELL_BOTTOM_LEFT_IMAGE_PARAM	= "cellBottomLeftImage";

	protected final String		CELL_BOTTOM_BORDER_COLOR		= "cellBottomBorderColor";

	protected final String		CELL_LEFT_BORDER_COLOR			= "cellLeftBorderColor";

	protected final String		CELL_RIGHT_BORDER_COLOR			= "cellRightBorderColor";

	protected final String		CELL_TOP_LEFT_IMAGE				= "cellTopLeftImage";

	protected final String		CELL_TOP_RIGHT_IMAGE			= "cellTopRightImage";

	protected final String		CELL_BOTTOM_RIGHT_IMAGE			= "cellBottomRightImage";


	public LyJQCell(LyCell cell)
	{
		super(cell);
		_cell = cell;
		_parentTable = getParentTable();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalCell#setLabel(java.lang.String)
	 */
	public void setLabel(String label)
	{
		if ((_cell.getObjectId() != null) && (_cell.getObjectId().length() > 0))
		{
			reloadTable();
			return;
		}

		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(TABLE_ID_TEMPLATE_PARAM, ((LyJQTable) _parentTable.getInternalView()).getId());
		params.put(CELL_ID_TEMPLATE_PARAM, getId());
		if (_cell.getType() != LyCell.Type.RICH)
		{
		    params.put(CELL_LABEL_PARAM, getHtmlString(label));
		}
		else
		{
		    params.put(CELL_LABEL_PARAM, translate(label));
		}
		String newDiv = templateManager.mergeTemplate(CELL_LABEL_TEMPLATE, params);
		_viewManager.addToUpdateScript(newDiv);
		params = null;

	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalCell#setObjectId(java.lang.String)
	 */
	public void setObjectId(String objectId)
	{
		reloadTable();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalCell#setEditable(boolean)
	 */
	public void setEditable(boolean editable)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(TABLE_ID_TEMPLATE_PARAM, ((LyJQTable) _parentTable.getInternalView()).getId());
		params.put(CELL_ID_TEMPLATE_PARAM, getId());
		params.put(IS_CELL_EDITABLE_PARAM, editable);

		String newDiv = templateManager.mergeTemplate(CELL_EDITABLE_TEMPLATE, params);

		_viewManager.addToUpdateScript(newDiv);
		params = null;
	}

	public LyTable getParentTable()
	{
		if(_parentTable == null){
		LyComponent parent = _cell;
		while (!(parent instanceof LyTable))
			parent = parent.getParent();

		_parentTable = (LyTable) parent;
		}
		return _parentTable;
	}

	/**
	 * Gets the cell alignment.
	 */
	public String getAlign(int columnIndex)
	{
		LyTable			table = getParentTable();
		LyHeaders       headers = table.getHeaders();
		String			align = null;

		if (headers != null)
		{
			LyHeader header = headers.getHeader(columnIndex);

			if (header.getAlign() == LyHeader.Align.CENTER)
				align = "center";
			else if (header.getAlign() == LyHeader.Align.RIGHT)
				align = "right";
		}

		return align;
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalCell#setSelected(boolean)
	 */
	public void setSelected(boolean selected)
	{

		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(TABLE_ID_TEMPLATE_PARAM, ((LyJQTable) _parentTable.getInternalView()).getId());
		params.put(CELL_ID_TEMPLATE_PARAM, getId());
		params.put(GRID_PARAM, "grid_" + ((LyJQTable) _parentTable.getInternalView()).getId());
		params.put(IS_CELL_SELECTED_PARM, selected);
		String newDiv = templateManager.mergeTemplate(CELL_SELECTED_TEMPLATE, params);
		_viewManager.addToUpdateScript(newDiv);
		params = null;

	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalCell#setImage(java.lang.String)
	 */
	public void setImage(String image)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(TABLE_ID_TEMPLATE_PARAM, ((LyJQTable) _parentTable.getInternalView()).getId());
		params.put(CELL_ID_TEMPLATE_PARAM, getId());
		params.put(IMAGE_PARAM, getImageCellUrl(image));
		String newDiv = templateManager.mergeTemplate(CELL_IMAGE_TEMPLATE, params);
		_viewManager.addToUpdateScript(newDiv);
		params = null;

	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalCell#setTooltip(java.lang.String)
	 */
	public void setTooltip(String tooltip)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(TABLE_ID_TEMPLATE_PARAM, ((LyJQTable) _parentTable.getInternalView()).getId());
		params.put(CELL_ID_TEMPLATE_PARAM, getId());
		params.put(CELL_TOOLTIP_PARM, tooltip);
		String newDiv = templateManager.mergeTemplate(CELL_TIP_TEMPLATE, params);
		_viewManager.addToUpdateScript(newDiv);
		params = null;

	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalCell#setTopBorderColor(java.lang.String)
	 */
	public void setTopBorderColor(String topBorderColor)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(TABLE_ID_TEMPLATE_PARAM, ((LyJQTable) _parentTable.getInternalView()).getId());
		params.put(CELL_ID_TEMPLATE_PARAM, getId());
		params.put(CELL_TOP_BORDER_COLOR_PARAM, topBorderColor);
		String newDiv = templateManager.mergeTemplate(CELL_BORDER_COLOR_TEMPLATE, params);
		_viewManager.addToUpdateScript(newDiv);
		params = null;

	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalCell#setBottomBorderColor(java.lang.String)
	 */
	public void setBottomBorderColor(String bottomBorderColor)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(TABLE_ID_TEMPLATE_PARAM, ((LyJQTable) _parentTable.getInternalView()).getId());
		params.put(CELL_ID_TEMPLATE_PARAM, getId());
		params.put(CELL_BOTTOM_BORDER_COLOR, bottomBorderColor);
		String newDiv = templateManager.mergeTemplate(CELL_BORDER_COLOR_TEMPLATE, params);
		_viewManager.addToUpdateScript(newDiv);
		params = null;

	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalCell#setLeftBorderColor(java.lang.String)
	 */
	public void setLeftBorderColor(String leftBorderColor)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(TABLE_ID_TEMPLATE_PARAM, ((LyJQTable) _parentTable.getInternalView()).getId());
		params.put(CELL_ID_TEMPLATE_PARAM, getId());
		params.put(CELL_LEFT_BORDER_COLOR, leftBorderColor);
		String newDiv = templateManager.mergeTemplate(CELL_BORDER_COLOR_TEMPLATE, params);
		_viewManager.addToUpdateScript(newDiv);
		params = null;

	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalCell#setRightBorderColor(java.lang.String)
	 */
	public void setRightBorderColor(String rightBorderColor)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(TABLE_ID_TEMPLATE_PARAM, ((LyJQTable) _parentTable.getInternalView()).getId());
		params.put(CELL_ID_TEMPLATE_PARAM, getId());
		params.put(CELL_RIGHT_BORDER_COLOR, rightBorderColor);
		String newDiv = templateManager.mergeTemplate(CELL_BORDER_COLOR_TEMPLATE, params);
		_viewManager.addToUpdateScript(newDiv);
		params = null;

	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalCell#setTopLeftImage(java.lang.String)
	 */
	public void setTopLeftImage(String topLeftImage)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(TABLE_ID_TEMPLATE_PARAM, ((LyJQTable) _parentTable.getInternalView()).getId());
		params.put(CELL_ID_TEMPLATE_PARAM, getId());
		params.put(CELL_TOP_LEFT_IMAGE, _cell.getEnvironment().getImageUrl(topLeftImage));
		String newDiv = templateManager.mergeTemplate(CELL_POS_IMAGE_TEMPLATE, params);
		_viewManager.addToUpdateScript(newDiv);
		params = null;

	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalCell#setTopRightImage(java.lang.String)
	 */
	public void setTopRightImage(String topRightImage)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(TABLE_ID_TEMPLATE_PARAM, ((LyJQTable) _parentTable.getInternalView()).getId());
		params.put(CELL_ID_TEMPLATE_PARAM, getId());
		params.put(CELL_TOP_RIGHT_IMAGE, _cell.getEnvironment().getImageUrl(topRightImage));
		String newDiv = templateManager.mergeTemplate(CELL_POS_IMAGE_TEMPLATE, params);
		_viewManager.addToUpdateScript(newDiv);
		params = null;

	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalCell#setBottomLeftImage(java.lang.String)
	 */
	public void setBottomLeftImage(String bottomLeftImage)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(TABLE_ID_TEMPLATE_PARAM, ((LyJQTable) _parentTable.getInternalView()).getId());
		params.put(CELL_ID_TEMPLATE_PARAM, getId());
		params.put(CELL_BOTTOM_LEFT_IMAGE_PARAM, _cell.getEnvironment()
				.getImageUrl(bottomLeftImage));
		String newDiv = templateManager.mergeTemplate(CELL_POS_IMAGE_TEMPLATE, params);
		_viewManager.addToUpdateScript(newDiv);
		params = null;

	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalCell#setBottomRightImage(java.lang.String)
	 */
	public void setBottomRightImage(String bottomRightImage)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(TABLE_ID_TEMPLATE_PARAM, ((LyJQTable) _parentTable.getInternalView()).getId());
		params.put(CELL_ID_TEMPLATE_PARAM, getId());
		params.put(CELL_BOTTOM_RIGHT_IMAGE, _cell.getEnvironment().getImageUrl(bottomRightImage));
		String newDiv = templateManager.mergeTemplate(CELL_POS_IMAGE_TEMPLATE, params);
		_viewManager.addToUpdateScript(newDiv);
		params = null;

	}

	public void setColor(String color)
	{
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalCell#addCell(leon.view.LyCell, int)
	 */
	public void addCell(LyCell cells, int pos)
	{
		reloadTable();

	}

	public void removeCell(LyCell cells)
	{
		reloadTable();

	}

	/**
	 * get sub cells list
	 * @return sub cell list
	 */
	public List<LyCell> getSubCellList(LyCell cell)
	{
		List<LyCell> cellList = new ArrayList<LyCell>();
		if (cell.getCellCount() == 0)
			cellList.add(cell);
		else
			for (int i = 0; i < cell.getCellCount(); i++)
			{
				cellList.addAll(getSubCellList(cell.getCell(i)));
			}

		return cellList;
	}

	public void writeHtml(LyWebPageWriter pw)
	{
		// TODO Auto-generated method stub
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.jquery.component.LyJQInternalView#getReloadScript()
	 */
	@Override
	public String getReloadScript()
	{
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * get image cell url
	 */
	public String getImageCellUrl(String image)
	{
		String url = null;
		String docRoot = expandUrl(getEnv(LyJQConstants.LY_APP_DOC));
		String docDir = expandUrl(getEnv(LyJQConstants.LY_APP_DOC_DIR));

		if ((image != null) && (image.length() > 0))
		{
			image = expandUrl(image);

			if (image.startsWith(docDir))
			{
				image = image.substring(docDir.length());
				image = getEnv(LyJQConstants.LY_APP_DOC) + image;
			}
			else
			{
				docDir = expandUrl(getEnv(LyJQConstants.LY_LEON_DOC_DIR));
				if (image.startsWith(docDir))
				{
					image = image.substring(docDir.length());
					image = getEnv(LyJQConstants.LY_LEON_DOC) + image;
				}
			}

			if (image.startsWith(docRoot))
				url = expandUrl(image);
			else if (image.startsWith("file"))
				url = LyString.encodeFileURL(image);
			else if (image.startsWith("http") || image.startsWith("ftp"))
				url = image;
			else
				url = getImageUrl(image);

			LyEnvironment environment = getEnvironment();
			return environment.getPathWithoutSize(url);
		}

		return url;
	}

	/**
	 * reload table
	 */
	public void reloadTable()
	{
		_viewManager.addToReloadComponents(getParentTable());
	}
}
