package leon.view.web.jquery.component.chart;

import java.awt.Color;
import java.awt.Paint;
import java.text.*;
import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.LyChartBase.GraphType;
import leon.view.common.*;
import leon.view.getinterface.*;
import leon.view.graphic.*;
import leon.view.setinterface.*;
import leon.view.web.*;
import leon.view.web.jquery.*;
import leon.view.web.jquery.component.panel.*;
import leon.view.web.jquery.controller.*;

/**
 * Type : leon.view.web.jquery.component.chart The peer component of LyTab for the web2 viewer.
 * @version $Revision: -1M $
 * @author abkristou
 */
public class LyJQChart extends LyJQPanel implements
										LyGraphicalChart,
										LyGetChartConverterInterface,
										LySetBoundsInterface
{
	/** Max length for labels displayed in pie charts. */
	public static int				PIE_LABEL_MAX_LENGTH = 20;
	
	/** Default colors used for charts. */
	public static String[]			DEFAULT_COLORS = new String[] {"#5D8AA8","#E32636","#FFBF00",
											"#FF7E00","#FF033E","#9966CC","#CD9575","#915C83",
											"#0000FF","#008000","#8DB600","#4B5320","#3B444B",
											"#6D351A","#6E7F80","#FF2052","#007FFF","#66FF00"};
	
	/** The chart generic component. */
	protected LyChart						_chart;

	/** The generic chart converter for this component. */
	protected LyJQChartConverter			_chartConv;

	/**
	 * Constructor
	 * @param chart
	 */
	public LyJQChart(LyChart chart)
	{
		super(chart);
		_chart = chart;
		initialize();
		setContent();
		
		if (_chart.getGraphType().equals(GraphType.PIE))
			_chart = getChartConverter().invertChart(_chart);
	}

	/**
	 * Initialization method.
	 */
	public void initialize()
	{
		_chartConv = new LyJQChartConverter(_chart);
	}

	public int getLastIndex()
	{
		int	columnCount = _chart.getColumnCount() - 1;

		return (int) Math.round(Math.sqrt(columnCount) + 0.5f);
	}
	
	/**
	 * generate chart image.
	 */
	public String generateChartImage()
	{
		StringBuilder	chartHtml = new StringBuilder();
		String			chartId = _chart.getId();
		String			dateStamp = "";
		String			colorString = "";
		Paint 			paint = _chartConv.getFreeChart().getBackgroundPaint();

		if (paint instanceof Color)
		{
			Color color = (Color)paint;

			colorString = " bgColor='#" + LyColors.getHexString(color.getRed(), color.getGreen(), color.getBlue()) + "'";
		}

		chartHtml.append("<script type='text/javascript'>var selectionContext='true'</script>");

		if ((chartId != null) && (chartId.endsWith("_chart")))
			dateStamp = "_" + (new Date()).getTime();

		// Create the image
		LyViewManager viewManager = getViewManager();
		String        tmpDir = viewManager.getTemporaryDir();
		String        tmpRoot = viewManager.getTemporaryLocation();
		String        imageFile = tmpDir + "/chart_" + chartId + dateStamp;
		String        imagePath = tmpRoot + "/chart_" + chartId + dateStamp;
		StringBuffer  buffer = new StringBuffer();
      
		saveChartFile(tmpDir, imageFile, buffer);

		// Always generate image map: it handles tooltips
		chartHtml.append(buffer.toString());

		chartHtml.append("<center>\n");
		chartHtml.append("<table width='100%' height='100%' cellpadding='0' cellspacing='0'>\n");
		chartHtml.append("<tr width='100%' height='100%' valign='middle'  align='center'>\n");
		chartHtml.append("<td" + colorString + ">\n");

		chartHtml.append(getHtmlLink(imagePath));

		chartHtml.append("</td>\n");
		chartHtml.append("</tr>\n</table>\n");
		chartHtml.append("</center>\n");

		System.gc(); // Force clean up here since use a large amount of memory
	
		return  chartHtml.toString();	
	}
	
	/**
	 * Save the file to the http directory.
	 *
	 * @param tmpDir    temporary directory for the image/svg
	 * @param imageFile name of the file
	 * @param buffer    the buffer where imagemap must be generated
	 */
	public void saveChartFile(String tmpDir, String imageFile, StringBuffer buffer)
	{
		if ((tmpDir == null) || (imageFile == null))
			return;

		// add the extension
		imageFile += ".png";

        _chartConv.setDimension(getWidth(), getHeight());
		_chartConv.saveImg(tmpDir, imageFile, buffer, true);
	}

	/**
	 * Gets the HTML tag that links the image to the ressource.
	 *
	 * @param  imagePath the image path
	 * @return the HTML tag for the image
	 */
	public String getHtmlLink(String imagePath)
	{
		return "<img src='" + imagePath + ".png' border='0' usemap='#chart_imagemap'/>\n";
	}
	
	/**
	 * Sets the content of this view.
	 */
	public void setContent()
	{
		_chartConv.getFreeChart();
	}

	/**
	 * Get Max value of the column
	 * @return float Max value
	 */
	public float yMax()
	{
		int				nbChartValue, i, j;
		LyColumn		colum;
		int				nbColum = _chart.getColumnCount();
		float			max = 0;
		
		if ((nbColum > 0) && (_chart.getColumn(0).getChartValue(0) != null))
			max = Float.valueOf(_chart.getColumn(0).getChartValue(0).getYValue());
		
		LyChartValue	chartValue;
		
		for (j = 0; j < nbColum; j++)
		{
			colum = _chart.getColumn(j);
			nbChartValue = colum.getChartValueCount();
			
			for (i = 0; i < nbChartValue; i++)
			{
				chartValue = colum.getChartValue(i);
				
				if (max < Float.valueOf(chartValue.getYValue()))
					max = Float.valueOf(chartValue.getYValue());
			}
		}

		return max;
	}

	/**
	 * Get Sum max value of the columns
	 * @return float Max value
	 */
	public float ySumMax()
	{
		int			i, j;
		float		sumChartValue;
		LyColumn	colum;
		float		max = Float.valueOf(0);

		int				nbColum = _chart.getColumnCount();
		LyChartValue	chartValue;
		int				nbChartValue = (_chart.getColumn(0) == null ? 0 : _chart.getColumn(0).getChartValueCount());
		
		for (i = 0; i < nbChartValue; i++)
		{
			sumChartValue = 0;
			
			for (j = 0; j < nbColum; j++)
			{
				colum = _chart.getColumn(j);
				chartValue = colum.getChartValue(i);
				sumChartValue += Float.valueOf(chartValue.getYValue());
			}
			
			if (max < sumChartValue)
				max = sumChartValue;
		}
		
		return max;
	}

	/**
	 * Get step for y axis
	 * @return int step
	 */
	public int yStep()
	{
		return Math.round(ySumMax() / 20);
	}

	/**
	 * Format date of x value
	 * @param LyChartValue cv, LyChart c
	 * @return string formated date
	 */
	public String xFormatDate(LyChartValue cv, LyChart c)
	{
		String			returnVal;
		String			destFormat = "";
		LyEnvironment	environment = _chart.getEnvironment();

		try
		{
			switch (c.getIntervals().getXTimeUnit())
			{
			case YEARS:
				destFormat = "yyyy";
				break;
			case MONTHS:
				destFormat = "MMMMM yyyy";
				break;
			case DAYS:
				destFormat = environment.getStringDateFormat(LyEnvironment.TYPE_DATE);
				break;
			case HOURS:
				destFormat = environment.getStringDateFormat(LyEnvironment.TYPE_DATE_HOUR_MIN);
				break;
			case MINS:
				destFormat = environment.getStringDateFormat(LyEnvironment.TYPE_DATE_HOUR_MIN);
				break;
			case SECS:
				destFormat = environment.getStringDateFormat(LyEnvironment.TYPE_DATE_TIME);
				break;
			default:
				destFormat = c.getXFormat();
			}
			
			String sourceFormat = environment.getStringDateFormat(LyTools.intFromString(c.getXDateType()));
			
			SimpleDateFormat	sdfSource = new SimpleDateFormat(sourceFormat);
			SimpleDateFormat	sdfDestination = new SimpleDateFormat(destFormat);
			Date				date = sdfSource.parse(cv.getXValue());
			
			returnVal = sdfDestination.format(date);
		}
		catch (ParseException e)
		{
			returnVal = "";
		}
		
		return returnVal;
	}

	/**
	 * Adds carriage returns in a label in order to shorten its length.
	 */
	public String addLabelCR(String label)
	{
		if (label == null)
			return null;
		
		label = translate(label);
		
		String[]		list = LyString.stringToList(label, ' ');
		int				pos = 0;
		StringBuffer	buffer = new StringBuffer(label.length());

		for (int i = 0; i < list.length; i++)
		{
			if (pos >= PIE_LABEL_MAX_LENGTH)
			{
				buffer.append("\n");
				pos = 0;
			}
			else if (i > 0)
			{
				buffer.append(' ');
				pos++;
			}
			
			pos += list[i].length();
			buffer.append(list[i]);
		}
		
		String	result = buffer.toString();
		
		result = LyString.replace(result, "\n", "\\n");
		
		return result;
	}
	
	/**
	 * Returns the label to be displayed for a section of a pie chart.
	 * 
	 * @param label  the label of the pie section
	 * @param value  the value of the pie section
	 * @param column the column containing the value
	 */
	public String getPieValueLabel(String label, String value, LyColumn column)
	{
		label = addLabelCR(label);
		
		if (label == null)
			return "";
		
		StringBuffer	buffer = new StringBuffer(label);

		if (value != null)
		{
			NumberFormat	format = (_chart.isRatios() ? NumberFormat.getPercentInstance() : NumberFormat.getNumberInstance());
			Double			doubleValue = LyTools.doubleFromString(value);
			
			if ((column != null) && _chart.isRatios() && (_chart.getYType() == LyChart.Type.CHARTNUMBER))
			{
				// If ratios and numeric values, convert value to a ratio
				int				i, n = column.getChartValueCount();
				LyChartValue	chartValue;
				String			yValue;
				double			yValueDouble, ratioTotalValue = 0.0;
				
				for (i = 0; i < n; i++)
				{
					chartValue = column.getChartValue(i);
					yValue = chartValue.getYValue();
					yValueDouble = LyTools.doubleFromString(yValue);
					ratioTotalValue += yValueDouble;
				}
				
				if (ratioTotalValue > 0)
					doubleValue = doubleValue / ratioTotalValue;
			}
			
			value = format.format(doubleValue);
			buffer.append(" : " + value);
		}

		return buffer.toString();
	}
	
	/**
	 * Get Min value of the column
	 * @return float Min value
	 */
	public float yMin()
	{
		float min;
		
		if (_chart.getAxis() != null && _chart.getAxis().getYBegin() != null)
			min = Float.valueOf(_chart.getAxis().getYBegin());
		else
			min = 0;
		
		return min;
	}

	/**
	 * Returns the width of the chart that must be displayed.
	 */
	public int getWidth()
	{
		int	width = getEnvironment().getIntResource("LY_CHART_WIDTH");
		
		if (width < 0)
			width = LyCommonFreeChartConverter.DEFAULT_CHART_WIDTH;
		
		return width;
	}

	/**
	 * Returns the height of the chart that must be displayed.
	 */
	public int getHeight()
	{
		int	height = getEnvironment().getIntResource("LY_CHART_HEIGHT");
		
		if (height < 0)
			height = LyCommonFreeChartConverter.DEFAULT_CHART_HEIGHT;
		
		return height;
	}
	
	/**
	 * Indicates whether this chart is in a popup view.
	 */
	public boolean isInPopup()
	{
		LyView	view = _chart.getParentView();
		
		if ((view instanceof LyDialog) && ((LyDialog)view).getTarget() != null)
			return true;
		
		return false;
	}
	
	/**
	 * Returns the colors that must be used by chart component.
	 */
	public String[] getColors()
	{
		String[]	colors = DEFAULT_COLORS;

		LyJQChartConverter	converter = getChartConverter();
		
		if (converter != null)
		{
			Paint[]			paints = converter.getPaints();
			
			if (paints != null)
			{
				int				i, n = paints.length;
				Color			color;
				String			hexColor;
				String[]		customColors = new String[n];
				
				for (i = 0; i < n; i++)
				{
					if (paints[i] instanceof Color)
					{
						color = (Color)paints[i];
						hexColor = getHexColor(color.getRed() + " " + color.getGreen() + " " + color.getBlue());
						customColors[i] = hexColor;
					}
				}
				
				return customColors;
			}
		}
		
		return colors;
	}
	
	@Override
	public void free()
	{
	}

	public void setLocation(int x, int y)
	{
	}

	public void setSize(int w, int h)
	{
	}

	public LyJQChartConverter getChartConverter()
	{
		return _chartConv;
	}

	public int[] getComponentDimension()
	{
		return null;
	}

	public void addColumn(LyColumn columns, int pos)
	{
	}

	public void removeColumn(LyColumn columns)
	{
	}

	public boolean isChart()
	{
		return true;
	}
	
	public void writeHtml(LyWebPageWriter pw)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();
		
		params.put("lyChart", _chart);
		
		String	result = templateManager.mergeTemplate("chart/lyChartTemplate.vm", params);
		
		pw.append(result);
	}
}
