package leon.view.web.jquery.component.dialog;

import java.util.*;

import javax.servlet.http.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.setinterface.*;
import leon.view.web.*;
import leon.view.web.jquery.*;
import leon.view.web.jquery.common.*;
import leon.view.web.jquery.component.area.*;
import leon.view.web.jquery.controller.*;

/**
 * This class implements the Dialog object (as described in the dtd) for web viewer.
 * @author atalbi
 */
public class LyJQDialog extends LyJQView implements
										LyGraphicalDialog, LySetBoundsInterface
{
	protected static String		CONTENT_PARAM			= "content";

	protected static String		DIALOG_TEMPLATE			= "dialog/lyDialogTemplate.vm";

	protected final String		SHOW_DIALOG_TEMPLATE	= "ajax/dialog/showDialog.vm";

	protected final String		CLOSE_DIALOG_TEMPLATE	= "ajax/dialog/closeDialog.vm";

	protected final String		DIALOG_ID_PARAM			= "dialogId";

	protected final String		DIALOG_LABEL_PARAM		= "dialogLabel";

	protected final String		DIALOG_CONTROLLER_PARAM	= "dController";

	protected final String		DIALOG_AREA_ID_PARAM	= "dialogAreaId";

	protected final String		DIALOG_RESIZABLE_PARAM	= "isResizable";
	
	protected final static String		SHOW_STICKER_DIALOG_TEMPLATE	= "ajax/dialog/showStickerDialog.vm";
	
	protected final static String		NOTICE_TEMPLATE					= "panel/notice/lyNoticeTemplate.vm";
	
	protected static final String 		NOTICE_PARAM 					= "notice";

	/** The dialog generic component. */
	protected LyDialog			_dialog;

	public LyJQDialog(LyDialog dialog)
	{
		super(dialog);
		_dialog = dialog;
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalDialog#setArea(leon.view.LyArea)
	 */
	public void setArea(LyArea area)
	{
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.web.jquery.component.LyJQInternalView#getReloadScript()
	 */
	@Override
	public String getReloadScript()
	{
		return null;
	}

	/**
	 * @see leon.view.web.jquery.LyJQView#setVisible(boolean)
	 */
	@Override
	public void setVisible(boolean status)
	{
		_viewManager.setServletContext(LyTemplatesUtilsBean.getContext());
		
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();
		String					target = _dialog.getTarget();
		boolean					printMode = _viewManager.isPrintMode();
		
		if (status && (status != _visible))
		{
			if (target != null)
			{
				if (!printMode)
				{
					if (target == LyJQConstants.STICKER_TARGET)
					{
						LyNotice notice = getNoticeComponentFromDialog(_dialog);	

						params.put(NOTICE_PARAM, notice);
						
						String noticeDiv = templateManager.mergeTemplate(NOTICE_TEMPLATE, params);

						noticeDiv = TemplateUtils.getFormattedJs(noticeDiv);
						params = new HashMap<String, Object>();
						params.put(LyJQConstants.DIV_TO_APPEND_PARAM, noticeDiv);
						_viewManager.addToUpdateScript(templateManager.mergeTemplate(SHOW_STICKER_DIALOG_TEMPLATE, params));
					}
					else
					{
						String dialogTitle = getEnvironment().translate(_dialog.getLabel());
						
						params.put("viewManager", _viewManager);
						params.put(LyJQConstants.DIALOG_PARAM, _dialog);
						params.put(DIALOG_LABEL_PARAM, TemplateUtils.getConvertedHTML(dialogTitle));
						
						String dialogDiv = templateManager.mergeTemplate(DIALOG_TEMPLATE, params);
						
						dialogDiv = TemplateUtils.getFormattedJs(dialogDiv);
		
						params = new HashMap<String, Object>();
						params.put(LyJQConstants.DIV_TO_APPEND_PARAM, dialogDiv);
						params.put(DIALOG_CONTROLLER_PARAM, _dialog.getController());
						params.put(DIALOG_ID_PARAM, getId());
		
						LyArea lyArea = _dialog.getArea();
						
						if (_dialog.isResizable() && lyArea != null)
						{
							params.put(DIALOG_AREA_ID_PARAM, ((LyJQArea) _dialog.getArea().getInternalView()).getId());
							params.put(DIALOG_RESIZABLE_PARAM, _dialog.isResizable());
						}
		
						_viewManager.addToUpdateScript(templateManager.mergeTemplate(SHOW_DIALOG_TEMPLATE, params));
					}
				}
			}
			else
			{
				if (!printMode)
				{
					LyWebRequestInfo	requestInfo = LyJQServlet.getWebRequestInfo();
					HttpServletRequest	request = (requestInfo == null ? null : requestInfo.getRequest());
					String				isAjaxMode = (request == null ? "true" : request.getParameter(LyJQConstants.AJAX_MODE));
					
					if ((isAjaxMode != null) && !isAjaxMode.equalsIgnoreCase("false"))
						_viewManager.redirectToHomePage();
				}
				else
					_viewManager.setCurrentPrintView(_dialog);
			}
		}
		else if (!status)
		{
			if (target != null)
			{
				params.put(DIALOG_ID_PARAM, getId());

				String	closeEvent = templateManager.mergeTemplate(CLOSE_DIALOG_TEMPLATE, params);

				_viewManager.addToUpdateScript(closeEvent);
			}
			else
			{
				if (_viewManager.getCurrentPrintView() != null)
					_viewManager.setCurrentPrintView(null);

				if (!printMode)
					_viewManager.redirectToHomePage(_dialog);
			}
		}

		params = null;
		super.setVisible(status);
	}

	private LyNotice getNoticeComponentFromDialog(LyDialog lyDialog)
	{
		LyArea		areaComponent = lyDialog.getArea();
		LyNotice	noticeComponent =null;
		
		if (areaComponent!=null	)
			noticeComponent =(LyNotice) areaComponent.getPanel(0);
						
		return noticeComponent;
	}

	/**
	 * add the dialog script
	 */
	public String addDialogRefreshScript()
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();
		String					result = null;
		
		if (_dialog.getTarget()==LyJQConstants.STICKER_TARGET)
		{
			LyNotice notice = getNoticeComponentFromDialog(_dialog);	

			params.put(NOTICE_PARAM, notice);
			
			String noticeDiv = templateManager.mergeTemplate(NOTICE_TEMPLATE, params);
			
			noticeDiv = TemplateUtils.getFormattedJs(noticeDiv);
			noticeDiv = LyString.replace(noticeDiv, "<script", "<scr\" + \"ipt");
			noticeDiv = LyString.replace(noticeDiv, "</script", "</scr\" + \"ipt");
			params = new HashMap<String, Object>();
			params.put(LyJQConstants.DIV_TO_APPEND_PARAM, noticeDiv);
			result = templateManager.mergeTemplate(SHOW_STICKER_DIALOG_TEMPLATE, params);
			params = null;
		}
		else
		{
			String dialogTitle = _dialog.getEnvironment().translate(_dialog.getLabel());
			
			dialogTitle = TemplateUtils.getConvertedHTML(dialogTitle);
			params.put("viewManager", _viewManager);
			params.put(LyJQConstants.DIALOG_PARAM, _dialog);
			params.put(DIALOG_LABEL_PARAM, dialogTitle);

			String dialogDiv = templateManager.mergeTemplate(DIALOG_TEMPLATE, params);
			
			dialogDiv = TemplateUtils.getFormattedJs(dialogDiv);
			dialogDiv = LyString.replace(dialogDiv, "<script", "<scr\" + \"ipt");
			dialogDiv = LyString.replace(dialogDiv, "</script", "</scr\" + \"ipt");
			params = new HashMap<String, Object>();
			params.put(LyJQConstants.DIV_TO_APPEND_PARAM, dialogDiv);
			params.put(DIALOG_CONTROLLER_PARAM, _dialog.getController());
			params.put(DIALOG_ID_PARAM, getId());

			LyArea lyArea = _dialog.getArea();
			
			if (_dialog.isResizable() && lyArea != null)
			{
				params.put(DIALOG_AREA_ID_PARAM, ((LyJQArea) _dialog.getArea().getInternalView()).getId());
				params.put(DIALOG_RESIZABLE_PARAM, _dialog.isResizable());
			}

			result = templateManager.mergeTemplate(SHOW_DIALOG_TEMPLATE, params);

			params = null;
		}
		
		return result;
	}

	public void writeHtml(LyWebPageWriter pw)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();
		
		params.put("lyDialog", _dialog);
		
		String	result = templateManager.mergeTemplate("dialog/lyDialogTemplate.vm", params);
		
		pw.append(result);
	}

	public void setLabel(String label)
	{
	}

	/**
	 * Accessor to the location.
	 *
	 * @see LySetBoundsInterface#setLocation
	 */
	public void setLocation(int x, int y)
	{
		_dialog.setX(x);
		_dialog.setY(y);
	}

	/**
	 * Accessor to the Size.
	 *
	 * @see LySetBoundsInterface#setSize
	 */
	public void setSize(int w, int h)
	{
		_dialog.setWidth(w);
		_dialog.setHeight(h);
	}

	/**
	 * Returns the label that must be used when dialog is minimized.
	 */
	public String getMinimizedLabel()
	{
		String				label = _dialog.getLabel();
		Vector<LyTitleBar>	titlebars = new Vector<LyTitleBar>();
		
		_dialog.getComponents(LyTitleBar.class, titlebars);
		
		if (titlebars.size() == 1)
		{
			LyTitleBar	titlebar = titlebars.get(0);
			
			label = titlebar.getLeftLabel();
		}
		
		label = getEnvironment().translate(label);
		label = TemplateUtils.getConvertedHTML(label);

		return label;
	}
	
	public boolean isVisible()
	{
		return _visible;
	}
}