package leon.view.web.jquery.component.fieldItem;

import java.util.HashMap;
import java.util.Map;

import leon.view.LyCaption;
import leon.view.LyComponent;
import leon.view.LyDialog;
import leon.view.LyFieldItem;
import leon.view.LyForm;
import leon.view.LyFormField;
import leon.view.LyFrame;
import leon.view.LyLabel;
import leon.view.LyTable;
import leon.view.getinterface.LyGetControllerInterface;
import leon.view.graphic.LyGraphicalFieldItem;
import leon.view.setinterface.LySetLabelInterface;
import leon.view.web.LyWebConstants;
import leon.view.web.LyWebPageWriter;
import leon.view.web.LyWebViewManager;
import leon.view.web.jquery.common.TemplateUtils;
import leon.view.web.jquery.component.LyJQInternalView;
import leon.view.web.jquery.controller.LyTemplatesUtilsBean;

/**
 * LyField Jquery internal view
 * @author ychaari
 */
public class LyJQFieldItem extends LyJQInternalView	implements
													LyGraphicalFieldItem,
													LySetLabelInterface
{
	public static final String	__VERSION		= "$Id: LyJQFieldItem.java -1M 2010-12-05 14:01:33Z (local) $";

	/** The form field component that contains this item. */
	protected LyFormField		_formField;
	
	protected LyFieldItem 		_fieldItem;

	/** The name of the html form where this item is generated. */
	protected String			_formName;

	/**
	 * Class constructor of a Field Item object for the struts target.
	 * @param fieldItem the generic Field Item defined for every type of view targets
	 */
	public LyJQFieldItem(LyFieldItem fieldItem)
	{
		super(fieldItem);
		
		_fieldItem = fieldItem;

		LyComponent parent = fieldItem.getParent();

		if (parent instanceof LyFormField)
			_formField = (LyFormField) parent;
		else if (parent.getParent() instanceof LyFormField)
			_formField = (LyFormField) parent.getParent();
		else
			_formField = null;

		_formName = null;
	}

	/**
	 * Gets the field item.
	 * @return the field item
	 */
	public LyFieldItem getFieldItem()
	{
		return (LyFieldItem) _viewComponent;
	}

	/**
	 * Gets the form field that contains this item.
	 * @return the form field
	 */
	public LyFormField getFormField()
	{
		return _formField;
	}

	/**
	 * (non-Javadoc).
	 * @see leon.view.graphic.LyGraphicalObject#getController()
	 */
	public String getController()
	{
		if (_formField.getParent() instanceof LyGetControllerInterface)
			return ((LyGetControllerInterface) _formField.getParent()).getController();

		return null;
	}

	/**
	 * Gets the caption of the parent form field.
	 * @return the caption
	 */
	public String getCaption()
	{
		LyCaption caption = _formField.getCaption();

		return ((caption != null) ? caption.getLabel() : _formField.getId());
	}

	/**
	 * Gets the form ancestor.
	 * @return the form ancestor
	 */
	public LyForm getForm()
	{
		// Look for an ancestor of type LyForm
		LyComponent ancestor = _formField;

		do
			ancestor = ancestor.getParent();
		while ((ancestor != null) && !(ancestor instanceof LyForm));

		return (LyForm) ancestor;
	}

	/**
	 * Indicates if the field item is editable.
	 * @return true if it is editable, false otherwise
	 */
	public boolean isEditable()
	{
		if (!_formField.isEditable() || !_formField.isEnabled())
			return false;

		LyForm form = ((_formField.getParent() instanceof LyForm) ? (LyForm) _formField.getParent()
				: null);

		return ((form == null) || form.isEditable());
	}

	/**
	 * Indicates if the field item is optional.
	 * @return true if it is optional, false otherwise
	 */
	public boolean isOptional()
	{
		return _formField.isOptional();
	}

	public boolean isLabelInstance(LyFieldItem fieldItem)
	{
		return (fieldItem instanceof LyLabel);
	}

	/**
	 * Check if text is a mail
	 * @param label
	 * @return
	 */
	public boolean isMail(String label)
	{
		if ((label.indexOf(' ') < 0) && (label.indexOf('@') > 0) && (label.indexOf("mailto:") < 0))
		{
			// Checks that the domain part of the label is an IP address
			String		domain = label.substring(label.lastIndexOf('@') + 1);
			String[]	domainParts = domain.split("\\.");
			String		part;
			boolean		isIp = false;
			
			if (domainParts.length == 4)
			{
				isIp = true;
				
				for (int i = 0; i < domainParts.length && isIp; i++)
				{
					part = domainParts[i];
					isIp &= part.matches("(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)");
				}
			}

			// If the domain is not an IP address, then the label is
			// an email
			if (!isIp)
				return true;
		}
		
		return false;
	}

	/**
	 * Check if text is an url recurse
	 * @param label
	 * @return
	 */
	public boolean isUrlRecurse(String label)
	{
		return (!isMail(label) && (label != null) && ((label.indexOf("http://") >= 0) || (label
				.indexOf("mailto:") >= 0)));
	}

	/**
	 * Check if text is an url
	 * @param label
	 * @return
	 */
	public boolean isUrl(String label)
	{
		return (label != null) && (isUrlRecurse(label) || (label.indexOf("www.") == 0));
	}

	/**
	 * Check if text is in a print mode
	 * @param text
	 * @return
	 */
	public boolean isPrintMode()
	{
		return _viewManager.getPageWriter().isPrintMode();
	}

	/**
	 * Gets target string
	 * @return
	 */
	public String getTarget()
	{
		if (LyWebViewManager.WEB_FILE_NEW_WINDOW)
			return "_blank";

		return LyWebConstants.TARGET_FRAME;
	}

	/**
	 * Indicates if the Web browser can disabled its elements.
	 * @return true if it can, false otherwise
	 */
	public boolean canDisabled()
	{
		return (!isNetscape() || (_viewManager.getBrowserVersion() >= 5.0));
	}

	/**
	 * Indicates if the Web browser used is Netscape.
	 * @return true if it is Netscape, false otherwise
	 */
	public boolean isNetscape()
	{
		return (_viewManager.getBrowser() == LyWebViewManager.BROWSER_NETSCAPE);
	}

	/**
	 * Indicates if the field item needs check.
	 * @return true if it needs check, false otherwise
	 */
	public boolean needCheck()
	{
		return _formField.isNeedCheck();
	}

	public void setLabel(String label)
	{
		if (label != null)
		{
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("componentId", getId());
		label = TemplateUtils.getFormattedJs(label);
		params.put("value", label);
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		String script = templateManager.mergeTemplate("ajax/form/field/setLabel.vm", params);
		_viewManager.addToUpdateScript(script);
		params = null;
		}
	}

	public void writeHtml(LyWebPageWriter pw)
	{
		// Nop
	}

	@Override
	public String getReloadScript()
	{
		// TODO Auto-generated method stub
		return null;
	}

	public boolean isEnabled()
	{
		return _formField.isEnabled();
	}
	
	/**
	 * get parent table if text is belonging to a table component
	 * @return
	 */
	protected LyTable getParentTable()
	{
		LyComponent parent = _fieldItem.getParent();

		while (!(parent instanceof LyTable || parent instanceof LyFrame || parent instanceof LyDialog))
		{
			parent = parent.getParent();
		}

		if (parent instanceof LyTable)
			return (LyTable) parent;

		return null;
	}
}