package leon.view.web.jquery.component.footer;

import java.util.HashMap;
import java.util.Map;

import leon.view.*;
import leon.view.graphic.LyGraphicalFooter;
import leon.view.web.LyWebPageWriter;
import leon.view.web.jquery.component.LyJQInternalView;
import leon.view.web.jquery.controller.LyTemplatesUtilsBean;

/**
 * LyFooter : table footer JQuery internal view
 * @author bahaSmicket
 */
public class LyJQFooter extends LyJQInternalView implements
												LyGraphicalFooter
{

	/** The footer generic component. */
	protected LyFooter		_footer;

	protected final String	FOOTER_ID_TEMPLATE_PARAM	= "footerId";

	protected final String	FOOTER_LABEL_PARAM			= "footerLabel";

	protected final String	FOOTER_LABEL_TEMPLATE		= "table/actions/footerLabel.vm";

	/**
	 * Class constructor of a Footer object for the struts target.
	 * @param footer the generic Footer defined for every type of view targets
	 */
	public LyJQFooter(LyFooter footer)
	{
		super(footer);
		_footer = footer;
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
		// TODO Auto-generated method stub

	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalFooter#setLabel(java.lang.String)
	 */
	public void setLabel(String label)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(FOOTER_ID_TEMPLATE_PARAM, getId());
		params.put(FOOTER_LABEL_PARAM, getJavascriptString(label));

		String newDiv = templateManager.mergeTemplate(FOOTER_LABEL_TEMPLATE, params);

		_viewManager.addToUpdateScript(newDiv);
		newDiv = null;
		params = null;

	}
	
	/**
	 * Gets the footer alignment.
	 */
	public String getAlign()
	{
		LyFooters		footers = (LyFooters)_footer.getParent();
		LyTable		    table = (LyTable)footers.getParent();
		LyHeaders       headers = table.getHeaders();
		String			align = "left";

		if (headers != null)
		{
			int		 index = getFooterIndex();
			LyHeader header = headers.getHeader(index);

			if (header.getAlign() == LyHeader.Align.CENTER)
				align = "center";
			else if (header.getAlign() == LyHeader.Align.RIGHT)
				align = "right";
		}
		
		return align;
	}

	/**
	 * Gets the footer index in the parent footers.
	 *
	 * @return the current footer index
	 */
	public int getFooterIndex()
	{
		LyFooters footers = (LyFooters)_footer.getParent();
		int		  nFooters = footers.getFooterCount();

		for (int i = 0; i < nFooters; i++)
		{
			if (footers.getFooter(i).equals(_footer))
				return i;
		}
		return -1;
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.web.jquery.component.LyJQInternalView#getReloadScript()
	 */
	@Override
	public String getReloadScript()
	{
		// TODO Auto-generated method stub
		return null;
	}

}