/* 
 * Lyria-W4 - afficheur_web2
 * LyWeb2Form.java
 * Copyright (c) 2000-2009, Lyria-W4
 * All rights reserved.
 */
package leon.view.web.jquery.component.form;

import java.util.*;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.setinterface.*;
import leon.view.web.*;
import leon.view.web.jquery.component.form.field.*;
import leon.view.web.jquery.component.panel.*;
import leon.view.web.jquery.controller.*;

/**
 * Type : leon.view.web.web2.component.form.LyJQForm The peer component of LyForm for the web2 viewer.
 * @version $Revision: -1M $
 * @author ychaari$
 */
public class LyJQForm extends LyJQPanel	implements
										LyGraphicalForm,
										LySetEnabledInterface,
										LyResetContentInterface
{

	protected LyForm	_form;
	
	/** Workaround for IE 5.0 and tables width within forms. */
	public static boolean	   COMPOSITION_TABLE_WORKAROUND = true;
	
	/** Constant used to generate CSS class name for caption. */
	public static int		   DECO_CAPTION_PARAMETER = 0;
	
	/** Maximum length for not wrapped labels. */
    public static int          LABEL_LENGTH_WRAP = 50;
    
    /** Constant used to generate CSS class name for field item. */
	public static int		   DECO_ITEM_PARAMETER = 1;

	/**
	 * Constructor
	 * @param form
	 */
	public LyJQForm(LyForm form)
	{
		super(form);
		_form = form;
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.web.jquery.component.panel.LyJQPanel#setPopupMenu(leon.view.LyPopupMenu)
	 */
	@Override
	public void setPopupMenu(LyPopupMenu popup)
	{
		// Nop
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LyResetContentInterface#resetContent()
	 */
	public void resetContent()
	{
		// Nop
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LyResetContentInterface#resetContent(boolean)
	 */
	public void resetContent(boolean full)
	{
		// Nop
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetEnabledInterface#setEnabled(boolean)
	 */
	public void setEnabled(boolean status)
	{
		// Nop
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalForm#setEditable(boolean)
	 */
	public void setEditable(boolean editable)
	{
		// Nop
	}

	/**
	 * Get form align
	 * @param form
	 * @return
	 */
	public String getAlign(LyForm form)
	{
		String align = "left";
		if(_viewManager.isRtlMode())
			align="right";
		if (form.getAlign() == LyForm.Align.CENTER)
			align = "center";
		else if (_form.getAlign() == LyForm.Align.RIGHT)
			{
			if(_viewManager.isRtlMode())
				align="left";
			else
				align = "right";
			}
	
			
		return align;
	}

	/**
	 * @return ly form fields list
	 */
	public List<LyFormField> getFormFieldList()
	{
		List<LyFormField> formFieldList = new ArrayList<LyFormField>();
		for (int i = 0; i < _form.getFormFieldCount(); i++)
		{
			formFieldList.add(_form.getFormField(i));
		}
		return formFieldList;
	}
	
	public boolean isForm(){
		return true;
	}

	/**
	 * Compute ColSpans of the form fields.
	 *
	 * @param pw the page writer
	 */
	public void computeColSpans()
	{
		int				 i, nFields = _form.getFormFieldCount();
		LyFormField		 item;
		LyCaption		 caption;
		LyJQFormField	 internal;
		LyJQCaption      internalCaption;
		LyConstraint     constraint;
		int				 rowspan, colspan, currentCol = 0;
		boolean			 jump = false, captionJump, remainder;
	    LyConstraint.HAlign hAlignCode;
	    LyConstraint.VAlign vAlignCode;
	    int              colWeight;
		String			 hAlign = null;
		String			 vAlign = null;
		int				 maxNbCols = 2;
		int				 maxUnitNbCols = 1;
		int				 hightNbCols = 2;
		int				 tempNbCols = 0;
		int				 tempHightNbCols = 0;
		boolean			 hasColsOffset = false;
		String			 oldColsString = "", colsString = "";
		
		boolean hasFullWidthArea = false;
		BitSet	areaColspans = new BitSet();
	
		// compute the number ncols of the hight object (hightNbCols)
		// and compute the number ncols of hight line (maxNbCols)
		for (i = 0; i < nFields; i++)
		{
			item = _form.getFormField(i);
	
			caption = item.getCaption();
			if (caption != null)
			{
				constraint = caption.getConstraint();
				if (constraint != null)
				{
					colspan = constraint.getColSpan();
					remainder = constraint.isRemainder();
	
					if (colspan < 0)
						colspan = 1;
	
					if (remainder)
						jump = true;
				}
				else
					colspan = 1;
				
				colsString += colspan;
	
				if (jump)
				{
					tempNbCols = 0;
					tempHightNbCols = 0;
				}
				else
				{
					tempNbCols += colspan;
					tempHightNbCols += colspan;
				}
			}
	
			if (jump)
			{
				if (oldColsString.length() > 0 && !oldColsString.equals(colsString))
					hasColsOffset = true;
				
				if (tempNbCols > maxNbCols)
					maxNbCols = tempNbCols;
	
				tempNbCols = 0;
				jump = false;
				oldColsString = colsString;
				colsString = "";
			}
	
			constraint = item.getConstraint();
	
			if (constraint != null)
			{
				colspan = constraint.getColSpan();
				remainder = constraint.isRemainder();
	
				if (remainder || (colspan == LyConstraint.DEFAULT_VALUE))
					jump = true;
	
				if (colspan < 0)
					colspan = 1;
			}
			else
			{
				colspan = 1;
				jump = true;
			}
			tempNbCols += colspan;
			tempHightNbCols += colspan;
			colsString += colspan;
	
			if (tempHightNbCols > hightNbCols)
				hightNbCols = tempHightNbCols;
	
			if (colspan > maxUnitNbCols)
				maxUnitNbCols = colspan;
	
			tempHightNbCols = 0;
	
			if (jump)
			{
				if (oldColsString.length() > 0 && !oldColsString.equals(colsString))
					hasColsOffset = true;
				
				if (tempNbCols > maxNbCols)
					maxNbCols = tempNbCols;
	
				tempNbCols = 0;
				jump = false;
				oldColsString = colsString;
				colsString = "";
			}
			
			if (item.getFieldItem() instanceof LyArea)
				areaColspans.set(colspan);
		}
		
		if (areaColspans.get(hightNbCols))
			hasFullWidthArea = true;			
	
		jump = true;
	
		int		cellWidth/*, cellLeftPadding, cellRightPadding */;
		boolean	jumpAfterItem;

		for (i = 0; i < nFields; i++)
		{
			hAlign = "right";
			vAlign = "top";
			item = _form.getFormField(i);
			internal = (LyJQFormField)item.getInternalView();
	
			cellWidth = getCellWidth(item);
			//cellLeftPadding = getCellLeftPadding(item);
			//cellRightPadding = getCellRightPadding(item);
			
			if (item.isHidden())
			{
				// Call writeHtml() method in order to update refresh info of
				// the html field
				///internal.writeHtml(pw);
			}
	
			if (jump)
			{
				jumpAfterItem = true;
				
				constraint = item.getConstraint();
	
				if (constraint != null)
				{
					colspan = constraint.getColSpan();
					remainder = constraint.isRemainder();
	
					if (!remainder && (colspan != LyConstraint.DEFAULT_VALUE))
						jumpAfterItem = false;
				}
				
				///pw.append("<tr id='" + parentTRId + "'");
	
				if (jumpAfterItem)
				{
					///internal.setParentTRId(parentTRId);
					
					///if (item.isHidden())
						///pw.append(" style='display:none;'");
				}
				
				///pw.append(">");
				currentCol = 0;
			}
	
			caption = item.getCaption();
			captionJump = false;
	
			if (caption != null)
			{
				///pw.append("\n");
				jump = false;
				constraint = caption.getConstraint();
				internalCaption = (LyJQCaption)caption.getInternalView();
	
				if (constraint != null)
				{
					rowspan = constraint.getRowSpan();
					colspan = constraint.getColSpan();
					remainder = constraint.isRemainder();
					colWeight = (int)constraint.getColWeight();
	
					if (remainder)
						jump = true;
	
					if (colspan < 0)
						colspan = 1;
	
					if (rowspan == 0)
						rowspan = 1;
					else if (rowspan < 0)
						rowspan = 1;
	
					hAlignCode = constraint.getHAlign();
	
					if (getHtmlHAlignment(hAlignCode) != null)
						hAlign = getHtmlHAlignment(hAlignCode);
	
					if (_viewManager.isPrintMode())
						hAlign = getHtmlHAlignment(LyConstraint.HAlign.LEFT);
	
					vAlignCode = constraint.getVAlign();
					if (getHtmlVAlignment(vAlignCode) != null)
						vAlign = getHtmlVAlignment(vAlignCode);
				}
				else
				{
					rowspan = 1;
					colspan = 1;
					colWeight = 0;
				}
	
				int diffColspan = colspan;
	
				currentCol += colspan;
				if (jump)
				{
					// workaround for tables in forms : the table does not take
					// the entire line if alone
					if (COMPOSITION_TABLE_WORKAROUND)
						// colspan++;
						captionJump = true; // workaround
					if ((maxNbCols - currentCol) > 0)
						diffColspan += (maxNbCols - currentCol);
				}
				if (vAlign.equals("center"))
					vAlign = "middle"; // Consistency with HTML DTD
	
				internalCaption.setColSpan(diffColspan);
				internalCaption.setRowSpan(rowspan);
				internalCaption.setHAlign(hAlign);
				internalCaption.setVAlign(vAlign);
				
				///pw.append("<td colspan='" + diffColspan + "' rowspan='" + rowspan + "'");
				///pw.append(" align='" + hAlign + "'");
				///pw.append(" valign='" + vAlign + "'");
	
				if (colWeight <= 0)
				{
					if (_form.getAlign() != LyForm.Align.LEFT)
						colWeight = 30; // %
					else
						colWeight = 0; // %
				}
	
				internalCaption.setColWeight(colWeight);
				///pw.append(" nowrap='true' width='" + colWeight + "%' class='" +
				///	deco.getClassName(item, DECO_CAPTION_PARAMETER, pw) + "'>");
	
				///internal.writeCaption(pw);
				///pw.append("</td>");
	
				///if (jump)
					///pw.append("</tr><tr>");
			}
	
			if (jump)
			{
				jump = false;
				currentCol = 0;
			}
			
			hAlign = null;
			vAlign = null;
			colWeight = 0;
	
			constraint = item.getConstraint();
	
			if (constraint != null)
			{
				rowspan = constraint.getRowSpan();
				colspan = constraint.getColSpan();
				remainder = constraint.isRemainder();
				colWeight = (int)constraint.getColWeight();
	
				if (remainder || (colspan == LyConstraint.DEFAULT_VALUE))
					jump = true;
	
				if (colspan == LyConstraint.DEFAULT_VALUE)
					colspan = maxNbCols - currentCol;
				
				if (colspan <= 0)
					colspan = 1;
	
				if (rowspan == 0)
					rowspan = 1;
				else if (rowspan < 0)
					rowspan = 1;
	
				if (colWeight == -2)
					colWeight = 100;
	
				if (colWeight < 0)
					colWeight = 0;
	
				hAlignCode = constraint.getHAlign();
				hAlign = getHtmlHAlignment(hAlignCode);
				vAlignCode = constraint.getVAlign();
				vAlign = getHtmlVAlignment(vAlignCode);
			}
			else
			{
				rowspan = 1;
				colWeight = 0;
				colspan = 1;
				jump = true;
			}
	
			///pw.append("\n");
	
			currentCol += colspan;
	
			int diffColspan = colspan;
	
			if (((currentCol + 1) < hightNbCols) && (i < (nFields - 1)))
				jump = false;
			else if (i == (nFields - 1))
				jump = true;
	
			// String	style = "";
			
			// Complete colspan if field is the last one on its line and only
			// if it is left-aligned. Otherwise field may not be placed correctly.
			if (jump
					&& ((hAlign == null) || "left".equals(hAlign)))
			{
				if ((maxNbCols - currentCol) > 0)
					diffColspan += (maxNbCols - currentCol);
			}
	
			if (COMPOSITION_TABLE_WORKAROUND && captionJump)
			{
				// workaround for tables in forms : the table does not take the
				// entire line if alone
				// diffColspan += 1;
				internal.setColSpan(diffColspan);
				internal.setRowSpan(rowspan);
				///pw.append("<td " + width + height + "colspan='" + diffColspan + "' rowspan='" + rowspan + "'");
				captionJump = false;
			}
			else
			{
				if (hasColsOffset && cellWidth > 0)
				{
					//style += "width:2px;";
					internal.setColSpan(diffColspan);
					internal.setRowSpan(rowspan);
					///pw.append("<td " + height + "colspan='" + diffColspan + "' rowspan='" + rowspan + "'");
				}
				else
				{
					internal.setColSpan(diffColspan);
					internal.setRowSpan(rowspan);
					///pw.append("<td " + width + height + "colspan='" + diffColspan + "' rowspan='" + rowspan + "'");
				}
			}
	
			if (hAlign != null)
			{
				internal.setHAlign(hAlign);
				internal.setVAlign(vAlign);
				///pw.append(" float='" + hAlign + "' align='" + hAlign + "'");
			}
			if (vAlign == null)
				vAlign = "middle";
			else if (vAlign.equals("center"))
				vAlign = "middle"; // Consistency with HTML DTD
			
			internal.setVAlign(vAlign);
			///pw.append(" valign='" + vAlign + "'");
	
			if (jump)
			{
				if (((maxUnitNbCols != hightNbCols) || (colWeight != 100))
						&& (cellWidth < 0))
				{
					// Giving a 100% width to a cell does not work with IE nor Chrome.
					// Compute well width by Javascript.
					if (hasColsOffset
							&& ((getWebViewManager().getBrowser() == LyWebViewManager.BROWSER_MSIE)
									|| (getWebViewManager().getBrowser() == LyWebViewManager.BROWSER_CHROME)))
					{
						///pw.append(" id='form_cell_" + i + "_" + _form.getIdCode() + "'");
						///endScripts.add("<script>ExtendCellToFullWidth('form_cell_" + i + "_" + _form.getIdCode() +
								///"', 'form_table_" + _form.getIdCode() + "')</script>");
					}
					else
					{
						internal.setColWeight(colWeight);
						///pw.append(" width='" + colWeight + "%'");
					}
				}
				else
				{
					// If last field is contained in a form and area with an horizontal fill,
					// give it a 100% width.
					LyConstraint	formConstraint = _form.getConstraint();
					LyConstraint	areaConstraint = _form.getParentArea().getConstraint();
					
					if ((constraint != null)
							&& ((constraint.getFill() == LyConstraint.Fill.HORIZONTAL)
									|| (constraint.getFill() == LyConstraint.Fill.BOTH))
							&& ((formConstraint == null)
									|| (formConstraint.getFill() == LyConstraint.Fill.HORIZONTAL)
									|| (formConstraint.getFill() == LyConstraint.Fill.BOTH))
							&& (areaConstraint != null)
							&& ((areaConstraint.getFill() == LyConstraint.Fill.HORIZONTAL)
									|| (areaConstraint.getFill() == LyConstraint.Fill.BOTH)))
						
						internal.setColWeight(100);
						///pw.append(" width='100%'");
				}
			}
			else
			{
				if (cellWidth < 0)
					internal.setColWeight(0);
					///pw.append(" width='0%'");
			}
		
			/* if ((cellLeftPadding >  0) || (cellRightPadding > 0))
			{
				if (cellLeftPadding >  0)
					style += "padding-left:" + cellLeftPadding + "px;";
				
				if (cellRightPadding >  0)
					style += "padding-right:" + cellRightPadding + "px;";
			} */
	
			///if (style.length() > 0)
				///pw.append(" style='" + style + "'");
			
			///pw.append(" class='" + deco.getClassName(item, DECO_ITEM_PARAMETER, pw) + "'>");
	
			///if (hasColsOffset && cellWidth > 0)
				///pw.append("<table cellpadding='0' cellspacing='0' border='0' width='" + cellWidth + "'><tr><td>");
			
			///boolean currentInnerView = pw.isInnerView();
	
			///pw.setInnerView(true);
			///internal.writeItem(pw);
			///pw.setInnerView(currentInnerView);
	
			///if (hasColsOffset && cellWidth > 0)
				///pw.append("</td></tr></table>");
	
			///pw.append("</td>");
	
			if (jump)
			{
				///pw.append("</tr>");
				if (hasColsOffset && hasFullWidthArea && (i < nFields - 1))
				{
					// If one of the field is a full width area, generate another HTML table around it
					// to avoid graphical issues with the other fields.
					LyFormField	nextItem = _form.getFormField(i + 1);
	
					if ((item.getFieldItem() instanceof LyArea
								&& item.getConstraint() != null
								&& item.getConstraint().getColSpan() == hightNbCols)
							|| (nextItem.getFieldItem() instanceof LyArea
									&& nextItem.getConstraint() != null
									&& nextItem.getConstraint().getColSpan() == hightNbCols))
					{
						///pw.append("</table>");
						///pw.append("<table ");
						///pw.append("width='100%' ");
						///pw.append("height='100%' ");
						///pw.append("border='" + DEBUG_BORDER + "' cellspacing='" + getCellSpacing() + "' cellpadding='" + getCellPadding() + "' valign='top'>\n");
					}
				}
			}
		}
	
		///if (openedTR)
			///pw.append("</tr>");
		
		///for (String s : endScripts)
			///pw.addEndScript(s, false);
	} // end method writeFields
	
	/**
	 * Returns the width of the HTML cell containing given field if fixed.
	 * @return the width of the cell or a negative value if there is no fixed width.
	 */
	public int getCellWidth(LyFormField formField)
	{
		return -1;
	}

	/**
	 * Returns the height of the HTML cell containing given field if fixed.
	 * @return the height of the cell or a negative value if there is no fixed height.
	 */
	public int getCellHeight(LyFormField formField)
	{
		return -1;
	}
	
	/**
	 * Returns the left padding of the HTML cell containing given field.
	 * @return the left padding of the cell or a negative value if there is no left padding.
	 */
	public int getCellLeftPadding(LyFormField formField)
	{
		return 0;
	}
	
	/**
	 * Returns the right padding of the HTML cell containing given field.
	 * @return the right padding of the cell or a negative value if there is no right padding.
	 */
	public int getCellRightPadding(LyFormField formField)
	{
		return 0;
	}
	
	/**
	 * Gets the html horizontal alignment string corresponding to the LyConstraint alignment.
	 *
	 * @param  alignCode the LyConstraint halign code.
	 * @return the html horizontal alignment string.
	 */
	protected static String getHtmlHAlignment(LyConstraint.HAlign alignCode)
	{
		if (alignCode == LyConstraint.HAlign.LEFT)
			return "left";
		else if (alignCode == LyConstraint.HAlign.RIGHT)
			return "right";
		else if (alignCode == LyConstraint.HAlign.CENTER)
			return "center";
		else
			return null;
	}

	/**
	 * Gets the html vertical alignment string corresponding to the LyConstraint alignment.
	 *
	 * @param  alignCode the LyConstraint valign code.
	 * @return the html vertical alignment string.
	 */
	protected static String getHtmlVAlignment(LyConstraint.VAlign alignCode)
	{
		if (alignCode == LyConstraint.VAlign.TOP)
			return "top";
		else if (alignCode == LyConstraint.VAlign.MIDDLE)
			return "middle";
		else if (alignCode == LyConstraint.VAlign.BOTTOM)
			return "bottom";
		else
			return null;
	}
	
	public void writeHtml(LyWebPageWriter pw)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();
		
		params.put("lyForm", _form);
		
		String	result = templateManager.mergeTemplate("form/lyFormTemplate.vm", params);
		
		pw.append(result);
	}

}