/* 
 * Lyria-W4 - afficheur_web2
 * LyWeb2FormField.java
 * Copyright (c) 2000-2009, Lyria-W4
 * All rights reserved.
 */
package leon.view.web.jquery.component.form.field;

import java.util.*;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.setinterface.*;
import leon.view.web.*;
import leon.view.web.jquery.*;
import leon.view.web.jquery.common.*;
import leon.view.web.jquery.component.*;
import leon.view.web.jquery.controller.*;

/**
 * Type : leon.view.web.web2.component.form.field.LyWeb2FormField The peer component of LyFormField for the web2 viewer.
 * @version $Revision: 147 $
 * @author ychaari$
 */
public class LyJQFormField extends LyJQInternalView	implements
													LyGraphicalFormField,
													LyResetContentInterface,
													LySetFocusInterface
{

	protected static String	LEFT_ALIGN					= "left";

	protected static String	RIGHT_ALIGN					= "right";

	protected static String	CENTER_ALIGN				= "center";

	protected static String	TOP_ALIGN					= "top";

	protected static String	MIDDLE_ALIGN				= "middle";

	protected static String	BOTTOM_ALIGN				= "bottom";

	protected static String	LABEL_STATE_PREFIX			= "LY_LABEL_STATE_";

	protected static String	HIDE_PARAM					= "hide";

	protected static String	STATE_PARAM					= "state";

	protected static String	FORM_FIELD_PARAM			= "formField";

	protected static String	JQ_FORM_FIELD_PARAM			= "web2FormField";

	protected static String	SHOW_HIDE_TEMPLATE			= "form/field/actions/toggle.vm";

	protected static String	SET_STATE_TEMPLATE			= "form/field/actions/setFieldState.vm";

	protected static String	FORM_FIELD_TEMPLATE_CAPTION	= "form/field/lyFormFieldCaptionTemplate.vm";

	protected static String	FORM_FIELD_TEMPLATE_CONTENT	= "form/field/lyFormFieldContentTemplate.vm";
	
	protected static String	JQ_FORM_FIELD_ID_PARAM		= "formFieldId";
	
	protected static String	FORM_FIELD_MESSAGE_PARAM	= "message";
	
	protected static String	FORM_FIELD_STATE_PARAM		= "state";
	
	protected static String	SET_MESSAGE_TEMPLATE		= "ajax/form/field/setMessage.vm";
	
	 
	/** The form field generic component. */
	protected LyFormField	_field;

	private int				colSpan;

	private int				rowSpan;

	private int				colWeight;

	private String			vAlign;

	private String			hAlign;

	/**
	 * Constructor
	 * @param field
	 */
	public LyJQFormField(LyFormField field)
	{
		super(field);
		_field = field;
	}

	public boolean isMakeTable()
	{
		LyComponent parent = _field.getParent();
		return (parent instanceof LyForm);
	}

	/**
	 * Gets the warning state short
	 * @param field
	 * @return
	 */
	public short getWarningState(LyFormField field)
	{
		return LyFormField.STATE_WARNING;
	}

	/**
	 * Gets the warning color
	 * @param field
	 * @return
	 */
	public String getWarningColor(LyFormField field)
	{
		return getHexColor(LABEL_STATE_PREFIX + field.getState());
	}

	/**
	 * Gets the html horizontal alignment string corresponding to the LyConstraint alignment.
	 * @param alignCode the LyConstraint halign code.
	 * @return the html horizontal alignment string.
	 */
	
	public   String getHtmlHAlignment(LyConstraint.HAlign alignCode)
	{
		
		if (alignCode == LyConstraint.HAlign.LEFT)
		{	
			if(_viewManager.isRtlMode())
				return RIGHT_ALIGN;
			else
				return LEFT_ALIGN;
		}
		else if (alignCode == LyConstraint.HAlign.RIGHT)
		{
			if(_viewManager.isRtlMode())
				return LEFT_ALIGN;
			else
				return RIGHT_ALIGN;
		}
		else if (alignCode == LyConstraint.HAlign.CENTER)
			return CENTER_ALIGN;
		else
			return null;
	}

	/**
	 * Gets the html vertical alignment string corresponding to the LyConstraint alignment.
	 * @param alignCode the LyConstraint valign code.
	 * @return the html vertical alignment string.
	 */
	public static String getHtmlVAlignment(LyConstraint.VAlign alignCode)
	{
		if (alignCode == LyConstraint.VAlign.TOP)
			return TOP_ALIGN;
		else if (alignCode == LyConstraint.VAlign.MIDDLE)
			return MIDDLE_ALIGN;
		else if (alignCode == LyConstraint.VAlign.BOTTOM)
			return BOTTOM_ALIGN;
		else
			return null;
	}

	/**
	 * Get field's weight
	 * @param constraint
	 * @return
	 */
	public int getColWeight(LyConstraint constraint)
	{
		return (int) constraint.getColWeight();
	}

	/**
	 * Check if field is a LyText instance
	 * @param item
	 * @return
	 */
	public boolean isLyTextInstance(LyComponent item)
	{
		return (item instanceof LyText);
	}

	public boolean isLyDateInstance(LyComponent item)
	{
		return (item instanceof LyDate);
	}

	public boolean isLyHrefInstance(LyComponent item)
	{
		return (item instanceof LyHRef);
	}

	public boolean isLyHrefsInstance(LyComponent item)
	{
		return (item instanceof LyHRefs);
	}

	public boolean isPanelInstance(LyComponent item)
	{
		return (item instanceof LyPanel);
	}

	public boolean isLyChoiceInstance(LyComponent item)
	{
		return (item instanceof LyChoice);
	}

	public boolean isLyBooleanInstance(LyComponent item)
	{
		return (item instanceof LyBoolean);
	}

	public boolean isLyNumberInstance(LyComponent item)
	{
		return (item instanceof LyNumber);
	}

	public boolean isLyGaugeInstance(LyComponent item)
	{
		return (item instanceof LyGauge);
	}

	public boolean isLyListInstance(LyComponent item)
	{
		return (item instanceof LyList);
	}

	public boolean isLyFileInstance(LyComponent item)
	{
		return (item instanceof LyFile);
	}

	/**
	 * Indicates if the field item is a icon component.
	 * @return true if it is icon, false otherwise
	 */
	public boolean isIconInstance(LyFieldItem fieldItem)
	{
		return (fieldItem instanceof LyIcon);
	}

	/**
	 * Check if the field is a LyRelation instance.
	 * @param item : represent the field to be checked.
	 * @return test result {true | false}.
	 */
	public boolean isLyRelationInstance(LyComponent item)
	{
		return (item instanceof LyRelation);
	}

	/**
	 * Check if the field is a LyDictionary instance.
	 * @param item : represent the field to be checked.
	 * @return test result {true | false}.
	 */
	public boolean isLyDictionaryInstance(LyComponent item)
	{
		return (item instanceof LyDictionary);
	}

	/**
	 * Check if the field is a LyLink instance.
	 * @param item : represent the field to be checked.
	 * @return test result {true | false}.
	 */
	public boolean isLyLinkInstance(LyComponent item)
	{
		return (item instanceof LyLink);
	}

	/**
	 * Check if field is a LyText instance
	 * @param item
	 * @return
	 */
	public boolean isLyLabelInstance(LyComponent item)
	{
		return (item instanceof LyLabel);
	}

	/**
	 * Check if field is a lyArea
	 * @param item
	 * @return
	 */
	public boolean isLyAreaInstance(LyComponent item)
	{
		return (item instanceof LyArea);
	}

	/**
	 * Check if field is a lyTable
	 * @param item
	 * @return
	 */
	public boolean isLyTableInstance(LyComponent item)
	{
		return (item instanceof LyTable);
	}

	public void setChecked(boolean checked)
	{
		reloadComponent();
	}

	public void setEditable(boolean editable)
	{
		reloadComponent();
	}

	public void setEnabled(boolean enabled)
	{
		reloadComponent();
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalFormField#setHidden(boolean)
	 */
	public void setHidden(boolean hidden)
	{
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(HIDE_PARAM, hidden);
		params.put(JQ_FORM_FIELD_PARAM, this);

		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		String hideScript = templateManager.mergeTemplate(SHOW_HIDE_TEMPLATE, params);

		_viewManager.addToUpdateScript(hideScript);
		hideScript = null;
		params = null;
	}

	public void setOptional(boolean optional)
	{
		reloadComponent();
	}

	public void setState(short state)
	{
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(STATE_PARAM, state);
		params.put(JQ_FORM_FIELD_PARAM, this);

		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		String stateScript = templateManager.mergeTemplate(SET_STATE_TEMPLATE, params);

		_viewManager.addToUpdateScript(stateScript);
		stateScript = null;
		params = null;
	}

	public void setToolBar(LyToolBar toolBar)
	{
		reloadComponent();
	}

	public void resetContent()
	{
		reloadComponent();
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LyResetContentInterface#resetContent(boolean)
	 */
	public void resetContent(boolean full)
	{
		reloadComponent();
	}

	public void setFocus()
	{
		// Nop
	}

	public void writeHtml(LyWebPageWriter pw)
	{
		// Nop
	}

	public void reloadComponent()
	{
		_viewManager.addToReloadComponents(_field);
	}

	@Override
	public String getReloadScript()
	{
		StringBuffer	result = new StringBuffer();
		
		// update form field caption
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(FORM_FIELD_PARAM, _field);
		params.put("environment", getEnvironment());

		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		String caption = templateManager.mergeTemplate(FORM_FIELD_TEMPLATE_CAPTION, params);
		caption = TemplateUtils.getFormattedJs(caption);

		params = new HashMap<String, Object>();
		params.put("content", caption);
		params.put("componentId", getId() + "_caption");
		caption = null;
		result.append(templateManager.mergeTemplate(LyJQConstants.REPLACE_SCRIPT_TEMPLATE, params));
		params = null;

		// update form field content
		params = new HashMap<String, Object>();
		params.put(FORM_FIELD_PARAM, _field);

		String fieldContent = templateManager.mergeTemplate(FORM_FIELD_TEMPLATE_CONTENT, params);
		fieldContent = TemplateUtils.getFormattedJs(fieldContent);

		params = new HashMap<String, Object>();
		params.put("content", fieldContent);
		params.put("componentId", getId() + "_content");
		fieldContent = null;
		result.append(templateManager.mergeTemplate(LyJQConstants.REPLACE_SCRIPT_TEMPLATE, params));
		params = null;

		return result.toString();
	}

	/**
	 * @return the colSpan
	 */
	public int getColSpan()
	{
		return colSpan;
	}

	/**
	 * @param colSpan the colSpan to set
	 */
	public void setColSpan(int colSpan)
	{
		this.colSpan = colSpan;
	}

	/**
	 * @return the rowSpan
	 */
	public int getRowSpan()
	{
		return rowSpan;
	}

	/**
	 * @param rowSpan the rowSpan to set
	 */
	public void setRowSpan(int rowSpan)
	{
		this.rowSpan = rowSpan;
	}

	/**
	 * @return the colWeight
	 */
	public int getColWeight()
	{
		return colWeight;
	}

	/**
	 * @param colWeight the colWeight to set
	 */
	public void setColWeight(int colWeight)
	{
		this.colWeight = colWeight;
	}

	/**
	 * @return the vAlign
	 */
	public String getVAlign()
	{
		return vAlign;
	}

	/**
	 * @param vAlign the vAlign to set
	 */
	public void setVAlign(String vAlign)
	{
		this.vAlign = vAlign;
	}

	/**
	 * @return the hAlign
	 */
	public String getHAlign()
	{
		if (LEFT_ALIGN.equals(hAlign))
		{
			if(_viewManager.isRtlMode())
				hAlign = RIGHT_ALIGN;
			
		}
		else if (RIGHT_ALIGN.equals(hAlign))
		{
			if(_viewManager.isRtlMode())
				hAlign = LEFT_ALIGN;	
		}

		return hAlign;
	}

	/**
	 * @param hAlign the hAlign to set
	 */
	public void setHAlign(String hAlign)
	{
		this.hAlign = hAlign;
	}

	public void setMessage(String message)
	{
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(JQ_FORM_FIELD_ID_PARAM, getId());
		params.put(FORM_FIELD_MESSAGE_PARAM, message);
		params.put(FORM_FIELD_STATE_PARAM, _field.getState());
		
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		String stateScript = templateManager.mergeTemplate(SET_MESSAGE_TEMPLATE, params);

		_viewManager.addToUpdateScript(stateScript);
		stateScript = null;
		params = null;
	}
}