package leon.view.web.jquery.component.form.field.choice;

import java.util.HashMap;
import java.util.Map;

import leon.view.LyChoice;
import leon.view.LyComponent;
import leon.view.LyForm;
import leon.view.LyFormField;
import leon.view.LyOption;
import leon.view.graphic.LyGraphicalOption;
import leon.view.web.LyWebPageWriter;
import leon.view.web.jquery.component.LyJQInternalView;
import leon.view.web.jquery.controller.LyTemplatesUtilsBean;

/**
 * LyOption JQuery internal view
 * @author slachiheb
 */
public class LyJQOption extends LyJQInternalView implements
												LyGraphicalOption
{
	/** The option generic component. */
	protected LyOption			_option;

	/** The identifier of the parent form. */
	protected String			_formId;

	private static final String	OPTION_SET_ENABLED_TEMPLATE	= "form/field/choice/actions/setEnabled.vm";

	private static final String	OPTION_SET_CHECKED			= "form/field/choice/actions/setChecked.vm";

	public LyJQOption(LyOption option)
	{
		super(option);
		_option = option;
		_formId = null;
	}

	/**
	 * check if option is enabled
	 */
	public boolean isEnabled(LyOption lyOption)
	{
		LyChoice parent = (LyChoice) _option.getParent();
		LyFormField grandParent = (LyFormField) parent.getParent();
		LyForm form;

		// Look for an ancestor of type LyForm
		LyComponent ancestor = parent;

		do
			ancestor = ancestor.getParent();
		while ((ancestor != null) && !(ancestor instanceof LyForm));
		form = (LyForm) ancestor;

		return (_option.isEnabled() && ((form == null) || form.isEditable())
				&& grandParent.isEditable() && grandParent.isEnabled());
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.web.jquery.component.LyJQInternalView#getReloadScript()
	 */
	@Override
	public String getReloadScript()
	{
		// Nop
		return null;
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalOption#setChecked(boolean)
	 */
	public void setChecked(boolean checked)
	{
		LyChoice parent = (LyChoice)_option.getParent();
		
		if (!checked && !parent.isMultiple())
		{
			// If at least one other option is selected in a non-multiple choice,
			// do not generate script when option is unchecked.
			int			i, n = parent.getOptionCount();
			LyOption	option;
			boolean		oneChecked = false;
			
			for (i = 0; i < n; i++)
			{
				option = parent.getOption(i);
				
				if (option != _option)
					oneChecked = oneChecked || option.isChecked();
			}
			
			if (oneChecked)
				return;
		}
		
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("option", _option);
		params.put("checked", checked);

		String editableCell = templateManager.mergeTemplate(OPTION_SET_CHECKED, params);

		_viewManager.addToUpdateScript(editableCell);
		params = null;
		editableCell = null;
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalOption#setEnabled(boolean)
	 */
	public void setEnabled(boolean enabled)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("option", _option);
		params.put("enabled", enabled);

		String editableCell = templateManager.mergeTemplate(OPTION_SET_ENABLED_TEMPLATE, params);

		_viewManager.addToUpdateScript(editableCell);
		params = null;
		editableCell = null;
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
		// Nop
	}
}