package leon.view.web.jquery.component.form.field.date;

import java.text.*;
import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.jquery.common.*;
import leon.view.web.jquery.component.fieldItem.*;
import leon.view.web.jquery.controller.*;

/**
 * LyDate internal view implementation class
 * @author abkristou
 */
public class LyJQDate extends LyJQFieldItem	implements
											LyGraphicalDate
{
	/** The date generic component. */
	protected LyDate	_date;

	
	public LyJQDate(LyDate date)
	{
		super(date);
		_date = date;
	}

	/**
	 * get date labels
	 * @return date labels
	 */
	public String[] getDateLabels()
	{
		String		label = getHtmlString(_date.getLabel());
		String[]	labels = LyString.stringToList(label, ' ');
		String		dateLabel = "";
		String		timeLabel = "";
		String		amPmLabel = "";
		LyDate.Type	type = _date.getType();
		boolean		addDate = (type == LyDate.Type.DATE_TIME) || (type == LyDate.Type.DATE_HOUR_MIN)
				|| (type == LyDate.Type.DATE);
		boolean		addTime = (type == LyDate.Type.DATE_TIME) || (type == LyDate.Type.TIME)
				|| (type == LyDate.Type.DATE_HOUR_MIN) || (type == LyDate.Type.HOUR_MIN);
		
		// Both date and time
		if (addDate && addTime)
		{
			dateLabel = labels[0];
			
			if ((labels.length > 2) && (labels[1] != null) && (labels[1].length() > 0))
			{
				timeLabel = labels[1];
				amPmLabel = labels[2];
			}
			else if ((labels.length > 1) && (labels[1] != null) && (labels[1].length() > 0))
				timeLabel = labels[1];
			else if ((dateLabel != null) && (dateLabel.length() > 0))
			{
				// A date was given without any time : set time with current time
				Date		currentDate = new Date();
				int			timeType = (type == LyDate.Type.DATE_HOUR_MIN) || (type == LyDate.Type.HOUR_MIN) ?
								LyEnvironment.TYPE_HOUR_MIN : LyEnvironment.TYPE_TIME;
				String		format = getEnvironment().getStringDateFormat(timeType);
				String		formattedDate = LyDateFormatter.format(currentDate, format, getEnvironment());
				String[]	timeLabels = LyString.stringToList(formattedDate, ' ');

				if (timeLabels.length == 2)
				{
					timeLabel = timeLabels[0];
					amPmLabel = timeLabels[1];
				}
				else if (timeLabels.length == 1)
					timeLabel = timeLabels[0];
			}
		}
		// Only date
		else if (addDate)
			dateLabel = labels[0];
		// Only time
		else if (addTime)
		{
			if (labels.length > 2)
			{
				timeLabel = labels[1];
				amPmLabel = labels[2];
			}
			else if (labels.length > 1)
				timeLabel = labels[1];
			else
				timeLabel = labels[0];
		}

		String initialValue = dateLabel;

		if ((dateLabel != null) && (dateLabel.length() != 0)
				&& (timeLabel.length() != 0))
			initialValue += " ";
		
		initialValue += timeLabel;
		
		if ((timeLabel.length() != 0) && (amPmLabel.length() != 0))
			initialValue += " " + amPmLabel;

		return new String[] { initialValue, dateLabel, timeLabel, amPmLabel};
	} // end method getDateLabels

	/**
	 * Indicates if date input must be displayed.
	 */
	public boolean isDate()
	{
		LyDate.Type type = _date.getType();

		return (type == LyDate.Type.DATE_TIME) || (type == LyDate.Type.DATE_HOUR_MIN)
				|| (type == LyDate.Type.DATE);
	}

	/**
	 * Indicates if time input must be displayed.
	 */
	public boolean isTime()
	{
		LyDate.Type type = _date.getType();

		return (type == LyDate.Type.DATE_TIME) || (type == LyDate.Type.TIME)
				|| (type == LyDate.Type.DATE_HOUR_MIN) || (type == LyDate.Type.HOUR_MIN);
	}
	
	/**
	 * Indicates if seconds must be displayed.
	 */
	public boolean isSeconde()
	{
		LyDate.Type type = _date.getType();

		return (type == LyDate.Type.DATE_TIME) || (type == LyDate.Type.TIME);
	}
	
	/**
	 * Indicates if AM/PM selector must be displayed.
	 */
	public boolean isAmPm()
	{
		boolean	isTimeHourMin = (_date.getType()== LyDate.Type.DATE_HOUR_MIN) || (_date.getType()== LyDate.Type.HOUR_MIN);
		String	timeStringFormat = getEnvironment().getStringDateFormat(isTimeHourMin ? LyEnvironment.TYPE_HOUR_MIN : LyEnvironment.TYPE_TIME);
		boolean	addAmPm = false;

		if (timeStringFormat.endsWith(" a"))
			addAmPm = true;
		
		return addAmPm;
	}
	
	/**
	 * Indicates whether AM is selected in an AM/PM date label.
	 */
	public boolean isAmLabel(String label)
	{
		String				amLabel = getAmPmLabels()[0];
		
		return (amLabel.equals(label));
	}
	
	/**
	 * Indicates whether PM is selected in an AM/PM date label.
	 */
	public boolean isPmLabel(String label)
	{
		String				pmLabel = getAmPmLabels()[1];
		
		return (pmLabel.equals(label));
	}
	
	/**
	 * Returns the labels used for AM and PM depending on the locale.
	 */
	public String[] getAmPmLabels()
	{
		Locale				locale = _environment.getDateLocale();
		DateFormatSymbols	dfs = new DateFormatSymbols(locale);
		
		return dfs.getAmPmStrings();
	}
	
	public boolean isDateInstance()
	{
		return true;
	}

	public void setLabel(String label)
	{
		String[]				labels = getDateLabels();
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params;
		String					script;
		
		if (isDate())
		{
			params = new HashMap<String, Object>();
			params.put("componentId", "date_" + getId());
			params.put("value", TemplateUtils.getFormattedJs(labels[1]));
			script = templateManager.mergeTemplate("ajax/form/field/setLabel.vm", params);
			_viewManager.addToUpdateScript(script);
		}
		
		if (isTime())
		{
			params = new HashMap<String, Object>();
			params.put("componentId", "time_" + getId());
			params.put("value", TemplateUtils.getFormattedJs(labels[2]));
			script = templateManager.mergeTemplate("ajax/form/field/setLabel.vm", params);
			_viewManager.addToUpdateScript(script);
		}
		
		if (isAmPm())
		{
			params = new HashMap<String, Object>();
			params.put("componentId", "time_ampm_" + getId());
			params.put("content", getAmPmContent());
			script = templateManager.mergeTemplate("ajax/replaceContent.vm", params);
			_viewManager.addToUpdateScript(script);
		}

		params = null;
	}
	
	/**
	 * Returns the content of the AM/PM selector.
	 */
	public String getAmPmContent()
	{
		String			id = getId();
		String[]		label = getDateLabels();
		String[]		amPmLabels = getAmPmLabels();
		boolean			isAm = isAmLabel(label[3]);
		boolean			isPm = isPmLabel(label[3]);
		StringBuffer	buffer = new StringBuffer();
		
		buffer.append("<option id='time_noampm_").append(id).append("' ").append(!isAm && !isPm ? "selected='true'" : "").append("></option>");
		buffer.append("<option id='time_am_").append(id).append("' ").append(isAm ? "selected='true'" : "").append(">").append(amPmLabels[0]).append("</option>");
		buffer.append("<option id='time_pm_").append(id).append("' ").append(isPm ? "selected='true'" : "").append(">").append(amPmLabels[1]).append("</option>");
		
		return buffer.toString();
	}
}
