package leon.view.web.jquery.component.form.field.dictionary;

import leon.view.LyDictionary;
import leon.view.LyEntry;
import leon.view.LyForm;
import leon.view.LyFormField;
import leon.view.getinterface.*;
import leon.view.graphic.LyGraphicalDictionary;
import leon.view.web.LyWebParameterValues;
import leon.view.web.jquery.common.*;
import leon.view.web.jquery.component.fieldItem.LyJQFieldItem;
import leon.view.web.jquery.controller.*;

import java.util.*;


/**
 * LyDictionary internal view
 * @author oussema
 */
public class LyJQDictionary extends LyJQFieldItem implements LyGraphicalDictionary, LyGetValueInterface
{
	protected static final String	COMPONENT_ID_PARAM			= "componentId";

	protected static final String	CONTENT_PARAM				= "content";

	protected static final String	DICTIONARY					= "dictionary";

	protected static final String	DICTIONARY_TEMPLATE_PATH	= "form/field/dictionary/lyDictionaryTemplate.vm";

    /** The dictionary generic component. */
    protected LyDictionary _dictionary;

    protected StringBuffer value;

    protected StringBuffer displayedValue;

    public LyJQDictionary(LyDictionary dictionary)
    {
        super(dictionary);
        _dictionary = dictionary;
    }

    /**
     * @see leon.view.graphic.LyGraphicalComponent#free()
     */
    @Override public void free()
    {
    }

    /**
     * @see leon.view.graphic.LyGraphicalDictionary#setEntry(leon.view.LyEntry)
     */
    public void setEntry(LyEntry entry)
    {
    	_viewManager.addToReloadComponents(_dictionary);
    }

    /**
     * Indicates if the dictionary is editable.
     * @return true if it is editable, false otherwise
     */
    public boolean isEditable()
    {
        LyFormField parent = (LyFormField) _dictionary.getParent();

        if (!parent.isEditable() || !parent.isEnabled())
            return false;

        LyForm form = ((parent.getParent() instanceof LyForm) ? (LyForm) parent.getParent() : null);
        return ((form == null) || form.isEditable());
    }

    /**
     * @return
     */
    public StringBuffer getDisplayedValue()
    {
        char          visibleSep = getEnvironment().getString().VISIBLE_SEP2;
        List<LyEntry> subEntries = new ArrayList<LyEntry>();
        StringBuffer  displayedValue = new StringBuffer();

        // Populates the subEntries list with root sub-entries
        getSubEntries(subEntries, _dictionary.getEntry());

        // For each checked sub-entry add the values and displayed values of the checked sub-entries
        for (LyEntry subEntry : subEntries)
        {

            if (subEntry.isChecked())
            {

                if (displayedValue.length() > 0)
                    displayedValue.append(visibleSep);

                displayedValue.append(translate(subEntry.getLabel()));
            }
        }

        return displayedValue;
    }

    public String getValue()
    {
        char          sep = getEnvironment().getString().SEP;
        List<LyEntry> subEntries = new ArrayList<LyEntry>();
        StringBuffer  value = new StringBuffer();

        // Populates the subEntries list with root sub-entries
        getSubEntries(subEntries, _dictionary.getEntry());

        // For each checked sub-entry add the values and displayed values of the checked sub-entries
        for (LyEntry subEntry : subEntries)
        {
            if (subEntry.isChecked())
            {
                if (value.length() > 0)
                    value.append(sep);

                value.append(subEntry.getId());
            }
        }

        return value.toString();
    }

    /**
     * Populates the given list with the list of sub-entries of given entry. The entry itself is added to the list.
     * @param list the list to be populated
     * @param entry the entry to be parsed
     */
    protected void getSubEntries(List<LyEntry> list, LyEntry entry)
    {
        int     i, n = entry.getEntryCount();
        LyEntry subEntry;

        if (list != null)
        {
            list.add(entry);

            for (i = 0; i < n; i++)
            {
                subEntry = entry.getEntry(i);
                getSubEntries(list, subEntry);
            }
        }
    }

    /**
     * Get the visible separator char
     * @return separator
     */
    public char getDisplayedSeparator()
    {
        return getEnvironment().getString().VISIBLE_SEP2;
    }

    /**
     * Get the separator char
     * @return separator
     */
    public String getSeparator()
    {
        return LyWebParameterValues.HTML_SEP;
    }

    public boolean isDictionary()
    {
        return true;
    }

	/**
	 * (non-Javadoc)
	 * @see leon.view.web.jquery.component.fieldItem.LyJQFieldItem#getReloadScript()
	 */
	@Override
	public String getReloadScript()
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		String					removeDictionaryScript = "";
		String					newDictionaryScript = "";
		Map<String, Object>		params = new HashMap<String, Object>();	
		
		params = new HashMap<String, Object>();
		params.put(COMPONENT_ID_PARAM, getId());
		removeDictionaryScript = templateManager.mergeTemplate("ajax/removeDiv.vm", params);

		params = new HashMap<String, Object>();
		params.put(DICTIONARY, _dictionary);
		newDictionaryScript = templateManager.mergeTemplate(DICTIONARY_TEMPLATE_PATH, params);
		newDictionaryScript = TemplateUtils.getFormattedJs(newDictionaryScript);
		params = new HashMap<String, Object>();
		params.put(COMPONENT_ID_PARAM, getId() + "_span");
		params.put(CONTENT_PARAM, newDictionaryScript);
		newDictionaryScript = templateManager.mergeTemplate("ajax/replaceWith.vm", params);
		params = null;

		return removeDictionaryScript + newDictionaryScript;
	}
}
