package leon.view.web.jquery.component.form.field.file;

import java.awt.*;
import java.io.*;
import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.jquery.*;
import leon.view.web.jquery.component.fieldItem.*;
import leon.view.web.jquery.component.form.field.*;
import leon.view.web.jquery.controller.*;

/**
 * LyFile JQuery internal view
 * @author bas
 */
public class LyJQFile extends LyJQFieldItem	implements
											LyGraphicalFile
{
	/** Indicates whether the name or full path of previously selected file must be displayed. */
	public static boolean		DISPLAY_PREVIOUS_FILE_NAME = true;

	protected LyFile			_file;

	/**
	 * constructor
	 * @param file
	 */
	public LyJQFile(LyFile file)
	{
		super(file);
		_file = file;
	}

	/**
	 * @return is directory
	 */
	public boolean isDirectory()
	{
		return (_file.getType() == LyFile.Type.DIRECTORY);
	}

	/**
	 * check if file requires checkbox
	 */
	public boolean generateCheckbox()
	{
		return ((_formField == null) || _formField.isOptional());
	}

	/**
	 * @return previous value
	 */
	public String getPrevious()
	{
		String previousFileLabel = _file.getLabel();

		if ((previousFileLabel != null) && DISPLAY_PREVIOUS_FILE_NAME)
		{
			String	sep = File.separator;
			int		pos = previousFileLabel.lastIndexOf(sep);

			if (pos >= 0)
				previousFileLabel = previousFileLabel.substring(pos + sep.length());
		}

	    if (previousFileLabel == null)
	    	previousFileLabel = "";
		
		if (previousFileLabel != null && previousFileLabel.length() != 0)
		{
            if (previousFileLabel.startsWith("__"))
            {
            	int n1 = -1, n2 = -1;
            	n1 = previousFileLabel.indexOf("__");

            	if (n1 >= 0)
            		n2 = previousFileLabel.indexOf("__", n1+1);
            	
            	if ((n1 >= 0) && (n2 >= 0))
            		previousFileLabel = previousFileLabel.substring(0, n1) + previousFileLabel.substring(n2 + 2);
            }
		}

		return previousFileLabel;
	}
	
	/**
	 * Return the label used to indicate that no file was selected.
	 */
	public String getNoFileLabel()
	{
		return getString("LY_NO_FILE");
	}
	
	/**
	 * Return the label used to indicate that a file must be selected.
	 */
	public String getSelectLabel()
	{
		return getString("LY_SELECT");
	}
	
	/**
	 * Returns the width of the input component.
	 */
	@SuppressWarnings("deprecation")
	public int getWidth()
	{
        int     width = _file.getNCols();

		// If a constraint is set on the file, use the nCols value to compute file width in pixels.
		if ((_file.getParent() instanceof LyFormField)
				&& (((LyFormField)(_file.getParent())).getConstraint() != null))
		{
			LyConstraint	constraint = ((LyFormField)(_file.getParent())).getConstraint();

			if ((constraint != null) && (constraint.getNCols() > 0))
				width = constraint.getNCols();
		}

		if (width > 0)
			return width;
		else
		{
			int		nCols = getIntResource("LY_FILE_SELECTOR_WIDTH");

			if (nCols < 0)
				nCols = 40;

			Font			font = (Font)getEnvironment().getFont(_file.getFont());
			Toolkit			toolkit = Toolkit.getDefaultToolkit();
			FontMetrics		fm = toolkit.getFontMetrics(font);
			int				wCharWidth = fm.charWidth('W');
			int				averageCharWidth = fm.stringWidth("abcdefghijklmnopqrstuvwxyz") / 26;
			int sizeX = 0;

			if (nCols < 10)
	            // With small size, the text field is sometimes not wide enough...
				sizeX = wCharWidth * (nCols + 1);
	        else
	            // Note : the 1.5 factor allows to have almost the same width as AWT...
	        	sizeX = (averageCharWidth * nCols * 3) / 2;

			return sizeX + 4;
		}
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.web.jquery.component.fieldItem.LyJQFieldItem#setLabel(java.lang.String)
	 */
	@Override
	public void setLabel(String label)
	{
		LyComponent parent = _file.getParent();

        if ((parent instanceof LyFormField) && useCustomizedFilecomponent())
        {
            LyFormField       formField = (LyFormField)parent;
            LyJQFormField webField = (LyJQFormField)formField.getInternalView();

            webField.reloadComponent();
        }
        else
        {
	
			LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
			Map<String, Object> params = new HashMap<String, Object>();
	
			params.put("file", _file);
			params.put("label", label);
	
			String containt = templateManager.mergeTemplate("form/field/file/actions/setLabel.vm", params);
	
			_viewManager.addToUpdateScript(containt);
			params = null;
			containt = null;
        }
	}

	/**
	 * Indicates whether file component must be customized in web browser.
	 * Customization is not used with IE since there are security issues with IE file selector:
	 * use standard file selector
	 * 
	 * @return true if a customized file selector must be used, false otherwise
	 */
	public boolean useCustomizedFilecomponent()
	{
		return ((LyJQViewManager)getWebViewManager()).useCustomizedFilecomponent();
	}
	
	public String getAttachmentDirectory()
	{
		String	dir = _file.getAttachmentDirectory();
		
		dir = LyString.replace(dir, "\\", "\\\\");
		
		return dir;
	}
	
	public String getImagePath()
	{
		String	directory = _file.getDirectory();

		if (directory != null)
		{
			directory = LyString.replace(directory, "$LY_APP_DOC_DIR$", "$LY_APP_DOC$");
			directory = expand(directory);
			directory = LyString.replace(directory, "\\", "/");
		}
		
		String	label = _file.getLabel();
		String	path = (directory != null ? directory : "") + (label != null ? label : "");

		if (label == null)
			path = getImageUrl("LY_DEFAULT_FILE_IMAGE");

		if (path.startsWith("file://"))
		{
			path = path.substring(7);
			path = getImageUrl(path);
		}
		
		if (path.startsWith("http") || path.startsWith("ftp"))
			return path;

		if ((label != null) && (_file.getAttachmentHandler() != null))
		{
			String docURL = getEnvironment().getEnv("LY_APP_DOC");
			String docDir = getEnvironment().getEnv("LY_APP_DOC_DIR");
			String docDirSlashes = LyString.replace(docDir, "\\", "/");
			String labelSlashes = LyString.replace(label, "\\", "/");

			if (labelSlashes.startsWith(docDirSlashes))
				path = docURL + "/" + labelSlashes.substring(docDirSlashes.length());
			else if (!label.startsWith(docURL))
			{
				String attachmentDir = _file.getAttachmentDirectory();

				if (attachmentDir == null)
					attachmentDir = "files";
				
				if (attachmentDir.startsWith(docDir))
					attachmentDir = attachmentDir.substring(docDir.length());
				
				String fileDir = docURL + "/" + attachmentDir;
				
				path = getEnvironment().getString().expand(fileDir + "/" + label);
			}

			path = path.replace('\\', '/');
		}

		return path;
	}
	
	/**
	 * @return field id
	 */
	public String getFileId()
	{
		return _formField.getId();
	}

	public boolean isFile()
	{
		return true;
	}
}
