package leon.view.web.jquery.component.form.field.list;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import leon.view.LyList;
import leon.view.LyListOption;
import leon.view.getinterface.LyGetValueInterface;
import leon.view.graphic.LyGraphicalList;
import leon.view.setinterface.LySetIdleInterface;
import leon.view.web.jquery.component.fieldItem.LyJQFieldItem;
import leon.view.web.jquery.controller.LyTemplatesUtilsBean;

/**
 * LyList internal view implementation
 * @author abkristou
 */
public class LyJQList extends LyJQFieldItem	implements
											LyGraphicalList,
											LySetIdleInterface,
											LyGetValueInterface
{

	protected LyList			_list;

	/** Indicates if the list is idle or not. */
	protected boolean			_idle;

	public static final String	LIST_ADD_OPTION		= "form/field/list/actions/addListOption.vm";

	public static final String	LIST_REMOVE_OPTION	= "form/field/list/actions/removeListOption.vm";

	public static final String	LIST_REFRESH_SCRIPT	= "form/field/list/actions/refreshList.vm";

	public LyJQList(LyList list)
	{
		super(list);
		_list = list;
		_idle = false;
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.getinterface.LyGetValueInterface#getValue()
	 */
	public String getValue()
	{
		int i, n = _list.getListOptionCount();
		LyListOption listOption;
		StringBuffer sb = new StringBuffer();
		char sep = getEnvironment().getString().SEP;

		for (i = 0; i < n; i++)
		{
			listOption = _list.getListOption(i);

			if (!listOption.isSelected())
				continue;

			if (sb.length() > 0)
				sb.append(sep);

			sb.append(listOption.getId());
		}

		if (sb.length() == 0)
			return null;
		else
			return (sb.toString());
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetIdleInterface#setIdle(boolean)
	 */
	public void setIdle(boolean status)
	{
		_idle = status;
		if (!status)
		{

		}
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalList#addListOption(leon.view.LyListOption, int)
	 */
	public void addListOption(LyListOption listOption, int pos)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("list", _list);
		params.put("option", listOption);
		params.put("pos", pos);

		String editableCell = templateManager.mergeTemplate(LIST_ADD_OPTION, params);

		_viewManager.addToUpdateScript(editableCell);
		refreshList();
		editableCell = null;
		params = null;
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalList#removeListOption(leon.view.LyListOption)
	 */
	public void removeListOption(LyListOption listOption)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("option", listOption);
		params.put("list", _list);
		String removeOptionScript = templateManager.mergeTemplate(LIST_REMOVE_OPTION, params);

		_viewManager.addToUpdateScript(removeOptionScript);
		refreshList();
		params = null;
		removeOptionScript = null;
	}

	/**
	 * @return list options
	 */
	public List<LyListOption> getListOptions()
	{
		List<LyListOption> listOptions = new ArrayList<LyListOption>();
		LyListOption listOption;
		for (int listOptionIndex = 0; listOptionIndex < _list.getListOptionCount(); listOptionIndex++)
		{
			listOption = _list.getListOption(listOptionIndex);
			listOptions.add(listOption);
		}
		return listOptions;
	}

	/**
	 * reapplicate refresh script on list after a modif has been done
	 */
	public void refreshList()
	{
		_viewManager.addToReloadComponents(_list);
	}

	/**
	 * refresh script
	 */
	@Override
	public String getReloadScript()
	{
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("listId", getId());

		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		String refreshScript = templateManager.mergeTemplate(LIST_REFRESH_SCRIPT, params);
		params = null;
		return refreshScript;
	}

	public boolean isList()
	{
		return true;
	}
}
