package leon.view.web.jquery.component.form.field.number;

import java.text.*;
import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.jquery.component.fieldItem.*;

/**
 * LyNumber JQuery internal view
 * @author yoChaari
 */
public class LyJQNumber extends LyJQFieldItem
	implements LyGraphicalNumber
{

	/** The number generic component. */
	protected LyNumber		_number;

	protected static String	LY_MAX_VALUE	= "LY_MAX_VALUE";

	protected static String	LY_MIN_VALUE	= "LY_MIN_VALUE";

	/**
	 * Class constructor of a Number object for the struts target.
	 * @param number the generic Number defined for every type of view targets.
	 */
	public LyJQNumber(LyNumber number)
	{
		super(number);
		_number = number;
	}

	/**
	 * get number decimals
	 * @return nb decimals
	 */
	public int getDecimals()
	{
		return _number.getDecimals();
	}

	/**
	 * get min value
	 * @return minvalue
	 */
	public double getMin()
	{
		return _number.getVMin();
	}

	/**
	 * get max value
	 * @return max
	 */
	public double getMax()
	{
		return _number.getVMax();
	}

	/**
	 * Gets the displayable label for this number.
	 * @param label the initial label
	 * @param decimalSeparator the decimal separator to be used
	 * @return the displayable label for this number
	 */
	public String getDisplayableLabel(String label)
	{
		LyEnvironment environment = getEnvironment();
		DecimalFormatSymbols format = new DecimalFormatSymbols(environment.getLocale());
		char decimalSeparator = format.getDecimalSeparator();

		int _decimals = getDecimals();
		double _min = getMin();
		double _max = getMax();

		if (label == null)
			return "";

		// FFT00021 : If there is no label we let an empty string.
		if (label.length() != 0)
		{
			label = getHtmlString(label);

			// Check the value (must be <= max, >= min and
			// number of decimals <= decimals)
			if (_decimals > 0)
				label = LyTools.checkDecimals(label, _decimals);

			// Be sure to replace decimal separator by '.'
			label = label.replace(decimalSeparator, '.');

			Number value;


			// FFT00529 // FFT04832 // FFT04744
			if (_decimals <= 0 && !(_fieldItem.getFieldValue() != null &&  _fieldItem.getFieldValue() instanceof Double))
			{
				value = new Long(LyTools.longFromString(label));
				if (value.doubleValue() <= _min)
					value = new Long(Math.round(_min));
				else if (value.doubleValue() >= _max)
					value = new Long(Math.round(_max));
			}
			else
			{
				value = new Double(LyTools.doubleFromString(label));
				if (value.doubleValue() <= _min)
					value = new Double(_min);
				else if (value.doubleValue() >= _max)
					value = new Double(_max);
			}

			label = LyTools.checkDecimals(String.valueOf(value), _decimals);

			// Check the number of decimals and add '0' at the end if needed
			if (_decimals > 0)
			{
				int pos = label.indexOf('.');
				int decimalsNb = label.length() - pos - 1;

				if ((pos > 0) && (decimalsNb < _decimals))
				{
					int i, n = _decimals - decimalsNb;

					for (i = 0; i < n; i++)
						label = label + '0';
				}
			}

			// label = label.replace('.', decimalSeparator);
		}

		return label;
	}

	/**
	 * get max chars
	 */
	public int getMaxChars(LyNumber number)
	{
		int maxChars = 1;
		double var = 1;
		int i = 0;
		boolean noMin = (getMin() == Double.NEGATIVE_INFINITY);
		boolean noMax = (getMax() == Double.POSITIVE_INFINITY);

		if (!noMax && !noMin)
		{
			// Get the max between min and max (abs values)
			double absMax = Math.max(Math.abs(getMax()), Math.abs(getMin()));

			for (i = 0; (var <= absMax) && (i < 20); i++)
				var *= 10;

			maxChars = i + 1
					+ ((getDecimals() == 0) ? 0 : ((getDecimals() < 0) ? 10 : (getDecimals() + 1)));

			if (maxChars > 25)
				maxChars = 25;
		}
		else if (getDecimals() == 0)
			maxChars = 10;
		else
			maxChars = 20;

		return maxChars;
	}

	/**
	 * get decimal separator
	 * @return
	 */
	public char getDecimalSep()
	{
		Locale locale = getEnvironment().getLocale();
		DecimalFormatSymbols format = new DecimalFormatSymbols(locale);
		return format.getDecimalSeparator();
	}

	/**
	 * get increase spinner label
	 * @param number
	 * @return label
	 */
	public String getLabelIncrease(LyNumber number)
	{
		String maxStr = null;
		String unit = translate(number.getUnit());

		if (getDecimals() == 0)
		{
			maxStr = String.valueOf((long) getMax()).replace('.', getDecimalSep());
		}
		else
		{
			maxStr = String.valueOf(getMax()).replace('.', getDecimalSep());
		}

		return getString(LY_MAX_VALUE, new String[] { maxStr, unit });
	}

	/**
	 * Get decrease spinner lable
	 * @param number
	 * @return label
	 */
	public String getLabelDecrease(LyNumber number)
	{
		String minStr = null;
		String unit = translate(number.getUnit());

		if (getDecimals() == 0)
		{
			minStr = String.valueOf((long) getMin()).replace('.', getDecimalSep());
		}
		else
		{
			minStr = String.valueOf(getMin()).replace('.', getDecimalSep());
		}
		return getString(LY_MIN_VALUE, new String[] { minStr, unit });
	}

	public void setDecimals(int decimals)
	{
		_viewManager.addToReloadComponents(_number.getParent());

	}

	public void setFormat(String format)
	{
		// TODO Auto-generated method stub

	}

	public void setPitch(double pitch)
	{
		_viewManager.addToReloadComponents(_number.getParent());

	}

	public void setVMax(double vMax)
	{
		_viewManager.addToReloadComponents(_number.getParent());

	}

	public void setVMin(double vMin)
	{
		_viewManager.addToReloadComponents(_number.getParent());

	}

	public boolean isNumber(){
		return true;
	}

	@Override
	public void setLabel(String label)
	{
		if (_updating && !getWebViewManager().isShared())
			return;

		_viewManager.addToReloadComponents(_number.getParent());
	}
}
