package leon.view.web.jquery.component.form.field.relation;

import java.awt.*;
import java.util.*;
import java.util.List;

import leon.misc.*;
import leon.view.*;
import leon.view.getinterface.*;
import leon.view.graphic.*;
import leon.view.web.jquery.common.*;
import leon.view.web.jquery.component.fieldItem.*;
import leon.view.web.jquery.controller.*;

/**
 * This class implements the Relation object (as described in the DTD) for web2 viewer.
 * <p>
 * <b>RELATION: Input Field of Relation Type</b><br>
 * An input field of Relation type allows to choose one or more values among a list of values. This list offers
 * generally application objects belonging to a same class and is used to establish a relation between an object and one
 * or more objects of this class. It is possible to specify if the relation is multiple or not.
 * </p>
 * @author ous
 * @see leon.view.LyRelation
 */
public class LyJQRelation extends LyJQFieldItem	implements
												LyGraphicalRelation,
												LyGetValueInterface
{

	/** Default number of rows for list selectors. */
	static int						DEFAULT_HEIGHT				= 6;

	/** Default width for list selectors. */
	static int						DEFAULT_WIDTH				= 20;

	protected LyRelation			_relation;

	protected static final String	COMPONENT_ID_PARAM			= "componentId";

	protected static final String	RELATION					= "relation";

	protected static final String	RELATION_ID_PARAM			= "relationId";

	protected static final String	RELATION_TEMPLATE_PATH		= "form/field/relation/lyRelationTemplate.vm";

	protected static final String	RELATION_AC_LINK_TEMPLATE_PATH	= "form/field/relation/ajax/addRelationAcLink.vm";

	protected static final String	CONTENT_PARAM				= "content";

	protected static final String	LINK_ID_PARAM				= "linkId";

	protected static final String	LINK_PARAM					= "link";

	protected static final String	POS_PARAM					= "pos";

	private static final String		LINK_TEMPLATE_PATH			= "form/field/relation/lyLinkTemplate.vm";

	private static final String		ADD_LINK_TEMPLATE_PATH		= "form/field/relation/ajax/addLink.vm";

	private static final String		REMOVE_LINK_TEMPLATE_PATH	= "form/field/relation/ajax/removeLink.vm";

	public LyJQRelation(LyRelation relation)
	{
		super(relation);
		_relation = relation;
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.getinterface.LyGetValueInterface#getValue()
	 */
	public String getValue()
	{
		int          i, n = _relation.getLinkCount();
		LyLink       link;
		StringBuffer sb = new StringBuffer();
		char         sep = getEnvironment().getString().SEP;

		for (i = 0; i < n; i++)
		{
			link = _relation.getLink(i);

			if ((link == null) || !link.isSelected())
				continue;

			if (sb.length() > 0)
				sb.append(sep);

			sb.append(link.getId());
		}

		if (sb.length() == 0)
			return null;
		else
			return (sb.toString());
	}
	
	public String getInitialName()
	{
		int          i, n = _relation.getLinkCount();
		LyLink       link;
		StringBuffer sb = new StringBuffer();

		for (i = 0; i < n; i++)
		{
			link = _relation.getLink(i);

			if ((link != null) && link.isSelected())
			{
				sb.append(link.getLabel());
				break;
			}
		}

		if (sb.length() == 0)
			return "";
		else
			return (sb.toString());
	}

	/**
	 * Check if relation has autoCompletion
	 * @return
	 */
	public boolean isAutoCompletion()
	{
		return (_relation.getAutoCompletionMax() != 0);
	}
	
	/**
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalRelation#addLink(leon.view.LyLink, int)
	 */
	public void addLink(LyLink link, int pos)
	{
		if (_updating && !getWebViewManager().isShared())
			return;
		
		if (_relation.isMultiple())
			reloadComponent();
		else
		{
			if (_viewManager.isAutoCompleteEvent() || isAutoCompletion())
			{
				//dont refresh all the relation. Just prepares the link list for the auto complete plugin
				_viewManager.addToReloadComponents(_relation);
			}
			else
			{
				LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
				Map<String, Object>		params = new HashMap<String, Object>();

				params.put(LINK_PARAM, link);
		
				String linkContent = templateManager.mergeTemplate(LINK_TEMPLATE_PATH, params);
				
				linkContent = TemplateUtils.getFormattedJs(linkContent);
				params = new HashMap<String, Object>();
				params.put(CONTENT_PARAM, linkContent);
				params.put(RELATION_ID_PARAM, getId());
				params.put(POS_PARAM, pos);
		
				String addLinkScript = templateManager.mergeTemplate(ADD_LINK_TEMPLATE_PATH, params);

				_viewManager.addToUpdateScript(addLinkScript);
				params = null;
				linkContent = null;
				addLinkScript = null;
			}
		}
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalRelation#removeLink(leon.view.LyLink)
	 */
	public void removeLink(LyLink link)
	{
		if (_updating && !getWebViewManager().isShared())
			return;
		
		if (_relation.isMultiple())
			reloadComponent();
		else
		{
			if (_viewManager.isAutoCompleteEvent() || isAutoCompletion())
			{
				// dont refresh all the relation. Just prepares the link list for the auto complete plugin
				_viewManager.addToReloadComponents(_relation);
			}
			else
			{
				LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
				Map<String, Object>		params = new HashMap<String, Object>();

				params.put(LINK_ID_PARAM, ((LyJQLink) link.getInternalView()).getId());
		
				String editableCell = templateManager.mergeTemplate(REMOVE_LINK_TEMPLATE_PATH, params);
		
				_viewManager.addToUpdateScript(editableCell);
				params = null;
				editableCell = null;
			}
		}
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.web.jquery.component.fieldItem.LyJQFieldItem#getReloadScript()
	 */
	@Override
	public String getReloadScript()
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		
		String newRelationScript="";
		if(_viewManager.isAutoCompleteEvent())
		{
			//dont refresh all the relation. Just prepares the link list for the auto complete plugin
			for(int currentLinkId =0 ;currentLinkId < _relation.getLinkCount();currentLinkId++)
			{
				Map<String, Object> params = new HashMap<String, Object>();	
				params.put(LINK_PARAM, _relation.getLink(currentLinkId));
				String currentLinkParams="";
				currentLinkParams= templateManager.mergeTemplate(RELATION_AC_LINK_TEMPLATE_PATH, params);
				newRelationScript=newRelationScript+((currentLinkId ==0)? "":"\n")+currentLinkParams;
				params = null;
			}
			_viewManager.setAutoCompleteEvent(false);
		}
		else
		{
			Map<String, Object> params = new HashMap<String, Object>();	
			params.put(RELATION, _relation);
			newRelationScript = templateManager.mergeTemplate(RELATION_TEMPLATE_PATH, params);
			newRelationScript = TemplateUtils.getFormattedJs(newRelationScript);
			params = new HashMap<String, Object>();
			params.put(COMPONENT_ID_PARAM, getId());
			params.put(CONTENT_PARAM, newRelationScript);
			newRelationScript = templateManager.mergeTemplate("ajax/replaceWith.vm", params);
			params = null;
		}
		return newRelationScript;

	}

	/**
	 * Collect the links of the relation in a list.
	 * @return list of links.
	 */
	public List<LyLink> getLinksList()
	{
		int c = _relation.getLinkCount();
		List<LyLink> links = new ArrayList<LyLink>(c);
		for (int i = 0; i < c; i++)
		{
			links.add(_relation.getLink(i));
		}
		return links;
	}

	/**
	 * reload relation component
	 */
	protected void reloadComponent()
	{
		if (_updating && !getWebViewManager().isShared())
			return;
				
		_viewManager.addToReloadComponents(_relation);
	}

	public int getHeight()
	{
		int height = _relation.getNRows();

		if (height <= 0)
			height = getIntResource("LY_MULTIPLE_RELATION_HEIGHT");

		if (height <= DEFAULT_HEIGHT)
			height = DEFAULT_HEIGHT;

		return height;
	}

	/**
	 * Return the width of the relation in pixels.
	 */
	@SuppressWarnings("deprecation")
	public int getWidth()
	{
        int     nCols = _relation.getNCols();

        if (nCols > 0)
        {
			// Calculate estimation of size in pixels
			LyEnvironment	environment = getEnvironment();
			Font			font = (Font) environment.getFont("LY_DEFAULT_FONT");
			Toolkit			toolkit = Toolkit.getDefaultToolkit();
			FontMetrics		fm = toolkit.getFontMetrics(font);

			if (_relation.isMultiple())
				return fm.charWidth('W') * nCols * 2;

			return fm.charWidth('W') * nCols + 20;
        }
        
		return -1;
	}

	/**
	 * Return the minimum width of the relation in pixels.
	 */
	@SuppressWarnings("deprecation")
	public int getMinWidth()
	{
		int		minWidth = -1;
        int     nCols = _relation.getNCols();
		
        if (nCols <= 0)
        {
            minWidth = _relation.getMinWidth();
            
            if (_relation.isMultiple() && (minWidth > 0))
            {
        		// Calculate estimation of size in pixels
        		LyEnvironment	environment = getEnvironment();
        		Font			font = (Font) environment.getFont("LY_DEFAULT_FONT");
        		Toolkit			toolkit = Toolkit.getDefaultToolkit();
        		FontMetrics		fm = toolkit.getFontMetrics(font);

    			if (_relation.isMultiple())
            		minWidth = fm.charWidth('W') * minWidth * 2;
    			else
    				minWidth = fm.charWidth('W') * minWidth + 20;
            }
        }
        
        return minWidth;
	}

	/**
	 * Return the maximum width of the relation in pixels.
	 */
	@SuppressWarnings("deprecation")
	public int getMaxWidth()
	{
		int		maxWidth = -1;
        int     nCols = _relation.getNCols();
		
        if (nCols <= 0)
        {
        	maxWidth = _relation.getMaxWidth();
            
            if (_relation.isMultiple() && (maxWidth > 0))
            {
        		// Calculate estimation of size in pixels
        		LyEnvironment	environment = getEnvironment();
        		Font			font = (Font) environment.getFont("LY_DEFAULT_FONT");
        		Toolkit			toolkit = Toolkit.getDefaultToolkit();
        		FontMetrics		fm = toolkit.getFontMetrics(font);

    			if (_relation.isMultiple())
            		maxWidth = fm.charWidth('W') * maxWidth * 2;
    			else
    				maxWidth = fm.charWidth('W') * maxWidth + 20;
            }
        }
        
        return maxWidth;
	}

	/**
	 * @return relation size
	 */
	public int getSize()
	{
		List<LyLink>	tempLyLinks = getLinksList();
		int				size = 0;
		int				linkLength = 0;
		
		for (LyLink lyLink : tempLyLinks)
		{
			linkLength = lyLink.getLabel().length();
			
			if (linkLength > size)
				size = linkLength;
		}

		return size;
	}

	public String getSelectAllLabel()
	{
		return getHtmlString("LY_SELECT_ALL");
	}

	public String getUnselectAllLabel()
	{
		return getHtmlString("LY_UNSELECT_ALL");
	}

	public String getLeftHeaderLabel()
	{
		return getHtmlString("LY_MULTISELECTOR_LEFT_HEADER");
	}

	public String getRightHeaderLabel()
	{
		return getHtmlString("LY_MULTISELECTOR_RIGHT_HEADER");
	}

	public boolean isRelation()
	{
		return true;
	}
}
