/*
 * Lyria-W4 - afficheur_web_2
 * LyJQText.java
 * Copyright (c) 2000-2011, Lyria-W4
 * All rights reserved.
 */
package leon.view.web.jquery.component.form.field.text;

import java.awt.*;
import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;
import leon.view.web.jquery.common.*;
import leon.view.web.jquery.component.fieldItem.*;
import leon.view.web.jquery.controller.*;

/**
 * LyWeb2Text
 * @version $Revision$
 * @author slachiheb, last modified by bsmichet
 */
public class LyJQText extends LyJQFieldItem
	implements LyGraphicalText
{
	/** Indicates whether non editable text must be displayed as labels. */
	public static boolean	USE_LABEL_FOR_NON_EDITABLE_TEXT	= false;

	/** The text generic component. */
	protected LyText		_text;

	/**
	 * LyWeb2Text Constructor
	 * @param fieldItem
	 */
	protected LyJQText(LyFieldItem fieldItem)
	{
		super(fieldItem);
	}

	/**
	 * Class constructor of a Text object for the struts target.
	 * @param text the generic Text defined for every type of view targets
	 */
	public LyJQText(LyText text)
	{
		super(text);
		_text = text;
	}

	/**
	 * Check if field is enabled
	 * @param text
	 * @return
	 */
	public boolean isEnabled()
	{
		return _formField.isEnabled();
	}

	/**
	 * Check if parent form is editable
	 * @param text
	 * @return
	 */
	public boolean isEditable()
	{
		return (_text.isEditable() && super.isEditable());
	}

	/**
	 * Check if text is a rich text
	 * @param text
	 * @return
	 */
	public boolean isRichText()
	{
		return (_text.getType() == LyText.Type.RICH);
	}

	public boolean isEditableText()
	{
		return ((USE_LABEL_FOR_NON_EDITABLE_TEXT /* || pw.isPrintMode() */) && !_text.isEditable());
	}

	public String getFile()
	{
		return _environment.getFile("LY_RICH_TEXT_EDITOR_FILE");
	}
	
	/**
	 * Returns the width of the text component.
	 * If a fixed width must be used, result is a positive integer.
	 * If a minimum width must be used, result is a negative integer.
	 */
	@SuppressWarnings("deprecation")
	public int getTextWidth()
	{
		int	 			 textWidth = -1;

		// If a constraint is set on the text, use the nCols and nRows values to compute
		// text size in pixels.
		if ((_text.getParent() instanceof LyFormField)
				&& (((LyFormField)(_text.getParent())).getConstraint() != null))
		{
			LyFormField		formField = (LyFormField)_text.getParent();
			LyConstraint	constraint = formField.getConstraint();
			
			if ((constraint != null) && (constraint.getNCols() > 0))
			{
				textWidth = constraint.getNCols();
				
				if (formField.getFieldUnit() != null)
				{
					LyFieldUnit	unit = formField.getFieldUnit();
					LyFieldItem unitItem = unit.getFieldItem();

					if (unitItem instanceof LyLabel)
					{
						LyLabel			label = (LyLabel)unitItem;
						String			value = label.getLabel();
						Font			font = (Font)getEnvironment().getFont(label.getFont());
						Toolkit			toolkit = Toolkit.getDefaultToolkit();
						FontMetrics		fm = toolkit.getFontMetrics(font);
						
						value = " " + translate(value);
						textWidth -= fm.stringWidth(value) + 1;
					}
				}
			}
		}

		if (textWidth < 0)
		{
			textWidth = getTextWidth((_text.getNCols() < 0) ? LyText.DEFAULT_NCOLS : _text.getNCols());
			
			// If ncols is set on text field, use a minimum width. Otherwise, use the width set on the constraint.
			if (_text.getType() == LyText.Type.RICH)
				textWidth = Math.max(480, textWidth);
		}

		return textWidth;
	}
	
	/**
	 * Returns the CSS style to be used for date input.
	 */
	@SuppressWarnings("deprecation")
	public int getTextWidth(int nCols)
	{
		Font			font = (Font)getEnvironment().getFont(_text.getFont());
		Toolkit			toolkit = Toolkit.getDefaultToolkit();
		FontMetrics		fm = toolkit.getFontMetrics(font);
		int				wCharWidth = fm.charWidth('W');
		int				averageCharWidth = fm.stringWidth("abcdefghijklmnopqrstuvwxyz") / 26;
		int sizeX = 0;
		
		if (nCols < 10)
            // With small size, the text field is sometimes not wide enough...
			sizeX = wCharWidth * (nCols + 1);
        else
            // Note : the 1.5 factor allows to have almost the same width as AWT...
        	sizeX = (averageCharWidth * nCols * 3) / 2;

		return sizeX + 4;
	}

	/**
	 * Returns the height of the text component.
	 */
	@SuppressWarnings("deprecation")
	public int getTextHeight()
	{
		Font			font = (Font)getEnvironment().getFont("LY_DEFAULT_FONT");
		Toolkit			toolkit = Toolkit.getDefaultToolkit();
		FontMetrics		fm = toolkit.getFontMetrics(font);
		int				rows = (_text.getNRows() <= 0 ? 1 : _text.getNRows());
		int				height = fm.getHeight() * rows;
		
		if ((_text.getType() == LyText.Type.RICH) && isEditable())
			height += 30;
		
		return height + 2;
	}

	/**
	 * Gets width style
	 * @param text
	 * @return
	 */
	public String getWidthStyle()
	{
		if (getTextWidth() < 0)
			return "100%";
		else
			return Math.abs(getTextWidth()) + "px";
	}

	/**
	 * Check if text has autoCompletion property
	 * @return
	 */
	public boolean isAutoCompletion()
	{
		return (_text.getAutoCompletionValues() != null)
				&& (_text.getAutoCompletionValues().length() > 0);
	}
	
	/**
	 * Gets a auto completion items
	 * @return completion items
	 */
	public String[] getAutoCompletionItems() {
		String autoCompletionValues  = _text.getAutoCompletionValues().split("\\"+LyTools.PIPE_SEP)[1];
		return autoCompletionValues.split(Character.toString(LyTools.VALUE_SEP));
	}

	/**
	 * Gets a auto completion max items
	 * @return completion items
	 */
	public int getAutoCompletionMaxItems() {
		return Integer.valueOf(_text.getAutoCompletionValues().split("\\"+LyTools.PIPE_SEP)[0]).intValue();
	}
	
	/**
	 * Gets a text number of columns
	 * @param text
	 * @return
	 */
	public int getNbCols()
	{
		return (_text.getNCols() < 0) ? LyText.DEFAULT_NCOLS : _text.getNCols();
	}

	/**
	 * Gets the secret correspondance of a label
	 * @param label
	 * @return
	 */
	public String getSecretLabel(String label)
	{
		int c, nc = ((label == null) ? 0 : label.length());

		for (c = 0, label = ""; c < nc; c++)
			label += '*';
		return label;
	}

	/**
	 * Check if field parent is a form
	 * @param text
	 * @return
	 */
	public boolean isParentForm()
	{
		return (_text.getParent().getParent() instanceof LyForm);
	}

	/**
	 * Gets url from a label
	 */
	public String getUrl(String label)
	{
		LyWebPageWriter pageWriter = getWebViewManager().getPageWriter();
		String target = getTarget();
		return replaceUrl(label, target, pageWriter.isPrintMode());
	}

	public boolean isText()
	{
		return true;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see leon.view.web.struts.LyStrutsFieldItem#setLabel(java.lang.String)
	 */
	public void setLabel(String label)
	{
		LyComponent parent = _text.getParent();

		if (_updating && !getWebViewManager().isShared())
			return;
		
		if (parent instanceof LyFormField)
		{
			if (!isRichText() && !_text.isNotifyValueChanged())
			{
				Map<String, Object>		params = new HashMap<String, Object>();

				params.put("componentId", getId());
				
				if (label == null)
					label = "";
				
				label = TemplateUtils.getFormattedJs(label);
				params.put("value", label);

				LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
				String					script = templateManager.mergeTemplate("ajax/form/field/setLabel.vm", params);

				_viewManager.addToUpdateScript(script);
				params = null;
			}
			else if (isRichText() && isEditable())
			{
				Map<String, Object>		params = new HashMap<String, Object>();

				params.put("viewManager", _viewManager);
				params.put("componentId", getId());
				
				if (label == null)
					label = "";
				else
				{
					label = getConvertedHTML(label);
					label = getFormattedRichText(label);
					label = TemplateUtils.getFormattedJs(label);
				}
				
				params.put("value", label);

				LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
				String					script = templateManager.mergeTemplate("ajax/form/field/setRichTextLabel.vm", params);

				_viewManager.addToUpdateScript(script);
				params = null;
			}
			else
				_viewManager.addToReloadComponents(_text.getParent());
		}
	}
	
	public String getFormattedRichText(String label)
	{
		return LyString.replace(label, "<br />\n", "<br />");
	}
}