/*
 * LyWeb2Frame.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.jquery.component.frame;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.*;

import leon.misc.*;
import leon.view.LyArea;
import leon.view.LyFrame;
import leon.view.LyMessages;
import leon.view.LyToolBar;
import leon.view.getinterface.LyIsVisibleInterface;
import leon.view.graphic.LyGraphicalFrame;
import leon.view.setinterface.LySetVisibleInterface;
import leon.view.web.*;
import leon.view.web.jquery.*;
import leon.view.web.jquery.common.TemplateUtils;
import leon.view.web.jquery.component.area.LyJQArea;
import leon.view.web.jquery.controller.LyTemplatesUtilsBean;

/**
 * Type : leon.view.web.web2.frame.LyJQFrame The peer component of LyFrame for the jquery viewer.
 * @version $Revision: -1M $
 * @author ychaari, last modified by $Author: (local) $
 */
public class LyJQFrame extends LyJQView	implements
										LyGraphicalFrame,
										LyIsVisibleInterface,
										LySetVisibleInterface
{
	/** The frame generic component. */
	protected LyFrame			_frame;

	protected LyArea			_area;

	protected static String		SWICH_TOOLBAR_EVENT			= "switchToolbarState";

	protected static String		FRAME_DIALOG_PARAM			= "lyFrameDialog";

	protected static String		CONTENT_PARAM				= "content";

	protected static String		FRAME_DIALOG_TEMPLATE		= "dialog/lyFrameDialogTemplate.vm";

	protected final String		SHOW_DIALOG_TEMPLATE		= "ajax/dialog/showDialog.vm";

	protected final String		CLOSE_DIALOG_TEMPLATE		= "ajax/dialog/closeDialog.vm";

	protected final String		DIALOG_ID_PARAM				= "dialogId";

	protected final String		DIALOG_LABEL_PARAM			= "dialogLabel";

	protected final String		DIALOG_CONTROLLER_PARAM		= "dController";

	protected final String		DIALOG_AREA_ID_PARAM		= "dialogAreaId";

	protected final String		DIALOG_RESIZABLE_PARAM		= "isResizable";

	protected final String		DIALOG_ISFRAME_PARAM		= "isFrameDialog";

	protected final String		DIALOG_ISPRINT_MODE_PARAM	= "isPrintMode";

	/**
	 * Constructor.
	 * @param frame the frame component
	 */
	public LyJQFrame(LyFrame frame)
	{
		super(frame);
		_frame = frame;
	}

	/**
	 * Get LyFrame object
	 * @return frame
	 */
	public LyFrame getLyFrame()
	{
		return _frame;
	}

	/**
	 * Set LyFrame
	 * @param _frame
	 */
	public void setLyFrame(LyFrame _frame)
	{
		this._frame = _frame;
	}

	/**
	 * Set label
	 */
	public void setLabel(String label)
	{
		// empty
	}

	/**
	 * Set LyMessages
	 */
	public void setMessages(LyMessages messages)
	{
		_viewManager.addToReloadComponents(messages);
		messages.getInternalView(true);
	}

	/**
	 * @see leon.view.web.jquery.LyJQView#setVisible(boolean)
	 */
	@Override
	public void setVisible(boolean status)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		
		if (templateManager != null)
		{
			Map<String, Object>		params = new HashMap<String, Object>();
			String					target = _frame.getTarget();
			
			if (status && (status != _visible))
			{
				if (LyWebConstants.POPUP_VIEW.equals(target))
				{
					String dialogTitle = getEnvironment().translate(_frame.getLabel());
	
					params.put("viewManager", _viewManager);
					params.put(FRAME_DIALOG_PARAM, _frame);
					params.put(DIALOG_LABEL_PARAM, TemplateUtils.getConvertedHTML(dialogTitle));
	
					String frameDialogDiv = templateManager.mergeTemplate(FRAME_DIALOG_TEMPLATE, params);
	
					frameDialogDiv = TemplateUtils.getFormattedJs(frameDialogDiv);
	
					params = new HashMap<String, Object>();
					params.put(LyJQConstants.DIV_TO_APPEND_PARAM, frameDialogDiv);
					params.put(DIALOG_ID_PARAM, getId());
					params.put(DIALOG_CONTROLLER_PARAM, _frame.getController());
					params.put(DIALOG_ISFRAME_PARAM, true);
					params.put(DIALOG_ISPRINT_MODE_PARAM, _viewManager.isPrintMode());
					LyArea lyArea = _frame.getArea();
					if (lyArea != null)
					{
						params.put(DIALOG_AREA_ID_PARAM,
								((LyJQArea) (lyArea.getInternalView())).getId());
						params.put(DIALOG_RESIZABLE_PARAM, true);
					}
					_viewManager.addToUpdateScript(templateManager.mergeTemplate(SHOW_DIALOG_TEMPLATE,
							params));
					frameDialogDiv = null;
				}
				else
				{
					LyWebRequestInfo	requestInfo = LyJQServlet.getWebRequestInfo();
					HttpServletRequest	request = (requestInfo == null ? null : requestInfo.getRequest());
					String				isAjaxMode = (request == null ? "true" : request.getParameter(LyJQConstants.AJAX_MODE));
					
					if ((isAjaxMode != null) && !isAjaxMode.equalsIgnoreCase("false"))
						_viewManager.redirectToHomePage();
				}
			}
			else if (!status)
			{
				if (LyWebConstants.POPUP_VIEW.equals(target))
				{
					params.put(DIALOG_ID_PARAM, getId());
					String closeEvent = templateManager.mergeTemplate(CLOSE_DIALOG_TEMPLATE, params);
					_viewManager.addToUpdateScript(closeEvent);
				}
				else
				{
					LyWebRequestInfo	requestInfo = LyJQServlet.getWebRequestInfo();
					HttpServletRequest	request = (requestInfo == null ? null : requestInfo.getRequest());
					String				isAjaxMode = (request == null ? "true" : request.getParameter(LyJQConstants.AJAX_MODE));
					
					if (!_viewManager.isSessionClosing() && (isAjaxMode != null) && !isAjaxMode.equalsIgnoreCase("false"))
						_viewManager.redirectToHomePage();
				}
			}
		
			params = null;
		}
		
		super.setVisible(status);
	}

	/**
	 * Getting LyWeb2Area
	 * @return area
	 */
	public LyJQArea getArea()
	{
		if (_frame.getArea() != null)
			return (LyJQArea) _frame.getArea().getInternalView();
		return null;
	}

	@Override
	public void setProperty(String name, String value)
	{
		super.setProperty(name, value);
		if (SWICH_TOOLBAR_EVENT.equals(name))
		{
			LyToolBar toolbar = _frame.getArea().getLeftToolBar();
			toolbar.setOpen(!(toolbar).isOpen());
		}
	}

	public boolean isVisible()
	{
		return _visible;
	}

	@Override
	public String getReloadScript()
	{
		return null;
	}

	/**
	 * add the dialog frame script
	 */
	public String addDialogFrameRefreshScript()
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();

		params.put("viewManager", _viewManager);
		params.put(FRAME_DIALOG_PARAM, _frame);
		
		String	frameDialogDiv = templateManager.mergeTemplate(FRAME_DIALOG_TEMPLATE, params);
		
		frameDialogDiv = TemplateUtils.getFormattedJs(frameDialogDiv);
		params = new HashMap<String, Object>();
		params.put(LyJQConstants.DIV_TO_APPEND_PARAM, frameDialogDiv);
		frameDialogDiv = LyString.replace(frameDialogDiv, "<script", "<scr\" + \"ipt");
		frameDialogDiv = LyString.replace(frameDialogDiv, "</script", "</scr\" + \"ipt");
		params.put(DIALOG_ID_PARAM, getId());
		params.put(DIALOG_LABEL_PARAM, getEnvironment().translate(_frame.getLabel()));
		params.put(DIALOG_CONTROLLER_PARAM, _frame.getController());
		params.put(DIALOG_ISFRAME_PARAM, true);

		LyArea	lyArea = _frame.getArea();
		
		if (lyArea != null)
		{
			params.put(DIALOG_AREA_ID_PARAM, ((LyJQArea) _frame.getArea().getInternalView()).getId());
			params.put(DIALOG_RESIZABLE_PARAM, true);
		}
		
		String	result = templateManager.mergeTemplate(SHOW_DIALOG_TEMPLATE, params);

		params = null;
		frameDialogDiv = null;

		return result;
	}

	public void writeHtml(LyWebPageWriter pw)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();
		
		params.put("lyFrame", _frame);
		
		String	result = templateManager.mergeTemplate("frame/lyFrameTemplate.vm", params);
		
		pw.append(result);
	}

}