/* 
 * Lyria-W4 - afficheur_web_2
 * LyJQInterval.java
 * Copyright (c) 2000-2009, Lyria-W4
 * All rights reserved.
 */
package leon.view.web.jquery.component.gantt.interval;

import java.text.DateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Vector;

import leon.misc.LyString;
import leon.view.LyComponent;
import leon.view.LyGantt;
import leon.view.LyInterval;
import leon.view.LyLine;
import leon.view.LyPeriod;
import leon.view.graphic.LyGraphicalInterval;
import leon.view.web.LyWebPageWriter;
import leon.view.web.jquery.component.LyJQInternalView;
import leon.view.web.jquery.component.gantt.LyJQGantt;
import leon.view.web.jquery.component.gantt.line.LyJQLine;

/**
 * LyJQInterval
 * @version $Revision$
 * @author slachiheb, last modified by $Author$
 */
public class LyJQInterval extends LyJQInternalView	implements
													LyGraphicalInterval
{

	/** The interval generic component. */
	protected LyInterval	_interval;

	/** Begin date cache **/
	protected Date			_beginDate;

	/** End date cache **/
	protected Date			_endDate;

	protected LyGantt		_parentGantt;

	protected LyJQGantt		_htmlGantt;

	/**
	 * Class constructor of an Interval object for the struts target.
	 * @param interval the generic Interval defined for every type of view targets
	 */
	public LyJQInterval(LyInterval interval)
	{
		super(interval);
		_interval = interval;
		_parentGantt = getParentGantt();
		_htmlGantt = (LyJQGantt) _parentGantt.getInternalView();
	}


	public LyJQGantt getJQGantt()
	{
		return _htmlGantt;
	}
	
	/**
	 * This method does nothing.
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	}

	/**
	 * Return the begin date of the interval using the date format.
	 * @return the begin date
	 */
	public Date getBeginDate()
	{
		if (_beginDate == null)
			_beginDate = parseDate(_interval.getBeginDate());
		return _beginDate;
	}


	public float getXStartPosition()
	 {
		Calendar beginCalendar = Calendar.getInstance();
		Date	date = getBeginDate();
		
		if (date == null)
			return 0;
		
		beginCalendar.setTime(date);
	  return _htmlGantt.getXPositionOfGivenDate(beginCalendar);	  
	 }
	
	public float getXEndPosition()
	 {
		Calendar endCalendar = Calendar.getInstance();
		Date	date = getEndDate();
		
		if (date == null)
			return 0;
		
		endCalendar.setTime(date);
	  return _htmlGantt.getXPositionOfGivenDate(endCalendar);	  
	 }
	
	public float getIntervalPeriod()
	 {
		long intervalPeriod = getEndDate().getTime()- getBeginDate().getTime();
		long stepWidth = _htmlGantt.getStepWidthInMilliSecond();
		return (float) intervalPeriod/stepWidth;
	 }
	
	public Vector<LyInterval> getDependenciesVector() {					// Compute dependencies
		String dependencies = _interval.getDependencies();
		Vector<LyInterval> result = new Vector<LyInterval>();

		if ((dependencies != null) && (dependencies.length() > 0))
		{
			String[] dependenciesArray = LyString.stringToList(dependencies, ' ');

			if (dependenciesArray != null)
			{
				String intervalId;
				LyInterval intervalTmp;
				// For each dependency, retrieve the pointed
				// interval and its Gantt element
				// and add it to the list of dependencies
				for (int k = 0; k < dependenciesArray.length; k++)
				{
					intervalId = dependenciesArray[k];
					intervalTmp = (LyInterval) _parentGantt.getComponent(intervalId);

					if (intervalTmp != null)
						result.addElement(intervalTmp);
				}

			}
		}
		return result;
		}
	
	/**
	 * Return the end date of the interval using the date format.
	 * @return the end date
	 */
	public Date getEndDate()
	{
		if (_endDate == null)
			_endDate = parseDate(_interval.getEndDate());
		return _endDate;
	}

	/**
	 * Parses a date value using date time format and date format.
	 * @param value the date value
	 * @return the parsed date
	 */
	public Date parseDate(String value)
	{
		DateFormat dateFormat = getDateTimeFormat();
		Date date = null;

		try
		{
			date = dateFormat.parse(value);
		}
		catch (Exception e)
		{
			try
			{
				// If date could not be parsed with date and time format, try
				// with date only
				dateFormat = getDateFormat();
				date = dateFormat.parse(value);
			}
			catch (Exception e2)
			{
			}
		}

		return date;
	}

	/**
	 * Gets the date and time format associated with this interval.
	 * @return the date and time format
	 */
	protected DateFormat getDateTimeFormat()
	{
		// Get the date format from the Gantt component
		DateFormat dateFormat = _htmlGantt.getDateTimeFormat();

		return dateFormat;
	}

	/**
	 * Gets the date format associated with this interval.
	 * @return the date format
	 */
	protected DateFormat getDateFormat()
	{
		// Get the date format from the Gantt component
		DateFormat dateFormat = _htmlGantt.getDateFormat();

		return dateFormat;
	}

	/**
	 * Gets the parent Gantt component of the interval.
	 * @return the parent Gantt component
	 */
	protected LyGantt getParentGantt()
	{
		LyComponent parent = _interval.getParent();

		if (parent instanceof LyLine)
		{
			LyLine line = (LyLine) parent;
			LyJQLine htmlLine = (LyJQLine) line.getInternalView();

			return htmlLine.getParentGantt();
		}

		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalInterval#addPeriod(leon.view.LyPeriod, int)
	 */
	public void addPeriod(LyPeriod period, int pos)
	{
		refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalInterval#removePeriod(leon.view.LyPeriod)
	 */
	public void removePeriod(LyPeriod period)
	{
		refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalInterval#setBeginDate(java.lang.String)
	 */
	public void setBeginDate(String beginDate)
	{
		refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetColorInterface#setColor(java.lang.String)
	 */
	public void setColor(String color)
	{
		refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalInterval#setDependencies(java.lang.String)
	 */
	public void setDependencies(String dependencies)
	{
		refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalInterval#setDrawVerticalLines(boolean)
	 */
	public void setDrawVerticalLines(boolean drawVerticalLines)
	{
		refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalInterval#setEndDate(java.lang.String)
	 */
	public void setEndDate(String endDate)
	{
		refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetFontInterface#setFont(java.lang.String)
	 */
	public void setFont(String font)
	{
		refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalInterval#setForegroundColor(java.lang.String)
	 */
	public void setForegroundColor(String foregroundColor)
	{
		refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetLabelInterface#setLabel(java.lang.String)
	 */
	public void setLabel(String label)
	{
		refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetLabelInterface#setTooltip(java.lang.String)
	 */
	public void setTooltip(String tooltip)
	{
		refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetLabelInterface#setCenterLabel(java.lang.String)
	 */
	public void setCenterLabel(String label)
	{
		refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalInterval#setLeftResizable(boolean)
	 */
	public void setLeftResizable(boolean leftResizable)
	{
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalInterval#setMovable(boolean)
	 */
	public void setMovable(boolean movable)
	{
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalInterval#setRightResizable(boolean)
	 */
	public void setRightResizable(boolean rightResizable)
	{
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalInterval#setVerticalLinesColor(java.lang.String)
	 */
	public void setVerticalLinesColor(String verticalLinesColor)
	{
	}

	/**
	 * Refresh frame containing this web object.
	 */
	@Override
	public void refresh()
	{
		// clear cache
		_beginDate = null;
		_endDate = null;
		_viewManager.addToReloadComponents(_parentGantt);

	}

	/**
	 * @see leon.view.web.jquery.component.LyJQInternalView#getReloadScript()
	 */
	@Override
	public String getReloadScript()
	{
		// TODO slachiheb Auto-generated method stub
		return null;
	}
} // end class LyJQInterval
