/* 
 * Lyria-W4 - afficheur_web_2
 * LyJQLine.java
 * Copyright (c) 2000-2009, Lyria-W4
 * All rights reserved.
 */
package leon.view.web.jquery.component.gantt.line;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import leon.misc.LyString;
import leon.view.LyComponent;
import leon.view.LyGantt;
import leon.view.LyInterval;
import leon.view.LyLine;
import leon.view.graphic.LyGraphicalLine;
import leon.view.web.LyWebPageWriter;
import leon.view.web.jquery.common.TemplateUtils;
import leon.view.web.jquery.component.LyJQInternalView;
import leon.view.web.jquery.component.gantt.LyJQGantt;
import leon.view.web.jquery.component.gantt.interval.LyJQInterval;
import leon.view.web.jquery.controller.LyTemplatesUtilsBean;

/**
 * LyJQLine
 * @version $Revision$
 * @author slachiheb, last modified by $Author$
 */
public class LyJQLine extends LyJQInternalView	implements
												LyGraphicalLine
{

	/** The line generic component. */
	protected LyLine		_line;

	protected static String	GANTT_PLANNING_TEMPLATE	= "gantt/lyGanttPlanningTemplate.vm";

	/**
	 * LyJQLine constructor
	 * @param viewComponent
	 */
	public LyJQLine(LyLine lyLine)
	{
		super(lyLine);
		_line = lyLine;
	}

	/**
	 * Computes the height of the line.
	 * @return the line height
	 */
	public int computeHeight()
	{
		int lineHeight = 22;
		int[] imageSize = getImageSize("LY_NODE");

		if ((imageSize != null) && (imageSize.length > 0))
			lineHeight = imageSize[1];

		return lineHeight;
	}

	/**
	 * Gets the parent Gantt component of the line.
	 * @return the parent Gantt component
	 */
	public LyGantt getParentGantt()
	{
		LyComponent parent = _line.getParent();

		if (parent instanceof LyLine)
		{
			LyLine line = (LyLine) parent;
			LyJQLine webLine = (LyJQLine) line.getInternalView();

			return webLine.getParentGantt();
		}
		else if (parent instanceof LyGantt)
			return (LyGantt) parent;

		return null;
	}

	/**
	 * get lines
	 * @return lines list
	 */
	public List<LyLine> getLines()
	{
		List<LyLine> lines = new ArrayList<LyLine>();
		LyComponent currentComp;
		for (int i = 0; i < _line.getComponentCount(); i++)
		{
			currentComp = _line.getComponent(i);
			if (!(currentComp instanceof LyLine) || ((LyLine) currentComp).isHidden())
				continue;
			lines.add((LyLine) currentComp);
		}
		return lines;
	}

	/**
	 * @see leon.view.graphic.LyGraphicalComponent#free()
	 */
	@Override
	public void free()
	{

	}

	/**
	 * @see leon.view.graphic.LyGraphicalLine#setLabel(java.lang.String)
	 */
	public void setLabel(String label)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("label", label);
		params.put("lineId", getId());
		params.put("ganttId", ((LyJQGantt) getParentGantt().getInternalView()).getId());
		_viewManager.addToUpdateScript(templateManager.mergeTemplate(
				"ajax/gantt/line/setLineLabel.vm", params));
		params = null;

	}

	/**
	 * @see leon.view.graphic.LyGraphicalLine#setImage(java.lang.String)
	 */
	public void setImage(String image)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("lineId", getId());
		params.put("imageUrl", getImageUrl(image));
		_viewManager.addToUpdateScript(templateManager.mergeTemplate(
				"ajax/gantt/line/setLineImage.vm", params));
		params = null;
	}

	/**
	 * @see leon.view.graphic.LyGraphicalLine#setTooltip(java.lang.String)
	 */
	public void setTooltip(String tooltip)
	{
		// TODO slachiheb Auto-generated method stub

	}

	/**
	 * @see leon.view.graphic.LyGraphicalLine#setSelected(boolean)
	 */
	public void setSelected(boolean selected)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("lineId", getId());
		params.put("selected", selected);
		params.put("ganttId", ((LyJQGantt) getParentGantt().getInternalView()).getId());
		_viewManager.addToUpdateScript(templateManager.mergeTemplate(
				"ajax/gantt/line/setLineSelected.vm", params));
		params = null;
	}

	/**
	 * @see leon.view.graphic.LyGraphicalLine#setOpen(boolean)
	 */
	public void setOpen(boolean open)
	{
		LyGantt gantt = getParentGantt();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("lyPlanning", gantt);
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		String planningHtml = templateManager.mergeTemplate(GANTT_PLANNING_TEMPLATE, params);
		planningHtml = TemplateUtils.getFormattedJs(planningHtml);
		params = new HashMap<String, Object>();
		params.put("lineId", getId());
		params.put("open", open);
		params.put("planningHtml", planningHtml);
		params.put("ganttId", ((LyJQGantt) getParentGantt().getInternalView()).getId());
		_viewManager.addToUpdateScript(templateManager.mergeTemplate(
				"ajax/gantt/line/setLineOpen.vm", params));
		params = null;
	}

	/**
	 * get the fmk id
	 */
	public String getFmkId()
	{
		return LyString.replace(_line.getId(), "\\", "\\\\");
	}

	/**
	 * TODO slachiheb Add a comment
	 * @see leon.view.graphic.LyGraphicalLine#setHidden(boolean)
	 */
	public void setHidden(boolean hidden)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("lineId", getId());
		params.put("hidden", hidden);
		_viewManager.addToUpdateScript(templateManager.mergeTemplate(
				"ajax/gantt/line/setLineHidden.vm", params));
		params = null;
	}

	/**
	 * @see leon.view.graphic.LyGraphicalLine#setFont(java.lang.String)
	 */
	public void setFont(String font)
	{

	}

	/**
	 * TODO slachiheb Add a comment
	 * @see leon.view.graphic.LyGraphicalLine#setColor(java.lang.String)
	 */
	public void setColor(String color)
	{
		// TODO slachiheb Auto-generated method stub

	}

	/**
	 * @see leon.view.graphic.LyGraphicalLine#setSuperLineColor(java.lang.String)
	 */
	public void setSuperLineColor(String superLineColor)
	{
		// TODO slachiheb Auto-generated method stub

	}

	/**
	 * @see leon.view.graphic.LyGraphicalLine#setSuperLineLabel(java.lang.String)
	 */
	public void setSuperLineLabel(String superLineLabel)
	{
		// TODO slachiheb Auto-generated method stub

	}

	/**
	 * @see leon.view.graphic.LyGraphicalLine#setSuperLineHidden(boolean)
	 */
	public void setSuperLineHidden(boolean superLineHidden)
	{
		// TODO slachiheb Auto-generated method stub

	}

	/**
	 * @see leon.view.graphic.LyGraphicalLine#addComponent(leon.view.LyComponent, int)
	 */
	public void addComponent(LyComponent components, int pos)
	{
		refresh();

	}

	/**
	 * @see leon.view.graphic.LyGraphicalLine#removeComponent(leon.view.LyComponent)
	 */
	public void removeComponent(LyComponent components)
	{
		refresh();

	}

	/**
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
		// TODO slachiheb Auto-generated method stub

	}

	/**
	 * Indicates if current line is a super line. (A super line is a parent line).
	 * @return true if current line is a super line, false otherwise
	 */
	protected boolean isSuperLine()
	{
		int i, n = _line.getComponentCount();
		LyComponent component;
		for (i = 0; i < n; i++)
		{
			component = _line.getComponent(i);

			if (component instanceof LyLine)
				return true;
		}

		return false;
	}

	/**
	 * TODO slachiheb Add a comment
	 * @see leon.view.web.jquery.component.LyJQInternalView#getReloadScript()
	 */
	@Override
	public String getReloadScript()
	{
		// TODO slachiheb Auto-generated method stub
		return null;
	}

	/**
	 * Gets the limit date for this line (minimum or maximum). Browse all line sub-components and retrieve their minimum
	 * (or maximum) dates.
	 * @param max indicate if maximum or minimum date must be returned and if end date is null, we consider begin date
	 * instead.
	 * @return the maximum date if max is true and if the end date is not null, the minimum date otherwise
	 */
	public Date getLimitDate(boolean max)
	{
		Date date = null;
		Date dateTmp = null;
		int i, n = _line.getComponentCount();

		// Browse all the sub-component
		LyLine line;
		LyJQLine internal1;
		LyInterval interval;
		LyJQInterval internal;
		LyComponent component;
		for (i = 0; i < n; i++)
		{
			component = _line.getComponent(i);

			dateTmp = null;

			if (component instanceof LyLine)
			{
				line = (LyLine) component;
				internal1 = (LyJQLine) line.getInternalView();

				dateTmp = internal1.getLimitDate(max);
			}
			else if (component instanceof LyInterval)
			{
				interval = (LyInterval) component;
				internal = (LyJQInterval) interval.getInternalView();

				if (max)
				{
					dateTmp = internal.getEndDate();

					if (dateTmp == null)
						// If end date is null, we consider the begin date
						dateTmp = internal.getBeginDate();
				}
				else
					dateTmp = internal.getBeginDate();
			}

			if (dateTmp == null)
				continue;

			if (date == null)
				date = dateTmp;
			else
			{
				if (max && dateTmp.after(date))
					date = dateTmp;

				if (!max && dateTmp.before(date))
					date = dateTmp;
			}

		}

		return date;
	} // end method getLimitDate

	/**
	 * Refresh frame containing this web object.
	 */
	@Override
	public void refresh()
	{
		_viewManager.addToReloadComponents(getParentGantt());

	}
	
	public float getXStartPosition()
	{
		Date		limitDate = getLimitDate(false);
		
		if (limitDate == null)
			return 0;
		
		Calendar	beginCalendar = Calendar.getInstance();

		beginCalendar.setTime(limitDate);

		return  ((LyJQGantt)getParentGantt().getInternalView()).getXPositionOfGivenDate(beginCalendar);	  
	}
	
	public float getXEndPosition()
	{
		Date		limitDate = getLimitDate(true);
		
		if (limitDate == null)
			return 0;
		
		Calendar	endCalendar = Calendar.getInstance();
		
		endCalendar.setTime(limitDate);
	
		return ((LyJQGantt) getParentGantt().getInternalView()).getXPositionOfGivenDate(endCalendar);	
	}
}
