package leon.view.web.jquery.component.header;

import java.util.HashMap;
import java.util.Map;

import leon.view.LyHeader;
import leon.view.LyHeaders;
import leon.view.LyTable;
import leon.view.graphic.LyGraphicalHeader;
import leon.view.web.LyWebPageWriter;
import leon.view.web.jquery.LyJQViewManager;
import leon.view.web.jquery.component.LyJQInternalView;
import leon.view.web.jquery.component.headers.LyJQHeaders;
import leon.view.web.jquery.component.table.LyJQTable;
import leon.view.web.jquery.controller.LyTemplatesUtilsBean;

/**
 * LyHeader JQuery internal view
 * @author bahaSmichet
 */
public class LyJQHeader extends LyJQInternalView implements
												LyGraphicalHeader
{
	public static final String	__VERSION = "$Id: LyStrutsHeader.java 25434 2010-05-10 08:32:31Z nhamon $";

	/** The Header generic component. */
	protected LyHeader			_header;

	/** Image for increase button. */
	protected String			_increaseButton;

	/** Image for decrease button. */
	protected String			_decreaseButton;

	/** Sort tooltip shown to the user. */
	protected String			_sortBy;

	protected LyTable			_parentTable;

	protected final String		TABLE_ID_TEMPLATE_PARAM			= "tableId";
	
	protected static String		HEADER_ID_PARAM					= "headerId";

	protected static String		HEADER_SORT_PARAM				= "headerSort";

	protected static String		HEADER_INCREASING_PARAM			= "headerIncreasing";

	protected static String		HEADER_BORDER_WIDTH				= "headerBorderWidth";

	protected static String		HEADER_BODER_COLOR				= "headerBorderColor";

	protected static String		HEADER_SORT_TEMPLATE			= "table/actions/headerSort.vm";

	protected static String		HEADER_INCREASING_TEMPLATE		= "table/actions/headerIncreasing.vm";

	protected static String		HEADER_BORDER_WIDTH_TEMPLATE	= "table/actions/headerBorderWidth.vm";

	protected static String		HEADER_BORDER_COLOR_TEMPLATE	= "table/actions/headerBorderColor.vm";

	public LyJQHeader(LyHeader header)
	{
		super(header);
		_header = header;
		
		LyHeaders	headers = (LyHeaders) _header.getParent();

		_parentTable = headers.getParentTable();
	}

	/**
	 * sort table ajax response
	 */
	public void setSort(int sort)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();

		params.put(TABLE_ID_TEMPLATE_PARAM, ((LyJQTable) _parentTable.getInternalView()).getId());
		params.put(HEADER_ID_PARAM, getId());
		params.put(HEADER_SORT_PARAM, sort);

		String	newDiv = templateManager.mergeTemplate(HEADER_SORT_TEMPLATE, params);

		_viewManager.addToUpdateScript(newDiv);
		newDiv = null;
		params = null;
	}

	/**
	 * header increasing ajax response
	 */
	public void setIncreasing(boolean increasing)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();

		params.put(TABLE_ID_TEMPLATE_PARAM, ((LyJQTable) _parentTable.getInternalView()).getId());
		params.put(HEADER_ID_PARAM, getId());
		params.put(HEADER_INCREASING_PARAM, increasing);

		String	newDiv = templateManager.mergeTemplate(HEADER_INCREASING_TEMPLATE, params);

		_viewManager.addToUpdateScript(newDiv);
		newDiv = null;
		params = null;
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalHeader#setFixed(boolean)
	 */
	public void setFixed(boolean fixed)
	{
	}

	/**
	 * set border width ajax response
	 */
	public void setBorderWidth(int borderWidth)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();

		params.put(TABLE_ID_TEMPLATE_PARAM, ((LyJQTable) _parentTable.getInternalView()).getId());
		params.put(HEADER_ID_PARAM, getId());
		params.put(HEADER_BORDER_WIDTH, borderWidth);

		String	newDiv = templateManager.mergeTemplate(HEADER_BORDER_WIDTH_TEMPLATE, params);

		_viewManager.addToUpdateScript(newDiv);
		newDiv = null;
		params = null;
	}

	/**
	 * set border color ajax response
	 */
	public void setBorderColor(String borderColor)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();
		
		params.put(TABLE_ID_TEMPLATE_PARAM, ((LyJQTable) _parentTable.getInternalView()).getId());
		params.put(HEADER_ID_PARAM, getId());
		params.put(HEADER_BODER_COLOR, borderColor);

		String	newDiv = templateManager.mergeTemplate(HEADER_BORDER_COLOR_TEMPLATE, params);

		_viewManager.addToUpdateScript(newDiv);
		params = null;
		newDiv = null;
	}

	/** get current header index **/
	public int getHeaderIndex()
	{
		// Retrieve top level headers
		LyHeaders	headers = (LyHeaders) _header.getParent();
		LyHeader	header;

		headers = _parentTable.getHeaders();

		int i, n = headers.getHeaderCount();

		for (i = 0; i < n; i++)
		{
			header = headers.getHeader(i);
			
			if (header == _header)
				return i;
		}

		return -1;
	}

	/**
	 * get row span
	 **/
	public int getRowSpan()
	{
		// Retrieve top level headers
		LyHeaders	headers = (LyHeaders) _header.getParent();
		LyHeaders	totalHeaders = _parentTable.getHeaders();
		int			i = ((LyJQHeaders) totalHeaders.getInternalView()).getChildsDepth();
		
		while (headers != totalHeaders)
		{
			i--;
			headers = (LyHeaders) headers.getParent();
		}

		return i;
	}

	/**
	 * get colspan
	 * @return colspan
	 */
	public int getColSpan()
	{
		return 1;
	}

	public void writeHtml(LyWebPageWriter pw)
	{
	}

	@Override
	public String getReloadScript()
	{
		return null;
	}

	/**
	 * Sets attribute columnWidth to the given value.
	 * @param columnWidth New value of columnWidth.
	 */
	public void setColumnWidth(double columnWidth)
	{
		LyJQViewManager	viewManager = (LyJQViewManager)getViewManager();
		LyJQTable		jqTable = (LyJQTable)_parentTable.getInternalView();
		
		// Optimization : reload table only if sharing mode is activated
		if (viewManager.isShared() || viewManager.isSharing())
			jqTable.reloadTable();
	}
}