/*
 * LyWeb2Icon.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.jquery.component.icon;

import java.util.HashMap;
import java.util.Map;

import leon.misc.LyString;
import leon.view.LyIcon;
import leon.view.LyIcon.Border;
import leon.view.graphic.LyGraphicalIcon;
import leon.view.web.LyWebPageWriter;
import leon.view.web.jquery.LyJQConstants;
import leon.view.web.jquery.common.TemplateUtils;
import leon.view.web.jquery.component.LyJQInternalView;
import leon.view.web.jquery.controller.LyTemplatesUtilsBean;

/**
 * Type : leon.view.web.web2.LyWeb2Icon The peer component of LyIcon for the web2 viewer.
 * @version $Revision: 17 $
 * @author ychaari, last modified by $Author: ychaari $
 */
public class LyJQIcon extends LyJQInternalView	implements
												LyGraphicalIcon
{
	/** The icon generic component. */
	protected LyIcon			_icon;

	/**
	 * Class constructor of an Icon object for the web2 target.
	 * @param icon the generic Icon defined for every type of view targets
	 */
	public LyJQIcon(LyIcon icon)
	{
		super(icon);
		_icon = icon;

	}

	/**
	 * Check if icon element has border
	 * @param lyIcon
	 * @return
	 */
	public boolean hasBorder(LyIcon lyIcon)
	{
		Border border = lyIcon.getBorder();
		return (border != null && border != LyIcon.Border.NONE);
	}

	/**
	 * Returns the path associated with the given image.
	 * @param image the image that must be displayed.
	 * @return the path associated with the given image.
	 */
	public String getImagePath(String image)
	{
		String path;
		String docDir;
		String docRoot = expandUrl(getEnv(LyJQConstants.LY_APP_DOC));

		image = expandUrl(image);
		docDir = expandUrl(getEnv(LyJQConstants.LY_APP_DOC_DIR));

		if (image.startsWith(docDir))
		{
			image = image.substring(docDir.length());
			image = getEnv(LyJQConstants.LY_APP_DOC) + image;
		}
		else
		{
			docDir = expandUrl(getEnv(LyJQConstants.LY_LEON_DOC_DIR));
			if (image.startsWith(docDir))
			{
				image = image.substring(docDir.length());
				image = getEnv(LyJQConstants.LY_LEON_DOC) + image;
			}
		}

		if (image.startsWith(docRoot))
			path = expandUrl(image);
		else if (image.startsWith("file"))
			path = LyString.encodeFileURL(image);
		else if (image.startsWith("http") || image.startsWith("ftp"))
			path = image;
		else
			path = getImageUrl(image);

		return path;
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalIcon#setImage(java.lang.String)
	 */
	public void setImage(String imageSrc)
	{
		_viewManager.addToReloadComponents(_icon);
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter arg0)
	{
		// TODO Auto-generated method stub

	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.web.jquery.component.LyJQInternalView#getReloadScript()
	 */
	@Override
	public String getReloadScript()
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();

		Map<String, Object> params = new HashMap<String, Object>();
		params.put("icon", _icon);
		String newHTML = templateManager.mergeTemplate("icon/lyIconTemplate.vm", params);
		newHTML = TemplateUtils.getFormattedJs(newHTML);
		params = new HashMap<String, Object>();
		params.put("content", newHTML);
		params.put("componentId", getId());
		newHTML = templateManager.mergeTemplate("ajax/replaceWith.vm", params);
		params = null;
		return newHTML;
	}

	public boolean isIcon()
	{
		return true;
	}
}