/* 
 * Lyria-W4 - afficheur_web_2
 * LyWeb2Item.java
 * Copyright (c) 2000-2009, Lyria-W4
 * All rights reserved.
 */
package leon.view.web.jquery.component.item;

import java.util.*;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.setinterface.*;
import leon.view.web.*;
import leon.view.web.jquery.*;
import leon.view.web.jquery.common.*;
import leon.view.web.jquery.component.*;
import leon.view.web.jquery.controller.*;

/**
 * LyWeb2Item
 * @version $Revision$
 * @author slachiheb, last modified by $Author$
 */
public class LyJQItem extends LyJQInternalView	implements
												LyGraphicalItem,
												LySetVisibleInterface
{
	protected static String		ITEM_ID_PARAM			= "componentId";

	protected static String		ITEM_PARAM				= "lyItem";

	protected static String		ITEM_CONTENT_PARAM		= "content";

	protected static String		MENUBAR_PARAM			= "lyMenuBar";

	protected static String		MENUBAR_ITEM_TEMPLATE	= "menuBar/item/lyItemTemplate.vm";

	protected static String		MENU_BAR_SCRIPT_TEMPLATE	= "menuBar/lyMenuBarScriptTemplate.vm";

	/* set visible template */
	protected static String		SET_VISIBLE_TEMPLATE	= "ajax/menuBar/setVisible.vm";

	public LyItem				_item;

	protected String			_dialogSuffix			= "...";

	/** Visibility of the item. */
	protected boolean			_visible;

	/**
	 * @param viewComponent
	 */
	public LyJQItem(LyItem item)
	{
		super(item);
		_item = item;
		_visible = true;
	}

	/**
	 * Get dialog
	 */
	public String getLabel()
	{
		// Add "..." if dialog item
		String itemLabel = translate(_item.getLabel());
		if (_item.isDialog() && !itemLabel.endsWith(_dialogSuffix))
			return itemLabel + _dialogSuffix;
		return itemLabel;

	}

	/**
	 * getImageWidth return width of the image
	 */
	public int getImageWidth(String key)
	{
		int[] img_sizes = getImageSize(key);
		if (img_sizes != null)
			return img_sizes[0];
		return 16;
	}

	/**
	 * getImageHeight return height of the image
	 */
	public int getImageHeight(String key)
	{
		int[] img_sizes = getImageSize(key);
		if (img_sizes != null && img_sizes.length > 1)
			return img_sizes[1];
		return 16;
	}

	/**
	 * method used by the lymenu velocity template
	 * @return true
	 */
	public boolean isItem()
	{
		return true;
	}

	/**
	 * method used by the lymenu velocity template
	 * @return false
	 */
	public boolean isMenu()
	{
		return false;
	}

	/**
	 * Check if the component is a seperator.
	 * @return boolean
	 */
	public boolean isSep()
	{
		if ((_item == null) || (_item.getId() == LyComponent.SEP_ID))
			return true;
		return false;

	}

	/**
	 * @see leon.view.setinterface.LySetVisibleInterface#setVisible(boolean)
	 */
	public void setVisible(boolean status)
	{
		if (status != _visible)
		{
			_visible = status;
			Map<String, Object> params = new HashMap<String, Object>();
			params.put(ITEM_ID_PARAM, getId());
			params.put("visible", status);
			params.put("isSep", isSep());
			LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
			_viewManager.addToUpdateScript(templateManager.mergeTemplate(SET_VISIBLE_TEMPLATE,
					params));
			params = null;
		}
	}

	/**
	 * @see leon.view.graphic.LyGraphicalItem#setLabel(java.lang.String)
	 */
	public void setLabel(String label)
	{
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(ITEM_PARAM, _item);
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		String itemScript = templateManager.mergeTemplate(MENUBAR_ITEM_TEMPLATE, params);
		itemScript = TemplateUtils.getFormattedJs(itemScript);
		params = new HashMap<String, Object>();
		params.put(ITEM_ID_PARAM, getId());
		params.put(ITEM_CONTENT_PARAM, itemScript);
		_viewManager.addToUpdateScript(templateManager.mergeTemplate(
				LyJQConstants.REPLACE_SCRIPT_TEMPLATE, params));
		params = null;

	}

	/**
	 * @see leon.view.graphic.LyGraphicalItem#setEnabled(boolean)
	 */
	public void setEnabled(boolean enabled)
	{
		// Update item content
		Map<String, Object>		params = new HashMap<String, Object>();

		params.put(ITEM_PARAM, _item);
		
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		String					itemScript = templateManager.mergeTemplate(MENUBAR_ITEM_TEMPLATE, params);
		
		itemScript = TemplateUtils.getFormattedJs(itemScript);
		params = new HashMap<String, Object>();
		params.put(ITEM_ID_PARAM, getId());
		params.put(ITEM_CONTENT_PARAM, itemScript);
		_viewManager.addToUpdateScript(templateManager.mergeTemplate(LyJQConstants.REPLACE_SCRIPT_TEMPLATE, params));

		// Call createMenu() JS function on the menubar in order to set the hover styles
		if (enabled)
		{
			LyComponent parentComponent = _item.getParent();
			
			while (!(parentComponent instanceof LyMenuBar))
			{
				parentComponent = parentComponent.getParent();
			}
			
			if (parentComponent instanceof LyMenuBar)
			{
				params = new HashMap<String, Object>();
				params.put(MENUBAR_PARAM, parentComponent);
				
				String menuScript = templateManager.mergeTemplate(MENU_BAR_SCRIPT_TEMPLATE, params);
				
				_viewManager.addToUpdateScript(menuScript);
			}
		}
		
		params = null;
	}

	/**
	 * @see leon.view.graphic.LyGraphicalItem#setChecked(boolean)
	 */
	public void setChecked(boolean checked)
	{
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(ITEM_PARAM, _item);
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		String itemScript = templateManager.mergeTemplate(MENUBAR_ITEM_TEMPLATE, params);
		itemScript = TemplateUtils.getFormattedJs(itemScript);
		params = new HashMap<String, Object>();
		params.put(ITEM_ID_PARAM, getId());
		params.put(ITEM_CONTENT_PARAM, itemScript);
		_viewManager.addToUpdateScript(templateManager.mergeTemplate(
				LyJQConstants.REPLACE_SCRIPT_TEMPLATE, params));
		params = null;
	}

	/**
	 * @see leon.view.graphic.LyGraphicalItem#setImage(java.lang.String)
	 */
	public void setImage(String image)
	{
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(ITEM_PARAM, _item);
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		String itemScript = templateManager.mergeTemplate(MENUBAR_ITEM_TEMPLATE, params);
		itemScript = TemplateUtils.getFormattedJs(itemScript);
		params = new HashMap<String, Object>();
		params.put(ITEM_ID_PARAM, getId());
		params.put(ITEM_CONTENT_PARAM, itemScript);
		_viewManager.addToUpdateScript(templateManager.mergeTemplate(
				LyJQConstants.REPLACE_SCRIPT_TEMPLATE, params));
		params = null;
	}

	/**
	 * @return _visible
	 */
	public boolean isVisible()
	{
		return _visible;
	}

	/**
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
		//
	}

	/**
	 * @see leon.view.web.jquery.component.LyJQInternalView#getReloadScript()
	 */
	@Override
	public String getReloadScript()
	{
		return null;
	}

}
