package leon.view.web.jquery.component.panel.notice;

import leon.view.LyConstraint;
import leon.view.LyNotice;
import leon.view.graphic.LyGraphicalNotice;
import leon.view.web.jquery.component.panel.LyJQPanel;

/**
 * This class implements the Notice object (as described in the dtd) for web viewer.
 * <p>
 * <b>NOTICE: Information Panel</b><br>
 * An information panel allows displaying an insert having an informative purpose. It is possible to assign an icon, a
 * label and a progress bar to an information panel.
 * </p>
 * @author ous
 * @see leon.view.LyNotice
 */
public class LyJQNotice extends LyJQPanel	implements
											LyGraphicalNotice
{

	/** The notice generic component. */
	protected LyNotice	_notice;

	/**
	 * Class constructor of a Notice object for the web2 target.
	 * @param notice the generic Notice defined for every type of view targets.
	 */
	public LyJQNotice(LyNotice notice)
	{
		super(notice);
		_notice = notice;
	}

	/**
	 * @return notice height
	 */
	public String getHeight()
	{
		LyConstraint constraint = _notice.getConstraint();

		if ((constraint != null)
				&& ((constraint.getFill() == LyConstraint.Fill.BOTH) || (constraint.getFill() == LyConstraint.Fill.VERTICAL)))
		{
			return " height='100%'";
		}
		return null;
	}

	/**
	 * get notice width
	 */
	public String getWidth()
	{
		LyConstraint constraint = _notice.getConstraint();

		if ((constraint != null)
				&& ((constraint.getFill() == LyConstraint.Fill.BOTH) || (constraint.getFill() == LyConstraint.Fill.HORIZONTAL)))
		{
			return " width='100%'";
		}
		return null;
	}

	public String getFont()
	{
		return getFontString("LY_DEFAULT_FONT");
	}

	public boolean isNotice()
	{
		return true;
	}
}