package leon.view.web.jquery.component.row;

import java.util.*;

import leon.misc.LyString;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;
import leon.view.web.jquery.*;
import leon.view.web.jquery.common.*;
import leon.view.web.jquery.component.*;
import leon.view.web.jquery.component.table.*;
import leon.view.web.jquery.controller.*;

/**
 * Ly table Row jQuery internal view
 * @author bahaSmichet
 */
public class LyJQRow extends LyJQInternalView	implements
												LyGraphicalRow
{
	protected final String		ROW_IMAGE_TEMPLATE				= "table/actions/rowImage.vm";

	protected final String		TABLE_ID_TEMPLATE_PARAM			= "tableId";

	protected final String		ROW_ID_TEMPLATE_PARAM			= "rowId";

	protected final String		IMAGE_PARAM						= "image";

	protected LyTable _table;

	/** The row generic component. */
	protected LyRow				_row;

	/** Index of row in the table. */
	protected int				_rowIndex;

	/** Alignment of the row. */
	protected LyHeader.Align	_headerAlign;

	/**
	 * Class constructor of a Row object for the Struts target. Calls the super implementation
	 * @param row the generic Row defined for every type of view targets.
	 */
	public LyJQRow(LyRow row)
	{
		super(row);

		_row = row;
		_table = ((LyRows) _row.getParent()).getParentTable();
		_headerAlign = LyHeader.Align.LEFT;
		_rowIndex = -1;
	}

	/**
	 * @see leon.view.graphic.LyGraphicalRow#setOpen(boolean)
	 */
	public void setOpen(boolean open)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();

		params.put("open", open);
		params.put("row", _row);

		String	openRowScript = templateManager.mergeTemplate("table/actions/setOpenRow.vm", params);

		_viewManager.addToUpdateScript(openRowScript);
		params = null;
		openRowScript = null;
	}

	/**
	 * @see leon.view.graphic.LyGraphicalRow#setImage(java.lang.String)
	 */
	public void setImage(String image)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();

		params.put(TABLE_ID_TEMPLATE_PARAM, ((LyJQTable) _table.getInternalView()).getId());
		params.put(ROW_ID_TEMPLATE_PARAM, getId());
		params.put(IMAGE_PARAM, getImageUrl(image));

		String	updateImageTemplate = templateManager.mergeTemplate(ROW_IMAGE_TEMPLATE, params);

		_viewManager.addToUpdateScript(updateImageTemplate);
		params = null;
	}

	/**
	 * @see leon.view.graphic.LyGraphicalRow#setLabel(java.lang.String)
	 */
	public void setLabel(String label)
	{
	}

	/**
	 * @see leon.view.setinterface.LyGraphicalRow#setIndex(java.lang.String)
	 */
	public void setIndex(String index)
	{
	}

	/**
	 * @see leon.view.graphic.LyGraphicalRow#setTooltip(java.lang.String)
	 */
	public void setTooltip(String tooltip)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();

        tooltip = translate(tooltip);
        tooltip = LyString.getJavascriptString(tooltip);
        tooltip = LyString.replace(tooltip, "\\n", "<br/>");

        params.put(TABLE_ID_TEMPLATE_PARAM, ((LyJQTable) _table.getInternalView()).getId());
		params.put("cellId", "img_" + getId());
		params.put("cellTooltip", tooltip);

		String	updateImageTemplate = templateManager.mergeTemplate("table/actions/cellTooltip.vm", params);

		_viewManager.addToUpdateScript(updateImageTemplate);
		params = null;
	}

	@Override
	public String getJavascriptStrId()
	{
	    String id = getViewComponent().getId();

        if (id == null)
            id = getViewComponent().getInternalId();

        final char[]   jsSearches = { '\\', '<', '>', '"','\'',  '\n' };
        final String[] jsReplaces = { "\\\\", "&lt;", "&gt;", "\\\"", "&#39;", "\\n"};

        id = LyString.encodeAmpersand(id);
        return LyString.replace(id, jsSearches, jsReplaces);
	}

	/**
	 * @see leon.view.graphic.LyGraphicalRow#setHidden(boolean)
	 */
	public void setHidden(boolean hidden)
	{
	}

	/**
	 * @see leon.view.graphic.LyGraphicalRow#setBorderHeight(int)
	 */
	public void setBorderHeight(int borderHeight)
	{
	}

	/**
	 * @see leon.view.graphic.LyGraphicalRow#setBorderColor(java.lang.String)
	 */
	public void setBorderColor(String borderColor)
	{
	}

	/**
	 * @see leon.view.graphic.LyGraphicalRow#addComponent(leon.view.LyComponent, int)
	 */
	public void addComponent(LyComponent component, int pos)
	{
		if (_viewManager.getBrowser() == LyWebViewManager.BROWSER_MSIE
				&& _viewManager.getBrowserVersion() <= 7.99)
		{
			_viewManager.addToReloadComponents(_table);
		}
		else
		{

			LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
			Map<String, Object>		params = new HashMap<String, Object>();

			if (component instanceof LyFormField)
				params.put("typeComp", "formField");
			else if (component instanceof LyToolBar)
				params.put("typeComp", "toolBar");
			else if (component instanceof LyCell)
				params.put("typeComp", "cell");

			params.put("tableId", ((LyJQTable) _table.getInternalView()).getId());
			params.put("component", component);
			params.put("row", _row);
			params.put("pos", pos);

			String	editableCell = templateManager.mergeTemplate("table/actions/editCell.vm", params);

			editableCell = TemplateUtils.getFormattedJs(editableCell);
			params = new HashMap<String, Object>();
			params.put("content", editableCell);
			params.put("tableId", ((LyJQTable) _table.getInternalView()).getId());
			params.put("rowId", getId());
			params.put("pos", pos);
			String newCell = templateManager.mergeTemplate("table/actions/replaceCell.vm", params);

			_viewManager.addToUpdateScript(newCell);
			newCell = null;
			params = null;
		}
	}

	/**
	 * @see leon.view.graphic.LyGraphicalRow#removeComponent(leon.view.LyComponent)
	 */
	public void removeComponent(LyComponent component)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();

		params.put("tableId", ((LyJQTable) _table.getInternalView()).getId());
		params.put("component", component);
		params.put("row", _row);

		String	editableCell = templateManager.mergeTemplate("table/actions/editCell.vm", params);

		_viewManager.addToUpdateScript(editableCell);
		params = null;
		editableCell = null;
	}

	/**
	 * Gets the depth of current row in the table.
	 * @return the depth of current row in the table if hierarchical, 0 otherwise.
	 */
	public int getDepthInTable()
	{
		LyRows	rows = (LyRows) _row.getParent();
		LyTable	table = rows.getParentTable();
		LyRows	rootRows = table.getRows();
		LyRow	row;
		int		depth = 0;

		while (rows != rootRows)
		{
			depth++;
			row = (LyRow) rows.getParent();
			rows = (LyRows) row.getParent();
		}

		return depth;
	}

	/**
	 * get Row Span for a given cell
	 * @return
	 */
	public int getRowSpan()
	{
		int	rowSpan = 1;
		int	i, n = _row.getComponentCount(), cellNbChild;

		for (i = 0; i < n; i++)
		{
			LyComponent	comp = _row.getComponent(i);

			if (comp instanceof LyCell)
			{
				cellNbChild = getNbChild((LyCell) comp);

				if (cellNbChild > rowSpan)
					rowSpan = cellNbChild;
			}
		}

		return rowSpan;
	}

	public int getNbChild(LyCell cell)
	{
		if (cell.getCellCount() == 0)
			return 1;
		else
		{
			int		i, n = cell.getCellCount(), total = 0;

			for (i = 0; i < n; i++)
				total = total + getNbChild(cell.getCell(i));

			return total;
		}
	}

	/**
	 * get Row Index for a given cell
	 * @return
	 */
	public int getRowIndex()
	{
		int			i = -1;
		LyTable		table = ((LyRows) _row.getParent()).getParentTable();
		List<LyRow>	indexedList = getIndexedRowList(table.getRows());

		for (LyRow row_i : indexedList)
		{
			i++;

			if (row_i == _row)
				return i;
		}

		return -1;
	}

	public List<LyRow> getIndexedRowList(LyRows rows)
	{
		List<LyRow>	rowList = new ArrayList<LyRow>();
		LyRow		row;
		int			i, n = rows.getRowCount();

		for (i = 0; i < n; i++)
		{
			row = rows.getRow(i);
			rowList.add(row);

			if ((row.getRows() != null) && row.isOpen())
				rowList.addAll(getIndexedRowList(row.getRows()));
		}

		return rowList;
	}

	/**
     * Indicates if current row contains at least one component (field,
     * editable cell or toolbar).
     *
     * @param testToolbar indicates if we must test the presence of a toolbar
     * @return true if contains at least one component, false otherwise
     */
    public boolean hasComponents(boolean testToolbar)
    {
        int         i, n = _row.getComponentCount();
        LyComponent comp;

        for (i = 0; i < n; i++)
        {
            comp = _row.getComponent(i);

            if (((comp instanceof LyCell) && ((LyCell)comp).isEditable())
                    || (testToolbar && (comp instanceof LyToolBar))
                    || (comp instanceof LyFormField))
                return true;
        }

        if ((n == 0) && (_row.getRows() != null))
        {
        	LyRows	rows = _row.getRows();

    		if (rows.getRow(0) != null)
    		{
    			LyRow	row = rows.getRow(0);
    			LyJQRow	webRow = (LyJQRow)row.getInternalView();

    			if ((webRow != null) && webRow.hasComponents(false))
    				return true;
    		}
        }

        return false;
    }

    public String getJSId()
    {
    	String	id = getJSWithoutTranslateString(_row.getId());

    	id = LyString.replace(id, "&#39;", "\\&#39;");

    	return id;
    }

	public void writeHtml(LyWebPageWriter pw)
	{
	}

	@Override
	public String getReloadScript()
	{
		return null;
	}

	public void setColor(String color)
	{
		LyJQViewManager viewManager = (LyJQViewManager) getViewManager();
		LyTemplatesUtilsBean templateManager = viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("row", _row);
		params.put("color", color);

		String setColorScript = templateManager.mergeTemplate("table/actions/setRowColor.vm", params);

		viewManager.addToUpdateScript(setColorScript);
		params = null;
		setColorScript = null;
	}
}
