package leon.view.web.jquery.component.splitter;

import java.util.*;

import leon.view.*;
import leon.view.LySplitter.ActivePart;
import leon.view.graphic.*;
import leon.view.web.*;
import leon.view.web.jquery.*;
import leon.view.web.jquery.controller.*;

/**
 * Type : leon.view.web.web2.splitter.LyWeb2Splitter The peer component of LySplitter for the web2 viewer.
 * @version $Revision: -1M $
 * @author bsmichet, last modified by $Author: (local) $
 */
public class LyJQSplitter extends LyJQContainer	implements LyGraphicalSplitter
{

	public static final String	__VERSION					= "$Id: LyJQSplitter.java -1M 2010-12-05 14:01:48Z (local) $";

	/** The splitter generic component. */
	protected LySplitter		_splitter;

	protected static String		SPLITTER_ID_PARAM			= "splitterId";

	protected static String		SPLITTER_STYLE_PARAM		= "splitterStyle";

	protected static String		FIRST_SIZE_PARAM			= "firstSize";

	protected static String		SECOND_SIZE_PARAM			= "secondSize";

	/** Maximum height of the components displayed in a splitter. */
	public static int			MAXIMUM_COMPONENT_HEIGHT	= 450;

	public LyJQSplitter(LySplitter splitter)
	{
		super(splitter);
		_splitter = splitter;
	}

	/**
	 * getSplitterPanes
	 */
	public List<LyPanel> getSplitterPanes()
	{
		List<LyPanel>	panels = new ArrayList<LyPanel>();
		LyPanel			firstPanel = _splitter.getPanel(0);
		LyPanel			secondPanel = _splitter.getPanel(1);
		if(_viewManager.isRtlMode() && _splitter.getStyle().equals(LySplitter.Style.HORIZONTAL))
		{
			// if rtl mode and horizontal style inverse panels
			if (secondPanel != null)
			{
				panels.add(secondPanel);
				panels.add(firstPanel);
				
			}else if (firstPanel != null)
				panels.add(firstPanel);
		} else
		{
			if (firstPanel != null)
				panels.add(firstPanel);

			if (secondPanel != null)
				panels.add(secondPanel);	
		}
		return panels;
	}

	public void setSizes(String sizes)
	{
		if (_splitter.getEnvironment().getEnv("LY_TINY_SPLITTER") != null)
		{
			Map<String, Object>	params = new HashMap<String, Object>();

			params.put(SPLITTER_ID_PARAM, getId());
			params.put(SPLITTER_STYLE_PARAM, _splitter.getStyle());
			
			int	i = sizes.indexOf(" ");
			if(_viewManager.isRtlMode() && _splitter.getStyle().equals(LySplitter.Style.HORIZONTAL))
			{
				// if rtl mode and horizontal style inverse sizes
				params.put(SECOND_SIZE_PARAM, sizes.substring(0, i));
				params.put(FIRST_SIZE_PARAM, sizes.substring(i + 1));
			}else
			{
				params.put(FIRST_SIZE_PARAM, sizes.substring(0, i));
				params.put(SECOND_SIZE_PARAM, sizes.substring(i + 1));	
			}


			LyTemplatesUtilsBean	_templateManager = _viewManager.getVeloTemplatesBean();
			String					setSizesScript = _templateManager.mergeTemplate("ajax/splitter/setSizes.vm", params);

			_viewManager.addToUpdateScript(setSizesScript);
			params = null;
		}
	}

	/**
	 * get first panel size
	 * @param splitter
	 * @return size
	 */
	public String getSecondSize()
	{
		if (_splitter.getSizes() != null)
		{
			String	sizes = _splitter.getSizes();
			int		i = sizes.indexOf(" ");
			if(_viewManager.isRtlMode() && _splitter.getStyle().equals(LySplitter.Style.HORIZONTAL))
				return sizes.substring(0, i);
			return sizes.substring(i + 1);
		}
		
		return "50";
	}

	/**
	 * get second panel size
	 * @param splitter
	 * @return size
	 */
	public String getFirstSize()
	{
		if (_splitter.getSizes() != null)
		{
			String	sizes = _splitter.getSizes();
			int		i = sizes.indexOf(" ");
			if(_viewManager.isRtlMode() && _splitter.getStyle().equals(LySplitter.Style.HORIZONTAL))
				return sizes.substring(i + 1);
			return sizes.substring(0, i);
		}
		return "50";
	}

	public void writeHtml(LyWebPageWriter pw)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();

		params.put("splitter", _splitter);

		String	result = templateManager.mergeTemplate("splitter/lySplitterTemplate.vm", params);

		pw.append(result);
	}


	@Override
	public String getReloadScript()
	{
		return null;
	}

	public void setSelected(boolean status)
	{
		// TODO Auto-generated method stub
		
	}

	public void setActivePart(ActivePart activePart)
	{
		// TODO Auto-generated method stub
		
	}
}