package leon.view.web.jquery.component.tab;

import java.util.*;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;
import leon.view.web.jquery.component.*;
import leon.view.web.jquery.controller.*;

/**
 * Type : leon.view.web.web2.tab.LyWeb2Tab The peer component of LyTab for the web2 viewer.
 * @version $Revision: -1M $
 * @author bsmichet, last modified by $Author: (local) $
 */
public class LyJQTab extends LyJQInternalView implements LyGraphicalTab
{
	protected final String		TAB_COLOR_TEMPLATE				= "tabs/setTabColor.vm";
	
	protected final String		TABS_TEMPLATE_PARAM				= "tabs";

	protected final String		COLOR_TEMPLATE_PARAM			= "color";

	protected final String		INDEX_TEMPLATE_PARAM			= "index";

	public static final String	__VERSION	= "$Id: LyJQTab.java -1M 2010-12-05 14:01:33Z (local) $";

	/** The tab generic component. */
	protected LyTab				_tab;

	/** The calculated ID. */
	protected String			_id;

	/**
	 * Class constructor of a Tab object.
	 * @param tab the generic Tab defined for every type of view targets
	 */
	public LyJQTab(LyTab tab)
	{
		super(tab);
		_tab = tab;
	}

	public void writeHtml(LyWebPageWriter pw)
	{
	}

	public void setEnabled(boolean enabled)
	{
        LyTabs		tabs = _tab.getParentTabs();
        
        _viewManager.addToReloadComponents(tabs);
	}

	public void setColor(String color)
	{
		String			 		tabColor = ((color == null) ? "" : getHexColor(color));
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();
		LyTabs					tabs = _tab.getParentTabs();
		int						i, n = tabs.getTabCount();
		
		for (i = 0; i < n; i++)
		{
			if (tabs.getTab(i) == _tab)
			{
				params.put(TABS_TEMPLATE_PARAM, tabs);
				params.put(COLOR_TEMPLATE_PARAM, tabColor);
				params.put(INDEX_TEMPLATE_PARAM, i);
				
				String updateImageTemplate = templateManager.mergeTemplate(TAB_COLOR_TEMPLATE, params);
				
				_viewManager.addToUpdateScript(updateImageTemplate);
				params = null;
				break;
			}
		}
	}
	
	/**
	 * Indicates whether focus must be requested when tab is displayed.
	 */
	public boolean requestFocus()
	{
		// If tab contains a form, give focus to first input in the form
		Vector<LyForm>	forms = new Vector<LyForm>();
		boolean			hasForm = false;
		LyArea			parentArea;
		
		_tab.getComponents(LyForm.class, forms);
		
		// Check if form is not a sub-form of a tool
		for (LyForm form : forms)
		{
			parentArea = form.getParentArea();
			
			if ((parentArea != null) && !(parentArea.getParent() instanceof LyTool))
			{
				hasForm = true;
				break;
			}
		}
		
		return hasForm;
	}

	@Override
	public String getReloadScript()
	{
		return null;
	}
}