package leon.view.web.jquery.component.table;

import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.setinterface.*;
import leon.view.web.*;
import leon.view.web.jquery.*;
import leon.view.web.jquery.component.footers.*;
import leon.view.web.jquery.component.headers.*;
import leon.view.web.jquery.component.panel.*;
import leon.view.web.jquery.component.row.*;
import leon.view.web.jquery.component.rows.*;
import leon.view.web.jquery.controller.*;

/**
 * <b>LyTable</b> JQUERY internal view implementation
 * @author bahaSmichet
 */
public class LyJQTable extends LyJQPanel implements
										LyGraphicalTable,
										LySetVisibleInterface,
										LySetTableInterface,
										LySetIdleInterface
{
	/** The table generic component. */
	protected LyTable		_table;

	protected LyJQFooters	_footers;

	protected LyJQRows		_rows;

	protected LyJQHeaders	_headers;

    protected LyRow			_activeRow;

	protected static String	TABLE_ID_PARAM		= "tableId";

	protected static String	ROW_PARAM			= "row";

	protected static String	SELECT_PARAM		= "select";

	protected static String	ROW_IDX_PARAM		= "rowIdx";

	protected static String	UPDATE_VIEW_PARAM	= "updateView";

	protected static String	SELECT_ROW_TEMPLATE	= "table/actions/tableSelectRow.vm";

	protected static String	SHOW_ROW_TEMPLATE	= "table/actions/tableShowRow.vm";

	public LyJQTable(LyTable table)
	{
		super(table);
		_table = table;
		_activeRow = null;
		setJQRows();
		setJQHeaders();
		setJQFooters();
	}

	/**
	 * get Lytable component
	 */
	public LyTable getLyTable()
	{
		return _table;
	}
	
	@Override
	public void writeHtml(LyWebPageWriter pw)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();
		
		params.put("table", _table);
		
		String	result = templateManager.mergeTemplate("table/lyTableTemplate.vm", params);
		
		pw.append(result);
	}

	/**
	 * get headers imbrication depth
	 * @return depth
	 */
	public int getHeadersDepth()
	{
		LyHeaders headers = _table.getHeaders();

		if (headers == null)
			return 0;
		else
		{
			LyJQHeaders internal = (LyJQHeaders) headers.getInternalView();

			return internal.getDepth();
		}
	}

	/**
	 * @see leon.view.graphic.LyGraphicalTable#setEnabled(boolean)
	 */
	public void setEnabled(boolean enabled)
	{
	}

	/**
	 * @see leon.view.graphic.LyGraphicalTable#setHeaders(leon.view.LyHeaders)
	 */
	public void setHeaders(LyHeaders headers)
	{
		 reloadTable();
	}

	/**
	 * @see leon.view.graphic.LyGraphicalTable#setFooters(leon.view.LyFooters)
	 */
	public void setFooters(LyFooters footers)
	{
		 reloadTable();
	}

	/**
	 * get headers tree
	 * @return headers tree
	 */
	public List<List<LyComponent>> getHeadersArray()
	{
		LyHeaders rootHeaders = _table.getHeaders();
		List<List<LyComponent>> headMap = null;
		List<LyComponent> tmpHeadArray = new ArrayList<LyComponent>();

		if (rootHeaders != null)
		{
			headMap = new ArrayList<List<LyComponent>>();

			for (int i = 0; i < rootHeaders.getChildCount(); i++)
			{
				tmpHeadArray.add(rootHeaders.getChild(i));
			}

			headMap.add(tmpHeadArray);

			int i = 1;
			boolean eot = false;
			while (!eot)
			{
				tmpHeadArray = new ArrayList<LyComponent>();

				for (LyComponent head : headMap.get(i - 1))
				{
					if (head instanceof LyHeaders)
					{
						for (int j = 0; j < head.getChildCount(); j++)
						{
							tmpHeadArray.add(head.getChild(j));
						}
					}
				}

				if (tmpHeadArray.size() > 0)
				{
					headMap.add(tmpHeadArray);
					i++;
				}
				else
					eot = true;
			}
		}
		tmpHeadArray = null;
		return headMap;
	}

	/**
	 * @return table column number
	 */
	public int getTableNbColumns()
	{
        LyHeaders headers = _table.getHeaders();
        
        if (headers != null)
        	return headers.getHeaderCount();

        int		nb = 0;
		int		componentCount;
		LyRows	rows = _table.getRows();
		int		rowsCount = (rows == null ? 0 : rows.getRowCount());
		LyRow	row;
		
		for (int i = 0; i < rowsCount; i++)
		{
			row = rows.getRow(i);
			componentCount = row.getComponentCount();

			if (componentCount > nb)
				nb = componentCount;
			
			if (row.getRows() != null)
			{
				if (nb < 1)
					nb = 1;
				
				if (((LyJQRows) row.getRows().getInternalView()).getRowsNbColumns() > nb)
					nb = ((LyJQRows) row.getRows().getInternalView()).getRowsNbColumns();
			}
		}

		return nb;
	}

	/**
	 * @see leon.view.setinterface.LySetIdleInterface#setIdle(boolean)
	 */
	public void setIdle(boolean status)
	{
        if (!status)
            _activeRow = null;
	}

	/**
	 * @see leon.view.setinterface.LySetTableInterface#addRow(leon.view.LyRow)
	 */
	public void addRow(LyRow row)
	{
		reloadTable();
	}

	/**
	 * @see leon.view.setinterface.LySetTableInterface#addRow(leon.view.LyRow, int)
	 */
	public void addRow(LyRow row, int idx)
	{

		reloadTable();

	}

	/**
	 * @see leon.view.setinterface.LySetTableInterface#removeRow(leon.view.LyRow)
	 */
	public void removeRow(LyRow row)
	{
		reloadTable();

	}

	/**
	 * @see leon.view.setinterface.LySetTableInterface#removeRow(int)
	 */
	public void removeRow(int idx)
	{
		reloadTable();

	}

	/**
	 * @see leon.view.setinterface.LySetTableInterface#swapRows(int, int)
	 */
	public void swapRows(int idx1, int idx2)
	{
		reloadTable();

	}

	/**
	 * @see leon.view.setinterface.LySetTableInterface#selectRow(leon.view.LyRow, boolean)
	 */
	public void selectRow(LyRow row, boolean select)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(TABLE_ID_PARAM, getId());
		params.put(ROW_PARAM, row);
		params.put(SELECT_PARAM, select);

		String newDiv = templateManager.mergeTemplate(SELECT_ROW_TEMPLATE, params);

		_viewManager.addToUpdateScript(newDiv);
		params = null;
		newDiv = null;

	}

	/**
	 * @see leon.view.setinterface.LySetTableInterface#selectRow(int, boolean)
	 */
	public void selectRow(int idx, boolean select)
	{
		LyJQViewManager viewManager = (LyJQViewManager) getViewManager();
		LyTemplatesUtilsBean templateManager = viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(TABLE_ID_PARAM, getId());
		params.put(ROW_IDX_PARAM, idx);
		params.put(SELECT_PARAM, select);

		String newDiv = templateManager.mergeTemplate(SELECT_ROW_TEMPLATE, params);

		viewManager.addToUpdateScript(newDiv);
		params = null;
		newDiv = null;

	}

	/**
	 * @see leon.view.setinterface.LySetTableInterface#selectAll(boolean, boolean)
	 */
	public void selectAll(boolean select, boolean updateView)
	{
		LyJQViewManager viewManager = (LyJQViewManager) getViewManager();
		LyTemplatesUtilsBean templateManager = viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(TABLE_ID_PARAM, getId());
		params.put(UPDATE_VIEW_PARAM, updateView);
		params.put(SELECT_PARAM, select);

		String newDiv = templateManager.mergeTemplate(SELECT_ROW_TEMPLATE, params);

		viewManager.addToUpdateScript(newDiv);
		params = null;
		newDiv = null;

	}

	/**
	 * @see leon.view.setinterface.LySetTableInterface#showRow(int)
	 */
	public void showRow(int idx)
	{
        String parentFieldId = _table.getParentFormFieldId();

        // If a row has been activated and table is not in a form, show it
        if ((parentFieldId == null) || (parentFieldId.length() == 0))
        {
    		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
    		Map<String, Object> params = new HashMap<String, Object>();
    		
    		_activeRow = getRow(idx);
    		params.put(TABLE_ID_PARAM, getId());
    		params.put(ROW_PARAM, _activeRow);

    		String newDiv = templateManager.mergeTemplate(SHOW_ROW_TEMPLATE, params);

    		_viewManager.addToUpdateScript(newDiv);
    		params = null;
    		newDiv = null;
        }
	}
	
	public LyRow getActiveRow()
	{
		return _activeRow;
	}

    /**
     * Gets the row at the specified position.
     *
     * @param  i the row position
     * @return the LySwingRow at position i
     */
    public LyRow getRow(int i)
    {
        LyRows rows = _table.getRows();

        if (rows == null)
            return null;

        if (!rows.isHierarchical()
            && !rows.hasHiddenRow())
        {
            LyRow subRow = rows.getRow(i);
            
            return subRow;
        }
        else
        {
            Vector<LyRow>       subRows = new Vector<LyRow>();

            rows.getDisplayedSubRows(subRows);
            
            if (i >= subRows.size())
                return null;
            else
                return subRows.elementAt(i);
        }
    }

	public void reloadTable()
	{
		if (_table.getParent() instanceof LyFormField)
			_viewManager.addToReloadComponents(_table.getParent());
		else
			_viewManager.addToReloadComponents(_table);
	}

	/**
	 * @see leon.view.setinterface.LySetTableInterface#hideRow(int, boolean)
	 */
	public void hideRow(int idx, boolean hide)
	{
		reloadTable();
	}

	/**
	 * @see leon.view.setinterface.LySetTableInterface#freezeColumn(int)
	 */
	public void freezeColumn(int idx)
	{
	}

	/**
	 * @see leon.view.setinterface.LySetTableInterface#updateRow(leon.view.LyRow)
	 */
	public void updateRow(LyRow row)
	{
		reloadTable();
	}

	/**
	 * @see leon.view.setinterface.LySetVisibleInterface#setVisible(boolean)
	 */
	public void setVisible(boolean status)
	{
//		reloadTable();
	}

	/**
	 * @return if row contains an image
	 */
	public boolean containsImage()
	{
		boolean containsImg = false;
		LyRows rows = _table.getRows();
		int rowsCount = (rows == null ? 0 : rows.getRowCount());
		LyRow row;
		LyJQRows rowsIView;
		for (int i = 0; i < rowsCount; i++)
		// for (LyRow row : _rows.getSubRowList())
		{
			row = rows.getRow(i);
			if (row.getImage() != null && row.getImage() != "")
			{
				containsImg = true;
				break;
			}
			if (row.getRows() != null)
			{
				rowsIView = (LyJQRows) row.getRows().getInternalView();
				containsImg = rowsIView.containsImage();
				if (containsImg)
					break;
			}
		}
		return containsImg;
	}

	public boolean isTable()
	{
		return true;
	}

	public LyJQFooters getJQFooters()
	{
		return _footers;
	}

	public void setJQFooters()
	{
		if (_table.getFooters() != null)
			_footers = (LyJQFooters) _table.getFooters().getInternalView();
	}

	public LyJQRows getJQRows()
	{
		if (_rows == null)
			setJQRows();
		
		return _rows;
	}

	public void setJQRows()
	{
		LyRows rows = _table.getRows();
		_rows = (rows == null ? null : (LyJQRows) rows.getInternalView());
	}

	public LyJQHeaders getJQHeaders()
	{
		return _headers;
	}

	public void setJQHeaders()
	{
		LyHeaders headers = _table.getHeaders();
		_headers = (headers == null ? null : (LyJQHeaders) headers.getInternalView());
	}

	public int getNbRows(LyRows rows)
	{
		if (rows == null)
			return 0;
		
		int		nbRows = 0;
		LyRow	row;
		
		for (int i = 0; i < rows.getRowCount(); i++)
		{
			row = rows.getRow(i);
			
			if (row.getRows() != null)
				nbRows += getNbRows(row.getRows());
			else
				nbRows += 1;
		}

		return nbRows;
	}
	
	public void setNRows(int nRows)
	{
		reloadTable();
	}

	public boolean isEditable()
	{
		LyRows	rows = _table.getRows();
		
		if ((rows != null) && (rows.getRow(0) != null))
		{
			// Check if table contains components (for performance, just check it on the first row).
			// If so, do not disable selection mode.
			LyRow	row = rows.getRow(0);
			LyJQRow	webRow = (LyJQRow)row.getInternalView();
			
			if ((webRow != null) && webRow.hasComponents(false))
				return true;
		}
		
		return false;
	}

	/**
	 * Returns the string that must be displayed in headers for sort.
	 */
	public String getSortString(int sortIndex)
	{
		LyEnvironment	environment = getEnvironment();
		boolean			isArabic = environment.getLanguage().equals("ar");
		
		if (isArabic)
			return environment.translate("LY_SORT_" + sortIndex);
		
		return sortIndex + "";
	}
	
	/**
	 * Indicates whether table content must be loaded in a separate Ajax request.
	 */
	public boolean useAjaxLoading(boolean isPrintMode)
	{
		LyView	parentView = _table.getParentView();
		String	target = (parentView instanceof LyDialog ? ((LyDialog)parentView).getTarget() : null);
		
		if (!"popup_view".equals(target) && !isPrintMode)
		{
			// If table is not in popup and printMode is not set, check table size
			int	nRows = getNbRows(_table.getRows());
			
			if (nRows > 30)
				return true;
		}
		
		return false;
	}
}