/* 
 * Lyria-W4 - afficheur_web2
 * LyWeb2TitleBar.java
 * Copyright (c) 2000-2009, Lyria-W4
 * All rights reserved.
 */
package leon.view.web.jquery.component.titleBar;

import java.util.HashMap;
import java.util.Map;

import leon.misc.LyString;
import leon.view.LyTitleBar;
import leon.view.graphic.LyGraphicalTitleBar;
import leon.view.web.LyWebObject;
import leon.view.web.LyWebPageWriter;
import leon.view.web.LyWebViewManager;
import leon.view.web.jquery.common.TemplateUtils;
import leon.view.web.jquery.component.LyJQInternalView;
import leon.view.web.jquery.controller.LyTemplatesUtilsBean;

/**
 * Type : leon.view.web.web2.titleBar.LyWeb2TitleBar The peer component of LyTitleBar for the web2 viewer.
 * @version $Revision: -1M $
 * @author ychaari, last modified by $Author: (local) $
 */
public class LyJQTitleBar extends LyJQInternalView	implements
													LyGraphicalTitleBar
{

	/** The title bar generic component. */
	protected LyTitleBar			_titleBar;

	protected static final String	TITLE_BAR_PARAM		= "title_bar";

	protected static final String	TITLE_BAR_TEMPLATE	= "titleBar/lyTitleBarTemplate.vm";

	/**
	 * Contructor
	 * @param titleBar
	 */
	public LyJQTitleBar(LyTitleBar titleBar)
	{
		super(titleBar);
		_titleBar = titleBar;
	}

	/**
	 * Check if close icon has to be displayed
	 * @return
	 */
	public boolean isDisplayCloseIcon(LyTitleBar titleBar)
	{
		boolean displayUpPage = LyWebObject.displayUpPage(getEnvironment());
		return ((titleBar.getImage() == null) || displayUpPage);
	}

	/**
	 * Format given label
	 * @param label
	 */
	public void formatLabel(String label)
	{
		LyString.replace(label, "\\n", " ");
	}

	/**
	 * Get the application context url
	 * @return
	 */
	public String getBaseUrl()
	{
		return ((LyWebViewManager) getViewManager()).getRootURL();
	}

	public int getLength(String label)
	{
		return label.length();
	}

	public void setCenterLabel(String centerLabel)
	{
		reloadComponent();
	}

	public void setImage(String image)
	{
		reloadComponent();
	}

	public void setImageToolTip(String imageToolTip)
	{
		reloadComponent();
	}

	public void setLeftLabel(String leftLabel)
	{
		reloadComponent();
	}

	public void setRightImage(String rightImage)
	{
		reloadComponent();
	}

	public void setRightImageToolTip(String rightImageToolTip)
	{
		reloadComponent();
	}

	public void setRightLabel(String rightLabel)
	{
		reloadComponent();
	}

	public void writeHtml(LyWebPageWriter pw)
	{
		// Nop
	}

	@Override
	public String getReloadScript()
	{
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(TITLE_BAR_PARAM, _titleBar);
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		String titleBar = templateManager.mergeTemplate(TITLE_BAR_TEMPLATE, params);

		titleBar = TemplateUtils.getFormattedJs(titleBar);

		params = new HashMap<String, Object>();
		params.put("content", titleBar);
		params.put("componentId", getId());
		titleBar = templateManager.mergeTemplate("ajax/replaceWith.vm", params);
		params = null;
		return titleBar;

	}

	private void reloadComponent()
	{
		_viewManager.addToReloadComponents(_titleBar);
	}
}