/* 
 * Lyria-W4 - afficheur_web2
 * LyWeb2Toolbar.java
 * Copyright (c) 2000-2009, Lyria-W4
 * All rights reserved.
 */
package leon.view.web.jquery.component.toolbar;

import java.util.*;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;
import leon.view.web.jquery.*;
import leon.view.web.jquery.component.*;
import leon.view.web.jquery.controller.*;

/**
 * Type : leon.view.web.web2.component.toolbar.LyWeb2Toolbar The peer component
 * of LyToolBar for the web2 viewer.
 * 
 * @version $Revision: -1M $
 * @author ychaari, last modified by $Author: (local) $
 */
public class LyJQToolbar extends LyJQInternalView implements LyGraphicalToolBar {

	protected static String		TOOLBAR_PARAM			= "lyToolBar";

	protected static String		TOOLBAR_OPEN			= "open";

	protected static String		SET_OPEN_TEMPLATE		= "ajax/toolbar/setOpen.vm";

	protected LyToolBar _toolBar;

	protected boolean _isExpandableToolBarVar = true;

	protected boolean _isHorizontalDashBoard = testResource(LyJQConstants.LY_HORIZONTAL_DASHBOARD);

	/**
	 * Constructor
	 * 
	 * @param toolbar
	 */
	public LyJQToolbar(LyToolBar toolbar)
	{
		super(toolbar);
		_toolBar = toolbar;
	}

	/**
	 * Check if toolbar is horizontal
	 * 
	 * @param toolbar
	 * @return
	 */
	public boolean isHorizontal(LyToolBar toolbar)
	{
		if (_isHorizontalDashBoard)
			return true;

		return (toolbar.getFill() == LyToolBar.Fill.HORIZONTAL);
	}

	/**
	 * Check if toolbar is a top expandable toolbar
	 */
	public boolean isTopExpandToolbar(LyToolBar toolbar)
	{		
		return ((toolbar.getStyle() == LyToolBar.Style.EXPAND) && (toolbar.getParent() instanceof LyArea));
	}

	/**
	 * Check if toolbar has an image and a top-right toolbar as its first
	 * component
	 * 
	 * @param toolbar
	 * @return
	 */
	public boolean hasImageandTopRightToolbar(LyToolBar toolbar)
	{
		return ((getImagePath(toolbar) != null)
				&& (toolbar.getComponentCount() > 0)
				&& (toolbar.getComponent(0) instanceof LyToolBar)
				&& (((LyToolBar) toolbar.getComponent(0)).getHAlign() == LyToolBar.HAlign.RIGHT) && (((LyToolBar) toolbar
				.getComponent(0)).getVAlign() == LyToolBar.VAlign.TOP));
	}

	/**
	 * Get toolbar alignment
	 * 
	 * @param toolbar
	 * @return
	 */
	public String getAlign(LyToolBar toolbar)
	{
		String align = "";

		if (toolbar.getHAlign() == LyToolBar.HAlign.LEFT)
		{
			if(_viewManager.isRtlMode())
				align = "right";
			else
				align = "left";
		}
		else
		{
			if(_viewManager.isRtlMode())
				align = "left";
			else
				align = "right";
		}
		 		
		return align;
	}

	/**
	 * Indicates if the toolbar is expandable or not.
	 * 
	 * @return false if dashboards can't be expanded, true otherwise
	 */
	public boolean isExpandable(LyToolBar toolbar)
	{
		if (toolbar.getParent() instanceof LyFormField)
			return false;

		if (!_isExpandableToolBarVar || isHorizontal(toolbar))
			return false;

		return (toolbar.getStyle() == LyToolBar.Style.EXPAND);
	}

	/**
	 * Check the "LY_TOOL_LABEL_RIGHT_POSITION" attribute
	 * 
	 * @param toolbar
	 * @return
	 */
	public boolean isLabelRightPosition(LyToolBar toolbar)
	{
		return _environment.testResource(LyJQConstants.LY_TOOL_LABEL_RIGHT_POSITION);
	}

	/**
	 * Check the "LY_WRITE_TOOL_LABEL" attribute
	 * 
	 * @param toolbar
	 * @return
	 */
	public boolean isWriteLabel(LyToolBar toolbar)
	{
		return _environment.testResource(LyJQConstants.LY_WRITE_TOOL_LABEL);
	}

	/**
	 * Gets the tooltip image URL.
	 * 
	 * @return the tooltip image URL if there is an image, null otherwise
	 */
	public String getImagePath(LyToolBar toolbar)
	{
		String image = toolbar.getImage();
		
		return (((image == null) || (image.length() == 0)) ? null : _environment.getImageUrl(image));
	}

	/**
	 * Indicates if at least one of the tools of the toolbar contains a label.
	 * 
	 * @return true if at least one of the tools of the toolbar contains a
	 *         label, false otherwise
	 */
	public boolean containsLabel()
	{
		int			i, nTools = _toolBar.getComponentCount();
		LyComponent	component;
		String		label;

		// Parse the tools contained in this toolBar and search if at least one
		// is a toolBar and has a label
		for (i = 0; i < nTools; i++)
		{
			component = _toolBar.getComponent(i);

			if (component instanceof LyToolBar)
			{
				label = ((LyToolBar) component).getLabel();
				
				if ((label != null) && (label.length() != 0))
					return true;
			}
		}
		
		return false;
	}

	/**
	 * Check if the component is a toolbar
	 * 
	 * @param component
	 * @return
	 */
	public boolean isToolbarInstance(LyComponent component)
	{
		return (component instanceof LyToolBar);
	}

	/**
	 * Check if the component is a tool
	 * 
	 * @param component
	 * @return
	 */
	public boolean isToolInstance(LyComponent component)
	{
		return (component instanceof LyTool);
	}

	/**
	 * Check if the component is a field
	 * 
	 * @param component
	 * @return
	 */
	public boolean isFieldInstance(LyComponent component)
	{
		return (component instanceof LyFormField);
	}

	/**
	 * Check if the component is a label
	 * 
	 * @param component
	 * @return
	 */
	public boolean isLabelInstance(LyComponent component)
	{
		return (component instanceof LyLabel);
	}

	/**
	 * Get the toolbar's elements
	 * 
	 * @param toolbar
	 * @return
	 */
	public List<LyComponent> getToolbarList(LyToolBar toolbar)
	{
		List<LyComponent>	toolbarList = new ArrayList<LyComponent>();
		int					elementNb = toolbar.getComponentCount();
		 if(_viewManager.isRtlMode() && isHorizontal(toolbar))
		 {
				for (int i = elementNb-1; i >= 0; i--)
				{
					toolbarList.add(toolbar.getComponent(i));
				}
		 }
		 else
		 {
				for (int i = 0; i < elementNb; i++)
				{
					toolbarList.add(toolbar.getComponent(i));
				}
		 }
		
		return toolbarList;
	}

	/**
	 * Get a subList from the given index
	 * 
	 * @param list
	 * @param i
	 * @return
	 */
	public List<LyComponent> removeElementFromList(List<LyComponent> list, int i)
	{
		return list.subList(i, list.size());
	}

	/**
	 * Check if the given index is not empty
	 * 
	 * @param label
	 * @return
	 */
	public boolean isLabelNotEmpty(String label)
	{
		return (label != null && !label.trim().equals(""));
	}

	public void addComponent(LyComponent arg0, int arg1)
	{
		// Nop
	}

	public void removeComponent(LyComponent arg0)
	{
		// Nop
	}

	public void setImage(String arg0)
	{
		// Nop
	}

	public void setColor(String color)
	{
	}

	public void setOpen(boolean open)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();

		params.put(TOOLBAR_PARAM, _toolBar);
		params.put(TOOLBAR_OPEN, open);
		
		String	script = templateManager.mergeTemplate(SET_OPEN_TEMPLATE, params);
		
		_viewManager.addToUpdateScript(script);
		params = null;
		script = null;
	}

	public void writeHtml(LyWebPageWriter arg0)
	{
		// Nop
	}

	@Override
	public String getReloadScript()
	{
		// TODO Auto-generated method stub
		return null;
	}
}