/*
 * Lyria-W4 - afficheur_web2
 * LyWeb2Tool.java
 * Copyright (c) 2000-2009, Lyria-W4
 * All rights reserved.
 */
package leon.view.web.jquery.component.toolbar.tool;

import java.util.HashMap;
import java.util.Map;

import leon.misc.LyString;
import leon.view.LyArea;
import leon.view.LyComponent;
import leon.view.LyDialog;
import leon.view.LyFrame;
import leon.view.LyTable;
import leon.view.LyTool;
import leon.view.LyToolBar;
import leon.view.graphic.LyGraphicalTool;
import leon.view.web.LyWebPageWriter;
import leon.view.web.LyWebViewManager;
import leon.view.web.jquery.component.LyJQInternalView;
import leon.view.web.jquery.controller.LyTemplatesUtilsBean;

/**
 * Type : leon.view.web.jquery.component.toolbar.tool.LyJQTool The peer component of LyTool for the jquery viewer.
 * @version $Revision: 312 $
 * @author ychaari, last modified by $Author: ychaari $
 */
public class LyJQTool extends LyJQInternalView	implements
												LyGraphicalTool
{
	protected LyTool			_tool;

	protected static String		COMPONENT_PARAM			= "lyTool";

	protected static String		COMPONENT_ENABLED		= "enabled";

	protected static String   COMPONENT_PRESSED   = "pressed";

	protected static String		SET_ENABLED_TEMPLATE	= "ajax/toolbar/setEnabled.vm";

	protected static String   SET_PRESSED_TEMPLATE = "ajax/toolbar/setPressed.vm";

	/**
	 * Constructor
	 * @param tool
	 */
	public LyJQTool(LyTool tool)
	{
		super(tool);
		_tool = tool;
	}

	/**
	 * Check if the tool has the toggle style
	 * @param tool
	 * @return
	 */
	public boolean isToggleStyle(LyTool tool)
	{
		return (tool.getStyle() == LyTool.Style.TOGGLE);
	}

	/**
	 * Check if the tool is a separator
	 * @param tool
	 * @return
	 */
	public boolean isSeparator(LyTool tool)
	{
		return ((tool == null) || (tool.getId() == LyComponent.SEP_ID));
	}

	/**
	 * Indicates if the tool contains an area which contains only a toolbar or not.
	 * @return true if the tool contains an area which contains only a toolbar, false otherwise
	 */
	public boolean containsToolBar(LyTool tool)
	{
		LyComponent component = tool.getComponent();
		return ((component != null) && (component instanceof LyArea)
				&& (((LyArea) component).getPanelCount() == 0) && (((LyArea) component)
				.getToolBarCount() > 0));
	}

	/**
	 * Check if subArea is opened
	 * @param tool
	 * @return
	 */
	public boolean isAreaOpened(LyTool tool)
	{
		return getSubToolbar(tool).isOpen();
	}

	/**
	 * Gets the sub toolbar (toolbar in tool's area)
	 * @param tool
	 * @return
	 */
	public LyToolBar getSubToolbar(LyTool tool)
	{
		if (tool.getComponent() instanceof LyArea)
			return ((LyArea)tool.getComponent()).getToolBar(0);
		else
			return null;
	}

	public boolean isSmallDashboard(LyTool tool)
	{
	  return false;
	}

	/**
	 * Gets the tooltip
	 * @param label
	 * @return
	 */
	public String getToolTip(String label)
	{
		return LyString.getJavascriptString(label);
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalTool#setEnabled(boolean)
	 */
	public void setEnabled(boolean enabled)
	{
		LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(COMPONENT_PARAM, _tool);
		params.put(COMPONENT_ENABLED, enabled);
		String script = templateManager.mergeTemplate(SET_ENABLED_TEMPLATE, params);
		_viewManager.addToUpdateScript(script);
		params = null;
		script = null;
	}

	/**
	 * Check if the given label is empty
	 * @param label
	 * @return
	 */
	public boolean isLabelEmpty(String label)
	{
		return label == null || label.trim().length() == 0;
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalTool#setImage(java.lang.String)
	 */
	public void setImage(String image)
	{
		if (_viewManager.getBrowser() == LyWebViewManager.BROWSER_MSIE
				&& _viewManager.getBrowserVersion() <= 7.99 && getParentTable() != null)
		{
			_viewManager.addToReloadComponents(getParentTable());
		}
		else
		{
			refreshTool();
		}
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalTool#setLabel(java.lang.String)
	 */
	public void setLabel(String label)
	{
		if (_viewManager.getBrowser() == LyWebViewManager.BROWSER_MSIE
				&& _viewManager.getBrowserVersion() <= 7.99 && getParentTable() != null)
		{
			_viewManager.addToReloadComponents(getParentTable());
		}
		else
		{
			refreshTool();
		}
	}

	/**
	 * get parent table if tool is belonging to a table component
	 * @return
	 */
	protected LyTable getParentTable()
	{
		// TODO Auto-generated method stub
		LyComponent parent = _tool.getParent();

		while (!(parent instanceof LyTable || parent instanceof LyFrame || parent instanceof LyDialog))
		{
			parent = parent.getParent();
		}

		if (parent instanceof LyTable)
			return (LyTable) parent;

		return null;
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalTool#setPressed(boolean)
	 */
	public void setPressed(boolean pressed)
	{
	  LyTemplatesUtilsBean templateManager = _viewManager.getVeloTemplatesBean();
    Map<String, Object> params = new HashMap<String, Object>();
    params.put(COMPONENT_PARAM, _tool);
    params.put(COMPONENT_PRESSED, pressed);
    String script = templateManager.mergeTemplate(SET_PRESSED_TEMPLATE, params);
    _viewManager.addToUpdateScript(script);
    params = null;
    script = null;
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalTool#setTooltip(java.lang.String)
	 */
	public void setTooltip(String tooltip)
	{
		refreshTool();
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
		// Nop
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.web.jquery.component.LyJQInternalView#getReloadScript()
	 */
	@Override
	public String getReloadScript()
	{
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * refresh tool
	 */
	protected void refreshTool()
	{
		_viewManager.addToReloadComponents(_tool);
	}

	protected void refreshToolBar()
	{
		LyComponent parent = _tool.getParent();

		while (!(parent instanceof LyToolBar))
		{
			parent = parent.getParent();
		}
		_viewManager.addToReloadComponents(parent);
	}

	public int getImageFirstSize()
	{
		if (_tool.getImage() != null && getImageSize(_tool.getImage(), true) != null
				&& getImageSize(_tool.getImage(), true).length > 0)
		{
			return getImageSize(_tool.getImage(), true)[0];
		}
		return -1;
	}

	public int getImageSecondSize()
	{
		if (_tool.getImage() != null && getImageSize(_tool.getImage(), true) != null
				&& getImageSize(_tool.getImage(), true).length > 1)
		{
			return getImageSize(_tool.getImage(), true)[1];
		}
		return -1;
	}
}