package leon.view.web.jquery.component.tree;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import leon.misc.LyString;
import leon.view.LyComponent;
import leon.view.LyImage;
import leon.view.LyTree;
import leon.view.LyTreeNode;
import leon.view.LyTreeNode.Validity;
import leon.view.graphic.LyGraphicalTreeNode;
import leon.view.web.LyWebPageWriter;
import leon.view.web.jquery.component.LyJQInternalView;
import leon.view.web.jquery.controller.LyTemplatesUtilsBean;

public class LyJQTreeNode extends LyJQInternalView	implements
													LyGraphicalTreeNode
{
	protected LyTreeNode		_lyTreeNode;

	protected static String		POSITION_PARAM	= "position";

	/**
	 * get the fmk id
	 */
	public String getFmkId()
	{
		return LyString.replace(_lyTreeNode.getId(), "\\", "\\\\");
	}

	public String getJSLabel()
	{
		String	label = _lyTreeNode.getLabel();

		if (label == null)
			return "";

		label = translate(label);

		label = LyString.getJavascriptString(label);

		return LyString.replace(label, "\"", "\\\"");
	}

	public LyJQTreeNode(LyTreeNode lyTreeNode)
	{
		super(lyTreeNode);
		_lyTreeNode = lyTreeNode;
	}

	/**
	 * Gets parent tree instance.
	 * @return parent tree instance
	 */
	public LyTree getParentTree()
	{
		LyComponent	parent = _lyTreeNode.getParent();

		while ((parent != null) && !(parent instanceof LyTree))
			parent = parent.getParent();

		return (LyTree) parent;
	}

	/**
	 * get tree treeNodes
	 * @return treeNodes list
	 */
	public List<LyTreeNode> getTreeNodes()
	{
		List<LyTreeNode>	treeNodes = new ArrayList<LyTreeNode>();
		LyTreeNode			treeNode;

		for (int treeNodeIndex = 0; treeNodeIndex < _lyTreeNode.getTreeNodeCount(); treeNodeIndex++)
		{
			treeNode = _lyTreeNode.getTreeNode(treeNodeIndex);
			treeNodes.add(treeNode);
		}

		return treeNodes;
	}

	public void writeHtml(LyWebPageWriter pw)
	{
	}

	public void setLabel(String label)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();

		params.put("label", getJSLabel());
		params.put("treeNodeId", getId());
		params.put("treeId", ((LyJQTree) getParentTree().getInternalView()).getId());
		_viewManager.addToUpdateScript(templateManager.mergeTemplate("ajax/tree/setTreeNodeLabel.vm", params));
		params = null;
	}

	public void setTooltip(String tooltip)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();

		params.put("tooltip", tooltip);
		params.put("treeNodeId", getId());
		_viewManager.addToUpdateScript(templateManager.mergeTemplate("ajax/tree/setTreeNodeTooltip.vm", params));
		params = null;
	}

	public void setSelected(boolean selected)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();

		params.put("selected", selected);
		params.put("treeNodeId", getId());
		params.put("lyTreeNode", _lyTreeNode);

		String	parentTreeId = ((LyJQTree) getParentTree().getInternalView()).getId();

		params.put("parentTreeId", parentTreeId);
		params.put("pTreeSelectionMode", getParentTree().getSelectionMode());

		String	selectedCheckboxImage = _lyTreeNode.getSelectedCheckboxImage();

		if ((selectedCheckboxImage != null) && !selectedCheckboxImage.equals(""))
			params.put("selectedCheckboxImageUrl", getImageUrl(selectedCheckboxImage));

		String	emptyCheckboxImage = _lyTreeNode.getEmptyCheckboxImage();

		if (emptyCheckboxImage != null && !emptyCheckboxImage.equals(""))
			params.put("emptyCheckboxImageUrl", getImageUrl(emptyCheckboxImage));

		String	openNodeScript = templateManager.mergeTemplate("ajax/tree/setSelected.vm", params);

		_viewManager.addToUpdateScript(openNodeScript);
		params = null;
	}

	public void setOpen(boolean open)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		LyTree					tree =getParentTree();
 		LyJQTree				web2Tree = ((LyJQTree) getParentTree().getInternalView());
		Map<String, Object>		params = new HashMap<String, Object>();

		params.put("lyTreeNode", _lyTreeNode);
		params.put("web2Tree", web2Tree);
		params.put("open", open);
		params.put("parentId", getId());
		params.put("parentNodeFmkId", getFmkId());
		params.put("isParentNodeOpen", _lyTreeNode.isOpen());
		params.put("parentTreeNodeController", _lyTreeNode.getRootController());
		params.put("treeId", web2Tree.getId());
		params.put("treeDefaultId", tree.getId());
		_viewManager.addToUpdateScript(templateManager.mergeTemplate("ajax/tree/setOpen.vm", params));
		params = null;
	}

	public void setHidden(boolean hidden)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();

		params.put("hidden", hidden);
		params.put("treeNodeId", getId());
		_viewManager.addToUpdateScript(templateManager.mergeTemplate("ajax/tree/setTreeNodeHidden.vm", params));
		params = null;
	}

	public void setExpandable(boolean expandable)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();

		params.put("lyTreeNode", _lyTreeNode);
		params.put("treeNodeId", getId());
		params.put("expandable", expandable);
		_viewManager.addToUpdateScript(templateManager.mergeTemplate("ajax/tree/setTreeNodeExpandable.vm", params));
		params = null;
	}

	public void setImage(String image)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();
		String					defaultImageHeight = _environment.getResource("LY_DEFAULT_IMAGE_HEIGHT");

		params.put("treeNodeId", getId());
		params.put("imageDefaultHeight", defaultImageHeight);
		params.put("imageDefaultWidth", defaultImageHeight);
		params.put("imageUrl", getImageUrl(image));
		_viewManager.addToUpdateScript(templateManager.mergeTemplate("ajax/tree/setTreeNodeImage.vm", params));
		params = null;
	}

	public void setValidity(Validity validity)
	{
	}

	public void setFont(String font)
	{
	}

	public void setColor(String color)
	{
	}

	public void addImage(LyImage images, int pos)
	{
	}

	public void removeImage(LyImage images)
	{
	}

	public void addTreeNode(LyTreeNode treeNode, int position)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		LyJQTreeNode			jQTreeNode = (LyJQTreeNode) treeNode.getInternalView();
		LyJQTree				tree = ((LyJQTree) getParentTree().getInternalView());
		Map<String, Object>		params = new HashMap<String, Object>();

		params.put("lyTreeNode", treeNode);
		params.put(POSITION_PARAM, position);
		params.put("web2Tree", tree);
		params.put("parentId", getId());
		params.put("parentNodeFmkId", getFmkId());
		params.put("isParentNodeOpen", _lyTreeNode.isOpen());
		params.put("parentTreeNodeController", _lyTreeNode.getRootController());
		params.put("newTreeNodeId", jQTreeNode.getId());
		params.put("treeId", tree.getId());
		_viewManager.addToUpdateScript(templateManager.mergeTemplate("ajax/tree/addTreeNode.vm", params));
		params = null;
	}

	public void removeTreeNode(LyTreeNode treeNode)
	{
		LyTemplatesUtilsBean	templateManager = _viewManager.getVeloTemplatesBean();
		Map<String, Object>		params = new HashMap<String, Object>();
		LyJQTreeNode			jQTreeNode = (LyJQTreeNode) treeNode.getInternalView();

		params.put("treeId", ((LyJQTree) getParentTree().getInternalView()).getId());
		params.put("treeNodeId", jQTreeNode.getId());

		String	removeNodeScript = templateManager.mergeTemplate("ajax/tree/removeTreeNode.vm", params);

		_viewManager.addToUpdateScript(removeNodeScript);
		params = null;
	}

	@Override
	public String getReloadScript()
	{
		return null;
	}
}
