package leon.view.web.jquery.controller;

import java.util.*;

import javax.servlet.http.*;

import leon.app.*;
import leon.misc.*;
import leon.view.event.*;
import leon.view.web.*;
import leon.view.web.jquery.*;

import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.*;

/**
 * Answer Action controller
 * @author atalbi
 */
public class LyJQAnswerController extends LyJQController
{
	@Override
	@RequestMapping("/answer")
	public ModelAndView handleRequest(HttpServletRequest request, HttpServletResponse response)
			throws Exception
	{
		if (checkSession(request))
		{
			LySession		session = getSession();
			LyJQViewManager	viewManager = getViewManager(session);
			String			message = getParameter(request, LyWebConstants.MESSAGE);
			String			answer = getParameter(request, LyWebConstants.ANSWER);
			String			rootUrl = viewManager.getRootURL();
			LyWebViewManager	sessionViewManager = (LyWebViewManager)session.getViewManager();
			
			try
			{
				session.getSoftLock();

				beginHandleRequest(viewManager);
				setRequestAttributes(request, session);
	
				// at this point, the message should be not null
				if (message == null)
					return null;
	
				if (isSessionClosed(session))
				{
					LyTemplatesUtilsBean	templateManager = viewManager.getVeloTemplatesBean();
					Map<String, Object>		params = new HashMap<String, Object>();
					
					params.put("redirectUrl", rootUrl + "/" + LyJQConstants.PATH_ACTION_EXIT);
					viewManager.addToUpdateScript(templateManager.mergeTemplate("ajax/redirectionScript.vm", params));
				}
				
				// if we are in login mode, session is not valid yet
				boolean login = !session.isValid();
	
				// Answer the interface by calling the answer on the view manager
				sessionViewManager.answerMessage(message, LyTools.shortFromString(answer));
				
				// Session, environment and view manager may have changed when answering message (reconnection system)
				checkSession(request);
				
				if (!login && isSessionClosed(session))
				{
					LyTemplatesUtilsBean	templateManager = viewManager.getVeloTemplatesBean();
					Map<String, Object>		params = new HashMap<String, Object>();
					
					params.put("redirectUrl", rootUrl + "/" + LyJQConstants.PATH_ACTION_EXIT);
					viewManager.addToUpdateScript(templateManager.mergeTemplate("ajax/redirectionScript.vm", params));
				}
	
				writeUpdateScripts(response, viewManager);
				endHandleRequest(viewManager);
			}
			finally
			{
				session.releaseSoftLock();
			}
		}
		else
			return redirectToPage(request, null, LyJQConstants.PATH_ACTION_HOME);
		
		return null;
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.web.jquery.controller.LyJQController#getDemand(javax.servlet.http.HttpServletRequest,
	 * leon.misc.LyEnvironment)
	 */
	@Override
	public LyDemand getDemand(HttpServletRequest request, LyEnvironment environment)
	{
		return null;
	}
}
