package leon.view.web.jquery.controller;

import javax.servlet.http.*;

import leon.app.*;
import leon.info.*;
import leon.misc.*;
import leon.view.event.*;
import leon.view.web.*;
import leon.view.web.jquery.*;

import org.springframework.stereotype.*;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.*;

/**
 * Command action controller
 * @author atalbi
 */
@Controller
public class LyJQCommandController extends LyJQController
{
	@Override
	public LyDemand getDemand(HttpServletRequest request, LyEnvironment environment)
	{
		return null;
	}

	@RequestMapping("/command")
	public ModelAndView handleRequest(HttpServletRequest request, HttpServletResponse response)
			throws Exception
	{
		String		controller = getParameter(request, LyWebConstants.CONTROLLER);
		String		requestRootUrl = getRootUrlFromRequest(request);
		boolean		checkSession = checkSession(request);
		
		// If command action is received from login page, try to register session and to
		// validate login form.
		if (!checkSession)
		{
			checkAndCreateSession(request);
			
			LySession		session = getSession();
			LyJQViewManager	viewManager = getViewManager(session);
			
			viewManager.setServletContext(request.getSession().getServletContext());
			// No caching for this page.
			setNoCacheHeaders(response, 0);
			// Start the session which invokes root Action
			session.register(true);
			// Call the getDecoration() method to be sure that CSS file is
			// generated
			viewManager.getDecoration();
		}
		
		if (checkSession(request))
		{
			LySession		session = getSession();
			LyEnvironment	environment = session.getEnvironment();
			LyJQViewManager	viewManager = getViewManager(session);
			LyWebViewManager	sessionViewManager = (LyWebViewManager)session.getViewManager();

			try
			{
				session.getSoftLock();

				beginHandleRequest(viewManager);
				
				// Check if root action is _login
				LyApplication	application = session.getApplication();
				LyAction		rootAction = application.getProject().getRootAction();
				String			rootActionId = (rootAction == null ? null : rootAction.getId());
	
				if (controller.equals(rootActionId))
				{
					request.getSession().getServletContext().getRequestDispatcher("/" + LyJQConstants.PATH_ACTION_LOGIN)
						.forward(request, response);
					
					return null;
				}
				
				if (!checkSessionValidity(session))
					response.getWriter().write(requestRootUrl + "/" + LyJQConstants.PATH_ACTION_HOME);
				
				String	source = getParameter(request, LyWebConstants.SOURCE);
				String	value = getParameter(request, LyWebConstants.VALUE);
	
				if (source != null)
					source = source.intern();
	
				// Manage null controller
				if (source != null)
				{
					LyActionDemand demand = new LyActionDemand(LyActionDemand.TYPE_ACTION, source);
	
					demand.setValue(value);
					demand.setParameterValues(new LyWebParameterValues(request, environment.getString()));
					sessionViewManager.processDemand(controller, demand);
				}
				
				String redirect = (String)session.getValue("CLOSE_APPLICATION_ON_VALIDATE");
				
				if (redirect != null && redirect.equals("true"))
				{
					session.close();
					endHandleRequest(viewManager);
					
					return redirectToPage(request, null, LyJQConstants.PATH_ACTION_HOME);
				}
				
				writeUpdateScripts(response, viewManager);
				endHandleRequest(viewManager);
			}
			finally
			{
				session.releaseSoftLock();
			}
		}
		else
			return redirectToPage(request, null, LyJQConstants.PATH_ACTION_HOME);
		
		return null;
	}
}
