package leon.view.web.jquery.controller;

import java.io.*;

import javax.servlet.*;
import javax.servlet.http.*;

import leon.app.*;
import leon.control.*;
import leon.misc.*;
import leon.view.*;
import leon.view.event.*;
import leon.view.mobile.*;
import leon.view.web.*;
import leon.view.web.jquery.*;

import org.springframework.stereotype.*;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.*;


/**
 * Controller used to generate a page for a given component.
 */
@Controller public class LyJQComponentController extends LyJQController
{
    /**
     * @see leon.view.web.jquery.controller.LyJQEventController#handleRequest(javax.servlet.http.HttpServletRequest,
     * javax.servlet.http.HttpServletResponse)
     */
    @RequestMapping("/component")
    public ModelAndView handleRequest(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
		checkAndCreateSession(request);
		getApplicationId(request);
        
        String			mappedSessionId = getParameter(request, "_sessionId").intern();
        String			controllerId = getParameter(request, "_controllerId");
        String			componentId = getParameter(request, "_componentId");
		LyApplication	application = LyApplication.getApplication(_applicationId);
        LySession		mappedSession = (application != null ? (LySession)application.getSession(mappedSessionId) : null);

        if ((application == null) || (mappedSession == null))
        	return null;
        
        response.setHeader("Content-Type", LyString.DEFAULT_CHARSET);

        LyMobileViewManager mobileViewManager = (LyMobileViewManager)mappedSession.getViewManager();
        LyWebViewManager	viewManager = mobileViewManager.getWebViewManager();
        
        // Register last called url
        String	startUrl = viewManager.getRootURL() + request.getServletPath() + "?" + request.getQueryString();
        
        viewManager.setMobileStartURL(startUrl);
    	
    	LyEnvironment	mappedEnv = mappedSession.getEnvironment();
        LyController	controller = mappedSession.getController(controllerId);
        LyComponent		topComponent = (controller != null ? controller.getComponent() : null);
        LyComponent		component = (topComponent != null ? topComponent.getComponent(componentId) : null);
        
        if ((component == null) || !(component instanceof LyPanel))
        	return null;
        
        // Generate another component in the JQuery session
     /*   StringWriter	sw = new StringWriter();
        PrintWriter		pw = new PrintWriter(sw);
        
		pw.println("<?xml version='1.0' encoding='" + LyString.DEFAULT_CHARSET + "'?>");
		pw.println("<!DOCTYPE AREA SYSTEM 'leon/conf/dtd/views.dtd'>");
		pw.println("<AREA>");
        component.writeXml(pw, true);
		pw.println("</AREA>");
        
        StringBuffer			sb = sw.getBuffer();
        ByteArrayInputStream	bis = new ByteArrayInputStream(sb.toString().getBytes(LyString.DEFAULT_CHARSET));
        LyViewParser			parser = new LyViewParser(env, bis);
        LyArea					newComponent = (LyArea)parser.getRootElement();
        
        newComponent.setEnvironment(env);
        
        LyPanel					panel = newComponent.getPanel(0);*/
        LyWebComponent			webComponent = (component != null ? (LyWebComponent)component.getInternalView() : null);
        
        if (webComponent == null)
        	return null;

        LyJQPage		jqPage = new LyJQPage(component, true);
        
        webComponent.writeHtml(jqPage);

        StringBuffer	result = jqPage.getBuffer(jqPage.getActiveName());
        
        ModelAndView modelAndView = new ModelAndView(LyJQConstants.HOME_TEMPLATE);

        modelAndView.addObject("generateComponent", result.toString());
        modelAndView.addObject("environment", mappedEnv);
        modelAndView.addObject("viewManager", viewManager);
        modelAndView.addObject("charset", LyString.DEFAULT_CHARSET);
        modelAndView.addObject("showThemeSelector", false);

        return modelAndView;
    }

    /**
     * @see leon.view.web.jquery.controller.LyJQController#getDemand(javax.servlet.http.HttpServletRequest,
     * leon.misc.LyEnvironment)
     */
    @Override public LyDemand getDemand(HttpServletRequest request, LyEnvironment environment)
    {
        return null;
    }
}
