package leon.view.web.jquery.controller;

import java.io.*;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

import leon.app.*;
import leon.misc.*;
import leon.view.event.*;
import leon.view.web.*;
import leon.view.web.jquery.*;

import org.springframework.context.*;
import org.springframework.web.context.support.*;
import org.springframework.web.servlet.*;

/**
 * Ly jquery controllers abstract class
 * @author amt
 */

public abstract class LyJQController extends LyJQEventController
{
	/** Identifier of the default session. */
	public final static String	DEFAULT_SESSION = "DEFAULT_SESSION";

	protected String			_applicationId;

	protected LySpringUtilsBean	_applicationContext;

	/** The list of associated LySession instances of the HttpSession. */
	public static transient HashMap<String, LyJQSession>	_sessions = new HashMap<String, LyJQSession>();

	/** The list of associated LySession instances of the HttpSession. */
	private static transient HashMap<Thread, LyJQSession>	_sessionThreads = new HashMap<Thread, LyJQSession>();


	public static void swapSession(Object newSession, Object oldSession, String applicationId)
	{
	}

	/**
	 * Checks the existence of a Leonardi session.<br />
	 * @param request current http request
	 * @return
	 */
	protected boolean checkSession(HttpServletRequest request)
	{
		HttpSession		httpSession = request.getSession();
		String			target = request.getParameter(LyWebConstants.TARGET_PARAM);
		String			httpSessionKey = getSessionKey(httpSession, target);
		LySession		currentSession = _sessions.get(httpSessionKey);

		// Try to get session registered on http session
		if (currentSession == null)
			currentSession = (LySession)httpSession.getAttribute(httpSessionKey);

		if (!isSessionClosed(currentSession))
		{
			setSession(currentSession);

			return true;
		}

		setSession(null);

		return false;
	}

	public static String getSessionKey(HttpSession httpSession, String target)
	{
		if ((target == null) || "null".equals(target) || "".equals(target))
			target = DEFAULT_SESSION;

		return httpSession.getId() + "#" + target;
	}

	public LyDemand getDemand(HttpServletRequest request, LyEnvironment environment)
	{
		return null;
	}

	/**
	 * Checks the existence of a Leonardi session.<br />
	 * If no session exists, a new Leonardi session is created.
	 * @param request current http request
	 */
	protected void checkAndCreateSession(HttpServletRequest request)
	{
		HttpSession		httpSession = request.getSession();
		String			target = request.getParameter(LyWebConstants.TARGET_PARAM);
		String			httpSessionKey = getSessionKey(httpSession, target);
		LyJQSession		currentSession = _sessions.get(httpSessionKey);

		if (currentSession == null)
			currentSession = (LyJQSession)httpSession.getAttribute(httpSessionKey);

		if (isSessionClosed(currentSession))
		{
			if (_sessions.containsKey(httpSessionKey))
				_sessions.remove(httpSessionKey);

			currentSession = createSession(request, target);

			String redirect = (String) request.getAttribute("REDIRECT_ACTION_URL");

			if (redirect != null && !redirect.equals(""))
			{
				currentSession.addValue("REDIRECT_ACTION_URL", redirect);
				request.removeAttribute(redirect);
			}

			currentSession.setHttpSession(httpSession, httpSessionKey);
			setSession(currentSession);

			int timer = currentSession.getSessionTimeout();

			if (timer > 0)
				httpSession.setMaxInactiveInterval(timer);

			LyApplication	application = getApplication();
			LyEnvironment	environment = application.getEnvironment();

			if (environment == null)
			{
				request.setAttribute("leonardi.exception", application.getError());
				return;
			}

			if (httpSession.getAttribute(application.getId()) == null)
				httpSession.setAttribute(application.getId(), new _LyJQSessionListener());

			LyJQViewManager	viewManager = (LyJQViewManager)currentSession.getViewManager();
			String			rootUrl = null;

			if (LyJQViewManager.USE_ABSOLUTE_ROOT_URL)
			{
				if (request.getRequestURL() != null)
				{
					rootUrl = request.getRequestURL().toString();

					int index = rootUrl.lastIndexOf('/');

					if (index > 0)
						rootUrl = rootUrl.substring(0, index);
				}
			}

			if (rootUrl == null)
				rootUrl = request.getContextPath();

			// change the scheme according to SSL layer
			String httpScheme;

			if (environment.testEnv("LY_USE_HTTPS"))
				httpScheme = "https:";
			else
				httpScheme = request.getScheme() + ":";

			rootUrl = LyString.replace(rootUrl, "http:", httpScheme);
			rootUrl = LyString.replace(rootUrl, "https:", httpScheme);

			viewManager.setRootURL(rootUrl);

			if (currentSession.getMasterSession() != null
					|| ((target != null) && !DEFAULT_SESSION.equals(target)))
				viewManager.setTargetCode(target);
		}
		else
			setSession(currentSession);
	}

	/**
	 * Gets the application of the current session.<br>
	 * It wakes it up if necessary.
	 * @return the application of the current session.
	 */
	public LyWebApplication getApplication()
	{
		if (getSession() == null)
			return null;

		LyApplication application = getSession().getApplication();

		if (application.isSleeping())
		{
			LyEnvironment environment = getEnvironment();

			environment.logTrace("Waking up application " + application.getId());
			application.wakeUp();
		}

		return (LyWebApplication) application;
	}

	/**
	 * Gets the environment of the current session.
	 * @return the environment of the current session.
	 */
	public LyEnvironment getEnvironment()
	{
		if (getSession() == null)
			return null;

		return getSession().getEnvironment();
	}

	/**
	 * Creates a new Leonardi session.
	 * @return the newly created session.
	 */
	protected LyJQSession createSession(HttpServletRequest request, String sessionCode)
	{
		getApplicationId(request);

		LyApplication application = LyApplication.getApplication(_applicationId);

		if (application == null)
		{
			HttpSession    httpSession = request.getSession();
			ServletContext context = httpSession.getServletContext();

			application = LyJQController.createApplication(_applicationId, _applicationContext.getConfigFile(),
					context.getRealPath(""));
		}

		LyJQSession session;

		if (sessionCode != null)
		{
            session = (LyJQSession)application.getSession(sessionCode.intern());
            if (session != null)
                return session;
		}

		session = new LyJQSession(application, false);

		return session;
	}

	/**
	 * Write file in another. transform image url path in css to match new dest
	 * @param bw dest file
	 * @param file file to write
	 * @throws IOException IO errors
	 */
	private static void aggregateCssFile(BufferedWriter bw, String file, LyEnvironment env) throws IOException
    {
        File f = new File(file);
        if (f.exists())
        {
            FileReader fr;
            fr = new FileReader(f);
            BufferedReader br = new BufferedReader(fr);
            String line = null;
            String base = "./";
            String docDir = env.getEnv("LY_APP_DOC_DIR");
            String docDirSlashes = LyString.replace(docDir, "\\", "/");
            String labelSlashes = LyString.replace(f.getPath(), "\\", "/");
            labelSlashes = labelSlashes.substring(docDirSlashes.length());
            labelSlashes = labelSlashes.replace("/leon/css/", base);
            labelSlashes = labelSlashes.replace(f.getName(), "");
            while ((line = br.readLine()) != null)
            {
                line = line.replace("url(images/", "url(" + labelSlashes + "images/");
                line = line.replace("url(./images/", "url(" + labelSlashes + "images/");
                line = line.replace("url(../../../../../leon/images/", "url(../images/");
                line = line.replace("url(../../../images/form/field/", "url(./images/form/field/");

                line = line.replace("url('images/", "url('" + labelSlashes + "images/");
                line = line.replace("url('./images/", "url('" + labelSlashes + "images/");
                line = line.replace("url('../../../../../leon/images/", "url('../images/");
                line = line.replace("url('../../../images/form/field/", "url('./images/form/field/");

                line = line.replace("url(\"images/", "url(\"" + labelSlashes + "images/");
                line = line.replace("url(\"./images/", "url(\"" + labelSlashes + "images/");
                line = line.replace("url(\"../../../../../leon/images/", "url(\"../images/");
                line = line.replace("url(\"../../../images/form/field/", "url(\"./images/form/field/");

                bw.append(line);
                bw.newLine();
            }
            br.close();
        }
    }

	/**
     * Creates a new Leonardi application.
     *
     * @return the newly created application.
     */
    public static LyWebApplication createApplication(String applicationId, String configFile, String realPath)
    {
        LyWebApplication application = new LyWebApplication(applicationId, configFile, realPath);

        LyJQController.generateCssFile(application);

        LyEnvironment env = application.getEnvironment();
        String skin = env.getSkin();

        String leonDocDir = env.getString().expand("$LY_LEON_DOC_DIR$");
        File f = new File(leonDocDir + "/css/concat.css");
        BufferedWriter bufferWritter = null;

        try
        {
            if (f.exists())
            {
                f.delete();
            }
            f.createNewFile();
            FileWriter fw = new FileWriter(f, true);

            bufferWritter = new BufferedWriter(fw);
            aggregateCssFile(bufferWritter, leonDocDir + "/css/layout/layout.css", env);
            aggregateCssFile(bufferWritter, leonDocDir + "/css/ly-global.css", env);
            aggregateCssFile(bufferWritter, leonDocDir + "/css/jquery-ui/themes/" + skin + "/jquery-ui.css", env);
            aggregateCssFile(bufferWritter, leonDocDir + "/css/form/formular.css", env);
            aggregateCssFile(bufferWritter, leonDocDir + "/css/prettyPhoto/prettyPhoto.css", env);
            aggregateCssFile(bufferWritter, leonDocDir + "/css/form/validationEngine.jquery.css", env);
            aggregateCssFile(bufferWritter, leonDocDir + "/css/form/field/date/jquery-dateplustimepicker.css", env);
            aggregateCssFile(bufferWritter, leonDocDir + "/css/form/field/choice/ui.checkbox-ui.radiobutton.css", env);
            aggregateCssFile(bufferWritter, leonDocDir + "/css/form/field/relation/ui.multiselect.css", env);
            aggregateCssFile(bufferWritter, leonDocDir + "/css/form/field/number/ui.spinner.css", env);
            aggregateCssFile(bufferWritter, leonDocDir + "/css/form/jquery.multiselect.css", env);
            aggregateCssFile(bufferWritter, leonDocDir + "/css/form/field/boolean/boolean.css", env);
            aggregateCssFile(bufferWritter, leonDocDir + "/css/kiketable/jquery.kiketable.colsizable.css", env);
            aggregateCssFile(bufferWritter, leonDocDir + "/css/sticker/jquery.stickynotes.css", env);
            aggregateCssFile(bufferWritter, leonDocDir + "/css/autocomplete/jquery.autocomplete.css", env);
            aggregateCssFile(bufferWritter, leonDocDir + "/css/qtip/jquery.qtip.min.css", env);

        }
        catch (IOException e)
        {
            env.logError("Error when aggregating Css files : " + e.getMessage());
        }
        finally
        {
            try
            {
                if (bufferWritter != null)
                {
                    bufferWritter.flush();
                    bufferWritter.close();
                }
            }
            catch (IOException ex)
            {
                env.logError("Error when aggregating Css files : " + ex.getMessage());
            }
        }

        return application;
    }

    /**
     * Generate CSS File.
     * Replace all $XX$ variables in skin.
     */
    public static void generateCssFile(LyApplication application)
    {
        LyEnvironment  env = application.getEnvironment();
        LyJQDecoration decoration = new LyJQDecoration(env);
        String         skin = env.getSkin();
        String         leonDocDir = env.getString().expand("$LY_LEON_DOC_DIR$");
        String         skinPath = leonDocDir + "/css/jquery-ui/themes/" + skin + "/jquery-ui.css";
        String         skinAqualightPath = leonDocDir + "/css/jquery-ui/themes/aqualight/jquery-ui.css";

        env.setSkin("aqualight");
        decoration.setSkinFile(skinAqualightPath, env);
        env.setSkin(skin);
        decoration.setSkinFile(skinPath, env);
    }

	/**
	 * Sets an expiration delay used by the cache : modify the HTTP response header.
	 * @param response Current HTTP response.
	 * @param delay The expiration delay.
	 */
	public void setNoCacheHeaders(HttpServletResponse response, int delay)
	{
		try
		{
			// Always control the pages.
			response.setHeader("Cache-control", "no-cache");
			response.setHeader("Pragma", "no-cache");
			response.setDateHeader("Expires", (new Date()).getTime() + delay);
		}
		catch (Exception e)
		{
			// An exception has been reported in some cases when using WebSphere
		}
	}

	/**
	 * Retrieves a parameter in an http request and decode its value.<br>
	 * If the value is multiple, this method just returns the last value.
	 * @param request The HTTP request
	 * @param name The name of the parameter.
	 * @return The value found in the given HTTP request.
	 */
	protected String getParameter(HttpServletRequest request, String name)
	{
		String[] values = request.getParameterValues(name);

		if (values == null)
			return null;

		// Just return the last value
		String value = values[values.length - 1];

		value = decodeUrl(value);

		return value;
	}

	/**
	 * Removes the context path from URLS to enable forwarding into a JSP.<br>
	 * Make a path relative to the Servlet context for the given page.<br>
	 * The path beeing build is relative to the root directory of the current WEB application.
	 * @param request Current HTTP request.
	 * @param page A JSP or HTML page with possibly a context path.
	 * @return the page ready for a forward.
	 */
	public String getContextRelativePath(HttpServletRequest request, String page)
	{
		if ((request == null) || (page == null))
			return page;

		String contextPath = request.getContextPath();

		// We do a forward, so this url may be relative to current servlet
		if (page.startsWith(contextPath))
			page = page.substring(contextPath.length());

		page = completeUrlParams(page, request, null);

		return page;
	}

	/**
	 * Decodes an URL by replacing unauthorized characters.
	 * @param url The URL to decode.
	 * @return The URL without unauthorized characters.
	 */
	protected static String decodeUrl(String url)
	{
		String newUrl;

		newUrl = LyString.replace(url, "%20", " ");
		newUrl = LyString.replace(newUrl, "\\\\", "\\");

		return newUrl;
	}

	/**
	 * Method that search selected items inside the current HTTP request.
	 * @param request Current HTTP request
	 * @return a Vector that contains the selected items.
	 */
	protected Vector<String> getSelectionIds(HttpServletRequest request)
	{
		// search for selection parameters
		Vector<String>	selectionIds = new Vector<String>();
		String			parameter, selection, newElement;
		Enumeration<?>	e = request.getParameterNames();

		while (e.hasMoreElements())
		{
			// get parameter
			parameter = (String) e.nextElement();

			if (parameter.indexOf(LyWebConstants.SELECTION_ITEM) == 0)
			{
				// get selection value
				selection = request.getParameterValues(parameter)[0];

				// The selection value may be in the name or in the value
				// parameter
				// the first to try is the value, then the name
				if ((selection != null) && (selection.indexOf(LyWebConstants.SELECTION_ITEM) >= 0))
					newElement = selection.substring(LyWebConstants.SELECTION_ITEM.length());
				else
					newElement = parameter.substring(LyWebConstants.SELECTION_ITEM.length());

				// if the element is already in the selection list, it is
				// already selected
				// so it's an UNSELECT_ITEM demand
				if (newElement.length() != 0)
				{
					if (selectionIds.contains(newElement))
						selectionIds.removeElement(newElement);
					else
						selectionIds.addElement(newElement);
				}
			}
		}

		// return list of selected items
		return selectionIds;
	} // end method getSelectionIds

	protected void beginHandleRequest(LyJQViewManager viewManager)
	{
		viewManager.setRequestInProgress(true);
	}

	protected void endHandleRequest(LyJQViewManager viewManager)
	{
		if (viewManager != null)
		{
			viewManager.setRequestInProgress(false);
			viewManager.setCurrentWindowName(null);
		}

		setSession(null);
	}

	/**
	 * @param _session the _session to set
	 */
	public void setSession(LySession session)
	{
		if (session == null)
			_sessionThreads.remove(Thread.currentThread());
		else
			_sessionThreads.put(Thread.currentThread(), (LyJQSession)session);
	}

	public LySession getSession()
	{
		return _sessionThreads.get(Thread.currentThread());
	}

	/**
	 * getRootUrlFromRequest
	 * @param request
	 * @return root url
	 */
	public String getRootUrlFromRequest(HttpServletRequest request)
	{
		return request.getScheme() + "://" + request.getServerName() + ":"
				+ request.getServerPort() + request.getContextPath();
	}

	/**
	 * Tests if a session is closed.
	 * @param session the session to test.
	 * @return true if the specified session is closed.
	 */
	protected boolean isSessionClosed(LySession session)
	{
		return ((session == null) || (session.getState() == LySession.State.CLOSED) || (session
				.getState() == LySession.State.CLOSING));
	}

	/**
	 * @return login view
	 */
	public ModelAndView getLoginView()
	{
		ModelAndView modelAndView = new ModelAndView(LyJQConstants.VM_LOGIN_NAME);

		LyEnvironment	environment = getEnvironment();
		String			loginPage = environment.getUrl("LY_JQUERY_LOGIN_PAGE");

		if ((loginPage != null) && (!loginPage.equals("LY_JQUERY_LOGIN_PAGE"))
				&& (loginPage.trim().length() > 0))
		{
			modelAndView.addObject("loginPage", loginPage);
		}

		modelAndView.addObject("homeUrl", LyJQConstants.PATH_ACTION_HOME);
		modelAndView.addObject("frenchFlag", environment.getImageUrl("LY_FRENCH_FLAG"));
		modelAndView.addObject("englishFlag", environment.getImageUrl("LY_ENGLISH_FLAG"));

		return modelAndView;
	}

	/**
	 * return application Context Spring bean
	 * @param http request
	 */
	public void getApplicationContext(HttpServletRequest request)
	{
		ApplicationContext appContext = WebApplicationContextUtils.getWebApplicationContext(request
				.getSession().getServletContext());

		_applicationContext = (LySpringUtilsBean) appContext.getBean("applicationContext");
	}

	/**
	 * @return application id
	 */
	public void getApplicationId(HttpServletRequest request)
	{
		getApplicationContext(request);
		_applicationId = _applicationContext.getApplicationId();
	}

	/**
	 * Internal class used to listen to session bounds / unbounds.
	 */
	class _LyJQSessionListener implements HttpSessionBindingListener
	{
		/**
		 * Implementation of HttpSessionBindingListener interface.<br>
		 * Creates the LySession and initialize the timeout.
		 *
		 * @see HttpSessionBindingListener#valueBound(javax.servlet.http.HttpSessionBindingEvent)
		 */
		public void valueBound(HttpSessionBindingEvent event)
		{
		}

		/**
		 * Implementation of HttpSessionBindingListener interface.<br>
		 * Destroys the related LySession.
		 *
		 * @see HttpSessionBindingListener#valueUnbound(javax.servlet.http.HttpSessionBindingEvent)
		 */
		public void valueUnbound(HttpSessionBindingEvent event)
		{
			LyApplication	application = getApplication();
			HttpSession		httpSession = event.getSession();

			try
			{
				String		httpSessionId = getSessionKey(httpSession, null);
				LyJQSession	jqSession = _sessions.get(httpSessionId);

				_sessions.remove(httpSessionId);

				// Closing the user session.
				if ((jqSession != null) && (!isSessionClosed(jqSession)))
					jqSession.sessionTimeout(false);

				try
				{
					String	appId = (application == null ? null : application.getId());

					if (appId != null)
						httpSession.setAttribute(appId, null);
				}
				catch (IllegalStateException e)
				{
					// With some servers, this produces an exception "Session already invalidated"
				}
			}
			catch (Exception e)
			{
				// Prevent any crash
				if ((application != null) && (application.getEnvironment() != null))
					application.getEnvironment().writeException(e);
				else
					e.printStackTrace();
			}
		}
	}
}
