package leon.view.web.jquery.controller;

import java.io.*;
import java.util.zip.*;

import javax.servlet.http.*;

import leon.app.*;
import leon.control.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;
import leon.view.event.*;
import leon.view.mobile.*;
import leon.view.web.*;
import leon.view.web.jquery.*;
import leon.view.web.jquery.component.*;
import leon.view.web.jquery.component.map.*;

import org.springframework.web.servlet.*;
import org.springframework.web.servlet.mvc.*;

/**
 * Ly Web 2 controllers abstract class
 * @author amt
 */
public abstract class LyJQEventController	implements
											Controller
{
	protected static String		CLOSE_DIALOG_EVENT	= "_close";

	protected static String		SET_SIZE_EVENT		= "setSize";
	
	protected static String		AUTO_COMPLETE_EVENT		= "completeValue";
	
	protected static String		TREE_MAP_EVENT		= "treeMap";

	protected static String		PARAM_SEPARATOR		= "_SEP_";

	
	public ModelAndView handleRequest(HttpServletRequest request, HttpServletResponse response)
			throws Exception
	{
		if (checkSession(request))
		{
			LySession		session = getSession();
			LyJQViewManager	viewManager = getViewManager(session);
			LyWebViewManager	sessionViewManager = (LyWebViewManager)session.getViewManager();

			try
			{
				session.getSoftLock();

				beginHandleRequest(viewManager);
				setRequestAttributes(request, session);
				
				String requestRootUrl = viewManager.getRootURL();
				
				if (!checkSessionValidity(session))
					response.getWriter().write(requestRootUrl + "/" + LyJQConstants.PATH_ACTION_HOME);
				
				LyApplication application = session.getApplication();
	
				// get some usefull request parameters
				String controllerId = getParameter(request, LyWebConstants.CONTROLLER);
				String source = getParameter(request, LyWebConstants.SOURCE);
				String frame = getParameter(request, LyWebConstants.FRAME);
				String value = getParameter(request, LyWebConstants.VALUE);
	
				// If controller is not found in the parameters, try to retrieve it from source
				// that may be the identifier of the component that sent the request.
				if ((controllerId == null) || (controllerId.length() == 0) || "ROOT_CONTROLLER".equals(controllerId))
				{
					String stackName = frame;
	
					if (LyWebConstants.TARGET_FRAME.equals(frame))
						stackName = null;
	
					LyViewController viewController = session.getStackLastController(stackName);
	
					if ((viewController != null) && !viewController.isFreed())
					{
						LyController		parentController = viewController;
						LyViewController	rootController = viewController;
						
						while (parentController.getParent() != null)
						{
							parentController = parentController.getParent();
							
							if (parentController instanceof LyViewController)
								rootController = (LyViewController)parentController;
						}
						
						LyView view = rootController.getView();
	
						if (view != null)
						{
							LyComponent component = view.getComponent(source);
							LyComponent parentComponent = null;
							
							if (view instanceof LyDialog)
							{
								String		parentId = ((LyDialog)view).getParentId();
								LyView		parentView = viewManager.getView(parentId);
								
								parentComponent = (parentView == null ? null : parentView.getComponent(source));
							}
	
							if ("ROOT_CONTROLLER".equals(controllerId))
								controllerId = view.getRootController();
							else if (parentComponent != null)
								controllerId = parentComponent.getRootController();
							else if (component != null)
								controllerId = component.getRootController();
							else
							{
								LyJQInternalView jqInternalView = ((LyJQInternalView)view.getInternalView());
								jqInternalView.setProperty(source, value);
							}
						}
					}
				}
	
				if ((source != null) && source.equals(SET_SIZE_EVENT) && (controllerId != null))
				{
					LyController	viewController = (LyController)session.getController(controllerId);
					LyComponent		component = viewController.getComponent();
	
					if (component instanceof LyArea)
					{
						component = (LyArea) component;
						for (int i = 0; i < component.getChildCount(); i++)
						{
							if (component.getChild(i) instanceof LySplitter)
							{
								((LySplitter) component.getChild(i)).setSizes(value + " "
										+ (100 - Integer.parseInt(value)));
								break;
							}
						}
					}
					else if (component instanceof LySplitter)
						((LySplitter) component).setSizes(value + " " + (100 - Integer.parseInt(value)));
				}
	
				if ((source != null) && source.equals(TREE_MAP_EVENT) && (controllerId != null))
				{
					LyComponent currentTreeMap = viewManager.getCurrentTreeMapView();
					
					if (currentTreeMap != null)
					{
						LyJQMap	currentJQTreeMap =  (LyJQMap) currentTreeMap.getInternalView();
	
						currentJQTreeMap.getMapContent(value);
						writeUpdateScripts(response, viewManager);
						endHandleRequest(viewManager);
					}
					
					return null;
				}
				
				if ((source != null) && "stopSharingSession".equals(source))
				{
					LyJQViewManager	sharingManager = viewManager.getSharingManager(request.getSession());
	
					viewManager.stopSharingSession();
					
					if (sharingManager != null)
					{
						endHandleRequest(viewManager);
						viewManager = sharingManager;
					}
				}
				
				// get the controller that receive this event
				LyController controller = (controllerId == null) ? null : session.getController(controllerId);
	
				// Check if the action is a print action
				String[] parameters = LyString.stringToList(source, LyWebConstants.CONTROLLER_ID_SEP);
	
				if ((parameters != null) && (parameters.length > 0) && (parameters[0] != null))
				{
					String id = parameters[0].intern();
					LyAction action = application.getAction(id);
					boolean isPrintAction = false;
	
					if (action != null)
					{
						action = action.getRoot();
						isPrintAction = ((action != null)
								&& (LyAction.ID_PRINT.equals(action.getId()) || LyAction.ID_PRINT_TABLE.equals(id)));
					}
					else
						isPrintAction = id.startsWith(LyAction.ID_PRINT);
	
					if (isPrintAction)
					{
						request.getSession().getServletContext().getRequestDispatcher("/"+LyJQConstants.PATH_ACTION_PRINT).forward(request, response);
						endHandleRequest(viewManager);
						return null;
					}
				}
	
				if ((source != null) && source.equals(AUTO_COMPLETE_EVENT) && (controllerId != null))
					viewManager.setAutoCompleteEvent(true);		
				else
					viewManager.setAutoCompleteEvent(false);
				
				// get the demand from the request
				LyDemand demand = (controller == null ? null : getDemand(request, session.getEnvironment()));
				
				// process the demand to the areaController
				if (demand != null)
				{
					if ((controller instanceof LyViewController)
							&& !LyWebConstants.FRAME_CONTROLLER_DEMAND_CLOSE.equals(source))
					{
						LyAreaController areaController = ((LyViewController) controller).getAreaController();
	
						if (areaController != null)
							controllerId = areaController.getId();
					}
					
					sessionViewManager.processDemand(controllerId, demand);
				}
	
				writeUpdateScripts(response, viewManager);
				endHandleRequest(viewManager);
			}
			finally
			{
				session.releaseSoftLock();
			}
		}
		else
			return redirectToPage(request, null, LyJQConstants.PATH_ACTION_HOME);

		return null;
	}

	public void writeUpdateScripts(HttpServletResponse response, LyJQViewManager viewManager) throws IOException
	{
		if (viewManager != null)
		{
			LyEnvironment	environment = viewManager.getEnvironment();
			boolean			compress = ((environment != null) && environment.testResource("LY_WEB_USE_COMPRESSION"));
			String			updateScript = viewManager.getUpdateScripts(true);

			response.setContentType("text/html; charset=" + LyString.DEFAULT_CHARSET);

			if(compress)
			{	
				response.setHeader("Content-Encoding", "gzip");	

				GZIPOutputStream	gzos = new GZIPOutputStream(response.getOutputStream());

				gzos.write(updateScript.getBytes(LyString.DEFAULT_CHARSET));
				gzos.close();
				response.setHeader("Content-Length", gzos.toString().length() + "");
			}
			else
				response.getWriter().write(updateScript);			

			viewManager.setPrintMode(false);
			//System.gc();
		}
	}

	/**
	 * Stores some usefull information as attribute inside an HTTP request. Then forward to a JSP:
	 * <ul>
	 * <li>leonardiSession : The current LySession</li>
	 * <li>servletRootURL : The current absolute URL of the servlet</li>
	 * <li>leonardiRootURL : The URL to the leonardi documents: $LY_LEON_DOC$</li>
	 * <li>appRootURL : The URL to the application documens: $LY_APP_DOC$</li>
	 * </ul>
	 * @param request Current HTTP request.
	 * @param session Current user session
	 */
	public void setRequestAttributes(HttpServletRequest request, LySession session)
	{
		if (session == null)
			return;

		request.setAttribute(LyWebConstants.RA_LY_SESSION, session);
		request.getSession().setAttribute(LyWebConstants.RA_LY_SESSION, session);

		LyEnvironment environment = session.getEnvironment();

		if (environment != null)
		{
			LyJQViewManager viewManager = getViewManager(session);

			if (viewManager != null)
				request.setAttribute(LyWebConstants.RA_LY_ROOT_URL, viewManager.getRootURL());

			// set the leonardi root url
			String leonardiRootUrl;
			String leonardiExternalUrl;
			String LEON_DOC = "$LY_LEON_DOC$";
			String LEON_DOC_EXTERNAL = "$LY_LEON_DOC$";
			String cssPath = environment.getUrl("LY_CSS_FILE");

			if (environment.getEnv("_LY_LEON_DOC_EXTERNAL") != null)
				LEON_DOC_EXTERNAL = "$_LY_LEON_DOC_EXTERNAL$";

			leonardiRootUrl = environment.getString().expandUrl(LEON_DOC);
			leonardiExternalUrl = environment.getString().expandUrl(LEON_DOC_EXTERNAL);

			request.setAttribute(LyWebConstants.RA_LY_LEON_DOC_URL, leonardiRootUrl);
			request.setAttribute(LyWebConstants.RA_LY_LEON_DOC_EXTERNAL_URL, leonardiExternalUrl);
			request.setAttribute(LyWebConstants.RA_LY_SKIN_URL, cssPath);
			request.setAttribute(LyWebConstants.RA_LY_APP_DOC_URL, environment.getString()
					.expandUrl("$LY_APP_DOC$"));
			request.getSession().setAttribute(LyWebConstants.RA_LY_LEON_DOC_URL, leonardiRootUrl);
			request.getSession().setAttribute(LyWebConstants.RA_LY_LEON_DOC_EXTERNAL_URL,
					leonardiExternalUrl);
			request.getSession().setAttribute(LyWebConstants.RA_LY_SKIN_URL, cssPath);
			request.getSession().setAttribute(LyWebConstants.RA_LY_APP_DOC_URL,
					environment.getString().expandUrl("$LY_APP_DOC$"));

			String	logo = environment.getImageUrl("LY_COMPANY_LOGO");
			String	cookieTheme = (viewManager != null ? viewManager.getCookieTheme(request) : null);
			
			request.getSession().setAttribute("language", environment.getLanguage());
			request.getSession().setAttribute("logo", logo);
			request.getSession().setAttribute("LY_JQUERY_EXIT_PAGE", environment.getUrl("LY_JQUERY_EXIT_PAGE"));

			if (cookieTheme != null)
				request.getSession().setAttribute(LyJQConstants.COOKIE_THEME, cookieTheme);
		}
	} // end method setRequestAttributes

	/**
	 * redirect to home page.<br />
	 * @param request current http request
	 * @param response current http response
	 * @return
	 * @return
	 */
	protected ModelAndView redirectToPage(HttpServletRequest request, LyJQViewManager viewManager, String path)
	{
		ModelAndView	modelAndView = new ModelAndView(LyJQConstants.VM_AJAX_REDIRECTION_NAME);
		String			requestRootUrl = ((viewManager != null) && (viewManager.getRootURL() != null) ? viewManager.getRootURL() + "/" : "");
		boolean			isAjaxMode = ((request != null) && "true".equals(request.getParameter(LyJQConstants.AJAX_MODE)));
		
		path = LyJQController.completeUrlParams(path, request, viewManager);
		modelAndView.addObject("redirectUrl", requestRootUrl + path);
		modelAndView.addObject("ajaxMode", isAjaxMode);

		if (viewManager != null)
			viewManager.emptyRefreshScripts();
		
		return modelAndView;
	}
	
	/**
	 * Completes a given URL with needed parameters such as target code.
	 * 
	 * @param url         the URL that must be completed
	 * @param request     current Http request (may be null)
	 * @param viewManager current view manager (may be null)
	 * @return the URL completed with needed parameters
	 */
	public static String completeUrlParams(String url, HttpServletRequest request, LyJQViewManager viewManager)
	{
		String	target = null;
		
		if (request != null)
			target = (String)request.getParameter(LyWebConstants.TARGET_PARAM);
		
		if ((target == null) && (viewManager != null))
			target = viewManager.getTargetCode();
		
		if ((target != null) && (target.length() > 0) && !target.equals("null"))
			url += "?" + LyWebConstants.TARGET_PARAM + "=" + target;
		
		return url;
	}
	
	/**
	 * Check if current session is valid
	 */
	public boolean checkSessionValidity(LySession session) throws Exception
	{
		boolean isSessionValid = ((session != null) && session.isRegistered()
				&& (session.getState() != LySession.State.CLOSING) && (session.getState() != LySession.State.CLOSED));
		return isSessionValid;
	}

	/**
	 * Gets the view manager related to the current session.
	 * 
	 * @return the view manager related to the current session.
	 */
	public LyJQViewManager getViewManager(LySession session)
	{
		if (session == null)
			return null;
		
		LyWebViewManager	viewManager = (LyWebViewManager)session.getViewManager();
		
		if (viewManager instanceof LyMobileViewManager)
			viewManager = ((LyMobileViewManager) viewManager).getWebViewManager();
		
		return (LyJQViewManager)viewManager;
	}

	public abstract LySession getSession();

	public abstract LyDemand getDemand(HttpServletRequest request, LyEnvironment environment);

	protected abstract void beginHandleRequest(LyJQViewManager viewManager);

	protected abstract void endHandleRequest(LyJQViewManager viewManager);
	
	protected abstract boolean checkSession(HttpServletRequest request);

	protected abstract String getParameter(HttpServletRequest request, String name);
}
