package leon.view.web.jquery.controller;

import java.io.*;

import javax.servlet.*;
import javax.servlet.http.*;

import leon.app.*;
import leon.misc.*;
import leon.view.event.*;
import leon.view.web.*;
import leon.view.web.jquery.*;

import org.springframework.stereotype.*;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.*;

/**
 * Type : leon.view.web.struts.action.LyStrutsCloseAction<br>
 * This is the Struts action for closing a Leonardi user session.
 * @version $Id: LyStrutsCloseAction.java 26282 2010-07-15 13:54:42Z nhamon $
 * @author Lyria S.A.
 */
@Controller
public class LyJQExitController extends LyJQController
{

	@Override
	@RequestMapping("/exit")
	public ModelAndView handleRequest(HttpServletRequest request, HttpServletResponse response)
			throws Exception
	{
		LySession		session = getSession();
		LyApplication	application = (session == null ? null : session.getApplication());
		LyEnvironment	environment = (application == null ? null : application.getEnvironment());
		String			exitPage = getExitPage(request, environment);
		
		// Get the HTTP session
		if (exitPage != null)
			goToExitPage(request, response, environment);
		else
			return redirectToPage(request, null, LyJQConstants.PATH_ACTION_HOME);

		return null;
	}
	
	/**
	 * Computes the url of the exit page.
	 * 
	 * @param request     The HTTP request we are processing
	 * @param environment Current environment
	 * @return Default implementation returns the value for LY_JQUERY_EXIT_PAGE resource.
	 */
	protected String getExitPage(HttpServletRequest request, LyEnvironment environment)
	{
		String		exitPage = null;
		HttpSession	session = request.getSession(false);
		
		if (session != null)
			exitPage = (String)session.getAttribute("LY_JQUERY_EXIT_PAGE");
		else if (environment != null)
			exitPage = environment.getUrl("LY_JQUERY_EXIT_PAGE");
		
		return exitPage;
	}

	@Override
	public LyDemand getDemand(HttpServletRequest request, LyEnvironment environment)
	{
		return null;
	}

	public void goToExitPage(HttpServletRequest request, HttpServletResponse response,
			LyEnvironment environment) throws ServletException, IOException
	{
		String	exitLink = getExitPage(request, environment);

		if ((exitLink != null) && (!exitLink.equals("LY_JQUERY_EXIT_PAGE"))
				&& (exitLink.trim().length() > 0))
		{
			String language = (String) request.getSession().getAttribute("language");
			String logo = (String) request.getSession().getAttribute("logo");
			String cookieTheme = (String) request.getSession().getAttribute(LyJQConstants.COOKIE_THEME);

			exitLink = getContextRelativePath(request, exitLink);
			
			if (request.getSession(false) != null)
			{
				if (language == null)
					language = "fr";
			}
			
			exitLink += "?" + LyWebConstants.LANGUAGE + "=" + language;
			exitLink += "&logo=" + logo;
			
			if (cookieTheme != null)
				exitLink += "&" + LyJQConstants.COOKIE_THEME + "=" + cookieTheme;
			
			request.getSession(false).getServletContext().getRequestDispatcher(exitLink).forward(request, response);
		}
	}
}
