package leon.view.web.jquery.controller;

import java.io.*;

import javax.servlet.*;
import javax.servlet.http.*;

import leon.app.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;
import leon.view.web.*;
import leon.view.web.jquery.*;

import org.springframework.stereotype.*;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.*;

/**
 * Invoke action controller
 * 
 * @author atalbi
 */
@Controller
public class LyJQInvokeActionController extends LyJQController
{
	public static final String INVOKE_URL_MAPPING = "/invoke";

	@Override
	@RequestMapping(INVOKE_URL_MAPPING)
	public ModelAndView handleRequest(HttpServletRequest request, HttpServletResponse response)
		throws ServletException, IOException
	{
		if (checkSession(request))
		{
			LySession		session = getSession();
			LyApplication	application = session.getApplication();			
			String			queryString = request.getQueryString();
			
			try
			{
				session.getSoftLock();

				session.resetTimeout();
	
				String		controllerId = getParameter(request, LyWebConstants.CONTROLLER);
				String		action = getParameter(request, LyWebConstants.ACTION);
				LyAction	actionInfo = (action == null) ? null : application.getAction(action);
	
				// if the action is null, this action is useless
				if (actionInfo == null)
					return null;
	
				// To avoid the creation of multiple controllers for the same action
				// search for a static action previously stored.
				LyViewController	viewController = null;
				boolean				invokeAction = true;
				boolean				contextualAction = (actionInfo.getMin() > 0);
	
				// If action has already been display, retrieve the view controller
				if (!contextualAction)
				{
					if (controllerId != null)
					{
						viewController = (LyViewController)session.getController(controllerId);
						invokeAction = (viewController == null);
					}
				}
	
				if (invokeAction)
				{
					// get the controller and the view controller
					LyController controller = null;
	
					if (getEnvironment().testResource("LY_WEB_DIRECT_URL_NAVIGATION"))
					{
	                    if (!queryString.contains("_dashboard=true"))
	                        session.addValue("CLOSE_APPLICATION_ON_VALIDATE", "true");
	
						LyClassInfo		classInfo = null;
						LyObjectList	context = null;
						String			classInfoParam = getParameter(request, LyWebConstants.CLASSINFO);
	
						if (classInfoParam != null)
						{
							classInfo = session.getApplication().getClassInfo(classInfoParam);
	
							if (classInfo != null)
							{
								String[] objectIdStr = request.getParameterValues(LyWebConstants.OBJECTID);
								String[] objectNameStr = request.getParameterValues(LyWebConstants.OBJECTNAME);
	
								context = getDirectUrlNavigationContext(session, classInfo, objectIdStr, objectNameStr);
	
	                            if (context != null)
	                            {
	                                // Find real class info
	                                LyInfoList<LyClassInfo> classes = context.getClasses();
	
	                                if ((classes != null)
	                                        && (classes.getSize() == 1))
	                                    classInfo = classes.getInfo(0);
	                            }
							}
						}
						controller = invokeLeonardiAction(session, actionInfo, classInfo, context);
					}
					else
						controller = invokeLeonardiAction(session, actionInfo, null, null);
	
					viewController = (controller == null) ? null : controller.getViewController();
	
					// Register the new static action
					if (viewController != null)
					{
						// Do not free this controller when closing it (may happen
						// when the 'close all views' item
						// is used from the dashboard menubar.
						if (!contextualAction)
							viewController.setFreeOnClose(false);
					}
				}
	
				if (viewController != null)
				{
					if (viewController.getView() instanceof LyDialog)
						((LyDialog)viewController.getView()).setTarget(null);
	                
					viewController.showView();
	
	                if (contextualAction)
	                    session.controllerHidden(viewController);
	
	                LyView			currentMainView = viewController.getView();
					LyJQViewManager	viewManager = getViewManager(session);
					ModelAndView	modelAndView = new ModelAndView(LyJQConstants.HOME_TEMPLATE);;
					
					viewManager.emptyRefreshScripts();
					
					modelAndView.addObject("environment", session.getEnvironment());
					modelAndView.addObject("charset", LyString.DEFAULT_CHARSET);
									
					if (currentMainView != null)
					{
						modelAndView.addObject(LyJQConstants.DIALOG_PARAM, currentMainView);
						modelAndView.addObject("title", currentMainView.getEnvironment().translate(currentMainView instanceof LyDialog ?
								((LyDialog)currentMainView).getLabel() : ((LyFrame)currentMainView).getLabel()));
						modelAndView.addObject("targetCode", viewManager.getTargetCode());
						modelAndView.addObject("rootUrl", getRootUrlFromRequest(request));
						modelAndView.addObject("fullDialogPage", true);
						//System.gc();
						
						return modelAndView;
					}
				}
			}
			finally
			{
				session.releaseSoftLock();
			}
		}
		else
		{
			storeRedirectionUrl(request);
			
			request.getSession().getServletContext().getRequestDispatcher("/" + LyJQConstants.PATH_ACTION_HOME).forward(request, response);
		}

		return null;
	}

	protected void storeRedirectionUrl(HttpServletRequest request)
	{
		String scheme = request.getScheme();
		String serverName = request.getServerName();
		int serverPort = request.getServerPort();
		String servletPath = request.getServletPath();
		String contextPath = request.getContextPath();
		String queryString = request.getQueryString();

		if (queryString.contains("&" + LyWebConstants.NEW_SESSION + "=true"))
			queryString = queryString.replace("&" + LyWebConstants.NEW_SESSION
					+ "=true", "");

		request.setAttribute("REDIRECT_ACTION_URL", scheme + "://" + serverName
				+ ":" + serverPort + contextPath + servletPath + "?"
				+ queryString);
	}
	
	/**
	 * Direct url navigation : compute the context for the request.
	 * 
	 * @param session
	 *            current user session
	 * @param classInfo
	 *            targetClassinfo to look into
	 * @param objectIdStr
	 *            list of ids to add in the context
	 * @param objectNameStr
	 *            list of object names to adsd in the context
	 * @return the list of objects if any
	 */
	protected LyObjectList getDirectUrlNavigationContext(LySession session,
			LyClassInfo classInfo, String[] objectIdStr, String[] objectNameStr)
	{
		if (classInfo == null)
			return null;

		// Try to retrieve objects by their identifiers
		LySimpleObjectList	context = null;
		int					i, n = ((objectIdStr == null) ? 0 : objectIdStr.length);
		LyObjectId			objectId;

		for (i = 0; i < n; i++)
		{
			objectId = new LyObjectId(classInfo, objectIdStr[i]);

			if (objectId.getLogicalObject() != null)
			{
				if (context == null)
					context = new LySimpleObjectList();

				context.addObject(objectId.getLogicalObject());
			}
		}

		// Try to retrieve objects by their names
		LyObjectList objectsByName = session.getObjectsByName(new LyInfoList<LyClassInfo>(classInfo), objectNameStr);

		if ((objectsByName != null) && (objectsByName.getSize() > 0))
		{
			if (context == null)
				context = new LySimpleObjectList();

			context.addList(objectsByName);
		}

		return context;
	} // end method getDirectUrlNavigationContext

	/**
	 * Asks the given session to invoke the given leonardi action.<br>
	 * It uses the last view controller on the session to manage the context on
	 * the newly invoked action.
	 * 
	 * @param bean
	 *            Current session bean.
	 * @param action
	 *            a Leonardi action.
	 * @param classInfo
	 *            a classinfo parameter for the action (default is
	 *            action.getClassInfo() when set to null)
	 * @return a controller.
	 */
	protected LyController invokeLeonardiAction(LySession session,
			LyAction action, LyClassInfo classInfo, LyObjectList context)
	{
		if (classInfo == null)
			classInfo = action.getClassInfo();

		int					steps = 0;
		LyViewController	viewController = session.getLastViewController(steps);
		LyAreaController	parent = null;

		if (viewController != null)
		{
			parent = viewController.getAreaController();

			if ((parent instanceof LyDashboardController)
					&& (((LyDashboardController) parent).getDisplayedAreaController() != null))
				parent = ((LyDashboardController) parent).getDisplayedAreaController();
		}

		if (action.getMin() < 0)
		{
			// MIN < 0 : means any selection allowed, use content if nothing selected
			if (context == null)
			{
				if (action.getMax() == 0)
					context = parent.getContent();
				else if (parent != null)
				{
					// take selection, even if any...
					context = parent.getSelection();

					// ... otherwise take content
					if ((context == null) || (context.getSize() == 0))
						context = parent.getContent();
				}
			}
		}
		else if (action.getMin() == 0)
		{
			// MIN = 0 : means any selection allowed, use empty if nothing selected
			if (action.getMax() == 0)
			{
				// MIN = 0 and MAX = 0 action is independant from selection
				if (classInfo == null)
					classInfo = action.getClassInfo();

				if (classInfo == null)
					classInfo = parent.getClassInfo();
			}
			else if (context == null)
				// take selection, even if empty
				context = parent.getSelection();
		}
		else if ((parent != null) && (context == null))
			// take selection, even if empty
			context = parent.getSelection();

		if ((classInfo == null) && (parent != null))
			classInfo = parent.getClassInfo();

		// ask the session to process the action
		return session.invokeAction(parent, action, classInfo, context, false);
	} // end method invokeLeonardiAction
}
