package leon.view.web.jquery.controller;

import java.awt.*;

import javax.servlet.http.*;

import leon.app.*;
import leon.misc.*;
import leon.view.*;
import leon.view.event.*;
import leon.view.web.*;
import leon.view.web.jquery.*;
import leon.view.web.jquery.component.map.*;

import org.springframework.stereotype.*;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.*;

/**
 * Selection event spring controller
 * @author bsmichet
 */

@Controller
public class LyJQMapController extends LyJQController
{
	public LyJQMapController()
	{
		super();
	}

	/**
	 * @see leon.view.web.jquery.controller.LyJQEventController#handleRequest(javax.servlet.http.HttpServletRequest,
	 * javax.servlet.http.HttpServletResponse)
	 */
	@RequestMapping("/map")
	public ModelAndView handleRequest(HttpServletRequest request, HttpServletResponse response)
			throws Exception
	{
		// Check if session is registered, otherwise redirect to home page
		if (checkSession(request))
		{
			LySession		session = getSession();
			LyJQViewManager	viewManager = getViewManager(session);
			LyWebViewManager	sessionViewManager = (LyWebViewManager)session.getViewManager();
			
			try
			{
				session.getSoftLock();

				beginHandleRequest(viewManager);
	
				// Get some request parameters
				String source = getParameter(request, LyWebConstants.SOURCE);
				String value = getParameter(request, LyWebConstants.VALUE);
				String index = getParameter(request, LyWebConstants.INDEX);
				String controller = getParameter(request, LyWebConstants.CONTROLLER);
		
				// Transmit the action demand
				if ((source != null) && (controller != null))
				{
					// get the demand type
					int demandType = LyActionDemand.TYPE_ACTION;
		
					LyActionDemand demand = new LyActionDemand(demandType, source);
		
					demand.setValue(value);
					demand.setParameterValues(new LyJQParameterValues(request, session.getEnvironment().getString()));
		
					sessionViewManager.processDemand(controller, demand);
				}
				
				LyJQMap		jqMap = (LyJQMap)viewManager.getCurrentTreeMapView().getInternalView();
				LyMap		map = (jqMap != null ? (LyMap)jqMap.getViewComponent() : null);
		
				if ((jqMap != null) && "loadImage".equals(source) && (value != null))
				{
					String[]	divIds = LyString.stringToList(value, '*');
					int			i, nDivs = (divIds == null ? 0 : divIds.length);
					String[]	values;
					for (i = 0; i < nDivs; i++)
					{
						values = LyString.stringToList(divIds[i], '_');
						
						if ((values != null) && (values.length == 5))
						{
							StringBuffer     script = new StringBuffer();
							String			imagePath = jqMap.generateMapImage(LyTools.intFromString(values[1]), LyTools.intFromString(values[2]),
									LyTools.intFromString(values[3]), LyTools.intFromString(values[4]), false);
							
							script.append("replaceDivInnerHtml(\"");
							script.append(jqMap.getLastFrameName()).append("\",\"");
							script.append(divIds[i]).append("\",\"<img src='" + imagePath + "' />\");");
			
							viewManager.addToUpdateScript(script.toString());
						}
					}
				}
				else if ((jqMap != null) && "translateMap".equals(source) && (value != null))
				{
					int	tx = 0, ty = 0;
					
					// Remove 'px' strings
					value = LyString.replace(value, "px", "");
					
					String[]	values = LyString.stringToList(value, '_');
					
					if ((values != null) && (values.length == 2))
					{
						tx = (int)LyTools.floatFromString(values[0]);
						ty = (int)LyTools.floatFromString(values[1]);
						jqMap.getMapCanvas()._scrollx += tx;
						jqMap.getMapCanvas()._scrolly += ty;
					}
					else
					{
						jqMap.getMapCanvas()._scrollx = 0;
						jqMap.getMapCanvas()._scrolly = 0;
					}
		
					jqMap.updateViewerImage();
				}
				else if ((jqMap != null) && "setMapZoomScale".equals(source) && (value != null))
				{
					// Remove 'px' strings
					value = LyString.replace(value, "px", "");
					
					String[]	values = LyString.stringToList(value, '_');
					float		offsetX = 0;
					float		offsetY = 0;
		
					float	zoomStep = map.getZoomStep();
					
					if (zoomStep <= 1)
						zoomStep = 2;
					
					int		scale = LyTools.intFromString(values[0]);
					float	zoomFactor = (float)(Math.pow(zoomStep, scale));
					boolean	wasIdle = map.isIdle();
					
					map.setIdle(true);
		
					int	xx, yy;
					
					if (values.length == 3)
					{
						xx = (int)LyTools.floatFromString(values[1]);
						yy = (int)LyTools.floatFromString(values[2]);
					}
					else
					{
						xx = -jqMap.getMapCanvas()._scrollx;
						yy = -jqMap.getMapCanvas()._scrolly;
					}
					
					float	x1 = (xx / map.getZoomFactor()) * zoomFactor;
					float	y1 = (yy / map.getZoomFactor()) * zoomFactor;
					float	x2 = xx;
					float	y2 = yy;
					
					offsetX = x2 - x1;
					offsetY = y2 - y1;
					jqMap.getMapCanvas()._scrollx += offsetX;
					jqMap.getMapCanvas()._scrolly += offsetY;
		
					map.setZoomFactor(zoomFactor);
					map.setIdle(wasIdle);
				}
				else if ((jqMap != null) && "translateViewer".equals(source) && (value != null))
				{
					// Remove 'px' strings
					value = LyString.replace(value, "px", "");
					
					String[]	values = LyString.stringToList(value, '_');
					
					if ((values != null) && (values.length == 2))
					{
						LyJQMapCanvas	mapCanvas = jqMap.getMapCanvas();
						
						mapCanvas.resetTranslation();
						
						Rectangle scrollRectangle = mapCanvas.getScrollRectangle();
						Dimension dimension = jqMap.getMapDimension();
						float	  deltaX = -LyTools.intFromString(values[0]) + LyJQMapCanvas.VIEWER_SIZE / 2;
						float	  deltaY = -LyTools.intFromString(values[1]) + LyJQMapCanvas.VIEWER_SIZE / 2;
						float     ratioX = ((float)scrollRectangle.width) / (LyJQMapCanvas.VIEWER_SIZE);
						float     ratioY = ((float)scrollRectangle.height) / (LyJQMapCanvas.VIEWER_SIZE);
						float	  ratio = Math.max(ratioX, ratioY);
						int		  tX = (int)(deltaX * ratio);
						int		  tY = (int)(deltaY * ratio);
		
						mapCanvas._scrollx = tX;
						mapCanvas._scrolly = tY;
						mapCanvas.setSize(dimension.width, dimension.height);
						mapCanvas.setZoom(map.getZoomFactor());
						mapCanvas.resetTranslation();
		
						Rectangle		boundingRect = mapCanvas.getBoundingRectangle();
						StringBuffer    script = new StringBuffer();
						String requestRootUrl = getRootUrlFromRequest(request);
						String			actionUrl = requestRootUrl + "/" + LyJQConstants.PATH_ACTION_MAP;
						
						script.append("setMapOffset(\"mapContent_");
						script.append(map.getIdCode());
						script.append("\",");
						script.append((boundingRect.x + mapCanvas._scrollx));
						script.append(",");
						script.append((boundingRect.y + mapCanvas._scrolly));
						script.append(");");
						script.append("updateLoadedImages(\"" + jqMap.getId() + "\", \"" + map.getRootController() + "\", \"" + actionUrl + "\");");
						
						viewManager.addToUpdateScript(script.toString());
						jqMap.updateViewerImage();
					}
				}
				else if ((jqMap != null) && "moveElements".equals(source) && (value != null))
				{
					// Remove 'px' strings
					index = LyString.replace(index, "px", "");
					
					String[]	indexes = LyString.stringToList(index, '|');
					
					if ((indexes != null) && (indexes.length == 2))
					{
						LyActionDemand	demand = new LyActionDemand(LyActionDemand.TYPE_MOVE, value);
						
						if (map.getStyle() == LyMap.Style.STANDARD)
						{
							LyJQMapCanvas	mapCanvas = jqMap.getMapCanvas();
							
							mapCanvas.setSize(jqMap.getMapDimension());
							mapCanvas.setZoom(map.getZoomFactor());
							mapCanvas.resetTranslation();
							
							Rectangle		boundingRect = mapCanvas.getBoundingRectangle();
							Point			position = new Point(LyTools.intFromString(indexes[0]) + boundingRect.x,
									LyTools.intFromString(indexes[1]) + boundingRect.y);
							LyPoint			point = mapCanvas.revert(position);

							demand.setValue(point.getX() + "|" + point.getY());
						}
						else
							demand.setValue(LyTools.floatFromString(indexes[0]) + "|" + LyTools.floatFromString(indexes[1]));

						demand.setParameterValues(new LyJQParameterValues(request, session.getEnvironment().getString()));
		
						sessionViewManager.processDemand(controller, demand);

						if (map.getStyle() == LyMap.Style.STANDARD)
						{
							jqMap.updateBackgroundImage();
							jqMap.updateViewerImage();
						}
					}
				}
				
				writeUpdateScripts(response, viewManager);
				endHandleRequest(viewManager);
			}
			finally
			{
				session.releaseSoftLock();
			}
		}
		else
			return redirectToPage(request, null, LyJQConstants.PATH_ACTION_HOME);

		return null;
	}
}