package leon.view.web.jquery.controller;

import leon.app.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;
import leon.view.event.*;
import leon.view.web.*;
import leon.view.web.jquery.*;
import leon.view.web.jquery.component.dialog.*;
import leon.view.web.jquery.component.frame.*;

import org.springframework.stereotype.*;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.*;

import java.io.*;

import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;


/**
 * Web 2 page controller
 * @author atalbi
 */
@Controller public class LyJQPageController extends LyJQController
{

    protected static String FRAME_CONTAINS_DIALOGS_PARAM = "containsDialogs";

    /**
     * @see leon.view.web.jquery.controller.LyJQEventController#handleRequest(javax.servlet.http.HttpServletRequest,
     * javax.servlet.http.HttpServletResponse)
     */
    @RequestMapping("/home")
    public ModelAndView handleRequest(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {

        /** Check if a leonardi session exist. create it if necessary **/

        checkAndCreateSession(request);

        LySession       session = getSession();
        LyEnvironment   environment = getEnvironment();
        LyJQViewManager viewManager = getViewManager(session);
        viewManager.setIsPortlet(false);

        response.setHeader("Content-Type", LyString.DEFAULT_CHARSET);

        /** If the session is not valid : invoke 'root action' **/
        if (!session.isRegistered())
        {
            viewManager.setServletContext(request.getSession().getServletContext());
            // No caching for this page.
            setNoCacheHeaders(response, 0);

            // Start the session which invokes root Action
            session.register(true);
            // Call the getDecoration() method to be sure that CSS file is
            // generated
            viewManager.getDecoration();

            // set the navigator type
            String userAgent = request.getHeader("User-Agent");

            if (userAgent != null)
                viewManager.setBrowser(userAgent);
        }
        else
        {
            ServletContext context = viewManager.getServletContext();

            if (context == null)
                viewManager.setServletContext(request.getSession().getServletContext());
        }

        try
        {
            session.getSoftLock();

            // Update language or skin if parameter is found from the request
            String language = getParameter(request, LyWebConstants.LANGUAGE);
            String ajax = getParameter(request, LyWebConstants.AJAX);

            // If a reload of the main page is performed, stop push process : another push will be performed
            // when main page will be loaded.
            if ((ajax == null) || !ajax.equalsIgnoreCase("true"))
                viewManager.setStopPush();

            boolean useThemeSelector = environment.testResource("LY_JQUERY_SHOW_THEME_SELECTOR");

            if ((language != null) && !language.equals(""))
                session.setLanguage(language);

            if (!session.isValid())
            {
                // Check if root action is _login
                LyApplication application = session.getApplication();
                LyAction      rootAction = application.getProject().getRootAction();
                String        rootId = rootAction.getRoot().getId();

                if (rootId.equals(LyAction.ID_LOGIN))
                {
                    String loginPage = environment.getUrl("LY_JQUERY_LOGIN_PAGE");

                    if ((loginPage != null) && (!loginPage.equals("LY_JQUERY_LOGIN_PAGE"))
                            && (loginPage.trim().length() > 0))
                    {
                        LyTemplatesUtilsBean templateManager = viewManager.getVeloTemplatesBean();
                        Map<String, Object>  params = new HashMap<String, Object>();

                        if (!viewManager.getViewList().isEmpty())
                            params.put(LyJQConstants.AREA_PARAM,
                                viewManager.getViewList().get(0).getArea());

                        params.put("isLoginPage", true);
                        params.put("environment", environment);

                        String areaDiv = templateManager.mergeTemplate(
                                LyJQConstants.VM_AREA_TEMPLATE, params);

                        loginPage = getContextRelativePath(request, loginPage);
                        setRequestAttributes(request, session);
                        request.setAttribute("loginFormContent", areaDiv);
                        request.getSession(false).getServletContext().getRequestDispatcher(
                            loginPage).forward(request, response);
                        endHandleRequest(viewManager);

                        return null;
                    }
                    else
                    {
                        viewManager.emptyRefreshScripts();

                        ModelAndView modelAndView = new ModelAndView(LyJQConstants.HOME_TEMPLATE);

                        modelAndView.addObject("environment", session.getEnvironment());
                        modelAndView.addObject("charset", LyString.DEFAULT_CHARSET);
                        modelAndView.addObject("showFmkLoginDialog", true);
                        modelAndView.addObject("targetCode", viewManager.getTargetCode());
                        modelAndView.addObject("showThemeSelector", false);
                        modelAndView.addObject(LyJQConstants.DIALOG_PARAM,
                            viewManager.getViewList().get(0));
                        endHandleRequest(viewManager);

                        return modelAndView;
                    }
                }
            }

            session.resetTimeout();

            String redirect = (String) session.getValue("REDIRECT_ACTION_URL");

            if ((redirect != null) && !redirect.equals(""))
            {
                session.addValue("REDIRECT_ACTION_URL", "");
                response.sendRedirect(redirect);
                endHandleRequest(viewManager);

                return null;
            }

            // set no cache
            setNoCacheHeaders(response, 0);

            // store usefull attributes
            setRequestAttributes(request, session);

            String idComponent = request.getParameter("id");
            String idController = request.getParameter("controller");

            ModelAndView modelAndView = new ModelAndView(LyJQConstants.HOME_TEMPLATE);

            modelAndView.addObject("environment", session.getEnvironment());
            modelAndView.addObject("viewManager", viewManager);
            modelAndView.addObject("charset", LyString.DEFAULT_CHARSET);
            modelAndView.addObject("showThemeSelector", useThemeSelector);

            String cookieTheme = viewManager.getCookieTheme(request);

            if (cookieTheme != null)
                modelAndView.addObject("cookieTheme", cookieTheme);

            if ((idComponent != null) && (idController != null))
            {
                LyActionDemand demand = new LyActionDemand(LyActionDemand.TYPE_ACTION, idComponent);

                session.getViewManager().processDemand(idController, demand);
            }

            // Get the LY_UP_PAGE resource
            String upPage = environment.getUrl("LY_JQUERY_UP_PAGE");

            if ((upPage != null) && !upPage.equals("LY_JQUERY_UP_PAGE"))
            {
                // get the path relative to the context
                String upPageActionUrl = getRootUrlFromRequest(request) + "/"
                    + getContextRelativePath(request, LyJQConstants.PATH_ACTION_UPFRAME);

                modelAndView.addObject("useUpFrame", true);
                modelAndView.addObject("upPageActionUrl", upPageActionUrl);
            }

            Vector<LyView> views = viewManager.getViewList();
            String         containsDialogs = null;

            if (views.size() <= 0)
                return null;

            int    i, n = views.size();
            LyView mainView = null, viewTmp;
            String target;

            // Retrieve first main view (with no target)
            for (i = n - 1; i >= 0; i--)
            {
                viewTmp = views.elementAt(i);
                target = null;

                if (viewTmp != null)
                {

                    if (viewTmp instanceof LyDialog)
                        target = ((LyDialog) viewTmp).getTarget();
                    else if (viewTmp instanceof LyFrame)
                        target = ((LyFrame) viewTmp).getTarget();

                    if ((target == null)
                            || (!LyWebConstants.POPUP_VIEW.equals(target)
                                && !LyJQConstants.STICKER_TARGET.equals(target)))
                    {
                        mainView = viewTmp;
                        break;
                    }
                }
            }

            if (mainView == null)
                mainView = views.elementAt(n - 1);

            if (mainView instanceof LyDialog)
            {
                modelAndView.addObject(LyJQConstants.DIALOG_PARAM, mainView);
                modelAndView.addObject("title",
                    environment.translate(((LyDialog) mainView).getLabel()));
                modelAndView.addObject("targetCode", viewManager.getTargetCode());
                modelAndView.addObject("rootUrl", getRootUrlFromRequest(request));
                modelAndView.addObject("fullDialogPage", true);
                endHandleRequest(viewManager);

                return modelAndView;
            }

            modelAndView.addObject(LyJQConstants.FRAME_PARAM, mainView);
            modelAndView.addObject("title", environment.translate(((LyFrame) mainView).getLabel()));
            modelAndView.addObject("targetCode", viewManager.getTargetCode());
            modelAndView.addObject("rootUrl", getRootUrlFromRequest(request));

            if (views.size() > 1)
            {
                // Retrieve popup views (with a given target)
                LyView     view;
                LyFrame    frame;
                LyDialog   dialog;
                LyJQFrame  jqFrame;
                LyJQDialog jqDialog;

                containsDialogs = "";

                for (i = 0; i < n; i++)
                {
                    view = views.get(i);

                    if (view instanceof LyFrame)
                    {
                        frame = (LyFrame) view;
                        target = frame.getTarget();

                        if (LyWebConstants.POPUP_VIEW.equals(target))
                        {
                            jqFrame = (LyJQFrame) frame.getInternalView();

                            if (jqFrame.isVisible())
                                containsDialogs += jqFrame.addDialogFrameRefreshScript();
                        }
                    }
                    else if (view instanceof LyDialog)
                    {
                        dialog = (LyDialog) view;
                        target = dialog.getTarget();

                        if (LyWebConstants.POPUP_VIEW.equals(target)
                                || LyJQConstants.STICKER_TARGET.equals(target))
                        {
                            jqDialog = (LyJQDialog) dialog.getInternalView();

                            if (jqDialog.isVisible())
                                containsDialogs += jqDialog.addDialogRefreshScript();
                        }
                    }
                }

                modelAndView.addObject(FRAME_CONTAINS_DIALOGS_PARAM, containsDialogs);
            }

            views = null;

            endHandleRequest(viewManager);

            return modelAndView;
        }
        finally
        {
            session.releaseSoftLock();
        }
    }

    /**
     * @see leon.view.web.jquery.controller.LyJQController#getDemand(javax.servlet.http.HttpServletRequest,
     * leon.misc.LyEnvironment)
     */
    @Override public LyDemand getDemand(HttpServletRequest request, LyEnvironment environment)
    {
        return null;
    }

    /**
     * Returns the JSP used for the login.
     *
     * @param LySession current session for this request
     * @return the url of the JSP used for the login.
     */
    protected String getLoginPage(LySession session)
    {
        LyEnvironment environment = session.getEnvironment();
        // Get the LY_JQUERY_LOGIN_PAGE ressource
        return environment.getUrl("LY_JQUERY_LOGIN_PAGE");
    }
}
