package leon.view.web.jquery.controller;

import java.io.*;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

import leon.app.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;
import leon.view.web.*;
import leon.view.web.jquery.*;

import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.*;

public class LyJQPortletController extends LyJQController
{
	public static final String PORTLET_URL_MAPPING = "/portlet";
	public String _controllerId = "";

	@RequestMapping(PORTLET_URL_MAPPING)
	public ModelAndView handleRequest(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException
	{
	    checkAndCreateSession(request);

		if (checkSession(request))
		{
			LySession		session = getSession();
			LyApplication	application = session.getApplication();
			LyJQViewManager	viewManager = getViewManager(session);

			viewManager.setIsPortlet(true);
			response.setHeader("Content-Type", LyString.DEFAULT_CHARSET);

			/** If the session is not valid : invoke 'root action' **/
			if (!session.isRegistered())
			{
				viewManager.setServletContext(request.getSession().getServletContext());

				// No caching for this page.
				setNoCacheHeaders(response, 0);

				// Start the session which invokes root Action
				session.register(true);
				// Call the getDecoration() method to be sure that CSS file is
				// generated
				viewManager.getDecoration();

				// set the navigator type
				String userAgent = request.getHeader("User-Agent");

				if (userAgent != null)
					viewManager.setBrowser(userAgent);
			}

			// Update language or skin if parameter is found from the request
			String language = getParameter(request, LyWebConstants.LANGUAGE);
			String ajax = getParameter(request, LyWebConstants.AJAX);

			// If a reload of the main page is performed, stop push process :
			// another push will be performed
			// when main page will be loaded.
			if ((ajax == null) || !ajax.equalsIgnoreCase("true"))
				viewManager.setStopPush();

			if ((language != null) && !language.equals(""))
				session.setLanguage(language);

			session.resetTimeout();

			String		controllerId = getParameter(request, LyWebConstants.CONTROLLER);
			String		action = getParameter(request, LyWebConstants.ACTION);
			String		hideTitlebarParameter = getParameter(request, "hideTitlebar");
			String		hideToolbarParameter = getParameter(request, "hideToolbar");
			String		hideCommandbarParameter = getParameter(request, "hideCommandbar");
			boolean		hideTitlebar = "yes".equals(hideTitlebarParameter);
			boolean		hideToolbar = "yes".equals(hideToolbarParameter);
			boolean		hideCommandbar = "yes".equals(hideCommandbarParameter);
			LyAction	actionInfo = (action == null) ? null : application.getAction(action);


			if (actionInfo == null)
			{
				return null;
			}

			boolean actionValid = session.validateAction(actionInfo, session.getController(0));
			if (!actionValid)
			{
			    return null;
			}

			// To avoid the creation of multiple controllers for the same action
			// search for a static action previously stored.
			LyViewController	viewController = null;
			boolean				invokeAction = true;
			boolean				contextualAction = (actionInfo.getMin() > 0);

			// If action has already been display, retrieve the view controller
			if (!contextualAction)
			{
				if (controllerId != null)
				{
					viewController = (LyViewController)session.getController(controllerId);
					invokeAction = (viewController == null);
				}
			}

			if (invokeAction)
			{
				// get the controller and the view controller
				LyController	controller = null;

				controller = invokeLeonardiAction(session, actionInfo, hideTitlebar, hideToolbar, hideCommandbar);
				viewController = (controller == null) ? null : controller.getViewController();

				// Register the new static action
				if (viewController != null)
				{
					// Do not free this controller when closing it (may happen
					// when the 'close all views' item
					// is used from the dashboard menubar.
					if (!contextualAction)
						viewController.setFreeOnClose(false);
				}
			}

			if (viewController != null)
			{
				if (viewController.getView() instanceof LyDialog)
					((LyDialog) viewController.getView()).setTarget(null);

				viewController.showView();

				if (contextualAction)
					session.controllerHidden(viewController);

				LyView currentMainView = viewController.getView();
				ModelAndView modelAndView = new ModelAndView(LyJQConstants.HOME_TEMPLATE);

				String portletNamespace = getParameter(request, "portletNamespace");

				viewManager.emptyRefreshScripts();

				modelAndView.addObject("environment", session.getEnvironment());
				modelAndView.addObject("charset", LyString.DEFAULT_CHARSET);
				modelAndView.addObject("portletNamespace", portletNamespace);

				if (currentMainView != null)
				{
					modelAndView.addObject(LyJQConstants.DIALOG_PARAM, currentMainView);
					modelAndView.addObject("title", currentMainView.getEnvironment().translate(
						currentMainView instanceof LyDialog ? ((LyDialog)currentMainView).getLabel()
								: ((LyFrame)currentMainView).getLabel()));
					modelAndView.addObject("targetCode", viewManager.getTargetCode());
					modelAndView.addObject("rootUrl", getRootUrlFromRequest(request));
					modelAndView.addObject("fullDialogPage", true);

					return modelAndView;
				}
			}
		}
		else
		{
			storeRedirectionUrl(request);

			request.getSession().getServletContext()
					.getRequestDispatcher("/" + LyJQConstants.PATH_ACTION_HOME)
					.forward(request, response);
		}

		return null;
	}

	protected void storeRedirectionUrl(HttpServletRequest request)
	{
		String	scheme = request.getScheme();
		String	serverName = request.getServerName();
		int		serverPort = request.getServerPort();
		String	servletPath = request.getServletPath();
		String	contextPath = request.getContextPath();
		String	queryString = request.getQueryString();

		if (queryString != null)
		{
			request.setAttribute("REDIRECT_ACTION_URL", scheme + "://"
					+ serverName + ":" + serverPort + contextPath + servletPath
					+ "?" + queryString);
		}
	}

	/**
	 * Direct url navigation : compute the context for the request.
	 *
	 * @param session
	 *            current user session
	 * @param classInfo
	 *            targetClassinfo to look into
	 * @param objectIdStr
	 *            list of ids to add in the context
	 * @param objectNameStr
	 *            list of object names to adsd in the context
	 * @return the list of objects if any
	 */
	protected LyObjectList getDirectUrlNavigationContext(LySession session,
			LyClassInfo classInfo, String[] objectIdStr, String[] objectNameStr)
	{
		if (classInfo == null)
			return null;

		// Try to retrieve objects by their identifiers
		LySimpleObjectList	context = null;
		int					i, n = ((objectIdStr == null) ? 0 : objectIdStr.length);
		LyObjectId			objectId;

		for (i = 0; i < n; i++)
		{
			objectId = new LyObjectId(classInfo, objectIdStr[i]);

			if (objectId.getLogicalObject() != null)
			{
				if (context == null)
					context = new LySimpleObjectList();

				context.addObject(objectId.getLogicalObject());
			}
		}

		// Try to retrieve objects by their names
		LyObjectList objectsByName = session.getObjectsByName(new LyInfoList<LyClassInfo>(classInfo), objectNameStr);

		if ((objectsByName != null) && (objectsByName.getSize() > 0))
		{
			if (context == null)
				context = new LySimpleObjectList();

			context.addList(objectsByName);
		}

		return context;
	} // end method getDirectUrlNavigationContext

	/**
	 * Asks the given session to invoke the given leonardi action.<br>
	 * It uses the last view controller on the session to manage the context on
	 * the newly invoked action.
	 *
	 * @param bean
	 *            Current session bean.
	 * @param action
	 *            a Leonardi action.
	 * @param classInfo
	 *            a classinfo parameter for the action (default is
	 *            action.getClassInfo() when set to null)
	 * @return a controller.
	 */
	protected LyController invokeLeonardiAction(LySession session,
			LyAction action, boolean hideTitlebar, boolean hideToolbar, boolean hideCommandbar)
	{
		try
		{
			LyClassInfo			classInfo = action.getClassInfo();
			LyObjectList		context = null;
			int					steps = 0;
			LyViewController	viewController = session.getLastViewController(steps);
			LyAreaController	parent = null;

			if (viewController != null)
			{
				parent = viewController.getAreaController();

				if ((parent instanceof LyDashboardController)
						&& (((LyDashboardController) parent).getDisplayedAreaController() != null))
					parent = ((LyDashboardController) parent).getDisplayedAreaController();
			}

			if (action.getMin() < 0)
			{
				// MIN < 0 : means any selection allowed, use content if nothing
				// selected
				if (action.getMax() == 0)
					context = parent.getContent();
				else if (parent != null)
				{
					// take selection, even if any...
					context = parent.getSelection();

					// ... otherwise take content
					if ((context == null) || (context.getSize() == 0))
						context = parent.getContent();
				}
			}
			else if (action.getMin() == 0)
			{
				// MIN = 0 : means any selection allowed, use empty if nothing
				// selected
				if (action.getMax() == 0)
				{
					// MIN = 0 and MAX = 0 action is independant from selection
					if (classInfo == null)
						classInfo = action.getClassInfo();

					if (classInfo == null)
						classInfo = parent.getClassInfo();
				}
				// take selection, even if empty
				context = parent.getSelection();
			}
			else if ((parent != null) && (context == null))
				// take selection, even if empty
				context = parent.getSelection();

			if ((classInfo == null) && (parent != null))
				classInfo = parent.getClassInfo();

			// If titlebar, toolbar or commandbar must be hidden, create a new action
			if (hideTitlebar || hideToolbar || hideCommandbar)
			{
				// Check if action already exists
				String			computedActionId = computeNewInfoId(action, hideTitlebar, hideToolbar, hideCommandbar);
				LyApplication	application = session.getApplication();
				LyAction		newAction = application.getAction(computedActionId);

				if (newAction == null)
				{
					Hashtable<String, Object>	parameters = new Hashtable<String, Object>();

					if (hideTitlebar)
						parameters.put("A_MARKS.noTitlebar", Boolean.TRUE);

					if (hideToolbar)
						parameters.put("A_MARKS.noToolbar", Boolean.TRUE);

					if (hideCommandbar)
						parameters.put("A_MARKS.noCommands", Boolean.TRUE);

					newAction = new LyAction(computedActionId, action, classInfo, parameters);

					// If preferences exist on action, create new preferences for newly created action
					LyInfoList<LyPreference>	preferences = application.getInfoList(LyPreference.class);
					LyPreference				preference, newPreference;
					String						newPreferenceId;
					int							i, n = (preferences != null ? preferences.getSize() : 0);

					for (i = 0; i < n; i++)
					{
						preference = preferences.getInfo(i);

						if (preference.getObjectInfo() == action)
						{
							newPreferenceId = computeNewInfoId(preference, hideTitlebar, hideToolbar, hideCommandbar);
							newPreference = (LyPreference)application.getInfo(newPreferenceId);

							if (newPreference == null)
							{
								newPreference = preference.clonePreference(newPreferenceId);
								newPreference.setObjectInfo(newAction);
								newPreference.setApplication(application);
							}
						}
					}
				}

				action = newAction;
			}

			// ask the session to process the action
			LyController	controller = session.invokeAction(parent, action, classInfo, null, false);

			return controller;
		}
		catch (Exception ex)
		{
			session.getEnvironment().writeException(ex);
			return null;
		}
	} // end method invokeLeonardiAction

	/**
	 * Computes the identifier of a newly created info instance.
	 * @param info template info instance
	 * @param hideTitlebar indicates whether the title bar is hidden in the portlet view
	 * @param hideToolbar indicates whether the tool bar is hidden in the portlet view
	 * @param hideCommandbar indicates whether the command bar is hidden in the portlet view
	 * @return the computed identifier of a newly created info instance
	 */
	protected String computeNewInfoId(LyInfo info, boolean hideTitlebar, boolean hideToolbar, boolean hideCommandbar)
	{
		String	computedInfoId = info.getId();

		if (hideTitlebar)
			computedInfoId += "_hideTitlebar";

		if (hideToolbar)
			computedInfoId += "_hideToolbar";

		if (hideCommandbar)
			computedInfoId += "_hideCommandbar";

		return computedInfoId;
	}
}
