package leon.view.web.jquery.controller;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import leon.app.LySession;
import leon.control.*;
import leon.misc.*;
import leon.view.LyComponent;
import leon.view.event.LyActionDemand;
import leon.view.event.LyDemand;
import leon.view.setinterface.LySetPrintModeInterface;
import leon.view.web.*;
import leon.view.web.jquery.LyJQConstants;
import leon.view.web.jquery.LyJQViewManager;
import leon.view.web.struts.LyStrutsParameterValues;

import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.ModelAndView;

/**
 * JQuery print controller
 * @author amt
 */
@Controller
public class LyJQPrintController extends LyJQController
{

	/**
	 * Print forward Url
	 */
	public static final String	PRINT_URL					= "/print";

	protected static String		NEW_PRINT_WINDOW_TEMPLATE	= "ajax/newPrintWindowScript.vm";

	protected static String		PRINT_MODE_PARAM			= "printMode";

	/**
	 * @see leon.view.web.jquery.controller.LyJQController#getDemand(javax.servlet.http.HttpServletRequest,
	 * leon.misc.LyEnvironment)
	 */
	@Override
	public LyDemand getDemand(HttpServletRequest request, LyEnvironment environment)
	{
		return null;
	}

	/**
	 * handleRequest
	 * @see org.springframework.web.servlet.mvc.Controller#handleRequest(javax.servlet.http.HttpServletRequest,
	 * javax.servlet.http.HttpServletResponse)
	 */
	@RequestMapping(PRINT_URL)
	@Override
	public ModelAndView handleRequest(HttpServletRequest request, HttpServletResponse response)
			throws Exception
	{
		if (checkSession(request))
		{
			String controllerId = getParameter(request, LyWebConstants.CONTROLLER);
			String source = getParameter(request, LyWebConstants.SOURCE);
			String value = getParameter(request, LyWebConstants.VALUE);
			String index = getParameter(request, LyWebConstants.VALUE);
			String forContent = getParameter(request, "_forContent");

			LySession		session = getSession();
			LyEnvironment	environment = session.getEnvironment();
			LyJQViewManager	viewManager = getViewManager(session);
			LyWebViewManager	sessionViewManager = (LyWebViewManager)session.getViewManager();

			try
			{
				session.getSoftLock();

				beginHandleRequest(viewManager);
				viewManager.setPrintMode(true);

				if ("true".equals(forContent))
				{
					LyController    controllerUsingSourceId = null;
					LyController    controller = getController(request, session, viewManager);
					int			    printPos = source.indexOf("_print");

					source = source.substring(printPos + 6);

					// Try to get controller from source
					controllerUsingSourceId = session.getController(source);

					if ((controllerUsingSourceId != null) && (controllerUsingSourceId != controller))
						controller = controllerUsingSourceId;

					LyComponent		 component = controller.getComponent();

					if (controller instanceof LySetPrintModeInterface)
						((LySetPrintModeInterface)controller).setPrintMode(true);

					ModelAndView modelAndView = new ModelAndView(LyJQConstants.HOME_TEMPLATE);

					modelAndView.addObject(PRINT_MODE_PARAM, true);
					modelAndView.addObject("environment", session.getEnvironment());
					modelAndView.addObject("charset", LyString.DEFAULT_CHARSET);
					modelAndView.addObject("component", component);
					endHandleRequest(viewManager);

					return modelAndView;
				}
				else
				{
					if (source == null) // at this point the source is supposed to be
					{
						LyComponent		lyView = viewManager.getCurrentPrintView();

						if (lyView != null)
						{
							viewManager.setCurrentPrintView(null);

							ModelAndView		modelAndView = new ModelAndView(LyJQConstants.HOME_TEMPLATE);
							LyViewController	lastViewController = session.getLastViewController();

							modelAndView.addObject(LyJQConstants.DIALOG_PARAM, lyView);
							modelAndView.addObject("environment", session.getEnvironment());
							modelAndView.addObject("charset", LyString.DEFAULT_CHARSET);
							modelAndView.addObject("controllerToFree", lastViewController);
							modelAndView.addObject(PRINT_MODE_PARAM, true);
							viewManager.emptyRefreshScripts();
							endHandleRequest(viewManager);

							return modelAndView;
						}
						else
						{
							if (controllerId != null)
							{
								LyController	controller = null;

								controller = session.getController(controllerId);

								LyComponent component = controller.getComponent();

								if (component != null)
								{
									viewManager.setCurrentPrintView(null);
									ModelAndView modelAndView = new ModelAndView(LyJQConstants.HOME_TEMPLATE);

									modelAndView.addObject(LyJQConstants.AREA_PRINT_PARAM, component);
									modelAndView.addObject("environment", session.getEnvironment());
									modelAndView.addObject("charset", LyString.DEFAULT_CHARSET);
									modelAndView.addObject(PRINT_MODE_PARAM, true);
									endHandleRequest(viewManager);

									return modelAndView;
								}
							}
						}

						endHandleRequest(viewManager);

						return null;
					}

					if (controllerId != null)
					{
						// process an action demand on the current controller
						LyActionDemand demand = new LyActionDemand(LyActionDemand.TYPE_ACTION,
								source.intern());

						demand.setValue(value);
						demand.setParameterValues(new LyStrutsParameterValues(request, environment
								.getString()));
						demand.addIndex(LyTools.intFromString(index));

						// Invoke action on specified controller
						sessionViewManager.processDemand(controllerId, demand);
					}

					if (source != null && source.equals(LyJQConstants.PRINT_LIST))
						viewManager.addToUpdateScript(getNewPrintWidowScript(controllerId, source, true, viewManager));
					else
						viewManager.addToUpdateScript(getNewPrintWidowScript(controllerId, null, false, viewManager));
				}

				writeUpdateScripts(response, viewManager);
				endHandleRequest(viewManager);
			}
			finally
			{
				session.releaseSoftLock();
			}
		}
		else
			return redirectToPage(request, null, LyJQConstants.PATH_ACTION_HOME);

		return null;
	}

	/**
	 * Searches the controller concerned by the printing.
	 *
	 * @param  request     Current HTTP request that contains information about the controller.
	 * @param  session     Current Leonardi session.
	 * @param  viewManager Current view manager.
	 * @return the controller involved in the request.
	 */
	protected LyController getController(HttpServletRequest request, LySession session,
		LyJQViewManager viewManager)
	{
		LyController controller = null;
		String		 pageId = getParameter(request, LyWebConstants.PAGE);
		String		 controllerId = getParameter(request, LyWebConstants.CONTROLLER);

		if (pageId != null)
		{
			LyWebPageWriter pw = viewManager.getPageWriter(pageId);

			if (pw != null)
			{
				LyComponent component = pw.getViewComponent();

				controllerId = component.getRootController();
			}
		}

		if (controllerId != null)
			controller = session.getController(controllerId);

		return controller;
	}

	private String getNewPrintWidowScript(String controller, String source, boolean forContent, LyJQViewManager viewManager)
	{
		String	url = LyJQConstants.PATH_ACTION_PRINT;

		if (viewManager.isPortletMode())
        {
            String appDoc = getEnvironment().getEnv("LY_APP_DOC");
            url = appDoc + '/' + url;
        }

		if (((controller != null) && !controller.equals(""))
				|| ((source != null) && !source.equals(""))
				|| forContent)
		{
			url = url + "?";

			if ((controller != null) && !controller.equals(""))
				url = url + LyWebConstants.CONTROLLER + "=" + controller;

			if ((source != null) && !source.equals(""))
				url = url + "&" + LyWebConstants.SOURCE + "=" + source;

			if (forContent)
				url = url + "&" + "_forContent" + "=" + forContent;
		}


		Map<String, Object>	params = new HashMap<String, Object>();

		params.put("url", url);

		LyTemplatesUtilsBean	templateManager = viewManager.getVeloTemplatesBean();
		String					newWindowScript = templateManager.mergeTemplate(NEW_PRINT_WINDOW_TEMPLATE, params);

		params = null;

		return newWindowScript;
	}

}
