package leon.view.web.jquery.controller;

import java.util.*;
import java.util.zip.*;

import javax.servlet.http.*;

import leon.app.*;
import leon.misc.*;
import leon.view.event.*;
import leon.view.web.jquery.*;

import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.*;

/**
 * Push Spring controller
 * @author atalbi
 */
public class LyJQPushController extends LyJQController
{	
	/** Time to sleep for push thread (in milliseconds). */
	public static int		PUSH_CHECK_DELAY = 500;
	
	/** Max delay for push response (in milliseconds). */
	public static int		PUSH_MAX_DELAY = 60000;
	
	/** Number of iterations for push response. */
	public static int		PUSH_ITERATIONS = PUSH_MAX_DELAY / PUSH_CHECK_DELAY;
	
	/** Last time when a push refresh came from the browser. */
	protected Hashtable<HttpSession, Long>		_lastPushTimes;
	
	
	public LyJQPushController()
	{
		_lastPushTimes = new Hashtable<HttpSession, Long>();
	}

	@Override
	public LyDemand getDemand(HttpServletRequest request, LyEnvironment environment)
	{
		return null;
	}

	@Override
	@RequestMapping("/push")
	public ModelAndView handleRequest(HttpServletRequest request, HttpServletResponse response)
			throws Exception
	{
		if (checkSession(request))
		{
			LySession		session = getSession();
			LyJQViewManager	viewManager = getViewManager(session);
			LyJQViewManager	sharingViewManager = viewManager.getSharingManager(request.getSession());
			
			if (sharingViewManager == null)
				sharingViewManager = viewManager;
			
			String	      timeParam = getParameter(request, "_time");
			long		  pushStartTime = System.currentTimeMillis();
			long          time = LyTools.longFromString(timeParam);
			Long          lastPushTime;
			int           loop = 0;
			boolean	      stop = false;
			boolean	      recurse = true;
			HttpSession   httpSession = request.getSession();
			StringBuffer  scripts = new StringBuffer();
			
			lastPushTime = _lastPushTimes.get(httpSession);
			
			if ((lastPushTime == null)
			        || (time > lastPushTime.longValue()))
				_lastPushTimes.put(httpSession, time);
			
			while ((loop < PUSH_ITERATIONS)
					&& !stop
					&& (time >= (_lastPushTimes.get(httpSession)).longValue()))
			{
				Thread.sleep(PUSH_CHECK_DELAY);

				if (isSessionClosed(session))
				{
					stop = true;
					recurse = false;

	                LyEnvironment    environment = (session != null ? session.getEnvironment() : null);
	                
	                if ((environment != null) && (environment.testEnv("LY_REPLAY")))
	                {
						LyTemplatesUtilsBean	templateManager = viewManager.getVeloTemplatesBean();
						Map<String, Object>		params = new HashMap<String, Object>();
						
						params.put("redirectUrl", "../../" + LyJQConstants.PATH_ACTION_EXIT);
						scripts.append(templateManager.mergeTemplate("ajax/redirectionScript.vm", params));
               		}

	                break;
				}
				
				try
				{
					session.getSoftLock();
					
					String	updateScripts = null;
					
					if (!sharingViewManager.isRequestInProgress())
						updateScripts = getUpdateScripts(sharingViewManager);
					
					// If view manager has a more recent stop push time, stop push process.
					if (pushStartTime < sharingViewManager.getStopPushTime())
					{
						endHandleRequest(null);
						return null;
					}
					
					if ((updateScripts != null) && (updateScripts.length() > 0))
					{
	    				stop = true;
	    				updateScripts = sharingViewManager.getUpdateScripts(true);
	    				scripts.append(updateScripts);
					}
				}
	            catch (Exception e)
	            {
	                // An exception occurred but it will logged on next refresh, trace only.
	                LyEnvironment    environment = session.getEnvironment();
	                
	                environment.logTrace("Leonardi : refresh error detected on push.");
	            }
				finally
				{
					session.releaseSoftLock();
				}
				
				loop++;
			}

			if (recurse && (time >= (_lastPushTimes.get(httpSession)).longValue()))
				scripts.append("this.frames['push_frame'].PushRefresh();");

			response.setContentType("text/html; charset=" + LyString.DEFAULT_CHARSET);
			response.setHeader("Content-Encoding", "gzip");

			GZIPOutputStream gzos = new GZIPOutputStream(response.getOutputStream());
			
			gzos.write(scripts.toString().getBytes(LyString.DEFAULT_CHARSET));
			gzos.close();
			viewManager.setPrintMode(false);
			response.setHeader("Content-Length", gzos.toString().length() + "");
			endHandleRequest(null);
		}
		else
		{
			return redirectToPage(request, null, "../../" + LyJQConstants.PATH_ACTION_HOME);
		}

		return null;
	}
	
	/**
	 * Returns the update scripts registered on view manager.
	 */
	protected String getUpdateScripts(LyJQViewManager viewManager)
	{
		return viewManager.getUpdateScripts(false);
	}
}
