package leon.view.web.jquery.controller;

import java.io.*;

import javax.servlet.http.*;

import leon.app.*;
import leon.control.*;
import leon.control.form.*;
import leon.misc.*;
import leon.view.*;
import leon.view.web.*;
import leon.view.web.jquery.*;

import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.*;

/**
 * File Upload controller
 * 
 * @author atalbi
 */
public class LyJQUploadController extends LyJQController
{
	@RequestMapping("/upload")
	@Override
	public ModelAndView handleRequest(HttpServletRequest request, HttpServletResponse response) throws Exception
	{
		if (checkSession(request))
		{
			String contentType = request.getContentType();

			if ((contentType != null)
					&& (contentType.indexOf("multipart/form-data") >= 0))
			{
				LySession			session = getSession();
				String				attachmentDirectory = getParameter(request, "directory");
				String				attachmentHandler = getParameter(request, "handler");
				LyView				view = null;
				String				controllerId = getParameter(request, LyWebConstants.CONTROLLER);
				LyController		controller = (controllerId == null ? null : session.getController(controllerId));
				LyJQViewManager		viewManager = getViewManager(session);
				boolean				useCustomizedFilecomponent = viewManager.useCustomizedFilecomponent();

				try
				{
					session.getSoftLock();

					beginHandleRequest(viewManager);
	
					if (attachmentDirectory == null)
						attachmentDirectory = "files";
	
					if ((attachmentHandler == null) || attachmentHandler.equals("default"))
						attachmentHandler = "leon.view.web.LyFileAttachmentHandler";
	
					Class<?> handlerClass = Class.forName(attachmentHandler);
	
					LyFileAttachmentHandler	handler = (LyFileAttachmentHandler)handlerClass.newInstance();
	
					handler.setAttachmentDirectory(attachmentDirectory);
	
					// Set parameters
					handler.setEnvironment(session.getEnvironment());
					handler.setRequest(request);
	
					// Parse stream content and attachment
					try
					{
						handler.parseContent();
					}
					catch (Exception e)
					{
						session.getEnvironment().writeException(e);
						viewManager.setMessage(e.getMessage());
					}
	
					// get parameter part request
					HttpServletRequest parameterPartRequest = handler.getParameterPartRequest();
					LyFilePartWriter   writer = handler.getFilePartWriter();
	
					// may save to delete list : delete temp file when controller is freed
					if (controller instanceof LyFileFieldSetController) 
						((LyFileFieldSetController)controller).addTempFilePath(writer.getServerFileName());
					
					writer.commit(true);
	
					// free the file handler
					handler.free();
	
					if (controller != null)
						view = controller.getView();
	
					if (view != null)
					{
						LyJQParameterValues	values = new LyJQParameterValues(parameterPartRequest, session.getEnvironment().getString());
						
						values.updateViewFromValues(view, true);
					}
					
					endHandleRequest(viewManager);
					
					if (!useCustomizedFilecomponent)
					{
						StringBuffer	buffer = new StringBuffer();
						
						buffer.append("<html>");
						buffer.append("<head>");
						buffer.append("<script type='text/javascript' src='./leon/scripts/jquery/jquery/jquery.min.js'></script>");
						buffer.append("<script type='text/javascript' src='./leon/scripts/jquery/jquery/ajax/eventsUtils.js'></script>");
						buffer.append("<script type='text/javascript' src='./leon/scripts/LyUtilsScript.js'></script>");
						buffer.append("<script type='text/javascript' src='./leon/scripts/jquery/LyJQUtils.js'></script>");
						buffer.append("<script type='text/javascript' src='./leon/scripts/jquery/jquery/form/text/ckeditor/ckeditor.js'></script>");
						buffer.append("</head> ");
						buffer.append("<body>\n");
						buffer.append("<script type='text/javascript'>jQuery(document).ready(function() {\n");
						buffer.append("    getLeonardiTopFrame().handleUploadFinished();");
						buffer.append("});</script>\n");
						buffer.append("</body>\n");
						buffer.append("</html>\n");
	
						response.setContentType("text/html; charset=" + LyString.DEFAULT_CHARSET);
						
						PrintWriter	pw = response.getWriter();
	
						pw.print(buffer);
						pw.close();
						getViewManager(session).setPrintMode(false);
						response.setHeader("Content-Length", buffer.length() + "");
					}
				}
				finally
				{
					session.releaseSoftLock();
				}
			}
		}
		else
		{
			return redirectToPage(request, null, LyJQConstants.PATH_ACTION_HOME);
		}

		return null;
	}
}
