package leon.view.web.jquery.controller;

import java.io.IOException;
import java.security.Principal;
import java.util.Map;
import java.util.Vector;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletContext;
import javax.portlet.PortletException;
import javax.portlet.PortletMode;
import javax.portlet.PortletModeException;
import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequestDispatcher;
import javax.portlet.ReadOnlyException;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.portlet.ValidatorException;

import leon.data.LyObjectId;

import leon.app.LyDataSession;

import leon.app.LySession;

import leon.app.LyApplication;
import leon.info.LyAction;
import leon.info.LyCompoundAction;
import leon.info.LyInfoList;
import leon.view.web.jquery.LyJQConstants;
import org.springframework.context.ApplicationContext;
import org.springframework.web.portlet.context.PortletApplicationContextUtils;

public class LyPortletManager extends GenericPortlet
{
    /**
     * Url that must be called to display the portlet.
     */
    public static final String PORTLET_URL = "/portlet.htm";

    /**
     * Leonardi application instance.
     */
    private LyApplication      _application;

    @Override
    protected void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException
    {
        getApplication(request, response);

        if (isLoginNeeded(request, response) && !isPortalUserLogged(request))
        {
            if (_application != null && _application.getEnvironment() != null)
            {
                request.setAttribute("message",
                        _application.getEnvironment().getMessage("LY_ER_CONFIG_PORTLET_MUST_LOG"));
            }
            else
            {
                request.setAttribute("message", "You must sign in to access this page");
            }

            getPortletContext().getRequestDispatcher(LyJQConstants.PORTLET_ERROR_PAGE).include(request, response);
            return;
        }

        String portalUser = getPortalLoggedUser(request);

        PortletPreferences preferences = request.getPreferences();
        String action = preferences.getValue("action", "");
        String hideTitlebar = preferences.getValue("hideTitlebar", "");
        String hideToolbar = preferences.getValue("hideToolbar", "");
        String hideCommandbar = preferences.getValue("hideCommandbar", "");

        String portletNameSpace = response.getNamespace();
        response.setContentType(request.getResponseContentType());

        String url = PORTLET_URL;

        url = addParameterToUrl(url, "_action", action);
        url = addParameterToUrl(url, "portletNamespace", portletNameSpace);
        url = addParameterToUrl(url, "hideTitlebar", hideTitlebar);
        url = addParameterToUrl(url, "hideToolbar", hideToolbar);
        url = addParameterToUrl(url, "hideCommandbar", hideCommandbar);
        url = addParameterToUrl(url, "_portlet_user_login", portalUser);

        PortletContext portletContext = getPortletContext();

        if (portletContext != null)
        {
            PortletRequestDispatcher requestDispatcher = portletContext.getRequestDispatcher(url);
            if (requestDispatcher != null)
            {
                requestDispatcher.include(request, response);
            }
            else
            {
                throw new PortletException("Portlet unavailable : cannot dispatch request");
            }
        }
    }

    public LySession getLeonardiSessionForPortletUser(RenderRequest request)
    {
        String userName = getPortalLoggedUser(request);

        if (_application != null)
        {
            Vector<LyDataSession> sessions = _application.getSessions();
           for (LyDataSession lyDataSession : sessions)
           {
               LyObjectId sessionUser = (LyObjectId)lyDataSession.getValue("$USER");

               if (sessionUser != null && userName.equals(sessionUser.toName()))
                   return (LySession)lyDataSession;
           }
        }
        return null;
    }

    public boolean isLoginNeeded(RenderRequest request, RenderResponse response) throws PortletException, IOException
    {
        LyAction rootAction = getRootAction(request, response);
        LyAction loginAction = _application.getAction(LyAction.ID_LOGIN);
        if (rootAction != null
                && (LyAction.ID_LOGIN.equals(rootAction.getId()) ||  rootAction.hasTemplate(loginAction)))
            return true;
        return false;
    }


    /**
    * Add http parameter to url.
    * add ?param=value after url if no parameter, and &param=value if there is already a parameter
    * @param url root url
    * @param parameter parameter
    * @param value value of parameter
    * @return url with added parameter
    */
    private String addParameterToUrl(String url, String parameter, String value)
    {
        if (!"".equals(value))
        {
            int qpos = url.indexOf('?');
            int hpos = url.indexOf('#');
            char sep = ((qpos == -1) ? '?' : '&');
            String seg = sep + parameter + '=' + value;
            return hpos == -1 ? url + seg : url.substring(0, hpos) + seg + url.substring(hpos);
        }
        else
        {
            return url;
        }
    }

    private boolean isPortalUserLogged(RenderRequest request)
    {
        Principal userPrincipal = request.getUserPrincipal();

        if (userPrincipal == null)
        {
            return false;
        }
        return true;
    }

    /**
     * Get portal current user login according to LyApplication configuration.
     *
     * @param request
     *            request.
     * @return user login
     */
    @SuppressWarnings("unchecked")
    private String getPortalLoggedUser(RenderRequest request)
    {
        Map<String, String> userInfo = null;

        if (request.getAttribute("javax.portlet.userinfo") != null)
        {
            userInfo = (Map<String, String>) request.getAttribute("javax.portlet.userinfo");
            if (_application != null && _application.getEnvironment() != null && userInfo != null)
            {
                String authField = _application.getEnvironment().getResource("LY_PORTLET_AUTHENTICATION_FIELD");
                return userInfo.get(authField);
            }
        }
        return "";
    }

    @Override
    public void processAction(ActionRequest request, ActionResponse response) throws PortletException, IOException
    {
        if (request.getPortletMode().equals(PortletMode.EDIT))
            processEditionAction(request, response);
    }

    /**
     * Processes the edition action.
     */
    private void processEditionAction(ActionRequest request, ActionResponse response) throws PortletModeException,
            ReadOnlyException, IOException, ValidatorException
    {
        // Retrieve parameters from edition form and store them on the
        // preferences
        String action = request.getParameter("action");
        String hideTitlebar = request.getParameter("hideTitlebar");
        String hideToolbar = request.getParameter("hideToolbar");
        String hideCommandbar = request.getParameter("hideCommandbar");
        PortletPreferences prefs = request.getPreferences();
        prefs.setValue("action", action);
        prefs.setValue("hideTitlebar", hideTitlebar);
        prefs.setValue("hideToolbar", hideToolbar);
        prefs.setValue("hideCommandbar", hideCommandbar);
        prefs.store();
        response.setPortletMode(PortletMode.VIEW);
    }


    @Override
    protected void doEdit(RenderRequest request, RenderResponse response) throws PortletException, IOException
    {
        getApplication(request, response);

        LyAction rootAction = getRootAction(request, response);

        response.setContentType(request.getResponseContentType());
        response.setContentType("text/html");
        if (rootAction == null)
        {
            if (_application != null && _application.getEnvironment() != null)
                request.setAttribute("message",
                        _application.getEnvironment().getMessage("LY_ER_CONFIG_PORTLET_NO_ACTION"));
            else
                throw new PortletException("Application unavailable");


            getPortletContext().getRequestDispatcher(LyJQConstants.PORTLET_ERROR_PAGE).include(request, response);
        }
        else
        {
            // Set attributes used to build edit page
            LyInfoList<LyAction> actions = new LyInfoList<LyAction>();
            getActions(rootAction, actions);

            if (isLoginNeeded(request, response))
            {
                actions = checkActionsList(actions, request);
            }

            request.setAttribute("application", _application);
            request.setAttribute("availableActions", actions);

            // Set values stored on preferences on the edit page
            PortletPreferences prefs = request.getPreferences();
            String action = prefs.getValue("action", " ");
            String hideTitlebar = prefs.getValue("hideTitlebar", " ");
            String hideToolbar = prefs.getValue("hideToolbar", " ");
            String hideCommandbar = prefs.getValue("hideCommandbar", " ");

            request.setAttribute("action", action);
            request.setAttribute("hideTitlebar", hideTitlebar);
            request.setAttribute("hideToolbar", hideToolbar);
            request.setAttribute("hideCommandbar", hideCommandbar);
            getPortletContext().getRequestDispatcher(LyJQConstants.PORTLET_EDIT_PAGE).include(request, response);
        }
    }

    private LyInfoList<LyAction> checkActionsList(LyInfoList<LyAction> actions, RenderRequest request)
    {
        LySession leonardiSession = getLeonardiSessionForPortletUser(request);
        if (leonardiSession == null)
            return actions;
        for (int i = 0; i < actions.getSize(); i++)
        {
            LyAction action = actions.getInfo(i);
            if (!leonardiSession.validateAction(action, leonardiSession.getController(0))
                    || !leonardiSession.enableAction(action, leonardiSession.getController(0), null, false))
            {
               actions.removeInfo(action);
            }
        }
        return actions;
    }

    /**
     * Retrieves the sub-actions of a given action.
     *
     * @param action
     *            the action whose sub-actions must be retrieved
     * @param actions
     *            the list that must be populated with the sub-actions
     */
    private void getActions(LyAction action, LyInfoList<LyAction> actions)
    {
        if (action == null)
        {
            return;
        }
        if (_application == null)
        {
            return;
        }

        LyInfoList<LyAction> subActions = null;
        if (action instanceof LyCompoundAction)
            subActions = ((LyCompoundAction) action).getSubActions();
        else
            subActions = action.getActions();

        if ((subActions == null) || (subActions.getSize() == 0))
            return;

        int i, n = subActions.getSize();
        LyAction subAction;

        for (i = 0; i < n; i++)
        {
            subAction = subActions.getInfo(i);
            actions.addInfo(subAction);
            getActions(subAction, actions);
        }
    }

    /**
     * Retrieve the application context for a given request.
     */
    private LySpringUtilsBean getApplicationContext(RenderRequest request)
    {
        PortletContext portletContext = this.getPortletContext();
        ApplicationContext context = PortletApplicationContextUtils.getWebApplicationContext(portletContext);
        LySpringUtilsBean applicationContext = (LySpringUtilsBean) context.getBean("applicationContext");

        return applicationContext;
    }

    /**
     * Retrieve the Leonardi application instance for a given request.
     * @throws PortletException
     * @throws IOException
     */
    private LyApplication getApplication(RenderRequest request, RenderResponse response) throws PortletException, IOException
    {
        LySpringUtilsBean applicationContext = getApplicationContext(request);

        if (applicationContext != null)
        {
            _application = LyApplication.getApplication(applicationContext.getApplicationId());
            if (_application == null)
            {
                PortletContext portletContext = this.getPortletContext();
                _application = LyJQController.createApplication(applicationContext.getApplicationId(),
                                                                applicationContext.getConfigFile(),
                                                                portletContext.getRealPath(""));
            }
        }
        else
        {
            throw new PortletException("Cannot find application context");
        }
        return _application;
    }

    /**
     * Retrieve the root action for a given request.
     * @throws PortletException
     */
    private LyAction getRootAction(RenderRequest request, RenderResponse response) throws PortletException, IOException
    {
        getApplication(request, response);
        if (_application != null && _application.getProject() != null)
        {
            return _application.getProject().getRootAction();
        }
        return null;
    }
}
