package leon.view.web.jquery.controller;

import java.util.*;

import javax.servlet.*;

import leon.app.*;

import org.apache.velocity.app.*;
import org.springframework.ui.velocity.*;
import org.springframework.web.context.*;

/**
 * templates utils class : used for instantiating velocity engine and merging templates
 * @author atalbi
 */
public class LyTemplatesUtilsBean	implements
									ServletContextAware
{
	public static final String		TEMPLATES_PACKAGE = "leon/view/web/jquery/templates/";
	
	/** templates properties resource **/
	protected Properties			_templatesProperties;

	protected VelocityEngine		_velocityEngine;	

	protected static ServletContext	_context;

	protected VelocityEngine		_engine;

	protected boolean				_isProdEnvironment	= true;
	
	protected boolean				_isPreloadApplication = false;
	
	protected String				_initName = null;
	
	protected String				_initFile = null;

	protected Thread				_preloadApplicationThread = null;

	
	public LyTemplatesUtilsBean()
	{
		_velocityEngine = new VelocityEngine();
	}

	public static ServletContext getContext()
	{
		return _context;
	}

	public void setVelocityEngine(VelocityEngine engine)
	{
		this._engine = engine;
	}

	/**
	 * @param templatePath
	 * @param params
	 * @return merged template ready to be rendered
	 */
	public String mergeTemplate(String templatePath, Map<String, Object> params)
	{
		return VelocityEngineUtils.mergeTemplateIntoString(_velocityEngine, templatePath, params);
	}

	public Properties getTemplatesProperties()
	{
		return _templatesProperties;
	}

	public void setIsProdMode(boolean isProdMode)
	{
		this._isProdEnvironment = isProdMode;
	}

	public void setIsPreloadApplication(boolean isPreloadApplication)
	{
		this._isPreloadApplication = isPreloadApplication;
	}

	public void setInitName(String initName)
	{
		this._initName = initName;
	}

	public void setInitFile(String initFile)
	{
		this._initFile = initFile;
	}

	/**
	 * @param templatesProperties
	 */
	public void setTemplatesProperties(Properties templatesProperties)
	{
		this._templatesProperties = templatesProperties;
	}

	/**
	 * Get template path for a given component class
	 * @param className
	 * @return template path
	 */
	public String getClassTemplate(String className)
	{
		return _templatesProperties.getProperty(className);
	}

	/**
	 * @see org.springframework.web.context.ServletContextAware#setServletContext (javax.servlet.ServletContext)
	 */
	public void setServletContext(ServletContext context)
	{
		_context = context;
		_velocityEngine.setApplicationAttribute("javax.servlet.ServletContext", context);
		
		if (_isProdEnvironment)
		{
			_velocityEngine.setProperty("resource.loader", "class");
			_velocityEngine.setProperty("class.resource.loader.class", "leon.view.web.jquery.controller.LyClasspathResourceLoader");
			_velocityEngine.setProperty("class.resource.loader.cache", "true");
			_velocityEngine.setProperty("class.resource.loader.modificationCheckInterval", "-1");
		}
		else
		{
			_velocityEngine.setProperty("resource.loader", "webapp");
			_velocityEngine.setProperty("webapp.resource.loader.class", "org.apache.velocity.tools.view.servlet.WebappLoader");
			_velocityEngine.setProperty("webapp.resource.loader.path", "/templates/");
			_velocityEngine.setProperty("webapp.resource.loader.cache", "false");
		}
		
		_velocityEngine.setProperty("runtime.log.logsystem.class", "org.apache.velocity.runtime.log.NullLogSystem");
		_velocityEngine.setProperty("directive.set.null.allowed", true);
		_velocityEngine.setProperty("directive.parse.max.depth", "100");
		_velocityEngine.setProperty("resource.manager.cache.size", 100);
		_velocityEngine.setProperty("resource.manager.defaultcache.size", 100);
		_velocityEngine.setProperty("velocimacro.permissions.allow.inline.local.scope", true);
		_velocityEngine.setProperty("velocimacro.library", "");

		try
		{
			_velocityEngine.init();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		
		if (_isPreloadApplication)
		{
			LyApplication		application = LyApplication.getApplication(_initName);
			
			if ((application == null) && !isLoadingData())
			{
				Runnable r = new Runnable()
				{
					public void run()
					{
						LyJQController.createApplication(_initName, _initFile, _context.getRealPath(""));
						_preloadApplicationThread = null;
					};
				};
				
				_preloadApplicationThread = new Thread(r);
				_preloadApplicationThread.start();
			}
		}
	}
	
	public boolean isLoadingData()
	{
		return ((_preloadApplicationThread != null) && _preloadApplicationThread.isAlive());
	}
}
