package leon.view.web.jquery.mobile;

import leon.app.*;
import leon.control.*;
import leon.control.form.*;
import leon.misc.*;
import leon.view.*;
import leon.view.event.*;
import leon.view.mobile.*;
import leon.view.web.*;
import leon.view.web.jquery.*;
import leon.view.web.jquery.controller.*;
import leon.view.web.mobile.*;

import org.springframework.context.*;
import org.springframework.web.context.support.*;

import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;


public class LyJQMobileDecoder extends LyMobileDecodeRequest
{

    /**
     * Design Pattern : singleton.
     */
    private static LyJQMobileDecoder _instance = new LyJQMobileDecoder();

    /**
     * Constructeur.
     */
    public LyJQMobileDecoder()
    {
        super();
    }

    /**
     * Renvoie l'unique instance traitant les requêtes.
     * @return Cette classe.
     */
    public static LyJQMobileDecoder getInstance()
    {
        return _instance;
    }

    public void swapSession(Object newSession, Object oldSession, String applicationId)
    {
        LyWebRequestInfo requestInfo = LyJQServlet.getWebRequestInfo();
        HttpSession      httpSession = requestInfo.getRequest().getSession();
        String           httpSessionKey = httpSession.getId();
        _map.remove(httpSessionKey);
        _map.put(httpSessionKey, (LySession) oldSession);
    }

    @Override public void createSession(HttpServletRequest request, HttpServletResponse response)
    {
        HttpSession    httpSession = request.getSession();
        ServletContext context = httpSession.getServletContext();


        ApplicationContext appContext = WebApplicationContextUtils.getWebApplicationContext(
                context);
        LySpringUtilsBean  applicationContext = (LySpringUtilsBean) appContext.getBean(
                "applicationContext");

        _applicationId = applicationContext.getApplicationId();
        String initFile = applicationContext.getConfigFile();

        // Compute Base dir from request
        String path = context.getRealPath("");
        path = LyString.replace(path, "\\", "/");

        LyApplication application = LyApplication.getApplication(_applicationId);

        if (application == null)
        {
            application = new LyWebApplication(_applicationId, initFile, path, true);
            LyJQController.generateCssFile(application);
        }
        
        //Insertion de la variable pour l'affichage des labels.
        application.getEnvironment().putResource("LY_WRITE_FIELD_TOOL_LABEL", "true");

        LyJQSession         session = new LyJQSession(application, false);
        LyEnvironment       environment = session.getEnvironment();
        LyMobileViewManager viewManager = (LyMobileViewManager) environment.getViewManager(true);

        if ((viewManager != null) && (viewManager.getEnvironment() != null)
                && (viewManager.getRootURL() == null))
        {
            String rootUrl = computeRootUrl(request, environment);
            viewManager.setSession(session);
            viewManager.setRootURL(rootUrl);
            viewManager.setApplicationName(request.getContextPath());
            ((LyJQViewManager)viewManager.getWebViewManager()).
            	setServletContext(request.getSession().getServletContext());
        }

        String			httpSessionKey = LyJQController.getSessionKey(httpSession, null);

        session.setHttpSession(httpSession, httpSessionKey);
        
        _map.put(request.getSession(true).getId(), session);
        home(request, response);

    }

    @Override public void upload(String controllerId, String frame, String source, String dir,
        HttpServletRequest req) throws Exception
    {
        LySession session = getSession(req);

        if (LyWebConstants.POPUP_VIEW.equals(frame))
            frame = null;
        else if (LyWebConstants.TARGET_FRAME.equals(frame))
            frame = null;

        // Do we have any specific decoder here ?
        LyWebPageWriter pageWriter = null;
        LyView          view = null;
        LyController    controller = ((controllerId == null) ? null
                                                             : session.getController(controllerId));

        if (controller != null)
        {
            view = controller.getView();
            pageWriter = (LyWebPageWriter) view.getInternalView();
        }

        LyFileAttachmentHandler handler = null;

        if (pageWriter == null) // FFT 597
        {
            LyViewController lastViewController = session.getLastViewController();

            if (lastViewController != null)
                lastViewController.showView();
        }
        else
        {
            String attachmentHandler = pageWriter.getAttachmentHandler(frame);
            //String attachmentDirectory = pageWriter.getAttachmentDirectory(frame);

            if ((attachmentHandler == null) || attachmentHandler.equals("default"))
                attachmentHandler = "leon.view.web.LyFileAttachmentHandler";

            Class<?> handlerClass = Class.forName(attachmentHandler);

            handler = (LyFileAttachmentHandler) handlerClass.newInstance();
            handler.setAttachmentDirectory(dir);
        }

        // Use default one if none specified.
        if (handler == null)
        {
            handler = new LyFileAttachmentHandler();
            handler.setAttachmentDirectory("files");
        }

        // Set parameters
        handler.setEnvironment(session.getEnvironment());
        handler.setRequest(req);

        // Parse stream content and attachment
        try
        {
            handler.parseContent();
        }
        catch (Exception e)
        {
            session.getEnvironment().writeException(e);

            getViewManager(session).setMessage(e.getMessage());
        }

        // get parameter part request
        HttpServletRequest parameterPartRequest = handler.getParameterPartRequest();
        LyFilePartWriter   writer = handler.getFilePartWriter();

        // may save to delete list : delete temp file when controller is freed
        if (controller instanceof LyFileFieldSetController)
            ((LyFileFieldSetController) controller).addTempFilePath(writer.getServerFileName());

        writer.commit(true);

        // free the file handler
        handler.free();

        if (view != null)
        {
            LyJQParameterValues values = new LyJQParameterValues(parameterPartRequest,
                    session.getEnvironment().getString());

            values.updateViewFromValues(view, true);
        }

    }

    @Override public boolean commandAction(HttpServletRequest request, HttpServletResponse response)
    {
        LySession session = getSession(request);
        String    controller = getParameter(request, LyWebConstants.CONTROLLER);

        try
        {
            session.getSoftLock();

            // Get some request parameters
            String frame = getParameter(request, LyWebConstants.FRAME);
            String source = getParameter(request, LyWebConstants.SOURCE);
            String value = getParameter(request, LyWebConstants.VALUE);
            String command = getParameter(request, LyWebConstants.COMMAND);

            if (source != null)
                source = source.intern();

            if (command != null)
                command = command.intern();

            // Manage null controller
            if ((controller == null) || (session.getController(controller) == null))
            {
                controller = manageNullController(request, response, session.getApplication(),
                        session, session.getEnvironment(), session.getViewManager(true));

                if (controller == null)
                    return false;
            }

            // if we are in login mode, session is not valid yet
            boolean login = !session.isValid();

            // Transmit the action demand
            if (source != null)
            {
                // get the demand type
                int demandType = LyActionDemand.TYPE_ACTION;

                LyActionDemand demand = new LyActionDemand(demandType, source);

                demand.setValue(value);
                demand.setParameterValues(new LyJQParameterValues(request,
                        session.getEnvironment().getString()));

                getViewManager(session).processDemand(controller, demand);
            }

            if (LyWebConstants.POPUP_VIEW.equals(frame))
                frame = LyWebConstants.TARGET_FRAME;
            else if (!login && ((frame == null) || (frame.trim().length() == 0)))
                frame = LyWebConstants.TARGET_FRAME;

            // We mustn't have pending transactions on the session.
            if (session.hasTransaction())
            {
                Exception e = new IllegalStateException(
                        "BF AppEngine : Remaining pending transactions");

                session.handleException(e);
            }


            String redirect = (String) session.getValue("CLOSE_APPLICATION_ON_VALIDATE");

            if ((redirect != null) && redirect.equals("true"))
                request.getSession().invalidate();

            if (getViewManager(session) == null)
            {
                response.addHeader(LyMobileConstants.EXIT, LyMobileConstants.TRUE);
                return true;
            }
            else
            {
                String message = getViewManager(session).getMessage();

                if (message != null)
                {
                    response.addHeader(LyMobileConstants.MESSAGE, LyMobileConstants.TRUE);
                    response.getWriter().print(message);
                    return true;
                }
            }

        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            session.releaseSoftLock();
        }

        return false;
    }

    @Override public LyDemand getDemand(HttpServletRequest request, LyEnvironment environment)
    {
        // get the event type
        String type = getParameter(request, LyWebConstants.TYPE);

        // get the source component
        String source = getParameter(request, LyWebConstants.SOURCE);

        source = ((source == null) ? "" : source.intern());

        // get the value of the demand
        String value = getParameter(request, LyWebConstants.VALUE);

        // get the index of the graphical event
        String index = getParameter(request, LyWebConstants.INDEX);
        int    subIndex = -1;

        // Manage sub-index given in value or index parameter
        if (value != null)
        {
            int pos = value.indexOf(LyWebConstants.SUB_INDEX_SEPARATOR);

            if (pos >= 0)
            {
                String subIndexValue = value.substring(pos
                        + LyWebConstants.SUB_INDEX_SEPARATOR.length());

                subIndex = LyTools.intFromString(subIndexValue);
                value = value.substring(0, pos);
            }
            else if (index != null)
            {
                pos = index.indexOf(LyWebConstants.SUB_INDEX_SEPARATOR);

                if (pos >= 0)
                {
                    String subIndexValue = index.substring(pos
                            + LyWebConstants.SUB_INDEX_SEPARATOR.length());

                    subIndex = LyTools.intFromString(subIndexValue);
                    index = index.substring(0, pos);
                }
            }
        }

        // compute the demand type
        int            demandType = LyActionDemand.TYPE_ACTION;
        Vector<String> selectionIds = getSelectionIds(request);

        if (LyWebConstants.POPUP_MENU.equals(source)
                || LyWebConstants.POPUP_MENU_EXTEND.equals(source))
        {
            String       positionX = getParameter(request, LyWebConstants.POSITION_X);
            String       positionY = getParameter(request, LyWebConstants.POSITION_Y);
            LyMenuDemand menuDemand = new LyMenuDemand(LyWebConstants.POPUP_MENU.equals(source)
                    ? LyMenuDemand.TYPE_POPUP : LyMenuDemand.TYPE_EXTEND_POPUP, value);

            if ((index != null) && (index.trim().length() != 0))
                menuDemand.addIndex(LyTools.intFromString(index));

            if (subIndex >= 0)
                menuDemand.setSubIndex(subIndex);

            menuDemand.setItem(index);
            menuDemand.setPosition(
                new int[] { LyTools.intFromString(positionX), LyTools.intFromString(positionY) });

            return menuDemand;
        }

        // Check if we have more accurate action type information.
        if ((type != null) && (type.length() != 0))
        {
            type = type.intern();

            if (type == "sort")
                demandType = LyActionDemand.TYPE_SORT;
            else if (type == "extendSort")
                demandType = LyActionDemand.TYPE_EXTEND_SORT;
            else if (type == "href")
                demandType = LyActionDemand.TYPE_HREF;
            else if (type == "expand")
                demandType = LyActionDemand.TYPE_EXPAND;
            else if (type == "open")
                demandType = LyActionDemand.TYPE_OPEN;
            else if (type == "move")
                demandType = LyActionDemand.TYPE_MOVE;
            else if (type == "delete")
                demandType = LyActionDemand.TYPE_DELETE;
            else if (type == "closeTab")
                demandType = LyActionDemand.TYPE_CLOSE_TAB;
            else if ((type != null) && (type.equals("_checkField")))
            {

                if ((source != null) && (source.startsWith("LyUnitRelationSelector.")))
                    demandType = LyActionDemand.TYPE_UNIT_CHANGED;
                else
                    demandType = LyActionDemand.TYPE_VALUE_CHANGED;
            }
            else if ((type != null) && (type.equals("_validateField")))
                // Validation of a field using the 'Enter' key
                value = type;
            // Manage tab next / previous in editable tables
            else if ((type != null) && (type.equals("_tabNext")))
                demandType = LyActionDemand.TYPE_TAB_NEXT;
            else if ((type != null) && (type.equals("_tabPrevious")))
                demandType = LyActionDemand.TYPE_TAB_PREVIOUS;
            else if ((type != null)
                    && (type.equals("_altRight") || type.equals("_altUp") || type.equals("_altLeft")
                        || type.equals("_altDown") || type.equals("_altAdd")))
                source = type;
        }

        // test the source component may be the status
        if (LyWebConstants.STATUS == source)
            demandType = LyActionDemand.TYPE_STATUS;

        // build the action demand
        LyActionDemand demand = new LyActionDemand(demandType, source);

        demand.setValue(value);
        demand.setParameterValues(new LyJQParameterValues(request, environment.getString()));

        if ((index != null) && (index.length() > 0))
            demand.addIndex(LyTools.intFromString(index));

        if (subIndex >= 0)
            demand.setSubIndex(subIndex);

        // JUST INDICATIVE : view may not know all selected items and
        // final decision is made by the controller
        demand.setViewSelection(selectionIds);

        // return the action demand
        return demand;
    }

    @Override protected String computeRootUrl(HttpServletRequest request, LyEnvironment environment)
    {
        String endUrl = request.getContextPath();
        String rootUrl = request.getRequestURL().toString();

        if (rootUrl.lastIndexOf(endUrl) != -1)
            rootUrl = rootUrl.substring(0, rootUrl.lastIndexOf(endUrl) + endUrl.length()+1);

        int index = rootUrl.lastIndexOf('/');

        if (index > 0)
            rootUrl = rootUrl.substring(0, index);

        // change the scheme according to SSL layer
        String httpScheme;

        if (environment.testEnv("LY_USE_HTTPS"))
            httpScheme = "https:";
        else
            httpScheme = request.getScheme() + ":";

        rootUrl = LyString.replace(rootUrl, "http:", httpScheme);
        rootUrl = LyString.replace(rootUrl, "https:", httpScheme);

        return rootUrl;
    }
}
