(function(jQuery) {

	jQuery.fn.stickyNotes = function(options) {
		jQuery.fn.stickyNotes.options = jQuery.extend({}, jQuery.fn.stickyNotes.defaults, options);
		jQuery.fn.stickyNotes.prepareContainer(this);
		jQuery.each(jQuery.fn.stickyNotes.options.notes, function(index, note){
			jQuery.fn.stickyNotes.renderNote(note);
			jQuery.fn.stickyNotes.notes.push(note);
		});
	};


	jQuery.fn.stickyNotes.getNote = function(note_id) {
		var result = null;
		jQuery.each(jQuery.fn.stickyNotes.notes, function(index, note) {
			if (note.id == note_id) {
				result = note;
				return false;
			}
		});
		return result;
	}

	jQuery.fn.stickyNotes.getNotes = function() {
		return jQuery.fn.stickyNotes.notes;
	}

	jQuery.fn.stickyNotes.deleteNote =  function(note_id) {
		jQuery.each(jQuery.fn.stickyNotes.notes, function(index, note) {
			if (note.id == note_id) {
				jQuery.fn.stickyNotes.notes.splice(index, 1);
				return false;
			}
		});
	}

	jQuery.fn.stickyNotes.prepareContainer = function(container) {
		jQuery(container).append('<div id="sticky-container" class="sticky-container"></div>');
		if (jQuery.fn.stickyNotes.options.controls) {
			jQuery("#sticky-container").html('<button id="add_note">Add Note</button>');
			jQuery("#add_note").click(function() {
				jQuery.fn.stickyNotes.createNote();
				return false;
			});
		}
		jQuery("body").click(function() {
			var note_ids = jQuery.fn.stickyNotes.currentlyEditedNotes();
			for (var i = note_ids.length - 1; i >= 0; i--){
				var note_id = note_ids[i];
				if (note_id != null) {
					jQuery.fn.stickyNotes.stopEditing(note_id);
				}
			};
		});

	};

	jQuery.fn.stickyNotes.createNote = function(noteFmkId,noteTextContent, noteController, noteX, noteY, noteWidth, noteHeight) {
		if(jQuery('iframe').contents().length > 0){
			jQuery('iframe').contents().click(function() {
			var note_ids = jQuery.fn.stickyNotes.currentlyEditedNotes();
				for (var i = note_ids.length - 1; i >= 0; i--){
				var note_id = note_ids[i];
				if (note_id != null) {
					jQuery.fn.stickyNotes.stopEditing(note_id);
					}
				};
			});
		}
		var pos_x = noteX;
		var pos_y = noteY;
		var note_id = jQuery.fn.stickyNotes.notes.length + 1;
		var _note_content = jQuery(document.createElement('textarea'));

		var _div_note 	= 	jQuery(document.createElement('div')).addClass('jStickyNote');
        var _div_background = jQuery.fn.stickyNotes.createNoteBackground();
        _note_content.text(noteTextContent);
		_div_note.append(_note_content);
		var _div_delete = 	jQuery(document.createElement('div'))
							.addClass('jSticky-delete')
							.click(function(){jQuery.fn.stickyNotes.deleteNote(this);});
		var _div_wrap 	= 	jQuery(document.createElement('div'))
							.css({'position':'absolute','top':pos_y,'left':pos_x, 'float' : 'left','width':noteWidth,'height':noteHeight})
							.attr("id", "note-" + note_id)
							.attr("controller", noteController)
							.attr("noteFmkId", noteFmkId)
							.append(_div_background)
							.append(_div_note)
							.append(_div_delete);
		_div_wrap.addClass('jSticky-medium');
		if (jQuery.fn.stickyNotes.options.resizable) {
			_div_wrap.resizable({stop: function(event, ui) { jQuery.fn.stickyNotes.resizedNote(note_id)}});
		}
		_div_wrap.draggable({containment: '#sticky-container', scroll: false, stop: function(event, ui){jQuery.fn.stickyNotes.movedNote(note_id)}});


		jQuery('#sticky-container').append(_div_wrap);

		jQuery.fn.stickyNotes.setCurrentlyEditedNote(note_id);
		jQuery("#note-" + note_id).click(function() {
			return false;
		})
		jQuery("#note-" + note_id).css("z-index","800");
		jQuery("#note-" + note_id).find("textarea").focus();
		var note = {"id":note_id,
		      "text":"",
			  "pos_x": pos_x,
			  "pos_y": pos_y,
			  "controller": noteController,
			  "width": jQuery(_div_wrap).width(),
			  "height": jQuery(_div_wrap).height()};
		jQuery.fn.stickyNotes.notes.push(note);
        jQuery(_note_content).css({
            'width': jQuery("#note-" + note_id).width() - 44,
            'height': jQuery("#note-" + note_id).height() - 15
        });
		if (jQuery.fn.stickyNotes.options.createCallback) {
			jQuery.fn.stickyNotes.options.createCallback(note);
		}

		if (noteTextContent != "")
			jQuery.fn.stickyNotes.stopEditing(note_id);
	}

	jQuery.fn.stickyNotes.stopEditing = function(note_id) {
		var note = jQuery.fn.stickyNotes.getNote(note_id);
		var _note_text = jQuery("#note-" + note_id).find('textarea').val();

		if (typeof _note_text != "undefined")
			_note_text = _note_text.replace(/(\r\n|\n|\r)/gm,"<br>");

		note.text = _note_text;
		note.controller = jQuery("#note-" + note_id).attr('controller');
		note.noteFmkId = jQuery("#note-" + note_id).attr('noteFmkId');
		var _p_note_text = 	jQuery(document.createElement('p')).attr("id", "p-note-" + note_id)
							.html(note.text);

		if ((typeof note.text == "undefined") || (note.text == ""))
			_p_note_text.width('50px').height('20px');

		jQuery("#note-" + note_id).find('textarea').replaceWith(_p_note_text);
		jQuery("#p-note-" + note_id).dblclick(function() {
			jQuery.fn.stickyNotes.editNote(this);
		});
		jQuery.fn.stickyNotes.removeCurrentlyEditedNote(note_id);
		if (jQuery.fn.stickyNotes.options.editCallback) {
			jQuery.fn.stickyNotes.options.editCallback(note);
		}
	};

	jQuery.fn.stickyNotes.deleteNote = function(delete_button) {
		if(jQuery(delete_button).parent().attr("id"))
			{
		var note_id = jQuery(delete_button).parent().attr("id").replace(/note-/, "");
		var note = jQuery.fn.stickyNotes.getNote(note_id);
		jQuery("#note-" + note_id).remove();
		if (jQuery.fn.stickyNotes.options.deleteCallback) {
			jQuery.fn.stickyNotes.options.deleteCallback(note);
		}
		jQuery.fn.stickyNotes.deleteNote(note_id);
			}
	}

	jQuery.fn.stickyNotes.editNote = function(paragraph) {
		var note_id = jQuery(paragraph).parent().parent().attr("id").replace(/note-/, "");

		var _note_text = jQuery("#note-" + note_id).find('p').html();

		_note_text = ReplaceString(_note_text, "<BR>", "\n");
		_note_text = ReplaceString(_note_text, "<br>", "\n");

		var textarea = 	jQuery(document.createElement('textarea')).attr("id", "textarea-note-" + note_id)
							.val(_note_text);

		jQuery("#p-note-" + note_id).replaceWith(textarea);

        jQuery(textarea).css({
            'width': jQuery("#note-" + note_id).width() - 44,
            'height': jQuery("#note-" + note_id).height() - 15
        });
		jQuery("#note-" + note_id).find("textarea").focus();
		jQuery.fn.stickyNotes.setCurrentlyEditedNote(note_id);
	}

	jQuery.fn.stickyNotes.currentlyEditedNotes = function() {
		return jQuery.fn.stickyNotes.currentlyEditedNoteIds;
	}

	jQuery.fn.stickyNotes.setCurrentlyEditedNote = function(note_id) {
		jQuery.fn.stickyNotes.currentlyEditedNoteIds.push(note_id);
	}

	jQuery.fn.stickyNotes.removeCurrentlyEditedNote = function(note_id) {
		var notes = jQuery.fn.stickyNotes.currentlyEditedNotes();
		var pos = -1;
		for (var i = notes.length - 1; i >= 0; i--){
			if (notes[i] == note_id) {
				pos = i;
				break;
			}
		};
		jQuery.fn.stickyNotes.currentlyEditedNoteIds.splice(pos, 1);
	}

	jQuery.fn.stickyNotes.renderNote = function(note) {
		var _p_note_text = 	jQuery(document.createElement('p')).attr("id", "p-note-" + note.id)
							.html( note.text);
		var _div_note 	= 	jQuery(document.createElement('div')).addClass('jStickyNote');
        var _div_background = jQuery.fn.stickyNotes.createNoteBackground();
		_div_note.append(_p_note_text);
		var _div_delete = 	jQuery(document.createElement('div'))
							.addClass('jSticky-delete')
							.click(function(){jQuery.fn.stickyNotes.deleteNote(this);});
		var _div_wrap 	= 	jQuery(document.createElement('div'))
							.css({'position':'absolute','top':note.pos_y,'left':note.pos_x, 'float': 'left',"width":note.width,"height":note.height})
							.attr("id", "note-" + note.id)
							.append(_div_background)
							.append(_div_note)
							.append(_div_delete);
		_div_wrap.addClass('jSticky-medium');
		if (jQuery.fn.stickyNotes.options.resizable) {
			_div_wrap.resizable({stop: function(event, ui) { jQuery.fn.stickyNotes.resizedNote(note.id)}});
		}
		_div_wrap.draggable({containment: '#sticky-container', scroll: false, stop: function(event, ui){jQuery.fn.stickyNotes.movedNote(note.id)}});


		jQuery('#sticky-container').append(_div_wrap);
		jQuery("#note-" + note.id).click(function() {
			return false;
		})
		jQuery(_p_note_text).dblclick(function() {
			jQuery.fn.stickyNotes.editNote(this);
		});
	}

	jQuery.fn.stickyNotes.movedNote = function(note_id) {
		var note = jQuery.fn.stickyNotes.getNote(note_id);
		note.pos_x=jQuery("#note-" + note_id).css("left").replace(/px/, "");
		note.pos_y=jQuery("#note-" + note_id).css("top").replace(/px/, "");
		if (jQuery.fn.stickyNotes.options.moveCallback) {
			jQuery.fn.stickyNotes.options.moveCallback(note);
		}
	}

	jQuery.fn.stickyNotes.resizedNote = function(note_id) {
		var note = jQuery.fn.stickyNotes.getNote(note_id);
		note.width=jQuery("#note-" + note_id).width();
		note.height=jQuery("#note-" + note_id).height();
		if (jQuery.fn.stickyNotes.options.resizeCallback) {
			jQuery.fn.stickyNotes.options.resizeCallback(note);
		}
	}
    jQuery.fn.stickyNotes.createNoteBackground = function() {
        var background = null;
        if (jQuery.browser.msie && jQuery.browser.version <= 6)  {
            background = jQuery(document.createElement('div')).addClass("background").html('<img src="images/spacer.gif" class="stretch" style="margin-top:5px;filter:progid:DXImageTransform.Microsoft.AlphaImageLoader(src=\'images/sticky-bg.png\',sizingMethod=\'scale\'");" alt="" />');
        } else {
            background = jQuery(document.createElement('div')).addClass("background").html('<img src="./leon/css/sticker/images/sticky-bg.png" class="stretch" style="margin-top:5px;" alt="" />');
        }
        return background;

    }


	jQuery.fn.stickyNotes.defaults = {
		notes 	: [],
		resizable : true,
		controls: true,
		createCallback: false,
		editCallback: false,
		deleteCallback: false,
		moveCallback: false,
		resizeCallback: false
	};


	jQuery.fn.stickyNotes.options = null;
	jQuery.fn.stickyNotes.currentlyEditedNoteIds = new Array();
	jQuery.fn.stickyNotes.notes = new Array();
})(jQuery);

	var stickyEdited = function(note) {
		sendStickyEvent(note.controller,note.noteFmkId,note.text);
	}

	var stickyDeleted = function(note) {
		sendStickyEvent(note.controller,"_close");
	}

	var stickyMoved = function(note) {
		sendStickyEvent(note.controller,"move",note.pos_x + "|" + note.pos_y);
	}

	var stickyResized = function(note) {
		sendStickyEvent(note.controller,"resize",note.width + "|" + note.height);
	}
