package leon.view.web.mobile;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;

import javax.servlet.http.*;

import leon.app.*;
import leon.app.behavior.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;
import leon.view.event.*;
import leon.view.getinterface.*;
import leon.view.mobile.*;
import leon.view.web.*;

import org.apache.commons.fileupload.servlet.*;

/**
 * Classe dcodant l'ensemble des requtes reues par la servlet 
 * et renvoyant la vue au format XML.
 * 
 * @author kchenal
 *
 */
public abstract class LyMobileDecodeRequest
{

	/**
	 * Design Pattern : singleton.
	 */
	private static LyMobileDecodeRequest _instance = null;
	/**
	 * Utilis pour l'upload d'un fichier.
	 */
	protected static String 					_controller;
	/**
	 * Utilis pour l'upload d'un fichier.
	 */
	protected static String 					_dir;
	/**
	 * Utilis pour l'upload d'un fichier.
	 */
	protected static String 					_source;
	/**
	 * Contient l'ensemble des sessions en cours.
	 */
	protected HashMap<String, LySession> 		_map;	
	/**
	 * L'id de l'application
	 */
	protected String 							_applicationId;

	/**
	 * Constructeur.
	 */
	protected LyMobileDecodeRequest()
	{
		_map = new HashMap<String, LySession>(1);
	}

	public static LyMobileDecodeRequest getInstance(boolean struts)
	{
		if(_instance == null)
		{
			String viewerClassName;
			if(struts)
			{
				viewerClassName = "leon.view.web.struts.mobile.LyStrutsMobileDecoder";
			}
			else
			{
				viewerClassName = "leon.view.web.jquery.mobile.LyJQMobileDecoder";
			}
			try
			{
				Class<?>	viewerClass = Class.forName(viewerClassName);
				Constructor<?> constructor = viewerClass.getConstructor();

				_instance = (LyMobileDecodeRequest) constructor.newInstance();
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}

		return _instance;

	}

	public static LyMobileDecodeRequest getInstance()
	{
		return _instance;
	}

	public abstract void swapSession(Object newSession, Object oldSession, String applicationId);



	/**
	 * Renvoie la session utilisateur en fonction de son identifiant.
	 * @param request la requte permettant d'identifier la session.
	 * @return La session correspondante.
	 */
	protected LySession getSession(HttpServletRequest request)
	{
		return _map.get(request.getSession().getId());
	}

	/**
	 * Supprime la session correspondante  l'identifiant de la requte
	 * @param request la requte permettant d'identifier la session.
	 */
	private void deleteSession(HttpServletRequest request)
	{
		LySession session = getSession(request);
		_map.remove(request.getSession(true).getId());
		HttpSession	   httpSession = request.getSession();
		httpSession.invalidate();
		session.close();
	}

	/**
	 * Rcupre le viewManager d'une session.
	 * @param session La session
	 * @return Le viewManager.
	 */
	protected LyWebViewManager getViewManager(LySession session)
	{
		return (LyWebViewManager) session.getEnvironment().getViewManager(true);
	}

	/**
	 * Rcupre l'environnement d'une session.
	 * @param session La session
	 * @return L'environnement
	 */
	private LyEnvironment getEnvironment(LySession session)
	{
		return session.getEnvironment();
	}


	//********************************************************************
	//					Dcodage de la requte
	//********************************************************************

	/**
	 * Dcode la requte et construit la vue XML.
	 * @param req La requte envoye par le mobile.
	 * @param resp La vue XML.
	 */
	public void decode(HttpServletRequest req, HttpServletResponse resp) 
	{
		long debut = System.currentTimeMillis();
		LyController controller = null;

		LySession session = getSession(req);

		String[] type = req.getParameterValues(LyMobileConstants.TYPE);

		Boolean exit = false;
		Boolean message = false;

		if(session == null)
		{
			createSession(req, resp);
			session = getSession(req);
			controller = session.getController(0);
		} 

		int initialControllersNumber = session.getControllerCount();
		String initialId = null;
		if(session.getController(initialControllersNumber) != null)
		{
			initialId = session.getController(initialControllersNumber).getId();
		}

		if(type != null && type[0] != null) 
		{
			if(type[0].equals(LyMobileConstants.COMMAND))
			{
				message = commandAction(req, resp);		
				controller = session.getLastViewController();
			}
			else if(type[0].equals(LyMobileConstants.EVENT))
			{
				message = graphicalEvent(req, resp, false);
				controller = session.getLastViewController();
			}
			else if(type[0].equals(LyMobileConstants.SELECT))
			{
				message = graphicalEvent(req, resp, true);
				controller = session.getLastViewController();
			}
			else if(type[0].equals(LyMobileConstants.RELOAD))
			{
				controller = session.getLastViewController();
			}
			else if(type[0].equals(LyMobileConstants.EXIT))
			{
				exit = true;
				deleteSession(req);
			}
			else if(type[0].equals(LyMobileConstants.RESPOND))
			{
				respond(req, resp);

				controller = session.getController(0);
				if(controller == null)
				{
					session = getSession(req);
				}
				controller = session.getLastViewController();
			}
			else if(type[0].equals(LyMobileConstants.DOWNLOAD))
			{
				download(req, resp);
			}
			else if(type[0].equals(LyMobileConstants.UPLOAD))
			{
				String[] control = req.getParameterValues(LyMobileConstants._controller);
				_controller = control[0];
				String[] source = req.getParameterValues(LyMobileConstants._source);
				_source = source[0];
				String[] dir = req.getParameterValues(LyMobileConstants._attachmentDirectory);
				_dir = dir[0];
				exit = true;
			}
			else if(type[0].equals(LyMobileConstants.INFO_PLUS))
			{
				getInfoPlus(req, resp);
				exit = true;
			}
			else if(type[0].equals(LyMobileConstants.GET_LOGO))
			{
				exit = true;
				PrintWriter out;
				try
				{
					out = resp.getWriter();
					String value = ((LyMobileViewManager) getViewManager(session)).getRealRootURL();
					String value2 = session.getApplication().getProject().getFilePrefix();
					String value3 = getImageUrl(session, value2);
					String value4 = session.getApplication().getProject().getTip();
					out.print(value);
					out.print(value3);
					out.print("////");
					out.print(value4);

				}
				catch (IOException e)
				{
					e.printStackTrace();
				}

			}
			else if(type[0].equals(LyMobileConstants.GET_VERSION))
			{
				resp.addHeader(LyMobileConstants.VERSION, LyMobileConstants.TRUE);
				exit = true;
			}
			else
			{
				message = graphicalEvent(req, resp, false);
				controller = session.getLastViewController();
			}		
		}
		else
		{
			boolean isMultipart = ServletFileUpload.isMultipartContent(req);

			if(isMultipart)
			{
				try
				{
					upload(_controller, null, _source, _dir,req);
					//exit = true;
				}
				catch (Exception e)
				{
					e.printStackTrace();
				}
				finally
				{
					_controller = null;
					_source = null;
					_dir = null;
				}
			}
		}

		if(exit == false && message == false)
		{
			int finalControllersNumber = session.getControllerCount();
			try
			{
				PrintWriter out = new PrintWriter(new OutputStreamWriter(resp.getOutputStream(), "UTF8"), true);
				resp.setContentType("text/xml; charset="+LyString.DEFAULT_CHARSET);
				if(controller == null)
					controller = session.getLastViewController();
				if(controller == null)
					controller = session.getController(0);
				if(controller == null)
				{
					deleteSession(req);
					decode(req, resp);
					return;
				}

				if((finalControllersNumber > initialControllersNumber) && (initialId == null || (initialId != null &&
						initialId.equals(session.getController(initialControllersNumber).getId()))))
				{
					resp.addHeader(LyMobileConstants.NEW_VIEW, LyMobileConstants.TRUE);
				}

				if(controller instanceof LyViewController)
				{
					controller = ((LyViewController)controller).getAreaController();
				}

				LyView view = controller.getView();

				if(controller instanceof LyLoginController)
				{
					resp.addHeader(LyMobileConstants.LOGIN, LyMobileConstants.TRUE);
				}
				else if(controller instanceof LyDashboardController)
				{
					resp.addHeader(LyMobileConstants.MENU, LyMobileConstants.TRUE);
				}
				else if(view.getInternalView() instanceof LyMobileDialog)
				{
					resp.addHeader(LyMobileConstants.DIALOG, LyMobileConstants.TRUE);
				}
				else
				{
					resp.addHeader(LyMobileConstants.FRAME, LyMobileConstants.TRUE);
				}

				LyMobileViewManager viewManager = ((LyMobileViewManager)getViewManager(session));
				if(viewManager.is_sessionSwapped())
				{
					viewManager.set_sessionSwapped(false);					
					LyAreaController tmpController = ((LyAreaController) controller).getParentAreaController();

					boolean dashboardPassed = false;
					while(!dashboardPassed)
					{
						if(tmpController == null)
						{
							dashboardPassed = true;
						}
						else if(tmpController instanceof LyDashboardController)
						{
							dashboardPassed = true;
							String label = ((LyGetLabelInterface)tmpController.getView()).getLabel();
							resp.addHeader(LyMobileConstants.SWAP_TITLE, 
									viewManager.getEnvironment().translate(label));
							tmpController = tmpController.getParentAreaController();

						}
						else
						{
							String label = ((LyGetLabelInterface)tmpController.getView()).getLabel();
							resp.addHeader(LyMobileConstants.SWAP_TITLE, 
									viewManager.getEnvironment().translate(label));
							tmpController = tmpController.getParentAreaController();
						}
					}
				}
				//out.print("<?xml version='1.0' encoding='" + LyString.DEFAULT_CHARSET + "'?>");

				//((LyMobileArea)view.getArea().getInternalView()).writeXml(out);

				LyViewComponent component = view.getArea().getInternalView();
				if(component instanceof LyWebObject)
				{
					((LyWebObject)view.getArea().getInternalView(true)).writeJson(out, session.getId());
				}
				else
				{
					((LyMobileArea)view.getArea().getInternalView(true)).writeJson(out, session.getId());
				}
				

				LyEnvironment environment = getEnvironment(session);
				if(environment.isDebugLevel(LyEnvironment.DEBUG_ALL))
				{
					String fileJson;
					String fileXml;

					String path = environment.getEnv("LY_APP_DIR");

					if (environment.testEnv("LY_DATA_IN_TEMP_FILE"))
						path = environment.getEnv("LY_TEMP_DIR") + "/data";

					if ((path != null) && (!path.equals("LY_APP_DIR")))
					{
						fileJson = path + "/" + "mobileDumpJson.vml";
						fileXml = path + "/" + "mobileDumpXml.vml";
					}						
					else
					{
						fileJson = "mobileDumpJson.vml";
						fileXml = "mobileDumpXml.vml";
					}
						
					FileOutputStream outputJson = new FileOutputStream(fileJson);
					PrintWriter		 pwJson = new PrintWriter(outputJson);
					((LyMobileArea)view.getArea().getInternalView()).writeJson(pwJson, session.getId());
					pwJson.close();
					
					FileOutputStream outputXml = new FileOutputStream(fileXml);
					PrintWriter		 pwXml = new PrintWriter(outputXml);
					pwXml.print("<?xml version='1.0' encoding='" + LyString.DEFAULT_CHARSET + "'?>");
					((LyMobileArea)view.getArea().getInternalView()).writeXml(pwXml);
					pwXml.close();
				}


			}
			catch (IOException e)
			{
				e.printStackTrace();
			}			
		}
		long fin = System.currentTimeMillis();
		getEnvironment(session).logInfo("Temps de traitement de la servlet : "+(fin-debut)+" ms / "+(fin-debut)/1000+" s");
	}

	public String getImageUrl(LySession session, String image)
	{
		String imageUrl;
		String docRoot = expandUrl(session, getEnvironment(session).getEnv("LY_APP_DOC"));
		if (image.startsWith(docRoot))
			imageUrl = expandUrl(session, image);
		else if (image.startsWith("file"))
			imageUrl = LyString.encodeFileURL(image);
		else if (image.startsWith("http") || image.startsWith("ftp"))
			imageUrl = image;
		else
			imageUrl = getEnvironment(session).getImageUrl(image);

		int index = imageUrl.indexOf(' ');
		if(index != -1)
		{
			imageUrl = imageUrl.substring(0, index);
		}
		return imageUrl;
	}

	/**
	 * Replaces within a given String some $properties$ found in the system environment. This method
	 * is looking for macros between '$' and replaces these macros by a property if any (such as
	 * $LY_APP_DIR$/data/test.html).
	 *
	 * @param  text The original string, before any substitution.
	 * @return The string after all substitutions.
	 */
	public final String expandUrl(LySession session, String text)
	{
		return getEnvironment(session).getString().expandUrl(text);
	}


	/**
	 * Crer une session (et une application si besoin).
	 * @param request La requte envoye par le mobile.
	 * @param response La vue XML.
	 */
	public abstract void createSession(HttpServletRequest request,HttpServletResponse response);

	/**
	 * This method computes the root URL of the current leonardi application.<br>
	 * This is the path to the current server + the path of the current WEB application.<br>
	 * Change also the scheme from http to https if LY_USE_HTTPS is on.
	 *
	 * @param  request     The current HTTP request.
	 * @param  environment The current environment
	 * @return the root URL of the current leonardi application.
	 */
	protected abstract String computeRootUrl(HttpServletRequest request, LyEnvironment environment);


	/**
	 * Retourne les menus effectifs pour un objet prcis.
	 * @param request La requte envoye par le mobile.
	 * @param response La vue XML.
	 */
	public void getInfoPlus(HttpServletRequest request, HttpServletResponse response)
	{
		LySession session = getSession(request);
		try
		{
			session.getSoftLock();

			String[] actionId = request.getParameterValues(LyWebConstants.ACTION);
			String[] id = request.getParameterValues(LyWebConstants.VALUE);

			String controllerId = getParameter(request, LyMobileConstants._controller);

			LyController controller = (controllerId == null) ? null
					: session.getController(controllerId);
			LySessionBehavior behav = (LySessionBehavior) session.getSessionBehavior();

			int size;
			if(actionId != null)
			{
				size = actionId.length;
			}
			else
			{
				size = 0;
			}
			HashMap<String, Boolean> doublonTest = new HashMap<String, Boolean>();
			for(int i = 0; i < size; i++)
			{
				String res;
				int index = actionId[i].indexOf(':');
				if(index != -1)
				{
					res = actionId[i].substring(0, index);
				}
				else
				{
					res = actionId[i];
				}
				boolean ok;
				if(doublonTest.get(res) != null)
				{
					ok = false;
				}
				else
				{
					LyAction action = session.getApplication().getAction(res);

					LyObjectId oid = controller.getObjectId(id[i]);

					LyObjectList list = new LySimpleObjectList(oid.getLogicalObject());

					ok = behav.enableAction(session, action, controller, list, false);

					doublonTest.put(res, true);
				}


				if(ok)
				{
					response.addHeader(LyMobileConstants.ACTION, LyMobileConstants.TRUE);
				}
				else
				{
					response.addHeader(LyMobileConstants.ACTION, LyMobileConstants.FALSE);
				}
			}
		}
		finally
		{
			session.releaseSoftLock();
		}
	}

	/**
	 * S'occupe du dmarrage de l'appli.
	 * @param request La requte envoye par le mobile.
	 * @param response La vue XML.
	 */
	public void home(HttpServletRequest request, HttpServletResponse response)
	{
		LySession session = getSession(request);

		try
		{
			session.getSoftLock();

			// If the session is not valid : invoke 'root action'
			if (!session.isRegistered())
			{
				session.getEnvironment().setMobile(true);
				// Start the session which invokes root Action
				session.register(true); 
				session.setValid(false);
			}
			// Update language or skin if parameter is found from the request
			String[] language = request.getParameterValues("LANG");

			if ((language != null) && !language[0].equals(""))
				session.setLanguage(language[0]);

			if (!session.isValid())
			{
				// Check if root action is _login
				LyApplication application = session.getApplication();
				LyController  controller = session.getController(0);
				LyAction	  rootAction = application.getProject().getRootAction();
				String		  rootId = rootAction.getRoot().getId();

				session.setValid(true);

				if (rootId.equals(LyAction.ID_LOGIN))
				{
					// Make a redirect to the the login action
					Map<String, String> parameters = new HashMap<String, String>();

					parameters.put(LyWebConstants.ACTION, rootAction.getId());

					if (controller != null)
						parameters.put(LyWebConstants.CONTROLLER, controller.getId());

					try
					{
						login(request,response);
					}
					catch (Exception e)
					{
						e.printStackTrace();
					}

					return;
				}
			}
			session.resetTimeout();
		}
		finally
		{
			session.releaseSoftLock();
		}
		LyEnvironment.setCurrentEnvironment(getEnvironment(session));
	}

	/**
	 * S'occupe de la connexion d'un utilisateur.
	 * @param request La requte envoye par le mobile.
	 * @param response La vue XML.
	 * @throws Exception
	 */
	public void login(HttpServletRequest request, HttpServletResponse response)
			throws Exception
			{
		LySession session = getSession(request);

		try
		{
			session.getSoftLock();

			// Get the controller id
			String		 controllerId = getParameter(request, LyWebConstants.CONTROLLER);
			LyController controller = (controllerId == null) ? null
					: session.getController(controllerId);

			response.addHeader(LyMobileConstants.LOGIN, LyMobileConstants.TRUE);

			if(controller == null)
				return;
			// show the controller
			controller.showView();
		}
		finally
		{
			session.releaseSoftLock();
		}
		return;
			} // end method execute

	/**
	 * Upload d'un fichier.
	 * @param controllerId
	 * @param frame
	 * @param source
	 * @param dir
	 * @param req
	 * @throws Exception
	 */
	public abstract void upload(String controllerId, String frame, String source, String dir, HttpServletRequest req)
			throws Exception; // end method getFileAttachment

	//LyStrutsGraphicalEventAction+AnswerAction
	/**
	 * S'occupe de transmettre les vnements graphiques.
	 * @param request La requte envoye par le mobile.
	 * @param response La vue XML.
	 * @param select Cas d'une slection dans une table
	 * @return false si aucun message n'est apparu.
	 */
	public boolean graphicalEvent(HttpServletRequest request,HttpServletResponse response, boolean select)
	{
		LySession session = getSession(request);

		try
		{
			session.getSoftLock();
			// get some usefull request parameters
			String		 controllerId = getParameter(request, LyMobileConstants._controller);
			String		 source = getParameter(request, LyMobileConstants._source);
			String		 frame = getParameter(request, LyMobileConstants._frame);

			// If controller is not found in the parameters, try to retrieve it from source
			// that may be the identifier of the component that sent the request.
			if ((controllerId == null) || (controllerId.length() == 0))
			{
				String	stackName = frame;

				if (LyWebConstants.TARGET_FRAME.equals(frame))
					stackName = null;

				LyViewController viewController = session.getStackLastController(stackName);

				if ((viewController != null) && !viewController.isFreed())
				{
					LyView	view = viewController.getView();

					if (view != null)
					{
						LyComponent	component = view.getComponent(source);

						if (component != null)
						{	
							controllerId = component.getRootController();
						}
					}
				}
			}

			LyWebViewManager viewManager = getViewManager(session);

			// get the controller that receive this event
			LyDemandTransmitter	transmitter = viewManager.getDemandTransmitter();
			LyDemandListener	controller = (controllerId == null) || (transmitter == null)? null
					: transmitter.getListenerById(controllerId);

			LyDemand demand = (controller == null ? null : 
				((select == true || source.equals("_select")) ? getDemandSelection(request, session.getEnvironment()) : 
					getDemand(request, session.getEnvironment())));

			// process the demand to the areaController
			if (demand != null)
			{
				if (controller instanceof LyViewController)
				{
					LyAreaController areaController = ((LyViewController)controller)
							.getAreaController();

					if (areaController != null)
						controllerId = areaController.getId();
				}

				viewManager.processDemand(controllerId, demand);
			}

			String message = viewManager.getMessage();
			if(message != null)
			{
				response.addHeader(LyMobileConstants.MESSAGE, LyMobileConstants.TRUE);
				response.getWriter().print(message);
				return true;
			}
			if(((LyMobileViewManager)viewManager).hasUrl())
			{
				response.addHeader(LyMobileConstants.URL, LyMobileConstants.TRUE);
				response.getWriter().print(((LyMobileViewManager)viewManager).getUrl());
				return true;
			}
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		finally
		{
			session.releaseSoftLock();
		}
		return false;
	}

	//LyStrutsCommandAction
	/**
	 * S'occupe de traiter les vnements de type command.
	 * @param request La requte envoye par le mobile.
	 * @param response La vue XML.
	 * @return false si aucun message n'est apparu.
	 */
	public abstract boolean commandAction(HttpServletRequest request,HttpServletResponse response);


	/*public static void swapSession(Object newSession, Object oldSession, String applicationId)
	{
		String id = getId();
		HttpSession			httpSession = requestInfo.getRequest().getSession();
		String				httpSessionKey = httpSession.getId() + "#" + DEFAULT_SESSION;

		httpSession.setAttribute(httpSessionKey, (LySession)oldSession);
		_sessions.put(httpSessionKey, (LySession)oldSession);
	}

	public abstract String getId();*/

	/**
	 * S'occupe du tlchargement d'un fichier.
	 * @param request La requte envoye par le mobile.
	 * @param response La vue XML.
	 */
	public void download(HttpServletRequest request, HttpServletResponse response)
	{
		String sessionId = getParameter(request, LyMobileConstants.SESSION_ID_MOBILE);
		LySession session;	
		if(sessionId != null)
		{
			LyApplication application = LyApplication.getApplication(_applicationId);
			session = (LySession) application.getSession(sessionId.intern());
		}
		else
		{
			session = getSession(request);
		}

		try
		{
			session.getSoftLock();
			String		fileId = getParameter(request, LyWebConstants.FILE);
			String[]	files = session.getTempFile(fileId);

			String		fileDesc;
			String		filePath;

			if (files == null)
			{
				LyViewManager manager = session.getViewManager(true);
				filePath = manager.getTemporaryDir()+"\\"+fileId;
				fileDesc = fileId;
			}
			else
			{
				fileDesc = files[1];
				filePath = files[0];
			}

			File	file = new File(filePath);

			if (!file.exists())
				return;

			FileInputStream fis;
			try
			{
				fis = new FileInputStream(file);
				OutputStream			ros = response.getOutputStream();
				BufferedOutputStream	bos = new BufferedOutputStream(ros);
				byte[]					buf = new byte[1024];
				int						len;

				response.reset();
				fileDesc = fileDesc.replace(" ", "_");
				response.setHeader("content-Length", Long.toString(file.length()));
				response.setHeader("content-disposition", "attachment; filename="+fileDesc);
				while ((len = fis.read(buf, 0, buf.length)) != -1)
				{
					bos.write(buf, 0, len);
				}

				bos.flush();
				bos.close();

				ros.flush();
				ros.close();

				fis.close();
			}
			catch (FileNotFoundException e)
			{
				e.printStackTrace();
			}
			catch (IOException e)
			{
				e.printStackTrace();
			}
		}
		finally
		{
			session.releaseSoftLock();
		}
	}

	/**
	 * S'occupe de la rponse  un message.
	 * @param request La requte envoye par le mobile.
	 * @param response La vue XML.
	 */
	public void respond(HttpServletRequest request, HttpServletResponse response)
	{

		LySession session = getSession(request);

		String answer;
		String[] type = request.getParameterValues(LyMobileConstants.RESPONSE);
		String[] id = request.getParameterValues(LyMobileConstants.ID);
		try
		{
			session.getSoftLock();

			if(type[0].equals(LyMobileConstants.YES))
			{
				answer = new Short(LyViewManager.COMMAND_YES).toString();
			}
			else if(type[0].equals(LyMobileConstants.NO))
			{
				answer = new Short(LyViewManager.COMMAND_NO).toString();
			}
			else
			{
				answer = new Short(LyViewManager.COMMAND_OK).toString();
			}

			LyWebViewManager viewManager = getViewManager(session);
			viewManager.answerMessage(id[0], LyTools.shortFromString(answer));

			//FFT05040
			/*LyWebViewManager viewManager = getViewManager(session);
			viewManager.answerMessage(id[0], LyTools.shortFromString(type[0]));*/
			viewManager.setMessage(null);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			session.releaseSoftLock();
		}
	}


	/**
	 * Rcupre un paramtre de la requte.
	 * @param request La requte envoye par le mobile.
	 * @param name Le nom du praramtre.
	 * @return La valeur du paramtre.
	 */
	public static String getParameter(HttpServletRequest request, String name)
	{
		String[] values = request.getParameterValues(name);

		if (values == null)
			return null;

		// Just return the last value
		String value = values[values.length - 1];

		value = decodeUrl(value);
		return value;
	}

	/**
	 * Dcode une url
	 * @param url L'URL.
	 * @return L'URL dcode.
	 */
	public static String decodeUrl(String url)
	{
		String newUrl;

		newUrl = LyString.replace(url, "%20", " ");
		newUrl = LyString.replace(newUrl, "\\\\", "\\");

		return newUrl;
	}

	/**
	 * @param request
	 * @param environment
	 * @return
	 */
	public abstract LyDemand getDemand(HttpServletRequest request, LyEnvironment environment);

	/**
	 * @param request
	 * @return
	 */
	protected Vector<String> getSelectionIds(HttpServletRequest request)
	{
		// search for selection parameters
		Vector<String>     selectionIds = new Vector<String>();
		String		       parameter, selection, newElement;
		Enumeration<?>     e = request.getParameterNames();

		while (e.hasMoreElements())
		{
			// get parameter
			parameter = (String)e.nextElement();
			if (parameter.indexOf(LyWebConstants.SELECTION_ITEM) == 0)
			{
				// get selection value
				selection = request.getParameterValues(parameter)[0];

				// The selection value may be in the name or in the value
				// parameter
				// the first to try is the value, then the name
				if ((selection != null) && (selection.indexOf(LyWebConstants.SELECTION_ITEM) >= 0))
					newElement = selection.substring(LyWebConstants.SELECTION_ITEM.length());
				else
					newElement = parameter.substring(LyWebConstants.SELECTION_ITEM.length());

				// if the element is already in the selection list, it is
				// already selected
				// so it's an UNSELECT_ITEM demand
				if (newElement.length() != 0)
					if (selectionIds.contains(newElement))
						selectionIds.removeElement(newElement);
					else
						selectionIds.addElement(newElement);
			}
		}

		// return list of selected items
		return selectionIds;
	} // end method getSelectionIds

	/**
	 * @param request
	 * @param response
	 * @param application
	 * @param session
	 * @param environment
	 * @param lyViewManager
	 * @return
	 * @throws Exception
	 */
	protected String manageNullController(HttpServletRequest request, HttpServletResponse response,
			LyApplication application, LySession session, LyEnvironment environment,
			LyViewManager lyViewManager)
					throws Exception
					{
		// get some request parameters
		String frame = getParameter(request, LyWebConstants.FRAME);
		String controller = null;

		// This happens when validating a login form
		// Check if valid : bug on Netscape when reusing a login page after exit
		// or timeout or server restart
		if ((frame == null) || (!session.isValid()))
		{
			// If not found, redirect on root controller
			LyController rootController = session.getController(0);

			if (rootController != null)
			{
				controller = rootController.getId();

				// May cause a few problems
				if (rootController.getViewController() != null)
					rootController.getViewController().showView();
			}
		}
		// This happens when more than one submit are sent.
		// Display the last view controller or root page if there
		// is no last view controller.
		else
		{
			LyViewController lvc = session.getLastViewController();

			if ((lvc != null) && (!lvc.isFreed()))
			{
				LyAreaController ac = lvc.getAreaController();

				// Do not send any action demand to the controller.
				// Just show it.
				controller = ac.getId();
				lvc.showView();
			}
		}
		return controller;
					} // end method manageNullController


	/**
	 * @param request
	 * @param environment
	 * @return
	 */
	public LyDemand getDemandSelection(HttpServletRequest request, LyEnvironment environment)
	{
		// prepare a selection demand
		LySelectionDemand selectionDemand = null;

		// get the source component
		String			  source = getParameter(request, LyWebConstants.SOURCE);

		source = ((source == null) ? "" : source.intern());

		// get the extend parameter
		String extend = getParameter(request, LyWebConstants.EXTEND);
		String value = getParameter(request, LyWebConstants.VALUE);

		// Use the same separator as used in the getSelectionIds() function in LyUtilsScript.js
		Vector<String>	selectionIds = new Vector<String>();
		String[]		ids = LyString.stringToList(value, "__|__");

		if ((ids != null) && (ids.length > 0))
		{
			for (int i = 0; i < ids.length; i++)
			{
				if ((ids[i] != null) && (ids[i].length() > 0))
					selectionIds.addElement(ids[i]);
			}
		}

		int		i, nSelections = selectionIds.size();

		// create the demand depending on the source component
		if (LyWebConstants.SELECT_ITEM == source || LyWebConstants.UNSELECT_ITEM.equals(source))
		{
			if (LyWebConstants.SELECT_ITEM == source)
				selectionDemand = new LySelectionDemand(LySelectionDemand.TYPE_SELECT, null);
			else
				selectionDemand = new LySelectionDemand(LySelectionDemand.TYPE_UNSELECT, null);

			// set the extend property
			selectionDemand.setExtend("true".equals(extend));

			// set the selected items
			for (i = 0; i < nSelections; i++)
				selectionDemand.addSelectionId(selectionIds.elementAt(i));
		}
		else if (LyWebConstants.SELECT_ALL.equals(source))
			selectionDemand = new LySelectionDemand(LySelectionDemand.TYPE_SELECT_ALL, null);
		else if (LyWebConstants.UNSELECT_ALL.equals(source))
			selectionDemand = new LySelectionDemand(LySelectionDemand.TYPE_UNSELECT_ALL, null);

		return selectionDemand;
	}
}
