/*
 * LyStrutsArea.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts;

import java.io.*;
import java.util.*;

import javax.servlet.http.*;

import leon.misc.*;
import leon.view.*;
import leon.view.getinterface.*;
import leon.view.graphic.*;
import leon.view.setinterface.*;
import leon.view.web.*;


/**
 * This class implements the Area object (as described in the dtd) for web viewer.
 *
 * <p><b>AREA: Area</b><br>
 * An area allows the positioning of various graphical components inside a main window or a dialog
 * window. It can contains any type of panel as well as a title bar, a tool bar and a command bar.
 * </p>
 *
 * @version $Id: LyStrutsArea.java 41032 2012-10-04 13:18:14Z nhamon $
 * @author  Lyria S.A.
 */
public class LyStrutsArea extends LyStrutsContainer
    implements LyGraphicalArea,
               LySetVisibleInterface,
               LySetBorderPanelInterface
{
	public static final String __VERSION =
		"$Id: LyStrutsArea.java 41032 2012-10-04 13:18:14Z nhamon $";


	/** Indicates the max heightof a frame in dhtml. */
	 public static int						 FRAME_MAX_HEIGHT = -1;

	/** Indicates the max width of a frame in dhtml.*/
	 public static int						 FRAME_MAX_WIDTH = 600;
	
	   /** Constant used to generate CSS class name for area. */
    public static int                        DECO_AREA_PARAMETER = 0;

    /** Constant used to generate CSS class name for tip. */
    public static int                        DECO_TIP_PARAMETER = 1;

    /** The toolbar size. */
    public static final int                  TOOLBAR_SIZE = 15;

    /** The toolbar image size. */
    public static final int                  IMAGE_TOOLBAR_SIZE = 20;

    /** The toolbar label size. */
    public static final int                  LABEL_TOOLBAR_SIZE = 15;

    /** The scrollbar size. */
    public static final int                  SCROLLBAR_SIZE = 30;

    /** The titlebar size. */
    public static final int                  TITLEBAR_SIZE = 30;

    /** Minimum height of the border panels. */
    public static final int                  BORDER_PANEL_MIN_HEIGHT = 37;

    /** The <b>tooltitle</b> string representation used to build frame with tooltitle names. */
    public static final String               TOOLTITLE = "tooltitle".intern();

    /** The <b>northpanel</b> string representation used to build frame with northpanel names. */
    public static final String               NORTHPANEL = "northpanel".intern();

    /** The <b>southpanel</b> string representation used to build frame with southpanel names. */
    public static final String               SOUTHPANEL = "southpanel".intern();

    /** The <b>westpanel</b> string representation used to build frame with westpanel names. */
    public static final String               WESTPANEL = "westpanel".intern();

    /** The <b>eastpanel</b> string representation used to build frame with eastpanel names. */
    public static final String               EASTPANEL = "eastpanel".intern();

    /** The <b>selectbar</b> string representation used within frames names. */
    public static final String               SELECTBAR = "selectbar".intern();

    /** The <b>commands</b> string representation used within frames names. */
    public static final String               COMMANDS = "commands".intern();

    /** The <b>element</b> string representation used within frames names. */
    public static final String               ELEMENT = "element".intern();

    /** The <b>left</b> string representation used within frames names. */
    public static final String               LEFT = "left".intern();

    /** The <b>center</b> string representation used within frames names. */
    public static final String               CENTER = "center".intern();

    /** The <b>right</b> string representation used within frames names. */
    public static final String               RIGHT = "right".intern();

    /** The <b>bottomtoolbar</b> string representation used within frames names. */
    public static final String               BOTTOMTOOLBAR = "bottomtoolbar".intern();

	/**
	 * Indicates whether Ajax must be used to refresh border panels.
	 * If set, empty panels may be visible even if they are empty.
	 */
	public static boolean					 USE_AJAX_FOR_BORDER_PANELS = true;

    /**
     * List of string indicating if elements of the area are framesets or not. If the name of the
     * frame is in the list, the corresponding frame is a frameset.
     */
    protected Vector<String>                 _frames;

    /** The area generic component. */
    protected LyArea                         _area;

    /** Address for redirection. */
    protected String                         _redirect;

    /**
     * Internal table used to get Ids of panels inside the area. Used for elements that do not
     * implement {@link LyGetIdInterface} or that do not have an id.
     */
    protected Hashtable<LyComponent, String> _subElementIds;

    /** An available Id. */
    protected String                         _availableId;

    /** The area content width. */
    protected int                            _width;

    /** The area content height. */
    protected int                            _height;

    /** Identifier of the form generated in the frame that contains the border panels. */
    protected String                         _borderPanelsFormId;

    /** Indicates whether border panels are framesets. */
    protected boolean[]                      _isBorderPanelFrameset;

    /** Indicates whether this area is visible or not. */
    protected boolean                       _isVisible;

	/**
	 * Constructor.
	 *
	 * @param area the area component
	 */
	public LyStrutsArea(LyArea area)
	{
        super(area);

        _area = area;
        init();
	}

	public void writeHtml(LyWebPageWriter pw)
	{
	    updateRefreshInfo(pw);

	    if (_area.isHidden())
	        return;

	    pw.startComponent(this);

		LyWebDecoration	deco = getWebViewManager().getDecoration();
		LyTitleBar		titleBar = getTitleBar();
		int				border = (_area.getBorder() == LyArea.Border.SOLID ? 1 : 0);
		LyToolBar		topLeftToolbar = _area.getTopLeftToolBar();
		LyToolBar		topRightToolbar = _area.getTopRightToolBar();
		LyToolBar		leftToolbar = _area.getLeftToolBar();
		LyToolBar		rightToolbar = _area.getRightToolBar();
		String			tip = _area.getTip();
		LyPanel			northBorderPanel = _area.getBorderPanel(LyArea.NORTH);
		LyPanel			southBorderPanel = _area.getBorderPanel(LyArea.SOUTH);
		LyPanel			eastBorderPanel = _area.getBorderPanel(LyArea.EAST);
		LyPanel			westBorderPanel = _area.getBorderPanel(LyArea.WEST);
		LyToolBar		bottomLeftToolbar = _area.getBottomLeftToolBar();
		LyToolBar		bottomRightToolbar = _area.getBottomRightToolBar();
		LySelectBar		selectbar = _area.getSelectBar();
		LyCommands		commands = _area.getCommands();
		int				nPanel = _area.getPanelCount();
		String			label = _area.getLabel();
		boolean			generateBorder = (_area.getBorder() != LyArea.Border.NONE) && (_area.getBorder() != LyArea.Border.SOLID);
		LyConstraint	areaConstraint = _area.getConstraint();
		boolean			fullWidth = false;
		boolean			fullHeight = false;
		String			align = "";
		boolean			isInBorderPanel = (getParentBorderPanel() != null);
		LyComponent		parentComponent = _area.getParent();
		boolean			generateExternalDiv = ((parentComponent instanceof LySplitter) || isInBorderPanel)
				&& (!pw.writeDivOnly() || (pw.getViewComponent() != _area)) && !LyWebConstants.POPUP_VIEW.equals(_lastWindowName);
		boolean			useScroll = _area.getEnvironment().testResource("LY_SCROLL_DASHBOARD")
				&& (_area.getParent() instanceof LyFrame)
				&& (leftToolbar != null)
				&& (leftToolbar.getStyle() == LyToolBar.Style.EXPAND);

		// Add base url
		String baseUrl = getBaseUrl(pw);

		pw.addBeginScript("<script type='text/javascript'>\n//<![CDATA[\nvar _rootUrl='"
				+ LyWebObject.encodeUrl(baseUrl) + "';\n//]]>\n</script>", false);

		if (areaConstraint != null)
		{
			LyConstraint.Fill	fill = areaConstraint.getFill();
			LyConstraint.HAlign	hAlign = areaConstraint.getHAlign();

			if ((fill == LyConstraint.Fill.HORIZONTAL)
					|| (fill == LyConstraint.Fill.BOTH)
					|| (areaConstraint.getNCols() > 0))
				fullWidth = true;

			if ((fill == LyConstraint.Fill.VERTICAL) || (fill == LyConstraint.Fill.BOTH))
				fullHeight = true;

			if (hAlign == LyConstraint.HAlign.CENTER)
				align = "center";
			else if (hAlign == LyConstraint.HAlign.RIGHT)
				align = "right";
		}

		int		areaWidth = getAreaWidth();
		int		areaHeight = getAreaHeight();
		String	width = "";
		String	height = "";

		if (areaWidth > 0)
			width = "width='" + areaWidth + "'";
		else
			width = (fullWidth ?  "width='100%'" : "");

		if (isInBorderPanel)
			fullHeight = false;

		if (areaHeight > 0)
			height = "height='" + areaHeight + "'";
		else
			height = (fullHeight ? "height='100%'" : "");

		if (_area.getRedirect() != null)
		{
			String	redirect = _area.getRedirect();
			boolean isIndexPage = false;

			// If redirection to default page, check if index page mode is activated.
			if ("LY_DEFAULT_PAGE".equals(redirect))
			{
				isIndexPage = testResource("LY_WEB_INDEX_PAGE");

				if (isIndexPage)
				{
					// get the url to the event handler
					LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();
					String			    actionAbsoluteUrl = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_INDEX_PAGE, null, false);

					redirect = actionAbsoluteUrl;
				}
			}

			if (!isIndexPage)
				redirect = getUrl(redirect);

			pw.append("<iframe src='" + redirect + "' " + width + " " + height + " frameborder='0'></iframe>");
			return;
		}

		if (generateExternalDiv)
		{
			pw.append("<div id='area" + _area.getIdCode() + "'");

			if (!LyWebConstants.POPUP_VIEW.equals(_lastWindowName)
					|| (fullHeight && fullWidth))
			{
				Vector<LySplitter>	subSplitters = new Vector<LySplitter>();

				_area.getComponents(LySplitter.class, subSplitters);

				if (subSplitters.size() > 0)
					pw.append(" style='width:100%;height:100%;'");
			}

			pw.append(">\n");
		}

		// Generate an additional table with the area background if needed
		String	bgImage = _area.getEnvironment().getImageUrl("LY_AREA_BACKGROUND_IMAGE");
		LyArea	parentArea = _area.getParentArea();
		String	parentBgImage = ((parentArea == null)
				? null : parentArea.getEnvironment().getImageUrl("LY_AREA_BACKGROUND_IMAGE"));
		boolean	generateBackground = (bgImage != null) && (parentBgImage != null) && !bgImage.equals(parentBgImage);
		int		borderWidth = _area.getBorderWidth();
		String  stylearea = "";

		if (pw.isPrintMode())
		{
			generateBackground = false;
			border = (_area.getBorder() != LyArea.Border.NONE ? 1 : 0);
			generateBorder &= (border == 0);
		}

		Object		paramLeft = null;
		Object		paramBottom = null;

		if (generateBorder)
		{
			if ((label != null) && (label.length() > 0))
				label = "&nbsp;" + translate(label) + "&nbsp;";
			else if (label != null)
				label = null;

		    Object      paramBegin = label;

			if (_area.isClosable())
			{
				paramBegin = new String[] {label, getOpenAreaScript(pw), _area.isOpen() + "", _area.getIdCode() + ""};
				paramLeft = (_area.isOpen() ? new String[] {"true", _area.getIdCode() + ""} : new String[] {"false", _area.getIdCode() + ""});
				paramBottom = (_area.isOpen() ? new String[] {"true", _area.getIdCode() + ""} : new String[] {"false", _area.getIdCode() + ""});
			}

			int beginStyle, leftStyle;

			if (fullWidth)
			{
				beginStyle = (fullHeight ? LyWebDecoration.AREA_BORDER_BEGIN : LyWebDecoration.AREA_BORDER_BEGIN_NOT_FULL_HEIGHT);
				leftStyle = (fullHeight ? LyWebDecoration.AREA_BORDER_LEFT : LyWebDecoration.AREA_BORDER_LEFT_NOT_FULL_HEIGHT);
			}
			else
			{
				beginStyle = (fullHeight ? LyWebDecoration.AREA_BORDER_BEGIN_NOT_FULL_WIDTH : LyWebDecoration.AREA_BORDER_BEGIN_NOT_FULL_WIDTH_AND_HEIGHT);
				leftStyle = (fullHeight ? LyWebDecoration.AREA_BORDER_LEFT_NOT_FULL_WIDTH : LyWebDecoration.AREA_BORDER_LEFT_NOT_FULL_WIDTH_AND_HEIGHT);
			}

			deco.writePageDecoration(pw, beginStyle, paramBegin);
			deco.writePageDecoration(pw, leftStyle, paramLeft);
		}

		if (!_area.isDefaultFont() || !_area.isDefaultFontColor())
			stylearea += getFontString(_area.getFont(),_area.getFontColor());

		if (!_area.isDefaultColor())
		{
			stylearea += " background-color:"+getHexColor(_area.getColor())+";";
			stylearea += " background-image:none;";
		}

		if (stylearea.trim().length() > 0)
			stylearea +=" style=\'"+stylearea+"\'";

		// Generate a sub-table only if needed (only if area has a border)
		boolean		generateSubTable = (borderWidth > 0)
			|| generateBorder
			|| ((borderWidth <= 0) && !_area.isDefaultColor() && (_area.getColor() != null));
		int			colspanValue = 1;
		String		colspanString = "";
		boolean		generateGlobalTable = (topLeftToolbar != null)
			|| (topRightToolbar != null)
			|| (titleBar != null)
			|| ((tip != null) && (tip.length() > 0))
			|| (borderWidth > 0)
			|| (bottomLeftToolbar != null)
			|| (bottomRightToolbar != null)
			|| (commands != null)
			|| (selectbar != null)
			|| !generateBorder
			|| (stylearea.trim().length() > 0)
			|| (border > 0)
			|| (areaWidth > 0)
			|| (areaHeight > 0);

		boolean	isEmpty = ((leftToolbar == null)
				&& (rightToolbar == null)
				&& (topLeftToolbar == null)
				&& (topRightToolbar == null)
				&& (northBorderPanel == null)
				&& (southBorderPanel == null)
				&& (eastBorderPanel == null)
				&& (westBorderPanel == null)
				&& (bottomLeftToolbar == null)
				&& (bottomRightToolbar == null)
				&& (selectbar == null)
				&& (commands == null));

		if (isEmpty
				&& (titleBar == null)
				&& (tip == null)
				&& !generateBorder
				&& ((label == null) || (label.length() == 0))
				&& (nPanel == 1)
				&& (_area.getPanel(0) instanceof LyPanel)
				&& (_area.getPanel(0) instanceof LyGetConstraintInterface)
				&& compareAreas(_area, (LyGetConstraintInterface)_area.getPanel(0)))
		{
			LyPanel		panel = _area.getPanel(0);
			LyWebObject	internal = (LyWebObject)panel.getInternalView();

			internal.writeHtml(pw);

			return;
		}

		if (generateBackground)
			pw.append("<table height='100%' width='100%' style='background:url(" + bgImage + ");'><tr><td>");

		if (generateGlobalTable)
		{
			pw.append("<table " + width + " " + height
					+ " cellpadding='0' cellspacing='0' border='" + border
					+ "' " + (align.length() > 0 ? "align='" + align + "' " : "") + stylearea +">\n");
		}

		if (!generateSubTable)
		{
			colspanValue = 0;

			if (leftToolbar != null)
				colspanValue++;

			if (rightToolbar != null)
				colspanValue++;

			if (nPanel > 0)
			{
				if (USE_AJAX_FOR_BORDER_PANELS)
					colspanValue += 3;
				else
				{
					if (westBorderPanel != null)
						colspanValue++;

					if (eastBorderPanel != null)
						colspanValue++;
				}
			}
		}

		if (colspanValue > 1)
			colspanString = " colspan='" + colspanValue + "'";

		if (titleBar != null)
		{
			LyStrutsTitleBar internal = (LyStrutsTitleBar)titleBar.getInternalView();

			pw.append("<tr valign='top' height='10'>\n");
			pw.append("<td valign='top'" + colspanString + ">\n");
			internal.writeHtml(pw);
			pw.append("</td>\n");
			pw.append("</tr>\n");
		}

		boolean	optimizeTables = false;

		// If area contains only panels, just write them in a table to avoid too many table elements
		if (optimizeTables
				&& isEmpty)
		{
			_area.computePanelsPositions();

			writePanels(pw, false, false);
		}
		else
		{
			// Write top toolbars
			if (((topLeftToolbar != null) && (topLeftToolbar.getComponentCount() > 0))
					|| ((topRightToolbar != null) && (topRightToolbar.getComponentCount() > 0)))
			{
				pw.append("<tr valign='top' height='10'>\n");
				pw.append("<td valign='top'" + colspanString + ">\n");
				pw.append("<table border='0' cellpadding='2' cellspacing='0' width='100%' height='100%'>\n");
				pw.append("<tr>\n");

				if (topLeftToolbar != null)
				{
					LyStrutsToolBar internal = (LyStrutsToolBar)topLeftToolbar.getInternalView();

					pw.append("<td align='left' valign='top' class='" + getToolbarClass() + "'>\n");
					internal.writeHtml(pw);
					pw.append("</td>\n");
				}

				if (topRightToolbar != null)
				{
					LyStrutsToolBar internal = (LyStrutsToolBar)topRightToolbar.getInternalView();

					pw.append("<td align='right' valign='top' class='" + getToolbarClass() + "'>\n");
					internal.writeHtml(pw);
					pw.append("</td>\n");
				}

				pw.append("</tr>\n");
				pw.append("</table>\n");
				pw.append("</td>\n");
				pw.append("</tr>\n");
			}

			// Write tip
			writeTip(pw, colspanString);

			if (!generateBorder && (label != null) && (label.length() > 0))
				pw.append("<tr><td height='20' valign='top'" + colspanString + "><font class='areaGroup'>&nbsp;" + translate(label) + "&nbsp;</font></td></tr>\n");

			if (generateSubTable)
			{
				if (generateGlobalTable)
				{
					pw.append("<tr valign='top'");

					if ((borderWidth <= 0) && !_area.isDefaultColor() && (_area.getColor() != null))
						pw.append(" bgColor='" + getHexColor(_area.getColor()) + "'");

					pw.append(">\n");
					pw.append("<td valign='top'>\n");

					if (borderWidth > 0)
					{
						pw.append("<table border='0' cellpadding='0' cellspacing='" + borderWidth +"' width='100%' height='100%'><tr><td");

						if (!_area.isDefaultColor() && (_area.getColor() != null))
							pw.append(" bgColor='" + getHexColor(_area.getColor()) + "'");

						pw.append(">\n");
					}
				}
			}

			// Write center zone : left toolbar, panels and right toolbar
			if ((nPanel > 0)
					|| (northBorderPanel != null)
					|| (southBorderPanel != null)
					|| (eastBorderPanel != null)
					|| (westBorderPanel != null)
					|| (leftToolbar != null)
					|| (rightToolbar != null))
			{
				if (generateSubTable)
					pw.append("<table border='0' cellpadding='0' cellspacing='0' width='100%' height='100%'>\n");

				pw.append("<tr>\n");

				if (leftToolbar != null)
				{
					LyStrutsToolBar internal = (LyStrutsToolBar)leftToolbar.getInternalView();

					if (internal != null)
					{
						pw.append("<td id='expandToolbarCell" + _area.getIdCode() + "' align='left' valign='middle' height='100%'");

						if ((leftToolbar.getStyle() == LyToolBar.Style.EXPAND) && !leftToolbar.isOpen())
							pw.append(" style='display:none;'");

						pw.append(">\n");

						if (leftToolbar.getStyle() != LyToolBar.Style.EXPAND)
							pw.append("<table cellpadding='0' cellspacing='0' border='0'><tr><td>\n");

						if (useScroll)
							pw.append("<div style='overflow:auto;height:100%;'>");

						internal.writeHtml(pw);

						if (useScroll)
							pw.append("</div>");

						if (leftToolbar.getStyle() != LyToolBar.Style.EXPAND)
							pw.append("</td></tr></table>\n");

						pw.append("</td>\n");

						if (leftToolbar.getStyle() == LyToolBar.Style.EXPAND)
						{
							String			imageCollapse= getImageUrl("LY_EXPANDBAR_COLLAPSE");
							String			imageExpand = getImageUrl("LY_EXPANDBAR_EXPAND");
							LyView			view = _area.getParentView();
							LyWebPageWriter	pageWriter = (LyWebPageWriter)view.getInternalView();

							LyWebRequestInfo		info = LyStrutsServlet.getWebRequestInfo();
							HttpServletRequest		request = (info == null ? null : info.getRequest());
							String					target = (request == null ? null : (String)request.getParameter(LyWebConstants.TARGET_PARAM));

							pw.append("<td valign='middle' class='expandBarSwitch'>");
							pw.append("<a href=\"javascript:SwitchExpandBar('expandToolbarCell" + _area.getIdCode() + "', 'expandImg_" + leftToolbar.getIdCode() + "', '" +
									imageCollapse + "', '" + imageExpand + "', '" + pageWriter.getId() + "', '" + leftToolbar.getId() + "','" + target + "');\">");
							pw.append("<img id='expandImg_" + leftToolbar.getIdCode() + "' src='" + (leftToolbar.isOpen() ? imageCollapse : imageExpand) + "' border='0'/></a>");
							pw.append("</td>");
						}
					}
				}

				if (nPanel > 0)
				{
					if (USE_AJAX_FOR_BORDER_PANELS || (westBorderPanel != null))
					{
						// Write cell for west panel
						pw.append("<td width='1' align='left' valign='top' id='west_panel_cell_" + hashCode() + "'>\n");

						if (westBorderPanel != null)
						{
							LyWebObject internal = (LyWebObject)westBorderPanel.getInternalView();
							internal.writeHtml(pw);
						}

						pw.append("</td>\n");
					}

					boolean		hFill = (_area.getLayout() == LyArea.Layout.BORDER) || (_area.getLayout() == LyArea.Layout.FILL);
					boolean		vFill = (_area.getLayout() == LyArea.Layout.FILL);

					// For a grid layout, check if at least one component has a vertical fill constraint
					if (_area.getLayout() == LyArea.Layout.GRID)
					{
						int				i;
						LyPanel			panel;
						LyConstraint	constraint;

						for (i = 0; i < nPanel; i++)
						{
							panel = _area.getPanel(i);

							if (panel instanceof LyGetConstraintInterface)
							{
								constraint = ((LyGetConstraintInterface)panel).getConstraint();

								if ((constraint != null)
										&& ((constraint.getFill() == LyConstraint.Fill.VERTICAL)
												|| (constraint.getFill() == LyConstraint.Fill.BOTH)
												|| (constraint.getVAlign() == LyConstraint.VAlign.MIDDLE)
												|| (constraint.getVAlign() == LyConstraint.VAlign.BOTTOM)))
									vFill = true;

								if ((constraint != null)
										&& ((constraint.getFill() == LyConstraint.Fill.HORIZONTAL)
												|| (constraint.getFill() == LyConstraint.Fill.BOTH)
												|| (constraint.getHAlign() == LyConstraint.HAlign.CENTER)
												|| (constraint.getHAlign() == LyConstraint.HAlign.RIGHT)))
									hFill = true;
							}
						}
					}
					// For a border layout, check if west, center or east panel is set
					else if (_area.getLayout() == LyArea.Layout.BORDER)
					{
						String	layoutInfo = _area.getLayoutInfo();

						if (((layoutInfo == null) && (_area.getPanelCount() > 1))
								|| (layoutInfo.indexOf('E') >= 0)
								|| (layoutInfo.indexOf('C') >= 0)
								|| (layoutInfo.indexOf('W') >= 0))
							vFill = true;
					}
					else if (_area.getLayout() == LyArea.Layout.FLOW)
					{
						String	layoutInfo = _area.getLayoutInfo();

						if ((layoutInfo != null)
								&& ((layoutInfo.indexOf("MIDDLE") >= 0)
									|| (layoutInfo.indexOf("BOTTOM") >= 0)))
							vFill = true;
					}

					// Write center zone : north panel, panels, south panel
					String	parentId = "mainScroll_" + _area.getIdCode();

					pw.append("<td");

					if ((pw.getActiveName() != LyWebConstants.TOOL_POPUP)
							&& !LyWebConstants.POPUP_VIEW.equals(_lastWindowName))
					{
						pw.append(" id='" + parentId + "'");
						pw.append(" width='100%'");
					}

					pw.append(" vAlign='top'>\n");

					if (useScroll)
						pw.append("<div id='" + parentId + "_div' style='overflow:auto;width:100%;height:100%;'>");

					pw.append("<table border='0' cellpadding='0' cellspacing='0'");

					// An area with a flow layout must not fill its parent width
					if ((_area.getLayout() != LyArea.Layout.FLOW) && hFill)
						pw.append(" width='100%'");
					else if (_area.getLayout() == LyArea.Layout.FLOW)
					{
						// For a flow layout, retrieve horizontal align
						String	layoutInfo = _area.getLayoutInfo();

						if (layoutInfo == null)
							layoutInfo = "";

						if (layoutInfo.indexOf("RIGHT") >= 0)
							pw.append(" align='right'");
						else if (layoutInfo.indexOf("CENTER") >= 0)
							pw.append(" align='center'");
					}

					if (vFill)
						pw.append(" height='100%'");

					pw.append(">\n");

					// Do not generate north nor south border panels when area contains a splitter otherwise it will add
					// add a 1-pixel margin that may disturb the Javascript function that computes splitter sizes.
					boolean		generateVerticalBorderPanels = ((_area.getPanelCount() == 0) || !(_area.getPanel(0) instanceof LySplitter));

					// Write cell for north panel
					if (generateVerticalBorderPanels && (USE_AJAX_FOR_BORDER_PANELS || (northBorderPanel != null)))
					{
						// Write cell for north panel
						pw.append("<tr height='0'>\n");
						pw.append("<td align='left' valign='top' id='north_panel_cell_" + hashCode() + "'>\n");

						if (northBorderPanel != null)
						{
							LyWebObject	internal = (LyWebObject)northBorderPanel.getInternalView();
							internal.writeHtml(pw);
						}

						pw.append("</td>\n");
						pw.append("</tr>\n");
					}

					writePanels(pw, hFill, vFill);

					if (generateVerticalBorderPanels && (USE_AJAX_FOR_BORDER_PANELS || (southBorderPanel != null)))
					{
						// Write cell for south panel
						pw.append("<tr height='0'>\n");
						pw.append("<td align='left' valign='bottom' id='south_panel_cell_" + hashCode() + "'>\n");

						if (southBorderPanel != null)
						{
							LyWebObject	internal = (LyWebObject)southBorderPanel.getInternalView();

							internal.writeHtml(pw);
						}

						pw.append("</td>\n");
						pw.append("</tr>\n");
					}

					pw.append("</table>\n");

					if (useScroll)
					{
						pw.append("</div>");

						StringBuffer	script = new StringBuffer();

						script.append("<script type='text/javascript'>");
						script.append("getLeonardiTopFrame().addSplitterIds(new Array(false");
						script.append(",'" + parentId + "','" + parentId + "_div','','full',false));");
						script.append("</script>");

						pw.addEndScript(script.toString(), false);
					}

					pw.append("</td>\n");

					if (USE_AJAX_FOR_BORDER_PANELS || (eastBorderPanel != null))
					{
						// Write cell for east panel
						pw.append("<td width='1' align='right' valign='top' id='east_panel_cell_" + hashCode() + "'>\n");

						if (eastBorderPanel != null)
						{
							LyWebObject internal = (LyWebObject)eastBorderPanel.getInternalView();
							internal.writeHtml(pw);
						}

						pw.append("</td>\n");
					}
				}

				if (rightToolbar != null)
				{
					LyStrutsToolBar internal = (LyStrutsToolBar)rightToolbar.getInternalView();

					pw.append("<td align='right' valign='top'>\n");
					internal.writeHtml(pw);
					pw.append("</td>\n");
				}

				pw.append("</tr>\n");

				if (generateSubTable)
					pw.append("</table>\n");
			}

			if (generateSubTable)
			{
				if (generateGlobalTable)
				{
					if (borderWidth > 0)
					{
						pw.append("</td></tr></table>\n");
					}

					pw.append("</td>\n");
					pw.append("</tr>\n");
				}
			}

			// Write bottom toolbars
			if ((bottomLeftToolbar != null) || (bottomRightToolbar != null))
			{
				pw.append("<tr valign='top' height='10'>\n");
				pw.append("<td valign='top'" + colspanString + ">\n");
				pw.append("<table border='0' cellpadding='0' cellspacing='0' width='100%'>\n");
				pw.append("<tr>\n");

				if (bottomLeftToolbar != null)
				{
					LyStrutsToolBar internal = (LyStrutsToolBar)bottomLeftToolbar.getInternalView();

					pw.append("<td align='left' valign='top'>\n");
					internal.writeHtml(pw);
					pw.append("</td>\n");
				}

				if (bottomRightToolbar != null)
				{
					LyStrutsToolBar internal = (LyStrutsToolBar)bottomRightToolbar.getInternalView();

					pw.append("<td align='right' valign='top'>\n");
					internal.writeHtml(pw);
					pw.append("</td>\n");
				}

				pw.append("</tr>\n");
				pw.append("</table>\n");
				pw.append("</td>\n");
				pw.append("</tr>\n");
			}

			// Write selectbar
			if (selectbar != null)
			{
				LyStrutsSelectBar internal = (LyStrutsSelectBar)selectbar.getInternalView();

				pw.append("<tr valign='top' height='10'>\n");
				pw.append("<td valign='top'" + colspanString + ">\n");
				internal.writeHtml(pw);
				pw.append("</td>\n");
				pw.append("</tr>\n");
			}

			// Write commands
			if (commands != null)
			{
				LyStrutsCommands internal = (LyStrutsCommands)commands.getInternalView();

				pw.append("<tr valign='top' height='10'>\n");
				pw.append("<td valign='top'" + colspanString + ">\n");
				internal.writeHtml(pw);
				pw.append("</td>\n");
				pw.append("</tr>\n");
			}
		}

		if (generateGlobalTable)
			pw.append("</table>");

		if (generateBackground)
			pw.append("</td></tr></table>");

		if (generateBorder)
		{
			deco.writePageDecoration(pw, LyWebDecoration.AREA_BORDER_RIGHT);
			deco.writePageDecoration(pw, LyWebDecoration.AREA_BORDER_END, paramBottom);
		}

		if (generateExternalDiv)
			pw.append("</div>\n");

		boolean	replayMode = getEnvironment().testEnv("LY_REPLAY");
		int		refresh = _area.getRefresh();

		if (!replayMode && (refresh > 0))
		{
			refresh = getWebViewManager().getRefreshRate();

			if (refresh > 0)
			{
				StringBuffer	buffer = new StringBuffer();

				buffer.append("<script type='text/javascript'>\n//<![CDATA[\n");
				buffer.append("setTimeout(\"getLeonardiTopFrame().AutoRefresh(false)\", " + (refresh * 1000) + ");");
				buffer.append("\n//]]>\n</script>\n");
				pw.addEndScript(buffer.toString(), false);
			}
		}

		pw.endComponent(this);
	}

	/**
	 * Check if two areas have the same constraint infos.
	 */
	protected boolean compareAreas(LyGetConstraintInterface area1, LyGetConstraintInterface area2)
	{
		// If tabs is contained in an area and in a parent frame, force the generation of HTML
		// code for parent area (needed for Javascript code generated in the writeHtml() method
		// of the LyStrutsTabs for IE.
		if ((area2 instanceof LyTabs)
				&& (((LyTabs)area2).getParentView() instanceof LyFrame))
			return false;

		if ((area1 == null) && (area2 == null))
			return true;
		if ((area1 == null) || (area2 == null))
			return false;

		LyConstraint		constraint1 = area1.getConstraint();
		LyConstraint		constraint2 = area2.getConstraint();
		LyConstraint.Fill	fill1 = LyConstraint.Fill.HORIZONTAL;
		LyConstraint.HAlign	hAlign1 = LyConstraint.HAlign.LEFT;
		LyConstraint.Fill	fill2 = LyConstraint.Fill.HORIZONTAL;
		LyConstraint.HAlign	hAlign2 = LyConstraint.HAlign.LEFT;

		if (constraint1 != null)
		{
			fill1 = constraint1.getFill();
			hAlign1 = constraint1.getHAlign();
		}

		if (constraint2 != null)
		{
			fill2 = constraint2.getFill();
			hAlign2 = constraint2.getHAlign();
		}

		// If fill1 == BOTH, allow a more specific constraint
		if ((fill1 != LyConstraint.Fill.BOTH) && (fill1 != fill2))
			return false;

		if (hAlign1 == LyConstraint.HAlign.DEFAULT)
			hAlign1 = LyConstraint.HAlign.LEFT;

		if (hAlign2 == LyConstraint.HAlign.DEFAULT)
			hAlign2 = LyConstraint.HAlign.LEFT;

		if (hAlign1 != hAlign2)
			return false;

		return true;
	}

	/**
	 * Writes the panels contained in this area.
	 *
	 * @param pw current page writer
	 * @param hFill indicates whether the table fill the horizontal space
	 * @param vFill indicates whether the table fill the vertical space
	 */
	protected void writePanels(LyWebPageWriter pw, boolean hFill, boolean vFill)
	{
		// For a card layout, retrieve visible panel. If no layout info is given, or if it does
		// not match any panel, show the last one.
		int		i, nPanels = _area.getPanelCount();
		int		horizontalSpacing = _area.getHorizontalSpacing();
		int		verticalSpacing = _area.getVerticalSpacing();
		String	horizontalSpacingString = "";
		String	verticalSpacingString = "";

		if (horizontalSpacing > 0)
			horizontalSpacingString = "<td><div style='width:" + horizontalSpacing + "px;'></div></td>";

		if (verticalSpacing > 0)
			verticalSpacingString = "<tr height='" + verticalSpacing + "'><td></td></tr>";

		if (_area.getLayout() == LyArea.Layout.BORDER)
		{
			// For a border layout, use the layout info to retrieve each panel position.
			Hashtable<String, LyPanel>	panelsPositions = new Hashtable<String, LyPanel>();
			String						layoutInfo = _area.getLayoutInfo();

			if (layoutInfo != null)
				layoutInfo = layoutInfo.trim();

			String[]	layoutPositions = LyString.stringToList(layoutInfo, ' ');

			if (layoutPositions != null)
			{
				for (i = 0; (i < layoutPositions.length) && (i < _area.getPanelCount()); i++)
					panelsPositions.put(layoutPositions[i], _area.getPanel(i));
			}
			else
			{
				for (i = 0; i < _area.getPanelCount(); i++)
				{
					String	position;

					switch (i)
					{
					case 0:
						position = "N";
						break;
					case 1:
						position = "W";
						break;
					case 2:
						position = "C";
						break;
					case 3:
						position = "E";
						break;
					default:
						position = "S";
						break;
					}

					panelsPositions.put(position, _area.getPanel(i));
				}
			}

			LyPanel		panel;
			LyWebComponent	internal;
			boolean		jump = false;
			LyPanel		westPanel = panelsPositions.get("W");
			LyPanel		centerPanel = panelsPositions.get("C");
			LyPanel		eastPanel = panelsPositions.get("E");
			int			colspan = 3;

			if (horizontalSpacing > 0)
				colspan += (westPanel != null ? 1 : 0) + (centerPanel != null ? 1 : 0) + (eastPanel != null ? 1 : 0) - 1;

			panel = panelsPositions.get("N");

			if (panel != null)
			{
				pw.append("<tr>\n");
				internal = (LyWebComponent)panel.getInternalView();
				pw.append("<td align='left' valign='top' colspan='" + colspan + "'" + getPanelStyle(panel) + ">\n");
				internal.writeHtml(pw);
				pw.append("</td>\n");
				pw.append("</tr>\n");
				jump = true;
			}

			if (jump)
			{
				jump = false;
				pw.append(verticalSpacingString);
			}

			if ((westPanel != null) || (centerPanel != null) || (eastPanel != null))
				pw.append("<tr height='100%'>\n");

			if (westPanel != null)
			{
				internal = (LyWebComponent)westPanel.getInternalView();
				pw.append("<td align='left' valign='top'" + getPanelStyle(westPanel) + ">\n");
				internal.writeHtml(pw);
				pw.append("</td>\n");
				jump = true;

				if ((centerPanel != null) || (eastPanel != null))
					pw.append(horizontalSpacingString);
			}

			if (centerPanel != null)
			{
				internal = (LyWebComponent)centerPanel.getInternalView();
				pw.append("<td align='left' valign='top' width='100%'" + getPanelStyle(centerPanel) + ">\n");
				internal.writeHtml(pw);
				pw.append("</td>\n");
				jump = true;

				if (eastPanel != null)
					pw.append(horizontalSpacingString);
			}

			if (eastPanel != null)
			{
				internal = (LyWebComponent)eastPanel.getInternalView();
				pw.append("<td align='left' valign='top'" + getPanelStyle(eastPanel) + ">\n");
				internal.writeHtml(pw);
				pw.append("</td>\n");
				jump = true;
			}

			if (jump)
				pw.append("</tr>" + verticalSpacingString + "<tr>\n");

			panel = panelsPositions.get("S");

			if (panel != null)
			{
				internal = (LyWebComponent)panel.getInternalView();
				pw.append("<td align='left' valign='top' colspan='" + colspan + "'" + getPanelStyle(panel) + ">\n");
				internal.writeHtml(pw);
				pw.append("</td>\n");
			}
		}
		else
		{
			LyPanel						panel;
			LyWebComponent					internal;
			LyConstraint				constraint;
			int							colspan, rowspan;
			boolean						jump;
			String						align, vAlign, width;
			int							row = 0;
			BitSet						fillVertical = new BitSet();
			Hashtable<Integer, BitSet>	fillHorizontal = new Hashtable<Integer, BitSet>();
			String						fillHeight = "";
			boolean						isPreviousRightAlign = false;

			// For grid layout, compute each <tr> height
			for (i = 0; i < nPanels; i++)
			{
				panel = _area.getPanel(i);
				internal = (LyWebComponent)panel.getInternalView();
				constraint = (panel instanceof LyGetConstraintInterface ? ((LyGetConstraintInterface)panel).getConstraint() : null);

				if ((constraint != null)
						&& ((constraint.getFill() == LyConstraint.Fill.VERTICAL)
							|| (constraint.getFill() == LyConstraint.Fill.BOTH)
							|| (constraint.getVAlign() == LyConstraint.VAlign.MIDDLE)
							|| (constraint.getVAlign() == LyConstraint.VAlign.BOTTOM)))
					fillVertical.set(row);

				if ((constraint != null)
						&& ((constraint.getFill() == LyConstraint.Fill.HORIZONTAL)
							|| (constraint.getFill() == LyConstraint.Fill.BOTH)
							|| (constraint.getHAlign() == LyConstraint.HAlign.CENTER)
							|| ((constraint.getHAlign() == LyConstraint.HAlign.RIGHT) && !isPreviousRightAlign)))
				{
					BitSet	nbFill = fillHorizontal.get(new Integer(row));

					if (nbFill == null)
						nbFill = new BitSet();

					nbFill.set(i);
					fillHorizontal.put(new Integer(row), nbFill);
				}

				if (((constraint == null) || constraint.isRemainder()) && (i < nPanels - 1))
					row++;

				// If panel is the first right align, give it a full width
				isPreviousRightAlign = false;

				if ((constraint != null)
						&& !constraint.isRemainder()
						&& (constraint.getHAlign() == LyConstraint.HAlign.RIGHT))
					isPreviousRightAlign = true;
			}

			if (fillVertical.cardinality() > 0)
				fillHeight = " height='" + (100 / fillVertical.cardinality()) + "%'";

			pw.append("<tr");

			if (fillVertical.get(0))
				pw.append(fillHeight);

			pw.append(">\n");

			LyPanel	cardPanel = null;

			if (_area.getLayout() == LyArea.Layout.CARD)
			{
				for (i = 0; i < nPanels; i++)
				{
					LyPanel item = _area.getPanel(i);

					if ((_area.getLayoutInfo() != null)
							&& _area.getLayoutInfo().equals(item.getId()))
						cardPanel = item;
				}

				if (cardPanel == null)
					cardPanel = _area.getPanel(nPanels - 1);
			}

			row = 0;

			String	defaultVAlign = "top";

			// For a flow layout, retrieve vertical align
			if (_area.getLayout() == LyArea.Layout.FLOW)
			{
				defaultVAlign = "top";

				String	layoutInfo = _area.getLayoutInfo();

				if (layoutInfo == null)
					layoutInfo = "";

				if (layoutInfo.indexOf("MIDDLE") >= 0)
					defaultVAlign = "middle";
				else if (layoutInfo.indexOf("BOTTOM") >= 0)
					defaultVAlign = "bottom";
			}

			for (i = 0; i < nPanels; i++)
			{
				panel = _area.getPanel(i);
				internal = (LyWebComponent)panel.getInternalView();
				constraint = (panel instanceof LyGetConstraintInterface ? ((LyGetConstraintInterface)panel).getConstraint() : null);
				colspan = (constraint != null ? constraint.getColSpan() : 1);
				rowspan = (constraint != null ? constraint.getRowSpan() : 1);
				jump = false;
				align = "left";
				vAlign = defaultVAlign;
				width = "";

				if ((colspan > 1) && (horizontalSpacing > 0))
					colspan = 2 * colspan - 1;

				if (constraint != null)
				{
					if (constraint.getHAlign() == LyConstraint.HAlign.CENTER)
						align = "center";
					else if (constraint.getHAlign() == LyConstraint.HAlign.RIGHT)
						align = "right";

					if (constraint.getVAlign() == LyConstraint.VAlign.BOTTOM)
						vAlign = "bottom";
					else if (constraint.getVAlign() == LyConstraint.VAlign.MIDDLE)
						vAlign = "middle";
				}

	            // If card layout, only add the visible panel
	            if ((cardPanel != null) && (panel != cardPanel))
	            	continue;
				else if (((constraint == null) || constraint.isRemainder()) && (i < nPanels - 1))
					jump = true;

	            // In a fill layout, components occupy the same place
	            if (_area.getLayout() == LyArea.Layout.FILL)
	            	width = "width='" + (100 / nPanels) + "%'";
	            else
	            {
	            	BitSet	nbFill = fillHorizontal.get(new Integer(row));

	            	if (hFill
	            			&& (nbFill != null)
	            			&& (nbFill.size() > 0)
	            			&& nbFill.get(i)
	            			&& (constraint != null)
	            			&& ((constraint.getFill() == LyConstraint.Fill.HORIZONTAL)
	            					|| (constraint.getFill() == LyConstraint.Fill.BOTH)
	            					|| (constraint.getHAlign() == LyConstraint.HAlign.CENTER)
	            					|| (constraint.getHAlign() == LyConstraint.HAlign.RIGHT)))
		            	width = "width='" + (100 / nbFill.cardinality()) + "%'";
	            	else if (hFill
	            			&& jump
	            			&& (((nbFill != null)
	            					&& (nbFill.cardinality() <= 0)) || (nbFill == null)))
		            	width = "width='99%'";
	            }

				pw.append("<td align='" + align + "' valign='" + vAlign + "'");

				if (colspan > 1)
					pw.append(" colspan='" + colspan + "'");

				if (rowspan > 1)
					pw.append(" rowspan='" + rowspan + "'");

				pw.append(" " + width + getPanelStyle(panel) + ">\n");

				// Another table must be generated for tools to manage their horizontal alignment correctly
				if (panel instanceof LyTool)
					pw.append("<table border='0' cellpadding='0' cellspacing='0'><tr><td>");

				internal.writeHtml(pw);

				if (panel instanceof LyTool)
					pw.append("</td></tr></table>");

				pw.append("</td>\n");

				if ((_area.getLayout() == LyArea.Layout.FILL) || (_area.getLayout() == LyArea.Layout.FLOW))
					continue;

				if (jump)
				{
					row++;
					pw.append("</tr>" + verticalSpacingString + "<tr");

					if (fillVertical.get(row))
						pw.append(fillHeight);

					pw.append(">\n");
				}
				else if (i < nPanels - 1)
					pw.append(horizontalSpacingString);
			}
		}

		if (nPanels == 0)
			pw.append("<td></td>");

		pw.append("</tr>\n");
	}

	/**
	 * Gets the style to be used for the cell containing a given panel.
	 *
	 * @param panel the panel contained in the cell.
	 * @return a string representing the style to be used.
	 */
	protected String getPanelStyle(LyPanel panel)
	{
		if (!(panel instanceof LyGetConstraintInterface))
			return "";

		LyConstraint	constraint = ((LyGetConstraintInterface)panel).getConstraint();

		if (constraint == null)
			return "";

		String			style = "";

		if (constraint.getCellPadding() > 0)
			style += "padding:" + constraint.getCellPadding() + "px;";

		if (constraint.getLeftPadding() > 0)
			style += "padding-left:" + constraint.getLeftPadding() + "px;";

		if (constraint.getRightPadding() > 0)
			style += "padding-right:" + constraint.getRightPadding() + "px;";

		if (constraint.getTopPadding() > 0)
			style += "padding-top:" + constraint.getTopPadding() + "px;";

		if (constraint.getBottomPadding() > 0)
			style += "padding-bottom:" + constraint.getBottomPadding() + "px;";

		if (style.trim().length() > 0)
			style = " style='" + style + "'";

		return style;
	}

	/**
	 * Set a border panel at the given position.
	 *
	 * @param panel    the border panel.
	 * @param position Either NORTH, SOUTH, EAST or WEST position.
	 */
	public void setBorderPanel(LyPanel panel, short position)
	{
		boolean		generateVerticalBorderPanels = USE_AJAX_FOR_BORDER_PANELS && ((_area.getPanelCount() == 0) || !(_area.getPanel(0) instanceof LySplitter));

		if (!generateVerticalBorderPanels)
		{
			refresh();
			return;
		}

		LyWebViewManager viewManager = getWebViewManager();
		StringBuffer     buffer = new StringBuffer();
		String			 innerHtml = "";
		String			 id = getIdCode();

		if (position == LyArea.NORTH)
			id = "north_panel_cell_" + id;
		else if (position == LyArea.SOUTH)
			id = "south_panel_cell_" + id;
		else if (position == LyArea.WEST)
			id = "west_panel_cell_" + id;
		else if (position == LyArea.EAST)
			id = "east_panel_cell_" + id;

		// Activate buffer for additional refresh scripts since additional refresh scripts generated
		// by border panel must be added after replaceDivInnerHtml() script generated in current method.
		viewManager.activateAdditionalRefreshScriptsBuffer(true);

		if (panel != null)
		{
			// Generate the content of the border panel with a page writer that has the
			// same form id as the page used to generate HTML page. Therefore, generated
			// fields contained in this border panel will have the same form id as the
			// form they are contained in.
			LyStrutsArea 	internal = (LyStrutsArea)panel.getInternalView();
			LyStrutsPage		pw = new LyStrutsPage(_area, true);
			LyView			view = _area.getParentView();
			LyWebPageWriter	viewWriter = (LyWebPageWriter)view.getInternalView();

			pw.setFormId(viewWriter.getActiveFormId());
			pw.setActiveBuffer(viewWriter.getActiveName());
			pw.setWindowName(viewWriter.getWindowName());
			internal.writeHtml(pw);
			innerHtml = pw.getBuffer(pw.getActiveName()).toString();
			pw.free();
		}

		String innerHtmlStr = LyString.replace(innerHtml, "\n", "\\n");

		innerHtmlStr = LyString.replace(innerHtmlStr, "\\", "\\\\");
		innerHtmlStr = LyString.replace(innerHtmlStr, "\\\\n", "\\n");
		innerHtmlStr = LyString.replace(innerHtmlStr, "\"", "\\\"");
		innerHtmlStr = LyString.replace(innerHtmlStr, "\\'", "\\\\'");

		buffer.append("<script type='text/javascript'>\n//<![CDATA[\nreplaceDivInnerHtml('").append(
			_lastFrameName).append("','").append(id).append("',\"").append(innerHtmlStr).append(
			"\");");

		if (panel != null)
			buffer.append("ResetFormFocusImmediatly(false);");

		// If area is displayed in a popup, check popup size
		if (LyWebConstants.POPUP_VIEW.equals(_lastWindowName))
			buffer.append("checkSizePopupView(null, false)\n");

		buffer.append("//]]>\n</script>");

		viewManager.addAdditionalRefreshScripts(buffer.toString(), false);
		viewManager.activateAdditionalRefreshScriptsBuffer(false);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.graphic.LyGraphicalArea#addPanel(leon.view.LyPanel, int)
	 */
	public void addPanel(LyPanel panel, int pos)
	{
		LyPanel		panelObject = getParentBorderPanel();
		LyComponent parentComponent = _area.getParent();

		if ((panelObject != null) ||(parentComponent instanceof LySplitter))
		{
			LyStrutsViewManager  viewManager = (LyStrutsViewManager)getViewManager();
			String				id = "area" + _area.getIdCode();

			viewManager.addReloadDiv(_lastFrameName, id, this);
		}
		else
		{
			if (LyWebConstants.TOOL_POPUP.equals(_lastFrameName))
			{
				LyComponent	parent = _area;

				while (parent != null)
				{
					parent = parent.getParent();

					if (parent instanceof LyTool)
					{
						LyStrutsTool	webTool = (LyStrutsTool)parent.getInternalView();

						webTool.refresh();
						break;
					}
				}
			}
			else
			{
		        getSubElementId(panel);

		        refresh();
			}
		}
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.graphic.LyGraphicalArea#removePanel(leon.view.LyPanel)
	 */
	public void removePanel(LyPanel panel)
	{
		LyPanel		panelObject = getParentBorderPanel();
		LyComponent parentComponent = _area.getParent();

		if ((panelObject != null) ||(parentComponent instanceof LySplitter))
		{
			LyStrutsViewManager  viewManager = (LyStrutsViewManager)getViewManager();
			String				id = "area" + _area.getIdCode();

			viewManager.addReloadDiv(_lastFrameName, id, this);
		}
		else
		{
	        if ((_subElementIds != null) && _subElementIds.containsKey(panel))
	        {
	            String id = getSubElementId(panel);

	            _subElementIds.remove(panel);
	            _availableId = id;
	        }

	        LyWebViewManager viewManager = getWebViewManager();
	        LyWebComponent   webComponent = (LyWebComponent)panel.getInternalView();

	        if (webComponent instanceof LyWebPageWriter)
	            viewManager.removePageWriter((LyWebPageWriter)webComponent);

	        refresh();
		}
	}

	/**
	 * Retrieve the first parent border panel if existing.
	 *
	 * @return the first parent border panel if existing, null otherwise.
	 */
	protected LyPanel getParentBorderPanel()
	{
		LyPanel	panelObject = _area;
		LyArea	parentArea = null;

		while (panelObject != null)
		{
			parentArea = panelObject.getParentArea();

			if ((parentArea != null) && (parentArea.isBorderPanel(panelObject)))
				return panelObject;

			panelObject = parentArea;
		}

		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalArea#setOpen(boolean)
	 */
	public void setOpen(boolean open)
	{
		if (_area.isClosable())
		{
			String	parentLeftTRId = "borderLeftTR" + _area.getIdCode();
			String	parentBottomTRId = "borderBottomTR" + _area.getIdCode();

			LyWebViewManager viewManager = getWebViewManager();
			StringBuffer     buffer = new StringBuffer();
			LyEnvironment    environment = getEnvironment();

			buffer.append("<script type='text/javascript'>\n//<![CDATA[\n");
			buffer.append("displayElement('");
			buffer.append(_lastFrameName);
			buffer.append("','");
			buffer.append(parentLeftTRId);
			buffer.append("',");
			buffer.append(open);
			buffer.append(");");
			buffer.append("displayElement('");
			buffer.append(_lastFrameName);
			buffer.append("','");
			buffer.append(parentBottomTRId);
			buffer.append("',");
			buffer.append(open);
			buffer.append(");");

			buffer.append("SetImageSrc('imgOpen" + _area.getIdCode() + "','" +
					(open ? getImageUrl("LY_AREA_BUTTON_CLOSE") : getImageUrl("LY_AREA_BUTTON_OPEN")) + "');");

			buffer.append("SetImageSrc('imgLeftOpen" + _area.getIdCode() + "','" +
					(open ? getImageUrl("LY_AREA_BORDER_TOP_LEFT") : getImageUrl("LY_AREA_BORDER_TOP_LEFT_CLOSED")) + "');");

			buffer.append("SetImageSrc('imgRightOpen" + _area.getIdCode() + "','" +
					(open ? getImageUrl("LY_AREA_BORDER_TOP_RIGHT") : getImageUrl("LY_AREA_BORDER_TOP_RIGHT_CLOSED")) + "');");

			buffer.append("SetImageSrc('imgLabelLeftOpen" + _area.getIdCode() + "','" +
					(open ? environment.getImageUrl("LY_AREA_BORDER_LABEL_LEFT", false, false) :
					    environment.getImageUrl("LY_AREA_BORDER_LABEL_LEFT_CLOSED", false, false)) + "');");

			buffer.append("SetImageSrc('imgLabelRightOpen" + _area.getIdCode() + "','" +
					(open ? environment.getImageUrl("LY_AREA_BORDER_LABEL_RIGHT", false, false) :
					    environment.getImageUrl("LY_AREA_BORDER_LABEL_RIGHT_CLOSED", false, false)) + "');");

			if (getEnvironment().getImageUrl("LY_AREA_BORDER_LABEL", false, false) != null)
				buffer.append("SetObjectBackgroundImage('td1" + _area.getIdCode() + "','url(" +
						(open ? getImageUrl("LY_AREA_BORDER_LABEL") : getImageUrl("LY_AREA_BORDER_LABEL_CLOSED")) + ")','repeat-x');");

			buffer.append("SetObjectBackgroundImage('td2" + _area.getIdCode() + "','url(" +
					(open ? getImageUrl("LY_AREA_BORDER_TOP") : getImageUrl("LY_AREA_BORDER_TOP_CLOSED")) + ")','repeat-x');");

			buffer.append("SetObjectBackgroundImage('td3" + _area.getIdCode() + "','url(" +
					(open ? getImageUrl("LY_AREA_BORDER_TOP") : getImageUrl("LY_AREA_BORDER_TOP_CLOSED")) + ")','repeat-x');");

			if (LyWebConstants.POPUP_VIEW.equals(_lastWindowName))
				buffer.append("checkSizePopupView(null, false);");
			else
				buffer.append("checkSizeMainView();");

			buffer.append("\n//]]>\n</script>");

			viewManager.addAdditionalRefreshScripts(buffer.toString());
		}
	}

	/**
	 * Gets the script used to open the area.
	 *
	 * @param  pw the current page writer.
	 * @return The javascript for opening a toolbar.
	 */
	protected String getOpenAreaScript(LyWebPageWriter pw)
	{
		// get toolbar infos
		String			    controllerId = _area.getRootController();

		// get the url to the event handler
		LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();
		String			    actionUrl = viewManager.getActionURL(LyStrutsConstants.PATH_ACTION_GRAPHICEVENT);

		// build the javascript
		StringBuffer	    script = new StringBuffer();

		script.append("InvokeAction('");
		script.append(_area.getId());
		script.append("', '");
		script.append(actionUrl);
		script.append("', null, '");
		script.append(controllerId);
		script.append("', '");
		script.append(!_area.isOpen());
		script.append("', null)");

		return script.toString();
	} // end method getOpenAreaScript

	/**
	 * Method overriden to redirect the root URL to the page action handler.
	 */
	public String getRootUrl()
	{
		LyWebViewManager viewManager = getWebViewManager();

		if (!(viewManager instanceof LyStrutsViewManager))
			return super.getRootUrl();

		// get the url to the page action
		String			    url = ((LyStrutsViewManager)viewManager).getActionAbsoluteUrl(
				LyStrutsConstants.PATH_ACTION_PAGE, null, true);

		return url;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.struts.LyStrutsArea#getBaseUrl(leon.view.web.LyWebPageWriter)
	 */
	protected String getBaseUrl(LyWebPageWriter pw)
	{
		// get the application context url.
		return ((LyWebViewManager)getViewManager()).getRootURL();
	}

	/**
	 * Overriden to return the URL to the Leoanrdi struts 'page' action.
	 *
	 * @return The base url for the internal frames and elements of the frame.
	 */
	protected String getFrameBaseUrl()
	{
		LyWebViewManager viewManager = getWebViewManager();

		// get the url to the page action
		String    url = ((LyStrutsViewManager)viewManager).getActionAbsoluteUrl(
				LyStrutsConstants.PATH_ACTION_PAGE, null, true);

		return url;
	}

	/**
	 * Method overriden to return the URL to the "Command" struts action path.
	 */
	public String getActionUrl()
	{
		// get the action type, default is hevent handler
		String		   actionPath = LyStrutsConstants.PATH_ACTION_GRAPHICEVENT;
		// check if we have a form, then the action is command handler
		Vector<LyForm> v = new Vector<LyForm>(2);

		_area.getComponents(LyForm.class, v);

		if (!v.isEmpty())
			actionPath = LyStrutsConstants.PATH_ACTION_COMMAND;

		// get the view manager
		LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();

		// build the URL for the Leonardi struts command action
		String			    url = viewManager.getActionAbsoluteUrl(actionPath, null, true);

		return url;
	}

	/**
	 * Returns the width of area if fixed.
	 * @return the width of area in pixels or a negative value if there is no fixed width.
	 */
	public int getAreaWidth()
	{
		return -1;
	}

	/**
	 * Returns the height of area if fixed.
	 * @return the height of area in pixels or a negative value if there is no fixed height.
	 */
	public int getAreaHeight()
	{
		return -1;
	}

	   /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebPage#init()
     */
    public void init()
    {
        super.init();

        _frames = new Vector<String>();
        _title = _area.getLabel();
        _redirect = _area.getRedirect();
        _subElementIds = new Hashtable<LyComponent, String>();
        _availableId = null;
        _isVisible = !_area.isHidden();
        _borderPanelsFormId = null;

        // Activate autorefresh.
        int refresh = _area.getRefresh();

        if (refresh > 0)
            getWebViewManager().addRefreshingView(_area);

        _width = 100;
        _height = 0;

        int     nElements = _area.getPanelCount();
        boolean formInside = false;

        for (int i = 0; i < nElements; i++)
        {
            LyPanel panel = _area.getPanel(i);

            if (panel instanceof LyForm)
            {
                formInside = true;
                break;
            }
        }

        if (nElements == 0)
            formInside = true;

        if (formInside)
            _height = 100;
        else
            _height = 100; // Frame height=100%
    } // end method init

    /**
     * Gets a unique identifier for a sub-element of the current area. This is used to avoid
     * confusion between frames that may have the same name inside the navigator.
     *
     * @param  component reference of a component inside the current area.
     * @return a unique identifier.
     */
    protected String getSubElementId(LyComponent component)
    {
        String id = null;

        if (component instanceof LyGetIdInterface)
            id = ((LyGetIdInterface)component).getId();

        if (id != null)
            return id;

        // if the component does not have an Id then
        // look for inside the internal table or create it
        if (_subElementIds.containsKey(component))
            return _subElementIds.get(component);
        else
        {
            if (_availableId != null)
            {
                id = _availableId;
                _availableId = null;
            }
            else
                id = String.valueOf(LyString.getRandomId());

            _subElementIds.put(component, id);
            return id;
        }
    } // end method getSubElementId

    /**
     * Gets the css class of the toolbars for this area.
     *
     * @return the name of the css class to be used for toolbars of this area.
     */
    public String getToolbarClass()
    {
        if (_area.getParentArea() == null)
            return "topToolbar";

        return "toolbar";
    }

    /**
     * Gets name of the main frame.
     *
     * @return the main frame name
     */
    public String getMainFrameName()
    {
        // If single-frame return MAIN_BUFFER
        if (!isFrameSet())
            return MAIN_BUFFER;

        // If there is at least one panel, return ELEMENT + Id of Panel(0)
        int nElements = _area.getPanelCount();

        if (nElements > 0)
            return (ELEMENT + getSubElementId(_area.getPanel(0)));

        _redirect = _area.getRedirect();

        // If there is a titlebar return TOOLTITLE
        // LyToolBar toolbar = _area.getToolBar();
        LyTitleBar titleBar = getTitleBar();

        if ((_area.getTopLeftToolBar() != null) || (_area.getTopRightToolBar() != null)
                || (titleBar != null))
            return (_area.getId() + TOOLTITLE);

        // If there is a command bar return COMMANDS
        LyCommands commands = _area.getCommands();

        if (commands != null)
            return COMMANDS + "_" + _area.getController();

        // If there is nothing in the area (!) return MAIN_BUFFER
        return MAIN_BUFFER;
    } // end method getMainFrameName

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebPage#writeBeginPage(java.io.Writer, java.lang.String)
     */
    public void writeBeginPage(Writer pw, String frame)
        throws IOException
    {
        if (frame == null)
            frame = MAIN_BUFFER;

        if (!isSubFrameSet(frame))
            super.writeBeginPage(pw, frame);
        else
        {
            pw.write("<!-- Generated by Lyria -->\n");
            pw.write("<html><head>\n");
            pw.write("<meta name=\"AUTHOR\" content=\"Lyria\"/>\n");
            pw.write("<meta name=\"GENERATOR\" content=\"Leonardi\"/>\n");
            pw.write("<title>" + _title + "</title>\n");
            pw.write("</head>\n");
        }
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebPage#writeEndPage(java.io.Writer, java.lang.String)
     */
    public void writeEndPage(Writer pw, String frame)
        throws IOException
    {
        if (frame == null)
            frame = MAIN_BUFFER;

        if (!isSubFrameSet(frame))
            super.writeEndPage(pw, frame);
        else
            pw.write("</html>\n");
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebPageWriter#getRedirect(java.lang.String)
     */
    public String getRedirect(String frame)
    {
        _redirect = _area.getRedirect();

        if (_redirect != null)
            return _redirect;

        // maybe other panel can be redirected
        int            i, nElements = _area.getPanelCount();

        LyComponent    element;
        LyWebComponent internal;

        for (i = 0; i < nElements; i++)
        {
            element = _area.getPanel(i);
            internal = (LyWebComponent)element.getInternalView();

            if ((internal instanceof LyWebPageWriter)
                    && (((LyWebPageWriter)internal).getRedirect(frame) != null))
                return ((LyWebPageWriter)internal).getRedirect(frame);
        }

        return null;
    }

    /**
     * Writes area frame set.
     *
     * @param pw           the page writer
     * @param writeToolBar true if the tollbar has to be writed
     */
    protected void writeFrameSet(LyWebPageWriter pw, boolean writeToolBar)
    {
        if (!hasMenuBar())
        {
            writeVirtualFrameSet(pw, writeToolBar);
            return;
        }

        LyMenuBar      menuBar = getMenuBar();
        LyWebComponent internalView = (LyWebComponent)menuBar.getInternalView();

        // CLEANCODE : this code does nothing !
        // if ((internalView != null) && !isPrintMode())
        // ;

        internalView.writeHtml(pw);

        String forceReload = "";

        if (getForceReloadState())
        {
            _lastReloadTime = new Date().getTime();
            setForceReloadState(false);
        }

        if (_lastReloadTime > 0)
            forceReload = "&#38;reload=" + _lastReloadTime;

        // Add base url
        String baseUrl = getBaseUrl(pw);

        pw.addBeginScript("<script type='text/javascript'>\n//<![CDATA[\nvar _rootUrl='"
            + LyWebObject.encodeUrl(baseUrl) + "';\n//]]>\n</script>", false);

        // 4% of total height for the menubar...
        int         i, nElements = _area.getPanelCount();
        int         height = 96 / nElements;
        LyComponent element;

        pw.append("<!-- Layout renderer : LyStrutsArea. 1 -->\n");

        for (i = 0; i < nElements; i++)
        {
            element = _area.getPanel(i);
            pw.append("<iframe src='" + getFrameBaseUrl());
            pw.append("?" + LyWebConstants.PAGE + "=" + getId() + "&#38;" + LyWebConstants.FRAME
                + "=" + ELEMENT + getSubElementId(element) + forceReload + "'");
            pw.append(" name='" + ELEMENT + getSubElementId(element) + "' ");
            pw.append("width='100%' height='" + height + "%' scrolling='auto'>&nbsp;</iframe>\n");
        }
    } // end method writeFrameSet

    /**
     * Gets elements frame size.
     *
     * @return a String containing all elements frame size separated by a colon.
     */
    protected String getElementsFrameSize()
    {
        int          i, nElements = _area.getPanelCount();
        LySelectBar  selectBar = _area.getSelectBar();
        LyCommands   commands = _area.getCommands();
        LyComponent  panel;
        Object       internal;
        int[]        d;
        StringBuffer rows = new StringBuffer();

        for (i = 0; i < nElements; i++)
        {
            panel = _area.getPanel(i);
            internal = panel.getInternalView(false);

            // Asks to the internal component the requested size
            if (internal instanceof LyGetBoundsInterface)
            {
                d = ((LyGetBoundsInterface)internal).getSize();
                rows.append(Integer.toString(d[1]));
            }
            else
                rows.append("*");

            if (i != (nElements - 1))
                rows.append(",");
        }

        if (selectBar != null)
            rows.append(",25");
        if (commands != null)
            rows.append(",35");

        return rows.toString();
    } // end method getElementsFrameSize

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebPage#setVisible(boolean)
     */
    public void setVisible(boolean status)
    {
        boolean refresh = (_isVisible != status);

        if (refresh)
            refresh();

        _isVisible = status;

        if (status && refresh)
        {
            LySetVisibleInterface visible = _area.getSetVisibleParent();

            if (visible != null)
                visible.setVisible(true);
        }
    }

    /**
     * Gets the title bar of the area. If the toolbar is vertical, no label is displayed in the
     * titlebar.
     *
     * @return the title bar if the area has ome, null otherwise
     */
    protected LyTitleBar getTitleBar()
    {
        LyTitleBar titlebar = _area.getTitleBar();

        return titlebar;
    }

    /**
     * Utility method to write a frame.
     *
     * @param pw          the page writer
     * @param rootURL     the root URL
     * @param frameName   the name of the frame
     * @param forceReload the value to print in URL to indicates that the relaod is forced
     * @param scrolling   indcates if the frame is scrolled. If false, set the HTML frame attribute
     *                    'scrolling' to auto or no, depending the skin.
     */
    protected void writeFrame(LyWebPageWriter pw, String rootURL, String frameName,
        String forceReload, boolean scrolling)
    {
        pw.append("<frame src='" + rootURL + "?");
        pw.append(LyWebConstants.PAGE + "=" + getId() + "&#38;");
        pw.append(LyWebConstants.FRAME + "=" + frameName + forceReload);

        if (getWindowName() != null)
            pw.append("&#38;" + LyWebConstants.TARGET_WINDOW + "=" + getWindowName());

        pw.append("' name='" + frameName + "'");

        if (!scrolling)
            pw.append(" scrolling='auto'"); // Workaround for frame width

        pw.append(" marginheight='0' marginwidth='0'/>\n");
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebPage#setNoFrames(boolean)
     */
    public void setNoFrames(boolean noFrames)
    {
        super.setNoFrames(noFrames);

        int            i, nElements = _area.getPanelCount();
        LyComponent    element;
        LyWebComponent internal;

        for (i = 0; i < nElements; i++)
        {
            element = _area.getPanel(i);
            internal = (LyWebComponent)element.getInternalView();

            if (internal instanceof LyWebPageWriter)
                ((LyWebPageWriter)internal).setNoFrames(noFrames);
        }
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebPageWriter#setPrintMode(boolean)
     */
    public void setPrintMode(boolean printMode)
    {
        super.setPrintMode(printMode);

        int            i, nElements = _area.getPanelCount();
        LyComponent    element;
        LyWebComponent internal;

        for (i = 0; i < nElements; i++)
        {
            element = _area.getPanel(i);
            internal = (LyWebComponent)element.getInternalView();

            if (internal instanceof LyWebPageWriter)
                ((LyWebPageWriter)internal).setPrintMode(printMode);
        }
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebPageWriter#getPageWriters()
     */
    public Vector<LyWebPageWriter> getPageWriters()
    {
        Vector<LyWebPageWriter> pageWriters = new Vector<LyWebPageWriter>();
        Vector<LyWebPageWriter> subPageWriters;
        int                     i, j, nSubElements, nElements = _area.getPanelCount();
        LyComponent             element;
        LyWebComponent          internal;

        for (i = 0; i < nElements; i++)
        {
            element = _area.getPanel(i);
            internal = (LyWebComponent)element.getInternalView();

            if (internal instanceof LyWebPageWriter)
            {
                subPageWriters = ((LyWebPageWriter)internal).getPageWriters();
                nSubElements = subPageWriters.size();

                for (j = 0; j < nSubElements; j++)
                    pageWriters.addElement(subPageWriters.elementAt(j));
            }
        }

        pageWriters.addElement(this);
        return pageWriters;
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebPageWriter#setForceReloadState(boolean)
     */
    public void setForceReloadState(boolean force)
    {
        _forceReloadState = force;

        // if force is true, we set the state of all the sub pageWriter
        if (force)
        {
            int            i, nElements = _area.getPanelCount();
            LyComponent    element;
            LyWebComponent internal;

            for (i = 0; i < nElements; i++)
            {
                element = _area.getPanel(i);
                internal = (LyWebComponent)element.getInternalView();

                if (internal instanceof LyWebPageWriter)
                    ((LyWebPageWriter)internal).setForceReloadState(true);
            }
        }
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebPage#getBuffer(java.lang.String)
     */
    public StringBuffer getBuffer(String key)
    {
        StringBuffer result = super.getBuffer(key);

        if ((result == null) && (_area != null))
        {
            int            i, nElements = _area.getPanelCount();
            LyComponent    element;
            LyWebComponent internal;

            for (i = 0; i < nElements; i++)
            {
                element = _area.getPanel(i);
                internal = (LyWebComponent)element.getInternalView();

                if (internal instanceof LyWebPageWriter)
                {
                    result = ((LyWebPageWriter)internal).getBuffer(key);
                    if (result != null)
                        break;
                }
            }
        }

        return result;
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.graphic.LyGraphicalArea#setRefresh(int)
     */
    public void setRefresh(int duration)
    {
        if (duration > 0)
            getWebViewManager().addRefreshingView(_area);
        else
            getWebViewManager().clearRefreshingView(_area);
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.setinterface.LyFreeInterface#free()
     */
    public void free()
    {
        // Activate autorefresh.
        int refresh = ((_area == null) ? 0 : _area.getRefresh());

        if (refresh > 0)
            getWebViewManager().clearRefreshingView(_area);

        super.free();
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebObject#refresh()
     */
    public void refresh()
    {
        LyWebViewManager viewManager = getWebViewManager();

        // Get current frame name and set refresh
        String           frameName = getFrameName();

        if (frameName == null)
            frameName = _lastFrameName;

        if (frameName == null)
            return;

        // The name can be something like xxx.yy.zzz
        // The frame name is zzz, and not all the string
        int i = frameName.lastIndexOf('.');

        frameName = frameName.substring(i + 1);
        viewManager.addReloadFrame(frameName);
        setForceReloadState(true);
    }

    /**
     * Parses the tools contained in this toolBar and search if at least one has a label.
     *
     * @param  toolBar the toolbar to parse
     * @return true if there is at least one label, false otherwise
     */
    public boolean containsLabel(LyToolBar toolBar)
    {
        int         i, nTools = toolBar.getComponentCount();
        LyComponent component;
        String      label;

        // Parse the tools contained in this toolBar and search if at least one
        // is a toolBar and has a label
        for (i = 0; i < nTools; i++)
        {
            component = toolBar.getComponent(i);
            if (component instanceof LyToolBar)
            {
                label = ((LyToolBar)component).getLabel();
                if ((label != null) && (label.length() != 0))
                    return true;
            }
        }

        return false;
    }

    /**
     * Prepare the html page section for the specified element.
     *
     * @param elementId    current element idenfitier.
     * @param targetWindow target window indicates that the result has to be shown into a popup
     *                     frame. This parameter is the name of the popup frame, or null if in
     *                     Leonardi main window
     */
    protected void prepareElementPage(String elementId, String targetWindow)
    {
        LyComponent    element;
        LyWebComponent webElement;

        // Write the buffers for elements
        element = _area.getComponent(elementId);
        // if element is not found look into internal hashtable
        if ((element == null) && (_subElementIds.containsValue(elementId)))
        {
            LyComponent key;
            String      value;

            for (Enumeration<LyComponent> e = _subElementIds.keys(); (e.hasMoreElements() && (element == null));)
            {
                key = e.nextElement();
                value = _subElementIds.get(key);
                if (elementId.equals(value))
                    element = key;
            }
        }

        if (element == null)
            return;

        clearBuffer(ELEMENT + elementId);
        setActiveBuffer(ELEMENT + elementId);

        webElement = (LyWebComponent)element.getInternalView();

        if ((webElement instanceof LyWebPageWriter)
                && (((LyWebPageWriter)webElement).isFrameSet() || _noFrames))
        {
            _frames.addElement(ELEMENT + elementId);
            ((LyWebPageWriter)webElement).preparePage(null, targetWindow);
        }

        webElement.writeHtml(this);

    } // end method prepareElementPage

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebPageWriter#clearBeginScripts()
     */
    public void clearBeginScripts()
    {
        super.clearBeginScripts();

        _beginScripts.put(_area.getId() + TOOLTITLE, new Vector<_LyScript>());
        _beginScripts.put(_area.getId() + SELECTBAR, new Vector<_LyScript>());
        _beginScripts.put(COMMANDS + "_" + _area.getController(), new Vector<_LyScript>());

        int i, nElements = _area.getPanelCount();

        for (i = 0; i < nElements; i++)
            _beginScripts.put(ELEMENT + getSubElementId(_area.getPanel(i)),
                new Vector<_LyScript>());
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebPageWriter#clearEndScripts()
     */
    public void clearEndScripts()
    {
        super.clearEndScripts();

        _endScripts.put(_area.getId() + TOOLTITLE, new Vector<_LyScript>());
        _endScripts.put(_area.getId() + SELECTBAR, new Vector<_LyScript>());
        _endScripts.put(COMMANDS + "_" + _area.getController(), new Vector<_LyScript>());

        int i, nElements = _area.getPanelCount();

        for (i = 0; i < nElements; i++)
            _endScripts.put(ELEMENT + getSubElementId(_area.getPanel(i)), new Vector<_LyScript>());
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebPageWriter#clearCssScripts()
     */
    public void clearCssScripts()
    {
        super.clearEndScripts();

        _cssScripts.put(_area.getId() + TOOLTITLE, new Vector<_LyCssScript>());
        _cssScripts.put(_area.getId() + SELECTBAR, new Vector<_LyCssScript>());
        _cssScripts.put(COMMANDS + "_" + _area.getController(), new Vector<_LyCssScript>());

        int i, nElements = _area.getPanelCount();

        for (i = 0; i < nElements; i++)
            _cssScripts.put(ELEMENT + getSubElementId(_area.getPanel(i)),
                new Vector<_LyCssScript>());
    }

    /**
     * Computes the number of needed cells to place all area elements in a table.
     *
     * @return an array containing the number of needed rows and cols.
     */
    protected int[] computeNeededCells()
    {
        // Set the maximum number of cells to 100 : may be unsufficient.
        // The algorithm create an array of integers (0 is initial value)
        // representing the number of cells for each rows.
        // For each field, its colspan is added to the array values between pos
        // (current element row) and (pos + rowspan).
        int          maxLength = 100;
        int[]        cols = new int[maxLength];
        int          i, pos = 0;
        LyConstraint constraint;

        // Treatment of the titlebar
        LyTitleBar   titleBar = getTitleBar();

        if (titleBar != null)
        {
            constraint = titleBar.getConstraint();
            if (computeNeededCells(constraint, cols, pos, false))
                pos++;
        }

        // Treatment of the toolbars

        int       nElements = _area.getToolBarCount();
        LyToolBar toolBar;

        for (i = 0; i < nElements; i++)
        {
            toolBar = _area.getToolBar(i);
            if (toolBar == null)
                continue;
            constraint = toolBar.getConstraint();
            if (computeNeededCells(constraint, cols, pos, false))
                pos++;
        }

        // Treatment of the elements
        nElements = _area.getPanelCount();
        LyComponent element;

        for (i = 0; i < nElements; i++)
        {
            element = _area.getPanel(i);
            if (element instanceof LyGetConstraintInterface)
                constraint = ((LyGetConstraintInterface)element).getConstraint();
            else
                constraint = null;

            if (computeNeededCells(constraint, cols, pos, true))
                pos++;
        }

        // Treatment of the selectBar
        LySelectBar selectBar = _area.getSelectBar();

        if (selectBar != null)
        {
            constraint = selectBar.getConstraint();
            if (computeNeededCells(constraint, cols, pos, false))
                pos++;
        }

        // Treatment of the commands
        LyCommands commands = _area.getCommands();

        if (commands != null)
        {
            constraint = commands.getConstraint();
            if (computeNeededCells(constraint, cols, pos, false))
                pos++;
        }

        int col = 0;

        for (i = 0; (i < (pos + 1)) && (cols[i] != 0); i++)
        {
            if (cols[i] > col)
                col = cols[i];
        }

        return new int[] { i, col };
    } // end method computeNeededCells

    /**
     * Computes the number of needed cells to place all area elements in a table.
     *
     * @param  constraint     the area constraints
     * @param  cols           array of integers representing the number of cells for each rows.
     * @param  pos            colspan is added to the array values between pos (current element row)
     *                        and (pos + rowspan).
     * @param  jumpIfNegative true if a row position has to be incremented when the constraint
     *                        colspan is negative
     * @return true if a row position has to be incremented, false otherwise
     */
    boolean computeNeededCells(LyConstraint constraint, int[] cols, int pos, boolean jumpIfNegative)
    {
        int     rowspan, colspan;
        boolean jump = false;

        if (constraint != null)
        {
            int maxLength = 100;

            rowspan = constraint.getRowSpan();
            colspan = constraint.getColSpan();

            // If colspan is <= 0 replace the value with 1 and jump to the next
            // row if equals 0
            if (colspan <= 0)
                colspan = 1;
            if ((colspan == 0) || (jumpIfNegative && (colspan < 0)))
                jump = true;

            if (rowspan == 0)
                rowspan = maxLength;
            else if (rowspan < 0)
                rowspan = 1;
        }
        else
        {
            rowspan = 1;
            colspan = 1;
            jump = true;
        }

        int i;

        for (i = pos; i < (pos + rowspan); i++)
            cols[i] += colspan;

        return jump;
    } // end method computeNeededCells

    /**
     * Computes the global size of list of toolbars.
     *
     * @param  toolBar the toolbar to compute
     * @return [0,0] if the toolbar is null, toolbar size otherwise.
     */
    protected int[] computeToolBarsSize(LyToolBar toolBar)
    {
        LyStrutsToolBar internal;
        int[]          size;
        int[]          result = new int[2];

        result[0] = 0;
        result[1] = 0;

        // Add toolBar separation (5 pixels)
        if (toolBar == null)
            return result;

        internal = (LyStrutsToolBar)(toolBar.getInternalView());
        size = internal.computeSize();

        return size;
    }

    /**
     * Indicates if the specified frame is also a sub frame set. Root frame set may contain
     * sub-frameset.
     *
     * @param  frame the frame
     * @return true if the frame is also a sub frame set, false otherwise
     */
    public boolean isSubFrameSet(String frame)
    {
        if (frame == null)
            return false;

        return _frames.contains(frame);
    }

    /**
     * Write virtual frame set.
     *
     * @param pw           the page writer
     * @param writeToolBar if true write the toolbar (if it exists), false otherwise
     */
    protected void writeVirtualFrameSet(LyWebPageWriter pw, boolean writeToolBar)
    {
        LySelectBar selectBar = _area.getSelectBar();
        LyCommands  commands = _area.getCommands();
        int         i, nElements = _area.getPanelCount();
        String      forceReload = "";

        if (getForceReloadState())
        {
            _lastReloadTime = new Date().getTime();
            setForceReloadState(false);
        }

        if (_lastReloadTime > 0)
            forceReload = "&#38;reload=" + _lastReloadTime;

        pw.append("<!-- Layout renderer : LyStrutsVirtualFrameset 0 -->\n");
        pw.append(
            "<body leftmargin='0' topmargin='0' marginwidth='0' marginheight='0' scroll='no'>\n");
        pw.append(
            "<table height='100%' width='100%' border='0' cellpadding='0' cellspacing='0'>\n");

        if (writeToolBar && ((_area.getRightToolBar() != null) || (_area.getLeftToolBar() != null)))
        {
            LyToolBar leftToolBar = _area.getLeftToolBar();
            LyToolBar rightToolBar = _area.getRightToolBar();

            pw.append("<!-- Layout renderer : LyStrutsVirtualFrameset 1 -->\n");
            pw.append("<tr>\n");

            // compute teh sizes for each iframe in the table
            if (leftToolBar != null)
            {
                int[] sizeLeft = computeToolBarsSize(leftToolBar);

                pw.append("<!-- Layout leftToolBar -->\n\n");
                pw.append("<td width='" + sizeLeft[0] + "'>\n");
                writeFrame(getId(), null, pw, getFrameBaseUrl(), LyStrutsArea.LEFT, forceReload,
                    false, false, false, "", "");
                pw.append("</td>\n");
            }

            pw.append("<!-- Layout centerToolBar -->\n\n");
            pw.append("<td>\n");
            writeFrame(getId(), null, pw, getFrameBaseUrl(), LyStrutsArea.CENTER, forceReload, false,
                false, false, "", "");
            pw.append("</td>\n");

            if (rightToolBar != null)
            {
                int[] sizeRight = computeToolBarsSize(rightToolBar);

                pw.append("<!-- Layout rightToolBar -->\n\n");
                pw.append("<td width='" + sizeRight[0] + "'>\n");
                writeFrame(getId(), null, pw, getFrameBaseUrl(), LyStrutsArea.RIGHT, forceReload,
                    false, false, false, "", "");
                pw.append("</td>\n");
            }
            pw.append("</tr>\n");
        }
        else
        {
            pw.append("<!-- Layout renderer : LyStrutsVirtualFrameset 2 -->\n");

            String      rows = "";
            String[]    sizesTab;
            LyComponent panel;
            Object      internal;
            int[]       d;
            String      rootURL = getFrameBaseUrl();
            LyComponent element;

            for (i = 0; i < nElements; i++)
            {

                panel = _area.getPanel(i);
                internal = panel.getInternalView(false);

                // Asks to the internal component the requested size
                if (internal instanceof LyGetBoundsInterface)
                {
                    d = ((LyGetBoundsInterface)internal).getSize();
                    rows = rows + Integer.toString(d[1]);
                }
                else
                    rows = rows + "*";

                if (i != (nElements - 1))
                    rows = rows + ",";
            }

            sizesTab = LyString.stringToList(rows, ',');

            for (i = 0; i < nElements; i++)
            {
                if (sizesTab[i].equals("*"))
                    pw.append("<tr><td>\n");
                else
                    pw.append("<tr height='" + sizesTab[i] + "%'><td>\n");
                element = _area.getPanel(i);
                writeFrame(getId(), null, pw, rootURL,
                    LyStrutsArea.ELEMENT
                    + getSubElementId(element), forceReload, false, false, false, "", "");
                pw.append("</td></tr>\n");
            }

            if (selectBar != null)
            {
                pw.append("<tr height='25'><td>\n");
                writeFrame(getId(), null, pw, rootURL, _area.getId() + SELECTBAR, forceReload,
                    false, false, false, "", "");
                pw.append("</td></tr>\n");
            }

            if (commands != null)
            {
                int height;

                if (testResource("LY_USE_IMAGE_FOR_COMMAND"))
                    height = 50;
                else
                    height = 35;
                pw.append("<tr height='" + height + "'><td>\n");
                writeFrame(getId(), null, pw, rootURL,
                    LyStrutsArea.COMMANDS + "_"
                    + _area.getController(), forceReload, false, false, false, "", "");
                pw.append("</td></tr>\n");
            }
        }
        pw.append("</table></body>");
    } // end method writeVirtualFrameSet

    /**
     * Writes area's titlebar.
     *
     * @param pw    the page writer
     * @param nRows the number of rows
     * @param nCols the number of columns
     */
    protected void writeTitleBar(LyWebPageWriter pw, int nRows, int nCols)
    {
        LyTitleBar titleBar = getTitleBar();
        LyMenuBar  menuBar = getMenuBar();

        if ((titleBar == null) && (menuBar == null))
            return;

        titleBar = getTitleBar();
        if (titleBar == null)
            return;

        LyStrutsTitleBar internal = (LyStrutsTitleBar)titleBar.getInternalView();

        pw.append("<tr valign='top' height='10'>");
        pw.append("<td valign='top' colspan='" + nCols + "' rowspan='1'>");
        internal.writeHtml(pw);
        pw.append("</td>");
        pw.append("</tr>");
    } // end method writeTitleBar

    /**
     * Writes toolbar.
     *
     * @param pw      the page writer
     * @param toolBar the toolbar to write
     */
    protected void writeToolBar(LyWebPageWriter pw, LyToolBar toolBar)
    {
        if (toolBar == null)
            return;
        LyStrutsToolBar internal = (LyStrutsToolBar)toolBar.getInternalView();

        internal.writeHtml(pw);
    }

    /**
     * Writes area's tip.
     *
     * @param pw the page writer
     */
    protected void writeTip(LyWebPageWriter pw)
    {
    	writeTip(pw, "");
    }

    /**
     * Writes area's tip.
     *
     * @param pw the page writer
     * @param colspanString string value representing colspan for tip
     */
    protected void writeTip(LyWebPageWriter pw, String colspanString)
    {
        String tip = _area.getTip();

        if (tip == null)
            return;

        String          htmlString = getHtmlString(tip);
        LyWebDecoration deco = getWebViewManager().getDecoration();

        pw.append("\n");
        pw.append("<tr height='10'><td " + colspanString + "align='left' class='"
            + deco.getClassName(_area, DECO_TIP_PARAMETER, pw) + "'>");
        pw.append(htmlString);
        pw.append("</td></tr >");
    }

    /**
     * Writes one of the border panels.
     *
     * @param pw the page writer.
     * @param pw the panel that must be written.
     * @param pw the position of the panel (may be one of the values <code>LyArea.NORTH</code>,
     *           <code>LyArea.SOUTH</code>, <code>LyArea.EAST</code>, <code>LyArea.WEST</code>).
     */
    protected void writeBorderPanel(LyWebPageWriter pw, LyPanel panel, short position)
    {
        LyWebViewManager viewManager = getWebViewManager();
        boolean          isFirefox = (viewManager.getBrowser() == LyWebViewManager.BROWSER_FIREFOX);
        int              height = ((panel == null) ? 0 : BORDER_PANEL_MIN_HEIGHT);
        String           id = getIdCode();
        boolean          generateTr = ((position == LyArea.NORTH) || (position == LyArea.SOUTH));
        boolean          isDialog = (_area.getParentView() instanceof LyDialog);
        String           heightString = ((isFirefox && generateTr && isDialog)
                ? " height='0%'" : (" height='" + height + "'"));

        // If print mode and panel is null, do not add a new line.
        if (generateTr && (panel == null) && pw.isPrintMode())
            return;

        if (position == LyArea.NORTH)
            id += "_north_panel_cell";
        else if (position == LyArea.SOUTH)
            id += "_south_panel_cell";
        else if (position == LyArea.WEST)
            id += "_west_panel_cell";
        else if (position == LyArea.EAST)
            id += "_east_panel_cell";

        updateRefreshInfo(pw);

        // If no toolbar, do not generate HTML tags used to place the border panels.
        // This is done to avoid too many empty <tr><td></td></tr> in the HTML code.
        if ((panel == null) && (_area.getToolBarCount() == 0))
            return;

        if (generateTr)
            pw.append("<tr" + heightString + ">");

        pw.append("<td" + heightString + " id='" + id + "'>");

        if (panel != null)
        {
            LyStrutsArea internal = (LyStrutsArea)panel.getInternalView();

            internal.writeHtml(pw);
            _borderPanelsFormId = pw.getFormId(pw.getActiveName());
        }
        else
            _borderPanelsFormId = pw.getFormId(pw.getActiveName());

        pw.append("</td>");

        if (generateTr)
            pw.append("</tr>");
    } // end method writeBorderPanel

    /**
     * Writes area's elements.
     *
     * @param pw    the page writer
     * @param nRows the number of rows
     * @param nCols the number of columns
     */
    protected void writeElements(LyWebPageWriter pw, int nRows, int nCols)
    {
        int nElements = _area.getPanelCount();

        if (nElements <= 0)
        {
            pw.append("<td width='100%' height='100%'>");
            pw.append("&nbsp;</td>");
            return;
        }

        int    height = 100 / nElements;
        String forceReload = "";

        if (getForceReloadState())
        {
            _lastReloadTime = System.currentTimeMillis();
            setForceReloadState(false);
        }

        if (_lastReloadTime > 0)
            forceReload = "&#38;reload=" + _lastReloadTime;

        boolean        jump, hasJump = false;
        LyComponent    element;
        LyWebComponent internal;
        LyConstraint   constraint;
        int            i, colspan, rowspan, currentCol = 0;

        _area.computePanelsPositions();

        int[] gridSize = _area.getGridSize();

        // First check if structure has at least one jump (one HTML <tr> tag).
        for (i = 0; i < nElements; i++)
        {
            element = _area.getPanel(i);
            jump = false;

            if (element instanceof LyGetConstraintInterface)
                constraint = ((LyGetConstraintInterface)element).getConstraint();
            else
                constraint = null;

            if (constraint != null)
            {
                rowspan = constraint.getRowSpan();
                colspan = constraint.getColSpan();

                if (colspan <= 0)
                {
                    colspan = nCols - currentCol;
                    jump = true;
                }
            }
            else
                jump = true;

            if (jump && (i != (nElements - 1)))
                hasJump = true;
        }

        pw.append("<tr");

        if (!hasJump)
            pw.append(" height='100%'");

        pw.append(">\n");

        for (i = 0; i < nElements; i++)
        {
            element = _area.getPanel(i);
            internal = (LyWebComponent)element.getInternalView();
            jump = false;

            if (element instanceof LyGetConstraintInterface)
                constraint = ((LyGetConstraintInterface)element).getConstraint();
            else
                constraint = null;

            if (constraint != null)
            {
                rowspan = constraint.getRowSpan();
                colspan = constraint.getColSpan();

                if (colspan <= 0)
                {
                    colspan = nCols - currentCol;
                    jump = true;
                }

                if (constraint.isRemainder())
                    jump = true;

                if (rowspan == 0)
                    rowspan = nRows;
                else if (rowspan < 0)
                    rowspan = 1;
            }
            else
            {
                rowspan = 1;
                colspan = nCols - currentCol;
                jump = true;
            }

            currentCol += colspan;

            boolean     isFormNoScroll = (element instanceof LyForm)
                && ((nElements > 1)
                    || (_area.getScrolling() == LyArea.Scrolling.NO)
                    || LyWebConstants.POPUP_VIEW.equals(pw.getWindowName())
                    || ((element.getParentView() instanceof LyDialog)
                            && (element.getParent() != null)
                            && (element.getParent().getParent() instanceof LyTab)
                            && !LyWebConstants.POPUP_VIEW.equals(pw.getWindowName()))
                    || LyWebConstants.TOOL_POPUP.equals(pw.getActiveName()));

            // Check if area is placed in a tabs that contains an tab with empty label : this occurs
            // when some fields have a tab and some others don't. In this case do not add scroll
            // (a global scroll will be performed).
            LyTabs  tabs = _area.getParentTabs();

            if ((tabs != null) && (tabs.getTabCount() > 0))
            {
                LyTab   tab = tabs.getTab(0);

                if ((tab != null)
                        && ((tab.getLabel() == null) || (tab.getLabel().length() == 0)))
                    isFormNoScroll = true;
            }

            // If there is a grid structure with at least two columns and two rows, generate a grid table
            if ((gridSize[0] > 1)
                    && (gridSize[1] > 1)
                    && (element instanceof LyArea)
                    && (((LyArea)element).getBorder() == LyArea.Border.SOLID))
            {
                int elementWidth = (colspan * 100 / gridSize[0]) + 1;
                int elementHeight = (rowspan * 100 / gridSize[1]) + 1;

                pw.append("<td ");

                if (_area.getParentArea() == null)
                    pw.append("class='" + getClassName(pw) + "' ");

                pw.append("valign='top' colspan='" + colspan + "' rowspan='" + rowspan + "' ");
                pw.append("width='" + elementWidth + "%' height='" + elementHeight + "%'>\n");
                pw.append("<!-- Layout renderer : LyStrutsArea. 2 -->\n");
                pw.append("<iframe src='" + getFrameBaseUrl());
                pw.append("?" + LyWebConstants.PAGE + "=" + getId() + "&#38;"
                    + LyWebConstants.FRAME);
                pw.append("=" + ELEMENT + getSubElementId(element) + forceReload + "'");
                pw.append(" name='" + ELEMENT + getSubElementId(element) + "' id='" + ELEMENT
                    + getSubElementId(element) + "'");
                pw.append(
                    " frameborder='0' align='left' scrolling='auto' width='100%' height='100%'");
                pw.append(" ly_height_scroll='1' ly_width_scroll='1'");
                pw.append("></iframe>");
            }
            else if (!isFormNoScroll && !(element instanceof LyArea)
                    && !pw.isPrintMode())
            {
                pw.append("<td ");

                if (_area.getParentArea() == null)
                    pw.append("class='" + getClassName(pw) + "' ");

                pw.append("valign='top' colspan='" + colspan);

                if ((element.getParent().getParent() != null)
                        && (element.getParent().getParent() instanceof LyFormField))
                    pw.append("' rowspan='" + rowspan + "' height='100'>\n");
                else
                    pw.append("' rowspan='" + rowspan + "' height='100%'>\n");

                String scrolling = "no";

                if ((_area.getScrolling() == LyArea.Scrolling.AUTO)
                        || (_area.getScrolling() == LyArea.Scrolling.YES)
                        || (pw.getWebViewManager().getBrowser() == LyWebViewManager.BROWSER_FIREFOX))
                    scrolling = "auto";

                pw.append("<!-- Layout renderer : LyStrutsArea. 2 -->\n");
                pw.append("<iframe src='" + getFrameBaseUrl());
                pw.append("?" + LyWebConstants.PAGE + "=" + getId() + "&#38;"
                    + LyWebConstants.FRAME);
                pw.append("=" + ELEMENT + getSubElementId(element) + forceReload + "'");
                pw.append(" name='" + ELEMENT + getSubElementId(element) + "' id='" + ELEMENT
                    + getSubElementId(element) + "'");
                pw.append(" frameborder='0' width='100%' height='" + height);
                pw.append("%' align='left' scrolling='" + scrolling + "'");

                // Special case for an area placed in a tab and containing a
                // splitter
                if (!pw.isInnerView()
                        && (!(element instanceof LySplitter)
                            || !(_area.getParent() instanceof LyTab)))
                    pw.append(" ly_height_scroll='1' ly_width_scroll='1'");
                pw.append("></iframe>");
            }
            else
            {
                pw.append("<td ");

                if (_area.getParentArea() == null)
                    pw.append("class='" + getClassName(pw) + "' ");

                pw.append("valign='top' colspan='" + colspan + "' rowspan='" + rowspan
                    + "'>\n");
                if (internal != null)
                    internal.writeHtml(pw);
            }

            pw.append("</td>\n");

            if (jump && (i != (nElements - 1)))
                pw.append("</tr><tr>\n");
        }

        pw.append("</tr>\n");

        // Generate an empty area to occupy all space left
        if (hasJump && (nElements > 1))
            pw.append("<tr height='100%'><td></td></tr>\n");
    } // end method writeElements

    /**
     * Returns the name of the css class to be used for current area.
     *
     * @return The name of the css class to be used for current area or an empty string if no css
     *         class has to be used.
     */
    protected String getClassName(LyWebPageWriter pw)
    {
        LyWebDecoration deco = getWebViewManager().getDecoration();

        return deco.getClassName(_area, DECO_AREA_PARAMETER, pw);
    }

    /**
     * Writes area's selectbar.
     *
     * @param pw    the page writer
     * @param nRows the number of rows
     * @param nCols the number of columns
     */
    protected void writeSelectBar(LyWebPageWriter pw, int nRows, int nCols)
    {
        LySelectBar selectBar = _area.getSelectBar();

        if (selectBar == null)
            return;

        LyStrutsSelectBar internal = (LyStrutsSelectBar)selectBar.getInternalView();
        LyConstraint     constraint = selectBar.getConstraint();

        writeComponent(pw, internal, constraint, nRows, nCols);
    }

    /**
     * Writes commands.
     *
     * @param pw    the page writer
     * @param nRows the number of rows
     * @param nCols the number of columns
     */
    protected void writeCommands(LyWebPageWriter pw, int nRows, int nCols)
    {
        LyCommands commands = _area.getCommands();

        if (commands == null)
            return;

        LyStrutsCommands internal = (LyStrutsCommands)commands.getInternalView();
        LyConstraint    constraint = commands.getConstraint();

        writeComponent(pw, internal, constraint, nRows, nCols);
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebPageWriter#preparePage(java.lang.String, java.lang.String)
     */
    public void preparePage(String frame, String targetWindow)
    {
        // Write the buffers for elements
        LyComponent    element;
        LyWebComponent webElement;
        int            i, nElements = _area.getPanelCount();

        // FFT00470
        boolean        mustRecurse = true;

        // ** Association element <-> frame name
        if ((frame == null) || (frame.equals(LyWebConstants.TARGET_FRAME)))
        {
            for (i = 0; i < nElements; i++)
            {
                element = _area.getPanel(i);
                webElement = (LyWebComponent)element.getInternalView();

                if (webElement instanceof LyWebPageWriter)
                    ((LyWebPageWriter)webElement).setFrameName(_name + "." + ELEMENT
                        + getSubElementId(element));
            }

            LyView view = _area.getParentView();

            if (view != null)
            {
                LyArea topArea = view.getArea();

                // Compute noFrame mode for top area only
                if (_area == topArea)
                    computeNoFrame();
            }
        }

        super.preparePage(frame, targetWindow);

        // Write the buffer for toolbar and titlebar
        if (((_area.getId() + TOOLTITLE).equals(frame)) || (_noFrames))
        {
            // FFT00470
            mustRecurse = false;
            clearBuffer(_area.getId() + TOOLTITLE);
            setActiveBuffer(_area.getId() + TOOLTITLE);

            LyTitleBar      titleBar = getTitleBar();
            String          tip = _area.getTip();

            LyWebDecoration deco = getWebViewManager().getDecoration();

            if (getWebViewManager().writeDivs())
                this.append("<div id=\"noBackground\">");
            else
                this.append(
                    "<table border='0' cellpadding='0' cellspacing='0' width='100%' height='100%'><tr><td id=\"background\" valign='top'>");

            deco.writePageDecoration(this, LyWebDecoration.WINDOW_BEGIN);

            if (titleBar != null)
            {
                deco.writePageDecoration(this, LyWebDecoration.WINDOW_TOP);

                deco.writePageDecoration(this, LyWebDecoration.WINDOW_HEADER_LEFT);

                LyStrutsTitleBar webTitlebar = (LyStrutsTitleBar)titleBar.getInternalView();

                webTitlebar.writeHtml(this);
                deco.writePageDecoration(this, LyWebDecoration.WINDOW_HEADER_RIGHT);
            }

            deco.writePageDecoration(this, LyWebDecoration.WINDOW_HEADER_LEFT);
            // write toolBars
            LyToolBar topLeftToolBar = _area.getTopLeftToolBar();
            LyToolBar topRightToolBar = _area.getTopRightToolBar();

            // if one of the two toolbar has a vertical separator style
            // show it
            int       border = 0;
            String    toolbarFrame = "";

            if (((topRightToolBar != null)
                        && (topRightToolBar.getStyle() == LyToolBar.Style.SEPARATOR))
                    || ((topLeftToolBar != null)
                        && (topLeftToolBar.getStyle() == LyToolBar.Style.SEPARATOR)))
            {
                // border = 1;
                border = 0;
                toolbarFrame = " frame='hsides' rules='none'";
            }

            if (getEnvironment().getEnv("LY_VIEWER_TYPE").equals("HTML"))
                border = 1;

            if (border == 1)
            {
                this.append("<table width='100%' cellpadding='0' cellspacing='0'");
                this.append(" border='" + border + "'");
                this.append(toolbarFrame);
                this.append("><tr><td>");
            }
            this.append("<table width='100%' cellpadding='0' cellspacing='0'");
            this.append(" border='0'");
            this.append(toolbarFrame);
            this.append("><tr valign='middle'>");
            this.append("<td align='left'>");
            this.append("<table width='100%' cellpadding='0' cellspacing='0'");
            this.append("><tr>\n");

            if ((topLeftToolBar != null) && (topLeftToolBar.getComponentCount() > 0))
            {
                this.append("<td class='" + getToolbarClass() + "'>\n");
                writeToolBar(this, topLeftToolBar);
                this.append("</td>\n");
            }

            this.append("</tr></table>\n");
            this.append("</td>\n");

            if ((topRightToolBar != null) && (topRightToolBar.getComponentCount() > 0))
            {
                this.append("<td align='right' class='" + getToolbarClass() + "'>");
                writeToolBar(this, topRightToolBar);
                this.append("</td>");
            }

            this.append("</tr></table>");

            if (border == 1)
                this.append("</td></tr></table>");
            deco.writePageDecoration(this, LyWebDecoration.WINDOW_HEADER_RIGHT);

            if (tip != null)
            {
                String htmlString = getHtmlString(tip);

                this.append("<table width='100%' border='0' cellpadding='3' cellspacing='0'>");
                this.append("<tr valign='middle'>");

                this.append("<td align='left' class='"
                    + deco.getClassName(_area, DECO_TIP_PARAMETER, this) + "'>");
                this.append(htmlString);
                this.append("</td>");
                this.append("</tr>");
                this.append("</table>");
            }
            deco.writePageDecoration(this, LyWebDecoration.WINDOW_END);

            if (getWebViewManager().writeDivs())
                this.append("</div>");
            else
                this.append("</td></tr></table>");
        }

        if ((_area.getId() + BOTTOMTOOLBAR).equals(frame) || _noFrames)
        {
            mustRecurse = false;
            clearBuffer(_area.getId() + BOTTOMTOOLBAR);
            setActiveBuffer(_area.getId() + BOTTOMTOOLBAR);
            // Write bottom toolBar
            this.append("<!-- Starts Writing bottom toolbar -->");

            this.append(
                "<table  width='100%' border='0' bordercolor='#FF0000' cellpadding='0' cellspacing='0'>");
            this.append("<tr valign='middle'>");

            this.append("<td align='left'>");
            if (_area.getBottomLeftToolBar() != null)
                writeToolBar(this, _area.getBottomLeftToolBar());
            else
                this.append("&nbsp;");
            this.append("</td>");
            this.append("<td align='right'>");
            if (_area.getBottomRightToolBar() != null)
                writeToolBar(this, _area.getBottomRightToolBar());
            else
                this.append("&nbsp;");
            this.append("</tr>");
            this.append("</table>");
            this.append("<!-- End Writing bottom toolbar -->");
        }

        // Write the buffer for titlebar and elements
        // (happens only if vertical toolbar)
        if ((_area.getId() + CENTER).equals(frame) || _noFrames)
        {
            // FFT00470
            mustRecurse = false;
            clearBuffer(_area.getId() + CENTER);
            setActiveBuffer(_area.getId() + CENTER);

            writeFrameSet(this, true);

            _frames.addElement(_area.getId() + CENTER);
        }

        // Write the buffer for titlebar and elements
        // (happens only if vertical toolbar)
        if (CENTER.equals(frame) || _noFrames)
        {
            // FFT00470
            mustRecurse = false;
            clearBuffer(CENTER);
            setActiveBuffer(CENTER);

            writeFrameSet(this, false);

            _frames.addElement(CENTER);
        }

        // Write the buffer for toolbar (happens only if vertical toolbar)
        if (RIGHT.equals(frame) || _noFrames)
        {
            // FFT00470
            mustRecurse = false;
            clearBuffer(RIGHT);
            setActiveBuffer(RIGHT);
            writeToolBar(this, _area.getRightToolBar());

        }

        // Write the buffer for toolbar (happens only if vertical toolbar)
        if (LEFT.equals(frame) || _noFrames)
        {
            // FFT00470
            mustRecurse = false;
            clearBuffer(LEFT);
            setActiveBuffer(LEFT);
            writeToolBar(this, _area.getLeftToolBar());

        }

        if ((_area.getId() + SELECTBAR).equals(frame) || (_noFrames))
        {
            // FFT00470
            mustRecurse = false;
            // Write the buffer for selectBar
            LySelectBar selectBar = _area.getSelectBar();

            if (selectBar != null)
            {
                clearBuffer(_area.getId() + SELECTBAR);
                setActiveBuffer(_area.getId() + SELECTBAR);

                LyStrutsSelectBar webSelectBar = (LyStrutsSelectBar)selectBar.getInternalView();

                webSelectBar.writeHtml(this);
            }
        }

        if (((COMMANDS + "_" + _area.getController()).equals(frame)) || (_noFrames))
        {
            // FFT00470
            mustRecurse = false;
            // Write the buffer for commands
            LyCommands commands = _area.getCommands();

            if (commands != null)
            {
                clearBuffer(COMMANDS + "_" + _area.getController());
                setActiveBuffer(COMMANDS + "_" + _area.getController());

                LyStrutsCommands webCommands = (LyStrutsCommands)commands.getInternalView();

                webCommands.writeHtml(this);
            }
        }

        if ((frame != null) && (frame.startsWith(ELEMENT)))
        {
            // int pos =
            // LyTools.intFromString(frame.substring(ELEMENT.length()));
            String elementId = frame.substring(ELEMENT.length());

            prepareElementPage(elementId, targetWindow);
        }
        else if (_noFrames)
        {
            for (i = 0; i < nElements; i++)
                prepareElementPage(getSubElementId(_area.getPanel(i)), targetWindow);
        }
        // FFT00470
        else if ((frame != null) && mustRecurse)
        {
            // This area doesn't know what to do with the given frame name
            // But maybe one of its son can handle it, so we call preparePage
            // for
            // each son
            for (i = 0; i < nElements; i++)
            {
                element = _area.getPanel(i);
                webElement = (LyWebComponent)element.getInternalView();

                if (webElement instanceof LyWebPageWriter)
                    ((LyWebPageWriter)webElement).preparePage(frame, targetWindow);
            }
        }
    } // end method preparePage

    /**
     * Computes the frame mode for this web area. By default, frames are used. But if there are more
     * than one form in the view, a single frame may be used because only one html form may be
     * submitted at a time. This method retrieves the controller id for each form and groups the
     * forms with the same controller in Vectors. Then it calls the computeNoFrame(Vector) method.
     */
    protected void computeNoFrame()
    {
        LyView         view = _area.getParentView();
        LyForm         form, formTmp;
        String         controllerId, controllerIdTmp;
        Vector<LyForm> formsTmp, forms = new Vector<LyForm>();
        int            i, n;

        // Retrieve all the forms in the view
        view.getComponents(LyForm.class, forms);

        while (forms.size() > 1)
        {
            n = forms.size();

            // Working with the first one, we build a Vector containing all
            // the forms controller by the same controller
            form = forms.elementAt(n - 1);
            controllerId = form.getRootController();

            if (controllerId == null)
                continue;

            // if (form.getId().equals("_formContext") )
            if (!form.isEditable())
            {
                forms.removeElement(form);
                continue;
            }

            formsTmp = new Vector<LyForm>(n);

            for (i = n - 1; i >= 0; i--)
            {
                formTmp = forms.elementAt(i);
                controllerIdTmp = formTmp.getRootController();

                // if (formTmp.getId().equals("_formContext") )
                if (!formTmp.isEditable())
                    continue;

                // If same controller, add it to the Vector
                if (controllerId.equals(controllerIdTmp))
                {
                    formsTmp.addElement(formTmp);
                    forms.removeElement(formTmp);
                }
            }

            // If not contained in a tabs (noFrame mode for tabs is managed in
            // LyStrutsTabs), set the noFrame mode if needed
            if (form.getParentTabs() == null)
                computeNoFrame(formsTmp);
        }
    } // end method computeNoFrame

    /**
     * Computes the frame mode for this web area for a group of forms. Retrieves the smallest area
     * containing all the forms and set the noFrame mode on it.
     *
     * @param forms Vector containing all the forms managed by the same controller.
     */
    protected void computeNoFrame(Vector<LyForm> forms)
    {
        if ((forms == null) || (forms.size() < 2))
            return;

        LyForm  form, formTmp;
        LyView  view = _area.getParentView();
        LyArea  area, areaTmp, topArea = view.getArea();
        boolean found = false, foundTmp = true;
        int     i, n = forms.size();

        // Working with the first form, we consider its parent area and look
        // if other forms have this area as a parent. If not, we take this
        // area's parent and do the same. And so on.
        form = forms.elementAt(0);
        area = form.getParentArea();

        while (!found && (area != topArea))
        {
            foundTmp = true;

            for (i = 1; (i < n) && foundTmp; i++)
            {
                foundTmp = false;
                formTmp = forms.elementAt(i);
                areaTmp = formTmp.getParentArea();

                while (!foundTmp && (areaTmp != null))
                {
                    if (area == areaTmp)
                        foundTmp = true;
                    else
                        areaTmp = areaTmp.getParentArea();
                }
            }

            if ((i == n) && foundTmp)
                found = true;
            else
                area = area.getParentArea();
        }

        ((LyStrutsArea)area.getInternalView()).setNoFrames(true);
    } // end method computeNoFrame

    /**
     * Gets the area's parent's menu bar.
     *
     * @return the menu bar if exists, null otherwise
     */
    public LyMenuBar getMenuBar()
    {
        LyComponent parent = _area.getParent();

        if (parent instanceof LyFrame)
            return ((LyFrame)parent).getMenuBar();
        else
            return null;
    }

    /**
     * Check if this area has a menu bar.
     *
     * @return true if it has a menu bar, false otherwise
     */
    public boolean hasMenuBar()
    {
        return (getMenuBar() != null);
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebPageWriter#isFrameSet()
     */
    public boolean isFrameSet()
    {
        if (_noFrames)
            return false;

        int         nElements = _area.getPanelCount();
        LyComponent parent = _area.getParent();

        if ((nElements > 0)
                && (_area.getCommands() != null)
                && !(parent instanceof LyTab)
                && !(parent instanceof LyFrame))
            return true;

        LyComponent    element;
        LyWebComponent internal;
        int            i;

        for (i = 0; i < nElements; i++)
        {
            element = _area.getPanel(i);
            internal = (LyWebComponent)element.getInternalView();

            if ((internal instanceof LyWebPageWriter) && ((LyWebPageWriter)internal).isFrameSet())
                return false;
        }

        return false;
    } // end method isFrameSet

    /**
     * Indicates if content decoration must be generated for current area.
     *
     * @return true if content decoration must be generated, false otherwise.
     */
    protected boolean writeContentDecoration()
    {
        // If parent view is a frame, check on this frame if it manages the content
        // decoration.
        LyView view = _area.getParentView();

        if ((view != null) && (view instanceof LyFrame))
        {
            LyFrame      frame = (LyFrame)view;
            LyStrutsFrame webFrame = (LyStrutsFrame)frame.getInternalView();

            if (webFrame.isFrameSet())
                return false;
        }

        return true;
    }

    /**
     * Add style sheet for form font (needed for printing forms).
     *
     * @param pw the page writer
     */
    protected void writeFontScripts(LyWebPageWriter pw)
    {
        String labelFont = getFontString("LY_DEFAULT_FONT");
        String labelNormalColor = getHexColor("LY_LABEL_NORMAL_STATE");

        // Handle CSS for current Area
        pw.addCssScript("input{" + labelFont + "}", false);
        pw.addCssScript("textarea{" + labelFont + "}", false);
        pw.addCssScript("select{" + labelFont + "}", false);
        pw.addCssScript("font.label{" + labelFont + "}", false);
        pw.addCssScript("td.lblObl{" + labelFont + "font-weight:bold;color:" + labelNormalColor
            + ";}", false);
        pw.addCssScript("td.lbl{" + labelFont + "color:" + labelNormalColor + ";}", false);
    }

    /**
     * Writes area content.
     *
     * @param pw the page writer
     */
    protected void writeContent(LyWebPageWriter pw)
    {
        // If there are no elements in the area, create a javascript var
        // to indicate there is no selection to submit.
        int nElements = _area.getPanelCount();

        if (nElements <= 0)
            pw.addBeginScript(
                "<script type='text/javascript'>var selectionContext='false'</script>", false);
        else
            pw.addBeginScript("<script type='text/javascript'>var selectionContext='true'</script>",
                false);

        // Add scripts for fonts
        writeFontScripts(pw);

        // Write area content
        int[]           neededCells = computeNeededCells();
        int             nRows = neededCells[0];
        int             nCols = neededCells[1];
        LyWebDecoration deco = getWebViewManager().getDecoration();

        if (_area.getBorder() == LyArea.Border.SOLID)
            pw.append(
                "<table border='1' cellpadding='0' cellspacing='0' width='100%' height='100%'><tr><td>");
        else if (_area.getBorder() != LyArea.Border.NONE)
        {
            String label = _area.getLabel();

            if ((_area.getLabel() != null) && (_area.getLabel().length() > 0))
                label = "<font class='areaGroup'>&nbsp;" + translate(label) + "&nbsp;</font>";

            deco.writePageDecoration(pw, LyWebDecoration.AREA_BORDER_BEGIN, label);
            deco.writePageDecoration(pw, LyWebDecoration.AREA_BORDER_LEFT);
        }

        pw.append("<table border='0' cellspacing='0' cellpadding='0'");

        if (_width > 0)
            pw.append(" width='" + _width + "%'");

        String activeName = pw.getActiveName();

        if (_height > 0)
            pw.append(" height='" + _height + "%'");
        else
        {
            if (LyStrutsFrame.MAIN_BUFFER.equals(activeName) // for the login
                    || LyWebConstants.PILOT_FRAME.equals(activeName))
                // page
                pw.append(" height='100%'");
        }

        boolean writeTitleBar = false;

        if (LyWebConstants.HEADER_FRAME.equals(activeName))
        {
            writeTitleBar = true;

            LyToolBar toolBar = _area.getTopRightToolBar();

            if (toolBar != null)
                toolBar.setStyle(LyToolBar.Style.DEFAULT);
        }
        else if (LyWebConstants.PILOT_FRAME.equals(activeName))
        {
            // erase title
            LyTitleBar titleBar = getTitleBar();

            if (titleBar != null)
                titleBar.setLeftLabel("");
        }

        String bgImage = _area.getEnvironment().getImageUrl("LY_AREA_BACKGROUND_IMAGE");
        LyArea parentArea = _area.getParentArea();
        String parentBgImage = ((parentArea == null)
                ? null : parentArea.getEnvironment().getImageUrl("LY_AREA_BACKGROUND_IMAGE"));

        if ((bgImage != null) && (parentBgImage != null) && !bgImage.equals(parentBgImage))
            pw.append(" style='background:url(" + bgImage + ");'>\n");
        else if (_area.getParent() != parentArea)
            pw.append(" id='background'>\n");
        else
            pw.append(">\n");

        if (writeTitleBar)
            // Write the titlebar
            writeTitleBar(pw, nRows, nCols);

        if (LyWebConstants.PILOT_FRAME.equals(activeName))
        {
            // We are writing the dashboard
            // change properties of rightDahsboard

            pw.append("<tr>\n <td vAlign='top'>\n");
            writeToolBar(pw, _area.getTopLeftToolBar());

            boolean displayUpPage = LyWebObject.displayUpPage(getEnvironment());

            if (!displayUpPage)
            {
                pw.append("<hr />");
                LyToolBar toolbar = _area.getTopRightToolBar();

                if (toolbar != null)
                {
                    LyStrutsToolBar  webToolbar = (LyStrutsToolBar)toolbar.getInternalView();

                    webToolbar.setSpecificStyle(true);
                    writeToolBar(pw, toolbar);
                }
            }
            pw.append("</td></tr>\n");
        }
        else
        {
            // Write the toolbar
            if ((_area.getTopLeftToolBar() != null) || (_area.getTopRightToolBar() != null))
            {
                pw.append("<tr>\n<td valign='top' colspan='1' rowspan='1'>\n");
                pw.append("<table width='100%' cellpadding='0' cellspacing='0'><tr><td");
                pw.append(" class='" + getToolbarClass() + "'>\n");
                pw.append("<table width='100%' cellpadding='4' cellspacing='0'><tr><td>\n");
                writeToolBar(pw, _area.getTopLeftToolBar());
                pw.append("</td>\n<td>\n");
                writeToolBar(pw, _area.getTopRightToolBar());
                pw.append("</td>\n</tr>\n</table>\n");
                pw.append("</td>\n</tr>\n</table>\n");
                pw.append("</td>\n</tr>");
            }

        }

        if (!LyWebConstants.HEADER_FRAME.equals(activeName)
            && !LyWebConstants.PILOT_FRAME.equals(activeName))
        {
            // Write tip
            writeTip(pw);

            // Write north panel
            LyPanel northPanel = _area.getBorderPanel(LyArea.NORTH);

            writeBorderPanel(pw, northPanel, LyArea.NORTH);

            LyToolBar leftToolBar = _area.getLeftToolBar();
            LyToolBar rightToolBar = _area.getRightToolBar();
            LyPanel   westPanel = _area.getBorderPanel(LyArea.WEST);
            LyPanel   eastPanel = _area.getBorderPanel(LyArea.EAST);

            if ((leftToolBar != null)
                    || (rightToolBar != null)
                    || (westPanel != null)
                    || (eastPanel != null))
            {
                pw.append("<tr>\n<td valign='top' colspan='1' rowspan='1' height='100%'>\n");
                pw.append(
                    "<table width='100%' height='100%' cellpadding='0' cellspacing='0'>\n<tr>\n");

                if (leftToolBar != null)
                {
                    String vAlign = "top";
                    String width = " width='1%'";

                    if (leftToolBar.getVAlign() == LyToolBar.VAlign.MIDDLE)
                        vAlign = "middle";
                    else if (leftToolBar.getVAlign() == LyToolBar.VAlign.BOTTOM)
                        vAlign = "bottom";

                    if (leftToolBar.getFill() == LyToolBar.Fill.HORIZONTAL)
                        width = "";

                    pw.append("<td" + width + " valign='" + vAlign + "'>\n");
                    writeToolBar(pw, leftToolBar);
                    pw.append("</td>\n");
                }

                if (westPanel != null)
                    writeBorderPanel(pw, westPanel, LyArea.WEST);

                pw.append(
                    "<td>\n<table width='100%' height='100%' cellpadding='0' cellspacing='0'>\n");
                // Write the elements
                writeElements(pw, nRows, nCols);
                pw.append("</table>\n</td>\n");

                if (rightToolBar != null)
                {
                    String vAlign = "top";
                    String width = " width='1%'";

                    if (rightToolBar.getVAlign() == LyToolBar.VAlign.MIDDLE)
                        vAlign = "middle";
                    else if (rightToolBar.getVAlign() == LyToolBar.VAlign.BOTTOM)
                        vAlign = "bottom";

                    if (rightToolBar.getFill() == LyToolBar.Fill.HORIZONTAL)
                        width = "";

                    pw.append("<td" + width + " valign='" + vAlign + "'>\n");
                    writeToolBar(pw, rightToolBar);
                    pw.append("</td>\n");
                }

                if (eastPanel != null)
                    writeBorderPanel(pw, eastPanel, LyArea.EAST);

                pw.append("</tr>\n</table>\n");
                pw.append("</td>\n</tr>\n");
            }
            else
            {
                pw.append("<tr>\n<td valign='top' colspan='1' rowspan='1' height='100%'>\n");
                pw.append("<table width='100%' height='100%' cellpadding='0' cellspacing='0'>\n");

                // Write the elements
                writeElements(pw, nRows, nCols);
                pw.append("</table>\n");
                pw.append("</td>\n</tr>\n");
            }

            // Write the selectbar
            writeSelectBar(pw, nRows, nCols);

            // Write south panel
            LyPanel southPanel = _area.getBorderPanel(LyArea.SOUTH);

            writeBorderPanel(pw, southPanel, LyArea.SOUTH);

            // Write the commands
            writeCommands(pw, nRows, nCols);

            // Write bottom toolbars

            if ((_area.getBottomLeftToolBar() != null) || (_area.getBottomRightToolBar() != null))
            {
                pw.append("<tr><td valign='top' colspan='1' rowspan='1'>");

                pw.append("<table width='100%'><tr><td>");
                writeToolBar(pw, _area.getBottomLeftToolBar());
                pw.append("</td><td>");
                writeToolBar(pw, _area.getBottomRightToolBar());
                pw.append("</td></tr></table>");
                pw.append("</td></tr>");
            }
        }

        pw.append("</table>\n");

        if (_area.getBorder() == LyArea.Border.SOLID)
            pw.append("</td></tr></table>");
        else if (_area.getBorder() != LyArea.Border.NONE)
        {
            deco.writePageDecoration(pw, LyWebDecoration.AREA_BORDER_RIGHT);
            deco.writePageDecoration(pw, LyWebDecoration.AREA_BORDER_END);
        }
    } // end method writeContent

    /**
     * Utility method to write html component with a given constraint.
     *
     * @param pw         the page writer
     * @param component  the component to write
     * @param constraint the given constraint
     * @param nRows      the number of rows
     * @param nCols      the number of columns
     */
    protected void writeComponent(LyWebPageWriter pw, LyWebComponent component,
        LyConstraint constraint, int nRows, int nCols)
    {
        int     rowspan, colspan, currentCol = 0;
        boolean jump = false;

        if (constraint != null)
        {
            rowspan = constraint.getRowSpan();
            colspan = constraint.getColSpan();

            if (colspan <= 0)
            {
                colspan = nCols - currentCol;
                jump = true;
            }

            if (rowspan == 0)
                rowspan = nRows;
            else if (rowspan < 0)
                rowspan = 1;
        }
        else
        {
            rowspan = 1;
            colspan = nCols - currentCol;
            jump = true;
        }

        if (jump)
        {
            pw.append("<tr");

            if ((constraint != null)
                    && ((constraint.getFill() == LyConstraint.Fill.BOTH)
                        || (constraint.getFill() == LyConstraint.Fill.VERTICAL)))
                pw.append(" height='100%'");

            pw.append(">");
        }

        String vAlign = " valign='top'";

        if (constraint != null)
        {
            if (constraint.getVAlign() == LyConstraint.VAlign.BOTTOM)
                vAlign = " valign='bottom'";
            else if (constraint.getVAlign() == LyConstraint.VAlign.MIDDLE)
                vAlign = " valign='middle'";
        }

        pw.append("<td" + vAlign + " colspan='" + colspan + "' rowspan='" + rowspan + "'>");

        currentCol += colspan;

        component.writeHtml(pw);

        pw.append("</td>");

        if (jump)
            pw.append("</tr>\n");
    } // end method writeComponent

    /**
     * Writes frame.
     *
     * @param pw          the page writer
     * @param rootURL     the root URL
     * @param frameName   the frame name
     * @param forceReload the value to print in URL to indicates that the relaod is forced
     * @param scrolling   indicates if the frame is scrolled.
     * @param autoWidth   auto resize the iframe in javascript (width parameter)
     * @param autoHeight  auto resize the iframe in javascript (height parameter)
     */
    protected void writeFrame(LyWebPageWriter pw, String rootURL, String frameName,
        String forceReload, boolean scrolling, boolean autoWidth, boolean autoHeight)
    {
        String maxHeight = " ly_max_height='" + FRAME_MAX_HEIGHT + "'";
        String maxWidth = " ly_max_width='" + FRAME_MAX_WIDTH + "'";

        writeFrame(getId(), getWindowName(), pw, rootURL, frameName, forceReload, scrolling,
            autoWidth, autoHeight, maxHeight, maxWidth);
    }

    /**
     * Write an iframe to fetch a subelement.
     *
     * @param id          the frame id
     * @param windowName  the window name
     * @param pw          the page writer
     * @param rootURL     the root URL
     * @param frameName   the frame name
     * @param forceReload the value to print in URL to indicates that the relaod is forced
     * @param scrolling   indicates if the frame is scrolled. If true, set the HTML frame attribute
     *                    'scrolling' to 'auto', if false, set it to 'no' .
     * @param autoWidth   auto resize the iframe in javascript (width parameter)
     * @param autoHeight  auto resize the iframe in javascript (height parameter)
     * @param maxHeight   the maximum height
     * @param maxWidth    the maximum width
     */
    public static void writeFrame(String id, String windowName, LyWebPageWriter pw, String rootURL,
        String frameName, String forceReload, boolean scrolling, boolean autoWidth,
        boolean autoHeight, String maxHeight, String maxWidth)
    {
        pw.append("<!-- Layout renderer : LyStrutsArea. 2bis -->\n");
        pw.append("<iframe id='" + frameName + "' name='" + frameName + "'");

        String  heightValue = null, widthValue = null;

        if (autoWidth)
            pw.append(" width='0' height='100%'");
        else if (autoHeight)
            pw.append(" width='100%' height='30'");
        else
        {
            pw.append(" width='100%' height='100%' ");

            if (!pw.isInnerView())
            {
                heightValue = "1";
                widthValue = "1";
            }

            pw.append(maxHeight + "" + maxWidth);
        }

        if (LyWebConstants.POPUP_VIEW.equals(pw.getWindowName())
                && (pw.getWebViewManager().getBrowser() == LyWebViewManager.BROWSER_FIREFOX))
        {
            heightValue = "reset";
        }

        if (heightValue != null)
            pw.append(" ly_height_scroll='" + heightValue + "'");

        if (widthValue != null)
            pw.append(" ly_width_scroll='" + widthValue + "'");

        pw.append("  marginwidth='0' marginheight='0' frameborder='0'\n");

        /*
         * if (scrolling) if (autoHeight) pw.append(" scrolling='no'"); else
         * pw.append(" scrolling='no'"); else
         */
        if (scrolling)
            pw.append(" scrolling='auto'");
        else
            pw.append(" scrolling='no'");

        pw.append(" src='" + rootURL + "?");
        pw.append(LyWebConstants.PAGE + "=" + id + "&#38;");
        pw.append(LyWebConstants.FRAME + "=" + frameName + forceReload);
        if (windowName != null)
            pw.append("&#38;" + LyWebConstants.TARGET_WINDOW + "=" + windowName);
        pw.append("'></iframe>\n");
    } // end method writeFrame

    /**
     * Write the FRAMESET structure of the page.
     *
     * @param pw the page writer
     */
    public void writeGlobalFrameSet(LyWebPageWriter pw)
    {
        boolean inPopupView = (LyWebConstants.POPUP_VIEW.equals(pw.getWindowName()));

        writeGlobalFrameSet(pw, false, inPopupView);
    }

    /**
     * Write the FRAMESET structure of the page.
     *
     * @param pw         the page writer
     * @param autoWidth  auto resize the iframe in javascript (width parameter)
     * @param autoHeight auto resize the iframe in javascript (height parameter)
     */
    public void writeGlobalFrameSet(LyWebPageWriter pw, boolean autoWidth, boolean autoHeight)
    {
        boolean          sideToolBar = false;
        boolean          bottomToolBar = false;
        boolean          scrollToolbar = false;
        LyTitleBar       titleBar = getTitleBar();
        int              toolTitleSize = 0;
        String           forceReload = "";
        LyWebViewManager viewManager = getWebViewManager();

        if (getForceReloadState())
        {
            _lastReloadTime = System.currentTimeMillis();
            setForceReloadState(false);
        }

        if (_lastReloadTime > 0)
            forceReload = "&#38;reload=" + _lastReloadTime;

        if (testResource("LY_WEB_SCROLL_TOOLBAR"))
            scrollToolbar = true;

        if ((_area.getRightToolBar() != null) || (_area.getLeftToolBar() != null))
            sideToolBar = true;

        if ((_area.getBottomRightToolBar() != null) || (_area.getBottomLeftToolBar() != null))
            bottomToolBar = true;

        if (titleBar != null)
        {
            int    imageHeight = 20;

            String titlebarImageHeightResource = getResource("LY_TITLEBAR_IMAGE_HEIGHT");
            int    titlebarImageHeight = -1;

            if ((titlebarImageHeightResource != null) && (titlebarImageHeightResource.length() > 0))
                titlebarImageHeight = Integer.parseInt(titlebarImageHeightResource);

            String defaultImageHeightResource = getResource("LY_DEFAULT_IMAGE_HEIGHT");
            int    defaultImageHeight = -1;

            if ((defaultImageHeightResource != null) && (defaultImageHeightResource.length() > 0))
                defaultImageHeight = Integer.parseInt(defaultImageHeightResource);

            if (titlebarImageHeight > 0)
                imageHeight = titlebarImageHeight;
            else if (defaultImageHeight > 0)
                imageHeight = defaultImageHeight;

            // Add 10 pixels for space above and below the image
            toolTitleSize += imageHeight + 10;
        }
        int toolSize = 0;

        if (_area.getTopLeftToolBar() != null)
        {
            int[] size = computeToolBarsSize(_area.getTopLeftToolBar());

            toolSize = size[1];
        }

        if (_area.getTopRightToolBar() != null)
        {
            int[] size = computeToolBarsSize(_area.getTopRightToolBar());

            if (size[1] > toolSize)
                toolSize = size[1];
        }
        toolTitleSize += toolSize;

        if (_area.getTip() != null)
        {
            // find the number of line for the text
            // From the police, get the size.
            // the height of the text will be approximatively size * 1.5
            // Add extra space for layout
            String tip = translate(_area.getTip());
            int    nbLine = LyString.getNbLines(tip);
            int    size = getFontSize("LY_TIP_FONT");
            int    finalSize = (3 * size * nbLine) / 2;

            toolTitleSize += 5 + finalSize;
        }

        // Write the begin scripts
        StringBuffer scripts = getBeginScriptsAsHtml(MAIN_BUFFER);

        pw.append(scripts.toString());

        StringBuffer rows = new StringBuffer();

        if (toolTitleSize != 0)
            rows.append("" + toolTitleSize);

        String elementsSize = getElementsFrameSize();

        if (toolTitleSize != 0)
            rows.append(",");

        // Add center frame
        if (sideToolBar)
            rows.append("*");
        else
            rows.append(elementsSize);

        if (bottomToolBar)
        {
            toolSize = 0;
            if (_area.getBottomLeftToolBar() != null)
            {
                int[] size = computeToolBarsSize(_area.getBottomLeftToolBar());

                toolSize = size[1];
            }
            if (_area.getBottomRightToolBar() != null)
            {
                int[] size = computeToolBarsSize(_area.getBottomRightToolBar());

                if (toolSize < size[1])
                    toolSize = size[1];
            }

            rows.append("," + toolSize);
        }

        if (rows.toString().equals("*"))
        {
            rows = new StringBuffer();
            rows.append("100%,*");
        }
        // Store pageWriterId
        pw.addBeginScript("<script type='text/javascript'>var pageWriterId='" + pw.getId()
            + "';</script>\n", false);

        String rootURL = getFrameBaseUrl();

        pw.append("<!-- Layout renderer : LyStrutsArea. 3 -->\n");

        if (viewManager.getBrowser() == LyWebViewManager.BROWSER_FIREFOX)
            pw.append("<table width='100%' height='100%' border='0' cellspacing='0' cellpadding='0'>\n");
        else
            pw.append(
                "<table width='100%' height='100%' border='0' cellspacing='0' cellpadding='0'>\n");

        pw.append("<tr height='0'>\n");
        pw.append(" <td valign='top'>\n");

        if ((toolTitleSize != 0) && scrollToolbar)
            writeFrame(pw, rootURL, _area.getId() + TOOLTITLE, forceReload, true, false, true);
        else if ((toolTitleSize != 0) && !scrollToolbar)
        {
            String          tip = _area.getTip();

            LyWebDecoration deco = getWebViewManager().getDecoration();

            pw.append("<div id=\"noBackground\">");

            deco.writePageDecoration(pw, LyWebDecoration.WINDOW_BEGIN);

            if (titleBar != null)
            {
                deco.writePageDecoration(pw, LyWebDecoration.WINDOW_TOP);

                deco.writePageDecoration(pw, LyWebDecoration.WINDOW_HEADER_LEFT);

                LyStrutsTitleBar webTitlebar = (LyStrutsTitleBar)titleBar.getInternalView();

                webTitlebar.writeHtml(pw);
                deco.writePageDecoration(pw, LyWebDecoration.WINDOW_HEADER_RIGHT);
            }

            deco.writePageDecoration(pw, LyWebDecoration.WINDOW_HEADER_LEFT);
            // write toolBars
            LyToolBar topLeftToolBar = _area.getTopLeftToolBar();
            LyToolBar topRightToolBar = _area.getTopRightToolBar();

            // if one of the two toolbar has a vertical separator style
            // show it
            String    toolbarFrame = "";

            if (((topRightToolBar != null)
                        && (topRightToolBar.getStyle() == LyToolBar.Style.SEPARATOR))
                    || ((topLeftToolBar != null)
                        && (topLeftToolBar.getStyle() == LyToolBar.Style.SEPARATOR)))
                toolbarFrame = " frame='hsides' rules='none'";

            pw.append("<table width='100%' cellpadding='0' cellspacing='0'");
            pw.append(" border='0'");
            pw.append(toolbarFrame);
            pw.append("><tr valign='middle'>");
            pw.append("<td align='left'>");
            pw.append("<table width='100%' cellpadding='0' cellspacing='0'>");
            pw.append("<tr>\n");

            if ((topLeftToolBar != null) && (topLeftToolBar.getComponentCount() > 0))
            {
                pw.append("<td class='" + getToolbarClass() + "'>\n");
                writeToolBar(pw, topLeftToolBar);
                pw.append("</td>\n");
            }

            pw.append("</tr></table>\n");
            pw.append("</td>\n");

            if ((topRightToolBar != null) && (topRightToolBar.getComponentCount() > 0))
            {
                pw.append("</td>\n<td align='right' class='" + getToolbarClass() + "'>");
                writeToolBar(pw, topRightToolBar);
                pw.append("</td>");
            }

            pw.append("</td></tr></table>");

            if (tip != null)
            {
                String htmlString = getHtmlString(tip);

                pw.append("<table width='100%' border='0' cellpadding='3' cellspacing='0'>");
                pw.append("<tr valign='middle'>");

                pw.append("<td align='left' class='"
                    + deco.getClassName(_area, DECO_TIP_PARAMETER, pw) + "'>");
                pw.append(htmlString);
                pw.append("</td>");
                pw.append("</tr>");
                pw.append("</table>");
            }
            deco.writePageDecoration(pw, LyWebDecoration.WINDOW_HEADER_RIGHT);
            deco.writePageDecoration(pw, LyWebDecoration.WINDOW_END);

            pw.append("</div>");
        }

        pw.append(" </td>\n");
        pw.append("</tr>\n");

        // Write north panel
        LyPanel northPanel = _area.getBorderPanel(LyArea.NORTH);
        LyPanel southPanel = _area.getBorderPanel(LyArea.SOUTH);
        LyPanel westPanel = _area.getBorderPanel(LyArea.WEST);
        LyPanel eastPanel = _area.getBorderPanel(LyArea.EAST);

        writeBorderPanel(pw, northPanel, LyArea.NORTH);

        // Work around of Firefox
        if (viewManager.getBrowser() == LyWebViewManager.BROWSER_FIREFOX)
        {
            if (toolSize != 0)
                pw.append("<tr height='90%'>\n");
            else
                pw.append("<tr height='95%'>\n");
        }
        else
            pw.append("<tr height='100%'>\n");

        pw.append("<td>\n");

        if ((westPanel != null) || (eastPanel != null))
        {
            pw.append(
                "<table width='100%' height='100%' border='0' cellpadding='0' cellspacing='0'><tr width='100%' height='100%'><td>\n");
            writeBorderPanel(pw, westPanel, LyArea.WEST);
            pw.append("</td><td>\n");
        }

        if (sideToolBar)
            writeFrame(pw, rootURL, _area.getId() + CENTER, forceReload, false, false, false);
        else
            writeElementsFrame(pw, rootURL, forceReload, autoWidth, autoHeight);

        if ((westPanel != null) || (eastPanel != null))
        {
            pw.append("</td><td>\n");
            writeBorderPanel(pw, eastPanel, LyArea.EAST);
            pw.append("</td></tr></table>\n");
        }

        pw.append("</td>\n");
        pw.append("</tr>\n");

        writeBorderPanel(pw, southPanel, LyArea.SOUTH);

        if (bottomToolBar)
        {
            pw.append("<tr height='0%'>\n");
            pw.append(" <td>\n");
            writeFrame(pw, rootURL, _area.getId() + BOTTOMTOOLBAR, forceReload, false, false, true);
            pw.append(" </td>\n");
            pw.append("</tr>\n");
        }
        pw.append("</table>\n");

        // Write the end scripts
        scripts = getEndScriptsAsHtml(MAIN_BUFFER);
        pw.append(scripts.toString());
    } // end method writeGlobalFrameSet

    /**
     * Writes elements frame.
     *
     * @param pw          the page writer
     * @param rootURL     the root URL
     * @param forceReload the value to print in URL to indicates that the relaod is forced
     */
    protected void writeElementsFrame(LyWebPageWriter pw, String rootURL, String forceReload)
    {
        writeElementsFrame(pw, rootURL, forceReload, false, true);
    }

    /**
     * Writes elements frame.
     *
     * @param pw          the page writer
     * @param rootURL     the root URL
     * @param forceReload the value to print in URL to indicates that the relaod is forced
     * @param autoWidth   auto resize the iframe in javascript (width parameter)
     * @param autoHeight  auto resize the iframe in javascript (height parameter)
     */
    protected void writeElementsFrame(LyWebPageWriter pw, String rootURL, String forceReload,
        boolean autoWidth, boolean autoHeight)
    {
        int              i, nElements = _area.getPanelCount();
        LySelectBar      selectBar = _area.getSelectBar();
        LyCommands       commands = _area.getCommands();
        LyComponent      element;
        LyWebViewManager viewManager = getWebViewManager();

        pw.append("<!-- Layout renderer : LyStrutsArea. 4 -->\n");
        pw.append("<table width='100%' height='100%' border='0' "
            + "cellspacing='0' cellpadding='0'>\n");

        for (i = 0; i < nElements; i++)
        {
            if (viewManager.getBrowser() == LyWebViewManager.BROWSER_FIREFOX)
                pw.append("<tr>\n");
            else
                pw.append("<tr height='100%'>\n");
            pw.append("<td><!-- " + (i + 1) + " element of " + nElements + " -->\n");
            element = _area.getPanel(i);

            boolean scrolling = false;

            if ((element instanceof LyArea) && getWebViewManager().forceScrolling() && !isInPopUp())
            {
                LyArea area = (LyArea)element;

                if ((area.getScrolling() == LyArea.Scrolling.AUTO)
                        || (area.getScrolling() == LyArea.Scrolling.YES))
                    scrolling = true;
            }
            else if ((commands != null) && getWebViewManager().forceScrolling()
                    && !isInPopUp()
                    && !(element instanceof LyTabs))
                scrolling = true;

            writeFrame(pw, rootURL, ELEMENT + getSubElementId(element), forceReload, scrolling,
                autoWidth, autoHeight);
            pw.append(" </td>\n");
            pw.append("</tr>\n");
        }

        if (viewManager.getBrowser() == LyWebViewManager.BROWSER_FIREFOX)
        {
            if ((selectBar != null) || (commands != null))
            {
                int height = 0;

                if (selectBar != null)
                    height += 25;
                if (commands != null)
                {
                    if (testResource("LY_USE_IMAGE_FOR_COMMAND"))
                        height += 50;
                    else
                        height += 35;
                }

                pw.append("<tr height='" + height + "'>\n");
                pw.append("<td>\n");

                if (selectBar != null)
                {
                    pw.append("<!-- select bar -->\n");
                    writeFrame(pw, rootURL, _area.getId() + SELECTBAR, forceReload, false, false,
                        true);
                }

                if (commands != null)
                {
                    pw.append("<!-- command bar -->\n");
                    writeFrame(pw, rootURL, COMMANDS + "_" + _area.getController(), forceReload,
                        false, false, true);
                }

                pw.append("</td>\n");
                pw.append("</tr>\n");
            }
        }
        else
        {
            if (selectBar != null)
            {
                pw.append("<tr>\n");
                pw.append(" <td><!-- select bar -->\n");
                writeFrame(pw, rootURL, _area.getId() + SELECTBAR, forceReload, false, false, true);
                pw.append(" </td>\n");
                pw.append("</tr>\n");
            }

            if (commands != null)
            {
                pw.append("<tr>\n");
                pw.append(" <td height='0'><!-- command bar -->\n");
                writeFrame(pw, rootURL, COMMANDS + "_" + _area.getController(), forceReload, false,
                    false, true);
                pw.append(" </td>\n");
                pw.append("</tr>\n");
            }
        }

        pw.append("</table>\n");

    } // end method writeElementsFrame

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebPage#getBodyScript()
     */
    protected String getBodyScript()
    {
        if (LyWebConstants.POPUP_VIEW.equals(getWindowName()))
            return "onload='checkSizePopupView(null, true);ResetFormFocus(false);'";
        else
        {
            LyComponent parent = _area;

            do
            {
                parent = parent.getParent();
                if ((parent != null) && (parent instanceof LyView))
                {
                    if (parent instanceof LyDialog)
                    {
                        LyDialog dialog = (LyDialog)parent;

                        if (LyWebConstants.POPUP_VIEW.equals(dialog.getTarget()))
                            return "onload='checkSizePopupView(null, true);ResetFormFocus(false);'";
                    }
                }
            }
            while (parent != null);
            return super.getBodyScript();
        }
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebPage#isInPopUp()
     */
    protected boolean isInPopUp()
    {
        if (LyWebConstants.POPUP_VIEW.equals(getWindowName()))
            return true;
        else
        {
            LyComponent parent = _area;

            do
            {
                parent = parent.getParent();
                if ((parent != null) && (parent instanceof LyView))
                {
                    if (parent instanceof LyDialog)
                    {
                        LyDialog dialog = (LyDialog)parent;

                        if (LyWebConstants.POPUP_VIEW.equals(dialog.getTarget()))
                            return true;
                    }
                }
            }
            while (parent != null);
            return super.isInPopUp();
        }
    }

    /**
     * (non-Javadoc)
     *
     * @see leon.view.graphic.LyGraphicalArea#setRedirect(java.lang.String)
     */
    public void setRedirect(String redirect)
    {
        getWebViewManager().addReloadFrame(_lastFrameName);
    }

    /*
     * (non-Javadoc)
     * @see leon.view.graphic.LyGraphicalArea#setLayoutInfo(java.lang.String)
     */
    public void setLayoutInfo(String layoutInfo)
    {
        if ((_area.getLayout() == LyArea.Layout.BORDER)
                || (_area.getLayout() == LyArea.Layout.CARD))
            refresh();
    }
} // end class LyStrutsArea

