/*
 * LyStrutsBoolean.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Boolean object (as described in the dtd) for web viewer.
 *
 * <p><b>BOOLEAN: Input Field of Boolean Type</b><br>
 * An input field of boolean type allows to display a graphical component with a three state
 * checkbox : not specified (empty), true (checked with a green sign), false (with a red cross). It
 * is possible to specify a label for each of these states and to make a negation of the boolean
 * value.</p>
 *
 * @version $Id: LyStrutsBoolean.java 16172 2009-01-06 11:04:44Z jls $
 * @author  Lyria
 * @see     leon.view.LyBoolean
 */
public class LyStrutsBoolean extends LyStrutsFieldItem
	implements LyGraphicalBoolean
{
	public static final String __VERSION =
		"$Id: LyStrutsBoolean.java 16172 2009-01-06 11:04:44Z jls $";

	/** The boolean generic component. */
	protected LyBoolean		   _boolean;

	/**
	 * Class constructor of a Boolean object for the struts target.
	 *
	 * @param booleanViewComp the generic Boolean defined for every type of view targets.
	 */
	public LyStrutsBoolean(LyBoolean booleanViewComp)
	{
		super(booleanViewComp);
		_boolean = booleanViewComp;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
		updateRefreshInfo(pw);

		String  checked = getImageUrl("LY_BOOLEAN_CHECKED_BACKGROUND");
		String  unchecked = getImageUrl("LY_BOOLEAN_UNCHECKED_BACKGROUND");
		String  checkedHoover = getImageUrl("LY_BOOLEAN_CHECKED_HOOVER");
		String  uncheckedHoover = getImageUrl("LY_BOOLEAN_UNCHECKED_HOOVER");
		String  empty = getImageUrl("LY_BOOLEAN_EMPTY_BACKGROUND");
		String  emptyHoover = getImageUrl("LY_BOOLEAN_EMPTY_HOOVER");
		String  formName = "forms[0]";//getFormName(pw);
		String  name = getId();
		String  nameSubst = LyString.replace(name, "\\", "\\\\");
		String  labelChecked = translate(_boolean.getLabelChecked());
		String  labelUnchecked = translate(_boolean.getLabelUnchecked());
		String  labelEmpty = translate(_boolean.getLabelEmpty());
		boolean not = _boolean.isNot();
		boolean needCheck = needCheck();
		boolean isEditable = isEditable();
		boolean isOptional = isOptional();
		LyBoolean.Status   status = _boolean.getStatus();
		String  value = LyBoolean.Status.EMPTY.toString(), shownValue = LyBoolean.Status.EMPTY.toString();
		String  image = empty;
		String  label = labelEmpty;
		String  imageName = name + "_image";
		String  imageNameSubst = LyString.replace(imageName, "\\", "\\\\");
		String  textName = name + "_text";
		String  textNameSubst = LyString.replace(textName, "\\", "\\\\");

		pw.addBeginScript("<script type='text/javascript'>" + "window.defaultStatus='';"
			+ "</script>", false);

		if (status == LyBoolean.Status.CHECKED)
		{
			label = (not ? labelUnchecked : labelChecked);
			value = LyBoolean.Status.CHECKED.toString();
			shownValue = (not ? LyBoolean.Status.UNCHECKED.toString() : LyBoolean.Status.CHECKED.toString());
			image = (not ? unchecked : checked);
		}
		else if (status == LyBoolean.Status.UNCHECKED)
		{
			label = (not ? labelChecked : labelUnchecked);
			value = LyBoolean.Status.UNCHECKED.toString();
			shownValue = (not ? LyBoolean.Status.CHECKED.toString() : LyBoolean.Status.UNCHECKED.toString());
			image = (not ? checked : unchecked);
		}

		if (isEditable)
		{
			pw.append("\n<input type='hidden' name='" + name + "' value='" + value + "'/>\n");
			pw.append("\n<input type='hidden' name='" + name + "_shown_value' value='" + shownValue
				+ "'/>\n");
			pw.append("<a href=\"javascript:");
			pw.append("ChangeBooleanState(window.document.");
			pw.append(formName);
			pw.append(",'");
			pw.append(nameSubst);
			pw.append("','");
			pw.append(nameSubst + "_shown_value");
			pw.append("','");
			pw.append(imageNameSubst);
			pw.append("','");
			pw.append(textNameSubst);
			pw.append("',");
			pw.append(isOptional + "");
			pw.append(",'");
			pw.append(LyString.getJavascriptString(labelEmpty));
			pw.append("','");
			pw.append(LyString.getJavascriptString(labelUnchecked));
			pw.append("','");
			pw.append(LyString.getJavascriptString(labelChecked));
			pw.append("','");
			pw.append(emptyHoover);
			pw.append("','");
			pw.append(uncheckedHoover);
			pw.append("','");
			pw.append(checkedHoover);
			pw.append("',");
			pw.append(not + "");
			pw.append(");");

			if (needCheck)
			{
				pw.append("FieldChange(");
				pw.append("window.document." + formName);
				pw.append(",window.document." + formName + ".elements['" + nameSubst + "'],'"
					+ _boolean.getRootController() + "');");
			}

			pw.append("\"");
			pw.append(" onmouseover=\"javascript:SwapBooleanImage(window.document.");
			pw.append(formName);
			pw.append(",'");
			pw.append(nameSubst + "_shown_value");
			pw.append("','");
			pw.append(imageNameSubst);
			pw.append("','");
			pw.append(emptyHoover);
			pw.append("','");
			pw.append(uncheckedHoover);
			pw.append("','");
			pw.append(checkedHoover);
			pw.append("');return true;\" ");
			pw.append(" onmouseout=\"javascript:SwapBooleanImage(window.document.");
			pw.append(formName);
			pw.append(",'");
			pw.append(nameSubst + "_shown_value");
			pw.append("','");
			pw.append(imageNameSubst);
			pw.append("','");
			pw.append(empty);
			pw.append("','");
			pw.append(unchecked);
			pw.append("','");
			pw.append(checked);
			pw.append("');return true;\">");
		}

        // Write the image
        pw.append("<img src='");
        pw.append(image);
        pw.append("' name='" + name + "_image' ");
        if (!isEditable)
            pw.append(" class='disabledTool'");
        pw.append("align='middle' border='0'/>");


		if (isEditable)
			pw.append("</a>\n");
		else
			pw.append("&nbsp;");

		// Write the label
		LyWebDecoration	deco = getWebViewManager().getDecoration();
		String			className = deco.getClassName(_boolean, pw);

		pw.append("<font class='" + className + "' id='" + textName + "'>");
		if (label != null)
			pw.append(label);
		pw.append("</font>\n");
	} // end method writeHtml

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.setinterface.LySetStateInterface#setState(short)
	 */
	public void setStatus(LyBoolean.Status status)
	{
		LyComponent parent = _boolean.getParent();

		if (parent instanceof LyFormField)
		{
			LyFormField		  formField = (LyFormField)parent;
			LyStrutsFormField webField = (LyStrutsFormField)formField.getInternalView();

			webField.refreshDivContent();
		}
		else
			refresh();
	}
} // end class LyStrutsBoolean
