/*
 * LyStrutsCaption.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Caption object (as described in the dtd) for web viewer.
 *
 * <p><b>CAPTION: Legend</b><br>
 * A legend allows to assign to an input field a text area that will be displayed with the field.
 * The text of the legend is eventually followed by a colon (' : ').</p>
 *
 * @version $Id: LyStrutsCaption.java 23300 2009-12-17 13:56:41Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.LyCaption
 */
public class LyStrutsCaption extends LyWebObject
	implements LyGraphicalCaption
{

	public static final String __VERSION =
		"$Id: LyStrutsCaption.java 23300 2009-12-17 13:56:41Z jlsaouli $";

	/** The caption generic component. */
	protected LyCaption		   _caption;

	/**
	 * Class constructor of a Caption object for the struts target.
	 *
	 * @param caption the generic Caption defined for every type of view targets.
	 */
	public LyStrutsCaption(LyCaption caption)
	{
		super(caption);
		_caption = caption;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	    pw.startComponent(this);
	    
		updateRefreshInfo(pw);
		
		String label = _caption.getLabel();

		if ((label == null) || (label.trim().length() == 0))
			pw.append("&nbsp;");
		else
		{
			label = getHtmlString(label);
			if ((label == null) || (label.trim().length() == 0))
				pw.append("&nbsp;");
			else
			{
				pw.append(label);
				
				LyFormField field = (LyFormField)_caption.getParent();
				    
				if (!field.isOptional())
				{
	                String  str = getString("LY_CAPTION_ASTERISK", "");
	                
	                if ((str != null) && (str.trim().length() > 0))
	                    pw.append("<span class=\"asterisk\">" + str + "</span>");
				}
				
                if (_caption.isColon())
                    pw.append(getString("LY_CAPTION_COLON", ""));
                else
                    pw.append("\n");
			}
		}
		
	    pw.endComponent(this);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.setinterface.LySetLabelInterface#setLabel(java.lang.String)
	 */
	public void setLabel(String label)
	{
		LyComponent		parent = _caption.getParent();
		
		if (parent instanceof LyFormField)
		{
			LyStrutsFormField	strutsFormField = (LyStrutsFormField)parent.getInternalView();
			
			strutsFormField.refreshCaption();
		}
		else
			refresh();
	}
	
	/* (non-Javadoc)
	 * @see leon.view.setinterface.LySetEnabledInterface#setEnabled(boolean)
	 */
	public void setEnabled(boolean status)
	{
		LyComponent		parent = _caption.getParent();
		
		if (parent instanceof LyFormField)
		{
			LyStrutsFormField	strutsFormField = (LyStrutsFormField)parent.getInternalView();
			
			strutsFormField.refreshCaption();
		}
		else
			refresh();
	}
} // end class LyStrutsCaption
