/*
 * LyStrutsChoice.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import leon.view.*;
import leon.view.getinterface.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Choice object (as described in the dtd) for web viewer.
 *
 * <p><b>CHOICE: Input Field of Choice Type</b><br>
 * An input field of Choice type allows to display a graphical component containing a set of check
 * boxes. It allows to choose one or more values among a discrete list of values. It is possible to
 * specify whether the choice is multiple or not and whether the boxes should be arranged
 * horizontally or vertically.</p>
 *
 * @version $Id: LyStrutsChoice.java 23300 2009-12-17 13:56:41Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.LyChoice
 */
public class LyStrutsChoice extends LyStrutsFieldItem
	implements LyGraphicalChoice, LyGetValueInterface
{
	public static final String __VERSION =
		"$Id: LyStrutsChoice.java 23300 2009-12-17 13:56:41Z jlsaouli $";

	/** The choice generic component. */
	protected LyChoice		   _choice;

	/**
	 * Class constructor of a Choice object for the struts target.
	 *
	 * @param choice the generic Choice defined for every type of view targets.
	 */
	public LyStrutsChoice(LyChoice choice)
	{
		super(choice);
		_choice = choice;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	    pw.startComponent(this);
	    
		updateRefreshInfo(pw);
	  
		String  style = "";
		
		if (!_choice.isDefaultFont() || !_choice.isDefaultFontColor())
			style += getFontString(_choice.getFont(),_choice.getFontColor());
		
		if (!_choice.isDefaultColor())
		{
			style += " background-color:" + getHexColor(_choice.getColor()) + ";";
			style += " background-image:none;";
		}
    
		int     i, n = _choice.getOptionCount();

		// FFT00263 : bug if nothing checked : nothing sent to the servlet
		boolean isMultiple = _choice.isMultiple();

		if (isMultiple)
		{
			StringBuffer value = new StringBuffer();
			String		 sep = LyWebParameterValues.HTML_ENCODED_SEP;
			LyOption     option;

			for (i = 0; i < n; i++)
			{
				option = _choice.getOption(i);
				if (option.isChecked())
				{
					if (value.length() > 0)
						value.append(sep);
					value.append(option.getId());
				}
			}

			pw.append("<input type='hidden' name='" + getId() + "' value='" + value.toString()
				+ "' />\n");
		}

		// If vertical and if there is a maxspan, use as many tables as needed
		// rows.
		boolean isVertical = (_choice.getFill() == LyChoice.Fill.VERTICAL);
		int     maxSpan = _choice.getMaxSpan();
		int		width = -1;
		int		height = -1;
		
		// If a constraint is set on the choice, use the nCols and nRows values to compute
		// choice size in pixels.
		if ((_choice.getParent() instanceof LyFormField)
				&& (((LyFormField)(_choice.getParent())).getConstraint() != null))
		{
			LyConstraint	constraint = ((LyFormField)(_choice.getParent())).getConstraint();
			
			if (constraint != null)
			{
				if (constraint.getNCols() > 0)
					width = constraint.getNCols();

				if (constraint.getNRows() > 0)
					height = constraint.getNRows();
			}
		}

		if ((width > 0) || (height > 0))
		{
			pw.append("<table border='0' cellpadding='0' cellspacing='0' style='" + style);
			
			if (width > 0)
				pw.append("width:" + width + ";");
			
			if (height > 0)
				pw.append("height:" + height + ";");
			
			pw.append("'><tr><td vAlign='top' align='left'>");
		}
	    
		if (style.trim().length() > 0)
			style = " style=\'" + style + "\'";

		if (isVertical && (maxSpan > 0) && (n > maxSpan))
			pw.append(
				"<table border='0' cellspacing='0' cellpadding='0' "+ style +">\n<tr valign='top'>\n<td nowrap='true'>\n");

		pw.append("<table border='0' cellspacing='0' cellpadding='0' "+ style +">\n");

		if (!isVertical)
			pw.append("<tr valign='top'>\n");
		
		for (i = 0; i < n; i++)
			writeOption(pw, i);

		if (!isVertical)
			pw.append("</tr>\n");

		pw.append("</table>\n");

		if (isVertical && (maxSpan > 0) && (n > maxSpan))
			pw.append("</td>\n</tr>\n</table>\n");

		if ((width > 0) || (height > 0))
			pw.append("</td></tr></table>");
	} // end method writeHtml

	/**
	 * Write HTML code for option at the given index.
	 *
	 * @param pw the page writer
	 * @param i  the given index
	 */
	protected void writeOption(LyWebPageWriter pw, int i)
	{
		LyOption	  option = _choice.getOption(i);
		LyStrutsOption internal = (LyStrutsOption)option.getInternalView();
		boolean		  isVertical = (_choice.getFill() == LyChoice.Fill.VERTICAL);
		int			  maxSpan = _choice.getMaxSpan();

		if (isVertical)
		{
			// If vertical and maxspan is reached, create a new
			// table in the table that contains all the others.
			if ((maxSpan > 0) && ((i % maxSpan) == 0) && (i != 0))
				pw.append(
					"</table>\n</td>\n<td>&nbsp;&nbsp;</td>\n<td nowrap='true'><table border='0' cellspacing='0' cellpadding='0'>\n");

			pw.append("<tr valign='top'>\n");
		}
		else if ((maxSpan > 0) && ((i % maxSpan) == 0) && (i != 0))
			pw.append("</tr>\n<tr valign='top'>\n");

		pw.append("<td nowrap='true'");
		
		String	onMouseOverFunctions = option.getEventFunctions(LyComponent.EventType.MOUSE_ENTERED, ';');
		String	onMouseOutFunctions = option.getEventFunctions(LyComponent.EventType.MOUSE_EXITED, ';');

		if (onMouseOverFunctions.length() > 0)
			pw.append(" onmouseover=\"" + onMouseOverFunctions + "\"");
		
		if (onMouseOutFunctions.length() > 0)
			pw.append(" onmouseout=\"" + onMouseOutFunctions + "\"");

		if (!option.isDefaultColor())
		{
			pw.append(" style='background-color:"+getHexColor(option.getColor())+";");
			pw.append(" background-image:none;'");
	    }

		pw.append(">\n");

		if (!isVertical && (((maxSpan > 0) && ((i % maxSpan) != 0)) || (maxSpan <= 0)) && (i != 0))
			pw.append("&nbsp;&nbsp;");

		internal.writeHtml(pw);

		pw.append("</td>\n");

		if (isVertical)
			pw.append("</tr>\n");
	} // end method writeOption

	/*
     * (non-Javadoc)
     * @see leon.view.getinterface.LyGetValueInterface#getValue()
     */
    public String getValue()
    {
		int				 i, n = _choice.getOptionCount();
		StringBuffer     sb = new StringBuffer();
		LyOption		 option;
		char			 sep = getEnvironment().getString().SEP;

		for (i = 0; i < n; i++)
		{
			option = _choice.getOption(i);

			if (option.isChecked())
			{
				if (sb.length() > 0)
					sb.append(sep);
				sb.append(option.getId());
			}
		}
		
		if (sb.length() == 0)
			return null;
		return (sb.toString());
    }
} // end class LyStrutsChoice
