/*
 * LyStrutsCommand.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Command object (as described in the dtd) for web viewer.
 *
 * <p><b>COMMAND: Command Button</b><br>
 * A command button allows to execute an action. There are predefined types of command giving
 * information about the nature of the action to execute (closing, cancellation, help, etc.).</p>
 *
 * @version $Id: LyStrutsCommand.java 35717 2012-06-04 14:26:42Z nhamon $
 * @author  Lyria S.A.
 */
public class LyStrutsCommand extends LyWebObject
    implements LyGraphicalCommand
{
	public static final String __VERSION =
		"$Id: LyStrutsCommand.java 35717 2012-06-04 14:26:42Z nhamon $";

	/** The command generic component. */
    protected LyCommand        _command;
    
    /** Enabled state. */
    protected boolean          _enabled;
    
	/**
	 * Constructor.
	 *
	 * @param command The generic parent component.
	 */
	public LyStrutsCommand(LyCommand command)
	{
        super(command);
        
        _command = command;
        _enabled = true;
	}

	/**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
     */
    public void writeHtml(LyWebPageWriter pw)
    {
        pw.startComponent(this);
        
        updateRefreshInfo(pw);

        String  id = _command.getId();
        String  htmlId = _command.getId() + _command.getIdCode();
        LyCommand.Action   action = _command.getAction();
        String  label = _command.getLabel();
        String  image = _command.getImage();
        String  targetWindow = pw.getWindowName();
        String  errorMessage = getMessage("LY_FORM_ARLEADY_SUBMITTED");
        String  fileErrorMessage = getMessage("LY_FORM_FILE_ERROR");
        String  left = getImageUrl("LY_COMMAND_LEFT");
        String  right = getImageUrl("LY_COMMAND_RIGHT");
        String  leftOver = getImageUrl("LY_COMMAND_LEFT_OVER");
        String  rightOver = getImageUrl("LY_COMMAND_RIGHT_OVER");
        boolean useButtons = testResource("LY_USE_IMAGE_FOR_COMMAND");
        boolean enabled = isEnabled();
        String  controller = _command.getRootController();

        label = getHtmlString(label);

        LyWebDecoration  deco = getWebViewManager().getDecoration();

        // build the url
        String url = buildUrl(targetWindow);

        pw.addBeginScript("<script type='text/javascript'>window.defaultStatus='';</script>",
            false);

        if (!pw.writeDivOnly())
            pw.append("<div id='command_" + _command.getIdCode() + "'>");
        else
        {
            errorMessage = LyString.replace(errorMessage, "\\n", "\\\\n");
            fileErrorMessage = LyString.replace(fileErrorMessage, "\\n", "\\\\n");
        }
        
        if (useButtons)
        {
            if (testResource("LY_USE_IMAGE_FOR_COMMAND") && (image == null))
                image = getEnvironment().getImageUrl(_command.getLabel(), false, false);
            else if (image != null)
                image = getImageUrl(image);

            if (image == null)
            {
                pw.append("<input type='button' name='");
                pw.append(LyWebConstants.COMMAND_ITEM);
                pw.append(id);
                pw.append("' value='");
            }
            else
            {
                pw.append("<img src='" + image + "' title='");
            }

            pw.append(label);
            pw.append("'");

            // FFT 597
            if (enabled)
            {
                if (image == null)
                    pw.append(" onClick=\"javascript:InvokeCommand('");
                else
                    pw.append(" onmousedown=\"javascript:InvokeCommand('");
                pw.append(id);
                pw.append("','");
                pw.append(url);
                pw.append("','" + (action == LyCommand.Action.CANCEL) + "','false','" + errorMessage
                        + "','" + fileErrorMessage + "','" + controller + "')\"");
            }
            else
                pw.append("disabled='true' ");
            // Not supported by all browsers XXX
            // Has been removed in commands
            pw.append("/>");
        }
        else
        {
            // FFT 819 Compute left image size if possible
            String imageSize = "";
            int[]  size = getImageSize("LY_COMMAND_LEFT", true);

            if (size != null)
            {
                if (size.length == 2)
                    imageSize = "width='" + size[0] + "' height='" + size[1] + "'";
                else
                    imageSize = "height='" + size[0] + "'";
            }
			
			StringBuffer	hrefBuffer = new StringBuffer();
			
			hrefBuffer.append("<a");
			hrefBuffer.append(" href=\"javascript:InvokeCommand('");
			hrefBuffer.append(id);
			hrefBuffer.append("','");
			hrefBuffer.append(url);
			hrefBuffer.append("','" + (action == LyCommand.Action.CANCEL) + "','false','"
					+ errorMessage + "','" + fileErrorMessage + "','" + controller + "')\"");

			// On mouse down, try to give focus to current command so that last edited input looses focus and
			// transmits its new value before validating form.
			hrefBuffer.append(" onMouseDown=\"try {this.focus();} catch (ex) {};\"");
			
			hrefBuffer.append(" onMouseOver=\"rollover(imageLeft");
			hrefBuffer.append(htmlId);
			hrefBuffer.append(",\'");
			hrefBuffer.append(leftOver);
			hrefBuffer.append("\',\'\');rollover(imageRight");
			hrefBuffer.append(htmlId);
			hrefBuffer.append(",\'");
			hrefBuffer.append(rightOver);
			hrefBuffer.append("\',\'\');");
			hrefBuffer.append("document.getElementById('imageBg" + htmlId + "').className='commandButtonOver';");
			hrefBuffer.append("return true\"");

			hrefBuffer.append(" onMouseOut=\"rollout(imageLeft");
			hrefBuffer.append(htmlId);
			hrefBuffer.append(",\'");
			hrefBuffer.append(left);
			hrefBuffer.append("\',\'\');rollout(imageRight");
			hrefBuffer.append(htmlId);
			hrefBuffer.append(",\'");
			hrefBuffer.append(right);
			hrefBuffer.append("\',\'\');");
			hrefBuffer.append("document.getElementById('imageBg" + htmlId + "').className='commandButton';");
			hrefBuffer.append("\">\n");
			

            pw.append("<table border='0' cellspacing='0' cellpadding='0' onmousedown='return false;'>\n");
            // FFT 819 : FORCED right align + size because of MSIE bug
            pw.append("<tr><td align='right'");

            if ((size != null) && (size.length == 2))
                pw.append(" width='" + size[0] + "'");

			pw.append(">");

			if (enabled)
				pw.append(hrefBuffer.toString());

			pw.append("<img src='");
            pw.append(left);
            pw.append("' ");
            pw.append(imageSize);
            pw.append(" name=\"imageLeft");
            pw.append(htmlId);
            pw.append("\" ");
			pw.append("border='0'/>");

			if (enabled)
				pw.append("</a>");

			pw.append("</td>\n");
            pw.append("<td class='commandButton' nowrap='true'");
            pw.append(" id=\"imageBg");
            pw.append(htmlId);
            pw.append("\" >\n");

            boolean addSpace = false;
            
            if (image != null)
            {
                int[] commandImageSize = getImageSize(image);

                image = (enabled ? getImageUrl(image) : getDisabledImageUrl(image));
                pw.append("<table border='0' cellspacing='0' cellpadding='0'>\n");
                pw.append("<tr><td valign='middle'>\n");
                
    			if (enabled)
    				pw.append(hrefBuffer.toString());
                
                pw.append("<img src='" + image + "' title='" + label + "' border='0' align='center'");
                
                if (enabled)
                    pw.append(" class='commandImage'");
                else
                    pw.append(" class='commandImageDisabled'");

                if (commandImageSize != null)
                {
                    if (commandImageSize.length == 2)
                        pw.append(" width='" + commandImageSize[0] + "' height='" + commandImageSize[1] + "'");
                    else
                        pw.append(" height='" + commandImageSize[0] + "'");
                }

                pw.append("/>");
                
                if (enabled)
                    pw.append("</a>");

                pw.append("</td><td valign='middle'>\n");
                addSpace = true;
            }
            
			if (enabled)
				pw.append(hrefBuffer.toString());
            
            pw.append("<font class='" + deco.getClassName(_command, pw) + "'>");
            
            if (addSpace)
                pw.append("&nbsp;");
            
            pw.append(label);
            pw.append("</font>");
            
            if (enabled)
                pw.append("</a>");

            if (image != null)
            {
            	pw.append("</td></tr>\n");
            	pw.append("</table>\n");
            }

            pw.append("</td>\n");

            // FFT 819 Compute right image size if possible
            imageSize = "";
            size = getImageSize("LY_COMMAND_RIGHT", true);

            if (size != null)
            {
                if (size.length == 2)
                    imageSize = "width='" + size[0] + "' height='" + size[1] + "'";
                else
                    imageSize = "height='" + size[0] + "'";
            }
            pw.append("<td align='left' ");
            if ((size != null) && (size.length == 2))
                pw.append(" width='" + size[0] + "'");
			pw.append(">");

			if (enabled)
				pw.append(hrefBuffer.toString());

			pw.append("<img src='");
            pw.append(right);
            pw.append("' ");
            pw.append(imageSize);
            pw.append(" name=\"imageRight");
            pw.append(htmlId);
            pw.append("\" ");
			pw.append("border='0'/>");

			if (enabled)
				pw.append("</a>");

			pw.append("</td>\n");
            pw.append("</tr>\n");
            pw.append("</table>\n");
        }

        if (!pw.writeDivOnly())
            pw.append("</div>");
        
        pw.endComponent(this);

        // get the script to define the base url
        LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();

        StringBuffer        script = new StringBuffer();

        script.append("<script type='text/javascript'>\n//<![CDATA[\nvar _rootUrl='");
        script.append(viewManager.getRootURL());
        script.append("';\n//]]>\n</script>");

        pw.addBeginScript(script.toString(), false);
    } // end method writeHtml

    /**
     * (non-Javadoc).
     *
     * @see leon.view.setinterface.LySetLabelInterface#setLabel(java.lang.String)
     */
    public void setLabel(String label)
    {
        LyStrutsViewManager  manager = (LyStrutsViewManager)getWebViewManager();
        
        manager.addReloadDiv(_lastFrameName, "command_" + _command.getIdCode(), this);
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.setinterface.LySetEnabledInterface#setEnabled(boolean)
     */
    public void setEnabled(boolean status)
    {
        _enabled = status;
        
        LyStrutsViewManager  manager = (LyStrutsViewManager)getWebViewManager();
        
        manager.addReloadDiv(_lastFrameName, "command_" + _command.getIdCode(), this);
    }

    /**
     * Builds the command URL.
     *
     * @param  targetWindow The target window in the frameset.
     * @return The url of the command button.
     */
    protected String buildUrl(String targetWindow)
    {
        // get the view manager
        LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();

        // get the action URL
        String              actionUrl = viewManager.getActionURL(LyStrutsConstants.PATH_ACTION_COMMAND);

        return actionUrl.toString();
    }

	/**
	 * Indicates whether this command is enabled or not.
	 * @return
	 */
	public boolean isEnabled()
	{
		return _enabled && _command.isEnabled();
	}
} // end class LyStrutsCommand
