/*
 * LyStrutsCommands.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Commands object (as described in the dtd) for web viewer.
 *
 * <p><b>COMMANDS: Command Bar</b><br>
 * A command bar allows to display a set of command buttons (Validate, Cancel, Close, etc.). This
 * bar is generally located at the bottom of a page or window.</p>
 *
 * @version $Id: LyStrutsCommands.java 40708 2012-09-07 08:32:47Z jlsaouli $
 * @author  Lyria S.A.
 */
public class LyStrutsCommands extends LyWebObject
    implements LyGraphicalCommands
{
	public static final String __VERSION =
		"$Id: LyStrutsCommands.java 40708 2012-09-07 08:32:47Z jlsaouli $";

	/** The commands generic component. */
    protected LyCommands       _commands;
    
	/**
	 * Constructor.
	 *
	 * @param command The generic parent component.
	 */
	public LyStrutsCommands(LyCommands commands)
	{
        super(commands);
        _commands = commands;

        init();
	}
	
	/**
     * Initialization method. Sets the commands font defined in the resources
     *
     * @see LyWebObject#getFontString(String, String)
     */
    public void init()
    {
    }

    /**
     * Gets the html horizontal alignment string corresponding to the Commands generic component
     * alignment.
     *
     * @param  alignCode the Commands generic component halign code.
     * @return the HTML horizontal alignment string.
     */
    protected static String getHtmlHAligment(LyCommands.Align alignCode)
    {
        if (alignCode == LyCommands.Align.LEFT)
            return "left";
        else if (alignCode == LyCommands.Align.RIGHT)
            return "right";
        else if (alignCode == LyCommands.Align.CENTER)
            return "center";
        else
            return null;
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
     */
    public void writeHtml(LyWebPageWriter pw)
    {
        // Don't print commands
        if (pw.isPrintMode())
            return;

        pw.startComponent(this);
        
        String controllerId = _commands.getRootController();

        // Add needed script
        pw.addBeginScript("<script type='text/javascript'>var _controllerId='" + controllerId
            + "';</script>", false);

        String bgColor = getHexColor("LY_FORM_BACKGROUND");

        if (bgColor != null)
            // Set the background color
            pw.setBackgroundColor(bgColor, pw.getActiveName());

        int             i, n = _commands.getCommandCount();
        LyCommand       command;
        LyStrutsCommand webCommand;
        String          align = getHtmlHAligment(_commands.getAlign());

        // FFT00146 : use one single form for a same frame
        pw.setFormId(controllerId);

        // Write content
        LyWebDecoration deco = getWebViewManager().getDecoration();

        deco.writePageDecoration(pw, LyWebDecoration.WINDOW_BEGIN);
        deco.writePageDecoration(pw, LyWebDecoration.WINDOW_TOP);
        deco.writePageDecoration(pw, LyWebDecoration.WINDOW_LEFT);
        
        LyWebViewManager viewManager = getWebViewManager();
        
        if (viewManager.usePopupView()
                && LyWebConstants.POPUP_VIEW.equals(pw.getWindowName())
                && (viewManager.getBrowser() == LyWebViewManager.BROWSER_FIREFOX))
        {
            // With Firefox, we must give the command bar height otherwise command buttons
            int[]   sizes = getEnvironment().getImageSize("LY_COMMAND_LEFT");
            int     imageHeight = (sizes != null ? sizes[1] : 16);

            pw.append("<div style='height:" + (imageHeight + 15) + "px;'>");
        }
        else
            pw.append("<div>");

        pw.append("<table border='0' align='center' cellspacing='0' cellpadding='0' ");
        pw.append("class='" + deco.getClassName(_commands, pw) + "'");
        pw.append(" width='100%' height='100%'>");
        pw.append("<tr><td>\n");
        pw.append("<table border='0' align='" + align + "' cellspacing='1' cellpadding='1'><tr>\n");
        
        for (i = 0; i < n; i++)
        {
            command = _commands.getCommand(i);

            pw.append("<td>");

            if ((command == null)
                || (command.getId() == LyComponent.SEP_ID))
            {
                pw.append("<img src='");
                pw.append(getImageUrl("LY_VERTICAL_SEPARATOR"));
                pw.append("'/></td>\n");
            }
            else
            {
                // if command is enable it will write something, else it will write
                // nothing
                // But it will allow it to initialize itself
                webCommand = (LyStrutsCommand)command.getInternalView();
                webCommand.writeHtml(pw);
            }

            pw.append("</td>\n");
        }

        pw.append("</tr>\n</table>\n");
        pw.append("</td></tr></table>");
        pw.append("</div>");

        deco.writePageDecoration(pw, LyWebDecoration.WINDOW_RIGHT);
        deco.writePageDecoration(pw, LyWebDecoration.WINDOW_BOTTOM);
        deco.writePageDecoration(pw, LyWebDecoration.WINDOW_END);

        pw.endComponent(this);
    } // end method writeHtml
} // end class LyStrutsCommands
