/*
 * LyStrutsDate.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import java.awt.*;
import java.util.*;
import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Date object (as described in the dtd) for web viewer.
 *
 * <p><b>DATE: Input Field of Date Type</b><br>
 * An input field of Date type allows to display a graphical component allowing to input or to
 * consult date and/or hour. It does not comprise any element. It is possible to specify the type of
 * the date as well as the display format.</p>
 *
 * @version $Id: LyStrutsDate.java 24563 2010-03-04 08:15:19Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.LyDate
 */
public class LyStrutsDate extends LyStrutsFieldItem
	implements LyGraphicalDate
{
	public static final String __VERSION =
		"$Id: LyStrutsDate.java 24563 2010-03-04 08:15:19Z jlsaouli $";

	/** Image for opening date selector button. */
	protected String		   _openSelectorButton;

	/** Disabled image for opening date selector button. */
	protected String		   _disabledOpenSelectorButton;

	/** Image for year decreasing. */
	protected String		   _yearDecreaseButton;

	/** Image for month decreasing. */
	protected String		   _monthDecreaseButton;

	/** Image for month increasing. */
	protected String		   _monthIncreaseButton;

	/** Image for year increasing. */
	protected String		   _yearIncreaseButton;

	/** The date generic component. */
	protected LyDate		   _date;

	/**
	 * Class constructor of a Date object for the struts target.
	 *
	 * @param date the generic Date defined for every type of view targets.
	 */
	public LyStrutsDate(LyDate date)
	{
		super(date);
		_date = date;

		initialize();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.struts.LyStrutsFieldItem#setLabel(java.lang.String)
	 */
	@Override
  public void setLabel(String label)
	{
		LyComponent parent = _date.getParent();

		if (parent instanceof LyFormField)
		{
			LyFormField		 formField = (LyFormField)parent;
			LyStrutsFormField webField = (LyStrutsFormField)formField.getInternalView();

			webField.refreshDivContent();
		}
		else
			refresh();
	} // end method setLabel

	/**
	 * Gets the date labels.
	 *
	 * @return an array of String with initial value at index 0, date label at index 1 and time
	 *         label at index 2.
	 */
	protected String[] getDateLabels()
	{
		String   label = getHtmlString(_date.getLabel());
		String[] labels = LyString.stringToList(label, ' ');
		String   dateLabel = "";
		String   timeLabel = "";
		String   amPmLabel = "";
		LyDate.Type    type = _date.getType();
		boolean  addDate = (type == LyDate.Type.DATE_TIME) || (type == LyDate.Type.DATE_HOUR_MIN)
				|| (type == LyDate.Type.DATE);
		boolean  addTime = (type == LyDate.Type.DATE_TIME) || (type == LyDate.Type.TIME)
				|| (type == LyDate.Type.DATE_HOUR_MIN) || (type == LyDate.Type.HOUR_MIN);

		// Both date and time
		if (addDate && addTime)
		{
			dateLabel = labels[0];

			if (labels.length > 2)
			{
				timeLabel = labels[1];
				amPmLabel = labels[2];
			}
			else if (labels.length > 1)
				timeLabel = labels[1];
		}
		// Only date
		else if (addDate)
			dateLabel = labels[0];
		// Only time
		else if (addTime)
		{
			if (labels.length > 2)
			{
				timeLabel = labels[1];
				amPmLabel = labels[2];
			}
			else if (labels.length > 1)
				timeLabel = labels[1];
			else
				timeLabel = labels[0];
		}

		String initialValue = dateLabel;

		if ((dateLabel.length() > 0)
				&& (timeLabel.length() > 0))
			initialValue += " ";

		initialValue += timeLabel;

		if ((timeLabel.length() != 0) && (amPmLabel.length() != 0))
			initialValue += " " + amPmLabel;

		return new String[] { initialValue, dateLabel, timeLabel, amPmLabel};
	} // end method getDateLabels

	/**
	 * Initialization method.
	 */
	public void initialize()
	{
		_openSelectorButton = getImageUrl("LY_ARROW_DOWN_BIG_BUTTON_SPECIAL_DATE");
		_disabledOpenSelectorButton = getDisabledImageUrl("LY_ARROW_DOWN_BIG_BUTTON_SPECIAL_DATE");
		_yearDecreaseButton = getImageUrl("LY_ARROW_YEAR_LEFT");
		_monthDecreaseButton = getImageUrl("LY_ARROW_MONTH_LEFT");
		_monthIncreaseButton = getImageUrl("LY_ARROW_MONTH_RIGHT");
		_yearIncreaseButton = getImageUrl("LY_ARROW_YEAR_RIGHT");
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	    pw.startComponent(this);

		updateRefreshInfo(pw);

		String  label = getHtmlString(_date.getLabel());
		boolean isEditable = isEditable();
		boolean canDisabled = canDisabled();
		String  style = "";

    if(!_date.isDefaultFont() || !_date.isDefaultFontColor()) {
      style += getFontString(_date.getFont(),_date.getFontColor());
    }
    if(!_date.isDefaultColor()) {
      style += " background-color:"+getHexColor(_date.getColor())+";";
      style += " background-image:none;";
    }

		// If not editable and control can be disabled, just write text
		if (!isEditable && !canDisabled)
		{
			if (label != null)
				pw.append("<font class='label'>" + label + "</font>\n");

			pw.endComponent(this);
			return;
		}

		// global (editable or not)

		// First, add the needed scripts to the page
		pw.addBeginScript("<script type='text/javascript'>window.defaultStatus='';</script>",
			false);

		// Retrieve the two labels (date and time)
		LyDate.Type   type = _date.getType();
		boolean addDate = (type == LyDate.Type.DATE_TIME) || (type == LyDate.Type.DATE_HOUR_MIN)
				|| (type == LyDate.Type.DATE);
		boolean addTime = (type == LyDate.Type.DATE_TIME) || (type == LyDate.Type.TIME)
				|| (type == LyDate.Type.DATE_HOUR_MIN) || (type == LyDate.Type.HOUR_MIN);

		String  errorMessage = "";

		if (isEditable)
			errorMessage = getMessage("LY_ER_MATCH_FORMAT", new String[] { "%1", getCaption() });

		// Add a hidden input for the date that will be sent as the new value.
		// This input must be updated each time the date or the time are
		// modified.
		String   name = getId();
		String[] labels = getDateLabels();
		String   initialValue = labels[0];
		String   dateLabel = labels[1];
		String   timeLabel = labels[2];
		String   amPmLabel = labels[3];

		pw.append("<input type='hidden' name='" + name + "' value='" + initialValue + "' />\n");

		// NeedCheck on date fields : add a hidden input for needCheck
		// (used by the UpdateDate() method of LyDateScript).
		boolean needCheck = needCheck();

		if (needCheck && isEditable)
			pw.append("<input type='hidden' name='_checkDate" + name + "' value='"
				+ getFormField().getController() + "'/>\n");

		// Add the date selector
		String		  disabledColor = getHexColor("LY_DISABLED_BACKGROUND");
		LyEnvironment environment = getEnvironment();
		Locale		  locale = environment.getDateLocale();
		String		  formName = "forms[0]";//getFormName(pw);
		String		  nameSubst = LyString.replace(name, "\\", "\\\\");
		boolean		  isOptional = isOptional();
		int			  width = -1;
		int			  height = -1;

		// If a constraint is set on the date, use the nCols and nRows values to compute
		// date size in pixels.
		if ((_date.getParent() instanceof LyFormField)
				&& (((LyFormField)(_date.getParent())).getConstraint() != null))
		{
			LyConstraint	constraint = ((LyFormField)(_date.getParent())).getConstraint();

			if (constraint != null)
			{
				if (constraint.getNCols() > 0)
				{
					width = constraint.getNCols();

					if (_date.isShowArrows() && addDate)
						width -= 20;
					else if (!_date.isShowArrows() && addDate && addTime)
						width -= 5;

					if (addDate && addTime)
						width = width / 2;
				}

				if (constraint.getNRows() > 0)
					height = constraint.getNRows();
			}
		}

		String	onMouseOverFunctions = _date.getEventFunctions(LyComponent.EventType.MOUSE_ENTERED, ';');
		String	onMouseOutFunctions = _date.getEventFunctions(LyComponent.EventType.MOUSE_EXITED, ';');

		if (addDate)
		{
			String	dateStyle = getDateStyle();

			String format = environment.getStringDateFormat(LyEnvironment.TYPE_DATE);
			String parseFormat = LyString.replace(format, "MM", "[1-12]");

			parseFormat = LyString.replace(parseFormat, "dd", "[1-31]");
			parseFormat = LyString.replace(parseFormat, "yyyy", "[1-9999]");

			pw.append("<table border='0' cellpadding='0' cellspacing='0'>\n");
			pw.append("<tr valign='middle'  align='center'>\n<td>\n");
			pw.append("<input");
			pw.append(" name='" + name + "_date'" + " size='10'");
			pw.append(" dateFormat='" + format + "'");

			if ((width > 0) || (height > 0))
			{
				pw.append(" style='");

				if (width > 0)
					pw.append("width:" + width + "px;");

				if (height > 0)
					pw.append("height:" + height + "px;");

				pw.append("'");
			}

			if (onMouseOverFunctions.length() > 0)
				pw.append(" onmouseover=\"" + onMouseOverFunctions + "\"");

			if (onMouseOutFunctions.length() > 0)
				pw.append(" onmouseout=\"" + onMouseOutFunctions + "\"");

			pw.append(" value='" + dateLabel + "'");

			if (isEditable)
			{
				// store the old value on focus event
				pw.append(" onfocus=\"StorePreviousValue(this)\"");

				// check the date on the blur event
				pw.append(" onBlur=\"javascript:UpdateDateOnBlur(window.document." + formName + ",'"
					+ nameSubst + "','" + nameSubst + "_date','" + nameSubst + "_time','"
					+ type + "','" + locale + "','" + isOptional + "','"
					+ parseFormat + "','" + errorMessage + "',true)\" ");

				// check the date on the key up event
				pw.append(" onKeyUp=\"javascript:UpdateDateOnBlur(window.document." + formName + ",'"
					+ nameSubst + "','" + nameSubst + "_date','" + nameSubst + "_time','"
					+ type + "','" + locale + "','" + isOptional + "','"
					+ parseFormat + "',null,false)\" ");
			}
			else
			{
				pw.append(" disabled='true'");

				if (disabledColor != null)
				{
					if (dateStyle == null)
						dateStyle = "";

					dateStyle += "background-color:" + disabledColor + ";";
				}
			}

			if (style.trim().length() > 0)
				dateStyle += style;

			if (dateStyle != null)
				pw.append(" style='" + dateStyle + "'");

			pw.append(" maxlength='10'/>\n");
			pw.append("</td>\n<td>\n");

			if (_date.isShowArrows())
			{
				if (isEditable)
				{
					String scriptRoot = getWebViewManager().getScriptsDir();

					pw.append("<img src=\"" + _openSelectorButton + "\" border='0'  align='middle'");

					if (onMouseOverFunctions.length() > 0)
						pw.append(" onmouseover=\"" + onMouseOverFunctions + "\"");

					if (onMouseOutFunctions.length() > 0)
						pw.append(" onmouseout=\"" + onMouseOutFunctions + "\"");

					pw.append(" onClick=\"javascript:ShowDateSelector('" + nameSubst + "_date_selector" + _date.getIdCode() + "', '"
					+ scriptRoot
					+ "',window.document." + formName + ",'" + nameSubst + "','" + nameSubst
					+ "_date','" + nameSubst + "_time" + "','" + locale + "','"
					+ _yearDecreaseButton + "','" + _monthDecreaseButton + "','"
					+ _monthIncreaseButton + "','" + _yearIncreaseButton + "','" + type
					+ "');\"/>");
			}
			else
				pw.append("<img src=\"" + _disabledOpenSelectorButton
						+ "\" border='0'  align='middle' class='disabledTool'/>");
			}

			pw.append("</td><td nowrap='yes'>\n");
		}

		// Add the time selector
		if (addTime)
		{
			boolean	isTimeHourMin = (_date.getType()== LyDate.Type.DATE_HOUR_MIN) || (_date.getType()== LyDate.Type.HOUR_MIN);
			String	timeStringFormat = environment.getStringDateFormat(isTimeHourMin ? LyEnvironment.TYPE_HOUR_MIN : LyEnvironment.TYPE_TIME);
			boolean	addAmPm = false;

			if (timeStringFormat.endsWith(" a"))
				addAmPm = true;

			if (addAmPm)
			{
				pw.append("<table border='0' cellpadding='0' cellspacing='0'>\n");
				pw.append("<tr>\n<td valign='middle' style='padding-left:5px;'>\n");
			}

			String timeStyle = getTimeStyle();
			int    size;

			if ((type == LyDate.Type.DATE_TIME) || (type == LyDate.Type.TIME))
				size = 8;
			else if (type == LyDate.Type.DATE_HOUR_MIN)
				size = 5;
			else if (type == LyDate.Type.HOUR_MIN)
				size = 5;
			else
				size = 2;

			String	timeParseFormat = (addAmPm ? "[1-12]:[0-59]" : "[0-23]:[0-59]");

			if (!isTimeHourMin)
				timeParseFormat += ":[0-59]";

			pw.append("<input");
			pw.append(" name='" + name + "_time' size='" + size + "'");

			if ((width > 0) || (height > 0))
			{
				pw.append(" style='");

				if (width > 0)
					pw.append("width:" + width + "px;");

				if (height > 0)
					pw.append("height:" + height + "px;");

				pw.append("'");
			}

			if (onMouseOverFunctions.length() > 0)
				pw.append(" onmouseover=\"" + onMouseOverFunctions + "\"");

			if (onMouseOutFunctions.length() > 0)
				pw.append(" onmouseout=\"" + onMouseOutFunctions + "\"");

			pw.append(" value='" + timeLabel + "'");

			if (isEditable)
			{
				// store the old value on focus event
				pw.append(" onfocus=\"StorePreviousValue(this)\"");
				pw.append(" onBlur=\"javascript:UpdateTimeOnBlur(window.document." + formName
					+ ",'" + nameSubst + "','" + nameSubst + "_date','" + nameSubst
					+ "_time','" + type + "','" + locale + "','" + isOptional
					+ "','" + timeParseFormat + "','" + errorMessage + "',true)\" ");
				pw.append(" onKeyUp=\"javascript:UpdateTimeOnBlur(window.document." + formName
					+ ",'" + nameSubst + "','" + nameSubst + "_date','" + nameSubst
					+ "_time','" + type + "','" + locale + "','" + isOptional
					+ "','" + timeParseFormat + "',null,false)\" ");
			}
			else
			{
				pw.append(" disabled='true'");

				if (disabledColor != null)
				{
					if (timeStyle == null)
						timeStyle = "";

					timeStyle += "background-color:" + disabledColor + ";";
				}
			}

			if (timeStyle != null)
				style += timeStyle;

			if (style.trim().length() > 0)
				pw.append(" style='" + style + "'");

			pw.append(" maxlength='" + size + "'/>");

			if (addAmPm)
			{
				pw.append("</td><td valign='middle'>");
				pw.append("");
				pw.append("<select name='" + name + "_time_ampm'");
				pw.append(" onChange=\"javascript:UpdateTimeOnBlur(window.document." + formName
						+ ",'" + nameSubst + "','" + nameSubst + "_date','" + nameSubst
						+ "_time','" + type + "','" + locale + "','" + isOptional
						+ "','" + timeParseFormat + "','" + errorMessage + "',true)\" ");
				pw.append(">\n");
				pw.append("<option id='" + name + "AM'"  + ("AM".equals(amPmLabel) ? " selected='true'" : "") + " value='AM'>AM</option>\n");
				pw.append("<option id='" + name + "PM'" + ("PM".equals(amPmLabel) ? " selected='true'" : "") + " value='PM'>PM</option>\n");
				pw.append("</select>\n");
				pw.append("</td></tr></table>\n");
			}
		}

		if (addDate)
		{
			if (!addTime && (pw.getWindowName() == LyWebConstants.POPUP_VIEW))
			{
				// If no time input is added, generate a hidden div used to leave enough space
				// for calendar to be displayed.
				pw.append("</td><td>\n");
				pw.append("<div style='width:70px;height:10px;visibility:hidden;'></div>");
			}

			pw.append("</td></tr>\n");
			pw.append("<tr><td colspan='4'>\n");

			pw.append("\n<iframe src='");
			pw.append(getWebViewManager().getScriptsDir() + "/blank.html");
			pw.append("' id='" + name + "_date_selector" + _date.getIdCode() + "' ");
			pw.append("name='" + name
				+ "_date_selector" + _date.getIdCode() + "' style='position:absolute;width:155;height:155;visibility:hidden;z-index:50;' ");
			pw.append("scrolling='no' frameborder='0'>");
			pw.append("</iframe>\n");
			pw.append("</td></tr>\n");
			pw.append("</table>\n");
		}

		pw.endComponent(this);
	} // end method writeHtml

	/**
	 * Returns the CSS style to be used for date input.
	 */
	@SuppressWarnings("deprecation")
	protected String getDateStyle()
	{
		Font			font = (Font)getEnvironment().getFont(_date.getFont());
		Toolkit			toolkit = Toolkit.getDefaultToolkit();
		FontMetrics		fm = toolkit.getFontMetrics(font);
		float			charWidth = ((float)fm.stringWidth("0123456789")) / 10;

		return "width:" + ((int)(charWidth * 10) + 4) + "px;";
	}

	/**
	 * Returns the CSS style to be used for time input.
	 */
	protected String getTimeStyle()
	{
		return null;
	}
} // end class LyStrutsDate
