/*
 * LyStrutsDialog.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts;

import java.io.*;
import java.util.*;

import javax.servlet.http.*;

import leon.app.*;
import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Dialog object (as described in the dtd) for web viewer.
 *
 * <p><b>DIALOG: Dialog Window</b><br>
 * A dialog window is, generally, linked to a main window of an application. Unlike the main window,
 * it contains neither a menu, nor a message bar. Its display can be modal (blocking) or not. This
 * type of window is generally used to allow a user to interact with an application: input request,
 * presentation of the results, warning messages, questions, etc.</p>
 *
 * @version $Id: LyStrutsDialog.java 35296 2012-04-19 09:44:54Z jlsaouli $
 * @author  Lyria S.A.
 */
public class LyStrutsDialog extends LyStrutsView
    implements LyGraphicalDialog
{
	public static final String __VERSION =
		"$Id: LyStrutsDialog.java 35296 2012-04-19 09:44:54Z jlsaouli $";

	/** The dialog generic component. */
    protected LyDialog         _dialog;

    /** Indicates if the dialog is open in a pop up or not. */
    protected boolean          _inPopUp = false;
    
	/**
	 * Constructor.
	 *
	 * @param dialog the dialog component
	 */
	public LyStrutsDialog(LyDialog dialog)
	{
        super(dialog);
        _dialog = dialog;

        init();
        setContent();
	}
	
	public void preparePage(String frame, String targetWindow)
	{	       
		if (targetWindow != null)
			setWindowName(targetWindow); // keep the name of the window where

		// Add up frame if needed
		LyWebRequestInfo		info = LyStrutsServlet.getWebRequestInfo();
		HttpServletRequest		request = (info == null ? null : info.getRequest());
		LySession				session = (request == null ? null : (LySession)request.getAttribute(LyWebConstants.RA_LY_SESSION));
		boolean					addUpFrame = (LyWebConstants.TARGET_FRAME.equals(frame))
			&& LyWebPageWriter.displayUpPage(getEnvironment()) && (session != null);

		if ((frame == null) || frame.equals(LyWebConstants.TARGET_FRAME))
			frame = MAIN_BUFFER;

		clearBuffer(frame);
		clearBeginScripts(frame);
		clearEndScripts(frame);
		clearCssScripts(frame);
		setActiveBuffer(frame);

	    startComponent(this);
	      
		updateRefreshInfo(this);
		
		if (addUpFrame)
		{
			String					target = (request == null ? null : (String)request.getParameter(LyWebConstants.TARGET_PARAM));
			HashMap<String, String>	parameters = new HashMap<String, String>();
			LyStrutsViewManager		viewManager = (LyStrutsViewManager)getViewManager();
			
			if (target != null)
				parameters.put(LyWebConstants.TARGET_PARAM, target);

			String upFrame = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_UPFRAME, parameters, true);
			
			append("<table width='100%' height='100%' border='0' cellspacing='0' cellpadding='0' valign='top'>\n");
			append("<tr>\n");
			append("<td vAlign='top'>\n");
			append("<iframe src='" + upFrame + "' id='up_frame' name='up_frame' class='upFrame' frameborder='0'></iframe>\n");
			append("</td>\n");
			append("</tr>\n");
			append("<tr>\n");
			append("<td valign='top' align='left'>\n");
		}

		if (!LyWebConstants.POPUP_VIEW.equals(getWindowName()))
		{
			append("<table border='0' cellspacing='0' cellpadding='0' align='center' valign='top' class='mainDialog'>\n");
			append("<tr>\n");
			append("<td vAlign='top'>\n");
		}

		writeArea(this);

		if (!LyWebConstants.POPUP_VIEW.equals(getWindowName()))
		{
			append("</td>\n");
			append("</tr>\n");
			append("</table>\n");
		}
		
		if (addUpFrame)
		{
			append("</td>\n");
			append("</tr>\n");
			append("</table>\n");
		}
		
	    endComponent(this);
	}
	
	public void writeBeginPage(Writer pw, String frame)
		throws IOException
	{
		LyWebViewManager	manager = getWebViewManager();
		LyEnvironment		env = getEnvironment();

		pw.write("<html><head>\n");
		pw.write("<meta name=\"AUTHOR\" content=\"Lyria\"/>\n");
		pw.write("<meta name=\"GENERATOR\" content=\"Leonardi\"/>\n");
		pw.write("<meta name=\"Content-Type\" content=\"text/html;charset=" + LyString.DEFAULT_CHARSET + "\"/>\n");
		pw.write("<title>");
		pw.write((_title == null) ? "" : translate(_title));
		pw.write("</title>\n");
		writeCssScripts(pw, frame);
		pw.write("</head>\n");

		if ((frame == null) || frame.equals(LyWebConstants.TARGET_FRAME))
			frame = MAIN_BUFFER;

		pw.write("<body ");
		
		String	bodyClass = getBodyClassString();
		
		if (bodyClass != null)
			pw.write(" " + bodyClass);

		pw.write(" topmargin='0' leftmargin='0' style='width:100%;' " + getBodyScript()
			+ ">\n");
		
		String	scripts = manager.generateScriptsInclusion();
		
		pw.write(scripts);
		pw.write("<script type='text/javascript'>var pageWriterId='" + getId() + "';</script>\n");

		if (!env.testEnv("LY_REPLAY")
				&& ((getAttachmentHandler(frame) == null) || (manager instanceof LyStrutsViewManager))
				&& (manager.hasProperty("DYNAMIC")))
			pw.write("<script type='text/javascript'>var AJAX_MODE='" + (FORCE_AJAX ? "force" : true) + "';</script>\n");
		else
			pw.write("<script type='text/javascript'>var AJAX_MODE='false';</script>\n");

		pw.write("<script type='text/javascript'>var isInPopUp='" + isInPopUp() + "';</script>\n");

		if (manager instanceof LyStrutsViewManager)
		{
			String	actionUrl = ((LyStrutsViewManager)manager).getActionURL(LyStrutsConstants.PATH_ACTION_GRAPHICEVENT);

			pw.write("<script type='text/javascript'>var STRUTS_EVENT_URL='" + actionUrl + "';</script>\n");
		}
	}
	
	protected String getBodyScript()
	{
		if (LyWebConstants.POPUP_VIEW.equals(getWindowName()))
		{
			String	title = _dialog.getLabel();
			boolean	modal = false;
			int		width = Math.max(_dialog.getWidth(), _dialog.getMinWidth());
			int		height = Math.max(_dialog.getHeight(), _dialog.getMinHeight());
			
			// Check if one of the parent is modal
			LyView			parentView = _dialog;
			LyViewManager	manager = getViewManager();

			while ((parentView != null)
					&& (parentView instanceof LyDialog))
			{
				if (((LyDialog)parentView).isModal())
				{
					modal = true;
					break;
				}

				parentView = manager.getView(((LyDialog)parentView).getParentId());
			}
			
			return "onload='setPopupDimension(" + width + "," + height + ");checkSizePopupView(null, true, "
				+ !_dialog.isDecorated() + ", \"" + getHtmlEntitiesString(translate(title))
				+ "\", " + modal + ", " + (_dialog.isModal() && _dialog.isDecorated())
				+ ");ResetFormFocusImmediatly(false);'";
		}
		else
			return "onload=\"checkSizeMainView('true', 'false');ResetFormFocusImmediatly(false);\" onResize=\"checkSizeMainView('true', 'true');\"";
	}

	public void writePageContent(Writer pw, String frame) throws IOException
	{
		// Generate the first table around the form otherwise a blank line may appear with Firefox
		pw.write("<table width='100%' border='0' cellspacing='0' cellpadding='0' valign='top'>\n");
		pw.write("<tr height='100%'>\n");
		pw.write("<td height='100%' vAlign='top'>\n");

		// Use one single form for a same frame
		pw.write(getOpenFormContent(frame, true));

		if (frame == null)
			frame = MAIN_BUFFER;
		
		StringBuffer buffer = getBuffer(frame);

		pw.write(buffer.toString());
		pw.write("\n");
		
		pw.write("<!-- popup menu view is hidden but ready -->\n");
		pw.write("<div id='");
		pw.write(LyWebConstants.POPUP_MENU + _dialog.getIdCode());
		pw.write("' name='");
		pw.write(LyWebConstants.POPUP_MENU);
		pw.write(_dialog.getIdCode());
		pw.write("' style='z-index:10000;height:10;width:10;position:absolute;left:10px;top:10px;visibility:hidden;'>");
		pw.write("</div>");
		
		writeSplitterDivs(pw, frame);
		
		pw.write("</form>\n");
		
		pw.write("</td>\n");
		pw.write("</tr>\n");
		pw.write("</table>\n");
	}
	
	/**
	 * Writes the divs needed for splitter movements.
	 * @param pw
	 * @param frame
	 * @throws IOException
	 */
	protected void writeSplitterDivs(Writer pw, String frame) throws IOException
	{
		// Write hidden div used to move the splitter bar
		String		 image = getImageUrl("LY_SPLITTER_MOVEMENT");

		pw.write("<div id='hidden_splitter' style='width:5;height:5;top:5;left:5;z-index:100;background-color:#000000;filter:alpha(opacity=0);opacity:0;-moz-opacity:0;");
		pw.write("display:none;position:absolute;'></div>");

		// Write moving div used to move the vertical splitter bar
		pw.write("<div id='moving_splitterv' style='cursor:s-resize;width:5;height:8;top:5;left:5;z-index:110;");
		pw.write("display:none;position:absolute;'>");
		pw.write("<table height='100%' width='100%' border='0' cellpadding='0' cellspacing='0'><tr><td valign='middle'>");
		pw.write("<table border='0' width='100%' cellpadding='0' cellspacing='0'>");
		pw.write("<tr height='3'><td background='");
		pw.write(image);
		pw.write("'></td></tr></table>");
		pw.write("</td></tr></table>");
		pw.write("</div>\n");

		// Write moving div used to move the horizontal splitter bar
		pw.write("<div id='moving_splitterh' style='cursor:w-resize;width:8;height:5;top:5;left:5;z-index:110;");
		pw.write("display:none;position:absolute;'>");
		pw.write("<table align='center' border='0' width='100%' height='100%' cellpadding='0' cellspacing='0'>");
		pw.write("<tr><td align='center'><table border='0' height='100%' cellpadding='0' cellspacing='0'><tr><td background='");
		pw.write(image);
		pw.write("' width='3'>&nbsp;</td></tr></table></td></tr></table>");
		pw.write("</div>\n");
	}
	
	public void writeEndPage(Writer pw, String frame)
		throws IOException
	{
		if (frame == LyWebConstants.TARGET_FRAME)
		{
			LyWebViewManager	viewManager = getWebViewManager();
			String				message = viewManager.getMessage();
			
			if (message != null)
				pw.write(message);
		}
		
		pw.write("</body>\n");
		pw.write("</html>\n");
	}

	/**
	 * Method overriden to return the URL to the "Command" struts action path.
	 */
	public String getActionUrl()
	{
		// get the view manager
		LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();

		// build the URL for the Leonardi struts command action
		String			    url = viewManager.getActionAbsoluteUrl(
				LyStrutsConstants.PATH_ACTION_COMMAND, null, true);

		return url;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.struts.LyStrutsDialog#getPopupBaseUrl()
	 */
	protected String getPopupBaseUrl()
	{
		// get the parameters
		Map<String, String> parameters = new HashMap<String, String>(2);

		parameters.put(LyWebConstants.SOURCE, LyWebConstants.POPUP_VIEW);
		parameters.put(LyWebConstants.RELOAD, String.valueOf(System.currentTimeMillis()));

		// get the view manager
		LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();

		// build the URL for the Leonardi struts page action
		String			    url = viewManager.getActionAbsoluteUrl(
				LyStrutsConstants.PATH_ACTION_PAGE, parameters, false);

		return url;
	}
	
	   /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPageWriter#addBeginScript(java.lang.String,
     *      boolean)
     */
    public void addBeginScript(String text, boolean isFile)
    {
        if (text == null)
            return;

        LyArea      area = _dialog.getArea();
        LyStrutsArea webArea = (LyStrutsArea)(area.getInternalView());
        boolean     isFrameSet = webArea.isFrameSet();

        if (isFrameSet)
        {
            super.addBeginScript(text, isFile);
            return;
        }

        // Retrieve the area main frame
        String frame = webArea.getMainFrameName();

        webArea.setActiveBuffer(frame);
        webArea.addBeginScript(text, isFile);
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPageWriter#addEndScript(java.lang.String,
     *      boolean)
     */
    public void addEndScript(String text, boolean isFile)
    {
        if (text == null)
            return;

        LyArea      area = _dialog.getArea();
        LyStrutsArea webArea = (LyStrutsArea)(area.getInternalView());
        boolean     isFrameSet = webArea.isFrameSet();

        if (isFrameSet)
        {
            super.addEndScript(text, isFile);
            return;
        }

        // Retrieve the area main frame
        String frame = webArea.getMainFrameName();

        webArea.setActiveBuffer(frame);
        webArea.addEndScript(text, isFile);
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPageWriter#getRedirect(java.lang.String)
     */
    public String getRedirect(String frame)
    {
        LyArea area = _dialog.getArea();

        if (area != null)
            return area.getRedirect();

        return null;
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPageWriter#getTarget()
     */
    public String getTarget()
    {
        return _dialog.getTarget();
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPageWriter#isFrameSet()
     */
    public boolean isFrameSet()
    {
        return false;
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPage#setBackgroundColor(java.lang.String,
     *      java.lang.String)
     */
    public void setBackgroundColor(String hexColor, String frame)
    {
        super.setBackgroundColor(hexColor, frame);

        LyArea      area = _dialog.getArea();
        LyStrutsArea webArea = (LyStrutsArea)(area.getInternalView());

        if (frame == null)
            webArea.setBackgroundColor(hexColor, null);
    }

    /**
     * Sets the content of the dialog.
     */
    public void setContent()
    {
        LyWebViewManager viewManager = getWebViewManager();
        String           mainTarget = viewManager.getMainTargetFrame();

        if ((mainTarget != null) && (mainTarget.length() != 0))
            _name += mainTarget;
        else
            _name = "";
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPage#setVisible(boolean)
     */
    public void setVisible(boolean status)
    {
        if (status && (status != _visible))
        {
            String              target = _dialog.getTarget();
            LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();
            
            if (target == null)
                viewManager.addReloadFrame(LyWebConstants.TARGET_FRAME);
            else
            {
                StringBuffer        buffer = new StringBuffer();
                String              title = _dialog.getLabel();
                String              popupUrl = viewManager.getPopupFrameAbsoluteUrl();
                
                title = translate(title);
                buffer.append("<script type='text/javascript'>\n//<![CDATA[\n");
                buffer.append("ShowPopupView(true, '" + getJavascriptString(title) + "', '" + popupUrl + "');");
                buffer.append("\n//]]>\n</script>");
                
                viewManager.addAdditionalRefreshScripts(buffer.toString());
            }
        }
        
        if (status && (status != super.isVisible()))
            // Always refresh since we keep existing dialogs in the session
            refresh();

        super.setVisible(status);
    }

    /*
     * (non-Javadoc)
     * @see leon.view.graphic.LyGraphicalDialog#setArea(leon.view.LyArea)
     */
    public void setArea(LyArea area)
    {
        refresh();
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPage#writeBeginScripts(java.io.Writer,
     *      java.lang.String)
     */
    public void writeBeginScripts(Writer pw, String frame)
        throws IOException
    {
        super.writeBeginScripts(pw, frame);

        LyArea      area = _dialog.getArea();
        LyStrutsArea webArea = (LyStrutsArea)(area.getInternalView());
        boolean     isFrameSet = webArea.isFrameSet();

        if (!isFrameSet)
            webArea.writeBeginScripts(pw, frame);
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPage#writeEndScripts(java.io.Writer,
     *      java.lang.String)
     */
    public void writeEndScripts(Writer pw, String frame)
        throws IOException
    {
        super.writeEndScripts(pw, frame);

        LyArea      area = _dialog.getArea();
        LyStrutsArea webArea = (LyStrutsArea)(area.getInternalView());
        boolean     isFrameSet = webArea.isFrameSet();

        if (!isFrameSet)
            webArea.writeEndScripts(pw, frame);
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.struts.LyStrutsView#getPopupView()
     */
    protected String getPopupView()
    {
        StringBuffer sb = new StringBuffer();

        if (_inPopUp
        		&& LyStrutsViewManager.DHTML_POPUP_VIEW
                && !LyWebConstants.POPUP_VIEW.equals(_dialog.getTarget()))
            // Never add a popup IN the popup !
        {
            String           frameId = LyWebConstants.POPUP_VIEW;
            String           baseUrl = getPopupBaseUrl();

            // If the POPUP_VIEW stack is not empty, show the popup right now,
            // otherwise hide it.
            String           visibility = "hidden";
            LyWebViewManager viewManager = getWebViewManager();

            if (viewManager.getControllerStackSize(LyWebConstants.POPUP_VIEW) > 0)
                visibility = "visible";

            if (!viewManager.getEnvironment().testEnv("LY_REPLAY") && !isPrintMode())
            {
                sb.append("<!-- popup view is hidden but ready -->\n");
                sb.append("<iframe class='popupView' id='").append(frameId).append("' name='").append(frameId);
                sb.append("' src='"
                    + LyWebObject.encodeUrlXhtml(baseUrl)
                    + "' style='z-index: 10000; height:20; width: 100; position: absolute; left: 50px; top: 20px; visibility: ");
                sb.append(visibility).append(";'\n");
                sb.append(
                    " scrolling='no' marginheight='0' marginwidth='0' frameborder='0' ly_debug='' ly_width_scroll='1' ly_height_scroll='1'></iframe>\n");

                sb.append("<div id='popupMovingDiv'\n");
                sb.append("     onMouseDown='startPopupMoving(event)'\n");
                sb.append("     onMouseUp='endPopupMoving(event)'\n");
                sb.append("     onMouseMove='movePopup(event)'\n");
                sb.append("     onMouseOut=\"endPopupMoving(event);this.style.visibility = 'hidden';\"\n");
                sb.append(" style='cursor:move;z-index:20;background-color:#000000;filter:alpha(opacity=0);opacity:0;-moz-opacity:0;visibility:hidden;position:absolute;'>\n");
                sb.append("</div>\n");
            }
        }
        return sb.toString();
    } // end method getPopupView

} // end class LyStrutsDialog
