/*
 * LyStrutsDictionary.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import java.io.*;
import java.util.*;
import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Dictionary object (as described in the dtd) for web viewer.
 *
 * <p><b>DICTIONARY: Input Field of Dictionary Type</b><br>
 * An input field of dictionary type allows to display a graphical component containing a combo with
 * a tree.</p>
 *
 * @version $Id: LyStrutsDictionary.java 23300 2009-12-17 13:56:41Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.LyDictionary
 */
public class LyStrutsDictionary extends LyWebPageWriter
	implements LyGraphicalDictionary
{
	public static final String    __VERSION =
		"$Id: LyStrutsDictionary.java 23300 2009-12-17 13:56:41Z jlsaouli $";

	/** The <b>popup</b> String value used within names of frames that are shown in popup. */
	protected static final String POPUP = "popup".intern();

	/**
	 * The <b>tree</b> String value used within names of frames that contains a combo with a tree.
	 */
	protected static final String TREE = "tree".intern();

	/** Image for opening selector button. */
	protected String			  _openSelectorButton;

	/** Disabled image for opening selector button. */
	protected String			  _disabledOpenSelectorButton;

	/** The last opened node image URL. */
	protected String			  _lastNodeOpen;

	/** The last closed node image URL. */
	protected String			  _lastNodeClosed;

	/** The open node image URL. */
	protected String			  _nodeOpen;

	/** The closed node image URL. */
	protected String			  _nodeClosed;

	/** The last node image URL. */
	protected String			  _lastNode;

	/** The blank node image URL. */
	protected String			  _blankNode;

	/** The vertical line node image URL. */
	protected String			  _verticalLineNode;

	/** The simple node image URL. */
	protected String			  _simpleNode;

	/** The empty checkbox image URL. */
	protected String			  _emptyCheckbox;

	/** The selected checkbox image URL. */
	protected String			  _checkedCheckbox;

	/** The dicable checkbox image URL. */
	protected String			  _disabledCheckbox;

	/** The dictionary generic component. */
	protected LyDictionary		  _dictionary;

	/**
	 * Class constructor of a Dictionary object for the struts target.
	 *
	 * @param dictionary the generic Dictionary defined for every type of view targets
	 */
	public LyStrutsDictionary(LyDictionary dictionary)
	{
		super(dictionary);
		_dictionary = dictionary;

		init();
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.LyWebPage#init()
	 */
	public void init()
	{
		super.init();

		_openSelectorButton = getImageUrl("LY_ARROW_DOWN_BIG_BUTTON");
		_disabledOpenSelectorButton = getDisabledImageUrl("LY_ARROW_DOWN_BIG_BUTTON");

		LyFormField parent = (LyFormField)_dictionary.getParent();
		LyCaption   caption = parent.getCaption();

		if ((caption != null) && (caption.getLabel() != null))
			_title = translate(caption.getLabel());
		else
			_title = translate("LY_DICTIONARY");

		_simpleNode = getImageUrl("LY_NODE");
		_lastNodeOpen = getImageUrl("LY_LAST_NODE_OPEN");
		_lastNodeClosed = getImageUrl("LY_LAST_NODE_CLOSED");
		_nodeOpen = getImageUrl("LY_NODE_OPEN");
		_nodeClosed = getImageUrl("LY_NODE_CLOSED");
		_lastNode = getImageUrl("LY_LAST_NODE");
		_blankNode = getImageUrl("LY_EMPTY_NODE");
		_verticalLineNode = getImageUrl("LY_VERTICAL_LINE");
		_emptyCheckbox = getImageUrl("LY_TREENODE_EMPTY_CHECKBOX");
		_checkedCheckbox = getImageUrl("LY_TREENODE_CHECKED_CHECKBOX");
		_disabledCheckbox = getImageUrl("LY_ENTRY_VALIDITY_PARTIALLY");
		setBodyClassName("dictionary");
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.LyWebPageWriter#getPageWriters()
	 */
	public Vector<LyWebPageWriter> getPageWriters()
	{
		Vector<LyWebPageWriter> v = new Vector<LyWebPageWriter>();

		v.addElement(this);
		return v;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.LyWebPageWriter#getRedirect(java.lang.String)
	 */
	public String getRedirect(String frame)
	{
		return null;
	}

	/**
	 * Indicates if the Web browser used is Netscape.
	 *
	 * @return true if it is Netscape, false otherwise
	 */
	public boolean isNetscape()
	{
		LyWebViewManager viewManager = getWebViewManager();

		return (viewManager.getBrowser() == LyWebViewManager.BROWSER_NETSCAPE);
	}

	/**
	 * Indicates if the Web browser can disabled its elements.
	 *
	 * @return true if it can, false otherwise
	 */
	public boolean canDisabled()
	{
		LyWebViewManager viewManager = getWebViewManager();

		return (!isNetscape() || (viewManager.getBrowserVersion() >= 5.0));
	}

	/**
	 * Indicates if the dictionary is editable.
	 *
	 * @return true if it is editable, false otherwise
	 */
	public boolean isEditable()
	{
		LyFormField parent = (LyFormField)_dictionary.getParent();

		if (!parent.isEditable() || !parent.isEnabled())
			return false;

		LyForm form = ((parent.getParent() instanceof LyForm) ? (LyForm)parent.getParent() : null);

		return ((form == null) || form.isEditable());
	}

	/**
	 * Indicates if the dictionary is optional.
	 *
	 * @return true if it is optional, false otherwise
	 */
	public boolean isOptional()
	{
		LyFormField parent = (LyFormField)_dictionary.getParent();

		return parent.isOptional();
	}

	/**
	 * Indicates if the dictionary needs check.
	 *
	 * @return true if it needs check, false otherwise
	 */
	public boolean needCheck()
	{
		LyFormField parent = (LyFormField)_dictionary.getParent();

		return parent.isNeedCheck();
	}

	/**
	 * Gets the dictionary's form name.
	 *
	 * @param      pw the page writer
	 * @return     the parent from name
	 * @deprecated replaced by {@link #getFormName()} because the parameter <code>pw</code> is not
	 *             used.
	 */
	public String getFormName(LyWebPageWriter pw)
	{
		return getFormName();
	}

	/**
	 * Gets the dictionary's form name.
	 *
	 * @return the parent from name
	 * @since  leonardi V4.0, replace {@link #getFormName(LyWebPageWriter)} where the attribute was
	 *         not used.
	 */
	public String getFormName()
	{
		//CLEANCODE : parent is never read
		//LyFormField parent = (LyFormField) _dictionary.getParent();
		_dictionary.getParent();

		LyStrutsForm form = getHtmlForm();

		return ((form == null) ? "default" : form.getName());
	}

	/**
	 * Gets the dictionary's form.
	 *
	 * @return the dictionary's form
	 */
	public LyForm getForm()
	{
		// Look for an ancestor of type LyForm
		LyComponent ancestor = _dictionary;

		do
			ancestor = ancestor.getParent();
		while ((ancestor != null) && !(ancestor instanceof LyForm));

		return (LyForm)ancestor;
	}

	/**
	 * Gets the Struts dictionary's form .
	 *
	 * @return the struts dictionary's form
	 */
	public LyStrutsForm getHtmlForm()
	{
		LyForm form = getForm();

		return ((form == null) ? null : (LyStrutsForm)form.getInternalView());
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	    pw.startComponent(this);
	    
		updateRefreshInfo(pw);

		boolean isEditable = isEditable();
		boolean canDisabled = canDisabled();

		String		    sep = LyWebParameterValues.HTML_SEP;
		char		    visibleSep = getEnvironment().getString().VISIBLE_SEP2;
		StringBuffer    value = new StringBuffer();
		StringBuffer    displayedValue = new StringBuffer();
		LyEntry		    root = _dictionary.getEntry(), subEntry;
		Vector<LyEntry> subEntries = new Vector<LyEntry>();
		int			    i, n;
		int			    height = 200;
		int			    width = computeWidth();
		int			    minComboWidth = getIntResource("LY_COMBO_MIN_WIDTH");
		int			    fontSize = getFontSize("LY_DEFAULT_FONT");
		int			    textSize = minComboWidth / fontSize;

		if (_dictionary.getNRows() > 0)
			height = 22 * _dictionary.getNRows();

		getSubEntries(subEntries, root);
		n = subEntries.size();

		for (i = 0; i < n; i++)
		{
			subEntry = subEntries.elementAt(i);

			if (subEntry.isChecked())
			{
				if (value.length() > 0)
					value.append(sep);

				if (displayedValue.length() > 0)
					displayedValue.append(visibleSep);

				value.append(subEntry.getId());
				
				if (subEntry.getLabel() != null)
					displayedValue.append(translate(subEntry.getLabel()));
			}
		}

		// If not editable and control can be disabled, just write text
		if (!isEditable && !canDisabled)
		{
			if (displayedValue != null)
				pw.append("<font class='label'>" + displayedValue + "</font>\n");
			return;
		}

		// First, add the needed scripts to the page
		pw.addBeginScript("<script type='text/javascript'>window.defaultStatus='';</script>",
			false);

		// Add a hidden input that will be sent as the new value.
		// This input must be updated each time the value is modified.
		String fieldName = ((LyFormField)_dictionary.getParent()).getId();
		String name = getId();

		pw.append("<input type='hidden' name='" + fieldName + "' value='" + value + "' />\n");

		// Add the selector
		String disabledColor = getHexColor("LY_DISABLED_BACKGROUND");
		String formName = getFormName();

		if (testResource("LY_USE_TREE_FOR_DICTIONARY"))
		{
			String treeScript = getTreeScript(false);

			pw.addCssScript(".link { text-decoration:NONE;" + getFontString("LY_TREENODE_FONT")
				+ "}", false);
			pw.addCssScript("a.hover {color:blue}", false);
			pw.addCssScript("a.link {color:black}", false);
			pw.addCssScript("a.alink {color:black}", false);
			pw.addCssScript("a.vlink {color:black}", false);
			pw.append(treeScript);
		}
		else
		{
			pw.append("<table border='0' cellpadding='0' cellspacing='0'>\n");
			pw.append("<tr valign='middle'  align='center'>\n<td>\n");

			if (isEditable)
			{
				// Hidden inputs used to keep point where mouse was clicked
				// in order to make the selector appear under the date field
				pw.append("<input type='hidden' name='screenX' />\n");
				pw.append("<input type='hidden' name='screenY' />\n");
			}

			pw.append("<input");
			pw.append(" name='" + name + "_text' size='" + textSize + "'");
			pw.append(" value='" + displayedValue + "'");

			// Functions used for reset the imputs and disable write
			pw.append(" onKeyPress=\"return OnKeyPress(event, window.document." + formName + ", '"
				+ name + "_text', '" + fieldName + "')\" ");
			pw.append(" onKeyDown=\"OnKeyDown(event, window.document." + formName + ", '" + name
				+ "_text', '" + fieldName + "')\" ");

			if ((disabledColor != null) && !isEditable)
				pw.append(" style='background-color:" + disabledColor + "'");

			pw.append("/>\n");
			pw.append("</td>\n<td>\n");

			if (isEditable)
			{
				preparePage(pw.getFrameName(), pw.getWindowName());
				
				pw.append("<a onClick=\"window.document." + formName
					+ ".screenX.value=event.screenX;");
				pw.append("window.document." + formName + ".screenY.value=event.screenY\"");
				pw.append(" href=\"javascript:OpenDictionaryPopup('dictionary_selector" + _dictionary.getIdCode() + "');\"");
				pw.append(" onmouseover=\"window.status='';return true\">\n");
			}

			if (isEditable)
				pw.append("<img src=\"" + _openSelectorButton + "\" border='0'  align='middle'/>");
			else
				pw.append("<img src=\"" + _disabledOpenSelectorButton
					+ "\" border='0'  align='middle' class='disabledTool'/>");

			if (isEditable)
				pw.append("</a>");

			pw.append("</td><td>&nbsp;\n");
			pw.append("</td></tr><tr><td colspan='3'>\n");
			
			if (isEditable)
			{
				LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();
				String				pageUrl = viewManager.getActionURL(LyStrutsConstants.PATH_ACTION_PAGE);

				pw.append("<iframe src='" + getRootUrl() + pageUrl + "?"
						+ LyWebConstants.PAGE + "=" + getId() + "&#38;" + LyWebConstants.FRAME
						+ "=" + POPUP);
				pw.append("' id='dictionary_selector" + _dictionary.getIdCode() + "' ");
				pw.append("name='dictionary_selector" + _dictionary.getIdCode() + "' style='position:absolute;width:" + width + ";height:" + height
					+ ";visibility:hidden;z-index:50;border:1px solid #aaaaaa;background-color:" + getBackgroundColor(TREE) + "' ");
				pw.append("scrolling='no' frameborder='0'>");
				pw.append("</iframe>\n");
			}
			
			pw.append("</td></tr></table>\n");
		}
		
		pw.endComponent(this);
	} // end method writeHtml

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.LyWebPage#writeBeginPage(java.io.Writer, java.lang.String)
	 */
	public void writeBeginPage(Writer pw, String frame)
		throws IOException
	{
		if (!POPUP.equals(frame))
		{
			super.writeBeginPage(pw, frame);
			return;
		}

		pw.write("<!-- Generated by Leonardi (Lyria) -->\n");
		pw.write("<html><head>\n");
		pw.write("<meta name=\"AUTHOR\" content=\"Lyria\"/>\n");
		pw.write("<meta name=\"GENERATOR\" content=\"Leonardi\"/>\n");
		pw.write("<title>");
		pw.write((_title == null) ? "" : translate(_title));
		pw.write("</title>\n");
		pw.write("</head>\n");
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.LyWebPage#getBackgroundColor(java.lang.String)
	 */
	public String getBackgroundColor(String frame)
	{
		if (!TREE.equals(frame))
			return super.getBackgroundColor(frame);
		else
			return getHexColor("LY_TREE_BACKGROUND");
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.LyWebPage#writeEndPage(java.io.Writer, java.lang.String)
	 */
	public void writeEndPage(Writer pw, String frame)
		throws IOException
	{
		if (!POPUP.equals(frame))
			super.writeEndPage(pw, frame);
		else
		{
			pw.write("</html>\n");
			pw.write("<!-- End of HTML generation by Lyria -->\n");
		}
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.LyWebPageWriter#preparePage(java.lang.String, java.lang.String)
	 */
	public void preparePage(String frame, String targetWindow)
	{
		if (targetWindow == null)
			return;
		
		super.preparePage(frame, targetWindow);

		clearBuffer(POPUP);
		setActiveBuffer(POPUP);

		writePopup(this);

		clearBuffer(TREE);
		setActiveBuffer(TREE);

		writeTreeFrame(this);
	}

	/**
	 * Writes the content of the popup.
	 *
	 * @param pw the current page writer
	 */
	protected void writePopup(LyWebPageWriter pw)
	{
		LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();
		String				pageUrl = viewManager.getActionURL(LyStrutsConstants.PATH_ACTION_PAGE);

		pw.append("<frameset rows='0,*' border='0'>");
		pw.append("<frame name='hidden_frame' src='");
		pw.append(getWebViewManager().getScriptsDir());
		pw.append("/blank.html'/>");
		pw.append("<frame name='tree_frame' src='" + getRootUrl() + pageUrl + "?" + LyWebConstants.PAGE + "="
			+ getId() + "&#38;" + LyWebConstants.FRAME + "=" + TREE + "' scrolling='auto'/>");
		pw.append("</frameset>");
	}

	/**
	 * Writes the content of the tree frame that manages the dictionary.
	 *
	 * @param pw the current page writer
	 */
	protected void writeTreeFrame(LyWebPageWriter pw)
	{
		pw.addCssScript("font.label{" + getFontString("LY_DEFAULT_FONT") + "}", false);
		pw.addCssScript(".link { text-decoration:NONE;" + getFontString("LY_TREENODE_FONT") + "}",
			false);
		pw.addCssScript("a.hover {color:blue}", false);
		pw.addCssScript("a.link {color:black}", false);
		pw.addCssScript("a.alink {color:black}", false);
		pw.addCssScript("a.vlink {color:black}", false);
		pw.addCssScript(".dictionary {background-color: " + getBackgroundColor(TREE) + ";}", false);

		pw.addEndScript(getTreeScript(true), false, true);
	}

	/**
	 * Builds the html text that will be used to display the dictionary component.
	 *
	 * @param      pw      the current page writer
	 * @param      isPopup indicates if dictionary is displayed in a popup
	 * @return     the html text describing the dictionary component
	 * @deprecated replaced by {@link #getTreeScript(boolean)} because the parameter <code>pw</code>
	 *             is not used.
	 */
	protected String getTreeScript(LyWebPageWriter pw, boolean isPopup)
	{
		return getTreeScript(isPopup);
	}

	/**
	 * Builds the html text that will be used to display the dictionary component.
	 *
	 * @param  isPopup indicates if dictionary is displayed in a popup
	 * @return the html text describing the dictionary component
	 * @since  leonardi V4.0, replace {@link #getTreeScript(LyWebPageWriter, boolean)} where the
	 *         page writer parameter was not used.
	 */
	protected String getTreeScript(boolean isPopup)
	{
		StringBuffer buffer = new StringBuffer();
		String		 cellId = "tree_cell_" + getId();
		String		 sep = LyWebParameterValues.HTML_SEP;
		char		 visibleSep = getEnvironment().getString().VISIBLE_SEP2;
		String		 formName = getFormName();
		String		 name = getId();
		String		 fieldName = ((LyFormField)_dictionary.getParent()).getId();

		buffer.append("<script type='text/javascript'>\n//<![CDATA[\n");

		// Initialize the javascript variables
		if (_dictionary.isMultiple())
			buffer.append("selectionMode = 2;\n");
		else
			buffer.append("selectionMode = 1;\n");

		buffer.append("empty_checkbox = '" + _emptyCheckbox + "';\n");
		buffer.append("checked_checkbox = '" + _checkedCheckbox + "';\n");
		buffer.append("disabled_checkbox = '" + _disabledCheckbox + "';\n");
		buffer.append("simple_node = '" + _simpleNode + "';\n");
		buffer.append("last_node_open = '" + _lastNodeOpen + "';\n");
		buffer.append("last_node_closed = '" + _lastNodeClosed + "';\n");
		buffer.append("node_open = '" + _nodeOpen + "';\n");
		buffer.append("node_closed = '" + _nodeClosed + "';\n");
		buffer.append("last_node = '" + _lastNode + "';\n");
		buffer.append("blank_node = '" + _blankNode + "';\n");
		buffer.append("vertical_line_node = '" + _verticalLineNode + "';\n");
		buffer.append("dictionaryTagName = '" + cellId + "';\n");
		buffer.append("sep = '" + sep + "';\n");
		buffer.append("visibleSep = '" + visibleSep + "';\n");

		// Initialize the javascript variables for popup and not popup modes
		if (!isPopup)
		{
			buffer.append("parent_form = window.document." + formName + ";\n");
			buffer.append("parent_field_name = '" + fieldName + "';\n");
			buffer.append("hidden_field_name = '" + fieldName + "';\n");
			buffer.append("root_window = this;\n");
			buffer.append("is_popup = false;\n");
		}
		else
		{
			buffer.append("var popup = findTarget(null, '" + _lastFrameName + "');\n");
			buffer.append("root_window = popup;\n");
			buffer.append("parent_form = root_window.document." + formName + ";\n");
			buffer.append("parent_field_name = '" + name + "_text';\n");
			buffer.append("hidden_field_name = '" + fieldName + "';\n");
			buffer.append("is_popup = true;\n");
		}

		// Retrieve all the entries and create the javascript structure
		Vector<LyEntry> subEntries = new Vector<LyEntry>();
		LyEntry		    root = _dictionary.getEntry();

		getSubEntries(subEntries, root);

		int     i, n = subEntries.size();
		LyEntry subEntry;
		String  varName, parentVarName, image;

		if (_dictionary.getSelectionType() == LyDictionary.SelectionType.ALL)
			buffer.append("var selection_type = '1';\n");
		else
			buffer.append("var selection_type = '0';\n");

		for (i = 0; i < n; i++)
		{
			subEntry = subEntries.elementAt(i);
			image = ((subEntry.getImage() != null) ? getImageUrl(subEntry.getImage()) : "");
			varName = ((subEntry == root) ? "dictionaryRoot" : ("node_" + subEntry.getId()));
			varName = LyString.getJavascriptStringVar(varName);

			buffer.append("var " + varName + " = entry('" + subEntry.getId() + "', '"
				+ (subEntry.getLabel() == null ? "" : translate(subEntry.getLabel())) + "', '" + image + "', '" + image + "', "
				+ subEntry.isOpen() + ", false);\n");

			if (subEntry.getParent() instanceof LyEntry)
			{
				parentVarName = ((subEntry.getParent() == root)
						? "dictionaryRoot" : ("node_"
							+ ((LyEntry)subEntry.getParent()).getId()));
				parentVarName = LyString.getJavascriptStringVar(parentVarName);

				buffer.append("addChild(" + parentVarName + ", " + varName + ");\n");
			}
		}

		buffer.append("\n//]]>\n</script>\n");

		// Create the table and the cell that will contain the dictionary
		String tableParams = " border='0' cellpadding='1' cellspacing='1'";

		if (!isPopup)
			tableParams = " border='0' cellpadding='2' cellspacing='0' style='border:1px solid #aaaaaa;background-color:" + getBackgroundColor(TREE) + "'";

		buffer.append("<table" + tableParams + ">\n");
		buffer.append("<tr><td id='" + cellId + "'></td></tr>\n");
		buffer.append("</table>\n");

		if (root != null)
		{
			buffer.append("<script type='text/javascript'>\n");
			buffer.append("writeDictionary(true");

			// NeedCheck on field : add a hidden input for needCheck
			boolean needCheck = needCheck();
			
			if (needCheck && isEditable())
				buffer.append(",'" + _dictionary.getRootController() + "'");
				
			buffer.append(");\n");
			buffer.append("</script>\n");
		}

		return buffer.toString();
	} // end method getTreeScript

	/**
	 * Populates the given list with the list of sub-entries of given entry. The entry itself is
	 * added to the list.
	 *
	 * @param list  the list to be populated
	 * @param entry the entry to be parsed
	 */
	protected void getSubEntries(Vector<LyEntry> list, LyEntry entry)
	{
		if ((list == null) || (entry == null))
			return;

		list.addElement(entry);

		int     i, n = entry.getEntryCount();
		LyEntry subEntry;

		for (i = 0; i < n; i++)
		{
			subEntry = entry.getEntry(i);
			getSubEntries(list, subEntry);
		}
	}

	/**
	 * Compute approximative width for the popup window.
	 *
	 * @return the width that must be used for the popup window
	 */
	protected int computeWidth()
	{
		// Default min width is 50
		int			    width = 50, currentWidth;
		int			    defaultImageSize = 16;
		Vector<LyEntry> entries = new Vector<LyEntry>();
		int			    fontSize = getFontSize("LY_DEFAULT_FONT");

		getSubEntries(entries, _dictionary.getEntry());

		int     i, n = entries.size();
		LyEntry entry, parent, root = _dictionary.getEntry();
		String  label;

		for (i = 0; i < n; i++)
		{
			currentWidth = defaultImageSize;
			entry = entries.elementAt(i);
			label = entry.getLabel();

			if ((label != null) && (label.length() > 0))
			{
				label = translate(label);
				currentWidth = label.length() * fontSize;

				parent = entry;

				while (parent != root)
				{
					currentWidth += defaultImageSize;
					parent = (LyEntry)parent.getParent();
				}

				if (entry.getImage() != null)
					currentWidth += defaultImageSize;

				if (_dictionary.isMultiple())
					currentWidth += defaultImageSize;

				if (currentWidth > width)
					width = currentWidth;
			}
		}

		return width;
	} // end method computeWidth

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.setinterface.LySetEntryInterface#setEntry(leon.view.LyEntry)
	 */
	public void setEntry(LyEntry entry)
	{
		refresh();
	}
} // end class LyStrutsDictionary
