/*
 * LyStrutsFooter.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Footer object (as described in the dtd) for web viewer.
 *
 * <p><b>FOOTER: List Footer Cell</b><br>
 * Footer object is a List Footer Cell.</p>
 *
 * @version $Id: LyStrutsFooter.java 24563 2010-03-04 08:15:19Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.LyFooter
 */
public class LyStrutsFooter extends LyWebObject
	implements LyGraphicalFooter
{
	public static final String __VERSION =
		"$Id: LyStrutsFooter.java 24563 2010-03-04 08:15:19Z jlsaouli $";

	/** The footer generic component. */
	protected LyFooter		   _footer;

	/**
	 * Class constructor of a Footer object for the struts target.
	 *
	 * @param footer the generic Footer defined for every type of view targets
	 */
	public LyStrutsFooter(LyFooter footer)
	{
		super(footer);
		_footer = footer;
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	    pw.startComponent(this);
	    
		updateRefreshInfo(pw);

		LyFooters footers = (LyFooters)_footer.getParent();
		String    label = translate(_footer.getLabel());

		label = getHtmlString(label);
		boolean isWrap = _footer.isWrap();
		String localStyle = "";

		if (!footers.isDefaultColor())
		{
			localStyle += " background-image:none;";
			localStyle += " background-color:" + getHexColor(footers.getColor()) + ";";
		}

		if (!footers.isDefaultFont() || !footers.isDefaultFontColor())
			localStyle += getFontString(footers.getFont(), footers.getFontColor());

		if (localStyle.trim().length() > 0)
			localStyle = " style=\"" + localStyle + "\"";

		LyWebDecoration	deco = getWebViewManager().getDecoration();
		LyTable		    table = (LyTable)footers.getParent();
		LyHeaders       headers = table.getHeaders();
		String			align = "left";

		if (headers != null)
		{
			int		 index = getFooterIndex();
			LyHeader header = headers.getHeader(index);

			if (header.getAlign() == LyHeader.Align.CENTER)
				align = "center";
			else if (header.getAlign() == LyHeader.Align.RIGHT)
				align = "right";
		}

		pw.append("<td id='footer_" + _footer.getIdCode() + "' class='" + deco.getClassName(_footer, pw) + "' align='" + align + "' " + localStyle);

		if (!isWrap)
			pw.append(" nowrap='true'");

		pw.append(">");
		pw.append(label);

		if ((label == null) || (label.length() == 0))
			pw.append("&nbsp;");

		pw.append("</td>\n");

		pw.endComponent(this);
	} // end method writeHtml

	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetLabelInterface#setLabel(java.lang.String)
	 */
	public void setLabel(String label)
	{
		LyWebViewManager viewManager = getWebViewManager();
		StringBuffer     buffer = new StringBuffer();
		String			 id = "footer_" + _footer.getIdCode();

		if ((label == null) || (label.length() == 0))
			label = "&nbsp;";

		buffer.append("<script type='text/javascript'>\n//<![CDATA[\nreplaceDivInnerHtml('")
			.append(_lastFrameName).append("','").append(id).append("',\"")
			.append(label).append("\");\n//]]>\n</script>");

		viewManager.addAdditionalRefreshScripts(buffer.toString());
	}

	/**
	 * Gets the footer index in the parent footers.
	 *
	 * @return the current footer index
	 */
	public int getFooterIndex()
	{
		LyFooters footers = (LyFooters)_footer.getParent();
		int		  nFooters = footers.getFooterCount();

		for (int i = 0; i < nFooters; i++)
		{
			if (footers.getFooter(i).equals(_footer))
				return i;
		}
		return -1;
	}

} // end class LyStrutsFooter
