/*
 * LyStrutsFooters.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Footers object (as described in the dtd) for web viewer.
 *
 * <p><b>FOOTERS: List Footer</b><br>
 * A list footer allows to display cells at the end of a list. A list footer does not contain any
 * element: this is only a non-empty list of cells of footer list.</p>
 *
 * @version $Id: LyStrutsFooters.java 23300 2009-12-17 13:56:41Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.LyFooters
 */
public class LyStrutsFooters extends LyWebObject
	implements LyGraphicalFooters
{
	public static final String __VERSION =
		"$Id: LyStrutsFooters.java 23300 2009-12-17 13:56:41Z jlsaouli $";

	/** The footers generic component. */
	protected LyFooters		   _footers;

	/**
	 * Class constructor of a Footers object for the struts target.
	 *
	 * @param footers the generic Footers defined for every type of view targets
	 */
	public LyStrutsFooters(LyFooters footers)
	{
		super(footers);
		_footers = footers;
		initialize();
	}

	/**
	 * Initialization method.<br>
	 * Sets the footers font and color.
	 */
	public void initialize()
	{
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	    pw.startComponent(this);
	    
		LyTable		 table = (LyTable)_footers.getParent();
		LyStrutsTable webTable = (LyStrutsTable)table.getInternalView();

		LyFooter     footer;
		int			 i, nFooters = _footers.getFooterCount();

		// Add styleSheet for footers
		writeStyleSheet(pw);
		
		String localStyle = "";
		
		if (!_footers.isDefaultColor())
		{
			localStyle += " background-image:none;";
			localStyle += " background-color:" + getHexColor(_footers.getColor()) + ";";
		}

		if (!_footers.isDefaultFont() || !_footers.isDefaultFontColor())
			localStyle += getFontString(_footers.getFont(), _footers.getFontColor());

		if (localStyle.trim().length() > 0)
			localStyle = " style=\"" + localStyle + "\"";
    
		pw.append("<tr>");
		
		if (webTable.generateExternalBorder(pw))
			pw.append("<td class='tableLeft' width='1'></td>\n");

		if (!pw.isPrintMode())
		{
			if (table.getSelectionMode() == LyTable.SelectionMode.NONE)
			{
				// Nothing to do...
			}
		}

		// Add a cell for images
		if (webTable.drawImages() && !webTable.sortOnStatus())
			pw.append("<td class='footer'>&nbsp;</td>\n");
		
		for (i = 0; i < nFooters; i++)
		{
			footer = _footers.getFooter(i);
			((LyWebComponent)footer.getInternalView()).writeHtml(pw);
		}
		
		// Adding a cell for last empty column
		if (webTable.addLastCell(pw))
			pw.append("<td class='footer' " + localStyle + " width='100%'>&nbsp;</td>\n");
		
		if (webTable.generateExternalBorder(pw))
			pw.append("<td class='tableRight' width='1'></td>\n");
		
		pw.append("</tr>\n");
		
		pw.endComponent(this);
	} // end method writeHtml

	/**
	 * Writes the needed styleSheet for this component.
	 *
	 * @param pw the page writer
	 */
	protected void writeStyleSheet(LyWebPageWriter pw)
	{
	}

} // end class LyStrutsFooters
