/*
 * LyStrutsFrame.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts;

import java.io.*;
import java.util.*;

import javax.servlet.http.*;

import leon.app.*;
import leon.misc.*;
import leon.view.*;
import leon.view.getinterface.*;
import leon.view.graphic.*;
import leon.view.setinterface.*;
import leon.view.web.*;
import leon.view.web.struts.action.*;


/**
 * Type : leon.view.web.struts.LyStrutsFrame<br>
 * The peer component of LyFrame for the Struts viewer.
 *
 * @version $Id: LyStrutsFrame.java 40694 2012-09-06 15:32:54Z nhamon $
 * @author  Lyria S.A.
 */
public class LyStrutsFrame extends LyStrutsView
    implements LyGraphicalFrame,
               LyIsVisibleInterface,
               LySetVisibleInterface
{
	public static final String __VERSION =
		"$Id: LyStrutsFrame.java 40694 2012-09-06 15:32:54Z nhamon $";

	   /** The <b>menubar</b> String value used within frames names. */
    public static final String MENUBAR = "menubar".intern();

    /** The <b>area</b> String value used within frames names. */
    public static final String AREA = "area".intern();

    /** The <b>messages</b> String value used within frames names. */
    public static final String MESSAGES = "messages".intern();

    /** The frame generic component. */
    protected LyFrame          _frame;

    /**
     * Internal parameter indicating if the view is always visible or not. Prevent from clearing
     * view cache for pilot frame. Default value is false.
     */
    protected boolean          _visibleStatus;
    
	/**
	 * Constructor.
	 *
	 * @param frame the frame component
	 */
	public LyStrutsFrame(LyFrame frame)
	{
        super(frame);
        _frame = frame;

        init();
        setContent();

        init();
	}
	
	public void preparePage(String frame, String targetWindow)
	{
		if (targetWindow != null)
			setWindowName(targetWindow); // keep the name of the window where

		clearBuffer(frame);
		clearBeginScripts(frame);
		clearEndScripts(frame);
		clearCssScripts(frame);
		setActiveBuffer(frame);

        startComponent(this);
        
		updateRefreshInfo(this);
		setTitle(_frame.getLabel());

		// Retrieve request info
		LyWebRequestInfo		info = LyStrutsServlet.getWebRequestInfo();
		HttpServletRequest		request = (info == null ? null : info.getRequest());
		LySession				session = (request == null ? null : (LySession)request.getAttribute(LyWebConstants.RA_LY_SESSION));

		int		height = ((session != null) && (session.getMasterSession() == null) ? _frame.getHeight() : 0);
		String	heightValue = "100%";
		
		if (height > 0)
			heightValue = height + "";
		
		if (!LyWebViewManager.OPTIMIZE_TABLE_GENERATION)
			append("<table width='100%' height='" + heightValue + "' border='0' cellspacing='0' cellpadding='0' valign='top'");

		// Add an id to this main frame with full height to compute its height in the checkSizeMainView() Javascript function
		if (height <= 0)
			append(" id='mainFrameFullHeightTable'");

		append(">\n");

		// Add up frame if needed
		boolean		addUpFrame = (LyWebConstants.TARGET_FRAME.equals(frame));
		
		if (addUpFrame
				&& LyWebPageWriter.displayUpPage(getEnvironment())
				&& (session != null))
		{
			String					target = (request == null ? null : (String)request.getParameter(LyWebConstants.TARGET_PARAM));
			HashMap<String, String>	parameters = new HashMap<String, String>();
			LyStrutsViewManager		viewManager = (LyStrutsViewManager)getViewManager();
			
			if (target != null)
				parameters.put(LyWebConstants.TARGET_PARAM, target);

			String upFrame = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_UPFRAME, parameters, true);
			
			append("<tr>\n");
			append("<td vAlign='top'>\n");
			append("<iframe src='" + upFrame + "' id='up_frame' name='up_frame' class='upFrame' frameborder='0'></iframe>\n");
			append("</td>\n");
			append("</tr>\n");
		}
		
		// Write menubar
		LyMenuBar menubar = _frame.getMenuBar();

		if (menubar != null)
		{
			if (!LyWebViewManager.OPTIMIZE_TABLE_GENERATION)
			{
				append("<tr>\n");
				append("<td vAlign='top'>\n");
			}
			
			writeMenuBar(this, frame, targetWindow);
			
			if (!LyWebViewManager.OPTIMIZE_TABLE_GENERATION)
			{
				append("</td>\n");
				append("</tr>\n");
			}
		}

		// Write area
		if (!LyWebViewManager.OPTIMIZE_TABLE_GENERATION)
		{
			append("<tr height='100%'>\n");
			append("<td height='100%' vAlign='top'>\n");
		}
		
		writeArea(this);

		if (!LyWebViewManager.OPTIMIZE_TABLE_GENERATION)
		{
			append("</td>\n");
			append("</tr>\n");
		}

		// Write messages
		if (_frame.getMessages() != null)
		{
			if (!LyWebViewManager.OPTIMIZE_TABLE_GENERATION)
			{
				append("<tr>\n");
				append("<td>\n");
			}
			
			writeMessages(this);

			if (!LyWebViewManager.OPTIMIZE_TABLE_GENERATION)
			{
				append("</td>\n");
				append("</tr>\n");
			}
		}
		
		if (!LyWebViewManager.OPTIMIZE_TABLE_GENERATION)
			append("</table>\n");
		
	    endComponent(this);
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetVisibleInterface#setVisible(boolean)
	 */
	public void setVisible(boolean status)
	{
		if (status && (status != _visible))
		{
			String				target = _frame.getTarget();
			LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();
			
			if (LyWebConstants.POPUP_VIEW.equals(target))
			{
				StringBuffer		buffer = new StringBuffer();
				String				title = _frame.getLabel();
				String				popupUrl = viewManager.getPopupFrameAbsoluteUrl();
				
				title = translate(title);
				buffer.append("<script type='text/javascript'>\n//<![CDATA[\n");
				buffer.append("ShowPopupView(true, '" + getJavascriptString(title) + "', '" + popupUrl + "');");
				buffer.append("\n//]]>\n</script>");
				
				viewManager.addAdditionalRefreshScripts(buffer.toString());
			}
		}

        if (status && (status != isSuperVisible()))
            // XXX do NOT forget super here : workaround
            refresh();
        
		super.setVisible(status);
	}

	public void writeBeginPage(Writer pw, String frame)
		throws IOException
	{
		LyWebViewManager	manager = getWebViewManager();
		LyEnvironment		env = getEnvironment();

		pw.write("<html><head>\n");
		pw.write("<meta name=\"AUTHOR\" content=\"Lyria\"/>\n");
		pw.write("<meta name=\"GENERATOR\" content=\"Leonardi\"/>\n");
		pw.write("<meta name=\"Content-Type\" content=\"text/html;charset=" + LyString.DEFAULT_CHARSET + "\"/>\n");
		pw.write("<title>");
		pw.write((_title == null) ? "" : translate(_title));
		pw.write("</title>\n");
		writeCssScripts(pw, frame);
		pw.write("</head>\n");

		if ((frame == null) || frame.equals(LyWebConstants.TARGET_FRAME))
			frame = MAIN_BUFFER;

		pw.write("<body ");
		
		String	bodyClass = getBodyClassString();
		
		if (bodyClass != null)
			pw.write(" " + bodyClass);

		pw.write(" topmargin='0' leftmargin='0' style='height:100%;width:100%;' " + getBodyScript()
			+ ">\n");
		
		String	scripts = manager.generateScriptsInclusion();
		
		pw.write(scripts);
		pw.write("<script type='text/javascript'>var pageWriterId='" + getId() + "';</script>\n");

		if (!env.testEnv("LY_REPLAY")
				&& ((getAttachmentHandler(frame) == null) || (manager instanceof LyStrutsViewManager))
				&& (manager.hasProperty("DYNAMIC")))
			pw.write("<script type='text/javascript'>var AJAX_MODE='" + (FORCE_AJAX ? "force" : true) + "';</script>\n");
		else
			pw.write("<script type='text/javascript'>var AJAX_MODE='false';</script>\n");

		pw.write("<script type='text/javascript'>var isInPopUp='" + isInPopUp() + "';</script>\n");

		if (manager instanceof LyStrutsViewManager)
		{
			String	actionUrl = ((LyStrutsViewManager)manager).getActionURL(LyStrutsConstants.PATH_ACTION_GRAPHICEVENT);

			pw.write("<script type='text/javascript'>var STRUTS_EVENT_URL='" + actionUrl + "';</script>\n");
		}
	}
	
	protected String getBodyScript()
	{
		if (LyWebConstants.POPUP_VIEW.equals(getWindowName()))
		{
			String	title = _frame.getLabel();
			int		width = Math.max(_frame.getWidth(), _frame.getMinWidth());
			int		height = Math.max(_frame.getHeight(), _frame.getMinHeight());
			
			return "onload='setPopupDimension(" + width + "," + height + ");checkSizePopupView(null, true, false, \"" +
				getHtmlEntitiesString(translate(title)) + "\", false, false);'";
		}
		else
			return "onload=\"checkSizeMainView('true', 'false');\" onResize=\"checkSizeMainView('true', 'true');\"";
	}

	public void writePageContent(Writer pw, String frame) throws IOException
	{
		// Use one single form for a same frame
		pw.write(getOpenFormContent(frame, true));

		if (frame == null)
			frame = MAIN_BUFFER;
		
		StringBuffer buffer = getBuffer(frame);
		String		 bufferString = buffer.toString();

		// Page size is greater than GENERATED_PAGE_MAX_SIZE : display an error page and dump page content
		// to temporary directory.
		if ((LyStrutsLeonardiAction.GENERATED_PAGE_MAX_SIZE > 0)
			&& (bufferString.length() > LyStrutsLeonardiAction.GENERATED_PAGE_MAX_SIZE))
		{
			String				fileName = "buffer_" + System.currentTimeMillis() + ".dump";
			String				tmpFilePath = getWebViewManager().getTemporaryDir() + File.separator + fileName;
			LyStrutsViewManager	viewManager = (LyStrutsViewManager)getWebViewManager();
			FileOutputStream	outputStream = new FileOutputStream(tmpFilePath);
			
			outputStream.write(bufferString.getBytes());
			outputStream.flush();
			outputStream.close();

			pw.write("<center><br/>Error while generating page.<br/>Page will be refreshed automatically in a few seconds.<br/><br/>");

			// Retrieve request info
			LyWebRequestInfo	info = LyStrutsServlet.getWebRequestInfo();
			HttpServletRequest	request = (info == null ? null : info.getRequest());
			LySession			session = (request == null ? null : (LySession)request.getAttribute(LyWebConstants.RA_LY_SESSION));

			if (session != null)
			{
				String	fileId = session.storeTempFile(tmpFilePath, fileName);
				String	url = viewManager.getDownloadAction() + fileId;

				pw.write("<a href='" + url + "'>See dump file</a></center>");
			}
			
			pw.write("<script type='text/javascript'>\n//<![CDATA[\n");
			pw.write("setTimeout('this.location.reload()', 5000);");
			pw.write("\n//]]>\n</script>\n");

			return;
		}

		pw.write(bufferString);
		pw.write("\n");

		pw.write("<!-- popup menu view is hidden but ready -->\n");
		pw.write("<div id='");
		pw.write(LyWebConstants.POPUP_MENU + _frame.getIdCode());
		pw.write("' name='");
		pw.write(LyWebConstants.POPUP_MENU);
		pw.write(_frame.getIdCode());
		pw.write("' style='z-index:10000;height:10;width:10;position:absolute;left:10px;top:10px;visibility:hidden;'>");
		pw.write("</div>");
		
		writeSplitterDivs(pw, frame);
		
		pw.write("</form>\n");
	}
	
	/**
	 * Writes the divs needed for splitter movements.
	 * @param pw
	 * @param frame
	 * @throws IOException
	 */
	protected void writeSplitterDivs(Writer pw, String frame) throws IOException
	{
		// Write hidden div used to move the splitter bar
		String		 image = getImageUrl("LY_SPLITTER_MOVEMENT");

		pw.write("<div id='hidden_splitter' style='width:5;height:5;top:5;left:5;z-index:100;background-color:#000000;filter:alpha(opacity=0);opacity:0;-moz-opacity:0;");
		pw.write("display:none;position:absolute;'></div>");

		// Write moving div used to move the vertical splitter bar
		pw.write("<div id='moving_splitterv' style='cursor:s-resize;width:5;height:8;top:5;left:5;z-index:110;");
		pw.write("display:none;position:absolute;'>");
		pw.write("<table height='100%' width='100%' border='0' cellpadding='0' cellspacing='0'><tr><td valign='middle'>");
		pw.write("<table border='0' width='100%' cellpadding='0' cellspacing='0'>");
		pw.write("<tr height='3'><td background='");
		pw.write(image);
		pw.write("'></td></tr></table>");
		pw.write("</td></tr></table>");
		pw.write("</div>\n");

		// Write moving div used to move the horizontal splitter bar
		pw.write("<div id='moving_splitterh' style='cursor:w-resize;width:8;height:5;top:5;left:5;z-index:110;");
		pw.write("display:none;position:absolute;'>");
		pw.write("<table align='center' border='0' width='100%' height='100%' cellpadding='0' cellspacing='0'>");
		pw.write("<tr><td align='center'><table border='0' height='100%' cellpadding='0' cellspacing='0'><tr><td background='");
		pw.write(image);
		pw.write("' width='3'>&nbsp;</td></tr></table></td></tr></table>");
		pw.write("</div>\n");
	}
	
	public void writeEndPage(Writer pw, String frame)
		throws IOException
	{
		if (_frame.isRootFrame()
		        && (LyWebConstants.TARGET_FRAME.equals(frame))
		        && (_title != null))
			pw.write("<script type='text/javascript'>setTopTitle(\"" + translate(_title) + "\");</script>\n");
		
		if (frame == LyWebConstants.TARGET_FRAME)
		{
			LyWebViewManager	viewManager = getWebViewManager();
			String				message = viewManager.getMessage();
			
			if (message != null)
				pw.write(message);
		}
		
		pw.write("</body>\n");
		pw.write("</html>\n");
	}

	/**
	 * Method overriden to set the IFRAME URL to the page action handler.
	 */
	public String getFrameBaseUrl()
	{
		// get the url to the page action
		LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();
		String			    url = viewManager.getActionAbsoluteUrl(
				LyStrutsConstants.PATH_ACTION_PAGE, null, true);

		return url;
	}

	/**
	 * Method overriden to set the popup URL to the page action handler.
	 *
	 * @return The base URL for a popup view.
	 */
	protected String getPopupBaseUrl()
	{
		// get the url to the page action
		LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();
		String			    url = viewManager.getActionAbsoluteUrl(
				LyStrutsConstants.PATH_ACTION_POPUPFRAME, null, true);

		return url;
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebPageWriter#setProperty(java.lang.String, java.lang.String)
	 */
	public void setProperty(String name, String value)
	{
		super.setProperty(name, value);
		
		if ("switchToolbarState".equals(name))
		{
			LyComponent	toolbar = _frame.getComponent(value);
			
			if (toolbar instanceof LyToolBar)
			{
				((LyToolBar)toolbar).setOpen(!((LyToolBar)toolbar).isOpen());
				
                getWebViewManager().addAdditionalRefreshScripts("<script type='text/javascript'>\n//<![CDATA[\ncheckSizeMainView(null);\n//]]>\n</script>");
			}
		}
	}
	
	   /**
     * Initialization method. Overriden to add initialization of the visible status to <code>
     * false</code>.
     *
     * @see leon.view.web.LyWebPage#init()
     */
    public void init()
    {
        super.init();
        _visibleStatus = false;
    }

    /**
     * Sets the content of the frame.
     */
    public void setContent()
    {
        LyWebViewManager viewManager = getWebViewManager();

        _title = _frame.getLabel();
        _title = translate(_title);
        _title = getHtmlString(_title);

        String mainTarget = viewManager.getMainTargetFrame();

        if ((mainTarget != null) && (mainTarget.length() != 0))
            _name = mainTarget;
        else
            _name = "";
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPage#setBackgroundColor(java.lang.String,
     *      java.lang.String)
     */
    public void setBackgroundColor(String hexColor, String frame)
    {
        super.setBackgroundColor(hexColor, frame);

        LyArea      area = _frame.getArea();
        LyStrutsArea webArea = (LyStrutsArea)(area.getInternalView());

        if (AREA.equals(frame))
            webArea.setBackgroundColor(hexColor, null);
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPageWriter#getRedirect(java.lang.String)
     */
    public String getRedirect(String frame)
    {
        LyArea area = _frame.getArea();

        if (area != null)
            return area.getRedirect();

        return null;
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPageWriter#clearBeginScripts()
     */
    public void clearBeginScripts()
    {
        super.clearBeginScripts();

        _beginScripts.put(MENUBAR, new Vector<_LyScript>());
        _beginScripts.put(AREA, new Vector<_LyScript>());
        _beginScripts.put(MESSAGES, new Vector<_LyScript>());
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPageWriter#clearEndScripts()
     */
    public void clearEndScripts()
    {
        super.clearEndScripts();

        _endScripts.put(MENUBAR, new Vector<_LyScript>());
        _endScripts.put(AREA, new Vector<_LyScript>());
        _endScripts.put(MESSAGES, new Vector<_LyScript>());
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPageWriter#clearCssScripts()
     */
    public void clearCssScripts()
    {
        super.clearCssScripts();

        _cssScripts.put(MENUBAR, new Vector<_LyCssScript>());
        _cssScripts.put(AREA, new Vector<_LyCssScript>());
        _cssScripts.put(MESSAGES, new Vector<_LyCssScript>());
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPageWriter#isFrameSet()
     */
    public boolean isFrameSet()
    {
        if (_noFrames)
            return false;

        LyMessages messages = _frame.getMessages();
        LyMenuBar  menubar = _frame.getMenuBar();
        LyArea     area = _frame.getArea();

        // If there is a message bar or a menu bar, use multi-framing
        if (((messages != null) || (menubar != null))
                && ((area != null) && (area.getPanelCount() > 0)))
            return true;
        else
            return false;
    }

    /**
     * Force the visibility status of a Frame Internal use : may be used to prevent view from being
     * cleared / freed. Default value is false. This is not showing the view : see setVisible()
     * instead.
     *
     * @param status the new status
     * @see   #setVisible
     */
    public void setVisibleStatus(boolean status)
    {
        _visibleStatus = status;
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPage#isVisible()
     */
    public boolean isVisible()
    {
        return (_visibleStatus);
    }

    /**
     * Indicates if the component is visible or not.
     *
     * @return true if the super component is shown (a visible component is never freed)
     * @see    leon.view.web.LyWebPage#isVisible()
     */
    public boolean isSuperVisible()
    {
        return (super.isVisible());
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPageWriter#getTarget()
     */
    public String getTarget()
    {
        return _frame.getTarget();
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.struts.LyStrutsView#getPopupView()
     */
    protected String getPopupView()
    {
        StringBuffer sb = new StringBuffer();

        // Never add a popup IN the popup !
        if (LyStrutsViewManager.DHTML_POPUP_VIEW
        	&& !LyWebConstants.POPUP_VIEW.equals(_frame.getTarget()))
        {
            String           frameId = LyWebConstants.POPUP_VIEW;
            String           baseUrl = getPopupBaseUrl() + "?_source=" + LyWebConstants.POPUP_VIEW
                + "&_reload=" + System.currentTimeMillis();

            // If the POPUP_VIEW stack is not empty, show the popup right now,
            // otherwise hide it.
            String           visibility = "hidden";
            LyWebViewManager viewManager = getWebViewManager();

            if (viewManager.getControllerStackSize(LyWebConstants.POPUP_VIEW) > 0)
                visibility = "visible";

            sb.append("<!-- popup view is hidden but ready -->\n");
            sb.append("<iframe class='popupView' id='").append(frameId).append("' name='").append(
                frameId);
            sb.append("' src='"
                + LyWebObject.encodeUrlXhtml(baseUrl)
                + "' style='z-index: 10000; height:20; width: 100; position: absolute; left: 50px; top: 20px; visibility: ");
            sb.append(visibility).append(";'\n");
            sb.append(
                " scrolling='no' marginheight='0' marginwidth='0' frameborder='0' ly_debug='' ly_width_scroll='1' ly_height_scroll='1'></iframe>\n");

            sb.append("<div id='popupMovingDiv'\n");
            sb.append("     onMouseDown='startPopupMoving(event)'\n");
            sb.append("     onMouseUp='endPopupMoving(event)'\n");
            sb.append("     onMouseMove='movePopup(event)'\n");
            sb.append("     onMouseOut=\"endPopupMoving(event);this.style.visibility = 'hidden';\"\n");
            sb.append(" style='cursor:move;z-index:20;background-color:#000000;filter:alpha(opacity=0);opacity:0;-moz-opacity:0;visibility:hidden;position:absolute;'>\n");
            sb.append("</div>\n");
        }
        return sb.toString();
    } // end method getPopupView

    /**
     * Writes HTML description corresponding to the frame messages.
     *
     * @param pw the page writer
     */
    public void writeMessages(LyWebPageWriter pw)
    {
        LyMessages messages = _frame.getMessages();

        if (messages != null)
        {
            LyStrutsMessages webMessages = (LyStrutsMessages)(messages.getInternalView());

            webMessages.writeHtml(pw);
        }
    }

    /**
     * Writes HTML description corresponding to the frame menubar.
     *
     * @param pw the page writer
     */
    public void writeMenuBar(LyWebPageWriter pw, String targetWindow)
    {
        writeMenuBar(pw, null, targetWindow);
    }
    
    /**
     * Writes HTML description corresponding to the frame menubar.
     *
     * @param pw the page writer
     */
    public void writeMenuBar(LyWebPageWriter pw, String frame, String targetWindow)
    {
        LyMenuBar menubar = _frame.getMenuBar();

        if (menubar != null)
        {
            LyStrutsMenuBar webMenubar = (LyStrutsMenuBar)(menubar.getInternalView());

            webMenubar.writeHtml(pw);
            webMenubar.setLastFrameName(((targetWindow == null) ? (frame == null ? LyWebConstants.TARGET_FRAME : frame)
                                                                : targetWindow));
        }
    }

    /*
     * (non-Javadoc)
     * @see leon.view.graphic.LyGraphicalFrame#setMessages(LyMessages)
     */
    public void setMessages(LyMessages messages)
    {
        refresh();
    }
} // end class LyStrutsFrame
