/*
 * LyStrutsFreeChart.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import java.awt.*;
import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.common.*;
import leon.view.getinterface.*;
import leon.view.graphic.*;
import leon.view.setinterface.*;
import leon.view.web.*;


/**
 * This class implements the Chart object (as described in the dtd) for web viewer.
 *
 * <p><b>CHART: Statistic Chart</b><br>
 * A chart allows to display a graph representing statistics on data.</p>
 *
 * @version $Id: LyStrutsFreeChart.java 23300 2009-12-17 13:56:41Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.common.LyCommonFreeChartConverter
 */
public class LyStrutsFreeChart extends LyStrutsPanel
	implements LyGraphicalChart,
		LyGetChartConverterInterface,
		LySetBoundsInterface
{
	public static final String			 __VERSION =
		"$Id: LyStrutsFreeChart.java 23300 2009-12-17 13:56:41Z jlsaouli $";

	/** The chart generic component. */
	protected LyChart					 _chart;

	/** The generic chart converter for this component. */
	protected LyCommonFreeChartConverter _chartConv;

	/** Width of the generated image. */
	protected int						 _width;

	/** Height of the generated image. */
	protected int						 _height;

	/**
	 * Class constructor of a Chart object for the struts target.
	 *
	 * @param chart the generic Chart defined for every type of view targets
	 * @see   LyChartController
	 */
	public LyStrutsFreeChart(LyChart chart)
	{
		super(chart);    
        
		_chart = chart;
        
        LyEnvironment   environment = chart.getEnvironment();
        
        _width = environment.getIntResource("LY_CHART_WIDTH");
        if (_width < 0)
            _width = LyCommonFreeChartConverter.DEFAULT_CHART_WIDTH;
        _height = environment.getIntResource("LY_CHART_HEIGHT"); 
        if (_height < 0)
            _height = LyCommonFreeChartConverter.DEFAULT_CHART_HEIGHT;
        
		initialize();
		setContent();
	}

	/**
	 * Initialization method.
	 */
	public void initialize()
	{
		_chartConv = new LyCommonFreeChartConverter(_chart);
	}

	/**
	 * Sets the content of this view.
	 */
	public void setContent()
	{
		_chartConv.getFreeChart();
	}

	/**
	 * Sets the width and height of the generated image.
	 *
	 * @param width  the new width of the generated image
	 * @param height the new height of the generated image
	 * @deprecated use setSize()
	 */
	public void setDimension(int width, int height)
	{
		setSize(width, height);
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.getinterface.LyGetChartConverterInterface#getChartConverter()
	 */
	public LyChartConverter getChartConverter()
	{
		return _chartConv;
	}

	/**
	 * Get the dimension of the component used to represent the chart.
	 * 
	 * @return an array of two integers (width and height of the component).
	 */
	public int[] getComponentDimension()
	{
		return new int[] {_width, _height};
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	    pw.startComponent(this);
	    
		updateRefreshInfo(pw);

		String chartId = _chart.getId();
		String dateStamp = "";
		String controllerId = _chart.getRootController();
		String colorString = "";

		Paint  paint = _chartConv.getFreeChart().getBackgroundPaint();

		if (paint instanceof Color)
		{
			Color color = (Color)paint;

			colorString = " bgColor='#"
				+ LyColors.getHexString(color.getRed(), color.getGreen(), color.getBlue())
				+ "'";
		}

		pw.addBeginScript("<script type='text/javascript'>var selectionContext='true'</script>",
			false);
		pw.setFormId(controllerId);

		if ((chartId != null) && (chartId.endsWith("_chart")))
			dateStamp = "_" + (new Date()).getTime();

		// Create the image
		LyViewManager viewManager = getViewManager();
		String        tmpDir = viewManager.getTemporaryDir();
		String        tmpRoot = viewManager.getTemporaryLocation();
		String        imageFile = tmpDir + "/chart_" + chartId + dateStamp;
		String        imagePath = tmpRoot + "/chart_" + chartId + dateStamp;
		StringBuffer  buffer = new StringBuffer();
      
		saveChartFile(tmpDir, imageFile, buffer);

		// Always generate image map: it handles tooltips
		pw.append(buffer.toString());

		pw.append("<center>\n");
		pw.append("<table width='100%' height='100%' cellpadding='0' cellspacing='0'>\n");
		pw.append("<tr width='100%' height='100%' valign='middle'  align='center'>\n");
		pw.append("<td" + colorString + ">\n");

		pw.append(getHtmlLink(imagePath));

		pw.append("</td>\n");
		pw.append("</tr>\n</table>\n");
		pw.append("</center>\n");

		System.gc(); // Force clean up here since use a large amount of memory
		
		pw.endComponent(this);
	} // end method writeHtml

	/**
	 * Save the file to the http directory.
	 *
	 * @param tmpDir    temporary directory for the image/svg
	 * @param imageFile name of the file
	 * @param buffer    the buffer where imagemap must be generated
	 */
	public void saveChartFile(String tmpDir, String imageFile, StringBuffer buffer)
	{
		if ((tmpDir == null) || (imageFile == null))
			return;

		// add the extension
		imageFile += ".png";

        _chartConv.setDimension(_width, _height);
		_chartConv.saveImg(tmpDir, imageFile, buffer, true);
	}

	/**
	 * Gets the HTML tag that links the image to the ressource.
	 *
	 * @param  imagePath the image path
	 * @return the HTML tag for the image
	 */
	public String getHtmlLink(String imagePath)
	{
		return "<img src='" + imagePath + ".png' border='0' usemap='#chart_imagemap'/>\n";
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LyFreeInterface#free()
	 */
	public void free()
	{
		_chart = null;
		_chartConv = null;
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalChart#addColumn(leon.view.LyColumn,
	 *      int)
	 */
	public void addColumn(LyColumn column, int pos)
	{
		refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalChart#removeColumn(leon.view.LyColumn)
	 */
	public void removeColumn(LyColumn column)
	{
		refresh();
	}
	
	public void setLocation(int x, int y) 
	{
		// nothing to do
	}

	/**
	 * Sets the width and height of the generated image.
	 *
	 * @param width  the new width of the generated image
	 * @param height the new height of the generated image
	 */
	public void setSize(int width, int height) 
	{
		if (width > 0)
			_width = width;

		if (height > 0)
			_height = height;
	}
	
} // end class LyStrutsFreeChart
