/*
 * LyStrutsGauge.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Gauge object (as described in the dtd) for web viewer.
 *
 * <p><b>GAUGE: Progress Bar</b><br>
 * A gauge is a progress bar that show to the user some information about the current process in
 * progress.</p>
 *
 * @version $Id: LyStrutsGauge.java 24563 2010-03-04 08:15:19Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.LyGauge
 */
public class LyStrutsGauge extends LyWebObject
	implements LyGraphicalGauge
{
	public static final String __VERSION =
		"$Id: LyStrutsGauge.java 24563 2010-03-04 08:15:19Z jlsaouli $";

	/** Number of cells that must be drawn. */
	public static final int    N_CELLS = 25;

	/** The gauge generic component. */
	protected LyGauge		   _gauge;

	/** The progess color. */
	protected String		   _progressColor;

	/** The backgroung color. */
	protected String		   _backgroundColor;

	/** The progress label color . */
	protected String		   _progressLabelColor;

	/** The gauge progress label font and color. */
	protected String		   _gaugeFontProgress;

	/**
	 * Class constructor of a Gauge object for the struts target.
	 *
	 * @param gauge the generic Gauge defined for every type of view targets
	 */
	public LyStrutsGauge(LyGauge gauge)
	{
		super(gauge);
		_gauge = gauge;

		initialize();
	}

	/**
	 * Initialization method.<br>
	 * Sets colors and font with values defined in resources files.
	 */
	public void initialize()
	{
		_progressColor = getHexColor(_gauge.getProgressColor());
		_backgroundColor = getHexColor(_gauge.getColor());
		_progressLabelColor = getHexColor(_gauge.getFontColor());
		
		if (_progressLabelColor == null)
			_progressLabelColor = _backgroundColor;

		_gaugeFontProgress = getFontString(_gauge.getFont(), _progressLabelColor);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	    pw.startComponent(this);
	       
		updateRefreshInfo(pw, true);

		int    nMin = _gauge.getNMin();
		int    nMax = _gauge.getNMax();
		int    nValue = _gauge.getNValue();
		
		if (nMin < 0)
			nMin = 0;
		
		if (nMax < 0)
			nMax = 100;
		
		if (nValue < 0)
			nValue = 0;
		
		String label = _gauge.getLabel();
		float  percent = ((float)nValue) / (nMax - nMin);
		int	   totalWidth = 150;

		if ((label == null) || (label.length() == 0))
		{
			int     prop = (int)(percent * 100);
			Integer propInteger = new Integer(prop);

			label = propInteger.toString();
			label = label + "%";
		}

		// add Css Scripts to set color and font
		pw.addCssScript("font.progressLabel{" + _gaugeFontProgress + "}", false);
		pw.addCssScript("td.progressColor{background-color:" + _progressColor + ";}", false);
		pw.addCssScript("td.progressBackColor{background-color:" + _backgroundColor + ";}", false);

		// Add a 1 pixel offset in Firefox to take into account the div border
		int			progressWidth = (int)(totalWidth * percent);

		// Write a first table to draw a border around the gauge
		if (!pw.writeDivOnly())
			pw.append("<div id='gaugeDiv_" + _gauge.getIdCode() + "'>");
		
		pw.append("<table id='gaugeTable_" + _gauge.getIdCode() + "' height='20' border='0' cellspacing='0' cellpadding='0'>\n");
		pw.append("<tr>\n");
		pw.append("<td align='left' nowrap='yes' width='" + totalWidth + "'>\n");

		pw.append("<div style='position:absolute;'>");

		// DIV with the background color
		pw.append("<div style='position:absolute;z-index:1;height:21;width:" +
				(totalWidth + (getWebViewManager().getBrowser() == LyWebViewManager.BROWSER_MSIE ? 0 : 1)) + ";background-color:" + _backgroundColor + ";'>");
		pw.append("</div>\n");

		// DIV with the label
		pw.append("<div class='gauge' style='width:" + totalWidth + ";'>");
		pw.append("<table height='100%' width='100%' cellpadding='0' cellspacing='0'><tr><td align='center'><font class='progressLabel'>");
		pw.append(label);
		pw.append("</font></td></tr></table></div>\n");

		// DIV with the progress color
		pw.append("<div style='width:" + progressWidth + ";height:21;background-color:");
		pw.append(_progressColor);
		pw.append(";z-index:2;position:absolute;'></div>\n");

		pw.append("</div>");
		
		pw.append("&nbsp;\n");
		pw.append("</td>\n");
		pw.append("</tr>\n");
		pw.append("</table>\n");
		
		if (!pw.writeDivOnly())
			pw.append("</div>");
		
	    pw.endComponent(this);
	} // end method writeHtml

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.setinterface.LySetLabelInterface#setLabel(java.lang.String)
	 */
	public void setLabel(String label)
	{
		refresh();
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.graphic.LyGraphicalGauge#setNValue(int)
	 */
	public void setNValue(int nValue)
	{
		refresh();
	}
	
	/**
	 * Refresh frame containing this web object.
	 */
	public void refresh()
	{
		LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();
		String			   id = "gaugeDiv_" + _gauge.getIdCode();

		viewManager.addReloadDiv(_lastFrameName, id, this);
	}
} // end class LyStrutsGauge
