/*
 * LyStrutsHRef.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the HRef object (as described in the dtd) for web viewer.
 *
 * <p><b>HREF: Input Field of Hyperlink Type.</b><br>
 * An input field of Hyperlink type allows to display a graphical component containing an hyperlink.
 * This link allows to launch an action by a simple click. It does not contain any element.</p>
 *
 * @version $Id: LyStrutsHRef.java 35209 2012-04-12 08:14:02Z jlsaouli $
 * @author  Lyria S.A.
 */
public class LyStrutsHRef extends LyStrutsFieldItem
    implements LyGraphicalHRef
{
	public static final String __VERSION =
		"$Id: LyStrutsHRef.java 35209 2012-04-12 08:14:02Z jlsaouli $";

	/** The <b>location://</b> String value used within Path representation. */
    public static String       LOCATION = "location://";

    /** The HRef generic component. */
    protected LyHRef           _hRef;
    
	/**
	 * Constructor.
	 *
	 * @param hRef the href component
	 */
	public LyStrutsHRef(LyHRef hRef)
	{
        super(hRef);
        _hRef = hRef;

        if (_formField == null)
        {
            LyComponent grandParent = hRef.getParent().getParent();

            if (grandParent instanceof LyFormField)
                _formField = (LyFormField)grandParent;
        }
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.struts.LyStrutsHRef#getBaseUrl(leon.view.web.LyWebPageWriter)
	 */
	protected String getBaseUrl(LyWebPageWriter pw)
	{
		// get the application context url.
		return ((LyWebViewManager)getViewManager()).getRootURL();
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.struts.LyStrutsHRef#getLink()
	 */
	protected String getLink()
	{
		// get the url to the event action
		LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();
		String			    actionUrl = viewManager.getActionURL(LyStrutsConstants.PATH_ACTION_GRAPHICEVENT);

		return actionUrl;
	} // end method getLink
	
	/**
	 * Returns the url sent to Javascript function for popup menus.
	 * 
	 * @return the graphical action URL.
	 */
	protected String getPopupMenuUrl()
	{
		// get the url to the event handler
		LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();
		String			    actionUrl = viewManager.getActionURL(LyStrutsConstants.PATH_ACTION_GRAPHICEVENT);
		
		return actionUrl;
	}
	
	   /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
     */
    public void writeHtml(LyWebPageWriter pw)
    {
        pw.startComponent(this);
        
        updateRefreshInfo(pw);

        String image = getImage();

        // Add needed script
        pw.addBeginScript("<script type='text/javascript'>window.defaultStatus='';</script>",
            false);

        String baseUrl = getBaseUrl(pw);

        pw.addBeginScript("<script type='text/javascript'>\n//<![CDATA[\nvar _rootUrl='" + baseUrl
            + "';\n//]]>\n</script>", false);

        pw.append("<table border='0' cellspacing='0' cellpadding='1'>\n");
        pw.append("<tr valign='middle'>\n");

        if (image != null)
        {
            pw.append("<td width='0'>");
            writeImage(pw);
            pw.append("</td>\n");
        }
        
        LyForm  form = _hRef.getParentForm();
        
        if ((form != null)
            && !pw.isPrintMode())
        {
            String id = _hRef.getId();
            
            id = LyString.getJavascriptString(id);
    
            if (pw.writeDivOnly())
                id = LyString.replace(id, "\\", "\\\\");
            
            pw.append("<td valign='middle' nowrap='yes' onContextMenu=\"InvokePopup('");
            pw.append(id);
            pw.append("','");
            pw.append(id);
            pw.append("','" + getPopupMenuUrl() + "','");
            pw.append(form.getController() + "',event.ctrlKey, event.clientX, event.clientY);");
            pw.append("return false;\">");
        }
        else
            pw.append("<td>");

        writeLabel(pw);

        pw.append("</td>\n</tr>\n");
        pw.append("</table>\n");
        
        pw.endComponent(this);
    } // end method writeHtml

    /**
     * Writes HTML description corresponding to the href image.
     *
     * @param pw the page writer
     */
    public void writeImage(LyWebPageWriter pw)
    {
        String image = getImage();

        if (image == null)
            return;

        if (_hRef.isSensitive() && !pw.isPrintMode())
        {
            String label = _hRef.getLabel();
            String toolTip = getJavascriptString("LY_LINK_TO" + " " + label);

            pw.append("<a href=\"javascript:InvokeAction('" + _hRef.getParentFormFieldId() + "','"
                    + getLink() + "','" + LyWebConstants.RELOAD_FRAMES + "','" + _hRef.getRootController() + "','"
                    + LyString.replace(_hRef.getId(), "\\", "\\\\") + "','href');");

            if (LyWebConstants.TOOL_POPUP.equals(pw.getActiveName()))
            {
                LyComponent component = pw.getViewComponent();
                LyComponent parent = null;

                if (component instanceof LyView)
                {
                    LyArea area = ((LyView)component).getArea();
                    parent = area.getParent();
                }
                else
                {
                    parent = component.getParent();
                }

                pw.append("javascript:SetDivVisibility('" + "popupText_" + parent.getIdCode()
                    + "', false);");
            }

            pw.append("\"");
            pw.append(" onmouseover=\'window.status=\"" + toolTip + "\";return true\'>");
        }

        String imageSize = getImageSize();

        pw.append("<img src='" + image + "'  align='middle' border='0'");
        if (imageSize != null)
            pw.append(" " + imageSize);
        pw.append("/>");

        if (_hRef.isSensitive() && !pw.isPrintMode())
            pw.append("</a>");
    } // end method writeImage

    /**
     * Writes HTML description corresponding to the href label.
     *
     * @param pw the page writer
     */
    public void writeLabel(LyWebPageWriter pw)
    {
        String label = _hRef.getLabel();

        label = getHtmlString(label);

        if ((label == null) || (label.length() == 0))
            return;

        if (_hRef.isSensitive() && !pw.isPrintMode())
        {
            String appDoc = getEnv("LY_APP_DOC");
            String target = LyWebConstants.TARGET_FRAME;
            String id = _hRef.getId();

            if (id == null)
                id = "";
            
            if (LyWebViewManager.WEB_FILE_NEW_WINDOW)
                target = "_blank";

            if (id.startsWith("http://")
                || id.startsWith("https://")
                || id.startsWith(appDoc)) // case of a file: do not use javascript!
            {
                pw.append("<a href=\"" + id + "\" target=\"" + target + "\""); // Direct
                // download
            }
            else if (id.startsWith(LOCATION)) // it's a text/x-url type ->
            // Direct download
            {
                id = id.substring(LOCATION.length());
                pw.append("<a href=\"" + id + "\" target=\"" + target + "\""); // Direct
                // download
            }
            else
            {
                pw.append("<a href=\"javascript:InvokeAction('" + _hRef.getParentFormFieldId() + "','"
                        + getLink() + "','" + LyWebConstants.RELOAD_FRAMES + "','" + _hRef.getRootController() + "','"
                        + LyString.replace(id, "\\", "\\\\") + "','href');");

                if (LyWebConstants.TOOL_POPUP.equals(pw.getActiveName()))
                {
                    LyComponent component = pw.getViewComponent();
                    LyComponent parent = null;

                    if (component instanceof LyView)
                    {
                        LyArea area = ((LyView)component).getArea();
                        parent = area.getParent();
                    }
                    else
                    {
                        parent = component.getParent();
                    }

                    pw.append("javascript:SetDivVisibility('" + "popupText_" + parent.getIdCode()
                        + "', false);");
                }

                pw.append("\"");
            }
            String toolTip = getJavascriptString("LY_LINK_TO" + " " + label);

            pw.append(" onmouseover=\'window.status=\"" + toolTip + "\";return true\'>");
        }

        LyArea  parentArea = _hRef.getParentArea();
        LyArea  grandParentArea = (parentArea == null ? null : parentArea.getParentArea());
        boolean classGenerated = false;
        
        if ((parentArea != null) && (grandParentArea != null))
        {
            Object  bgImage = parentArea.getEnvironment().getColor("LY_HREF_COLOR");
            Object  parentBgImage = grandParentArea.getEnvironment().getColor("LY_HREF_COLOR");

            if ((bgImage != null) && (parentBgImage != null) && !bgImage.equals(parentBgImage))
            {
                String  font = null;
                
                if (!_hRef.isSensitive())
                    font = getFontString("LY_DEFAULT_FONT", "LY_HREF_DISABLE");
                else if (_hRef.isVisited())
                    font = getFontString("LY_DEFAULT_FONT", "LY_HREF_VISITED");
                else
                    font = getFontString("LY_DEFAULT_FONT", "LY_HREF_COLOR");

                pw.append("<font style='" + font + "'>" + label + "</font>");
                classGenerated = true;
            }
        }
        
        if (!classGenerated)
        {
            LyWebDecoration deco = getWebViewManager().getDecoration();

            pw.append("<font class='" + deco.getClassName(_hRef, pw) + "'>" + label + "</font>");
        }

        if (_hRef.isSensitive() && !pw.isPrintMode())
            pw.append("</a>");
    } // end method writeLabel

    /**
     * Generates the command URL link to open a consultation form on an object of the application.
     *
     * @param      pw Current page writer.
     * @return     a href request wich will be interpreted as a command if clicked by the user
     * @deprecated replaced by {@link #getLink()} because the parameter <code>pw</code> is not used
     */
    protected String getLink(LyWebPageWriter pw)
    {
        return getLink();
    }

    /**
     * Gets the HRef image.
     *
     * @return the image
     * @see    #setImage(String)
     */
    protected String getImage()
    {
		String			 path;
		String			 docDir;
		String			 docRoot = expandUrl(getEnv("LY_APP_DOC"));
		String			 image = _hRef.getImage();

		if (image == null)
			return null;
		image = expandUrl(image);
		docDir = expandUrl(getEnv("LY_APP_DOC_DIR"));

		if (image.startsWith(docDir))
		{
			image = image.substring(docDir.length());
			image = getEnv("LY_APP_DOC") + image;
		}
		else
		{
			docDir = expandUrl(getEnv("LY_LEON_DOC_DIR"));
			if (image.startsWith(docDir))
			{
				image = image.substring(docDir.length());
				image = getEnv("LY_LEON_DOC") + image;
			}
		}

		if (image.startsWith(docRoot))
			path = expandUrl(image);
		else if (image.startsWith("file"))
			path = LyString.encodeFileURL(image);
		else if (image.startsWith("http") || image.startsWith("ftp"))
			path = image;
		else
			path = getImageUrl(image);
		
		return path;
    }

    /**
     * Gets the HRef image size.
     *
     * @return the image size
     */
    protected String getImageSize()
    {
        String image = _hRef.getImage();

        if (image == null)
            return null;

        int[] size = getImageSize(image);

        if (size == null)
            return null;

        String imageSize = null;

        if (size.length == 2)
            imageSize = "width='" + size[0] + "' height='" + size[1] + "'";
        else
            imageSize = "height='" + size[0] + "'";

        return imageSize;
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.setinterface.LySetImageInterface#setImage(java.lang.String)
     */
    public void setImage(String image)
    {
        refresh();
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.setinterface.LySetSensitiveInterface#setSensitive(boolean)
     */
    public void setSensitive(boolean sensitive)
    {
        refresh();
    }

    /**
     * (Modification) Accessor (LySetVisitedInterface interface).
     */
    public void setVisited(boolean visited)
    {
        refresh();
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebObject#refresh()
     */
    public void refresh()
    {
        LyComponent parent = _hRef.getParent();

        if (parent instanceof LyFormField)
        {
            LyFormField       formField = (LyFormField)parent;
            LyStrutsFormField webField = (LyStrutsFormField)formField.getInternalView();

            webField.refreshDivContent();
        }
        else
            super.refresh();
    }
} // end class LyStrutsHRef
