/*
 * LyStrutsHeaders.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import java.util.*;

import leon.misc.*;
import leon.view.*;
import leon.view.LyTable.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Headers object (as described in the dtd) for web viewer.
 *
 * <p><b>HEADERS: Headers line</b><br>
 * A headers allows to display header cells at the beginning of a list. A header does not contain
 * any element: it is only a non-empty list of header cells or sub-headers.</p>
 *
 * @version $Id: LyStrutsHeaders.java 23300 2009-12-17 13:56:41Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.LyHeaders
 */
public class LyStrutsHeaders extends LyWebObject
	implements LyGraphicalHeaders
{
	public static final String __VERSION =
		"$Id: LyStrutsHeaders.java 23300 2009-12-17 13:56:41Z jlsaouli $";

	/** The headers generic component. */
	protected LyHeaders		   _headers;

	/** Image for extended sort button. */
	protected String		   _extendedSortButton;

	/** Image for simple sort button. */
	protected String		   _simpleSortButton;

	/** Tooltip for sort mode swaping. */
	protected String		   _swapSortMode;

	/** Rowspan to be used for header of this headers (for optimization). */
	protected int			   _rowspan;

	/**
	 * Class constructor of a Headers object for the struts target.
	 *
	 * @param headers the generic Headers defined for every type of view targets
	 */
	public LyStrutsHeaders(LyHeaders headers)
	{
		super(headers);
		_headers = headers;
		initialize();
	}

	/**
	 * Initialization method.<br>
	 * Sets default value to the row span of header components.<br>
	 * Sets font, colors, images and tooltip with values defined in resources files.
	 */
	public void initialize()
	{
		_rowspan = 1;
		_swapSortMode = translate("LY_SWAP_SORT_MODE");
		_swapSortMode = getHtmlString(_swapSortMode);

	}

	/**
	 * Writes the needed styleSheet for this component.
	 *
	 * @param pw the pageWriter
	 */
	protected void writeStyleSheet(LyWebPageWriter pw)
	{
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	    pw.startComponent(this);
	    
		String    align = LyStrutsTable.getHtmlHAligment(_headers.getAlign());
		String    label = translate(_headers.getLabel());

		LyString.getJavascriptString(label);

		computeRowSpan();
		
		String htmlLabel = getHtmlString(label);
		String colspan = " colspan='" + _headers.getHeaderCount() + "'";

		pw.append("<td class='header' nowrap='true'");
		pw.append(align);
		pw.append(colspan);
		pw.append(">");
		pw.append(htmlLabel);
		pw.append("</td>\n");
		
		pw.endComponent(this);
	}

	/**
	 * Writes the HTML representation of this headers.
	 *
	 * @param pw           the page writer
	 * @param drawImages   indicates if there is an image to draw
	 * @param sortOnStatus indicates if there is a sort on status
	 * @param topHeaders   the top headers
	 * @deprecated
	 * @see   leon.view.web.LyWebComponent#writeHtml(LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw, boolean drawImages, boolean sortOnStatus,
		boolean topHeaders)
	{
		writeHtml(pw, drawImages, sortOnStatus);
	}

	/**
	 * Writes the HTML representation of this headers.
	 *
	 * @param pw           the page writer
	 * @param drawImages   indicates if there is an image to draw
	 * @param sortOnStatus indicates if there is a sort on status
	 * @see   leon.view.web.LyWebComponent#writeHtml(LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw, boolean drawImages, boolean sortOnStatus)
	{
		if (_headers.isFreed() || _headers.isFreeing())
			return;
		pw.startComponent(this);

		LyTable			table = _headers.getParentTable();
		LyStrutsTable	webTable = (LyStrutsTable)table.getInternalView();
		LyComponent		subComponent;
		LyWebComponent	webComp;
		boolean   checkboxSelection = false ;//testResource("LY_DHTML_TABLE_ROW_SELECTION_BY_CHECKBOX");
    
    // Set the checkbox selection value
    if(table.getSelectionMode() == SelectionMode.SINGLE_CHECKBOX 
        || table.getSelectionMode() == SelectionMode.MULTIPLE_CHECKBOX ) {
      checkboxSelection = true;
    }
    
		computeRowSpan();

		Hashtable<Integer, Vector<LyComponent>>	subComponents = new Hashtable<Integer, Vector<LyComponent>>();
		
		getSubComponents(subComponents, 0);

		// Write styleSheet
		writeStyleSheet(pw);
		pw.append("<tr>");

		if (webTable.generateExternalBorder(pw))
		{
			String	topLeft = getEnvironment().getImageUrl("LY_TABLE_TOP_LEFT", false, false);

			pw.append("<td width='1' rowspan='" + _rowspan + "' align='right' float='right' valign='top' class='tableTopLeft'>");
			pw.append("<img src='" + topLeft + "'/>");
			pw.append("</td>");
		}

		// Add the checkbox to select a row
		if(checkboxSelection)
		{
			pw.append("<td class='header'");

			if (!_headers.isDefaultColor())
			{
				pw.append(" style='background-image:none;");
				pw.append("background-color:" + getHexColor(_headers.getColor()) + ";'");
			}

			pw.append(" rowspan='" + _rowspan + "'>&nbsp;</td>\n");
		}
		
		// Add a cell for images
		if (drawImages && !sortOnStatus)
		{
			pw.append("<td class='header'");

			if (!_headers.isDefaultColor())
			{
				pw.append(" style='background-image:none;");
				pw.append("background-color:" + getHexColor(_headers.getColor()) + ";'");
			}

			pw.append(" rowspan='" + _rowspan + "'>&nbsp;</td>\n");
		}
		


		int		i, j, depth = getDepth();
		
		for (i = 0; i <= depth; i++)
		{
			Vector<LyComponent>	componentList = subComponents.get(new Integer(i));
			
			for (j = 0; (componentList != null) && (j < componentList.size()); j++)
			{
				subComponent = componentList.elementAt(j);
				if (subComponent.isFreed() || subComponent.isFreeing())
					continue;
				webComp = (LyWebComponent)subComponent.getInternalView();
				webComp.writeHtml(pw);
			}
			
			String localStyle = "";
			
			if (!_headers.isDefaultColor())
	    	{
	    		localStyle += " background-image:none;";
	    		localStyle += " background-color:"+getHexColor(_headers.getColor())+";";
	    	}
			
			if (localStyle.trim().length()>0)
				localStyle = " style=\"" + localStyle + "\"";
			
			if (i == 0)
			{
				if (webTable.addLastCell(pw))
					pw.append("<td class='emptyHeader' " + localStyle + " rowspan='" + _rowspan + "'>&nbsp;</td>\n");

				if (webTable.generateExternalBorder(pw))
				{
					String		topRight = getEnvironment().getImageUrl("LY_TABLE_TOP_RIGHT", false, false);
					String		topRightBottom = getEnvironment().getImageUrl("LY_TABLE_TOP_RIGHT_BOTTOM", false, false);

					pw.append("<td width='1' rowspan='" + _rowspan + "' align='left' float='left' valign='top' class='tableTopRight'>");
					pw.append("<div style='position:relative;'><div style='left:0px;top:0px;'><img src='" + topRight + "'/></div>");
					pw.append("<div style='position:absolute;left:0px;bottom:0px;' class='tableTopRight'><img src='" + topRightBottom + "'/></div></div>");
					pw.append("</td>");
				}
			}
			
			if (i < depth)
				pw.append("</tr>\n<tr>\n");
		}

		pw.append("</tr>\n");
		
		pw.endComponent(this);
	} // end method writeHtml
	
	/**
	 * Retrieve the list of components at given depth.
	 * 
	 * @param subComponents the list of components to be populated.
	 * @param depth the depth of components to put in the hastable.
	 */
	protected void getSubComponents(Hashtable<Integer, Vector<LyComponent>> subComponents, int depth)
	{
		LyComponent			subComponent;
		LyStrutsHeaders		webComp;
		int					i, nComponents = _headers.getComponentCount();
		Vector<LyComponent>	test = subComponents.get(new Integer(depth));

		for (i = 0; i < nComponents; i++)
		{
			subComponent = _headers.getComponent(i);
			
			if (subComponent instanceof LyHeaders)
			{
				webComp = (LyStrutsHeaders)subComponent.getInternalView();
				webComp.getSubComponents(subComponents, depth + 1);
			}
			
			if (test == null)
				test = new Vector<LyComponent>(1);
			
			test.addElement(subComponent);
			subComponents.put(new Integer(depth), test);
		}
	}

	/**
	 * Builds the string of sorted columns.<br>
	 * Example of the string format: "<code>sort+2-3+6</code>" indicates that colums 2 and 6 have an
	 * increasing sort and the column 6 has a decreasing sort. Other columns are not sorted.
	 *
	 * @return string of sorted columns
	 */
	public String getSortedCols()
	{
		// Build the string of sorted columns (format:'+/- ncol' ; eg:-1+5)
		LyHeader header;
		int		 nHeaders = _headers.getHeaderCount();
		String   sortedCols = "sort";
		int		 sort;
		boolean  increasing;

		for (int i = 0; i < nHeaders; i++)
		{
			header = _headers.getHeader(i);
			sort = header.getSort();
			increasing = header.isIncreasing();

			if (sort >= 0)
			{
				if (increasing)
					sortedCols += "+";
				else
					sortedCols += "-";
				sortedCols += i;
			}
		}

		return sortedCols;
	}

	/**
	 * Gets the depth of current sub-headers in its parent table component.
	 *
	 * @return the depth of current sub-headers in its parent table component.
	 */
	protected int getDepthInTable()
	{
		int		    depth = -1;
		LyComponent parent = _headers;

		while (!(parent instanceof LyTable) && (parent != null))
		{
			depth++;
			parent = parent.getParent();
		}

		return depth;
	}

	/**
	 * Gets the depth of sub-headers for current table component.
	 *
	 * @return the depth of sub-headers.
	 */
	protected int getDepth()
	{
		if (_headers == null)
			return 0;

		int			   i, n = _headers.getHeadersCount();
		int			   subDepth, depth = 0;
		LyHeaders	   subHeaders;
		LyStrutsHeaders subComponent;

		for (i = 0; i < n; i++)
		{
			subHeaders = _headers.getHeaders(i);
			subComponent = (LyStrutsHeaders)subHeaders.getInternalView();
			subDepth = subComponent.getDepth();

			if (subDepth >= depth)
				depth = subDepth + 1;
		}

		return depth;
	}

	/**
	 * Computes row span.
	 */
	protected void computeRowSpan()
	{
		LyTable		 table = _headers.getParentTable();
		LyStrutsTable htmlTable = (LyStrutsTable)table.getInternalView();
		int			 tableDepth = htmlTable.getHeadersDepth();
		int			 headersDepth = getDepthInTable();

		_rowspan = 1 + tableDepth - headersDepth;
	}

	/**
	 * Gets row span.
	 *
	 * @return row span
	 */
	public int getRowSpan()
	{
		return _rowspan;
	}

} // end class LyStrutsHeaders
