/*
 * LyStrutsIcon.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Icon object (as described in the dtd) for web viewer.
 *
 * <p><b>ICON: Icon</b><br>
 * An icon allows to define an image to which a text can be associated.</p>
 *
 * @version $Id: LyStrutsIcon.java 24563 2010-03-04 08:15:19Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.LyIcon
 */
public class LyStrutsIcon extends LyStrutsFieldItem
	implements LyGraphicalIcon
{
	public static final String __VERSION =
		"$Id: LyStrutsIcon.java 24563 2010-03-04 08:15:19Z jlsaouli $";

	/** The icon generic component. */
	protected LyIcon		   _icon;

	/**
	 * Class constructor of an Idon object for the struts target.
	 *
	 * @param icon the generic Icon defined for every type of view targets
	 */
	public LyStrutsIcon(LyIcon icon)
	{
		super(icon);
		_icon = icon;

	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
		updateRefreshInfo(pw);
		
		// FFT00067 : if no image, do not display anything
		String image = _icon.getImage();

		if ((image == null) || (image.length() == 0))
			return;

		pw.startComponent(this);
		
		// In all cases, we must use url expanding strings before making
		// comparisons
		// to be sure to have the same file separators.
		String			 label = translate(_icon.getLabel());
		int				 height = _icon.getHeight();
		int				 width = _icon.getWidth();
		boolean			 border = (_icon.getBorder() != LyIcon.Border.NONE);
		String			 path = getImagePath(image);

		label = getHtmlString(label);

		if (border)
			pw.append(
				"<table style='border: solid 1px #888888;' cellspacing='0' cellpadding='0'><tr><td  align='center'>\n");

		pw.append("<img name=\"icon_" + _icon.getIdCode() + "\" id=\"icon_" + _icon.getIdCode()
				+ "\" src=\"" + path + "\" hspace='0' vspace='0' title=\"" + label + "\"");
		
		if (height > 0)
			pw.append(" height=" + height);
		if (width > 0)
			pw.append(" width=" + width);
		pw.append("/>");

		if (border)
			pw.append("</td></tr></table>\n");
		
		pw.endComponent(this);
	} // end method writeHtml

	/**
	 * Returns the path associated with the given image.
	 * 
	 * @param image the image that must be displayed.
	 * @return the path associated with the given image.
	 */
	protected String getImagePath(String image)
	{
		String			 path;
		String			 docDir;
		String			 docRoot = expandUrl(getEnv("LY_APP_DOC"));

		image = expandUrl(image);
		docDir = expandUrl(getEnv("LY_APP_DOC_DIR"));

		if (image.startsWith(docDir))
		{
			image = image.substring(docDir.length());
			image = getEnv("LY_APP_DOC") + image;
		}
		else
		{
			docDir = expandUrl(getEnv("LY_LEON_DOC_DIR"));
			if (image.startsWith(docDir))
			{
				image = image.substring(docDir.length());
				image = getEnv("LY_LEON_DOC") + image;
			}
		}

		if (image.startsWith(docRoot))
			path = expandUrl(image);
		else if (image.startsWith("file"))
			path = LyString.encodeFileURL(image);
		else if (image.startsWith("http") || image.startsWith("ftp"))
			path = image;
		else
			path = getImageUrl(image);
		
		return path;
	}
	
	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetImageInterface#setImage(java.lang.String)
	 */
	public void setImage(String image)
	{
		LyWebViewManager viewManager = getWebViewManager();

		// Manage Ajax mode
		StringBuffer    buffer = new StringBuffer();
		String			label = translate(image);
		String			path = getImagePath(image);

		label = getHtmlString(label);
		
		// Update the image
		buffer.append("<script type='text/javascript'>SetImageSrc('icon_").append(_icon.getIdCode())
			.append("','").append(path).append("');</script>");
		buffer.append("<script type='text/javascript'>SetImageTooltip('icon_").append(_icon.getIdCode())
			.append("','").append(label).append("');</script>");
		
		if (LyWebConstants.POPUP_VIEW.equals(_lastWindowName))
			buffer.append("<script type='text/javascript'>checkSizePopupView();</script>");
		
		viewManager.addAdditionalRefreshScripts(buffer.toString());
	}
} // end class LyStrutsIcon
