/*
 * LyStrutsItem.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.setinterface.*;
import leon.view.web.*;


/**
 * Type : leon.view.web.struts.LyStrutsItem<br>
 * The peer component of LyItem for the Struts viewer.
 *
 * @version $Id: LyStrutsItem.java 30766 2011-05-04 13:31:17Z nhamon $
 * @author  Lyria S.A.
 */
public class LyStrutsItem extends LyWebObject
    implements LyGraphicalItem,
               LySetVisibleInterface
{
	public static final String __VERSION =
		"$Id: LyStrutsItem.java 30766 2011-05-04 13:31:17Z nhamon $";

	/** The item generic component. */
    protected LyItem           _item;

    /** Visibility of the item. */
    protected boolean          _visible;
    
	/**
	 * Constructor.
	 *
	 * @param item the Item component
	 */
	public LyStrutsItem(LyItem item)
	{
        super(item);
        _item = item;
        _visible = true;
	}

	/**
	 * Indicates whether a specific case is made for print items.
	 */
	protected boolean managePrintItem()
	{
		return false;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.struts.LyStrutsItem#getItemUrl(leon.view.web.LyWebPageWriter)
	 */
	protected String getItemUrl()
	{
		// get the item id
		String itemId = _item.getId();

		// get the target struts action
		String actionPath = LyStrutsConstants.PATH_ACTION_GRAPHICEVENT;

		if (itemId.equals(LyWebConstants.SELECT_ALL)
				|| itemId.equals(LyWebConstants.SELECT_ITEM)
				|| itemId.equals(LyWebConstants.UNSELECT_ALL)
				|| itemId.equals(LyWebConstants.UNSELECT_ITEM))
			actionPath = LyStrutsConstants.PATH_ACTION_SELECTEVENT;
		else if (itemId.equals(LyWebConstants.ACTION_ABOUT))
			actionPath = LyStrutsConstants.PATH_ACTION_ABOUT;

		// get the url to the action
		LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();
		String			    actionUrl = viewManager.getActionURL(actionPath);

		return actionUrl;
	} // end method getItemUrl
	
	   /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
     */
    public void writeHtml(LyWebPageWriter pw)
    {
        pw.startComponent(this);
        
        pw.append(writeContent(pw, null));
        
        pw.endComponent(this);
    }
    
    /**
     * Returns the HTML content for this menu component.
     * 
     * @param pw page writer
     * @param frameVar name of the Javascript var used for frame containing menubar
     * @return the HTML string representing the content of this menu component.
     */
    protected String writeContent(LyWebPageWriter pw, String frameVar)
    {
        if (pw != null)
            updateRefreshInfo(pw);

        if (!_visible)
            return "";
        
        StringBuffer buffer = new StringBuffer();
        String       label = _item.getLabel();
        String       controllerId = _item.getRootController();
        String       itemId = _item.getId();
        String       url = getItemUrl();
        StringBuffer action = new StringBuffer();
        String       image = null;
        String       imgKey = _item.getImage();
        int[]        img_sizes = getImageSize(imgKey);
        LyWebDecoration deco = getWebViewManager().getDecoration();
        String          imgClass = deco.getClassName(_item, pw);
        
        itemId = LyString.replace(itemId, "\\", "\\\\\\\\");

        if ((imgKey == null) && !_item.isChecked())
            imgKey = "LY_EMPTY_IMAGE";
        
        if (imgKey != null)
        {
            if (_item.isEnabled())
                image = getImageUrl(imgKey);
            else
                image = getDisabledImageUrl(imgKey);
        }
        
        // Use default sizes
        if (img_sizes == null)
            img_sizes = new int[] { 16 };

        StringBuffer imageBuffer = new StringBuffer();

        if (_item.isChecked())
        {
            if (image == null)
                imageBuffer.append("&nbsp;");
            
            imageBuffer.append("<img src=\"");
            
            if (_item.isEnabled())
                imageBuffer.append(getImageUrl("LY_MENU_CHECKED"));
            else
                imageBuffer.append(getDisabledImageUrl("LY_MENU_CHECKED"));

            imageBuffer.append("\" border=\"0\"");
            
            if (imgClass != null)
                imageBuffer.append(" class=\"" + imgClass + "\"");

            imageBuffer.append("/>");
        }

        if (image != null)
        {
            if (_item.isChecked())
                imageBuffer.append("&nbsp;");
            
            imageBuffer.append("<img src=\"");
            imageBuffer.append(image);
            imageBuffer.append("\" width=\"");
            imageBuffer.append(Integer.toString(img_sizes[0]));
            imageBuffer.append("\" border=\"0\"");
            
            if (imgClass != null)
                imageBuffer.append(" class=\"" + imgClass + "\"");

            imageBuffer.append("/>");
        }
        else
            imageBuffer.append("&nbsp;");

        String target = "\"" + LyWebConstants.RELOAD_FRAMES + "\"";

        // Specific for printing, see LyStrutsTool
        if (managePrintItem()
                && itemId.startsWith("_print") && !itemId.startsWith("_printXSLFO")
                && !itemId.startsWith("_print" + LyWebConstants.CONTROLLER_ID_SEP)
                && !itemId.equals("_print"))
        {
            action.append("javascript:InvokePrint(\"");
            action.append(LyWebObject.encodeUrl(url));
            action.append("\",null,\"");
            action.append(controllerId);
            action.append("\",\"");
            action.append(itemId);
            action.append("\")");
        }
        else
        {
            action.append("javascript:InvokeAction(\"");
            action.append(itemId);
            action.append("\",\"");
            action.append(LyWebObject.encodeUrl(url));
            action.append("\",");
            action.append(target);
            action.append(",\"" + controllerId + "\")");
        }

        if ((label != null) && (label.length() > 0))
        {
            label = translate(label);

            // Add "..." if dialog item
            if (_item.isDialog() && !label.endsWith("..."))
                label += "...";
			
			label = leon.misc.LyString.replace(label, "\n", "<br />");
            
            String  tooltip = leon.misc.LyString.replace(label, "'", "\\'");
            
            label = leon.misc.LyString.replace(label, "'", "&#39;");
            
            if (_item.isEnabled())
            {
                buffer.append("['" + imageBuffer + "','" + label + "', '" + action + "', null, '" + tooltip + "']");
            }
            else
            {
                buffer.append("[");

                if (frameVar != null)
                    buffer.append(frameVar + ".");
                
                buffer.append("_cmNoClick, '<td class=\"LeonardiMenuItemLeft\">" + imageBuffer + "</td>");
                buffer.append("<td colspan=\"2\" nowrap=\"yes\"><div class=\"LeonardiDisabledMenuItem\">");
                buffer.append(label);
                buffer.append("</div></td>', null, null, '");
                buffer.append(label);
                buffer.append("']");
            }
        }
        
        return buffer.toString();
    } // end method writeContent
    
    /*
     * (non-Javadoc)
     * @see leon.view.setinterface.LySetEnabledInterface#setEnabled(boolean)
     */
    public void setEnabled(boolean enabled)
    {
        refresh();
    }

    /*
     * (non-Javadoc)
     * @see leon.view.graphic.LyGraphicalItem#setChecked(boolean)
     */
    public void setChecked(boolean checked)
    {
        refresh();
    }

    /*
     * (non-Javadoc)
     * @see leon.view.setinterface.LySetImageInterface#setImage(java.lang.String)
     */
    public void setImage(String image)
    {
        refresh();
    }

    /*
     * (non-Javadoc)
     * @see leon.view.setinterface.LySetLabelInterface#setLabel(java.lang.String)
     */
    public void setLabel(String label)
    {
        refresh();
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebObject#refresh()
     */
    public void refresh()
    {
        // In DHTML the refresh is more clever : we refresh the parent DIV  content
        LyItem          item = (LyItem)_viewComponent;
        LyMenu          menu = (LyMenu)item.getParent(); // xx something else in DTD?
        LyViewComponent component = menu.getInternalView(false);

        if (component instanceof LyWebObject)
            ((LyWebObject)component).refresh();
        else
            super.refresh();
    }

    /**
     * Compute the URL of the item.<br>
     * Depending on the action of the item (printing, selection, action), the resulting URL may
     * vary.
     *
     * @param      pw the page writer.
     * @return     The URL of the menu item
     * @deprecated replaced by {@link #getItemUrl()} because parameter <code>pw</code> is not used.
     */
    protected String getItemUrl(LyWebPageWriter pw)
    {
        return getItemUrl();
    }
    
    /*
     * (non-Javadoc)
     * @see leon.view.setinterface.LySetVisibleInterface#setVisible(boolean)
     */
    public void setVisible(boolean status)
    {
		boolean	wasVisible = _visible;
		
		_visible = status;
		
		if (_visible != wasVisible)
			refresh();
    }
} // end class LyStrutsItem
