/*
 * LyStrutsLabel.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Label object (as described in the dtd) for web viewer.
 *
 * <p><b>LABEL: Input Field of Label Type</b><br>
 * An input field of label type allows to display a simple, non-editable label inside a form. It is
 * used in the consultation forms to show different types of fields (date, number, choice,...). It
 * can be made up of one or several lines. It does not comprise any element.</p>
 *
 * @version $Id: LyStrutsLabel.java 23300 2009-12-17 13:56:41Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.LyLabel
 */
public class LyStrutsLabel extends LyStrutsFieldItem
	implements LyGraphicalLabel
{
	public static final String __VERSION =
		"$Id: LyStrutsLabel.java 23300 2009-12-17 13:56:41Z jlsaouli $";

	/** The label generic component. */
	protected LyLabel		   _label;

	/**
	 * Class constructor of a Label object for the struts target.
	 *
	 * @param label the generic Label defined for every type of view targets.
	 */
	public LyStrutsLabel(LyLabel label)
	{
		super(label);
		_label = label;
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.struts.LyStrutsFieldItem#setLabel(java.lang.String)
	 */
	public void setLabel(String label)
	{
		LyComponent parent = _label.getParent();

		if (parent instanceof LyFormField)
		{
			LyFormField		 formField = (LyFormField)parent;
			LyStrutsFormField webField = (LyStrutsFormField)formField.getInternalView();

			webField.refreshDivContent();
		}
		else
			refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	    pw.startComponent(this);
	    
		updateRefreshInfo(pw, true);

		String  label = _label.getLabel();
		String  font = _label.getFont();
		String  fontColor = _label.getFontColor();
		String  color = _label.getColor();
		boolean isUrlRecurse;
		boolean isUrl;
		boolean isMail = false;
		// FFT00180/IpBoss : Chanes de caractres contenant @
		if ((label != null) && (label.indexOf(' ') < 0) && (label.indexOf('@') > 0)
				&& (label.indexOf("mailto:") < 0))
		{
			// Checks that the domain part of the label is an IP address
			String domain = label.substring(label.lastIndexOf('@') + 1);
			String[] domainParts = domain.split("\\.");
			boolean isIp = true;
			if (domainParts.length == 4)
			{
				for (int i = 0; i < domainParts.length && isIp; i++)
				{
					String part = domainParts[i];
					isIp &= part.matches("(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)");
				}
			}
			// If the domain is not an IP address, then the label is an email
			if (!isIp)
				isMail = true;
		}
		LyWebDecoration	deco = getWebViewManager().getDecoration();
		String			className = deco.getClassName(_label, pw);

		label = getHtmlString(label);

		// Contains one or more http:// or starts with www.
		isUrlRecurse = !isMail && (label != null)
				&& ((label.indexOf("http://") >= 0)
				    || (label.indexOf("https://") >= 0)
				    || (label.indexOf("mailto:") >= 0));
		isUrl = (label != null) && (isUrlRecurse || (label.indexOf("www.") == 0));

		String style = "";
		
		if (!_label.isDefaultFont() || !_label.isDefaultFontColor())
			style += getFontString(font,fontColor);
		
		if (!_label.isDefaultColor())
		{
		  style += " background-color:"+getHexColor(color)+";";
		  style += " background-image:none;";
		}
		
		String			onMouseOverFunctions = _label.getEventFunctions(LyComponent.EventType.MOUSE_ENTERED, ';');
		String			onMouseOutFunctions = _label.getEventFunctions(LyComponent.EventType.MOUSE_EXITED, ';');
		boolean			generateTable = (_label.getImage() != null) || !(_label.getParent().getParent() instanceof LyFormField)
			|| (onMouseOverFunctions.length() > 0) || (onMouseOutFunctions.length() > 0);
		LyConstraint	constraint = null;
		String			width = "", height = "";
		
		// If a constraint is set on the label, use the nCols and nRows values to compute
		// label size in pixels.
		if ((_label.getParent() instanceof LyFormField)
				&& (((LyFormField)(_label.getParent())).getConstraint() != null))
		{
			constraint = ((LyFormField)(_label.getParent())).getConstraint();
			
			if (constraint.getNCols() > 0)
			{
				width = constraint.getNCols() + "";
				generateTable = true;
			}
			else if ((constraint.getFill() == LyConstraint.Fill.HORIZONTAL) || (constraint.getFill() == LyConstraint.Fill.BOTH))
				width = "100%";
			
			if (constraint.getNRows() > 0)
			{
				height = constraint.getNRows() + "";
				generateTable = true;
			}
			else if ((constraint.getFill() == LyConstraint.Fill.VERTICAL) || (constraint.getFill() == LyConstraint.Fill.BOTH))
				height = "100%";
		}
		
		if (generateTable)
		{
			pw.append("<table border='0' cellpadding='0' cellspacing='0' ");
			
			if (onMouseOverFunctions.length() > 0)
				pw.append("onmouseover=\"" + onMouseOverFunctions + "\" ");
			
			if (onMouseOutFunctions.length() > 0)
				pw.append("onmouseout=\"" + onMouseOutFunctions + "\" ");
			
			if (width.length() > 0)
				style += "width:" + width + ";";
			
			if (height.length() > 0)
				style += "height:" + height + ";";

			if (style.trim().length() > 0)
				pw.append(" style='" + style + "'");

			pw.append("><tr><td nowrap='yes' ");

			if (_label.getHAlign() == LyLabel.HAlign.RIGHT)
				pw.append("align='right'");
			else if (_label.getHAlign() == LyLabel.HAlign.CENTER)
				pw.append("align='center'");
			else
				pw.append("align='left'");
			
			pw.append(">");

			if (_label.getImage() != null)
			{
				pw.append("<img src='" + getImageUrl(_label.getImage()) + "'/>");
				pw.append("</td><td nowrap='yes'>");
			}
		}
		
		pw.append("<font class='" + className + "'");

		if (style.trim().length() > 0)
			pw.append(" style='" + style + "'");

		pw.append(">");
		
		if (label == null)
			pw.append("&nbsp;");
		else
		{
			if (isMail && !pw.isPrintMode())
				pw.append("<a href='mailto:" + label + "' class='" + className + "'>");
			else if (isUrl)
			{
				String target = LyWebConstants.TARGET_FRAME;

				if (LyWebViewManager.WEB_FILE_NEW_WINDOW)
					target = "_blank";

				if (isUrlRecurse) {
					pw.append(replaceUrl(label, target, pw.isPrintMode())); 
				}
				else if (!pw.isPrintMode()) {
					pw.append("<a href='http://" + label + "' target=\"" + target + "\">");
				}
			}

			if (!isUrlRecurse)
			{
				pw.append(label);

				if ((isMail || isUrl) && !pw.isPrintMode())
					pw.append("</a>");
			}

			pw.append("\n");
		}

		pw.append("</font>");
		
		if (generateTable)
			pw.append("</td></tr></table>");
		
		pw.endComponent(this);
	} // end method writeHtml

	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetFontInterface#setFont(java.lang.String)
	 */
	public void setFont(String font)
	{
		refresh();
	}
	
	/*
     * (non-Javadoc)
     * @see leon.view.setinterface.LySetImageInterface#setImage(java.lang.String)
     */
    public void setImage(String key)
    {
        LyComponent parent = _label.getParent();

        if (parent instanceof LyFormField)
        {
            LyFormField      formField = (LyFormField)parent;
            LyStrutsFormField dhtmlField = (LyStrutsFormField)formField.getInternalView();

            dhtmlField.refreshDivContent();
        }
        else
            refresh();
    }
} // end class LyStrutsLabel
