/*
 * LyStrutsLayout.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;
import leon.view.web.struts.map.*;


/**
 * This class implements the Layout object (as described in the dtd) for web viewer.
 *
 * <p><b>LAYOUT: Layout Manager</b><br>
 * A layout manager allows to place nodes of a map according to a predefined layout. In most cases,
 * the coordinates of nodes are not used.</p>
 *
 * @version $Id: LyStrutsLayout.java 16172 2009-01-06 11:04:44Z jls $
 * @author  Lyria
 * @see     leon.view.LyLayout
 */
public class LyStrutsLayout extends LyWebObject
	implements LyGraphicalLayout
{
	public static final String     __VERSION =
		"$Id: LyStrutsLayout.java 16172 2009-01-06 11:04:44Z jls $";

	/** The layout generic component. */
	protected LyLayout			   _layout;

	/** Reference to the layout manager. */
	protected LyStrutsLayoutManager _layoutManager;

	/**
	 * Class constructor of a Layout object for the struts target.
	 *
	 * @param layout the generic Layout defined for every type of view targets.
	 */
	public LyStrutsLayout(LyLayout layout)
	{
		super(layout);
		_layout = layout;
		_layoutManager = null;
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
		// Nothing to do
	}

	/**
	 * Builds the appropriate layout manager switch the layout style.
	 *
	 * @param      map parameter not used
	 * @return     the appropriate layout manager
	 * @deprecated replaced by {@link #layoutFactory()} because the parameter <code>map</code> is
	 *             not used.
	 */
	public LyStrutsLayoutManager layoutFactory(LyStrutsMap map)
	{
		return layoutFactory();
	}

	/**
	 * Builds the appropriate layout manager switch the layout style.
	 *
	 * @return the appropriate layout manager
	 * @since  leonardi V4.0, replace {@link #layoutFactory(LyStrutsMap)} where the parameter was not
	 *         used.
	 */
	public LyStrutsLayoutManager layoutFactory()
	{
		if (_layout.getStyle() == LyLayout.Style.MATRIX)
			_layoutManager = new LyStrutsMatrixLayout(_layout.getHAlign(), _layout.getVAlign(),
					_layout.getDeltaNode(), _layout.getDeltaBorder());
		else if (_layout.getStyle() == LyLayout.Style.ELLIPSE)
			_layoutManager = new LyStrutsEllipseLayout(_layout.getDeltaBorder(), false);
		else if (_layout.getStyle() == LyLayout.Style.CIRCLE)
			_layoutManager = new LyStrutsEllipseLayout(_layout.getDeltaBorder(), true);
		else if (_layout.getStyle() == LyLayout.Style.LINEAR)
			_layoutManager = new LyStrutsLinearLayout(_layout.getDeltaNode(),
					_layout.getDeltaBorder(), _layout.getFill(), _layout.getHAlign(),
					_layout.getVAlign());
		else if (_layout.getStyle() == LyLayout.Style.TREEMAP)
			_layoutManager = new LyStrutsTreemapLayout(_layout.getBorderSizes(),
					_layout.getSelectBorderSize(), _layout.getLabelStyles());

		return _layoutManager;
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalLayout#setBorderSizes(java.lang.String)
	 */
	public void setBorderSizes(String borderSizes)
	{
		if (_layout.getStyle() != LyLayout.Style.TREEMAP)
			return;

		((LyStrutsTreemapLayout)_layoutManager).setBorderSizes(borderSizes);
		refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalLayout#setLabelStyles(java.lang.String)
	 */
	public void setLabelStyles(String labelStyles)
	{
		if (_layout.getStyle() != LyLayout.Style.TREEMAP)
			return;

		((LyStrutsTreemapLayout)_layoutManager).setLabelStyles(labelStyles);
		refresh();
	}
} // end class LyStrutsLayout
