/*
 * LyStrutsLink.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Link object (as described in the dtd) for web viewer.
 *
 * <p><b>LINK: Link</b><br>
 * A link represents a possible value in an input field of Relation type.</p>
 *
 * @version $Id: LyStrutsLink.java 23300 2009-12-17 13:56:41Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.LyLink
 */
public class LyStrutsLink extends LyWebObject
	implements LyGraphicalLink
{
	public static final String __VERSION =
		"$Id: LyStrutsLink.java 23300 2009-12-17 13:56:41Z jlsaouli $";

	/** The link generic component. */
	protected LyLink		   _link;

	/**
	 * Class constructor of a Link object for the struts target.
	 *
	 * @param link the generic Link defined for every type of view targets.
	 */
	public LyStrutsLink(LyLink link)
	{
		super(link);

		_link = link;
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetSelectedInterface#setSelected(boolean)
	 */
	public void setSelected(boolean selected)
	{
		// Manage Ajax mode
		LyWebViewManager viewManager = getWebViewManager();
		LyRelation		 relation = (LyRelation)_link.getParent();
		LyStrutsRelation  htmlRelation = (LyStrutsRelation)relation.getInternalView();

		if (!htmlRelation._notify)
			return;
		
		// For Firefox do not use Ajax refresh since name of the select input generated in
		// the HTML code depends on the time (see the writeHtml() method in the
		// LyStrutsRelation class).
		if (!relation.isMultiple()
			&& (getWebViewManager().getBrowser() == LyWebViewManager.BROWSER_FIREFOX && getWebViewManager().getBrowserVersion() >= 4.0)
			&& (relation.getInternalView() instanceof LyStrutsRelation)
			&& (relation.getAutoCompletionMax() == 0))
		{
			String			 relationId = LyString.getJavascriptString(htmlRelation.getId());
			String			 linkId = LyString.getJavascriptString(_link.getId());
			LyStrutsForm		 form = htmlRelation.getHtmlForm();
			String			 formName = ((form == null) ? "default" : form.getName());
			StringBuffer     buffer = new StringBuffer();
			LyTabs			 tabs = _link.getParentTabs();
			boolean			 refresh = true;

			if (tabs != null)
			{
				LyStrutsTabs htmlTabs = (LyStrutsTabs)tabs.getInternalView();
				LyTab	    selectedTab = htmlTabs.getSelectedTab();
				LyTab	    fieldTab = _link.getParentTab();

				// Refresh only if field is displayed. Test if field tab is selected tab and its label
				// is not null (tab with a null label contains fields that are allways displayed)
				if ((selectedTab != null) && (fieldTab != null) && (fieldTab != selectedTab)
						&& (fieldTab.getLabel() != null) && (fieldTab.getLabel().length() > 0))
					refresh = false;
			}

			if (refresh)
			{
				buffer.append("<script type='text/javascript'>\n//<![CDATA[\nSelectOption('")
					.append(formName).append("','").append(relationId).append("','").append(linkId)
					.append("',").append(selected).append(");\n//]]>\n</script>");

				viewManager.addAdditionalRefreshScripts(buffer.toString());
			}
		}
		else
		{
			LyComponent parent = relation.getParent();

			if (parent instanceof LyFormField)
			{
				LyFormField		 formField = (LyFormField)parent;
				LyStrutsFormField webField = (LyStrutsFormField)formField.getInternalView();

				webField.refreshDivContent();
			}
			else
				refresh();
		}
	} // end method setSelected

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	    pw.startComponent(this);
	    
		LyComponent parent = _link.getParent();
		boolean     multiple = false;
		String	    label = _link.getLabel();

		label = getHtmlString(label);
		label = LyString.replace(label, " ", "&#160;");

		if (parent instanceof LyRelation)
			multiple = ((LyRelation)parent).isMultiple();

		String	linkId = _link.getId();
		
		if (multiple)
			linkId = LyString.replace(linkId, "\\n", LyWebParameterValues.HTML_LINK_CR_SEP);
		linkId = LyString.getHtmlEntitiesString(linkId);
		
		pw.append("\n<option value='" + linkId + "'");
		
		if (_link.isSelected() && !multiple)
			pw.append(" selected='true'");
		
		pw.append(">" + label + "</option>");
		
		pw.endComponent(this);
	}
} // end class LyStrutsLink
