/*
 * LyStrutsList.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts;

import leon.misc.*;
import leon.view.*;
import leon.view.getinterface.*;
import leon.view.graphic.*;
import leon.view.setinterface.*;
import leon.view.web.*;


/**
 * This class implements the List object (as described in the dtd) for web viewer.
 *
 * <p><b>LIST: Input Field of List Type</b><br>
 * An input field of List type allows to choose one or more values among a discrete list of values.
 * It is possible to specify if the selection is multiple or not as well as the height of the list.
 * </p>
 *
 * @version $Id: LyStrutsList.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria S.A.
 */
public class LyStrutsList extends LyStrutsFieldItem
    implements LyGraphicalList,
               LySetIdleInterface,
               LyGetValueInterface
{
	public static final String __VERSION =
		"$Id: LyStrutsList.java 28422 2010-11-03 12:03:38Z ehelmste $";

	   /** Indicates if the list is idle or not. */
    protected boolean          _idle;

    /** The list generic component. */
    protected LyList           _list;
    
	/**
	 * Constructor.
	 *
	 * @param table the list component.
	 */
	public LyStrutsList(LyList list)
	{
        super(list);
        
        _list = list;
        _idle = false;
	}

	/**
	 * Gets the Javascript code used when double-clicking on the list.
	 *
	 * @return the String describing the Javascript code when double-clicking on the list
	 */
	public String getDoubleClickScript()
	{
		String			name = getId();
		StringBuffer	script = new StringBuffer();

		// Retrieve action URL
		LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();
		String			    actionUrl = viewManager.getActionURL(LyStrutsConstants.PATH_ACTION_GRAPHICEVENT);
		
		script.append("javascript:InvokeAction('");
		script.append(name);
		script.append("','");
		script.append(actionUrl);
		script.append("','','");
		script.append(_list.getRootController());
		script.append("',this.value,null)");
		
		return script.toString();
	}
	
	   /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
     */
    public void writeHtml(LyWebPageWriter pw)
    {
        pw.startComponent(this);
        
        updateRefreshInfo(pw);

        String            name = getId();
        String            formName = getFormName(pw);
        boolean           needCheck = needCheck();
        int               size = _list.getNRows();
        int               i, nOption = _list.getListOptionCount();
        LyListOption      option;
        LyStrutsListOption internal;
        StringBuffer      initialValue = new StringBuffer();
        String            sep = LyWebParameterValues.HTML_ENCODED_SEP;

        // If size negative, use default value
        if (size <= 0)
            size = 5;

        for (i = 0; i < nOption; i++)
        {
            option = _list.getListOption(i);
            
            if ((option != null) && option.isSelected())
            {
                if (initialValue.length() > 0)
                    initialValue.append(sep);
                
                initialValue.append(option.getId());
            }
        }

        pw.append("<input type='hidden' name='" + name + "' value='" + initialValue.toString() + "'/>\n");
        pw.append("<select name='" + name + "_list' size='" + size + "'");

        if (_list.getSelectionMode() != LyList.SelectionMode.SINGLE)
            pw.append(" multiple='yes'");

        pw.addBeginScript("<script type='text/javascript'>var _controllerId='"
            + _list.getRootController() + "';</script>", false);
        pw.append(" onDblClick=\"");
        pw.append(getDoubleClickScript());
        pw.append("\"");

        pw.append(" onChange=\"javascript:UpdateList(window.document." + formName
                + ",'" + name + "','" + name + "_list','" + sep
                + "', " + needCheck + ",'" + _list.getRootController() + "');");

        pw.append("\">\n");

        for (i = 0; i < nOption; i++)
        {
            option = _list.getListOption(i);
			
			if (option != null)
			{
				internal = (LyStrutsListOption)option.getInternalView();
				
				if (internal != null)
					internal.writeHtml(pw);
			}
        }

        pw.append("</select>\n");
        
        pw.endComponent(this);
    } // end method writeHtml
    
    /*
     * (non-Javadoc)
     * @see leon.view.setinterface.LySetListOptionsInterface#addListOption(leon.view.LyListOption,
     *      int)
     */
    public void addListOption(LyListOption listOption, int pos)
    {
        if (_idle)
            return;

        LyWebViewManager viewManager = getWebViewManager();
        LyStrutsForm      form = getHtmlForm();
        String           formName = ((form == null) ? "default" : form.getName());
        StringBuffer     buffer = new StringBuffer();
        String           name = getId();
        String           itemLabel = listOption.getLabel();

        itemLabel = translate(itemLabel);
        itemLabel = LyString.replace(itemLabel, "'", "\\'");
        name = LyString.getJavascriptString(name);

        buffer.append("<script type='text/javascript'>\n//<![CDATA[\nAddSelectOption('").append(
            formName).append("','").append(name).append("_list','").append(LyString.getJavascriptString(
                listOption.getId())).append("','" + itemLabel).append("',").append(pos).append(
            ");\n//]]>\n</script>");

        viewManager.addAdditionalRefreshScripts(buffer.toString());
    }

    /*
     * (non-Javadoc)
     * @see leon.view.setinterface.LySetListOptionsInterface#removeListOption(leon.view.LyListOption)
     */
    public void removeListOption(LyListOption listOption)
    {
        if (_idle)
            return;

        LyWebViewManager viewManager = getWebViewManager();
        LyStrutsForm      form = getHtmlForm();
        String           formName = ((form == null) ? "default" : form.getName());
        StringBuffer     buffer = new StringBuffer();
        String           name = getId();

        name = LyString.getJavascriptString(name);

        buffer.append("<script type='text/javascript'>\n//<![CDATA[\nRemoveSelectOption('");
        buffer.append(formName);
        buffer.append("','");
        buffer.append(name);
        buffer.append("_list','");
        buffer.append(LyString.getJavascriptString(listOption.getId()));
        buffer.append("');\n//]]>\n</script>");

        viewManager.addAdditionalRefreshScripts(buffer.toString());
    }

    /*
     * (non-Javadoc)
     * @see leon.view.setinterface.LySetIdleInterface#setIdle(boolean)
     */
    public void setIdle(boolean status)
    {
        _idle = status;

        if (!status)
        {
            LyComponent parent = _list.getParent();

            // Manage Ajax mode
            if (parent instanceof LyFormField)
            {
                LyFormField      formField = (LyFormField)parent;
                LyStrutsFormField webField = (LyStrutsFormField)formField.getInternalView();

                webField.refreshDivContent();
            }
            else
                refresh();
        }
    }
    
    public String getValue()
    {
        int          i, n = _list.getListOptionCount();
        LyListOption listOption;
        StringBuffer sb = new StringBuffer();
        char         sep = getEnvironment().getString().SEP;

        for (i = 0; i < n; i++)
        {
            listOption = _list.getListOption(i);
            
            if (!listOption.isSelected())
                continue;
            
            if (sb.length() > 0)
                sb.append(sep);

            sb.append(listOption.getId());
        }

        if (sb.length() == 0)
            return null;
        else
            return (sb.toString());
    }
} // end class LyStrutsList
