/*
 * LyStrutsMapContent.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the MapContent object (as described in the dtd) for win viewer.
 *
 * <p><b>MAP_CONTENT: Map Content</b><br>
 * A map content allows to group together in a same group various elements that can be placed on a
 * map (nodes, links, images, regions).</p>
 *
 * @version $Id: LyStrutsMapContent.java 16172 2009-01-06 11:04:44Z jls $
 * @author  Lyria
 * @see     leon.view.LyMapContent
 */
public class LyStrutsMapContent extends LyWebObject
	implements LyGraphicalMapContent
{
	public static final String __VERSION =
		"$Id: LyStrutsMapContent.java 16172 2009-01-06 11:04:44Z jls $";

	/** The map content generic component. */
	protected LyMapContent     _mapContent;

	/**
	 * Class constructor of a Map Content for the struts target.
	 *
	 * @param mapContent the generic Map Content defined for every type of view targets
	 */
	public LyStrutsMapContent(LyMapContent mapContent)
	{
		super(mapContent);
		_mapContent = mapContent;
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalMapContent#addComponent(leon.view.LyComponent,
	 *      int)
	 */
	@SuppressWarnings("deprecation")
	public void addComponent(LyComponent component, int pos)
	{
		LyMap map = (LyMap)_mapContent.getParent();

		if (map != null)
		{
			LyStrutsMap webMap = (LyStrutsMap)map.getInternalView();

			webMap.addComponent(component, pos, null);
		}
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalMapContent#removeComponent(leon.view.LyComponent)
	 */
	public void removeComponent(LyComponent component)
	{
		LyMap map = (LyMap)_mapContent.getParent();

		if (map != null)
		{
			LyStrutsMap webMap = (LyStrutsMap)map.getInternalView();

			webMap.removeComponent(component);
		}
	}
} // end class LyStrutsMapContent
