/*
 * LyStrutsMapObject.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import java.awt.*;
import leon.misc.*;
import leon.view.*;
import leon.view.web.*;


/**
 * This class implements a map object for web viewer.
 *
 * @version $Id: LyStrutsMapObject.java 16172 2009-01-06 11:04:44Z jls $
 * @author  Lyria
 */
public abstract class LyStrutsMapObject extends LyWebObject
{
	public static final String __VERSION =
		"$Id: LyStrutsMapObject.java 16172 2009-01-06 11:04:44Z jls $";

	/**
	 * Gets the hexadecimal string representing the given color.
	 *
	 * @param  color The color to convert.
	 * @return The hexadecimal code for the color.
	 */
	public static String getHexString(Color color)
	{
		if (color == null)
			return "000000";

		return LyColors.getHexString(color.getRed(), color.getGreen(), color.getBlue());
	}

	/**
	 * Class constructor of a Map Object object for the struts target.
	 *
	 * @param mapObject the generic Map Object defined for every type of view targets
	 */
	public LyStrutsMapObject(LyComponent mapObject)
	{
		super(mapObject);
	}

	/**
	 * Retrieves a Color from the current environment.
	 *
	 * @param  key The ressource name (key) of the Color.
	 * @return A reference to the requested Color, or null if it could not be found.
	 * @see    LyEnvironment#getColor(String)
	 */
	public final Color getColor(String key)
	{
		return (Color)getEnvironment().getColor(key);
	}

	/**
	 * Retrieves a Font from the current environment.
	 *
	 * @param  key The ressource name (key) of the font.
	 * @return A reference to the requested Font, or a default font if it could not be found.
	 * @see    LyEnvironment#getFont(String)
	 */
	public final Font getFont(String key)
	{
		return (Font)getEnvironment().getFont(key);
	}

	/**
	 * Retrieves an image from the current environment.
	 *
	 * @param  key The ressource name (key) of the image.
	 * @return A reference to the image object, or null if it could not be found.
	 * @see    LyEnvironment#getImage(String)
	 */
	public final Image getImage(String key)
	{
		return (Image)getEnvironment().getImage(key);
	}
} // end class LyStrutsMapObject
