/*
 * LyStrutsMenu.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.setinterface.*;
import leon.view.web.*;


/**
 * This class implements the Menu object (as described in the dtd) for struts viewer.
 *
 * <p><b>MENU: Menu</b><br>
 * A menu contains a set of menu, sub-menus entries and separators.</p>
 *
 * @version $Id: LyStrutsMenu.java 23300 2009-12-17 13:56:41Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.LyMenu
 */

public class LyStrutsMenu extends LyWebObject
	implements LyGraphicalMenu,
	LySetVisibleInterface
{
	public static final String __VERSION =
		"$Id: LyStrutsMenu.java 23300 2009-12-17 13:56:41Z jlsaouli $";

	/** The menu generic component. */
	protected LyMenu		   _menu;

	/** Visibility of the item. */
	protected boolean		   _visible;

	/**
	 * Class constructor of a Menu object for the struts target.
	 *
	 * @param menu the generic Menu defined for every type of view targets
	 */
	public LyStrutsMenu(LyMenu menu)
	{
		super(menu);
		_menu = menu;
		_visible = true;
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	    pw.startComponent(this);
	    
		pw.append(writeContent(pw, null));
		
		pw.endComponent(this);
	}
	
	/**
	 * Returns the HTML content for this menu component.
	 * 
	 * @param pw page writer
	 * @param frameVar name of the Javascript var used for frame containing menubar
	 * @return the HTML string representing the content of this menu component.
	 */
	protected String writeContent(LyWebPageWriter pw, String frameVar)
	{
		if (pw != null)
			updateRefreshInfo(pw);

		if (!_visible)
			return "";
		
		int			   i, componentCount = _menu.getComponentCount();
		LyComponent    component;
		LyStrutsMenu   webMenu;
		LyStrutsItem   webItem;
		StringBuffer   buffer = new StringBuffer();
		
		String	label = translate(_menu.getLabel());
		String	tooltip = leon.misc.LyString.replace(label, "'", "\\'");
		
		label = leon.misc.LyString.replace(label, "'", "&#39;");
		
		StringBuffer	imageBuffer = new StringBuffer();
		
		if (_menu.getParent() instanceof LyMenuBar)
			imageBuffer.append("&nbsp;");
		else
		{
			imageBuffer.append("<img src=\"");
			imageBuffer.append(getImageUrl("LY_EMPTY_IMAGE"));
			imageBuffer.append("\" width=\"16\" border=\"0\"/>");
		}
		
		if (!_menu.isEnabled())
			label = "<font class=\"LeonardiDisabledMenuItem\">" + label + "</font>";
		
		buffer.append("['" + imageBuffer.toString() + "','" + label + "', null, null, '" + tooltip + "'");
		
		if (_menu.isEnabled())
		{
			for (i = 0; i < componentCount ; i++)
			{
				buffer.append(",");
				
				component = _menu.getComponent(i);
				
                if ((component == null)
                    || (component.getId() == LyComponent.SEP_ID))
                {
                    // it's a separator
                    if (frameVar != null)
                        buffer.append(frameVar + ".");
                    
                    buffer.append("_cmSplit");
                }
                else if (component instanceof LyMenu)
				{
					webMenu = (LyStrutsMenu)component.getInternalView();
					buffer.append(webMenu.writeContent(pw, frameVar));
				}
				else if (component instanceof LyItem)
				{
					webItem = (LyStrutsItem)component.getInternalView();
					buffer.append(webItem.writeContent(pw, frameVar));
				}
			}
		}
		
		buffer.append(']');
		
		return buffer.toString();
	} // end method writeContent

	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetEnabledInterface#setEnabled(boolean)
	 */
	public void setEnabled(boolean enabled)
	{
		refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalMenu#addComponent(leon.view.LyComponent,
	 *      int)
	 */
	public void addComponent(LyComponent component, int pos)
	{
		refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalMenu#removeComponent(leon.view.LyComponent)
	 */
	public void removeComponent(LyComponent component)
	{
		refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebObject#refresh()
	 */
	public void refresh()
	{
		LyComponent		parent = _menu.getParent();
		
		while ((parent != null) && !(parent instanceof LyMenuBar))
		{
			parent = parent.getParent();
		}
		
		if (parent instanceof LyMenuBar)
		{
			LyMenuBar		menuBar = (LyMenuBar)parent;
			LyStrutsMenuBar	webMenuBar = (LyStrutsMenuBar)menuBar.getInternalView();
			
			webMenuBar.refresh();
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetVisibleInterface#setVisible(boolean)
	 */
	public void setVisible(boolean status)
	{
		boolean	wasVisible = _visible;
		
		_visible = status;
		
		if (_visible != wasVisible)
			refresh();
	}
} // end class LyStrutsMenu
