/*
 * LyStrutsMenuBar.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the MenuBar object (as described in the dtd) for struts viewer.
 *
 * <p><b>MENUBAR: Menu Bar</b><br>
 * A menu bar allows displaying a set of menus. It can be displayed in a main window of a Leonardi
 * application.</p>
 *
 * @version $Id: LyStrutsMenuBar.java 30778 2011-05-05 08:51:43Z nhamon $
 * @author  Lyria S.A.
 */
public class LyStrutsMenuBar extends LyWebObject
    implements LyGraphicalMenuBar
{
	public static final String __VERSION =
		"$Id: LyStrutsMenuBar.java 30778 2011-05-05 08:51:43Z nhamon $";

	/** The menu bar generic component. */
    protected LyMenuBar        _menuBar;
    
	/**
	 * Constructor.
	 *
	 * @param menuBar the menu bar component
	 */
	public LyStrutsMenuBar(LyMenuBar menuBar)
	{
        super(menuBar);
        
        _menuBar = menuBar;

        init();
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.struts.LyStrutsMenuBar#getBaseUrl(leon.view.web.LyWebPageWriter)
	 */
	protected String getBaseUrl(LyWebPageWriter pw)
	{
		// get the application context url.
		return ((LyWebViewManager)getViewManager()).getRootURL();
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.struts.LyStrutsMenuBar#getCloseUrl(leon.view.web.LyWebPageWriter)
	 */
	protected String getCloseUrl(LyWebPageWriter pw)
	{
		// get menu bar infos
		String			    controllerId = _menuBar.getRootController();

		// get the url to the event handler
		LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();
		String			    actionUrl = viewManager.getActionURL(LyStrutsConstants.PATH_ACTION_GRAPHICEVENT);

		// build the URL
		StringBuffer	    closeUrl = new StringBuffer();

		closeUrl.append(actionUrl);
		closeUrl.append("?");
		closeUrl.append(LyWebConstants.CONTROLLER);
		closeUrl.append("=");
		closeUrl.append(controllerId);
		closeUrl.append("&#38;");
		closeUrl.append(LyWebConstants.SOURCE);
		closeUrl.append("=");
		closeUrl.append(LyWebConstants.FRAME_CONTROLLER_DEMAND_CLOSE);

		return closeUrl.toString();
	}
	
	   /**
     * Initialization method.<br>
     * Sets the menu font.
     */
    public void init()
    {
    }
    
    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
     */
    public void writeHtml(LyWebPageWriter pw)
    {
        pw.startComponent(this);
        
        updateRefreshInfo(pw);

        // Now write menuBar
        LyWebDecoration deco = getWebViewManager().getDecoration();
        
        pw.append(
            "<table border='0' cellpadding='0' cellspacing='0' frame='hsides' width='100%' id='menuBackground'>\n");
        pw.append("<tr>\n<td>\n");
        pw.append("<div class='" + deco.getClassName(_menuBar, pw) + "' style='width:90%;' id='menubar_labels_"
            + _menuBar.getIdCode() + "'></div>\n");
        pw.append("</td>\n");

        // Generate Javascript content used to create the menubar
        String      content = writeContent(pw, null);
        
        pw.addBeginScript("<script type='text/javascript'>" + content + "</script>", false);
        pw.addEndScript("<script type='text/javascript'>cmLeonardi.effect = new CMSlidingEffect(10);" +
            "cmDraw ('menubar_labels_" + _menuBar.getIdCode() +
            "', mainMenu, 'hbr', cmLeonardi, 'Leonardi');</script>", false);

        pw.append("<td width='100%' align='right' valign='middle'>\n");

        String       controllerId = _menuBar.getRootController();
        String       baseUrl = getBaseUrl(pw);

        pw.addBeginScript("<script type='text/javascript'>\n//<![CDATA[\nvar _rootUrl='" + baseUrl
            + "';\n//]]>\n</script>", false);
        pw.addBeginScript("<script type='text/javascript'>var _controllerId='" + controllerId
            + "';</script>", false);

        if (!LyWebConstants.POPUP_VIEW.equals(pw.getWindowName()))
        {
            String       url = getCloseUrl(pw);
            String       javascriptToolTip = getJavascriptString("LY_CLOSE_VIEW");
            StringBuffer action = new StringBuffer();
            String       target = "'" + LyWebConstants.RELOAD_FRAMES + "'";
            String       buttonClose = getImageUrl("LY_CLOSE_WINDOW");
            String       buttonCloseOver = getImageUrl("LY_CLOSE_WINDOW_OVER");

            action.append("InvokeAction('");
            action.append(LyWebConstants.FRAME_CONTROLLER_DEMAND_CLOSE);
            action.append("','");
            action.append(LyWebObject.encodeUrl(url));
            action.append("',");
            action.append(target);
            action.append(",'" + controllerId + "')");

            pw.append("<table cellpadding='1' cellspacing='0' border='0'><tr><td><a href=\"javascript:" + action.toString());
            pw.append("\" onmouseover=\'");
            pw.append("rollover(");
            pw.append(LyWebConstants.FRAME_CONTROLLER_DEMAND_CLOSE);
            pw.append(",\"");
            pw.append(buttonCloseOver);
            pw.append("\",\"\");");
            pw.append("window.status=\"");
            pw.append(javascriptToolTip);
            pw.append("\";return true\'");
            pw.append(" onMouseOut=\'rollout(");
            pw.append(LyWebConstants.FRAME_CONTROLLER_DEMAND_CLOSE);
            pw.append(",\"");
            pw.append(buttonClose);
            pw.append("\",\"\");\'");
            pw.append(">");

            pw.append("<img align='middle' src='" + buttonClose + "'");
            pw.append(" name=\"");
            pw.append(LyWebConstants.FRAME_CONTROLLER_DEMAND_CLOSE);
            pw.append("\" ");
            pw.append(" border='0'/>");
            pw.append("</a></td></tr></table>");
        }

        pw.append("</td>\n");
        pw.append("<td>\n");
        pw.append("</td>\n</tr>\n</table>");
        
        pw.endComponent(this);
    } // end method writeHtml

    /**
     * Writes HTML description corresponding to sub-menu labels.
     *
     * @param pw page writer
     * @param frameVar name of the Javascript var used for frame containing menubar
     */
    public String writeContent(LyWebPageWriter pw, String frameVar)
    {
        int             i, menuCount = _menuBar.getMenuCount();
        LyMenu          menu;
        LyStrutsMenu    webMenu;
        StringBuffer    buffer = new StringBuffer();
        
        buffer.append("var mainMenu = [");

        for (i = 0; i < menuCount; i++)
        {
            if (i > 0)
                buffer.append(",");
            
            menu = _menuBar.getMenu(i);
            webMenu = (LyStrutsMenu)menu.getInternalView();
            buffer.append(webMenu.writeContent(pw, frameVar));
        }
        
        buffer.append(" ];");
        buffer.append("if ((typeof cmLeonardi != 'undefined') && (cmLeonardi != null))\n\tcmLeonardi.folderRight = '<img src=\"" + getImageUrl("LY_MENU_ARROW") + "\"/>';");
        
        return buffer.toString();
    } // end method writeContent

    /**
     * Force the value of the <code>_lastFrameName</code> variable.
     * 
     * @param frame the new value of the <code>_lastFrameName</code> variable.
     */
    public void setLastFrameName(String frame)
    {
        _lastFrameName = frame;
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.graphic.LyGraphicalMenuBar#addComponent(leon.view.LyComponent, int)
     */
    public void addMenu(LyMenu menu, int pos)
    {
        refresh();
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.graphic.LyGraphicalMenuBar#removeComponent(leon.view.LyComponent)
     */
    public void removeMenu(LyMenu menu)
    {
        refresh();
    }

    /**
     * Returns the Javascript content that must be used to refresh the menubar. 
     * 
     * @param  pw Current page writer.
     * @return the Javascript that performs a refresh of the menubar struts component.
     */
    public String getRefreshContent(LyWebPageWriter pw)
    {
        StringBuffer        buffer = new StringBuffer();
        String              content = writeContent(pw, "menuBarFrame");

        buffer.append("var menuBarFrame = findTarget(null, '" + _lastFrameName + "');");
        buffer.append("if ((menuBarFrame != null) && (typeof menuBarFrame.cmLeonardi != 'undefined') && (menuBarFrame.cmLeonardi != null)) {");
        buffer.append(content);
        buffer.append("menuBarFrame.mainMenu = mainMenu;");
        buffer.append("menuBarFrame.cmLeonardi.effect = new menuBarFrame.CMSlidingEffect(10);");
        buffer.append("menuBarFrame.cmDraw ('menubar_labels_" + _menuBar.getIdCode() +
            "', menuBarFrame.mainMenu, 'hbr', menuBarFrame.cmLeonardi, 'Leonardi');}");
        
        return buffer.toString();
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebObject#refresh()
     */
    public void refresh()
    {
        LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();

        viewManager.addReloadDiv(_lastFrameName, "menubar_labels_" + _menuBar.getIdCode(), this);
    } // end method refresh
} // end class LyStrutsMenuBar
