/*
 * LyStrutsMessage.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Message object (as described in the dtd) for struts viewer.
 *
 * <p><b>MESSAGE: Message</b><br>
 * A message is placed inside a message bar. It allows displaying a text and an image.</p>
 *
 * @version $Id: LyStrutsMessage.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria S.A.
 */
public class LyStrutsMessage extends LyWebObject
    implements LyGraphicalMessage
{
	public static final String __VERSION =
		"$Id: LyStrutsMessage.java 28422 2010-11-03 12:03:38Z ehelmste $";

	   /**
     * Gets the HTML horizontal alignment string corresponding to the Message generic component
     * alignment.
     *
     * @param  alignCode the LyMessage halign code ({@link LyMessage#Align.LEFT}, {@link
     *                   LyMessage#Align.CENTER} or {@link LyMessage#Align.RIGHT}).
     * @return the HTML horizontal alignment string.
     */
    protected static String getHtmlHAligment(LyMessage.Align alignCode)
    {
        if (alignCode == LyMessage.Align.LEFT)
            return "left";
        else if (alignCode == LyMessage.Align.RIGHT)
            return "right";
        else if (alignCode == LyMessage.Align.CENTER)
            return "center";
        else
            return null;
    }

    /** The message generic component. */
    protected LyMessage _message;
    
	/**
	 * Constructor.
	 *
	 * @param message the message component
	 */
	public LyStrutsMessage(LyMessage message)
	{
        super(message);
        
        _message = message;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.struts.LyStrutsMessage#getLinkUrl(leon.view.web.LyWebPageWriter)
	 */
	protected String getLinkUrl(LyWebPageWriter pw)
	{
		// get the url to the selection handler
		LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();
		String			    actionUrl = viewManager.getActionURL(LyStrutsConstants.PATH_ACTION_GRAPHICEVENT);
		String		 		controllerId = _message.getController();
		String		 		messageId = _message.getId();

		return "javascript:InvokeAction('" + messageId + "','" +
			actionUrl + "',null,'" + controllerId + "','" + messageId + "',null)";
	}
	
	   /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
     */
    public void writeHtml(LyWebPageWriter pw)
    {
        pw.startComponent(this);
        
        updateRefreshInfo(pw);

        String      label = _message.getLabel();
        String      image = null;
        String      imgKey = _message.getImage();
        LyComponent component = pw.getViewComponent();
        boolean     writeDiv = !pw.writeDivOnly() || (component != _message);
        LyWebDecoration deco = getWebViewManager().getDecoration();

        if (imgKey != null)
            image = getImageUrl(imgKey);

        if ((label == null) && (image == null))
        {
            pw.endComponent(this);
            return;
        }

        String align = getHtmlHAligment(_message.getAlign());
        int    size;
        int[]  img_sizes = getImageSize(imgKey);

        if (_message.getSize() > 0)
            size = _message.getSize();
        else
            size = 1;

        if (writeDiv)
        {
            pw.append("<td align='");
            pw.append(align);
            pw.append("' nowrap='true' width='");
            pw.append(Integer.toString(size));
            pw.append("%'>");
        }
        writeMessageBegin(pw);

        label = getHtmlString(label);

        if (image != null)
        {
            // Use default sizes
            if (img_sizes == null)
                img_sizes = new int[] { 15, 15 };

            // Add needed scripts
            pw.addBeginScript("<script type='text/javascript'>window.defaultStatus='';</script>",
                false);

            String linkUrl = getLinkUrl(pw);
            String controllerId = _message.getController();

            // Create a sub table to handle correctly middle alignment
            pw.append("<table border='0' cellspacing='0' cellpadding='0'><tr>");
            
            // Add an empty space on Firefox otherwise image may touch separator
            if (getWebViewManager().getBrowser() == LyWebViewManager.BROWSER_FIREFOX)
                pw.append("<td width='4'></td>");
            
            pw.append("<td>");

            if (controllerId != null)
            {
                pw.append("\n<a href=\"");
                pw.append(linkUrl);
                pw.append("\"");
                pw.append(" onMouseOver=\"window.status='';return true\">");
            }

            pw.append("\n<img src='");
            pw.append(image);
            pw.append("'  align='middle' width='");
            pw.append(Integer.toString(img_sizes[0]));
            pw.append("' height='");
            pw.append(Integer.toString(img_sizes[1]));
            pw.append("' border='0'/>");

            if (controllerId != null)
                pw.append("</a>");

            pw.append("</td><td nowrap='true'>");
        }

        pw.append("<font class='" + deco.getClassName(_message, pw) + "'>&nbsp;");
        pw.append(label);
        pw.append("</font>");

        if (image != null)
            pw.append("</td></tr></table>");
        writeMessageEnd(pw);

        if (writeDiv)
            pw.append("</td>");
        
        pw.endComponent(this);
    } // end method writeHtml

    /**
     * Writes the HTML description corresponding to the message begining.
     *
     * @param pw the page writer
     */
    protected void writeMessageBegin(LyWebPageWriter pw)
    {
        LyComponent component = pw.getViewComponent();
        boolean     writeDiv = !pw.writeDivOnly() || (component != _message);

        if (writeDiv)
            pw.append("<div id='message_" + _message.getIdCode() + "'>\n");
    }

    /**
     * Writes the HTML description corresponding to the message end.
     *
     * @param pw the page writer
     */
    protected void writeMessageEnd(LyWebPageWriter pw)
    {
        LyComponent component = pw.getViewComponent();
        boolean     writeDiv = !pw.writeDivOnly() || (component != _message);

        if (writeDiv)
            pw.append("\n</div>\n");
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebObject#refresh()
     */
    public void refresh()
    {
        // In DHTML the refresh is more clever : we refresh the DIV content
        String             id = "message_" + _message.getIdCode();
        LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();

        viewManager.addReloadDiv(_lastFrameName, id, this);
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.setinterface.LySetLabelInterface#setLabel(java.lang.String)
     */
    public void setLabel(String label)
    {
        refresh();
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.setinterface.LySetImageInterface#setImage(java.lang.String)
     */
    public void setImage(String image)
    {
        refresh();
    }

} // end class LyStrutsMessage
