/*
 * LyStrutsMessageWindow.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts;

import java.util.*;

import javax.servlet.http.*;

import leon.misc.*;
import leon.view.*;
import leon.view.event.*;
import leon.view.setinterface.*;
import leon.view.web.*;


/**
 * This class implements the Message window (as described in the dtd) for the Struts viewer.
 *
 * @version $Id: LyStrutsMessageWindow.java 35072 2012-03-28 12:27:27Z bangiole $
 * @author  Lyria S.A.
 */
public class LyStrutsMessageWindow implements LyMessageWindowInterface
{
	public static final String __VERSION =
		"$Id: LyStrutsMessageWindow.java 35072 2012-03-28 12:27:27Z bangiole $";

	/** View manager of the message. */
	LyViewManager _viewManager;

	/** Text of the message. */
	String _text;

	/** Title of the message. */
	String _title;

	/** Icon of the message. */
	String _icon;

	/** Background color of the message. */
	String _bgColor;

	/** Indicates whether this message is a question or not. */
	int    _type;

	/** Id of the info answer that must be called when answer is given. */
	String _infoId;

	/** Commands of the message. */
	BitSet _commands;

	/** The url of the question answer. */
	String _answerUrl;

	/**
	 * Constructor.
	 *
	 * @param viewManager the view manager
	 * @param answerUrl the URL that must be called to answer message
	 */
	public LyStrutsMessageWindow(LyViewManager viewManager, String answerUrl)
	{
		_viewManager = viewManager;
		_answerUrl = answerUrl;
	}

	/**
	 * Accessor to the client Data.
	 *
	 * @return the client data
	 */
	public Object getClientData()
	{
		return null;
	}

	/**
	 * Set the client data.
	 *
	 * @param clientData
	 */
	public void setClientData(Object clientData)
	{
		_infoId = (String)clientData;
	}

	/**
	 * Show the message window.
	 *
	 * @param status    the visible status of the message box
	 * @param newThread true if the window must be started in a new thread
	 */
	public void setVisible(boolean status, boolean newThread)
	{
	}

	/**
	 * Set the icon of the message box.
	 *
	 * @param icon
	 */
	public void setIcon(String icon)
	{
		_icon = icon;
	}

	/**
	 * Set the message displayed in the message box.
	 *
	 * @param message
	 */
	public void setMessage(String message)
	{
		_text = message;
	}

	/**
	 * Set the title of the message box.
	 *
	 * @param title
	 */
	public void setTitle(String title)
	{
		_title = title;
	}

	/**
	 * Add a listener on event of the message box.
	 *
	 * @param listener
	 */
	public void addActionListener(LyActionListener listener)
	{
	}

	/**
	 * Get the result of the message box (which button was pressed..).
	 *
	 * @return the result of the question
	 * @see    leon.view.LyViewManager
	 */
	public int getCommandResult()
	{
		return -1;
	}

	/**
	 * Set the commands (buttons) that are present in the message box.
	 *
	 * @param commands
	 * @see   leon.view.LyViewManager
	 */
	public void setCommands(BitSet commands)
	{
		_commands = commands;

		if ((_commands == null) || (_commands.size() == 0))
		{
			_commands = new BitSet();
			_commands.set(LyViewManager.COMMAND_OK);
		}
	}

	/**
	 * Set the background color of the message box.
	 *
	 * @param bgColor
	 */
	public void setBackgroundColor(String bgColor)
	{
		_bgColor = bgColor;
	}

	/**
	 * Set the modal state of the message window.
	 *
	 * @param modal
	 */
	public void setModal(boolean modal)
	{
	}

	/**
	 * Initialize the window This method is called when all properties are set.
	 */
	public void initialize()
	{
	}

	/**
	 * Free the window. This method is called to hide and free the window.
	 */
	public void free()
	{
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
	    LyEnvironment   environment = _viewManager.getEnvironment();
		StringBuffer	script = new StringBuffer();
		String			title = LyWebViewManager.encodeString(_title);
		String			text = LyWebViewManager.encodeString(LyString.getHtmlString(_text));
		String			icon = LyWebViewManager.encodeString(_icon);
		String			left = environment.getImageUrl("LY_COMMAND_LEFT");
		String			right = environment.getImageUrl("LY_COMMAND_RIGHT");
		String			leftOver = environment.getImageUrl("LY_COMMAND_LEFT_OVER");
		String			rightOver = environment.getImageUrl("LY_COMMAND_RIGHT_OVER");
		LyWebRequestInfo		info = LyStrutsServlet.getWebRequestInfo();
		HttpServletRequest		request = (info == null ? null : info.getRequest());
		String					target = (request == null ? null : (String)request.getParameter(LyWebConstants.TARGET_PARAM));

		script.append("<script type='text/javascript'>\n//<![CDATA[\n");
		script.append("var values = new Array();\n");
		script.append("var labels = new Array();\n");
		script.append("var images = new Array();\n");

		if (_commands.get(LyViewManager.COMMAND_OK))
		{
			script.append("values[values.length] = '" + LyViewManager.COMMAND_OK + "';");
			script.append("labels[labels.length] = '" + environment.translate("LY_OK") + "';");
			script.append("images[images.length] = '" + environment.getImageUrl("LY_OK", false, false) + "';");
		}
		if (_commands.get(LyViewManager.COMMAND_YES))
		{
			script.append("values[values.length] = '" + LyViewManager.COMMAND_YES + "';");
			script.append("labels[labels.length] = '" + environment.translate("LY_YES") + "';");
			script.append("images[images.length] = '" + environment.getImageUrl("LY_YES", false, false) + "';");
		}
		if (_commands.get(LyViewManager.COMMAND_ALL))
		{
			script.append("values[values.length] = '" + LyViewManager.COMMAND_ALL + "';");
			script.append("labels[labels.length] = '" + environment.translate("LY_ALL") + "';");
			script.append("images[images.length] = '" + environment.getImageUrl("LY_ALL", false, false) + "';");
		}
		if (_commands.get(LyViewManager.COMMAND_NONE))
		{
			script.append("values[values.length] = '" + LyViewManager.COMMAND_NONE + "';");
			script.append("labels[labels.length] = '" + environment.translate("LY_NONE") + "';");
			script.append("images[images.length] = '" + environment.getImageUrl("LY_NONE", false, false) + "';");
		}
		if (_commands.get(LyViewManager.COMMAND_NO))
		{
			script.append("values[values.length] = '" + LyViewManager.COMMAND_NO + "';");
			script.append("labels[labels.length] = '" + environment.translate("LY_NO") + "';");
			script.append("images[images.length] = '" + environment.getImageUrl("LY_NO", false, false) + "';");
		}
		if (_commands.get(LyViewManager.COMMAND_CANCEL))
		{
			script.append("values[values.length] = '" + LyViewManager.COMMAND_CANCEL + "';");
			script.append("labels[labels.length] = '" + environment.translate("LY_CANCEL") + "';");
			script.append("images[images.length] = '" + environment.getImageUrl("LY_CANCEL", false, false) + "';");
		}
		if (_commands.get(LyViewManager.COMMAND_HELP))
		{
			script.append("values[values.length] = '" + LyViewManager.COMMAND_HELP + "';");
			script.append("labels[labels.length] = '" + environment.translate("LY_HELP") + "';");
			script.append("images[images.length] = '" + environment.getImageUrl("LY_HELP", false, false) + "';");
		}

		script.append("ShowMessageWindow(\"" + title + "\", \"" + icon +
			"\", \"" + text + "\", values, labels, images, \"" + _infoId + "\", \"" + _answerUrl + "\", \""
			+ left + "\", \"" + leftOver + "\", \"" + right + "\", \"" + rightOver + "\", \"" + target
			+ "\", \"center\");");
		script.append("\n//]]>\n</script>");

		return script.toString();
	}
} // end class LyStrutsMessageWindow
