/*
 * LyStrutsMessages.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Messages object (as described in the dtd) for struts viewer.
 *
 * <p><b>MESSAGES: Message Bar</b><br>
 * A message bar can be used in a main window to provide the user with information about the
 * displayed data (number of objects, printing in progress...).</p>
 *
 * @version $Id: LyStrutsMessages.java 24563 2010-03-04 08:15:19Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.LyMessages
 */
public class LyStrutsMessages extends LyWebObject
	implements LyGraphicalMessages
{
	public static final String __VERSION =
		"$Id: LyStrutsMessages.java 24563 2010-03-04 08:15:19Z jlsaouli $";

	/** Constant used to generate CSS class name for border. */
	public static int		   DECO_BORDER_PARAMETER = 0;

	/** Constant used to generate CSS class name for messages. */
	public static int		   DECO_MESSAGES_PARAMETER = 1;

	/** The messages generic component. */
	protected LyMessages	   _messages;

	/**
	 * Class constructor of a Messages object for the struts target.
	 *
	 * @param messages the generic Messages defined for every type of view targets.
	 */
	public LyStrutsMessages(LyMessages messages)
	{
		super(messages);
		_messages = messages;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	    pw.startComponent(this);
	    
		if (!pw.writeDivOnly())
			pw.append("<div id='messages_" + _messages.getIdCode() + "'>\n");

		updateRefreshInfo(pw);

		if (pw.isPrintMode())
		{
		    pw.endComponent(this);
			return;
		}

		int			   i, n = _messages.getMessageCount();
		LyMessage	   message;
		LyStrutsMessage webMessage;
		String		   messageSeparator = getImageUrl("LY_MESSAGE_SEPARATOR");
		String		   color = getHexColor("LY_MESSAGES_BACKGROUND");

		if (color == null)
			color = getHexColor("LY_DEFAULT_BACKGROUND");
		if (color != null)
			// Set the background color
			pw.setBackgroundColor(color, pw.getActiveName());

		LyWebDecoration deco = getWebViewManager().getDecoration();

		deco.writePageDecoration(pw, LyWebDecoration.WINDOW_BEGIN);
		deco.writePageDecoration(pw, LyWebDecoration.WINDOW_TOP);
		deco.writePageDecoration(pw, LyWebDecoration.WINDOW_LEFT);

		pw.append("<table class='" + deco.getClassName(_messages, DECO_BORDER_PARAMETER, pw) +
			"' width='100%' height='100%' border='0' cellpadding='0' cellspacing='0'><tr><td>\n");
		pw.append("<table class='" + deco.getClassName(_messages, DECO_MESSAGES_PARAMETER, pw) +
			"' width='100%' height='100%' cellpadding='0' cellspacing='0' border='0'><tr>\n");
		
		for (i = 0; i < n; i++)
		{
			if (i > 0)
				pw.append("<td align='right'><img src='" + messageSeparator + "'/></td>");
			
			message = _messages.getMessage(i);

			// Separator
			if (message == null)
				pw.append("<td></td>");
			else
			{
				webMessage = (LyStrutsMessage)message.getInternalView();
				webMessage.writeHtml(pw);
				pw.append("\n");
			}
		}
		
		pw.append("</tr></table>");
		pw.append("</td></tr></table>");
		deco.writePageDecoration(pw, LyWebDecoration.WINDOW_RIGHT);
		deco.writePageDecoration(pw, LyWebDecoration.WINDOW_BOTTOM);
		deco.writePageDecoration(pw, LyWebDecoration.WINDOW_END);

		if (!pw.writeDivOnly())
			pw.append("</div>\n");
		
		pw.endComponent(this);
	} // end method writeHtml

	/*
	 * (non-Javadoc)
	 *
	 * @see leon.view.web.LyWebObject#refresh()
	 */
	public void refresh()
	{
		LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();
		String			   id = "messages_" + _messages.getIdCode();

		viewManager.addReloadDiv(_lastFrameName, id, this);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see leon.view.graphic.LyGraphicalMessages#addMessage(leon.view.LyMessage,
	 *      int)
	 */
	public void addMessage(LyMessage message, int pos)
	{
		refresh();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see leon.view.graphic.LyGraphicalMessages#removeMessage(leon.view.LyMessage)
	 */
	public void removeMessage(LyMessage message)
	{
		refresh();
	}
} // end class LyStrutsMessages
