/*
 * LyStrutsNotice.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Notice object (as described in the dtd) for web viewer.
 *
 * <p><b>NOTICE: Information Panel</b><br>
 * An information panel allows displaying an insert having an informative purpose. It is possible to
 * assign an icon, a label and a progress bar to an information panel.</p>
 *
 * @version $Id: LyStrutsNotice.java 23300 2009-12-17 13:56:41Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.LyNotice
 */
public class LyStrutsNotice extends LyStrutsPanel
	implements LyGraphicalNotice
{
	public static final String __VERSION =
		"$Id: LyStrutsNotice.java 23300 2009-12-17 13:56:41Z jlsaouli $";

	/** The notice generic component. */
	protected LyNotice		   _notice;

	/**
	 * Class constructor of a Notice object for the struts target.
	 *
	 * @param notice the generic Notice defined for every type of view targets.
	 */
	public LyStrutsNotice(LyNotice notice)
	{
		super(notice);
		_notice = notice;
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	    pw.startComponent(this);
	    
		LyIcon		  icon = _notice.getIcon();
		LyLabel		  label = _notice.getLabel();
		LyGauge		  gauge = _notice.getGauge();
		LyConstraint  constraint = _notice.getConstraint();
		String		  constraintString = "";
		
		if (constraint != null)
		{
			if ((constraint.getFill() == LyConstraint.Fill.BOTH)
					|| (constraint.getFill() == LyConstraint.Fill.HORIZONTAL))
				constraintString = " width='100%'";
			
			if ((constraint.getFill() == LyConstraint.Fill.BOTH)
					|| (constraint.getFill() == LyConstraint.Fill.VERTICAL))
				constraintString += " height='100%'";
		}

		pw.append("<center>\n<table border='0' cellspacing='10'");
		pw.append(constraintString);
		pw.append(">\n");
		pw.append("<tr  align='center'>\n");

		if (icon != null)
		{
			LyStrutsIcon webIcon = (LyStrutsIcon)icon.getInternalView();

			pw.append("<td>\n");
			webIcon.writeHtml(pw);
			pw.append("</td>\n");
		}

		if (label != null)
		{
			LyStrutsLabel webLabel = (LyStrutsLabel)label.getInternalView();

			pw.addCssScript("font.label{" + getFontString("LY_DEFAULT_FONT") + "}", false);
			pw.append("<td nowrap='yes'>\n");
			webLabel.writeHtml(pw);
			pw.append("</td>\n");
		}

		pw.append("</tr>\n<tr  align='center'>\n");

		if (gauge != null)
		{
			LyStrutsGauge webGauge = (LyStrutsGauge)gauge.getInternalView();

			pw.append("<td colspan='2'>\n");
			webGauge.writeHtml(pw);
			pw.append("</td>\n");
		}

		pw.append("</tr>\n");
		pw.append("</table>\n</center>\n");
		
		pw.endComponent(this);
	} // end method writeHtml
} // end class LyStrutsNotice
