/*
 * LyStrutsNumber.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import java.text.*;
import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Number object (as described in the dtd) for web viewer.
 *
 * <p><b>NUMBER: Input Field of Number Type</b><br>
 * An input field of Number type allows displaying a graphical component allowing to input or to
 * consult a number value. It does not comprise any element. It is possible to specify the minimum
 * and the maximum value, the incrementation step, the number of decimals and the unit.</p>
 *
 * @version $Id: LyStrutsNumber.java 23300 2009-12-17 13:56:41Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.LyNumber
 */
public class LyStrutsNumber extends LyStrutsFieldItem
	implements LyGraphicalNumber
{
	public static final String __VERSION =
		"$Id: LyStrutsNumber.java 23300 2009-12-17 13:56:41Z jlsaouli $";

	/** Image for increase button. */
	protected String		   _increaseButton;

	/** Image for decrease button. */
	protected String		   _decreaseButton;

	/** Disabled image for increase button. */
	protected String		   _disabledIncreaseButton;

	/** Disabled image for decrease button. */
	protected String		   _disabledDecreaseButton;

	/** Maximum number of digits used to represents the current number. */
	protected int			   _maxChars;

	/** Label of the tooltip shown aver the increase button. */
	protected String		   _labelIncrease;

	/** Label of the tooltip shown aver the decrease button. */
	protected String		   _labelDecrease;

	/** The number generic component. */
	protected LyNumber		   _number;

	/** The label associated to the number component. */
	protected String		   _label;

	/** The minimun value of the number. */
	protected double		   _min;

	/** The maximun value of the number. */
	protected double		   _max;

	/** The number of decimals that the number can have. */
	protected int			   _decimals;

	/**
	 * Class constructor of a Number object for the struts target.
	 *
	 * @param number the generic Number defined for every type of view targets.
	 */
	public LyStrutsNumber(LyNumber number)
	{
		super(number);
		_number = number;

		initialize();
	}

	/**
	 * Initialization method.<br>
	 * Sets button images with default images, the maximum number of chars with a default value and
	 * the label, min, max and decimals with the number generic component values.
	 */
	public void initialize()
	{
		_increaseButton = getImageUrl("LY_ARROW_UP_BUTTON");
		_decreaseButton = getImageUrl("LY_ARROW_DOWN_BUTTON");
		_disabledIncreaseButton = getDisabledImageUrl("LY_ARROW_UP_BUTTON");
		_disabledDecreaseButton = getDisabledImageUrl("LY_ARROW_DOWN_BUTTON");
		_maxChars = 1;
		_label = _number.getLabel();
		_min = _number.getVMin();
		_max = _number.getVMax();
		_decimals = _number.getDecimals();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.struts.LyStrutsFieldItem#setLabel(java.lang.String)
	 */
	public void setLabel(String label)
	{
		LyComponent parent = _number.getParent();

		// Manage Ajax mode
		if (parent instanceof LyFormField)
		{
			LyFormField		 formField = (LyFormField)parent;
			LyStrutsFormField webField = (LyStrutsFormField)formField.getInternalView();

			webField.refreshDivContent();
		}
		else
			refresh();
	}

	/**
	 * Sets the content of the number component.
	 */
	public void setContent()
	{
		// Retrieve the maximum number of chars

		double  var = 1;
		int     i = 0;
		boolean noMin = (_min == Double.NEGATIVE_INFINITY);
		boolean noMax = (_max == Double.POSITIVE_INFINITY);
		String  unit = translate(_number.getUnit());

		if (!noMax && !noMin)
		{
			// Get the max between min and max (abs values)
			double absMax = Math.max(Math.abs(_max), Math.abs(_min));

			for (i = 0; (var <= absMax) && (i < 20); i++)
				var *= 10;

			_maxChars = i + 1 + ((_decimals == 0) ? 0 : ((_decimals < 0) ? 10 : (_decimals + 1)));

			if (_maxChars > 25)
				_maxChars = 25;
		}
		else if (_decimals == 0)
			_maxChars = 10;
		else
			_maxChars = 20;

		// Construct the buttons labels
		LyEnvironment		 environment = getEnvironment();
		DecimalFormatSymbols format = new DecimalFormatSymbols(environment.getLocale());
		char				 decimalSeparator = format.getDecimalSeparator();
		String				 maxStr = null;
		String				 minStr = null;

		if (_decimals == 0)
		{
			maxStr = String.valueOf((long)_max).replace('.', decimalSeparator);
			minStr = String.valueOf((long)_min).replace('.', decimalSeparator);
		}
		else
		{
			maxStr = String.valueOf(_max).replace('.', decimalSeparator);
			minStr = String.valueOf(_min).replace('.', decimalSeparator);
		}

		_labelIncrease = getString("LY_MAX_VALUE", new String[] { maxStr, unit });
		_labelDecrease = getString("LY_MIN_VALUE", new String[] { minStr, unit });
		_label = _number.getLabel();
		if (_label == null)
			_label = "";
	} // end method setContent

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	    pw.startComponent(this);
	    
		updateRefreshInfo(pw);

		String				 name = getId();
		String				 nameSubst = LyString.replace(name, "\\", "\\\\");
		String				 formName = "forms[0]";//getFormName(pw);
		String				 unit = _number.getUnit();
		boolean				 needCheck = needCheck();
		String				 disabledColor = getHexColor("LY_DISABLED_BACKGROUND");
		String				 label;
		double				 pitch = _number.getPitch();

		// Retrieve the decimal separator
		LyEnvironment		 environment = getEnvironment();
		DecimalFormatSymbols format = new DecimalFormatSymbols(environment.getLocale());
		char				 decimalSeparator = format.getDecimalSeparator();

		setContent();

		// Here setContent has computed all the labels.
		unit = getHtmlString(unit);
		label = _label;
		label = getDisplayableLabel(label, decimalSeparator);
		pitch = LyTools.getPitch(pitch, _decimals);

		boolean isEditable = isEditable();
		boolean canDisabled = canDisabled();

		// If not editable and control can be disabled, just write text
		if (!isEditable && !canDisabled)
		{
			pw.append("<font class='label'>");
			pw.append(label + "&nbsp;");
			pw.append(unit);
			pw.append("</font>");
			return;
		}

		// global (editable or not)
		String errorMessage = null;
		String incrementStatus = null, decrementStatus = null;

		// First, add the needed script to the page
		pw.addBeginScript("<script type='text/javascript'>window.defaultStatus='';</script>",
			false);

		if (isEditable)
		{
			errorMessage = getMessage("LY_ER_MATCH_FORMAT", new String[] { "%1", getCaption() });

			incrementStatus = getHtmlString("LY_INCREMENT_VALUE");
			decrementStatus = getHtmlString("LY_DECREMENT_VALUE");
		}

		String type = "Float";

		if (_decimals == 0)
			type = "Integer";

		pw.append("\n<table border='0' cellpadding='0' cellspacing='0'>");
		pw.append("\n<tr>");
		pw.append("<td>\n");
		pw.append("\n<input name='" + name + "' value='" + label + "'" + " size='" + _maxChars
			+ "'");

		if (isEditable)
		{
			pw.append(" onChange=\"");

			// FFT00059 : NeedCheck on number fields.
			if (needCheck)
				pw.append(";javascript:FieldChange(window.document." + formName + ",this,'"
					+ _number.getRootController() + "')");
			else
			{
				pw.append("javascript:Check" + type + "Field" + "(window.document."
						+ formName + ",'" + nameSubst + "'," + _min + "," + _max + ",'" + errorMessage
						+ "'");

				if (_decimals != 0)
					pw.append(",'" + decimalSeparator + "'," + _decimals);
				
				pw.append(")");
			}

			pw.append("\"");
		}
		else
		{
			pw.append(" disabled='true'");
			if (disabledColor != null)
				pw.append(" style='background-color:" + disabledColor + "'");
		}

		pw.append("/>");
		pw.append("</td>");

		// Adding increment button
		if (_number.isShowArrows())
		{
			pw.append("\n<td>");
			pw.append("\n<div>\n");
			pw.append("<div>\n");

			if (isEditable)
			{
				pw.append("<img style='cursor:hand;cursor:pointer;' onClick=\"javascript:Increment" + type
					+ "Field(window.document." + formName
					+ ",'" + nameSubst + "'," + _min + "," + _max + "," + pitch);

				if (_decimals != 0)
					pw.append(",'" + decimalSeparator + "'");

				// FFT00059 : NeedCheck on number fields.
				if (needCheck)
					pw.append(",'" + _number.getRootController() + "'");
				else
					pw.append(",'false'");

				pw.append(")\"");

				pw.append(" onmouseover=\"window.status='" + incrementStatus + "';return true\"");
				pw.append(" src=\"" + _increaseButton + "\" align='bottom' border='0' title=\"" + _labelIncrease + "\"/>");
			}
			else
				pw.append("\n<img src=\"" + _disabledIncreaseButton
					+ "\" align='bottom' border='0' class='disabledTool'/>");
			
			pw.append("</div>\n");
			// Adding decrement button
			pw.append("<div>\n");

			if (isEditable)
			{
				pw.append("<img style='cursor:hand;cursor:pointer;' onClick=\"javascript:Increment" + type
					+ "Field(window.document." + formName
					+ ",'" + nameSubst + "'," + _min + "," + _max + ",-" + pitch);

				if (_decimals != 0)
					pw.append(",'" + decimalSeparator + "'");

				// FFT00059 : NeedCheck on number fields.
				if (needCheck)
					pw.append(",'" + _number.getRootController() + "'");
				else
					pw.append(",'false'");

				pw.append(")\"");
				pw.append(" onmouseover=\"window.status='" + decrementStatus + "';return true\"");
				pw.append(" src=\"" + _decreaseButton + "\" align='bottom' border='0' title=\"" + _labelDecrease + "\"/></a>");
			}
			else
				pw.append("\n<img src=\"" + _disabledDecreaseButton
					+ "\" align='bottom' border='0' class='disabledTool'/>");

			pw.append("<div>\n");
			pw.append("</div>\n");
			pw.append("</td>\n");
		}
		else
			pw.append("\n");

		// Adding unit
		pw.append("<td nowrap='yes'><font class='label'>&nbsp;" + unit + "</font></td>");

		pw.append("</tr>");
		pw.append("</table>\n");
		
		pw.endComponent(this);
	} // end method writeHtml

	/**
	 * Gets the displayable label for this number.
	 *
	 * @param  label            the initial label
	 * @param  decimalSeparator the decimal separator to be used
	 * @return the displayable label for this number
	 */
	public String getDisplayableLabel(String label, char decimalSeparator)
	{
		if (label == null)
			return "";

		// FFT00021 : If there is no label we let an empty string.
		if (label.length() != 0)
		{
			label = getHtmlString(label);

			// Check the value (must be <= max, >= min and
			// number of decimals <= decimals)
			if (_decimals > 0)
				label = LyTools.checkDecimals(label, _decimals);

			// Be sure to replace decimal separator by '.'
			label = label.replace(decimalSeparator, '.');

			Number value;

			// FFT00529
			if (_decimals <= 0)
			{
				value = new Long(LyTools.longFromString(label));
				if (value.doubleValue() <= _min)
					value = new Long(Math.round(_min));
				else if (value.doubleValue() >= _max)
					value = new Long(Math.round(_max));
			}
			else
			{
				value = new Double(LyTools.doubleFromString(label));
				if (value.doubleValue() <= _min)
					value = new Double(_min);
				else if (value.doubleValue() >= _max)
					value = new Double(_max);
			}

			label = LyTools.checkDecimals(String.valueOf(value), _decimals);

			// Check the number of decimals and add '0' at the end if needed
			if (_decimals > 0)
			{
				int pos = label.indexOf('.');
				int decimalsNb = label.length() - pos - 1;

				if ((pos > 0) && (decimalsNb < _decimals))
				{
					int i, n = _decimals - decimalsNb;

					for (i = 0; i < n; i++)
						label = label + '0';
				}
			}

			label = label.replace('.', decimalSeparator);
		}

		return label;
	} // end method getDisplayableLabel

	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetNumberInterface#setFormat(java.lang.String)
	 */
	public void setFormat(String format)
	{
		refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetNumberInterface#setVMin(double)
	 */
	public void setVMin(double vMin)
	{
		_min = vMin;

		LyComponent parent = _number.getParent();

		if (parent instanceof LyFormField)
		{
			LyFormField		 formField = (LyFormField)parent;
			LyStrutsFormField webField = (LyStrutsFormField)formField.getInternalView();

			webField.refreshDivContent();
		}
		else
			refresh();
	}

	/**
	 * Sets the maximum value.
	 *
	 * @param vMax New value of maximum.
	 */
	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetNumberInterface#setVMax(double)
	 */
	public void setVMax(double vMax)
	{
		_max = vMax;

		LyComponent parent = _number.getParent();

		if (parent instanceof LyFormField)
		{
			LyFormField		 formField = (LyFormField)parent;
			LyStrutsFormField webField = (LyStrutsFormField)formField.getInternalView();

			webField.refreshDivContent();
		}
		else
			refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetNumberInterface#setPitch(double)
	 */
	public void setPitch(double pitch)
	{
		refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetNumberInterface#setDecimals(int)
	 */
	public void setDecimals(int decimals)
	{
		_decimals = decimals;
		refresh();
	}

} // end class LyStrutsNumber
