/*
 * LyDtmlOption.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Option object (as described in the dtd) for web viewer.
 *
 * <p><b>OPTION: Option</b><br>
 * An option represents a possible value in an input field of {@link
 * leon.view.web.struts.LyStrutsChoice Choice} type. It can be represented by a check box (case of a
 * multiple choice) or else by a radio button (case of a simple choice).</p>
 *
 * @version $Id: LyStrutsOption.java 23300 2009-12-17 13:56:41Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.LyOption
 */
public class LyStrutsOption extends LyWebObject
	implements LyGraphicalOption
{
	public static final String __VERSION =
		"$Id: LyStrutsOption.java 23300 2009-12-17 13:56:41Z jlsaouli $";
	
	/** The option generic component. */
	protected LyOption		   _option;

	/** The identifier of the parent form. */
	protected String		   _formId;

	/**
	 * Class constructor of an Option object for the struts target.
	 *
	 * @param option the generic Option defined for every type of view targets.
	 */
	public LyStrutsOption(LyOption option)
	{
		super(option);
		_option = option;
		_formId = null;
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalOption#setChecked(boolean)
	 */
	public void setChecked(boolean checked)
	{
		// Manage Ajax mode
		LyWebViewManager viewManager = getWebViewManager();
		LyChoice		 choice = (LyChoice)_option.getParent();
		LyStrutsChoice    htmlChoice = (LyStrutsChoice)choice.getInternalView();
		
		if (choice.isMultiple())
		{
			LyFormField		 formField = (LyFormField)choice.getParent();
			LyStrutsFormField webField = (LyStrutsFormField)formField.getInternalView();

			webField.refreshDivContent();
			return;
		}
		
		LyStrutsForm	form = htmlChoice.getHtmlForm();
		String			formName = ((form == null) ? "default" : form.getName());
		StringBuffer    buffer = new StringBuffer();

		buffer.append("<script type='text/javascript'>\n//<![CDATA[\nSelectCheckbox('");
		buffer.append(formName);
		buffer.append("','");
		buffer.append(htmlChoice.getId());
		buffer.append("','");
		buffer.append(LyString.getJavascriptString(_option.getId()));
		buffer.append("',");
		buffer.append(checked);
		buffer.append(");\n//]]>\n</script>");

		viewManager.addAdditionalRefreshScripts(buffer.toString());
	}
	
	/*
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	    pw.startComponent(this);
	    
		updateRefreshInfo(pw);

		LyChoice    parent = (LyChoice)_option.getParent();
		LyFormField grandParent = (LyFormField)parent.getParent();
		LyForm	    form;

		// Look for an ancestor of type LyForm
		LyComponent ancestor = parent;

		do
			ancestor = ancestor.getParent();
		while ((ancestor != null) && !(ancestor instanceof LyForm));
		form = (LyForm)ancestor;

//		LyStrutsForm webForm = ((form == null) ? null : (LyStrutsForm)form.getInternalView());
		String	    formName = "forms[0]";//((webForm == null) ? "forms[0]" : webForm.getName());
		boolean     isEnabled = (_option.isEnabled() && ((form == null) || form.isEditable())
					&& grandParent.isEditable() && grandParent.isEnabled());

		if (_formId == null)
			_formId = formName;
		
		boolean multiple = parent.isMultiple();

		pw.append("\n<input name=\"");
		pw.append(grandParent.getId());
		if (multiple)
			pw.append("_multiple\" type='checkbox'");
		else
			pw.append("\" type='radio'");
		pw.append(" value='");
		pw.append(_option.getId());
		pw.append("'");
		
		if (_option.isChecked())
			pw.append(" checked='true'");

		// FFT00263 : bug if nothing checked : nothing sent to the servlet
		StringBuffer onClickBuffer = null;
		String		 idSubst = LyString.replace(grandParent.getId(), "\\", "\\\\");
		String		 sep = LyWebParameterValues.HTML_ENCODED_SEP;
		boolean		 needCheck = grandParent.isNeedCheck();

		if (isEnabled && multiple)
		{
			if (onClickBuffer == null)
				onClickBuffer = new StringBuffer();

			if (needCheck)
				onClickBuffer.append("UpdateMultipleChoice(window.document." + _formId + ",'" + idSubst + "','"
					+ idSubst + "_multiple','" + sep + "', true,'"
					+ _option.getRootController() + "');");
			else
				onClickBuffer.append("UpdateMultipleChoice(window.document." + _formId + ",'" + idSubst + "','"
					+ idSubst + "_multiple','" + sep + "');");
		}

		if (!isEnabled)
		{
			String errorMessage = getMessage("LY_ER_SELECT");

			pw.append(" disabled='true'" + " onMouseDown=\"javascript:ClickOnDisabledComponent('"
				+ errorMessage + "')\"");
		}
		else if (needCheck && !multiple)
		{
			if (onClickBuffer == null)
				onClickBuffer = new StringBuffer();

			onClickBuffer.append("FieldChange(window.document." + _formId + ",this,'"
				+ _option.getRootController() + "');");
		}

		if (onClickBuffer != null)
			pw.append(" onClick=\"" + onClickBuffer.toString() + "\"");

		pw.append(" />");

		String label = _option.getLabel();

		label = translate(label);
		pw.append("<font ");
		
		String  style = "";
    if(!_option.isDefaultFont() || !_option.isDefaultFontColor()) {
      style += getFontString(_option.getFont(),_option.getFontColor());
    }
    
		if(style.trim().length() > 0) {
		  pw.append(" style=\'"+style+"\'");
		} 
		
		if(_option.isDefaultFont() && parent.isDefaultFont()) {
		   LyWebDecoration	deco = getWebViewManager().getDecoration();
		   pw.append(" class='" + deco.getClassName(_option, pw) + "'");
		}
		
		if (!isEnabled)
			pw.append(" color='#909090'");
		pw.append(">");
		pw.append(getHtmlString(label));
		pw.append("</font>");
		
		pw.endComponent(this);
	} // end method writeHtml

	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetEnabledInterface#setEnabled(boolean)
	 */
	public void setEnabled(boolean enabled)
	{
		LyChoice    	 choice = (LyChoice)_option.getParent();
		LyFormField		 formField = (LyFormField)choice.getParent();
		LyStrutsFormField webField = (LyStrutsFormField)formField.getInternalView();

		webField.refreshDivContent();
	}

} // end class LyStrutsOption
