/*
 * LyStrutsPanel.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Current Version: $Id: LyStrutsPanel.java 24563 2010-03-04 08:15:19Z jlsaouli $.
 */

package leon.view.web.struts;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.setinterface.*;
import leon.view.web.*;


/**
 * This class implements the common behavior for Panel objects for the HTML viewer.
 *
 * <p>The panel entity corresponds to any type of high-level graphical component (form, list, tree,
 * map, etc.) or to a container.</p>
 *
 * @version $Id: LyStrutsPanel.java 24563 2010-03-04 08:15:19Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.LyPanel
 */
public abstract class LyStrutsPanel extends LyStrutsFieldItem
	implements	LyGraphicalPanel,
				LySetPopupMenuInterface
{
	public static final String __VERSION =
		"$Id: LyStrutsPanel.java 24563 2010-03-04 08:15:19Z jlsaouli $";

	/**
	 * Class constructor of a Panel object for the struts target.
	 *
	 * @param panel the generic Panel defined for every type of view targets.
	 */
	public LyStrutsPanel(LyPanel panel)
	{
		super(panel);
	}

	/**
	 * Gets the HTML tags to create a popup menu view.
	 *
	 * @return the value or "" if no popup menu view is available.
	 */
	public String getPopupMenuView()
	{
		// Popup menu is generated directly in the view
		return "";
	}

	/**
	 * Sets the popup menu with the given value.
	 *
	 * @param popup the new popup menu
	 */
	public void setPopupMenu(LyPopupMenu popup)
	{
		LyStrutsViewManager	viewManager = (LyStrutsViewManager)getWebViewManager();
		LyView				view = _viewComponent.getParentView();
		
		viewManager.setPopupMenu(popup, LyWebConstants.POPUP_MENU + view.getIdCode());
	}
}