/*
 * LyStrutsRows.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import java.util.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Rows object (as described in the dtd) for struts viewer.
 *
 * <p><b>ROWS: Rows</b><br>
 * This element allows displaying rows within a list. It does not contain any attribute: this is
 * only a non-empty list of rows.</p>
 *
 * @version $Id: LyStrutsRows.java 23300 2009-12-17 13:56:41Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.LyRows
 */
public class LyStrutsRows extends LyWebObject
	implements LyGraphicalRows
{
	public static final String __VERSION =
		"$Id: LyStrutsRows.java 23300 2009-12-17 13:56:41Z jlsaouli $";

	/** The rows generic component. */
	protected LyRows		   _rows;

	/**
	 * Class constructor of a Rows object for the struts target.
	 *
	 * @param rows the generic Rows defined for every type of view targets.
	 */
	public LyStrutsRows(LyRows rows)
	{
		super(rows);
		_rows = rows;
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LyFreeInterface#free()
	 */
	public void free()
	{
	}

	/**
	 * Writes the needed styleSheet for this component. This method does nothing.
	 *
	 * @param pw the pageWriter
	 */
	protected void writeStyleSheet(LyWebPageWriter pw)
	{
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	    pw.startComponent(this);
	    
		LyTable table = (_rows == null ? null : (LyTable)_rows.getParent());
		int     i;
		Object  lock = ((_rows == null) ? new Object() : _rows);

		// Write styleSheet
		writeStyleSheet(pw);

		synchronized (lock)
		{
			if (_rows == null)
			{
			    pw.endComponent(this);
				return;
			}

			// Duplicate list to avoid changes during loop
			Vector<LyRow>   list = null;
			int		   nRows;
			LyRow	   row;
			LyStrutsRow comp;
			boolean    hierarchical = _rows.isHierarchical();

			if (!hierarchical)
			{
				do
				{
					nRows = ((_rows == null) ? 0 : _rows.getRowCount());
					list = new Vector<LyRow>(nRows);
					for (i = 0; i < nRows; i++)
					{
						row = _rows.getRow(i);

						if ((row != null) && !list.contains(row))
							list.add(row);
					}
				}
				while (list.size() != _rows.getRowCount());
			}
			else
			{
				do
				{
					nRows = ((_rows == null) ? 0 : _rows.getSubRowCount(true));
					list = new Vector<LyRow>(nRows);
					_rows.getDisplayedSubRows(list);
				}
				while (list.size() != _rows.getSubRowCount(true));
			}

			// Remove hidden objects
			for (nRows = list.size(), i = nRows - 1; i >= 0; i--)
			{
				row = list.elementAt(i);
				if (row.isHidden())
					list.remove(i);
			}

			int		subIndex;
			
			for (i = 0, nRows = list.size(); i < nRows; i++)
			{
				row = list.elementAt(i);
				comp = (LyStrutsRow)row.getInternalView();
				subIndex = -1;
				
				if (hierarchical)
				{
					LyRows	rows = (LyRows)row.getParent();
					
					subIndex = rows.getRowIndex(row);
				}
				
				comp.writeHtml(pw, i, subIndex);
			}

			if (table.getSelectionMode() == LyTable.SelectionMode.NONE)
			{
				// Nothing to do...
			}
			else if (table.getSelectionMode() == LyTable.SelectionMode.SINGLE)
			{
				pw.append("<input type='hidden' name='_selectedRow' value='");
				pw.append("'/>\n");
			}
		}
		
		pw.endComponent(this);
	} // end method writeHtml
} // end class LyStrutsRows
