/*
 * LyStrutsSelectItem.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the selectable item object (as described in the dtd) for web viewer.
 *
 * <p><b>SELECTITEM: Selection Item</b><br>
 * A selection item allows pointing to a page of data that can be reached by simple selection.</p>
 *
 * @version $Id: LyStrutsSelectItem.java 29032 2011-01-03 10:33:25Z nhamon $
 * @author  Lyria S.A.
 */
public class LyStrutsSelectItem extends LyWebObject
    implements LyGraphicalSelectItem
{
	public static final String __VERSION =
		"$Id: LyStrutsSelectItem.java 29032 2011-01-03 10:33:25Z nhamon $";

	/** The select item generic component. */
    protected LySelectItem     _selectItem;
    
	/**
	 * Constructor.
	 *
	 * @param selectItem the select item component
	 */
	public LyStrutsSelectItem(LySelectItem selectItem)
	{
        super(selectItem);
        
        _selectItem = selectItem;
	}

	/**
	 * Build the path associated to this selectable item.<br>
	 * Method overriden to redirect to the Leonardi struts event handler.
	 */
	public String buildPath()
	{
		// get the url to the event handler
		LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();
		String			    actionUrl = viewManager.getActionURL(LyStrutsConstants.PATH_ACTION_GRAPHICEVENT);

		// build the URL
		StringBuffer	    itemUrl = new StringBuffer();

		itemUrl.append(actionUrl);
		itemUrl.append("?");
		itemUrl.append(LyWebConstants.CONTROLLER);
		itemUrl.append("=");
		itemUrl.append(_selectItem.getRootController());
		itemUrl.append("&#38;");
		itemUrl.append(LyWebConstants.SOURCE);
		itemUrl.append("=");
		itemUrl.append(_selectItem.getId());

		return "javascript:InvokeAction('" + _selectItem.getId() + "','" + itemUrl.toString()
		+ "','" + LyWebConstants.RELOAD_FRAMES + "','" + _selectItem.getRootController() + "');";
	}
	
	   /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebObject#refresh()
     */
    public void refresh()
    {
        // In DHTML the refresh is more clever : we refresh the parent DIV content
        LySelectItem    selectItem = (LySelectItem)_viewComponent;
        LySelectBar     selectBar = (LySelectBar)selectItem.getParent();

        LyViewComponent component = selectBar.getInternalView(false);

        if (component instanceof LyWebObject)
            ((LyWebObject)component).refresh();
        else
            super.refresh();

    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
     */
    public void writeHtml(LyWebPageWriter pw)
    {
        pw.startComponent(this);
        
        updateRefreshInfo(pw);

        String label = getHtmlString(_selectItem.getLabel());
        LyWebDecoration deco = getWebViewManager().getDecoration();

        if (_selectItem.isSelected())
        {
            pw.append("<font class='" + deco.getClassName(_selectItem, pw) + "'>");
            pw.append(label);
            pw.append("</font>");
            
            pw.endComponent(this);
            return;
        }

        String path = buildPath();
        String tooltip = getHtmlString(_selectItem.getTooltip());
        String javascriptToolTip = getJavascriptString("LY_SELECT_ITEM" + " " + label);

		pw.append("\n<a href=\"");
		pw.append(path);
		pw.append("\"");

		if ((tooltip != null) && (tooltip.length() > 0))
		{
			pw.append(" title='");
			pw.append(tooltip);
			pw.append("'");
		}

		pw.append(" onmouseover=\'window.status=\"");
        pw.append(javascriptToolTip);
        pw.append("\";return true\'>");
        pw.append("<font class='" + deco.getClassName(_selectItem, pw) + "'>");
        pw.append(label);
        pw.append("</font>");
        pw.append("</a>");
        
        pw.endComponent(this);
    } // end method writeHtml

    /**
     * (non-Javadoc).
     *
     * @see leon.view.setinterface.LySetLabelInterface#setLabel(java.lang.String)
     */
    public void setLabel(String label)
    {
        refresh();
    }

    /**
     * Updates attribute tooltip to the given value.
     *
     * @param tooltip New value of tooltip.
     */
    public void setTooltip(String tooltip)
    {
        refresh();
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.setinterface.LySetImageInterface#setImage(java.lang.String)
     */
    public void setImage(String image)
    {
        refresh();
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.setinterface.LySetSelectedInterface#setSelected(boolean)
     */
    public void setSelected(boolean selected)
    {
        refresh();
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.setinterface.LySetEnabledInterface#setEnabled(boolean)
     */
    public void setEnabled(boolean enabled)
    {
        refresh();
    }
} // end class LyStrutsSelectItem
