/*
 * LyStrutsServlet.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts;


import java.awt.*;
import java.io.*;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

import leon.app.*;
import leon.control.*;
import leon.control.form.*;
import leon.misc.*;
import leon.view.*;
import leon.view.web.*;
import leon.view.web.mobile.*;
import leon.view.web.struts.action.*;

import org.apache.struts.action.*;


/**
 * @version $Id: LyStrutsServlet.java 36431 2012-08-06 09:03:20Z nhamon $
 * @author  Lyria S.A.
 */
public class LyStrutsServlet extends ActionServlet
{
	/**
	 * 
	 */
	private static final long serialVersionUID = -5955068839831742253L;

	public static final String __VERSION =
		"$Id: LyStrutsServlet.java 36431 2012-08-06 09:03:20Z nhamon $";

	/** Keep the information : which request / Servlet / response are currently being processed. */
	public static HashMap<Thread, LyWebRequestInfo> _webRequestInfos =
		new HashMap<Thread, LyWebRequestInfo>();

	
	/**
	 * Constructor.
	 *
	 * @param area the area component
	 */
	public LyStrutsServlet()
	{
		super();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see javax.servlet.Servlet#init(javax.servlet.ServletConfig)
	 */
	public void init(ServletConfig config)
		throws ServletException
	{
		super.init(config);

		try
		{
			// FFT SITA002
			if (!GraphicsEnvironment.isHeadless()) 
				System.setProperty("java.awt.headless", "true");
		}
		catch (Throwable t)
		{
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.apache.struts.action.ActionServlet#doGet(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 */
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
	{
		HttpSession				httpSession = request.getSession();
		ServletContext			context = httpSession.getServletContext();
		LyStrutsLeonardiPlugin	plugin = LyStrutsLeonardiPlugin.getPlugin(context);
		
		if ((plugin != null) && plugin.isLoadingData())
		{
			generateLoadingDataPage(request, response);
			return;
		}

		int	requestCount = LyWebConstants.REQU_COUNT;

		LyWebConstants.REQU_COUNT++;

		try
		{
			if ("true".equals(request.getParameter(LyWebConstants.NEW_SESSION)))
			{
				request.getSession().invalidate();
			}
			
			synchronized (_webRequestInfos)
			{
				_webRequestInfos.put(Thread.currentThread(),
						new LyWebRequestInfo(requestCount, this, request, response));
			}
			
			super.doGet(request, response);
		}
		finally
		{
			synchronized (_webRequestInfos)
			{
				_webRequestInfos.remove(Thread.currentThread());
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * @see org.apache.struts.action.ActionServlet#doPost(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 */
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
	{
		int	requestCount = LyWebConstants.REQU_COUNT;

		LyWebConstants.REQU_COUNT++;

		try
		{
			synchronized (_webRequestInfos)
			{
				_webRequestInfos.put(Thread.currentThread(),
						new LyWebRequestInfo(requestCount, this, request, response));
			}
			
			String mobile = request.getParameter(LyWebConstants.MOBILE);
			
			if((mobile != null && mobile.equals("true")) ||
					(request.getHeader(LyWebConstants.MOBILE)!=null && request.getHeader(LyWebConstants.MOBILE).equals("true")))
			{
				LyMobileDecodeRequest.getInstance(true).decode(request, response);
				return;
			}
			// If a file is attached, manage the upload directly in this servlet to disable the Struts standard mechanism.
			if (isFileAttached(request))
			{
				HttpSession			httpSession = request.getSession();
				LyStrutsSessionBean	sessionBean = (LyStrutsSessionBean)httpSession.getAttribute("leonardiSessionBean");
				
				sessionBean.checkSession(request);
				
				LySession		    session = sessionBean.getSession();
				LyStrutsViewManager viewManager = (LyStrutsViewManager)session.getViewManager();
				LyEnvironment		environment = viewManager.getEnvironment();
	
				try
				{
					// Upload the file
					request = getFileAttachment(request, session, viewManager);
				}
				catch (Throwable e)
				{
					environment.writeException(e);
					if ((environment.getResource("LY_MAX_BLOB_SIZE") != null)
							&& (environment.getIntResource("LY_MAX_BLOB_SIZE") != -1))
					{
						String errorMessage = environment.getString("ERROR_BLOB_SIZE",environment.getIntResource("LY_MAX_BLOB_SIZE")/1000000);
						String controllerId = request.getParameter(LyWebConstants.CONTROLLER);
						LyController	controller = (controllerId == null ? null : session.getController(controllerId));
						controller.showError(errorMessage);
					}
				}
				finally
				{
					// get a print writer
					Writer pw = response.getWriter();
					response.setContentType("text/html;charset=" + LyString.DEFAULT_CHARSET);
					response.setStatus(HttpServletResponse.SC_OK);
					// The response sent to the client is only a script turning off the wait mechanism.
					pw.write("<html><head>");
					pw.write(viewManager.generateScriptsInclusion());
					pw.write("<script>getLeonardiTopFrame().showWaitDiv(false, true);</script>");
					
					LyStrutsLeonardiAction.generateAdditionalScripts(session, request, response, pw);
					
					pw.write("</head></html>");
					pw.close();
				}
				
				return;
			}
			
			super.doPost(request, response);
		}
		finally
		{
			synchronized (_webRequestInfos)
			{
				_webRequestInfos.remove(Thread.currentThread());
			}
		}
	}

	/**
	 * Accessor to the information about current request in progress : TyServlet, request, response
	 * Current means for 'current Thread.
	 *
	 * @return the information if available or null if it could not bee found.
	 */
	public static LyWebRequestInfo getWebRequestInfo()
	{
		synchronized (_webRequestInfos)
		{
			return _webRequestInfos.get(Thread.currentThread());
		}
	}

	/**
	 * Check if the given request contains an attached file.
	 *
	 * @param  request the request
	 * @return true if an attachement is found.
	 */
	protected boolean isFileAttached(HttpServletRequest request)
	{
		String upload = request.getParameter(LyWebConstants.UPLOAD);
		
		if (upload != null)
			upload = upload.intern();
		
		return (upload == "true");
	}

	/**
	 * Manage the file attachement during the submit of a Leonardi form.
	 *
	 * @param  request     Current HTTP request.
	 * @param  session     The Leonardi session.
	 * @param  viewManager The view manager.
	 * @return a non-multipart HTTP request.
	 * @throws Exception if something is wrong during the management.
	 */
	protected HttpServletRequest getFileAttachment(HttpServletRequest request, LySession session,
		LyStrutsViewManager viewManager)
		throws Exception
	{
		// get some request parameters
		String frame = request.getParameter(LyWebConstants.FRAME);
		String controllerId = request.getParameter(LyWebConstants.CONTROLLER);

		if (LyWebConstants.POPUP_VIEW.equals(frame))
			frame = null;
		else if (LyWebConstants.TARGET_FRAME.equals(frame))
			frame = null;
		
		// Do we have any specific decoder here ?
		LyWebPageWriter	pageWriter = null;
		LyView			view = null;
		LyController	controller = (controllerId == null ? null : session.getController(controllerId));
		
		if (controller != null)
		{
			view = controller.getView();
			pageWriter = (LyWebPageWriter)view.getInternalView();
		}

		LyFileAttachmentHandler handler = null;

		if (pageWriter == null) // FFT 597
		{
			LyViewController lastViewController = session.getLastViewController();

			if (lastViewController != null)
				lastViewController.showView();
		}
		else
		{
			String attachmentHandler = pageWriter.getAttachmentHandler(controllerId);
			String attachmentDirectory = pageWriter.getAttachmentDirectory(controllerId);

			if ((attachmentHandler == null) || attachmentHandler.equals("default"))
				attachmentHandler = "leon.view.web.LyFileAttachmentHandler";

			Class<?> handlerClass = Class.forName(attachmentHandler);

			handler = (LyFileAttachmentHandler)handlerClass.newInstance();
			handler.setAttachmentDirectory(attachmentDirectory);
		}

		// Use default one if none specified.
		if (handler == null)
		{
			handler = new LyFileAttachmentHandler();
			handler.setAttachmentDirectory("files");
		}

		// Set parameters
		handler.setEnvironment(session.getEnvironment());
		handler.setRequest(request);

		// Parse stream content and attachment
		try
		{
			handler.parseContent();
		}
		catch (Exception e)
		{
			session.getEnvironment().writeException(e);

			viewManager.setMessage(e.getMessage());
		}

		// get parameter part request
		HttpServletRequest parameterPartRequest = handler.getParameterPartRequest();
		LyFilePartWriter   writer = handler.getFilePartWriter();

		// may save to delete list : delete temp file when controller is freed
		if (controller instanceof LyFileFieldSetController) 
			((LyFileFieldSetController)controller).addTempFilePath(writer.getServerFileName());
		
		writer.commit(true);

		// Manage CKEditor upload mechanism
		if ("true".equals(request.getParameter("_ckeditor")))
			generateCKEditorScript(request, session, viewManager, writer);
		
		// free the file handler
		handler.free();

		if (view != null)
		{
			LyStrutsParameterValues	values = new LyStrutsParameterValues(parameterPartRequest, session.getEnvironment().getString());
			
			values.updateViewFromValues(view, true);
		}
		
		// return the parameter part request
		return parameterPartRequest;
	} // end method getFileAttachment
	
	/**
	 * Generate the Javascript sent to CKEditor when uploading an image file.
	 *
	 * @param  request     Current HTTP request.
	 * @param  session     The Leonardi session.
	 * @param  viewManager The view manager.
	 * @param  writer      The Leonardi file part writer.
	 */
	protected void generateCKEditorScript(HttpServletRequest request, LySession session,
			LyStrutsViewManager viewManager, LyFilePartWriter writer)
	{
		String			formName = request.getParameter("_formName");
		String			file = writer.getServerFileName();
		StringBuffer	buffer = new StringBuffer();
		String			rootUrl = viewManager.getRootURL();
		String			appDir = session.getEnvironment().getEnv("LY_APP_DIR");
		
		if ((file != null) && file.startsWith(appDir))
			file = LyString.replace(file, appDir, rootUrl);
		
		file = LyString.replace(file, "\\", "/");
		buffer.append("<script type='text/javascript'>\n");
		buffer.append("var ckeditorFrame = findTargetByContent(null, '" + formName + "');\n");
		buffer.append("if (ckeditorFrame != null) {\n");
		buffer.append("var serverFile = ckeditorFrame.document.getElementById('" + formName + "');\n");
		buffer.append("ckeditorFrame.SetCKEditorServerFile('" + formName + "','" + file + "');\n");
		buffer.append("}\n");
		buffer.append("</script>\n");
		
		viewManager.addAdditionalRefreshScripts(buffer.toString());
	}
	
	/**
	 * Generates the page indicating that application data is being loaded.
	 */
	protected void generateLoadingDataPage(HttpServletRequest request, HttpServletResponse response)
		throws ServletException, IOException
	{
		PrintWriter	pw = response.getWriter();
		
		pw.write("<html>\n");
		pw.write("<body>\n");
		pw.write("<font style='font-face:Arial;font-size:20px;'>Loading Data. Please wait...</font>\n");
		pw.write("</body>\n");
		pw.write("</html>\n");
		pw.close();
	}
} // end class LyStrutsServlet
