/**
 * LySession.java Copyright (c) 2000-2009,
 * Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import javax.servlet.http.*;

import leon.app.*;
import leon.misc.*;
import leon.view.web.*;
import leon.view.web.struts.action.*;


/**
 * The instance of session for web viewers.
 *
 * @see     leon.app.LySession
 */
public class LyStrutsSession extends LyWebSession
{
	public static final String __VERSION =
		"$Id: LyWebSession.java 31632 2011-07-08 08:30:48Z nhamon $";
    
	/**
	 * Session bean managing this web session.
	 */
	protected LyStrutsSessionBean	_bean;
	
    /**
     * Constructor of web session.
     *
     * @param application     The application owner of the session.
     * @param registerSession Indicates if the session must be registered on the application
     *                        (default is true), or if this is a temporary session which should be
     *                        ignored.
     */
    public LyStrutsSession(LyApplication application, boolean registerSession)
    {
        super(application, registerSession);
    }

    /**
     * Constructor of web session.
     *
     * @param application     The application owner of the session.
     * @param registerSession Indicates if the session must be registered on the application
     *                        (default is true), or if this is a temporary session which should be
     *                        ignored.
     * @param id              The identifier of the session.
     */
    public LyStrutsSession(LyApplication application, boolean registerSession, String sessionId)
    {
        this(application, registerSession);
        
        if ((sessionId != null) && (sessionId != LyStrutsSessionBean.DEFAULT_SESSION))
        	_id = sessionId;
    }

    /**
     * Gets the current request.
     *
     * @return the request
     */
    public HttpServletRequest getCurrentRequest()
    {
        LyWebRequestInfo    info = LyStrutsServlet.getWebRequestInfo();
        
        return (info == null ? null : info.getRequest());
    }

    /**
     * Gets the current response.
     *
     * @return the response
     */
    public HttpServletResponse getCurrentResponse()
    {
        LyWebRequestInfo    info = LyStrutsServlet.getWebRequestInfo();
        
        return (info == null ? null : info.getResponse());
    }
    
    /**
     * Sets the session bean and Http session for this web session.
     */
    public void setSessionBean(LyStrutsSessionBean bean, HttpSession httpSession)
    {
    	_bean = bean;
    	_httpSession = httpSession;
    }
    
    /**
     * Returns the Http session associated with this web session.
     */
    public HttpSession getHttpSession()
    {
    	return _httpSession;
    }
    
    /*
     * (non-Javadoc)
     * @see leon.app.LySession#close()
     */
    public void close()
    {
        LyEnvironment environment = getEnvironment();
        boolean closeHttpSession = ((environment == null) || environment.testResource("LY_EXIT_PAGE_CLOSING_HTTPSESSION"));

        super.close();
    	
    	if (_bean != null)
    	{
    		LySession	oldSession = _bean.getSession();
    		
    		_bean.setSession(this);
    		_bean.closeCurrentSession(null, _httpSession, closeHttpSession);
    		_bean.setSession(oldSession);
    		_bean = null;
    		_httpSession = null;
    	}
    }
} // end class LyWebSession
