/*
 * LyStrutsSplitter.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts;

import java.io.*;
import java.util.*;

import javax.servlet.http.*;

import leon.misc.*;
import leon.view.*;
import leon.view.LySplitter.ActivePart;
import leon.view.getinterface.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Splitter object (as described in the dtd) for web viewer.
 *
 * <p><b>SPLITTER: Panel with Splitter</b><br>
 * A container of panel type with splitter allows displaying different panels that are splitted by
 * vertical or horizontal separation bars. These bars can be moved so as to allocating more or less
 * place to each of the panels. The minimum number of panels that is required is two panels. There
 * is no constraint on the maximum number of panels allowed in such a container. Including such
 * containers in containers of same type allows getting windows combining vertical and horizontal
 * separators.</p>
 *
 * @version $Id: LyStrutsSplitter.java 36450 2012-08-09 09:35:21Z nhamon $
 * @author  Lyria S.A.
 */
public class LyStrutsSplitter extends LyStrutsContainer
    implements LyGraphicalSplitter
{
	public static final String __VERSION =
		"$Id: LyStrutsSplitter.java 36450 2012-08-09 09:35:21Z nhamon $";

	/** Maximum height of the components displayed in a splitter. */
	public static int	MAXIMUM_COMPONENT_HEIGHT = 450;
	
	/** The <b>panel</b> String value used within panel ids. */
    protected static final String PANEL = "panel".intern();

    /**
     * List of string indicating if elements of the area are framesets or not. If the name of the
     * frame is in the list, the corresponding frame is a frameset.
     */
    protected Vector<String>      _framesets;

    /** The splitter generic component. */
    protected LySplitter          _splitter;

	/** Indicates if a modification of the text must send an Ajax update. */
	protected boolean			  _notify;
    
	/**
	 * Constructor.
	 *
	 * @param splitter the splitter component
	 */
	public LyStrutsSplitter(LySplitter splitter)
	{
        super(splitter);
        
        _splitter = splitter;
        _notify = true;

        init();
	}
	
	/*
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	    pw.startComponent(this);
	    
	    updateRefreshInfo(pw);
	    
		int				 i, nAreas = _splitter.getPanelCount();
		String			 sizes = _splitter.getSizes();
		int				 size;
		String[]		 sizesTab = LyString.stringToList(sizes, ' ');
		int				 ns = ((sizesTab == null) ? 0 : sizesTab.length);
		boolean			 isVertical = (_splitter.getStyle() == LySplitter.Style.VERTICAL);
		
		if ((ns == 0) && (nAreas > 0))
		{
			// no value : use the same for all areas
			int  count = 0;
			
			sizes = "";
			
			for (i = 0; i < nAreas; i++)
			{
				if (i != 0)
					sizes += ",";
				
				if (i != (nAreas - 1))
				{
					sizes += 100/nAreas;
					count += 100/nAreas;
				}
				else
					sizes += 100 - count; // use remaining
			}
		}
		else if (ns < nAreas)
		{
			sizes = "";

			for (i = 0; i < ns; i++)
			{
				size = LyTools.intFromString(sizesTab[i]);
				if (i != 0)
					sizes += ",";
				if (size < 0)
					sizes += "*";
				else
					sizes += sizesTab[i];
			}

			if (ns > 0)
				sizes += ",";

			for (i = 0; i < (nAreas - ns); i++)
			{
				if (i != 0)
					sizes += ",";
				sizes += "*";
			}
		}
		else if (ns > nAreas)
		{
			sizes = "";

			for (i = 0; i < ns; i++)
			{
				size = LyTools.intFromString(sizesTab[i]);
				if (i != 0)
					sizes += ",";
				if (size < 0)
					sizes += "*";
				else
					sizes += sizesTab[i];
			}
		}
		else
		{
			sizes = "";
			for (i = 0; i < ns; i++)
			{
				size = LyTools.intFromString(sizesTab[i]);
				
				if (i != 0)
					sizes += ",";
				
				if (sizesTab[i].indexOf("px") > 0)
					sizes += sizesTab[i];
				else if (size < 0)
					sizes += "*";
				else
					sizes += sizesTab[i];
			}
		}

		sizesTab = LyString.stringToList(sizes, ',');

		boolean	hasFixedPart = ((sizes.indexOf('*') >= 0) || (sizes.indexOf("px") > 0)) && (nAreas == 2);
		String	divId = "splitter_" + _splitter.getIdCode() + "_" + i;
		String	parentId = divId + "_table";

		pw.append("<table border='0' cellpadding='0' cellspacing='0' id='" + parentId + "' width='100%' height='100%'>\n");

		if (!isVertical)
			pw.append("<tr>");
		
		Vector<LySplitter>	subSplitters;
		boolean				hasVisibleSubSplitters;
		LyArea				parentArea = _splitter.getParentArea();
		boolean				isInBorderPanel = false;
		
		if (parentArea != null)
		{
			LyArea	grandParentArea = parentArea.getParentArea();
			
			if (grandParentArea != null)
				isInBorderPanel = grandParentArea.isBorderPanel(parentArea);
		}
		
		LyTabs			parentTabs = _splitter.getParentTabs();
		LySplitter		parentSplitter = _splitter.getParentSplitter();
		boolean			fullSize = isVertical;
		boolean			setToAutoByScript = false;
		StringBuffer	script = new StringBuffer();
		
		if (!isInBorderPanel && (parentSplitter != null))
		{
			fullSize = true;
			
			LyComponent	parent = _splitter;
			LyPanel		firstParentPanel = parentSplitter.getPanel(0);
			LyPanel		secondParentPanel = parentSplitter.getPanel(1);
			boolean		found = false;
			
			while (parent != parentSplitter)
			{
				parent = parent.getParent();
				
				if (parent == firstParentPanel)
				{
					found = true;
					break;
				}
			}
			
			if (found)
				parent = firstParentPanel;
			else
				parent = secondParentPanel;
			
			parentId = "splitter_" + parent.getIdCode() + "_div";
		}
		else if (!isInBorderPanel && (parentTabs != null))
		{
			LyTabs	grandParentTabs = parentTabs.getParentTabs();
			
			if (grandParentTabs != null)
				parentTabs = grandParentTabs;
			
			fullSize = true;
			parentId = "tab_content_" + parentTabs.getIdCode();
		}
		
		script.append("<script type='text/javascript'>");
		script.append("getLeonardiTopFrame().addSplitterIds(new Array(");
		script.append(isVertical + ",'" + parentId + "'");
		
		String	arrowLeft = getImageUrl("LY_SPLITTER_ARROW_LEFT");
		String	arrowRight = getImageUrl("LY_SPLITTER_ARROW_RIGHT");
		String	arrowDown = getImageUrl("LY_SPLITTER_ARROW_DOWN");
		String	arrowUp = getImageUrl("LY_SPLITTER_ARROW_UP");

		for (i = 0; i < nAreas; i++)
		{
			LyPanel		panel = _splitter.getPanel(i);
			LyWebObject	internal = (LyWebObject)panel.getInternalView();
			String		partId = "splitter_" + panel.getIdCode();

			script.append(",");
			script.append("'" + partId + "_div'");

			hasVisibleSubSplitters = false;
			subSplitters = new Vector<LySplitter>();
			panel.getComponents(LySplitter.class, subSplitters);
			
			for (int j = 0; j < subSplitters.size(); j++)
			{
				LySplitter	subSplitter = subSplitters.elementAt(j);
				
				if (subSplitter.getParentTabs() != null)
				{
					LyStrutsTabs	webTabs = (LyStrutsTabs)subSplitter.getParentTabs().getInternalView();
					
					if (webTabs.getSelectedTab() == subSplitter.getParentTab())
					{
						hasVisibleSubSplitters = true;
						break;
					}
				}
			}

			if (i > 0)
			{
				String				splitterClassName = (isVertical ? "verticalSplitter" : "horizontalSplitter");
				LyPanel				prevPanel = _splitter.getPanel(i - 1);
				String				prevPartId = "splitter_" + prevPanel.getIdCode();
				StringBuffer		mouseDown = new StringBuffer();
				LyWebRequestInfo	info = LyStrutsServlet.getWebRequestInfo();
				HttpServletRequest	request = (info == null ? null : info.getRequest());
				String				target = (request == null ? null : (String)request.getParameter(LyWebConstants.TARGET_PARAM));
				boolean				resizable = _splitter.isResizable() && !hasFixedPart;

				if (resizable)
				{
					splitterClassName += "Resizable";
					mouseDown.append("javascript:startMoveSplitter(event, '");
					mouseDown.append(parentId);
					mouseDown.append("','");
					mouseDown.append(prevPartId + "_div");
					mouseDown.append("','");
					mouseDown.append(partId + "_div");
					mouseDown.append("','hidden_splitter','moving_splitter");
					mouseDown.append(isVertical ? "v" : "h");
					mouseDown.append("',");
					mouseDown.append(isVertical);
					mouseDown.append(",'" + getId() + "'");
					mouseDown.append(",'" + target + "');");
				}
				
				if (isVertical)
				{
					pw.append("<tr>");
					pw.append("<td height='4'><table id='" + splitterClassName + "' width='100%' height='4' cellpadding='1' cellspacing='0'");
					pw.append(" onMouseDown=\"" + mouseDown + "\">");
					pw.append("<tr><td align='right'>");
					
					if (resizable)
					{
						pw.append("<img class='splitterArrow' onMouseDown=\"javascript:hideSplitter('" + prevPartId + "_div',false,'" + getId() + "','" + target + "');\" src='");
						pw.append(arrowUp + "' border='0' title=\"" + translate(_splitter.getFirstTooltip()) + "\"/>");
						pw.append("</td><td align='left'>");
						pw.append("<img class='splitterArrow' onMouseDown=\"javascript:hideSplitter('" + prevPartId + "_div',true,'" + getId() + "','" + target + "');\" src='");
						pw.append(arrowDown + "' border='0' title=\"" + translate(_splitter.getSecondTooltip()) + "\"/>");
					}
					
					pw.append("</td></tr></table></td>\n");
					pw.append("</tr>\n");
				}
				else
				{
					pw.append("<td width='4'><table id='" + splitterClassName + "' width='4' height='100%' cellpadding='1' cellspacing='0'");
					pw.append(" onMouseDown=\"" + mouseDown + "\">");
					pw.append("<tr><td valign='bottom'>");
					
					if (resizable)
					{
						pw.append("<img class='splitterArrow' onMouseDown=\"javascript:hideSplitter('" + prevPartId + "_div',false,'" + getId() + "','" + target + "');\" src='");
						pw.append(arrowLeft + "' border='0' title=\"" + translate(_splitter.getFirstTooltip()) + "\"/>");
						pw.append("</td></tr><tr><td valign='top'>");
						pw.append("<img class='splitterArrow' onMouseDown=\"javascript:hideSplitter('" + prevPartId + "_div',true,'" + getId() + "','" + target + "');\" src='");
						pw.append(arrowRight + "' border='0' title=\"" + translate(_splitter.getSecondTooltip()) + "\"/>");
					}
					
					pw.append("</td></tr></table></td>\n");
				}
			}

			if (isVertical)
			{
				pw.append("<tr>\n");
				pw.append("<td id='" + partId + "' vAlign='top'>\n");
			}
			else
				pw.append("<td id='" + partId + "' vAlign='top' align='left' width='50px'>\n");

			pw.append("<div id='" + partId + "_div' style='");
			
			if (!LyWebConstants.POPUP_VIEW.equals(pw.getWindowName()))
			{
				if (isVertical)
					pw.append("width:150px;");
				else
					pw.append("width:100%;");
			}
			else
				setToAutoByScript = true;
			
			if ((!"*".equals(sizesTab[i]) || !isVertical)
					&& !isInBorderPanel
					&& !LyWebConstants.POPUP_VIEW.equals(pw.getWindowName()))
				pw.append("overflow:auto;");
			else if (isInBorderPanel && !isVertical && !LyWebConstants.POPUP_VIEW.equals(pw.getWindowName()))
				pw.append("overflow:auto;");

			if ((sizesTab[i].indexOf("px") > 0) && hasFixedPart)
				pw.append("height:" + sizesTab[i] + ";");
			else if (fullSize && !hasFixedPart)
				pw.append("height:50px;");
			else
			{
				if (!hasVisibleSubSplitters
						&& (nAreas > 1)
						&& (!(hasFixedPart && "*".equals(sizesTab[i]))))
				{
					int				scrollHeight = -1;
					Vector<LyPanel>	subPanels = new Vector<LyPanel>();
					LyPanel         subPanel;
	                LyWebObject     webSubPanel;
	                int             subScrollHeight;
					
					_splitter.getComponents(LyPanel.class, subPanels);

					int             j, n = subPanels.size();
					
					for (j = 0; j < n; j++)
					{
						subPanel = subPanels.elementAt(j);
						webSubPanel = (LyWebObject)subPanel.getInternalView();
						subScrollHeight = (webSubPanel == null ? 0 : webSubPanel.getScrollHeightForSplitter(pw));
						scrollHeight = Math.max(scrollHeight, subScrollHeight);
					}

					if (!isVertical && (scrollHeight > 0))
					{
						if ((sizesTab.length > nAreas) && (sizesTab[nAreas].indexOf("px") > 0))
							pw.append("height:" + sizesTab[nAreas] + ";");
						else
							pw.append("height:" + scrollHeight + "px;");
					}
					
					if ((scrollHeight > 0) && !isVertical)
						pw.append("width:100%;");
				}
			}
			
			pw.append("'>");
			internal.writeHtml(pw);
			pw.append("</div>");
			pw.append("</td>\n");

			if (isVertical)
				pw.append("</tr>\n");
		}
		
		if (nAreas < 2)
			script.append(",''");
		
		script.append(",'");
		script.append(sizesTab[0]);
		script.append("',");
		script.append(setToAutoByScript);
		script.append("));");
		
		// Do not generate script if top vertical splitter with a fixed part in a popup view
		boolean		generateScript = true;
		
		if ((parentSplitter == null)
				&& (parentTabs == null)
				&& isVertical
				&& hasFixedPart
				&& (_splitter.getParent() != null)
				&& (_splitter.getParent().getParent() instanceof LyView)
				&& LyWebConstants.POPUP_VIEW.equals(pw.getTarget()))
			generateScript = false;
		
		if (generateScript)
		{
			getWebViewManager().addAdditionalRefreshScripts(script.toString() + "checkSizeMainView('true', 'false');</script>", false);
			pw.addEndScript(script.toString() + "</script>", false, false);
		}
		
		if (isVertical)
			pw.append("</tr>");

		pw.append("</table>\n");
		
		pw.endComponent(this);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.LyWebPageWriter#setProperty(java.lang.String, java.lang.String)
	 */
	public void setProperty(String name, String value)
	{
		super.setProperty(name, value);
		
		if ("setSize".equals(name) && (value != null))
		{
			_notify = false;
			
			if (value.indexOf("0%") == 0)
				_splitter.setSizes("0 100");
			else if (value.indexOf("100%") == 0)
				_splitter.setSizes("100 0");
			else
			{
				int	integer = LyTools.intFromString(value);
				
				_splitter.setSizes(integer + " " + (100 - integer));
			}

			_notify = true;
		}
	}

	/**
	 * Recursively computes the list of panels that must be resized when current splitter is moved.
	 * 
	 * @param panel the parent panel to extract panels from.
	 * @param bottom indicates if bottom or right panels must be retrieved.
	 * @param vertical indicates whether the splitter is vertical or not.
	 * @param panels
	 */
	protected void getResizablePanels(LyPanel panel, boolean bottom, boolean vertical, ArrayList<LyPanel> panels)
	{
		// Retrieve the deepest area
		LyPanel		newPanel = panel;
		
		while (newPanel instanceof LyArea)
		{
			LyArea	area = (LyArea)newPanel;
			
			if ((area.getPanelCount() == 1) && (area.getPanel(0) instanceof LyArea))
				newPanel = area.getPanel(0);
			else
			{
				if (area.getPanelCount() == 1)
					newPanel = area.getPanel(0);
				
				break;
			}
		}
		
		// For a splitter, call the method recursively
		if (newPanel instanceof LySplitter)
		{
			LySplitter	subSplitter = (LySplitter)newPanel;
			
			if (vertical == (subSplitter.getStyle() == LySplitter.Style.VERTICAL))
				getResizablePanels(subSplitter.getPanel(bottom ? 1 : 0), bottom, vertical, panels);
			else
			{
				getResizablePanels(subSplitter.getPanel(0), bottom, vertical, panels);
				getResizablePanels(subSplitter.getPanel(1), bottom, vertical, panels);
			}
		}
		else
			panels.add(panel);
	}

	/**
	 * Overriden to return the URL to the Leoanrdi struts 'page' action.
	 *
	 * @return The base url for the internal frames and elements of the frame.
	 */
	protected String getFrameBaseUrl()
	{
		// get the url to the page action
		LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();
		String			    url = viewManager.getActionAbsoluteUrl(
				LyStrutsConstants.PATH_ACTION_PAGE, null, true);

		return url;
	}
	
	   /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPage#init()
     */
    public void init()
    {
        super.init();
        _framesets = new Vector<String>();
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPageWriter#getRedirect(java.lang.String)
     */
    public String getRedirect(String frame)
    {
        int            i, nAreas = _splitter.getPanelCount();
        LyPanel        panel;
        LyWebComponent webComponent;
        String         idPanel;

        for (i = 0; i < nAreas; i++)
        {
            panel = _splitter.getPanel(i);
            idPanel = getFrameName(i);

            // FFT00924
            // We are asking for a specific frame
            // We don't care about the other frames...
            if ((frame == null) || !frame.equals(idPanel))
                continue;

            webComponent = (LyWebComponent)panel.getInternalView();

            if (webComponent instanceof LyWebPageWriter)
                return ((LyWebPageWriter)webComponent).getRedirect(frame);
        }
        return null;
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPage#writeBeginPage(java.io.Writer,
     *      java.lang.String)
     */
    public void writeBeginPage(Writer pw, String frame)
        throws IOException
    {
        boolean isFrameSet = _framesets.contains(frame);

        if (!isFrameSet)
            super.writeBeginPage(pw, frame);
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPage#writeEndPage(java.io.Writer,
     *      java.lang.String)
     */
    public void writeEndPage(Writer pw, String frame)
        throws IOException
    {
        boolean isFrameSet = _framesets.contains(frame);

        if (!isFrameSet)
            super.writeEndPage(pw, frame);
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPageWriter#preparePage(java.lang.String,
     *      java.lang.String)
     */
    public void preparePage(String frame, String targetWindow)
    {
        if (frame == null)
            return;

        int            i, nAreas = _splitter.getPanelCount();
        LyPanel        panel;
        LyWebComponent webComponent;
        String         idPanel;

        for (i = 0; i < nAreas; i++)
        {
            idPanel = getFrameName(i);
            panel = _splitter.getPanel(i);
            webComponent = (LyWebComponent)panel.getInternalView();

            if (webComponent instanceof LyWebPageWriter)
                ((LyWebPageWriter)webComponent).setFrameName(_name + "." + idPanel);
        }

        super.preparePage(frame, targetWindow);

        // Write the buffers for areas
        for (i = 0; i < nAreas; i++)
        {
            panel = _splitter.getPanel(i);
            idPanel = getFrameName(i);

            // FFT00924
            // We are asking for a specific frame
            // We don't care about the other frames...
            if (!frame.equals(idPanel))
                continue;

            clearBuffer(idPanel);
            setActiveBuffer(idPanel);

            webComponent = (LyWebComponent)panel.getInternalView();

            if ((webComponent instanceof LyWebPageWriter)
                    && (((LyWebPageWriter)webComponent).isFrameSet() || _noFrames))
            {
                _framesets.addElement(idPanel);
                ((LyWebPageWriter)webComponent).preparePage(frame, null);
            }

            if (webComponent instanceof LyWebPageWriter)
                ((LyWebPageWriter)webComponent).setFrameName(_name + "." + idPanel);

            webComponent.writeHtml(this);
        }
    } // end method preparePage

    protected String getScrolling(LyArea area)
    {
        LyArea.Scrolling    areaScrolling = area.getScrolling();
      
        if (areaScrolling == LyArea.Scrolling.AUTO)
            return "auto";
        else if (areaScrolling == LyArea.Scrolling.YES)
            return "yes";
            
        if ((area.getChildCount() == 1)
                && (area.getPanel(0) instanceof LyArea))
            return getScrolling((LyArea)area.getPanel(0));
        else
            return "no";
        }
        
    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPageWriter#isFrameSet()
     */
    public boolean isFrameSet()
    {
        return (!_noFrames);
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPage#setNoFrames(boolean)
     */
    public void setNoFrames(boolean noFrames)
    {
        super.setNoFrames(noFrames);

        int         i, nElements = _splitter.getPanelCount();
        LyComponent element;
        Object      internal;

        for (i = 0; i < nElements; i++)
        {
            element = _splitter.getPanel(i);
            internal = element.getInternalView();

            if (internal instanceof LyWebPageWriter)
                ((LyWebPageWriter)internal).setNoFrames(noFrames);
        }
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPageWriter#setPrintMode(boolean)
     */
    public void setPrintMode(boolean printMode)
    {
        super.setPrintMode(printMode);

        int         i, nElements = _splitter.getPanelCount();
        LyComponent element;
        Object      internal;

        for (i = 0; i < nElements; i++)
        {
            element = _splitter.getPanel(i);
            internal = element.getInternalView();

            if (internal instanceof LyWebPageWriter)
                ((LyWebPageWriter)internal).setPrintMode(printMode);
        }
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPageWriter#getPageWriters()
     */
    public Vector<LyWebPageWriter> getPageWriters()
    {
        Vector<LyWebPageWriter>        pageWriters = new Vector<LyWebPageWriter>();
        Vector<LyWebPageWriter>        subPageWriters;
        int            i, j, nSubElements, nElements = _splitter.getPanelCount();
        LyComponent    element;
        LyWebComponent internal;

        for (i = 0; i < nElements; i++)
        {
            element = _splitter.getPanel(i);
            internal = (LyWebComponent)element.getInternalView();

            if (internal instanceof LyWebPageWriter)
            {
                subPageWriters = ((LyWebPageWriter)internal).getPageWriters();
                nSubElements = subPageWriters.size();

                for (j = 0; j < nSubElements; j++)
                    pageWriters.addElement(subPageWriters.elementAt(j));
            }
        }

        pageWriters.addElement(this);
        return pageWriters;
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPageWriter#setForceReloadState(boolean)
     */
    public void setForceReloadState(boolean force)
    {
        int            i, nElements = _splitter.getPanelCount();
        LyComponent    element;
        LyWebComponent internal;

        _forceReloadState = force;

        // if force is true, we set the state of all the sub pageWriter
        if (force)
        {
            for (i = 0; i < nElements; i++)
            {
                element = _splitter.getPanel(i);
                internal = (LyWebComponent)element.getInternalView();

                if (internal instanceof LyWebPageWriter)
                    ((LyWebPageWriter)internal).setForceReloadState(true);
            }
        }
    }

    /**
     * Gets the name of the frame at the given index.<br>
     * The name of a frame is the id of the splitter's panel in which it is shown.
     *
     * @param  i the position of the frame (or panel) in this splitter
     * @return the name of the frame (= id of the panel)
     */
    protected String getFrameName(int i)
    {
        int     nAreas = _splitter.getPanelCount();
        LyPanel panel;
        String  idPanel = null;

        if (i >= nAreas)
            return null;

        panel = _splitter.getPanel(i);
        idPanel = null;

        if (panel instanceof LyGetIdInterface)
            idPanel = ((LyGetIdInterface)panel).getId();

        if ((idPanel == null) || (idPanel.length() == 0))
            idPanel = PANEL + panel.getIdCode();

        return idPanel;
    }

    /*
     * (non-Javadoc)
     * @see leon.view.web.LyWebPage#getBuffer(java.lang.String)
     */
    public StringBuffer getBuffer(String key)
    {
        StringBuffer result = super.getBuffer(key);

        if ((result == null) && (_splitter != null))
        {
            int            i, nElements = _splitter.getPanelCount();
            LyComponent    element;
            LyWebComponent internal;

            for (i = 0; i < nElements; i++)
            {
                element = _splitter.getPanel(i);
                internal = (LyWebComponent)element.getInternalView();

                if (internal instanceof LyWebPageWriter)
                {
                    result = ((LyWebPageWriter)internal).getBuffer(key);

                    if (result != null)
                        break;
                }
            }
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * @see leon.view.graphic.LyGraphicalSplitter#setSizes(java.lang.String)
     */
    public void setSizes(String sizes)
    {
		if (!_notify)
			return;
		
    	LyTabs	tabs = _splitter.getParentTabs();
    	boolean	refresh = true;
    	
    	if (tabs != null)
    	{
    		LyStrutsTabs	strutsTabs = (LyStrutsTabs)tabs.getInternalView();
    		
    		if (strutsTabs != null)
    		{
    			strutsTabs.refreshFrames(true);
    			refresh = false;
    		}
    	}
    	
    	if (refresh)
    		refresh();
    }

	public void setSelected(boolean status)
	{
		// TODO Auto-generated method stub
		
	}

	public void setActivePart(ActivePart activePart)
	{
		// TODO Auto-generated method stub
		
	}

} // end class LyStrutsSplitter
