/*
 * LyStrutsTab.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Tab object (as described in the dtd) for web viewer.
 *
 * <p><b>TAB: Tab</b><br>
 * A tab belongs to a container of tab type. It is identified by a name (label) and it allows
 * displaying a panel.</p>
 *
 * @version $Id: LyStrutsTab.java 23300 2009-12-17 13:56:41Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.LyTab
 */
public class LyStrutsTab extends LyWebObject
	implements LyGraphicalTab
{
	public static final String __VERSION =
		"$Id: LyStrutsTab.java 23300 2009-12-17 13:56:41Z jlsaouli $";

	/** The tab generic component. */
	protected LyTab			   _tab;

	/**
	 * Class constructor of a Tab object for the struts target.
	 *
	 * @param tab the generic Tab defined for every type of view targets
	 */
	public LyStrutsTab(LyTab tab)
	{
		super(tab);
		_tab = tab;
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
		LyPanel		   panel = _tab.getPanel();

		if (!_tab.isEnabled() || (panel == null))
			return;

		pw.startComponent(this);
		
		LyWebComponent internal = (LyWebComponent)panel.getInternalView();

		internal.writeHtml(pw);
		
		pw.endComponent(this);
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetColorInterface#setColor(java.lang.String)
	 */
	public void setColor(String color)
	{
		refresh();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetEnabledInterface#setEnabled(boolean)
	 */
	public void setEnabled(boolean enabled)
	{
        LyTabs       tabs = _tab.getParentTabs();
        LyStrutsTabs webTabs = (LyStrutsTabs)tabs.getInternalView();
        
		webTabs.refreshFrames(true);
	}
} // end class LyStrutsTab
