/*
 * LyStrutsText.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import java.awt.*;

import leon.misc.*;
import leon.view.*;
import leon.view.LyTextBase.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Text object (as described in the dtd) for web viewer.
 *
 * <p><b>TEXT: Input Field of Text Type</b><br>
 * An input field of Text type allows displaying a graphical input or text consultation component.
 * It does not comprise any element. It is possible to specify if the text should be hidden, if it
 * should respect a given format, if it can be divided into several lines and the maximum number of
 * characters that it can contain.</p>
 *
 * @version $Id: LyStrutsText.java 24563 2010-03-04 08:15:19Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.LyText
 */
public class LyStrutsText extends LyStrutsFieldItem
	implements LyGraphicalText
{
	public static final String __VERSION =
		"$Id: LyStrutsText.java 24563 2010-03-04 08:15:19Z jlsaouli $";

	/** Indicates whether non editable text must be displayed as labels. */
	public static boolean	   USE_LABEL_FOR_NON_EDITABLE_TEXT = false;

	/** The text generic component. */
	protected LyText		   _text;

	/**
	 * Class constructor of a Text object for the struts target.
	 *
	 * @param text the generic Text defined for every type of view targets
	 */
	public LyStrutsText(LyText text)
	{
		super(text);
		_text = text;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see leon.view.web.struts.LyStrutsFieldItem#isEditable()
	 */
	public boolean isEditable()
	{
		return (_text.isEditable() && super.isEditable());
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see leon.view.setinterface.LySetEditableInterface#setEditable(boolean)
	 */
	public void setEditable(boolean editable)
	{
		LyComponent		parent = _text.getParent();

		if (parent instanceof LyFormField)
		{
			if (!_text.isNotifyValueChanged())
			{
				LyFormField			formField = (LyFormField)parent;
				LyStrutsFormField	webField = (LyStrutsFormField)formField.getInternalView();

				webField.refreshDivContent();
			}
		}
		else
			refresh();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see leon.view.web.struts.LyStrutsFieldItem#setLabel(java.lang.String)
	 */
	public void setLabel(String label)
	{
		LyComponent parent = _text.getParent();

		if (_updating)
			return;

		if (parent instanceof LyFormField)
		{
			if (!_text.isNotifyValueChanged())
			{
				boolean jsRefresh = (isEditable() || !USE_LABEL_FOR_NON_EDITABLE_TEXT) && (_text.getType() != LyText.Type.RICH);
				
				if ((label != null) && (label.indexOf("</script>") >= 0))
					jsRefresh = false;
				
				if ((_text.getType() == LyText.Type.RICH) && isEditable())
				{
					refresh();
				}
				else if (jsRefresh)
				{
					// For simple and editable text, just change label
					String  		name = getId();
					StringBuffer	buffer = new StringBuffer();

	                if (label == null)
	                    label = "";
	                {
    					label = LyString.replace(label, "\n", "\\n");
    					label = LyString.replace(label, "\\", "\\\\");
    					label = LyString.replace(label, "\\\\n", "\\n");
    					label = LyString.replace(label, "\"", "\\\"");
    					label = LyString.replace(label, "\\'", "\\\\'");
    	                label = label.replace((char)128, '');
    	                label = label.replace("", "\\u20AC");
	                }

					buffer.append("<script type='text/javascript'>\n//<![CDATA[\nSetFieldValue(\"" + _formName + "\",\"" + name + "\",\"" + label + "\");\n//]]>\n</script>");
					getWebViewManager().addAdditionalRefreshScripts(buffer.toString());
				}
				else
				{
					LyFormField		 formField = (LyFormField)parent;
					LyStrutsFormField webField = (LyStrutsFormField)formField.getInternalView();

					webField.refreshDivContent();
				}
			}
		}
		else
			refresh();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	    pw.startComponent(this);

		updateRefreshInfo(pw);

		String  formName = "forms[0]";//getFormName(pw);
		String  name = getId();
		String  label = _text.getLabel();
		int     maxChars = _text.getNChars();
		boolean needCheck = needCheck();
		boolean isSecret = _text.isSecret();
		boolean isEditable = _text.isEditable();
		boolean isEnabled = true;
		String  textId = LyString.getJavascriptStringVar(_text.getIdCode());
		String  style = "";

		if (!_text.isDefaultFont() || !_text.isDefaultFontColor())
			style += getFontString(_text.getFont(),_text.getFontColor());

		// Set the text align value, by default align left.
		if (_text.getTextAlign() == TextAlign.RIGHT)
			style += "text-align:right;";
		else if (_text.getTextAlign() == TextAlign.CENTER)
			style += "text-align:center;";

		LyForm form = ((_formField.getParent() instanceof LyForm) ? (LyForm)_formField.getParent()
				  : null);

		if (!_formField.isEnabled())
		{
			isEditable = false;
			isEnabled = false;
		}
		else if (!_formField.isEditable())
			isEditable = false;

		if ((form != null) && !form.isEditable())
			isEditable = false;

		if (isEnabled)
		{
			if (!_text.isDefaultEnabledColor())
			{
				style += "background-color:" + getHexColor(_text.getEnabledColor()) + ";";
				style += "background-image:none;";
			}
		}
		else
		{
			if (!_text.isDefaultDisabledColor())
			{
				style += "background-color:" + getHexColor(_text.getDisabledColor()) + ";";
				style += "background-image:none;";
			}
		}

		if (label == null)
			label = "";

		// If we have a format, the max number of chars is the number of columns
		if (_text.getFormat() != null)
			maxChars = _text.getNCols();

		if (_text.getType() == LyText.Type.RICH)
		{
			writeRichText(pw);
		}
		// If not editable, just write text
		else if ((USE_LABEL_FOR_NON_EDITABLE_TEXT || pw.isPrintMode()) && !isEditable)
		{
			label = getHtmlString(label);

			if (label != null)
			{
				if (!isSecret)
				{
					boolean mail = false;
					boolean isUrl;
					boolean isUrlRecurse;
					boolean isWrap = _text.isWrap();

					// FFT00180/IpBoss : Chanes de caractres contenant @
					if ((label.indexOf(' ') < 0) && (label.indexOf('@') > 0)
							&& (label.indexOf("mailto:") < 0))
					{
						// Checks that the domain part of the label is an IP address
						String domain = label.substring(label.lastIndexOf('@') + 1);
						String[] domainParts = domain.split("\\.");
						boolean isIp = true;
						if (domainParts.length == 4)
						{
							for (int i = 0; i < domainParts.length && isIp; i++)
							{
								String part = domainParts[i];
								isIp &= part.matches("(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)");
							}
						}
						// If the domain is not an IP address, then the label is an email
						if (!isIp)
							mail = true;
					}

					// Contains one or more http:// or starts with www.
					isUrlRecurse = !mail && (label != null)
							&& ((label.indexOf("http://") >= 0)
							    || (label.indexOf("https://") >= 0)
							    || (label.indexOf("mailto:") >= 0));
					isUrl = (label != null) && (isUrlRecurse || (label.indexOf("www.") == 0));

					if (!isWrap)
						pw.append("<table><tr><td nowrap='true'>\n");

					if (mail && !pw.isPrintMode())
						pw.append("<a href='mailto:" + label + "'>");
					else if (isUrl)
					{
						String target = LyWebConstants.TARGET_FRAME;

						if (LyWebViewManager.WEB_FILE_NEW_WINDOW)
							target = "_blank";

						if (isUrlRecurse)
						{
							pw.append("<font class='label'");

							if (style.trim().length() > 0)
								pw.append(" style=\"" + style + "\"");

							pw.append(">");
							pw.append(replaceUrl(label, target, pw.isPrintMode()) + "</font>");
						}
						else if (!pw.isPrintMode())
							pw.append("<a href='http://" + label + "' target=\"" + target + "\">");
					}

					if (!isUrlRecurse)
					{
						pw.append("<font class='label'");

						if (style.trim().length() > 0)
							pw.append(" style=\"" + style + "\"");

						pw.append(">");

						pw.append(label);
						pw.append("</font>");

						if ((mail || isUrl) && !pw.isPrintMode())
							pw.append("</a>");
					}

					if (!isWrap)
						pw.append("</td></tr></table>\n");
				}
				else
				{
					int			 i, n = label.length();
					StringBuffer buffer = new StringBuffer(n);

					for (i = 0; i < n; i++)
						buffer.append('*');

					pw.append(buffer.toString());
				}
			}
		}
		else
		{
			// We just need to change the entities
			label = getHtmlEntitiesString(label);

			if (pw.writeDivOnly())
			{
				label = LyString.replace(label, "\r\n", "\\n");
				label = LyString.replace(label, "\n", "\\n");
			}

			String caption = getCaption();
			String maxCharsStr = "";
			String lengthErrorMessage = "";

			LyWebDecoration	deco = getWebViewManager().getDecoration();
			String			className = deco.getClassName(_text, pw);

			if (maxChars > 0)
			{
				maxCharsStr = " maxlength='" + maxChars + "'";
				lengthErrorMessage = getMessage("LY_ER_TEXT_NCHARS_WEB",
						new String[]
						{
							String.valueOf(maxChars), caption, "%1"
						});
			}

			int			textWidth = getTextWidth();
			int			textHeight = getTextHeight();

			if (textWidth < 0)
			{
				style += "width:100%;";

				String	script = "<script type='text/javascript'>CheckComponentMinWidth('" + name + "'," + Math.abs(textWidth) + ");</script>";

				getWebViewManager().addAdditionalRefreshScripts(script);
				pw.addEndScript(script, false);
			}
			else
				style += "width:" + textWidth + ";";

			if (textHeight > 0)
				style += "height:" + textHeight + ";";

			String	onKeyUpFunctions = _text.getEventFunctions(LyComponent.EventType.KEY_RELEASED, ';');
			String	onKeyDownFunctions = _text.getEventFunctions(LyComponent.EventType.KEY_TYPED, ';');
			String	onKeyPressedFunctions = _text.getEventFunctions(LyComponent.EventType.KEY_PRESSED, ';');
			String	onMouseOverFunctions = _text.getEventFunctions(LyComponent.EventType.MOUSE_ENTERED, ';');
			String	onMouseOutFunctions = _text.getEventFunctions(LyComponent.EventType.MOUSE_EXITED, ';');
			String	onFocusFunctions = _text.getEventFunctions(LyComponent.EventType.FOCUS_GAINED, ';');
			String	onBlurFunctions = _text.getEventFunctions(LyComponent.EventType.FOCUS_LOST, ';');
			String	onPasteFunctions = _text.getEventFunctions(LyComponent.EventType.PASTE, ';');

			if (_text.getNRows() > 1)
			{
				pw.append("<textarea id=\"" + name + "\" name=\"" + name + "\"");

				if (style.trim().length() > 0)
					pw.append(" style='" + style + "'");

				if (className != null)
					pw.append(" class='" + className + "'");

				if (!isEnabled)
					pw.append(" disabled='true'");
				else if (!isEditable)
				{
					pw.append(" readonly='true'");

					if (className == null)
						pw.append(" class='readOnlyField'");
				}

				if (needCheck || (maxChars > 0))
				{
					pw.addBeginScript("<script type='text/javascript'>var message_"
						+ textId + " = \"" + lengthErrorMessage + "\";</script>", false);

					LyWebViewManager viewManager = getWebViewManager();

					viewManager.addAdditionalRefreshScripts(
						"<script type='text/javascript'>\n//<![CDATA[\nvar textFrame = findTarget(null, '" + _lastFrameName + "', null);"
						+ "if (textFrame != null) textFrame.message_"
						+ textId
						+ " = \""
						+ lengthErrorMessage
						+ "\";\n//]]>\n</script>");
				}

				// NeedCheck on text fields.
				if (needCheck)
				{
					pw.append(" onChange=\"javascript:");
					pw.append("FieldChange(");
					pw.append("window.document." + formName);
					pw.append(",this,'" + _text.getRootController() + "');\"");
				}

				if (maxChars > 0)
					pw.append(" onpaste=\"return checkTextAreaPaste(this, " + maxChars + ");\" onkeypress=\"return checkTextAreaLength(this, " + maxChars + ", event);\"");

				if (onKeyUpFunctions.length() > 0)
					pw.append(" onkeyup=\"" + onKeyUpFunctions + "\"");

				if (onKeyDownFunctions.length() > 0)
					pw.append(" onkeydown=\"" + onKeyDownFunctions + "\"");

				if (onKeyPressedFunctions.length() > 0)
					pw.append(" onkeypress=\"" + onKeyPressedFunctions + "\"");

				if (onMouseOverFunctions.length() > 0)
					pw.append(" onmouseover=\"" + onMouseOverFunctions + "\"");

				if (onMouseOutFunctions.length() > 0)
					pw.append(" onmouseout=\"" + onMouseOutFunctions + "\"");

				if (onFocusFunctions.length() > 0)
					pw.append(" onfocus=\"" + onFocusFunctions + "\"");

				if (onBlurFunctions.length() > 0)
					pw.append(" onblur=\"" + onBlurFunctions + "\"");

				if (onPasteFunctions.length() > 0)
					pw.append(" onpaste=\"" + onPasteFunctions + "\"");

				pw.append(">");
				pw.append(label);
				pw.append("</textarea>");
			}
			else
			{
				int     autoCompletionHeight = 0;
				int     autoCompletionwidth = (((_text.getNCols() < 0) ? LyText.DEFAULT_NCOLS : _text.getNCols())
							* 5) + 33;
				boolean autoCompletion = (_text.getAutoCompletionValues() != null)
						&& (_text.getAutoCompletionValues().length() > 0);

				// For auto-completion, add a table to get a correct position for auto-completion div
				if (autoCompletion)
					pw.append("<table border='0' cellpadding='0' cellspacing='0'><tr><td>");

				pw.append("<input id='" + name + "'");
				pw.append(" name='" + name + "'");

				if (style.trim().length() > 0)
					pw.append(" style='" + style + "'");

				if (className != null)
					pw.append(" class='" + className + "'");

				if (!isEnabled)
					pw.append(" disabled='true'");
				else if (!isEditable)
				{
					pw.append(" readonly='true'");

					if (className == null)
						pw.append(" class='readOnlyField'");
				}

				if (isSecret)
				{
					pw.append(" type='password'");

					int c, nc = ((label == null) ? 0 : label.length());

					for (c = 0, label = ""; c < nc; c++)
						label += '*';
				}

				pw.append(" value=\"" + label + "\"");

				if (_text.isNotifyValueChanged() && !autoCompletion)
				{
					if (_text.useEventDefaultFunction(LyComponent.EventType.KEY_RELEASED))
					{
						onKeyUpFunctions = "javascript:FieldChange(window.document." + formName + ",this,'" +
							_text.getRootController() + "');" + onKeyUpFunctions;
					}
				}
				// FFT00059 : NeedCheck on text fields.
				else if (needCheck && !autoCompletion)
				{
					pw.append(" onChange=\"javascript:FieldChange(window.document." + formName);
					pw.append(",this,'" + _text.getRootController() + "');\"");
				}
				else if (autoCompletion)
				{
					String jsName = "AutoCompletionListe_" + _text.getIdCode();
					String autoCompletionString = _text.getAutoCompletionValues();
					int    pos = autoCompletionString.indexOf(LyTools.PIPE_SEP);
					int    autoCompletionMax = -1;

					if (pos > 0)
					{
						String autoCompletionValue = autoCompletionString.substring(0, pos);

						autoCompletionMax = LyTools.intFromString(autoCompletionValue);
						autoCompletionString = autoCompletionString.substring(pos + 1);
					}

					String[]     autoCompletionList = LyString.stringToList(autoCompletionString,
							LyTools.VALUE_SEP);
					String		 autoCompletionItem;
					StringBuffer listeVar = new StringBuffer();
					int			 i,
								 n = ((autoCompletionList == null) ? 0 : autoCompletionList.length);
					String		 colorCode = "LY_HIGHLIGHTED_BACKGROUND";
					String		 bgColor = "";
					String		 controllerParam = (needCheck ? ", '" + _text.getRootController()
							+ "'" : ", ''");

					bgColor = getHexColor(colorCode);
					listeVar.append("<script type='text/javascript'>\n");
					listeVar.append("var " + jsName + " = new Array();\n");

					for (i = 0; i < n; i++)
					{
						autoCompletionItem = autoCompletionList[i];
						autoCompletionItem = LyString.replace(autoCompletionItem, "\\", "\\\\");
						autoCompletionItem = LyString.replace(autoCompletionItem, "\"", "\\\"");
						listeVar.append(jsName + "[" + i + "] = [\"" + autoCompletionItem + "\", \"" + autoCompletionItem + "\", true];\n");
					}

					listeVar.append("var textFrame = findTarget(null, '" + _lastFrameName + "', null);\n");
					listeVar.append("if (textFrame != null) textFrame.addAutoCompletionValues('" + jsName + "', " + jsName + ");\n");
					listeVar.append("else addAutoCompletionValues('" + jsName + "', " + jsName + ")\n");
					listeVar.append("</script>\n");
					pw.addBeginScript(listeVar.toString(), false);
					getWebViewManager().addAdditionalRefreshScripts(listeVar.toString());

					if (autoCompletionMax > 0)
						n = autoCompletionMax;

					autoCompletionHeight = Math.min(150, (n * 21) + 2);

					if (_text.useEventDefaultFunction(LyComponent.EventType.KEY_RELEASED))
					{
						onKeyUpFunctions = "javascript:refreshAutoCompletion('" + formName
							+ "', '" + _text.getIdCode() + "', '" + getJavascriptString(name) + "', '"
							+ jsName + "', '" + bgColor + "'," + autoCompletionMax
							+ ",'" + (autoCompletionwidth - 3) + "', event" + controllerParam + ", false, true);" + onKeyUpFunctions;
					}

					if (_text.useEventDefaultFunction(LyComponent.EventType.FOCUS_GAINED))
					{
						onFocusFunctions = "javascript:resetAutoCompletionFocus('" + formName
							+ "', '" + _text.getIdCode() + "', '" + getJavascriptString(name) + "');" + onFocusFunctions;
					}

					if (_text.useEventDefaultFunction(LyComponent.EventType.FOCUS_LOST))
					{
						onBlurFunctions = "javascript:hideAutoCompletionTimeout('" + formName
							+ "', '" + _text.getIdCode() + "', '" + getJavascriptString(name) + "');" + onBlurFunctions;
					}

					if (needCheck)
					{
						pw.append(" onChange=\"javascript:hideAutoCompletionTimeout('" + formName
							+ "', '" + _text.getIdCode() + "', '" + getJavascriptString(name) + "'" + controllerParam + ");\"");
					}

					pw.append(" autocomplete='off'");
				}

				if (onKeyUpFunctions.length() > 0)
					pw.append(" onkeyup=\"" + onKeyUpFunctions + "\"");

				if (onFocusFunctions.length() > 0)
					pw.append(" onfocus=\"" + onFocusFunctions + "\"");

				if (onBlurFunctions.length() > 0)
					pw.append(" onblur=\"" + onBlurFunctions + "\"");

				if (onKeyDownFunctions.length() > 0)
					pw.append(" onkeydown=\"" + onKeyDownFunctions + "\"");

				if (onKeyPressedFunctions.length() > 0)
					pw.append(" onkeypress=\"" + onKeyPressedFunctions + "\"");

				if (onMouseOverFunctions.length() > 0)
					pw.append(" onmouseover=\"" + onMouseOverFunctions + "\"");

				if (onMouseOutFunctions.length() > 0)
					pw.append(" onmouseout=\"" + onMouseOutFunctions + "\"");

				if (onPasteFunctions.length() > 0)
					pw.append(" onpaste=\"" + onPasteFunctions + "\"");

				pw.append(maxCharsStr + "/>");

				// Auto-completion
				if (autoCompletion)
				{
					String			controllerParam = (needCheck ? ", '" + _text.getRootController() + "'" : "");

					pw.append("</td></tr></table>");
					pw.append("<input name='AutoCompletionFocus_" + _text.getIdCode()
						+ "' type='hidden' value='false'/>\n");
					pw.append("<input name='AutoCompletionIndex_" + _text.getIdCode()
						+ "' type='hidden' value='-1'/>\n");
					pw.append("<div class='autoCompletion' id='AutoCompletion_" + _text.getIdCode()
						+ "'");
					pw.append(" style='position:absolute;width:" + autoCompletionwidth + ";height:"
						+ autoCompletionHeight);
					pw.append(";visibility:hidden;overflow:auto;z-index:100;table-layout:fixed;border:solid 1px #000000;background-color:#FFFFFF;'");
					pw.append(" scrolling='auto' frameborder='0'");
					pw.append(" onFocus=\"resetAutoCompletionFocus('" + formName + "', '"
						+ _text.getIdCode() + "', '" + getJavascriptString(name) + "');\"");
					pw.append(" onBlur=\"hideAutoCompletionTimeout('" + formName + "', '"
						+ _text.getIdCode() + "', '" + getJavascriptString(name) + "'" + controllerParam
						+ ");\">");
					pw.append("</div>\n");

					// Add an iframe to hide other components that may appear over the div
					// (bug on IE with div and select components).
					if (getWebViewManager().getBrowser() == LyWebViewManager.BROWSER_MSIE)
					{
						pw.append("<iframe name='AutoCompletionFrame_" + _text.getIdCode() + "'");
						pw.append(" style='position:absolute;width=" + autoCompletionwidth
							+ ";height=" + autoCompletionHeight);
						pw.append(";visibility:hidden;z-index:90;'");
						pw.append(" scrolling='no' frameborder='0'>");
						pw.append("</iframe>\n");
					}
				}
			}
		}

		pw.endComponent(this);
	} // end method writeHtml

	/**
	 * Returns the width of the text component.
	 * If a fixed width must be used, result is a positive integer.
	 * If a minimum width must be used, result is a negative integer.
	 */
	@SuppressWarnings("deprecation")
	public int getTextWidth()
	{
		boolean			 fill = false;
		int	 			 textWidth = -1;

		// If a constraint is set on the text, use the nCols and nRows values to compute
		// text size in pixels.
		if ((_text.getParent() instanceof LyFormField)
				&& (((LyFormField)(_text.getParent())).getConstraint() != null))
		{
			LyFormField		formField = (LyFormField)_text.getParent();
			LyConstraint	constraint = formField.getConstraint();

			if ((constraint != null)
					&& ((constraint.getFill() == LyConstraint.Fill.BOTH) || (constraint.getFill() == LyConstraint.Fill.HORIZONTAL)))
				fill = true;

			if ((constraint != null) && (constraint.getNCols() > 0))
			{
				textWidth = constraint.getNCols();

				if (formField.getFieldUnit() != null)
				{
					LyFieldUnit	unit = formField.getFieldUnit();
					LyFieldItem unitItem = unit.getFieldItem();

					if (unitItem instanceof LyLabel)
					{
						LyLabel			label = (LyLabel)unitItem;
						String			value = label.getLabel();
						Font			font = (Font)getEnvironment().getFont(label.getFont());
						Toolkit			toolkit = Toolkit.getDefaultToolkit();
						FontMetrics		fm = toolkit.getFontMetrics(font);

						value = " " + translate(value);
						textWidth -= fm.stringWidth(value) + 1;
					}
				}
			}
		}

		if (textWidth < 0)
			textWidth = getTextWidth((_text.getNCols() < 0) ? LyText.DEFAULT_NCOLS : _text.getNCols());

		if (_text.getType() == LyText.Type.RICH)
			textWidth = Math.max(480, textWidth);

		if (!fill)
			return textWidth;
		else
			return -textWidth;
	}

	/**
	 * Returns the CSS style to be used for date input.
	 */
	@SuppressWarnings("deprecation")
	protected int getTextWidth(int nCols)
	{
		Font			font = (Font)getEnvironment().getFont(_text.getFont());
		Toolkit			toolkit = Toolkit.getDefaultToolkit();
		FontMetrics		fm = toolkit.getFontMetrics(font);
		int				wCharWidth = fm.charWidth('W');
		float			averageCharWidth = ((float)fm.stringWidth("abcdefghijklmnopqrstuvwxyz")) / 26;
		int sizeX = 0;

		if (nCols < 10)
            // With small size, the text field is sometimes not wide enough...
			sizeX = wCharWidth * (nCols + 1);
        else
            // Note : the 1.25 factor allows to have almost the same width as AWT...
        	sizeX = (int)((averageCharWidth * nCols * 5) / 4);

		return sizeX + 4;
	}

	/**
	 * Returns the height of the text component.
	 */
	@SuppressWarnings("deprecation")
	public int getTextHeight()
	{
		// If a constraint is set on the text, use the nCols and nRows values to compute
		// text size in pixels.
		if ((_text.getParent() instanceof LyFormField)
				&& (((LyFormField)(_text.getParent())).getConstraint() != null))
		{
			LyConstraint	constraint = ((LyFormField)(_text.getParent())).getConstraint();

			if ((constraint != null) && (constraint.getNRows() > 0))
				return constraint.getNRows();
		}

		// For a simple text without constraint, use navigator default height
		if (_text.getNRows() <= 1)
			return -1;

		Font			font = (Font)getEnvironment().getFont(_text.getFont());
		Toolkit			toolkit = Toolkit.getDefaultToolkit();
		FontMetrics		fm = toolkit.getFontMetrics(font);
		int				rows = (_text.getNRows() <= 0 ? 1 : _text.getNRows());
		int				height = fm.getHeight() * rows;

		if ((_text.getType() == LyText.Type.RICH) && isEditable())
			height += 18;

		return height + 2;
	}

	/**
	 * Generates the HTML code used to replace current text by a rich text editor.
	 */
	protected void writeRichText(LyWebPageWriter pw)
	{
		int				width = getTextWidth();
		int				height = getTextHeight();
		String			name = getId();
		String			label = _text.getLabel();
		String			editorFile = getEnvironment().getFile("LY_RICH_TEXT_EDITOR_FILE");

		editorFile = getEnvironment().getString().expand(editorFile);
		editorFile = editorFile.replace('\\', '/');

		if (label == null)
			label = "";

		label = LyString.replace(label, "<br />\n", "<br />");
//		label = LyString.replace(label, "\n", "<br />");

		pw.addBeginScript("<script type='text/javascript' src='" + editorFile + "'></script>", false);

		if (!isEditable())
		{
			if (!USE_LABEL_FOR_NON_EDITABLE_TEXT && !pw.isPrintMode())
			{
				pw.append("<div class='readOnlyField' style='overflow:auto;padding:1px;width:" + Math.abs(width) + "px;");

				if (_text.getNRows() > 1)
					pw.append("height:" + height + ";");

				pw.append("'>");
			}

			pw.append("<font class='label'>");
			pw.append(label);
			pw.append("</font>");

			if (!USE_LABEL_FOR_NON_EDITABLE_TEXT && !pw.isPrintMode())
				pw.append("</div>");
		}
		else
		{
			label = LyString.replace(label, "&", "&amp;");

			pw.append("<textarea id=\"" + name + "\" name=\"" + name + "\" style='width:" + Math.abs(width) + ";height:" + (height + 28) + "px'>");
			pw.append(label);
			pw.append("</textarea>");
			pw.append("<input type='hidden' id=\"" + name + "_focus_hidden_input\" name=\"" + name + "_focus\" value=\"false\"/>");

			if (pw instanceof LyStrutsPage)
			{
				// If code is generated in a LyStrutsPage, it means that component is generated by Ajax.
				// In this case, we need to generate the whole page in order to get CKEditor scripts.
				refresh();
			}
			else
			{
				StringBuffer endBuffer = new StringBuffer();

				endBuffer.append("<script type='text/javascript'>\n");
				endBuffer.append("CKEDITOR.replace( '" + name + "',\n");
				endBuffer.append("{\n");
				endBuffer.append(getRichTextConfig(Math.abs(width), height));
				endBuffer.append("});\n");

				if (needCheck())
				{
					endBuffer.append("CKEDITOR.instances['" + name + "'].on( 'blur', function() { FieldChange(window.document.forms[0],'" + name + "','" +
							_text.getRootController() + "'); });\n");
					endBuffer.append("CKEDITOR.instances['" + name + "'].on( 'dataReady', function() { ").
							append("CheckCKEditorFocus(window.document.forms[0],'" + name + "'); });\n");
				}

				endBuffer.append("</script>\n");
				pw.addEndScript(endBuffer.toString(), false);
			}
		}
	}

	/**
	 * Return the configuration of the rich text editor.
	 */
	protected String getRichTextConfig(int width, int height)
	{
		StringBuffer buffer = new StringBuffer();

		buffer.append("toolbar :\n");
		buffer.append("[\n");
		buffer.append("	['Font','FontSize'],\n");
		buffer.append("	['Bold','Italic','Underline','Strike'],\n");
		buffer.append("	['NumberedList','BulletedList'],\n");
		buffer.append("	['TextColor','BGColor'],\n");
		buffer.append("	['Leonuploadimage']\n");
//		buffer.append("	['Subscript','Superscript'],\n");
//		buffer.append("	['Outdent','Indent','Blockquote','CreateDiv'],\n");
//		buffer.append("	['JustifyLeft','JustifyCenter','JustifyRight','JustifyBlock'],\n");
//		buffer.append("	'/',\n");
//		buffer.append("	['Maximize']\n");
		buffer.append("],\n");
		buffer.append("extraPlugins : 'leonuploadimage',\n");
		buffer.append("language : '" + getEnvironment().getLocale().getLanguage() + "',\n");
		buffer.append("uiColor : '" + getHexColor("LY_DEFAULT_BACKGROUND") + "',\n");
		buffer.append("removePlugins : 'elementspath',\n");
		buffer.append("enterMode : CKEDITOR.ENTER_BR,\n");
		buffer.append("shiftEnterMode  : CKEDITOR.ENTER_BR,\n");
		buffer.append("width : " + width + ",\n");
		buffer.append("height : " + height + ",\n");
		buffer.append("resize_enabled  : false,\n");
		buffer.append("scayt_autoStartup  : false,\n");
		buffer.append("toolbarCanCollapse  : true,\n");
		buffer.append("htmlEncodeOutput  : false,\n");
		buffer.append("toolbarStartupExpanded  : false\n");

		return buffer.toString();
	}
} // end class LyStrutsText
