/*
 * LyStrutsTip.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the Tip object (as described in the dtd) for web viewer.
 *
 * <p><b>TIP: Tooltip</b><br>
 * A tooltip allows associating a help text to a field. The text is displayed under the form of a
 * screen tip associated to the graphical component of the field.</p>
 *
 * @version $Id: LyStrutsTip.java 23300 2009-12-17 13:56:41Z jlsaouli $
 * @author  Lyria
 * @see     leon.view.LyTip
 */
public class LyStrutsTip extends LyWebObject
	implements LyGraphicalTip
{
	public static final String __VERSION =
		"$Id: LyStrutsTip.java 23300 2009-12-17 13:56:41Z jlsaouli $";

	/** The tip generic component. */
	protected LyTip			   _tip;

	/**
	 * Class constructor of a Tip object for the struts target.
	 *
	 * @param tip the generic Tip defined for every type of view targets
	 */
	public LyStrutsTip(LyTip tip)
	{
		super(tip);
		_tip = tip;
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	    pw.startComponent(this);
	    
		String label = _tip.getLabel();

		// translate
		label = _tip.getEnvironment().translate(label);
		label = getHtmlEntitiesString(label);

		String	image = getImageUrl("LY_TIP_ICON");

		pw.append("<img src='" + image + "' title='" + label + "'/>");
		
		pw.endComponent(this);
	}
} // end class LyStrutsTip
