/*
 * LyStrutsTitleBar.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts;

import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.web.*;


/**
 * This class implements the TitleBar object (as described in the dtd) for web viewer.
 *
 * <p><b>TITLEBAR: Title Bar</b><br>
 * A title bar allows displaying a colored horizontal bar containing images and texts. The text
 * comprises three independent parts and each one of them is optional: a left, a central and a right
 * part. The color of the title bar is defined in the resources of the application.</p>
 *
 * @version $Id: LyStrutsTitleBar.java 35064 2012-03-27 12:42:57Z jlsaouli $
 * @author  Lyria S.A.
 */
public class LyStrutsTitleBar extends LyWebObject
    implements LyGraphicalTitleBar
{
	public static final String __VERSION =
		"$Id: LyStrutsTitleBar.java 35064 2012-03-27 12:42:57Z jlsaouli $";

	/** Constant used to generate CSS class name for titlebar. */
    public static int          DECO_TITLEBAR_PARAMETER = 0;

    /** Constant used to generate CSS class name for label. */
    public static int          DECO_LABEL_PARAMETER = 1;
    

    /** The title bar generic component. */
    protected LyTitleBar       _titleBar;

    /** Title bar color. */
    protected String           _barColor;

    /** Title font. */
    protected String           _titleFont;

    /** Title bar image String representation. */
    protected String           _image;

    /** Tooltip shown over the title bar image. */
    protected String           _imageToolTip;

    /** Title bar right image. */
    protected String           _rightImage;

    /** Tooltip shown over the title bar right image. */
    protected String           _rightImageToolTip;

    /**
     * HTML attribute String representation of the image size.<br>
     * Format : <i>"width='<code>image_width</code>' height='<code>image_height</code>"</i><br>
     * or <i>"height='<code>image_height</code>"</i> if the width is not specified.
     */
    protected String           _imageSize;

    /**
     * HTML attribute String representation of the title size.<br>
     * Format : <i>"height='<code>title_height</code>"</i>.
     */
    protected String           _titleSize;

    /**
     * HTML attribute String representation of the right image size.<br>
     * The format is the same that {@link LyStrutsTitleBar#_imageSize}.
     */
    protected String           _rightImageSize;
    
	/**
	 * Constructor.
	 *
	 * @param titleBar the title bar component
	 */
	public LyStrutsTitleBar(LyTitleBar titleBar)
	{
        super(titleBar);
        
        _titleBar = titleBar;

        initialize();
        setContent();
	}

	/**
	 * Method overriden to get the application context URL.
	 *
	 * @see LyWebViewManager#getRootURL()
	 */
	protected String getBaseUrl(LyWebPageWriter pw)
	{
		// get the application context url.
		return ((LyWebViewManager)getViewManager()).getRootURL();
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.struts.LyStrutsTitleBar#getCloseUrl(leon.view.web.LyWebPageWriter,
	 *      java.lang.String)
	 */
	protected String getCloseUrl(LyWebPageWriter pw, String idTitleBar)
	{
		// get the url to the event handler
		LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();
		String			    actionUrl = viewManager.getActionURL(LyStrutsConstants.PATH_ACTION_GRAPHICEVENT);

		// build the URL
		StringBuffer	    closeUrl = new StringBuffer();

		closeUrl.append(actionUrl);
		closeUrl.append("?");
		closeUrl.append(LyWebConstants.CONTROLLER);
		closeUrl.append("=");
		closeUrl.append(_titleBar.getRootController());

		closeUrl.append("&#38;");
		closeUrl.append(LyWebConstants.SOURCE);
		closeUrl.append("=");
		closeUrl.append(idTitleBar);

		return closeUrl.toString();
	}

	/**
	 * Indicates whether popup movement is managed by this titlebar.
	 */
	protected boolean managePopupMovement(LyWebPageWriter pw)
	{
		// Popup movement is managed by the home JSP
		return false;
	}

	/**
	 * Writes the HTML description of the icon used to close the tool bar view.
	 *
	 * @param pw the page writer
	 */
	public void writeCloseIcon(LyWebPageWriter pw)
	{
		String	closeIconStr = getResource("LY_WEB_TITLEBAR_CLOSE_ICON");
		
        if ((closeIconStr != null)
        		&& !testResource("LY_WEB_TITLEBAR_CLOSE_ICON"))
            return;
        
		// Close icon is managed by the home JSP in Struts viewer
		LyWebViewManager viewManager = getWebViewManager();

		if (viewManager.usePopupView()
				&& LyWebConstants.POPUP_VIEW.equals(pw.getWindowName()))
			return;

        String      activeName = pw.getActiveName();
        LyArea      area = _titleBar.getParentArea();
        LyMenuBar   menubar = null;
        LyComponent parent = area.getParent();

        while (!(parent instanceof LyFrame) && (parent != null))
            parent = parent.getParent();
        if ((parent != null) && (parent instanceof LyFrame))
            menubar = ((LyFrame)parent).getMenuBar();

        boolean displayUpPage = LyWebObject.displayUpPage(getEnvironment());
        
        if ((LyWebConstants.PILOT_FRAME.equals(activeName)
                && ((_image == null) || displayUpPage))
            || (!LyWebConstants.PILOT_FRAME.equals(activeName) && (menubar == null)))
        {
            pw.append("<td width='1%'>");

            if (!pw.isPrintMode())
            {
                String javascriptToolTip = getJavascriptString("LY_CLOSE_VIEW");
                String idTitleBar;

                if (_titleBar.getId() != null)
                    idTitleBar = _titleBar.getId() + ".leftImage";
                else
                    idTitleBar = "leftImage";

                String url = getCloseUrl(pw, idTitleBar);
                String buttonClose = getImageUrl("LY_CLOSE_WINDOW");
                String buttonCloseOver = getImageUrl("LY_CLOSE_WINDOW_OVER");

                // If the LY_CLOSE_VIEW resource is set to null do not display image
                if ((javascriptToolTip != null) && !"_null".equals(javascriptToolTip))
                {
                    pw.append("<a href=\"javascript:InvokeAction('" + idTitleBar + "','"
                        + LyWebObject.encodeUrl(url) + "','" + LyWebConstants.TARGET_FRAME);
                    pw.append("')");
                    pw.append("\" onmouseover=\'");
                    pw.append("rollover(");
                    pw.append(LyWebConstants.FRAME_CONTROLLER_DEMAND_CLOSE);
                    pw.append(",\"");
                    pw.append(buttonCloseOver);
                    pw.append("\",\"\");");
                    pw.append("window.status=\"");
                    pw.append(javascriptToolTip);
                    pw.append("\";return true\'");
                    pw.append(" onMouseOut=\'rollout(");
                    pw.append(LyWebConstants.FRAME_CONTROLLER_DEMAND_CLOSE);
                    pw.append(",\"");
                    pw.append(buttonClose);
                    pw.append("\",\"\");\'");
                    pw.append(">");
    
                    pw.append("<img src='" + buttonClose + "'");
                    pw.append(" name=\"");
                    pw.append(LyWebConstants.FRAME_CONTROLLER_DEMAND_CLOSE);
                    pw.append("\" ");
                    pw.append(" border='0'/>");
                    pw.append("</a>");
                }
            }

            pw.append("</td>\n");
        }
	}
	
	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.struts.LyStrutsTitleBar#getRightLabelUrl(leon.view.web.LyWebPageWriter)
	 */
	protected String getRightLabelUrl(LyWebPageWriter pw)
	{
		// get the url to the event handler
		LyStrutsViewManager viewManager = (LyStrutsViewManager)getViewManager();
		String			    actionUrl = viewManager.getActionURL(LyStrutsConstants.PATH_ACTION_GRAPHICEVENT);

		// build the URL
		StringBuffer	    labelUrl = new StringBuffer();

		labelUrl.append(actionUrl);
		labelUrl.append("?");
		labelUrl.append(LyWebConstants.CONTROLLER);
		labelUrl.append("=");
		labelUrl.append(_titleBar.getRootController());

		labelUrl.append("&#38;");
		labelUrl.append(LyWebConstants.SOURCE);
		labelUrl.append("=");
		labelUrl.append(_titleBar.getId() + ".rightLabel");

		return labelUrl.toString();
	}
	
	   /**
     * Initialization method.<br>
     * Sets the title bar background color and title font with values defined in resources files.
     * <br>
     * Sets images to <code>null</code>.
     */
    public void initialize()
    {
		_barColor = (_titleBar.isDefaultColor() ? null : getHexColor(_titleBar.getColor()));
		
		String	font = (_titleBar.isDefaultFont() ? null : _titleBar.getFont());
		String	fontColor = (_titleBar.isDefaultFontColor() ? null : _titleBar.getFontColor());
		
		if ((font != null) || (fontColor != null))
			_titleFont = getFontString(font, fontColor);

        _image = null;
        _imageToolTip = null;
        _rightImage = null;
        _rightImageToolTip = null;
    }

    /**
     * Sets the image and right image of the title bar.
     */
    public void setContent()
    {
        _image = computeImage();
        _rightImage = computeRightImage();
    }

    /**
     * Computes the title bar image properties: its size, tooltip, String representation and the
     * position of the title relative to the image.
     *
     * @return the string representation of the image.
     */
    protected String computeImage()
    {
        if (_titleBar.getImage() == null)
            return null;

        _imageToolTip = getHtmlString(_titleBar.getImageToolTip());

        int[] size = getImageSize(_titleBar.getImage());

        if (size != null)
        {
            _titleSize = "height='" + (size[0] + 10) + "'";
            if (size.length == 2)
                _imageSize = "width='" + size[0] + "' height='" + size[1] + "'";
            else
                _imageSize = "height='" + size[0] + "'";

        }

        return getImageUrl(_titleBar.getImage());
    }

    /**
     * Computes the right image properties: its size, tooltip and String representation.
     *
     * @return the string representation of the right image.
     */
    protected String computeRightImage()
    {
        if (_titleBar.getRightImage() == null)
            return null;

        _rightImageToolTip = getHtmlString(_titleBar.getRightImageToolTip());

        int[] size = getImageSize(_titleBar.getRightImage());

        if (size != null)
        {
            if (size.length == 2)
                _rightImageSize = "width='" + size[0] + "' height='" + size[1] + "'";
            else
                _rightImageSize = "height='" + size[0] + "'";
        }

        return getImageUrl(_titleBar.getRightImage());
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
     */
    public void writeHtml(LyWebPageWriter pw)
    {
        pw.startComponent(this);
        
        updateRefreshInfo(pw);

        String controllerId = _titleBar.getRootController();
        String baseUrl = getBaseUrl(pw);

        // Add needed script
        pw.addBeginScript("<script type='text/javascript'>window.defaultStatus='';</script>",
            false);

        pw.addBeginScript("<script type='text/javascript'>\n//<![CDATA[\nvar _rootUrl='" + baseUrl
            + "';\n//]]>\n</script>", false);
        pw.addBeginScript("<script type='text/javascript'>var _controllerId='" + controllerId
            + "';</script>", false);

        pw.addCssScript("br.newPage{page-break-after:always;}", false);

        LyWebDecoration deco = getWebViewManager().getDecoration();
        String          activeName = pw.getActiveName();

		pw.append("<table border='0' width='100%' cellspacing='0' cellpadding='0'");
		
		if (_barColor != null)
			pw.append(" style='background-color=" + _barColor + "'");
		else
			pw.append(" class='" + deco.getClassName(_titleBar, DECO_TITLEBAR_PARAMETER, pw) + "'");

		pw.append("><tr><td>\n");
        pw.append("<table width='100%'");
        pw.append(" border='0' cellpadding='0' cellspacing='0'>\n");
        if (_titleSize == null)
            pw.append(" <tr valign='middle'>\n");
        else
            pw.append(" <tr valign='middle' " + _titleSize + ">\n");

        boolean displayUpPage = LyWebObject.displayUpPage(getEnvironment());
        
        if (LyWebConstants.PILOT_FRAME.equals(activeName)
            && !LyStrutsToolBar.HORIZONTAL_DASHBOARD
            && !LyStrutsToolBar.EXPAND_DASHBOARD)
        {
            String down = getImageUrl("LY_ARROW_DOWN_BIG");
            String downPopup = translate("LY_GO_BOTTOM_PAGE");

            pw.append("<td align='left'>&nbsp;");

            pw.addBeginScript("<a name='_top'></a>", false);

            pw.append("<a href='#_bottom'");
            pw.append(" onmouseover='window.status=\"");
            pw.append(downPopup);
            pw.append("\";return true;'>");
            pw.append("<img src='");
            pw.append(down);
            pw.append("' border='0' title='");
            pw.append(downPopup);
            pw.append("'/>");
            pw.append("</a></td>");
        }
        else
            pw.append(" <td width='1'>&nbsp;</td>\n");

        // Display image only if the titlebar is not in the pilot frame or if titlebar
        // is in the pilot frame and there is no up page.
        if ((LyWebConstants.PILOT_FRAME.equals(activeName) && (_image != null) && !displayUpPage)
                || (!LyWebConstants.PILOT_FRAME.equals(activeName) && (_image != null)))
        {
            pw.append("\t<td width='1%'>\n");

            if (_titleBar.isImageSensitive() && !pw.isPrintMode())
            {
                String javascriptToolTip = getJavascriptString("LY_CLOSE_VIEW");
                String idTitleBar;

                if (_titleBar.getId() != null)
                    idTitleBar = _titleBar.getId() + ".leftImage";
                else
                    idTitleBar = "leftImage";

                String url = getCloseUrl(pw, idTitleBar);

                pw.append("<a href=\"javascript:InvokeAction('" + idTitleBar + "','"
                    + LyWebObject.encodeUrl(url) + "','" + LyWebConstants.TARGET_FRAME);
                pw.append("')\" onmouseover=\'window.status=\"");
                pw.append(javascriptToolTip);
                pw.append("\";return true\'>");
            }

            pw.append(getImageHtml());

            if (_titleBar.isImageSensitive() && !pw.isPrintMode())
                pw.append("</a>");

            pw.append("</td><td width='10'>&nbsp;&nbsp;</td>\n");
        }
        
        pw.append("  <td align='left' nowrap='true'");

        if (managePopupMovement(pw))
            pw.append(" style='cursor:move' onMouseOver='javascript:startMovePopup(event);'");

        pw.append(">\n");
		pw.append("   <font class='" + deco.getClassName(_titleBar, DECO_LABEL_PARAMETER, pw) + "'");

		if (_titleFont != null)
			pw.append(" style='" + _titleFont + "'");
		
		pw.append(">");
		
        // If titlebar is in the pilot frame, do not display label
        if (!LyWebConstants.PILOT_FRAME.equals(activeName))
            pw.append(getLeftLabelHtml());
        
        pw.append("</font>\n");
        pw.append("  </td>\n");

		if (_titleBar.getCenterLabel() != null)
			pw.append("  <td width='10'>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>\n");
		
        pw.append("  <td  align='center' nowrap='true'>\n");
		pw.append("   <font class='" + deco.getClassName(_titleBar, DECO_LABEL_PARAMETER, pw) + "'");

		if (_titleFont != null)
			pw.append(" style='" + _titleFont + "'");
		
		pw.append(">");
        pw.append(getCenterLabelHtml());
        pw.append("</font>\n");
        pw.append("  </td>\n");

		if (_titleBar.getRightLabel() != null)
			pw.append("  <td width='10'>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>\n");

        pw.append("  <td align='right' nowrap='true'>\n");

        if (_titleBar.isRightLabelSensitive() && !(pw.isPrintMode()))
        {
            String url = getRightLabelUrl(pw);
            String javascriptToolTip = getJavascriptString("LY_SHOW_CONTEXT");

            pw.append("<a href=\"javascript:InvokeAction('" + _titleBar.getId() + ".rightLabel','"
                + LyWebObject.encodeUrl(url) + "','" + LyWebConstants.TARGET_FRAME);
            pw.append("')\" onmouseover=\'window.status=\"");
            pw.append(javascriptToolTip);
            pw.append("\";return true\'>");
        }

		pw.append("   <font class='" + deco.getClassName(_titleBar, DECO_LABEL_PARAMETER, pw) + "'");

		if (_titleFont != null)
			pw.append(" style='" + _titleFont + "'");
		
		pw.append(">");
        pw.append(getRightLabelHtml());
        pw.append("</font>\n");

        if (_titleBar.isRightLabelSensitive() && !(pw.isPrintMode()))
            pw.append("</a>");
        pw.append("  </td>\n");

        if (_rightImage != null)
        {
            pw.append("  <td width='1%'>\n");
            pw.append("   <img src='" + _rightImage + "'");

            if (_rightImageSize != null)
                pw.append(" " + _rightImageSize);

            if ((_rightImageToolTip != null) && (_rightImageToolTip.length() > 0))
                pw.append(" title='" + _rightImageToolTip + "'");
            
            pw.append(" border='0'/>");
            pw.append("</td>");
        }

        // Generate HTML code for right toolbar if set
        LyToolBar   toolbar = _titleBar.getToolBar();
        
        if (toolbar != null)
        {
            pw.append(" <td width='1'>&nbsp;</td>\n");
            pw.append(" <td width='1'>");
            
            LyStrutsToolBar      internal = (LyStrutsToolBar)toolbar.getInternalView();
            
            internal.writeHtml(pw);
            
            pw.append(" </td>");
        }

        pw.append(" <td width='1'>&nbsp;</td>\n");

        writeCloseIcon(pw);
        
        pw.append("</tr>\n");
        pw.append("</table>\n");

        deco.writePageDecoration(pw, LyWebDecoration.HEADER_BAR_END);

        pw.endComponent(this);
    } // end method writeHtml
    
    /**
     * (non-Javadoc).
     *
     * @see leon.view.graphic.LyGraphicalTitleBar#setRightImage(java.lang.String)
     */
    public void setRightImage(String image)
    {
        computeRightImage();
        refresh();
    }

    /**
     * Gets the HTML tag corresponding to the left label.
     *
     * @return the HTML tag corresponding to the left label
     */
    public String getLeftLabelHtml()
    {
        StringBuffer buffer = new StringBuffer();

        buffer.append("<div id=\"");
        buffer.append(_titleBar.getIdCode());
        buffer.append("_leftLabel\">");
        String leftLabel = getHtmlString(_titleBar.getLeftLabel());

        if (leftLabel != null)
            leftLabel = LyString.replace(leftLabel, "\\n", " ");

        buffer.append(leftLabel);
        buffer.append("</div>");

        return buffer.toString();
    }

    /**
     * Gets the HTML tag corresponding to the center label.
     *
     * @return the HTML tag corresponding to the center label
     */
    public String getCenterLabelHtml()
    {
        StringBuffer buffer = new StringBuffer();

        buffer.append("<div id=\"").append(_titleBar.getIdCode()).append("_centerLabel\">");
        String centerLabel = getHtmlString(_titleBar.getCenterLabel());

        buffer.append(centerLabel);
        buffer.append("</div>");

        return buffer.toString();
    }

    /**
     * Gets the HTML tag corresponding to the right label.
     *
     * @return the HTML tag corresponding to the right label
     */
    public String getRightLabelHtml()
    {
        StringBuffer buffer = new StringBuffer();

        buffer.append("<div id=\"").append(_titleBar.getIdCode()).append("_rightLabel\">");
        buffer.append(getHtmlString(_titleBar.getRightLabel()));
        buffer.append("</div>");

        return buffer.toString();
    }

    /**
     * Gets the HTML tag corresponding to the image.
     *
     * @return the HTML tag corresponding to the image
     */
    public String getImageHtml()
    {
        StringBuffer buffer = new StringBuffer();

        buffer.append("<div id=\"");
        buffer.append(_titleBar.getIdCode());
        buffer.append("_image_div\">");

        buffer.append("<img id='" + _titleBar.getIdCode() + "_image' src='" + _image + "'");

        if (_imageSize != null)
            buffer.append(" " + _imageSize);

        if ((_imageToolTip != null) && (_imageToolTip.length() > 0))
            buffer.append(" title='" + _imageToolTip + "'");

        buffer.append(" border='0'/>");
        buffer.append("</div>");

        return buffer.toString();
    }

    /**
     * Transforms the given label into another one that can be used in a refresh script.
     *
     * @param  label the initial label
     * @return the label transformed for a refresh script
     */
    protected String getScriptString(String label)
    {
        String newLabel = label;

        newLabel = LyString.replace(label, "\\", "\\\\");
        newLabel = LyString.replace(newLabel, "\"", "\\\"");
        newLabel = LyString.replace(newLabel, "\\'", "\\\\'");

        return newLabel;
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.graphic.LyGraphicalTitleBar#setLeftLabel(java.lang.String)
     */
    public void setLeftLabel(String leftLabel)
    {
        // If titlebar is in the pilot frame, do not display label
        if (LyWebConstants.PILOT_FRAME.equals(_lastFrameName))
            return;
        
        LyWebViewManager viewManager = getWebViewManager();
        StringBuffer     script = new StringBuffer();
        String           label = getHtmlString(_titleBar.getLeftLabel());

        label = getScriptString(label);

        script.append("<script type='text/javascript'>\n//<![CDATA[\nreplaceDivInnerHtml(\"");
        script.append(_lastFrameName).append("\",\"");
        script.append(_titleBar.getIdCode()).append("_leftLabel\",\"");
        script.append(label);
        script.append("\");\n//]]>\n</script>");

        viewManager.addAdditionalRefreshScripts(script.toString());

        if (LyWebConstants.POPUP_VIEW.equals(_lastWindowName))
            viewManager.addAdditionalRefreshScripts("<script type='text/javascript'>\n//<![CDATA[\n" +
                "checkSizePopupView(null, false);\n//]]>\n</script>");
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.graphic.LyGraphicalTitleBar#setCenterLabel(java.lang.String)
     */
    public void setCenterLabel(String centerLabel)
    {
        LyWebViewManager viewManager = getWebViewManager();
        StringBuffer     script = new StringBuffer();
        String           label = getHtmlString(_titleBar.getCenterLabel());

        label = getScriptString(label);

        script.append("<script type='text/javascript'>\n//<![CDATA[\nreplaceDivInnerHtml(\"");
        script.append(_lastFrameName).append("\",\"");
        script.append(_titleBar.getIdCode()).append("_centerLabel\",\"");
        script.append(label);
        script.append("\");\n//]]>\n</script>");

        viewManager.addAdditionalRefreshScripts(script.toString());
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.graphic.LyGraphicalTitleBar#setRightLabel(java.lang.String)
     */
    public void setRightLabel(String rightLabel)
    {
        LyWebViewManager viewManager = getWebViewManager();
        StringBuffer     script = new StringBuffer();
        String           label = getHtmlString(_titleBar.getRightLabel());

        label = getScriptString(label);

        script.append("<script type='text/javascript'>\n//<![CDATA[\nreplaceDivInnerHtml(\"");
        script.append(_lastFrameName).append("\",\"");
        script.append(_titleBar.getIdCode()).append("_rightLabel\",\"");
        script.append(label);
        script.append("\");\n//]]>\n</script>");

        viewManager.addAdditionalRefreshScripts(script.toString());
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.graphic.LyGraphicalTitleBar#setRightImageToolTip(java.lang.String)
     */
    public void setRightImageToolTip(String toolTip)
    {
        if (_titleBar.getRightImage() != null)
        {
            computeRightImage();
            refresh();

        }
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.graphic.LyGraphicalTitleBar#setImageToolTip(java.lang.String)
     */
    public void setImageToolTip(String toolTip)
    {
        computeImage();

        LyWebViewManager viewManager = getWebViewManager();
        StringBuffer     buffer = new StringBuffer();

        toolTip = translate(toolTip);
        toolTip = LyString.getJavascriptString(toolTip);

        // Update the tooltip of the image
        buffer.append("<script type='text/javascript'>\n//<![CDATA[\nSetImageTooltip('").append(
            _titleBar.getIdCode()).append("_image',\"").append(toolTip).append(
            "\");\n//]]>\n</script>");

        viewManager.addAdditionalRefreshScripts(buffer.toString());
    }

    /**
     * (non-Javadoc).
     *
     * @see leon.view.setinterface.LySetImageInterface#setImage(java.lang.String)
     */
    public void setImage(String image)
    {
        computeImage();

        LyWebViewManager viewManager = getWebViewManager();
        StringBuffer     script = new StringBuffer();
        StringBuffer     buffer = new StringBuffer();

        buffer.append("<img id='" + _titleBar.getIdCode() + "_image' src='" + _image + "'");
        
        if (_imageSize != null)
            buffer.append(" " + _imageSize);
        
        if ((_imageToolTip != null) && (_imageToolTip.length() > 0))
            buffer.append(" title='" + _imageToolTip + "'");
        
        buffer.append(" border='0'/>");
        
        String label = buffer.toString();

        label = getScriptString(label);

        script.append("<script type='text/javascript'>\n//<![CDATA[\nreplaceDivInnerHtml(\"");
        script.append(_lastFrameName).append("\",\"");
        script.append(_titleBar.getIdCode()).append("_image_div\",\"");
        script.append(label);
        script.append("\");\n//]]>\n</script>");

        viewManager.addAdditionalRefreshScripts(script.toString());
    } // end method setImage
} // end class LyStrutsTitleBar
