/*
 * LyStrutsView.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.web.struts;

import java.util.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.setinterface.*;
import leon.view.web.*;


/**
 * This class implements the common behavior of the View object for struts viewer.
 *
 * @version $Id: LyStrutsView.java 22936 2009-12-15 16:08:47Z rmartin $
 * @author  Lyria
 * @see     leon.view.LyView
 */
public abstract class LyStrutsView extends LyWebPageWriter
	implements LyGraphicalView,
		LySetVisibleInterface,
		LySetLabelInterface
{
	public static final String __VERSION =
		"$Id: LyStrutsView.java 22936 2009-12-15 16:08:47Z rmartin $";

	/**
	 * Class constructor of a View object for the struts target.
	 *
	 * @param view the generic View defined for every type of view targets
	 */
	public LyStrutsView(LyView view)
	{
		super(view);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.LyWebComponent#writeHtml(leon.view.web.LyWebPageWriter)
	 */
	public void writeHtml(LyWebPageWriter pw)
	{
	}

	/**
	 * Gets the view.
	 *
	 * @return the view
	 * @see    leon.view.graphic.LyGraphicalObject#getViewComponent()
	 */
	public final LyView getView()
	{
		return (LyView)getViewComponent();
	}

	/**
	 * Gets the view area.
	 *
	 * @return the area
	 * @see    LyView#getArea()
	 */
	public final LyArea getArea()
	{
		return getView().getArea();
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.setinterface.LySetLabelInterface#setLabel(java.lang.String)
	 */
	public void setLabel(String label)
	{
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.LyWebPageWriter#setPrintMode(boolean)
	 */
	public void setPrintMode(boolean printMode)
	{
		super.setPrintMode(printMode);

		LyArea	    area = getArea();
		LyStrutsArea webArea = (LyStrutsArea)(area.getInternalView());

		webArea.setPrintMode(printMode);
	}

	/**
	 * Write HTML description corresponding to the area.
	 *
	 * @param pw the page writer
	 */
	public void writeArea(LyWebPageWriter pw)
	{
		LyArea	    area = getArea();
		LyStrutsArea webArea = (LyStrutsArea)(area.getInternalView());

		webArea.writeHtml(pw);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.LyWebPage#setNoFrames(boolean)
	 */
	public void setNoFrames(boolean noFrames)
	{
		super.setNoFrames(noFrames);

		LyArea	    area = getArea();
		LyStrutsArea webArea = (LyStrutsArea)(area.getInternalView());

		webArea.setNoFrames(noFrames);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.LyWebPageWriter#getPageWriters()
	 */
	public Vector<LyWebPageWriter> getPageWriters()
	{
		LyArea				    area = getArea();
		LyStrutsArea			    webArea = (LyStrutsArea)(area.getInternalView());
		Vector<LyWebPageWriter> pageWriters = webArea.getPageWriters();

		pageWriters.addElement(this);
		return pageWriters;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.LyWebPageWriter#setForceReloadState(boolean)
	 */
	public void setForceReloadState(boolean force)
	{
		_forceReloadState = force;

		if (force)
		{
			LyArea	    area = getArea();
			LyStrutsArea webArea = (LyStrutsArea)(area.getInternalView());

			webArea.setForceReloadState(true);
		}
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.web.LyWebPage#getBuffer(java.lang.String)
	 */
	public StringBuffer getBuffer(String key)
	{
		StringBuffer result = super.getBuffer(key);

		if ((result == null) && (_viewComponent != null))
		{
			LyArea	    area = getArea();
			LyStrutsArea webArea = (LyStrutsArea)(area.getInternalView());

			result = webArea.getBuffer(key);
		}

		return result;
	}

	/**
	 * Gets the HTML tags to create a popup view.
	 *
	 * @return the value or "" if no popup view is available
	 */
	protected String getPopupView()
	{
		return ""; // DHTML Only
	}
} // end class LyStrutsView
