/*
 * LyStrutsAboutAction.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts.action;

import java.io.*;

import javax.servlet.http.*;

import leon.app.*;
import leon.control.*;
import leon.view.web.struts.*;

import org.apache.struts.action.*;


/**
 * Type : leon.view.web.struts.action.LyStrutsAboutAction<br>
 * This action just handles the about action.
 *
 * @version $Id: LyStrutsAboutAction.java 30054 2011-03-14 15:10:34Z bangiole $
 * @author  Lyria S.A.
 */
public class LyStrutsAboutAction extends LyStrutsLeonardiAction
{
	public static final String __VERSION =
		"$Id: LyStrutsAboutAction.java 30054 2011-03-14 15:10:34Z bangiole $";

	/**
	 * Default constructor without parameters.<br>
	 * This constructor should be kept for Java reflection.
	 */
	public LyStrutsAboutAction()
	{
	}

	/**
	 * This action displays a pop-up with the about message.
	 *
	 * @param  mappings - The ActionMapping used to select this instance
	 * @param  bean     - The optional ActionForm bean for this request (if any)
	 * @param  request  - The HTTP request we are processing
	 * @param  response - The HTTP response we are creating
	 * @return null : an empty action forward while the action has written the response.
	 */
	public ActionForward execute(ActionMapping mappings, ActionForm bean,
		HttpServletRequest request, HttpServletResponse response)
		throws Exception
	{

		// Get the session from the session bean
		LyStrutsSessionBean sessionBean = (LyStrutsSessionBean)bean;
		LySession		    session = sessionBean.getSession();
		LyStrutsViewManager viewManager = sessionBean.getViewManager();
		
		// Check if session is registered, otherwise redirect to home page
		if (!checkSessionRegistered(sessionBean, request, response))
			return null;

		// begin the request
		try
		{
			session.getSoftLock();
			Writer	out = beginRequest(request, response, viewManager);
	
			// set no cache
			setNoCacheHeaders(response, 0);
	
			// Set current session
			sessionBean.beginRequest();
	
			// get the first controller
			LyController controller = session.getController(0);
	
			// The session will handle this
			session.processAboutAction(controller);
	
			generateAdditionalScripts(session, request, response, out);
	
			// Unset current session
			sessionBean.endRequest();
	
			// end the request
			endRequest(request, response, viewManager, out);
		}
		finally
		{
			session.releaseSoftLock();
		}
		// the response has been written
		return null;
	} // end method execute

} // end class LyStrutsAboutAction
