/*
 * LyStrutsAnswerAction.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts.action;

import java.io.*;

import javax.servlet.http.*;

import leon.app.*;
import leon.misc.*;
import leon.view.web.*;
import leon.view.web.struts.*;

import org.apache.struts.action.*;


/**
 * Type : leon.view.web.struts.action.LyStrutsAnswerAction<br>
 * The purpose of this action is just to handle the messages answers from the User Interface.
 *
 * @version $Id: LyStrutsAnswerAction.java 31956 2011-08-12 11:53:18Z nhamon $
 * @author  Lyria S.A.
 */
public class LyStrutsAnswerAction extends LyStrutsLeonardiAction
{
	public static final String __VERSION =
		"$Id: LyStrutsAnswerAction.java 31956 2011-08-12 11:53:18Z nhamon $";

	/**
	 * Default constructor without parameters.<br>
	 * This constructor should be kept for Java reflection.
	 */
	public LyStrutsAnswerAction()
	{
	}

	/**
	 * This mehtod handles the answer from a user to a question.
	 *
	 * @param  mappings - The ActionMapping used to select this instance
	 * @param  bean     - The optional ActionForm bean for this request (if any)
	 * @param  request  - The HTTP request we are processing
	 * @param  response - The HTTP response we are creating
	 * @return null : an empty action forward while the action has written the response.
	 */
	public ActionForward execute(ActionMapping mappings, ActionForm bean,
		HttpServletRequest request, HttpServletResponse response)
		throws Exception
	{
		// Get the session from the session bean
		LyStrutsSessionBean sessionBean = (LyStrutsSessionBean)bean;
		LySession		    session = sessionBean.getSession();
		LySession			oldSession = null;
		LyEnvironment	    environment = sessionBean.getEnvironment();
		LyStrutsViewManager viewManager = sessionBean.getViewManager();
		
		// Check if session is registered, otherwise redirect to home page
		if (!checkSessionRegistered(sessionBean, request, response))
			return null;


		try
		{
			session.getSoftLock();

			// get some usefull request parameters
			String message = getParameter(request, LyWebConstants.MESSAGE);
			String answer = getParameter(request, LyWebConstants.ANSWER);
	
			// at this point, the message should be not null
			if (message == null)
				return null;
	
			// Set current session
			sessionBean.beginRequest();
	
			// if we are in login mode, session is not valid yet
			boolean login = !session.isValid();
	
			// begin the request
			Writer	pw = beginRequest(request, response, viewManager);
	
			// Register exit info if answer leads to the exit
			addRequestAttributes(request, response, viewManager);
			
			ActionForward closeForward = getForwardToAction(LyStrutsConstants.PATH_ACTION_CLOSE, viewManager,
					mappings.getModuleConfig());
	
			// Answer the interface by calling the answer on the view manager
			viewManager.answerMessage(message, LyTools.shortFromString(answer));
	
			// Session, environment and view manager may have changed when answering message (reconnection system)
			if (sessionBean.getSession() != session)
			{
				oldSession = session;
				session = sessionBean.getSession(); 
			}
			
			environment = sessionBean.getEnvironment();
			viewManager = sessionBean.getViewManager();
	
			// If session is not valid or session has just become valid after login,
			// forward to home page.
			if ((session != null)
					&& (!session.isValid()
							|| (session.isValid() && login)))
			{
				if (!sessionBean.sessionIsClosed(session))
				{
					if (session.isValid())
					{
						// write url page to home action
						String url = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_HOME,
								null, true);
	
						writeUrlPage(url, environment, pw);
	
						// Unset current session
						sessionBean.endRequest();
	
						// end the request
						endRequest(request, response, viewManager, pw);
	
						return null;
					}
				}
				else if (!session.isValid())
				{
					// forward to close action
					return closeForward;
				}
			}
			else if (!login && sessionBean.sessionIsClosed(session))
			{
				// forward to close action
				return closeForward;
			}
		
			generateAdditionalScripts(session, request, response, pw);
	
			// Unset current session
			sessionBean.endRequest();
	
			// end the request
			endRequest(request, response, viewManager, pw);
		}
		finally
		{
			if (oldSession != null)
				oldSession.releaseSoftLock();
			else
				session.releaseSoftLock();
		}

		return null;
	} // end method execute
} // end class LyStrutsAnswerAction

