/*
 * LyStrutsCloseAction.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts.action;

import org.apache.struts.action.*;
import java.io.*;
import javax.servlet.http.*;
import leon.misc.*;
import leon.view.web.*;


/**
 * Type : leon.view.web.struts.action.LyStrutsCloseAction<br>
 * This is the Struts action for closing a Leonardi user session.
 *
 * @version $Id: LyStrutsCloseAction.java 31634 2011-07-08 08:33:05Z nhamon $
 * @author  Lyria S.A.
 */
public class LyStrutsCloseAction extends LyStrutsLeonardiAction
{
	public static final String __VERSION =
		"$Id: LyStrutsCloseAction.java 31634 2011-07-08 08:33:05Z nhamon $";

	/**
	 * Default constructor without parameters.<br>
	 * This constructor should be kept for Java reflection.
	 */
	public LyStrutsCloseAction()
	{
	}

	/**
	 * This is the action for closing a Leoanrdi application.<br>
	 * It performs the following tasks :<br>
	 *
	 * <ul>
	 *   <li>Strore some usefull attributes in the HttpRequest</li>
	 *   <li>Writes the exit page</li>
	 *   <li>Invalidate the HTTP session</li>
	 * </ul>
	 *
	 * @param  mappings - The ActionMapping used to select this instance
	 * @param  bean     - The optional ActionForm bean for this request (if any)
	 * @param  request  - The HTTP request we are processing
	 * @param  response - The HTTP response we are creating
	 * @return null : an empty action forward while the action has written the response.
	 */
	public ActionForward execute(ActionMapping mappings, ActionForm bean,
		HttpServletRequest request, HttpServletResponse response)
		throws Exception
	{
		// Get the HTTP session
		LyStrutsSessionBean sessionBean = (LyStrutsSessionBean)bean;
		LyEnvironment	    environment = sessionBean.getEnvironment();

		// Get the exit link
		String	exitLink = getExitPage(sessionBean, request);
		String	params = getExitParameters(sessionBean, request);
		
		if ((params != null) && (params.length() > 0))
			exitLink += "?" + params;

		// get the usual printwriter and write the exit page
		PrintWriter pw = response.getWriter();

		pw.write("<html>\n<head></head>\n<body>\n");

		// add javascripts
		writeJavaScriptFile(environment, pw, LyWebConstants.JS_STRING);
		writeJavaScriptFile(environment, pw, LyWebConstants.JS_UTILS);

		// write message if needed
		if ((environment != null) && (environment.getViewManager() != null))
		{
			LyWebViewManager viewManager = (LyWebViewManager)environment.getViewManager();
			String			 message = viewManager.getMessage();

			if (message != null)
			{
				writeMessage(message, environment, pw);
				viewManager.setMessage(null);
			}
		}

		// Check if slave-sessions exist and close their associated windows.
		String	slaveSessionsString = (String)request.getAttribute("slaveSessions");
		
		if (slaveSessionsString != null)
		{
			String[]	sessionCodes = LyString.stringToList(slaveSessionsString, '|');
			int			i, n = sessionCodes.length;
			
			for (i = 0; i < n; i++)
			{
				writeJavaScriptCode(pw, "getLeonardiTopFrame().CloseWindow('" + LyWebConstants.TARGET_PARAM + sessionCodes[i] + "');");
			}
		}
		
		// Add javascript to load the page
		writeJavaScriptCode(pw, "loadUrlPage('" + exitLink + "');");

		// Close page
		pw.write("\n</body>\n</html>\n");

		return null;
	} // end method execute
	
	/**
	 * Computes the url of the exit page.
	 * 
	 * @param sessionBean The current session bean
	 * @param request	  The HTTP request we are processing
	 * @return Default implementation returns the value for LY_EXIT_PAGE resource.
	 */
	protected String getExitPage(LyStrutsSessionBean sessionBean, HttpServletRequest request)
	{
	   HttpSession		httpSession = request.getSession(false);
	   LyEnvironment	environment = sessionBean.getEnvironment();
	   String			exitLink = null;
	   
	   if (httpSession != null)
	      exitLink = (String)httpSession.getAttribute("LY_EXIT_PAGE");
	   else if (environment != null)
	      exitLink = environment.getUrl("LY_EXIT_PAGE");
	   
	   return exitLink;
	}

	/**
	 * Computes the parameters of the exit page.
	 * 
	 * @param sessionBean The current session bean
	 * @param request	  The HTTP request we are processing
	 * @return Default implementation returns the language, cssPath and logo parameters.
	 */
	protected String getExitParameters(LyStrutsSessionBean sessionBean, HttpServletRequest request)
	{
	   HttpSession	httpSession = request.getSession(false);
	   String		language = (String)request.getAttribute("language");
	   String		cssPath = (String)request.getAttribute("cssPath");
	   String		logo = (String)request.getAttribute("logo");
	   String		target = (String)request.getParameter(LyWebConstants.TARGET_PARAM);
	   
	   if (httpSession != null)
	   {
	      if (language == null)
	         language = (String)httpSession.getAttribute("language");

	      if (cssPath == null)
	         cssPath = (String)httpSession.getAttribute("cssPath");
	 
	      if (logo == null)
	         logo = (String)httpSession.getAttribute("logo");
	   } 

	   String parameters = LyWebConstants.LANGUAGE + "=" + language;
	   
	   parameters += "&" + LyWebConstants.RA_LY_SKIN_URL + "=" + cssPath;
	   parameters += "&logo=" + logo;

	   if ((target != null) && !"null".equals(target))
		   parameters += "&" + LyWebConstants.TARGET_PARAM + "=" + target;
	 
	   return parameters;
	}
} // end class LyStrutsCloseAction
