/*
 * LyStrutsClosePopupAction.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts.action;

import java.io.*;

import org.apache.struts.action.*;
import javax.servlet.http.*;
import leon.app.*;
import leon.control.*;
import leon.view.event.*;
import leon.view.web.*;
import leon.view.web.struts.*;


/**
 * Type : leon.view.web.struts.action.LyStrutsCloseAction<br>
 * This is the Struts action for closing the popup view.
 *
 * @version $Id: LyStrutsClosePopupAction.java 30054 2011-03-14 15:10:34Z bangiole $
 * @author  Lyria S.A.
 */
public class LyStrutsClosePopupAction extends LyStrutsLeonardiAction
{
	public static final String __VERSION =
		"$Id: LyStrutsClosePopupAction.java 30054 2011-03-14 15:10:34Z bangiole $";

	/**
	 * Default constructor without parameters.<br>
	 * This constructor should be kept for Java reflection.
	 */
	public LyStrutsClosePopupAction()
	{
	}

	/**
	 * This is the action for closing the popup view.
	 *
	 * @param  mappings - The ActionMapping used to select this instance
	 * @param  bean     - The optional ActionForm bean for this request (if any)
	 * @param  request  - The HTTP request we are processing
	 * @param  response - The HTTP response we are creating
	 * @return null : an empty action forward while the action has written the response.
	 */
	public ActionForward execute(ActionMapping mappings, ActionForm bean,
		HttpServletRequest request, HttpServletResponse response)
		throws Exception
	{
		// Get the session from the session bean
		LyStrutsSessionBean sessionBean = (LyStrutsSessionBean)bean;
		LySession		    session = sessionBean.getSession();
		LyStrutsViewManager viewManager = sessionBean.getViewManager();

		try
		{
			session.getSoftLock();
			// Set the request attritbutes
			setRequestAttributes(request, session);
	
			LyController	controller = viewManager.getLastPopupViewController();
			
			// get the last popup controller
			if (controller != null)
			{
				// build the action demand
				LyActionDemand demand = new LyActionDemand(LyActionDemand.TYPE_ACTION, LyWebConstants.FRAME_CONTROLLER_DEMAND_CLOSE);
	
				viewManager.processDemand(controller.getId(), demand);
			}
			
			// begin the request
			Writer	out = beginRequest(request, response, viewManager);
	
			generateAdditionalScripts(session, request, response, out);
	
			// end the request
			endRequest(request, response, viewManager, out);
		}
		finally
		{
			session.releaseSoftLock();
		}
		return null;
	} // end method execute

} // end class LyStrutsCloseAction
