
/*
 * LyStrutsCommandAction.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts.action;

import java.io.*;
import java.util.*;

import javax.servlet.http.*;

import leon.app.*;
import leon.control.*;
import leon.misc.*;
import leon.view.event.*;
import leon.view.web.*;
import leon.view.web.struts.*;

import org.apache.struts.action.*;


/**
 * Type : leon.view.web.struts.action.LyStrutsCommandAction<br>
 * This action manages all the requests comming from the command buttons inside Leonardi user
 * interfaces.
 *
 * @version $Id: LyStrutsCommandAction.java 35103 2012-04-02 13:15:12Z nhamon $
 * @author  Lyria S.A.
 */
public class LyStrutsCommandAction extends LyStrutsLeonardiAction
{
	public static final String __VERSION =
		"$Id: LyStrutsCommandAction.java 35103 2012-04-02 13:15:12Z nhamon $";

	/**
	 * Default constructor without parameters.<br>
	 * This constructor should be kept for Java reflection.
	 */
	public LyStrutsCommandAction()
	{
	}

	/**
	 * Generic treatment for the command actions of the Leonardi User Interface.
	 *
	 * @param  mappings - The ActionMapping used to select this instance
	 * @param  bean     - The optional ActionForm bean for this request (if any)
	 * @param  request  - The HTTP request we are processing
	 * @param  response - The HTTP response we are creating
	 * @return null : an empty action forward while the action has written the response.
	 */
	public ActionForward execute(ActionMapping mappings, ActionForm bean,
		HttpServletRequest request, HttpServletResponse response)
		throws Exception
	{
		// Get the session from the session bean
		LyStrutsSessionBean sessionBean = (LyStrutsSessionBean)bean;
		LySession		    session = sessionBean.getSession();
		LySession		    initialSession = session;
		LyWebApplication    application = sessionBean.getApplication();
		LyEnvironment	    environment = sessionBean.getEnvironment();
		LyStrutsViewManager viewManager = sessionBean.getViewManager();
		String				controller = getParameter(request, LyWebConstants.CONTROLLER);
		
		// If command action is received from login page, try to register session and to
		// validate login form.
		if ("LOGIN_CONTROLLER".equals(controller)
			&& !session.isRegistered())
		{
			// No caching for this page.
			setNoCacheHeaders(response, 0);
			// Start the session which invokes root Action
			session.register(true);
			// Call the getDecoration() method to be sure that CSS file is generated
			viewManager.getDecoration();
		}
		// Check if session is registered, otherwise redirect to home page
		else if (!checkSessionRegistered(sessionBean, request, response))
			return null;

		try
		{
			session.getSoftLock();
		
			// Get some request parameters
			String targetWindow = getParameter(request, LyWebConstants.TARGET_WINDOW);
			String frame = getParameter(request, LyWebConstants.FRAME);
			String source = getParameter(request, LyWebConstants.SOURCE);
			String value = getParameter(request, LyWebConstants.VALUE);
			String command = getParameter(request, LyWebConstants.COMMAND);
			String page = getParameter(request, LyWebConstants.PAGE);
			String ajax = getParameter(request, LyWebConstants.AJAX);
			
			boolean	isAjaxMode = ((ajax != null) && ("true".equalsIgnoreCase(ajax)));
	
			if (source != null)
				source = source.intern();
	
			if (command != null)
				command = command.intern();
	
			// store the current window name
			storeCurrentWindow(viewManager, targetWindow, frame, source);
	
			// The default is to reload the view.
			viewManager.setReload(true);
	
			// Set current session
			sessionBean.beginRequest();
			
			// Manage null controller
			if ((controller == null) || (session.getController(controller) == null))
			{
				controller = manageNullController(request, response, application, session, environment, viewManager);
				
				if (controller == null)
				{
					Writer pw = beginRequest(request, response, viewManager);
					
					generateAdditionalScripts(session, request, response, pw);
					endRequest(request, response, viewManager, pw);
					
					sessionBean.endRequest(); 
					return null;
				}
			}
	
			// if we are in login mode, session is not valid yet
			boolean login = !session.isValid();
	
			// keep the last view controller
			LyViewController beginViewController = session.getStackLastController(null);
			LyController	 targetController = session.getController(controller);
			
	        // Register exit info if answer leads to the exit
	        addRequestAttributes(request, response, viewManager);
	
			// Transmit the action demand
			if (source != null)
			{
				// get the demand type
				int demandType = LyActionDemand.TYPE_ACTION;
	
				LyActionDemand demand = new LyActionDemand(demandType, source);
	
				demand.setValue(value);
				demand.setParameterValues(new LyStrutsParameterValues(request, environment.getString()));
	
				viewManager.processDemand(controller, demand);
			}
			
			if (beginViewController != session.getStackLastController(null))
				viewManager.addReloadFrame(LyWebConstants.TARGET_FRAME);
			
			// Retrieve the close forward now because viewManager may has been set to null on the session bean
			ActionForward	closeActionForward = getForwardToAction(LyStrutsConstants.PATH_ACTION_CLOSE, viewManager,
					mappings.getModuleConfig());
	
			// Session may have changed (session swap when user has logged in)
			session = sessionBean.getSession();
			
			if (session != initialSession)
				session.getSoftLock(); // lock the new session
			
			environment = sessionBean.getEnvironment();
			viewManager = sessionBean.getViewManager();
	
			// if session is closing, forward to close action
			if (sessionBean.sessionIsClosed(session))
			{
			    if (viewManager != null)
					viewManager.setReload(false);
	
				// remove the current session
				sessionBean.endRequest(); 
				
				// forward to close action
				return closeActionForward;
			}
		
			// begin the request
			Writer pw = beginRequest(request, response, viewManager);
	
			// If session is not valid or session hast just become valid after login,
			// forward to home page.
			if (!session.isValid()
					|| (session.isValid() && login))
			{
				// No caching for this page.
				setNoCacheHeaders(response, 0);
	
				if (!sessionBean.sessionIsClosed(session))
				{
					boolean	homeRedirect = false;
					
					if (session.isValid())
					{
						String				target = getParameter(request, LyWebConstants.TARGET_PARAM);
						Map<String, String> parameters = new HashMap<String, String>();

						if ((target != null) && !"null".equals(target))
							parameters.put(LyWebConstants.TARGET_PARAM, target);

						// write url page to home action
						String url = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_HOME,
								parameters, true);
	
						writeUrlPage(url, environment, pw);
						homeRedirect = true;
					}
					
					if (login && !session.isValid() && (viewManager.getMessage() == null))
					{
						// If login and session is still unvalid and no message to display : try to reload
						// page since a specific JSP page may need to do some stuff.
						viewManager.clearReloadFrame(LyWebConstants.TARGET_FRAME);
						viewManager.addReloadFrame(LyWebConstants.TOP_FRAME);
					}
	
					// If a redirection to home page has been performed, there is no need to send
					// additional scripts.
					if (!homeRedirect)
						generateAdditionalScripts(session, request, response, pw);
	
					// end request
					endRequest(request, response, viewManager, pw);
					
					// remove the current session
					sessionBean.endRequest(); 
					return null;
				}
			}
	
			if (LyWebConstants.POPUP_VIEW.equals(frame))
			{
				targetWindow = frame;
				frame = LyWebConstants.TARGET_FRAME;
			}
			else if (!login && ((frame == null) || (frame.trim().length() == 0)))
				frame = LyWebConstants.TARGET_FRAME;
	
			// ### the post METHOD always submit the page= parameter.
			// the page parameter may be found in the current URL and not in
			// the submited values. We need to distinguish the post of command
			// (create, set ...) which submit the content of the view (view is
			// closed or shown if error)
			// and the post of an action bar (open a NEW page : ignore the current
			// page info).
			if (((command != null) && (command.length() != 0)))
			{
				if (page != null)
				{
					// ## prepare to write a part of the view : titlebar, list,
					// messagebar ...
					// The page is given and a frame name may be given.
					// Retrieve the page writer by its id
					LyWebPageWriter pageWriter = viewManager.getPageWriter(page);
	
					// Set this page writer as the current one
					if ((viewManager.getPageWriter() == null) && (pageWriter != null))
						viewManager.setPageWriter(pageWriter);
					else if (viewManager.getPageWriter() != null)
						// FFT868 : Another pagewriter may be ready may this is not
						// related to current page and frame.
						// Example : and action both closed a page and opened or
						// made visible another one.
						// We may have a page and a pagewriter ready, but they to
						// not correspond to the request parameters.
						frame = LyWebConstants.TARGET_FRAME;
				}
			}
			
			if (!isAjaxMode)
			{
				pw.write("<html>");
				pw.write(viewManager.generateScriptsInclusion());
			}
			
			String	closeAppli = (String)session.getValue("CLOSE_APPLICATION_ON_VALIDATE");
			
			if ((closeAppli != null) && !closeAppli.equals("true"))
			{
				boolean		close = closeAppli.equals("dashboard") || closeAppli.equals(controller);
				
				close = close && ((targetController == null) || targetController.isFreed());
				
				if (close)
				{
					String url = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_HOME, null, false);
					
					viewManager.addAdditionalRefreshScripts("<script type='text/javascript'>getLeonardiTopFrame().location.replace('" + url + "');</script>");
					session.addValue("CLOSE_APPLICATION_ON_VALIDATE", null);
					session.addValue("REDIRECT_ACTION_URL", "");
					session.addValue("TARGET_ACTION_URL", "");
				}
			}

			generateAdditionalScripts(session, request, response, pw);
	
			if (!isAjaxMode)
			{
				pw.write("</html>");
			}
	
			// We mustn't have pending transactions on the session.
			if (session.hasTransaction())
			{
				Exception e = new IllegalStateException("BF AppEngine : Remaining pending transactions");
	
				session.handleException(e);
			}
	
			// unset current session
			sessionBean.endRequest();
	
			// end request
			endRequest(request, response, viewManager, pw);
			
			if ((closeAppli != null) && closeAppli.equals("true"))
				request.getSession().invalidate();
		}
		finally
		{
			if ((initialSession != session) && (initialSession != null))// session swap
				initialSession.releaseSoftLock();

			if(session != null)
				session.releaseSoftLock();
		}
		
		return null;
	} // end method execute

	/**
	 * Manages the case of a controller that has not been found in the current HTTP request.<br>
	 * If session is not valid, try to get the root login controller. After, if the last view
	 * controller is not freed, get it. Then try to close a popup view.
	 *
	 * @param  request     Current HTTP request.
	 * @param  response    Current HTTP response.
	 * @param  application The Leonardi application.
	 * @param  session     The Leonardi user session.
	 * @param  environment The current environment.
	 * @param  viewManager The current view manager.
	 * @return The identifier of a controller.
	 * @throws Exception if soemthing is wrong during management.
	 */
	protected String manageNullController(HttpServletRequest request, HttpServletResponse response,
		LyApplication application, LySession session, LyEnvironment environment,
		LyStrutsViewManager viewManager)
		throws Exception
	{
		// get some request parameters
		String frame = getParameter(request, LyWebConstants.FRAME);
		String controller = null;

		// This happens when validating a login form
		// Check if valid : bug on Netscape when reusing a login page after exit
		// or timeout or server restart
		if ((frame == null) || (!session.isValid()))
		{
			// If not found, redirect on root controller
			LyController rootController = session.getController(0);

			if (rootController != null)
			{
				controller = rootController.getId();

				// May cause a few problems
				if (rootController.getViewController() != null)
					rootController.getViewController().showView();
			}
		}
		// This happens when more than one submit are sent.
		// Display the last view controller or root page if there
		// is no last view controller.
		else
		{
			LyViewController lvc = session.getLastViewController();

			if ((lvc != null) && (!lvc.isFreed()))
				// Do not send any action demand to the controller, just show it.
				lvc.showView();
		}

		return controller;
	} // end method manageNullController

} // end class LyStrutsCommandAction

