/*
 * LyStrutsRefreshAction.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts.action;

import java.io.*;
import java.net.*;

import javax.servlet.http.*;

import leon.app.*;
import leon.view.web.*;

import org.apache.struts.action.*;

/**
 * Type : leon.view.web.struts.action.LyStrutsPageAction<br>
 * This action generates push mechanism for graphical updates.<br>
 *
 * @version $Id: LyStrutsPushAction.java 17533 2009-08-17 15:22:53Z nha $
 * @author  Lyria S.A.
 */
public class LyStrutsDownloadAction extends LyStrutsLeonardiAction
{
	public static final String __VERSION =
		"$Id: LyStrutsPushAction.java 17533 2009-08-17 15:22:53Z nha $";

	/**
	 * Default constructor without parameters.<br>
	 * This constructor should be kept for Java reflection.
	 */
	public LyStrutsDownloadAction()
	{
	}

    /**
     * Get mimeType of file. Special case for csv.
     * @param filepath Path of file.
     * @return mimetype
     */
    private String getMimeTypeFromFile(String filepath)
    {
        if (filepath == null)
            return null;
        if (filepath.toLowerCase().endsWith(".csv"))
        {
            return ("text/csv");
        }
        else
        {
            FileNameMap fileNameMap = URLConnection.getFileNameMap();
            return fileNameMap.getContentTypeFor(filepath);
        }
    }

	/**
	 * Generic treatment for the push mechanism of the Leoanrdi User Interface.
	 *
	 * @param  mappings - The ActionMapping used to select this instance
	 * @param  bean     - The optional ActionForm bean for this request (if any)
	 * @param  request  - The HTTP request we are processing
	 * @param  response - The HTTP response we are creating
	 * @return null : an empty action forward while the action has written the response.
	 */
	public ActionForward execute(ActionMapping mappings, ActionForm bean,
		HttpServletRequest request, HttpServletResponse response)
		throws Exception
	{
		// Get the session from the session bean
		LyStrutsSessionBean sessionBean = (LyStrutsSessionBean)bean;
		LySession		    mainSession = sessionBean.getSession();

		// Check if session is registered, otherwise redirect to home page
		if (!checkSessionRegistered(sessionBean, request, response))
			return null;
		try
		{
			mainSession.getSoftLock();
			String					fileId = getParameter(request, LyWebConstants.FILE);
			String[]				files = mainSession.getTempFile(fileId);

			if (files == null)
				return null;

			String					fileDesc = files[1];
			String					filePath = files[0];
			File					file = new File(filePath);

			if (!file.exists())
				return null;

			FileInputStream			fis = new FileInputStream(file);
			OutputStream			ros = response.getOutputStream();
			BufferedOutputStream	bos = new BufferedOutputStream(ros);
			byte[]					buf = new byte[1024];
			int						len;

			response.reset();
			fileDesc = fileDesc.replace(" ", "_");

			String mimeType = getMimeTypeFromFile(filePath);

            if (mimeType != null && !"".equals(mimeType))
            {
                response.setContentType(mimeType);
            }


			response.setHeader("content-Length", Long.toString(file.length()));
			response.setHeader("content-disposition", "attachment; filename="+fileDesc);
			while ((len = fis.read(buf, 0, buf.length)) != -1)
				{
					bos.write(buf, 0, len);
			}

			bos.flush();
			bos.close();

			ros.flush();
			ros.close();

			fis.close();
		}
		finally
		{
			mainSession.releaseSoftLock();
		}

		return null;
	} // end method execute
} // end class LyStrutsPageAction
