/*
 * LyStrutsEventAction.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts.action;

import org.apache.struts.action.*;

import java.io.*;
import java.util.*;

import javax.servlet.http.*;

import leon.app.*;
import leon.control.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;
import leon.view.event.*;
import leon.view.web.*;
import leon.view.web.struts.*;


/**
 * Type : leon.view.web.struts.action.LyStrutsEventAction<br>
 * This class is a common template for the Leonardi struts actions that manage events comming from
 * the User Interface.
 *
 * @version $Id: LyStrutsEventAction.java 34234 2012-02-21 09:20:20Z nhamon $
 * @author  Lyria S.A.
 */
public abstract class LyStrutsEventAction extends LyStrutsLeonardiAction
{

	public static final String __VERSION =
		"$Id: LyStrutsEventAction.java 34234 2012-02-21 09:20:20Z nhamon $";

	/**
	 * This common implementation of the Struts Action Model for managing events comming from the
	 * user interface.
	 *
	 * @param  mappings - The ActionMapping used to select this instance
	 * @param  bean     - The optional ActionForm bean for this request (if any)
	 * @param  request  - The HTTP request we are processing
	 * @param  response - The HTTP response we are creating
	 * @return null : no response.
	 */
	public ActionForward execute(ActionMapping mappings, ActionForm bean,
		HttpServletRequest request, HttpServletResponse response)
		throws Exception
	{
		// Get the session from the session bean
		LyStrutsSessionBean sessionBean = (LyStrutsSessionBean)bean;
		LySession		    session = sessionBean.getSession();
		LyEnvironment	    environment = sessionBean.getEnvironment();
		LyStrutsViewManager viewManager = sessionBean.getViewManager();
		
		// Check if session is registered, otherwise redirect to home page
		if (!checkSessionRegistered(sessionBean, request, response))
			return null;

		try
		{
			session.getSoftLock();
			// get some usefull request parameters
			String		 controllerId = getParameter(request, LyWebConstants.CONTROLLER);
			String		 source = getParameter(request, LyWebConstants.SOURCE);
			String		 frame = getParameter(request, LyWebConstants.FRAME);
			String		 targetWindow = getParameter(request, LyWebConstants.TARGET_WINDOW);
	
			// If controller is not found in the parameters, try to retrieve it from source
			// that may be the identifier of the component that sent the request.
			if ((controllerId == null) || (controllerId.length() == 0))
			{
				String	stackName = frame;
				
				if (LyWebConstants.TARGET_FRAME.equals(frame))
					stackName = null;
				
				LyViewController	viewController = session.getStackLastController(stackName);
				
				if ((viewController != null) && !viewController.isFreed())
				{
					LyView		view = viewController.getView();
					
					if (view != null)
					{
						LyComponent	component = view.getComponent(source);
						
						if (component != null)
							controllerId = component.getRootController();
					}
				}
			}
			
			// get the controller that receive this event
			LyDemandTransmitter	transmitter = viewManager.getDemandTransmitter();
			LyDemandListener	controller = (controllerId == null) || (transmitter == null)? null
					: transmitter.getListenerById(controllerId);
	
			// at this point, if no controller is defined, then the request is discarded
			if (controller == null)
			{
				String			 page = getParameter(request, LyWebConstants.PAGE);
				String			 value = getParameter(request, LyWebConstants.VALUE);
				LyWebPageWriter	 pageWriter = viewManager.getPageWriter(page);
				
				// Try to set property on the given page writer (for example, splitter component
				// use this to set splitter size from browser, map component use this to zoom, translate...).
				if (pageWriter != null)
					pageWriter.setProperty(source, value);
				else
				{
			        Vector<LySession> slaveSessions = session.getSlaveSessions();
			
			        if (slaveSessions != null)
			        {
						for (LySession slaveSession : slaveSessions)
						{
							LyStrutsViewManager slaveViewManager = (LyStrutsViewManager) slaveSession.getViewManager();
							
							pageWriter = slaveViewManager.getPageWriter(page);
							
							// Set this page writer as the current one
							if (pageWriter != null)
							{
								pageWriter.setProperty(source, value);
								break;
							}
						}
			        }
				}
			}
			
			// set current session
			sessionBean.beginRequest();
	
			// strore the current window name
			storeCurrentWindow(viewManager, targetWindow, frame, source);
	
			// keep the last view controller
			LyViewController beginViewController = session.getStackLastController(null);
	
			// Check if the action is a print action
			String[] parameters = LyString.stringToList(source, LyWebConstants.CONTROLLER_ID_SEP);
	
			if ((parameters != null) && (parameters.length > 0) && (parameters[0] != null))
			{
				String			 id = parameters[0].intern();
				LyWebApplication application = sessionBean.getApplication();
				LyAction		 action = application.getAction(id);
				boolean			 isPrintAction = false;
				
				if (action != null)
				{
					action = action.getRoot();
					
					if ((action != null)
							&& (LyAction.ID_PRINT.equals(action.getId()) 
									|| LyAction.ID_PRINT_TABLE.equals(id)))
						isPrintAction = true;
				}
				else if ("_printChart".equals(id))
					isPrintAction = true;
				else if ("_printMap".equals(id))
					isPrintAction = true;
				else if ("_printList".equals(id))
					isPrintAction = true;
				else if ("_printGantt".equals(id))
					isPrintAction = true;
					
				if (isPrintAction)
				{
					// remove the current session
					sessionBean.endRequest(); 
					
					return getForwardToAction(LyStrutsConstants.PATH_ACTION_PRINT, viewManager,
							mappings.getModuleConfig());
				}
			}
	
			// begin the request
			Writer	pw = beginRequest(request, response, viewManager);
	
			// get the demand from the request
			LyDemand demand = (controller == null ? null : getDemand(request, environment));
	
			// process the demand to the areaController
			if (demand != null)
			{
				if (controller instanceof LyViewController)
				{
					LyAreaController areaController = ((LyViewController)controller)
							.getAreaController();
	
					if (areaController != null)
						controllerId = areaController.getId();
				}
	
				viewManager.processDemand(controllerId, demand);
			}
			
			if (beginViewController != session.getStackLastController(null))
				viewManager.addReloadFrame(LyWebConstants.TARGET_FRAME);
			
			String ajax = getParameter(request, LyWebConstants.AJAX);
	
			if ("true".equals(ajax))
				generateAdditionalScripts(session, request, response, pw);
			else
			{
				// write the HTML page
				pw.write("<html>\n");
	
				// add javascript files
				writeJavaScriptFile(environment, pw, LyWebConstants.JS_UTILS);
				writeJavaScriptFile(environment, pw, LyWebConstants.JS_ACTIONS);
	
				// reload the entire view, because additional scripts can not be used
				writeJavaScriptCode(pw, "getLeonardiTopFrame().location.reload();");
	
				// close the page
				pw.write("</html>\n");
			}
			
			// remove the current session
			sessionBean.endRequest(); 
			
			// end request
			endRequest(request, response, viewManager, pw);
		}
		finally
		{
			session.releaseSoftLock();
		}
		return null;
	} // end method execute

	/**
	 * Method that search selected items inside the current HTTP request.
	 *
	 * @param  request Current HTTP request
	 * @return a Vector that contains the selected items.
	 */
	protected Vector<String> getSelectionIds(HttpServletRequest request)
	{
		// search for selection parameters
		Vector<String>     selectionIds = new Vector<String>();
		String		       parameter, selection, newElement;
		Enumeration<?>     e = request.getParameterNames();

		while (e.hasMoreElements())
		{
			// get parameter
			parameter = (String)e.nextElement();
			if (parameter.indexOf(LyWebConstants.SELECTION_ITEM) == 0)
			{
				// get selection value
				selection = request.getParameterValues(parameter)[0];

				// The selection value may be in the name or in the value
				// parameter
				// the first to try is the value, then the name
				if ((selection != null) && (selection.indexOf(LyWebConstants.SELECTION_ITEM) >= 0))
					newElement = selection.substring(LyWebConstants.SELECTION_ITEM.length());
				else
					newElement = parameter.substring(LyWebConstants.SELECTION_ITEM.length());

				// if the element is already in the selection list, it is
				// already selected
				// so it's an UNSELECT_ITEM demand
				if (newElement.length() != 0)
					if (selectionIds.contains(newElement))
						selectionIds.removeElement(newElement);
					else
						selectionIds.addElement(newElement);
			}
		}

		// return list of selected items
		return selectionIds;
	} // end method getSelectionIds

	/**
	 * This method creates a Logical Leonardi event with a User Interface physical event (the HTTP
	 * request).
	 *
	 * @param  request     Current HTTP request.
	 * @param  environment Current Leonardi environment.
	 * @return a logical event (LySelectionDemand, LyActionDemand, etc...)
	 */
	public abstract LyDemand getDemand(HttpServletRequest request, LyEnvironment environment);

} // end class LyStrutsEventAction
