/*
 * LyStrutsExceptionHandler.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts.action;

import java.io.*;

import javax.servlet.*;
import javax.servlet.http.*;

import leon.app.*;
import leon.misc.*;
import leon.view.web.struts.*;

import org.apache.struts.action.*;
import org.apache.struts.config.*;


/**
 * Type : leon.view.web.struts.LyStrutsExceptionHandler<br>
 * This handler should be configured with the leonardi actions in the <code>struts-config.xml</code>
 * file with the <code>&lt;exception&gt;</code> tag.<br>
 *
 * @version $Id: LyStrutsExceptionHandler.java 30054 2011-03-14 15:10:34Z bangiole $
 * @author  Lyria S.A.
 */
public class LyStrutsExceptionHandler extends ExceptionHandler
{
	public static final String __VERSION =
		"$Id: LyStrutsExceptionHandler.java 30054 2011-03-14 15:10:34Z bangiole $";

	/**
	 * Default constructor without parameters.<br>
	 * This constructor should be kept for Java reflection.
	 */
	public LyStrutsExceptionHandler()
	{
	}

	/**
	 * Handle the exception.<br>
	 * Return the <code>ActionForward</code> instance (if any) returned by the called <code>
	 * ExceptionHandler</code>.
	 *
	 * @param  ex           The exception to handle
	 * @param  ae           The ExceptionConfig corresponding to the exception
	 * @param  mapping      The ActionMapping we are processing
	 * @param  formInstance The ActionForm we are processing
	 * @param  request      The servlet request we are processing
	 * @param  response     The servlet response we are creating
	 * @return the <code>ActionForward</code> instance (if any) returned by the called <code>
	 *         ExceptionHandler</code>
	 * @throws ServletException if a servlet exception occurs
	 */
	public ActionForward execute(Exception ex, ExceptionConfig ae, ActionMapping mapping,
		ActionForm formInstance, HttpServletRequest request, HttpServletResponse response)
		throws ServletException
	{
		// test if this error occured during the process of a Leonardi action
		String  actionType = mapping.getType();
		boolean leonardiAction = ((actionType != null)
					&& actionType.startsWith("leon.view.web.struts.action"));

		// test the session bean
		if (leonardiAction
				&& ((formInstance == null) || !(formInstance instanceof LyStrutsSessionBean)))
		{
			// write a message to tell a misconfiguration error
			try
			{
				response.setContentType("text/html;");
				response.setStatus(HttpServletResponse.SC_OK);
				PrintWriter pw = response.getWriter();

				pw.println("<html><head><title>Leonardi configuration error</title></head>");
				pw.println("<body><h1>Leonardi configuration error :</h1><br>");
				pw.println("<body>The form bean should be of type ");
				pw.println("<code><b>leon.view.web.struts.LyStrutsSessionBean</b></code><br><br>");
				pw.println("Please check the struts config file :<br>");
				pw.println("<ul><li>Check <code>&ltform-bean name=\"leonardiSessionBean\" ");
				pw.println(
					"type=\"leon.view.web.struts.LyStrutsSessionBean\" /&gt</code> is defined</li>");
				pw.println(
					"<li>Check your action maping uses this bean with the <b>name</b> attribute</li>");
				pw.println("<br><br><code>");
				pw.print(ex);
				pw.println("<br>");
				StackTraceElement[] elements = ex.getStackTrace();

				if (elements != null)
				{
					for (int i = 0; i < elements.length; i++)
					{
						pw.print(elements[i]);
						pw.println("<br>");
					}
				}
				pw.println("</code></body></html>");
				return null;
			}
			catch (IOException e)
			{
				return null;
			}
		}

		// get the Session Bean
		LyStrutsSessionBean bean = (LyStrutsSessionBean)formInstance;
		LySession		    session = bean.getSession();
		LyEnvironment	    environment = null;

		if (session != null)
		{
			// The session should handle the exception
			session.handleException(ex);
			environment = session.getEnvironment();
		}
		else
		{
			// get the application from the plugin
			HttpSession			   httpSession = request.getSession();
			ServletContext		   context = httpSession.getServletContext();
			LyStrutsLeonardiPlugin plugin = LyStrutsLeonardiPlugin.getPlugin(context);
			LyApplication		   application = LyApplication.getApplication(plugin.getInitName());

			if (application != null)
				environment = application.getEnvironment();
		}

		// store the exception message in the "leonardi.exception" key
		Object	error = request.getAttribute(ae.getKey());
		
		if (error == null)
			request.setAttribute(ae.getKey(), ex);

		// if the environment is found : search for the error page
		String errorPage = null;

		if (environment != null)
			errorPage = environment.getUrl("LY_ERROR_PAGE");
		else
			errorPage = LyStrutsConstants.DEFAULT_ERROR_PAGE;
		
		// if the error page is found , makes a forward
		if (errorPage != null)
		{
			String contextPath = request.getContextPath();

			if (errorPage.startsWith(contextPath))
				errorPage = errorPage.substring(contextPath.length());

			return new ActionForward(errorPage);
		}
		else
			// makes a forward to the default configured error page
			return super.execute(ex, ae, mapping, formInstance, request, response);
	} // end method execute
} // end class LyStrutsExceptionHandler
