/*
 * LyStrutsExitAction.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts.action;

import leon.app.*;
import leon.control.*;
import leon.misc.*;
import leon.view.*;
import leon.view.event.*;
import leon.view.web.*;
import leon.view.web.struts.*;

import org.apache.struts.action.*;

import java.io.*;

import java.util.*;

import javax.servlet.http.*;


/**
 * Type : leon.view.web.struts.action.LyStrutsExitAction<br>
 * This action handles the exit action : ask a question to the user and manages the answer.
 *
 * @version $Id: LyStrutsExitAction.java 34828 2012-03-19 10:41:21Z jlsaouli $
 * @author  Lyria S.A.
 */
public class LyStrutsExitAction extends LyStrutsLeonardiAction
{
    public static final String __VERSION =
        "$Id: LyStrutsExitAction.java 34828 2012-03-19 10:41:21Z jlsaouli $";

    /**
     * Default constructor without parameters.<br>
     * This constructor should be kept for Java reflection.
     */
    public LyStrutsExitAction()
    {
    }

    /**
     * This action executes the Leonardi exit: displays a window to question the user about the
     * exit.
     *
     * @param  mappings - The ActionMapping used to select this instance
     * @param  bean     - The optional ActionForm bean for this request (if any)
     * @param  request  - The HTTP request we are processing
     * @param  response - The HTTP response we are creating
     * @return null : an empty action forward while the action has written the response.
     */
    public ActionForward execute(ActionMapping mappings, ActionForm bean,
        HttpServletRequest request, HttpServletResponse response) throws Exception
    {
        // Get the session from the session bean
        LyStrutsSessionBean sessionBean = (LyStrutsSessionBean) bean;
        LySession           session = sessionBean.getSession();
        LyStrutsViewManager viewManager = sessionBean.getViewManager();
        String              target = getParameter(request, LyWebConstants.TARGET_PARAM);

        // set no cache
        setNoCacheHeaders(response, 0);

        // Check if session is registered, otherwise redirect to home page
        if (!checkSessionRegistered(sessionBean, request, response))
            return null;

        try
        {
            session.getSoftLock();

            // get the anwser parameter
            String answer = getParameter(request, LyWebConstants.ANSWER);
            short  answerValue = LyTools.shortFromString(answer);
            Writer pw = null;

            // Set current session
            sessionBean.beginRequest();

            if ((answer != null)
                    && ((answerValue == LyViewManager.COMMAND_OK)
                        || (answerValue == LyViewManager.COMMAND_YES)))
            {
                // Register exit info if answer leads to the exit
                addRequestAttributes(request, response, viewManager);

                // Check if slave-sessions exist and close their associated windows.
                if (session.getSlaveSessions() != null)
                {
                    Vector<LySession> slaveSessions = session.getSlaveSessions();
                    LySession         slaveSession;
                    String            slaveSessionsString = "";
                    int               i, n = slaveSessions.size();

                    for (i = 0; i < n; i++)
                    {
                        slaveSession = slaveSessions.elementAt(i);

                        if (i > 0)
                            slaveSessionsString += "|";

                        slaveSessionsString += slaveSession.getId();
                    }

                    slaveSessionsString = LyString.replace(slaveSessionsString, ".", "");
                    request.setAttribute("slaveSessions", slaveSessionsString);
                }

                // Answer the interface by calling the answer on the view manager
                String message = getParameter(request, LyWebConstants.MESSAGE);

                viewManager.answerMessage(message, LyViewManager.COMMAND_YES);

                // Unset current session
                sessionBean.endRequest();

                // forward to close action
                return getForwardToAction(LyStrutsConstants.PATH_ACTION_CLOSE, viewManager,
                        mappings.getModuleConfig());
            }
            else if ("forceExit".equals(answer))
            {
                // Register exit info if answer leads to the exit
                addRequestAttributes(request, response, viewManager);

                session.confirmExit();

                // Unset current session
                sessionBean.endRequest();

                // forward to close action
                return getForwardToAction(LyStrutsConstants.PATH_ACTION_CLOSE, viewManager,
                        mappings.getModuleConfig());
            }
            else if (answer != null)
            {
                // begin the request
                pw = beginRequest(request, response, viewManager);

                // Answer the interface by calling the answer on the view manager
                String message = getParameter(request, LyWebConstants.MESSAGE);

                viewManager.answerMessage(message, LyViewManager.COMMAND_NO);

                generateAdditionalScripts(session, request, response, pw);
            }
            else
            {
                // begin the request
                pw = beginRequest(request, response, viewManager);

                // start the exit action
                LyViewController lvc = session.getLastViewController();
                LyController     parentController = lvc;
                String           mainViewControllerId = null;

                while (parentController != null)
                {

                    if (parentController instanceof LyViewController)
                        mainViewControllerId = parentController.getId();

                    parentController = parentController.getParent();
                }

                if (mainViewControllerId != null)
                {
                    LyActionDemand demand = new LyActionDemand(LyActionDemand.TYPE_WINDOW_CLOSED,
                            mainViewControllerId);

                    viewManager.processDemand(mainViewControllerId, demand);
                }
                else
                    session.exit();

                // set the answer URL to this action
                Map<String, String> parameters = new HashMap<String, String>();

                if ((target != null) && !"null".equals(target))
                    parameters.put(LyWebConstants.TARGET_PARAM, target);

                String answerUrl = viewManager.getActionAbsoluteUrl(
                        LyStrutsConstants.PATH_ACTION_EXIT, parameters, true);

                viewManager.setMessageAnswerUrl(answerUrl);

                generateAdditionalScripts(session, request, response, pw);
            }

            // Unset current session
            sessionBean.endRequest();

            // end the request
            endRequest(request, response, viewManager, pw);

        }
        finally
        {
            session.releaseSoftLock();
        }
        // the response has been written
        return null;
    } // end method execute
} // end class LyStrutsExitAction
