/*
 * LyStrutsGraphicalEventAction.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts.action;

import java.util.*;
import javax.servlet.http.*;
import leon.misc.*;
import leon.view.event.*;
import leon.view.web.*;
import leon.view.web.struts.*;


/**
 * Type : leon.view.web.struts.action.LyStrutsGraphicalEventAction.<br>
 * This action handles graphical events from the User Interface:
 *
 * <ul>
 *   <li>Table sort</li>
 *   <li>Toolbar opening</li>
 * </ul>
 *
 * @version $Id: LyStrutsGraphicalEventAction.java 32866 2011-10-19 13:58:22Z nhamon $
 * @author  Lyria S.A.
 */
public class LyStrutsGraphicalEventAction extends LyStrutsEventAction
{

	/** Constants. */

	public static final String __VERSION =
		"$Id: LyStrutsGraphicalEventAction.java 32866 2011-10-19 13:58:22Z nhamon $";

	/**
	 * Default constructor without parameters.<br>
	 * This constructor should be kept for Java reflection.
	 */
	public LyStrutsGraphicalEventAction()
	{
	}

	/*
	 * (non-Javadoc)
	 * @see leon.view.web.struts.action.LyStrutsEventAction#getDemand(javax.servlet.http.HttpServletRequest, leon.misc.LyEnvironment)
	 */
	public LyDemand getDemand(HttpServletRequest request, LyEnvironment environment)
	{
		// get the event type
		String type = getParameter(request, LyWebConstants.TYPE);

		// get the source component
		String source = getParameter(request, LyWebConstants.SOURCE);

		source = ((source == null) ? "" : source.intern());

		// get the value of the demand
		String value = getParameter(request, LyWebConstants.VALUE);

		// get the index of the graphical event
		String	index = getParameter(request, LyWebConstants.INDEX);
		int		subIndex = -1;

		// Manage sub-index given in value or index parameter
		if (value != null)
		{
			int pos = value.indexOf(LyWebConstants.SUB_INDEX_SEPARATOR);

			if (pos >= 0)
			{
				String subIndexValue = value.substring(pos
						+ LyWebConstants.SUB_INDEX_SEPARATOR.length());
				
				subIndex = LyTools.intFromString(subIndexValue);
				value = value.substring(0, pos);
			}
			else if (index != null)
			{
				pos = index.indexOf(LyWebConstants.SUB_INDEX_SEPARATOR);

				if (pos >= 0)
				{
					String subIndexValue = index.substring(pos
							+ LyWebConstants.SUB_INDEX_SEPARATOR.length());
					
					subIndex = LyTools.intFromString(subIndexValue);
					index = index.substring(0, pos);
				}
			}
		}

		// compute the demand type
		int            demandType = LyActionDemand.TYPE_ACTION;
		Vector<String> selectionIds = getSelectionIds(request);
		
		if (LyWebConstants.POPUP_MENU.equals(source)
				|| LyWebConstants.POPUP_MENU_EXTEND.equals(source))
		{
			String positionX = getParameter(request, LyWebConstants.POSITION_X);
			String positionY = getParameter(request, LyWebConstants.POSITION_Y);
			LyMenuDemand menuDemand = new LyMenuDemand(LyWebConstants.POPUP_MENU.equals(source) ?
					LyMenuDemand.TYPE_POPUP : LyMenuDemand.TYPE_EXTEND_POPUP, value);

			if ((index != null) && (index.trim().length() != 0))
				menuDemand.addIndex(LyTools.intFromString(index));
			
			if (subIndex >= 0)
				menuDemand.setSubIndex(subIndex);
			
			menuDemand.setItem(index);
			menuDemand.setPosition(
				new int[]
				{
					LyTools.intFromString(positionX),
					LyTools.intFromString(positionY)
				});
			
			return menuDemand;
		}

		// Check if we have more accurate action type information.
		if ((type != null) && (type.length() != 0))
		{
			type = type.intern();
			
			if (type == "sort")
				demandType = LyActionDemand.TYPE_SORT;
			else if (type == "extendSort")
				demandType = LyActionDemand.TYPE_EXTEND_SORT;
			else if (type == "href")
				demandType = LyActionDemand.TYPE_HREF;
			else if (type == "expand")
				demandType = LyActionDemand.TYPE_EXPAND;
			else if (type == "open")
				demandType = LyActionDemand.TYPE_OPEN;
			else if (type == "move")
				demandType = LyActionDemand.TYPE_MOVE;
			else if (type == "delete")
				demandType = LyActionDemand.TYPE_DELETE;
			else if (type == "closeTab")
				demandType = LyActionDemand.TYPE_CLOSE_TAB;
			else if (type == "completeValue")
				demandType = LyActionDemand.TYPE_COMPLETE_VALUE;
			else if ((type != null) && (type.equals("_checkField")))
			{
				if ((source != null) && (source.startsWith("LyUnitRelationSelector.")))
					demandType = LyActionDemand.TYPE_UNIT_CHANGED;
				else
					demandType = LyActionDemand.TYPE_VALUE_CHANGED;
			}
			else if ((type != null) && (type.equals("_validateField")))
			{
				// Validation of a field using the 'Enter' key
				value = type;
			}
			// Manage tab next / previous in editable tables
			else if ((type != null) && (type.equals("_tabNext")))
			{
				demandType = LyActionDemand.TYPE_TAB_NEXT;
			}
			else if ((type != null) && (type.equals("_tabPrevious")))
			{
				demandType = LyActionDemand.TYPE_TAB_PREVIOUS;
			}
			else if ((type != null)
					&& (type.equals("_altRight")
							|| type.equals("_altUp")
							|| type.equals("_altLeft")
							|| type.equals("_altDown")
							|| type.equals("_altAdd")))
			{
				source = type;
			}
		}

		// test the source component may be the status
		if (LyWebConstants.STATUS == source)
			demandType = LyActionDemand.TYPE_STATUS;

		// build the action demand
		LyActionDemand demand = new LyActionDemand(demandType, source);

		demand.setValue(value);
		demand.setParameterValues(new LyStrutsParameterValues(request, environment.getString()));
		
		if ((index != null) && (index.length() > 0))
			demand.addIndex(LyTools.intFromString(index));
		
		if (subIndex >= 0)
			demand.setSubIndex(subIndex);
		
		// JUST INDICATIVE : view may not know all selected items and
		// final decision is made by the controller
		demand.setViewSelection(selectionIds);

		// return the action demand
		return demand;
	} // end method getDemand

} // end class LyStrutsGraphicalEventAction
