/*
 * LyStrutsHomeAction.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts.action;

import java.util.*;

import org.apache.struts.action.*;
import javax.servlet.http.*;

import leon.app.*;
import leon.control.*;
import leon.info.*;
import leon.misc.*;
import leon.view.web.*;
import leon.view.web.struts.*;


/**
 * Type : leon.view.web.struts.LyStrutsHomeAction<br>
 * This action manages the building of the whole Leonardi frameset : finds the LY_HOME_PAGE page and
 * forward to this JSP.
 *
 * @version $Id: LyStrutsHomeAction.java 31634 2011-07-08 08:33:05Z nhamon $
 * @author  Lyria S.A.
 */
public class LyStrutsHomeAction extends LyStrutsLeonardiAction
{
	public static final String __VERSION =
		"$Id: LyStrutsHomeAction.java 31634 2011-07-08 08:33:05Z nhamon $";

	/**
	 * Default constructor without parameters.<br>
	 * This constructor should be kept for Java reflection.
	 */
	public LyStrutsHomeAction()
	{
	}

	/**
	 * This is the entry point from the Struts Framework to the Leonardi Framework.<br>
	 * This basic implementation of the Struts Action Model calls the Leonardi Servlet that builds
	 * the user interface in the response.
	 *
	 * @param  mappings - The ActionMapping used to select this instance
	 * @param  bean     - The optional ActionForm bean for this request (if any)
	 * @param  request  - The HTTP request we are processing
	 * @param  response - The HTTP response we are creating
	 * @return null : an empty action forward while the action has written the response.
	 */
	public ActionForward execute(ActionMapping mappings, ActionForm bean,
		HttpServletRequest request, HttpServletResponse response)
		throws Exception
	{
		// Get the session from the session bean
		LyStrutsSessionBean sessionBean = (LyStrutsSessionBean)bean;
		LySession		    session = sessionBean.getSession();
		LyEnvironment	    environment = sessionBean.getEnvironment();
		LyStrutsViewManager viewManager = sessionBean.getViewManager();
		String				homePage = null;
		
		try
		{
			session.getSoftLock();
		
			// If the session is not valid : invoke 'root action'
			if (!session.isRegistered())
			{
				// No caching for this page.
				setNoCacheHeaders(response, 0);
				// Start the session which invokes root Action
				session.register(true);
				// Call the getDecoration() method to be sure that CSS file is generated
				viewManager.getDecoration();
			}
			
			checkSessionAttributes(session, request);
			
			// Update language or skin if parameter is found from the request
			String	    language = getParameter(request, LyWebConstants.LANGUAGE);
			String	    skin = getParameter(request, LyWebConstants.SKIN);
	
			if ((language != null) && !language.equals(""))
				session.setLanguage(language);
	
			if ((skin != null) && !skin.equals(""))
			{
				session.setSkin(skin);
	
				LyWebDecoration deco = viewManager.getDecoration();
	
				deco.setEnvironment(session.getEnvironment());
			}
	
			if (!session.isValid())
			{
				// Check if root action is _login
				LyApplication application = session.getApplication();
				LyController  controller = session.getController(0);
				LyAction	  rootAction = application.getProject().getRootAction();
				String		  rootId = rootAction.getRoot().getId();
	
				if (rootId.equals(LyAction.ID_LOGIN))
				{
					// Make a redirect to the the login action
					Map<String, String> parameters = new HashMap<String, String>();
			
					parameters.put(LyWebConstants.ACTION, rootAction.getId());

					String		target = getParameter(request, LyWebConstants.TARGET_PARAM);
					
					if (controller != null)
						parameters.put(LyWebConstants.CONTROLLER, controller.getId());
					
					if ((target != null) && !"null".equals(target))
						parameters.put(LyWebConstants.TARGET_PARAM, target);
			
					// make the redir
					String url = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_LOGIN,
							parameters, false);
			
					response.sendRedirect(url);
			
					return null;
				}
			}
			
			session.resetTimeout();
			
			String redirect = (String)session.getValue("REDIRECT_ACTION_URL");
			if(redirect != null && !redirect.equals(""))
			{
				session.addValue("REDIRECT_ACTION_URL", "");
				response.sendRedirect(redirect);
				return null;
			}
			
			// Get the LY_HOME_PAGE ressource
			homePage = environment.getUrl("LY_HOME_PAGE");
	
			// get the path relative to the context
			homePage = getContextReativePath(request, homePage);
	
			// set no cache
			setNoCacheHeaders(response, 0);
	
			// store usefull attributes
			setRequestAttributes(request, sessionBean.getSession());
	
			// search for the up and target actions
			HashMap<String, String>	parameters = new HashMap<String, String>();
			String					target = request.getParameter(LyWebConstants.TARGET_PARAM);
			
			if (target != null)
				parameters.put(LyWebConstants.TARGET_PARAM, target);
	
			String upFrame = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_UPFRAME, parameters, true);
			String targetFrame = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_TARGETFRAME, parameters, true);
			String closePopup = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_CLOSE_POPUP, parameters, true);
			String refresh = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_REFRESH, parameters, true);
			String push = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_PUSH, parameters, true);
			String popup = viewManager.getPopupFrameAbsoluteUrl();
			
			request.setAttribute("leonardiUpFrame", upFrame);
			request.setAttribute("leonardiTargetFrame", targetFrame);
			request.setAttribute("leonardiPopupFrame", popup);
			request.setAttribute("leonardiClosePopupFrame", closePopup);
			request.setAttribute("leonardiRefreshUrl", refresh);
			request.setAttribute("leonardiPushUrl", push);

		}
		finally
		{
			session.releaseSoftLock();
		}
		return new ActionForward(homePage);
	} // end method execute
} // end class LyStrutsHomeAction
