/*
 * LyStrutsInvokeAction.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts.action;

import java.io.*;
import java.util.*;

import javax.servlet.http.*;

import leon.app.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;
import leon.view.web.*;
import leon.view.web.struts.*;

import org.apache.struts.action.*;


/**
 * Type : leon.view.web.struts.action.LyStrutsInvokeAction.<br>
 *
 * @version  $Id: LyStrutsInvokeAction.java 35778 2012-06-07 16:22:14Z nhamon $
 * @author   Lyria S.A.
 */
public class LyStrutsInvokeAction extends LyStrutsLeonardiAction
{

    public static final String __VERSION = "$Id: LyStrutsInvokeAction.java 35778 2012-06-07 16:22:14Z nhamon $";

    protected String[] _queryParams;

    /**
     * Default constructor without parameters.<br>
     * This constructor should be kept for Java reflection.
     */
    public LyStrutsInvokeAction()
    {
    }

    protected void storeRedirectionUrl(LySession session, HttpServletRequest request)
    {
        String scheme = request.getScheme();
        String serverName = request.getServerName();
        int    serverPort = request.getServerPort();
        String servletPath = request.getServletPath();
        String contextPath = request.getContextPath();
        String queryString = request.getQueryString();

        if (queryString.contains("&" + LyWebConstants.NEW_SESSION + "=true"))
            queryString = queryString.replace("&" + LyWebConstants.NEW_SESSION + "=true", "");

        session.addValue("REDIRECT_ACTION_URL",
            scheme + "://" + serverName + ":" + serverPort + contextPath + servletPath + "?" + queryString);
    }

    /**
     * This action executes the Leonardi invoke action: start a Leonardi logical action and displays
     * the created view.
     *
     * @param   mappings  - The ActionMapping used to select this instance
     * @param   bean      - The optional ActionForm bean for this request (if any)
     * @param   request   - The HTTP request we are processing
     * @param   response  - The HTTP response we are creating
     *
     * @return  null : an empty action forward while the action has written the response.
     */
    public ActionForward execute(ActionMapping mappings, ActionForm bean, HttpServletRequest request,
        HttpServletResponse response) throws Exception
    {
        // Get the session from the session bean
        LyStrutsSessionBean sessionBean = (LyStrutsSessionBean)bean;
        LySession           session = sessionBean.getSession();
        LyEnvironment       environment = sessionBean.getEnvironment();
        LyStrutsViewManager viewManager = sessionBean.getViewManager();
		String				navigationMode = session.getEnvironment().getResource("LY_WEB_DIRECT_URL_NAVIGATION");
        boolean				directUrlNavigation = session.getEnvironment().testResource("LY_WEB_DIRECT_URL_NAVIGATION");
        boolean				directUrlNavigationSession = "session".equals(navigationMode);

        if (!session.isRegistered())
        {
            // No caching for this page.
            setNoCacheHeaders(response, 0);
            // Start the session which invokes root Action
            session.register(true);
            // Call the getDecoration() method to be sure that CSS file is generated
            viewManager.getDecoration();

            storeRedirectionUrl(session, request);
        }

        // Check if session is registered, otherwise redirect to home page
        if (!checkSessionRegistered(sessionBean, request, response))
        {
    		if (directUrlNavigation || directUrlNavigationSession)
            {
                // accept auto connect on session.
                // No caching for this page.
                setNoCacheHeaders(response, 0);
                // Start the session which invokes root Action
                session.register(false);
                // Call the getDecoration() method to be sure that CSS file is generated
                viewManager.getDecoration();

                if (!checkSessionRegistered(sessionBean, request, response))
                    return null;
            }
            else
                return null;
        }

        try
        {
            session.getSoftLock();
            checkSessionAttributes(session, request);

            String displayUpPage = getParameter(request, LyWebConstants.UP_PAGE);

            if ("true".equals(displayUpPage))
	            request.setAttribute(LyWebConstants.RA_LY_SESSION, session);
            
            // Update language or skin if parameter is found from the request
            String language = getParameter(request, LyWebConstants.LANGUAGE);
            String skin = getParameter(request, LyWebConstants.SKIN);

            if ((language != null) && !language.equals(""))
                session.setLanguage(language);

            if ((skin != null) && !skin.equals(""))
            {
                session.setSkin(skin);

                LyWebDecoration deco = viewManager.getDecoration();

                deco.setEnvironment(session.getEnvironment());
            }

            if (!session.isValid())
            {
                // Check if root action is _login
                LyApplication application = session.getApplication();
                LyController  controller = session.getController(0);
                LyAction      rootAction = application.getProject().getRootAction();
                String        rootId = rootAction.getRoot().getId();

                if (rootId.equals(LyAction.ID_LOGIN))
                {
                    // Make a redirect to the the login action
                    Map<String, String> parameters = new HashMap<String, String>();

                    parameters.put(LyWebConstants.ACTION, rootAction.getId());

                    if (controller != null)
                        parameters.put(LyWebConstants.CONTROLLER, controller.getId());

                    // make the redir
                    String url = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_LOGIN, parameters,
                            false);

                    storeRedirectionUrl(session, request);

                    response.sendRedirect(url);

                    return null;
                }
            }

            String	queryString = request.getQueryString();
        	String	initialControllerId = getParameter(request, "_initialController");
            
        	if (initialControllerId != null)
            {
            	LyController	initialController = session.getController(initialControllerId);
    			int				steps = 0;
    			LyController	controller = session.getLastViewController(steps);
    	
    			// Check if controller was created from a static action (in this case, controller
    			// should not be displayed here)
    			while ((controller != null)
    					&& !isParentController(controller, initialController))
    			{
    				steps++;
    				controller = session.getLastViewController(steps);
    			}
    	
    			if (controller == null)
    				controller = initialController;
    			
    			// get the area controller
    			if (controller != null)
    			{
                    LyWebPageWriter pageWriter = (LyWebPageWriter)controller.getView().getInternalView();
        			String		    frame = LyWebConstants.TARGET_FRAME;
        	
        			if (pageWriter != null)
        			{
        				// unset current session
        				sessionBean.beginRequest();
        				
        				// begin the request
        				Writer	pw = beginRequest(request, response, viewManager);

        				synchronized (pageWriter)
        				{
        					// check the reload state of the page writer
        					checkReloadState(pageWriter, controller, response);
        	
        					// We prepare the current page
        					preparePage(pageWriter, frame, null);
        	
        					// write the page
        					writePage(request, session, environment, viewManager, pageWriter, pw, frame, null);
        				}
        		    	
            			viewManager.clearReloadFrames();
            			viewManager.clearAdditionalRefreshScripts();
            			viewManager.setMessage(null);
            			
            			// unset current session
            			sessionBean.endRequest();
            	
            			// end request
            			endRequest(request, response, viewManager, pw);
            			
            			return null;
        			}
    			}
            }
            // First we load the homePage
        	else if (!queryString.contains("&_redirect=done"))
                return loadHomePage(request, response, environment, sessionBean, viewManager);

            // Then we redirect to the invoked action
            queryString = queryString.replace("&_redirect=done", "");

            String controllerId = getParameter(request, LyWebConstants.CONTROLLER);
            String frame = getParameter(request, LyWebConstants.FRAME);
            String targetWindow = getParameter(request, LyWebConstants.TARGET_WINDOW);

            if (frame == null)
    			frame = LyWebConstants.TARGET_FRAME;
            
            LyWebApplication application = sessionBean.getApplication();

            // begin the request
            Writer pw = beginRequest(request, response, viewManager);

            // get the action parameter
            String action = getParameter(request, LyWebConstants.ACTION);
            // get the info action
            LyAction actionInfo = (action == null) ? null : application.getAction(action);

            // if the action is null, this action is useless
            if (actionInfo == null)
                return null;

            // set the current session
            sessionBean.beginRequest();

            // set no cache
            setNoCacheHeaders(response, 0);

            // To avoid the creation of multiple controllers for the same action
            // search for a static action previously stored.
            LyViewController viewController = null;
            boolean          invokeAction = true;
            boolean          contextualAction = (actionInfo.getMin() > 0);

            // If action has already been display, retrieve the view controller
            if (!contextualAction)
            {
                controllerId = sessionBean.getStaticAction(actionInfo.getId());

                if (controllerId != null)
                {
                    viewController = (LyViewController)session.getController(controllerId);
                    invokeAction = (viewController == null);
                }
            }

            // invoke the action if needed
            if (invokeAction)
            {
                // get the controller and the view controller
                LyController controller = null;

        		if (directUrlNavigation || directUrlNavigationSession)
                {
                    LyClassInfo  classInfo = null;
                    LyObjectList context = null;
                    String       classInfoParam = getParameter(request, LyWebConstants.CLASSINFO);

                    if (classInfoParam != null)
                    {
                        classInfo = session.getApplication().getClassInfo(classInfoParam);

                        if (classInfo != null)
                        {
                            String[] objectIdStr = request.getParameterValues(LyWebConstants.OBJECTID);
                            String[] objectNameStr = request.getParameterValues(LyWebConstants.OBJECTNAME);

                            context = getDirectUrlNavigationContext(session, classInfo, objectIdStr, objectNameStr);

                            if (context != null)
                            {
                                // Find real class info
                                LyInfoList<LyClassInfo> classes = context.getClasses();

                                if ((classes != null)
                                        && (classes.getSize() == 1))
                                    classInfo = classes.getInfo(0);
                            }
                        }
                    }

                    controller = invokeLeonardiAction(sessionBean, actionInfo, classInfo, context);

                    if (queryString.contains("_dashboard=true"))
                    {
                    	if (controller != null)
                            session.addValue("CLOSE_APPLICATION_ON_VALIDATE", controller.getId());
                    	else
                    		session.addValue("CLOSE_APPLICATION_ON_VALIDATE", "dashboard");
                    }
                    else
                    	session.addValue("CLOSE_APPLICATION_ON_VALIDATE", "true");
                }
                else
                    controller = invokeLeonardiAction(sessionBean, actionInfo);

                viewController = (controller == null) ? null : controller.getViewController();

                // Register the new static action
                if (viewController != null)
                {
                    // Do not free this controller when closing it (may happen when the 'close all views' item
                    // is used from the dashboard menubar.
                    if (!contextualAction)
                        viewController.setFreeOnClose(false);
                    
                    String scheme = request.getScheme();
                    String serverName = request.getServerName();
                    int    serverPort = request.getServerPort();
                    String servletPath = request.getServletPath();
                    String contextPath = request.getContextPath();

                    session.addValue("TARGET_ACTION_URL", scheme + "://" + serverName + ":" + serverPort
                    	+ contextPath + servletPath + "?" + queryString + "&_initialController=" + controller.getId());
                }
            }

            boolean	generateEmptyPage = (viewController == null);
            
            // if we have a view controller, show the view
            if (viewController != null)
            {
                if (targetWindow != null)
                {
                    if (viewController.getView() instanceof LyFrame)
                        ((LyFrame)viewController.getView()).setTarget(targetWindow);
                    else if (viewController.getView() instanceof LyDialog)
                        ((LyDialog)viewController.getView()).setTarget(targetWindow);
                }

                viewController.showView();

                viewManager.clearReloadFrames();

                if (!directUrlNavigationSession)
                {
	                if (!viewController.isFreed())
	                    sessionBean.setStaticAction(actionInfo.getId(), viewController.getId());
	                else
	                {
	                    controllerId = sessionBean.getStaticAction(actionInfo.getId());
	
	                    if (controllerId != null)
	                        viewController = (LyViewController)session.getController(controllerId);
	                }
	
	                if (contextualAction)
	                    session.controllerHidden(viewController);
	
	                LyWebPageWriter pageWriter = (LyWebPageWriter)viewController.getView().getInternalView();
	
	                // From this point (till we write it), the pagewriter must not change
	                if (pageWriter != null)
	                {
	                    synchronized (pageWriter)
	                    {
	                        // We prepare the current page
	                        preparePage(pageWriter, frame, null);
	
	                        // write the page
	                        writePage(request, session, environment, viewManager, pageWriter, pw, frame, null);
	                    }
	                }
	            	
	    			viewManager.clearReloadFrames();
	    			viewManager.clearAdditionalRefreshScripts();
	    			viewManager.setMessage(null);
                }
                else
                {
					String url = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_HOME, null, false);
					
					viewManager.addAdditionalRefreshScripts("<script type='text/javascript'>getLeonardiTopFrame().location.replace('" + url + "');</script>");
					session.addValue("CLOSE_APPLICATION_ON_VALIDATE", null);
					session.addValue("REDIRECT_ACTION_URL", "");
					session.addValue("TARGET_ACTION_URL", "");
                	generateEmptyPage = true;
                }
            }
            
            if (generateEmptyPage)
            {
        		pw.write("<html>\n<head>\n");
        		pw.write("<meta name=\"Content-Type\" content=\"text/html;charset=" + LyString.DEFAULT_CHARSET + "\"/>\n");
        		pw.write("</head>\n<body>\n");

        		// add javascripts
        		writeJavaScriptFile(environment, pw, LyWebConstants.JS_STRING);
        		writeJavaScriptFile(environment, pw, LyWebConstants.JS_UTILS);
        		writeJavaScriptFile(environment, pw, LyWebConstants.JS_ACTIONS);
        		writeJavaScriptFile(environment, pw, LyWebConstants.JS_MESSAGE);

        		generateAdditionalScripts(session, request, response, pw);

        		// Close page
        		pw.write("\n</body>\n</html>\n");
            }

            // unset the current session
            sessionBean.endRequest();

            // end the request
            endRequest(request, response, viewManager, pw);
        }
        finally
        {
            session.releaseSoftLock();
        }
        // the response has been written
        return null;
    } // end method execute

    /**
     * Check if a given controller is a parent of another controller.
     */
    protected boolean isParentController(LyController controller, LyController parentController)
    {
    	if ((controller == null) || (parentController == null))
    		return false;
    	
    	LyController	tmpController = controller;
    	
    	while (tmpController != null)
    	{
    		tmpController = tmpController.getParent();
    		
    		if (tmpController == parentController)
    			return true;
    	}
    	
    	return false;
    }
    
    /**
     * Load home page with the invoke.do url for target.
     *
     * @param   request      The http request
     * @param   response     The http response
     * @param   environment  The current environment
     * @param   sessionBean  The current session
     * @param   viewManager  The current view manager
     *
     * @return  The Home Page
     */
    protected ActionForward loadHomePage(HttpServletRequest request, HttpServletResponse response,
        LyEnvironment environment, LyStrutsSessionBean sessionBean, LyStrutsViewManager viewManager)
    {
        String queryString = request.getQueryString();

        queryString = queryString + "&_redirect=done";

        if (queryString.contains("&" + LyWebConstants.NEW_SESSION + "=true"))
            queryString = queryString.replace("&" + LyWebConstants.NEW_SESSION + "=true", "");

        String scheme = request.getScheme();
        String serverName = request.getServerName();
        int    serverPort = request.getServerPort();
        String servletPath = request.getServletPath();
        String contextPath = request.getContextPath();

        String targetFrame = scheme + "://" + serverName + ":" + serverPort + contextPath + servletPath + "?"
            + queryString;

        // Get the LY_HOME_PAGE ressource
        String homePage = environment.getUrl("LY_HOME_PAGE");

        // get the path relative to the context
        homePage = getContextReativePath(request, homePage);

        // set no cache
        setNoCacheHeaders(response, 0);

        // store usefull attributes
        setRequestAttributes(request, sessionBean.getSession());

        // search for the up and target actions
        HashMap<String, String> parameters = new HashMap<String, String>();
        String                  target = request.getParameter(LyWebConstants.TARGET_PARAM);

        if (target != null)
            parameters.put(LyWebConstants.TARGET_PARAM, target);

        String upFrame = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_UPFRAME, parameters, true);
        String closePopup = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_CLOSE_POPUP, parameters,
                true);
        String refresh = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_REFRESH, parameters, true);
        String push = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_PUSH, parameters, true);
        String popup = viewManager.getPopupFrameAbsoluteUrl();

        request.setAttribute("leonardiUpFrame", upFrame);
        request.setAttribute("leonardiTargetFrame", targetFrame);
        request.setAttribute("leonardiPopupFrame", popup);
        request.setAttribute("leonardiClosePopupFrame", closePopup);
        request.setAttribute("leonardiRefreshUrl", refresh);
        request.setAttribute("leonardiPushUrl", push);

        return new ActionForward(homePage);
    }

    /**
     * Get the paramName parameter value.
     *
     * @param   paramName  The parameter name
     *
     * @return  The associate value
     */
    protected String getQueryParam(String paramName)
    {
        if ((_queryParams == null) || (_queryParams.length == 0))
            return "";

        int i = 0;

        while ((i < _queryParams.length) && !_queryParams[i].contains(paramName + "="))
            i++;

        if (i < _queryParams.length)
            return _queryParams[i].replace(paramName + "=", "");

        return "";
    }

    /**
     * Direct url navigation : compute the context for the request.
     *
     * @param   session        current user session
     * @param   classInfo      targetClassinfo to look into
     * @param   objectIdStr    list of ids to add in the context
     * @param   objectNameStr  list of object names to adsd in the context
     *
     * @return  the list of objects if any
     */
    protected LyObjectList getDirectUrlNavigationContext(LySession session, LyClassInfo classInfo, String[] objectIdStr,
        String[] objectNameStr)
    {
        if (classInfo == null)
            return null;

        // Try to retrieve objects by their identifiers
        LySimpleObjectList context = null;
        int                i, n = ((objectIdStr == null) ? 0 : objectIdStr.length);
        LyObjectId         objectId;

        for (i = 0; i < n; i++)
        {
            objectId = new LyObjectId(classInfo, objectIdStr[i]);

            if (objectId.getLogicalObject() != null)
            {
                if (context == null)
                    context = new LySimpleObjectList();

                context.addObject(objectId.getLogicalObject());
            }
        }

        // Try to retrieve objects by their names
        LyObjectList objectsByName = session.getObjectsByName(new LyInfoList<LyClassInfo>(classInfo), objectNameStr);

        if ((objectsByName != null) && (objectsByName.getSize() > 0))
        {
            if (context == null)
                context = new LySimpleObjectList();

            context.addList(objectsByName);
        }

        return context;
    } // end method getDirectUrlNavigationContext

} // end class LyStrutsInvokeAction
