/*
 * LyStrutsLoginAction.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts.action;

import org.apache.struts.action.*;
import java.io.*;
import java.util.*;

import javax.servlet.http.*;

import leon.app.*;
import leon.control.*;
import leon.info.*;
import leon.misc.*;
import leon.view.web.*;
import leon.view.web.struts.*;


/**
 * Type : leon.view.web.struts.LyStrutsLoginAction<br>
 * This struts action manages the "Login" Leonardi action : forward to the login page or invoke the
 * "Login" Leonardi action.
 *
 * @version $Id: LyStrutsLoginAction.java 30054 2011-03-14 15:10:34Z bangiole $
 * @author  Lyria
 */
public class LyStrutsLoginAction extends LyStrutsLeonardiAction
{
	public static final String __VERSION =
		"$Id: LyStrutsLoginAction.java 30054 2011-03-14 15:10:34Z bangiole $";

	/**
	 * Default constructor without parameters.<br>
	 * This constructor should be kept for Java reflection.
	 */
	public LyStrutsLoginAction()
	{
	}

	/**
	 * This is the entry point from the Struts Framework to the Leonardi Framework.<br>
	 * This basic implementation of the Struts Action Model calls the Leonardi Servlet that builds
	 * the user interface in the response.
	 *
	 * @param  mappings - The ActionMapping used to select this instance
	 * @param  bean     - The optional ActionForm bean for this request (if any)
	 * @param  request  - The HTTP request we are processing
	 * @param  response - The HTTP response we are creating
	 * @return null : an empty action forward while the action has written the response.
	 */
	public ActionForward execute(ActionMapping mappings, ActionForm bean,
		HttpServletRequest request, HttpServletResponse response)
		throws Exception
	{
		// Get the session from the session bean
		LyStrutsSessionBean sessionBean = (LyStrutsSessionBean)bean;
		LySession		    session = sessionBean.getSession();
		LyWebApplication    application = sessionBean.getApplication();
		LyEnvironment	    environment = sessionBean.getEnvironment();
		LyStrutsViewManager viewManager = sessionBean.getViewManager();
		
		// Check if session is registered, otherwise redirect to home page
		if (!checkSessionRegistered(sessionBean, request, response))
			return null;
		
		try
		{
			session.getSoftLock();
	
			if (session.isValid())
			{
				// Make the redirection to the home page since session is already valid
				String url = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_HOME, null, false);
		
				response.sendRedirect(url);
		
				return null;
			}
	
			// Get the LY_LOGIN_PAGE ressource
			String loginPage = getLoginPage(sessionBean);
	
			if ((loginPage != null) && (loginPage.trim().length() > 0))
			{
				// get the path relative to the context
				loginPage = getContextReativePath(request, loginPage);
	
				// set no cache
				setNoCacheHeaders(response, 0);
	
				// set the navigator type
				String userAgent = request.getHeader("User-Agent");
	
				if (userAgent != null)
					viewManager.setBrowser(userAgent);
	
				// store usefull attributes
				HashMap<String, String>	parameters = new HashMap<String, String>();
				String					target = request.getParameter(LyWebConstants.TARGET_PARAM);
				
				if (target != null)
					parameters.put(LyWebConstants.TARGET_PARAM, target);
	
				setRequestAttributes(request, session);
		
				String refresh = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_REFRESH, null, true);
				String upFrame = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_UPFRAME, parameters, true);
				String targetFrame = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_TARGETFRAME, parameters, true);
				String closePopup = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_CLOSE_POPUP, parameters, true);
				String push = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_PUSH, parameters, true);
				String popup = viewManager.getPopupFrameAbsoluteUrl();
					
				request.setAttribute("leonardiUpFrame", upFrame);
				request.setAttribute("leonardiTargetFrame", targetFrame);
				request.setAttribute("leonardiPopupFrame", popup);
				request.setAttribute("leonardiClosePopupFrame", closePopup);
				request.setAttribute("leonardiPushUrl", push);
				request.setAttribute("leonardiRefreshUrl", refresh);
				request.setAttribute(LyWebConstants.RA_LY_ROOT_URL, viewManager.getRootURL());

				return new ActionForward(loginPage);
			}
	
			// begin the request
			Writer	pw = beginRequest(request, response, viewManager);

			// Get the controller id
			String		 controllerId = getParameter(request, LyWebConstants.CONTROLLER);
			LyController controller = (controllerId == null) ? null
															 : session.getController(controllerId);
	
			if (controller == null)
			{
				// Get the action
				String   actionId = getParameter(request, LyWebConstants.ACTION);
				LyAction action = application.getAction(actionId);
	
				// invoke action
				controller = invokeLeonardiAction(sessionBean, action);
			}
	
			// show the controller
			controller.showView();
	
			// get the view controller
			LyViewController viewController = controller.getViewController();

			// get the written page
			LyWebPageWriter  pageWriter = viewManager.getPageWriter();
	
			// write the page writer
			synchronized (pageWriter)
			{
				// Check the reload state of the page writter
				checkReloadState(pageWriter, viewController, null);
	
				// We prepare the current page
				preparePage(pageWriter, LyWebConstants.TARGET_FRAME, null);
	
				// write the page
				writePage(request, session, environment, viewManager, pageWriter, pw,
					LyWebConstants.TARGET_FRAME, null);
	
				// end request
				endRequest(request, response, viewManager, pw);
			}
		}
		finally
		{
			session.releaseSoftLock();
		}
		// Do not return any page while the page is written directly in the
		// response
		return null;
	} // end method execute
	
	/**
	 * Returns the JSP used for the login.
	 * 
	 * @param sessionBean current session bean for this request
	 * @return the url of the JSP used for the login.
	 */
	protected String getLoginPage(LyStrutsSessionBean sessionBean)
	{
		LyEnvironment	    environment = sessionBean.getEnvironment();

		// Get the LY_LOGIN_PAGE ressource
		return environment.getUrl("LY_LOGIN_PAGE");
	}
} // end class LyStrutsLoginAction
