/*
 * LyStrutsMapAction.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts.action;

import java.awt.*;
import java.io.*;

import javax.servlet.http.*;

import leon.app.*;
import leon.misc.*;
import leon.view.*;
import leon.view.event.*;
import leon.view.web.*;
import leon.view.web.struts.*;
import leon.view.web.struts.map.*;

import org.apache.struts.action.*;


/**
 * Type : leon.view.web.struts.action.LyStrutsCommandAction<br>
 * This action manages all the requests comming from the map components inside Leonardi user
 * interfaces.
 *
 * @version $Id: LyStrutsMapAction.java 34821 2012-03-19 10:31:58Z jlsaouli $
 * @author  Lyria S.A.
 */
public class LyStrutsMapAction extends LyStrutsLeonardiAction
{
	public static final String __VERSION =
		"$Id: LyStrutsMapAction.java 34821 2012-03-19 10:31:58Z jlsaouli $";

	/**
	 * Default constructor without parameters.<br>
	 * This constructor should be kept for Java reflection.
	 */
	public LyStrutsMapAction()
	{
	}

	/**
	 * Generic treatment for the command actions of the Leonardi User Interface.
	 *
	 * @param  mappings - The ActionMapping used to select this instance
	 * @param  bean     - The optional ActionForm bean for this request (if any)
	 * @param  request  - The HTTP request we are processing
	 * @param  response - The HTTP response we are creating
	 * @return null : an empty action forward while the action has written the response.
	 */
	public ActionForward execute(ActionMapping mappings, ActionForm bean,
		HttpServletRequest request, HttpServletResponse response)
		throws Exception
	{
		// Get the session from the session bean
		LyStrutsSessionBean sessionBean = (LyStrutsSessionBean)bean;
		LySession		    session = sessionBean.getSession();
		LyEnvironment	    environment = sessionBean.getEnvironment();
		LyStrutsViewManager viewManager = sessionBean.getViewManager();
		
		// Check if session is registered, otherwise redirect to home page
		if (!checkSessionRegistered(sessionBean, request, response))
			return null;

		try
		{
			session.getSoftLock();

			// Get some request parameters
			String source = getParameter(request, LyWebConstants.SOURCE);
			String value = getParameter(request, LyWebConstants.VALUE);
			String index = getParameter(request, LyWebConstants.INDEX);
			String controller = getParameter(request, LyWebConstants.CONTROLLER);
			String page = getParameter(request, LyWebConstants.PAGE);
	
			// Set current session
			sessionBean.beginRequest();
	
			// Transmit the action demand
			if ((source != null) && (controller != null))
			{
				// get the demand type
				int demandType = LyActionDemand.TYPE_ACTION;
	
				LyActionDemand demand = new LyActionDemand(demandType, source);
	
				demand.setValue(value);
				demand.setParameterValues(new LyStrutsParameterValues(request, environment.getString()));
	
				viewManager.processDemand(controller, demand);
			}
			
			LyStrutsMap	strutsMap = (LyStrutsMap)viewManager.getPageWriter(page);
			LyMap		map = (strutsMap != null ? (LyMap)strutsMap.getViewComponent() : null);
	
			if ((strutsMap != null) && "loadImage".equals(source) && (value != null))
			{
				String[]	divIds = LyString.stringToList(value, '*');
				int			i, nDivs = (divIds == null ? 0 : divIds.length);
				
				for (i = 0; i < nDivs; i++)
				{
					String[]	values = LyString.stringToList(divIds[i], '_');
					
					if ((values != null) && (values.length == 5))
					{
						StringBuffer     script = new StringBuffer();
						String			imagePath = strutsMap.generateMapImage(LyTools.intFromString(values[1]), LyTools.intFromString(values[2]),
								LyTools.intFromString(values[3]), LyTools.intFromString(values[4]), false);
						
						script.append("<script type='text/javascript'>\n//<![CDATA[\n");
						script.append("replaceDivInnerHtml(\"");
						script.append(strutsMap.getLastFrameName()).append("\",\"");
						script.append(divIds[i]).append("\",\"<img src='" + imagePath + "' />\");");
						script.append("\n//]]>\n</script>");
		
						viewManager.addAdditionalRefreshScripts(script.toString());
					}
				}
			}
			else if ((strutsMap != null) && "translateMap".equals(source) && (value != null))
			{
				int	tx = 0, ty = 0;
				
				// Remove 'px' strings
				value = LyString.replace(value, "px", "");
				
				String[]	values = LyString.stringToList(value, '_');
				
				if ((values != null) && (values.length == 2))
				{
					tx = (int)LyTools.floatFromString(values[0]);
					ty = (int)LyTools.floatFromString(values[1]);
					strutsMap.getMapCanvas()._scrollx += tx;
					strutsMap.getMapCanvas()._scrolly += ty;
				}
				else
				{
					strutsMap.getMapCanvas()._scrollx = 0;
					strutsMap.getMapCanvas()._scrolly = 0;
				}
	
				strutsMap.updateViewerImage();
			}
			else if ((strutsMap != null) && "setMapZoomScale".equals(source) && (value != null))
			{
				// Remove 'px' strings
				value = LyString.replace(value, "px", "");
				
				String[]	values = LyString.stringToList(value, '_');
				float		offsetX = 0;
				float		offsetY = 0;
	
				float	zoomStep = map.getZoomStep();
				
				if (zoomStep <= 1)
					zoomStep = 2;
				
				int		scale = (values != null ? LyTools.intFromString(values[0]) : 1);
				float	zoomFactor = (float)(Math.pow(zoomStep, scale));
				boolean	wasIdle = map.isIdle();
				
				map.setIdle(true);
	
				int	xx, yy;
				
				if ((values != null) && (values.length == 3))
				{
					xx = (int)LyTools.floatFromString(values[1]);
					yy = (int)LyTools.floatFromString(values[2]);
				}
				else
				{
					xx = -strutsMap.getMapCanvas()._scrollx;
					yy = -strutsMap.getMapCanvas()._scrolly;
				}
				
				float	x1 = (xx / map.getZoomFactor()) * zoomFactor;
				float	y1 = (yy / map.getZoomFactor()) * zoomFactor;
				float	x2 = xx;
				float	y2 = yy;
				
				offsetX = x2 - x1;
				offsetY = y2 - y1;
				strutsMap.getMapCanvas()._scrollx += offsetX;
				strutsMap.getMapCanvas()._scrolly += offsetY;
	
				map.setZoomFactor(zoomFactor);
				map.setIdle(wasIdle);
			}
			else if ((strutsMap != null) && "translateViewer".equals(source) && (value != null))
			{
				// Remove 'px' strings
				value = LyString.replace(value, "px", "");
				
				String[]	values = LyString.stringToList(value, '_');
				
				if ((values != null) && (values.length == 2))
				{
					LyStrutsMapCanvas	mapCanvas = strutsMap.getMapCanvas();
					
					mapCanvas.resetTranslation();
					
					Rectangle scrollRectangle = mapCanvas.getScrollRectangle();
					Dimension dimension = strutsMap.getMapDimension();
					float	  deltaX = -LyTools.intFromString(values[0]) + LyStrutsMapCanvas.VIEWER_SIZE / 2;
					float	  deltaY = -LyTools.intFromString(values[1]) + LyStrutsMapCanvas.VIEWER_SIZE / 2;
					float     ratioX = ((float)scrollRectangle.width) / (LyStrutsMapCanvas.VIEWER_SIZE);
					float     ratioY = ((float)scrollRectangle.height) / (LyStrutsMapCanvas.VIEWER_SIZE);
					float	  ratio = Math.max(ratioX, ratioY);
					int		  tX = (int)(deltaX * ratio);
					int		  tY = (int)(deltaY * ratio);
	
					mapCanvas._scrollx = tX;
					mapCanvas._scrolly = tY;
					mapCanvas.setSize(dimension.width, dimension.height);
					mapCanvas.setZoom(map.getZoomFactor());
					mapCanvas.resetTranslation();
	
					Rectangle		boundingRect = mapCanvas.getBoundingRectangle();
					StringBuffer    script = new StringBuffer();
					String			actionUrl = viewManager.getActionURL(LyStrutsConstants.PATH_ACTION_MAP);
					
					script.append("<script type='text/javascript'>\n//<![CDATA[\n");
					script.append("setMapOffset(\"mapContent_");
					script.append(map.getIdCode());
					script.append("\",");
					script.append((boundingRect.x + mapCanvas._scrollx));
					script.append(",");
					script.append((boundingRect.y + mapCanvas._scrolly));
					script.append(");");
					script.append("updateLoadedImages(\"" + strutsMap.getId() + "\", \"" + map.getRootController() + "\", \"" + actionUrl + "\");");
					script.append("\n//]]>\n</script>");
					
	
					viewManager.addAdditionalRefreshScripts(script.toString());
					strutsMap.updateViewerImage();
				}
			}
			else if ((strutsMap != null) && "moveElements".equals(source) && (value != null))
			{
			// Remove 'px' strings
				index = LyString.replace(index, "px", "");
				
				String[]	indexes = LyString.stringToList(index, '|');
				
				if ((indexes != null) && (indexes.length == 2))
				{
				LyStrutsMapCanvas	mapCanvas = strutsMap.getMapCanvas();
	
					mapCanvas.setSize(strutsMap.getMapDimension());
					mapCanvas.setZoom(map.getZoomFactor());
					mapCanvas.resetTranslation();
					
					Rectangle		boundingRect = mapCanvas.getBoundingRectangle();
	
					Point			position = new Point(LyTools.intFromString(indexes[0]) + boundingRect.x,
							LyTools.intFromString(indexes[1]) + boundingRect.y);
					LyPoint			point = mapCanvas.revert(position);
					LyActionDemand	demand = new LyActionDemand(LyActionDemand.TYPE_MOVE, value);
	
					demand.setValue(point.getX() + "|" + point.getY());
					demand.setParameterValues(new LyStrutsParameterValues(request, environment.getString()));
	
					viewManager.processDemand(controller, demand);
					strutsMap.updateBackgroundImage();
					strutsMap.updateViewerImage();
				}
			}
	
			// begin the request
			Writer pw = beginRequest(request, response, viewManager);
			
			generateAdditionalScripts(session, request, response, pw);
	
			// unset current session
			sessionBean.endRequest();
	
			// end request
			endRequest(request, response, viewManager, pw);
		}
		finally
		{
			session.releaseSoftLock();
		}
		return null;
	} // end method execute
} // end class LyStrutsMapAction