/*
 * LyStrutsPageAction.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts.action;

import java.io.*;
import java.util.*;

import javax.servlet.http.*;

import leon.app.*;
import leon.view.web.*;
import leon.view.web.struts.*;

import org.apache.struts.action.*;


/**
 * Type : leon.view.web.struts.action.LyStrutsPageAction<br>
 * This action writes an HTML page by using the '<code><b>_page</b></code>' request parameter.<br>
 *
 * @version $Id: LyStrutsPageAction.java 30054 2011-03-14 15:10:34Z bangiole $
 * @author  Lyria S.A.
 */
public class LyStrutsPageAction extends LyStrutsLeonardiAction
{
	public static final String __VERSION =
		"$Id: LyStrutsPageAction.java 30054 2011-03-14 15:10:34Z bangiole $";

	public static String	CODE_SEPARATOR = "____/____";
	
	/**
	 * Default constructor without parameters.<br>
	 * This constructor should be kept for Java reflection.
	 */
	public LyStrutsPageAction()
	{
	}

	/**
	 * Generic treatment for the command actions of the Leoanrdi User Interface.
	 *
	 * @param  mappings - The ActionMapping used to select this instance
	 * @param  bean     - The optional ActionForm bean for this request (if any)
	 * @param  request  - The HTTP request we are processing
	 * @param  response - The HTTP response we are creating
	 * @return null : an empty action forward while the action has written the response.
	 */
	public ActionForward execute(ActionMapping mappings, ActionForm bean,
		HttpServletRequest request, HttpServletResponse response)
		throws Exception
	{
		// get the session from the session bean
		LyStrutsSessionBean sessionBean = (LyStrutsSessionBean)bean;
		LySession		    session = sessionBean.getSession();
		LyStrutsViewManager viewManager = sessionBean.getViewManager();
		
		// Check if session is registered, otherwise redirect to home page
		if (!checkSessionRegistered(sessionBean, request, response))
			return null;

		try
		{
			session.getSoftLock();

			// get the page parameter
			String page = getParameter(request, LyWebConstants.PAGE);
	
			// at this point if the page is null, the action can do nothing, return.
			if ((page == null) || (page.trim().length() == 0))
				return null;
	
			// get other usefull parameters
			String		frame = getParameter(request, LyWebConstants.FRAME);
			String		ajax = getParameter(request, LyWebConstants.AJAX);
	
			// begin the request
			Writer pw = beginRequest(request, response, viewManager);
	
			// The page is given and a frame name may be given.
			// Retrieve the page writer by its id
			LyWebPageWriter pageWriter = viewManager.getPageWriter(page);
	
			// Set this page writer as the current one
			if (pageWriter != null)
			{
				pageWriter.writeHtml(pw, frame);
			}
			else
			{
		        Vector<LySession> slaveSessions = session.getSlaveSessions();
		
		        if (slaveSessions != null)
		        {
					for (LySession slaveSession : slaveSessions)
					{
						LyStrutsViewManager slaveViewManager = (LyStrutsViewManager) slaveSession.getViewManager();
						
						pageWriter = slaveViewManager.getPageWriter(page);
						
						// Set this page writer as the current one
						if (pageWriter != null)
						{
							pageWriter.writeHtml(pw, frame);
							break;
						}
					}
		        }
			}
			
			if ("true".equals(ajax))
			{
				pw.write(CODE_SEPARATOR);
				
				generateAdditionalScripts(session, request, response, pw);
			}
	
			// end request
			endRequest(request, response, viewManager, pw);
		}
		finally
		{
			session.releaseSoftLock();
		}
		return null;
	} // end method execute
} // end class LyStrutsPageAction
