/*
 * LyStrutsTargetFrameAction.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts.action;

import java.io.*;

import javax.servlet.http.*;

import leon.app.*;
import leon.control.*;
import leon.misc.*;
import leon.view.*;
import leon.view.web.*;
import leon.view.web.struts.*;

import org.apache.struts.action.*;


/**
 * Type : leon.view.web.struts.action.LyStrutsTargetFrameAction<br>
 * This action manages the building of the target frame of the Leonardi frameset.
 *
 * @version $Id: LyStrutsPopupFrameAction.java 33496 2011-12-09 10:06:04Z nhamon $
 * @author  Lyria S.A.
 */
public class LyStrutsPopupFrameAction extends LyStrutsLeonardiAction
{
	public static final String __VERSION =
		"$Id: LyStrutsPopupFrameAction.java 33496 2011-12-09 10:06:04Z nhamon $";

	/**
	 * Default constructor without parameters.<br>
	 * This constructor should be kept for Java reflection.
	 */
	public LyStrutsPopupFrameAction()
	{
	}

	/**
	 * This action builds the target frame of the Leonardi frameset.<br>
	 * It looks for the last view controller in the current session and writes the page.<br>
	 * If none are found, the LY_DEFAULT_PAGE page is displayed or the index page.
	 *
	 * @param  mappings - The ActionMapping used to select this instance
	 * @param  bean     - The optional ActionForm bean for this request (if any)
	 * @param  request  - The HTTP request we are processing
	 * @param  response - The HTTP response we are creating
	 * @return a forward to the default page or null if a controller has written itself into the
	 *         response.
	 */
	public ActionForward execute(ActionMapping mappings, ActionForm bean,
		HttpServletRequest request, HttpServletResponse response)
		throws Exception
	{
		// Get the session from the session bean
		LyStrutsSessionBean sessionBean = (LyStrutsSessionBean)bean;
		LySession		    session = sessionBean.getSession();
		LyEnvironment	    environment = sessionBean.getEnvironment();
		LyStrutsViewManager viewManager = sessionBean.getViewManager();
		
		// Check if session is registered, otherwise redirect to home page
		if (!checkSessionRegistered(sessionBean, request, response))
			return null;

		try
		{
			session.getSoftLock();
		
			// begin the request
			Writer	pw = beginRequest(request, response, viewManager);
	
			// No caching for this page.
			setNoCacheHeaders(response, 0);
			
			// Register the Leonardi session on the HTTP session
			request.setAttribute(LyWebConstants.RA_LY_SESSION, session);
			request.getSession().setAttribute(LyWebConstants.RA_LY_SESSION, session);
			
			// get the last view controller
			if (viewManager.getLastPopupViewController() != null)
			{
				LyViewController	viewController = session.getStackLastController(LyWebConstants.POPUP_VIEW);
				LyView				view = viewController.getView();
				LyWebPageWriter		pageWriter = (LyWebPageWriter)view.getInternalView();
	
				// From this point (till we write it), the pagewriter must not change
				if (pageWriter != null)
				{
					synchronized (pageWriter)
					{
						// We prepare the current page
						preparePage(pageWriter, null, LyWebConstants.POPUP_VIEW);
	
						// write the page
						writePage(request, session, environment, viewManager, pageWriter, pw, null, LyWebConstants.POPUP_VIEW);
					}
				}
			}
	
			if (!viewManager.isPushModeActivated())
			{
				viewManager.clearReloadFrames();
				viewManager.clearAdditionalRefreshScripts();
				viewManager.setMessage(null);
			}
			
			endRequest(request, response, viewManager, pw);
		}
		finally
		{
			session.releaseSoftLock();
		}
		return null;
	} // end method execute
} // end class LyStrutsTargetFrameAction
