/*
 * LyStrutsPrintAction.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 * Version: $Id: LyStrutsPrintAction.java 35365 2012-04-25 12:49:11Z nhamon $
 */
package leon.view.web.struts.action;

import org.apache.struts.action.*;
import java.io.*;
import java.util.*;

import javax.servlet.http.*;
import leon.app.*;
import leon.control.*;
import leon.misc.*;
import leon.view.*;
import leon.view.event.*;
import leon.view.setinterface.*;
import leon.view.web.*;
import leon.view.web.struts.*;


/**
 * Type : leon.view.web.struts.action.LyStrutsPrintAction<br>
 * This action handles all the print actions of the Struts viewer.
 *
 * @version $Id: LyStrutsPrintAction.java 35365 2012-04-25 12:49:11Z nhamon $
 * @author  Lyria S.A.
 */
public class LyStrutsPrintAction extends LyStrutsLeonardiAction
{
	public static final String __VERSION =
		"$Id: LyStrutsPrintAction.java 35365 2012-04-25 12:49:11Z nhamon $";

	/**
	 * Default constructor without parameters.<br>
	 * This constructor should be kept for Java reflection.
	 */
	public LyStrutsPrintAction()
	{
	}

	/**
	 * This action manages the printing of views.
	 *
	 * @param  mappings - The ActionMapping used to select this instance
	 * @param  bean     - The optional ActionForm bean for this request (if any)
	 * @param  request  - The HTTP request we are processing
	 * @param  response - The HTTP response we are creating
	 * @return null : an empty action forward while the action has written the response.
	 */
	public ActionForward execute(ActionMapping mappings, ActionForm bean,
		HttpServletRequest request, HttpServletResponse response)
		throws Exception
	{
		// Get the session from the session bean
		LyStrutsSessionBean sessionBean = (LyStrutsSessionBean)bean;
		LySession		    session = sessionBean.getSession();
		LyEnvironment	    environment = sessionBean.getEnvironment();
		LyStrutsViewManager viewManager = sessionBean.getViewManager();
		
		// Check if session is registered, otherwise redirect to home page
		if (!checkSessionRegistered(sessionBean, request, response))
			return null;

		try
		{
			session.getSoftLock();
		
			// get some usefull request parameters
			String			    controllerId = getParameter(request, LyWebConstants.CONTROLLER);
			String			    value = getParameter(request, LyWebConstants.VALUE);
			String			    index = getParameter(request, LyWebConstants.VALUE);
			String			    frame = getParameter(request, LyWebConstants.FRAME);
			String			    page = getParameter(request, LyWebConstants.PAGE);
			String			    hidden = getParameter(request, LyWebConstants.HIDDEN);
			String			    source = getParameter(request, LyWebConstants.SOURCE);
			String			    target = getParameter(request, LyWebConstants.TARGET_PARAM);
			String			    cancelParam = getParameter(request, "_cancel");
	
			if (source == null) // at this point the source is supposed to be non-null
				return null;
	
			// begin the request
			Writer pw = beginRequest(request, response, viewManager);
	
			// set no cache
			setNoCacheHeaders(response, 0);
	
			// print the page
			if ("true".equalsIgnoreCase(hidden))
			{
				if (controllerId != null)
				{
					// process an action demand on the current controller
					LyActionDemand demand = new LyActionDemand(LyActionDemand.TYPE_ACTION,
							source.intern());
	
					demand.setValue(value);
					demand.setParameterValues(new LyStrutsParameterValues(request,
							environment.getString()));
					demand.addIndex(LyTools.intFromString(index));
					
					Vector<String>	reloadFramesBefore = viewManager.getReloadFrames();
					
					// Invoke action on specified controller
					viewManager.processDemand(controllerId, demand);

					Vector<String>	reloadFramesAfter = viewManager.getReloadFrames();
					
					// Target frame may have been added to the list or frame to reload but should not
					if ((reloadFramesBefore == null)
							|| (!reloadFramesBefore.contains(LyWebConstants.TARGET_FRAME)
									&& (reloadFramesAfter != null)
									&& reloadFramesAfter.contains(LyWebConstants.TARGET_FRAME)))
						viewManager.clearReloadFrame(LyWebConstants.TARGET_FRAME);
				}
	
				// write the pop-up
				writeOpenPrintPopup(environment, viewManager, pw, controllerId, target);
			}
			else
			{
				// Retrieve the page writer by its id
				LyWebPageWriter pageWriter = viewManager.getPageWriter(page);
	
				if (pageWriter != null)
				{
					// get the error message
					String errorMessage = environment.getMessage("LY_PRINT_UNSUPPORTED");
	
					errorMessage = LyWebViewManager.encodeString(errorMessage);
	
					synchronized (pageWriter)
					{
						pageWriter.setNoFrames(true);
						pageWriter.setPrintMode(true);
	
						pageWriter.preparePage(null, null);
	
						// write the script for printing
						if (!"true".equals(cancelParam))
							writePrintScript(pageWriter, errorMessage);
	
						// Dump the HTML in the output stream
						pageWriter.writeHtml(pw, frame);
					}
	
					// Free the print controller (since close action is not
					// called by web browser).
					LyViewController lastViewController = session.getLastViewController();
	
					lastViewController.free();
				}
				else
					writePrintPage(request, session, environment, viewManager, pw);
			}
			
            if (!viewManager.isPushModeActivated())
            {
				viewManager.clearReloadFrames();
				viewManager.clearAdditionalRefreshScripts();
            }
	
			// end the request
			endRequest(request, response, viewManager, pw);
		}
		finally
		{
			session.releaseSoftLock();
		}
		return null;
	} // end method execute

	/**
	 * Searches the controller concerned by the printing.
	 *
	 * @param  request     Current HTTP request that contains information about the controller.
	 * @param  session     Current Leonardi session.
	 * @param  viewManager Current view manager.
	 * @return the controller involved in the request.
	 */
	protected LyController getController(HttpServletRequest request, LySession session,
		LyStrutsViewManager viewManager)
	{
		LyController controller = null;
		String		 pageId = getParameter(request, LyWebConstants.PAGE);
		String		 controllerId = getParameter(request, LyWebConstants.CONTROLLER);

		if (pageId != null)
		{
			LyWebPageWriter pw = viewManager.getPageWriter(pageId);

			if (pw != null)
			{
				LyComponent component = pw.getViewComponent();

				controllerId = component.getRootController();
			}
		}

		if (controllerId != null)
			controller = session.getController(controllerId);

		return controller;
	}

	/**
	 * Writes a printing page for the given controller.
	 *
	 * @param  request     Current HTTP request that contains information about the controller.
	 * @param  session     Current Leonardi session.
	 * @param  environment Current Leonardi environment.
	 * @param  viewManager Current view manager.
	 * @param  pw          The current writer to the ouput stream.
	 * @throws IOException
	 */
	protected void writePrintPage(HttpServletRequest request, LySession session,
		LyEnvironment environment, LyStrutsViewManager viewManager, Writer pw)
		throws IOException
	{
		LyController    controllerUsingSourceId = null;
		LyController    controller = getController(request, session, viewManager);
		String		    source = getParameter(request, LyWebConstants.SOURCE);
		int			    printPos = source.indexOf("_print");

		source = source.substring(printPos + 6);

		// Try to get controller from source
		controllerUsingSourceId = session.getController(source);
		
		if ((controllerUsingSourceId != null) && (controllerUsingSourceId != controller))
			controller = controllerUsingSourceId;
		
		LyComponent		 component = controller.getComponent();
		LyWebComponent	 webComponent = (LyWebComponent)component.getInternalView();
		LyWebPageWriter  page = new LyStrutsPage(component, true);
		
		page.setActiveBuffer(null);
		page.setNoFrames(true);
		page.setPrintMode(true);
		
		// FFT 00953 Some controllers may need to change view content for
		// printing
		// e.g. LyMultipageTableController
		if (controller instanceof LySetPrintModeInterface)
			((LySetPrintModeInterface)controller).setPrintMode(true);
		
		webComponent.writeHtml(page);

		// get the error message
		String message = environment.getMessage("LY_PRINT_UNSUPPORTED");

		message = LyWebViewManager.encodeString(message);

		// write the printing script
		writePrintScript(page, message);

		// write the HTML code
		page.writeHtml(pw, null);
		page.free();

		// FFT 00953 Some controllers may need to change view content for
		// printing
		// e.g. LyMultipageTableController
		if (controller instanceof LySetPrintModeInterface)
			((LySetPrintModeInterface)controller).setPrintMode(false);
	} // end method writePrintPage

	/**
	 * Adds the javascript form printing in the given page writer.
	 *
	 * @param pageWriter   The current printed page.
	 * @param errorMessage The error message displayed when the navigator does not handle page
	 *                     printing.
	 */
	protected void writePrintScript(LyWebPageWriter pageWriter, String errorMessage)
	{
		pageWriter.addBeginScript(LyWebConstants.JS_PAGE, true);
		pageWriter.addBeginScript(LyWebConstants.JS_FORM, true);

		StringBuffer endScript = new StringBuffer();

		endScript.append("<script type='text/javascript'>\n");
		endScript.append("//<![CDATA[\n");
		endScript.append("javascript:PrintPage(\"").append(errorMessage).append("\");\n");
		endScript.append("//]]>\n");
		endScript.append("</script>");

		pageWriter.addEndScript(endScript.toString(), false);
	}

	/**
	 * Writes the javascript code in the hidden frame to open the printing pop-up window.
	 *
	 * @param  environment Current Leonardi enviroment
	 * @param  viewManager Current view manager.
	 * @param  pw          The current writer to the ouput stream.
	 * @param  target      The target parameter for multi-sessions.
	 * @throws IOException
	 */
	protected void writeOpenPrintPopup(LyEnvironment environment, LyStrutsViewManager viewManager,
		Writer pw, String controllerId, String target)
		throws IOException
	{
		LyWebPageWriter     pageWriter = viewManager.getPageWriter();
		
		// get the URL parameters
		Map<String, String> parameters = new HashMap<String, String>();

		parameters.put(LyWebConstants.CONTROLLER, (controllerId == null ? "" : controllerId));
		parameters.put(LyWebConstants.PAGE, (pageWriter == null ? "" : pageWriter.getId()));
		parameters.put(LyWebConstants.SOURCE, "_print");

		if (target != null)
			parameters.put(LyWebConstants.TARGET_PARAM, target);
		
		// get the absolute URL to this action
		String url = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_PRINT,
				parameters, false);

		// write the HTML page
		pw.write("<html>\n");

		// add javascript files
		writeJavaScriptFile(environment, pw, LyWebConstants.JS_UTILS);
		writeJavaScriptFile(environment, pw, LyWebConstants.JS_ACTIONS);

		// add the javascript code
		String message = environment.getMessage("LY_PRINT_POPUP_BLOCKED");

		message = LyWebViewManager.encodeString(message);
		writeJavaScriptCode(pw, "javascript:OpenPrintPopup('" + url + "', \"" + message + "\");");

		// close the page
		pw.write("</html>\n");
	}
} // end class LyStrutsPrintAction
