/*
 * LyStrutsRefreshAction.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts.action;

import java.io.*;

import javax.servlet.http.*;

import leon.app.*;
import leon.control.*;
import leon.view.web.struts.*;

import org.apache.struts.action.*;


/**
 * Type : leon.view.web.struts.action.LyStrutsPageAction<br>
 * This action generates Ajax responses for graphical updates.<br>
 *
 * @version $Id: LyStrutsRefreshAction.java 36028 2012-06-26 13:08:06Z nhamon $
 * @author  Lyria S.A.
 */
public class LyStrutsRefreshAction extends LyStrutsLeonardiAction
{
	public static final String __VERSION =
		"$Id: LyStrutsRefreshAction.java 36028 2012-06-26 13:08:06Z nhamon $";
	
	/**
	 * Default constructor without parameters.<br>
	 * This constructor should be kept for Java reflection.
	 */
	public LyStrutsRefreshAction()
	{
	}

	/**
	 * Generic treatment for the command actions of the Leoanrdi User Interface.
	 *
	 * @param  mappings - The ActionMapping used to select this instance
	 * @param  bean     - The optional ActionForm bean for this request (if any)
	 * @param  request  - The HTTP request we are processing
	 * @param  response - The HTTP response we are creating
	 * @return null : an empty action forward while the action has written the response.
	 */
	public ActionForward execute(ActionMapping mappings, ActionForm bean,
		HttpServletRequest request, HttpServletResponse response)
		throws Exception
	{
		// get the session from the session bean
		LyStrutsSessionBean sessionBean = (LyStrutsSessionBean)bean;
		LySession		    session = sessionBean.getSession();
		LyStrutsViewManager viewManager = sessionBean.getViewManager();
		
		try
		{
			session.getSoftLock();
		
			// Check if session is registered, otherwise redirect to home page
			if (!checkSessionRegistered(sessionBean, request, response))
			{
				// In replay mode, force the session to be closed if needed.
				if (sessionBean.isReplayMode()
						&& !session.isRegistered()
						&& !session.isValid())
				{
					sessionBean.closeCurrentSession(request);
					return new ActionForward(LyStrutsConstants.PATH_ACTION_CLOSE + ".do");
				}
				
				return null;
			}
			
			// begin the request
			Writer	pw = beginRequest(request, response, viewManager);
			String	message = viewManager.getMessage();
			
			if (!sessionBean.isSessionInProgress())
				generateAdditionalScripts(session, request, response, pw);
	
			// Replay mode : reload top frame if needed
			if (sessionBean.isReplayMode())
			{
				int			 steps = 0;
				LyController controller = session.getLastViewController(steps);
	
				// Check if controller was created from a static action (in this
				// case, controller
				// should not be displayed here)
				while ((controller != null) && sessionBean.isStaticActionController(controller))
				{
					steps++;
					controller = session.getLastViewController(steps);
				}
	
				if (controller == null)
					controller = session.getController(0);
				
				LyController	oldController = sessionBean.getLastShownController();
				
				sessionBean.setLastShownController(controller);
				
				if (oldController != controller)
				{
					// If last shown controller has changed, reload home page.
					String			actionAbsoluteUrl = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_HOME, null, false);
					StringBuffer	buffer = new StringBuffer();
					
					buffer.append("<script type='text/javascript'>\n//<![CDATA[\n");
					buffer.append("var reloadFrame = getLeonardiTopFrame();");
					buffer.append("if ((reloadFrame != null) && ((typeof reloadFrame) != 'undefined')) ");
					buffer.append("reloadFrame.location.replace('" + actionAbsoluteUrl + "');");
					buffer.append("\n//]]>\n</script>\n");
					pw.append(buffer.toString());
				}
				
				if (message == null)
				{
					// If no message has to be displayed, hide previous one.
					StringBuffer	buffer = new StringBuffer();
					
					buffer.append("<script type='text/javascript'>\n//<![CDATA[\n");
					buffer.append("HideMessageWindow(null, null);");
					buffer.append("\n//]]>\n</script>\n");
					pw.append(buffer.toString());
				}
			}
			
			String	recurse = getParameter(request, "_recurse");
	
			if ("true".equals(recurse))
			{
				int		refresh = viewManager.getRefreshRate();
	
				pw.write("<script type='text/javascript'>\n//<![CDATA[\n");
				pw.write("setTimeout(\"AutoRefresh(true)\", " + (refresh * 1000) + ");");
				pw.write("\n//]]>\n</script>\n");
			}
	
			// end request
			endRequest(request, response, viewManager, pw);
		}
		finally
		{
			session.releaseSoftLock();
		}
		return null;
	} // end method execute
} // end class LyStrutsPageAction
