/*
 * LyStrutsSelectionEventAction.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts.action;

import java.util.*;

import javax.servlet.http.*;
import leon.misc.*;
import leon.view.event.*;
import leon.view.web.*;

/**
 * Type : leon.view.web.struts.action.LyStrutsSelectionEventAction<br>
 * This action handles selection events from the User Interface.
 *
 * @version $Id: LyStrutsSelectionEventAction.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria S.A.
 */
public class LyStrutsSelectionEventAction extends LyStrutsEventAction
{
	public static final String __VERSION =
		"$Id: LyStrutsSelectionEventAction.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Default constructor without parameters.<br>
	 * This constructor should be kept for Java reflection.
	 */
	public LyStrutsSelectionEventAction()
	{
	}

	/**
	 * This method creates a Logical Leonardi event with a User Interface physical event (the HTTP
	 * request).<br>
	 * This method creates a Selection Demand that will be sent back to the related controller.
	 */
	public LyDemand getDemand(HttpServletRequest request, LyEnvironment environment)
	{
		// prepare a selection demand
		LySelectionDemand selectionDemand = null;

		// get the source component
		String			  source = getParameter(request, LyWebConstants.SOURCE);

		source = ((source == null) ? "" : source.intern());

		// get the extend parameter
		String extend = getParameter(request, LyWebConstants.EXTEND);
		String value = getParameter(request, LyWebConstants.VALUE);

		// Use the same separator as used in the getSelectionIds() function in LyUtilsScript.js
		Vector<String>	selectionIds = new Vector<String>();
		String[]		ids = LyString.stringToList(value, "__|__");
		
		if ((ids != null) && (ids.length > 0))
		{
			for (int i = 0; i < ids.length; i++)
			{
				if ((ids[i] != null) && (ids[i].length() > 0))
					selectionIds.addElement(ids[i]);
			}
		}
		
		int		i, nSelections = selectionIds.size();

		// create the demand depending on the source component
		if (LyWebConstants.SELECT_ITEM == source || LyWebConstants.UNSELECT_ITEM.equals(source))
		{
			if (LyWebConstants.SELECT_ITEM == source)
				selectionDemand = new LySelectionDemand(LySelectionDemand.TYPE_SELECT, null);
			else
				selectionDemand = new LySelectionDemand(LySelectionDemand.TYPE_UNSELECT, null);

			// set the extend property
			selectionDemand.setExtend("true".equals(extend));

			// set the selected items
			for (i = 0; i < nSelections; i++)
				selectionDemand.addSelectionId(selectionIds.elementAt(i));
		}
		else if (LyWebConstants.SELECT_ALL.equals(source))
			selectionDemand = new LySelectionDemand(LySelectionDemand.TYPE_SELECT_ALL, null);
		else if (LyWebConstants.UNSELECT_ALL.equals(source))
			selectionDemand = new LySelectionDemand(LySelectionDemand.TYPE_UNSELECT_ALL, null);

		return selectionDemand;
	} // end method getDemand

} // end class LyStrutsSelectionEventAction
