/*
 * LyStrutsTargetFrameAction.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts.action;

import java.io.*;

import javax.servlet.http.*;

import leon.app.*;
import leon.control.*;
import leon.misc.*;
import leon.view.web.*;
import leon.view.web.struts.*;

import org.apache.struts.action.*;


/**
 * Type : leon.view.web.struts.action.LyStrutsTargetFrameAction<br>
 * This action manages the building of the target frame of the Leonardi frameset.
 *
 * @version $Id: LyStrutsTargetFrameAction.java 33992 2012-02-01 16:54:01Z nhamon $
 * @author  Lyria S.A.
 */
public class LyStrutsTargetFrameAction extends LyStrutsLeonardiAction
{
	public static final String __VERSION =
		"$Id: LyStrutsTargetFrameAction.java 33992 2012-02-01 16:54:01Z nhamon $";

	/**
	 * Default constructor without parameters.<br>
	 * This constructor should be kept for Java reflection.
	 */
	public LyStrutsTargetFrameAction()
	{
	}

	/**
	 * This action builds the target frame of the Leonardi frameset.<br>
	 * It looks for the last view controller in the current session and writes the page.<br>
	 * If none are found, the LY_DEFAULT_PAGE page is displayed or the index page.
	 *
	 * @param  mappings - The ActionMapping used to select this instance
	 * @param  bean     - The optional ActionForm bean for this request (if any)
	 * @param  request  - The HTTP request we are processing
	 * @param  response - The HTTP response we are creating
	 * @return a forward to the default page or null if a controller has written itself into the
	 *         response.
	 */
	public ActionForward execute(ActionMapping mappings, ActionForm bean,
		HttpServletRequest request, HttpServletResponse response)
		throws Exception
	{
		// Get the session from the session bean
		LyStrutsSessionBean sessionBean = (LyStrutsSessionBean)bean;
		LySession		    session = sessionBean.getSession();
		LyEnvironment	    environment = sessionBean.getEnvironment();
		LyStrutsViewManager viewManager = sessionBean.getViewManager();
		
		// Check if session is registered, otherwise redirect to home page
		if (!checkSessionRegistered(sessionBean, request, response))
			return null;
		
		try
		{
			session.getSoftLock();
			// unset current session
			sessionBean.beginRequest();
			
			// begin the request
			Writer	pw = beginRequest(request, response, viewManager);
	
			// No caching for this page.
			setNoCacheHeaders(response, 0);
			
			// Register the Leonardi session on the HTTP session
			request.setAttribute(LyWebConstants.RA_LY_SESSION, session);
			request.getSession().setAttribute(LyWebConstants.RA_LY_SESSION, session);
			
			// get the last view controller
			int			 steps = 0;
			LyController controller = session.getLastViewController(steps);
	
			// Check if controller was created from a static action (in this
			// case, controller
			// should not be displayed here)
			while ((controller != null) && sessionBean.isStaticActionController(controller))
			{
				steps++;
				controller = session.getLastViewController(steps);
			}
	
			if (controller == null)
				controller = session.getController(0);
	
			// get the area controller
			LyAreaController areaController = null;
	
			if (controller != null)
			{
				if (controller instanceof LyViewController)
					areaController = ((LyViewController)controller).getAreaController();
				else
					areaController = (LyAreaController)controller;
	
				controller.showView();
			}
	
			// get the page writer
			LyWebPageWriter pageWriter = viewManager.getPageWriter();
	
			// the frame is the target frame
			String		    frame = LyWebConstants.TARGET_FRAME;
	
			// From this point (till we write it), the pagewriter must not change
			// !!!
			if (pageWriter != null)
			{
				synchronized (pageWriter)
				{
					// check the reload state of the page writer
					checkReloadState(pageWriter, areaController, response);
	
					// We prepare the current page
					preparePage(pageWriter, frame, null);
	
					// write the page
					writePage(request, session, environment, viewManager, pageWriter, pw, frame, null);
				}
			}

			viewManager.clearReloadFrame(LyWebConstants.TARGET_FRAME);
			
			if (!viewManager.isPushModeActivated())
			{
				viewManager.clearReloadFrames();
				viewManager.clearAdditionalRefreshScripts();
				viewManager.setMessage(null);
			}
			
			// unset current session
			sessionBean.endRequest();
	
			// end request
			endRequest(request, response, viewManager, pw);
		}
		finally
		{
			session.releaseSoftLock();
		}
		// Do not return any page while the page is written directly in the
		// response
		return null;
	} // end method execute
} // end class LyStrutsTargetFrameAction
