/*
 * LyStrutsUpFrameAction.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */
package leon.view.web.struts.action;

import org.apache.struts.action.*;
import javax.servlet.http.*;
import leon.misc.*;
import leon.view.web.struts.*;


/**
 * Type : leon.view.web.struts.action.LyStrutsUpFrameAction<br>
 * The only purpose of this action is to manage the up part of the Leonardi frameset.
 *
 * @version $Id: LyStrutsUpFrameAction.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria S.A.
 */
public class LyStrutsUpFrameAction extends LyStrutsLeonardiAction
{
	public static final String __VERSION =
		"$Id: LyStrutsUpFrameAction.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Default constructor without parameters.<br>
	 * This constructor should be kept for Java reflection.
	 */
	public LyStrutsUpFrameAction()
	{
	}

	/**
	 * This action makes a forward to the LY_UP_PAGE page.
	 *
	 * @param  mappings - The ActionMapping used to select this instance
	 * @param  bean     - The optional ActionForm bean for this request (if any)
	 * @param  request  - The HTTP request we are processing
	 * @param  response - The HTTP response we are creating
	 * @return null : an empty action forward while the action has written the response.
	 */
	public ActionForward execute(ActionMapping mappings, ActionForm bean,
		HttpServletRequest request, HttpServletResponse response)
		throws Exception
	{
		// Get the session from the session bean
		LyStrutsSessionBean sessionBean = (LyStrutsSessionBean)bean;
		LyEnvironment	    environment = sessionBean.getEnvironment();
		LyStrutsViewManager viewManager = sessionBean.getViewManager();
		
		// Check if session is registered, otherwise redirect to home page
		if (!checkSessionRegistered(sessionBean, request, response))
			return null;

		// Get the LY_UP_PAGE ressource
		String upPage = environment.getUrl("LY_UP_PAGE");

		// get the path relative to the context
		upPage = getContextReativePath(request, upPage);

		// set no cache
		setNoCacheHeaders(response, 0);

		// store usefull attributes
		setRequestAttributes(request, sessionBean.getSession());

		// add url for the about tool
		String aboutUrl = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_ABOUT,
				null, true);

		request.setAttribute("leonardiAboutUrl", aboutUrl);

		// add url for the exit action
		String exitUrl = viewManager.getActionAbsoluteUrl(LyStrutsConstants.PATH_ACTION_EXIT, null,
				true);

		request.setAttribute("leonardiExitUrl", exitUrl);

		return new ActionForward(upPage);
	} // end method execute
} // end class LyStrutsUpFrameAction
