/*
 * LySwingContainer.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing;

import java.awt.*;
import leon.view.*;
import leon.view.event.*;
import leon.view.graphic.*;


/**
 * This class implements the common behavior for container objects for swing viewer.
 *
 * @version $Id: LySwingContainer.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 * @see     LyContainer
 */
public abstract class LySwingContainer extends LySwingPanel
	implements LyGraphicalContainer,
		LyActionListener
{
	public static final String __VERSION = "$Id: LySwingContainer.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Class constructor of a container object.
	 *
	 * @param container a generic container
	 */
	public LySwingContainer(LyContainer container)
	{
		super(container);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.setinterface.LyFreeInterface#free()
	 */
	public void free()
	{
		_actionTransmitter = null;

		super.free();
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.event.LyActionListener#actionPerformed(leon.view.event.LyActionEvent)
	 */
	public void actionPerformed(LyActionEvent actionEvent)
	{
		// notify other listeners
		if (_actionTransmitter != null)
			_actionTransmitter.processActionEvent(actionEvent);
	}

	/**
	 * Updates the minimum size occupied by this component.
	 *
	 * @param child child component in this container. This parameter is not used in this
	 *              implementation.
	 * @param d     minimum size to use
	 */
	public void updateMinimumSize(LyComponent child, Dimension d)
	{
		// Nothing to do at this level : overriden in subclasses when something to do.
		if ((_viewComponent != null) && (_viewComponent.getParent() != null))
		{
			if (_viewComponent.getParent().getInternalView() instanceof LySwingContainer)
				((LySwingContainer)_viewComponent.getParent().getInternalView()).updateMinimumSize(
					_viewComponent, d);
			else if (_viewComponent.getParent().getInternalView() instanceof LySwingDialog)
				((LySwingDialog)_viewComponent.getParent().getInternalView()).updateMinimumSize(
					_viewComponent, d);
			else if (_viewComponent.getParent().getInternalView() instanceof LySwingDialog)
				((LySwingDialog)_viewComponent.getParent().getInternalView()).updateMinimumSize(
					_viewComponent, d);
		}
	}
} // end class LySwingContainer
