/*
 * LySwingFieldItem.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing;

import leon.view.*;
import leon.view.event.*;
import leon.view.graphic.*;


/**
 * This class implements the common behavior for field items objects for swing viewer.
 *
 * @version $Id: LySwingFieldItem.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 * @see     LyFieldItem
 */
public abstract class LySwingFieldItem extends LySwingObject
	implements LyGraphicalFieldItem,
		LyEventTransmitter
{
	public static final String    __VERSION = "$Id: LySwingFieldItem.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Provides notification events on component's actions. */
	protected LyActionTransmitter _actionTransmitter;

	/**
	 * Class constructor of an object for the Swing target.
	 *
	 * @param fieldItem a component defined for every type of view targets
	 */
	protected LySwingFieldItem(LyFieldItem fieldItem)
	{
		super(fieldItem);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.event.LyEventTransmitter#getActionTransmitter()
	 */
	public LyActionTransmitter getActionTransmitter()
	{
		if (_actionTransmitter == null)
			_actionTransmitter = new LySimpleActionTransmitter();

		return _actionTransmitter;
	}

	/**
	 * This field item doesn't handle selection events, so it has no selection transmitter.
	 *
	 * @return <code>null</code>
	 * @see    leon.view.event.LyEventTransmitter#getSelectionTransmitter()
	 */
	public LySelectionTransmitter getSelectionTransmitter()
	{
		return null;
	}
} // end class LySwingFieldItem
