/*
 * LySwingFooter.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing;

import leon.view.*;
import leon.view.getinterface.*;
import leon.view.graphic.*;
import leon.view.setinterface.*;


/**
 * This class implements the Footer object (as described in the dtd) for swing viewer.
 *
 * @version $Id: LySwingFooter.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 * @see     LyFooter
 */
public class LySwingFooter extends LySwingObject
	implements LySetLabelInterface,
		LyGetLabelInterface,
		LyGraphicalFooter
{
	public static final String __VERSION =
		"$Id: LySwingFooter.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** The view independent footer description. */
	protected LyFooter		   _footer;

	/** The index of this footer. */
	protected int			   _index;

	/**
	 * Class constructor of a Footer object for the Swing target.
	 *
	 * @param footer a FOOTER component defined for every type of view targets
	 */
	public LySwingFooter(LyFooter footer)
	{
		super(footer);
		_footer = footer;
		_index = getIndex();
	}

	/**
	 * Finds the index of this footer. This corresponds to the column in the footer list.
	 *
	 * @return the index of this footer or -1 if it cannot be found
	 */
	protected int getIndex()
	{
		LyFooters footers = (LyFooters)_footer.getParent();
		LyFooter  footer;
		int		  i, n = footers.getFooterCount();

		for (i = 0; i < n; i++)
		{
			footer = footers.getFooter(i);
			if (footer == _footer)
				return i;
		}

		return -1;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.setinterface.LySetLabelInterface#setLabel(java.lang.String)
	 */
	public void setLabel(String label)
	{
		LyTable table = (LyTable)_footer.getParent().getParent();

		((LySwingTable)table.getInternalView()).getTableFooter().setFooter(_index, label);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.getinterface.LyGetLabelInterface#getLabel()
	 */
	public String getLabel()
	{
		//Flandres/FFT00120
		String label = _footer.getLabel();

		label = translate(label);
		return label;
	}

	/**
	 * Gets the wrapping capability of this footer.
	 *
	 * @return true if this footer has wrapping capability
	 */
	public boolean isWrap()
	{
		return _footer.isWrap();
	}

	/**
	 * Gets the identifier of this footer.
	 *
	 * @return the id of this footer
	 */
	public String getId()
	{
		return _footer.getId();
	}

} // end class LySwingFooter
