/*
 * LySwingGauge.java Copyright (c) 2000-2009, Lyria-W4. All rights reserved.
 */

package leon.view.win.swing;

import java.awt.*;
import javax.swing.*;
import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;

/**
 * This class implements the Gauge object (as described in the dtd) for swing
 * viewer.
 * 
 * <p>
 * A gauge is defined with minimum and maximum values as well as a current
 * value.
 * </p>
 * 
 * @version $Id: LySwingGauge.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author Lyria
 * @see LyColor
 */
public class LySwingGauge extends LySwingObject implements LySwingComponent,
        LyGraphicalGauge
{
    public static final String __VERSION = "$Id: LySwingGauge.java 28422 2010-11-03 12:03:38Z ehelmste $";

    /** The view independent gauge description. */
    protected LyGauge          _gauge;

    /** The swing component used to display this progress bar. */
    protected Component        _comp;

    /**
     * Class constructor of a Gauge object for the Swing target.
     * 
     * @param gauge
     *            a GAUGE component defined for every type of view targets. This
     *            gauge is initialized specificaly for the Swing view.
     */
    public LySwingGauge(LyGauge gauge)
    {
        super(gauge);
        _gauge = gauge;

        initialize();
        setContent();
    }

    /**
     * Initializes this progress bar. Builds the graphic component that display
     * this object.
     */
    public void initialize()
    {
        Color label_color = getColor("LY_GAUGE_LABEL_PROGRESS_COLOR");
        if (label_color == null)
            label_color = getColor("LY_GAUGE_PROGRESS_COLOR");
        if (label_color != null)
            UIManager.put("ProgressBar.selectionBackground", label_color);

        label_color = getColor("LY_GAUGE_LABEL_BACKGROUND_COLOR");
        if (label_color != null)
            UIManager.put("ProgressBar.selectionForeground", label_color);

        _comp = createGauge(getEnvironment());

        Font font = getFont("LY_GAUGE_FONT");

        if (font != null)
            _comp.setFont(font);
    }

    /**
     * Sets the initial values.
     */
    public void setContent()
    {
        setInitialValues();
    }

    /**
     * Creates the gauge swing component with the default properties.
     * 
     * @param environment
     *            the current context. This is not used in this implementation.
     * @return the new progress bar
     */
    public Component createGauge(LyEnvironment environment)
    {
        JProgressBar comp;

        comp = new JProgressBar();

        comp.setStringPainted(true);

        Color c;

        c = getColor("LY_GAUGE_PROGRESS_COLOR");

        if (c != null)
            comp.setForeground(c);

        c = getColor("LY_GAUGE_BACKGROUND_COLOR");

        if (c == null)
            c = getColor("LY_DEFAULT_BACKGROUND");

        if (c != null)
            comp.setBackground(c);
        return comp;
    }

    /**
     * Initializes this gauge: sets the minimum, maximum and current values,
     * then sets the gauge label, if any.
     */
    public void setInitialValues()
    {
        ((JProgressBar)_comp).setMinimum(_gauge.getNMin());
        ((JProgressBar)_comp).setMaximum(_gauge.getNMax());
        ((JProgressBar)_comp).setValue(_gauge.getNValue());

        if ((_gauge.getLabel() != null) && (_gauge.getLabel().length() > 0))
            ((JProgressBar)_comp).setString(_gauge.getLabel());
    }

    /**
     * Sets the current component value.
     * 
     * @see LyGraphicalGauge#setNValue(int)
     */
    public void setNValue(int nValue)
    {
        if (_comp == null)
            return;
        
        ((JProgressBar)_comp).setValue(nValue);
        if (_gauge.getLabel() != null)
            setLabel(_gauge.getLabel());
    }

    /**
     * Sets the current component label.
     * 
     * @see leon.view.setinterface.LySetLabelInterface#setLabel(String)
     */
    public void setLabel(String label)
    {
        if (_comp == null)
            return;
        
        if ((label != null) && (label.length() > 0) && (_comp != null))
            ((JProgressBar)_comp).setString(label);
    }

    /**
     * (non-Javadoc).
     * 
     * @see leon.view.win.swing.LySwingComponent#getComponent()
     */
    public Component getComponent()
    {
        return _comp;
    }

} // end class LySwingGauge
