/*
 * LySwingHeader.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing;

import java.awt.*;
import javax.swing.*;
import leon.view.*;
import leon.view.getinterface.*;
import leon.view.graphic.*;


/**
 * This class implements the Header object (as described in the dtd) for swing viewer.
 *
 * @version $Id: LySwingHeader.java 32548 2011-09-29 12:59:26Z nhamon $
 * @author  Lyria
 * @see     LyHeader
 */
public class LySwingHeader extends LySwingObject
	implements LyGetLabelInterface,
		LyGetIdInterface,
		LyGraphicalHeader
{
	public static final String __VERSION =
		"$Id: LySwingHeader.java 32548 2011-09-29 12:59:26Z nhamon $";

	/** The view independent header description. */
	protected LyHeader		   _header;

	/** The sortable header font. */
	protected Font			   _sortableHeaderFont;

	/**
	 * Class constructor of a Header object for the Swing target.
	 *
	 * @param header a HEADER component defined for every type of view targets. This header is
	 *               initialized specificaly for the Swing view.
	 */
	public LySwingHeader(LyHeader header)
	{
		super(header);
		_header = header;

		initialize();
		setContent();
	}

	/**
	 * Initializes the {@link #_sortableHeaderFont} to null.
	 */
	public void initialize()
	{
		_sortableHeaderFont = null;
	}

	/**
	 * Nothing is done here, as a header has no content.
	 */
	public void setContent()
	{
	}

	/* Implementation of modification set interfaces */

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.graphic.LyGraphicalHeader#setIncreasing(boolean)
	 */
	public void setIncreasing(boolean increasing)
	{
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.graphic.LyGraphicalHeader#setFixed(boolean)
	 */
	public void setFixed(boolean fixed)
	{
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.graphic.LyGraphicalHeader#setSort(int)
	 */
	public void setSort(int sort)
	{
	}

	/**
	 * (Modification) Accessor.
	 *
	 * @see LyGraphicalHeader#setBorderColor(String)
	 */
	public void setBorderColor(String borderColor)
	{
	}

	/**
	 * (Modification) Accessor.
	 *
	 * @see LyGraphicalHeader#setBorderWidth(int)
	 */
	public void setBorderWidth(int borderWidth)
	{
	}

	/**
	 * Gets the header tooltip.
	 *
	 * @return the label to display
	 */
	public String getTooltip()
	{
		String label = _header.getTooltip();

		label = translate(label);
		return label;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.getinterface.LyGetLabelInterface#getLabel()
	 */
	public String getLabel()
	{
		String label = _header.getLabel();

		label = translate(label);
		return label;
	}

	/**
	 * Return the image associated to the header.
	 * 
	 * @return the image associated to the header or null if no image is associated to the header.
	 */
	public Image getImage()
	{
		String	imageKey = _header.getImage();
		
		return getImage(imageKey);
	}
	
	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.getinterface.LyGetIdInterface#getId()
	 */
	public String getId()
	{
		return _header.getId();
	}

	/**
	 * Gets the header sort level.
	 *
	 * @return the sort level of this header
	 * @see    #setSort(int)
	 */
	public int getSort()
	{
		return _header.getSort();
	}

	/**
	 * Gets the increasing property of this header.
	 *
	 * @return true if increasing is set for the header
	 */
	public boolean isIncreasing()
	{
		return _header.isIncreasing();
	}

	/**
	 * Gets the sortable property of this header.
	 *
	 * @return true if the header is sortable
	 */
	public boolean isSortable()
	{
		return _header.isSortable();
	}

	/**
	 * Gets the resizable property of this header.
	 *
	 * @return true if the header is resizable
	 */
	public boolean isResizable()
	{
		return _header.isResizable();
	}

	/**
	 * Gets the fixed property of this header.
	 *
	 * @return true if the header is fixed
	 */
	public boolean isFixed()
	{
		return _header.isFixed();
	}

	/**
	 * Gets the wrapped property of this header.
	 *
	 * @return true if the header is wrapped
	 */
	public boolean isWrap()
	{
		return _header.isWrap();
	}

	/**
	 * Gets the align property of this header.
	 *
	 * @return {@link LyHeader#Align.CENTER}, {@link LyHeader#Align.LEFT} or {@link
	 *         LyHeader#Align.RIGHT} depending on the align property of the header
	 */
	public short getAlign()
	{
		if (_header.getAlign() == LyHeader.Align.CENTER)
			return SwingConstants.CENTER;
		else if (_header.getAlign() == LyHeader.Align.RIGHT)
			return SwingConstants.RIGHT;
		else
			return SwingConstants.LEFT;
	}

	/**
	 * Gets the sortable header font.
	 *
	 * @return the sortable header font, or null if unspecified
	 */
	public Font getSortableHeaderFont()
	{
		if (_sortableHeaderFont != null)
			return _sortableHeaderFont;

		_sortableHeaderFont = getFont("LY_TABLE_LABEL_SORTABLE");

		return _sortableHeaderFont;
	}

	/**
	 * Sets attribute columnWidth to the given value.
	 * @param columnWidth New value of columnWidth.
	 */
	public void setColumnWidth(double columnWidth)
	{
	}
} // end class LySwingHeader
