/*
 * LySwingLayout.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing;

import leon.view.*;
import leon.view.graphic.*;
import leon.view.win.swing.map.*;


/**
 * This class implements the layout object (as described in the dtd) for swing viewer.
 *
 * <p>A layout manager allows to place the nodes of a map according to a predefined layout. In most
 * cases, the coordinates of the nodes are not used.</p>
 *
 * @version $Id: LySwingLayout.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 * @see     LyLayout
 */
public class LySwingLayout extends LySwingObject
	implements LyGraphicalLayout
{
	public static final String __VERSION =
		"$Id: LySwingLayout.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** The view independent layout description. */
	protected LyLayout		   _layout;

	/** Reference to the layout manager. */
	protected LyLayoutManager  _layoutManager;

	/**
	 * Class constructor of a Layout object for the Swing target.
	 *
	 * @param layout a LAYOUT component defined for every type of view targets. The associated
	 *               layout manager is initialized at <code>null</code>.
	 */
	public LySwingLayout(LyLayout layout)
	{
		super(layout);
		_layout = layout;
		_layoutManager = null;
	}

	/**
	 * Creates a factory manager to lay out the elements of this layout depending on the layout
	 * style. The specified map is not used in this implementation.
	 *
	 * @param  map a map object, not used in this implementation
	 * @return the layout manager to use for the current layout
	 */
	public LyLayoutManager layoutFactory(LySwingMap map)
	{
		if (_layout.getStyle() == LyLayout.Style.MATRIX)
			_layoutManager = new LyMatrixLayout(_layout.getHAlign(), _layout.getVAlign(),
					_layout.getDeltaNode(), _layout.getDeltaBorder());
		else if (_layout.getStyle() == LyLayout.Style.ELLIPSE)
			_layoutManager = new LyEllipseLayout(_layout.getDeltaBorder(), false);
		else if (_layout.getStyle() == LyLayout.Style.CIRCLE)
			_layoutManager = new LyEllipseLayout(_layout.getDeltaBorder(), true);
		else if (_layout.getStyle() == LyLayout.Style.LINEAR)
			_layoutManager = new LyLinearLayout(_layout.getDeltaNode(), _layout.getDeltaBorder(),
					_layout.getFill(), _layout.getHAlign(), _layout.getVAlign());
		else if (_layout.getStyle() == LyLayout.Style.GEOGRAPHIC)
			_layoutManager = new LyGeographicLayout();
		else if (_layout.getStyle() == LyLayout.Style.COORDINATES)
			_layoutManager = new LyCoordinatesLayout();
		else if (_layout.getStyle() == LyLayout.Style.TREEMAP)
			_layoutManager = new LyTreemapLayout(_layout.getBorderSizes(),
					_layout.getSelectBorderSize(), _layout.getLabelStyles());

		return _layoutManager;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.graphic.LyGraphicalLayout#setBorderSizes(java.lang.String)
	 */
	public void setBorderSizes(String borderSizes)
	{
		if (_layout.getStyle() != LyLayout.Style.TREEMAP)
			return;

		((LyTreemapLayout)_layoutManager).setBorderSizes(borderSizes);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.graphic.LyGraphicalLayout#setLabelStyles(java.lang.String)
	 */
	public void setLabelStyles(String labelStyles)
	{
		if (_layout.getStyle() != LyLayout.Style.TREEMAP)
			return;

		((LyTreemapLayout)_layoutManager).setLabelStyles(labelStyles);
	}

} // end class LySwingLayout
