/*
 * LySwingMapContent.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing;

import leon.view.*;
import leon.view.graphic.*;


/**
 * This class implements the Map content object (as described in the dtd) for swing viewer.
 *
 * <p>A map content allows to group together in a same group various elements that can be placed on
 * a map (nodes, links, images, regions).</p>
 *
 * @version $Id: LySwingMapContent.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 * @see     leon.view.LyMapContent
 */
public class LySwingMapContent extends LySwingObject
	implements LyGraphicalMapContent
{
	public static final String __VERSION =
		"$Id: LySwingMapContent.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** The view independent map content description. */
	protected LyMapContent     _mapContent;

	/**
	 * Class constructor of a Map content object for the Swing target.
	 *
	 * @param mapContent a MAP_CONTENT component defined for every type of view targets
	 */
	public LySwingMapContent(LyMapContent mapContent)
	{
		super(mapContent);
		_mapContent = mapContent;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.graphic.LyGraphicalMapContent#addComponent(leon.view.LyComponent, int)
	 */
	public void addComponent(LyComponent component, int pos)
	{
		LyMap map = (LyMap)_mapContent.getParent();

		if (map != null)
		{
			LyGraphicalMap commonMap = (LyGraphicalMap)map.getInternalView();

			commonMap.addComponent(component, pos);
		}
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.graphic.LyGraphicalMapContent#removeComponent(leon.view.LyComponent)
	 */
	public void removeComponent(LyComponent component)
	{
		LyMap map = (LyMap)_mapContent.getParent();

		if (map != null)
		{
			LyGraphicalMap commonMap = (LyGraphicalMap)map.getInternalView();

			commonMap.removeComponent(component);
		}
	}

} // end class LySwingMapContent
