/*
 * LySwingObject.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing;

import java.awt.*;
import leon.view.*;
import leon.view.graphic.*;


/**
 * The LySwingObject class implements root object for all swing components.
 */
public abstract class LySwingObject extends LyGraphicalObject
	implements LySwingComponent
{
	public static final String __VERSION = "$Id: LySwingObject.java 32287 2011-09-12 09:45:53Z jlsaouli $";

	/**
	 * Returns a default frame.
	 *
	 * @return The default frame.
	 */
	public static Frame getDefaultFrame()
	{
		return LySwingViewManager.getDefaultFrame();
	}

	/**
	 * Constructs abstract graphical component.
	 *
	 * @param viewComponent current view component
	 */
	public LySwingObject(LyComponent viewComponent)
	{
		super(viewComponent);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.win.swing.LySwingComponent#getComponent()
	 */
	public Component getComponent()
	{
		return null;
	}

	/**
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalObject#getBounds(leon.view.LyComponent)
	 */
	public int[] getBounds(LyComponent parent)
	{
		Component	parentcomponent = null;
		Component	component = getComponent();
		
		if (component == null || ! component.isShowing())
			return null;

		if (parent != null)
		{
			LySwingComponent	parentView = (LySwingComponent)parent.getInternalView(true);
			
			parentcomponent = parentView.getComponent();
		}
		
		Point		parentPosition = (parentcomponent == null ? new Point(0, 0) : parentcomponent.getLocationOnScreen());
		Point		position = component.getLocationOnScreen();
		Dimension	dimension = component.getSize();
		
		return new int[] {position.x - parentPosition.x,
				position.y - parentPosition.y,
				dimension.width,
				dimension.height};
	}
	
	/**
	 * (non-Javadoc)
	 * @see leon.view.graphic.LyGraphicalObject#getChildrenBounds(leon.view.LyComponent)
	 */
	public int[] getChildrenBounds(LyComponent children)
	{
		return children.getBounds(_viewComponent);
	}

	/**
	 * Retrieves a Color from the current environment.
	 *
	 * @param  key The resource name (key) of the Color.
	 * @return A reference to the requested Color, or null if it could not be found.
	 * @see    leon.misc.LyEnvironment#getColor(String)
	 */
	protected final Color getColor(String key)
	{
		if (getEnvironment() == null)
			return null;
		return (Color)getEnvironment().getColor(key);
	}

	/**
	 * Retrieves a Font from the current environment.
	 *
	 * @param  key The resource name (key) of the font.
	 * @return A reference to the requested Font, or a default font if it could not be found.
	 * @see    leon.misc.LyEnvironment#getFont(String)
	 */
	protected final Font getFont(String key)
	{
		if (getEnvironment() == null)
			return null;
		return (Font)getEnvironment().getFont(key);
	}

	/**
	 * Retrieves an image from the current environment.
	 *
	 * @param  key The resource name (key) of the image.
	 * @return A reference to the image object, or null if it could not be found.
	 * @see    leon.misc.LyEnvironment#getImage(String)
	 */
	protected final Image getImage(String key)
	{
		if (getEnvironment() == null)
			return null;
		return (Image)getEnvironment().getImage(key);
	}
} // end class LySwingObject
