/*
 * LySwingPanel.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing;

import javax.swing.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.setinterface.*;


/**
 * This class implements the common behavior for panel objects (as described in the dtd) for swing
 * viewer.
 *
 * @version $Id: LySwingPanel.java 28974 2010-12-21 13:52:49Z jlsaouli $
 * @author  Lyria
 * @see     LyPanel
 */
public abstract class LySwingPanel extends LySwingFieldItem
	implements LyGraphicalPanel,
				LySetPopupMenuInterface
{
	public static final String __VERSION = "$Id: LySwingPanel.java 28974 2010-12-21 13:52:49Z jlsaouli $";

	/** Keeps last popup menu to remove it. This saves memory. */
	protected JPopupMenu			 _popupMenu;

	/**
	 * Class constructor of a Swing panel.
	 *
	 * @param panel a view independant panel
	 */
	public LySwingPanel(LyPanel panel)
	{
		super(panel);
	}

	/**
	 * Adds the specified popup menu to the tree component and shows it.
	 *
	 * @param popupMenu the popup menu to show
	 * @see   LySetPopupMenuInterface#setPopupMenu(LyPopupMenu)
	 */
	public void setPopupMenu(LyPopupMenu popupMenu)
	{
        if (_popupMenu != null)
        {
            JComponent  parent = getPopupMenuParent();
            
            _popupMenu.setVisible(false);
            parent.remove(_popupMenu);
            _popupMenu = null;
        }
        
		if (popupMenu == null)
			return;

		LySwingPopupMenu internal = (LySwingPopupMenu)popupMenu.getInternalView();
		JPopupMenu		 popup = (JPopupMenu)internal.getMenu();
		JComponent		 parent = getPopupMenuParent();

		_popupMenu = popup;

		// Show popup
		parent.add(popup);

		/* TESTER SI LE MENU RENTRE DANS LA FENETRE */
		popup.show(parent, popupMenu.getX(), popupMenu.getY());
	}
	
	/**
	 * Returns the component used to display popup menus.
	 * 
	 * @return the parent component of popup menus.
	 */
	protected JComponent getPopupMenuParent()
	{
		return null;
	}
}