/*
 * LySwingSplashScreen.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing;

import java.awt.*;
import java.awt.geom.Rectangle2D;

import javax.swing.*;

import leon.misc.*;
import leon.view.setinterface.*;


/**
 * This class implements a splash screen for swing viewers.
 *
 * @version $Id: LySwingSplashScreen.java 31007 2011-05-13 14:59:20Z gtachon $
 * @author  Lyria
 */
public class LySwingSplashScreen extends JWindow
	implements LyFreeInterface
{
	private static final long  serialVersionUID = -8769199825268430165L;

	public static final String __VERSION =
		"$Id: LySwingSplashScreen.java 31007 2011-05-13 14:59:20Z gtachon $";

	/** Border size (in pixels) of the splash screen. */
	public static int    	   BORDERSIZE = 0;

	/** Color of the border if the size is visible (> 0). */
	public static Color  	   BORDERCOLOR = Color.BLACK;

	/** The image to show on the splash screen. */
	protected Image			   _splashImage;

	/** The splash image width. */
	protected int			   _imageWidth;

	/** The splash image height. */
	protected int			   _imageHeight;
	
	/** Font. */
	protected Font				_font;
	
	/** Font color */
	protected Color				_fontColor;
	
	/** Version string.*/
	protected String			_version;
	
	/** Copyright string */
	protected String			_copyright;

	/**
	 * Class constructor of a Swing splash screen. The image is retrieved and automatically
	 * displayed.
	 *
	 * @param environment the current environment
	 */
	public LySwingSplashScreen(LyEnvironment environment)
	{
		super(LySwingViewManager.getDefaultFrame());

		String imageName = environment.getEnv("LY_SPLASH_SCREEN");

		if (imageName == null)
			imageName = "LY_SPLASH_SCREEN";

		_splashImage = (Image)environment.getImage(imageName, null, false);

		if (_splashImage == null)
			return;
		
		_font = (Font)environment.getFont("LY_SPLASHSCREEN_FONT");
		_fontColor = (Color)environment.getColor("LY_SPLASHSCREEN_VERSION");
		_version = environment.getString("LY_PRODUCT_VERSION");
		_copyright = environment.getString("LY_COPYRIGHT");

		_imageWidth = _splashImage.getWidth(this);
		_imageHeight = _splashImage.getHeight(this);

		int[]     dscreen = environment.getScreenSize(this);
		Dimension screenSize = new Dimension(dscreen[0], dscreen[1]);
		int		  w = _imageWidth + (BORDERSIZE * 2);
		int		  h = _imageHeight + (BORDERSIZE * 2);
		// FFT00435/Flandres
		int		  x = (screenSize.width - w) / 2 + dscreen[2];
		int		  y = (screenSize.height - h) / 2 + dscreen[3];

	    Container	c = getContentPane();
		ImageIcon	icon = new ImageIcon(_splashImage);
		JLabel		label = new JLabel(icon);

		setBackground(BORDERCOLOR);
		((JPanel)c).setOpaque(false);
		getRootPane().setOpaque(false);
		label.setOpaque(false);
		c.add(label);
		pack();
		setBounds(x, y, w, h);
		setVisible(true); 
        setCursor(LySwingViewManager.WAIT_CURSOR);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see java.awt.Component#paint(java.awt.Graphics)
	 */
	public void paint(Graphics g)
	{
		g.drawImage(_splashImage, BORDERSIZE, BORDERSIZE, _imageWidth, _imageHeight, this);
		int		copyrightHeight = 0;
		
		g.setFont(_font);
		g.setColor(_fontColor);
		FontMetrics fm = g.getFontMetrics();
		if (_copyright != null)
		{
			Rectangle2D rCopyright = fm.getStringBounds(_copyright, g);
			copyrightHeight = (int)rCopyright.getHeight();
			g.drawString(_copyright, (int)(_imageWidth - rCopyright.getWidth() - 5),(int)( _imageHeight - rCopyright.getHeight()));
		}
		
		if (_version != null)
		{
			Rectangle2D rVersion = fm.getStringBounds(_version, g);
			
			g.drawString(_version, (int)(_imageWidth - rVersion.getWidth()- 5),(int)( _imageHeight - rVersion.getHeight() - copyrightHeight));
		}
	}

	/**
	 * Frees the current component. Unshows the splash screen frame then disposes it.
	 */
	public void free()
	{
		setVisible(false);
		dispose();
	}
} // end class LySwingSplashScreen
