/*
 * LySwingTip.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing;

import java.awt.*;
import leon.misc.*;
import leon.view.*;
import leon.view.graphic.*;
import leon.view.win.swing.component.*;


/**
 * This class implements the Tip object (as described in the dtd) for swing viewer.
 *
 * <p>A tooltip allows to associate an help text to a field. The text is displayed under the form of
 * a screen tip associated to the graphical component of the field.</p>
 *
 * @version $Id: LySwingTip.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 * @see     LyTip
 */
public class LySwingTip extends LySwingObject
	implements LySwingComponent,
		LyGraphicalTip
{
	public static final String __VERSION =
		"$Id: LySwingTip.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Default tip width. */
	public static int		   TIP_WIDTH = 15;

	/** Default tip height. */
	public static int		   TIP_HEIGHT = 15;

	/** The view independent help tip description. */
	protected LyTip			   _tip;

	/** The image displayed by this tip. */
	protected Image			   _image;

	/** The swing component used to display this tip. */
	protected Component		   _component;

	/**
	 * Class constructor of a Tip object for the Swing target.
	 *
	 * @param tip a TIP component defined for every type of view targets. This help tip component is
	 *            initialized specificaly for the Swing view.
	 */
	public LySwingTip(LyTip tip)
	{
		super(tip);
		_tip = tip;

		initialize();
	}

	/**
	 * Loads the tip image.
	 */
	protected void initialize()
	{
		LyEnvironment environment = getEnvironment();
		LyString	  string = environment.getString();
		StringBuffer  buffer = new StringBuffer();

		buffer.append(string.VISIBLE_SEP3);
		buffer.append(TIP_WIDTH);
		buffer.append(string.VISIBLE_SEP2);
		buffer.append(TIP_HEIGHT);
		buffer.append(string.VISIBLE_SEP3_BIS);

		_image = (Image)environment.getImage("LY_TIP_ICON", buffer.toString());
	}

	/**
	 * Creates the tip component.
	 *
	 * @return the new tip component
	 */
	protected Component createTip()
	{
		LyJIconJLabel label = new LyJIconJLabel(_image, translate(_tip.getLabel()), TIP_WIDTH,
				TIP_HEIGHT);

		return label;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.win.swing.LySwingComponent#getComponent()
	 */
	public Component getComponent()
	{
		if (_component == null)
			_component = createTip();

		return _component;
	}

} // end class LySwingTip
