/*
 * LyAbstractChoicePanelManager.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.component;

import java.awt.*;
import leon.view.*;


/**
 * Handles choice panel management.
 *
 * @version $Id: LyAbstractChoicePanelManager.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public abstract class LyAbstractChoicePanelManager
{
	public static final String __VERSION =
		"$Id: LyAbstractChoicePanelManager.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Number of choice elements on a same line. -1 for auto. */
	int						   _maxSpan;

	/**
	 * Indicates the orientation when adding new choice elements to the choice panel. {@link
	 * LyChoiceBase#FILL_HORIZONTAL} or {@link LyChoiceBase#FILL_VERTICAL}.
	 */
    LyChoiceBase.Fill		   _fill;

	/** Panel component that contains the choice elements. */
	Container				   _panel;

	/**
	 * Class constructor of a choice panel manager.
	 *
	 * @param panel the choice panel component
	 */
	public LyAbstractChoicePanelManager(Container panel)
	{
		_panel = panel;
		_maxSpan = -1;
		_fill = LyChoiceBase.Fill.HORIZONTAL;
	}

	/**
	 * Sets the maximum number of objects on one line. -1 for auto.
	 *
	 * @param maxSpan the number of objects in the panel
	 */
	public void setMaxSpan(int maxSpan)
	{
		_maxSpan = maxSpan;
	}

	/**
	 * Sets the filling policy of the choice panel container.
	 *
	 * @param fill LyChoiceBase.FILL_HORIZONTAL or LyChoiceBase.FILL_VERTICAL
	 */
	public void setFill(LyChoiceBase.Fill fill)
	{
		_fill = fill;
	}

	/**
	 * Gets the preferred size of the choice panel parent.
	 *
	 * @return the preferred size of the panel parent
	 */
	public abstract Dimension superGetPreferredSize();

	/**
	 * Retreives the preferred size for this Panel.
	 *
	 * @return the preferred size of the panel
	 */
	public Dimension getPreferredSize()
	{
		int n = _panel.getComponentCount();

		if (n == 0)
			return new Dimension(0, 0);

		Dimension d = superGetPreferredSize();

		Component comp;
		Dimension dmax;
		int		  i, pad = 1;
		boolean   layout_done = _panel.isValid();
		Dimension currentRow = new Dimension(0, 0);
		Dimension currentView = new Dimension(0, 0);
		int		  lineCount = 0;

		for (i = 0; i < n; i++)
		{
			comp = _panel.getComponent(i);

			if (!layout_done)
			{
				// Can not use location as long as components are not set
				_panel.doLayout();
				layout_done = true;
			}

			dmax = comp.getPreferredSize();

			if (_fill == LyChoiceBase.Fill.HORIZONTAL)
			{
				currentRow.width += (dmax.width + pad);

				if ((dmax.height + pad) > currentRow.height)
					currentRow.height = (dmax.height + pad);

				// check if we should go to next line
				if (_maxSpan > 0)
				{
					if ((i - (_maxSpan * lineCount) + 1) >= _maxSpan)
					{
						if (currentView.width < currentRow.width)
							currentView.width = currentRow.width;

						// check if components has already been put into next line
						// or not
						if (currentRow.height <= currentView.height)
							currentView.height += currentRow.height;
						else
							currentView.height = currentRow.height;
						lineCount++;
						currentRow = new Dimension(0, 0);
					}
				}
			}
			else if (_fill == LyChoiceBase.Fill.VERTICAL)
			{
				currentRow.height += (dmax.height + pad);

				if ((dmax.width + pad) > currentRow.width)
					currentRow.width = (dmax.width + pad);

				// check if we should go to next line
				if (_maxSpan > 0)
				{
					if ((i - (_maxSpan * lineCount) + 1) >= _maxSpan)
					{
						if (currentView.height < currentRow.height)
							currentView.height = currentRow.height;

						// check if components has already been put into next line
						// or not
						if (currentRow.width <= currentView.width)
							currentView.width += currentRow.width;
						else
							currentView.width = currentRow.width;
						lineCount++;
						currentRow = new Dimension(0, 0);
					}
				}
			}
		}

		if (_maxSpan > 0)
		{
			if (_fill == LyChoiceBase.Fill.HORIZONTAL)
			{
				if (currentView.width < currentRow.width)
					currentView.width = currentRow.width;

				currentView.height += currentRow.height;
			}
			else if (_fill == LyChoiceBase.Fill.VERTICAL)
			{
				if (currentView.height < currentRow.height)
					currentView.height = currentRow.height;

				currentView.width += currentRow.width;
			}
			d = currentView;
		}

		return d;
	} // end method getPreferredSize

	/**
	 * Gets the minimum size of this component. This is the same size as the preferred size.
	 *
	 * @return the minimum size used by this panel.
	 */
	public Dimension getMinimumSize()
	{
		return getPreferredSize();
	}

} // end class LyAbstractChoicePanelManager
