/*
 * LyJBackgroundPanel.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.component;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import leon.view.win.swing.*;


/**
 * This class implements a panel who can have an image as a background.
 *
 * @author  Lyria
 * @version $Id: LyJBackgroundPanel.java 28422 2010-11-03 12:03:38Z ehelmste $
 */
public class LyJBackgroundPanel extends JPanel implements MouseListener
{
	private static final long  serialVersionUID = 5981028498345306386L;

	public static final String __VERSION =
		"$Id: LyJBackgroundPanel.java 28422 2010-11-03 12:03:38Z ehelmste $";

	// Data
	private Image			   _bgImage;

	/**
	 * Default constructor.
	 */
	public LyJBackgroundPanel()
	{
		this(null);
	}

	/**
	 * Class constructor.
	 *
	 * @param image the background image of the panel
	 */
	public LyJBackgroundPanel(Image image)
	{
		this(null, image);
	}

	/**
	 * Class constructor.
	 *
	 * @param layout the layout manager of the panel
	 * @param image  the background image of the panel
	 */
	public LyJBackgroundPanel(LayoutManager layout, Image image)
	{
		super(layout);
		// initialize data
		_bgImage = image;
		
		addMouseListener(this);
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent e)
	{
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent e)
	{
		requestFocus();
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent e)
	{
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent e)
	{
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent e)
	{
	}

	/**
	 * Overwritten to draw the background image.
	 */
	protected void paintComponent(Graphics g)
	{
		super.paintComponent(g);

		if (_bgImage != null)
			LySwingUtils.fillBackground(g, getWidth(), getHeight(), _bgImage);
	}
} // end class LyJBackgroundPanel
