/*
 * LyJBooleanPanel.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.component;

import java.awt.*;
import javax.swing.*;
import leon.view.getinterface.*;


/**
 * Boolean Canvas.
 *
 * @version $Id: LyJBooleanPanel.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 * @see     Panel
 */
public class LyJBooleanPanel extends JPanel
	implements LyGetManagerInterface
{
	private static final long		  serialVersionUID = -9015903122078668058L;

	public static final String		  __VERSION =
		"$Id: LyJBooleanPanel.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Manages the offscreen painting of the canvas. */
	protected _LyBooleanCanvasManager _manager;

	/**
	 * Default boolean panel constructor.
	 */
	public LyJBooleanPanel()
	{
		super();

		_manager = new _LyBooleanCanvasManager(this);
	}

	/**
	 * Paints all.
	 *
	 * @see JComponent#paintComponent(java.awt.Graphics)
	 */
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		_manager.paintCommon(g);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see java.awt.Component#getMinimumSize()
	 */
	public Dimension getMinimumSize()
	{
		return _manager.getPreferredSize();
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see java.awt.Component#getPreferredSize()
	 */
	public Dimension getPreferredSize()
	{
		Dimension dim = _manager.getPreferredSize();

		return dim;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.getinterface.LyGetManagerInterface#getManager()
	 */
	public Object getManager()
	{
		return _manager;
	}

	/**
	 * This boolean panel is always focus traversable.
	 *
	 * @return <code>true</code>
	 * @see    java.awt.Component#isFocusTraversable()
	 */
	public boolean isFocusTraversable()
	{
		return true;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see java.awt.Component#setEnabled(boolean)
	 */
	public void setEnabled(boolean enabled)
	{
		_manager.setEnabled(enabled);
		_manager.redraw();
	}

	/* ******************************************************************************************/

	/**
	 * Implements the LyAbstractBooleanPanelManager to manage offscreen image.
	 *
	 * @see LyAbstractBooleanPanelManager
	 */
	public class _LyBooleanCanvasManager extends LyAbstractBooleanPanelManager
	{

		/**
		 * Sole constructor. (For invocation by subclass constructors, typically implicit.)
		 */
		public _LyBooleanCanvasManager()
		{
			// XXX this call should generate a NullPointerException when initializing the abstract
			// boolean panel manager.
			this(null);
		}

		/**
		 * Creates a boolean canvas manager for the specified component.
		 *
		 * @param comp the boolean component that need a canvas manager
		 */
		public _LyBooleanCanvasManager(Component comp)
		{
			super(comp);
		}

		/**
		 * This canvas manager doesn't have offscreen images.
		 *
		 * @param  offScreenSize the size of the image to return
		 * @return <code>null</code>
		 * @see    LyAbstractBooleanPanelManager#getImageOffScreen(Dimension)
		 */
		public Image getImageOffScreen(Dimension offScreenSize)
		{
			return null;
		}

		/**
		 * There is no offscreen image, so this is the specified default graphics context that is
		 * used.
		 *
		 * @param  offScreenImage  this image is not used in this implementation
		 * @param  defaultGraphics the graphics context that is returned
		 * @return the default graphics
		 * @see    LyAbstractBooleanPanelManager#getTargetGraphics(Image, Graphics)
		 */
		public Graphics getTargetGraphics(Image offScreenImage, Graphics defaultGraphics)
		{
			return defaultGraphics;
		}

		/**
		 * Doesn't do anything.
		 *
		 * @see LyAbstractBooleanPanelManager#paintOnScreen(Image, Graphics)
		 */
		public void paintOnScreen(Image offScreenImage, Graphics onScreenGraphics)
		{
		}
	} // end class _LyBooleanCanvasManager
} // end class LyJBooleanPanel
