/*
 * LyJCanvasSplitter.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.component;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import leon.view.getinterface.*;

/**
 * Splitter Canvas.
 *
 * @version $Id: LyJCanvasSplitter.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 * @see     Panel
 */
public class LyJCanvasSplitter extends JPanel
	implements LyGetManagerInterface
{
	private static final long		   serialVersionUID = 5000912433337682575L;

	public static final String		   __VERSION =
		"$Id: LyJCanvasSplitter.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Manages the offscreen painting of the canvas. */
	protected _LyCanvasSplitterManager _manager;

	/**
	 * Default canvas splitter constructor.
	 */
	public LyJCanvasSplitter()
	{
		super();

		_manager = new _LyCanvasSplitterManager(this);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see java.awt.Component#paint(java.awt.Graphics)
	 */
	public void paint(Graphics g)
	{
		_manager.paintCommon(g);
		super.paintComponents(g);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see java.awt.Component#getMinimumSize()
	 */
	public Dimension getMinimumSize()
	{
		return _manager.getPreferredSize();
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see java.awt.Component#getPreferredSize()
	 */
	public Dimension getPreferredSize()
	{
		Dimension dim = _manager.getPreferredSize();

		return dim;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.getinterface.LyGetManagerInterface#getManager()
	 */
	public Object getManager()
	{
		return _manager;
	}

	/**
	 * Gets the associated canvas splitter manager.
	 *
	 * @return the canvas splitter manager of this component
	 */
	public LyAbstractCanvasSplitterManager getCanvasSplitterManager()
	{
		return _manager;
	}

	/**
	 * This canvas splitter panel is always focus traversable.
	 *
	 * @return <code>true</code>
	 * @see    java.awt.Component#isFocusTraversable()
	 */
	public boolean isFocusTraversable()
	{
		return true;
	}

	/**
	 * .
	 */

	/**
	 * Implements the LyAbstractCanvasSplitterManager to manage offscreen image.
	 *
	 * @see LyAbstractCanvasSplitterManager
	 */
	public class _LyCanvasSplitterManager extends LyAbstractCanvasSplitterManager
	{

		/**
		 * Sole constructor. (For invocation by subclass constructors, typically implicit.)
		 */
		public _LyCanvasSplitterManager()
		{
			// XXX this call should generate a NullPointerException when initializing the canvas
			// splitter panel manager.
			this(null);
		}

		/**
		 * Creates a canvas splitter manager for the specified component.
		 *
		 * @param comp the splitter component that need a canvas manager
		 */
		public _LyCanvasSplitterManager(Container comp)
		{
			super(comp);
		}

		/*
		 * (non-Javadoc)
		 * @see leon.view.win.swing.component.LyAbstractCanvasSplitterManager#buildSplitter(boolean)
		 */
		public Component buildSplitter(boolean horizontal)
		{
			return new JScrollBar(horizontal ? Scrollbar.HORIZONTAL : Scrollbar.VERTICAL);
		}

		/*
		 * (non-Javadoc)
		 * @see leon.view.win.swing.component.LyAbstractCanvasSplitterManager#transformMouseEvent(java.awt.event.MouseEvent)
		 */
		protected MouseEvent transformMouseEvent(MouseEvent e)
		{
			MouseEvent result = super.transformMouseEvent(e);
			int		   offset = 0;

			if ((_mouseOverPart == LEFT_HEADER_TOP_PART)
					|| (_mouseOverPart == LEFT_HEADER_BOTTOM_PART))
				offset = getVerticalOffset(true);

			return new MouseEvent(result.getComponent(), result.getID(), result.getWhen(),
				result.getModifiers(), result.getX(), result.getY() + offset,
				result.getClickCount(), result.isPopupTrigger(), result.getButton());
		}
	} // end class _LyCanvasSplitterManager
} // end class LyJCanvasSplitter
