/*
 * LyJDictionaryPopupSelector.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.component;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import leon.misc.*;
import leon.view.event.*;


/**
 * This class implements a popup selection between a combo (choice) and a dictionary.
 *
 * @version $Id: LyJDictionaryPopupSelector.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public class LyJDictionaryPopupSelector extends LyJPopupSelector
{
	public static final String __VERSION =
		"$Id: LyJDictionaryPopupSelector.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Background color. */
	protected Color			   _bgColor;

	/**
	 * Creates a new popup selector between a combo and a dictionary. By default, the alignment
	 * policy is LyJPopupSelector.Align.RIGHT.
	 *
	 * @param choice      the component which ensures the display
	 * @param popup       a component to edit the value of the combo. Raised when opening the combo.
	 * @param environment the current environment
	 */
	public LyJDictionaryPopupSelector(Component choice, Component popup, LyEnvironment environment)
	{
		super(choice, popup, ALIGN_RIGHT, environment);

		_bgColor = null;
	}

	/**
	 * Creates a new popup selector between a combo and a dictionary.
	 *
	 * @param choice      the component which ensures the display
	 * @param popup       a component to edit the value of the combo. Raised when opening the combo.
	 * @param align       Specifies the alignment. One of the following:
	 *
	 *                    <ul>
	 *                      <li>{@link LyJPopupSelector#Align.RIGHT}</li>
	 *                      <li>{@link LyJPopupSelector#Align.LEFT}</li>
	 *                      <li>{@link LyJPopupSelector#Align.TOP}</li>
	 *                      <li>{@link LyJPopupSelector#Align.BOTTOM}</li>
	 *                    </ul>
	 * @param environment the current environment
	 */
	public LyJDictionaryPopupSelector(Component choice, Component popup, int align,
		LyEnvironment environment)
	{
		super(choice, popup, align, environment);

		_bgColor = null;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.event.LyActionListener#actionPerformed(leon.view.event.LyActionEvent)
	 */
	public void actionPerformed(LyActionEvent actionEvent)
	{
		if (actionEvent.getType() == LyActionDemand.TYPE_EXPAND)
			return;
		super.actionPerformed(actionEvent);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.win.swing.component.LyJPopupSelector#showTipWindow()
	 */
	protected void showTipWindow()
	{
		super.showTipWindow();

		((JScrollPane)_popup).getViewport().getComponent(0).requestFocus();
		_popup.setBackground(_bgColor);
	}

	/**
	 * Sets the background color.
	 *
	 * @param color the new background color
	 */
	public void setBackground(Color color)
	{
		_bgColor = color;
	}

	/**
	 * Nothing is done if the focus is lost.
	 *
	 * @see LyJPopupSelector#focusLost(FocusEvent)
	 */
	public void focusLost(FocusEvent e)
	{
	}
} // end class LyJDictionaryPopupSelector
