/*
 * LyJDictionarySelector.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.component;

import java.awt.*;
import java.awt.event.*;
import leon.misc.*;
import leon.view.event.*;
import leon.view.setinterface.*;


/**
 * This class implements a selection tool for a dictionary component.
 *
 * @version $Id: LyJDictionarySelector.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public class LyJDictionarySelector extends LyJSelector
	implements LyFreeInterface
{
	private static final long  serialVersionUID = 15606242287877348L;

	public static final String __VERSION =
		"$Id: LyJDictionarySelector.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** The button component. */
	protected LyJButton		   _button;

	/** The associated menu component. */
	protected Menu			   _menuComp;

	/** Indicates if this selector is enabled or not. */
	protected boolean		   _enabled;

	/**
	 * Creates a new dictionary selector.
	 *
	 * @param value  the label of the selector
	 * @param button the button image
	 * @param env    the current environment
	 */
	public LyJDictionarySelector(String value, Image button, LyEnvironment env)
	{
		super(value, env, true);
		_button = addButton(button, null);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.setinterface.LyFreeInterface#free()
	 */
	public void free()
	{
		_button = null;
	}

	/**
	 * Finishes the initialization and adds the default key listener.
	 *
	 * @param environment the current environment, not used in this implementation
	 */
	public void endInit(LyEnvironment environment)
	{
		_text.addKeyListener(this);
		_text.setEditable(false);
		setBackground(_enabledColor);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
	 */
	public void keyPressed(KeyEvent e)
	{
		int key = e.getKeyCode();

		if (key == KeyEvent.VK_DOWN)
		{
			if (!_enabled)
				return;
			LyActionEvent evt = new LyActionEvent(this);

			evt.setSourceId("selector_button");
			_actionTransmitter.processActionEvent(evt);
		}
		else if ((key == KeyEvent.VK_DELETE)
				|| ((key == KeyEvent.VK_BACK_SPACE) && (_text.getSelectionStart() > 0)
					&& (_text.getSelectionStart() == _text.getSelectionEnd())))
		{
			// Handle del content request
			setValue(null); // DO NOT let the LySelector handle notification. It doesn't handle null
			// value

			if (_actionTransmitter != null)
			{
				LyActionEvent evt = new LyActionEvent(this, null, null,
						LyActionDemand.TYPE_VALUE_CHANGED);

				_actionTransmitter.processActionEvent(evt);
			}
		}
		else
			super.keyPressed(e);
	} // end method keyPressed

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.win.swing.component.LyJSelector#checkButtons()
	 */
	protected void checkButtons()
	{
		if (_button != null)
			_button.setEnabled(_enabled);
	}

	/**
	 * Sets the enabled state of the button then refreshes it.
	 *
	 * @param enabled the new enabled state
	 */
	public void setButtonEnabled(boolean enabled)
	{
		_enabled = enabled;
		checkButtons();
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.setinterface.LySetEditableInterface#setEditable(boolean)
	 */
	public void setEditable(boolean editable)
	{
		super.setEditable(editable);

		if (editable && (_text != null) && _text.isEditable())
		{
			_text.setEditable(false);
			setBackground(_enabledColor);
		}
	}
} // end class LyJDictionarySelector
