/*
 * LyJExpandBar.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.component;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import leon.misc.*;


/**
 * This class defines an expand bar that may contains expand items.
 *
 * @see LyJExpandItem
 */
public class LyJExpandBar extends JPanel implements MouseListener
{
	private static final long  serialVersionUID = -4730687986574122090L;
	public static final String __VERSION =
		"$Id: LyJExpandBar.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Current environment. */
	protected LyEnvironment		_environment;

	/**
	 * Sole constructor.
	 */
	public LyJExpandBar(LyEnvironment environment)
	{
		super();
		
		_environment = environment;
		
		VFlowLayout	layout = new VFlowLayout(VFlowLayout.TOP, false);
		
		layout.setAlignment(FlowLayout.LEFT);
		setBackground((Color)_environment.getColor("LY_EXPANDBAR_CONTENT_COLOR"));
		setLayout(layout);
		
		addMouseListener(this);
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.awt.Container#addImpl(java.awt.Component, java.lang.Object, int)
	 */
	protected void addImpl(Component comp, Object constraints, int index)
	{
		super.addImpl(comp, constraints, index);

		// Update the width of expand items contained in this expand bar each time a
		// component is added.
		updateExpandItemsWidth();
		
		// If expand bar is contained in a scroll pane, set the preferred size of the scroll
		// pane to avoid resize of the scroll pane.
		if ((getParent() != null) && (getParent().getParent() instanceof JScrollPane))
		{
			JScrollPane	pane = (JScrollPane)getParent().getParent();
			Insets		insets = getParent().getInsets();
			Insets		scrollInsets = getParent().getParent().getInsets();
			int			scrollbarWidth = 15;
			Dimension	size = getPreferredSize();
			
			pane.setPreferredSize(new Dimension(size.width + insets.left + insets.right +
					scrollInsets.left + scrollInsets.right + scrollbarWidth,
					size.height));
		}
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.Container#doLayout()
	 */
	public void doLayout()
	{
		super.doLayout();
		updateExpandItemsWidth();
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.Component#isFocusTraversable()
	 */
	public boolean isFocusTraversable()
	{
		return true;
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent e)
	{
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent e)
	{
		requestFocus();
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent e)
	{
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent e)
	{
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent e)
	{
	}

	/**
	 * Compute the new minimum size of internal panel.
	 */
	protected void updateExpandItemsWidth()
	{
		int				i, n = getComponentCount();
		int				width = 0;
		JComponent		comp;
		
		for (i = 0; i < n; i++)
		{
			comp = (JComponent)getComponent(i);
			width = Math.max(width, comp.getPreferredSize().width);
		}
		
		for (i = 0; i < n; i++)
		{
			comp = (JComponent)getComponent(i);
			
			if (comp instanceof LyJExpandItem)
				((LyJExpandItem)comp).setWidth(width);
			else if (comp instanceof JPanel)
			{
				// Force preferred width for JPanel components (for example for top
				// panel containing toolbar image and button) but not for tools
				// that are placed in this expand bar.
				Dimension	dim = comp.getPreferredSize();
				
				if (dim.height > 0)
					comp.setPreferredSize(new Dimension(width, dim.height));
			}
		}
	}
}