/*
 * LyJFormattedDateInput.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.component;

import java.awt.*;
import java.text.*;
import java.util.*;
import javax.swing.*;

import leon.misc.*;
import leon.view.event.*;


/**
 * This class implements a specific formatted input text for date handling.
 *
 * @version $Id: LyJFormattedDateInput.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public class LyJFormattedDateInput extends LyJFormattedInput
{
	private static final long  serialVersionUID = 6751483463464556591L;

	public static final String __VERSION =
		"$Id: LyJFormattedDateInput.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** The format of the date. */
	DateFormat				   _format;

	/** The string format of the date. */
	String					   _formatDate;

	/**
	 * Creates a LyJFormattedDateInput object.
	 *
	 * @param value  the initial value of the selector
	 * @param format the input format
	 * @param image  the arrow image
	 * @param env    the environment
	 */
	public LyJFormattedDateInput(String value, String format, Image image, LyEnvironment env,
		boolean useArrows)
	{
		super(value, format, image, null, env, useArrows);
	}

	/**
	 * Sets the date format.
	 *
	 * @param format the date format
	 */
	public void setFormat(String format)
	{
		_format = new SimpleDateFormat(format);

		_formatDate = format;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.win.swing.component.LyJSelector#addButton(java.awt.Image, java.lang.String)
	 */
	public LyJButton addButton(Image image, String label)
	{
		LyJButton button = new LyJButton(image, label, null, "selector_button");

		button.setRollover(false);
		button.setOpaque(true);
		button.setContentAreaFilled(false);

		Icon icon = new ImageIcon(image);

		button.setIcon(icon);
		button.setPreferredSize(new Dimension(icon.getIconWidth() + IMAGE_W_OFFSET,
				icon.getIconHeight()
				+ IMAGE_H_OFFSET));
		button.setHorizontalAlignment(SwingConstants.CENTER);
		button.setVerticalAlignment(SwingConstants.CENTER);
		button.setMargin(new Insets(0, 0, 0, 1));

		LyActionTransmitter transmit = button.getActionTransmitter();

		if (transmit != null)
			transmit.addActionListener(this);
		
		button.setBorder(BorderFactory.createMatteBorder(1, 1, 1, 1,
				UIManager.getColor("TextField.darkShadow")));
		
		_buttons.add(button, buildButtonConstraint());

		return button;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.event.LyActionListener#actionPerformed(leon.view.event.LyActionEvent)
	 */
	public void actionPerformed(LyActionEvent actionEvent)
	{
		LyActionEvent evt = new LyActionEvent(this);

		evt.setSourceId("selector_button");
		evt.setValue("selector_button");
		_actionTransmitter.processActionEvent(evt);
	}

	/**
	 * Computes the correct value for the date.
	 */
	protected void computeValue()
	{
		String value = getValue();
		Date   date = null;

		if ((value == null) || (value.length() == 0))
			return;

		try
		{
			date = _format.parse(value);
		}
		catch (ParseException e)
		{
			e.printStackTrace();
		}

		// Special CASE
		// 31/02/2004 is correct for date parser and it becomes 02/03/2004 with parsing.
		// Check It and 31/02/2004 must become 29/02/2004
		if (date != null)
		{
			// Get index of month on date format.
			int firstIndexMonth = _formatDate.toString().indexOf("M");
			int secondIndexMonth = _formatDate.toString().indexOf("M",
					(firstIndexMonth > -1) ? (firstIndexMonth + 1) : 0);

			if (firstIndexMonth > -1)
			{
				// Compute index of month in Gregorian Calendar.
				String month = "" + value.charAt(firstIndexMonth);

				if (secondIndexMonth > -1)
					month = month + value.charAt(secondIndexMonth);

				try
				{
					int monthI = Integer.parseInt(month);

					monthI--;
					Calendar calendar = Calendar.getInstance();

					calendar.setTime(date);

					// Check if month is the same as parsed date.
					if (monthI != calendar.get(Calendar.MONTH))
					{
						calendar.roll(Calendar.MONTH, -1);
						calendar.set(Calendar.DAY_OF_MONTH,
							calendar.getActualMaximum(Calendar.DAY_OF_MONTH));
						date = calendar.getTime();
					}
				}
				// no correction
				catch (Exception e)
				{
				}
			}
		}

		if (date != null)
		{
			value = _format.format(date);
			setValue(value);
		}
	} // end method computeValue

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.win.swing.component.LyJFormattedInput#computeMinValue(leon.view.win.swing.component.LyJFormattedInput.LyStructureItem)
	 */
	public String computeMinValue(_LyStructureItem item)
	{
		if (item instanceof _LyNumericStructure)
		{
			_LyNumericStructure num = (_LyNumericStructure)item;
			GregorianCalendar   current = new GregorianCalendar();

			if (num._max == 12) // month
				return "" + (current.get(Calendar.MONTH) + 1);
			else if (num._max == 31) // day
				return "" + current.get(Calendar.DAY_OF_MONTH);
			else if (num._max == 9999) // year
				return "" + current.get(Calendar.YEAR);
		}

		return super.computeMinValue(item);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.win.swing.component.LyJSelector#checkButtons()
	 */
	protected void checkButtons()
	{
		if (_buttons == null)
			return;
		if (!isEnabled() || !_buttons.isEnabled())
		{
			int i, n = _buttons.getComponentCount();

			for (i = 0; i < n; i++)
				_buttons.getComponent(i).setEnabled(false);
		}
		else
		{
			int i, n = _buttons.getComponentCount();

			for (i = 0; i < n; i++)
				_buttons.getComponent(i).setEnabled(true);
		}
	}
} // end class LyJFormattedDateInput
