/*
 * LyJFormattedTimeInput.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.component;

import java.awt.*;
import java.text.*;
import java.util.*;
import leon.misc.*;


/**
 * This class implements a specific formatted input text for time handling.
 */
public class LyJFormattedTimeInput extends LyJFormattedInput
{
	private static final long  serialVersionUID = -1243104537809798252L;

	public static final String __VERSION =
		"$Id: LyJFormattedTimeInput.java 28997 2010-12-23 10:17:15Z bmichel $";

	/** The format of the date. */
	DateFormat				   _formatDate;

	/**
	 * Creates a formatted input selector with the specified attributes.
	 *
	 * @param value   the current value
	 * @param format  the format
	 * @param imgUp   the up button image
	 * @param imgDown the down button image
	 */
	public LyJFormattedTimeInput(String value, String format, Image imgUp, Image imgDown)
	{
		super(value, format, imgUp, imgDown);
	}

	/**
	 * Creates a formatted input selector with the specified attributes.
	 *
	 * @param value   the current value
	 * @param format  the format
	 * @param imgUp   the up button image
	 * @param imgDown the down button image
	 * @param env     the environment
	 */
	public LyJFormattedTimeInput(String value, String format, Image imgUp, Image imgDown,
		LyEnvironment env, boolean useArrows)
	{
		super(value, format, imgUp, imgDown, env, useArrows);
	}

	/**
	 * Sets the date format.
	 *
	 * @param format the date format
	 * @see   SimpleDateFormat
	 */
	public void setFormat(String format)
	{
		_formatDate = new SimpleDateFormat(format);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.win.swing.component.LyJFormattedInput#computeMinValue(leon.view.win.swing.component.LyJFormattedInput._LyStructureItem)
	 */
	public String computeMinValue(_LyStructureItem item)
	{
		if (item instanceof _LyNumericStructure)
		{
			_LyFormattedField   text = (_LyFormattedField)_text;
			Vector<_LyStructureItem>	list = text._structure._structureList;
			_LyNumericStructure num = (_LyNumericStructure)item;
			GregorianCalendar   current = new GregorianCalendar();
			_LyNumericStructure s;

			if (num._max == 23) // hour
				return "" + current.get(Calendar.HOUR_OF_DAY);
			else if (num._max == 59)
			{
				int idx = list.indexOf(num);

				if (idx == 0)
					// Minutes
					return "" + current.get(Calendar.MINUTE);
				else if (idx == 2)
				{
					s = (_LyNumericStructure)list.elementAt(0);
					if (s._max == 23)
						// Minutes
						return "" + current.get(Calendar.MINUTE);
					// Seconds
					return "" + current.get(Calendar.SECOND);
				}
				else
					// Seconds
					return "" + current.get(Calendar.SECOND);
			}
		}
		return super.computeMinValue(item);
	} // end method computeMinValue

} // end class LyJFormattedTimeInput
