/*
 * LyJMessageIcon.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.component;

import java.awt.*;
import javax.swing.*;
import leon.misc.*;
import leon.view.*;
import leon.view.event.*;
import leon.view.getinterface.*;


/**
 * This class implements a component that display an icon.
 *
 * @version $Id: LyJMessageIcon.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public class LyJMessageIcon extends JLabel
	implements LyGetManagerInterface
{
	private static final long  serialVersionUID = -4385523995086525369L;

	public static final String __VERSION =
		"$Id: LyJMessageIcon.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** The manager of this component. */
	_LyJMessageIconManager     _manager;

	/**
	 * Creates a message icon object with the specified attributes.
	 *
	 * @param environment the current environment
	 * @param text        the text of the message
	 * @param image       the image of the icon
	 * @param align       the alignment policy. One of the following:
	 *
	 *                    <ul>
	 *                      <li>{@link LyMessage#Align.LEFT}</li>
	 *                      <li>{@link LyMessage#Align.RIGHT}</li>
	 *                      <li>{@link LyMessage#Align.CENTER}</li>
	 *                    </ul>
	 * @param listener    the associated action listener
	 */
	public LyJMessageIcon(LyEnvironment environment, String text, String image,
        LyMessage.Align align, LyActionListener listener)
	{
		super();
        
		_manager = new _LyJMessageIconManager(environment, text, image, align, this, listener);
		setOpaque(false);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.getinterface.LyGetManagerInterface#getManager()
	 */
	public Object getManager()
	{
		return _manager;
	}

	/* **************************************************************************************/

	/**
	 * Swing customization of the message icon manager.
	 */
	public class _LyJMessageIconManager extends LyAbstractMessageIconManager
	{

		/**
		 * Creates a message icon manager with the specified attributes then sets the default values
		 * for the font and the foreground color.
		 *
		 * @param environment the current environment
		 * @param text        the text message
		 * @param image       the icon image
		 * @param align       the alignment policy. One of the following:
		 *
		 *                    <ul>
		 *                      <li>LyMessage.Align.LEFT</li>
		 *                      <li>LyMessage.Align.RIGHT</li>
		 *                      <li>LyMessage.Align.CENTER</li>
		 *                    </ul>
		 * @param comp        the component to manage
		 * @param listener    the associated action listener
		 */
		public _LyJMessageIconManager(LyEnvironment environment, String text, String image,
			LyMessage.Align align, Component comp, LyActionListener listener)
		{
			super(environment, text, image, align, comp, listener);
			_comp.setFont(_font);
			_comp.setForeground(Color.black);
		}

		/*
		 * (non-Javadoc)
		 * @see leon.view.win.swing.component.LyAbstractMessageIconManager#setText(java.lang.String)
		 */
		public void setText(String label)
		{
			super.setText(label);
			((JLabel)_comp).setText(_text);
		}

		/*
		 * (non-Javadoc)
		 * @see leon.view.win.swing.component.LyAbstractMessageIconManager#setImage(java.lang.String)
		 */
		public void setImage(String image)
		{
			super.setImage(image);
			if (image != null)
				((JLabel)_comp).setIcon(new ImageIcon(_image));
			else
				// FFT00968
				((JLabel)_comp).setIcon(null);
		}

		/*
		 * (non-Javadoc)
		 * @see leon.view.win.swing.component.LyAbstractMessageIconManager#setAlign(short)
		 */
		public void setAlign(LyMessage.Align align)
		{
			super.setAlign(align);
			int swingAlign = -1;

			if (_align == LyMessage.Align.LEFT)
				swingAlign = SwingConstants.LEFT;
			else if (_align == LyMessage.Align.CENTER)
				swingAlign = SwingConstants.CENTER;
			else if (_align == LyMessage.Align.RIGHT)
				swingAlign = SwingConstants.RIGHT;
			((JLabel)_comp).setHorizontalAlignment(swingAlign);
		}
	} // end class _LyJMessageIconManager

} // end class LyJMessageIcon
