/*
 * LyJPopupButton.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.component;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import leon.misc.*;
import leon.view.event.*;


/**
 * This class implements a panel with two buttons : one is the given button and the other is an
 * arrow button. A click on the arrow button opens the given popup component.
 *
 * @version $Id: LyJPopupButton.java 35025 2012-03-23 16:43:07Z jlsaouli $
 * @author  Lyria
 * @see     LyJPopupSelector
 */
public class LyJPopupButton extends LyJPopupSelector
	implements ActionListener
{
	public static final String __VERSION =
		"$Id: LyJPopupButton.java 35025 2012-03-23 16:43:07Z jlsaouli $";

	/** An associated button of this popup button. */
	protected LyJButton		   _button;

	/** Date when the focus is gained. */
	// Workaround for JDK1.3 BUG.
	protected Date			   _gainedFocusDate;

	/** Date when the button is pressed. */
	protected Date			   _mousePressedDate;

	/** Indicates that the mouse is over a toggle button. */
	protected boolean		   _mouseOverToggleButton;

	/**
	 * Creates a new popup button component with the specified attributes.
	 *
	 * @param button      the component which ensures the display
	 * @param popup       the component to edit the value of the combo. Raised when opening the
	 *                    combo.
	 * @param align       the popup alignment
	 * @param environment the current environment
	 */
	public LyJPopupButton(LyJButton button, Component popup, int align, LyEnvironment environment)
	{
		super(button, popup, align, environment);

		_button = button;
		((LyJOpenerButton)_choice).setNeighbor(_button);
	}

	/**
	 * Initializes this popup button component and registers its event listeners.
	 *
	 * @see leon.view.win.swing.component.LyJPopupSelector#init()
	 */
	public void init()
	{
		super.init();

		_choice = new LyJOpenerButton((Image)_environment.getImage("LY_ARROW_DOWN"));
		_mouseOverToggleButton = false;
		((LyJOpenerButton)_choice).addActionListener(this);
		((LyJOpenerButton)_choice).addMouseListener(this);
		((LyJOpenerButton)_choice).setBorder(BorderFactory.createMatteBorder(1, 1, 1, 1,
			(Icon)null));
		((LyJOpenerButton)_choice).setBorderPainted(true);
		((LyJOpenerButton)_choice).setRollover(true);
		((LyJOpenerButton)_choice).setOpaque(false);
		((LyJOpenerButton)_choice).setRolloverChangeBackground(true);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.win.swing.component.LyJPopupSelector#showTipWindow()
	 */
	protected void showTipWindow()
	{
		super.showTipWindow();
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see java.awt.event.FocusListener#focusGained(java.awt.event.FocusEvent)
	 */
	public void focusGained(FocusEvent e)
	{
		super.focusGained(e);
		// XXX REMOVE IF NOPATCH. Useful for JDK1.3 (PC / Solaris)
		_gainedFocusDate = new Date();
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see java.awt.event.FocusListener#focusLost(java.awt.event.FocusEvent)
	 */
	public void focusLost(FocusEvent e)
	{
		Date d = new Date();

		if ((_mousePressedDate != null) && ((d.getTime() - _mousePressedDate.getTime()) < 500))
			return;

		// XXX REMOVE IF NOPATCH. Useful for JDK1.3 (PC / Solaris)
		// if ((_gainedFocusDate != null) && ((d.getTime() - _gainedFocusDate.getTime()) < 200))
		// _tipWindow.requestFocus();
		// CHECK TEMPORARY to distinguish between focus lost by window
		// deactivation OR click on another window, from the click on a component
		// INSIDE the window
		// else

		if (e.isTemporary())
		{
			super.focusLost(e);
			if (!_mouseOverToggleButton)
				((LyJOpenerButton)_choice).setSelected(false);
		}
		else
			super.focusLost(e);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.win.swing.component.LyJPopupSelector#hideTipWindow()
	 */
	public void hideTipWindow()
	{
		super.hideTipWindow();
		if (!_mouseOverToggleButton)
			((LyJOpenerButton)_choice).setSelected(false);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.win.swing.component.LyJPopupSelector#getTipWindowLocation()
	 */
	protected Point getTipWindowLocation()
	{
		// Invert temporarly the button and the choice
		Component tempChoice = _choice;

		if (_align == ALIGN_LEFT)
			_choice = _button;

		Point result = super.getTipWindowLocation();

		if (_align == ALIGN_LEFT)
			_choice = tempChoice;

		return result;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent e)
	{
		_mousePressedDate = new Date();
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent e)
	{
		_mouseOverToggleButton = true;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent e)
	{
		_mouseOverToggleButton = false;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e)
	{
		// Swap visibility of the popup
		if ((_tipWindow != null) && _tipWindow.isShowing())
			hideTipWindow();
		else if (!userClosingRequest())
			showTipWindow();
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.event.LyActionListener#actionPerformed(leon.view.event.LyActionEvent)
	 */
	public void actionPerformed(LyActionEvent actionEvent)
	{
		// Don't open the popup if the button is pressed
		super.actionPerformed(actionEvent);
		
		((LyJOpenerButton)_choice).setSelected(false);
	}

	/**
	 * Gets the associated button.
	 *
	 * @return the button that is associated to this toggle button
	 */
	public Component getButton()
	{
		return _button;
	}

	/**
	 * Gets the popup opener parent button.
	 *
	 * @return the toggle button that open a popup window
	 */
	public Component getOpenerButton()
	{
		return _choice;
	}
} // end class LyJPopupButton
