/*
 * LyJPopupDateSelector.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.component;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import leon.misc.*;
import leon.view.win.swing.LySwingDate.*;


/**
 * This class implements a panel with two buttons: one is the given button and the other is an arrow
 * button. A click on the arrow button opens the given popup component.
 *
 * @version $Id: LyJPopupDateSelector.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 * @see     LyJPopupSelector
 */
public class LyJPopupDateSelector extends LyJPopupSelector
{
	public static final String __VERSION =
		"$Id: LyJPopupDateSelector.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** Date when focus is gained. */
	// Workaround for JDK1.3 BUG
	protected Date			   _gainedFocusDate;

	/** Button used to open the calendar. */
	protected JComponent	   _button;

	/**
	 * Creates a new popup button component with the specified attributes.
	 *
	 * @param choice      the component which ensure the display
	 * @param popup       the component that edit the value of the combo. Raised when opening the
	 *                    combo.
	 * @param environment the current environment
	 */
	public LyJPopupDateSelector(Component choice, Component popup, LyEnvironment environment)
	{
		super(choice, popup, environment);

		_button = (JComponent)((LyJFormattedInput)_choice).getButtons().getComponent(0);
		_button.addFocusListener(this);
	}

	/**
	 * Creates a new popup button component with the specified attributes.
	 *
	 * @param choice      the component which ensures the display
	 * @param popup       the component that edit the value of the combo. Raised when opening the
	 *                    combo.
	 * @param align       the popup alignment
	 * @param environment the current environment
	 */
	public LyJPopupDateSelector(Component choice, Component popup, int align,
		LyEnvironment environment)
	{
		super(choice, popup, align, environment);

		_button = (JComponent)((_LyJSpecialSelector)_choice).getButtons().getComponent(0);
		_button.addFocusListener(this);
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see leon.view.win.swing.component.LyJPopupSelector#showTipWindow()
	 */
	protected void showTipWindow()
	{
		focusGained(null);

		super.showTipWindow();
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see java.awt.event.FocusListener#focusGained(java.awt.event.FocusEvent)
	 */
	public void focusGained(FocusEvent e)
	{
		super.focusGained(e);
		// XXX REMOVE IF NOPATCH. Useful for JDK1.3 (PC / Solaris)
		_gainedFocusDate = new Date();
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see java.awt.event.FocusListener#focusLost(java.awt.event.FocusEvent)
	 */
	public void focusLost(FocusEvent e)
	{
		Date d = new Date();

		// XXX REMOVE IF NOPATCH. Useful for JDK1.3 (PC / Solaris)
		if ((d.getTime() - _gainedFocusDate.getTime()) < 200)
		{
			if (_tipWindow != null)
				_tipWindow.requestFocus();
		}

		// CHECK TEMPORARY to distinguish between focus lost by window
		// deactivation OR click on another window, from the click on a component
		// INSIDE the window
		else if ((e.isTemporary() && (e.getComponent() != _button))
				|| ((e.getComponent() == _button) && !e.isTemporary()))
			super.focusLost(e);
	}

} // end class LyJPopupDateSelector
