/*
 * LyJPopupMenu.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.component;

import java.awt.*;
import javax.swing.*;

import leon.misc.*;


/**
 * This class defines a JPopupMenu that never appears half-hidden.
 *
 * @version $Id: LyJPopupMenu.java 33062 2011-11-04 08:36:25Z nhamon $
 * @author  Lyria
 */
public class LyJPopupMenu extends JPopupMenu
{
	private static final long  serialVersionUID = 1055954314874689997L;
	public static final String __VERSION =
		"$Id: LyJPopupMenu.java 33062 2011-11-04 08:36:25Z nhamon $";

	/** The environment of this popup menu. */
	protected LyEnvironment		_environment;
	
	/** Indicates whether this popup menu must be "light". */
	protected boolean			_lightMode;
	
	/**
	 * Sole constructor.
	 */
	public LyJPopupMenu()
	{
		super();
		_lightMode = false;
	}

	/**
	 * Creates a popup menu with the specified environment.
	 *
	 * @param environment the environment of this menu
	 */
	public LyJPopupMenu(LyEnvironment environment)
	{
		super();
		
		_environment = environment;
		_lightMode = false;
	}

	/**
	 * Creates a popup menu with the specified environment.
	 *
	 * @param environment the environment of this menu
	 * @param lightMode   indicates whether this popup menu must be "light"
	 */
	public LyJPopupMenu(LyEnvironment environment, boolean lightMode)
	{
		this(environment);
		_lightMode = lightMode;
	}

	/**
	 * Creates a popup menu with the specified label.
	 *
	 * @param label the name of this menu
	 */
	public LyJPopupMenu(String label)
	{
		super(label);
		_lightMode = false;
	}
	
    public void paint(Graphics g)
    {
    	if (_lightMode)
    	{
			Graphics2D	g2 = (Graphics2D)g.create();
	
			g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));
			super.paint(g2);
			g2.dispose();
    	}
    	else
    		super.paint(g);
    }

	/**
	 * (non-Javadoc).
	 *
	 * @see javax.swing.JPopupMenu#show(java.awt.Component, int, int)
	 */
	public void show(Component invoker, int x, int y)
	{
		Point     origin = new Point(x, y);
		Point     temp = new Point(x, y);
		// plu for point at left upper corner, etc.
		boolean   pld, prd, pru;

		Dimension menuSize = getPreferredSize();
		Rectangle screenRect = null;
		Container root = getTopLevelAncestor();

		if (_environment != null)
		{
			int[]	  screenSize = _environment.getScreenSize(root);
			screenRect = new Rectangle(screenSize[2], screenSize[3], screenSize[0], screenSize[1]);
		}
		else
		{
			Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
			screenRect = new Rectangle(0, 0, screenDim.width, screenDim.height);
		}

		SwingUtilities.convertPointToScreen(temp, invoker);

		// plu is on the screen
		pld = screenRect.contains(temp.x, temp.y + menuSize.height);
		prd = screenRect.contains(temp.x + menuSize.width, temp.y + menuSize.height);
		pru = screenRect.contains(temp.x + menuSize.width, temp.y);

		if (!prd)
		{
			if (pld)
				origin.x -= menuSize.width;
			else if (pru)
				origin.y -= menuSize.height;
			else
			{
				origin.x -= menuSize.width;
				origin.y -= menuSize.height;
			}
		}
		super.show(invoker, origin.x, origin.y);
	} // end method show
} // end class LyJPopupMenu
