/*
 * LyJTabbedPane.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.component;

import java.awt.*;

import javax.swing.*;
import javax.swing.plaf.*;


/**
 * This class implements the Tabpanel object (as described in the dtd) for swing viewer.
 *
 * @version $Id: LyJTabbedPane.java 35433 2012-05-02 12:19:54Z nhamon $
 * @author  Lyria
 * @see     leon.view.win.swing.LySwingTabs
 */
public class LyJTabbedPane extends JTabbedPane
{
	private static final long  serialVersionUID = -9092252412443148188L;

	public static final String __VERSION =
		"$Id: LyJTabbedPane.java 35433 2012-05-02 12:19:54Z nhamon $";

	/** The preferred size of the component. */
	Dimension				   _preferredSize;
	
	boolean					   _addingPanel;

	/**
	 * Creates an empty <code>TabbedPane</code>. The default tab placement is {@link
	 * JTabbedPane#TOP}.
	 */
	public LyJTabbedPane()
	{
		super(JTabbedPane.TOP, JTabbedPane.SCROLL_TAB_LAYOUT);
		_addingPanel = false;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see javax.swing.JTabbedPane#setUI(javax.swing.plaf.TabbedPaneUI)
	 */
	public void setUI(TabbedPaneUI ui)
	{
		if (ui.getClass().getName().startsWith("javax.swing.plaf.metal"))
			ui = new LyJMetalTabbedPaneUI();
		else if (ui.getClass().getName().startsWith("javax.swing.plaf.basic"))
			ui = new LyJTabbedPaneUI();
		else if (ui.getClass().getName().startsWith("com.sun.java.swing.plaf.windows"))
			ui = new LyJWindowsTabbedPaneUI();

		super.setUI(ui);
	}

	/**
	 * Sets the image used for the close button.
	 * 
	 * @param closeImage the image used for the close button.
	 * @param closeOverImage the image used for the close over button.
	 */
	public void setCloseImage(Image closeImage, Image closeOverImage)
	{
		if (ui instanceof LyJMetalTabbedPaneUI)
			((LyJMetalTabbedPaneUI)ui).setCloseImage(closeImage, closeOverImage);
		else if (ui instanceof LyJTabbedPaneUI)
			((LyJTabbedPaneUI)ui).setCloseImage(closeImage, closeOverImage);
		else if (ui instanceof LyJWindowsTabbedPaneUI)
			((LyJWindowsTabbedPaneUI)ui).setCloseImage(closeImage, closeOverImage);
	}
	
	/**
	 * (non-Javadoc).
	 *
	 * @see java.awt.Component#getPreferredSize()
	 */
	public Dimension getPreferredSize()
	{
		if (_preferredSize != null)
			return _preferredSize;

		Dimension result = super.getPreferredSize();

		_preferredSize = new Dimension(result.width + 2, result.height + 2);
		return _preferredSize;
	}
	
	public void setAddingPanel(boolean addingPanel)
	{
		_addingPanel = addingPanel;
	}
	
	@Override
	public Rectangle getBoundsAt(int index)
	{
		if (_addingPanel)
			return null;
		
		return super.getBoundsAt(index);
	}
} // end class LyJTabbedPane
