/*
 * LyJTabbedPanelUI.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.component;

import java.awt.*;
import java.util.*;

import javax.swing.*;
import javax.swing.plaf.basic.*;


/**
 * Specific class for UI of the tab panel.
 *
 * @version $Id: LyJTabbedPaneUI.java 31025 2011-05-16 15:25:39Z nhamon $
 * @author  Lyria
 * @see     leon.view.win.swing.LySwingTabs
 */
public class LyJTabbedPaneUI extends BasicTabbedPaneUI
{
	public static final String __VERSION =
		"$Id: LyJTabbedPaneUI.java 31025 2011-05-16 15:25:39Z nhamon $";

	/** Font of the selected items. */
	protected Font								_boldFont = null;
	
	/** Positions of the items. */
	protected Hashtable<String, Rectangle>		_labelPositions = null;
	
	/** Positions of the close images. */
	protected Hashtable<Integer, Rectangle>		_closeImagePositions = null;
	
	/** Image used for the close button. */
	protected Image								_closeImage = null;
	
	/** Image used for the close over button. */
	protected Image								_closeOverImage = null;
	
	/** Index of the close button that has the mouse over. */
	protected int								_mouseOverCloseButton = -1;

	/**
	 * (non-Javadoc).
	 *
	 * @see javax.swing.plaf.basic.BasicTabbedPaneUI#paintText(java.awt.Graphics, int,
	 *      java.awt.Font, java.awt.FontMetrics, int, java.lang.String, java.awt.Rectangle, boolean)
	 */
	protected void paintText(Graphics g, int tabPlacement, Font font, FontMetrics metrics,
		int tabIndex, String title, Rectangle textRect, boolean isSelected)
	{
		if (_boldFont == null)
			_boldFont = font.deriveFont(Font.BOLD);

		if (isSelected)
		{
			font = _boldFont;
			
			FontMetrics	fm = g.getFontMetrics(font);
			int			width = fm.stringWidth(title);
			
			if (width > textRect.width)
				textRect.x -= (width - textRect.width) / 2;
		}
		
		if (_closeImage != null)
			textRect.x -= _closeImage.getWidth(tabPane) / 2;

		if (_labelPositions == null)
			_labelPositions = new Hashtable<String, Rectangle>();
		
		Rectangle	position = new Rectangle();
		
		getTabBounds(tabIndex, position);
		
		Component[] comps = this.tabPane.getComponents();
		
		if (comps != null)
		{
			for (int i = 0; i < comps.length; i++)
			{
				if (comps[i] instanceof JButton)
				{
					Rectangle	buttonBounds = comps[i].getBounds();
					
					if (position.intersects(buttonBounds)
							&& (position.x + position.width > buttonBounds.x)
							&& (position.x < buttonBounds.x))
						position.width = buttonBounds.x - position.x;
				}
			}
		}

		_labelPositions.put(title, new Rectangle(position.x, position.y, position.width, position.height));
		
		super.paintText(g, tabPlacement, font, metrics, tabIndex, title, textRect, isSelected);
		
		if (_closeImage != null)
		{			
			if (_closeImagePositions == null)
				_closeImagePositions = new Hashtable<Integer, Rectangle>();
			
			Rectangle	imagePosition = new Rectangle(position.x + position.width - _closeImage.getWidth(tabPane),
					position.y + (position.height - _closeImage.getHeight(tabPane)) / 2,
					_closeImage.getWidth(tabPane),
					_closeImage.getHeight(tabPane));
			
			if (_mouseOverCloseButton == tabIndex)
				g.drawImage(_closeOverImage, imagePosition.x + getXOffset(), imagePosition.y, tabPane);
			else
				g.drawImage(_closeImage, imagePosition.x + getXOffset(), imagePosition.y, tabPane);
			
			_closeImagePositions.put(new Integer(tabIndex), imagePosition);
		}
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.plaf.basic.BasicTabbedPaneUI#paintIcon(java.awt.Graphics, int, int, javax.swing.Icon, java.awt.Rectangle, boolean)
	 */
	protected void paintIcon(Graphics g, int tabPlacement, int tabIndex, Icon icon, Rectangle iconRect, boolean isSelected)
	{		
		Rectangle	position = new Rectangle();
		
		getTabBounds(tabIndex, position);
		
		iconRect.x = position.x + getXOffset() + 6;
		
		super.paintIcon(g, tabPlacement, tabIndex, icon, iconRect, isSelected);
	}
	
	/**
	 * Returns the horizontal offset of the tabs.
	 * @return A positive integer representing the horizontal offset of the tabs.
	 */
	protected int getXOffset()
	{
		for (int i = 0; i < tabPane.getComponentCount(); i++)
		{
			Component	comp = tabPane.getComponent(i);
			
			if (comp instanceof JViewport)
			{
				Rectangle	viewPort = ((JViewport)comp).getViewRect();
				
				return viewPort.x;
			}
		}
		
		return 0;
	}

	/**
	 * Sets the image used for the close button.
	 * 
	 * @param closeImage the image used for the close button.
	 */
	public void setCloseImage(Image closeImage, Image closeOverImage)
	{
		_closeImage = closeImage;
		_closeOverImage = closeOverImage;
	}
	
	/**
	 * Sets the close button that has the mouse over.
	 * 
	 * @param index index of the close button that has the mouse over.
	 */
	public void setMouseOverCloseButton(int index)
	{
		int		oldMouseOverCloseButton = _mouseOverCloseButton;
		
		_mouseOverCloseButton = index;
		
		if (oldMouseOverCloseButton != _mouseOverCloseButton)
			tabPane.repaint();
	}
	
	/**
	 * Return the index of the close button at given position.
	 * 
	 * @param point the position on the screen.
	 * @return the index of the close button at given position or -1 if no close button was found
	 * at given position.
	 */
	public int getCloseButtonAtPosition(Point point)
	{
		if (_closeImagePositions == null)
			return -1;
		
		Enumeration<Integer>	tabIndexes = _closeImagePositions.keys();
		Integer					tabIndex;
		Rectangle				rect;
		
		while (tabIndexes.hasMoreElements())
		{
			tabIndex = tabIndexes.nextElement();
			rect = _closeImagePositions.get(tabIndex);
			
			if (rect.contains(point))
				return tabIndex.intValue();
		}
		
		return -1;
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.plaf.basic.BasicTabbedPaneUI#calculateTabHeight(int, int, int)
	 */
	protected int calculateTabHeight(int tabPlacement, int tabIndex, int fontHeight)
	{
		int		result = super.calculateTabHeight(tabPlacement, tabIndex, fontHeight);
		Icon	icon = getIconForTab(tabIndex);
		
		if (icon != null)
			result = Math.max(result, icon.getIconHeight() + 6);

		if (_closeImage != null)
			result = Math.max(result, _closeImage.getHeight(tabPane) + 4);
		
		return result;
	}

	/**
	 * Overridden to compute tab width with bold font and close button if needed.
	 */
	protected int calculateTabWidth(int tabPlacement, int tabIndex, FontMetrics metrics)
	{
		Font	font = tabPane.getFont();
		
		if (_boldFont == null)
			_boldFont = font.deriveFont(Font.BOLD);

		tabPane.setFont(_boldFont);
		
		FontMetrics	fm = tabPane.getFontMetrics(_boldFont);
		int			result = super.calculateTabWidth(tabPlacement, tabIndex, fm);

		if (_closeImage != null)
			result += _closeImage.getWidth(tabPane);

		tabPane.setFont(font);
		
		return result;
	}
	
	/**
	 * Return the position of a given item.
	 * 
	 * @param label the label of the item.
	 * @return the position of the item whose label is given.
	 */
	public Rectangle getLabelBounds(String label)
	{
		if (_labelPositions == null)
			return null;
		
		return _labelPositions.get(label);
	}
} // end class LyJTabbedPaneUI
