/*
 * LySplittableCanvas.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.component;

import java.awt.*;
import java.awt.event.*;


/**
 * This interface defines an object than can be drawn in a canvas splitter.
 *
 * @version $Id: LySplittableCanvas.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 * @see     LyAbstractCanvasSplitterManager
 */
public interface LySplittableCanvas
{
	String __VERSION = "$Id: LySplittableCanvas.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Draws the canvas in a graphics context.
	 *
	 * @param gc      the graphical context
	 * @param xOffset the horizontal offset for the splitter drawing
	 * @param yOffset the vertical offset for the splitter drawing
	 */
	void drawInSplitter(Graphics gc, int xOffset, int yOffset);

	/**
	 * Sets the canvas splitter.
	 *
	 * @param splitter the splitter
	 */
	void setCanvasSplitter(LyAbstractCanvasSplitterManager splitter);

	/**
	 * Gets the scrollable size, which is equals to the planning size.
	 *
	 * @return the scrollable size
	 */
	Dimension getScrollableSize();

	/**
	 * Gets the top header height. It is the default height of the header.
	 *
	 * @return the top header height
	 */
	int getTopHeaderHeight();

	/**
	 * Gets the left header width. Its value is 0 by default.
	 *
	 * @return the left header width
	 */
	int getLeftHeaderWidth();

	/**
	 * Gets all the mouse motion listeners.
	 *
	 * @return the mouse motion listeners on all the splitted components
	 */
	MouseMotionListener[] getMouseMotionListeners();

	/**
	 * Gets all the mouse listeners.
	 *
	 * @return the mouse listeners on all the splitted components
	 */
	MouseListener[] getMouseListeners();

	/**
	 * Gets all the key listeners.
	 *
	 * @return the key listeners on all the splitted components
	 */
	KeyListener[] getKeyListeners();
}