/*
 * LyCellEditor.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.table;

import java.awt.*;
import javax.swing.*;
import leon.view.win.swing.*;


/**
 * The default editor for table and tree cells.
 *
 * @version $Id: LyCellEditor.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public class LyCellEditor extends DefaultCellEditor
{
	private static final long  serialVersionUID = -7388761004188602488L;

	public static final String __VERSION =
		"$Id: LyCellEditor.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Default cell editor constructor.
	 */
	public LyCellEditor()
	{
		super(new JTextField());

		setClickCountToStart(1);
	}

	/**
	 * Sets an initial value for the editor. This will cause the editor to stop editing and lose any
	 * partially edited value if the editor is editing when this method is called.
	 *
	 * @param  table      the <code>JTable</code> that is asking the editor to edit; can be <code>
	 *                    null</code>
	 * @param  value      the value of the cell to be edited; it is up to the specific editor to
	 *                    interpret and draw the value. For example, if value is the string "true",
	 *                    it could be rendered as a string or it could be rendered as a check box
	 *                    that is checked. <code>null</code> is a valid value
	 * @param  isSelected true if the cell is to be rendered with highlighting
	 * @param  row        the row of the cell being edited
	 * @param  column     the column of the cell being edited
	 * @return the component that should be added to the client's Component hierarchy. Once
	 *         installed in the client's hierarchy this component will then be able to draw and
	 *         receive user input.
	 */
	public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected,
		int row, int column)
	{
		if (value instanceof LySwingFormField)
			return ((LySwingFormField)value).getFullComponent();
		if (value instanceof LySwingToolBar)
			return ((LySwingToolBar)value).getComponent();
		return super.getTableCellEditorComponent(table, value, isSelected, row, column);
	}
} // end class LyCellEditor
