/*
 * LyJCellRenderer.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.table;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import leon.misc.*;
import leon.view.*;
import leon.view.win.swing.*;
import leon.view.win.swing.component.*;


/**
 * This class configures the style of a cell during display.
 *
 * @version $Id: LyJCellRenderer.java 41522 2012-11-14 14:07:44Z bangiole $
 * @author  Lyria
 */
public class LyJCellRenderer extends DefaultTableCellRenderer
{
	private static final long  serialVersionUID = -476484669348041065L;

	public static final String __VERSION =
		"$Id: LyJCellRenderer.java 41522 2012-11-14 14:07:44Z bangiole $";
	/** Offset used for cursors. */
	public static final int    HIERARCHY_OFFSET_WIDTH = 13;

	/** Border of a normal cell. */
	protected Border		   _normalCellBorder;

	/** Odd color of the cell. */
	protected Color			   _oddColor;

	/** Even color of the cell. */
	protected Color			   _evenColor;

	/** Alignment of the cells. */
	protected int[]			   _aligns;

	/** Alignment of current cell. */
	protected int			   _align;

	/** Image used for editable cells. */
	protected Image			   _editionImage;


	/**
	 * Creates a cell renderer with the specified attributes.
	 *
	 * @param oddColor  the odd color of the cell
	 * @param evenColor the even color of the cell
	 * @param aligns    the alignment of the cells
	 */
	public LyJCellRenderer(Color oddColor, Color evenColor, int[] aligns)
	{
		super();

		_normalCellBorder = null;
		_oddColor = oddColor;
		_evenColor = evenColor;
		_aligns = aligns;
		_align = -1;
		_editionImage = null;
	}

	/**
	 * (non-Javadoc).
	 * @param reste force a deeper updat (fonts, backgrounds, component ...), false for a simple repaint
	 * true when resetting table or cell. If false simply return the JComponent instance
	 * @see javax.swing.table.TableCellRenderer#getTableCellRendererComponent(javax.swing.JTable,
	 *      java.lang.Object, boolean, boolean, int, int)
	 */
	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected,
		boolean hasFocus, int row, int column)
	{
		// Call the super method to correctly configure background color and other parameter
		// Comp is equal to 'this'
		LyJTableComponent tableComp = (LyJTableComponent)table;
		LyJTableModel     tableModel = (LyJTableModel)table.getModel();
		LySwingRow		  swingRow = null;
		Color			  background = null;
		JComponent		  cellComp;
		boolean			  setBackground = true;
		Object			  labelValue = value;

		Object lock =tableModel.getLock();
		synchronized(lock)
		{
			if (row < tableModel.getDataRowCount())
			{
				swingRow = tableModel.getRowAt(row);
				background = swingRow.getColor();

				if (!swingRow.isSensitive())
					isSelected = false;
			}
			else
				background = null;

			setEditionImage(null);
			_align = -1;

			// Retrieve alignment for current cell
			if ((_aligns != null) && (_aligns.length != 0))
			{
				if (column < _aligns.length)
					_align = _aligns[column];
			}

			// Convert peer value to a displayable value
			if ((value == LyJTableModel.EMPTY_CELL_VALUE)
					|| (value == LyJTableModel.EMPTY_CELL_STATUS_VALUE))
			{
				labelValue = "";
			}
			else if (value instanceof LySwingCell)
			{
				LySwingCell cell = ((LySwingCell)value);
				String	    label = cell.getLabel();

				if ((label != null)
					&& (label.indexOf('\n') >= 0))
				{
                    label = LyString.replace(label, "&", "&amp;");
                    label = LyString.replace(label, "\"", "&quot;");
                    label = LyString.replace(label, "<", "&lt;");
                    label = LyString.replace(label, ">", "&gt;");

					label = LyString.replace(label, "\r\n", "<br>");
					label = LyString.replace(label, "\n", "<br>");

					label = new StringBuffer(label.length()+12).append("<html>").append(label).append("</html>").toString();
				}
				if (label == null)
					labelValue = "";
				else
					labelValue = label;
			}


			if (value instanceof LySwingFormField)
			{
				cellComp = (JComponent)((LySwingFormField)value).getFullComponent();

				// Do not change background for text fields, selectors nor combo boxes.
				if ((cellComp instanceof javax.swing.text.JTextComponent)
						|| (cellComp instanceof LySwingText._LyJScrollPane)
						|| (cellComp instanceof LyJSelector)
						|| (cellComp instanceof LyJMultiSelector)
						|| ((cellComp != null)
								&& (cellComp.getComponentCount() > 0)
								&& (cellComp.getComponent(0) instanceof LyJMultiSelector))
						|| (cellComp instanceof JComboBox))
					setBackground = false;
			}
			else if ((value instanceof LySwingCell)
					&& (((LyCell)((LySwingCell)value).getViewComponent()).getCellCount() > 0))
			{
				LySwingCell swingCell = (LySwingCell)value;

				cellComp = (JComponent)swingCell.getComponent();

				if (!(cellComp instanceof _LyJMultiCell))
				{
					cellComp = new _LyJMultiCell(tableComp, (LyCell)swingCell.getViewComponent(), row,
						table.getForeground(), table.getSelectionForeground(), table.getGridColor(),
						isSelected, table.getSelectionBackground(), _align);
					swingCell.setComponent(cellComp);
				}
				else
				{
					_LyJMultiCell	multiCell = (_LyJMultiCell)cellComp;

					multiCell._selected = isSelected;
					multiCell._align = _align;
					multiCell._row = row;
				}

				cellComp.setFont(table.getFont());
			}
			else if ((value instanceof LySwingCell)
				&& ((LyCell)((LySwingCell)value).getViewComponent()).isEditable())
			{
				LySwingCell swingCell = (LySwingCell)value;

				cellComp = (JComponent)super.getTableCellRendererComponent(table, labelValue, isSelected, hasFocus, row, column);

				setEditionImage(swingCell.getEditionImage());
				tableComp.setEditableCells(true);
			}
			else if (value instanceof LySwingToolBar)
			{
				cellComp = (JComponent)((LySwingToolBar)value).getComponent();
			}
			else
			{
				cellComp = (JComponent)super.getTableCellRendererComponent(table, labelValue, isSelected, hasFocus, row, column);
			}

			// This component must paint its background
			if (cellComp != null)
			{
				cellComp.setOpaque(true);

				if (!isSelected && setBackground)
				{
					if (background != null)
						cellComp.setBackground(background);
					else if ((row % 2) == 0)
						cellComp.setBackground(_oddColor);
					else
						cellComp.setBackground(_evenColor);
				}

				if (setBackground
					&& isSelected
					&& ((value instanceof LySwingFormField) || (value instanceof LySwingToolBar)))
				{
					cellComp.setBackground(table.getSelectionBackground());
				}
			}
			if (value instanceof LySwingCell)
			{
				LySwingCell cell = ((LySwingCell)value);
				Image	    image = cell.getImage();
				ImageIcon   icon;
				Color	    color = cell.getColor();
				Color	    fontColor = cell.getFontColor();
				Font	    font = cell.getFont();

				if (font != null)
					cellComp.setFont(font);
				if (!isSelected)
					cellComp.setForeground(fontColor);

				if ((color != null) && !isSelected)
					cellComp.setBackground(color);

				if (((LyCell)((LySwingCell)value).getViewComponent()).getCellCount() <= 0)
				{
					if (image != null)
					{
						icon = new ImageIcon(image);
						((JLabel)cellComp).setIcon(icon);
					}
					else
						((JLabel)cellComp).setIcon(null);


					// Retrieve alignment for current cell
					if ((_aligns != null) && (_aligns.length != 0))
					{
						if (column < _aligns.length)
						{
							int align = _aligns[column];

							((JLabel)cellComp).setHorizontalAlignment(align);
						}
					}
				}

				// Case of hierarchical rows : use an image to open / close row
				LyRow		 parentRow = cell.getParentRow();

				if (parentRow != null)
				{
					LyRows		 rows = parentRow.getRows();
					LyTable		 viewTable = cell.getParentTable();
					LySwingTable swingTable = (LySwingTable)viewTable.getInternalView();
					int			 firstColumn = (swingTable.hasStatusColumn() ? 1 : 0);
					boolean		 hasCursor = ((rows != null) && (rows.getRowCount() > 0) && (column == firstColumn));
					int			 offset = 0;

					// Compute the offset for first column only
					if (swingTable.isHierarchical() && (column == firstColumn))
					{
						swingRow = (LySwingRow)parentRow.getInternalView();
						offset = swingRow.getDepth();
					}

					if (hasCursor)
					{
						if (parentRow.isOpen())
							icon = new ImageIcon(swingRow.getOpenRowImage());
						else
							icon = new ImageIcon(swingRow.getClosedRowImage());

						((JLabel)cellComp).setIcon(icon);

						swingTable.addRowCursor(row, HIERARCHY_OFFSET_WIDTH * offset);
					}

					if ((offset > 0) && (cellComp != null) && !(cellComp instanceof _LyJCursorPanel))
					{
						cellComp = new _LyJCursorPanel(cellComp, HIERARCHY_OFFSET_WIDTH * offset);
						cell.setComponent(cellComp);
					}
				}
			}
			else if (value instanceof String)
			{
				LyRow		 parentRow = null;
				boolean		 hasCursor = false;

				if (swingRow != null)
					parentRow = (LyRow)swingRow.getViewComponent();

				// Case of hierarchical rows : use an image to open / close row
				if ((parentRow != null) && (parentRow.getRows() != null))
				{
					LyRows		 rows = parentRow.getRows();
					LyTable		 viewTable = swingRow.getParentTable();
					LySwingTable swingTable = (LySwingTable)viewTable.getInternalView();
					int			 firstColumn = (swingTable.hasStatusColumn() ? 1 : 0);

					hasCursor = ((rows != null) && (column == firstColumn));

					if (hasCursor)
					{
						int			 offset = 0;

						swingRow = (LySwingRow)parentRow.getInternalView();
						offset = swingRow.getDepth();

						if (!hasCursor)
							offset++;

						swingTable.addRowCursor(row, HIERARCHY_OFFSET_WIDTH * offset);
					}
				}

				// empty normal cell
				if (!hasCursor && value == LyJTableModel.EMPTY_CELL_VALUE)
				{
					((JLabel)cellComp).setIcon(null);

					if (row >= tableComp.getTableModel().getDataRowCount())
					{
						if ((row % 2) == 0)
							cellComp.setBackground(_oddColor);
						else
							cellComp.setBackground(_evenColor);
					}
				}
				// empty status cell
				else if (!hasCursor && value == LyJTableModel.EMPTY_CELL_STATUS_VALUE)
				{
					// do nothing since status column managed by its own component
				}
			}
			else if (value instanceof LySwingFormField)
			{
				// OK, just return
			}
			else if (value instanceof LySwingToolBar)
			{
				// OK, just return
				//cellComp = (JComponent)((LySwingToolBar)value).getComponent();
			}
			else
				// Unreachable
				System.out.println("LyJCellRenderer error : unrecognized value : " + (value == null ? value : value.getClass()));

			return cellComp;
		}
	} // end method getTableCellRendererComponent

	/**
	 * Switch two alignments in the table.
	 *
	 * @param index1 first index to be changed.
	 * @param index2 second index to be changed.
	 */
	public void switchAligns(int index1, int index2)
	{
		if ((index1 == index2)
				|| (index1 < 0)
				|| (index2 < 0)
				|| (index1 >= _aligns.length)
				|| (index2 >= _aligns.length))
			return;

		int		oldAlign1 = _aligns[index1];

		_aligns[index1] = _aligns[index2];
		_aligns[index2] = oldAlign1;
	}

	/**
	 * Computes the dimension of the specified row for a given font.
	 *
	 * @param  row         the parent component
	 * @param  currentFont a font
	 * @return the height of the row
	 */
	public int computeDimension(LyRow row, Font currentFont)
	{
		if ((row == null) || (row.getChildCount() == 0))
			return 0;

		int    i, n = row.getChildCount();
		int    height = 0;
		LyCell cell;

		for (i = 0; i < n; i++)
		{
			if (row.getChild(i) instanceof LyCell)
			{
				cell = (LyCell)row.getChild(i);
				height = Math.max(height, computeDimension(cell, currentFont).height);
			}
			else if (row.getChild(i) instanceof LyFormField)
			{
				LySwingFormField formField = (LySwingFormField)row.getChild(i).getInternalView();
				JComponent		 comp = (formField != null ? (JComponent)formField.getFullComponent() : null);

                if (comp != null)
                    height = Math.max(height, comp.getPreferredSize().height);
			}
			else if (row.getChild(i) instanceof LyToolBar)
			{
				LySwingToolBar	toolbar = (LySwingToolBar)row.getChild(i).getInternalView();
				JComponent		comp = (toolbar != null ? (JComponent)toolbar.getComponent() : null);

				if (comp != null)
					height = Math.max(height, comp.getPreferredSize().height);
			}
		}

		return height;
	}

	/**
	 * Compute the dimension of the specified cell for a given font.
	 *
	 * @param  cell        the parent component
	 * @param  currentFont a font
	 * @return the dimension of the cell
	 */
	public Dimension computeDimension(LyCell cell, Font currentFont)
	{
		int  width = 0, height = 2;
		Font font = currentFont;

		if (cell == null)
			return new Dimension(-1, -1);

		int i, n = cell.getCellCount();

		if (n > 0)
		{
			LyCell    subCell;
			int		  widthTmp = 0;
			Dimension dim;

			for (i = 0; i < n; i++)
			{
				subCell = cell.getCell(i);
				dim = computeDimension(subCell, font);
				widthTmp = Math.max(dim.width, widthTmp);
				height += dim.height;
			}

			width = widthTmp;
		}
		else
		{
			FontMetrics fm = getFontMetrics(font);
			int			labelHeight = fm.getHeight() + 2;
			String	    image = cell.getImage();
			LySwingCell internal = (LySwingCell)cell.getInternalView();
			String	    label = internal.getLabel();

			if (image != null)
			{
				Image	    img = internal.getImage();
				if (img != null)
				{
					width += img.getWidth(this);
					height = img.getHeight(this) + 2;
				}
			}

			if (label != null)
			{
				width += fm.stringWidth(label);

				if (label.indexOf('\n') >= 0)
				{
					String[]	list = LyString.stringToList(label, '\n');

					if ((list != null) && (list.length > 1))
						labelHeight += fm.getHeight() * (list.length - 1);
				}
			}

			height = Math.max(height, labelHeight);
		}

		String image = cell.getImage();

		if (image != null)
		{
			LySwingCell internal = (LySwingCell)cell.getInternalView();
			Image	    img = internal.getImage();

			if (img != null)
				height = Math.max(height, img.getHeight(this));
		}

		return new Dimension(width, height);
	} // end method computeDimension

	/**
	 * Set the image used for edition.
	 *
	 * @param editionImage the image for editable cells.
	 */
	protected void setEditionImage(Image editionImage)
	{
		_editionImage = editionImage;
	}

	/**
	 * Overridden to draw edition image if needed.
	 *
	 * @see javax.swing.JComponent#paintComponent(Graphics)
	 */
	protected void paintComponent(Graphics g)
	{
		super.paintComponent(g);

		if (_editionImage != null)
		{
			Dimension dim = getSize();

			if (_align == SwingConstants.RIGHT)
				g.drawImage(_editionImage, 1, 1, this);
			else
				g.drawImage(_editionImage, dim.width - _editionImage.getWidth(this) - 1, 1, this);
		}
	}


	/**
	 * This class represents a cell with a cursor.
	 */
	class _LyJCursorPanel extends JPanel
	{
		private static final long serialVersionUID = -3048245125660238140L;

		/**
		 * Creates a new panel that contains the specified component with a pointing cursor of the
		 * specified offset width.
		 *
		 * @param comp   the cell
		 * @param offset the width of the cursor
		 */
		_LyJCursorPanel(Component comp, int offset)
		{
			super();

			JPanel offsetPanel = new JPanel();

			setLayout(new BorderLayout(0, 0));
			setBackground(comp.getBackground());

			offsetPanel.setOpaque(false);
			offsetPanel.setPreferredSize(new Dimension(offset, 5));

			add(offsetPanel, BorderLayout.WEST);
			add(comp, BorderLayout.CENTER);
		}
	}


	/**
	 * This class represents a cell with sub-cells.
	 */
	public class _LyJMultiCell extends JPanel
	{
		private static final long serialVersionUID = -395930739150953030L;

		/** Instance of LyJTableComponent where cell is displayed. */
		LyJTableComponent		  _table;

		/** The cell associated with this component. */
		LyCell					  _cell;

		/** Row index of the cell in the table. */
		int						  _row;

		/** Color used for font. */
		Color					  _fontColor;

		/** Color used for lines. */
		Color					  _gridColor;

		/** Color used for background when selected. */
		Color					  _selectionColor;

		/** Color used for font when selected. */
		Color					  _selectedFontColor;

		/** Indicates that this cell is selected. */
		boolean					  _selected;

		/** Alignement of the cell. */
		int						  _align;

		/**
		 * Creates a new multi-cell object with the specified attributes.
		 *
		 * @param table             the table component
		 * @param cell              the cell component
		 * @param row               the row index of this cell
		 * @param fontColor         the font color
		 * @param selectedFontColor the selected font color
		 * @param gridColor         the color of the lines
		 * @param selected          Indicates that this cell is selected.
		 * @param selectionColor    the background color when selected
		 * @param align             the alignment policy of this cell
		 */
		public _LyJMultiCell(LyJTableComponent table, LyCell cell, int row, Color fontColor,
			Color selectedFontColor, Color gridColor, boolean selected, Color selectionColor,
			int align)
		{
			super();

			_table = table;
			_cell = cell;
			_row = row;
			_fontColor = fontColor;
			_gridColor = gridColor;
			_selectionColor = selectionColor;
			_selectedFontColor = selectedFontColor;
			_selected = selected;
			_align = align;
		}

		/*
		 * (non-Javadoc)
		 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
		 */
		public void paintComponent(Graphics g)
		{
			Dimension dim = getSize();

			// Set anti-alias
			((Graphics2D)g).setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
			((Graphics2D)g).setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
				RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

			if (_selected)
				g.setColor(_selectionColor);
			else
				g.setColor(getBackground());

			g.fillRect(0, 0, dim.width, dim.height);

			drawCell(g, _cell, 2, dim.width, dim.height, null);
		}

		/**
		 * Draws a the specified cell.
		 *
		 * @param g      the graphic context
		 * @param cell   the instance of LyCell to be drawn
		 * @param y      the y coordonate
		 * @param width  the width of the cell
		 * @param height the height of the cell
		 * @param level  the level of the cell in its row
		 */
		protected void drawCell(Graphics g, LyCell cell, int y, int width, int height, String level)
		{
			if (cell == null)
				return;

			if (level == null)
				level = "";

			int    i, n = cell.getCellCount();
			float  subHeight;
			LyCell subCell;

			if (n > 0)
			{
				for (i = 0; i < n; i++)
				{
					subCell = cell.getCell(i);
					subHeight = _table.computeLevelHeight(_row, level + LyTools.PIPE_SEP + i);

					drawCell(g, subCell, y, width, (int)subHeight, level + LyTools.PIPE_SEP + i);

					y += subHeight;

					if (i != (n - 1))
					{
						g.setColor(_gridColor);
						g.drawLine(-1, y - 1, width - 1, y - 1);
					}
				}
			}
			else
			{
				if (_selected)
					g.setColor(_selectedFontColor);
				else
					g.setColor(_fontColor);

				LySwingCell internal = (LySwingCell)cell.getInternalView();
				FontMetrics fm = getFontMetrics(getFont());
				int		    x = 1;
				int		    compWidth;
				Image	    image = internal.getImage();
				int		    labelOffset = 0;
				Color	    color = internal.getColor();
				Font	    font = internal.getFont();
				String		label = cell.getLabel();
				String		fullLabel = label;

				if (font != null)
					g.setFont(font);

				if ((color != null) && !_selected)
				{
					Color oldColor = g.getColor();

					g.setColor(color);
					g.fillRect(x, y, width, height);
					g.setColor(oldColor);
				}

				if ((fullLabel != null) && (fullLabel.indexOf('\n') < 0))
				{
					while ((fm.stringWidth(fullLabel) >= width) && (label.length() > 0))
					{
						label = label.substring(0, label.length() - 1);
						fullLabel = label + "...";
					}
				}

				compWidth = ((fullLabel == null) ? 0 : fm.stringWidth(fullLabel));

				if (image != null)
				{
					compWidth += image.getWidth(this);
					labelOffset += image.getWidth(this) + 1;
				}

				if (_align == SwingConstants.RIGHT)
					x = width - compWidth;
				else if (_align == SwingConstants.CENTER)
					x = (width - compWidth) / 2;

				if (image != null)
					g.drawImage(image, x, y + ((height - image.getHeight(this)) / 2), this);

				if (fullLabel != null)
				{
					if (fullLabel.indexOf('\n') < 0)
						g.drawString(fullLabel, x + labelOffset, y + ((height - fm.getHeight()) / 2) + fm.getAscent());
					else
					{
						String[]	list = LyString.stringToList(label, '\n');

						if ((list != null) && (list.length > 1))
						{
							height = (height / list.length);

							for (i = 0; i < list.length; i++)
							{
								int	newY = y + ((height - fm.getHeight()) / 2) + fm.getAscent() + (height * i);

								g.drawString(list[i], x + labelOffset, newY);
							}
						}
					}
				}
			}
		} // end method drawCell

		/**
		 * Computes the height of current cell and sub-cells and adds it to the list of heights with
		 * their level.
		 *
		 * @param heights     the list of cell heights
		 * @param level       the level for current cell
		 * @param cell        the cell to compute its height
		 * @param table       the table container
		 * @param currentFont a font
		 */
		public void computeCellHeights(Hashtable<String, Integer> heights, LyCell cell, String level,
			LyJTableComponent table, Font currentFont)
		{
			if (cell == null)
				return;

			if (level == null)
				level = "";

			int     height = computeDimension(cell, currentFont).height;
			Integer oldHeight = heights.get(level);

			if ((oldHeight == null) || (oldHeight.intValue() < height))
				heights.put(level, new Integer(height));

			// Recurse with sub-cells
			if (cell.getCellCount() > 0)
			{
				int    i, n = cell.getCellCount();
				LyCell subCell;

				for (i = 0; i < n; i++)
				{
					subCell = cell.getCell(i);
					computeCellHeights(heights, subCell, level + LyTools.PIPE_SEP + i, table,
						currentFont);
				}
			}
		} // end method computeCellHeights

		/*
		 * (non-Javadoc)
		 * @see java.awt.Component#getMinimumSize()
		 */
		public Dimension getMinimumSize()
		{
			return computeDimension(_cell, getFont());
		}

		/*
		 * (non-Javadoc)
		 * @see java.awt.Component#getPreferredSize()
		 */
		public Dimension getPreferredSize()
		{
			return getMinimumSize();
		}

		/**
		 * Gets the cell of this component.
		 *
		 * @return the associated cell
		 */
		public LyCell getCell()
		{
			return _cell;
		}
	} // end class _LyJMultiCell
} // end class LyJCellRenderer
