/*
 * LyJFooterCellBorder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.table;

import java.awt.*;


/**
 * This class defines a special border for status, header and footer cells in the table.
 *
 * @version $Id: LyJFooterCellBorder.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public class LyJFooterCellBorder extends LyJSpecialCellBorder
{
	private static final long  serialVersionUID = 3932242770968155710L;
	public static final String __VERSION =
		"$Id: LyJFooterCellBorder.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Creates a border with default top,bottom, right and left inset.
	 */
	public LyJFooterCellBorder()
	{
		this(true);
	}

	/**
	 * Creates a border with default top,bottom, right and left inset.
	 * 
	 * @param drawSides indicates if left and right borders must be drawn.
	 */
	public LyJFooterCellBorder(boolean drawSides)
	{
		this(2, 1, 1, 1, drawSides);
	}

	/**
	 * Creates a border object with the specified insets.
	 *
	 * @param top    the top inset, in pixels
	 * @param left   the left inset, in pixels
	 * @param bottom the bottom inset, in pixels
	 * @param right  the top inset, in pixels
	 */
	public LyJFooterCellBorder(int top, int left, int bottom, int right)
	{
		this(top, left, bottom, right, true);
	}

	/**
	 * Creates a border with default top,bottom, right and left inset.
	 * 
	 * @param top    the top inset, in pixels
	 * @param left   the left inset, in pixels
	 * @param bottom the bottom inset, in pixels
	 * @param right  the top inset, in pixels
	 * @param drawSides indicates if left and right borders must be drawn.
	 */
	public LyJFooterCellBorder(int top, int left, int bottom, int right, boolean drawSides)
	{
		super(top, left, bottom, right, drawSides);
	}

	/* (non-Javadoc)
	 * @see javax.swing.border.Border#isBorderOpaque()
	 */
	public boolean isBorderOpaque()
	{
		return false;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see javax.swing.border.Border#paintBorder(java.awt.Component, java.awt.Graphics, int, int,
	 *      int, int)
	 */
	public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)
	{
		g.setColor(Color.white);
		
		if (_drawLeftSide)
			g.drawLine(x, y, x, y + height);
		
		g.drawLine(x, y + 1, x + width, y + 1);
		g.setColor(Color.gray);
		
		if (_drawRightSide)
			g.drawLine(x + width - 1, y, x + width - 1, y + height);
		
		g.drawLine(x, y, x + width, y);
	}

} // end class LyJFooterCellBorder
