/*
 * LyJSpecialCellBorder.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.table;

import java.awt.*;
import javax.swing.border.*;


/**
 * This class defines a special border for status, header and footer cell in the table.
 *
 * @version $Id: LyJSpecialCellBorder.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public class LyJSpecialCellBorder extends EmptyBorder
{
	private static final long  serialVersionUID = 53489353348383346L;

	public static final String __VERSION =
		"$Id: LyJSpecialCellBorder.java 28422 2010-11-03 12:03:38Z ehelmste $";

	protected boolean	_drawLeftSide;
	protected boolean	_drawRightSide;
	
	/**
	 * Creates a border with default top,bottom, right and left inset.
	 */
	public LyJSpecialCellBorder()
	{
		this(true);
	}

	/**
	 * Creates a border with default top,bottom, right and left inset.
	 * 
	 * @param drawSides indicates if left and right borders must be drawn.
	 */
	public LyJSpecialCellBorder(boolean drawSides)
	{
		this(1, 1, 1, 1, drawSides);
	}

	/**
	 * Creates a special border object with the specified insets.
	 *
	 * @param top    the top inset, in pixels
	 * @param left   the left inset, in pixels
	 * @param bottom the bottom inset, in pixels
	 * @param right  the top inset, in pixels
	 */
	public LyJSpecialCellBorder(int top, int left, int bottom, int right)
	{
		this(top, left, bottom, right, true);
	}

	/**
	 * Creates a special border object with the specified insets.
	 *
	 * @param top    the top inset, in pixels
	 * @param left   the left inset, in pixels
	 * @param bottom the bottom inset, in pixels
	 * @param right  the top inset, in pixels
	 * @param drawSides indicates if left and right borders must be drawn.
	 */
	public LyJSpecialCellBorder(int top, int left, int bottom, int right, boolean drawSides)
	{
		this(top, left, bottom, right, drawSides, drawSides);
	}

	/**
	 * Creates a special border object with the specified insets.
	 *
	 * @param top    the top inset, in pixels
	 * @param left   the left inset, in pixels
	 * @param bottom the bottom inset, in pixels
	 * @param right  the top inset, in pixels
	 * @param drawLeftSide indicates if left border must be drawn.
	 * @param drawRightSide indicates if right border must be drawn.
	 */
	public LyJSpecialCellBorder(int top, int left, int bottom, int right, boolean drawLeftSide, boolean drawRightSide)
	{
		super(top, left, bottom, right);
		
		_drawLeftSide = drawLeftSide;
		_drawRightSide = drawRightSide;
	}

	/**
	 * This border never draw its background.
	 *
	 * @return <code>false</code>.
	 * @see    javax.swing.border.Border#isBorderOpaque()
	 */
	public boolean isBorderOpaque()
	{
		return false;
	}

	/**
	 * (non-Javadoc).
	 *
	 * @see javax.swing.border.Border#paintBorder(java.awt.Component, java.awt.Graphics, int, int,
	 *      int, int)
	 */
	public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)
	{
		g.setColor(Color.white);
		
		if (_drawLeftSide)
			g.drawLine(x, y, x, y + height);
		
		g.drawLine(x, y, x + width, y);
		g.setColor(Color.gray);
		
		if (_drawRightSide)
			g.drawLine(x + width - 1, y, x + width - 1, y + height);
		
		g.drawLine(x, y + height - 1, x + width, y + height - 1);
	}

} // end class LyJSpecialCellBorder
