/*
 * LyJTableColumn.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.table;

import javax.swing.table.*;


/**
 * This class implements the properties of a table column. It has been created to access the footer
 * value of a column.
 *
 * @version $Id: LyJTableColumn.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public class LyJTableColumn extends TableColumn
{
	private static final long  serialVersionUID = -3022781205883978184L;

	public static final String __VERSION =
		"$Id: LyJTableColumn.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** The alignment of the footer. */
	protected int			   _align;

	/** The value used to display the footer. */
	protected Object		   _footerValue;

	/** Tooltip of the header. */
	String					   _tooltip;

	/**
	 * Default constructor.
	 */
	public LyJTableColumn()
	{
		super();
	}

	/**
	 * Creates a table column object with the specified column model.
	 *
	 * @param modelIndex the index of this column in the columnModel
	 */
	public LyJTableColumn(int modelIndex)
	{
		super(modelIndex);
	}

	/**
	 * Creates a table column object with the specified attributes.
	 *
	 * @param modelIndex the index of this column in the columnModel
	 * @param width      the initial width of this column
	 */
	public LyJTableColumn(int modelIndex, int width)
	{
		super(modelIndex, width);
	}

	/**
	 * Creates a table column object with the specified attributes.
	 *
	 * @param modelIndex   the index of this column in the columnModel
	 * @param width        the initial width of this column
	 * @param cellRenderer the component factory used to render a cell
	 * @param cellEditor   the component factory used to render an editing cell
	 */
	public LyJTableColumn(int modelIndex, int width, TableCellRenderer cellRenderer,
		TableCellEditor cellEditor)
	{
		super(modelIndex, width, cellRenderer, cellEditor);
	}

	/**
	 * Gets the value of the footer.
	 *
	 * @return the value of the column's footer
	 * @see    #setFooterValue(Object)
	 */
	public Object getFooterValue()
	{
		return _footerValue;
	}

	/**
	 * Gets the alignment of this footer.
	 *
	 * @return the alignment policy of the column's footer
	 * @see    #setFooterAlign(int)
	 */
	public int getFooterAlign()
	{
		return _align;
	}

	/**
	 * Sets the value of the column's footer.
	 *
	 * @param value the new value of this footer
	 * @see   #getFooterValue()
	 */
	public void setFooterValue(Object value)
	{
		_footerValue = value;
	}

	/**
	 * Sets the alignment of the column's footer.
	 *
	 * @param align the new alignment of this footer
	 * @see   #getFooterAlign()
	 */
	public void setFooterAlign(int align)
	{
		_align = align;
	}

	/**
	 * Gets value of tooltip.
	 *
	 * @return the value of tooltip
	 * @see    #setTooltip(String)
	 */
	public String getTooltip()
	{
		return _tooltip;
	}

	/**
	 * Sets attribute tooltip to the given value.
	 *
	 * @param tooltip the new value of tooltip
	 * @see   #getTooltip()
	 */
	public void setTooltip(String tooltip)
	{
		_tooltip = tooltip;
	}

} // end class LyJTableColumn
