/*
 * LyJTableColumnModel.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.table;

import javax.swing.table.*;
import leon.view.win.swing.*;


/**
 * This class is used in the management of the columns. It prevents from moving the filled space
 * column (last column) and the status column (invisible) if table has one.
 *
 * @version $Id: LyJTableColumnModel.java 32173 2011-09-02 13:05:15Z nhamon $
 * @author  Lyria
 */
public class LyJTableColumnModel extends DefaultTableColumnModel
{
	private static final long  serialVersionUID = 3014182447012862461L;

	public static final String __VERSION =
		"$Id: LyJTableColumnModel.java 32173 2011-09-02 13:05:15Z nhamon $";

	/** The associated table manager. */
	LySwingTable			   _swingTable;

	/**
	 * Creates a column model for the specified table.
	 *
	 * @param swingTable the table to assign a column model
	 */
	public LyJTableColumnModel(LySwingTable swingTable)
	{
		super();
		_swingTable = swingTable;

		// FFT00974
		setColumnMargin(1);
	}
	
	/**
	 * Returns the associated table manager.
	 */
	public LySwingTable getSwingTable()
	{
		return _swingTable;
	}

	/**
	 * Swaps the two specified columns. This method makes sure that the last column and the status
	 * column are not swapable.
	 *
	 * @see javax.swing.table.TableColumnModel#moveColumn(int, int)
	 */
	public void moveColumn(int columnIndex, int newIndex)
	{
		int columnCount = getColumnCount();
		
		LyJCellRenderer	renderer = (LyJCellRenderer)_swingTable.getTableComponent().getCellRenderer(0, 0);

		if (columnIndex == (columnCount - 1))
			newIndex = columnIndex;
		else if (newIndex == (columnCount - 1))
			newIndex = columnIndex;
		else if (_swingTable.hasStatusColumn())
		{
			if (columnIndex == 0)
				newIndex = columnIndex;
			else if (newIndex == 0)
				newIndex = columnIndex;
		}
		
		renderer.switchAligns(columnIndex, newIndex);
		super.moveColumn(columnIndex, newIndex);
	}

} // end class LyJTableColumnModel
