/*
 * LyJTableSelectionListener.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.table;

import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import leon.view.event.*;
import leon.view.win.swing.*;


/**
 * This class manages the selection events from the view to the model.
 *
 * @version $Id: LyJTableSelectionListener.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public class LyJTableSelectionListener
	implements ListSelectionListener
{
	public static final String __VERSION =
		"$Id: LyJTableSelectionListener.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** The manager of the table component Used to access the table. */
	protected LySwingTable     _swingTable;

	/** workaround : force notification on select all fo drag and drop. */
	protected boolean		   _forceNotifySelectAll;

	/**
	 * Constructor. Initializes the swing table and set the force notification to false.
	 *
	 * @param swingTable the manager of the table
	 */
	public LyJTableSelectionListener(LySwingTable swingTable)
	{
		_swingTable = swingTable;
		_forceNotifySelectAll = false;
	}

	/**
	 * Called whenever the value of the selection changes.
	 *
	 * @param e the event that characterizes the change.
	 */
	public void valueChanged(ListSelectionEvent e)
	{
		// MODE SELECTION_NONE
		if (!_swingTable.getTableComponent().getRowSelectionAllowed())
			return;
		// A NON-DATA ROW HAS BEEN SELECTED
		if (e.getFirstIndex() >= _swingTable.getTableModel().getDataRowCount())
			return;
			// FFT 1112 A (UN)SELECT ALL EVENT is handled as a single event

		Vector<String> selectedIds;

		if (!e.getValueIsAdjusting())
		{
			_forceNotifySelectAll = false;

			selectedIds = getSelectedIds();
			if (_swingTable.getTableComponent().getSelectionTransmitter() != null)
			{
				LySelectionEvent select = new LySelectionEvent(this, selectedIds, true);

				select.addIndex(e.getFirstIndex());
				select.setSelectionSize(selectedIds.size());

				select.setExtend(true);

				_swingTable.getTableComponent().selectionPerformed(select);
			}
		}
	} // end method valueChanged

	/**
	 * Gets the selected rows ids.
	 *
	 * @return the ids of the selected rows
	 */
	public Vector<String> getSelectedIds()
	{
		Vector<String>	   selectedIds = new Vector<String>();
		ListSelectionModel selection = _swingTable.getTableComponent().getSelectionModel();
		int				   i, n = (_swingTable.getTableModel()).getDataRowCount();

		for (i = 0; i < n; i++)
		{
			if (selection.isSelectedIndex(i))
			{
				String id = _swingTable.getRow(i).getId();

				if (id != null)
					selectedIds.add(id);
			}
		}
		return selectedIds;
	}
} // end class LyJTableSelectionListener
