/*
 * LyJTableStatusHeader.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.table;

import java.awt.*;
import javax.swing.*;
import leon.view.win.swing.*;
import leon.view.win.swing.component.*;


/**
 * This class manages the header of the status column. It can display a sort icon.
 *
 * @version $Id: LyJTableStatusHeader.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public class LyJTableStatusHeader extends JLabel
{
	private static final long   serialVersionUID = -264684052333058889L;

	public static final String  __VERSION =
		"$Id: LyJTableStatusHeader.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** The model that contains all table data. */
	protected LyJTableModel     _model;

	/** The graphical component that diplay data in a table. */
	protected LyJTableComponent _tableComp;

	/**
	 * Constructor. Calls the super implementation then initializes the border, font, opacity,
	 * background color horizontal alignment model and table component.
	 *
	 * @param font      the table font
	 * @param color     the table background color
	 * @param model     the data model of the table
	 * @param tableComp the table graphical component
	 */
	public LyJTableStatusHeader(Font font, Color color, LyJTableModel model,
		LyJTableComponent tableComp)
	{
		super();
		setBorder(new LyJSpecialCellBorder());
		setFont(font);
		setOpaque(true);
		setBackground(color);
		setHorizontalAlignment(SwingConstants.CENTER);
		_model = model;
		_tableComp = tableComp;
	}

	/**
	 * Calls by the mouseListener to update the header.
	 */
	public void updateHeader()
	{
		LySwingHeader header = (LySwingHeader)_model.getColumnHeaderValue(0);

		if (header.getSort() >= 0)
			setIcon((header.isIncreasing()) ? (_tableComp.getSortUpIcon())
											: (_tableComp.getSortDownIcon()));
		else
			setIcon(null);
	}
} // end class LyJTableStatusHeader
