/*
 * LyJFormattedTextDocument.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.text;

import java.awt.*;
import javax.swing.text.*;
import leon.view.win.swing.component.*;


/**
 * This class implements controls on its associated JTextComponent.
 *
 * @version $Id: LyJFormattedTextDocument.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 * @see     leon.view.win.swing.component.LyJFormattedField
 */
public class LyJFormattedTextDocument extends PlainDocument
{
	private static final long   serialVersionUID = 7886545894141075198L;

	public static final String  __VERSION =
		"$Id: LyJFormattedTextDocument.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/** The format that must be respected by this document. */
	protected LyJFormat		    _format;

	/** The associated JTextComponent. */
	protected LyJFormattedField _comp;

	/**
	 * Constructor. Calls the super implementation and initializes class attributes with the given
	 * values.
	 *
	 * @param format the format to respect
	 * @param comp   the associated JTextComponent
	 */
	public LyJFormattedTextDocument(LyJFormat format, LyJFormattedField comp)
	{
		super();
		_format = format;
		_comp = comp;
	}

	/**
	 * Emit a beep.
	 */
	public void beep()
	{
		Toolkit.getDefaultToolkit().beep();
	}

	/**
	 * Method overriden to control the user's insertions to allow only well-formed inputs.
	 */
	public void insertString(int offset, String str, AttributeSet a)
		throws BadLocationException
	{
		if (str.length() > 0)
		{
			char[] chars = str.toCharArray();

			for (int i = 0; i < chars.length; i++)
				if (_format.isCharInsertionValid(chars[i], offset))
				{
					_format.replaceChar(offset, chars[i]);
					replaceChar(offset, chars[i], a);
				}
				else
				{
					beep();
					return;
				}
		}
	}

	/**
	 * Method overriden to control the user's deletion to allow only well-formed inputs.
	 */
	public void remove(int offs, int len)
		throws BadLocationException
	{
		for (int i = 0; i < len; i++)
		{
			if (_format.isCharDeletionValid(offs + i))
			{
				_format.replaceChar(offs + i, _format.EMPTY_CHAR);
				replaceChar(offs + i, _format.EMPTY_CHAR, null);
			}
			_comp.getCaret().setDot(offs);
		}
	}

	/**
	 * Replaces the character at pos offset by c.
	 *
	 * @param  offset the position of the replacement
	 * @param  c      the new char
	 * @param  a      the AttibuteSet, not used here
	 * @throws BadLocationException
	 */
	public void replaceChar(int offset, char c, AttributeSet a)
		throws BadLocationException
	{
		super.insertString(offset, String.valueOf(c), a);
		super.remove(offset + 1, 1);
	}

	/**
	 * Sets the new text contained by this document.
	 *
	 * @param  text the new text
	 * @throws BadLocationException
	 */
	public void setDisplayableString(String text)
		throws BadLocationException
	{
		int length = getLength();
		int dot = _comp.getCaret().getDot();

		_format.setValue(text);

		super.remove(0, length);
		super.insertString(0, text, null);
		_comp.getCaret().setDot(dot);
	}

} // end class LyJFormattedTextDocument
