/*
 * LyJTextFieldDocument.java
 * Copyright (c) 2000-2009, Lyria-W4.
 * All rights reserved.
 */

package leon.view.win.swing.text;

import javax.swing.text.*;


/**
 * This class manages the content of a one-lined text field.
 *
 * @version $Id: LyJTextFieldDocument.java 28422 2010-11-03 12:03:38Z ehelmste $
 * @author  Lyria
 */
public class LyJTextFieldDocument extends LyJTextAreaDocument
{
	private static final long  serialVersionUID = -5419641805954714031L;
	public static final String __VERSION =
		"$Id: LyJTextFieldDocument.java 28422 2010-11-03 12:03:38Z ehelmste $";

	/**
	 * Constructor. Call the super implementation.
	 *
	 * @param maxChars The maximum number of char of this text area
	 */
	public LyJTextFieldDocument(int maxChars)
	{
		super(maxChars);
	}

	/**
	 * Insert a string in the text field.<br>
	 * Method overriden to take off <b>\n</b> character if any at the end of the given String.
	 *
	 * @param  offset the position of this insert
	 * @param  str    the string to insert
	 * @param  a      the attributes to associate with the inserted content. This may be null if
	 *                there are no attributes
	 * @throws BadLocationException if the value of this field cannot be found
	 */
	public void insertString(int offset, String str, AttributeSet a)
		throws BadLocationException
	{
		if (str == null)
			str = "";
		if (str.indexOf("\n") != -1)
		{
			String newStr = str.substring(0, str.indexOf("\n"));

			insertString(offset, newStr, a);
		}
		else
			super.insertString(offset, str, a);

	}

} // end class LyJTextFieldDocument
